@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2002-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/backends/fs/abstract-fs.h $
- * $Id: abstract-fs.h 22424 2006-05-12 21:41:54Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/backends/fs/abstract-fs.h $
+ * $Id: abstract-fs.h 27024 2007-05-30 21:56:52Z fingolfin $
  */
 
 #ifndef BACKENDS_ABSTRACT_FS_H
@@ -25,7 +28,7 @@
 #include "common/array.h"
 #include "common/str.h"
 
-#include "backends/fs/fs.h"
+#include "common/fs.h"
 
 class AbstractFilesystemNode;
 
@@ -87,7 +90,7 @@ protected:
 	 * identical to oldNode. Hence, we can use the "path" value for persistent
 	 * storage e.g. in the config file.
 	 *
-	 * @TODO: This is of course a place where non-portable code easily will sneak
+	 * @todo: This is of course a place where non-portable code easily will sneak
 	 *        in, because the format of the path used here is not well-defined.
 	 *        So we really should reconsider this API and try to come up with
 	 *        something which is more portable but still flexible enough for our
@@ -99,7 +102,13 @@ protected:
 public:
 	virtual ~AbstractFilesystemNode() {}
 
-	virtual String displayName() const = 0;
+	virtual String name() const = 0;
+	
+	// By default, we use the actual file name as 'display name'.
+	virtual String displayName() const { return name(); }
+
+	virtual bool isValid() const = 0;
+
 	virtual bool isDirectory() const = 0;
 	
 	/**
