@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2005-2006 The ScummVM project, contribution by Hans-Jrg Frieden and Juha Niemimki
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/backends/fs/amigaos4/amigaos4-fs.cpp $
- * $Id: amigaos4-fs.cpp 22424 2006-05-12 21:41:54Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/backends/fs/amigaos4/amigaos4-fs.cpp $
+ * $Id: amigaos4-fs.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  */
 
 #if defined(__amigaos4__)
@@ -36,9 +39,8 @@
 
 #include "common/util.h"
 
-#include "base/engine.h"
+#include "engines/engine.h"
 #include "backends/fs/abstract-fs.h"
-#include "backends/fs/fs.h"
 
 #define ENTER() /* debug(6, "Enter") */
 #define LEAVE() /* debug(6, "Leave") */
@@ -65,6 +67,7 @@ class AmigaOSFilesystemNode : public AbstractFilesystemNode {
 		virtual ~AmigaOSFilesystemNode();
 
 		virtual String displayName() const { return _sDisplayName; };
+		virtual String name() const { return _sDisplayName; };
 		virtual bool isValid() const { return _bIsValid; };
 		virtual bool isDirectory() const { return _bIsDirectory; };
 		virtual String path() const { return _sPath; };
@@ -72,7 +75,7 @@ class AmigaOSFilesystemNode : public AbstractFilesystemNode {
 		virtual bool listDir(AbstractFSList &list, ListMode mode) const;
 		virtual AbstractFSList listVolumes() const;
 		virtual AbstractFilesystemNode *parent() const;
-		virtual AbstractFilesystemNode *child(const String &name) const;
+		virtual AbstractFilesystemNode *child(const String &n) const;
 };
 
 AbstractFilesystemNode *AbstractFilesystemNode::getCurrentDirectory() {
@@ -103,7 +106,12 @@ AmigaOSFilesystemNode::AmigaOSFilesystemNode(const String &p) {
 
 	int len = 0, offset = p.size();
 
-	assert(offset > 0);
+	//assert(offset > 0);
+
+	if (offset <= 0) {
+		debug(6, "Bad offset");
+		return;
+	}
 
 	_sPath = p;
 
@@ -120,6 +128,7 @@ AmigaOSFilesystemNode::AmigaOSFilesystemNode(const String &p) {
 
 	_sDisplayName = String(str + offset, len);
 	_pFileLock = 0;
+	_bIsDirectory = false;
 
 	struct FileInfoBlock *fib = (struct FileInfoBlock *)IDOS->AllocDosObject(DOS_FIB, NULL);
 	if (!fib) {
@@ -144,10 +153,12 @@ AmigaOSFilesystemNode::AmigaOSFilesystemNode(const String &p) {
 
 			}
 			else {
-				_bIsDirectory = false;
+				//_bIsDirectory = false;
 				_bIsValid = true;
 			}			 
 		}
+
+		IDOS->UnLock(pLock);
 	}
 
 	IDOS->FreeDosObject(DOS_FIB, fib);
@@ -160,11 +171,11 @@ AmigaOSFilesystemNode::AmigaOSFilesystemNode(BPTR pLock, const char *pDisplayNam
 	_pFileLock = 0;
 
 	while (1) {
-		char *name = new char[bufSize];
-		if (IDOS->NameFromLock(pLock, (STRPTR)name, bufSize) != DOSFALSE) {
-			_sPath = name;
-			_sDisplayName = pDisplayName ? pDisplayName : IDOS->FilePart((STRPTR)name);
-			delete [] name;
+		char *n = new char[bufSize];
+		if (IDOS->NameFromLock(pLock, (STRPTR)n, bufSize) != DOSFALSE) {
+			_sPath = n;
+			_sDisplayName = pDisplayName ? pDisplayName : IDOS->FilePart((STRPTR)n);
+			delete [] n;
 			break;
 		}
 
@@ -172,14 +183,15 @@ AmigaOSFilesystemNode::AmigaOSFilesystemNode(BPTR pLock, const char *pDisplayNam
 			_bIsValid = false;
 			debug(6, "IoErr() != ERROR_LINE_TOO_LONG");
 			LEAVE();
-			delete [] name;
+			delete [] n;
 			return;
 		}
 		bufSize *= 2;
-		delete [] name;
+		delete [] n;
 	}
 
 	_bIsValid =	false;
+	_bIsDirectory = false;
 
 	struct FileInfoBlock *fib = (struct	FileInfoBlock *)IDOS->AllocDosObject(DOS_FIB, NULL);
 	if (!fib) {
@@ -199,7 +211,7 @@ AmigaOSFilesystemNode::AmigaOSFilesystemNode(BPTR pLock, const char *pDisplayNam
 				_sPath += '/';
 		}
 		else {
-			_bIsDirectory = false;
+			//_bIsDirectory = false;
 			_bIsValid = true;
 		}		 
 	}
@@ -328,12 +340,29 @@ AbstractFilesystemNode *AmigaOSFilesystemNode::parent() const {
 	return node;
 }
 
-AbstractFilesystemNode *AmigaOSFilesystemNode::child(const String &name) const {
-	assert(_bIsDirectory);
+AbstractFilesystemNode *AmigaOSFilesystemNode::child(const String &n) const {
+	
+	if (!_bIsDirectory) {
+		debug(6, "Not a directory");
+		return 0;
+	}
+
 	String newPath(_sPath);
+
 	if (_sPath.lastChar() != '/')
 		newPath += '/';
-	newPath += name;
+
+	newPath += n;
+
+	BPTR lock = IDOS->Lock(newPath.c_str(), SHARED_LOCK);
+
+	if (!lock) {
+		debug(6, "Bad path");
+		return 0;
+	}
+
+	IDOS->UnLock(lock);
+
 	return new AmigaOSFilesystemNode(newPath);
 }
 
@@ -343,7 +372,7 @@ AbstractFSList AmigaOSFilesystemNode::listVolumes()	const {
 	AbstractFSList myList;
 
 	const uint32 kLockFlags = LDF_READ | LDF_VOLUMES;
-	char name[MAXPATHLEN];
+	char buffer[MAXPATHLEN];
 
 	struct DosList *dosList = IDOS->LockDosList(kLockFlags);
 	if (!dosList) {
@@ -352,30 +381,49 @@ AbstractFSList AmigaOSFilesystemNode::listVolumes()	const {
 		return myList;
 	}
 
-
 	dosList = IDOS->NextDosEntry(dosList, LDF_VOLUMES);
 	while (dosList) {
 		if (dosList->dol_Type == DLT_VOLUME &&
 			dosList->dol_Name &&
 			dosList->dol_Task) {
-			const char *volName = (const char *)BADDR(dosList->dol_Name)+1;
-			const char *devName = (const char *)((struct Task *)dosList->dol_Task->mp_SigTask)->tc_Node.ln_Name;
+			//const char *volName = (const char *)BADDR(dosList->dol_Name)+1;
+			
+			// Copy name to buffer
+			IDOS->CopyStringBSTRToC(dosList->dol_Name, buffer, MAXPATHLEN);
 
-			strcpy(name, volName);
-			strcat(name, ":");
+			//const char *devName = (const char *)((struct Task *)dosList->dol_Task->mp_SigTask)->tc_Node.ln_Name;
 
-			BPTR volumeLock = IDOS->Lock((STRPTR)name, SHARED_LOCK);
+			// Volume name + '\0'
+			char *volName = new char [strlen(buffer) + 1];
+
+			strcpy(volName, buffer);
+
+			strcat(buffer, ":");
+
+			BPTR volumeLock = IDOS->Lock((STRPTR)buffer, SHARED_LOCK);
 			if (volumeLock) {
-				sprintf(name, "%s (%s)", volName, devName);
-				AmigaOSFilesystemNode *entry = new AmigaOSFilesystemNode(volumeLock, name);
+
+				char *devName = new char [MAXPATHLEN];
+
+				// Find device name
+				IDOS->DevNameFromLock(volumeLock, devName, MAXPATHLEN, DN_DEVICEONLY);
+				
+				sprintf(buffer, "%s (%s)", volName, devName);
+
+				delete [] devName;
+
+				AmigaOSFilesystemNode *entry = new AmigaOSFilesystemNode(volumeLock, buffer);
 				if (entry) {
 					if (entry->isValid())
 						myList.push_back(entry);
 					else
 						delete entry;
 				}
+
 				IDOS->UnLock(volumeLock);
 			}
+
+			delete [] volName;
 		}
 		dosList	= IDOS->NextDosEntry(dosList, LDF_VOLUMES);
 	}
