@@ -1,8 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2001-2006 The ScummVM project
- * Copyright (C) 2002 Ph0x - GP32 Backend
- * Copyright (C) 2003-2004 DJWillis - GP32 Backend
- * Copyright (C) 2005-2006 Won Star - GP32 Backend
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -18,15 +18,14 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/trunk/backends/fs/gp32/gp32-fs.cpp $
- * $Id: gp32-fs.cpp 23274 2006-06-24 08:07:48Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/backends/fs/gp32/gp32-fs.cpp $
+ * $Id: gp32-fs.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
 #include "stdafx.h"
 
 #include "backends/fs/abstract-fs.h"
-#include "backends/fs/fs.h"
 
 class GP32FilesystemNode : public AbstractFilesystemNode {
 protected:
@@ -40,13 +39,16 @@ public:
 	GP32FilesystemNode(const String &path);
 
 	virtual String displayName() const { return _displayName; }
+	virtual String name() const { return _displayName; }
+	// FIXME: isValid should return false if this Node can't be used!
+	// client code can rely on the return value.
 	virtual bool isValid() const { return true; }
 	virtual bool isDirectory() const { return _isDirectory; }
 	virtual String path() const { return _path; }
 
 	virtual bool listDir(AbstractFSList &list, ListMode mode) const;
 	virtual AbstractFilesystemNode *parent() const;
-	virtual AbstractFilesystemNode *child(const String &name) const;
+	virtual AbstractFilesystemNode *child(const String &n) const;
 };
 
 #define MAX_PATH_SIZE 256
@@ -192,11 +194,11 @@ bool GP32FilesystemNode::listDir(AbstractFSList &myList, ListMode mode) const {
 	return true;
 }
 
-const char *lastPathComponent(const Common::String &str) {
+static const char *lastPathComponent(const Common::String &str) {
 	const char *start = str.c_str();
 	const char *cur = start + str.size() - 2;
 
-	while (cur > start && *cur != '\\') {
+	while (cur >= start && *cur != '\\') {
 		--cur;
 	}
 
@@ -217,14 +219,14 @@ AbstractFilesystemNode *GP32FilesystemNode::parent() const {
 	return p;
 }
 
-AbstractFilesystemNode *GP32FilesystemNode::child(const String &name) const {
+AbstractFilesystemNode *GP32FilesystemNode::child(const String &n) const {
 	// FIXME: Pretty lame implementation! We do no error checking to speak
 	// of, do not check if this is a special node, etc.
 	assert(_isDirectory);
 	String newPath(_path);
 	if (_path.lastChar() != '\\')
 		newPath += '\\';
-	newPath += name;
+	newPath += n;
 	GP32FilesystemNode *p = new GP32FilesystemNode(newPath);
 
 	return p;
