@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2002-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/backends/fs/morphos/abox-fs.cpp $
- * $Id: abox-fs.cpp 22424 2006-05-12 21:41:54Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/backends/fs/morphos/abox-fs.cpp $
+ * $Id: abox-fs.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  */
 
 #if defined(__MORPHOS__)
@@ -24,10 +27,13 @@
 #include <proto/dos.h>
 
 #include <stdio.h>
+#include <sys/stat.h>
 
+#include <common/stdafx.h>
+
+#include "common/util.h"
 #include "base/engine.h"
 #include "backends/fs/abstract-fs.h"
-#include "backends/fs/fs.h"
 
 /*
  * Implementation of the ScummVM file system API based on the MorphOS A-Box API.
@@ -44,9 +50,13 @@ class ABoxFilesystemNode : public AbstractFilesystemNode {
 	public:
 		ABoxFilesystemNode();
 		ABoxFilesystemNode(BPTR lock, CONST_STRPTR display_name = NULL);
+		ABoxFilesystemNode(const String &p);
+		ABoxFilesystemNode(const ABoxFilesystemNode &node);
+
 		~ABoxFilesystemNode();
 
 		virtual String displayName() const { return _displayName; }
+		virtual String name() const { return _displayName; };
 		virtual bool isValid() const { return _isValid; }
 		virtual bool isDirectory() const { return _isDirectory; }
 		virtual String path() const { return _path; }
@@ -62,6 +72,10 @@ AbstractFilesystemNode *AbstractFilesystemNode::getCurrentDirectory() {
 	return AbstractFilesystemNode::getRoot();
 }
 
+AbstractFilesystemNode *AbstractFilesystemNode::getNodeForPath(const String	&path) {
+	return new ABoxFilesystemNode(path);
+}
+
 AbstractFilesystemNode *AbstractFilesystemNode::getRoot()
 {
 	return new ABoxFilesystemNode();
@@ -93,18 +107,19 @@ ABoxFilesystemNode::ABoxFilesystemNode(BPTR lock, CONST_STRPTR display_name)
 		if (IoErr() != ERROR_LINE_TOO_LONG)
 		{
 			_isValid = false;
-			warning("Error while retrieving path name: %d", IoErr());
+			debug(6, "Error while retrieving path name: %ld", IoErr());
 			return;
 		}
 		bufsize *= 2;
 	}
 
+	_isDirectory = false;
 	_isValid = false;
 
 	FileInfoBlock *fib = (FileInfoBlock*) AllocDosObject(DOS_FIB, NULL);
 	if (fib == NULL)
 	{
-		warning("Failed to allocate memory for FileInfoBlock");
+		debug(6, "Failed to allocate memory for FileInfoBlock");
 		return;
 	}
 
@@ -119,11 +134,79 @@ ABoxFilesystemNode::ABoxFilesystemNode(BPTR lock, CONST_STRPTR display_name)
 			_isValid = (_lock != NULL);
 		}
 		else
+		{
 			_isValid = true;
+		}
 	}
 	FreeDosObject(DOS_FIB, fib);
 }
 
+ABoxFilesystemNode::ABoxFilesystemNode(const String &p) {
+	int len = 0, offset = p.size();
+
+	assert(offset > 0);
+
+	_path = p;
+
+	// Extract last component from path
+	const char *str = p.c_str();
+	while (offset > 0 && (str[offset-1] == '/' || str[offset-1] == ':') )
+		offset--;
+	while (offset > 0 && (str[offset-1] != '/' && str[offset-1] != ':')) {
+		len++;
+		offset--;
+	}
+	_displayName = String(str + offset, len);
+	_lock = NULL;
+	_isDirectory = false;
+
+	struct FileInfoBlock *fib = (struct FileInfoBlock *)AllocDosObject(DOS_FIB, NULL);
+	if (!fib)
+	{
+		debug(6, "FileInfoBlock is NULL");
+		return;
+	}
+
+	// Check whether the node exists and if it is a directory
+
+	BPTR pLock = Lock((STRPTR)_path.c_str(), SHARED_LOCK);
+	if (pLock)
+	{
+		if (Examine(pLock, fib) != DOSFALSE) {
+			if (fib->fib_EntryType > 0)
+			{
+				_isDirectory = true;
+				_lock = DupLock(pLock);
+				_isValid = (_lock != 0);
+
+				// Add a trailing slash if it is needed
+				const char c = _path.lastChar();
+				if (c != '/' && c != ':')
+					_path += '/';
+
+			}
+			else
+			{
+				_isDirectory = false;
+				_isValid = true;
+			}
+		}
+	
+		UnLock(pLock);
+	}
+
+	FreeDosObject(DOS_FIB, fib);
+}
+
+ABoxFilesystemNode::ABoxFilesystemNode(const ABoxFilesystemNode& node)
+{
+	_displayName = node._displayName;
+	_isValid = node._isValid;
+	_isDirectory = node._isDirectory;
+	_path = node._path;
+	_lock = DupLock(node._lock);
+}
+
 ABoxFilesystemNode::~ABoxFilesystemNode()
 {
 	if (_lock)
@@ -136,10 +219,15 @@ ABoxFilesystemNode::~ABoxFilesystemNode()
 bool ABoxFilesystemNode::listDir(AbstractFSList &myList, ListMode mode) const
 {
 	if (!_isValid)
-		error("listDir() called on invalid node");
-
+	{
+		debug(6, "listDir() called on invalid node");
+		return false;
+	}
 	if (!_isDirectory)
-		error("listDir() called on file node");
+	{
+		debug(6, "listDir() called on file node");
+		return false;
+	}
 
 	if (_lock == NULL)
 	{
@@ -153,7 +241,7 @@ bool ABoxFilesystemNode::listDir(AbstractFSList &myList, ListMode mode) const
 
 	if (fib == NULL)
 	{
-		warning("Failed to allocate memory for FileInfoBlock");
+		debug(6, "Failed to allocate memory for FileInfoBlock");
 		return false;
 	}
 
@@ -165,7 +253,8 @@ bool ABoxFilesystemNode::listDir(AbstractFSList &myList, ListMode mode) const
 			String full_path;
 			BPTR lock;
 
-			if ((fib->fib_EntryType > 0 && (mode & FilesystemNode::kListDirectoriesOnly)) ||
+			if ((mode == FilesystemNode::kListAll) ||
+			     (fib->fib_EntryType > 0 && (mode & FilesystemNode::kListDirectoriesOnly)) ||
 				 (fib->fib_EntryType < 0 && (mode & FilesystemNode::kListFilesOnly)))
 			{
 				full_path = _path;
@@ -173,7 +262,7 @@ bool ABoxFilesystemNode::listDir(AbstractFSList &myList, ListMode mode) const
 				lock = Lock(full_path.c_str(), SHARED_LOCK);
 				if (lock)
 				{
-					entry = new ABoxFilesystemNode(lock);
+					entry = new ABoxFilesystemNode(lock, fib->fib_FileName);
 					if (entry)
 					{
 						if (entry->isValid())
@@ -187,12 +276,12 @@ bool ABoxFilesystemNode::listDir(AbstractFSList &myList, ListMode mode) const
 		}
 
 		if (IoErr() != ERROR_NO_MORE_ENTRIES)
-			warning("Error while reading directory: %d", IoErr());
+			debug(6, "Error while reading directory: %ld", IoErr());
 	}
 
 	FreeDosObject(DOS_FIB, fib);
 
-	return tree;
+	return true;
 }
 
 AbstractFilesystemNode *ABoxFilesystemNode::parent() const
@@ -200,11 +289,14 @@ AbstractFilesystemNode *ABoxFilesystemNode::parent() const
 	AbstractFilesystemNode *node = NULL;
 
 	if (!_isDirectory)
-		error("parent() called on file node");
+	{
+		debug(6, "parent() called on file node");
+		return NULL;
+	}
 
 	if (_lock == NULL) {
 		/* Parent of the root is the root itself */
-		node = 0;
+		return new ABoxFilesystemNode(*this);
 	} else {
 		BPTR parent_lock = ParentDir(_lock);
 		if (parent_lock) {
@@ -218,7 +310,23 @@ AbstractFilesystemNode *ABoxFilesystemNode::parent() const
 }
 
 AbstractFilesystemNode *ABoxFilesystemNode::child(const String &name) const {
-	TODO
+	assert(_isDirectory);
+	String newPath(_path);
+
+	if (_path.lastChar() != '/')
+		newPath += '/';
+	newPath += name;
+
+	BPTR lock = Lock(newPath.c_str(), SHARED_LOCK);
+
+	if (!lock)
+	{
+		return 0;
+    }
+
+	UnLock(lock);
+
+	return new ABoxFilesystemNode(newPath);
 }
 
 AbstractFSList ABoxFilesystemNode::listRoot()
@@ -231,7 +339,6 @@ AbstractFSList ABoxFilesystemNode::listRoot()
 	dosList = LockDosList(lockDosListFlags);
 	if (dosList == NULL)
 	{
-		warning("Could not lock dos list");
 		return myList;
 	}
 
