@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2003-2006 The ScummVM Team
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,18 +18,17 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/backends/fs/palmos/palmos-fs.cpp $
- * $Id: palmos-fs.cpp 22867 2006-06-03 11:21:39Z chrilith $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/backends/fs/palmos/palmos-fs.cpp $
+ * $Id: palmos-fs.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  */
 
 #if defined(PALMOS_MODE)
 
+#include "PalmVersion.h"
+#include "globals.h"
+
 #include "common/stdafx.h"
 #include "backends/fs/abstract-fs.h"
-#include "backends/fs/fs.h"
-
-#include <stdio.h>
-#include <stdlib.h>
 
 /*
  * Implementation of the ScummVM file system API based on PalmOS VFS API.
@@ -45,13 +47,14 @@ public:
 	PalmOSFilesystemNode(const String &p);
 
 	virtual String displayName() const { return _displayName; }
+	virtual String name() const { return _displayName; }
 	virtual bool isValid() const { return _isValid; }
 	virtual bool isDirectory() const { return _isDirectory; }
 	virtual String path() const { return _path; }
 
 	virtual bool listDir(AbstractFSList &list, ListMode mode) const;
 	virtual AbstractFilesystemNode *parent() const;
-	virtual AbstractFilesystemNode *child(const String &name) const;
+	virtual AbstractFilesystemNode *child(const String &n) const;
 
 private:
 	static void addFile (AbstractFSList &list, ListMode mode, const Char *base, FileInfoType* find_data);
@@ -61,8 +64,9 @@ static const char *lastPathComponent(const Common::String &str) {
 	const char *start = str.c_str();
 	const char *cur = start + str.size() - 2;
 
-	while (cur > start && *cur != '/')
+	while (cur >= start && *cur != '/') {
 		--cur;
+	}
 
 	return cur + 1;
 }
@@ -180,13 +184,13 @@ AbstractFilesystemNode *PalmOSFilesystemNode::parent() const {
 }
 
 
-AbstractFilesystemNode *PalmOSFilesystemNode::child(const String &name) const {
+AbstractFilesystemNode *PalmOSFilesystemNode::child(const String &n) const {
 	assert(_isDirectory);
 	String newPath(_path);
 
 	if (_path.lastChar() != '/')
 		newPath += '/';
-	newPath += name;
+	newPath += n;
 
 	FileRef handle;
 	UInt32 attr;
