@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2002-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/backends/fs/posix/posix-fs.cpp $
- * $Id: posix-fs.cpp 22883 2006-06-03 14:09:04Z marcus_c $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/backends/fs/posix/posix-fs.cpp $
+ * $Id: posix-fs.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  */
 
 #if defined(UNIX)
@@ -24,7 +27,6 @@
 #include "common/stdafx.h"
 
 #include "backends/fs/abstract-fs.h"
-#include "backends/fs/fs.h"
 
 #ifdef MACOSX
 #include <sys/types.h>
@@ -51,13 +53,14 @@ public:
 	POSIXFilesystemNode(const String &path, bool verify);
 
 	virtual String displayName() const { return _displayName; }
+	virtual String name() const { return _displayName; }
 	virtual bool isValid() const { return _isValid; }
 	virtual bool isDirectory() const { return _isDirectory; }
 	virtual String path() const { return _path; }
 
 	virtual bool listDir(AbstractFSList &list, ListMode mode) const;
 	virtual AbstractFilesystemNode *parent() const;
-	virtual AbstractFilesystemNode *child(const String &name) const;
+	virtual AbstractFilesystemNode *child(const String &n) const;
 };
 
 
@@ -65,7 +68,7 @@ static const char *lastPathComponent(const Common::String &str) {
 	const char *start = str.c_str();
 	const char *cur = start + str.size() - 2;
 
-	while (cur > start && *cur != '/') {
+	while (cur >= start && *cur != '/') {
 		--cur;
 	}
 
@@ -209,14 +212,14 @@ AbstractFilesystemNode *POSIXFilesystemNode::parent() const {
 	return p;
 }
 
-AbstractFilesystemNode *POSIXFilesystemNode::child(const String &name) const {
+AbstractFilesystemNode *POSIXFilesystemNode::child(const String &n) const {
 	// FIXME: Pretty lame implementation! We do no error checking to speak
 	// of, do not check if this is a special node, etc.
 	assert(_isDirectory);
 	String newPath(_path);
 	if (_path.lastChar() != '/')
 		newPath += '/';
-	newPath += name;
+	newPath += n;
 	POSIXFilesystemNode *p = new POSIXFilesystemNode(newPath, true);
 
 	return p;
