@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2005-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,12 +18,11 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/backends/fs/ps2/ps2-fs.cpp $
- * $Id: ps2-fs.cpp 24185 2006-10-08 00:55:31Z lavosspawn $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/backends/fs/ps2/ps2-fs.cpp $
+ * $Id: ps2-fs.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  */
 
 #include "backends/fs/abstract-fs.h"
-#include "backends/fs/fs.h"
 #include <kernel.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -43,6 +45,7 @@ public:
 	Ps2FilesystemNode(const String &path);
 
 	virtual String displayName() const { return _displayName; }
+	virtual String name() const { return _displayName; }
 	virtual bool isValid() const { return !_isRoot; }
 	virtual bool isDirectory() const { return _isDirectory; }
 	virtual String path() const { return _path; }
@@ -51,7 +54,7 @@ public:
 	virtual bool listDir(AbstractFSList &list, ListMode mode) const;
 	virtual AbstractFilesystemNode *parent() const;
 	virtual AbstractFilesystemNode *clone() const { return new Ps2FilesystemNode(this); }
-	virtual AbstractFilesystemNode *child(const String &name) const;
+	virtual AbstractFilesystemNode *child(const String &n) const;
 };
 
 AbstractFilesystemNode *AbstractFilesystemNode::getCurrentDirectory() {
@@ -85,31 +88,15 @@ Ps2FilesystemNode::Ps2FilesystemNode(const String &path) {
 		while (*pos)
 			if (*pos++ == '/')
 				dsplName = pos;
-		if (dsplName) {
+		if (dsplName)
 			_displayName = String(dsplName);
-
-			// check if this is a file or directory
-			if (path.lastChar() != '/') {
-				char checkPath[256];
-				sprintf(checkPath, "%s/", path.c_str());
-				int fd = fio.dopen(checkPath);
-				printf("check: %s: %d\n", checkPath, fd);
-				if (fd < 0)
-					_isDirectory = false;
-				else
-					fio.dclose(fd);
-			} // if it ends with /, assume it's not a file
-		} else {
+		else {
 			if (strncmp(path.c_str(), "cdfs", 4) == 0)
 				_displayName = "DVD Drive";
 			else if (strncmp(path.c_str(), "mass", 4) == 0)
 				_displayName = "USB Mass Storage";
-			else if (strncmp(path.c_str(), "pfs0", 4) == 0)
+			else
 				_displayName = "Harddisk";
-			else {
-				printf("PS2FSNode ctor: invalid path %s\n", path.c_str());
-				_isDirectory = false;
-			}
 		}
 	}
 }
@@ -146,19 +133,6 @@ bool Ps2FilesystemNode::listDir(AbstractFSList &list, ListMode mode) const {
 		}
 		return true;
 	} else {
-		if (strncmp(_path.c_str(), "host:", 5) == 0) {
-			// special case, we can't list the host device.
-			Ps2FilesystemNode dirEntry;
-			dirEntry._isRoot = false;
-			dirEntry._isDirectory = false;
-			char *fakeFiles[4] = { "modern.ini", "modern.zip", "classic.ini", "classic.zip" };
-			for (int i = 0; i < 4; i++) {
-				dirEntry._path = _path + fakeFiles[i];
-				dirEntry._displayName = fakeFiles[i];
-				list.push_back(new Ps2FilesystemNode(&dirEntry));
-			}
-			return true;
-		}
 		char listDir[256];
 		int fd;
 		if (_path.lastChar() == '/')
@@ -218,7 +192,7 @@ AbstractFilesystemNode *Ps2FilesystemNode::parent() const {
 		return new Ps2FilesystemNode();
 }
 
-AbstractFilesystemNode *Ps2FilesystemNode::child(const String &name) const {
+AbstractFilesystemNode *Ps2FilesystemNode::child(const String &n) const {
 	if (!_isDirectory)
 		return NULL;
 
@@ -230,7 +204,7 @@ AbstractFilesystemNode *Ps2FilesystemNode::child(const String &name) const {
 		iox_dirent_t dirent;
 
 		while (fio.dread(fd, &dirent) > 0) {
-			if (strcmp(name.c_str(), dirent.name) == 0) {
+			if (strcmp(n.c_str(), dirent.name) == 0) {
 				Ps2FilesystemNode *dirEntry = new Ps2FilesystemNode();
 
 				dirEntry->_isDirectory = (bool)(dirent.stat.mode & FIO_S_IFDIR);
