@@ -1,8 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2005-2006 The ScummVM project
- * Copyright (C) 2005 Joost Peters PSP Backend
- * Copyright (C) 2005 Thomas Mayer PSP Backend
- * Copyright (C) 2005 Paolo Costabel PSP Backend
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -18,15 +18,14 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/backends/fs/psp/psp_fs.cpp $
- * $Id: psp_fs.cpp 22727 2006-05-28 22:02:38Z joostp $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/backends/fs/psp/psp_fs.cpp $
+ * $Id: psp_fs.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  */
 
 #ifdef __PSP__
-#include "base/engine.h"
+#include "engines/engine.h"
 
 #include "backends/fs/abstract-fs.h"
-#include "backends/fs/fs.h"
 
 #include <sys/stat.h>
 #include <unistd.h>
@@ -50,13 +49,14 @@ public:
 	PSPFilesystemNode(const Common::String &p, bool verify);
 
 	virtual String displayName() const { return _displayName; }
+	virtual String name() const { return _displayName; }
 	virtual bool isValid() const { return _isValid; }
 	virtual bool isDirectory() const { return _isDirectory; }
 	virtual String path() const { return _path; }
 
 	virtual bool listDir(AbstractFSList &list, ListMode mode) const;
 	virtual AbstractFilesystemNode *parent() const;
-	virtual AbstractFilesystemNode *child(const String &name) const;
+	virtual AbstractFilesystemNode *child(const String &n) const;
 };
 
 AbstractFilesystemNode *AbstractFilesystemNode::getCurrentDirectory() {
@@ -133,11 +133,11 @@ bool PSPFilesystemNode::listDir(AbstractFSList &myList, ListMode mode) const {
 	}
 }
 
-const char *lastPathComponent(const Common::String &str) {
+static const char *lastPathComponent(const Common::String &str) {
 	const char *start = str.c_str();
 	const char *cur = start + str.size() - 2;
 
-	while (cur > start && *cur != '/') {
+	while (cur >= start && *cur != '/') {
 		--cur;
 	}
 
@@ -158,14 +158,14 @@ AbstractFilesystemNode *PSPFilesystemNode::parent() const {
 	return p;
 }
 
-AbstractFilesystemNode *PSPFilesystemNode::child(const String &name) const {
+AbstractFilesystemNode *PSPFilesystemNode::child(const String &n) const {
 	// FIXME: Pretty lame implementation! We do no error checking to speak
 	// of, do not check if this is a special node, etc.
 	assert(_isDirectory);
 	String newPath(_path);
 	if (_path.lastChar() != '/')
 		newPath += '/';
-	newPath += name;
+	newPath += n;
 	PSPFilesystemNode *p = new PSPFilesystemNode(newPath, true);
 
 	return p;
