@@ -1,8 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2003-2005 Andreas 'Sprawl' Karlsson - Original EPOC port, ESDL
- * Copyright (C) 2003-2005 Lars 'AnotherGuest' Persson - Original EPOC port, Audio System
- * Copyright (C) 2005 Jurgen 'SumthinWicked' Braam - EPOC/CVS maintainer
- * Copyright (C) 2005-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -18,14 +18,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/backends/fs/symbian/symbian-fs.cpp $
- * $Id: symbian-fs.cpp 22424 2006-05-12 21:41:54Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/backends/fs/symbian/symbian-fs.cpp $
+ * $Id: symbian-fs.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  */
 
 #if defined (__SYMBIAN32__) 
 #include "common/stdafx.h"
 #include "backends/fs/abstract-fs.h"
-#include "backends/fs/fs.h"
 
 #include <dirent.h>
 #include <eikenv.h>
@@ -48,13 +47,14 @@ public:
 	SymbianFilesystemNode(bool aIsRoot);
 	SymbianFilesystemNode(const String &path);
 	virtual String displayName() const { return _displayName; }
+	virtual String name() const { return _displayName; }
 	virtual bool isValid() const { return _isValid; }
 	virtual bool isDirectory() const { return _isDirectory; }
 	virtual String path() const { return _path; }
 
 	virtual bool listDir(AbstractFSList &list, ListMode mode) const;
 	virtual AbstractFilesystemNode *parent() const;
-	virtual AbstractFilesystemNode *child(const String &name) const;
+	virtual AbstractFilesystemNode *child(const String &n) const;
 };
 
 
@@ -62,15 +62,27 @@ static const char *lastPathComponent(const Common::String &str) {
 	const char *start = str.c_str();
 	const char *cur = start + str.size() - 2;
 
-	while (cur > start && *cur != '\\') {
+	while (cur >= start && *cur != '\\') {
 		--cur;
 	}
 
 	return cur + 1;
 }
 
+static void fixFilePath(Common::String& path) {
+	TInt len = path.size();
+	
+	for (TInt index = 0; index < len; index++) {
+		if (path[index] == '/') {	
+			path[index] = '\\';
+		}
+	}
+}
+
 AbstractFilesystemNode *AbstractFilesystemNode::getCurrentDirectory() {
-	return AbstractFilesystemNode::getRoot();
+	char path[MAXPATHLEN];
+	getcwd(path, MAXPATHLEN);
+	return new SymbianFilesystemNode(path);
 }
 
 AbstractFilesystemNode *AbstractFilesystemNode::getRoot() {
@@ -97,16 +109,23 @@ SymbianFilesystemNode::SymbianFilesystemNode(const String &path) {
 		_isPseudoRoot = false;
 
 	_path = path;
-	const char *dsplName = NULL, *pos = path.c_str();
-	// FIXME -- why is this code scanning for a slash '/' when the rest of
-	// the code in this file uses backslashes '\' ?
-	// TODO: Use lastPathComponent here.
-	while (*pos)
-		if (*pos++ == '/')
-			dsplName = pos;
-	_displayName = String(dsplName);
-	_isValid = true;
-	_isDirectory = true;
+	
+	fixFilePath(_path);
+
+	_displayName = lastPathComponent(_path);
+
+	TEntry fileAttribs;
+	TFileName fname;
+	TPtrC8 ptr((const unsigned char*)_path.c_str(),_path.size());
+	fname.Copy(ptr);
+
+	if (CEikonEnv::Static()->FsSession().Entry(fname, fileAttribs) == KErrNone) {
+		_isValid = true;
+		_isDirectory = fileAttribs.IsDir();
+	} else {
+		_isValid = false;
+		_isDirectory = false;
+	}
 }
 
 bool SymbianFilesystemNode::listDir(AbstractFSList &myList, ListMode mode) const {
@@ -207,13 +226,13 @@ AbstractFilesystemNode *SymbianFilesystemNode::parent() const {
 	return p;
 }
 
-AbstractFilesystemNode *SymbianFilesystemNode::child(const String &name) const {
+AbstractFilesystemNode *SymbianFilesystemNode::child(const String &n) const {
 	assert(_isDirectory);
 	String newPath(_path);
 
 	if (_path.lastChar() != '\\')
 		newPath += '\\';
-	newPath += name;
+	newPath += n;
 
 	TPtrC8 ptr((const unsigned char*) newPath.c_str(), newPath.size());
 	TFileName fname;
