@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2002-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,18 +18,22 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/backends/fs/windows/windows-fs.cpp $
- * $Id: windows-fs.cpp 22923 2006-06-05 00:14:23Z kirben $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/backends/fs/windows/windows-fs.cpp $
+ * $Id: windows-fs.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  */
 
 #ifdef WIN32
 
+#ifdef _WIN32_WCE
+#include <windows.h>
+#endif
 #include "common/stdafx.h"
 #include "backends/fs/abstract-fs.h"
-#include "backends/fs/fs.h"
 #include <stdio.h>
 #include <stdlib.h>
+#ifndef _WIN32_WCE
 #include <windows.h>
+#endif
 #include <tchar.h>
 
 /*
@@ -46,13 +53,14 @@ public:
 	WindowsFilesystemNode(const String &path);
 
 	virtual String displayName() const { return _displayName; }
+	virtual String name() const { return _displayName; }
 	virtual bool isValid() const { return _isValid; }
 	virtual bool isDirectory() const { return _isDirectory; }
 	virtual String path() const { return _path; }
 
 	virtual bool listDir(AbstractFSList &list, ListMode mode) const;
 	virtual AbstractFilesystemNode *parent() const;
-	virtual AbstractFilesystemNode *child(const String &name) const;
+	virtual AbstractFilesystemNode *child(const String &n) const;
 
 private:
 	static char *toAscii(TCHAR *x);
@@ -65,7 +73,7 @@ static const char *lastPathComponent(const Common::String &str) {
 	const char *start = str.c_str();
 	const char *cur = start + str.size() - 2;
 
-	while (cur > start && *cur != '\\') {
+	while (cur >= start && *cur != '\\') {
 		--cur;
 	}
 
@@ -241,12 +249,12 @@ AbstractFilesystemNode *WindowsFilesystemNode::parent() const {
 	return p;
 }
 
-AbstractFilesystemNode *WindowsFilesystemNode::child(const String &name) const {
+AbstractFilesystemNode *WindowsFilesystemNode::child(const String &n) const {
 	assert(_isDirectory);
 	String newPath(_path);
 	if (_path.lastChar() != '\\')
 		newPath += '\\';
-	newPath += name;
+	newPath += n;
 
 	// Check whether the directory actually exists
 	DWORD fileAttribs = GetFileAttributes(toUnicode(newPath.c_str()));
