@@ -1,6 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2001  Ludvig Strigeus
- * Copyright (C) 2001-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -16,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/backends/midi/alsa.cpp $
- * $Id: alsa.cpp 21605 2006-04-04 21:45:13Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/backends/midi/alsa.cpp $
+ * $Id: alsa.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  */
 
 #include "common/stdafx.h"
@@ -161,6 +163,10 @@ void MidiDriver_ALSA::send(uint32 b) {
 		snd_seq_ev_set_noteon(&ev, chanID, midiCmd[1], midiCmd[2]);
 		send_event(1);
 		break;
+	case 0xA0:
+		snd_seq_ev_set_keypress(&ev, chanID, midiCmd[1], midiCmd[2]);
+		send_event(1);
+		break;
 	case 0xB0:
 		/* is it this simple ? Wow... */
 		snd_seq_ev_set_controller(&ev, chanID, midiCmd[1], midiCmd[2]);
@@ -172,7 +178,7 @@ void MidiDriver_ALSA::send(uint32 b) {
 		break;
 	case 0xD0:
 		snd_seq_ev_set_chanpress(&ev, chanID, midiCmd[1]);
-		send_event(0);
+		send_event(1);
 		break;
 	case 0xE0:{
 			// long theBend = ((((long)midiCmd[1] + (long)(midiCmd[2] << 7))) - 0x2000) / 4;
@@ -184,7 +190,7 @@ void MidiDriver_ALSA::send(uint32 b) {
 		break;
 
 	default:
-		error("Unknown Command: %08x\n", (int)b);
+		warning("Unknown MIDI Command: %08x", (int)b);
 		/* I don't know if this works but, well... */
 		send_event(1);
 		break;
@@ -192,15 +198,16 @@ void MidiDriver_ALSA::send(uint32 b) {
 }
 
 void MidiDriver_ALSA::sysEx(const byte *msg, uint16 length) {
-	unsigned char buf[1024];
+	unsigned char buf[256];
 
-	if (length > 254) {
-		warning("Cannot send SysEx block - data too large");
-		return;
-	}
+	assert(length + 2 <= 256);
+
+	// Add SysEx frame
 	buf[0] = 0xF0;
 	memcpy(buf + 1, msg, length);
 	buf[length + 1] = 0xF7;
+
+	// Send it
 	snd_seq_ev_set_sysex(&ev, length + 2, &buf);
 	send_event(1);
 }
