@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2001-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/backends/midi/coreaudio.cpp $
- * $Id: coreaudio.cpp 21894 2006-04-14 23:21:59Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/backends/midi/coreaudio.cpp $
+ * $Id: coreaudio.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  */
 
 #ifdef MACOSX
@@ -178,22 +181,18 @@ void MidiDriver_CORE::send(uint32 b) {
 }
 
 void MidiDriver_CORE::sysEx(const byte *msg, uint16 length) {
+	unsigned char buf[256];
+
+	assert(length + 2 <= 256);
 	assert(_auGraph != NULL);
 
-	// Add SysEx frame if missing
-	byte *buf = 0;
-	if (*msg != 0xF0) {
-		buf = (byte *)malloc(length + 2);
-		buf[0] = 0xF0;
-		memcpy(buf+1, msg, length);
-		buf[length+1] = 0xF7;
-		msg = buf;
-		length += 2;
-	}
+	// Add SysEx frame
+	buf[0] = 0xF0;
+	memcpy(buf + 1, msg, length);
+	buf[length + 1] = 0xF7;
 
-	MusicDeviceSysEx(_synth, msg, length);
-
-	free(buf);
+	// Send it
+	MusicDeviceSysEx(_synth, buf, length+2);
 }
 
 MidiDriver *MidiDriver_CORE_create() {
