@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2001-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/backends/midi/coremidi.cpp $
- * $Id: coremidi.cpp 20952 2006-02-27 01:59:07Z Jamieson630 $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/backends/midi/coremidi.cpp $
+ * $Id: coremidi.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  */
 
 #ifdef MACOSX
@@ -163,17 +166,13 @@ void MidiDriver_CoreMIDI::sysEx(const byte *msg, uint16 length) {
 
 	packet->timeStamp = 0;
 
-	// Add SysEx frame if missing
-	if (*msg != 0xF0) {
-		packet->length = length + 2;
-		packet->data[0] = 0xF0;
-		memcpy(packet->data + 1, msg, length);
-		packet->data[length + 1] = 0xF7;
-	} else {
-		packet->length = length;
-		memcpy(packet->data, msg, length);
-	}
+	// Add SysEx frame
+	packet->length = length + 2;
+	packet->data[0] = 0xF0;
+	memcpy(packet->data + 1, msg, length);
+	packet->data[length + 1] = 0xF7;
 
+	// Send it
 	MIDISend(mOutPort, mDest, packetList);
 }
 
