@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2001-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,12 +18,15 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/backends/midi/windows.cpp $
- * $Id: windows.cpp 20952 2006-02-27 01:59:07Z Jamieson630 $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/backends/midi/windows.cpp $
+ * $Id: windows.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  */
 
 #if defined(WIN32) && !defined(_WIN32_WCE)
 
+
+#include <windows.h>
+#include <mmsystem.h>
 #include "common/stdafx.h"
 #include "sound/mpu401.h"
 #include "common/util.h"
@@ -34,7 +40,7 @@
 class MidiDriver_WIN : public MidiDriver_MPU401 {
 private:
 	MIDIHDR _streamHeader;
-	byte _streamBuffer [258];	// SysEx blocks should be no larger than 256 bytes
+	byte _streamBuffer[256];	// SysEx blocks should be no larger than 256 bytes
 	HANDLE _streamEvent;
 	HMIDIOUT _mo;
 	bool _isOpen;
@@ -42,19 +48,19 @@ private:
 	void check_error(MMRESULT result);
 
 public:
-	MidiDriver_WIN() : _isOpen (false) { }
+	MidiDriver_WIN() : _isOpen(false) { }
 	int open();
 	void close();
 	void send(uint32 b);
-	void sysEx (const byte *msg, uint16 length);
+	void sysEx(const byte *msg, uint16 length);
 };
 
 int MidiDriver_WIN::open() {
 	if (_isOpen)
 		return MERR_ALREADY_OPEN;
 
-	_streamEvent = CreateEvent (NULL, true, true, NULL);
-	MMRESULT res = midiOutOpen((HMIDIOUT *)&_mo, MIDI_MAPPER, (unsigned long) _streamEvent, 0, CALLBACK_EVENT);
+	_streamEvent = CreateEvent(NULL, true, true, NULL);
+	MMRESULT res = midiOutOpen((HMIDIOUT *)&_mo, MIDI_MAPPER, (unsigned long)_streamEvent, 0, CALLBACK_EVENT);
 	if (res != MMSYSERR_NOERROR) {
 		check_error(res);
 		CloseHandle(_streamEvent);
@@ -70,9 +76,9 @@ void MidiDriver_WIN::close() {
 		return;
 	_isOpen = false;
 	MidiDriver_MPU401::close();
-	midiOutUnprepareHeader (_mo, &_streamHeader, sizeof (_streamHeader));
+	midiOutUnprepareHeader(_mo, &_streamHeader, sizeof(_streamHeader));
 	check_error(midiOutClose(_mo));
-	CloseHandle (_streamEvent);
+	CloseHandle(_streamEvent);
 }
 
 void MidiDriver_WIN::send(uint32 b) {
@@ -98,25 +104,29 @@ void MidiDriver_WIN::sysEx(const byte *msg, uint16 length) {
 		return;
 	}
 
-	midiOutUnprepareHeader (_mo, &_streamHeader, sizeof (_streamHeader));
-	_streamBuffer [0] = 0xF0;
-	memcpy(&_streamBuffer[1], msg, length);
-	_streamBuffer [length+1] = 0xF7;
+	assert(length+2 <= 256);
 
-	_streamHeader.lpData = (char *) _streamBuffer;
+	midiOutUnprepareHeader(_mo, &_streamHeader, sizeof(_streamHeader));
+
+	// Add SysEx frame
+	_streamBuffer[0] = 0xF0;
+	memcpy(&_streamBuffer[1], msg, length);
+	_streamBuffer[length+1] = 0xF7;
+
+	_streamHeader.lpData = (char *)_streamBuffer;
 	_streamHeader.dwBufferLength = length + 2;
 	_streamHeader.dwBytesRecorded = length + 2;
 	_streamHeader.dwUser = 0;
 	_streamHeader.dwFlags = 0;
 
-	MMRESULT result = midiOutPrepareHeader (_mo, &_streamHeader, sizeof (_streamHeader));
+	MMRESULT result = midiOutPrepareHeader(_mo, &_streamHeader, sizeof(_streamHeader));
 	if (result != MMSYSERR_NOERROR) {
 		check_error (result);
 		return;
 	}
 
 	ResetEvent(_streamEvent);
-	result = midiOutLongMsg (_mo, &_streamHeader, sizeof (_streamHeader));
+	result = midiOutLongMsg(_mo, &_streamHeader, sizeof(_streamHeader));
 	if (result != MMSYSERR_NOERROR) {
 		check_error(result);
 		SetEvent(_streamEvent);
