@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2001-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/backends/midi/zodiac.cpp $
- * $Id: zodiac.cpp 24418 2006-10-21 19:56:38Z chrilith $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/backends/midi/zodiac.cpp $
+ * $Id: zodiac.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  */
 
 #include "common/stdafx.h"
@@ -25,8 +28,7 @@
 
 #ifndef DISABLE_TAPWAVE
 
-#include "TwTraps.h"
-#include "TwMidi.h"
+#include <tapwave.h>
 
 
 class MidiDriver_Zodiac:public MidiDriver_MPU401 {
@@ -55,7 +57,7 @@ int MidiDriver_Zodiac::open() {
  		return MERR_DEVICE_NOT_AVAILABLE;
 
 	TwMidiGetMasterVolume(&_oldVol);
-	TwMidiSetMasterVolume(twMidiMaxVolume);
+	TwMidiSetMasterVolume(twMidiMaxVolume); // TODO : set volume based on gVars
 
 	_isOpen = true;
 	return 0;
@@ -110,7 +112,13 @@ void MidiDriver_Zodiac::send(uint32 b) {
 }
 
 void MidiDriver_Zodiac::sysEx(const byte *msg, uint16 length) {
-	TwMidiSysEx(_midiHandle, 0, (byte *)msg, length);
+	unsigned char buf[256];
+	
+	buf[0] = 0xF0;
+	memcpy(buf + 1, msg, length);
+	buf[length + 1] = 0xF7;
+
+	TwMidiSysEx(_midiHandle, 0, (byte *)buf, length + 2);
 }
 
 MidiDriver *MidiDriver_Zodiac_create() {
