@@ -1,7 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2001  Ludvig Strigeus
- * Copyright (C) 2001-2006 The ScummVM project
- * Copyright (C) 2002-2006 Chris Apers - PalmOS Backend
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -17,12 +18,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/backends/platform/PalmOS/Src/base_event.cpp $
- * $Id: base_event.cpp 24417 2006-10-21 18:36:35Z chrilith $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/backends/platform/PalmOS/Src/base_event.cpp $
+ * $Id: base_event.cpp 27448 2007-06-16 09:57:14Z chrilith $
  *
  */
 
 #include "be_base.h"
+#include "common/events.h"
 
 #ifdef STDLIB_TRACE_MEMORY
 #	include <stdlib.h>
@@ -80,7 +82,7 @@ void OSystem_PalmBase::battery_handler() {
 	}
 }
 
-bool OSystem_PalmBase::pollEvent(Event &event) {
+bool OSystem_PalmBase::pollEvent(Common::Event &event) {
 	::EventType ev;
 	Boolean handled;
 	UInt32 keyCurrentState;
@@ -91,85 +93,174 @@ bool OSystem_PalmBase::pollEvent(Event &event) {
 	sound_handler();
 
 	for(;;) {
+		// check for hardkey repeat for mouse emulation
+		keyCurrentState = KeyCurrentState();
+
+		// if it was a key pressed, let the keyup event raise
+		if (_keyExtraPressed) {
+			if (gVars->arrowKeys) {
+				if (_keyExtraPressed & _keyExtra.bitLeft) {
+					if (!(keyCurrentState & _keyExtra.bitLeft)) {
+						_keyExtraPressed &= ~_keyExtra.bitLeft;
+
+						event.type = Common::EVENT_KEYUP;
+						event.kbd.keycode = 276;
+						event.kbd.ascii = event.kbd.keycode;
+						event.kbd.flags = 0;
+						return true;
+					}
+				}
+				if (_keyExtraPressed & _keyExtra.bitRight) {
+					if (!(keyCurrentState & _keyExtra.bitRight)) {
+						_keyExtraPressed &= ~_keyExtra.bitRight;
+
+						event.type = Common::EVENT_KEYUP;
+						event.kbd.keycode = 275;
+						event.kbd.ascii = event.kbd.keycode;
+						event.kbd.flags = 0;
+						return true;
+					}
+				}
+				if (_keyExtraPressed & _keyExtra.bitUp) {
+					if (!(keyCurrentState & _keyExtra.bitUp)) {
+						_keyExtraPressed &= ~_keyExtra.bitUp;
+
+						event.type = Common::EVENT_KEYUP;
+						event.kbd.keycode = 273;
+						event.kbd.ascii = event.kbd.keycode;
+						event.kbd.flags = 0;
+						return true;
+					}
+				}
+				if (_keyExtraPressed & _keyExtra.bitDown) {
+					if (!(keyCurrentState & _keyExtra.bitDown)) {
+						_keyExtraPressed &= ~_keyExtra.bitDown;
+
+						event.type = Common::EVENT_KEYUP;
+						event.kbd.keycode = 274;
+						event.kbd.ascii = event.kbd.keycode;
+						event.kbd.flags = 0;
+						return true;
+					}
+				}
+			}
+
+			if (_keyExtraPressed & _keyExtra.bitActionA) {
+				if (!(keyCurrentState & _keyExtra.bitActionA)) {
+					_keyExtraPressed &= ~_keyExtra.bitActionA;
+
+					event.type = Common::EVENT_LBUTTONUP;
+					event.mouse.x = _mouseCurState.x;
+					event.mouse.y = _mouseCurState.y;
+					return true;
+				}
+			}
+
+			if (_keyExtraPressed & _keyExtra.bitActionB) {
+				if (!(keyCurrentState & _keyExtra.bitActionB)) {
+					_keyExtraPressed &= ~_keyExtra.bitActionB;
+
+					event.type = Common::EVENT_RBUTTONUP;
+					event.mouse.x = _mouseCurState.x;
+					event.mouse.y = _mouseCurState.y;
+					return true;
+				}
+			}
+
+			// no more event till up is raised
+			return false;
+		}
+
+		if (!(keyCurrentState & _keyExtraMask)) {
+			_lastKeyRepeat = 0;
+
+		} else if (getMillis() >= (_keyExtraRepeat + _keyExtraDelay)) {
+			_keyExtraRepeat = getMillis();
+
+			if (gVars->arrowKeys) {
+				if (keyCurrentState & _keyExtra.bitLeft) {
+					_keyExtraPressed |= _keyExtra.bitLeft;
+					event.kbd.keycode = 276;
+
+				} else if (keyCurrentState & _keyExtra.bitRight) {
+					_keyExtraPressed |= _keyExtra.bitRight;
+					event.kbd.keycode = 275;
+
+				} else if (keyCurrentState & _keyExtra.bitUp) {
+					_keyExtraPressed |= _keyExtra.bitUp;
+					event.kbd.keycode = 273;
+
+				} else if (keyCurrentState & _keyExtra.bitDown) {
+					_keyExtraPressed |= _keyExtra.bitDown;
+					event.kbd.keycode = 274;
+				}
+
+				event.type = Common::EVENT_KEYDOWN;
+				event.kbd.ascii = event.kbd.keycode;
+				event.kbd.flags = 0;
+				return true;
+
+			} else {
+				Int8 sx = 0;
+				Int8 sy = 0;
+
+				if (keyCurrentState & _keyExtra.bitUp)
+					sy = -1;
+				else if (keyCurrentState & _keyExtra.bitDown)
+					sy = +1;
+					
+				if (keyCurrentState & _keyExtra.bitLeft)
+					sx = -1;
+				else if (keyCurrentState & _keyExtra.bitRight)
+					sx = +1;
+
+				if (sx || sy) {
+					simulate_mouse(event, sx, sy, &x, &y);
+					event.type = Common::EVENT_MOUSEMOVE;
+					event.mouse.x = x;
+					event.mouse.y = y;
+					warpMouse(x, y);
+
+					return true;
+				}			
+			}
+		}
+
 #if defined(COMPILE_OS5) && defined(PALMOS_ARM)
 		SysEventGet(&ev, evtNoWait);
 #else
 		EvtGetEvent(&ev, evtNoWait);
 #endif
-		// check for hardkey repeat for mouse emulation
-		keyCurrentState = KeyCurrentState();
-		// check_hard_keys();
 
-		if (!(keyCurrentState & _keyMouseMask)) {
-			_lastKeyRepeat = 0;
-		} else {
-			if (getMillis() >= (_keyMouseRepeat + _keyMouseDelay)) {
-				_keyMouseRepeat = getMillis();
+		if (ev.eType == keyUpEvent) {
+			int k = 0;
+			switch (ev.data.keyUp.chr) {
 
-				if (gVars->arrowKeys) {				
-					if (keyCurrentState & _keyMouse.bitUp)
-						event.kbd.keycode = 273;
-					else if (keyCurrentState & _keyMouse.bitDown)
-						event.kbd.keycode = 274;
-					else if (keyCurrentState & _keyMouse.bitLeft)
-						event.kbd.keycode = 276;
-					else if (keyCurrentState & _keyMouse.bitRight)
-						event.kbd.keycode = 275;
-					else if (keyCurrentState & _keyMouse.bitButLeft)
-						event.kbd.keycode = chrLineFeed;
+			// arrow keys
+			case chrUpArrow:
+				k = 273; break;
+			case chrDownArrow:
+				k = 274; break;
+			case chrRightArrow:
+				k = 275; break;
+			case chrLeftArrow:
+				k = 276; break;
+			}
 
-					event.type = EVENT_KEYDOWN;
-					event.kbd.ascii = event.kbd.keycode;
-					event.kbd.flags = 0;
-
-				} else {
-					Int8 sx = 0;
-					Int8 sy = 0;
-
-					if (keyCurrentState & _keyMouse.bitUp)
-						sy = -1;
-					else if (keyCurrentState & _keyMouse.bitDown)
-						sy = +1;
-						
-					if (keyCurrentState & _keyMouse.bitLeft)
-						sx = -1;
-					else if (keyCurrentState & _keyMouse.bitRight)
-						sx = +1;
-
-					if (sx || sy) {
-						simulate_mouse(event, sx, sy, &x, &y);
-						event.type = EVENT_MOUSEMOVE;
-						_lastKey = kKeyMouseMove;
-
-					} else {			
-						x = _mouseCurState.x;
-						y = _mouseCurState.y;
-
-						if (keyCurrentState & _keyMouse.bitButLeft) {
-							event.type = EVENT_LBUTTONDOWN;
-							_lastKey = kKeyMouseLButton;
-
-						} else if (_lastKey == kKeyMouseLButton) {
-							event.type = EVENT_LBUTTONUP;
-							_lastKey = kKeyNone;
-						}
-					}
-
-					event.mouse.x = x;
-					event.mouse.y = y;
-					warpMouse(x, y);
-		//			updateCD();
-				}
+			if (k) {
+				event.type = Common::EVENT_KEYUP;
+				event.kbd.keycode = k;
+				event.kbd.ascii = k;
+				event.kbd.flags = 0;
 				return true;
 			}
-		}
 
-		if (ev.eType == keyDownEvent) {
+		} else if (ev.eType == keyDownEvent) {
+			int k = 0;
 			switch (ev.data.keyDown.chr) {
 			// ESC key
 			case vchrLaunch:
-				_lastKey = kKeyNone;
-				event.type = EVENT_KEYDOWN;
+				event.type = Common::EVENT_KEYDOWN;
 				event.kbd.keycode = 27;
 				event.kbd.ascii = 27;
 				event.kbd.flags = 0;
@@ -177,8 +268,7 @@ bool OSystem_PalmBase::pollEvent(Event &event) {
 
 			// F5 = menu
 			case vchrMenu:
-				_lastKey = kKeyNone;
-				event.type = EVENT_KEYDOWN;
+				event.type = Common::EVENT_KEYDOWN;
 				event.kbd.keycode = 319;
 				event.kbd.ascii = 319;
 				event.kbd.flags = 0;
@@ -192,14 +282,30 @@ bool OSystem_PalmBase::pollEvent(Event &event) {
 //			case vchrBrightness:	// volume control on Zodiac, let other backends disable it
 			case vchrContrast:
 				// do nothing
-				_lastKey = kKeyNone;
+				return true;
+
+			// arrow keys
+			case chrUpArrow:
+				k = 273; break;
+			case chrDownArrow:
+				k = 274; break;
+			case chrRightArrow:
+				k = 275; break;
+			case chrLeftArrow:
+				k = 276; break;
+			}
+
+			if (k) {
+				event.type = Common::EVENT_KEYDOWN;
+				event.kbd.keycode = k;
+				event.kbd.ascii = k;
+				event.kbd.flags = 0;
 				return true;
 			}
 		}
 
 		if (check_event(event, &ev))
 			return true;
-		_lastKey = kKeyNone;
 
 		// prevent crash when alarm is raised
 		handled = ((ev.eType == keyDownEvent) && 
@@ -219,11 +325,10 @@ bool OSystem_PalmBase::pollEvent(Event &event) {
 			if (y > _screenHeight || y < 0 || x > _screenWidth || x < 0)
 				return false;
 
-			if (_lastEvent != penMoveEvent && (abs(y - event.mouse.y) <= 2 || abs(x - event.mouse.x) <= 2)) // move only if
+			if (abs(y - event.mouse.y) <= 2 || abs(x - event.mouse.x) <= 2)
 				return false;
 
-			_lastEvent = penMoveEvent;
-			event.type = EVENT_MOUSEMOVE;
+			event.type = Common::EVENT_MOUSEMOVE;
 			event.mouse.x = x;
 			event.mouse.y = y;
 			warpMouse(x, y);
@@ -238,21 +343,20 @@ bool OSystem_PalmBase::pollEvent(Event &event) {
 				num += 9 -
 						(3 - (3 * x / _screenWidth )) -
 						(3 * (3 * y / _screenHeight));
-			
-				event.type = EVENT_KEYDOWN;
+
+				event.type = Common::EVENT_KEYDOWN;
 				event.kbd.keycode = num;
 				event.kbd.ascii = num;
 				event.kbd.flags = 0;
 
-				_lastEvent = keyDownEvent;
+				_wasKey = true;
 				return true;
 			}
 
-			_lastEvent = penDownEvent;				
 			if (y > _screenHeight || y < 0 || x > _screenWidth || x < 0)
 				return false;
 
-			event.type = ((gVars->stylusClick || _overlayVisible) ? EVENT_LBUTTONDOWN : EVENT_MOUSEMOVE);
+			event.type = ((gVars->stylusClick || _overlayVisible) ? Common::EVENT_LBUTTONDOWN : Common::EVENT_MOUSEMOVE);
 			event.mouse.x = x;
 			event.mouse.y = y;
 			warpMouse(x, y);
@@ -261,7 +365,7 @@ bool OSystem_PalmBase::pollEvent(Event &event) {
 		case penUpEvent:
 			get_coordinates(&ev, x, y);
 
-			event.type = ((gVars->stylusClick || _overlayVisible) ? EVENT_LBUTTONUP : EVENT_MOUSEMOVE);
+			event.type = ((gVars->stylusClick || _overlayVisible) ? Common::EVENT_LBUTTONUP : Common::EVENT_MOUSEMOVE);
 			if (y > _screenHeight || y < 0 || x > _screenWidth || x < 0)
 				return false;
 
@@ -269,7 +373,7 @@ bool OSystem_PalmBase::pollEvent(Event &event) {
 			event.mouse.y = y;
 			warpMouse(x, y);
 			return true;
-		
+
 		case keyDownEvent:
 			if (ev.data.keyDown.chr == vchrCommand &&
 				(ev.data.keyDown.modifiers & commandKeyMask)) {
@@ -290,15 +394,15 @@ bool OSystem_PalmBase::pollEvent(Event &event) {
 
 			if (_lastKeyModifier == kModifierNone) {
 				// for keyboard mode
-				if (ev.data.keyDown.modifiers & shiftKeyMask)	mask |= KBD_SHIFT;
-				if (ev.data.keyDown.modifiers & controlKeyMask)	mask |= KBD_CTRL;
-				if (ev.data.keyDown.modifiers & optionKeyMask)	mask |= KBD_ALT;
-				if (ev.data.keyDown.modifiers & commandKeyMask) mask |= KBD_CTRL|KBD_ALT;
+				if (ev.data.keyDown.modifiers & shiftKeyMask)	mask |= Common::KBD_SHIFT;
+				if (ev.data.keyDown.modifiers & controlKeyMask)	mask |= Common::KBD_CTRL;
+				if (ev.data.keyDown.modifiers & optionKeyMask)	mask |= Common::KBD_ALT;
+				if (ev.data.keyDown.modifiers & commandKeyMask) mask |= Common::KBD_CTRL|Common::KBD_ALT;
 			} else {
 				// for grafiti mode
-				if (_lastKeyModifier == kModifierCommand)	mask = KBD_CTRL|KBD_ALT;
-				if (_lastKeyModifier == kModifierAlt)		mask = KBD_ALT;
-				if (_lastKeyModifier == kModifierCtrl)		mask = KBD_CTRL;
+				if (_lastKeyModifier == kModifierCommand)	mask = Common::KBD_CTRL|Common::KBD_ALT;
+				if (_lastKeyModifier == kModifierAlt)		mask = Common::KBD_ALT;
+				if (_lastKeyModifier == kModifierCtrl)		mask = Common::KBD_CTRL;
 			}
 
 			if (_lastKeyModifier)
@@ -306,40 +410,40 @@ bool OSystem_PalmBase::pollEvent(Event &event) {
 			_lastKeyModifier = kModifierNone;
 
 			// F1 -> F10 key
-			if  (key >= '0' && key <= '9' && mask == (KBD_CTRL|KBD_ALT)) {
+			if  (key >= '0' && key <= '9' && mask == (Common::KBD_CTRL|Common::KBD_ALT)) {
 				key = (key == '0') ? 324 : (315 + key - '1');
 				mask = 0;
 
 #ifdef STDLIB_TRACE_MEMORY
 			// print memory
-			} else if  (key == 'm' && mask == (KBD_CTRL|KBD_ALT)) {
+			} else if  (key == 'm' && mask == (Common::KBD_CTRL|Common::KBD_ALT)) {
 				printf("Used memory: %d\n", __stdlib_trace_memory);
 #endif
 			// exit
-			} else if  ((key == 'z' && mask == KBD_CTRL) || (mask == KBD_ALT && key == 'x')) {
-				event.type = EVENT_QUIT;
+			} else if  ((key == 'z' && mask == Common::KBD_CTRL) || (mask == Common::KBD_ALT && key == 'x')) {
+				event.type = Common::EVENT_QUIT;
 				return true;
-			
+
 			// num pad (indy fight mode)
-			} else if (key == 'n' && mask == (KBD_CTRL|KBD_ALT) && !_overlayVisible) {
+			} else if (key == 'n' && mask == (Common::KBD_CTRL|Common::KBD_ALT) && !_overlayVisible) {
 				_useNumPad = !_useNumPad;
 				draw_osd(kDrawFight, _screenDest.w - 34, _screenDest.h + 2, _useNumPad, 1);
 				displayMessageOnOSD(_useNumPad ? "Fight mode on." : "Fight mode off.");
 				return false;
 			}
-			
+
 			// other keys
-			_lastEvent = keyDownEvent;
-			event.type = EVENT_KEYDOWN;
+			_wasKey = true;
+			event.type = Common::EVENT_KEYDOWN;
 			event.kbd.keycode = key;
 			event.kbd.ascii = key;
 			event.kbd.flags = mask;
 			return true;
 
 		default:
-			if (_lastEvent == keyDownEvent) {
-				event.type = EVENT_KEYUP;
-				_lastEvent = nilEvent;
+			if (_wasKey && ev.eType != keyHoldEvent) {
+				event.type = Common::EVENT_KEYUP;
+				_wasKey = false;
 				return true;
 			}
 			return false;
