@@ -1,7 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2001  Ludvig Strigeus
- * Copyright (C) 2001-2006 The ScummVM project
- * Copyright (C) 2002-2006 Chris Apers - PalmOS Backend
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -17,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/backends/platform/PalmOS/Src/be_os5ex.h $
- * $Id: be_os5ex.h 22911 2006-06-04 12:12:24Z sev $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/backends/platform/PalmOS/Src/be_os5ex.h $
+ * $Id: be_os5ex.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -45,9 +46,9 @@ typedef struct {
 
 class OSystem_PalmOS5Ex : public OSystem_PalmOS5 {
 private:
-//	void timer_handler() {};
-	void sound_handler() {};
-//	void int_quit();
+	void timer_handler() {}
+	void sound_handler() {}
+	void int_quit();
 
 	SndStreamVariableBufferCallback sound_callback();
 
@@ -55,7 +56,7 @@ public:
 	OSystem_PalmOS5Ex();
 	static OSystem *create();
 
-//	void setTimerCallback(TimerProc callback, int interval);
+	void setTimerCallback(TimerProc callback, int interval);
 
 	MutexRef createMutex();
 	void lockMutex(MutexRef mutex);
