@@ -1,7 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2001  Ludvig Strigeus
- * Copyright (C) 2001-2006 The ScummVM project
- * Copyright (C) 2002-2006 Chris Apers - PalmOS Backend
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -17,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/backends/platform/PalmOS/Src/be_zodiac.cpp $
- * $Id: be_zodiac.cpp 22911 2006-06-04 12:12:24Z sev $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/backends/platform/PalmOS/Src/be_zodiac.cpp $
+ * $Id: be_zodiac.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -33,39 +34,22 @@ OSystem_PalmZodiac::OSystem_PalmZodiac() : OSystem_PalmOS5Ex() {
 }
 
 void OSystem_PalmZodiac::int_initBackend() {
-	_keyMouse.bitUp		= keyBitRockerUp;
-	_keyMouse.bitDown	= keyBitRockerDown;
-	_keyMouse.bitLeft	= keyBitRockerLeft;
-	_keyMouse.bitRight	= keyBitRockerRight;
-	_keyMouse.bitButLeft= keyBitRockerCenter;
-	_keyMouse.hasMore	= true;
+	_keyExtra.bitUp			= keyBitRockerUp;
+	_keyExtra.bitDown		= keyBitRockerDown;
+	_keyExtra.bitLeft		= keyBitRockerLeft;
+	_keyExtra.bitRight		= keyBitRockerRight;
+	
+//	_keyExtra.bitActionA	= keyBitActionD;
+//	_keyExtra.bitActionB	= keyBitActionB;
 }
 
 void OSystem_PalmZodiac::calc_rect(Boolean fullscreen) {
-	Int32 w, h;
-	
-	if (fullscreen) {
-		w = (_ratio.adjustAspect == kRatioWidth) ? _ratio.width : gVars->screenFullWidth;
-		h = (_ratio.adjustAspect == kRatioHeight) ? _ratio.height : gVars->screenFullHeight;
-
-		_screenOffset.x = (_ratio.adjustAspect == kRatioWidth) ? (gVars->screenFullWidth - _ratio.width) / 2 : 0;
-		_screenOffset.y = (_ratio.adjustAspect == kRatioHeight) ? (gVars->screenFullHeight - _ratio.height) / 2 : 0;
-
-	} else {
-		w = gVars->screenWidth;
-		h = gVars->screenHeight - MIN_OFFSET * 2;
-
-		_screenOffset.x = 0;
-		_screenOffset.y = MIN_OFFSET;		
-	}
+	OSystem_PalmOS5::calc_rect(fullscreen);
 
 	_dstRect.x = _screenOffset.x;
 	_dstRect.y = _screenOffset.y;
-	_dstRect.w = w;
-	_dstRect.h = h;
-	
-	_screenDest.w = _dstRect.w;
-	_screenDest.h = _dstRect.h;
+	_dstRect.w = _screenDest.w;
+	_dstRect.h = _screenDest.h;
 }
 
 void OSystem_PalmZodiac::setFeatureState(Feature f, bool enable) {
