@@ -1,6 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2001  Ludvig Strigeus
- * Copyright (C) 2001-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -16,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/backends/platform/PalmOS/Src/cdaudio.h $
- * $Id: cdaudio.h 20535 2006-02-11 12:47:47Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/backends/platform/PalmOS/Src/cdaudio.h $
+ * $Id: cdaudio.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
  
@@ -54,8 +56,8 @@ public:
 	virtual void setVolume(int volume) { _volumeLevel = volume; }
 	virtual int getVolume() const { return _volumeLevel; }
 
-	virtual void upVolume(int value) {};
-	virtual void downVolume(int value) {};
+	virtual void upVolume(int value) {}
+	virtual void downVolume(int value) {}
 
 protected:
 	int _volumeLevel;
