@@ -1,7 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2001  Ludvig Strigeus
- * Copyright (C) 2001-2006 The ScummVM project
- * Copyright (C) 2002-2006 Chris Apers - PalmOS Backend
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -17,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/backends/platform/PalmOS/Src/launcher/forms/formCards.cpp $
- * $Id: formCards.cpp 20931 2006-02-26 16:36:51Z chrilith $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/backends/platform/PalmOS/Src/launcher/forms/formCards.cpp $
+ * $Id: formCards.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -191,6 +192,7 @@ static void ConfigTabInit(Boolean update = false) {
 		FldSetTextHandle(fld1P, cacheH);
 		CtlSetValue((ControlType *)GetObjectPtr(TabCardConfigCacheCheckbox), gPrefs->card.useCache);
 		CtlSetValue((ControlType *)GetObjectPtr(TabCardConfigLedCheckbox), gPrefs->card.showLED);
+		CtlSetValue((ControlType *)GetObjectPtr(TabCardConfigDetectCheckbox), gPrefs->card.autoDetect);
 	// update ? redraw the list
 	} else {
 		WinScreenLock(winLockCopy);
@@ -200,7 +202,7 @@ static void ConfigTabInit(Boolean update = false) {
 }
 
 static UInt16 ConfigTabSave() {
-	ControlType *cckP[2];
+	ControlType *cckP[3];
 	FieldType *fld1P;
 	ListPtr listP;
 	FormPtr frmP;
@@ -208,9 +210,11 @@ static UInt16 ConfigTabSave() {
 
 	cckP[0] = (ControlType *)GetObjectPtr(TabCardConfigCacheCheckbox);
 	cckP[1] = (ControlType *)GetObjectPtr(TabCardConfigLedCheckbox);
+	cckP[2] = (ControlType *)GetObjectPtr(TabCardConfigDetectCheckbox);
 
 	gPrefs->card.useCache = CtlGetValue(cckP[0]);
 	gPrefs->card.showLED = CtlGetValue(cckP[1]);
+	gPrefs->card.autoDetect = CtlGetValue(cckP[2]);
 
 	fld1P = (FieldType *)GetObjectPtr(TabCardConfigCacheSizeField);
 	frmP = FrmGetActiveForm();
@@ -361,6 +365,7 @@ void CardSlotCreateDirs() {
 		VFSDirCreate(gPrefs->card.volRefNum, "/PALM/Programs/ScummVM/Saved");
 		VFSDirCreate(gPrefs->card.volRefNum, "/PALM/Programs/ScummVM/Audio");
 		VFSDirCreate(gPrefs->card.volRefNum, "/PALM/Programs/ScummVM/Mods");
+		VFSDirCreate(gPrefs->card.volRefNum, "/PALM/Programs/ScummVM/Themes");
 	}
 }
 
