@@ -1,7 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2001  Ludvig Strigeus
- * Copyright (C) 2001-2006 The ScummVM project
- * Copyright (C) 2002-2006 Chris Apers - PalmOS Backend
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -17,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/backends/platform/PalmOS/Src/launcher/forms/formmain.cpp $
- * $Id: formmain.cpp 20931 2006-02-26 16:36:51Z chrilith $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/backends/platform/PalmOS/Src/launcher/forms/formmain.cpp $
+ * $Id: formmain.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -299,8 +300,10 @@ Boolean MainFormHandleEvent(EventPtr eventP)
 						case skinButtonGameStart:
 							if (gPrefs->card.volRefNum == vfsInvalidVolRef)
 								FrmCustomAlert(FrmWarnAlert,"Please select/insert a memory card.", 0, 0);
+							else if (GamGetSelected() != dmMaxRecordIndex)
+								StartScummVM();
 							else
-								bStartScumm = true;
+								FrmPopupForm(EngineForm);
 							handled = true;
 							break;
 
