@@ -1,7 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2001  Ludvig Strigeus
- * Copyright (C) 2001-2006 The ScummVM project
- * Copyright (C) 2002-2006 Chris Apers - PalmOS Backend
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -17,18 +18,18 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/backends/platform/PalmOS/Src/launcher/launch.cpp $
- * $Id: launch.cpp 22938 2006-06-05 19:17:30Z chrilith $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/backends/platform/PalmOS/Src/launcher/launch.cpp $
+ * $Id: launch.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
 #include <PalmOS.h>
+#include <PmPalmOSNVFS.h>
 #include "StarterRsc.h"
 
 #include "games.h"
 #include "start.h"
 #include "rumble.h"
-#include "extend.h"
 #include "globals.h"
 #include "features.h"
 #include "formUtil.h"
@@ -52,15 +53,20 @@
 	StrCat(filename, h);	\
 	StrCat(filename, m);
 
+#define FIND_FILE() \
+	if (*volRefNum == vfsInvalidVolRef) \
+		*volRefNum = ModFind(filename);
+
+
 #define CHECK_FILE(m) \
-	e = VFSFileOpen(volRefNum, filename, vfsModeRead, &file);	\
+	e = VFSFileOpen(*volRefNum, filename, vfsModeRead, &file);	\
 	if (e)	\
 		BUILD_ERROR(m)	\
 	else	\
 		VFSFileClose(file);
 
 #define IMPRT_FILE(m) \
-	e = VFSImportDatabaseFromFile(volRefNum, filename, &cardNo, &dbID);	\
+	e = VFSImportDatabaseFromFile(*volRefNum, filename, &cardNo, &dbID);	\
 	if (e)	\
 		BUILD_ERROR(m)
 
@@ -77,6 +83,22 @@ void ModDelete() {
 	DELET_FILE("ScummVM-Engine");
 }
 
+UInt16 ModFind(const Char *f) {
+	Err e;
+	UInt16 volRefNum;
+	FileRef r;
+	UInt32 volIterator = vfsIteratorStart|vfsIncludePrivateVolumes;
+	while (volIterator != vfsIteratorStop) {
+		e = VFSVolumeEnumerate(&volRefNum, &volIterator);
+
+		if (!e)	e = VFSFileOpen(volRefNum, f, vfsModeRead, &r);
+		if (!e) e = VFSFileClose(r);
+		if (!e) break;
+	}
+	
+	return volRefNum;
+}
+
 static void ModSetStack(UInt32 newSize, UInt16 cardNo, LocalID dbID) {
 	DmOpenRef dbRef = DmOpenDatabase(cardNo, dbID, dmModeReadWrite);
 
@@ -103,7 +125,7 @@ static void ModSetStack(UInt32 newSize, UInt16 cardNo, LocalID dbID) {
 	}
 }
 
-static Err ModImport(UInt16 volRefNum, UInt8 engine, Boolean *armP) {
+static Err ModImport(UInt16 *volRefNum, UInt8 engine, Boolean *armP) {
 #ifndef _DEBUG_ENGINE
 	char filename[256];
 	UInt16 cardNo;
@@ -124,6 +146,7 @@ static Err ModImport(UInt16 volRefNum, UInt8 engine, Boolean *armP) {
 #ifndef _DEBUG_ENGINE
 	// engine file ?
 	BUILD_FILE(engines[engine].fileP, ".engine");
+	FIND_FILE ();
 	CHECK_FILE("ScummVM engine file was not found !");
 	IMPRT_FILE("Cannot import engine file !");
 
@@ -132,6 +155,7 @@ static Err ModImport(UInt16 volRefNum, UInt8 engine, Boolean *armP) {
 	e = SysAppLaunch(cardNo, dbID, 0, sysAppLaunchCustomEngineGetInfo, 0, &result);
 	*armP = ((result & GET_MODEARM) == GET_MODEARM);
 
+/*	ARM ONLY FOR NOW, NOT REQUIRED
 	// common file ?
 	if (!e && (result & GET_DATACOMMON)) {
 		BUILD_FILE("common", ".data");
@@ -144,6 +168,7 @@ static Err ModImport(UInt16 volRefNum, UInt8 engine, Boolean *armP) {
 		CHECK_FILE("Engine data file was not found !");
 		IMPRT_FILE("Cannot import engine data file !");
 	}
+*/
 #endif
 	// if error, cleanup
 	if (e) ModDelete();
@@ -163,54 +188,23 @@ onError:
 #undef CHECK_FILE
 #undef BUILD_FILE
 
-Boolean StartScummVM() {
+Boolean StartScummVM(Int16 engine) {
 	Char **argvP;
 	UInt8 lightspeed, argc	= 0;
 	UInt32 stackSize;
 	Boolean toLauncher, direct, isARM;
-	UInt8 engine;
 	Char num[6];
 
-	UInt16 index = GamGetSelected();
-
 	argvP = ArgsInit();
 	direct = false;
 
 	// start command line (exec name)
 	ArgsAdd(&argvP[argc], "-", NULL, &argc);
 
+	UInt16 index = GamGetSelected();
 	// no game selected
 	if (index == dmMaxRecordIndex) {
-		ListPtr listP;
-		UInt16 whichButton;
-		
-		// init form
-		FormPtr frmP = FrmInitForm(EngineForm);
-		listP = (ListType *)FrmGetObjectPtr(frmP, FrmGetObjectIndex(frmP, EngineListList));
-		itemsText = (Char **)MemPtrNew(ENGINE_COUNT * sizeof(Char *));
-		
-		for (int i = 0; i < ENGINE_COUNT; i++)
-			itemsText[i] = (Char *)engines[i].nameP;
-			
-		LstSetListChoices (listP, itemsText, ENGINE_COUNT);
-		LstSetSelection(listP, 0);	
-		
-		whichButton = FrmDoDialog(frmP);
-		engine = LstGetSelection(listP);
-
-		FrmDeleteForm(frmP);
-		MemPtrFree(itemsText);
-		itemsText = NULL;
-		
-		if (whichButton == EngineCancelButton) {
-			if (bDirectMode) {
-				// and force exit if nothing selected
-				EventType event;
-				event.eType = keyDownEvent;
-				event.data.keyDown.chr = vchrLaunch;
-				event.data.keyDown.modifiers = commandKeyMask;
-				EvtAddUniqueEventToQueue(&event, 0, true);
-			}
+		if (engine == NO_ENGINE) {
 			// free args
 			ArgsFree(argvP);
 			return false;
@@ -219,6 +213,7 @@ Boolean StartScummVM() {
 		// default values
 		if (bDirectMode)
 			gPrefs->card.volRefNum = parseCards();	// always use the first removable card available (?)
+
 		gVars->filter		= true;
 		gVars->palmVolume	= 50;
 		gVars->fmQuality	= FM_QUALITY_INI;
@@ -303,6 +298,7 @@ Boolean StartScummVM() {
 				"3do",
 				"acorn",
 				"amiga",
+				"2gs",
 				"atari",
 				"c64",
 				"pc",
@@ -429,7 +425,11 @@ Boolean StartScummVM() {
 
 	// gVars values 
 	// (gVars->HRrefNum defined in checkHRmode on Cli)
+#ifndef _DEBUG_ENGINE
+	gVars->VFS.volRefNum	= (gPrefs->card.autoDetect ? vfsInvalidVolRef : gPrefs->card.volRefNum);
+#else
 	gVars->VFS.volRefNum	= gPrefs->card.volRefNum;
+#endif
 	gVars->vibrator			= gPrefs->vibrator;
 	gVars->stdPalette		= gPrefs->stdPalette;
 	gVars->VFS.cacheSize	= (gPrefs->card.useCache ? gPrefs->card.cacheSize : 0);
@@ -460,7 +460,7 @@ Boolean StartScummVM() {
 		HWR_SET(INIT_PA1LIB);
 	}
 */
-	if (ModImport(gVars->VFS.volRefNum, engine, &isARM) != errNone) {
+	if (ModImport(&gVars->VFS.volRefNum, engine, &isARM) != errNone) {
 		if (bDirectMode) {
 			// and force exit if nothing selected
 			EventType event;
