@@ -1,7 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2001  Ludvig Strigeus
- * Copyright (C) 2001-2006 The ScummVM project
- * Copyright (C) 2002-2006 Chris Apers - PalmOS Backend
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -17,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/backends/platform/PalmOS/Src/launcher/start.cpp $
- * $Id: start.cpp 20928 2006-02-26 16:30:21Z chrilith $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/backends/platform/PalmOS/Src/launcher/start.cpp $
+ * $Id: start.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -43,7 +44,6 @@ GlobalsPreferencePtr gPrefs;
 GlobalsDataPtr gVars;
 
 Boolean bDirectMode = false;
-Boolean bStartScumm = false;
 Boolean bLaunched	= false;
 
 /***********************************************************************
@@ -200,6 +200,10 @@ static Boolean AppHandleEvent(EventPtr eventP) {
 				FrmSetEventHandler(frmP, InfoFormHandleEvent);
 				break;
 
+			case EngineForm:
+				FrmSetEventHandler(frmP, SelectorFormHandleEvent);
+				break;
+
 			default:
 //				ErrFatalDisplay("Invalid Form Load Event");
 				break;
@@ -231,9 +235,6 @@ static void AppEventLoop(void) {
 	do {
 		EvtGetEvent(&event, evtNoWait);
 
-		if(bStartScumm)
-			bStartScumm = StartScummVM();
-
 		if (! SysHandleEvent(&event))
 			if (! MenuHandleEvent(0, &event, &error))
 				if (! AppHandleEvent(&event))
@@ -330,7 +331,7 @@ static UInt32 ScummVMPalmMain(UInt16 cmd, MemPtr cmdPBP, UInt16 launchFlags) {
 				FrmGotoForm(MainForm);
 			}else {
 				GamUnselect();
-				bStartScumm = true;
+				FrmGotoForm(EngineForm);
 			}
 
 			AppEventLoop();
