@@ -1,7 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2001  Ludvig Strigeus
- * Copyright (C) 2001-2006 The ScummVM project
- * Copyright (C) 2002-2006 Chris Apers - PalmOS Backend
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -17,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/backends/platform/PalmOS/Src/launcher/start.h $
- * $Id: start.h 22938 2006-06-05 19:17:30Z chrilith $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/backends/platform/PalmOS/Src/launcher/start.h $
+ * $Id: start.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
  
@@ -53,6 +54,7 @@ typedef	struct {
 		UInt32 cacheSize;
 		Boolean useCache;
 		Boolean showLED;
+		Boolean autoDetect;
 		Boolean moveDB;
 		Boolean deleteDB;
 		Boolean confirmMoveDB;
@@ -78,7 +80,6 @@ typedef	struct {
 extern GlobalsPreferencePtr gPrefs;
 
 extern Boolean bDirectMode;
-extern Boolean bStartScumm;
 extern Boolean bLaunched;
 
 #define appPrefID				0x00
@@ -91,7 +92,7 @@ extern Boolean bLaunched;
 
 Err AppStart(void);
 void AppStop(void);
-Boolean StartScummVM();
+Boolean StartScummVM(Int16 engine = -1);
 void SavePrefs();
 Err SendDatabase (UInt16 cardNo, LocalID dbID, Char *nameP, Char *descriptionP);
 #endif
