@@ -1,7 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2001  Ludvig Strigeus
- * Copyright (C) 2001-2006 The ScummVM project
- * Copyright (C) 2002-2006 Chris Apers - PalmOS Backend
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -17,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/backends/platform/PalmOS/Src/native/oscalls.cpp $
- * $Id: oscalls.cpp 22911 2006-06-04 12:12:24Z sev $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/backends/platform/PalmOS/Src/native/oscalls.cpp $
+ * $Id: oscalls.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -29,19 +30,33 @@ GlobalsType global;
 
 
 PACE_CLASS_WRAPPER(Err)
-	StatShow_68k(void) {
+	__68k_StatShow(void) {
 	PACE_PIN_EXEC_NP(pinStatShow, Err)
 }
 
 PACE_CLASS_WRAPPER(Err)
-	StatHide_68k(void) {
+	__68k_StatHide(void) {
 	PACE_PIN_EXEC_NP(pinStatHide, Err)
 }
 
 PACE_CLASS_WRAPPER(Err)
-	PINSetInputAreaState_68k(UInt16 state) {
+	__68k_PINSetInputAreaState(UInt16 state) {
 	PACE_PARAMS_INIT()
 	PACE_PARAMS_ADD16(state)
 	PACE_PARAMS_END()
 	PACE_PIN_EXEC(pinPINSetInputAreaState, Err)
 }
+
+PACE_CLASS_WRAPPER(Err)
+	__68k_SysSetOrientation(UInt16 orientation) {
+	PACE_PARAMS_INIT()
+	PACE_PARAMS_ADD16(orientation)
+	PACE_PARAMS_END()
+	PACE_PIN_EXEC(pinSysSetOrientation, Err)
+}
+
+PACE_CLASS_WRAPPER(UInt16)
+	__68k_SysGetOrientation(void) {
+	PACE_PIN_EXEC_NP(pinSysGetOrientation, UInt16)
+}
+
