@@ -1,7 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2001  Ludvig Strigeus
- * Copyright (C) 2001-2006 The ScummVM project
- * Copyright (C) 2002-2006 Chris Apers - PalmOS Backend
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -17,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/backends/platform/PalmOS/Src/native/zodiacStartup.cpp $
- * $Id: zodiacStartup.cpp 22911 2006-06-04 12:12:24Z sev $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/backends/platform/PalmOS/Src/native/zodiacStartup.cpp $
+ * $Id: zodiacStartup.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -56,6 +57,7 @@ UInt32 __ARMlet_Startup__(const void*, void*, Call68KFuncType*);
  * mark the start and end of the various data sections.
  */
 extern long __DataStart__[];
+extern long __sinit__[];
 extern long __RODataStart__[];
 extern long __BSSStart__[];
 extern long __BSSEnd__[];
@@ -64,6 +66,9 @@ extern long __CodeRelocEnd__[];
 extern long __DataRelocStart__[];
 extern long __DataRelocEnd__[];
 
+/*	simple function pointer	*/
+typedef void (*StaticInitializer)(void);
+
 /*
  * This function performs relocation for Tapwave Native Application.
  */
@@ -73,6 +78,14 @@ static void relocate(void)
     long base = (long) __ARMlet_Startup__;
     long *cur, *end;
 
+	// handle static initializers
+	if (__sinit__) {
+		long s, *p;
+
+		for (p = __sinit__; p && (s = *p) != 0; p++)
+			((StaticInitializer)(s + base))();
+	}
+
     // handle code-to-data relocation
     cur = __CodeRelocStart__;
     end = __CodeRelocEnd__;
@@ -88,31 +101,6 @@ static void relocate(void)
     }
 }
 
-/*
- * The following functions provide malloc/free support to Metrowerks
- * Standard Library (MSL). This feature requires the MSL library be
- * built with _MSL_OS_DIRECT_MALLOC enabled.
- */
-void*
-__sys_alloc(size_t size)
-{
-    void * ptr = MemPtrNew(size);
-    ErrFatalDisplayIf(ptr == NULL, "out of memory");
-    return ptr;
-}
-
-void
-__sys_free(void* ptr)
-{
-    (void) MemPtrFree(ptr);
-}
-
-size_t
-__sys_pointer_size(void* ptr)
-{
-    return (size_t) MemPtrSize(ptr);
-}
-
 /*
  * This is the real entrypoint for Tapwave Native Application. It
  * depends on various CodeWarrior 9.2 compiler/linker/runtime features.
