@@ -3,7 +3,7 @@
 
 #include "native_common.h"
 #undef DISABLE_SCUMM
-//#undef DISABLE_SCUMM_7_8
-//#undef DISABLE_HE
+#undef DISABLE_SCUMM_7_8
+#undef DISABLE_HE
 
 #endif
