@@ -1,6 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2001  Ludvig Strigeus
- * Copyright (C) 2001-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -16,14 +18,14 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/base/commandLine.cpp $
- * $Id: commandLine.cpp 23109 2006-06-14 19:33:17Z eriktorbjorn $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/base/commandLine.cpp $
+ * $Id: commandLine.cpp 27355 2007-06-11 08:41:26Z sev $
  *
  */
 
 #include "common/stdafx.h"
 
-#include "base/engine.h"
+#include "engines/engine.h"
 #include "base/commandLine.h"
 #include "base/plugins.h"
 #include "base/version.h"
@@ -35,23 +37,19 @@
 #include "sound/mixer.h"
 
 #ifdef UNIX
-#include <errno.h>
-#include <sys/stat.h>
 #ifdef MACOSX
 #define DEFAULT_SAVE_PATH "Documents/ScummVM Savegames"
 #else
 #define DEFAULT_SAVE_PATH ".scummvm"
 #endif
 #elif defined(__SYMBIAN32__)
-#include <errno.h>
-#include <sys/stat.h>
 #define DEFAULT_SAVE_PATH "Savegames"
 #endif
 
 #define DETECTOR_TESTING_HACK
 
 #ifdef DETECTOR_TESTING_HACK
-#include "backends/fs/fs.h"
+#include "common/fs.h"
 #endif
 
 namespace Base {
@@ -80,9 +78,9 @@ static const char HELP_STRING[] =
 	"  -x, --save-slot[=NUM]    Save game slot to load (default: autosave)\n"
 	"  -f, --fullscreen         Force full-screen mode\n"
 	"  -F, --no-fullscreen      Force windowed mode\n"
-	"  -g, --gfx-mode=MODE      Select graphics scaler (normal,2x,3x,2xsai,\n"
-	"                           super2xsai,supereagle,advmame2x,advmame3x,hq2x,\n"
-	"                           hq3x,tv2x,dotmatrix)\n"
+	"  -g, --gfx-mode=MODE      Select graphics scaler (1x,2x,3x,2xsai,super2xsai,\n"
+	"                           supereagle,advmame2x,advmame3x,hq2x,hq3x,tv2x,\n"
+	"                           dotmatrix)\n"
 	"  --gui-theme=THEME        Select GUI theme (default, modern, classic)\n"
 	"  --themepath=PATH         Path to where GUI themes are stored\n"
 	"  -e, --music-driver=MODE  Select music driver (see README for details)\n"
@@ -92,7 +90,7 @@ static const char HELP_STRING[] =
 	"  -s, --sfx-volume=NUM     Set the sfx volume, 0-255 (default: 192)\n"
 	"  -r, --speech-volume=NUM  Set the speech volume, 0-255 (default: 192)\n"
 	"  --midi-gain=NUM          Set the gain for MIDI playback, 0-1000 (default:\n"
-        "                           100) (only supported by some MIDI drivers)\n"
+	"                           100) (only supported by some MIDI drivers)\n"
 	"  -n, --subtitles          Enable subtitles (use with games that have voice)\n"
 	"  -b, --boot-param=NUM     Pass number to the boot script (boot param)\n"
 	"  -d, --debuglevel=NUM     Set debug verbosity level\n"
@@ -105,9 +103,9 @@ static const char HELP_STRING[] =
 	"                           drive)\n"
 	"  --joystick[=NUM]         Enable input with joystick (default: 0 = first\n"
 	"                           joystick)\n"
-	"  --platform=WORD          Specify platform of game (allowed values: 3do, acorn,\n"
-	"                           amiga, atari, c64, fmtowns, nes, mac, pc, segacd,\n"
-	"                           windows)\n"
+	"  --platform=WORD          Specify platform of game (allowed values: 2gs, 3do,\n"
+	"                           acorn, amiga, atari, c64, fmtowns, nes, mac, pc,\n"
+	"                           pce, segacd, windows)\n"
 	"  --savepath=PATH          Path to where savegames are stored\n"
 	"  --soundfont=FILE         Select the SoundFont for MIDI playback (only\n"
 	"                           supported by some MIDI drivers)\n"
@@ -137,11 +135,7 @@ static const char HELP_STRING[] =
 ;
 #endif
 
-#if !(defined(PALMOS_ARM) || defined(PALMOS_DEBUG) || defined(__GP32__) || defined (__SYMBIAN32__))
-static Common::String s_appName("scummvm");
-#else
 static const char *s_appName = "scummvm";
-#endif
 
 static void usage(const char *s, ...) GCC_PRINTF(1, 2);
 
@@ -154,7 +148,7 @@ static void usage(const char *s, ...) {
 	va_end(va);
 
 #if !(defined(PALMOS_ARM) || defined(PALMOS_DEBUG) || defined(__GP32__) || defined (__SYMBIAN32__))
-	printf(USAGE_STRING, s_appName.c_str(), buf, s_appName.c_str(), s_appName.c_str());
+	printf(USAGE_STRING, s_appName, buf, s_appName, s_appName);
 #endif
 	exit(1);
 }
@@ -221,36 +215,19 @@ void registerDefaults() {
 	ConfMan.registerDefault("alsa_port", "65:0");
 #endif
 
+	// Register default savepath
 #ifdef DEFAULT_SAVE_PATH
 	char savePath[MAXPATHLEN];
 #ifdef UNIX
-	struct stat sb;
 	const char *home = getenv("HOME");
 	if (home && *home && strlen(home) < MAXPATHLEN) {
 		snprintf(savePath, MAXPATHLEN, "%s/%s", home, DEFAULT_SAVE_PATH);
-		if (stat(savePath, &sb) == -1) {
-			/* create the dir if it does not exist */
-			if (errno == ENOENT) {
-				if (mkdir(savePath, 0755) != 0) {
-					perror("mkdir");
-					exit(1);
-				}
-			}
-		}
-		/* check that the dir is there */
-		if (stat(savePath, &sb) == 0) {
-			ConfMan.registerDefault("savepath", savePath);
-		}
+		ConfMan.registerDefault("savepath", savePath);
 	}
 #elif defined(__SYMBIAN32__)
 	strcpy(savePath, Symbian::GetExecutablePath());
 	strcat(savePath, DEFAULT_SAVE_PATH);
-	struct stat sb;
-	if (stat(savePath, &sb) == -1)
-		if (errno == ENOENT)// create the dir if it does not exist
-			if (mkdir(savePath, 0755) != 0)
-				warning("mkdir for '%s' failed!", savePath);
-	ConfMan.registerDefault("savepath", savePath); // this should be enough...
+	ConfMan.registerDefault("savepath", savePath);
 #endif
 #endif // #ifdef DEFAULT_SAVE_PATH
 }
@@ -261,24 +238,26 @@ void registerDefaults() {
 
 // Use this for options which have an *optional* value
 #define DO_OPTION_OPT(shortCmd, longCmd, defaultVal) \
-	if (isLongCmd ? (!memcmp(s+2, longCmd"=", sizeof(longCmd"=") - 1)) : (tolower(s[1]) == shortCmd)) { \
+	if (isLongCmd ? (!strcmp(s+2, longCmd) || !memcmp(s+2, longCmd"=", sizeof(longCmd"=") - 1)) : (tolower(s[1]) == shortCmd)) { \
 		s += 2; \
-		if (isLongCmd) \
-			s += sizeof(longCmd"=") - 1; \
+		if (isLongCmd) { \
+			s += sizeof(longCmd) - 1; \
+			if (*s == '=') \
+				s++; \
+		} \
 		const char *option = s; \
-		if (*s == '\0') { option = s2; i++; } \
-		if (!option) option = defaultVal; \
+		if (*s == '\0' && !isLongCmd) { option = s2; i++; } \
+		if (!option || *option == '\0') option = defaultVal; \
 		if (option) settings[longCmd] = option;
 
 // Use this for options which have a required (string) value
 #define DO_OPTION(shortCmd, longCmd) \
 	DO_OPTION_OPT(shortCmd, longCmd, 0) \
-	if (!option) usage("Option '%s' requires an argument", argv[i]);
+	if (!option) usage("Option '%s' requires an argument", argv[isLongCmd ? i : i-1]);
 
 // Use this for options which have a required integer value
 #define DO_OPTION_INT(shortCmd, longCmd) \
-	DO_OPTION_OPT(shortCmd, longCmd, 0) \
-	if (!option) usage("Option '%s' requires an argument", argv[i]); \
+	DO_OPTION(shortCmd, longCmd) \
 	char *endptr = 0; \
 	int intValue; intValue = (int)strtol(option, &endptr, 0); \
 	if (endptr == NULL || *endptr != 0) usage("--%s: Invalid number '%s'", longCmd, option);
@@ -438,6 +417,8 @@ Common::String parseCommandLine(Common::StringMap &settings, int argc, char **ar
 			END_OPTION
 
 			DO_LONG_OPTION_OPT("joystick", "0")
+				settings["joystick_num"] = option;
+				settings.erase("joystick");
 			END_OPTION
 
 			DO_LONG_OPTION("platform")
@@ -516,28 +497,26 @@ unknownOption:
 
 /** List all supported game IDs, i.e. all games which any loaded plugin supports. */
 static void listGames() {
-	const PluginList &plugins = PluginManager::instance().getPlugins();
-
 	printf("Game ID              Full Title                                            \n"
 	       "-------------------- ------------------------------------------------------\n");
 
+	const PluginList &plugins = PluginManager::instance().getPlugins();
 	PluginList::const_iterator iter = plugins.begin();
 	for (iter = plugins.begin(); iter != plugins.end(); ++iter) {
 		GameList list = (*iter)->getSupportedGames();
 		for (GameList::iterator v = list.begin(); v != list.end(); ++v) {
-			printf("%-20s %s\n", v->gameid.c_str(), v->description.c_str());
+			printf("%-20s %s\n", v->gameid().c_str(), v->description().c_str());
 		}
 	}
 }
 
 /** List all targets which are configured in the config file. */
 static void listTargets() {
-	using namespace Common;
-	const ConfigManager::DomainMap &domains = ConfMan.getGameDomains();
-
 	printf("Target               Description                                           \n"
 	       "-------------------- ------------------------------------------------------\n");
 
+	using namespace Common;
+	const ConfigManager::DomainMap &domains = ConfMan.getGameDomains();
 	ConfigManager::DomainMap::const_iterator iter = domains.begin();
 	for (iter = domains.begin(); iter != domains.end(); ++iter) {
 		Common::String name(iter->_key);
@@ -549,8 +528,8 @@ static void listTargets() {
 			// be taken into account, too.
 			Common::String gameid(name);
 			GameDescriptor g = Base::findGame(gameid);
-			if (g.description.size() > 0)
-				description = g.description;
+			if (g.description().size() > 0)
+				description = g.description();
 		}
 
 		printf("%-20s %s\n", name.c_str(), description.c_str());
@@ -588,11 +567,11 @@ static void runDetectorTest() {
 			continue;
 		}
 
-		DetectedGameList candidates(PluginManager::instance().detectGames(files));
+		GameList candidates(PluginManager::instance().detectGames(files));
 		bool gameidDiffers = false;
-		DetectedGameList::iterator x;
+		GameList::iterator x;
 		for (x = candidates.begin(); x != candidates.end(); ++x) {
-			gameidDiffers |= (scumm_stricmp(gameid.c_str(), x->gameid.c_str()) != 0);
+			gameidDiffers |= (scumm_stricmp(gameid.c_str(), x->gameid().c_str()) != 0);
 		}
 		
 		if (candidates.empty()) {
@@ -615,10 +594,10 @@ static void runDetectorTest() {
 		
 		for (x = candidates.begin(); x != candidates.end(); ++x) {
 			printf("    gameid '%s', desc '%s', language '%s', platform '%s'\n",
-					x->gameid.c_str(),
-					x->description.c_str(),
-					Common::getLanguageCode(x->language),
-					Common::getPlatformCode(x->platform));
+				   x->gameid().c_str(),
+				   x->description().c_str(),
+				   Common::getLanguageCode(x->language()),
+				   Common::getPlatformCode(x->platform()));
 		}
 	}
 	int total = domains.size();
@@ -646,11 +625,7 @@ bool processSettings(Common::String &command, Common::StringMap &settings) {
 		printf("Features compiled in: %s\n", gScummVMFeatures);
 		return false;
 	} else if (command == "help") {
-#if !(defined(PALMOS_ARM) || defined(PALMOS_DEBUG) || defined(__GP32__) || defined(__SYMBIAN32__))
-		printf(HELP_STRING, s_appName.c_str());
-#else
 		printf(HELP_STRING, s_appName);
-#endif
 		return false;
 	}
 #ifdef DETECTOR_TESTING_HACK
@@ -665,8 +640,25 @@ bool processSettings(Common::String &command, Common::StringMap &settings) {
 	// domain (i.e. a target) matching this argument, or alternatively
 	// whether there is a gameid matching that name.
 	if (!command.empty()) {
-		if (ConfMan.hasGameDomain(command) || Base::findGame(command).gameid.size() > 0) {
+		GameDescriptor gd = Base::findGame(command);
+		if (ConfMan.hasGameDomain(command) || !gd.gameid().empty()) {
+			bool idCameFromCommandLine = false;
+
+			// WORKAROUND: Fix for bug #1719463: "DETECTOR: Launching
+			// undefined target adds launcher entry"
+			//
+			// We designate gameids which come strictly from command line
+			// so AdvancedDetector will not save config file with invalid
+			// gameid in case target autoupgrade was performed
+			if (!ConfMan.hasGameDomain(command)) {
+				idCameFromCommandLine = true;
+			}
+
 			ConfMan.setActiveDomain(command);
+
+			if (idCameFromCommandLine)
+				ConfMan.set("id_came_from_command_line", "1");
+
 		} else {
 			usage("Unrecognized game target '%s'", command.c_str());
 		}
