@@ -1,6 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2001  Ludvig Strigeus
- * Copyright (C) 2001-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -16,15 +18,17 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/base/plugins.h $
- * $Id: plugins.h 22199 2006-04-29 00:27:20Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/base/plugins.h $
+ * $Id: plugins.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
 #ifndef BASE_PLUGINS_H
 #define BASE_PLUGINS_H
 
+#include "common/stdafx.h"
 #include "common/array.h"
+#include "common/list.h"
 #include "common/singleton.h"
 #include "common/util.h"
 #include "base/game.h"
@@ -33,38 +37,6 @@ class Engine;
 class FSList;
 class OSystem;
 
-/** List of games. */
-typedef Common::Array<GameDescriptor> GameList;
-
-/**
- * A detected game. Carries the GameDescriptor, but also (optionally)
- * information about the language and platform of the detected game.
- */
-struct DetectedGame : public GameDescriptor {
-	Common::Language language;
-	Common::Platform platform;
-	DetectedGame(const char *g = 0, const char *d = 0,
-	             Common::Language l = Common::UNK_LANG,
-	             Common::Platform p = Common::kPlatformUnknown)
-		: GameDescriptor(g, d), language(l), platform(p) {}
-
-	template <class T>
-	DetectedGame(const T &game,
-	             Common::Language l = Common::UNK_LANG,
-	             Common::Platform p = Common::kPlatformUnknown)
-		: GameDescriptor(game.gameid, game.description), language(l), platform(p) {}
-	
-	/**
-	 * Update the description string by appending (LANG/PLATFORM/EXTRA) to it.
-	 */
-	void updateDesc(const char *extra = 0);
-};
-
-
-/** List of detected games. */
-typedef Common::Array<DetectedGame> DetectedGameList;
-
-
 /**
  * Error codes which mayb be reported by plugins under various circumstances.
  * @todo Turn this into a global 'ErrorCode' enum used by all of ScummVM ?
@@ -86,17 +58,18 @@ enum PluginError {
  */
 class Plugin {
 public:
-	virtual ~Plugin()				{}
+	virtual ~Plugin() {}
 
-	virtual bool loadPlugin()		{ return true; }
-	virtual void unloadPlugin()		{}
+	virtual bool loadPlugin() = 0;
+	virtual void unloadPlugin() = 0;
 
 	virtual const char *getName() const = 0;
-	virtual int getVersion() const	{ return 0; }	// TODO!
+	virtual const char *getCopyright() const = 0;
+//	virtual int getVersion() const	{ return 0; }	// TODO!
 
 	virtual GameList getSupportedGames() const = 0;
 	virtual GameDescriptor findGame(const char *gameid) const = 0;
-	virtual DetectedGameList detectGames(const FSList &fslist) const = 0;
+	virtual GameList detectGames(const FSList &fslist) const = 0;
 
 	virtual PluginError createInstance(OSystem *syst, Engine **engine) const = 0;
 };
@@ -127,10 +100,10 @@ public:
  */
 
 #ifndef DYNAMIC_MODULES
-#define REGISTER_PLUGIN(ID,name) \
+#define REGISTER_PLUGIN(ID,name,copyright) \
 	PluginRegistrator *g_##ID##_PluginReg; \
 	void g_##ID##_PluginReg_alloc() { \
-		g_##ID##_PluginReg = new PluginRegistrator(name, \
+		g_##ID##_PluginReg = new PluginRegistrator(name, copyright, \
 			Engine_##ID##_gameIDList(), \
 			Engine_##ID##_findGameID, \
 			Engine_##ID##_create, \
@@ -139,13 +112,14 @@ public:
 	} \
 	void dummyFuncToAllowTrailingSemicolon()
 #else
-#define REGISTER_PLUGIN(ID,name) \
+#define REGISTER_PLUGIN(ID,name,copyright) \
 	extern "C" { \
 		PLUGIN_EXPORT const char *PLUGIN_name() { return name; } \
+		PLUGIN_EXPORT const char *PLUGIN_copyright() { return copyright; } \
 		PLUGIN_EXPORT GameList PLUGIN_gameIDList() { return Engine_##ID##_gameIDList(); } \
 		PLUGIN_EXPORT GameDescriptor PLUGIN_findGameID(const char *gameid) { return Engine_##ID##_findGameID(gameid); } \
 		PLUGIN_EXPORT PluginError PLUGIN_createEngine(OSystem *syst, Engine **engine) { return Engine_##ID##_create(syst, engine); } \
-		PLUGIN_EXPORT DetectedGameList PLUGIN_detectGames(const FSList &fslist) { return Engine_##ID##_detectGames(fslist); } \
+		PLUGIN_EXPORT GameList PLUGIN_detectGames(const FSList &fslist) { return Engine_##ID##_detectGames(fslist); } \
 	} \
 	void dummyFuncToAllowTrailingSemicolon()
 #endif
@@ -160,17 +134,19 @@ class PluginRegistrator {
 public:
 	typedef GameDescriptor (*GameIDQueryFunc)(const char *gameid);
 	typedef PluginError (*EngineFactory)(OSystem *syst, Engine **engine);
-	typedef DetectedGameList (*DetectFunc)(const FSList &fslist);
+	typedef GameList (*DetectFunc)(const FSList &fslist);
 
 protected:
 	const char *_name;
+	const char *_copyright;
 	GameIDQueryFunc _qf;
 	EngineFactory _ef;
 	DetectFunc _df;
 	GameList _games;
 
 public:
-	PluginRegistrator(const char *name, GameList games, GameIDQueryFunc qf, EngineFactory ef, DetectFunc df);
+	PluginRegistrator(const char *name, const char *copyright, GameList games, GameIDQueryFunc qf, EngineFactory ef, DetectFunc df)
+		: _name(name), _copyright(copyright), _qf(qf), _ef(ef), _df(df), _games(games) {}
 };
 #endif
 
@@ -179,7 +155,20 @@ public:
 typedef Common::Array<Plugin *> PluginList;
 
 
-class PluginManager;
+class PluginProvider {
+public:
+	virtual ~PluginProvider() {}
+	
+	/**
+	 * Return a list of Plugin objects. The caller is responsible for actually
+	 * loading/unloading them (by invoking the appropriate methods).
+	 * Furthermore, the caller is responsible for deleting these objects
+	 * eventually.
+	 */
+	virtual PluginList getPlugins() = 0;
+};
+
+//class PluginManager;
 
 /**
  * Instances of this class manage all plugins, including loading them,
@@ -188,8 +177,10 @@ class PluginManager;
  * @todo Add support for dynamic plugins (this may need additional API, e.g. for a plugin path)
  */
 class PluginManager : public Common::Singleton<PluginManager> {
+	typedef Common::List<PluginProvider *> ProviderList;
 private:
 	PluginList _plugins;
+	ProviderList _providers;
 
 	bool tryLoadPlugin(Plugin *plugin);
 
@@ -198,6 +189,8 @@ private:
 
 public:
 	~PluginManager();
+	
+	void addPluginProvider(PluginProvider *pp);
 
 	void loadPlugins();
 	void unloadPlugins();
@@ -205,7 +198,7 @@ public:
 
 	const PluginList &getPlugins()	{ return _plugins; }
 
-	DetectedGameList detectGames(const FSList &fslist) const;
+	GameList detectGames(const FSList &fslist) const;
 };
 
 #endif
