@@ -1,6 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2001  Ludvig Strigeus
- * Copyright (C) 2001-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -16,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/base/version.cpp $
- * $Id: version.cpp 22220 2006-04-29 13:17:22Z wjpalenstijn $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/base/version.cpp $
+ * $Id: version.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -95,7 +97,7 @@ const char *gScummVMFeatures = ""
 #ifdef __SYMBIAN32__
 // we want a list of compiled in engines visible in the program,
 // because we also release special builds with only one engine
-#include "backends/symbian/src/main_features.inl"
+#include "backends/platform/symbian/src/main_features.inl"
 #endif
 	;
 
