@@ -1,6 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2001  Ludvig Strigeus
- * Copyright (C) 2001-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -16,11 +18,17 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/common/config-manager.cpp $
- * $Id: config-manager.cpp 24241 2006-10-08 22:47:29Z djwillis $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/common/config-manager.cpp $
+ * $Id: config-manager.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
+#if defined(WIN32)
+#include <windows.h>
+// winnt.h defines ARRAYSIZE, but we want our own one...
+#undef ARRAYSIZE
+#endif
+
 #include "common/stdafx.h"
 
 #include "common/config-manager.h"
@@ -30,7 +38,7 @@
 DECLARE_SINGLETON(Common::ConfigManager);
 
 #ifdef __PLAYSTATION2__
-#include "backends/ps2/systemps2.h"
+#include "backends/platform/ps2/systemps2.h"
 #endif
 
 #if defined(UNIX)
@@ -45,19 +53,6 @@ DECLARE_SINGLETON(Common::ConfigManager);
 
 #define MAXLINELEN 256
 
-static char *ltrim(char *t) {
-	while (isspace(*t))
-		t++;
-	return t;
-}
-
-static char *rtrim(char *t) {
-	int l = strlen(t) - 1;
-	while (l >= 0 && isspace(t[l]))
-		t[l--] = 0;
-	return t;
-}
-
 static bool isValidDomainName(const Common::String &domName) {
 	const char *p = domName.c_str();
 	while (*p && (isalnum(*p) || *p == '-' || *p == '_'))
@@ -97,8 +92,49 @@ void ConfigManager::loadDefaultConfigFile() {
 		strcpy(configFile, DEFAULT_CONFIG_FILE);
 #else
 	#if defined (WIN32) && !defined(_WIN32_WCE) && !defined(__SYMBIAN32__)
-		GetWindowsDirectory(configFile, MAXPATHLEN);
-		strcat(configFile, "\\" DEFAULT_CONFIG_FILE);
+		OSVERSIONINFO win32OsVersion;
+		ZeroMemory(&win32OsVersion, sizeof(OSVERSIONINFO));
+		win32OsVersion.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
+		GetVersionEx(&win32OsVersion);
+		// Check for non-9X version of Windows.
+		if (win32OsVersion.dwPlatformId != VER_PLATFORM_WIN32_WINDOWS) {
+			// Use the Application Data directory of the user profile.
+			if (win32OsVersion.dwMajorVersion >= 5) {
+				if (!GetEnvironmentVariable("APPDATA", configFile, sizeof(configFile)))
+					error("Unable to access application data directory");
+			} else {
+				if (!GetEnvironmentVariable("USERPROFILE", configFile, sizeof(configFile)))
+					error("Unable to access user profile directory");
+
+				strcat(configFile, "\\Application Data");
+				CreateDirectory(configFile, NULL);
+			}
+
+			strcat(configFile, "\\ScummVM");
+			CreateDirectory(configFile, NULL);
+			strcat(configFile, "\\" DEFAULT_CONFIG_FILE);
+
+			if (fopen(configFile, "r") == NULL) {
+				// Check windows directory
+				char oldConfigFile[MAXPATHLEN];
+				GetWindowsDirectory(oldConfigFile, MAXPATHLEN);
+				strcat(oldConfigFile, "\\" DEFAULT_CONFIG_FILE);
+				if (fopen(oldConfigFile, "r")) {
+					printf("The default location of the config file (scummvm.ini) in ScummVM has changed,\n");
+					printf("under Windows NT4/2000/XP/Vista. You may want to consider moving your config\n");
+					printf("file from the old default location:\n");
+					printf("%s\n", oldConfigFile);
+					printf("to the new default location:\n");
+					printf("%s\n\n", configFile);
+					strcpy(configFile, oldConfigFile);
+				}
+			}
+		} else {
+			// Check windows directory
+			GetWindowsDirectory(configFile, MAXPATHLEN);
+			strcat(configFile, "\\" DEFAULT_CONFIG_FILE);
+		}
+
 	#elif defined(PALMOS_MODE)
 		strcpy(configFile,"/PALM/Programs/ScummVM/" DEFAULT_CONFIG_FILE);
 	#elif defined(__PLAYSTATION2__)
@@ -125,7 +161,7 @@ void ConfigManager::loadConfigFile(const String &filename) {
 	_filename = filename;
 	_domainSaveOrder.clear();
 	loadFile(_filename);
-	debug(1, "Switched to configuration %s", _filename.c_str());
+	printf("Using configuration file: %s\n", _filename.c_str());
 }
 
 void ConfigManager::loadFile(const String &filename) {
@@ -219,13 +255,13 @@ void ConfigManager::loadFile(const String &filename) {
 
 void ConfigManager::flushToDisk() {
 #ifndef __DC__
-	FILE *cfg_file;
+	File cfg_file;
 
 // TODO
 //	if (!willwrite)
 //		return;
 
-	if (!(cfg_file = fopen(_filename.c_str(), "w"))) {
+	if (!cfg_file.open(_filename, File::kFileWriteMode)) {
 		warning("Unable to write configuration file: %s", _filename.c_str());
 	} else {
 
@@ -250,13 +286,11 @@ void ConfigManager::flushToDisk() {
 			if (!_domainSaveOrder.contains(d->_key))
 				writeDomain(cfg_file, d->_key, d->_value);
 		}
-
-		fclose(cfg_file);
 	}
 #endif // !__DC__
 }
 
-void ConfigManager::writeDomain(FILE *file, const String &name, const Domain &domain) {
+void ConfigManager::writeDomain(WriteStream &stream, const String &name, const Domain &domain) {
 	if (domain.empty())
 		return;		// Don't bother writing empty domains.
 
@@ -265,26 +299,31 @@ void ConfigManager::writeDomain(FILE *file, const String &name, const Domain &do
 	// Write domain comment (if any)
 	comment = domain.getDomainComment();
 	if (!comment.empty())
-		fprintf(file, "%s", comment.c_str());
+		stream.writeString(comment);
 
 	// Write domain start
-	fprintf(file, "[%s]\n", name.c_str());
+	stream.writeByte('[');
+	stream.writeString(name);
+	stream.writeByte(']');
+	stream.writeByte('\n');
 
 	// Write all key/value pairs in this domain, including comments
 	Domain::const_iterator x;
 	for (x = domain.begin(); x != domain.end(); ++x) {
-		const String &value = x->_value;
-		if (!value.empty()) {
+		if (!x->_value.empty()) {
 			// Write comment (if any)
 			if (domain.hasKVComment(x->_key)) {
 				comment = domain.getKVComment(x->_key);
-				fprintf(file, "%s", comment.c_str());
+				stream.writeString(comment);
 			}
 			// Write the key/value pair
-			fprintf(file, "%s=%s\n", x->_key.c_str(), value.c_str());
+			stream.writeString(x->_key);
+			stream.writeByte('=');
+			stream.writeString(x->_value);
+			stream.writeByte('\n');
 		}
 	}
-	fprintf(file, "\n");
+	stream.writeByte('\n');
 }
 
 
@@ -431,7 +470,7 @@ int ConfigManager::getInt(const String &key, const String &domName) const {
 	if (value.empty())
 		return 0;
 
-	// We zse the special value '0' for the base passed to strtol. Doing that
+	// We use the special value '0' for the base passed to strtol. Doing that
 	// makes it possible to enter hex values as "0x1234", but also decimal
 	// values ("123") are still valid.
 	int ivalue = (int)strtol(value.c_str(), &errpos, 0);
