@@ -1,6 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2001  Ludvig Strigeus
- * Copyright (C) 2001-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -16,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/common/config-manager.h $
- * $Id: config-manager.h 21951 2006-04-16 19:23:14Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/common/config-manager.h $
+ * $Id: config-manager.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -29,22 +31,12 @@
 #include "common/hashmap.h"
 #include "common/singleton.h"
 #include "common/str.h"
+#include "common/hash-str.h"
 
 namespace Common {
 
-struct IgnoreCase_Less {
-  bool operator()(const String& x, const String& y) const { return scumm_stricmp(x.c_str(), y.c_str()) < 0; }
-};
+class WriteStream;
 
-struct IgnoreCase_EqualTo {
-  bool operator()(const String& x, const String& y) const { return scumm_stricmp(x.c_str(), y.c_str()) == 0; }
-};
-
-struct IgnoreCase_Hash {
-  uint operator()(const String& x) const { return hashit_lower(x.c_str()); }
-};
-
-typedef HashMap<String, String, IgnoreCase_Hash, IgnoreCase_EqualTo> StringMap;
 
 /**
  * The (singleton) configuration manager, used to query & set configuration
@@ -165,7 +157,7 @@ private:
 	ConfigManager();
 
 	void			loadFile(const String &filename);
-	void			writeDomain(FILE *file, const String &name, const Domain &domain);
+	void			writeDomain(WriteStream &stream, const String &name, const Domain &domain);
 
 	Domain			_transientDomain;
 	DomainMap		_gameDomains;
