@@ -1,6 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2001  Ludvig Strigeus
- * Copyright (C) 2001-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -16,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/common/endian.h $
- * $Id: endian.h 21919 2006-04-15 21:22:33Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/common/endian.h $
+ * $Id: endian.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -91,6 +93,9 @@ FORCEINLINE uint16 SWAP_BYTES_16(uint16 a) {
 	#define FROM_LE_32(a) ((uint32)(a))
 	#define FROM_LE_16(a) ((uint16)(a))
 
+	#define FROM_BE_32(a) SWAP_BYTES_32(a)
+	#define FROM_BE_16(a) SWAP_BYTES_16(a)
+
 	#define TO_LE_32(a) ((uint32)(a))
 	#define TO_LE_16(a) ((uint16)(a))
 
@@ -111,6 +116,9 @@ FORCEINLINE uint16 SWAP_BYTES_16(uint16 a) {
 	#define FROM_LE_32(a) SWAP_BYTES_32(a)
 	#define FROM_LE_16(a) SWAP_BYTES_16(a)
 
+	#define FROM_BE_32(a) ((uint32)(a))
+	#define FROM_BE_16(a) ((uint16)(a))
+
 	#define TO_LE_32(a) SWAP_BYTES_32(a)
 	#define TO_LE_16(a) SWAP_BYTES_16(a)
 
@@ -125,7 +133,7 @@ FORCEINLINE uint16 SWAP_BYTES_16(uint16 a) {
 #endif
 
 
-#if defined(SCUMM_NEED_ALIGNMENT) || defined(SCUMM_BIG_ENDIAN)
+#if defined(SCUMM_NEED_ALIGNMENT) || !defined(SCUMM_LITTLE_ENDIAN)
 	FORCEINLINE uint16 READ_LE_UINT16(const void *ptr) {
 		const byte *b = (const byte *)ptr;
 		return (b[1] << 8) + b[0];
@@ -162,7 +170,7 @@ FORCEINLINE uint16 SWAP_BYTES_16(uint16 a) {
 #endif
 
 
-#if defined(SCUMM_NEED_ALIGNMENT) || defined(SCUMM_LITTLE_ENDIAN)
+#if defined(SCUMM_NEED_ALIGNMENT) || !defined(SCUMM_BIG_ENDIAN)
 	FORCEINLINE uint16 READ_BE_UINT16(const void *ptr) {
 		const byte *b = (const byte *)ptr;
 		return (b[0] << 8) + b[1];
