@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2002-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,30 +18,128 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/common/file.cpp $
- * $Id: file.cpp 23151 2006-06-16 22:19:45Z anotherguest $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/common/file.cpp $
+ * $Id: file.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
 #include "common/file.h"
+#include "common/fs.h"
 #include "common/hashmap.h"
 #include "common/util.h"
-#include "backends/fs/fs.h"
+#include "common/hash-str.h"
 
 #ifdef MACOSX
 #include "CoreFoundation/CoreFoundation.h"
 #endif
 
+#ifdef __PLAYSTATION2__
+	// for those replaced fopen/fread/etc functions
+	typedef unsigned long	uint64;
+	typedef signed long	int64;
+	#include "backends/platform/ps2/fileio.h"
+
+	#define fopen(a, b)			ps2_fopen(a, b)
+	#define fclose(a)			ps2_fclose(a)
+	#define fseek(a, b, c)			ps2_fseek(a, b, c)
+	#define ftell(a)			ps2_ftell(a)
+	#define feof(a)				ps2_feof(a)
+	#define fread(a, b, c, d)		ps2_fread(a, b, c, d)
+	#define fwrite(a, b, c, d)		ps2_fwrite(a, b, c, d)
+
+	//#define fprintf				ps2_fprintf	// used in common/util.cpp
+	//#define fflush(a)			ps2_fflush(a)	// used in common/util.cpp
+
+	//#define fgetc(a)			ps2_fgetc(a)	// not used
+	//#define fgets(a, b, c)			ps2_fgets(a, b, c)	// not used
+	//#define fputc(a, b)			ps2_fputc(a, b)	// not used
+	//#define fputs(a, b)			ps2_fputs(a, b)	// not used
+
+	//#define fsize(a)			ps2_fsize(a)	// not used -- and it is not a standard function either
+#endif
+
+#ifdef __DS__
+
+	// These functions replease the standard library functions of the same name.
+	// As this header is included after the standard one, I have the chance to #define
+	// all of these to my own code.
+	//
+	// A #define is the only way, as redefinig the functions would cause linker errors.
+	
+	// These functions need to be #undef'ed, as their original definition 
+	// in devkitarm is done with #includes (ugh!)
+	#undef feof
+	#undef clearerr
+	//#undef getc
+	//#undef ferror
+	
+
+	//void 	std_fprintf(FILE* handle, const char* fmt, ...);	// used in common/util.cpp
+	//void 	std_fflush(FILE* handle);	// used in common/util.cpp
+
+	//char* 	std_fgets(char* str, int size, FILE* file);	// not used
+	//int 	std_getc(FILE* handle);	// not used
+	//char* 	std_getcwd(char* dir, int dunno);	// not used
+	//void 	std_cwd(char* dir);	// not used
+	//int 	std_ferror(FILE* handle);	// not used
+	
+	// Only functions used in the ScummVM source have been defined here!
+	#define fopen(name, mode) 					DS::std_fopen(name, mode)
+	#define fclose(handle) 						DS::std_fclose(handle)
+	#define fread(ptr, size, items, file)		DS::std_fread(ptr, size, items, file)
+	#define fwrite(ptr, size, items, file)		DS::std_fwrite(ptr, size, items, file)
+	#define feof(handle)						DS::std_feof(handle)
+	#define ftell(handle)						DS::std_ftell(handle)
+	#define fseek(handle, offset, whence)		DS::std_fseek(handle, offset, whence)
+	#define clearerr(handle)					DS::std_clearerr(handle)
+
+	//#define printf(fmt, ...)					consolePrintf(fmt, ##__VA_ARGS__)
+
+	//#define fprintf(file, fmt, ...)				{ char str[128]; sprintf(str, fmt, ##__VA_ARGS__); DS::std_fwrite(str, strlen(str), 1, file); }
+	//#define fflush(file)						DS::std_fflush(file)	// used in common/util.cpp
+
+	//#define fgets(str, size, file)				DS::std_fgets(str, size, file)	// not used
+	//#define getc(handle)						DS::std_getc(handle)	// not used
+	//#define getcwd(dir, dunno)					DS::std_getcwd(dir, dunno)	// not used
+	//#define ferror(handle)						DS::std_ferror(handle)	// not used
+
+#endif
+
+#ifdef __SYMBIAN32__
+	#undef feof
+	#undef clearerr
+	
+	#define FILE void
+	
+	FILE* 	symbian_fopen(const char* name, const char* mode);
+	void 	symbian_fclose(FILE* handle);
+	size_t 	symbian_fread(const void* ptr, size_t size, size_t numItems, FILE* handle);
+	size_t 	symbian_fwrite(const void* ptr, size_t size, size_t numItems, FILE* handle);
+	bool 	symbian_feof(FILE* handle);
+	long int symbian_ftell(FILE* handle);
+	int 	symbian_fseek(FILE* handle, long int offset, int whence);
+	void 	symbian_clearerr(FILE* handle);
+
+	// Only functions used in the ScummVM source have been defined here!
+	#define fopen(name, mode) 					symbian_fopen(name, mode)
+	#define fclose(handle) 						symbian_fclose(handle)
+	#define fread(ptr, size, items, file)		symbian_fread(ptr, size, items, file)
+	#define fwrite(ptr, size, items, file)		symbian_fwrite(ptr, size, items, file)
+	#define feof(handle)						symbian_feof(handle)
+	#define ftell(handle)						symbian_ftell(handle)
+	#define fseek(handle, offset, whence)		symbian_fseek(handle, offset, whence)
+	#define clearerr(handle)					symbian_clearerr(handle)
+#endif
+
 namespace Common {
 
-typedef HashMap<String, String> FilesMap;
-typedef HashMap<String, int> StringIntMap;
+typedef HashMap<String, int, CaseSensitiveString_Hash, CaseSensitiveString_EqualTo> StringIntMap;
 
 // The following two objects could be turned into static members of class
 // File. However, then we would be forced to #include hashmap in file.h
 // which seems to be a high price just for a simple beautification...
 static StringIntMap *_defaultDirectories;
-static FilesMap *_filesMap;
+static StringMap *_filesMap;
 
 static FILE *fopenNoCase(const String &filename, const String &directory, const char *mode) {
 	FILE *file;
@@ -142,14 +243,14 @@ void File::addDefaultDirectoryRecursive(const FilesystemNode &dir, int level, co
 	(*_defaultDirectories)[directory] = level;
 
 	if (!_filesMap)
-		_filesMap = new FilesMap;
+		_filesMap = new StringMap;
 
 	for (FSList::const_iterator file = fslist.begin(); file != fslist.end(); ++file) {
 		if (file->isDirectory()) {
-			addDefaultDirectoryRecursive(file->path(), level - 1, prefix + file->displayName() + "/");
+			addDefaultDirectoryRecursive(file->path(), level - 1, prefix + file->name() + "/");
 		} else {
 			String lfn(prefix);
-			lfn += file->displayName();
+			lfn += file->name();
 			lfn.toLowercase();
 			if (!_filesMap->contains(lfn)) {
 				(*_filesMap)[lfn] = file->path();
@@ -167,7 +268,7 @@ void File::resetDefaultDirectories() {
 }
 
 File::File()
-	: _handle(0), _ioFailed(false), _refcount(1) {
+	: _handle(0), _ioFailed(false) {
 }
 
 //#define DEBUG_FILE_REFCOUNT
@@ -178,21 +279,6 @@ File::~File() {
 #endif
 	close();
 }
-void File::incRef() {
-#ifdef DEBUG_FILE_REFCOUNT
-	warning("File::incRef on file '%s'", _name.c_str());
-#endif
-	_refcount++;
-}
-
-void File::decRef() {
-#ifdef DEBUG_FILE_REFCOUNT
-	warning("File::decRef on file '%s'", _name.c_str());
-#endif
-	if (--_refcount == 0) {
-		delete this;
-	}
-}
 
 
 bool File::open(const String &filename, AccessMode mode) {
@@ -277,7 +363,16 @@ bool File::open(const String &filename, AccessMode mode) {
 
 bool File::open(const FilesystemNode &node, AccessMode mode) {
 	assert(mode == kFileReadMode || mode == kFileWriteMode);
-	String filename(node.displayName());
+
+	if (!node.isValid()) {
+		warning("File::open: Trying to open an invalid FilesystemNode object");
+		return false;
+	} else if (node.isDirectory()) {
+		warning("File::open: Trying to open a FilesystemNode which is a directory");
+		return false;
+	}
+
+	String filename(node.name());
 
 	if (_handle) {
 		error("File::open: This file object already is opened (%s), won't open '%s'", _name.c_str(), filename.c_str());
@@ -288,10 +383,8 @@ bool File::open(const FilesystemNode &node, AccessMode mode) {
 
 	const char *modeStr = (mode == kFileReadMode) ? "rb" : "wb";
 
-
 	_handle = fopen(node.path().c_str(), modeStr);
 
-
 	if (_handle == NULL) {
 		if (mode == kFileReadMode)
 			debug(2, "File %s not found", filename.c_str());
@@ -313,6 +406,9 @@ bool File::exists(const String &filename) {
 	// First try to find the file it via a FilesystemNode (in case an absolute
 	// path was passed). But we only use this to filter out directories.
 	FilesystemNode file(filename);
+	// FIXME: can't use isValid() here since at the time of writing
+	// FilesystemNode is to be unable to find for example files
+	// added in extrapath
 	if (file.isDirectory())
 		return false;
 
@@ -342,7 +438,7 @@ bool File::exists(const String &filename) {
 
 void File::close() {
 	if (_handle)
-		fclose(_handle);
+		fclose((FILE *)_handle);
 	_handle = NULL;
 }
 
@@ -364,7 +460,7 @@ bool File::eof() const {
 		return false;
 	}
 
-	return feof(_handle) != 0;
+	return feof((FILE *)_handle) != 0;
 }
 
 uint32 File::pos() const {
@@ -373,7 +469,7 @@ uint32 File::pos() const {
 		return 0;
 	}
 
-	return ftell(_handle);
+	return ftell((FILE *)_handle);
 }
 
 uint32 File::size() const {
@@ -382,10 +478,10 @@ uint32 File::size() const {
 		return 0;
 	}
 
-	uint32 oldPos = ftell(_handle);
-	fseek(_handle, 0, SEEK_END);
-	uint32 length = ftell(_handle);
-	fseek(_handle, oldPos, SEEK_SET);
+	uint32 oldPos = ftell((FILE *)_handle);
+	fseek((FILE *)_handle, 0, SEEK_END);
+	uint32 length = ftell((FILE *)_handle);
+	fseek((FILE *)_handle, oldPos, SEEK_SET);
 
 	return length;
 }
@@ -396,8 +492,8 @@ void File::seek(int32 offs, int whence) {
 		return;
 	}
 
-	if (fseek(_handle, offs, whence) != 0)
-		clearerr(_handle);
+	if (fseek((FILE *)_handle, offs, whence) != 0)
+		clearerr((FILE *)_handle);
 }
 
 uint32 File::read(void *ptr, uint32 len) {
@@ -412,7 +508,7 @@ uint32 File::read(void *ptr, uint32 len) {
 	if (len == 0)
 		return 0;
 
-	real_len = fread(ptr2, 1, len, _handle);
+	real_len = fread(ptr2, 1, len, (FILE *)_handle);
 	if (real_len < len) {
 		_ioFailed = true;
 	}
@@ -429,7 +525,7 @@ uint32 File::write(const void *ptr, uint32 len) {
 	if (len == 0)
 		return 0;
 
-	if ((uint32)fwrite(ptr, 1, len, _handle) != len) {
+	if ((uint32)fwrite(ptr, 1, len, (FILE *)_handle) != len) {
 		_ioFailed = true;
 	}
 
