@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2002-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/common/file.h $
- * $Id: file.h 22549 2006-05-20 15:10:04Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/common/file.h $
+ * $Id: file.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -34,15 +37,12 @@ namespace Common {
 
 class File : public SeekableReadStream, public WriteStream {
 protected:
-	/** POSIX file handle to the actual file; 0 if no file is open. */
-	FILE *_handle;
+	/** File handle to the actual file; 0 if no file is open. */
+	void *_handle;
 
 	/** Status flag which tells about recent I/O failures. */
 	bool _ioFailed;
 
-	/** Simple ref-counter for File objects. */
-	int32 _refcount;
-
 	/** The name of this file, for debugging. */
 	String _name;
 
@@ -67,28 +67,52 @@ public:
 	static void addDefaultDirectoryRecursive(const FilesystemNode &directory, int level = 4, const String &prefix = "");
 
 	static void resetDefaultDirectories();
-	
 
 	File();
 	virtual ~File();
 
-
-	void incRef();
-	void decRef();
+	/**
+	 * Checks if a given file exists in any of the current default paths
+	 * (those were/are added by addDefaultDirectory and/or
+	 * addDefaultDirectoryRecursive).
+	 *
+	 * @param filename: the file to check for
+	 * @return: true if the file exists, else false
+	 */
+	static bool exists(const String &filename);
 
 	virtual bool open(const String &filename, AccessMode mode = kFileReadMode);
 	virtual bool open(const FilesystemNode &node, AccessMode mode = kFileReadMode);
-	static bool exists(const String &filename);
 
 	virtual void close();
+
+	/**
+	 * Checks if the object opened a file successfully.
+	 *
+	 * @return: true if any file is opened, false otherwise.
+	 */
 	bool isOpen() const;
+
+	/**
+	 * Returns the filename of the opened file.
+	 *
+	 * @return: the filename
+	 */
+	const char *name() const { return _name.c_str(); }
+
 	bool ioFailed() const;
 	void clearIOFailed();
 	bool eos() const { return eof(); }
+
+	/**
+	 * Checks for end of file.
+	 *
+	 * @return: true if the end of file is reached, false otherwise.
+	 */
 	bool eof() const;
+
 	uint32 pos() const;
 	uint32 size() const;
-	const char *name() const { return _name.c_str(); }
 	void seek(int32 offs, int whence = SEEK_SET);
 	uint32 read(void *dataPtr, uint32 dataSize);
 	uint32 write(const void *dataPtr, uint32 dataSize);
