@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/common/hashmap.cpp $
- * $Id: hashmap.cpp 21517 2006-03-31 22:19:39Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/common/hashmap.cpp $
+ * $Id: hashmap.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -49,7 +52,6 @@
  */
 
 #include "common/hashmap.h"
-#include <ctype.h>
 
 namespace Common {
 
@@ -57,21 +59,16 @@ namespace Common {
 uint hashit(const char *p) {
 	uint hash = 0;
 	byte c;
-
-	hash = 0;
 	while ((c = *p++))
 		hash = (hash * 31 + c);
-
 	return hash;
 }
 
 uint hashit_lower(const char *p) {
 	uint hash = 0;
 	byte c;
-
 	while ((c = *p++))
 		hash = (hash * 31 + tolower(c));
-
 	return hash;
 }
 
