@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/common/hashmap.h $
- * $Id: hashmap.h 22875 2006-06-03 13:56:50Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/common/hashmap.h $
+ * $Id: hashmap.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -58,43 +61,6 @@
 
 namespace Common { 
 
-uint hashit(const char *str);
-uint hashit_lower(const char *str);	// Generate a hash based on the lowercase version of the string
-
-// Specalization of the Hash functor for String objects.
-template <>
-struct Hash<String> {
-	uint operator()(const String& s) const {
-		return hashit(s.c_str());
-	}
-};
-
-template <>
-struct Hash<const char *> {
-	uint operator()(const char *s) const {
-		return hashit(s);
-	}
-};
-
-// data structure used by HashMap internally to keep
-// track of what's mapped to what.
-template <class Key, class Val>
-struct BaseNode {
-	Key _key;
-	Val _value;
-	BaseNode() {}
-	BaseNode(const Key &key) : _key(key) {}
-};
-	
-template <class Val>
-struct BaseNode<const char *, Val> {
-	char *_key;
-	Val _value;
-	BaseNode() {assert(0);}
-	BaseNode(const char *key) { _key = (char *)malloc(strlen(key)+1); strcpy(_key, key); }
-	~BaseNode() { free(_key); }
-};
-
 // The table sizes ideally are primes. We use a helper function to find
 // suitable table sizes.
 uint nextTableSize(uint x);
@@ -118,38 +84,52 @@ uint nextTableSize(uint x);
  * triggered instead. Hence if you are not sure whether a key is contained in
  * the map, use contains() first to check for its presence.
  */ 
-template <class Key, class Val, class HashFunc = Hash<Key>, class EqualFunc = EqualTo<Key>, class BaseNodeFunc = BaseNode<Key, Val> >
+template <class Key, class Val, class HashFunc = Hash<Key>, class EqualFunc = EqualTo<Key> >
 class HashMap {
+	friend class const_iterator;
 private:
-#if defined (_WIN32_WCE) || defined (_MSC_VER) || defined (__SYMBIAN32__) || defined (PALMOS_MODE)
-//FIXME evc4, msvc6,msvc7 & GCC 2.9x doesn't like it as private member
+#if defined (PALMOS_MODE)
 public:
 #endif
-	BaseNodeFunc **_arr;	// hashtable of size arrsize.
+
+	typedef HashMap<Key, Val, HashFunc, EqualFunc> HM_t;
+
+	struct Node {
+		const Key _key;
+		Val _value;
+		Node(const Key &key) : _key(key) {}
+	};
+
+	Node **_arr;	// hashtable of size arrsize.
 	uint _arrsize, _nele;
 	
 	HashFunc _hash;
 	EqualFunc _equal;
 	
+	// Default value, returned by the const getVal.
+	const Val _defaultVal;
+	
 #ifdef DEBUG_HASH_COLLISIONS
 	mutable int _collisions, _lookups;
 #endif
 
+	void assign(const HM_t& map);
 	int lookup(const Key &key) const;
+	int lookupAndCreateIfMissing(const Key &key);
 	void expand_array(uint newsize);
 
 public:
 	class const_iterator {
-		typedef const HashMap<Key, Val, HashFunc, EqualFunc, BaseNodeFunc> * hashmap_t;
-		friend class HashMap<Key, Val, HashFunc, EqualFunc, BaseNodeFunc>;
+		typedef const HashMap<Key, Val, HashFunc, EqualFunc> * hashmap_t;
+		friend class HashMap<Key, Val, HashFunc, EqualFunc>;
 	protected:
-		hashmap_t _hashmap;
 		uint _idx;
+		hashmap_t _hashmap;
 		const_iterator(uint idx, hashmap_t hashmap) : _idx(idx), _hashmap(hashmap) {}
 
-		const BaseNodeFunc *deref() const {
+		const Node *deref() const {
 			assert(_hashmap != 0);
-			BaseNodeFunc *node = _hashmap->_arr[_idx];
+			Node *node = _hashmap->_arr[_idx];
 			assert(node != 0);
 			return node;
 		}
@@ -157,8 +137,8 @@ public:
 	public:
 		const_iterator() : _idx(0), _hashmap(0) {}
 
-		const BaseNodeFunc &operator *() const { return *deref(); }
-		const BaseNodeFunc *operator->() const { return deref(); }
+		const Node &operator *() const { return *deref(); }
+		const Node *operator->() const { return deref(); }
 		bool operator ==(const const_iterator &iter) const { return _idx == iter._idx && _hashmap == iter._hashmap; }
 		bool operator !=(const const_iterator &iter) const { return !(*this == iter); }
 		const_iterator operator ++() {
@@ -172,19 +152,35 @@ public:
 			return *this;
 		}
 	};
-
+	
 	HashMap();
+	HashMap(const HM_t& map);
 	~HashMap();
 
+	HM_t &operator =(const HM_t &map) {
+		if (this == &map)
+			return *this;
+
+		// Remove the previous content and ...
+		clear();
+		delete[] _arr;
+		// ... copy the new stuff.
+		assign(map);
+		return *this;
+	}
+
 	bool contains(const Key &key) const;
 
 	Val &operator [](const Key &key);
 	const Val &operator [](const Key &key) const;
-	const Val &queryVal(const Key &key) const;
+
+	Val &getVal(const Key &key);
+	const Val &getVal(const Key &key) const;
+	void setVal(const Key &key, const Val &val);
 
 	void clear(bool shrinkArray = 0);
 
-	size_t erase(const Key &key);
+	void erase(const Key &key);
 
 	uint size() const { return _nele; }
 
@@ -217,12 +213,16 @@ public:
 //-------------------------------------------------------
 // HashMap functions
 
-template <class Key, class Val, class HashFunc, class EqualFunc, class BaseNodeFunc>
-HashMap<Key, Val, HashFunc, EqualFunc, BaseNodeFunc>::HashMap() {
+/**
+ * Base constructor, creates an empty hashmap.
+ */
+template <class Key, class Val, class HashFunc, class EqualFunc>
+HashMap<Key, Val, HashFunc, EqualFunc>::HashMap()
+	: _defaultVal() {
 	_arrsize = nextTableSize(0);
-	_arr = new BaseNodeFunc *[_arrsize];
+	_arr = new Node *[_arrsize];
 	assert(_arr != NULL);
-	memset(_arr, 0, _arrsize * sizeof(BaseNodeFunc *));
+	memset(_arr, 0, _arrsize * sizeof(Node *));
 
 	_nele = 0;
 	
@@ -232,20 +232,59 @@ HashMap<Key, Val, HashFunc, EqualFunc, BaseNodeFunc>::HashMap() {
 #endif
 }
 
-template <class Key, class Val, class HashFunc, class EqualFunc, class BaseNodeFunc>
-HashMap<Key, Val, HashFunc, EqualFunc, BaseNodeFunc>::~HashMap() {
-	uint ctr;
+/**
+ * Copy constructor, creates a full copy of the given hashmap.
+ * We must provide a custom copy constructor as we use pointers
+ * to heap buffers for the internal storage.
+ */
+template <class Key, class Val, class HashFunc, class EqualFunc>
+HashMap<Key, Val, HashFunc, EqualFunc>::HashMap(const HM_t& map)
+	: _defaultVal()  {
+	assign(map);
+}
 
-	for (ctr = 0; ctr < _arrsize; ctr++)
+/**
+ * Destructor, frees all used memory.
+ */
+template <class Key, class Val, class HashFunc, class EqualFunc>
+HashMap<Key, Val, HashFunc, EqualFunc>::~HashMap() {
+	for (uint ctr = 0; ctr < _arrsize; ++ctr)
 		if (_arr[ctr] != NULL)
 			delete _arr[ctr];
 
 	delete[] _arr;
 }
 
-template <class Key, class Val, class HashFunc, class EqualFunc, class BaseNodeFunc>
-void HashMap<Key, Val, HashFunc, EqualFunc, BaseNodeFunc>::clear(bool shrinkArray) {
-	for (uint ctr = 0; ctr < _arrsize; ctr++) {
+/**
+ * Internal method for assigning the content of another HashMap
+ * to this one.
+ *
+ * @note We do *not* deallocate the previous storage here -- the caller is
+ *       responsible for doing that!
+ */
+template <class Key, class Val, class HashFunc, class EqualFunc>
+void HashMap<Key, Val, HashFunc, EqualFunc>::assign(const HM_t& map) {
+	_arrsize = map._arrsize;
+	_arr = new Node *[_arrsize];
+	assert(_arr != NULL);
+	memset(_arr, 0, _arrsize * sizeof(Node *));
+
+	// Simply clone the map given to us, one by one.
+	_nele = 0;
+	for (uint ctr = 0; ctr < _arrsize; ++ctr) {
+		if (map._arr[ctr] != NULL) {
+			_arr[ctr] = new Node(*map._arr[ctr]);
+			_nele++;
+		}
+	}
+	// Perform a sanity check (to help track down hashmap corruption)
+	assert(_nele == map._nele);
+}
+
+
+template <class Key, class Val, class HashFunc, class EqualFunc>
+void HashMap<Key, Val, HashFunc, EqualFunc>::clear(bool shrinkArray) {
+	for (uint ctr = 0; ctr < _arrsize; ++ctr) {
 		if (_arr[ctr] != NULL) {
 			delete _arr[ctr];
 			_arr[ctr] = NULL;
@@ -256,39 +295,37 @@ void HashMap<Key, Val, HashFunc, EqualFunc, BaseNodeFunc>::clear(bool shrinkArra
 		delete[] _arr;
 
 		_arrsize = nextTableSize(0);
-		_arr = new BaseNodeFunc *[_arrsize];
+		_arr = new Node *[_arrsize];
 		assert(_arr != NULL);
-		memset(_arr, 0, _arrsize * sizeof(BaseNodeFunc *));
+		memset(_arr, 0, _arrsize * sizeof(Node *));
 	}
 
 	_nele = 0;
 }
 
-template <class Key, class Val, class HashFunc, class EqualFunc, class BaseNodeFunc>
-void HashMap<Key, Val, HashFunc, EqualFunc, BaseNodeFunc>::expand_array(uint newsize) {
+template <class Key, class Val, class HashFunc, class EqualFunc>
+void HashMap<Key, Val, HashFunc, EqualFunc>::expand_array(uint newsize) {
 	assert(newsize > _arrsize);
-	BaseNodeFunc **old_arr;
-	uint old_arrsize, old_nele, ctr, dex;
+	uint ctr, dex;
 
-	old_nele = _nele;
-	old_arr = _arr;
-	old_arrsize = _arrsize;
+	const uint old_nele = _nele;
+	const uint old_arrsize = _arrsize;
+	Node **old_arr = _arr;
 
 	// allocate a new array 
-	_arrsize = newsize;
-	_arr = new BaseNodeFunc *[_arrsize];
-	assert(_arr != NULL);
-	memset(_arr, 0, _arrsize * sizeof(BaseNodeFunc *));
-
 	_nele = 0;
+	_arrsize = newsize;
+	_arr = new Node *[_arrsize];
+	assert(_arr != NULL);
+	memset(_arr, 0, _arrsize * sizeof(Node *));
 
 	// rehash all the old elements
-	for (ctr = 0; ctr < old_arrsize; ctr++) {
+	for (ctr = 0; ctr < old_arrsize; ++ctr) {
 		if (old_arr[ctr] == NULL)
 			continue;
 
 		// Insert the element from the old table into the new table.
-		// Since we know that no key exists twice the old table, we
+		// Since we know that no key exists twice in the old table, we
 		// can do this slightly better than by calling lookup, since we
 		// don't have to call _equal().
 		dex = _hash(old_arr[ctr]->_key) % _arrsize;
@@ -301,6 +338,7 @@ void HashMap<Key, Val, HashFunc, EqualFunc, BaseNodeFunc>::expand_array(uint new
 	}
 
 	// Perform a sanity check: Old number of elements should match the new one!
+	// This check will fail if some previous operation corrupted this hashmap.
 	assert(_nele == old_nele);
 
 	delete[] old_arr;
@@ -308,8 +346,8 @@ void HashMap<Key, Val, HashFunc, EqualFunc, BaseNodeFunc>::expand_array(uint new
 	return;
 }
 
-template <class Key, class Val, class HashFunc, class EqualFunc, class BaseNodeFunc>
-int HashMap<Key, Val, HashFunc, EqualFunc, BaseNodeFunc>::lookup(const Key &key) const {
+template <class Key, class Val, class HashFunc, class EqualFunc>
+int HashMap<Key, Val, HashFunc, EqualFunc>::lookup(const Key &key) const {
 	uint ctr = _hash(key) % _arrsize;
 
 	while (_arr[ctr] != NULL && !_equal(_arr[ctr]->_key, key)) {
@@ -330,18 +368,12 @@ int HashMap<Key, Val, HashFunc, EqualFunc, BaseNodeFunc>::lookup(const Key &key)
 	return ctr;
 }
 
-template <class Key, class Val, class HashFunc, class EqualFunc, class BaseNodeFunc>
-bool HashMap<Key, Val, HashFunc, EqualFunc, BaseNodeFunc>::contains(const Key &key) const {
-	uint ctr = lookup(key);
-	return (_arr[ctr] != NULL);
-}
-
-template <class Key, class Val, class HashFunc, class EqualFunc, class BaseNodeFunc>
-Val &HashMap<Key, Val, HashFunc, EqualFunc, BaseNodeFunc>::operator [](const Key &key) {
+template <class Key, class Val, class HashFunc, class EqualFunc>
+int HashMap<Key, Val, HashFunc, EqualFunc>::lookupAndCreateIfMissing(const Key &key) {
 	uint ctr = lookup(key);
 
 	if (_arr[ctr] == NULL) {
-		_arr[ctr] = new BaseNodeFunc(key);
+		_arr[ctr] = new Node(key);
 		_nele++;
 
 		// Keep the load factor below 75%.
@@ -351,32 +383,68 @@ Val &HashMap<Key, Val, HashFunc, EqualFunc, BaseNodeFunc>::operator [](const Key
 		}
 	}
 
-	return _arr[ctr]->_value;
+	return ctr;
 }
 
-template <class Key, class Val, class HashFunc, class EqualFunc, class BaseNodeFunc>
-const Val &HashMap<Key, Val, HashFunc, EqualFunc, BaseNodeFunc>::operator [](const Key &key) const {
-	return queryVal(key);
-}
 
-template <class Key, class Val, class HashFunc, class EqualFunc, class BaseNodeFunc>
-const Val &HashMap<Key, Val, HashFunc, EqualFunc, BaseNodeFunc>::queryVal(const Key &key) const {
+template <class Key, class Val, class HashFunc, class EqualFunc>
+bool HashMap<Key, Val, HashFunc, EqualFunc>::contains(const Key &key) const {
 	uint ctr = lookup(key);
+	return (_arr[ctr] != NULL);
+}
+
+template <class Key, class Val, class HashFunc, class EqualFunc>
+Val &HashMap<Key, Val, HashFunc, EqualFunc>::operator [](const Key &key) {
+	return getVal(key);
+}
+
+template <class Key, class Val, class HashFunc, class EqualFunc>
+const Val &HashMap<Key, Val, HashFunc, EqualFunc>::operator [](const Key &key) const {
+	return getVal(key);
+}
+
+template <class Key, class Val, class HashFunc, class EqualFunc>
+Val &HashMap<Key, Val, HashFunc, EqualFunc>::getVal(const Key &key) {
+	uint ctr = lookupAndCreateIfMissing(key);
 	assert(_arr[ctr] != NULL);
 	return _arr[ctr]->_value;
 }
 
-template <class Key, class Val, class HashFunc, class EqualFunc, class BaseNodeFunc>
-size_t HashMap<Key, Val, HashFunc, EqualFunc, BaseNodeFunc>::erase(const Key &key) {
+template <class Key, class Val, class HashFunc, class EqualFunc>
+const Val &HashMap<Key, Val, HashFunc, EqualFunc>::getVal(const Key &key) const {
+	uint ctr = lookup(key);
+	if (_arr[ctr] != NULL)
+		return _arr[ctr]->_value;
+	else
+		return _defaultVal;
+}
+
+template <class Key, class Val, class HashFunc, class EqualFunc>
+void HashMap<Key, Val, HashFunc, EqualFunc>::setVal(const Key &key, const Val &val) {
+	uint ctr = lookupAndCreateIfMissing(key);
+	assert(_arr[ctr] != NULL);
+	_arr[ctr]->_value = val;
+}
+
+template <class Key, class Val, class HashFunc, class EqualFunc>
+void HashMap<Key, Val, HashFunc, EqualFunc>::erase(const Key &key) {
 	// This is based on code in the Wikipedia article on Hash tables.
 	uint i = lookup(key);
 	if (_arr[i] == NULL)
-		return 0; // key wasn't present, so no work has to be done
+		return; // key wasn't present, so no work has to be done
+	// If we remove a key, we must check all subsequent keys and possibly
+	// reinsert them.
 	uint j = i;
+	delete _arr[i];
+	_arr[i] = NULL;
 	while (true) {
+		// Look at the next table slot
 		j = (j + 1) % _arrsize;
+		// If the next slot is empty, we are done
 		if (_arr[j] == NULL)
 			break;
+		// Compute the slot where the content of the next slot should normally be,
+		// assuming an empty table, and check whether we have to move it.
 		uint k = _hash(_arr[j]->_key) % _arrsize;
 		if ((j > i && (k <= i || k > j)) ||
 		    (j < i && (k <= i && k > j)) ) {
@@ -385,7 +453,7 @@ size_t HashMap<Key, Val, HashFunc, EqualFunc, BaseNodeFunc>::erase(const Key &ke
 		}
 	}
 	_arr[i] = NULL;
-	return 1;
+	return;
 }
 
 }	// End of namespace Common
