@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2002-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/common/md5.cpp $
- * $Id: md5.cpp 22250 2006-04-30 22:58:44Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/common/md5.cpp $
+ * $Id: md5.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  */
 
 /*
@@ -25,8 +28,6 @@
  * this program is licensed under the GPL.
  */
 
-#include <string.h>
-
 #include "common/file.h"
 #include "common/md5.h"
 #include "common/util.h"
@@ -34,6 +35,17 @@
 
 namespace Common {
 
+typedef struct {
+	uint32 total[2];
+	uint32 state[4];
+	uint8 buffer[64];
+} md5_context;
+
+void md5_starts(md5_context *ctx);
+void md5_update(md5_context *ctx, const uint8 *input, uint32 length);
+void md5_finish(md5_context *ctx, uint8 digest[16]);
+
+
 #define GET_UINT32(n, b, i)	(n) = READ_LE_UINT32(b + i)
 #define PUT_UINT32(n, b, i)	WRITE_LE_UINT32(b + i, n)
 
@@ -233,6 +245,18 @@ void md5_finish(md5_context *ctx, uint8 digest[16]) {
 	PUT_UINT32(ctx->state[3], digest, 12);
 }
 
+bool md5_file(const FilesystemNode &file, uint8 digest[16], uint32 length) {
+	if (!file.isValid()) {
+		warning("md5_file: using an invalid FilesystemNode");
+		return false;
+	} else if (file.isDirectory()) {
+		warning("md5_file: using a diretory FilesystemNode");
+		return false;
+	}
+
+	return md5_file(file.path().c_str(), digest, length);
+}
+
 bool md5_file(const char *name, uint8 digest[16], uint32 length) {
 	File f;
 
@@ -241,6 +265,12 @@ bool md5_file(const char *name, uint8 digest[16], uint32 length) {
 		warning("md5_file couldn't open '%s'", name);
 		return false;
 	}
+	
+	return md5_file(f, digest, length);
+}
+	
+
+bool md5_file(ReadStream &stream, uint8 digest[16], uint32 length) {
 
 #ifdef DISABLE_MD5
 	memset(digest, 0, 16);
@@ -249,7 +279,7 @@ bool md5_file(const char *name, uint8 digest[16], uint32 length) {
 	int i;
 	unsigned char buf[1000];
 	bool restricted = (length != 0);
-	int readlen;
+	uint32 readlen;
 
 	if (!restricted || sizeof(buf) <= length)
 		readlen = sizeof(buf);
@@ -258,7 +288,7 @@ bool md5_file(const char *name, uint8 digest[16], uint32 length) {
 
 	md5_starts(&ctx);
 
-	while ((i = f.read(buf, readlen)) > 0) {
+	while ((i = stream.read(buf, readlen)) > 0) {
 		md5_update(&ctx, buf, i);
 
 		length -= i;
@@ -274,6 +304,42 @@ bool md5_file(const char *name, uint8 digest[16], uint32 length) {
 	return true;
 }
 
+bool md5_file_string(const FilesystemNode &file, char *md5str, uint32 length) {
+	uint8 digest[16];
+	if (!md5_file(file, digest, length))
+		return false;
+
+	for (int i = 0; i < 16; i++) {
+		snprintf(md5str + i*2, 3, "%02x", (int)digest[i]);
+	}
+
+	return true;
+}
+
+bool md5_file_string(const char *name, char *md5str, uint32 length) {
+	uint8 digest[16];
+	if (!md5_file(name, digest, length))
+		return false;
+
+	for (int i = 0; i < 16; i++) {
+		snprintf(md5str + i*2, 3, "%02x", (int)digest[i]);
+	}
+
+	return true;
+}
+
+bool md5_file_string(ReadStream &stream, char *md5str, uint32 length) {
+	uint8 digest[16];
+	if (!md5_file(stream, digest, length))
+		return false;
+
+	for (int i = 0; i < 16; i++) {
+		snprintf(md5str + i*2, 3, "%02x", (int)digest[i]);
+	}
+
+	return true;
+}
+
 } // End of namespace Common
 
 #ifdef TEST
