@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2002-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/common/rect.h $
- * $Id: rect.h 20510 2006-02-11 09:53:53Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/common/rect.h $
+ * $Id: rect.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -36,12 +39,30 @@ struct Point {
 	int16 x;	//!< The horizontal part of the point
 	int16 y;	//!< The vertical part of the point
 
-	Point() : x(0), y(0) {};
-	Point(const Point &p) : x(p.x), y(p.y) {};
-	explicit Point(int16 x1, int16 y1) : x(x1), y(y1) {};
+	Point() : x(0), y(0) {}
+	Point(const Point &p) : x(p.x), y(p.y) {}
+	explicit Point(int16 x1, int16 y1) : x(x1), y(y1) {}
 	Point & operator=(const Point & p) { x = p.x; y = p.y; return *this; };
 	bool operator==(const Point & p) const { return x == p.x && y == p.y; };
 	bool operator!=(const Point & p) const { return x != p.x || y != p.y; };
+
+	/**
+	 * Return the square of the distance between this point and the point p.
+	 *
+	 * @param p		the other point
+	 * @return the distance between this and p
+	 */
+	uint sqrDist(const Point & p) const {
+		int diffx = ABS(p.x - x);
+		if (diffx >= 0x1000)
+			return 0xFFFFFF;
+
+		int diffy = ABS(p.y - y);
+		if (diffy >= 0x1000)
+			return 0xFFFFFF;
+
+		return diffx*diffx + diffy*diffy;
+	}
 };
 
 /*! 	@brief simple class for handling a rectangular zone.
@@ -167,6 +188,11 @@ struct Rect {
 		left = x;
 	}
 
+	void translate(int16 dx, int16 dy) {
+		left += dx;	right += dx;
+		top += dy; bottom += dy;
+	}
+
 	void moveTo(const Point &p) {
 		moveTo(p.x, p.y);
 	}
