@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2002-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/common/savefile.h $
- * $Id: savefile.h 22180 2006-04-26 11:15:13Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/common/savefile.h $
+ * $Id: savefile.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -24,6 +27,7 @@
 #define COMMON_SAVEFILE_H
 
 #include "common/stdafx.h"
+#include "common/noncopyable.h"
 #include "common/scummsys.h"
 #include "common/stream.h"
 
@@ -34,8 +38,7 @@ namespace Common {
  * That typically means "save games", but also includes things like the
  * IQ points in Indy3.
  */
-class InSaveFile : public SeekableReadStream {
-};
+class InSaveFile : public SeekableReadStream {};
 
 /**
  * A class which allows game engines to save game state data.
@@ -43,16 +46,35 @@ class InSaveFile : public SeekableReadStream {
  * IQ points in Indy3.
  */
 class OutSaveFile : public WriteStream {
+public:
+	/**
+	 * Close this savefile, to be called right before destruction of this
+	 * savefile. The idea is that this ways, I/O errors that occur
+	 * during closing/flushing of the file can still be handled by the
+	 * game engine.
+	 *
+	 * By default, this just flushes the stream.
+	 */
+	virtual void finalize() {
+		flush();
+	}
 };
 
+
 /**
- * Convenience intermediate class, to be removed.
+ * The SaveFileManager is serving as a factor for InSaveFile
+ * and OutSaveFile objects.
+ *
+ * Engines and other code should use SaveFiles whenever they need to
+ * store data which they need to be able to retrieve later on --
+ * i.e. typically save states, but also configuration files and similar
+ * things.
+ *
+ * While not declared as a singleton,
+ * it is effectively used as such, with OSystem::getSavefileManager
+ * returning the single SaveFileManager instances to be used.
  */
-class SaveFile : public InSaveFile, public OutSaveFile {
-public:
-};
-
-class SaveFileManager {
+class SaveFileManager : NonCopyable {
 
 public:
 	virtual ~SaveFileManager() {}
@@ -81,19 +103,12 @@ public:
 
 	/**
 	 * Get the path to the save game directory.
-	 * Should only be used for error messages, and not to construct file paths
-	 * from it, since that is highl unportable.
+	 * Should only be used for error messages, *never* to construct file paths
+	 * from it, since that is highly unportable!
 	 */
 	virtual const char *getSavePath() const;
 };
 
-class DefaultSaveFileManager : public SaveFileManager {
-public:
-	virtual OutSaveFile *openForSaving(const char *filename);
-	virtual InSaveFile *openForLoading(const char *filename);
-	virtual void listSavefiles(const char * /* prefix */, bool *marks, int num);
-};
-
 } // End of namespace Common
 
 #endif
