@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2002-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,15 +18,14 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/common/stack.h $
- * $Id: stack.h 21472 2006-03-28 09:42:54Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/common/stack.h $
+ * $Id: stack.h 27024 2007-05-30 21:56:52Z fingolfin $
  */
 
 #ifndef COMMON_STACK_H
 #define COMMON_STACK_H
 
 #include "common/scummsys.h"
-#include <assert.h>
 #include "common/array.h"
 
 namespace Common {
@@ -49,23 +51,27 @@ public:
 		assert(_size < MAX_SIZE);
 		_stack[_size++] = x;
 	}
-	T top() const {
-		if (_size > 0)
-			return _stack[_size - 1];
-		else
-			return 0;
+	const T& top() const {
+		assert(_size > 0);
+		return _stack[_size - 1];
+	}
+	T& top() {
+		assert(_size > 0);
+		return _stack[_size - 1];
 	}
 	T pop() {
-		T tmp;
-		assert(_size > 0);
-		tmp = _stack[_size - 1];
-		_stack[--_size] = 0;
+		T tmp = top();
+		--_size;
 		return tmp;
 	}
 	int size() const {
 		return _size;
 	}
-	T operator [](int i) {
+	T& operator [](int i) {
+		assert(0 <= i && i < MAX_SIZE);
+		return _stack[i];
+	}
+	const T& operator [](int i) const {
 		assert(0 <= i && i < MAX_SIZE);
 		return _stack[i];
 	}
@@ -81,6 +87,7 @@ protected:
 	Array<T>	_stack;
 public:
 	Stack<T>() {}
+	Stack<T>(const Array<T> &stackContent) : _stack(stackContent) {}
 
 	bool empty() const {
 		return _stack.empty();
@@ -93,10 +100,7 @@ public:
 	}
 	T top() const {
 		const int s = size();
-		if (s > 0)
-			return _stack[s - 1];
-		else
-			return 0;
+		return _stack[s - 1];
 	}
 	T pop() {
 		T tmp = top();
