@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2004-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/common/stream.h $
- * $Id: stream.h 22178 2006-04-26 10:54:32Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/common/stream.h $
+ * $Id: stream.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -29,6 +32,7 @@
 namespace Common {
 
 class String;
+class MemoryReadStream;
 
 /**
  * Virtual base class for both ReadStream and WriteStream.
@@ -38,7 +42,7 @@ public:
 	virtual ~Stream() {}
 
 	/**
-	 * Returns true if any I/O failure occured.
+	 * Returns true if any I/O failure occurred.
 	 * This flag is never cleared automatically. In order to clear it,
 	 * client code has to call clearIOFailed() explicitly.
 	 *
@@ -150,57 +154,134 @@ public:
 	// The remaining methods all have default implementations; subclasses
 	// need not (and should not) overload them.
 
+	/**
+	 * Read am unsigned byte from the stream and return it.
+	 * Performs no error checking. The return value is undefined
+	 * if a read error occurred (for which client code can check by
+	 * calling ioFailed()).
+	 */
 	byte readByte() {
 		byte b = 0;
 		read(&b, 1);
 		return b;
 	}
 
+	/**
+	 * Read a signed byte from the stream and return it.
+	 * Performs no error checking. The return value is undefined
+	 * if a read error occurred (for which client code can check by
+	 * calling ioFailed()).
+	 */
 	int8 readSByte() {
 		int8 b = 0;
 		read(&b, 1);
 		return b;
 	}
 
+	/**
+	 * Read an unsigned 16-bit word stored in little endian (LSB first) order
+	 * from the stream and return it.
+	 * Performs no error checking. The return value is undefined
+	 * if a read error occurred (for which client code can check by
+	 * calling ioFailed()).
+	 */
 	uint16 readUint16LE() {
 		uint16 a = readByte();
 		uint16 b = readByte();
 		return a | (b << 8);
 	}
 
+	/**
+	 * Read an unsigned 32-bit word stored in little endian (LSB first) order
+	 * from the stream and return it.
+	 * Performs no error checking. The return value is undefined
+	 * if a read error occurred (for which client code can check by
+	 * calling ioFailed()).
+	 */
 	uint32 readUint32LE() {
 		uint32 a = readUint16LE();
 		uint32 b = readUint16LE();
 		return (b << 16) | a;
 	}
 
+	/**
+	 * Read an unsigned 16-bit word stored in big endian (MSB first) order
+	 * from the stream and return it.
+	 * Performs no error checking. The return value is undefined
+	 * if a read error occurred (for which client code can check by
+	 * calling ioFailed()).
+	 */
 	uint16 readUint16BE() {
 		uint16 b = readByte();
 		uint16 a = readByte();
 		return a | (b << 8);
 	}
 
+	/**
+	 * Read an unsigned 32-bit word stored in big endian (MSB first) order
+	 * from the stream and return it.
+	 * Performs no error checking. The return value is undefined
+	 * if a read error occurred (for which client code can check by
+	 * calling ioFailed()).
+	 */
 	uint32 readUint32BE() {
 		uint32 b = readUint16BE();
 		uint32 a = readUint16BE();
 		return (b << 16) | a;
 	}
 
+	/**
+	 * Read a signed 16-bit word stored in little endian (LSB first) order
+	 * from the stream and return it.
+	 * Performs no error checking. The return value is undefined
+	 * if a read error occurred (for which client code can check by
+	 * calling ioFailed()).
+	 */
 	int16 readSint16LE() {
 		return (int16)readUint16LE();
 	}
 
+	/**
+	 * Read a signed 32-bit word stored in little endian (LSB first) order
+	 * from the stream and return it.
+	 * Performs no error checking. The return value is undefined
+	 * if a read error occurred (for which client code can check by
+	 * calling ioFailed()).
+	 */
 	int32 readSint32LE() {
 		return (int32)readUint32LE();
 	}
 
+	/**
+	 * Read a signed 16-bit word stored in big endian (MSB first) order
+	 * from the stream and return it.
+	 * Performs no error checking. The return value is undefined
+	 * if a read error occurred (for which client code can check by
+	 * calling ioFailed()).
+	 */
 	int16 readSint16BE() {
 		return (int16)readUint16BE();
 	}
 
+	/**
+	 * Read a signed 32-bit word stored in big endian (MSB first) order
+	 * from the stream and return it.
+	 * Performs no error checking. The return value is undefined
+	 * if a read error occurred (for which client code can check by
+	 * calling ioFailed()).
+	 */
 	int32 readSint32BE() {
 		return (int32)readUint32BE();
 	}
+	
+	/**
+	 * Read the specified amount of data into a malloc'ed buffer
+	 * which then is wrapped into a MemoryReadStream.
+	 * The returned stream might contain less data than requested,
+	 * if reading more failed.
+	 */
+	MemoryReadStream *readStream(uint32 dataSize);
+
 };
 
 
@@ -210,14 +291,14 @@ public:
  * @todo We really need better error handling here!
  *       Like seek should somehow indicate whether it failed.
  */
-class SeekableReadStream : public ReadStream {
+class SeekableReadStream : virtual public ReadStream {
 public:
 
 	virtual uint32 pos() const = 0;
 	virtual uint32 size() const = 0;
 
 	virtual void seek(int32 offset, int whence = SEEK_SET) = 0;
-	
+
 	void skip(uint32 offset) { seek(offset, SEEK_CUR); }
 
 	/**
@@ -226,46 +307,56 @@ public:
 	 *
 	 * @param buf	the buffer to store into
 	 * @param bufSize	the size of the buffer
-	 * @return a pointer to the read string, or NULL if an error occured
+	 * @return a pointer to the read string, or NULL if an error occurred
 	 * @note The line terminator (CR or CR/LF) is stripped and not inserted
 	 *       into the buffer.
 	 */
 	virtual char *readLine(char *buf, size_t bufSize);
 };
 
-
 /**
- * XORReadStream is a wrapper around an arbitrary other ReadStream,
- * which 'decrypts' the data being read by XORing all data bytes with the given
- * encryption 'key'.
- *
- * Currently, this is not used anywhere, it's just a demo of how one can chain
- * streams if necessary.
+ * SubReadStream provides access to a ReadStream restricted to the range
+ * [currentPosition, currentPosition+end).
+ * Manipulating the parent stream directly /will/ mess up a substream.
+ * Likewise, manipulating two substreams of a parent stream will cause them to
+ * step on each others toes.
  */
-class XORReadStream : public ReadStream {
-private:
-	byte _encbyte;
-	ReadStream *_realStream;
+class SubReadStream : virtual public ReadStream {
+protected:
+	ReadStream *_parentStream;
+	uint32 _pos;
+	uint32 _end;
+	bool _disposeParentStream;
 public:
-	XORReadStream(ReadStream *in = 0, byte enc = 0) :  _encbyte(enc), _realStream(in) {}
-	void setStream(ReadStream *in) { _realStream = in; }
-	void setEnc(byte value) { _encbyte = value; }
-
-	virtual bool eos() const { return _realStream->eos(); }
-	virtual bool ioFailed() const { return _realStream->ioFailed(); }
-	virtual void clearIOFailed() { _realStream->clearIOFailed(); }
-
-	uint32 read(void *dataPtr, uint32 dataSize) {
-		assert(_realStream);
-		uint32 len = _realStream->read(dataPtr, dataSize);
-		if (_encbyte) {
-			byte *p = (byte *)dataPtr;
-			byte *end = p + len;
-			while (p < end)
-				*p++ ^= _encbyte;
-		}
-		return len;
+	SubReadStream(ReadStream *parentStream, uint32 end, bool disposeParentStream = false)
+		: _parentStream(parentStream),
+		  _pos(0),
+		  _end(end),
+		  _disposeParentStream(disposeParentStream) {}
+	~SubReadStream() {
+		if (_disposeParentStream) delete _parentStream;
 	}
+
+	virtual bool eos() const { return _pos == _end; }
+	virtual uint32 read(void *dataPtr, uint32 dataSize);
+};
+
+/*
+ * SeekableSubReadStream provides access to a SeekableReadStream restricted to
+ * the range [begin, end).
+ * The same caveats apply to SeekableSubReadStream as do to SeekableReadStream.
+ */
+class SeekableSubReadStream : public SubReadStream, public SeekableReadStream {
+protected:
+	SeekableReadStream *_parentStream;
+	uint32 _begin;
+public:
+	SeekableSubReadStream(SeekableReadStream *parentStream, uint32 begin, uint32 end, bool disposeParentStream = false);
+
+	virtual uint32 pos() const { return _pos - _begin; }
+	virtual uint32 size() const { return _end - _begin; }
+
+	virtual void seek(int32 offset, int whence = SEEK_SET);
 };
 
 /**
@@ -274,39 +365,40 @@ public:
  */
 class MemoryReadStream : public SeekableReadStream {
 private:
-	const byte *_ptr;
 	const byte * const _ptrOrig;
-	const uint32 _bufSize;
+	const byte *_ptr;
+	const uint32 _size;
 	uint32 _pos;
 	byte _encbyte;
+	bool _disposeMemory;
 
 public:
-	MemoryReadStream(const byte *buf, uint32 len) : _ptr(buf), _ptrOrig(buf), _bufSize(len), _pos(0), _encbyte(0) {}
+
+	/**
+	 * This constructor takes a pointer to a memory buffer and a length, and
+	 * wraps it. If disposeMemory is true, the MemoryReadStream takes ownership
+	 * of the buffer and hence free's it when destructed.
+	 */
+	MemoryReadStream(const byte *dataPtr, uint32 dataSize, bool disposeMemory = false) :
+		_ptrOrig(dataPtr),
+		_ptr(dataPtr),
+		_size(dataSize),
+		_pos(0),
+		_encbyte(0),
+		_disposeMemory(disposeMemory) {}
+
+	~MemoryReadStream() {
+		if (_disposeMemory)
+			free(const_cast<byte *>(_ptrOrig));
+	}
 
 	void setEnc(byte value) { _encbyte = value; }
 
-	uint32 read(void *dataPtr, uint32 dataSize) {
-		// Read at most as many bytes as are still available...
-		if (dataSize > _bufSize - _pos)
-			dataSize = _bufSize - _pos;
-		memcpy(dataPtr, _ptr, dataSize);
+	uint32 read(void *dataPtr, uint32 dataSize);
 
-		if (_encbyte) {
-			byte *p = (byte *)dataPtr;
-			byte *end = p + dataSize;
-			while (p < end)
-				*p++ ^= _encbyte;
-		}
-
-		_ptr += dataSize;
-		_pos += dataSize;
-
-		return dataSize;
-	}
-
-	bool eos() const { return _pos == _bufSize; }
+	bool eos() const { return _pos == _size; }
 	uint32 pos() const { return _pos; }
-	uint32 size() const { return _bufSize; }
+	uint32 size() const { return _size; }
 
 	void seek(int32 offs, int whence = SEEK_SET);
 };
@@ -318,11 +410,10 @@ public:
 class MemoryWriteStream : public WriteStream {
 private:
 	byte *_ptr;
-	const byte * const _ptrOrig;
 	const uint32 _bufSize;
 	uint32 _pos;
 public:
-	MemoryWriteStream(byte *buf, uint32 len) : _ptr(buf), _ptrOrig(buf), _bufSize(len), _pos(0) {}
+	MemoryWriteStream(byte *buf, uint32 len) : _ptr(buf), _bufSize(len), _pos(0) {}
 
 	uint32 write(const void *dataPtr, uint32 dataSize) {
 		// Write at most as many bytes as are still available...
