@@ -1,6 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2001  Ludvig Strigeus
- * Copyright (C) 2001-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -16,24 +18,33 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/common/system.cpp $
- * $Id: system.cpp 22370 2006-05-06 18:10:38Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/common/system.cpp $
+ * $Id: system.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
 #include "common/stdafx.h"
 
 #include "backends/intern.h"
+#include "backends/events/default/default-events.h"
 
 #include "gui/message.h"
 
 #include "common/config-manager.h"
-#include "common/savefile.h"
 #include "common/system.h"
+#include "common/timer.h"
 #include "common/util.h"
 
+#include "sound/mixer.h"
+
 OSystem *g_system = 0;
 
+OSystem::OSystem() {
+}
+
+OSystem::~OSystem() {
+}
+
 bool OSystem::setGraphicsMode(const char *name) {
 	if (!name)
 		return false;
@@ -61,10 +72,6 @@ void OSystem::displayMessageOnOSD(const char *msg) {
 	dialog.runModal();
 }
 
-Common::SaveFileManager *OSystem::getSavefileManager() {
-	return new Common::DefaultSaveFileManager();
-}
-
 
 bool OSystem::openCD(int drive) {
 	return false;
@@ -83,3 +90,14 @@ void OSystem::stopCD() {
 void OSystem::updateCD() {
 }
 
+static Common::EventManager *s_eventManager = 0;
+
+Common::EventManager *OSystem::getEventManager() {
+	// FIXME/TODO: Eventually this method should be turned into an abstract one,
+	// to force backends to implement this conciously (even if they 
+	// end up returning the default event manager anyway).
+	if (!s_eventManager)
+		s_eventManager = new DefaultEventManager(this);
+	return s_eventManager;
+}
+
