@@ -1,6 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2001  Ludvig Strigeus
- * Copyright (C) 2001-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -16,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/common/system.h $
- * $Id: system.h 23987 2006-09-25 21:03:24Z agent-q $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/common/system.h $
+ * $Id: system.h 27069 2007-06-03 18:44:03Z knakos $
  *
  */
 
@@ -26,14 +28,22 @@
 
 #include "common/scummsys.h"
 #include "common/mutex.h"
+#include "common/noncopyable.h"
 #include "common/rect.h"
 
+namespace Audio {
+	class Mixer;
+}
+
 namespace Graphics {
 	struct Surface;
 }
 
 namespace Common {
+	struct Event;
+	class EventManager;
 	class SaveFileManager;
+	class TimerManager;
 }
 
 /**
@@ -46,21 +56,20 @@ namespace Common {
  * methods to create timers, to handle user input events,
  * control audio CD playback, and sound output.
  */
-class OSystem {
-private:
-	// Prevent copying OSystem objects by accident.
-	OSystem(const OSystem&);
-	OSystem& operator= (const OSystem&);
-
+class OSystem : Common::NonCopyable {
 protected:
-	OSystem() { }
-	virtual ~OSystem() { }
+	OSystem();
+	virtual ~OSystem();
 
 public:
 
 	/**
 	 * The following method is called once, from main.cpp, after all
 	 * config data (including command line params etc.) are fully loaded.
+	 *
+	 * @note Subclasses should always invoke the implementation of their
+	 *       parent class. They should do so near the end of their own
+	 *       implementation.
 	 */
 	virtual void initBackend() { }
 
@@ -116,7 +125,7 @@ public:
 		kFeatureAutoComputeDirtyRects,
 
 		/**
-		 * This flags determines either cursor can have its own palette or not
+		 * This flag determines either cursor can have its own palette or not
 		 * It is currently used only by some Macintosh versions of Humongous
 		 * Entertainment games. If backend doesn't implement this feature then
 		 * engine switches to b/w version of cursors.
@@ -128,7 +137,27 @@ public:
 		 * This should only be set if it offers at least 3-4 bits of accuracy,
 		 * as opposed to a single alpha bit.
 		 */
-		kFeatureOverlaySupportsAlpha
+		kFeatureOverlaySupportsAlpha,
+
+		/**
+		 * Set to true to iconify the window.
+		 */
+		kFeatureIconifyWindow,
+
+		/**
+		 * This feature, set to true, is a hint toward the backend to disable all
+		 * key filtering/mapping, in cases where it would be beneficial to do so.
+		 * As an example case, this is used in the agi engine's predictive dialog.
+		 * When the dialog is displayed this feature is set so that backends with
+		 * phone-like keypad temporarily unmap all user actions which leads to
+		 * comfortable word entry. Conversely, when the dialog exits the feature
+		 * is set to false.
+		 * TODO: Fingolfin suggests that the way the feature is used can be 
+		 * generalized in this sense: Have a keyboard mapping feature, which the
+		 * engine queries for to assign keys to actions ("Here's my default key
+		 * map for these actions, what do you want them set to?").
+		 */
+		kFeatureDisableKeyFiltering
 	};
 
 	/**
@@ -309,6 +338,21 @@ public:
 	 */
 	virtual void initSize(uint width, uint height) = 0;
 
+	/**
+	 * Return an int value which is changed whenever any screen
+	 * parameters (like the resolution) change. That is, whenever a
+	 * EVENT_SCREEN_CHANGED would be sent. You can track this value
+	 * in your code to detect screen changes in case you do not have
+	 * full control over the event loop(s) being used (like the GUI
+	 * code).
+	 *
+	 * @return an integer which can be used to track screen changes
+	 *
+	 * @note Backends which generate EVENT_SCREEN_CHANGED events MUST
+	 *       overload this method appropriately.
+	 */
+	virtual int getScreenChangeID() const { return 0; }
+
 	/**
 	 * Begin a new GFX transaction, which is a sequence of GFX mode changes.
 	 * The idea behind GFX transactions is to make it possible to activate
@@ -325,14 +369,14 @@ public:
 	 *       of the relevant methods simply do nothing.
 	 * @see endGFXTransaction
 	 */
-	virtual void beginGFXTransaction() {};
+	virtual void beginGFXTransaction() {}
 
 
 	/**
 	 * End (and thereby commit) the current GFX transaction.
 	 * @see beginGFXTransaction
 	 */
-	virtual void endGFXTransaction() {};
+	virtual void endGFXTransaction() {}
 
 
 	/**
@@ -439,7 +483,7 @@ public:
 	 *       and instead implement the functionality in the frontend.
 	 */
 	virtual void setShakePos(int shakeOffset) = 0;
-
+		
 	/**
 	 * Sets the area of the screen that has the focus.  For example, when a character
 	 * is speaking, they will have the focus.  Allows for pan-and-scan style views
@@ -462,7 +506,6 @@ public:
 	 */
 	virtual void clearFocusRectangle() {}
 
-
 	//@}
 
 
@@ -630,7 +673,7 @@ public:
 	 * @see setPalette
 	 * @see kFeatureCursorHasPalette
 	 */
-	virtual void setCursorPalette(const byte *colors, uint start, uint num) {};
+	virtual void setCursorPalette(const byte *colors, uint start, uint num) {}
 
 	/**
 	 * Disable or enable cursor palette.
@@ -642,7 +685,7 @@ public:
 	 * @see setPalette
 	 * @see kFeatureCursorHasPalette
 	 */
-	virtual void disableCursorPalette(bool disable) {};
+	virtual void disableCursorPalette(bool disable) {}
 
 	//@}
 
@@ -651,122 +694,17 @@ public:
 	/** @name Events and Time */
 	//@{
 
-	typedef int (*TimerProc)(int interval);
-
-	/**
-	 * The types of events backends may generate.
-	 * @see Event
-	 *
-	 * @todo Merge EVENT_LBUTTONDOWN, EVENT_RBUTTONDOWN and EVENT_WHEELDOWN;
-	 *       likewiese EVENT_LBUTTONUP, EVENT_RBUTTONUP, EVENT_WHEELUP.
-	 *       To do that, we just have to add a field to the Event which
-	 *       indicates which button was pressed.
-	 */
-	enum EventType {
-		/** A key was pressed, details in Event::kbd. */
-		EVENT_KEYDOWN = 1,
-		/** A key was released, details in Event::kbd. */
-		EVENT_KEYUP = 2,
-		/** The mouse moved, details in Event::mouse. */
-		EVENT_MOUSEMOVE = 3,
-		EVENT_LBUTTONDOWN = 4,
-		EVENT_LBUTTONUP = 5,
-		EVENT_RBUTTONDOWN = 6,
-		EVENT_RBUTTONUP = 7,
-		EVENT_WHEELUP = 8,
-		EVENT_WHEELDOWN = 9,
-
-		EVENT_QUIT = 10,
-		EVENT_SCREEN_CHANGED = 11
-	};
-
-	/**
-	 * Keyboard modifier flags, used for Event::kbd::flags.
-	 */
-	enum {
-		KBD_CTRL  = 1 << 0,
-		KBD_ALT   = 1 << 1,
-		KBD_SHIFT = 1 << 2
-	};
-
-	/**
-	 * Data structure for an event. A pointer to an instance of Event
-	 * can be passed to pollEvent.
-	 * @todo Rework/document this structure. It should be made 100% clear which
-	 *       field is valid for which event type.
-	 *       Implementation wise, we might want to use the classic
-	 *       union-of-structs trick. It goes roughly like this:
-	 *       struct BasicEvent {
-	 *       	EventType type;
-	 *       };
-	 *       struct MouseMovedEvent : BasicEvent {
-	 *       	Common::Point pos;
-	 *       };
-	 *       struct MouseButtonEvent : MouseMovedEvent {
-	 *       	int button;
-	 *       };
-	 *       struct KeyEvent : BasicEvent {
-	 *       	...
-	 *       };
-	 *       ...
-	 *       union Event {
-	 *          EventType type;
-	 *       	MouseMovedEvent mouse;
-	 *       	MouseButtonEvent button;
-	 *       	KeyEvent key;
-	 *       	...
-	 *       };
-	 */
-	struct Event {
-		/** The type of the event. */
-		EventType type;
-		/**
-		  * Keyboard data; only valid for keyboard events (EVENT_KEYDOWN and
-		  * EVENT_KEYUP). For all other event types, content is undefined.
-		  */
-		struct {
-			/**
-			 * Abstract key code (will be the same for any given key regardless
-			 * of modifiers being held at the same time.
-			 * For example, this is the same for both 'A' and Shift-'A'.
-			 * @todo Document which values are to be used for non-ASCII keys
-			 * like F1-F10. For now, let's just say that our primary backend
-			 * is the SDL one, and it uses the values SDL uses... so until
-			 * we fix this, your best bet is to get a copy of SDL_keysym.h
-			 * and look at that, if you want to find out a key code.
-			 */
-			int keycode;
-			/**
-			 * ASCII-value of the pressed key (if any).
-			 * This depends on modifiers, i.e. pressing the 'A' key results in
-			 * different values here depending on the status of shift, alt and
-			 * caps lock.
-			 * For the function keys F1-F9, values of 315-323 are used.
-			 */
-			uint16 ascii;
-			/**
-			 * Status of the modifier keys. Bits are set in this for each
-			 * pressed modifier
-			 * @see KBD_CTRL, KBD_ALT, KBD_SHIFT
-			 */
-			byte flags;
-		} kbd;
-		/**
-		 * The mouse coordinates, in virtual screen coordinates. Only valid
-		 * for mouse events.
-		 * Virtual screen coordinates means: the coordinate system of the
-		 * screen area as defined by the most recent call to initSize().
-		 */
-		Common::Point mouse;
-	};
+protected:
+	friend class DefaultEventManager;
 
 	/**
 	 * Get the next event in the event queue.
-	 * @param event	point to an Event struct, which will be filled with the event data.
+	 * @param event	point to an Common::Event struct, which will be filled with the event data.
 	 * @return true if an event was retrieved.
 	 */
-	virtual bool pollEvent(Event &event) = 0;
+	virtual bool pollEvent(Common::Event &event) = 0;
 
+public:
 	/** Get the number of milliseconds since the program was started. */
 	virtual uint32 getMillis() = 0;
 
@@ -774,24 +712,16 @@ public:
 	virtual void delayMillis(uint msecs) = 0;
 
 	/**
-	 * Set the timer callback, a function which is periodically invoked by the
-	 * backend. This can for example be done via a background thread.
-	 * There is at most one active timer; if this method is called while there
-	 * is already an active timer, then the new timer callback should replace
-	 * the previous one. In particular, passing a callback pointer value of 0
-	 * is legal and can be used to clear the current timer callback.
-	 * @see Common::Timer
-	 * @note The implementation of this method must be 'atomic' in the sense
-	 *       that when the method returns, the previously set callback must
-	 *       not be in use anymore (in particular, if timers are implemented
-	 *       via threads, then it must be ensured that the timer thread is
-	 *       not using the old callback function anymore).
-	 *
-	 * @param callback	pointer to the callback. May be 0 to reset the timer
-	 * @param interval	the interval (in milliseconds) between invocations
-	 *                  of the callback
+	 * Return the timer manager singleton. For more information, refer
+	 * to the TimerManager documentation.
 	 */
-	virtual void setTimerCallback(TimerProc callback, int interval) = 0;
+	virtual Common::TimerManager *getTimerManager() = 0;
+
+	/**
+	 * Return the event manager singleton. For more information, refer
+	 * to the EventManager documentation.
+	 */
+	virtual Common::EventManager *getEventManager();
 
 	//@}
 
@@ -846,22 +776,12 @@ public:
 
 	/** @name Sound */
 	//@{
-	typedef void (*SoundProc)(void *param, byte *buf, int len);
 
 	/**
-	 * Set the audio callback which is invoked whenever samples need to be generated.
-	 * Currently, only the 16-bit signed mode is ever used for Simon & Scumm
-	 * @param proc		pointer to the callback.
-	 * @param param		an arbitrary parameter which is stored and passed to proc.
+	 * Returh the audio mixer. For more information, refer to the
+	 * Audio::Mixer documentation.
 	 */
-	virtual bool setSoundCallback(SoundProc proc, void *param) = 0;
-
-	/**
-	 * Remove any audio callback previously set via setSoundCallback, thus effectively
-	 * stopping all audio output immediately.
-	 * @see setSoundCallback
-	 */
-	virtual void clearSoundCallback() = 0;
+	virtual Audio::Mixer *getMixer() = 0;
 
 	/**
 	 * Determine the output sample rate. Audio data provided by the sound
@@ -948,8 +868,12 @@ public:
 	 */
 	virtual void displayMessageOnOSD(const char *msg);
 
-	/** Savefile management. */
-	virtual Common::SaveFileManager *getSavefileManager();
+	/**
+	 * Return the SaveFileManager, used to store and load savestates
+	 * and other modifiable persistent game data. For more information,
+	 * refer to the SaveFileManager documentation.
+	 */
+	virtual Common::SaveFileManager *getSavefileManager() = 0;
 
 	//@}
 };
@@ -958,5 +882,4 @@ public:
 /** The global OSystem instance. Initialised in main(). */
 extern OSystem *g_system;
 
-
 #endif
