@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2002-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,48 +18,23 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/common/timer.h $
- * $Id: timer.h 20510 2006-02-11 09:53:53Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/common/timer.h $
+ * $Id: timer.h 27024 2007-05-30 21:56:52Z fingolfin $
  */
 
 #ifndef COMMON_TIMER_H
 #define COMMON_TIMER_H
 
 #include "common/scummsys.h"
-#include "common/mutex.h"
-
-#define MAX_TIMERS 8
-
-
-#ifdef __MORPHOS__
-#include "morphos_timer.h"
-#else
-
-class OSystem;
+#include "common/noncopyable.h"
 
 namespace Common {
 
-class Timer {
+class TimerManager : NonCopyable {
 public:
 	typedef void (*TimerProc)(void *refCon);
 
-private:
-	OSystem *_system;
-	Mutex _mutex;
-	void *_timerHandler;
-	int32 _thisTime;
-	int32 _lastTime;
-
-	struct TimerSlots {
-		TimerProc procedure;
-		int32 interval;
-		int32 counter;
-		void *refCon;
-	} _timerSlots[MAX_TIMERS];
-
-public:
-	Timer(OSystem *system);
-	~Timer();
+	virtual ~TimerManager() {}
 
 	/**
 	 * Install a new timer callback. It will from now be called every interval microseconds.
@@ -70,22 +48,14 @@ public:
 	 * @param refCon	an arbitrary void pointer; will be passed to the timer callback
 	 * @return	true if the timer was installed successfully, false otherwise
 	 */
-	bool installTimerProc(TimerProc proc, int32 interval, void *refCon);
+	virtual bool installTimerProc(TimerProc proc, int32 interval, void *refCon) = 0;
 
 	/**
 	 * Remove the given timer callback. It will not be invoked anymore.
 	 */
-	void removeTimerProc(TimerProc proc);
-
-protected:
-	static int timer_handler(int t);
-	int handler(int t);
+	virtual void removeTimerProc(TimerProc proc) = 0;
 };
 
-extern Timer *g_timer;
-
 } // End of namespace Common
 
 #endif
-
-#endif
