@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/common/unzip.cpp $
- * $Id: unzip.cpp 23019 2006-06-11 02:23:17Z vinterstum $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/common/unzip.cpp $
+ * $Id: unzip.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  */
 
 /* unzip.c -- IO on .zip files using zlib
@@ -120,7 +123,7 @@ typedef struct
 	uInt  size_local_extrafield;/* size of the local extra field */
 	uLong pos_local_extrafield;   /* position in the local extra field in read*/
 
-	uLong crc32;                /* crc32 of all data uncompressed */
+	uLong crc32_data;                /* crc32 of all data uncompressed */
 	uLong crc32_wait;           /* crc32 we must obtain after decompress all */
 	uLong rest_read_compressed; /* number of byte to be decompressed */
 	uLong rest_read_uncompressed;/*number of byte to be obtained after decomp*/
@@ -884,7 +887,7 @@ extern int ZEXPORT unzOpenCurrentFile (unzFile file)
 	Store = s->cur_file_info.compression_method==0;
 
 	pfile_in_zip_read_info->crc32_wait=s->cur_file_info.crc;
-	pfile_in_zip_read_info->crc32=0;
+	pfile_in_zip_read_info->crc32_data=0;
 	pfile_in_zip_read_info->compression_method =
             s->cur_file_info.compression_method;
 	pfile_in_zip_read_info->file=&s->file;
@@ -1003,7 +1006,7 @@ extern int ZEXPORT unzReadCurrentFile  (unzFile file, voidp buf, unsigned len)
 				*(pfile_in_zip_read_info->stream.next_out+i) =
                         *(pfile_in_zip_read_info->stream.next_in+i);
 
-			pfile_in_zip_read_info->crc32 = crc32(pfile_in_zip_read_info->crc32,
+			pfile_in_zip_read_info->crc32_data = crc32(pfile_in_zip_read_info->crc32_data,
 								pfile_in_zip_read_info->stream.next_out,
 								uDoCopy);
 			pfile_in_zip_read_info->rest_read_uncompressed-=uDoCopy;
@@ -1035,8 +1038,8 @@ extern int ZEXPORT unzReadCurrentFile  (unzFile file, voidp buf, unsigned len)
 			uTotalOutAfter = pfile_in_zip_read_info->stream.total_out;
 			uOutThis = uTotalOutAfter-uTotalOutBefore;
 
-			pfile_in_zip_read_info->crc32 =
-                crc32(pfile_in_zip_read_info->crc32,bufBefore,
+			pfile_in_zip_read_info->crc32_data =
+                crc32(pfile_in_zip_read_info->crc32_data,bufBefore,
                         (uInt)(uOutThis));
 
 			pfile_in_zip_read_info->rest_read_uncompressed -=
@@ -1172,7 +1175,7 @@ extern int ZEXPORT unzCloseCurrentFile (unzFile file)
 
 	if (pfile_in_zip_read_info->rest_read_uncompressed == 0)
 	{
-		if (pfile_in_zip_read_info->crc32 != pfile_in_zip_read_info->crc32_wait)
+		if (pfile_in_zip_read_info->crc32_data != pfile_in_zip_read_info->crc32_wait)
 			err=UNZ_CRCERROR;
 	}
 
