@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2002-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,13 +18,47 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/common/util.cpp $
- * $Id: util.cpp 21812 2006-04-11 22:31:47Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/common/util.cpp $
+ * $Id: util.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  */
 
 #include "common/stdafx.h"
-#include "base/engine.h"
+#include "engines/engine.h"
 #include "common/util.h"
+#include "common/system.h"
+#include "gui/debugger.h"
+
+#ifdef _WIN32_WCE
+// This is required for the debugger attachment
+extern bool isSmartphone(void);
+#endif
+
+#ifdef __PLAYSTATION2__
+	// for those replaced fopen/fread/etc functions
+	typedef unsigned long	uint64;
+	typedef signed long	int64;
+	#include "backends/platform/ps2/fileio.h"
+
+	#define fprintf				ps2_fprintf
+	#define fflush(a)			ps2_fflush(a)
+#endif
+
+#ifdef __DS__
+	#undef stderr
+	#undef stdout
+	#undef stdin
+
+	#define stdout ((DS::fileHandle*) -1)
+	#define stderr ((DS::fileHandle*) -2)
+	#define stdin ((DS::fileHandle*) -3)
+	
+	void 	std_fprintf(FILE* handle, const char* fmt, ...);
+	void 	std_fflush(FILE* handle);
+
+	#define fprintf(file, fmt, ...)				{ char str[128]; sprintf(str, fmt, ##__VA_ARGS__); DS::std_fwrite(str, strlen(str), 1, file); }
+	#define fflush(file)						DS::std_fflush(file)
+#endif
+
 
 namespace Common {
 
@@ -110,24 +147,24 @@ uint RandomSource::getRandomNumberRng(uint min, uint max) {
 
 
 const LanguageDescription g_languages[] = {
-	{"en", "English", EN_ANY}, // Generic English (when only one game version exist)
-	{"us", "English (US)", EN_USA},
-	{"gb", "English (GB)", EN_GRB},
-	{"de", "German", DE_DEU},
-	{"fr", "French", FR_FRA},
-	{"it", "Italian", IT_ITA},
-	{"pt", "Portuguese", PT_BRA},
-	{"es", "Spanish", ES_ESP},
-	{"jp", "Japanese", JA_JPN},
 	{"zh", "Chinese (Taiwan)", ZH_TWN},
-	{"kr", "Korean", KO_KOR},
-	{"se", "Swedish", SE_SWE},
-	{"hb", "Hebrew", HB_ISR},
-	{"ru", "Russian", RU_RUS},
 	{"cz", "Czech", CZ_CZE},
 	{"nl", "Dutch", NL_NLD},
+	{"en", "English", EN_ANY}, // Generic English (when only one game version exist)
+	{"gb", "English (GB)", EN_GRB},
+	{"us", "English (US)", EN_USA},
+	{"fr", "French", FR_FRA},
+	{"de", "German", DE_DEU},
+	{"hb", "Hebrew", HB_ISR},
+	{"it", "Italian", IT_ITA},
+	{"jp", "Japanese", JA_JPN},
+	{"kr", "Korean", KO_KOR},
 	{"nb", "Norwegian Bokm\xE5l", NB_NOR},
 	{"pl", "Polish", PL_POL},
+	{"br", "Portuguese", PT_BRA},
+	{"ru", "Russian", RU_RUS},
+	{"es", "Spanish", ES_ESP},
+	{"se", "Swedish", SE_SWE},
 	{0, 0, UNK_LANG}
 };
 
@@ -168,25 +205,28 @@ const char *getLanguageDescription(Language id) {
 
 
 const PlatformDescription g_platforms[] = {
-	{"3do", "3do", "3DO", kPlatform3DO},
-	{"acorn", "acorn", "Acorn", kPlatformAcorn},
-	{"amiga", "ami", "Amiga", kPlatformAmiga},
-	{"atari", "atari-st", "Atari ST", kPlatformAtariST},
-	{"c64", "c64", "Commodore 64", kPlatformC64},
-	{"pc", "dos", "DOS", kPlatformPC},
+	{"2gs", "2gs", "2gs", "Apple IIgs", kPlatformApple2GS },
+	{"3do", "3do", "3do", "3DO", kPlatform3DO},
+	{"acorn", "acorn", "acorn", "Acorn", kPlatformAcorn},
+	{"amiga", "ami", "amiga", "Amiga", kPlatformAmiga},
+	{"atari", "atari-st", "st", "Atari ST", kPlatformAtariST},
+	{"c64", "c64", "c64", "Commodore 64", kPlatformC64},
+	{"pc", "dos", "ibm", "DOS", kPlatformPC},
 
 	// The 'official' spelling seems to be "FM-TOWNS" (e.g. in the Indy4 demo).
 	// However, on the net many variations can be seen, like "FMTOWNS",
 	// "FM TOWNS", "FmTowns", etc.
-	{"fmtowns", "towns", "FM-TOWNS", kPlatformFMTowns},
+	{"fmtowns", "towns", "fm", "FM-TOWNS", kPlatformFMTowns},
 
-	{"linux", "linux", "Linux", kPlatformLinux},
-	{"macintosh", "mac", "Macintosh", kPlatformMacintosh},
-	{"nes", "nes", "NES", kPlatformNES},
-	{"segacd", "segacd", "SegaCD", kPlatformSegaCD},
-	{"windows", "win", "Windows", kPlatformWindows},
+	{"linux", "linux", "linux", "Linux", kPlatformLinux},
+	{"macintosh", "mac", "mac", "Macintosh", kPlatformMacintosh},
+	{"pce", "pce", "pce", "PC-Engine", kPlatformPCEngine },
+	{"nes", "nes", "nes", "NES", kPlatformNES},
+	{"segacd", "segacd", "sega", "SegaCD", kPlatformSegaCD},
+	{"windows", "win", "win", "Windows", kPlatformWindows},
 
-	{0, 0, "Default", kPlatformUnknown}
+
+	{0, 0, 0, "Default", kPlatformUnknown}
 };
 
 Platform parsePlatform(const String &str) {
@@ -206,7 +246,7 @@ Platform parsePlatform(const String &str) {
 
 	const PlatformDescription *l = g_platforms;
 	for (; l->code; ++l) {
-		if (!scumm_stricmp(l->code, s) || !scumm_stricmp(l->code2, s))
+		if (!scumm_stricmp(l->code, s) || !scumm_stricmp(l->code2, s) || !scumm_stricmp(l->abbrev, s))
 			return l->id;
 	}
 
@@ -223,6 +263,15 @@ const char *getPlatformCode(Platform id) {
 	return 0;
 }
 
+const char *getPlatformAbbrev(Platform id) {
+	const PlatformDescription *l = g_platforms;
+	for (; l->code; ++l) {
+		if (l->id == id)
+			return l->abbrev;
+	}
+	return 0;
+}
+
 const char *getPlatformDescription(Platform id) {
 	const PlatformDescription *l = g_platforms;
 	for (; l->code; ++l) {
@@ -279,8 +328,8 @@ const char *getRenderModeDescription(RenderMode id) {
 
 #pragma mark -
 
-Array<EngineDebugLevel> gDebugLevels;
-uint32 gDebugLevelsEnabled = 0;
+static Array<EngineDebugLevel> gDebugLevels;
+static uint32 gDebugLevelsEnabled = 0;
 
 bool addSpecialDebugLevel(uint32 level, const String &option, const String &description) {
 	for (uint i = 0; i < gDebugLevels.size(); ++i) {
@@ -313,7 +362,7 @@ bool enableSpecialDebugLevel(const String &option) {
 void enableSpecialDebugLevelList(const String &option) {
 	uint start = 0;
 	uint end = 0;
-	
+
 	const char *str = option.c_str();
 	for (end = start + 1; end <= option.size(); ++end) {
 		if (str[end] == ',' || end == option.size()) {
@@ -340,6 +389,11 @@ const Array<EngineDebugLevel> &listSpecialDebugLevels() {
 	return gDebugLevels;
 }
 
+uint32 getEnabledSpecialDebugLevels() {
+	return gDebugLevelsEnabled;
+}
+
+
 }	// End of namespace Common
 
 
@@ -354,13 +408,20 @@ int gDebugLevel = -1;
 
 
 
-static void debugHelper(char *buf, bool caret = true) {
-#ifndef _WIN32_WCE
+static void debugHelper(const char *in_buf, bool caret = true) {
+	char buf[STRINGBUFLEN];
+
+	// Next, give the active engine (if any) a chance to augment the message
+	if (g_engine) {
+		g_engine->errorString(in_buf, buf);
+	} else {
+		strcpy(buf, in_buf);
+	}
+
 	if (caret)
 		printf("%s\n", buf);
 	else
 		printf("%s", buf);
-#endif
 
 #if defined( USE_WINDBG )
 	if (caret)
@@ -429,3 +490,105 @@ void CDECL debugC(int level, uint32 engine_level, const char *s, ...) {
 
 	debugHelper(buf);
 }
+
+void NORETURN CDECL error(const char *s, ...) {
+	char buf_input[STRINGBUFLEN];
+	char buf_output[STRINGBUFLEN];
+	va_list va;
+
+	// Generate the full error message
+	va_start(va, s);
+	vsnprintf(buf_input, STRINGBUFLEN, s, va);
+	va_end(va);
+
+
+	// Next, give the active engine (if any) a chance to augment the message
+	if (g_engine) {
+		g_engine->errorString(buf_input, buf_output);
+	} else {
+		strcpy(buf_output, buf_input);
+	}
+
+
+	// Print the error message to stderr
+#ifdef __GP32__
+	printf("ERROR: %s\n", buf_output);
+#else
+	fprintf(stderr, "%s!\n", buf_output);
+#endif
+
+
+#ifndef __GP32__
+	// Unless this error -originated- within the debugger itself, we
+	// now invoke the debugger, if available / supported.
+	if (g_engine) {
+		GUI::Debugger *debugger = g_engine->getDebugger();
+#ifdef _WIN32_WCE
+		if (isSmartphone())
+			debugger = 0;
+#endif
+		if (debugger && !debugger->isAttached()) {
+			debugger->attach(buf_output);
+			debugger->onFrame();
+		}
+	}
+#endif
+
+
+#if defined( USE_WINDBG )
+#if defined( _WIN32_WCE )
+	TCHAR buf_output_unicode[1024];
+	MultiByteToWideChar(CP_ACP, 0, buf_output, strlen(buf_output) + 1, buf_output_unicode, sizeof(buf_output_unicode));
+	OutputDebugString(buf_output_unicode);
+#ifndef DEBUG
+	drawError(buf_output);
+#else
+	int cmon_break_into_the_debugger_if_you_please = *(int *)(buf_output + 1);	// bus error
+	printf("%d", cmon_break_into_the_debugger_if_you_please);			// don't optimize the int out
+#endif
+#else
+	OutputDebugString(buf_output);
+#endif
+#endif
+
+#ifdef PALMOS_MODE
+	extern void PalmFatalError(const char *err);
+	PalmFatalError(buf_output);
+#endif
+
+#ifdef __SYMBIAN32__
+	Symbian::FatalError(buf_output);
+#endif
+	// Finally exit. quit() will terminate the program if g_system is present
+	if (g_system)
+		g_system->quit();
+
+	exit(1);
+}
+
+void CDECL warning(const char *s, ...) {
+	char buf[STRINGBUFLEN];
+	va_list va;
+
+	va_start(va, s);
+	vsnprintf(buf, STRINGBUFLEN, s, va);
+	va_end(va);
+
+#ifdef __GP32__ //ph0x FIXME: implement fprint?
+	printf("WARNING: %s\n", buf);
+#else
+#if !defined (__SYMBIAN32__)
+	fprintf(stderr, "WARNING: %s!\n", buf);
+#endif
+#endif
+#if defined( USE_WINDBG )
+	strcat(buf, "\n");
+#if defined( _WIN32_WCE )
+	TCHAR buf_unicode[1024];
+	MultiByteToWideChar(CP_ACP, 0, buf, strlen(buf) + 1, buf_unicode, sizeof(buf_unicode));
+	OutputDebugString(buf_unicode);
+#else
+	OutputDebugString(buf);
+#endif
+#endif
+}
