@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2002-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/common/util.h $
- * $Id: util.h 22079 2006-04-22 02:49:27Z sev $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/common/util.h $
+ * $Id: util.h 27024 2007-05-30 21:56:52Z fingolfin $
  */
 
 #ifndef COMMON_UTIL_H
@@ -37,7 +40,7 @@
 template<typename T> inline T ABS (T x)			{ return (x>=0) ? x : -x; }
 template<typename T> inline T MIN (T a, T b)	{ return (a<b) ? a : b; }
 template<typename T> inline T MAX (T a, T b)	{ return (a>b) ? a : b; }
-template<typename T> inline T CLIP (T v, T amin, T amax) 
+template<typename T> inline T CLIP (T v, T amin, T amax)
 		{ if (v < amin) return amin; else if (v > amax) return amax; else return v; }
 
 /**
@@ -89,25 +92,24 @@ public:
  * List of game language.
  */
 enum Language {
-	EN_ANY,     // Generic English (when only one game version exist)
-	EN_USA,
-	EN_GRB,
-
-	DE_DEU,
-	FR_FRA,
-	IT_ITA,
-	PT_BRA,
-	ES_ESP,
-	JA_JPN,
 	ZH_TWN,
-	KO_KOR,
-	SE_SWE,
-	HB_ISR,
-	RU_RUS,
 	CZ_CZE,
 	NL_NLD,
+	EN_ANY,     // Generic English (when only one game version exist)
+	EN_GRB,
+	EN_USA,
+	FR_FRA,
+	DE_DEU,
+	HB_ISR,
+	IT_ITA,
+	JA_JPN,
+	KO_KOR,
 	NB_NOR,
 	PL_POL,
+	PT_BRA,
+	RU_RUS,
+	ES_ESP,
+	SE_SWE,
 
 	UNK_LANG = -1	// Use default language (i.e. none specified)
 };
@@ -145,7 +147,9 @@ enum Platform {
 	kPlatformAcorn,
 	kPlatformSegaCD,
 	kPlatform3DO,
-//	kPlatformPCEngine,
+	kPlatformPCEngine,
+
+	kPlatformApple2GS,
 
 	kPlatformUnknown = -1
 };
@@ -153,6 +157,7 @@ enum Platform {
 struct PlatformDescription {
 	const char *code;
 	const char *code2;
+	const char *abbrev;
 	const char *description;
 	Common::Platform id;
 };
@@ -162,6 +167,7 @@ extern const PlatformDescription g_platforms[];
 /** Convert a string containing a platform name into a Platform enum value. */
 extern Platform parsePlatform(const String &str);
 extern const char *getPlatformCode(Platform id);
+extern const char *getPlatformAbbrev(Platform id);
 extern const char *getPlatformDescription(Platform id);
 
 /**
@@ -179,7 +185,7 @@ enum RenderMode {
 	kRenderAmiga = 5
 };
 
-enum HerculesDimesnions {
+enum HerculesDimensions {
 	kHercW = 720,
 	kHercH = 350
 };
@@ -258,6 +264,13 @@ bool disableSpecialDebugLevel(const String &option);
  */
 const Array<EngineDebugLevel> &listSpecialDebugLevels();
 
+/**
+ * Return the active debug flag mask (i.e. all active debug flags ORed
+ * together into a single uint32).
+ */
+uint32 getEnabledSpecialDebugLevels();
+
+
 }	// End of namespace Common
 
 
@@ -274,8 +287,6 @@ void CDECL debug(const char *s, ...) GCC_PRINTF(1, 2);
 void CDECL debugN(int level, const char *s, ...) GCC_PRINTF(2, 3);
 void CDECL debugC(int level, uint32 engine_level, const char *s, ...) GCC_PRINTF(3, 4);
 
-void checkHeap();
-
 extern int gDebugLevel;
 
 
