@@ -1,7 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
  *
- * Copyright (C) 1999-2001 Sarien Team
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -17,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/agi/agi_v2.cpp $
- * $Id: agi_v2.cpp 22614 2006-05-24 19:51:37Z eriktorbjorn $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/agi/agi_v2.cpp $
+ * $Id: agi_v2.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -29,38 +30,30 @@
 
 namespace Agi {
 
-static int agi_v2_init(void);
-static int agi_v2_deinit(void);
-static int agi_v2_detect_game();
-static int agi_v2_load_resource(int, int);
-static int agi_v2_unload_resource(int, int);
-static int agi_v2_load_objects(const char *);
-static int agi_v2_load_words(const char *);
+int AgiLoader_v2::version() {
+	return 2;
+}
 
-struct agi_loader agi_v2 = {
-	2,
-	0,
-	agi_v2_init,
-	agi_v2_deinit,
-	agi_v2_detect_game,
-	agi_v2_load_resource,
-	agi_v2_unload_resource,
-	agi_v2_load_objects,
-	agi_v2_load_words
-};
+void AgiLoader_v2::setIntVersion(int ver) {
+	_intVersion = ver;
+}
 
-static int agi_v2_detect_game() {
+int AgiLoader_v2::getIntVersion() {
+	return _intVersion;
+}
+
+int AgiLoader_v2::detectGame() {
 	if (!Common::File::exists(LOGDIR) ||
 			!Common::File::exists(PICDIR) ||
 			!Common::File::exists(SNDDIR) ||
 			!Common::File::exists(VIEWDIR))
-		return err_InvalidAGIFile;
+		return errInvalidAGIFile;
 
-	agi_v2.int_version = 0x2917;	/* setup for 2.917 */
-	return v2id_game();
+	_intVersion = 0x2917;	/* setup for 2.917 */
+	return _vm->v2IdGame();
 }
 
-static int agi_v2_load_dir(struct agi_dir *agid, const char *fname) {
+int AgiLoader_v2::loadDir(AgiDir *agid, const char *fname) {
 	Common::File fp;
 	uint8 *mem;
 	uint32 flen;
@@ -69,16 +62,16 @@ static int agi_v2_load_dir(struct agi_dir *agid, const char *fname) {
 	report("Loading directory: %s\n", fname);
 
 	if (!fp.open(fname)) {
-		return err_BadFileOpen;
+		return errBadFileOpen;
 	}
 
 	fp.seek(0, SEEK_END);
 	flen = fp.pos();
 	fp.seek(0, SEEK_SET);
 
-	if ((mem = (uint8 *) malloc(flen + 32)) == NULL) {
+	if ((mem = (uint8 *)malloc(flen + 32)) == NULL) {
 		fp.close();
-		return err_NotEnoughMemory;
+		return errNotEnoughMemory;
 	}
 
 	fp.read(mem, flen);
@@ -99,57 +92,57 @@ static int agi_v2_load_dir(struct agi_dir *agid, const char *fname) {
 	free(mem);
 	fp.close();
 
-	return err_OK;
+	return errOK;
 }
 
-static int agi_v2_init() {
-	int ec = err_OK;
+int AgiLoader_v2::init() {
+	int ec = errOK;
 
 	/* load directory files */
-	ec = agi_v2_load_dir(game.dir_logic, LOGDIR);
-	if (ec == err_OK)
-		ec = agi_v2_load_dir(game.dir_pic, PICDIR);
-	if (ec == err_OK)
-		ec = agi_v2_load_dir(game.dir_view, VIEWDIR);
-	if (ec == err_OK)
-		ec = agi_v2_load_dir(game.dir_sound, SNDDIR);
+	ec = loadDir(_vm->_game.dirLogic, LOGDIR);
+	if (ec == errOK)
+		ec = loadDir(_vm->_game.dirPic, PICDIR);
+	if (ec == errOK)
+		ec = loadDir(_vm->_game.dirView, VIEWDIR);
+	if (ec == errOK)
+		ec = loadDir(_vm->_game.dirSound, SNDDIR);
 
 	return ec;
 }
 
-static int agi_v2_deinit() {
-	int ec = err_OK;
+int AgiLoader_v2::deinit() {
+	int ec = errOK;
 
 #if 0
 	/* unload words */
-	agi_v2_unload_words();
+	agiV2UnloadWords();
 
 	/* unload objects */
-	agi_v2_unload_objects();
+	agiV2UnloadObjects();
 #endif
 
 	return ec;
 }
 
-static int agi_v2_unload_resource(int t, int n) {
+int AgiLoader_v2::unloadResource(int t, int n) {
 	debugC(3, kDebugLevelResources, "unload resource");
 
 	switch (t) {
 	case rLOGIC:
-		unload_logic(n);
+		_vm->unloadLogic(n);
 		break;
 	case rPICTURE:
-		unload_picture(n);
+		_vm->_picture->unloadPicture(n);
 		break;
 	case rVIEW:
-		unload_view(n);
+		_vm->unloadView(n);
 		break;
 	case rSOUND:
-		unload_sound(n);
+		_vm->_sound->unloadSound(n);
 		break;
 	}
 
-	return err_OK;
+	return errOK;
 }
 
 /*
@@ -157,7 +150,7 @@ static int agi_v2_unload_resource(int t, int n) {
  * if further decoding is required, it must be done by another
  * routine. NULL is returned if unsucsessfull.
  */
-static uint8 *agi_v2_load_vol_res(struct agi_dir *agid) {
+uint8 *AgiLoader_v2::loadVolRes(struct AgiDir *agid) {
 	uint8 *data = NULL;
 	char x[MAX_PATH], *path;
 	Common::File fp;
@@ -184,7 +177,7 @@ static uint8 *agi_v2_load_vol_res(struct agi_dir *agid) {
 			/* FIXME: call some panic handler instead of
 			 *        deiniting directly
 			 */
-			deinit_video_mode();
+			deinitVideoMode();
 #endif
 			report("Error: bad signature %04x\n", sig);
 			// fprintf (stderr, "ACK! BAD RESOURCE!!!\n");
@@ -204,33 +197,33 @@ static uint8 *agi_v2_load_vol_res(struct agi_dir *agid) {
  * Loads a resource into memory, a raw resource is loaded in
  * with above routine, then further decoded here.
  */
-int agi_v2_load_resource(int t, int n) {
-	int ec = err_OK;
+int AgiLoader_v2::loadResource(int t, int n) {
+	int ec = errOK;
 	uint8 *data = NULL;
 
 	debugC(3, kDebugLevelResources, "(t = %d, n = %d)", t, n);
 	if (n > MAX_DIRS)
-		return err_BadResource;
+		return errBadResource;
 
 	switch (t) {
 	case rLOGIC:
-		if (~game.dir_logic[n].flags & RES_LOADED) {
+		if (~_vm->_game.dirLogic[n].flags & RES_LOADED) {
 			debugC(3, kDebugLevelResources, "loading logic resource %d", n);
-			agi_v2.unload_resource(rLOGIC, n);
+			unloadResource(rLOGIC, n);
 
 			/* load raw resource into data */
-			data = agi_v2_load_vol_res(&game.dir_logic[n]);
+			data = loadVolRes(&_vm->_game.dirLogic[n]);
 
-			game.logics[n].data = data;
-			ec = data ? decode_logic(n) : err_BadResource;
+			_vm->_game.logics[n].data = data;
+			ec = data ? _vm->decodeLogic(n) : errBadResource;
 
-			game.logics[n].sIP = 2;
+			_vm->_game.logics[n].sIP = 2;
 		}
 
 		/* if logic was cached, we get here */
 		/* reset code pointers incase it was cached */
 
-		game.logics[n].cIP = game.logics[n].sIP;
+		_vm->_game.logics[n].cIP = _vm->_game.logics[n].sIP;
 		break;
 	case rPICTURE:
 		/* if picture is currently NOT loaded *OR* cacheing is off,
@@ -238,34 +231,34 @@ int agi_v2_load_resource(int t, int n) {
 		 */
 
 		debugC(3, kDebugLevelResources, "loading picture resource %d", n);
-		if (game.dir_pic[n].flags & RES_LOADED)
+		if (_vm->_game.dirPic[n].flags & RES_LOADED)
 			break;
 
 		/* if loaded but not cached, unload it */
 		/* if cached but not loaded, etc */
-		agi_v2.unload_resource(rPICTURE, n);
-		data = agi_v2_load_vol_res(&game.dir_pic[n]);
+		unloadResource(rPICTURE, n);
+		data = loadVolRes(&_vm->_game.dirPic[n]);
 
 		if (data != NULL) {
-			game.pictures[n].rdata = data;
-			game.dir_pic[n].flags |= RES_LOADED;
+			_vm->_game.pictures[n].rdata = data;
+			_vm->_game.dirPic[n].flags |= RES_LOADED;
 		} else {
-			ec = err_BadResource;
+			ec = errBadResource;
 		}
 		break;
 	case rSOUND:
 		debugC(3, kDebugLevelResources, "loading sound resource %d", n);
-		if (game.dir_sound[n].flags & RES_LOADED)
+		if (_vm->_game.dirSound[n].flags & RES_LOADED)
 			break;
 
-		data = agi_v2_load_vol_res(&game.dir_sound[n]);
+		data = loadVolRes(&_vm->_game.dirSound[n]);
 
 		if (data != NULL) {
-			game.sounds[n].rdata = data;
-			game.dir_sound[n].flags |= RES_LOADED;
-			decode_sound(n);
+			_vm->_game.sounds[n].rdata = data;
+			_vm->_game.dirSound[n].flags |= RES_LOADED;
+			_vm->_sound->decodeSound(n);
 		} else {
-			ec = err_BadResource;
+			ec = errBadResource;
 		}
 		break;
 	case rVIEW:
@@ -274,34 +267,34 @@ int agi_v2_load_resource(int t, int n) {
 		 * can we cache the view? or must we reload it all
 		 * the time?
 		 */
-		if (game.dir_view[n].flags & RES_LOADED)
+		if (_vm->_game.dirView[n].flags & RES_LOADED)
 			break;
 
 		debugC(3, kDebugLevelResources, "loading view resource %d", n);
-		agi_v2.unload_resource(rVIEW, n);
-		data = agi_v2_load_vol_res(&game.dir_view[n]);
+		unloadResource(rVIEW, n);
+		data = loadVolRes(&_vm->_game.dirView[n]);
 		if (data) {
-			game.views[n].rdata = data;
-			game.dir_view[n].flags |= RES_LOADED;
-			ec = decode_view(n);
+			_vm->_game.views[n].rdata = data;
+			_vm->_game.dirView[n].flags |= RES_LOADED;
+			ec = _vm->decodeView(n);
 		} else {
-			ec = err_BadResource;
+			ec = errBadResource;
 		}
 		break;
 	default:
-		ec = err_BadResource;
+		ec = errBadResource;
 		break;
 	}
 
 	return ec;
 }
 
-static int agi_v2_load_objects(const char *fname) {
-	return load_objects(fname);
+int AgiLoader_v2::loadObjects(const char *fname) {
+	return _vm->loadObjects(fname);
 }
 
-static int agi_v2_load_words(const char *fname) {
-	return load_words(fname);
+int AgiLoader_v2::loadWords(const char *fname) {
+	return _vm->loadWords(fname);
 }
 
-}                             // End of namespace Agi
+} // End of namespace Agi
