@@ -1,7 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
  *
- * Copyright (C) 1999-2001 Sarien Team
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -17,21 +18,21 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/agi/console.h $
- * $Id: console.h 22618 2006-05-24 21:40:24Z aquadran $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/agi/console.h $
+ * $Id: console.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
 #ifndef AGI_CONSOLE_H
 #define AGI_CONSOLE_H
 
-#include "common/debugger.h"
+#include "gui/debugger.h"
 
 namespace Agi {
 
 class AgiEngine;
 
-struct agi_debug {
+struct AgiDebug {
 	int enabled;
 	int opcodes;
 	int logic0;
@@ -41,7 +42,7 @@ struct agi_debug {
 	int ignoretriggers;
 };
 
-class Console : public Common::Debugger<Console> {
+class Console : public GUI::Debugger {
 public:
 	Console(AgiEngine *vm);
 	virtual ~Console(void);
@@ -51,8 +52,6 @@ protected:
 	virtual void postEnter();
 
 private:
-	bool Cmd_Exit(int argc, const char **argv);
-	bool Cmd_Help(int argc, const char **argv);
 	bool Cmd_SetVar(int argc, const char **argv);
 	bool Cmd_SetFlag(int argc, const char **argv);
 	bool Cmd_SetObj(int argc, const char **argv);
@@ -75,4 +74,4 @@ private:
 
 }                             // End of namespace Agi
 
-#endif				/* AGI_CONSOLE_H */
+#endif /* AGI_CONSOLE_H */
