@@ -1,7 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
  *
- * Copyright (C) 1999-2003 Sarien Team
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -17,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/agi/global.cpp $
- * $Id: global.cpp 22794 2006-05-31 10:53:04Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/agi/global.cpp $
+ * $Id: global.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -28,15 +29,15 @@
 
 namespace Agi {
 
-int getflag(int n) {
-	uint8 *set = (uint8 *) &game.flags;
+int AgiEngine::getflag(int n) {
+	uint8 *set = (uint8 *)&_game.flags;
 
 	set += n >> 3;
 	return (*set & (1 << (n & 0x07))) != 0;
 }
 
-void setflag(int n, int v) {
-	uint8 *set = (uint8 *) &game.flags;
+void AgiEngine::setflag(int n, int v) {
+	uint8 *set = (uint8 *)&_game.flags;
 
 	set += n >> 3;
 	if (v)
@@ -45,29 +46,30 @@ void setflag(int n, int v) {
 		*set &= ~(1 << (n & 0x07));	/* clear bit */
 }
 
-void flipflag(int n) {
-	uint8 *set = (uint8 *) & game.flags;
+void AgiEngine::flipflag(int n) {
+	uint8 *set = (uint8 *)&_game.flags;
 
 	set += n >> 3;
 	*set ^= 1 << (n & 0x07);	/* flip bit */
 }
 
-void setvar(int var, int val) {
-	game.vars[var] = val;
+void AgiEngine::setvar(int var, int val) {
+	_game.vars[var] = val;
 }
 
-int getvar(int var) {
-	return game.vars[var];
+int AgiEngine::getvar(int var) {
+	return _game.vars[var];
 }
 
-void decrypt(uint8 *mem, int len) {
+void AgiEngine::decrypt(uint8 *mem, int len) {
 	const uint8 *key;
 	int i;
 
-	key = opt.agds ? (const uint8 *)CRYPT_KEY_AGDS : (const uint8 *)CRYPT_KEY_SIERRA;
+	key = (getFeatures() & GF_AGDS) ? (const uint8 *)CRYPT_KEY_AGDS
+	                   : (const uint8 *)CRYPT_KEY_SIERRA;
 
 	for (i = 0; i < len; i++)
 		*(mem + i) ^= *(key + (i % 11));
 }
 
-}                             // End of namespace Agi
+} // End of namespace Agi
