@@ -1,7 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
  *
- * Copyright (C) 1999-2003 Sarien Team
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -17,11 +18,12 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/agi/graphics.cpp $
- * $Id: graphics.cpp 22639 2006-05-25 22:51:42Z eriktorbjorn $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/agi/graphics.cpp $
+ * $Id: graphics.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
+#include "common/file.h"
 #include "common/stdafx.h"
 
 #include "graphics/cursorman.h"
@@ -39,18 +41,14 @@ namespace Agi {
 #  define MAX_INT (int)((unsigned)~0 >> 1)
 #endif
 
-static uint8 *agi_screen;
-
-static unsigned char *screen;
-
-extern uint8 cur_font[];
+#include "agi/font.h"
 
 /**
  * 16 color RGB palette (plus 16 transparent colors).
  * This array contains the 6-bit RGB values of the EGA palette exported
  * to the console drivers.
  */
-uint8 ega_palette[16 * 3] = {
+uint8 egaPalette[16 * 3] = {
 	0x00, 0x00, 0x00,
 	0x00, 0x00, 0x2a,
 	0x00, 0x2a, 0x00,
@@ -72,7 +70,7 @@ uint8 ega_palette[16 * 3] = {
 /**
  * 16 color amiga-ish palette.
  */
-uint8 new_palette[16 * 3] = {
+uint8 newPalette[16 * 3] = {
 	0x00, 0x00, 0x00,
 	0x00, 0x00, 0x3f,
 	0x00, 0x2A, 0x00,
@@ -91,9 +89,7 @@ uint8 new_palette[16 * 3] = {
 	0x3F, 0x3F, 0x3F
 };
 
-uint8 palette[32 * 3];
-
-static uint16 cga_map[16] = {
+static uint16 cgaMap[16] = {
 	0x0000,			/*  0 - black */
 	0x0d00,			/*  1 - blue */
 	0x0b00,			/*  2 - green */
@@ -112,15 +108,22 @@ static uint16 cga_map[16] = {
 	0x0f0f			/* 15 - white */
 };
 
-struct update_block {
+struct UpdateBlock {
 	int x1, y1;
 	int x2, y2;
 };
 
-static struct update_block update = {
+static struct UpdateBlock update = {
 	MAX_INT, MAX_INT, 0, 0
 };
 
+GfxMgr::GfxMgr(AgiEngine *vm) : _vm(vm) {
+	_shakeH = NULL;
+	_shakeV = NULL;
+	_agipalFileNum = 0;
+}
+
+
 /*
  *  Layer 4:  640x480?  ==================  User display
  *                              ^
@@ -136,84 +139,93 @@ static struct update_block update = {
  */
 
 #define SHAKE_MAG 3
-static uint8 *shake_h, *shake_v;
 
-void shake_start() {
+void GfxMgr::shakeStart() {
 	int i;
 
-	if ((shake_h = (uint8 *)malloc(GFX_WIDTH * SHAKE_MAG)) == NULL)
+	if ((_shakeH = (uint8 *)malloc(GFX_WIDTH * SHAKE_MAG)) == NULL)
 		return;
 
-	if ((shake_v = (uint8 *)malloc(SHAKE_MAG * (GFX_HEIGHT - SHAKE_MAG))) == NULL) {
-		free(shake_h);
+	if ((_shakeV = (uint8 *)malloc(SHAKE_MAG * (GFX_HEIGHT - SHAKE_MAG))) == NULL) {
+		free(_shakeH);
 		return;
 	}
 
 	for (i = 0; i < GFX_HEIGHT - SHAKE_MAG; i++) {
-		memcpy(shake_v + i * SHAKE_MAG, agi_screen + i * GFX_WIDTH, SHAKE_MAG);
+		memcpy(_shakeV + i * SHAKE_MAG, _agiScreen + i * GFX_WIDTH, SHAKE_MAG);
 	}
 
 	for (i = 0; i < SHAKE_MAG; i++) {
-		memcpy(shake_h + i * GFX_WIDTH, agi_screen + i * GFX_WIDTH, GFX_WIDTH);
+		memcpy(_shakeH + i * GFX_WIDTH, _agiScreen + i * GFX_WIDTH, GFX_WIDTH);
 	}
 }
 
-void shake_screen(int n) {
+void GfxMgr::shakeScreen(int n) {
 	int i;
 
 	if (n == 0) {
 		for (i = 0; i < (GFX_HEIGHT - SHAKE_MAG); i++) {
-			memmove(&agi_screen[GFX_WIDTH * i],
-					&agi_screen[GFX_WIDTH * (i + SHAKE_MAG) + SHAKE_MAG],
+			memmove(&_agiScreen[GFX_WIDTH * i],
+					&_agiScreen[GFX_WIDTH * (i + SHAKE_MAG) + SHAKE_MAG],
 					GFX_WIDTH - SHAKE_MAG);
 		}
 	} else {
 		for (i = GFX_HEIGHT - SHAKE_MAG - 1; i >= 0; i--) {
-			memmove(&agi_screen[GFX_WIDTH * (i + SHAKE_MAG) + SHAKE_MAG],
-					&agi_screen[GFX_WIDTH * i], GFX_WIDTH - SHAKE_MAG);
+			memmove(&_agiScreen[GFX_WIDTH * (i + SHAKE_MAG) + SHAKE_MAG],
+					&_agiScreen[GFX_WIDTH * i], GFX_WIDTH - SHAKE_MAG);
 		}
 	}
 }
 
-void shake_end() {
+void GfxMgr::shakeEnd() {
 	int i;
 
 	for (i = 0; i < GFX_HEIGHT - SHAKE_MAG; i++) {
-		memcpy(agi_screen + i * GFX_WIDTH, shake_v + i * SHAKE_MAG, SHAKE_MAG);
+		memcpy(_agiScreen + i * GFX_WIDTH, _shakeV + i * SHAKE_MAG, SHAKE_MAG);
 	}
 
 	for (i = 0; i < SHAKE_MAG; i++) {
-		memcpy(agi_screen + i * GFX_WIDTH, shake_h + i * GFX_WIDTH, GFX_WIDTH);
+		memcpy(_agiScreen + i * GFX_WIDTH, _shakeH + i * GFX_WIDTH, GFX_WIDTH);
 	}
 
-	flush_block(0, 0, GFX_WIDTH - 1, GFX_HEIGHT - 1);
+	flushBlock(0, 0, GFX_WIDTH - 1, GFX_HEIGHT - 1);
 
-	free(shake_v);
-	free(shake_h);
+	free(_shakeV);
+	free(_shakeH);
 }
 
-void put_text_character(int l, int x, int y, unsigned int c, int fg, int bg) {
+void GfxMgr::putTextCharacter(int l, int x, int y, unsigned int c, int fg, int bg, bool checkerboard) {
 	int x1, y1, xx, yy, cc;
-	uint8 *p;
+	const uint8 *p;
 
-	p = cur_font + ((unsigned int)c * CHAR_LINES);
+	p = Agi::curFont + ((unsigned int)c * CHAR_LINES);
 	for (y1 = 0; y1 < CHAR_LINES; y1++) {
 		for (x1 = 0; x1 < CHAR_COLS; x1++) {
 			xx = x + x1;
 			yy = y + y1;
 			cc = (*p & (1 << (7 - x1))) ? fg : bg;
-			agi_screen[xx + yy * GFX_WIDTH] = cc;
+			_agiScreen[xx + yy * GFX_WIDTH] = cc;
 		}
 
 		p++;
 	}
+
+	// Simple checkerboard effect to simulate "greyed out" text.
+	// This is what Sierra's interpreter does for things like menu items
+	// that aren't selectable (such as separators). -- dsymonds
+	if (checkerboard) {
+		for (yy = y; yy < y + CHAR_LINES; yy++)
+			for (xx = x + (~yy & 1); xx < x + CHAR_COLS; xx += 2)
+				_agiScreen[xx + yy * GFX_WIDTH] = 15;
+	}
+
 	/* FIXME: we don't want this when we're writing on the
 	 *        console!
 	 */
-	flush_block(x, y, x + CHAR_COLS - 1, y + CHAR_LINES - 1);
+	flushBlock(x, y, x + CHAR_COLS - 1, y + CHAR_LINES - 1);
 }
 
-void draw_rectangle(int x1, int y1, int x2, int y2, int c) {
+void GfxMgr::drawRectangle(int x1, int y1, int x2, int y2, int c) {
 	int y, w, h;
 	uint8 *p0;
 
@@ -228,49 +240,49 @@ void draw_rectangle(int x1, int y1, int x2, int y2, int c) {
 
 	w = x2 - x1 + 1;
 	h = y2 - y1 + 1;
-	p0 = &agi_screen[x1 + y1 * GFX_WIDTH];
+	p0 = &_agiScreen[x1 + y1 * GFX_WIDTH];
 	for (y = 0; y < h; y++) {
 		memset(p0, c, w);
 		p0 += GFX_WIDTH;
 	}
 }
 
-static void draw_frame(int x1, int y1, int x2, int y2, int c1, int c2) {
+void GfxMgr::drawFrame(int x1, int y1, int x2, int y2, int c1, int c2) {
 	int y, w;
 	uint8 *p0;
 
 	/* top line */
 	w = x2 - x1 + 1;
-	p0 = &agi_screen[x1 + y1 * GFX_WIDTH];
+	p0 = &_agiScreen[x1 + y1 * GFX_WIDTH];
 	memset(p0, c1, w);
 
 	/* bottom line */
-	p0 = &agi_screen[x1 + y2 * GFX_WIDTH];
+	p0 = &_agiScreen[x1 + y2 * GFX_WIDTH];
 	memset(p0, c2, w);
 
 	/* side lines */
 	for (y = y1; y <= y2; y++) {
-		agi_screen[x1 + y * GFX_WIDTH] = c1;
-		agi_screen[x2 + y * GFX_WIDTH] = c2;
+		_agiScreen[x1 + y * GFX_WIDTH] = c1;
+		_agiScreen[x2 + y * GFX_WIDTH] = c2;
 	}
 }
 
-void draw_box(int x1, int y1, int x2, int y2, int colour1, int colour2, int m) {
+void GfxMgr::drawBox(int x1, int y1, int x2, int y2, int colour1, int colour2, int m) {
 	x1 += m;
 	y1 += m;
 	x2 -= m;
 	y2 -= m;
 
-	draw_rectangle(x1, y1, x2, y2, colour1);
-	draw_frame(x1 + 2, y1 + 2, x2 - 2, y2 - 2, colour2, colour2);
-	flush_block(x1, y1, x2, y2);
+	drawRectangle(x1, y1, x2, y2, colour1);
+	drawFrame(x1 + 2, y1 + 2, x2 - 2, y2 - 2, colour2, colour2);
+	flushBlock(x1, y1, x2, y2);
 }
 
-void print_character(int x, int y, char c, int fg, int bg) {
+void GfxMgr::printCharacter(int x, int y, char c, int fg, int bg) {
 	x *= CHAR_COLS;
 	y *= CHAR_LINES;
 
-	put_text_character(0, x, y, c, fg, bg);
+	putTextCharacter(0, x, y, c, fg, bg);
 	// redundant! already inside put_text_character!
 	// flush_block (x, y, x + CHAR_COLS - 1, y + CHAR_LINES - 1);
 }
@@ -282,7 +294,7 @@ void print_character(int x, int y, char c, int fg, int bg) {
  * @param a  set if the button has focus
  * @param p  set if the button is pressed
  */
-void draw_button(int x, int y, const char *s, int a, int p) {
+void GfxMgr::drawButton(int x, int y, const char *s, int a, int p, int fgcolor, int bgcolor) {
 	int len = strlen(s);
 	int x1, y1, x2, y2;
 
@@ -292,7 +304,7 @@ void draw_button(int x, int y, const char *s, int a, int p) {
 	y2 = y + CHAR_LINES + 2;
 
 	while (*s) {
-		put_text_character(0, x + (!!p), y + (!!p), *s++, a ? 15 : 0, a ? 0 : 15);
+		putTextCharacter(0, x + (!!p), y + (!!p), *s++, a ? bgcolor : fgcolor, a ? fgcolor : bgcolor);
 		x += CHAR_COLS;
 	}
 
@@ -301,10 +313,10 @@ void draw_button(int x, int y, const char *s, int a, int p) {
 	x2 += 2;
 	y2 += 2;
 
-	flush_block(x1, y1, x2, y2);
+	flushBlock(x1, y1, x2, y2);
 }
 
-int test_button(int x, int y, const char *s) {
+int GfxMgr::testButton(int x, int y, const char *s) {
 	int len = strlen(s);
 	int x1, y1, x2, y2;
 
@@ -313,30 +325,30 @@ int test_button(int x, int y, const char *s) {
 	x2 = x + CHAR_COLS * len + 2;
 	y2 = y + CHAR_LINES + 2;
 
-	if ((int)mouse.x >= x1 && (int)mouse.y >= y1 && (int)mouse.x <= x2 && (int)mouse.y <= y2)
+	if ((int)g_mouse.x >= x1 && (int)g_mouse.y >= y1 && (int)g_mouse.x <= x2 && (int)g_mouse.y <= y2)
 		return true;
 
 	return false;
 }
 
-void put_block(int x1, int y1, int x2, int y2) {
-	gfx_putblock(x1, y1, x2, y2);
+void GfxMgr::putBlock(int x1, int y1, int x2, int y2) {
+	gfxPutBlock(x1, y1, x2, y2);
 }
 
-void put_screen() {
-	put_block(0, 0, GFX_WIDTH - 1, GFX_HEIGHT - 1);
+void GfxMgr::putScreen() {
+	putBlock(0, 0, GFX_WIDTH - 1, GFX_HEIGHT - 1);
 }
 
-void poll_timer() {
-	agi_timer_low();
+void GfxMgr::pollTimer() {
+	_vm->agiTimerLow();
 }
 
-int get_key() {
-	return agi_get_keypress_low();
+int GfxMgr::getKey() {
+	return _vm->agiGetKeypressLow();
 }
 
-int keypress() {
-	return agi_is_keypress_low();
+int GfxMgr::keypress() {
+	return _vm->agiIsKeypressLow();
 }
 
 /*
@@ -350,30 +362,71 @@ int keypress() {
  * for the interpreter console.
  * @param p  A pointer to the 16-color RGB palette.
  */
-void init_palette(uint8 *p) {
+void GfxMgr::initPalette(uint8 *p) {
 	int i;
 
 	for (i = 0; i < 48; i++) {
-		palette[i] = p[i];
-		palette[i + 48] = (p[i] + 0x30) >> 2;
+		_palette[i] = p[i];
+		_palette[i + 48] = (p[i] + 0x30) >> 2;
 	}
 }
 
-void gfx_set_palette() {
+void GfxMgr::gfxSetPalette() {
 	int i;
 	byte pal[32 * 4];
 
 	for (i = 0; i < 32; i++) {
-		pal[i * 4 + 0] = palette[i * 3 + 0] << 2;
-		pal[i * 4 + 1] = palette[i * 3 + 1] << 2;
-		pal[i * 4 + 2] = palette[i * 3 + 2] << 2;
+		pal[i * 4 + 0] = _palette[i * 3 + 0] << 2;
+		pal[i * 4 + 1] = _palette[i * 3 + 1] << 2;
+		pal[i * 4 + 2] = _palette[i * 3 + 2] << 2;
 		pal[i * 4 + 3] = 0;
 	}
 	g_system->setPalette(pal, 0, 32);
 }
 
+//Gets AGIPAL Data
+void GfxMgr::setAGIPal(int p0) {
+	//If 0 from savefile, do not use
+	if (p0 == 0)
+		return;
+
+	char filename[15];
+	sprintf(filename, "pal.%d", p0);
+
+	Common::File agipal;
+	if (!agipal.open(filename))
+		error("Couldn't open AGIPAL palette file '%s'", filename);
+
+	//Chunk0 holds colors 0-7
+	agipal.read(&_agipalPalette[0], 24);
+
+	//Chunk1 is the same as the chunk0
+
+	//Chunk2 chunk holds colors 8-15
+	agipal.seek(24, SEEK_CUR);
+	agipal.read(&_agipalPalette[24], 24);
+
+	//Chunk3 is the same as the chunk2
+
+	//Chunks4-7 are duplicates of chunks0-3
+
+	if (agipal.ioFailed())
+		error("Couldn't read AGIPAL palette from '%s'", filename);
+
+	_agipalFileNum = p0;
+
+	initPalette(_agipalPalette);
+	gfxSetPalette();
+
+	debug(1, "Using AGIPAL palette from '%s'", filename);
+}
+
+int GfxMgr::getAGIPalFileNum() {
+	return _agipalFileNum;
+}
+
 /* put a block onto the screen */
-void gfx_putblock(int x1, int y1, int x2, int y2) {
+void GfxMgr::gfxPutBlock(int x1, int y1, int x2, int y2) {
 	if (x1 >= GFX_WIDTH)
 		x1 = GFX_WIDTH - 1;
 	if (y1 >= GFX_HEIGHT)
@@ -383,19 +436,19 @@ void gfx_putblock(int x1, int y1, int x2, int y2) {
 	if (y2 >= GFX_HEIGHT)
 		y2 = GFX_HEIGHT - 1;
 
-	g_system->copyRectToScreen(screen + y1 * 320 + x1, 320, x1, y1, x2 - x1 + 1, y2 - y1 + 1);
-	//g_system->copyRectToScreen(screen, 320, 0, 0, 320, 200);
+	g_system->copyRectToScreen(_screen + y1 * 320 + x1, 320, x1, y1, x2 - x1 + 1, y2 - y1 + 1);
 }
 
 static const byte mouseCursorArrow[] = {
+	// This is the same arrow cursor that was later used in early SCI games
 	0x00, 0x00, 0x40, 0x00, 0x60, 0x00, 0x70, 0x00,
 	0x78, 0x00, 0x7C, 0x00, 0x7E, 0x00, 0x7F, 0x00,
-	0x7F, 0x80, 0x7C, 0x00, 0x6C, 0x00, 0x46, 0x00,
-	0x06, 0x00, 0x03, 0x00, 0x03, 0x00, 0x00, 0x00,
-	0xC0, 0x00, 0xE0, 0x00, 0xF0, 0x00, 0xF8, 0x00,
-	0xFC, 0x00, 0xFE, 0x00, 0xFF, 0x00, 0xFF, 0x80,
-	0xFF, 0xC0, 0xFF, 0xC0, 0xFE, 0x00, 0xFF, 0x00,
-	0xCF, 0x00, 0x07, 0x80, 0x07, 0x80, 0x03, 0x80
+	0x7F, 0x80, 0x7F, 0xC0, 0x7C, 0x00, 0x46, 0x00,
+	0x06, 0x00, 0x03, 0x00, 0x03, 0x00, 0x01, 0x80,
+	0xC0, 0x00, 0xA0, 0x00, 0x90, 0x00, 0x88, 0x00,
+	0x84, 0x00, 0x82, 0x00, 0x81, 0x00, 0x80, 0x80,
+	0x80, 0x40, 0x80, 0x20, 0x82, 0x00, 0xA9, 0x00,
+	0xC9, 0x00, 0x04, 0x80, 0x04, 0x80, 0x02, 0x40
 };
 
 /**
@@ -403,16 +456,16 @@ static const byte mouseCursorArrow[] = {
  *
  * @see deinit_video()
  */
-int init_video() {
-	if (opt.renderMode == Common::kRenderEGA)
-		init_palette(ega_palette);
+int GfxMgr::initVideo() {
+	if (_vm->_renderMode == Common::kRenderEGA)
+		initPalette(egaPalette);
 	else
-		init_palette(new_palette);
+		initPalette(newPalette);
 
-	if ((agi_screen = (uint8 *)calloc(GFX_WIDTH, GFX_HEIGHT)) == NULL)
-		return err_NotEnoughMemory;
+	if ((_agiScreen = (uint8 *)calloc(GFX_WIDTH, GFX_HEIGHT)) == NULL)
+		return errNotEnoughMemory;
 
-	gfx_set_palette();
+	gfxSetPalette();
 
 	byte mouseCursor[16 * 16];
 	const byte *src = mouseCursorArrow;
@@ -430,9 +483,17 @@ int init_video() {
 		}
 		++src;
 	}
+
+	const byte cursorPalette[] = {
+		0, 0, 0, 0,
+		0, 0, 0, 0,
+		255, 255, 255, 0
+	};
+
+	CursorMan.replaceCursorPalette(cursorPalette, 0, 3);
 	CursorMan.replaceCursor(mouseCursor, 16, 16, 1, 1);
 
-	return err_OK;
+	return errOK;
 }
 
 /**
@@ -440,24 +501,23 @@ int init_video() {
  *
  * @see init_video()
  */
-int deinit_video() {
-	free(agi_screen);
+int GfxMgr::deinitVideo() {
+	free(_agiScreen);
 
-	return err_OK;
+	return errOK;
 }
 
-int init_machine() {
-	screen = (unsigned char *)malloc(320 * 200);
-	clock_count = 0;
-	clock_ticks = 0;
+int GfxMgr::initMachine() {
+	_screen = (unsigned char *)malloc(320 * 200);
+	_vm->_clockCount = 0;
 
-	return err_OK;
+	return errOK;
 }
 
-int deinit_machine() {
-	free(screen);
+int GfxMgr::deinitMachine() {
+	free(_screen);
 
-	return err_OK;
+	return errOK;
 }
 
 /**
@@ -470,34 +530,24 @@ int deinit_machine() {
  * @param n number of pixels in the row
  * @param p pointer to the row start in the AGI screen
  */
-void put_pixels_a(int x, int y, int n, uint8 *p) {
-	if (opt.renderMode == Common::kRenderCGA) {
+void GfxMgr::putPixelsA(int x, int y, int n, uint8 *p) {
+	if (_vm->_renderMode == Common::kRenderCGA) {
 		for (x *= 2; n--; p++, x += 2) {
-			register uint16 q = (cga_map[(*p & 0xf0) >> 4] << 4) | cga_map[*p & 0x0f];
-			if (debug_.priority)
+			register uint16 q = (cgaMap[(*p & 0xf0) >> 4] << 4) | cgaMap[*p & 0x0f];
+			if (_vm->_debug.priority)
 				q >>= 4;
-			*(uint16 *)&agi_screen[x + y * GFX_WIDTH] = q & 0x0f0f;
+			*(uint16 *)&_agiScreen[x + y * GFX_WIDTH] = q & 0x0f0f;
 		}
 	} else {
 		for (x *= 2; n--; p++, x += 2) {
 			register uint16 q = ((uint16) * p << 8) | *p;
-			if (debug_.priority)
+			if (_vm->_debug.priority)
 				q >>= 4;
-			*(uint16 *)&agi_screen[x + y * GFX_WIDTH] = q & 0x0f0f;
+			*(uint16 *)&_agiScreen[x + y * GFX_WIDTH] = q & 0x0f0f;
 		}
 	}
 }
 
-void put_pixels_hires(int x, int y, int n, uint8 *p) {
-	//y += CHAR_LINES;
-	for (; n--; p++, x++) {
-		uint8 q = *p;
-		if (debug_.priority)
-			q >>= 4;
-		agi_screen[x + y * GFX_WIDTH] = q & 0x0f;
-	}
-}
-
 /**
  * Schedule blocks for blitting on the output device.
  * This function gets the coordinates of a block in the AGI screen and
@@ -509,7 +559,7 @@ void put_pixels_hires(int x, int y, int n, uint8 *p) {
  *
  * @see do_update()
  */
-void schedule_update(int x1, int y1, int x2, int y2) {
+void GfxMgr::scheduleUpdate(int x1, int y1, int x2, int y2) {
 	if (x1 < update.x1)
 		update.x1 = x1;
 	if (y1 < update.y1)
@@ -527,9 +577,9 @@ void schedule_update(int x1, int y1, int x2, int y2) {
  *
  * @see schedule_update()
  */
-void do_update() {
+void GfxMgr::doUpdate() {
 	if (update.x1 <= update.x2 && update.y1 <= update.y2) {
-		gfx_putblock(update.x1, update.y1, update.x2, update.y2);
+		gfxPutBlock(update.x1, update.y1, update.x2, update.y2);
 	}
 
 	/* reset update block variables */
@@ -550,17 +600,17 @@ void do_update() {
  *
  * @see flush_block_a()
  */
-void flush_block(int x1, int y1, int x2, int y2) {
+void GfxMgr::flushBlock(int x1, int y1, int x2, int y2) {
 	int y, w;
 	uint8 *p0;
 
-	schedule_update(x1, y1, x2, y2);
+	scheduleUpdate(x1, y1, x2, y2);
 
-	p0 = &agi_screen[x1 + y1 * GFX_WIDTH];
+	p0 = &_agiScreen[x1 + y1 * GFX_WIDTH];
 	w = x2 - x1 + 1;
 
 	for (y = y1; y <= y2; y++) {
-		memcpy(screen + 320 * y + x1, p0, w);
+		memcpy(_screen + 320 * y + x1, p0, w);
 		p0 += GFX_WIDTH;
 	}
 }
@@ -574,10 +624,10 @@ void flush_block(int x1, int y1, int x2, int y2) {
  *
  * @see flush_block()
  */
-void flush_block_a(int x1, int y1, int x2, int y2) {
+void GfxMgr::flushBlockA(int x1, int y1, int x2, int y2) {
 	//y1 += 8;
 	//y2 += 8;
-	flush_block(DEV_X0(x1), DEV_Y(y1), DEV_X1(x2), DEV_Y(y2));
+	flushBlock(DEV_X0(x1), DEV_Y(y1), DEV_X1(x2), DEV_Y(y2));
 }
 
 /**
@@ -585,8 +635,8 @@ void flush_block_a(int x1, int y1, int x2, int y2) {
  * This function updates the output device with the contents of the AGI
  * screen, handling console transparency.
  */
-void flush_screen() {
-	flush_block(0, 0, GFX_WIDTH - 1, GFX_HEIGHT - 1);
+void GfxMgr::flushScreen() {
+	flushBlock(0, 0, GFX_WIDTH - 1, GFX_HEIGHT - 1);
 }
 
 /**
@@ -597,19 +647,19 @@ void flush_screen() {
  * a graphic-only device.
  * @param c  color to clear the screen
  */
-void clear_screen(int c) {
-	memset(agi_screen, c, GFX_WIDTH * GFX_HEIGHT);
-	flush_screen();
+void GfxMgr::clearScreen(int c) {
+	memset(_agiScreen, c, GFX_WIDTH * GFX_HEIGHT);
+	flushScreen();
 }
 
 /**
  * Save a block of the AGI engine screen
  */
-void save_block(int x1, int y1, int x2, int y2, uint8 *b) {
+void GfxMgr::saveBlock(int x1, int y1, int x2, int y2, uint8 *b) {
 	uint8 *p0;
 	int w, h;
 
-	p0 = &agi_screen[x1 + GFX_WIDTH * y1];
+	p0 = &_agiScreen[x1 + GFX_WIDTH * y1];
 	w = x2 - x1 + 1;
 	h = y2 - y1 + 1;
 	while (h--) {
@@ -622,11 +672,11 @@ void save_block(int x1, int y1, int x2, int y2, uint8 *b) {
 /**
  * Restore a block of the AGI engine screen
  */
-void restore_block(int x1, int y1, int x2, int y2, uint8 *b) {
+void GfxMgr::restoreBlock(int x1, int y1, int x2, int y2, uint8 *b) {
 	uint8 *p0;
 	int w, h;
 
-	p0 = &agi_screen[x1 + GFX_WIDTH * y1];
+	p0 = &_agiScreen[x1 + GFX_WIDTH * y1];
 	w = x2 - x1 + 1;
 	h = y2 - y1 + 1;
 	while (h--) {
@@ -634,7 +684,7 @@ void restore_block(int x1, int y1, int x2, int y2, uint8 *b) {
 		b += w;
 		p0 += GFX_WIDTH;
 	}
-	flush_block(x1, y1, x2, y2);
+	flushBlock(x1, y1, x2, y2);
 }
 
 } // End of namespace Agi
