@@ -1,7 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
  *
- * Copyright (C) 1999-2001 Sarien Team
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -17,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/agi/inv.cpp $
- * $Id: inv.cpp 23011 2006-06-10 13:20:29Z ender $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/agi/inv.cpp $
+ * $Id: inv.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -28,8 +29,6 @@
 #include "agi/sprite.h"
 #include "agi/graphics.h"
 #include "agi/keyboard.h"
-#include "agi/text.h"
-#include "agi/keyboard.h"
 
 namespace Agi {
 
@@ -53,19 +52,16 @@ namespace Agi {
 #define SELECT_Y	24
 #define SELECT_MSG	"Press ENTER to select, ESC to cancel"
 
-static uint8 *intobj = NULL;
-
-static void print_item(int n, int fg, int bg)
-{
-	_text->print_text(object_name(intobj[n]), 0, n % 2 ? 39 - strlen(object_name(intobj[n])) : 1,
+void AgiEngine::printItem(int n, int fg, int bg) {
+	printText(objectName(_intobj[n]), 0, n % 2 ? 39 - strlen(objectName(_intobj[n])) : 1,
 			(n / 2) + 2, 40, fg, bg);
 }
 
-static int find_item() {
+int AgiEngine::findItem() {
 	int r, c;
 
-	r = mouse.y / CHAR_LINES;
-	c = mouse.x / CHAR_COLS;
+	r = g_mouse.y / CHAR_LINES;
+	c = g_mouse.x / CHAR_COLS;
 
 	debugC(6, kDebugLevelInventory, "r = %d, c = %d", r, c);
 
@@ -75,39 +71,42 @@ static int find_item() {
 	return (r - 2) * 2 + (c > 20);
 }
 
-static int show_items() {
+int AgiEngine::showItems() {
 	unsigned int x, i;
 
-	for (x = i = 0; x < game.num_objects; x++) {
-		if (object_get_location(x) == EGO_OWNED) {
+	for (x = i = 0; x < _game.numObjects; x++) {
+		if (objectGetLocation(x) == EGO_OWNED) {
 			/* add object to our list! */
-			intobj[i] = x;
-			print_item(i, STATUS_FG, STATUS_BG);
+			_intobj[i] = x;
+			printItem(i, STATUS_FG, STATUS_BG);
 			i++;
 		}
 	}
 
 	if (i == 0) {
-		_text->print_text(NOTHING_MSG, 0, NOTHING_X, NOTHING_Y, 40, STATUS_FG, STATUS_BG);
+		printText(NOTHING_MSG, 0, NOTHING_X, NOTHING_Y, 40, STATUS_FG, STATUS_BG);
 	}
 
 	return i;
 }
 
-static void select_items(int n) {
+void AgiEngine::selectItems(int n) {
 	int fsel = 0;
+	bool exit_select = false;
 
-	while (42) {
+	while (!exit_select) {
 		if (n > 0)
-			print_item(fsel, STATUS_BG, STATUS_FG);
+			printItem(fsel, STATUS_BG, STATUS_FG);
 
-		switch (wait_any_key()) {
+		switch (waitAnyKey()) {
 		case KEY_ENTER:
-			setvar(V_sel_item, intobj[fsel]);
-			goto exit_select;
+			setvar(vSelItem, _intobj[fsel]);
+			exit_select = true;
+			break;
 		case KEY_ESCAPE:
-			setvar(V_sel_item, 0xff);
-			goto exit_select;
+			setvar(vSelItem, 0xff);
+			exit_select = true;
+			break;
 		case KEY_UP:
 			if (fsel >= 2)
 				fsel -= 2;
@@ -125,26 +124,27 @@ static void select_items(int n) {
 				fsel++;
 			break;
 		case BUTTON_LEFT:{
-				int i = find_item();
+				int i = findItem();
 				if (i >= 0 && i < n) {
-					setvar(V_sel_item, intobj[fsel = i]);
+					setvar(vSelItem, _intobj[fsel = i]);
 					debugC(6, kDebugLevelInventory, "item found: %d", fsel);
-					show_items();
-					print_item(fsel, STATUS_BG, STATUS_FG);
-					do_update();
-					goto exit_select;
+					showItems();
+					printItem(fsel, STATUS_BG, STATUS_FG);
+					_gfx->doUpdate();
+					exit_select = true;
 				}
 				break;
 			}
 		default:
 			break;
 		}
-
-		show_items();
-		do_update();
+		
+		if (!exit_select) {
+			showItems();
+			_gfx->doUpdate();
+		}
 	}
 
- exit_select:
 	debugC(6, kDebugLevelInventory, "selected: %d", fsel);
 }
 
@@ -155,54 +155,54 @@ static void select_items(int n) {
 /**
  * Display inventory items.
  */
-void inventory() {
-	int old_fg, old_bg;
+void AgiEngine::inventory() {
+	int oldFg, oldBg;
 	int n;
 
 	/* screen is white with black text */
-	old_fg = game.color_fg;
-	old_bg = game.color_bg;
-	game.color_fg = 0;
-	game.color_bg = 15;
-	clear_screen(game.color_bg);
+	oldFg = _game.colorFg;
+	oldBg = _game.colorBg;
+	_game.colorFg = 0;
+	_game.colorBg = 15;
+	_gfx->clearScreen(_game.colorBg);
 
-	_text->print_text(YOUHAVE_MSG, 0, YOUHAVE_X, YOUHAVE_Y, 40, STATUS_FG, STATUS_BG);
+	printText(YOUHAVE_MSG, 0, YOUHAVE_X, YOUHAVE_Y, 40, STATUS_FG, STATUS_BG);
 
 	/* FIXME: doesn't check if objects overflow off screen... */
 
-	intobj = (uint8 *) malloc(4 + game.num_objects);
-	memset(intobj, 0, (4 + game.num_objects));
+	_intobj = (uint8 *)malloc(4 + _game.numObjects);
+	memset(_intobj, 0, (4 + _game.numObjects));
 
-	n = show_items();
+	n = showItems();
 
-	if (getflag(F_status_selects_items)) {
-		_text->print_text(SELECT_MSG, 0, SELECT_X, SELECT_Y, 40, STATUS_FG, STATUS_BG);
+	if (getflag(fStatusSelectsItems)) {
+		printText(SELECT_MSG, 0, SELECT_X, SELECT_Y, 40, STATUS_FG, STATUS_BG);
 	} else {
-		_text->print_text(ANY_KEY_MSG, 0, ANY_KEY_X, ANY_KEY_Y, 40, STATUS_FG, STATUS_BG);
+		printText(ANY_KEY_MSG, 0, ANY_KEY_X, ANY_KEY_Y, 40, STATUS_FG, STATUS_BG);
 	}
 
-	flush_screen();
+	_gfx->flushScreen();
 
 	/* If flag 13 is set, we want to highlight & select an item.
 	 * opon selection, put objnum in var 25. Then on esc put in
 	 * var 25 = 0xff.
 	 */
 
-	if (getflag(F_status_selects_items))
-		select_items(n);
+	if (getflag(fStatusSelectsItems))
+		selectItems(n);
 
-	free(intobj);
+	free(_intobj);
 
-	if (!getflag(F_status_selects_items))
-		wait_any_key();
+	if (!getflag(fStatusSelectsItems))
+		waitAnyKey();
 
-	clear_screen(0);
-	_text->write_status();
-	show_pic();
-	game.color_fg = old_fg;
-	game.color_bg = old_bg;
-	game.has_prompt = 0;
-	_text->flush_lines(game.line_user_input, 24);
+	_gfx->clearScreen(0);
+	writeStatus();
+	_picture->showPic();
+	_game.colorFg = oldFg;
+	_game.colorBg = oldBg;
+	_game.hasPrompt = 0;
+	flushLines(_game.lineUserInput, 24);
 }
 
-}                             // End of namespace Agi
+} // End of namespace Agi
