@@ -1,7 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
  *
- * Copyright (C) 1999-2003 Sarien Team
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -17,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/agi/keyboard.cpp $
- * $Id: keyboard.cpp 23011 2006-06-10 13:20:29Z ender $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/agi/keyboard.cpp $
+ * $Id: keyboard.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -28,28 +29,13 @@
 #include "agi/graphics.h"
 #include "agi/keyboard.h"
 #include "agi/menu.h"
-#include "agi/text.h"		/* remove later */
 
 namespace Agi {
 
-char last_sentence[40];
-
-/* FIXME */
-extern int open_dialogue;
-
-struct string_data {
-	int x;
-	int y;
-	int len;
-	int str;
-};
-
-struct string_data stringdata;
-
 /*
  * IBM-PC keyboard scancodes
  */
-uint8 scancode_table[26] = {
+uint8 scancodeTable[26] = {
 	30,			/* A */
 	48,			/* B */
 	46,			/* C */
@@ -78,21 +64,21 @@ uint8 scancode_table[26] = {
 	44			/* Z */
 };
 
-void init_words() {
-	game.num_ego_words = 0;
+void AgiEngine::initWords() {
+	_game.numEgoWords = 0;
 }
 
-void clean_input() {
-	while (game.num_ego_words)
-		free(game.ego_words[--game.num_ego_words].word);
+void AgiEngine::cleanInput() {
+	while (_game.numEgoWords)
+		free(_game.egoWords[--_game.numEgoWords].word);
 }
 
-void get_string(int x, int y, int len, int str) {
-	new_input_mode(INPUT_GETSTRING);
-	stringdata.x = x;
-	stringdata.y = y;
-	stringdata.len = len;
-	stringdata.str = str;
+void AgiEngine::getString(int x, int y, int len, int str) {
+	newInputMode(INPUT_GETSTRING);
+	_stringdata.x = x;
+	_stringdata.y = y;
+	_stringdata.len = len;
+	_stringdata.str = str;
 }
 
 /**
@@ -101,45 +87,50 @@ void get_string(int x, int y, int len, int str) {
  * It handles console keys and insulates AGI from the console. In the main
  * loop, handle_keys() handles keyboard input and ego movement.
  */
-int do_poll_keyboard() {
+int AgiEngine::doPollKeyboard() {
 	int key = 0;
 
 	/* If a key is ready, rip it */
-	if (keypress()) {
-		key = get_key();
+	if (_gfx->keypress()) {
+		key = _gfx->getKey();
+
 		debugC(3, kDebugLevelInput, "key %02x pressed", key);
 	}
 
 	return key;
 }
 
-int handle_controller(int key) {
-	struct vt_entry *v = &game.view_table[0];
+int AgiEngine::handleController(int key) {
+	VtEntry *v = &_game.viewTable[0];
 	int i;
 
-	/* The Black Cauldron needs KEY_ESCAPE to use menus */
-	if (key == 0 /*|| key == KEY_ESCAPE */ )
+	/* AGI 3.149 games and The Black Cauldron need KEY_ESCAPE to use menus */
+	if (key == 0 || (key == KEY_ESCAPE && agiGetRelease() != 0x3149 && getGameID() != GID_BC) )
 		return false;
 
+	if ((getGameID() == GID_MH1 || getGameID() == GID_MH2) && (key == KEY_ENTER) &&
+		(_game.inputMode == INPUT_NONE))
+		key = 0x20; // Set Enter key to Space in Manhunter when there's no text input
+
 	debugC(3, kDebugLevelInput, "key = %04x", key);
 
 	for (i = 0; i < MAX_DIRS; i++) {
-		if (game.ev_keyp[i].data == key) {
+		if (_game.evKeyp[i].data == key) {
 			debugC(3, kDebugLevelInput, "event %d: key press", i);
-			game.ev_keyp[i].occured = true;
+			_game.evKeyp[i].occured = true;
 			report("event AC:%i occured\n", i);
 			return true;
 		}
 	}
 
 	if (key == BUTTON_LEFT) {
-		if (getflag(F_menus_work) && mouse.y <= CHAR_LINES) {
-			new_input_mode(INPUT_MENU);
+		if (getflag(fMenusWork) && g_mouse.y <= CHAR_LINES) {
+			newInputMode(INPUT_MENU);
 			return true;
 		}
 	}
 
-	if (game.player_control) {
+	if (_game.playerControl) {
 		int d = 0;
 
 		if (!KEY_ASCII(key)) {
@@ -170,12 +161,23 @@ int handle_controller(int key) {
 				break;
 			}
 		}
-		if (!opt.agimouse) {
+
+		if (key == BUTTON_LEFT &&
+				(int)g_mouse.y >= _game.lineUserInput * CHAR_LINES &&
+				(int)g_mouse.y <= (_game.lineUserInput + 1) * CHAR_LINES) {
+			if (predictiveDialog()) {
+				strcpy((char *)_game.inputBuffer, _predictiveResult);
+				handleKeys(KEY_ENTER);
+			}
+			return true;
+		}
+
+		if (!(getFeatures() & GF_AGIMOUSE)) {
 			/* Handle mouse button events */
 			if (key == BUTTON_LEFT) {
 				v->flags |= ADJ_EGO_XY;
-				v->parm1 = WIN_TO_PIC_X(mouse.x);
-				v->parm2 = WIN_TO_PIC_Y(mouse.y);
+				v->parm1 = WIN_TO_PIC_X(g_mouse.x);
+				v->parm2 = WIN_TO_PIC_Y(g_mouse.y);
 				return true;
 			}
 		}
@@ -191,7 +193,7 @@ int handle_controller(int key) {
 	return false;
 }
 
-void handle_getstring(int key) {
+void AgiEngine::handleGetstring(int key) {
 	static int pos = 0;	/* Cursor position */
 	static char buf[40];
 
@@ -201,32 +203,43 @@ void handle_getstring(int key) {
 	debugC(3, kDebugLevelInput, "handling key: %02x", key);
 
 	switch (key) {
+	case BUTTON_LEFT:
+		if ((int)g_mouse.y >= _stringdata.y * CHAR_LINES &&
+				(int)g_mouse.y <= (_stringdata.y + 1) * CHAR_LINES) {
+			if (predictiveDialog()) {
+				strcpy(_game.strings[_stringdata.str], _predictiveResult);
+				newInputMode(INPUT_NORMAL);
+				_gfx->printCharacter(_stringdata.x + strlen(_game.strings[_stringdata.str]) + 1,
+								_stringdata.y, ' ', _game.colorFg, _game.colorBg);
+				return;
+			}
+		}
+		break;
 	case KEY_ENTER:
 		debugC(3, kDebugLevelInput, "KEY_ENTER");
-		game.has_prompt = 0;
+		_game.hasPrompt = 0;
 		buf[pos] = 0;
-		strcpy(game.strings[stringdata.str], buf);
+		strcpy(_game.strings[_stringdata.str], buf);
 		debugC(3, kDebugLevelInput, "buffer=[%s]", buf);
 		buf[pos = 0] = 0;
-		new_input_mode(INPUT_NORMAL);
-		print_character(stringdata.x + strlen(game.strings[stringdata.str]) + 1,
-				stringdata.y, ' ', game.color_fg, game.color_bg);
+		newInputMode(INPUT_NORMAL);
+		_gfx->printCharacter(_stringdata.x + strlen(_game.strings[_stringdata.str]) + 1,
+				_stringdata.y, ' ', _game.colorFg, _game.colorBg);
 		return;
 	case KEY_ESCAPE:
 		debugC(3, kDebugLevelInput, "KEY_ESCAPE");
-		game.has_prompt = 0;
+		_game.hasPrompt = 0;
 		buf[pos = 0] = 0;
-		strcpy(game.strings[stringdata.str], buf);
-		new_input_mode(INPUT_NORMAL);
-		/* new_input_mode (INPUT_MENU); */
+		strcpy(_game.strings[_stringdata.str], buf);
+		newInputMode(INPUT_NORMAL);
+		/* newInputMode(INPUT_MENU); */
 		break;
 	case KEY_BACKSPACE:	/*0x08: */
 		if (!pos)
 			break;
 
-		print_character(stringdata.x + (pos + 1), stringdata.y,
-				' ', game.color_fg, game.color_bg);
-
+		_gfx->printCharacter(_stringdata.x + (pos + 1), _stringdata.y,
+				' ', _game.colorFg, _game.colorBg);
 		pos--;
 		buf[pos] = 0;
 		break;
@@ -234,42 +247,42 @@ void handle_getstring(int key) {
 		if (key < 0x20 || key > 0x7f)
 			break;
 
-		if (pos >= stringdata.len)
+		if (pos >= _stringdata.len)
 			break;
 
 		buf[pos++] = key;
 		buf[pos] = 0;
 
 		/* Echo */
-		print_character(stringdata.x + pos, stringdata.y, buf[pos - 1],
-				game.color_fg, game.color_bg);
+		_gfx->printCharacter(_stringdata.x + pos, _stringdata.y, buf[pos - 1],
+				_game.colorFg, _game.colorBg);
 
 		break;
 	}
 
 	/* print cursor */
-	print_character(stringdata.x + pos + 1, stringdata.y,
-			(char)game.cursor_char, game.color_fg, game.color_bg);
+	_gfx->printCharacter(_stringdata.x + pos + 1, _stringdata.y,
+			(char)_game.cursorChar, _game.colorFg, _game.colorBg);
 }
 
-void handle_keys(int key) {
+void AgiEngine::handleKeys(int key) {
 	uint8 *p = NULL;
 	int c = 0;
-	static uint8 formated_entry[256];
-	int l = game.line_user_input;
-	int fg = game.color_fg, bg = game.color_bg;
+	static uint8 formatedEntry[256];
+	int l = _game.lineUserInput;
+	int fg = _game.colorFg, bg = _game.colorBg;
 
-	setvar(V_word_not_found, 0);
+	setvar(vWordNotFound, 0);
 
 	debugC(3, kDebugLevelInput, "handling key: %02x", key);
 
 	switch (key) {
 	case KEY_ENTER:
 		debugC(3, kDebugLevelInput, "KEY_ENTER");
-		game.keypress = 0;
+		_game.keypress = 0;
 
 		/* Remove all leading spaces */
-		for (p = game.input_buffer; *p && *p == 0x20; p++);
+		for (p = _game.inputBuffer; *p && *p == 0x20; p++);
 
 		/* Copy to internal buffer */
 		for (; *p; p++) {
@@ -278,39 +291,39 @@ void handle_keys(int key) {
 				p++;
 				continue;
 			}
-			formated_entry[c++] = tolower(*p);
+			formatedEntry[c++] = tolower(*p);
 		}
-		formated_entry[c++] = 0;
+		formatedEntry[c++] = 0;
 
 		/* Handle string only if it's not empty */
-		if (formated_entry[0]) {
-			strcpy((char *)game.echo_buffer, (const char *)game.input_buffer);
-			strcpy(last_sentence, (const char *)formated_entry);
-			dictionary_words(last_sentence);
+		if (formatedEntry[0]) {
+			strcpy((char *)_game.echoBuffer, (const char *)_game.inputBuffer);
+			strcpy(_lastSentence, (const char *)formatedEntry);
+			dictionaryWords(_lastSentence);
 		}
 
 		/* Clear to start a new line */
-		game.has_prompt = 0;
-		game.input_buffer[game.cursor_pos = 0] = 0;
+		_game.hasPrompt = 0;
+		_game.inputBuffer[_game.cursorPos = 0] = 0;
 		debugC(3, kDebugLevelInput, "clear lines");
-		_text->clear_lines(l, l + 1, bg);
-		_text->flush_lines(l, l + 1);
+		clearLines(l, l + 1, bg);
+		flushLines(l, l + 1);
 
 		break;
 	case KEY_ESCAPE:
 		debugC(3, kDebugLevelInput, "KEY_ESCAPE");
-		new_input_mode(INPUT_MENU);
+		newInputMode(INPUT_MENU);
 		break;
 	case KEY_BACKSPACE:
 		/* Ignore backspace at start of line */
-		if (game.cursor_pos == 0)
+		if (_game.cursorPos == 0)
 			break;
 
 		/* erase cursor */
-		print_character(game.cursor_pos + 1, l, ' ', fg, bg);
-		game.input_buffer[--game.cursor_pos] = 0;
+		_gfx->printCharacter(_game.cursorPos + 1, l, ' ', fg, bg);
+		_game.inputBuffer[--_game.cursorPos] = 0;
 		/* Print cursor */
-		print_character(game.cursor_pos + 1, l, game.cursor_char, fg, bg);
+		_gfx->printCharacter(_game.cursorPos + 1, l, _game.cursorChar, fg, bg);
 		break;
 	default:
 		/* Ignore invalid keystrokes */
@@ -318,57 +331,57 @@ void handle_keys(int key) {
 			break;
 
 		/* Maximum input size reached */
-		if (game.cursor_pos >= getvar(V_max_input_chars))
+		if (_game.cursorPos >= getvar(vMaxInputChars))
 			break;
 
-		game.input_buffer[game.cursor_pos++] = key;
-		game.input_buffer[game.cursor_pos] = 0;
+		_game.inputBuffer[_game.cursorPos++] = key;
+		_game.inputBuffer[_game.cursorPos] = 0;
 
 		/* echo */
-		print_character(game.cursor_pos, l, game.input_buffer[game.cursor_pos - 1], fg, bg);
+		_gfx->printCharacter(_game.cursorPos, l, _game.inputBuffer[_game.cursorPos - 1], fg, bg);
 
 		/* Print cursor */
-		print_character(game.cursor_pos + 1, l, game.cursor_char, fg, bg);
+		_gfx->printCharacter(_game.cursorPos + 1, l, _game.cursorChar, fg, bg);
 		break;
 	}
 }
 
-int wait_key() {
+int AgiEngine::waitKey() {
 	int key;
 
 	/* clear key queue */
-	while (keypress()) {
-		get_key();
+	while (_gfx->keypress()) {
+		_gfx->getKey();
 	}
 
 	debugC(3, kDebugLevelInput, "waiting...");
-	while (42) {
-		poll_timer();	/* msdos driver -> does nothing */
-		key = do_poll_keyboard();
+	for (;;) {
+		_gfx->pollTimer();	/* msdos driver -> does nothing */
+		key = doPollKeyboard();
 		if (key == KEY_ENTER || key == KEY_ESCAPE || key == BUTTON_LEFT)
 			break;
-		do_update();
+		_gfx->doUpdate();
 	}
 	return key;
 }
 
-int wait_any_key() {
+int AgiEngine::waitAnyKey() {
 	int key;
 
 	/* clear key queue */
-	while (keypress()) {
-		get_key();
+	while (_gfx->keypress()) {
+		_gfx->getKey();
 	}
 
 	debugC(3, kDebugLevelInput, "waiting...");
-	while (42) {
-		poll_timer();	/* msdos driver -> does nothing */
-		key = do_poll_keyboard();
+	for (;;) {
+		_gfx->pollTimer();	/* msdos driver -> does nothing */
+		key = doPollKeyboard();
 		if (key)
 			break;
-		do_update();
+		_gfx->doUpdate();
 	}
 	return key;
 }
 
-}                             // End of namespace Agi
+} // End of namespace Agi
