@@ -1,7 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
  *
- * Copyright (C) 1999-2001 Sarien Team
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -17,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/agi/keyboard.h $
- * $Id: keyboard.h 22611 2006-05-24 14:25:42Z eriktorbjorn $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/agi/keyboard.h $
+ * $Id: keyboard.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -27,10 +28,29 @@
 
 #include "common/stdafx.h"
 
-#include "agi/agi.h"
-
 namespace Agi {
 
+#define keyEnqueue(k) do { _keyQueue[_keyQueueEnd++] = (k); \
+	_keyQueueEnd %= KEY_QUEUE_SIZE; } while (0)
+#define keyDequeue(k) do { (k) = _keyQueue[_keyQueueStart++]; \
+	_keyQueueStart %= KEY_QUEUE_SIZE; } while (0)
+
+// Class to turn on synthetic events temporarily. Usually until the end of the
+// current function.
+
+class AllowSyntheticEvents {
+private:
+	AgiEngine *_vm;
+public:
+	AllowSyntheticEvents(AgiEngine *vm) : _vm(vm) {
+		_vm->allowSynthetic(true);
+	}
+
+	~AllowSyntheticEvents() {
+		_vm->allowSynthetic(false);
+	}
+};
+
 /* QNX4 has a KEY_DOWN defined which we don't need to care about */
 #undef KEY_DOWN
 
@@ -69,24 +89,14 @@ namespace Agi {
 
 #define BUTTON_LEFT	0xF101	/* Left mouse button */
 #define BUTTON_RIGHT	0xF202	/* Right mouse button */
+#define WHEEL_UP	0xF203	/* Mouse wheel up */
+#define WHEEL_DOWN	0xF204	/* Mouse wheel down */
 
 #define KEY_SCAN(k)	(k >> 8)
 #define KEY_ASCII(k)	(k & 0xff)
 
-extern uint8 scancode_table[];
-
-void init_words(void);
-void clean_input(void);
-int do_poll_keyboard(void);
-void clean_keyboard(void);
-void handle_keys(int);
-void handle_getstring(int);
-int handle_controller(int);
-void get_string(int, int, int, int);
-uint16 agi_get_keypress(void);
-int wait_key(void);
-int wait_any_key(void);
+extern uint8 scancodeTable[];
 
 }                             // End of namespace Agi
 
-#endif				/* AGI_KEYBOARD_H */
+#endif /* AGI_KEYBOARD_H */
