@@ -1,7 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
  *
- * Copyright (C) 1999-2001 Sarien Team
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -17,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/agi/logic.cpp $
- * $Id: logic.cpp 22794 2006-05-31 10:53:04Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/agi/logic.cpp $
+ * $Id: logic.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -34,15 +35,15 @@ namespace Agi {
  * into a message list.
  * @param n  The number of the logic resource to decode.
  */
-int decode_logic(int n) {
-	int ec = err_OK;
+int AgiEngine::decodeLogic(int n) {
+	int ec = errOK;
 	int mstart, mend, mc;
 	uint8 *m0;
 
 	/* decrypt messages at end of logic + build message list */
 
 	/* report ("decoding logic #%d\n", n); */
-	m0 = game.logics[n].data;
+	m0 = _game.logics[n].data;
 
 	mstart = READ_LE_UINT16(m0) + 2;
 	mc = *(m0 + mstart);
@@ -53,41 +54,40 @@ int decode_logic(int n) {
 	/* if the logic was not compressed, decrypt the text messages
 	 * only if there are more than 0 messages
 	 */
-	if ((~game.dir_logic[n].flags & RES_COMPRESSED) && mc > 0)
+	if ((~_game.dirLogic[n].flags & RES_COMPRESSED) && mc > 0)
 		decrypt(m0 + mstart, mend - mstart);	/* decrypt messages */
 
 	/* build message list */
-	m0 = game.logics[n].data;
+	m0 = _game.logics[n].data;
 	mstart = READ_LE_UINT16(m0) + 2;	/* +2 covers pointer */
-	game.logics[n].num_texts = *(m0 + mstart);
+	_game.logics[n].numTexts = *(m0 + mstart);
 
 	/* resetp logic pointers */
-	game.logics[n].sIP = 2;
-	game.logics[n].cIP = 2;
-	game.logics[n].size = READ_LE_UINT16(m0) + 2;	/* logic end pointer */
+	_game.logics[n].sIP = 2;
+	_game.logics[n].cIP = 2;
+	_game.logics[n].size = READ_LE_UINT16(m0) + 2;	/* logic end pointer */
 
 	/* allocate list of pointers to point into our data */
 
-	game.logics[n].texts = (const char **)calloc(1 + game.logics[n].num_texts, sizeof(char *));
+	_game.logics[n].texts = (const char **)calloc(1 + _game.logics[n].numTexts, sizeof(char *));
 
 	/* cover header info */
 	m0 += mstart + 3;
 
-	if (game.logics[n].texts != NULL) {
+	if (_game.logics[n].texts != NULL) {
 		/* move list of strings into list to make real pointers */
-		for (mc = 0; mc < game.logics[n].num_texts; mc++) {
+		for (mc = 0; mc < _game.logics[n].numTexts; mc++) {
 			mend = READ_LE_UINT16(m0 + mc * 2);
-			game.logics[n].texts[mc] = mend ? (const char *)m0 + mend - 2 : (const char *)"";
+			_game.logics[n].texts[mc] = mend ? (const char *)m0 + mend - 2 : (const char *)"";
 		}
 		/* set loaded flag now its all completly loaded */
-		game.dir_logic[n].flags |= RES_LOADED;
+		_game.dirLogic[n].flags |= RES_LOADED;
 	} else {
 		/* unload data
-		 * blah DF YA WANKER!!@!@# frag. i'm so dumb. not every logic
-		 * has text
+		 * Note that not every logic has text
 		 */
-		free(game.logics[n].data);
-		ec = err_NotEnoughMemory;
+		free(_game.logics[n].data);
+		ec = errNotEnoughMemory;
 	}
 
 	return ec;
@@ -99,18 +99,18 @@ int decode_logic(int n) {
  * memory chunks allocated for this resource.
  * @param n  The number of the logic resource to unload
  */
-void unload_logic(int n) {
-	if (game.dir_logic[n].flags & RES_LOADED) {
-		free(game.logics[n].data);
-		if (game.logics[n].num_texts)
-			free(game.logics[n].texts);
-		game.logics[n].num_texts = 0;
-		game.dir_logic[n].flags &= ~RES_LOADED;
+void AgiEngine::unloadLogic(int n) {
+	if (_game.dirLogic[n].flags & RES_LOADED) {
+		free(_game.logics[n].data);
+		if (_game.logics[n].numTexts)
+			free(_game.logics[n].texts);
+		_game.logics[n].numTexts = 0;
+		_game.dirLogic[n].flags &= ~RES_LOADED;
 	}
 
 	/* if cached, we end up here */
-	game.logics[n].sIP = 2;
-	game.logics[n].cIP = 2;
+	_game.logics[n].sIP = 2;
+	_game.logics[n].cIP = 2;
 }
 
-}                             // End of namespace Agi
+} // End of namespace Agi
