@@ -1,7 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
  *
- * Copyright (C) 1999-2001 Sarien Team
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -17,33 +18,31 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/agi/logic.h $
- * $Id: logic.h 22794 2006-05-31 10:53:04Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/agi/logic.h $
+ * $Id: logic.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
 #ifndef AGI_LOGIC_H
 #define AGI_LOGIC_H
 
-#include "agi/agi.h"
+#include "common/stdafx.h"
+#include "common/scummsys.h"
 
 namespace Agi {
 
 /**
  * AGI logic resource structure.
  */
-struct agi_logic {
+struct AgiLogic {
 	uint8 *data;		/**< raw resource data */
 	int size;		/**< size of data */
 	int sIP;		/**< saved IP */
 	int cIP;		/**< current IP */
-	int num_texts;		/**< number of messages */
+	int numTexts;		/**< number of messages */
 	const char **texts;		/**< message list */
 };
 
-int decode_logic(int);
-void unload_logic(int);
-
 }                             // End of namespace Agi
 
-#endif				/* AGI_LOGIC_H */
+#endif /* AGI_LOGIC_H */
