@@ -1,7 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
  *
- * Copyright (C) 1999-2001 Sarien Team
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -17,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/agi/lzw.h $
- * $Id: lzw.h 22611 2006-05-24 14:25:42Z eriktorbjorn $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/agi/lzw.h $
+ * $Id: lzw.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -27,8 +28,8 @@
 
 namespace Agi {
 
-void LZW_expand(uint8 *, uint8 *, int32);
+void lzwExpand(uint8 *, uint8 *, int32);
 
 }				// End of namespace Agi
 
-#endif				/* AGI_LZW_H */
+#endif /* AGI_LZW_H */
