@@ -1,7 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
  *
- * Copyright (C) 1999-2001 Sarien Team
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -17,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/agi/motion.cpp $
- * $Id: motion.cpp 22638 2006-05-25 22:39:44Z wjpalenstijn $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/agi/motion.cpp $
+ * $Id: motion.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -28,71 +29,71 @@
 
 namespace Agi {
 
-static int check_step(int delta, int step) {
+int AgiEngine::checkStep(int delta, int step) {
 	return (-step >= delta) ? 0 : (step <= delta) ? 2 : 1;
 }
 
-static int check_block(int x, int y) {
-	if (x <= game.block.x1 || x >= game.block.x2)
+int AgiEngine::checkBlock(int x, int y) {
+	if (x <= _game.block.x1 || x >= _game.block.x2)
 		return false;
 
-	if (y <= game.block.y1 || y >= game.block.y2)
+	if (y <= _game.block.y1 || y >= _game.block.y2)
 		return false;
 
 	return true;
 }
 
-static void changepos(struct vt_entry *v) {
+void AgiEngine::changePos(VtEntry *v) {
 	int b, x, y;
 	int dx[9] = { 0, 0, 1, 1, 1, 0, -1, -1, -1 };
 	int dy[9] = { 0, -1, -1, 0, 1, 1, 1, 0, -1 };
 
-	x = v->x_pos;
-	y = v->y_pos;
-	b = check_block(x, y);
+	x = v->xPos;
+	y = v->yPos;
+	b = checkBlock(x, y);
 
-	x += v->step_size * dx[v->direction];
-	y += v->step_size * dy[v->direction];
+	x += v->stepSize * dx[v->direction];
+	y += v->stepSize * dy[v->direction];
 
-	if (check_block(x, y) == b) {
+	if (checkBlock(x, y) == b) {
 		v->flags &= ~MOTION;
 	} else {
 		v->flags |= MOTION;
 		v->direction = 0;
-		if (is_ego_view(v))
-			game.vars[V_ego_dir] = 0;
+		if (isEgoView(v))
+			_game.vars[vEgoDir] = 0;
 	}
 }
 
-static void motion_wander(struct vt_entry *v) {
+void AgiEngine::motionWander(VtEntry *v) {
 	if (v->parm1--) {
 		if (~v->flags & DIDNT_MOVE)
 			return;
 	}
 
-	v->direction = rnd->getRandomNumber(8);
+	v->direction = _rnd->getRandomNumber(8);
 
-	if (is_ego_view(v)) {
-		game.vars[V_ego_dir] = v->direction;
+	if (isEgoView(v)) {
+		_game.vars[vEgoDir] = v->direction;
 		while (v->parm1 < 6) {
-			v->parm1 = rnd->getRandomNumber(50);	/* huh? */
+			v->parm1 = _rnd->getRandomNumber(50);	/* huh? */
 		}
 	}
 }
 
-static void motion_followego(struct vt_entry *v) {
-	int ego_x, ego_y;
-	int obj_x, obj_y;
+void AgiEngine::motionFollowEgo(VtEntry *v) {
+	int egoX, egoY;
+	int objX, objY;
 	int dir;
 
-	ego_x = game.view_table[0].x_pos + game.view_table[0].x_size / 2;
-	ego_y = game.view_table[0].y_pos;
+	egoX = _game.viewTable[0].xPos + _game.viewTable[0].xSize / 2;
+	egoY = _game.viewTable[0].yPos;
 
-	obj_x = v->x_pos + v->x_size / 2;
-	obj_y = v->y_pos;
+	objX = v->xPos + v->xSize / 2;
+	objY = v->yPos;
 
 	/* Get direction to reach ego */
-	dir = get_direction(obj_x, obj_y, ego_x, ego_y, v->parm1);
+	dir = getDirection(objX, objY, egoX, egoY, v->parm1);
 
 	/* Already at ego coordinates */
 	if (dir == 0) {
@@ -107,17 +108,17 @@ static void motion_followego(struct vt_entry *v) {
 	} else if (v->flags & DIDNT_MOVE) {
 		int d;
 
-		while ((v->direction = rnd->getRandomNumber(8)) == 0) {
+		while ((v->direction = _rnd->getRandomNumber(8)) == 0) {
 		}
 
-		d = (abs(ego_y - obj_y) + abs(ego_x - obj_x)) / 2;
+		d = (ABS(egoY - objY) + ABS(egoX - objX)) / 2;
 
-		if (d < v->step_size) {
-			v->parm3 = v->step_size;
+		if (d < v->stepSize) {
+			v->parm3 = v->stepSize;
 			return;
 		}
 
-		while ((v->parm3 = rnd->getRandomNumber(d)) < v->step_size) {
+		while ((v->parm3 = _rnd->getRandomNumber(d)) < v->stepSize) {
 		}
 		return;
 	}
@@ -132,7 +133,7 @@ static void motion_followego(struct vt_entry *v) {
 		 *      v->parm3 = 0;
 		 */
 		k = v->parm3;
-		k -= v->step_size;
+		k -= v->stepSize;
 		v->parm3 = k;
 
 		if ((int8) v->parm3 < 0)
@@ -142,32 +143,32 @@ static void motion_followego(struct vt_entry *v) {
 	}
 }
 
-static void motion_moveobj(struct vt_entry *v) {
-	v->direction = get_direction(v->x_pos, v->y_pos, v->parm1, v->parm2, v->step_size);
+void AgiEngine::motionMoveObj(VtEntry *v) {
+	v->direction = getDirection(v->xPos, v->yPos, v->parm1, v->parm2, v->stepSize);
 
 	/* Update V6 if ego */
-	if (is_ego_view(v))
-		game.vars[V_ego_dir] = v->direction;
+	if (isEgoView(v))
+		_game.vars[vEgoDir] = v->direction;
 
 	if (v->direction == 0)
-		in_destination(v);
+		inDestination(v);
 }
 
-static void check_motion(struct vt_entry *v) {
+void AgiEngine::checkMotion(VtEntry *v) {
 	switch (v->motion) {
 	case MOTION_WANDER:
-		motion_wander(v);
+		motionWander(v);
 		break;
 	case MOTION_FOLLOW_EGO:
-		motion_followego(v);
+		motionFollowEgo(v);
 		break;
 	case MOTION_MOVE_OBJ:
-		motion_moveobj(v);
+		motionMoveObj(v);
 		break;
 	}
 
-	if ((game.block.active && (~v->flags & IGNORE_BLOCKS)) && v->direction)
-		changepos(v);
+	if ((_game.block.active && (~v->flags & IGNORE_BLOCKS)) && v->direction)
+		changePos(v);
 }
 
 /*
@@ -177,13 +178,13 @@ static void check_motion(struct vt_entry *v) {
 /**
  *
  */
-void check_all_motions() {
-	struct vt_entry *v;
+void AgiEngine::checkAllMotions() {
+	VtEntry *v;
 
-	for (v = game.view_table; v < &game.view_table[MAX_VIEWTABLE]; v++) {
+	for (v = _game.viewTable; v < &_game.viewTable[MAX_VIEWTABLE]; v++) {
 		if ((v->flags & (ANIMATED | UPDATE | DRAWN)) == (ANIMATED | UPDATE | DRAWN)
-				&& v->step_time_count == 1) {
-			check_motion(v);
+				&& v->stepTimeCount == 1) {
+			checkMotion(v);
 		}
 	}
 }
@@ -194,14 +195,14 @@ void check_all_motions() {
  * type motion that * has reached its final destination coordinates.
  * @param  v  Pointer to view table entry
  */
-void in_destination(struct vt_entry *v) {
+void AgiEngine::inDestination(VtEntry *v) {
 	if (v->motion == MOTION_MOVE_OBJ) {
-		v->step_size = v->parm3;
+		v->stepSize = v->parm3;
 		setflag(v->parm4, true);
 	}
 	v->motion = MOTION_NORMAL;
-	if (is_ego_view(v))
-		game.player_control = true;
+	if (isEgoView(v))
+		_game.playerControl = true;
 }
 
 /**
@@ -210,8 +211,8 @@ void in_destination(struct vt_entry *v) {
  * after setting the motion mode to MOTION_MOVE_OBJ.
  * @param  v  Pointer to view table entry
  */
-void move_obj(struct vt_entry *v) {
-	motion_moveobj(v);
+void AgiEngine::moveObj(VtEntry *v) {
+	motionMoveObj(v);
 }
 
 /**
@@ -224,9 +225,9 @@ void move_obj(struct vt_entry *v) {
  * @param  y   y coordinate of the object
  * @param  s   step size
  */
-int get_direction(int x0, int y0, int x, int y, int s) {
-	int dir_table[9] = { 8, 1, 2, 7, 0, 3, 6, 5, 4 };
-	return dir_table[check_step(x - x0, s) + 3 * check_step(y - y0, s)];
+int AgiEngine::getDirection(int x0, int y0, int x, int y, int s) {
+	int dirTable[9] = { 8, 1, 2, 7, 0, 3, 6, 5, 4 };
+	return dirTable[checkStep(x - x0, s) + 3 * checkStep(y - y0, s)];
 }
 
-}                             // End of namespace Agi
+} // End of namespace Agi
