@@ -1,7 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
  *
- * Copyright (C) 1999-2003 Sarien Team
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -17,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/agi/objects.cpp $
- * $Id: objects.cpp 22659 2006-05-26 14:00:52Z eriktorbjorn $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/agi/objects.cpp $
+ * $Id: objects.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -28,24 +29,20 @@
 
 namespace Agi {
 
-extern int decode_objects(uint8 *mem, uint32 flen);
+int AgiEngine::allocObjects(int n) {
+	if ((_objects = (AgiObject *)calloc(n, sizeof(struct AgiObject))) == NULL)
+		return errNotEnoughMemory;
 
-static struct agi_object *objects;	/* objects in the game */
-
-int alloc_objects(int n) {
-	if ((objects = (agi_object *) calloc(n, sizeof(struct agi_object))) == NULL)
-		return err_NotEnoughMemory;
-
-	return err_OK;
+	return errOK;
 }
 
-int decode_objects(uint8 *mem, uint32 flen) {
+int AgiEngine::decodeObjects(uint8 *mem, uint32 flen) {
 	unsigned int i, so, padsize;
 
-	padsize = game.game_flags & ID_AMIGA ? 4 : 3;
+	padsize = _game.gameFlags & ID_AMIGA ? 4 : 3;
 
-	game.num_objects = 0;
-	objects = NULL;
+	_game.numObjects = 0;
+	_objects = NULL;
 
 	/* check if first pointer exceeds file size
 	 * if so, its encrypted, else it is not
@@ -62,102 +59,101 @@ int decode_objects(uint8 *mem, uint32 flen) {
 	 */
 	if (READ_LE_UINT16(mem) / padsize >= 256) {
 		/* die with no error! AGDS game needs not to die to work!! :( */
-		return err_OK;
+		return errOK;
 	}
 
-	game.num_objects = READ_LE_UINT16(mem) / padsize;
-	debugC(5, kDebugLevelResources, "num_objects = %d (padsize = %d)", game.num_objects, padsize);
+	_game.numObjects = READ_LE_UINT16(mem) / padsize;
+	debugC(5, kDebugLevelResources, "num_objects = %d (padsize = %d)", _game.numObjects, padsize);
 
-	if (alloc_objects(game.num_objects) != err_OK)
-		return err_NotEnoughMemory;
+	if (allocObjects(_game.numObjects) != errOK)
+		return errNotEnoughMemory;
 
 	/* build the object list */
-	for (i = 0, so = padsize; i < game.num_objects; i++, so += padsize) {
+	for (i = 0, so = padsize; i < _game.numObjects; i++, so += padsize) {
 		int offset;
 
-		(objects + i)->location = *(mem + so + 2);
+		(_objects + i)->location = *(mem + so + 2);
 		offset = READ_LE_UINT16(mem + so) + padsize;
 
 		if ((uint) offset < flen) {
-			(objects + i)->name = (char *)strdup((const char *)mem + offset);
+			(_objects + i)->name = (char *)strdup((const char *)mem + offset);
 		} else {
 			printf("ERROR: object %i name beyond object filesize! "
 					"(%04x > %04x)\n", i, offset, flen);
-			(objects + i)->name = strdup("");
+			(_objects + i)->name = strdup("");
 		}
 	}
-	report("Reading objects: %d objects read.\n", game.num_objects);
-
-	return err_OK;
+	report("Reading objects: %d objects read.\n", _game.numObjects);
 
+	return errOK;
 }
 
-int load_objects(const char *fname) {
+int AgiEngine::loadObjects(const char *fname) {
 	Common::File fp;
 	uint32 flen;
 	uint8 *mem;
 
-	objects = NULL;
-	game.num_objects = 0;
+	_objects = NULL;
+	_game.numObjects = 0;
 
 	debugC(5, kDebugLevelResources, "(fname = %s)", fname);
 	report("Loading objects: %s\n", fname);
 
 	if (!fp.open(fname))
-		return err_BadFileOpen;
+		return errBadFileOpen;
 
 	fp.seek(0, SEEK_END);
 	flen = fp.pos();
 	fp.seek(0, SEEK_SET);
 
-	if ((mem = (uint8 *) calloc(1, flen + 32)) == NULL) {
+	if ((mem = (uint8 *)calloc(1, flen + 32)) == NULL) {
 		fp.close();
-		return err_NotEnoughMemory;
+		return errNotEnoughMemory;
 	}
 
 	fp.read(mem, flen);
 	fp.close();
 
-	decode_objects(mem, flen);
+	decodeObjects(mem, flen);
 	free(mem);
-	return err_OK;
+	return errOK;
 }
 
-void unload_objects() {
+void AgiEngine::unloadObjects() {
 	unsigned int i;
 
-	if (objects != NULL) {
-		for (i = 0; i < game.num_objects; i++) {
-			free(objects[i].name);
-			objects[i].name = NULL;
+	if (_objects != NULL) {
+		for (i = 0; i < _game.numObjects; i++) {
+			free(_objects[i].name);
+			_objects[i].name = NULL;
 		}
-		free(objects);
-		objects = NULL;
+		free(_objects);
+		_objects = NULL;
 	}
 }
 
-void object_set_location(unsigned int n, int i) {
-	if (n >= game.num_objects) {
+void AgiEngine::objectSetLocation(unsigned int n, int i) {
+	if (n >= _game.numObjects) {
 		report("Error: Can't access object %d.\n", n);
 		return;
 	}
-	objects[n].location = i;
+	_objects[n].location = i;
 }
 
-int object_get_location(unsigned int n) {
-	if (n >= game.num_objects) {
+int AgiEngine::objectGetLocation(unsigned int n) {
+	if (n >= _game.numObjects) {
 		report("Error: Can't access object %d.\n", n);
 		return 0;
 	}
-	return objects[n].location;
+	return _objects[n].location;
 }
 
-const char *object_name(unsigned int n) {
-	if (n >= game.num_objects) {
+const char *AgiEngine::objectName(unsigned int n) {
+	if (n >= _game.numObjects) {
 		report("Error: Can't access object %d.\n", n);
 		return "";
 	}
-	return objects[n].name;
+	return _objects[n].name;
 }
 
-}                             // End of namespace Agi
+} // End of namespace Agi
