@@ -1,7 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
  *
- * Copyright (C) 1999-2003 Sarien Team
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -17,12 +18,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/agi/op_cmd.cpp $
- * $Id: op_cmd.cpp 23011 2006-06-10 13:20:29Z ender $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/agi/op_cmd.cpp $
+ * $Id: op_cmd.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
 #include "common/stdafx.h"
+#include "base/version.h"
 
 #include "agi/agi.h"
 #include "agi/sprite.h"
@@ -30,8 +32,6 @@
 #include "agi/keyboard.h"
 #include "agi/opcodes.h"
 #include "agi/menu.h"
-#include "agi/savegame.h"
-#include "agi/text.h"
 
 namespace Agi {
 
@@ -43,12 +43,20 @@ namespace Agi {
 #define p5	(p[5])
 #define p6	(p[6])
 
-#define ip	cur_logic->cIP
-#define vt	game.view_table[p0]
+#define game g_agi->_game
+#define g_sprites g_agi->_sprites
+#define g_sound g_agi->_sound
+#define g_text g_agi->_text
+#define g_gfx g_agi->_gfx
+#define g_picture g_agi->_picture
 
-static struct agi_logic *cur_logic;
+#define ip	curLogic->cIP
+#define vt	game.viewTable[p0]
 
-int timer_hack;			/* Workaround for timer loop in MH1 */
+static struct AgiLogic *curLogic;
+static AgiEngine *g_agi;
+
+int timerHack;			/* Workaround for timer loop in MH1 */
 
 #define _v game.vars
 #define cmd(x) static void cmd_##x (uint8 *p)
@@ -104,7 +112,7 @@ cmd(div_v) {
 }
 
 cmd(random) {
-	_v[p2] = rnd->getRandomNumber(p1 - p0) + p0;
+	_v[p2] = g_agi->_rnd->getRandomNumber(p1 - p0) + p0;
 }
 
 cmd(lindirectn) {
@@ -120,59 +128,59 @@ cmd(rindirect) {
 }
 
 cmd(set) {
-	setflag(*p, true);
+	g_agi->setflag(*p, true);
 }
 
 cmd(reset) {
-	setflag(*p, false);
+	g_agi->setflag(*p, false);
 }
 
 cmd(toggle) {
-	setflag(*p, !getflag(*p));
+	g_agi->setflag(*p, !g_agi->getflag(*p));
 }
 
 cmd(set_v) {
-	setflag(_v[p0], true);
+	g_agi->setflag(_v[p0], true);
 }
 
 cmd(reset_v) {
-	setflag(_v[p0], false);
+	g_agi->setflag(_v[p0], false);
 }
 
 cmd(toggle_v) {
-	setflag(_v[p0], !getflag(_v[p0]));
+	g_agi->setflag(_v[p0], !g_agi->getflag(_v[p0]));
 }
 
 cmd(new_room) {
-	new_room(p0);
+	g_agi->newRoom(p0);
 }
 
 cmd(new_room_f) {
-	new_room(_v[p0]);
+	g_agi->newRoom(_v[p0]);
 }
 
 cmd(load_view) {
-	agi_load_resource(rVIEW, p0);
+	g_agi->agiLoadResource(rVIEW, p0);
 }
 
 cmd(load_logic) {
-	agi_load_resource(rLOGIC, p0);
+	g_agi->agiLoadResource(rLOGIC, p0);
 }
 
 cmd(load_sound) {
-	agi_load_resource(rSOUND, p0);
+	g_agi->agiLoadResource(rSOUND, p0);
 }
 
 cmd(load_view_f) {
-	agi_load_resource(rVIEW, _v[p0]);
+	g_agi->agiLoadResource(rVIEW, _v[p0]);
 }
 
 cmd(load_logic_f) {
-	agi_load_resource(rLOGIC, _v[p0]);
+	g_agi->agiLoadResource(rLOGIC, _v[p0]);
 }
 
 cmd(discard_view) {
-	agi_unload_resource(rVIEW, p0);
+	g_agi->agiUnloadResource(rVIEW, p0);
 }
 
 cmd(object_on_anything) {
@@ -231,6 +239,24 @@ cmd(get_priority) {
 cmd(set_priority) {
 	vt.flags |= FIXED_PRIORITY;
 	vt.priority = p1;
+
+	// WORKAROUND: this fixes bug #1712585 in KQ4 (dwarf sprite priority)
+	// For this scene, ego (Rosella) hasn't got a fixed priority till script 54
+	// explicitly sets priority 8 for her, so that she can walk back to the table
+	// without being drawn over the other dwarfs
+	// It seems that in this scene, ego's priority is set to 8, but the priority of
+	// the last dwarf with the soup bowls (view 152) is also set to 8, which causes
+	// the dwarf to be drawn behind ego
+	// With this workaround, when the game scripts set the priority of view 152
+	// (seventh dwarf with soup bowls), ego's priority is set to 7
+	// The game script itself sets priotity 8 for ego before she starts walking, 
+	// and then releases the fixed priority set on ego after ego is seated
+	// Therefore, this workaround only affects that specific part of this scene
+	// Ego is set to object 19 by script 54
+	if (g_agi->getGameID() == GID_KQ4 && vt.currentView == 152) {
+		game.viewTable[19].flags |= FIXED_PRIORITY;
+		game.viewTable[19].priority = 7;
+	}
 }
 
 cmd(set_priority_f) {
@@ -246,59 +272,59 @@ cmd(set_upper_left) {				/* do nothing (AGI 2.917) */
 }
 
 cmd(start_update) {
-	start_update(&vt);
+	g_agi->startUpdate(&vt);
 }
 
 cmd(stop_update) {
-	stop_update(&vt);
+	g_agi->stopUpdate(&vt);
 }
 
 cmd(current_view) {
-	_v[p1] = vt.current_view;
+	_v[p1] = vt.currentView;
 }
 
 cmd(current_cel) {
-	_v[p1] = vt.current_cel;
+	_v[p1] = vt.currentCel;
 	debugC(4, kDebugLevelScripts, "v%d=%d", p1, _v[p1]);
 }
 
 cmd(current_loop) {
-	_v[p1] = vt.current_loop;
+	_v[p1] = vt.currentLoop;
 }
 
 cmd(last_cel) {
-	_v[p1] = vt.loop_data->num_cels - 1;
+	_v[p1] = vt.loopData->numCels - 1;
 }
 
 cmd(set_cel) {
-	set_cel(&vt, p1);
+	g_agi->setCel(&vt, p1);
 	vt.flags &= ~DONTUPDATE;
 }
 
 cmd(set_cel_f) {
-	set_cel(&vt, _v[p1]);
+	g_agi->setCel(&vt, _v[p1]);
 	vt.flags &= ~DONTUPDATE;
 }
 
 cmd(set_view) {
 	debugC(4, kDebugLevelScripts, "o%d, %d", p0, p1);
-	set_view(&vt, p1);
+	g_agi->setView(&vt, p1);
 }
 
 cmd(set_view_f) {
-	set_view(&vt, _v[p1]);
+	g_agi->setView(&vt, _v[p1]);
 }
 
 cmd(set_loop) {
-	set_loop(&vt, p1);
+	g_agi->setLoop(&vt, p1);
 }
 
 cmd(set_loop_f) {
-	set_loop(&vt, _v[p1]);
+	g_agi->setLoop(&vt, _v[p1]);
 }
 
 cmd(number_of_loops) {
-	_v[p1] = vt.num_loops;
+	_v[p1] = vt.numLoops;
 }
 
 cmd(fix_loop) {
@@ -310,15 +336,15 @@ cmd(release_loop) {
 }
 
 cmd(step_size) {
-	vt.step_size = _v[p1];
+	vt.stepSize = _v[p1];
 }
 
 cmd(step_time) {
-	vt.step_time = vt.step_time_count = _v[p1];
+	vt.stepTime = vt.stepTimeCount = _v[p1];
 }
 
 cmd(cycle_time) {
-	vt.cycle_time = vt.cycle_time_count = _v[p1];
+	vt.cycleTime = vt.cycleTimeCount = _v[p1];
 }
 
 cmd(stop_cycling) {
@@ -348,104 +374,104 @@ cmd(get_dir) {
 }
 
 cmd(get_room_f) {
-	_v[p1] = object_get_location(_v[p0]);
+	_v[p1] = g_agi->objectGetLocation(_v[p0]);
 }
 
 cmd(put) {
-	object_set_location(p0, _v[p1]);
+	g_agi->objectSetLocation(p0, _v[p1]);
 }
 
 cmd(put_f) {
-	object_set_location(_v[p0], _v[p1]);
+	g_agi->objectSetLocation(_v[p0], _v[p1]);
 }
 
 cmd(drop) {
-	object_set_location(p0, 0);
+	g_agi->objectSetLocation(p0, 0);
 }
 
 cmd(get) {
-	object_set_location(p0, EGO_OWNED);
+	g_agi->objectSetLocation(p0, EGO_OWNED);
 }
 
 cmd(get_f) {
-	object_set_location(_v[p0], EGO_OWNED);
+	g_agi->objectSetLocation(_v[p0], EGO_OWNED);
 }
 
 cmd(word_to_string) {
-	strcpy(game.strings[p0], game.ego_words[p1].word);
+	strcpy(game.strings[p0], game.egoWords[p1].word);
 }
 
 cmd(open_dialogue) {
 	debugC(4, kDebugLevelScripts, "p0 = %d", p0);
-	game.has_window = true;
+	game.hasWindow = true;
 }
 
 cmd(close_dialogue) {
 	debugC(4, kDebugLevelScripts, "p0 = %d", p0);
-	game.has_window = false;
+	game.hasWindow = false;
 }
 
 cmd(close_window) {
-	_text->close_window();
+	g_agi->closeWindow();
 }
 
 cmd(status_line_on) {
-	game.status_line = true;
-	_text->write_status();
+	game.statusLine = true;
+	g_agi->writeStatus();
 }
 
 cmd(status_line_off) {
-	game.status_line = false;
-	_text->write_status();
+	game.statusLine = false;
+	g_agi->writeStatus();
 }
 
 cmd(show_obj) {
-	_sprites->show_obj(p0);
+	g_sprites->showObj(p0);
 }
 
 cmd(show_obj_v) {
-	_sprites->show_obj(_v[p0]);
+	g_sprites->showObj(_v[p0]);
 }
 
 cmd(sound) {
-	start_sound(p0, p1);
+	g_sound->startSound(p0, p1);
 }
 
 cmd(stop_sound) {
-	stop_sound();
+	g_sound->stopSound();
 }
 
 cmd(menu_input) {
-	new_input_mode(INPUT_MENU);
+	g_agi->newInputMode(INPUT_MENU);
 }
 
 cmd(enable_item) {
-	menu->set_item(p0, true);
+	g_agi->_menu->setItem(p0, true);
 }
 
 cmd(disable_item) {
-	menu->set_item(p0, false);
+	g_agi->_menu->setItem(p0, false);
 }
 
 cmd(submit_menu) {
-	menu->submit();
+	g_agi->_menu->submit();
 }
 
 cmd(set_scan_start) {
-	cur_logic->sIP = cur_logic->cIP;
+	curLogic->sIP = curLogic->cIP;
 }
 
 cmd(reset_scan_start) {
-	cur_logic->sIP = 2;
+	curLogic->sIP = 2;
 }
 
 cmd(save_game) {
-	game.simple_save ? savegame_simple() : savegame_dialog();
+	game.simpleSave ? g_agi->saveGameSimple() : g_agi->saveGameDialog();
 }
 
 cmd(load_game) {
 	assert(1);
-	game.simple_save ? loadgame_simple() : loadgame_dialog();
+	game.simpleSave ? g_agi->loadGameSimple() : g_agi->loadGameDialog();
 }
 
 cmd(init_disk) {				/* do nothing */
@@ -461,7 +487,7 @@ cmd(trace_info) {				/* do nothing */
 }
 
 cmd(show_mem) {
-	_text->message_box("Enough memory");
+	g_agi->messageBox("Enough memory");
 }
 
 cmd(init_joy) { /* do nothing */ ;
@@ -489,7 +515,7 @@ cmd(obj_status_f) {
  * unk_181: Deactivate keypressed control (default control of ego)
  */
 cmd(set_simple) {
-	game.simple_save = true;
+	game.simpleSave = true;
 }
 
 cmd(pop_script) {
@@ -525,32 +551,32 @@ cmd(release_key) {
 }
 
 cmd(adj_ego_move_to_x_y) {
-	game.view_table[0].flags |= ADJ_EGO_XY;
+	game.viewTable[0].flags |= ADJ_EGO_XY;
 }
 
 cmd(parse) {
-	_v[V_word_not_found] = 0;
-	setflag(F_entered_cli, false);
-	setflag(F_said_accepted_input, false);
+	_v[vWordNotFound] = 0;
+	g_agi->setflag(fEnteredCli, false);
+	g_agi->setflag(fSaidAcceptedInput, false);
 
-	dictionary_words(_text->agi_sprintf(game.strings[p0]));
+	g_agi->dictionaryWords(g_agi->agiSprintf(game.strings[p0]));
 }
 
 cmd(call) {
-	int old_cIP;
-	int old_lognum;
+	int oldCIP;
+	int oldLognum;
 
 	/* CM: we don't save sIP because set.scan.start can be
 	 *     used in a called script (fixes xmas demo)
 	 */
-	old_cIP = cur_logic->cIP;
-	old_lognum = game.lognum;
+	oldCIP = curLogic->cIP;
+	oldLognum = game.lognum;
 
-	run_logic(p0);
+	g_agi->runLogic(p0);
 
-	game.lognum = old_lognum;
-	cur_logic = &game.logics[game.lognum];
-	cur_logic->cIP = old_cIP;
+	game.lognum = oldLognum;
+	curLogic = &game.logics[game.lognum];
+	curLogic->cIP = oldCIP;
 }
 
 cmd(call_f) {
@@ -559,26 +585,41 @@ cmd(call_f) {
 
 cmd(draw_pic) {
 	debugC(6, kDebugLevelScripts, "=== draw pic %d ===", _v[p0]);
-	_sprites->erase_both();
-	decode_picture(_v[p0], true);
-	_sprites->blit_both();
-	game.picture_shown = 0;
+	g_sprites->eraseBoth();
+	g_picture->decodePicture(_v[p0], true);
+	g_sprites->blitBoth();
+	game.pictureShown = 0;
 	debugC(6, kDebugLevelScripts, "--- end of draw pic %d ---", _v[p0]);
+
+	// WORKAROUND for a script bug which exists in SQ1, logic scripts
+	// 20 and 110. Flag 103 is not reset correctly, which leads to erroneous
+	// behavior from view 46 (the spider droid). View 46 is supposed to
+	// follow ego and explode when it comes in contact with him. However, as
+	// flag 103 is not reset correctly, when the player goes down the path
+	// and back up, the spider is always at the base of the path (since it 
+	// can't go up) and kills the player when he goes down at ground level 
+	// (although the spider droid sprite itself seems to be correctly positioned). 
+	// With this workaround, when the player goes back to picture 20 (1 screen 
+	// above the ground), flag 103 is reset, thereby fixing this issue. Note 
+	// that this is a script bug and occurs in the original interpreter as well.
+	// Fixes bug #1658514: AGI: SQ1 (2.2 DOS ENG) bizzare exploding roger
+	if (g_agi->getGameID() == GID_SQ1 && _v[p0] == 20)
+		g_agi->setflag(103, false);
 }
 
 cmd(show_pic) {
 	debugC(6, kDebugLevelScripts, "=== show pic ===");
-	setflag(F_output_mode, false);
+	g_agi->setflag(fOutputMode, false);
 	cmd_close_window(NULL);
-	show_pic();
-	game.picture_shown = 1;
+	g_picture->showPic();
+	game.pictureShown = 1;
 	debugC(6, kDebugLevelScripts, "--- end of show pic ---");
 }
 
 cmd(load_pic) {
-	_sprites->erase_both();
-	agi_load_resource(rPICTURE, _v[p0]);
-	_sprites->blit_both();
+	g_sprites->eraseBoth();
+	g_agi->agiLoadResource(rPICTURE, _v[p0]);
+	g_sprites->blitBoth();
 }
 
 cmd(discard_pic) {
@@ -588,23 +629,23 @@ cmd(discard_pic) {
 
 cmd(overlay_pic) {
 	debugC(6, kDebugLevelScripts, "--- overlay pic ---");
-	_sprites->erase_both();
-	decode_picture(_v[p0], false);
-	_sprites->blit_both();
-	game.picture_shown = 0;
-	_sprites->commit_both();
+	g_sprites->eraseBoth();
+	g_picture->decodePicture(_v[p0], false);
+	g_sprites->blitBoth();
+	game.pictureShown = 0;
+	g_sprites->commitBoth();
 }
 
 cmd(show_pri_screen) {
-	debug_.priority = 1;
-	_sprites->erase_both();
-	show_pic();
-	_sprites->blit_both();
-	wait_key();
-	debug_.priority = 0;
-	_sprites->erase_both();
-	show_pic();
-	_sprites->blit_both();
+	g_agi->_debug.priority = 1;
+	g_sprites->eraseBoth();
+	g_picture->showPic();
+	g_sprites->blitBoth();
+	g_agi->waitKey();
+	g_agi->_debug.priority = 0;
+	g_sprites->eraseBoth();
+	g_picture->showPic();
+	g_sprites->blitBoth();
 }
 
 cmd(animate_obj) {
@@ -621,37 +662,48 @@ cmd(animate_obj) {
 cmd(unanimate_all) {
 	int i;
 	for (i = 0; i < MAX_VIEWTABLE; i++)
-		game.view_table[i].flags &= ~(ANIMATED | DRAWN);
+		game.viewTable[i].flags &= ~(ANIMATED | DRAWN);
 }
 
 cmd(draw) {
 	if (vt.flags & DRAWN)
 		return;
 
-	if (vt.y_size <= 0 || vt.x_size <= 0)
+	if (vt.ySize <= 0 || vt.xSize <= 0)
 		return;
 
 	debugC(4, kDebugLevelScripts, "draw entry %d", vt.entry);
 
 	vt.flags |= UPDATE;
-	if (agi_get_release() >= 0x3000) {
-		set_loop(&vt, vt.current_loop);
-		set_cel(&vt, vt.current_cel);
+	if (g_agi->agiGetRelease() >= 0x3000) {
+		g_agi->setLoop(&vt, vt.currentLoop);
+		g_agi->setCel(&vt, vt.currentCel);
 	}
-	fix_position(p0);
-	vt.x_pos2 = vt.x_pos;
-	vt.y_pos2 = vt.y_pos;
-	vt.cel_data_2 = vt.cel_data;
-	_sprites->erase_upd_sprites();
+	g_agi->fixPosition(p0);
+	vt.xPos2 = vt.xPos;
+	vt.yPos2 = vt.yPos;
+	vt.celData2 = vt.celData;
+	g_sprites->eraseUpdSprites();
 	vt.flags |= DRAWN;
 
-	if (agi_get_release() <= 0x2440)	/* See bug #546562 */
+	// WORKAROUND: This fixes a bug with AGI Fanmade game Space Trek.
+	// The original workaround checked if AGI version was <= 2.440, which could
+	// cause regressions with some AGI games. The original workaround no longer
+	// works for Space Trek in ScummVM, as all fanmade games are set to use
+	// AGI version 2.917, but it applies to all other games where AGI version is
+	// <= 2.440, which was not the original purpose of this workaround. It is 
+	// assumed that this bug is caused by AGI Studio, so this applies to all 
+	// fanmade games only.
+	// TODO: Investigate this further and check if any other fanmade AGI
+	// games are affected. If yes, then it'd be best to set this for Space
+	// Trek only
+	if (g_agi->getFeatures() & GF_FANMADE)	/* See Sarien bug #546562 */
 		vt.flags |= ANIMATED;
 
-	_sprites->blit_upd_sprites();
+	g_sprites->blitUpdSprites();
 	vt.flags &= ~DONTUPDATE;
 
-	_sprites->commit_block(vt.x_pos, vt.y_pos - vt.y_size + 1, vt.x_pos + vt.x_size - 1, vt.y_pos);
+	g_sprites->commitBlock(vt.xPos, vt.yPos - vt.ySize + 1, vt.xPos + vt.xSize - 1, vt.yPos);
 
 	debugC(4, kDebugLevelScripts, "vt entry #%d flags = %02x", p0, vt.flags);
 }
@@ -660,32 +712,32 @@ cmd(erase) {
 	if (~vt.flags & DRAWN)
 		return;
 
-	_sprites->erase_upd_sprites();
+	g_sprites->eraseUpdSprites();
 	if (vt.flags & UPDATE) {
 		vt.flags &= ~DRAWN;
 	} else {
-		_sprites->erase_nonupd_sprites();
+		g_sprites->eraseNonupdSprites();
 		vt.flags &= ~DRAWN;
-		_sprites->blit_nonupd_sprites();
+		g_sprites->blitNonupdSprites();
 	}
-	_sprites->blit_upd_sprites();
+	g_sprites->blitUpdSprites();
 
-	_sprites->commit_block(vt.x_pos, vt.y_pos - vt.y_size + 1, vt.x_pos + vt.x_size - 1, vt.y_pos);
+	g_sprites->commitBlock(vt.xPos, vt.yPos - vt.ySize + 1, vt.xPos + vt.xSize - 1, vt.yPos);
 }
 
 cmd(position) {
-	vt.x_pos = vt.x_pos2 = p1;
-	vt.y_pos = vt.y_pos2 = p2;
+	vt.xPos = vt.xPos2 = p1;
+	vt.yPos = vt.yPos2 = p2;
 }
 
 cmd(position_f) {
-	vt.x_pos = vt.x_pos2 = _v[p1];
-	vt.y_pos = vt.y_pos2 = _v[p2];
+	vt.xPos = vt.xPos2 = _v[p1];
+	vt.yPos = vt.yPos2 = _v[p2];
 }
 
 cmd(get_posn) {
-	game.vars[p1] = (unsigned char)vt.x_pos;
-	game.vars[p2] = (unsigned char)vt.y_pos;
+	game.vars[p1] = (unsigned char)vt.xPos;
+	game.vars[p2] = (unsigned char)vt.yPos;
 }
 
 cmd(reposition) {
@@ -694,45 +746,45 @@ cmd(reposition) {
 	debugC(4, kDebugLevelScripts, "dx=%d, dy=%d", dx, dy);
 	vt.flags |= UPDATE_POS;
 
-	if (dx < 0 && vt.x_pos < -dx)
-		vt.x_pos = 0;
+	if (dx < 0 && vt.xPos < -dx)
+		vt.xPos = 0;
 	else
-		vt.x_pos += dx;
+		vt.xPos += dx;
 
-	if (dy < 0 && vt.y_pos < -dy)
-		vt.y_pos = 0;
+	if (dy < 0 && vt.yPos < -dy)
+		vt.yPos = 0;
 	else
-		vt.y_pos += dy;
+		vt.yPos += dy;
 
-	fix_position(p0);
+	g_agi->fixPosition(p0);
 }
 
 cmd(reposition_to) {
-	vt.x_pos = p1;
-	vt.y_pos = p2;
+	vt.xPos = p1;
+	vt.yPos = p2;
 	vt.flags |= UPDATE_POS;
-	fix_position(p0);
+	g_agi->fixPosition(p0);
 }
 
 cmd(reposition_to_f) {
-	vt.x_pos = _v[p1];
-	vt.y_pos = _v[p2];
+	vt.xPos = _v[p1];
+	vt.yPos = _v[p2];
 	vt.flags |= UPDATE_POS;
-	fix_position(p0);
+	g_agi->fixPosition(p0);
 }
 
 cmd(add_to_pic) {
-	_sprites->add_to_pic(p0, p1, p2, p3, p4, p5, p6);
+	g_sprites->addToPic(p0, p1, p2, p3, p4, p5, p6);
 }
 
 cmd(add_to_pic_f) {
-	_sprites->add_to_pic(_v[p0], _v[p1], _v[p2], _v[p3], _v[p4], _v[p5], _v[p6]);
+	g_sprites->addToPic(_v[p0], _v[p1], _v[p2], _v[p3], _v[p4], _v[p5], _v[p6]);
 }
 
 cmd(force_update) {
-	_sprites->erase_both();
-	_sprites->blit_both();
-	_sprites->commit_both();
+	g_sprites->eraseBoth();
+	g_sprites->blitBoth();
+	g_sprites->commitBoth();
 }
 
 cmd(reverse_loop) {
@@ -740,7 +792,7 @@ cmd(reverse_loop) {
 	vt.cycle = CYCLE_REV_LOOP;
 	vt.flags |= (DONTUPDATE | UPDATE | CYCLING);
 	vt.parm1 = p1;
-	setflag(p1, false);
+	g_agi->setflag(p1, false);
 }
 
 cmd(end_of_loop) {
@@ -748,7 +800,7 @@ cmd(end_of_loop) {
 	vt.cycle = CYCLE_END_OF_LOOP;
 	vt.flags |= (DONTUPDATE | UPDATE | CYCLING);
 	vt.parm1 = p1;
-	setflag(p1, false);
+	g_agi->setflag(p1, false);
 }
 
 cmd(block) {
@@ -772,34 +824,34 @@ cmd(stop_motion) {
 	vt.direction = 0;
 	vt.motion = MOTION_NORMAL;
 	if (p0 == 0) {		/* ego only */
-		_v[V_ego_dir] = 0;
-		game.player_control = false;
+		_v[vEgoDir] = 0;
+		game.playerControl = false;
 	}
 }
 
 cmd(start_motion) {
 	vt.motion = MOTION_NORMAL;
 	if (p0 == 0) {		/* ego only */
-		_v[V_ego_dir] = 0;
-		game.player_control = true;
+		_v[vEgoDir] = 0;
+		game.playerControl = true;
 	}
 }
 
 cmd(player_control) {
-	game.player_control = true;
-	game.view_table[0].motion = MOTION_NORMAL;
+	game.playerControl = true;
+	game.viewTable[0].motion = MOTION_NORMAL;
 }
 
 cmd(program_control) {
-	game.player_control = false;
+	game.playerControl = false;
 }
 
 cmd(follow_ego) {
 	vt.motion = MOTION_FOLLOW_EGO;
-	vt.parm1 = p1 > vt.step_size ? p1 : vt.step_size;
+	vt.parm1 = p1 > vt.stepSize ? p1 : vt.stepSize;
 	vt.parm2 = p2;
 	vt.parm3 = 0xff;
-	setflag(p2, false);
+	g_agi->setflag(p2, false);
 	vt.flags |= UPDATE;
 }
 
@@ -809,54 +861,54 @@ cmd(move_obj) {
 	vt.motion = MOTION_MOVE_OBJ;
 	vt.parm1 = p1;
 	vt.parm2 = p2;
-	vt.parm3 = vt.step_size;
+	vt.parm3 = vt.stepSize;
 	vt.parm4 = p4;
 
 	if (p3 != 0)
-		vt.step_size = p3;
+		vt.stepSize = p3;
 
-	setflag(p4, false);
+	g_agi->setflag(p4, false);
 	vt.flags |= UPDATE;
 
 	if (p0 == 0)
-		game.player_control = false;
+		game.playerControl = false;
 
 	/* AGI 2.272 (ddp, xmas) doesn't call move_obj! */
-	if (agi_get_release() > 0x2272)
-		move_obj(&vt);
+	if (g_agi->agiGetRelease() > 0x2272)
+		g_agi->moveObj(&vt);
 }
 
 cmd(move_obj_f) {
 	vt.motion = MOTION_MOVE_OBJ;
 	vt.parm1 = _v[p1];
 	vt.parm2 = _v[p2];
-	vt.parm3 = vt.step_size;
+	vt.parm3 = vt.stepSize;
 	vt.parm4 = p4;
 
 	if (_v[p3] != 0)
-		vt.step_size = _v[p3];
+		vt.stepSize = _v[p3];
 
-	setflag(p4, false);
+	g_agi->setflag(p4, false);
 	vt.flags |= UPDATE;
 
 	if (p0 == 0)
-		game.player_control = false;
+		game.playerControl = false;
 
 	/* AGI 2.272 (ddp, xmas) doesn't call move_obj! */
-	if (agi_get_release() > 0x2272)
-		move_obj(&vt);
+	if (g_agi->agiGetRelease() > 0x2272)
+		g_agi->moveObj(&vt);
 }
 
 cmd(wander) {
 	if (p0 == 0)
-		game.player_control = false;
+		game.playerControl = false;
 	vt.motion = MOTION_WANDER;
 	vt.flags |= UPDATE;
 }
 
 cmd(set_game_id) {
-	if (cur_logic->texts && (p0 - 1) <= cur_logic->num_texts)
-		strncpy(game.id, cur_logic->texts[p0 - 1], 8);
+	if (curLogic->texts && (p0 - 1) <= curLogic->numTexts)
+		strncpy(game.id, curLogic->texts[p0 - 1], 8);
 	else
 		game.id[0] = 0;
 
@@ -864,33 +916,33 @@ cmd(set_game_id) {
 }
 
 cmd(pause) {
-	int tmp = game.clock_enabled;
+	int tmp = game.clockEnabled;
 	const char *b[] = { "Continue", NULL };
 
-	game.clock_enabled = false;
-	_text->selection_box("  Game is paused.  \n\n\n", b);
-	game.clock_enabled = tmp;
+	game.clockEnabled = false;
+	g_agi->selectionBox("  Game is paused.  \n\n\n", b);
+	game.clockEnabled = tmp;
 }
 
 cmd(set_menu) {
-	debugC(4, kDebugLevelScripts, "text %02x of %02x", p0, cur_logic->num_texts);
-	if (cur_logic->texts != NULL && p0 < cur_logic->num_texts)
-		menu->add(cur_logic->texts[p0 - 1]);
+	debugC(4, kDebugLevelScripts, "text %02x of %02x", p0, curLogic->numTexts);
+	if (curLogic->texts != NULL && p0 < curLogic->numTexts)
+		g_agi->_menu->add(curLogic->texts[p0 - 1]);
 }
 
 cmd(set_menu_item) {
-	debugC(4, kDebugLevelScripts, "text %02x of %02x", p0, cur_logic->num_texts);
-	if (cur_logic->texts != NULL && p0 <= cur_logic->num_texts)
-		menu->add_item(cur_logic->texts[p0 - 1], p1);
+	debugC(4, kDebugLevelScripts, "text %02x of %02x", p0, curLogic->numTexts);
+	if (curLogic->texts != NULL && p0 <= curLogic->numTexts)
+		g_agi->_menu->addItem(curLogic->texts[p0 - 1], p1);
 }
 
 cmd(version) {
-	char ver_msg[] = TITLE " v" VERSION;
-	char ver2_msg[] =
+	char verMsg[64];
+	char ver2Msg[] =
 	    "\n"
-	    "                             \n\n"
-	    "Emulating Sierra AGI v%x.%03x\n";
-	char ver3_msg[] =
+	    "                               \n\n"
+	    "  Emulating Sierra AGI v%x.%03x\n";
+	char ver3Msg[] =
 	    "\n"
 	    "                             \n\n"
 	    "  Emulating AGI v%x.002.%03x\n";
@@ -898,74 +950,86 @@ cmd(version) {
 	char *r, *q;
 	int ver, maj, min;
 	char msg[256];
+	int gap;
+	int len;
 
-	ver = agi_get_release();
+	sprintf(verMsg, TITLE " v%s", gScummVMVersion);
+
+	ver = g_agi->agiGetRelease();
 	maj = (ver >> 12) & 0xf;
 	min = ver & 0xfff;
 
-	q = maj == 2 ? ver2_msg : ver3_msg;
+	q = maj == 2 ? ver2Msg : ver3Msg;
 	r = strchr(q + 1, '\n');
 
-	strncpy(q + 1 + ((r - q > 0 ? r - q : 1) / 4), ver_msg, strlen(ver_msg));
+	/* insert our version into the other version */
+	len = strlen(verMsg);
+	gap = r - q;
+	if(gap < 0)
+		gap = 0;
+	else
+		gap = (gap - len) / 2;
+
+	strncpy(q + 1 + gap, verMsg, strlen(verMsg));
 	sprintf(msg, q, maj, min);
-	_text->message_box(msg);
+	g_agi->messageBox(msg);
 }
 
 cmd(configure_screen) {
-	game.line_min_print = p0;
-	game.line_user_input = p1;
-	game.line_status = p2;
+	game.lineMinPrint = p0;
+	game.lineUserInput = p1;
+	game.lineStatus = p2;
 }
 
 cmd(text_screen) {
 	debugC(4, kDebugLevelScripts, "switching to text mode");
-	game.gfx_mode = false;
+	game.gfxMode = false;
 	/*
 	 * Simulates the "bright background bit" of the PC video
 	 * controller.
 	 */
-	if (game.color_bg)
-		game.color_bg |= 0x08;
-	clear_screen(game.color_bg);
+	if (game.colorBg)
+		game.colorBg |= 0x08;
+	g_gfx->clearScreen(game.colorBg);
 }
 
 cmd(graphics) {
 	debugC(4, kDebugLevelScripts, "switching to graphics mode");
-	if (!game.gfx_mode) {
-		game.gfx_mode = true;
-		clear_screen(0);
-		show_pic();
-		_text->write_status();
-		_text->write_prompt();
+	if (!game.gfxMode) {
+		game.gfxMode = true;
+		g_gfx->clearScreen(0);
+		g_picture->showPic();
+		g_agi->writeStatus();
+		g_agi->writePrompt();
 	}
 }
 
 cmd(set_text_attribute) {
-	game.color_fg = p0;
-	game.color_bg = p1;
+	game.colorFg = p0;
+	game.colorBg = p1;
 
-	if (game.gfx_mode) {
-		if (game.color_bg != 0) {
-			game.color_fg = 0;
-			game.color_bg = 15;
+	if (game.gfxMode) {
+		if (game.colorBg != 0) {
+			game.colorFg = 0;
+			game.colorBg = 15;
 		}
 	}
 }
 
 cmd(status) {
-	inventory();
+	g_agi->inventory();
 }
 
 cmd(quit) {
 	const char *buttons[] = { "Quit", "Continue", NULL };
 
-	stop_sound();
+	g_sound->stopSound();
 	if (p0) {
-		game.quit_prog_now = true;
+		game.quitProgNow = true;
 	} else {
-		if (_text->selection_box
-		    (" Quit the game, or continue? \n\n\n", buttons) == 0) {
-			game.quit_prog_now = true;
+		if (g_agi->selectionBox
+				(" Quit the game, or continue? \n\n\n", buttons) == 0) {
+			game.quitProgNow = true;
 		}
 	}
 }
@@ -974,28 +1038,28 @@ cmd(restart_game) {
 	const char *buttons[] = { "Restart", "Continue", NULL };
 	int sel;
 
-	stop_sound();
-	sel = getflag(F_auto_restart) ? 1 :
-	    _text->selection_box(" Restart game, or continue? \n\n\n", buttons);
+	g_sound->stopSound();
+	sel = g_agi->getflag(fAutoRestart) ? 1 :
+		g_agi->selectionBox(" Restart game, or continue? \n\n\n", buttons);
 
 	if (sel == 0) {
-		game.quit_prog_now = 0xff;
-		setflag(F_restart_game, true);
-		menu->enable_all();
+		game.quitProgNow = 0xff;
+		g_agi->setflag(fRestartGame, true);
+		g_agi->_menu->enableAll();
 	}
 }
 
 cmd(distance) {
 	int16 x1, y1, x2, y2, d;
-	struct vt_entry *v0 = &game.view_table[p0];
-	struct vt_entry *v1 = &game.view_table[p1];
+	VtEntry *v0 = &game.viewTable[p0];
+	VtEntry *v1 = &game.viewTable[p1];
 
 	if (v0->flags & DRAWN && v1->flags & DRAWN) {
-		x1 = v0->x_pos + v0->x_size / 2;
-		y1 = v0->y_pos;
-		x2 = v1->x_pos + v1->x_size / 2;
-		y2 = v1->y_pos;
-		d = abs(x1 - x2) + abs(y1 - y2);
+		x1 = v0->xPos + v0->xSize / 2;
+		y1 = v0->yPos;
+		x2 = v1->xPos + v1->xSize / 2;
+		y2 = v1->yPos;
+		d = ABS(x1 - x2) + ABS(y1 - y2);
 		if (d > 0xfe)
 			d = 0xfe;
 	} else {
@@ -1006,14 +1070,14 @@ cmd(distance) {
 
 cmd(accept_input) {
 	debugC(4, kDebugLevelScripts | kDebugLevelInput, "input normal");
-	new_input_mode(INPUT_NORMAL);
-	game.input_enabled = true;
+	g_agi->newInputMode(INPUT_NORMAL);
+	game.inputEnabled = true;
 }
 
 cmd(prevent_input) {
 	debugC(4, kDebugLevelScripts | kDebugLevelInput, "no input");
-	new_input_mode(INPUT_NONE);
-	game.input_enabled = false;
+	g_agi->newInputMode(INPUT_NONE);
+	game.inputEnabled = false;
 }
 
 cmd(get_string) {
@@ -1026,58 +1090,58 @@ cmd(get_string) {
 	col = p3;
 
 	/* Workaround for SQLC bug.
-	 * See bug #792125 for details
+	 * See Sarien bug #792125 for details
 	 */
 	if (row > 24)
 		row = 24;
 	if (col > 39)
 		col = 39;
 
-	new_input_mode(INPUT_GETSTRING);
+	g_agi->newInputMode(INPUT_GETSTRING);
 
-	if (cur_logic->texts != NULL && cur_logic->num_texts >= tex) {
-		int len = strlen(cur_logic->texts[tex]);
-		_text->print_text(cur_logic->texts[tex], 0, col, row, len, game.color_fg, game.color_bg);
-		get_string(col + len - 1, row, p4, p0);
+	if (curLogic->texts != NULL && curLogic->numTexts >= tex) {
+		int len = strlen(curLogic->texts[tex]);
+		g_agi->printText(curLogic->texts[tex], 0, col, row, len, game.colorFg, game.colorBg);
+		g_agi->getString(col + len - 1, row, p4, p0);
 
 		/* SGEO: display input char */
-		print_character((col + len), row, game.cursor_char, game.color_fg, game.color_bg);
+		g_gfx->printCharacter((col + len), row, game.cursorChar, game.colorFg, game.colorBg);
 	}
 
 	do {
-		main_cycle();
-	} while (game.input_mode == INPUT_GETSTRING);
+		g_agi->mainCycle();
+	} while (game.inputMode == INPUT_GETSTRING);
 }
 
 cmd(get_num) {
 	debugC(4, kDebugLevelScripts, "%d %d", p0, p1);
-	new_input_mode(INPUT_GETSTRING);
+	g_agi->newInputMode(INPUT_GETSTRING);
 
-	if (cur_logic->texts != NULL && cur_logic->num_texts >= (p0 - 1)) {
-		int len = strlen(cur_logic->texts[p0 - 1]);
-		_text->print_text(cur_logic->texts[p0 - 1], 0, 0, 22, len, game.color_fg, game.color_bg);
-		get_string(len - 1, 22, 3, MAX_STRINGS);
+	if (curLogic->texts != NULL && curLogic->numTexts >= (p0 - 1)) {
+		int len = strlen(curLogic->texts[p0 - 1]);
+		g_agi->printText(curLogic->texts[p0 - 1], 0, 0, 22, len, game.colorFg, game.colorBg);
+		g_agi->getString(len - 1, 22, 3, MAX_STRINGS);
 
 		/* CM: display input char */
-		print_character((p3 + len), 22, game.cursor_char, game.color_fg, game.color_bg);
+		g_gfx->printCharacter((p3 + len), 22, game.cursorChar, game.colorFg, game.colorBg);
 	}
 
 	do {
-		main_cycle();
-	} while (game.input_mode == INPUT_GETSTRING);
+		g_agi->mainCycle();
+	} while (game.inputMode == INPUT_GETSTRING);
 
 	_v[p1] = atoi(game.strings[MAX_STRINGS]);
 	debugC(4, kDebugLevelScripts, "[%s] -> %d", game.strings[MAX_STRINGS], _v[p1]);
-	_text->clear_lines(22, 22, game.color_bg);
-	_text->flush_lines(22, 22);
+	g_agi->clearLines(22, 22, game.colorBg);
+	g_agi->flushLines(22, 22);
 }
 
 cmd(set_cursor_char) {
-	if (cur_logic->texts != NULL && (p0 - 1) <= cur_logic->num_texts) {
-		game.cursor_char = *cur_logic->texts[p0 - 1];
+	if (curLogic->texts != NULL && (p0 - 1) <= curLogic->numTexts) {
+		game.cursorChar = *curLogic->texts[p0 - 1];
 	} else {
 		/* default */
-		game.cursor_char = '_';
+		game.cursorChar = '_';
 	}
 }
 
@@ -1086,28 +1150,28 @@ cmd(set_key) {
 
 	debugC(4, kDebugLevelScripts, "%d %d %d", p0, p1, p2);
 
-	if (game.ev_keyp[p2].data != 0)	/* TBC sets c23 (ESC) twice! */
+	if (game.evKeyp[p2].data != 0)	/* TBC sets c23 (ESC) twice! */
 		return;
 
 	key = 256 * p1 + p0;
-	game.ev_keyp[p2].data = key;
-	game.ev_keyp[p2].occured = false;
+	game.evKeyp[p2].data = key;
+	game.evKeyp[p2].occured = false;
 }
 
 cmd(set_string) {
 	/* CM: to avoid crash in Groza (str = 150) */
 	if (p0 > MAX_STRINGS)
 		return;
-	strcpy(game.strings[p0], cur_logic->texts[p1 - 1]);
+	strcpy(game.strings[p0], curLogic->texts[p1 - 1]);
 }
 
 cmd(display) {
-	_text->print_text(cur_logic->texts[p2 - 1], p1, 0, p0, 40, game.color_fg, game.color_bg);
+	g_agi->printText(curLogic->texts[p2 - 1], p1, 0, p0, 40, game.colorFg, game.colorBg);
 }
 
 cmd(display_f) {
 	debugC(4, kDebugLevelScripts, "p0 = %d", p0);
-	_text->print_text(cur_logic->texts[_v[p2] - 1], _v[p1], 0, _v[p0], 40, game.color_fg, game.color_bg);
+	g_agi->printText(curLogic->texts[_v[p2] - 1], _v[p1], 0, _v[p0], 40, game.colorFg, game.colorBg);
 }
 
 cmd(clear_text_rect) {
@@ -1130,8 +1194,8 @@ cmd(clear_text_rect) {
 	if (y2 > GFX_HEIGHT)
 		y2 = GFX_HEIGHT - 1;
 
-	draw_rectangle(x1, y1, x2, y2, c);
-	flush_block(x1, y1, x2, y2);
+	g_gfx->drawRectangle(x1, y1, x2, y2, c);
+	g_gfx->flushBlock(x1, y1, x2, y2);
 }
 
 cmd(toggle_monitor) {
@@ -1139,47 +1203,47 @@ cmd(toggle_monitor) {
 }
 
 cmd(echo_line) {
-	strcpy((char *)game.input_buffer, (const char *)game.echo_buffer);
-	game.cursor_pos = strlen((char *)game.input_buffer);
-	game.has_prompt = 0;
+	strcpy((char *)game.inputBuffer, (const char *)game.echoBuffer);
+	game.cursorPos = strlen((char *)game.inputBuffer);
+	game.hasPrompt = 0;
 }
 
 cmd(clear_lines) {
 	uint8 l;
 
-	/* Residence 44 calls clear.lines(24,0,0), see bug #558423 */
+	/* Residence 44 calls clear.lines(24,0,0), see Sarien bug #558423 */
 	l = p1 ? p1 : p0;
 
-	_text->clear_lines(p0, l, p2);
-	_text->flush_lines(p0, l);
+	g_agi->clearLines(p0, l, p2);
+	g_agi->flushLines(p0, l);
 }
 
 cmd(print) {
 	int n = p0 < 1 ? 1 : p0;
-	_text->print(cur_logic->texts[n - 1], 0, 0, 0);
+	g_agi->print(curLogic->texts[n - 1], 0, 0, 0);
 }
 
 cmd(print_f) {
 	int n = _v[p0] < 1 ? 1 : _v[p0];
-	_text->print(cur_logic->texts[n - 1], 0, 0, 0);
+	g_agi->print(curLogic->texts[n - 1], 0, 0, 0);
 }
 
 cmd(print_at) {
 	int n = p0 < 1 ? 1 : p0;
 	debugC(4, kDebugLevelScripts, "%d %d %d %d", p0, p1, p2, p3);
-	_text->print(cur_logic->texts[n - 1], p1, p2, p3);
+	g_agi->print(curLogic->texts[n - 1], p1, p2, p3);
 }
 
 cmd(print_at_v) {
 	int n = _v[p0] < 1 ? 1 : _v[p0];
-	_text->print(cur_logic->texts[n - 1], p1, p2, p3);
+	g_agi->print(curLogic->texts[n - 1], p1, p2, p3);
 }
 
 cmd(push_script) {
-	if (opt.agimouse) {
-		game.vars[27] = mouse.button;
-		game.vars[28] = mouse.x / 2;
-		game.vars[29] = mouse.y;
+	if (g_agi->getFeatures() & GF_AGIMOUSE) {
+		game.vars[27] = g_mouse.button;
+		game.vars[28] = g_mouse.x / 2;
+		game.vars[29] = g_mouse.y;
 	} else
 		report("push.script\n");
 }
@@ -1196,37 +1260,49 @@ cmd(set_pri_base) {
 		pri = (i - p0) < 0 ? 4 : (i - p0) * _HEIGHT / x + 5;
 		if (pri > 15)
 			pri = 15;
-		game.pri_table[i] = pri;
+		game.priTable[i] = pri;
 	}
 }
 
 cmd(mouse_posn) {
-	_v[p0] = WIN_TO_PIC_X(mouse.x);
-	_v[p1] = WIN_TO_PIC_Y(mouse.y);
+	_v[p0] = WIN_TO_PIC_X(g_mouse.x);
+	_v[p1] = WIN_TO_PIC_Y(g_mouse.y);
 }
 
 cmd(shake_screen) {
 	int i;
 
-	/* AGI Mouse 1.1 uses shake.screen values between 100 and 109 to
-	 * set the palette.
-	 */
-	if (opt.agimouse && p0 >= 100 && p0 < 110) {
-		report("not implemented: AGI Mouse palettes\n");
-		return;
-	} else
-		shake_start();
+	// AGIPAL uses shake.screen values between 101 and 109 to
+	// set the palette.
+	if (p0 >= 101 && p0 < 110) {
+		if (g_agi->getFeatures() & GF_AGIPAL) {
+			g_gfx->setAGIPal(p0);
+			return;
+		} else {
+			warning("It looks like GF_AGIPAL flag is missing");
+		}
+	} 
 
-	_sprites->commit_both();		/* Fixes SQ1 demo */
+	// Disables input while shaking to prevent bug
+	// #1678230: AGI: Entering text while screen is shaking
+	int originalValue = game.inputEnabled;
+	game.inputEnabled = 0;
+
+	g_gfx->shakeStart();
+
+	g_sprites->commitBoth();		/* Fixes SQ1 demo */
 	for (i = 4 * p0; i; i--) {
-		shake_screen(i & 1);
-		flush_block(0, 0, GFX_WIDTH - 1, GFX_HEIGHT - 1);
-		main_cycle();
+		g_gfx->shakeScreen(i & 1);
+		g_gfx->flushBlock(0, 0, GFX_WIDTH - 1, GFX_HEIGHT - 1);
+		g_agi->mainCycle();
 	}
-	shake_end();
+	g_gfx->shakeEnd();
+
+	//Sets input back to what it was
+	game.inputEnabled = originalValue;
 }
 
-static void (*agi_command[183]) (uint8 *) = {
+static void (*agiCommand[183])(uint8 *) = {
 	NULL,			/* 0x00 */
 	cmd_increment,
 	cmd_decrement,
@@ -1418,44 +1494,45 @@ static void (*agi_command[183]) (uint8 *) = {
  * Execute a logic script
  * @param n  Number of the logic resource to execute
  */
-int run_logic(int n) {
+int AgiEngine::runLogic(int n) {
 	uint8 op = 0;
 	uint8 p[CMD_BSIZE] = { 0 };
 	uint8 *code = NULL;
+	g_agi = this;
 	int num = 0;
 
 	/* If logic not loaded, load it */
-	if (~game.dir_logic[n].flags & RES_LOADED) {
+	if (~_game.dirLogic[n].flags & RES_LOADED) {
 		debugC(4, kDebugLevelScripts, "logic %d not loaded!", n);
-		agi_load_resource(rLOGIC, n);
+		agiLoadResource(rLOGIC, n);
 	}
 
-	game.lognum = n;
-	cur_logic = &game.logics[game.lognum];
+	_game.lognum = n;
+	curLogic = &_game.logics[game.lognum];
 
-	code = cur_logic->data;
-	cur_logic->cIP = cur_logic->sIP;
+	code = curLogic->data;
+	curLogic->cIP = curLogic->sIP;
 
-	timer_hack = 0;
-	while (ip < game.logics[n].size && !game.quit_prog_now) {
-		if (debug_.enabled) {
-			if (debug_.steps > 0) {
-				if (debug_.logic0 || n) {
-					debug_console(n, lCOMMAND_MODE, NULL);
-					debug_.steps--;
+	timerHack = 0;
+	while (ip < _game.logics[n].size && !_game.quitProgNow) {
+		if (_debug.enabled) {
+			if (_debug.steps > 0) {
+				if (_debug.logic0 || n) {
+					debugConsole(n, lCOMMAND_MODE, NULL);
+					_debug.steps--;
 				}
 			} else {
-				_sprites->blit_both();
+				_sprites->blitBoth();
 				do {
-					main_cycle();
-				} while (!debug_.steps && debug_.enabled);
-				_sprites->erase_both();
+					mainCycle();
+				} while (!_debug.steps && _debug.enabled);
+				_sprites->eraseBoth();
 			}
 		}
 
 		switch (op = *(code + ip++)) {
 		case 0xff:	/* if (open/close) */
-			test_if_code(n);
+			testIfCode(n);
 			break;
 		case 0xfe:	/* goto */
 			/* +2 covers goto size */
@@ -1463,34 +1540,36 @@ int run_logic(int n) {
 			/* timer must keep running even in goto loops,
 			 * but AGI engine can't do that :(
 			 */
-			if (timer_hack > 20) {
-				poll_timer();
-				update_timer();
-				timer_hack = 0;
+			if (timerHack > 20) {
+				g_gfx->pollTimer();
+				updateTimer();
+				timerHack = 0;
 			}
 			break;
 		case 0x00:	/* return */
 			return 1;
 		default:
-			num = logic_names_cmd[op].num_args;
+			num = logicNamesCmd[op].numArgs;
 			memmove(p, code + ip, num);
 			memset(p + num, 0, CMD_BSIZE - num);
 
-			debugC(2, kDebugLevelScripts, "%s(%d %d %d)", logic_names_cmd[op].name, p[0], p[1], p[2]);
-			agi_command[op] (p);
+			debugC(2, kDebugLevelScripts, "%s(%d %d %d)", logicNamesCmd[op].name, p[0], p[1], p[2]);
+			agiCommand[op] (p);
 			ip += num;
 		}
 
-		if (game.exit_all_logics)
+		if (_game.exitAllLogics)
 			break;
 	}
 
 	return 0;		/* after executing new.room() */
 }
 
-void execute_agi_command(uint8 op, uint8 *p) {
-	debugC(2, kDebugLevelScripts, "%s(%d %d %d)", logic_names_cmd[op].name, p[0], p[1], p[2]);
-	agi_command[op] (p);
+void AgiEngine::executeAgiCommand(uint8 op, uint8 *p) {
+	debugC(2, kDebugLevelScripts, "%s(%d %d %d)", logicNamesCmd[op].name, p[0], p[1], p[2]);
+	g_agi = this;
+
+	agiCommand[op] (p);
 }
 
 } // End of namespace Agi
