@@ -1,7 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
  *
- * Copyright (C) 1999-2001 Sarien Team
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -17,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/agi/op_dbg.cpp $
- * $Id: op_dbg.cpp 22618 2006-05-24 21:40:24Z aquadran $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/agi/op_dbg.cpp $
+ * $Id: op_dbg.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -29,6 +30,9 @@
 
 namespace Agi {
 
+static AgiEngine *g_agi;
+#define game g_agi->_game
+
 #define ip	(game.logics[lognum].cIP)
 #define code	(game.logics[lognum].data)
 
@@ -38,7 +42,7 @@ namespace Agi {
 
 #define _L(a,b,c) { a, b, c }
 
-struct agi_logicnames logic_names_test[] = {
+struct AgiLogicnames logicNamesTest[] = {
 	_L("", 0, 0x00),
 	_L("equaln", 2, 0x80),
 	_L("equalv", 2, 0xC0),
@@ -63,14 +67,14 @@ struct agi_logicnames logic_names_test[] = {
 	_L("right.posn", 5, 0x00)
 };
 
-struct agi_logicnames logic_names_if[] = {
+struct AgiLogicnames logicNamesIf[] = {
 	_L("OR", 0, 0x00),
 	_L("NOT", 0, 0x00),
 	_L("ELSE", 0, 0x00),
 	_L("IF", 0, 0x00)
 };
 
-struct agi_logicnames logic_names_cmd[] = {
+struct AgiLogicnames logicNamesCmd[] = {
 	_L("return", 0, 0x00),	/* 00 */
 	_L("increment", 1, 0x80),	/* 01 */
 	_L("decrement", 1, 0x80),	/* 02 */
@@ -266,8 +270,9 @@ struct agi_logicnames logic_names_cmd[] = {
 	_L(NULL, 0, 0x00)
 };
 
-void debug_console(int lognum, int mode, const char *str) {
-	struct agi_logicnames *x;
+void AgiEngine::debugConsole(int lognum, int mode, const char *str) {
+	g_agi = this;
+	AgiLogicnames *x;
 	uint8 a, c, z;
 
 	if (str) {
@@ -282,9 +287,9 @@ void debug_console(int lognum, int mode, const char *str) {
 	case 0xFD:
 	case 0xFE:
 	case 0xFF:
-		x = logic_names_if;
+		x = logicNamesIf;
 
-		if (debug_.opcodes) {
+		if (_debug.opcodes) {
 			report("%02X %02X %02X %02X %02X %02X %02X %02X %02X\n"
 			    "         ",
 			    (uint8) * (code + (0 + ip)) & 0xFF,
@@ -300,22 +305,22 @@ void debug_console(int lognum, int mode, const char *str) {
 		report("%s ", (x + *(code + ip) - 0xFC)->name);
 		break;
 	default:
-		x = mode == lCOMMAND_MODE ? logic_names_cmd : logic_names_test;
-		a = (unsigned char)(x + *(code + ip))->num_args;
-		c = (unsigned char)(x + *(code + ip))->arg_mask;
+		x = mode == lCOMMAND_MODE ? logicNamesCmd : logicNamesTest;
+		a = (unsigned char)(x + *(code + ip))->numArgs;
+		c = (unsigned char)(x + *(code + ip))->argMask;
 
-		if (debug_.opcodes) {
+		if (_debug.opcodes) {
 			report("%02X %02X %02X %02X %02X %02X %02X %02X %02X\n"
 			    "         ",
-			    (uint8) * (code + (0 + ip)) & 0xFF,
-			    (uint8) * (code + (1 + ip)) & 0xFF,
-			    (uint8) * (code + (2 + ip)) & 0xFF,
-			    (uint8) * (code + (3 + ip)) & 0xFF,
-			    (uint8) * (code + (4 + ip)) & 0xFF,
-			    (uint8) * (code + (5 + ip)) & 0xFF,
-			    (uint8) * (code + (6 + ip)) & 0xFF,
-			    (uint8) * (code + (7 + ip)) & 0xFF,
-			    (uint8) * (code + (8 + ip)) & 0xFF);
+			    (uint8)*(code + (0 + ip)) & 0xFF,
+			    (uint8)*(code + (1 + ip)) & 0xFF,
+			    (uint8)*(code + (2 + ip)) & 0xFF,
+			    (uint8)*(code + (3 + ip)) & 0xFF,
+			    (uint8)*(code + (4 + ip)) & 0xFF,
+			    (uint8)*(code + (5 + ip)) & 0xFF,
+			    (uint8)*(code + (6 + ip)) & 0xFF,
+			    (uint8)*(code + (7 + ip)) & 0xFF,
+			    (uint8)*(code + (8 + ip)) & 0xFF);
 		}
 		report("%s ", (x + *(code + ip))->name);
 
@@ -336,4 +341,4 @@ void debug_console(int lognum, int mode, const char *str) {
 	report("\n");
 }
 
-}                             // End of namespace Agi
+} // End of namespace Agi
