@@ -1,7 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
  *
- * Copyright (C) 1999-2001 Sarien Team
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -17,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/agi/opcodes.h $
- * $Id: opcodes.h 22618 2006-05-24 21:40:24Z aquadran $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/agi/opcodes.h $
+ * $Id: opcodes.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -29,25 +30,16 @@
 
 namespace Agi {
 
-struct agi_logicnames {
+struct AgiLogicnames {
 	const char *name;
-	uint16 num_args;
-	uint16 arg_mask;
+	uint16 numArgs;
+	uint16 argMask;
 };
 
-extern struct agi_logicnames logic_names_test[];
-extern struct agi_logicnames logic_names_cmd[];
-extern struct agi_logicnames logic_names_if[];
+extern AgiLogicnames logicNamesTest[];
+extern AgiLogicnames logicNamesCmd[];
+extern AgiLogicnames logicNamesIf[];
 
-void debug_console(int, int, const char *);
-int test_if_code(int);
-void new_room(int);
-void execute_agi_command(uint8, uint8 *);
+} // End of namespace Agi
 
-#ifdef PATCH_LOGIC
-void patch_logic(int);
-#endif
-
-}                             // End of namespace Agi
-
-#endif				/* AGI_OPCODES_H */
+#endif /* AGI_OPCODES_H */
