@@ -1,7 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
  *
- * Copyright (C) 1999-2001 Sarien Team
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -17,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/agi/picture.cpp $
- * $Id: picture.cpp 22614 2006-05-24 19:51:37Z eriktorbjorn $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/agi/picture.cpp $
+ * $Id: picture.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -26,26 +27,26 @@
 
 #include "agi/agi.h"
 #include "agi/graphics.h"
-#include "agi/savegame.h"
 
 namespace Agi {
 
-#define next_byte data[foffs++]
+#define nextByte data[foffs++]
 
 static uint8 *data;
 static uint32 flen;
 static uint32 foffs;
 
-static uint8 pat_code;
-static uint8 pat_num;
-static uint8 pri_on;
-static uint8 scr_on;
-static uint8 scr_colour;
-static uint8 pri_colour;
+static uint8 patCode;
+static uint8 patNum;
+static uint8 priOn;
+static uint8 scrOn;
+static uint8 scrColour;
+static uint8 priColour;
 
 static uint8 circles[][15] = {	/* agi circle bitmaps */
 	{0x80},
-	{0xfc},
+	/* {0xfc}, */
+	{ 3 << 4 },	/* pattern data different from specs. fixes gold rush. does not seem to break any other v3 games */
 	{0x5f, 0xf4},
 	{0x66, 0xff, 0xf6, 0x60},
 	{0x23, 0xbf, 0xff, 0xff, 0xee, 0x20},
@@ -54,14 +55,14 @@ static uint8 circles[][15] = {	/* agi circle bitmaps */
 	{0x18, 0x3c, 0x7e, 0x7e, 0x7e, 0xff, 0xff, 0xff, 0xff, 0xff, 0x7e, 0x7e, 0x7e, 0x3c, 0x18}
 };
 
-static uint8 splatter_map[32] = {	/* splatter brush bitmaps */
+static uint8 splatterMap[32] = {	/* splatter brush bitmaps */
 	0x20, 0x94, 0x02, 0x24, 0x90, 0x82, 0xa4, 0xa2,
 	0x82, 0x09, 0x0a, 0x22, 0x12, 0x10, 0x42, 0x14,
 	0x91, 0x4a, 0x91, 0x11, 0x08, 0x12, 0x25, 0x10,
 	0x22, 0xa8, 0x14, 0x24, 0x00, 0x50, 0x24, 0x04
 };
 
-static uint8 splatter_start[128] = {	/* starting bit position */
+static uint8 splatterStart[128] = {	/* starting bit position */
 	0x00, 0x18, 0x30, 0xc4, 0xdc, 0x65, 0xeb, 0x48,
 	0x60, 0xbd, 0x89, 0x05, 0x0a, 0xf4, 0x7d, 0x7d,
 	0x85, 0xb0, 0x8e, 0x95, 0x1f, 0x22, 0x0d, 0xdf,
@@ -79,43 +80,40 @@ static uint8 splatter_start[128] = {	/* starting bit position */
 	0x06, 0x6f, 0xc6, 0x4a, 0xa4, 0x75, 0x97, 0xe1
 };
 
-static void fix_pixel_bothsides(int x, int y);
-
-static void put_virt_pixel(int x, int y, int res) {
+void PictureMgr::putVirtPixel(int x, int y) {
 	uint8 *p;
-	int width = _WIDTH * res;
 
-	if (x < 0 || y < 0 || x >= width || y >= _HEIGHT)
+	if (x < 0 || y < 0 || x >= _WIDTH || y >= _HEIGHT)
 		return;
 
-	p = res > 1 ? &game.hires[y * width + x] : &game.sbuf[y * width + x];
+	p = &_vm->_game.sbuf[y * _WIDTH + x];
 
-	if (pri_on)
-		*p = (pri_colour << 4) | (*p & 0x0f);
-	if (scr_on)
-		*p = scr_colour | (*p & 0xf0);
+	if (priOn)
+		*p = (priColour << 4) | (*p & 0x0f);
+	if (scrOn)
+		*p = scrColour | (*p & 0xf0);
 }
 
 /* For the flood fill routines */
 
 /* MH2 needs stack size > 300 */
 #define STACK_SIZE 512
-static unsigned int stack_ptr;
+static unsigned int stackPtr;
 static uint16 stack[STACK_SIZE];
 
-static INLINE void _PUSH(uint16 c) {
-	assert(stack_ptr < STACK_SIZE);
+static INLINE void lpush(uint16 c) {
+	assert(stackPtr < STACK_SIZE);
 
-	stack[stack_ptr] = c;
-	stack_ptr++;
+	stack[stackPtr] = c;
+	stackPtr++;
 }
 
-static INLINE uint16 _POP() {
-	if (stack_ptr == 0)
+static INLINE uint16 lpop() {
+	if (stackPtr == 0)
 		return 0xffff;
 
-	stack_ptr--;
-	return stack[stack_ptr];
+	stackPtr--;
+	return stack[stackPtr];
 }
 
 /**
@@ -126,16 +124,14 @@ static INLINE uint16 _POP() {
  * @param y1  y coordinate of start point
  * @param x2  x coordinate of end point
  * @param y2  y coordinate of end point
- * @param res horizontal resolution multiplier
  */
-static void draw_line(int x1, int y1, int x2, int y2, int res) {
+void PictureMgr::drawLine(int x1, int y1, int x2, int y2) {
 	int i, x, y, deltaX, deltaY, stepX, stepY, errorX, errorY, detdelta;
-	int width = _WIDTH * res;
 
 	/* CM: Do clipping */
 #define clip(x, y) if((x)>=(y)) (x)=(y)
-	clip(x1, width - 1);
-	clip(x2, width - 1);
+	clip(x1, _WIDTH - 1);
+	clip(x2, _WIDTH - 1);
 	clip(y1, _HEIGHT - 1);
 	clip(y2, _HEIGHT - 1);
 
@@ -148,11 +144,8 @@ static void draw_line(int x1, int y1, int x2, int y2, int res) {
 			y2 = y;
 		}
 
-		for (; y1 <= y2; y1++) {
-			put_virt_pixel(x1, y1, res);
-			if (res > 1)
-				fix_pixel_bothsides(x1, y1);
-		}
+		for (; y1 <= y2; y1++)
+			putVirtPixel(x1, y1);
 
 		return;
 	}
@@ -165,17 +158,8 @@ static void draw_line(int x1, int y1, int x2, int y2, int res) {
 			x1 = x2;
 			x2 = x;
 		}
-		if (res > 1)
-			fix_pixel_bothsides(x1, y1);
-
 		for (; x1 <= x2; x1++)
-			put_virt_pixel(x1, y1, res);
-
-		if (res > 1) {
-			put_virt_pixel(x1, y1, res);
-			fix_pixel_bothsides(x1, y1);
-		}
-
+			putVirtPixel(x1, y1);
 		return;
 	}
 
@@ -208,9 +192,7 @@ static void draw_line(int x1, int y1, int x2, int y2, int res) {
 		errorY = deltaX / 2;
 	}
 
-	put_virt_pixel(x, y, res);
-	if (res > 1)
-		fix_pixel_bothsides(x, y);
+	putVirtPixel(x, y);
 
 	do {
 		errorY += deltaY;
@@ -225,33 +207,25 @@ static void draw_line(int x1, int y1, int x2, int y2, int res) {
 			x += stepX;
 		}
 
-		put_virt_pixel(x, y, res);
-		if (res > 1)
-			fix_pixel_bothsides(x, y);
+		putVirtPixel(x, y);
 		i--;
 	} while (i > 0);
-
-	if (res > 1) {
-		put_virt_pixel(x, y, res);
-		fix_pixel_bothsides(x, y);
-	}
 }
 
 /**
  * Draw a relative AGI line.
  * Draws short lines relative to last position. (drawing action 0xF7)
- * @param res  horizontal resolution multiplier
  */
-static void dynamic_draw_line(int res) {
+void PictureMgr::dynamicDrawLine() {
 	int x1, y1, disp, dx, dy;
 
-	x1 = next_byte * res;
-	y1 = next_byte;
+	x1 = nextByte;
+	y1 = nextByte;
 
-	put_virt_pixel(x1, y1, res);
+	putVirtPixel(x1, y1);
 
-	while (42) {
-		if ((disp = next_byte) >= 0xf0)
+	for (;;) {
+		if ((disp = nextByte) >= 0xf0)
 			break;
 
 		dx = ((disp & 0xf0) >> 4) & 0x0f;
@@ -262,9 +236,7 @@ static void dynamic_draw_line(int res) {
 		if (dy & 0x08)
 			dy = -(dy & 0x07);
 
-		dx *= res;
-
-		draw_line(x1, y1, x1 + dx, y1 + dy, res);
+		drawLine(x1, y1, x1 + dx, y1 + dy);
 		x1 += dx;
 		y1 += dy;
 	}
@@ -276,23 +248,21 @@ static void dynamic_draw_line(int res) {
 **
 ** Draws long lines to actual locations (cf. relative) (drawing action 0xF6)
 **************************************************************************/
-static void absolute_draw_line(int res) {
+void PictureMgr::absoluteDrawLine() {
 	int x1, y1, x2, y2;
 
-	x1 = next_byte * res;
-	y1 = next_byte;
-	put_virt_pixel(x1, y1, res);
+	x1 = nextByte;
+	y1 = nextByte;
+	putVirtPixel(x1, y1);
 
-	while (42) {
-		if ((x2 = next_byte) >= 0xf0)
+	for (;;) {
+		if ((x2 = nextByte) >= 0xf0)
 			break;
 
-		if ((y2 = next_byte) >= 0xf0)
+		if ((y2 = nextByte) >= 0xf0)
 			break;
 
-		x2 *= res;
-
-		draw_line(x1, y1, x2, y2, res);
+		drawLine(x1, y1, x2, y2);
 		x1 = x2;
 		y1 = y2;
 	}
@@ -302,67 +272,67 @@ static void absolute_draw_line(int res) {
 /**************************************************************************
 ** okToFill
 **************************************************************************/
-static INLINE int is_ok_fill_here(int x, int y) {
+INLINE int PictureMgr::isOkFillHere(int x, int y) {
 	uint8 p;
 
 	if (x < 0 || x >= _WIDTH || y < 0 || y >= _HEIGHT)
 		return false;
 
-	if (!scr_on && !pri_on)
+	if (!scrOn && !priOn)
 		return false;
 
-	p = game.sbuf[y * _WIDTH + x];
+	p = _vm->_game.sbuf[y * _WIDTH + x];
 
-	if (!pri_on && scr_on && scr_colour != 15)
+	if (!priOn && scrOn && scrColour != 15)
 		return (p & 0x0f) == 15;
 
-	if (pri_on && !scr_on && pri_colour != 4)
+	if (priOn && !scrOn && priColour != 4)
 		return (p >> 4) == 4;
 
-	return (scr_on && (p & 0x0f) == 15 && scr_colour != 15);
+	return (scrOn && (p & 0x0f) == 15 && scrColour != 15);
 }
 
 /**************************************************************************
 ** agi_fill
 **************************************************************************/
-static void fill_scanline(int x, int y) {
+void PictureMgr::fillScanline(int x, int y) {
 	unsigned int c;
-	int newspan_up, newspan_down;
+	int newspanUp, newspanDown;
 
-	if (!is_ok_fill_here(x, y))
+	if (!isOkFillHere(x, y))
 		return;
 
 	/* Scan for left border */
-	for (c = x - 1; is_ok_fill_here(c, y); c--);
+	for (c = x - 1; isOkFillHere(c, y); c--);
 
-	newspan_up = newspan_down = 1;
-	for (c++; is_ok_fill_here(c, y); c++) {
-		put_virt_pixel(c, y, 1);
-		if (is_ok_fill_here(c, y - 1)) {
-			if (newspan_up) {
-				_PUSH(c + 320 * (y - 1));
-				newspan_up = 0;
+	newspanUp = newspanDown = 1;
+	for (c++; isOkFillHere(c, y); c++) {
+		putVirtPixel(c, y);
+		if (isOkFillHere(c, y - 1)) {
+			if (newspanUp) {
+				lpush(c + 320 * (y - 1));
+				newspanUp = 0;
 			}
 		} else {
-			newspan_up = 1;
+			newspanUp = 1;
 		}
 
-		if (is_ok_fill_here(c, y + 1)) {
-			if (newspan_down) {
-				_PUSH(c + 320 * (y + 1));
-				newspan_down = 0;
+		if (isOkFillHere(c, y + 1)) {
+			if (newspanDown) {
+				lpush(c + 320 * (y + 1));
+				newspanDown = 0;
 			}
 		} else {
-			newspan_down = 1;
+			newspanDown = 1;
 		}
 	}
 }
 
-static void agi_fill(unsigned int x, unsigned int y) {
-	_PUSH(x + 320 * y);
+void PictureMgr::agiFill(unsigned int x, unsigned int y) {
+	lpush(x + 320 * y);
 
-	while (42) {
-		uint16 c = _POP();
+	for (;;) {
+		uint16 c = lpop();
 
 		/* Exit if stack is empty */
 		if (c == 0xffff)
@@ -371,10 +341,10 @@ static void agi_fill(unsigned int x, unsigned int y) {
 		x = c % 320;
 		y = c / 320;
 
-		fill_scanline(x, y);
+		fillScanline(x, y);
 	}
 
-	stack_ptr = 0;
+	stackPtr = 0;
 }
 
 /**************************************************************************
@@ -382,29 +352,27 @@ static void agi_fill(unsigned int x, unsigned int y) {
 **
 ** Draws an xCorner  (drawing action 0xF5)
 **************************************************************************/
-static void x_corner(int res) {
+void PictureMgr::xCorner() {
 	int x1, x2, y1, y2;
 
-	x1 = next_byte * res;
-	y1 = next_byte;
-	put_virt_pixel(x1, y1, res);
+	x1 = nextByte;
+	y1 = nextByte;
+	putVirtPixel(x1, y1);
 
-	while (42) {
-		x2 = next_byte;
+	for (;;) {
+		x2 = nextByte;
 
 		if (x2 >= 0xf0)
 			break;
 
-		x2 *= res;
-
-		draw_line(x1, y1, x2, y1, res);
+		drawLine(x1, y1, x2, y1);
 		x1 = x2;
-		y2 = next_byte;
+		y2 = nextByte;
 
 		if (y2 >= 0xf0)
 			break;
 
-		draw_line(x1, y1, x1, y2, res);
+		drawLine(x1, y1, x1, y2);
 		y1 = y2;
 	}
 	foffs--;
@@ -415,29 +383,27 @@ static void x_corner(int res) {
 **
 ** Draws an yCorner  (drawing action 0xF4)
 **************************************************************************/
-static void y_corner(int res) {
+void PictureMgr::yCorner() {
 	int x1, x2, y1, y2;
 
-	x1 = next_byte * res;
-	y1 = next_byte;
-	put_virt_pixel(x1, y1, res);
+	x1 = nextByte;
+	y1 = nextByte;
+	putVirtPixel(x1, y1);
 
-	while (42) {
-		y2 = next_byte;
+	for (;;) {
+		y2 = nextByte;
 
 		if (y2 >= 0xF0)
 			break;
 
-		draw_line(x1, y1, x1, y2, res);
+		drawLine(x1, y1, x1, y2);
 		y1 = y2;
-		x2 = next_byte;
+		x2 = nextByte;
 
 		if (x2 >= 0xf0)
 			break;
 
-		x2 *= res;
-
-		draw_line(x1, y1, x2, y1, res);
+		drawLine(x1, y1, x2, y1);
 		x1 = x2;
 	}
 
@@ -449,11 +415,11 @@ static void y_corner(int res) {
 **
 ** AGI flood fill.  (drawing action 0xF8)
 **************************************************************************/
-static void fill() {
+void PictureMgr::fill() {
 	int x1, y1;
 
-	while ((x1 = next_byte) < 0xF0 && (y1 = next_byte) < 0xf0)
-		agi_fill(x1, y1);
+	while ((x1 = nextByte) < 0xF0 && (y1 = nextByte) < 0xf0)
+		agiFill(x1, y1);
 
 	foffs--;
 }
@@ -465,43 +431,25 @@ static void fill() {
 ** on the pattern code.
 **************************************************************************/
 
-static int plot_pattern_point(int x, int y, int bitpos, int res) {
-	if (pat_code & 0x20) {
-		if ((splatter_map[bitpos >> 3] >> (7 - (bitpos & 7))) & 1) {
-			if (res > 1) {
-				/* extra randomness in hi-res brush fill
-				 */
-				if (rnd->getRandomNumber(3))
-					put_virt_pixel(x * 2, y, 2);
-				if (!rnd->getRandomNumber(3))
-					put_virt_pixel(x * 2 + 1, y, 2);
-			} else {
-				put_virt_pixel(x, y, 1);
-			}
+int PictureMgr::plotPatternPoint(int x, int y, int bitpos) {
+	if (patCode & 0x20) {
+		if ((splatterMap[bitpos >> 3] >> (7 - (bitpos & 7))) & 1) {
+			putVirtPixel(x, y);
 		}
 		bitpos++;
 		if (bitpos == 0xff)
 			bitpos = 0;
-	} else {
-		if (res > 1) {
-			/* double width pixels make MUMG and others
-			 * look nicer
-			 */
-			put_virt_pixel(x * 2, y, 2);
-			put_virt_pixel(x * 2 + 1, y, 2);
-		} else {
-			put_virt_pixel(x, y, 1);
-		}
-	}
+	} else
+		putVirtPixel(x, y);
 
 	return bitpos;
 }
 
-static void plot_pattern(int x, int y, int res) {
+void PictureMgr::plotPattern(int x, int y) {
 	int32 circlePos = 0;
-	uint32 x1, y1, pensize, bitpos = splatter_start[pat_num];
+	uint32 x1, y1, pensize, bitpos = splatterStart[patNum];
 
-	pensize = (pat_code & 7);
+	pensize = (patCode & 7);
 
 	if (x < (int)pensize)
 		x = pensize - 1;
@@ -510,11 +458,11 @@ static void plot_pattern(int x, int y, int res) {
 
 	for (y1 = y - pensize; y1 <= y + pensize; y1++) {
 		for (x1 = x - (pensize + 1) / 2; x1 <= x + pensize / 2; x1++) {
-			if (pat_code & 0x10) {	/* Square */
-				bitpos = plot_pattern_point (x1, y1, bitpos, res);
+			if (patCode & 0x10) {	/* Square */
+				bitpos = plotPatternPoint (x1, y1, bitpos);
 			} else {	/* Circle */
-				if ((circles[pat_code & 7][circlePos >> 3] >> (7 - (circlePos & 7))) & 1) {
-					bitpos = plot_pattern_point(x1, y1, bitpos, res);
+				if ((circles[patCode & 7][circlePos >> 3] >> (7 - (circlePos & 7))) & 1) {
+					bitpos = plotPatternPoint(x1, y1, bitpos);
 				}
 				circlePos++;
 			}
@@ -527,336 +475,86 @@ static void plot_pattern(int x, int y, int res) {
 **
 ** Plots points and various brush patterns.
 **************************************************************************/
-static void plot_brush(int res) {
+void PictureMgr::plotBrush() {
 	int x1, y1;
 
-	while (42) {
-		if (pat_code & 0x20) {
-			if ((pat_num = next_byte) >= 0xF0)
+	for (;;) {
+		if (patCode & 0x20) {
+			if ((patNum = nextByte) >= 0xF0)
 				break;
-			pat_num = (pat_num >> 1) & 0x7f;
+			patNum = (patNum >> 1) & 0x7f;
 		}
 
-		if ((x1 = next_byte) >= 0xf0)
+		if ((x1 = nextByte) >= 0xf0)
 			break;
 
-		if ((y1 = next_byte) >= 0xf0)
+		if ((y1 = nextByte) >= 0xf0)
 			break;
 
-		plot_pattern(x1, y1, res);
+		plotPattern(x1, y1);
 	}
 
 	foffs--;
 }
 
-static void fix_pixel_bothsides(int x, int y) {
-	uint8 *p, *s;
-
-	if (x >= (_WIDTH * 2) - 2)
-		return;
-
-	/* Sometimes a solid color area in the lo-res pic is made
-	 * with lines, and we want to keep this  effect in the
-	 * hi-res pic.
-	 */
-	p = &game.hires[y * (_WIDTH * 2) + x];
-	if ((*(p - 2) & 0x0f) == scr_colour)
-		put_virt_pixel(x - 1, y, 2);
-	if ((*(p + 2) & 0x0f) == scr_colour)
-		put_virt_pixel(x + 1, y, 2);
-
-	/* If two lines are contiguous in the lo-res pic, make them
-	 * contiguous in the hi-res pic. This condition is needed
-	 * in some scenes like in front of Lefty's in LSL1, to draw
-	 * the pole. Note: it adds artifacts in some cases.
-	 */
-	s = &game.sbuf[y * _WIDTH + x / 2];
-	if ((*(p - 1) & 0x0f) != (*(s - 1) & 0x0f))
-		put_virt_pixel(x - 1, y, 2);
-}
-
-/**************************************************************************
-** okToFill
-**************************************************************************/
-static INLINE int hires_fill_here(int x, int y) {
-	uint8 *p, *s;
-
-	if (x < 0 || x >= _WIDTH || y < 0 || y >= _HEIGHT)
-		return false;
-
-	if (!scr_on && !pri_on)
-		return false;
-
-	p = &game.hires[(int32) y * (_WIDTH * 2) + x * 2];
-	s = &game.sbuf[y * _WIDTH + x];
-
-	if (scr_on) {
-		if (scr_colour == 0x0f)
-			return false;
-		if ((*p & 0x0f) != 0x0f || (*(p + 1) & 0x0f) != 0x0f)
-			return false;
-		if ((*s & 0x0f) != scr_colour)
-			return false;
-	}
-
-	if (pri_on) {
-		if (pri_colour == 0x04)
-			return false;
-		if ((*p >> 4) != 0x04 || (*(p + 1) >> 4) != 0x04)
-			return false;
-		if ((*s >> 4) != pri_colour)
-			return false;
-	}
-
-	return true;
-}
-
-static void fix_pixel_left(int x, int y) {
-	uint8 *p;
-
-	if (!scr_on)
-		return;
-
-	p = &game.hires[y * (_WIDTH * 2) + x * 2 + 1];
-	if ((*p & 0x0f) == 0x0f)
-		put_virt_pixel(2 * x + 1, y, 2);
-	else if ((*p & 0x0f) == (*(p - 1) & 0x0f))
-		put_virt_pixel(2 * x + 1, y, 2);
-}
-
-static void fix_pixel_right(int x, int y) {
-	int idx = y * (_WIDTH * 2) + x * 2;
-
-	if (idx >= 160 * 168)
-		return;
-
-	if (scr_on && (game.hires[idx] & 0x0f) == 0x0f)
-		put_virt_pixel(2 * x, y, 2);
-}
-
-static void fix_pixel_here(int x, int y) {
-	uint8 p;
-
-	p = game.hires[y * (_WIDTH * 2) + x * 2 + 1];
-	if (scr_on && (p & 0x0f) == 0x0f)
-		put_virt_pixel(2 * x + 1, y, 2);
-}
-
-/**************************************************************************
-** agiFill
-**************************************************************************/
-static void hires_fill_scanline(int x, int y) {
-	unsigned int c;
-	int newspan_up, newspan_down;
-
-	if (!hires_fill_here(x, y))
-		return;
-
-	/* Scan for left border */
-	for (c = x - 1; c > 0 && hires_fill_here(c, y); c--);
-	fix_pixel_left(c, y);
-
-	newspan_up = newspan_down = 1;
-	for (c++; hires_fill_here(c, y); c++) {
-		put_virt_pixel(c * 2, y, 2);
-		fix_pixel_here(c, y);
-
-		if (hires_fill_here(c, y - 1)) {
-			if (newspan_up) {
-				_PUSH(c + 320 * (y - 1));
-				newspan_up = 0;
-			}
-		} else {
-			newspan_up = 1;
-		}
-
-		if (hires_fill_here(c, y + 1)) {
-			if (newspan_down) {
-				_PUSH(c + 320 * (y + 1));
-				newspan_down = 0;
-			}
-		} else {
-			newspan_down = 1;
-		}
-	}
-
-	fix_pixel_right(c, y);
-}
-
-static void _hires_fill(unsigned int x, unsigned int y) {
-	_PUSH(x + 320 * y);
-
-	while (42) {
-		uint16 c = _POP();
-
-		/* Exit if stack is empty */
-		if (c == 0xffff)
-			break;
-
-		x = c % 320;
-		y = c / 320;
-
-		hires_fill_scanline(x, y);
-	}
-
-	stack_ptr = 0;
-}
-
 /**************************************************************************
 ** fill
 **
 ** AGI flood fill.  (drawing action 0xF8)
 **************************************************************************/
-static void hires_fill() {
-	int x1, y1;
 
-	while ((x1 = next_byte) < 0xf0 && (y1 = next_byte) < 0xf0) {
-		_hires_fill(x1, y1);
-	}
-
-	foffs--;
-}
-
-/**
- * Show AGI picture.
- * This function copies a ``hidden'' AGI picture to the output device.
- */
-void show_hires_pic() {
-	int y, offset;
-	int32 i;
-
-	i = 0;
-	offset = game.line_min_print * CHAR_LINES;
-	for (y = 0; y < _HEIGHT; y++) {
-		put_pixels_hires(0, y + offset, _WIDTH * 2, &game.hires[i]);
-		i += _WIDTH * 2;
-	}
-
-	flush_screen();
-}
-
-void fix_hires_picture() {
-	uint8 *p, *b;
-	int i;
-
-	p = game.hires;
-	b = game.sbuf;
-
-	for (i = 0; p < &game.hires[_WIDTH * _HEIGHT * 2] - 1; p++, i++) {
-		if ((*p & 0x0f) == 0x0f && (*b & 0x0f) != 0x0f) {
-			if ((*(p + 1) & 0x0f) != 0x0f)
-				*p = *(p + 1);
-			else
-				*p = *b;
-		}
-		if ((*p >> 4) == 4 && (*b >> 4) != 4 && (*(b + 1) >> 4) != 4) {
-			*p = (*p & 0x0f) | (*b & 0xf0);
-		}
-		b += (i & 1);
-	}
-}
-
-static void draw_picture() {
+void PictureMgr::drawPicture() {
 	uint8 act;
 	int drawing;
-	int save_foffs;
 
-	pat_code = 0;
-	pat_num = 0;
-	pri_on = scr_on = false;
-	scr_colour = 0xf;
-	pri_colour = 0x4;
+	patCode = 0;
+	patNum = 0;
+	priOn = scrOn = false;
+	scrColour = 0xf;
+	priColour = 0x4;
 
 	drawing = 1;
 
 	debugC(8, kDebugLevelMain, "Drawing picture");
 	for (drawing = 1; drawing && foffs < flen;) {
-		save_foffs = foffs;
-		act = next_byte;
+		act = nextByte;
 		switch (act) {
 		case 0xf0:	/* set colour on screen */
-			scr_colour = next_byte;
-			scr_colour &= 0xF;	/* for v3 drawing diff */
-			scr_on = true;
+			scrColour = nextByte;
+			scrColour &= 0xF;	/* for v3 drawing diff */
+			scrOn = true;
 			break;
 		case 0xf1:	/* disable screen drawing */
-			scr_on = false;
+			scrOn = false;
 			break;
 		case 0xf2:	/* set colour on priority */
-			pri_colour = next_byte;
-			pri_colour &= 0xf;	/* for v3 drawing diff */
-			pri_on = true;
+			priColour = nextByte;
+			priColour &= 0xf;	/* for v3 drawing diff */
+			priOn = true;
 			break;
 		case 0xf3:	/* disable priority screen */
-			pri_on = false;
+			priOn = false;
 			break;
 		case 0xf4:	/* y-corner */
-			y_corner(1);
+			yCorner();
 			break;
 		case 0xf5:	/* x-corner */
-			x_corner(1);
+			xCorner();
 			break;
 		case 0xf6:	/* absolute draw lines */
-			absolute_draw_line(1);
+			absoluteDrawLine();
 			break;
 		case 0xf7:	/* dynamic draw lines */
-			dynamic_draw_line(1);
+			dynamicDrawLine();
 			break;
 		case 0xf8:	/* fill */
 			fill();
 			break;
 		case 0xf9:	/* set pattern */
-			pat_code = next_byte;
+			patCode = nextByte;
 			break;
 		case 0xfA:	/* plot brush */
-			plot_brush(1);
-			break;
-		case 0xFF:	/* end of pic data */
-		default:
-			drawing = 0;
-			break;
-		}
-
-		foffs = save_foffs;
-
-		act = next_byte;
-		switch (act) {
-		case 0xf0:	/* set colour on screen */
-			scr_colour = next_byte;
-			scr_colour &= 0xF;	/* for v3 drawing diff */
-			scr_on = true;
-			break;
-		case 0xf1:	/* disable screen drawing */
-			scr_on = false;
-			break;
-		case 0xf2:	/* set colour on priority */
-			pri_colour = next_byte;
-			pri_colour &= 0xf;	/* for v3 drawing diff */
-			pri_on = true;
-			break;
-		case 0xf3:	/* disable priority screen */
-			pri_on = false;
-			break;
-		case 0xf4:	/* y-corner */
-			y_corner(2);
-			break;
-		case 0xf5:	/* x-corner */
-			x_corner(2);
-			break;
-		case 0xf6:	/* absolute draw lines */
-			absolute_draw_line(2);
-			break;
-		case 0xf7:	/* dynamic draw lines */
-			dynamic_draw_line(2);
-			break;
-		case 0xf8:	/* fill */
-			hires_fill();
-			break;
-		case 0xf9:	/* set pattern */
-			pat_code = next_byte;
-			break;
-		case 0xfA:	/* plot brush */
-			plot_brush(2);
+			plotBrush();
 			break;
 		case 0xFF:	/* end of pic data */
 		default:
@@ -873,11 +571,11 @@ static void draw_picture() {
 /**
  *
  */
-uint8 *convert_v3_pic(uint8 *src, uint32 len) {
+uint8 *PictureMgr::convertV3Pic(uint8 *src, uint32 len) {
 	uint8 d, old = 0, x, *in, *xdata, *out, mode = 0;
 	uint32 i, ulen;
 
-	xdata = (uint8 *) malloc(len + len / 2);
+	xdata = (uint8 *)malloc(len + len / 2);
 
 	out = xdata;
 	in = src;
@@ -917,38 +615,34 @@ uint8 *convert_v3_pic(uint8 *src, uint32 len) {
 /**
  * Decode an AGI picture resource.
  * This function decodes an AGI picture resource into the correct slot
- * and draws it on the AGI screen, optionally cleaning the screen before
+ * and draws it on the AGI screen, optionally clearing the screen before
  * drawing.
  * @param n      AGI picture resource number
  * @param clear  clear AGI screen before drawing
  */
-int decode_picture(int n, int clear) {
+int PictureMgr::decodePicture(int n, int clear) {
 	debugC(8, kDebugLevelResources, "(%d)", n);
 
-	pat_code = 0;
-	pat_num = 0;
-	pri_on = scr_on = false;
-	scr_colour = 0xF;
-	pri_colour = 0x4;
+	patCode = 0;
+	patNum = 0;
+	priOn = scrOn = false;
+	scrColour = 0xF;
+	priColour = 0x4;
 
-	data = game.pictures[n].rdata;
-	flen = game.dir_pic[n].len;
+	data = _vm->_game.pictures[n].rdata;
+	flen = _vm->_game.dirPic[n].len;
 	foffs = 0;
 
-	if (clear) {
-		memset(game.sbuf, 0x4f, _WIDTH * _HEIGHT);
-		memset(game.hires, 0x4f, _WIDTH * 2 * _HEIGHT);
-	}
+	if (clear)
+		memset(_vm->_game.sbuf, 0x4f, _WIDTH * _HEIGHT);
 
-	draw_picture();
-
-	fix_hires_picture();
+	drawPicture();
 
 	if (clear)
-		clear_image_stack();
-	record_image_stack_call(ADD_PIC, n, clear, 0, 0, 0, 0, 0);
+		_vm->clearImageStack();
+	_vm->recordImageStackCall(ADD_PIC, n, clear, 0, 0, 0, 0, 0);
 
-	return err_OK;
+	return errOK;
 }
 
 /**
@@ -957,38 +651,34 @@ int decode_picture(int n, int clear) {
  * resource data.
  * @param n AGI picture resource number
  */
-int unload_picture(int n) {
+int PictureMgr::unloadPicture(int n) {
 	/* remove visual buffer & priority buffer if they exist */
-	if (game.dir_pic[n].flags & RES_LOADED) {
-		free(game.pictures[n].rdata);
-		game.dir_pic[n].flags &= ~RES_LOADED;
+	if (_vm->_game.dirPic[n].flags & RES_LOADED) {
+		free(_vm->_game.pictures[n].rdata);
+		_vm->_game.dirPic[n].flags &= ~RES_LOADED;
 	}
 
-	return err_OK;
+	return errOK;
 }
 
 /**
  * Show AGI picture.
  * This function copies a ``hidden'' AGI picture to the output device.
  */
-void show_pic() {
+void PictureMgr::showPic() {
 	int i, y;
 	int offset;
 
 	debugC(8, kDebugLevelMain, "Show picture!");
-	if (opt.hires) {
-		show_hires_pic();
-		return;
-	}
 
 	i = 0;
-	offset = game.line_min_print * CHAR_LINES;
+	offset = _vm->_game.lineMinPrint * CHAR_LINES;
 	for (y = 0; y < _HEIGHT; y++) {
-		put_pixels_a(0, y + offset, _WIDTH, &game.sbuf[i]);
+		_gfx->putPixelsA(0, y + offset, _WIDTH, &_vm->_game.sbuf[i]);
 		i += _WIDTH;
 	}
 
-	flush_screen();
+	_gfx->flushScreen();
 }
 
-}                             // End of namespace Agi
+} // End of namespace Agi
