@@ -1,7 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
  *
- * Copyright (C) 1999-2001 Sarien Team
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -17,15 +18,16 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/agi/sound.h $
- * $Id: sound.h 22611 2006-05-24 14:25:42Z eriktorbjorn $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/agi/sound.h $
+ * $Id: sound.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
 #ifndef AGI_SOUND_H
 #define AGI_SOUND_H
 
-#include "agi/agi.h"
+#include "sound/audiostream.h"
+#include "sound/mixer.h"
 
 namespace Agi {
 
@@ -45,45 +47,40 @@ namespace Agi {
 #define ENV_RELEASE	7500		/**< envelope release rate */
 #define NUM_CHANNELS    7		/**< number of sound channels */
 
-/**
- * AGI engine sound driver structure.
- */
-struct sound_driver {
-	char *description;
-	int (*init) (int16 * buffer);
-	void (*deinit) (void);
-};
-
 /**
  * AGI sound resource structure.
  */
-struct agi_sound {
+struct AgiSound {
 	uint32 flen;		/**< size of raw data */
 	uint8 *rdata;		/**< raw sound data */
 	uint8 flags;		/**< sound flags */
 	uint16 type;		/**< sound resource type */
 };
 
+#include "common/pack-start.h"
+
 /**
  * AGI sound note structure.
  */
-struct agi_note {
-	uint8 dur_lo;		/**< LSB of note duration */
-	uint8 dur_hi;			/**< MSB of note duration */
-	uint8 frq_0;			/**< LSB of note frequency */
-	uint8 frq_1;			/**< MSB of note frequency */
+struct AgiNote {
+	uint8 durLo;		/**< LSB of note duration */
+	uint8 durHi;			/**< MSB of note duration */
+	uint8 frq0;			/**< LSB of note frequency */
+	uint8 frq1;			/**< MSB of note frequency */
 	uint8 vol;			/**< note volume */
 };
 
+#include "common/pack-end.h"
+
 /**
  * AGI engine sound channel structure.
  */
-struct channel_info {
+struct ChannelInfo {
 #define AGI_SOUND_SAMPLE	0x0001
 #define AGI_SOUND_MIDI		0x0002
 #define AGI_SOUND_4CHN		0x0008
 	uint32 type;
-	struct agi_note *ptr;
+	struct AgiNote *ptr;
 	int16 *ins;
 	int32 size;
 	uint32 phase;
@@ -102,38 +99,18 @@ struct channel_info {
 	uint32 env;
 };
 
-void decode_sound(int);
-void unload_sound(int);
-void play_sound(void);
-int init_sound(void);
-void deinit_sound(void);
-void start_sound(int, int);
-void stop_sound(void);
-uint32 mix_sound(void);
-int load_instruments(char *fname);
+class AgiEngine;
 
-extern struct sound_driver *snd;
+class SoundMgr : public Audio::AudioStream {
+	AgiEngine *_vm;
 
-#endif				/* AGI_SOUND_H */
-
-} // End of namespace Agi
-
-#include "sound/audiostream.h"
-
-namespace Audio {
-class Mixer;
-} // End of namespace Audio
-
-namespace Agi {
-
-class AGIMusic : public Audio::AudioStream {
 public:
-	AGIMusic(Audio::Mixer * pMixer);
-	~AGIMusic(void);
+	SoundMgr(AgiEngine *agi, Audio::Mixer *pMixer);
+	~SoundMgr();
 	virtual void setVolume(uint8 volume);
 
 	// AudioStream API
-	int readBuffer(int16 * buffer, const int numSamples) {
+	int readBuffer(int16 *buffer, const int numSamples) {
 		premixerCall(buffer, numSamples / 2);
 		return numSamples;
 	}
@@ -152,10 +129,32 @@ public:
 	}
 
 private:
-	Audio::Mixer * _mixer;
+	Audio::Mixer *_mixer;
+	Audio::SoundHandle _soundHandle;
 	uint32 _sampleRate;
 
-	void premixerCall(int16 * buf, uint len);
+	void premixerCall(int16 *buf, uint len);
+
+public:
+
+	void decodeSound(int);
+	void unloadSound(int);
+	void playSound();
+	int initSound();
+	void deinitSound();
+	void startSound(int, int);
+	void stopSound();
+	void stopNote(int i);
+	void playNote(int i, int freq, int vol);
+	void playAgiSound();
+	uint32 mixSound();
+	int loadInstruments(char *fname);
+#ifdef USE_IIGS_SOUND
+	void playMidiSound();
+	void playSampleSound();
+#endif
 };
 
 } // End of namespace Agi
+
+#endif /* AGI_SOUND_H */
