@@ -1,7 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
  *
- * Copyright (C) 1999-2003 Sarien Team
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -17,16 +18,14 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/agi/sprite.cpp $
- * $Id: sprite.cpp 23011 2006-06-10 13:20:29Z ender $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/agi/sprite.cpp $
+ * $Id: sprite.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
 #include "agi/agi.h"
 #include "agi/sprite.h"
 #include "agi/graphics.h"
-#include "agi/text.h"
-#include "agi/savegame.h"
 
 namespace Agi {
 
@@ -36,18 +35,15 @@ namespace Agi {
  * a rectangular area of the AGI screen. Sprites are chained in two
  * circular lists, one for updating and other for non-updating sprites.
  */
-struct sprite {
-	vt_entry *v;		/**< pointer to view table entry */
-	int16 x_pos;			/**< x coordinate of the sprite */
-	int16 y_pos;			/**< y coordinate of the sprite */
-	int16 x_size;			/**< width of the sprite */
-	int16 y_size;			/**< height of the sprite */
+struct Sprite {
+	VtEntry *v;		/**< pointer to view table entry */
+	int16 xPos;			/**< x coordinate of the sprite */
+	int16 yPos;			/**< y coordinate of the sprite */
+	int16 xSize;			/**< width of the sprite */
+	int16 ySize;			/**< height of the sprite */
 	uint8 *buffer;			/**< buffer to store background data */
-	uint8 *hires;			/**< buffer for hi-res background */
 };
 
-SpritesMan *_sprites;
-
 /*
  * Sprite pool replaces dynamic allocation
  */
@@ -57,7 +53,7 @@ SpritesMan *_sprites;
 #define POOL_SIZE 68000		/* Gold Rush mine room needs > 50000 */
 	/* Speeder bike challenge needs > 67000 */
 
-void *SpritesMan::pool_alloc(int size) {
+void *SpritesMgr::poolAlloc(int size) {
 	uint8 *x;
 
 	/* Adjust size to 32-bit boundary to prevent data misalignment
@@ -65,12 +61,12 @@ void *SpritesMan::pool_alloc(int size) {
 	 */
 	size = (size + 3) & ~3;
 
-	x = pool_top;
-	pool_top += size;
+	x = _poolTop;
+	_poolTop += size;
 
-	if (pool_top >= (uint8 *)sprite_pool + POOL_SIZE) {
+	if (_poolTop >= (uint8 *)_spritePool + POOL_SIZE) {
 		debugC(1, kDebugLevelMain | kDebugLevelResources, "not enough memory");
-		pool_top = x;
+		_poolTop = x;
 		return NULL;
 	}
 
@@ -80,8 +76,8 @@ void *SpritesMan::pool_alloc(int size) {
 /* Note: it's critical that pool_release() is called in the exact
          reverse order of pool_alloc()
 */
-void SpritesMan::pool_release(void *s) {
-	pool_top = (uint8 *)s;
+void SpritesMgr::poolRelease(void *s) {
+	_poolTop = (uint8 *)s;
 }
 
 /*
@@ -90,12 +86,12 @@ void SpritesMan::pool_release(void *s) {
 
 /* Blit one pixel considering the priorities */
 
-void SpritesMan::blit_pixel(uint8 *p, uint8 *end, uint8 col, int spr, int width, int *hidden) {
+void SpritesMgr::blitPixel(uint8 *p, uint8 *end, uint8 col, int spr, int width, int *hidden) {
 	int epr = 0, pr = 0;	/* effective and real priorities */
 
 	/* CM: priority 15 overrides control lines and is ignored when
 	 *     tracking effective priority. This tweak is needed to fix
-	 *     bug #451768, and should not affect Sierra games because
+	 *     Sarien bug #451768, and should not affect Sierra games because
 	 *     sprites shouldn't have priority 15 (like the AGI Mouse
 	 *     demo "mouse pointer")
 	 *
@@ -125,7 +121,7 @@ void SpritesMan::blit_pixel(uint8 *p, uint8 *end, uint8 col, int spr, int width,
 		*hidden = false;
 
 		/* Except if our priority is 15, which should never happen
-		 * (fixes bug #451768)
+		 * (fixes Sarien bug #451768)
 		 *
 		 * Update: breaks other games, can't be used
 		 *
@@ -136,46 +132,12 @@ void SpritesMan::blit_pixel(uint8 *p, uint8 *end, uint8 col, int spr, int width,
 }
 
 
-#define X_FACT 2		/* Horizontal hires factor */
-
-int SpritesMan::blit_hires_cel(int x, int y, int spr, view_cel *c) {
-	uint8 *q = NULL;
-	uint8 *h0, *h, *end;
-	int i, j, t, m, col;
-	int hidden = true;
-
-	q = c->data;
-	t = c->transparency;
-	m = c->mirror;
-	spr <<= 4;
-	h0 = &game.hires[(x + y * _WIDTH + m * (c->width - 1)) * X_FACT];
-
-	end = game.hires + _WIDTH * X_FACT * _HEIGHT;
-
-	for (i = 0; i < c->height; i++) {
-		h = h0;
-		while (*q) {
-			col = (*q & 0xf0) >> 4;
-			for (j = *q & 0x0f; j; j--, h += X_FACT * (1 - 2 * m)) {
-				if (col != t) {
-					blit_pixel(h, end, col, spr, _WIDTH * X_FACT, &hidden);
-					blit_pixel(h + 1, end, col, spr, _WIDTH * X_FACT, &hidden);
-				}
-			}
-			q++;
-		}
-		h0 += _WIDTH * X_FACT;
-		q++;
-	}
-	return hidden;
-}
-
-int SpritesMan::blit_cel(int x, int y, int spr, view_cel *c) {
+int SpritesMgr::blitCel(int x, int y, int spr, ViewCel *c) {
 	uint8 *p0, *p, *q = NULL, *end;
 	int i, j, t, m, col;
 	int hidden = true;
 
-	/* Fixes bug #477841 (crash in PQ1 map C4 when y == -2) */
+	/* Fixes Sarien bug #477841 (crash in PQ1 map C4 when y == -2) */
 	if (y < 0)
 		y = 0;
 	if (x < 0)
@@ -185,16 +147,13 @@ int SpritesMan::blit_cel(int x, int y, int spr, view_cel *c) {
 	if (x >= _WIDTH)
 		x = _WIDTH - 1;
 
-	if (opt.hires)
-		blit_hires_cel(x, y, spr, c);
-
 	q = c->data;
 	t = c->transparency;
 	m = c->mirror;
 	spr <<= 4;
-	p0 = &game.sbuf[x + y * _WIDTH + m * (c->width - 1)];
+	p0 = &_vm->_game.sbuf[x + y * _WIDTH + m * (c->width - 1)];
 
-	end = game.sbuf + _WIDTH * _HEIGHT;
+	end = _vm->_game.sbuf + _WIDTH * _HEIGHT;
 
 	for (i = 0; i < c->height; i++) {
 		p = p0;
@@ -202,7 +161,7 @@ int SpritesMan::blit_cel(int x, int y, int spr, view_cel *c) {
 			col = (*q & 0xf0) >> 4;
 			for (j = *q & 0x0f; j; j--, p += 1 - 2 * m) {
 				if (col != t) {
-					blit_pixel(p, end, col, spr, _WIDTH, &hidden);
+					blitPixel(p, end, col, spr, _WIDTH, &hidden);
 				}
 			}
 			q++;
@@ -214,88 +173,73 @@ int SpritesMan::blit_cel(int x, int y, int spr, view_cel *c) {
 	return hidden;
 }
 
-void SpritesMan::objs_savearea(sprite *s) {
+void SpritesMgr::objsSaveArea(Sprite *s) {
 	int y;
-	int16 x_pos = s->x_pos, y_pos = s->y_pos;
-	int16 x_size = s->x_size, y_size = s->y_size;
+	int16 xPos = s->xPos, yPos = s->yPos;
+	int16 xSize = s->xSize, ySize = s->ySize;
 	uint8 *p0, *q;
-	uint8 *h0, *k;
 
-	if (x_pos + x_size > _WIDTH)
-		x_size = _WIDTH - x_pos;
+	if (xPos + xSize > _WIDTH)
+		xSize = _WIDTH - xPos;
 
-	if (x_pos < 0) {
-		x_size += x_pos;
-		x_pos = 0;
+	if (xPos < 0) {
+		xSize += xPos;
+		xPos = 0;
 	}
 
-	if (y_pos + y_size > _HEIGHT)
-		y_size = _HEIGHT - y_pos;
+	if (yPos + ySize > _HEIGHT)
+		ySize = _HEIGHT - yPos;
 
-	if (y_pos < 0) {
-		y_size += y_pos;
-		y_pos = 0;
+	if (yPos < 0) {
+		ySize += yPos;
+		yPos = 0;
 	}
 
-	if (x_size <= 0 || y_size <= 0)
+	if (xSize <= 0 || ySize <= 0)
 		return;
 
-	p0 = &game.sbuf[x_pos + y_pos * _WIDTH];
+	p0 = &_vm->_game.sbuf[xPos + yPos * _WIDTH];
 	q = s->buffer;
-	h0 = &game.hires[(x_pos + y_pos * _WIDTH) * 2];
-	k = s->hires;
-	for (y = 0; y < y_size; y++) {
-		memcpy(q, p0, x_size);
-		q += x_size;
+	for (y = 0; y < ySize; y++) {
+		memcpy(q, p0, xSize);
+		q += xSize;
 		p0 += _WIDTH;
-		memcpy(k, h0, x_size * 2);
-		k += x_size * 2;
-		h0 += _WIDTH * 2;
 	}
 }
 
-void SpritesMan::objs_restorearea(sprite *s) {
+void SpritesMgr::objsRestoreArea(Sprite *s) {
 	int y, offset;
-	int16 x_pos = s->x_pos, y_pos = s->y_pos;
-	int16 x_size = s->x_size, y_size = s->y_size;
+	int16 xPos = s->xPos, yPos = s->yPos;
+	int16 xSize = s->xSize, ySize = s->ySize;
 	uint8 *p0, *q;
-	uint8 *h0, *k;
 
-	if (x_pos + x_size > _WIDTH)
-		x_size = _WIDTH - x_pos;
+	if (xPos + xSize > _WIDTH)
+		xSize = _WIDTH - xPos;
 
-	if (x_pos < 0) {
-		x_size += x_pos;
-		x_pos = 0;
+	if (xPos < 0) {
+		xSize += xPos;
+		xPos = 0;
 	}
 
-	if (y_pos + y_size > _HEIGHT)
-		y_size = _HEIGHT - y_pos;
+	if (yPos + ySize > _HEIGHT)
+		ySize = _HEIGHT - yPos;
 
-	if (y_pos < 0) {
-		y_size += y_pos;
-		y_pos = 0;
+	if (yPos < 0) {
+		ySize += yPos;
+		yPos = 0;
 	}
 
-	if (x_size <= 0 || y_size <= 0)
+	if (xSize <= 0 || ySize <= 0)
 		return;
 
-	p0 = &game.sbuf[x_pos + y_pos * _WIDTH];
+	p0 = &_vm->_game.sbuf[xPos + yPos * _WIDTH];
 	q = s->buffer;
-	h0 = &game.hires[(x_pos + y_pos * _WIDTH) * 2];
-	k = s->hires;
-	offset = game.line_min_print * CHAR_LINES;
-	for (y = 0; y < y_size; y++) {
-		memcpy(p0, q, x_size);
-		put_pixels_a(x_pos, y_pos + y + offset, x_size, p0);
-		q += x_size;
+	offset = _vm->_game.lineMinPrint * CHAR_LINES;
+	for (y = 0; y < ySize; y++) {
+		memcpy(p0, q, xSize);
+		_gfx->putPixelsA(xPos, yPos + y + offset, xSize, p0);
+		q += xSize;
 		p0 += _WIDTH;
-		memcpy(h0, k, x_size * 2);
-		if (opt.hires) {
-			put_pixels_hires(x_pos * 2, y_pos + y + offset, x_size * 2, h0);
-		}
-		k += x_size * 2;
-		h0 += _WIDTH * 2;
 	}
 }
 
@@ -303,9 +247,9 @@ void SpritesMan::objs_restorearea(sprite *s) {
 /**
  * Condition to determine whether a sprite will be in the 'updating' list.
  */
-static bool test_updating(vt_entry *v) {
-	/* Sanity check (see bug #779302) */
-	if (~game.dir_view[v->current_view].flags & RES_LOADED)
+bool SpritesMgr::testUpdating(VtEntry *v, AgiEngine *agi) {
+	/* Sanity check (see Sarien bug #779302) */
+	if (~agi->_game.dirView[v->currentView].flags & RES_LOADED)
 		return false;
 
 	return (v->flags & (ANIMATED | UPDATE | DRAWN)) == (ANIMATED | UPDATE | DRAWN);
@@ -314,9 +258,9 @@ static bool test_updating(vt_entry *v) {
 /**
  * Condition to determine whether a sprite will be in the 'non-updating' list.
  */
-static bool test_not_updating(vt_entry *v) {
-	/* Sanity check (see bug #779302) */
-	if (~game.dir_view[v->current_view].flags & RES_LOADED)
+bool SpritesMgr::testNotUpdating(VtEntry *v, AgiEngine *vm) {
+	/* Sanity check (see Sarien bug #779302) */
+	if (~vm->_game.dirView[v->currentView].flags & RES_LOADED)
 		return false;
 
 	return (v->flags & (ANIMATED | UPDATE | DRAWN)) == (ANIMATED | DRAWN);
@@ -325,14 +269,14 @@ static bool test_not_updating(vt_entry *v) {
 /**
  * Convert sprite priority to y value.
  */
-INLINE int SpritesMan::prio_to_y(int p) {
+INLINE int SpritesMgr::prioToY(int p) {
 	int i;
 
 	if (p == 0)
 		return -1;
 
 	for (i = 167; i >= 0; i--) {
-		if (game.pri_table[i] < p)
+		if (_vm->_game.priTable[i] < p)
 			return i;
 	}
 
@@ -342,19 +286,18 @@ INLINE int SpritesMan::prio_to_y(int p) {
 /**
  * Create and initialize a new sprite structure.
  */
-sprite *SpritesMan::new_sprite(vt_entry *v) {
-	sprite *s;
-	s = (sprite *)pool_alloc(sizeof(sprite));
+Sprite *SpritesMgr::newSprite(VtEntry *v) {
+	Sprite *s;
+	s = (Sprite *)poolAlloc(sizeof(Sprite));
 	if (s == NULL)
 		return NULL;
 
 	s->v = v;		/* link sprite to associated view table entry */
-	s->x_pos = v->x_pos;
-	s->y_pos = v->y_pos - v->y_size + 1;
-	s->x_size = v->x_size;
-	s->y_size = v->y_size;
-	s->buffer = (uint8 *) pool_alloc(s->x_size * s->y_size);
-	s->hires = (uint8 *) pool_alloc(s->x_size * s->y_size * 2);
+	s->xPos = v->xPos;
+	s->yPos = v->yPos - v->ySize + 1;
+	s->xSize = v->xSize;
+	s->ySize = v->ySize;
+	s->buffer = (uint8 *)poolAlloc(s->xSize * s->ySize);
 	v->s = s;		/* link view table entry to this sprite */
 
 	return s;
@@ -363,29 +306,29 @@ sprite *SpritesMan::new_sprite(vt_entry *v) {
 /**
  * Insert sprite in the specified sprite list.
  */
-void SpritesMan::spr_addlist(SpriteList& l, vt_entry *v) {
-	sprite *s = new_sprite(v);
+void SpritesMgr::sprAddlist(SpriteList &l, VtEntry *v) {
+	Sprite *s = newSprite(v);
 	l.push_back(s);
 }
 
 /**
  * Sort sprites from lower y values to build a sprite list.
  */
-void SpritesMan::build_list(SpriteList& l, bool (*test) (vt_entry *)) {
+void SpritesMgr::buildList(SpriteList &l, bool (*test)(VtEntry *, AgiEngine *)) {
 	int i, j, k;
-	vt_entry *v;
-	vt_entry *entry[0x100];
-	int y_val[0x100];
-	int min_y = 0xff, min_index = 0;
+	VtEntry *v;
+	VtEntry *entry[0x100];
+	int yVal[0x100];
+	int minY = 0xff, minIndex = 0;
 
 	/* fill the arrays with all sprites that satisfy the 'test'
 	 * condition and their y values
 	 */
 	i = 0;
-	for (v = game.view_table; v < &game.view_table[MAX_VIEWTABLE]; v++) {
-		if ((*test)(v)) {
+	for (v = _vm->_game.viewTable; v < &_vm->_game.viewTable[MAX_VIEWTABLE]; v++) {
+		if ((*test)(v, _vm)) {
 			entry[i] = v;
-			y_val[i] = v->flags & FIXED_PRIORITY ? prio_to_y(v->priority) : v->y_pos;
+			yVal[i] = v->flags & FIXED_PRIORITY ? prioToY(v->priority) : v->yPos;
 			i++;
 		}
 	}
@@ -394,43 +337,42 @@ void SpritesMan::build_list(SpriteList& l, bool (*test) (vt_entry *)) {
 	 * sprite in the list
 	 */
 	for (j = 0; j < i; j++) {
-		min_y = 0xff;
+		minY = 0xff;
 		for (k = 0; k < i; k++) {
-			if (y_val[k] < min_y) {
-				min_index = k;
-				min_y = y_val[k];
+			if (yVal[k] < minY) {
+				minIndex = k;
+				minY = yVal[k];
 			}
 		}
 
-		y_val[min_index] = 0xff;
-		spr_addlist(l, entry[min_index]);
+		yVal[minIndex] = 0xff;
+		sprAddlist(l, entry[minIndex]);
 	}
 }
 
 /**
  * Build list of updating sprites.
  */
-void SpritesMan::build_upd_blitlist() {
-	build_list(spr_upd, test_updating);
+void SpritesMgr::buildUpdBlitlist() {
+	buildList(_sprUpd, testUpdating);
 }
 
 /**
  * Build list of non-updating sprites.
  */
-void SpritesMan::build_nonupd_blitlist() {
-	build_list(spr_nonupd, test_not_updating);
+void SpritesMgr::buildNonupdBlitlist() {
+	buildList(_sprNonupd, testNotUpdating);
 }
 
 /**
  * Clear the given sprite list.
  */
-void SpritesMan::free_list(SpriteList& l) {
+void SpritesMgr::freeList(SpriteList &l) {
 	SpriteList::iterator iter;
 	for (iter = l.reverse_begin(); iter != l.end(); ) {
-		sprite* s = *iter;
-		pool_release(s->hires);
-		pool_release(s->buffer);
-		pool_release(s);
+		Sprite* s = *iter;
+		poolRelease(s->buffer);
+		poolRelease(s);
 		iter = l.reverse_erase(iter);
 	}
 }
@@ -439,49 +381,49 @@ void SpritesMan::free_list(SpriteList& l) {
  * Copy sprites from the pic buffer to the screen buffer, and check if
  * sprites of the given list have moved.
  */
-void SpritesMan::commit_sprites(SpriteList& l) {
+void SpritesMgr::commitSprites(SpriteList &l) {
 	SpriteList::iterator iter;
 	for (iter = l.begin(); iter != l.end(); ++iter) {
-		sprite *s = *iter;
+		Sprite *s = *iter;
 		int x1, y1, x2, y2, w, h;
 
-		w = (s->v->cel_data->width > s->v->cel_data_2->width) ?
-				s->v->cel_data->width : s->v->cel_data_2->width;
+		w = (s->v->celData->width > s->v->celData2->width) ?
+				s->v->celData->width : s->v->celData2->width;
 
-		h = (s->v->cel_data->height >
-				s->v->cel_data_2->height) ? s->v->cel_data->
-				height : s->v->cel_data_2->height;
+		h = (s->v->celData->height >
+				s->v->celData2->height) ? s->v->celData->
+				height : s->v->celData2->height;
 
-		s->v->cel_data_2 = s->v->cel_data;
+		s->v->celData2 = s->v->celData;
 
-		if (s->v->x_pos < s->v->x_pos2) {
-			x1 = s->v->x_pos;
-			x2 = s->v->x_pos2 + w - 1;
+		if (s->v->xPos < s->v->xPos2) {
+			x1 = s->v->xPos;
+			x2 = s->v->xPos2 + w - 1;
 		} else {
-			x1 = s->v->x_pos2;
-			x2 = s->v->x_pos + w - 1;
+			x1 = s->v->xPos2;
+			x2 = s->v->xPos + w - 1;
 		}
 
-		if (s->v->y_pos < s->v->y_pos2) {
-			y1 = s->v->y_pos - h + 1;
-			y2 = s->v->y_pos2;
+		if (s->v->yPos < s->v->yPos2) {
+			y1 = s->v->yPos - h + 1;
+			y2 = s->v->yPos2;
 		} else {
-			y1 = s->v->y_pos2 - h + 1;
-			y2 = s->v->y_pos;
+			y1 = s->v->yPos2 - h + 1;
+			y2 = s->v->yPos;
 		}
 
-		commit_block(x1, y1, x2, y2);
+		commitBlock(x1, y1, x2, y2);
 
-		if (s->v->step_time_count != s->v->step_time)
+		if (s->v->stepTimeCount != s->v->stepTime)
 			continue;
 
-		if (s->v->x_pos == s->v->x_pos2 && s->v->y_pos == s->v->y_pos2) {
+		if (s->v->xPos == s->v->xPos2 && s->v->yPos == s->v->yPos2) {
 			s->v->flags |= DIDNT_MOVE;
 			continue;
 		}
 
-		s->v->x_pos2 = s->v->x_pos;
-		s->v->y_pos2 = s->v->y_pos;
+		s->v->xPos2 = s->v->xPos;
+		s->v->yPos2 = s->v->yPos;
 		s->v->flags &= ~DIDNT_MOVE;
 	}
 }
@@ -489,29 +431,29 @@ void SpritesMan::commit_sprites(SpriteList& l) {
 /**
  * Erase all sprites in the given list.
  */
-void SpritesMan::erase_sprites(SpriteList& l) {
+void SpritesMgr::eraseSprites(SpriteList &l) {
 	SpriteList::iterator iter;
 	for (iter = l.reverse_begin(); iter != l.end(); --iter) {
-		sprite *s = *iter;
-		objs_restorearea(s);
+		Sprite *s = *iter;
+		objsRestoreArea(s);
 	}
 
-	free_list(l);
+	freeList(l);
 }
 
 /**
  * Blit all sprites in the given list.
  */
-void SpritesMan::blit_sprites(SpriteList& l) {
+void SpritesMgr::blitSprites(SpriteList& l) {
 	int hidden;
 	SpriteList::iterator iter;
 	for (iter = l.begin(); iter != l.end(); ++iter) {
-		sprite *s = *iter;
-		objs_savearea(s);
+		Sprite *s = *iter;
+		objsSaveArea(s);
 		debugC(8, kDebugLevelSprites, "s->v->entry = %d (prio %d)", s->v->entry, s->v->priority);
-		hidden = blit_cel(s->x_pos, s->y_pos, s->v->priority, s->v->cel_data);
+		hidden = blitCel(s->xPos, s->yPos, s->v->priority, s->v->celData);
 		if (s->v->entry == 0) {	/* if ego, update f1 */
-			setflag(F_ego_invisible, hidden);
+			_vm->setflag(fEgoInvisible, hidden);
 		}
 	}
 }
@@ -520,18 +462,18 @@ void SpritesMan::blit_sprites(SpriteList& l) {
  * Public functions
  */
 
-void SpritesMan::commit_upd_sprites() {
-	commit_sprites(spr_upd);
+void SpritesMgr::commitUpdSprites() {
+	commitSprites(_sprUpd);
 }
 
-void SpritesMan::commit_nonupd_sprites() {
-	commit_sprites(spr_nonupd);
+void SpritesMgr::commitNonupdSprites() {
+	commitSprites(_sprNonupd);
 }
 
 /* check moves in both lists */
-void SpritesMan::commit_both() {
-	commit_upd_sprites();
-	commit_nonupd_sprites();
+void SpritesMgr::commitBoth() {
+	commitUpdSprites();
+	commitNonupdSprites();
 }
 
 /**
@@ -543,8 +485,8 @@ void SpritesMan::commit_both() {
  * @see erase_nonupd_sprites()
  * @see erase_both()
  */
-void SpritesMan::erase_upd_sprites() {
-	erase_sprites(spr_upd);
+void SpritesMgr::eraseUpdSprites() {
+	eraseSprites(_sprUpd);
 }
 
 /**
@@ -556,8 +498,8 @@ void SpritesMan::erase_upd_sprites() {
  * @see erase_upd_sprites()
  * @see erase_both()
  */
-void SpritesMan::erase_nonupd_sprites() {
-	erase_sprites(spr_nonupd);
+void SpritesMgr::eraseNonupdSprites() {
+	eraseSprites(_sprNonupd);
 }
 
 /**
@@ -569,9 +511,9 @@ void SpritesMan::erase_nonupd_sprites() {
  * @see erase_upd_sprites()
  * @see erase_nonupd_sprites()
  */
-void SpritesMan::erase_both() {
-	erase_upd_sprites();
-	erase_nonupd_sprites();
+void SpritesMgr::eraseBoth() {
+	eraseUpdSprites();
+	eraseNonupdSprites();
 }
 
 /**
@@ -582,10 +524,10 @@ void SpritesMan::erase_both() {
  * @see blit_nonupd_sprites()
  * @see blit_both()
  */
-void SpritesMan::blit_upd_sprites() {
+void SpritesMgr::blitUpdSprites() {
 	debugC(7, kDebugLevelSprites, "blit updating");
-	build_upd_blitlist();
-	blit_sprites(spr_upd);
+	buildUpdBlitlist();
+	blitSprites(_sprUpd);
 }
 
 /**
@@ -596,10 +538,10 @@ void SpritesMan::blit_upd_sprites() {
  * @see blit_upd_sprites()
  * @see blit_both()
  */
-void SpritesMan::blit_nonupd_sprites() {
+void SpritesMgr::blitNonupdSprites() {
 	debugC(7, kDebugLevelSprites, "blit non-updating");
-	build_nonupd_blitlist();
-	blit_sprites(spr_nonupd);
+	buildNonupdBlitlist();
+	blitSprites(_sprNonupd);
 }
 
 /**
@@ -610,9 +552,9 @@ void SpritesMan::blit_nonupd_sprites() {
  * @see blit_upd_sprites()
  * @see blit_nonupd_sprites()
  */
-void SpritesMan::blit_both() {
-	blit_nonupd_sprites();
-	blit_upd_sprites();
+void SpritesMgr::blitBoth() {
+	blitNonupdSprites();
+	blitUpdSprites();
 }
 
 /**
@@ -628,23 +570,23 @@ void SpritesMan::blit_both() {
  * @param pri   priority to use
  * @param mar   if < 4, create a margin around the the base of the cel
  */
-void SpritesMan::add_to_pic(int view, int loop, int cel, int x, int y, int pri, int mar) {
-	view_cel *c = NULL;
+void SpritesMgr::addToPic(int view, int loop, int cel, int x, int y, int pri, int mar) {
+	ViewCel *c = NULL;
 	int x1, y1, x2, y2, y3;
 	uint8 *p1, *p2;
 
 	debugC(3, kDebugLevelSprites, "v=%d, l=%d, c=%d, x=%d, y=%d, p=%d, m=%d", view, loop, cel, x, y, pri, mar);
 
-	record_image_stack_call(ADD_VIEW, view, loop, cel, x, y, pri, mar);
+	_vm->recordImageStackCall(ADD_VIEW, view, loop, cel, x, y, pri, mar);
 
 	/*
 	 * Was hardcoded to 8, changed to pri_table[y] to fix Gold
-	 * Rush (see bug #587558)
+	 * Rush (see Sarien bug #587558)
 	 */
 	if (pri == 0)
-		pri = game.pri_table[y];
+		pri = _vm->_game.priTable[y];
 
-	c = &game.views[view].loop[loop].cel[cel];
+	c = &_vm->_game.views[view].loop[loop].cel[cel];
 
 	x1 = x;
 	y1 = y - c->height + 1;
@@ -664,10 +606,10 @@ void SpritesMan::add_to_pic(int view, int loop, int cel, int x, int y, int pri,
 	if (y2 >= _HEIGHT)
 		y2 = _HEIGHT - 1;
 
-	erase_both();
+	eraseBoth();
 
 	debugC(4, kDebugLevelSprites, "blit_cel (%d, %d, %d, c)", x, y, pri);
-	blit_cel(x1, y1, pri, c);
+	blitCel(x1, y1, pri, c);
 
 	/* If margin is 0, 1, 2, or 3, the base of the cel is
 	 * surrounded with a rectangle of the corresponding priority.
@@ -677,16 +619,18 @@ void SpritesMan::add_to_pic(int view, int loop, int cel, int x, int y, int pri,
 		/* add rectangle around object, don't clobber control
 		 * info in priority data. The box extends to the end of
 		 * its priority band!
-		 *
-		 * SQ1 needs +1 (see bug #810331)
 		 */
-		y3 = (y2 / 12) * 12 + 1;
+		y3 = (y2 / 12) * 12;
+
+		// SQ1 needs +1 (see Sarien bug #810331)
+		if (_vm->getGameID() == GID_SQ1)
+			y3++;
 
 		// don't let box extend below y.
 		if (y3 > y2) y3 = y2;
 
-		p1 = &game.sbuf[x1 + y3 * _WIDTH];
-		p2 = &game.sbuf[x2 + y3 * _WIDTH];
+		p1 = &_vm->_game.sbuf[x1 + y3 * _WIDTH];
+		p2 = &_vm->_game.sbuf[x2 + y3 * _WIDTH];
 
 		for (y = y3; y <= y2; y++) {
 			if ((*p1 >> 4) >= 4)
@@ -698,8 +642,8 @@ void SpritesMan::add_to_pic(int view, int loop, int cel, int x, int y, int pri,
 		}
 
 		debugC(4, kDebugLevelSprites, "pri box: %d %d %d %d (%d)", x1, y3, x2, y2, mar);
-		p1 = &game.sbuf[x1 + y3 * _WIDTH];
-		p2 = &game.sbuf[x1 + y2 * _WIDTH];
+		p1 = &_vm->_game.sbuf[x1 + y3 * _WIDTH];
+		p2 = &_vm->_game.sbuf[x1 + y2 * _WIDTH];
 		for (x = x1; x <= x2; x++) {
 			if ((*p1 >> 4) >= 4)
 				*p1 = (mar << 4) | (*p1 & 0x0f);
@@ -710,10 +654,10 @@ void SpritesMan::add_to_pic(int view, int loop, int cel, int x, int y, int pri,
 		}
 	}
 
-	blit_both();
+	blitBoth();
 
 	debugC(4, kDebugLevelSprites, "commit_block (%d, %d, %d, %d)", x1, y1, x2, y2);
-	commit_block(x1, y1, x2, y2);
+	commitBlock(x1, y1, x2, y2);
 }
 
 /**
@@ -722,13 +666,13 @@ void SpritesMan::add_to_pic(int view, int loop, int cel, int x, int y, int pri,
  * a message box with the object description.
  * @param n  Number of the object to show
  */
-void SpritesMan::show_obj(int n) {
-	view_cel *c;
-	sprite s;
+void SpritesMgr::showObj(int n) {
+	ViewCel *c;
+	Sprite s;
 	int x1, y1, x2, y2;
 
-	agi_load_resource(rVIEW, n);
-	if (!(c = &game.views[n].loop[0].cel[0]))
+	_vm->agiLoadResource(rVIEW, n);
+	if (!(c = &_vm->_game.views[n].loop[0].cel[0]))
 		return;
 
 	x1 = (_WIDTH - c->width) / 2;
@@ -736,32 +680,27 @@ void SpritesMan::show_obj(int n) {
 	x2 = x1 + c->width - 1;
 	y2 = y1 + c->height - 1;
 
-	s.x_pos = x1;
-	s.y_pos = y1;
-	s.x_size = c->width;
-	s.y_size = c->height;
-	s.buffer = (uint8 *)malloc(s.x_size * s.y_size);
-	s.hires = (uint8 *)malloc(s.x_size * s.y_size * 2);
+	s.xPos = x1;
+	s.yPos = y1;
+	s.xSize = c->width;
+	s.ySize = c->height;
+	s.buffer = (uint8 *)malloc(s.xSize * s.ySize);
 
-	objs_savearea(&s);
-	blit_cel(x1, y1, s.x_size, c);
-	commit_block(x1, y1, x2, y2);
-	_text->message_box(game.views[n].descr);
-	objs_restorearea(&s);
-	commit_block(x1, y1, x2, y2);
+	objsSaveArea(&s);
+	blitCel(x1, y1, s.xSize, c);
+	commitBlock(x1, y1, x2, y2);
+	_vm->messageBox(_vm->_game.views[n].descr);
+	objsRestoreArea(&s);
+	commitBlock(x1, y1, x2, y2);
 
 	free(s.buffer);
-
-	/* Added to fix a memory leak --Vasyl */
-	free(s.hires);
 }
 
-void SpritesMan::commit_block(int x1, int y1, int x2, int y2) {
+void SpritesMgr::commitBlock(int x1, int y1, int x2, int y2) {
 	int i, w, offset;
 	uint8 *q;
-	uint8 *h;
 
-	if (!game.picture_shown)
+	if (!_vm->_game.pictureShown)
 		return;
 
 	/* Clipping */
@@ -785,33 +724,26 @@ void SpritesMan::commit_block(int x1, int y1, int x2, int y2) {
 	debugC(7, kDebugLevelSprites, "%d, %d, %d, %d", x1, y1, x2, y2);
 
 	w = x2 - x1 + 1;
-	q = &game.sbuf[x1 + _WIDTH * y1];
-	h = &game.hires[(x1 + _WIDTH * y1) * 2];
-	offset = game.line_min_print * CHAR_LINES;
+	q = &_vm->_game.sbuf[x1 + _WIDTH * y1];
+	offset = _vm->_game.lineMinPrint * CHAR_LINES;
 	for (i = y1; i <= y2; i++) {
-		put_pixels_a(x1, i + offset, w, q);
+		_gfx->putPixelsA(x1, i + offset, w, q);
 		q += _WIDTH;
-		if (opt.hires) {
-			put_pixels_hires(x1 * 2, i + offset, w * 2, h);
-		}
-		h += _WIDTH * 2;
 	}
 
-	flush_block_a(x1, y1 + offset, x2, y2 + offset);
+	_gfx->flushBlockA(x1, y1 + offset, x2, y2 + offset);
 }
 
-SpritesMan::SpritesMan() {
-//	if ((sprite_pool = (uint8 *)malloc(POOL_SIZE)) == NULL)
-//		return err_NotEnoughMemory;
+SpritesMgr::SpritesMgr(AgiEngine *agi, GfxMgr *gfx) {
+	_vm = agi;
+	_gfx = gfx;
 
-	sprite_pool = (uint8 *)malloc(POOL_SIZE);
-	pool_top = sprite_pool;
-
-//	return err_OK;
+	_spritePool = (uint8 *)malloc(POOL_SIZE);
+	_poolTop = _spritePool;
 }
 
-SpritesMan::~SpritesMan() {
-	free(sprite_pool);
+SpritesMgr::~SpritesMgr() {
+	free(_spritePool);
 }
 
-}                             // End of namespace Agi
+} // End of namespace Agi
