@@ -1,7 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
  *
- * Copyright (C) 1999-2001 Sarien Team
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -17,36 +18,39 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/agi/view.h $
- * $Id: view.h 22640 2006-05-25 22:53:45Z wjpalenstijn $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/agi/view.h $
+ * $Id: view.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
 #ifndef AGI_VIEW_H
 #define AGI_VIEW_H
 
+#include "common/stdafx.h"
+#include "common/scummsys.h"
+
 namespace Agi {
 
-struct view_cel {
+struct ViewCel {
 	uint8 height;
 	uint8 width;
 	uint8 transparency;
-	uint8 mirror_loop;
+	uint8 mirrorLoop;
 	uint8 mirror;
 	uint8 *data;
 };
 
-struct view_loop {
-	int num_cels;
-	struct view_cel *cel;
+struct ViewLoop {
+	int numCels;
+	struct ViewCel *cel;
 };
 
 /**
  * AGI view resource structure.
  */
-struct agi_view {
-	int num_loops;
-	struct view_loop *loop;
+struct AgiView {
+	int numLoops;
+	struct ViewLoop *loop;
 	char *descr;
 	uint8 *rdata;
 };
@@ -54,29 +58,29 @@ struct agi_view {
 /**
  * AGI view table entry
  */
-struct vt_entry {
-	uint8 step_time;
-	uint8 step_time_count;
+struct VtEntry {
+	uint8 stepTime;
+	uint8 stepTimeCount;
 	uint8 entry;
-	int16 x_pos;
-	int16 y_pos;
-	uint8 current_view;
-	struct agi_view *view_data;
-	uint8 current_loop;
-	uint8 num_loops;
-	struct view_loop *loop_data;
-	uint8 current_cel;
-	uint8 num_cels;
-	struct view_cel *cel_data;
-	struct view_cel *cel_data_2;
-	int16 x_pos2;
-	int16 y_pos2;
+	int16 xPos;
+	int16 yPos;
+	uint8 currentView;
+	struct AgiView *viewData;
+	uint8 currentLoop;
+	uint8 numLoops;
+	struct ViewLoop *loopData;
+	uint8 currentCel;
+	uint8 numCels;
+	struct ViewCel *celData;
+	struct ViewCel *celData2;
+	int16 xPos2;
+	int16 yPos2;
 	void *s;
-	int16 x_size;
-	int16 y_size;
-	uint8 step_size;
-	uint8 cycle_time;
-	uint8 cycle_time_count;
+	int16 xSize;
+	int16 ySize;
+	uint8 stepSize;
+	uint8 cycleTime;
+	uint8 cycleTimeCount;
 	uint8 direction;
 
 #define MOTION_NORMAL		0
@@ -115,30 +119,8 @@ struct vt_entry {
 	uint8 parm2;
 	uint8 parm3;
 	uint8 parm4;
-};				/* struct vt_entry */
+}; /* struct vt_entry */
 
-/* Motion */
-void check_all_motions(void);
-void move_obj(vt_entry *);
-void in_destination(vt_entry *);
-void fix_position(int);
-void update_position(void);
+} // End of namespace Agi
 
-/* View table management */
-void set_cel(vt_entry *, int);
-void set_loop(vt_entry *, int);
-void set_view(vt_entry *, int);
-void start_update(vt_entry *);
-void stop_update(vt_entry *);
-void update_viewtable(void);
-
-void unload_view(int);
-int decode_view(int);
-void add_to_pic(int, int, int, int, int, int, int);
-void draw_obj(int);
-
-bool is_ego_view(const vt_entry *v);
-
-}                             // End of namespace Agi
-
-#endif				/* AGI_VIEW_H */
+#endif /* AGI_VIEW_H */
