@@ -1,7 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
  *
- * cinE Engine is (C) 2004-2005 by CinE Team
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -17,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/cine/anim.cpp $
- * $Id: anim.cpp 22651 2006-05-26 09:36:41Z eriktorbjorn $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/cine/anim.cpp $
+ * $Id: anim.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -45,7 +46,7 @@ struct animHeader2Struct {
 	uint16 field_E;
 };
 
-uint16 frameVar0 = 0;
+static uint16 animDataCount = 0;
 
 animHeaderStruct animHeader;
 
@@ -183,7 +184,30 @@ animDataEntry animData[] = {
 	{"FIN", 0x9},
 };
 
-byte getAnimTransparentColor(const char *animName) {
+static void freeAnimData(byte idx) {
+	assert(idx < NUM_MAX_ANIMDATA);
+	if (animDataTable[idx].ptr1) {
+		free(animDataTable[idx].ptr1);
+		free(animDataTable[idx].ptr2);
+		memset(&animDataTable[idx], 0, sizeof(AnimData));
+		animDataTable[idx].fileIdx = -1;
+		animDataTable[idx].frameIdx = -1;
+		if (animDataCount > 0)
+			animDataCount--;
+	}
+}
+
+void freeAnimDataRange(byte startIdx, byte numIdx) {
+	for (byte i = 0; i < numIdx; i++) {
+		freeAnimData(i + startIdx);
+	}
+}
+
+void freeAnimDataTable() {
+	freeAnimDataRange(0, NUM_MAX_ANIMDATA);
+}
+
+static byte getAnimTransparentColor(const char *animName) {
 	char name[15];
 
 	removeExtention(name, animName);
@@ -237,7 +261,7 @@ int16 allocFrame(uint16 width, uint16 height, int8 isMask) {
 		animDataTable[i].frameIdx = -1;
 	}
 
-	frameVar0++;
+	animDataCount++;
 
 	return i;
 }
@@ -287,7 +311,7 @@ int16 allocFrame2(uint16 width, uint16 height, uint16 type) {
 	animDataTable[i].fileIdx = -1;
 	animDataTable[i].frameIdx = -1;
 
-	frameVar0++;
+	animDataCount++;
 
 	return i;
 }
@@ -331,7 +355,7 @@ int16 reserveFrame(uint16 width, uint16 height, uint16 type, uint16 idx) {
 	animDataTable[i].fileIdx = -1;
 	animDataTable[i].frameIdx = -1;
 
-	frameVar0++;
+	animDataCount++;
 
 	return i;
 }
@@ -393,6 +417,22 @@ void loadSpl(const char *resourceName) {
 	animDataTable[entry].fileIdx = foundFileIdx;
 	animDataTable[entry].frameIdx = 0;
 	strcpy(animDataTable[entry].name, currentPartName);
+
+	free(dataPtr);
+}
+
+void loadSplAbs(const char *resourceName, uint16 idx) {
+	int16 foundFileIdx;
+	byte *dataPtr;
+	int16 entry;
+
+	foundFileIdx = findFileInBundle(resourceName);
+	dataPtr = readBundleFile(foundFileIdx);
+
+	entry = reserveFrame((uint16) partBuffer[foundFileIdx].unpackedSize, 1, 0, idx);
+	memcpy(animDataTable[entry].ptr1, dataPtr, partBuffer[foundFileIdx].unpackedSize);
+
+	free(dataPtr);
 }
 
 void loadMsk(const char *resourceName) {
@@ -440,6 +480,8 @@ void loadMsk(const char *resourceName) {
 		animDataTable[entry].frameIdx = i;
 		strcpy(animDataTable[entry].name, currentPartName);
 	}
+
+	free(dataPtr);
 }
 
 void loadAni(const char *resourceName) {
@@ -520,6 +562,8 @@ void loadAni(const char *resourceName) {
 		animDataTable[entry].frameIdx = i;
 		strcpy(animDataTable[entry].name, currentPartName);
 	}
+
+	free(dataPtr);
 }
 
 void convert8BBP(byte * dest, byte * source, int16 width, int16 height) {
@@ -719,6 +763,8 @@ void loadSet(const char *resourceName) {
 		animDataTable[entry].frameIdx = i;
 		strcpy(animDataTable[entry].name, currentPartName);
 	}
+
+	free(dataPtr);
 }
 
 void loadSetAbs(const char *resourceName, uint16 idx) {
@@ -806,6 +852,8 @@ void loadSetAbs(const char *resourceName, uint16 idx) {
 		animDataTable[entry].frameIdx = i;
 		strcpy(animDataTable[entry].name, currentPartName);
 	}
+
+	free(dataPtr);
 }
 
 void loadSeq(const char *resourceName) {
@@ -819,6 +867,8 @@ void loadSeq(const char *resourceName) {
 	entry = allocFrame2((uint16) partBuffer[foundFileIdx].unpackedSize, 1, 0);
 
 	memcpy(animDataTable[entry].ptr1, dataPtr + 0x16, (uint16) partBuffer[foundFileIdx].unpackedSize - 0x16);
+
+	free(dataPtr);
 }
 
 void loadSeqAbs(const char *resourceName, uint16 idx) {
@@ -832,6 +882,8 @@ void loadSeqAbs(const char *resourceName, uint16 idx) {
 	entry = reserveFrame((uint16) partBuffer[foundFileIdx].unpackedSize, 1, 0, idx);
 
 	memcpy(animDataTable[entry].ptr1, dataPtr + 0x16, (uint16) partBuffer[foundFileIdx].unpackedSize - 0x16);
+
+	free(dataPtr);
 }
 
 void loadResource(const char *resourceName) {
@@ -856,6 +908,9 @@ void loadResource(const char *resourceName) {
 	} else if (strstr(resourceName, ".SEQ")) {
 		loadSeq(resourceName);
 		return;
+	} else if (strstr(resourceName, "ECHEC")) {
+		exitEngine = 1;
+		return;
 	}
 
 	error("loadResource: Cannot determine type for '%s'", resourceName);
@@ -874,6 +929,7 @@ void loadAbs(const char *resourceName, uint16 idx) {
 		loadSeqAbs(resourceName, idx);
 		return;
 	} else if (strstr(resourceName, ".SPL")) {
+		loadSplAbs(resourceName, idx);
 		return;
 	} else if (strstr(resourceName, ".AMI")) {
 		return;
@@ -1021,6 +1077,7 @@ void loadResourcesFromSave() {
 				}
 			}
 
+			free(dataPtr);
 		}
 	}
 
