@@ -1,7 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
  *
- * cinE Engine is (C) 2004-2005 by CinE Team
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -17,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/cine/bg.cpp $
- * $Id: bg.cpp 22651 2006-05-26 09:36:41Z eriktorbjorn $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/cine/bg.cpp $
+ * $Id: bg.cpp 27208 2007-06-08 19:30:22Z eriktorbjorn $
  *
  */
 
@@ -39,12 +40,14 @@ void loadCtHigh(byte * currentPtr) {
 
 byte loadCt(const char *ctName) {
 	uint16 header[32];
+	byte *ptr, *dataPtr;
 
-	strcpy(currentCtName, ctName);
+	if (currentCtName != ctName)
+		strcpy(currentCtName, ctName);
 
-	byte *ptr = readBundleFile(findFileInBundle(ctName));
+	ptr = dataPtr = readBundleFile(findFileInBundle(ctName));
 
-	if (gameType == Cine::GID_OS) {
+	if (g_cine->getGameType() == Cine::GType_OS) {
 		uint16 bpp = READ_BE_UINT16(ptr); ptr += 2;
 		if (bpp == 8) {
 			ptr += 3 * 256;
@@ -68,6 +71,7 @@ byte loadCt(const char *ctName) {
 		gfxConvertSpriteToRaw(page3Raw, ptr + 0x80, 160, 200);
 	}
 
+	free(dataPtr);
 	return 0;
 }
 
@@ -81,10 +85,13 @@ void loadBgHigh(const char *currentPtr) {
 }
 
 byte loadBg(const char *bgName) {
-	strcpy(currentBgName[0], bgName);
+	byte *ptr, *dataPtr;
+
+	if (currentBgName[0] != bgName)
+		strcpy(currentBgName[0], bgName);
 
 	byte fileIdx = findFileInBundle(bgName);
-	byte *ptr = readBundleFile(fileIdx);
+	ptr = dataPtr = readBundleFile(fileIdx);
 
 	uint16 bpp = READ_BE_UINT16(ptr); ptr += 2;
 	if (bpp == 8) {
@@ -102,18 +109,21 @@ byte loadBg(const char *bgName) {
 		gfxResetRawPage(page2Raw);
 		gfxConvertSpriteToRaw(page2Raw, ptr, 160, 200);
 	}
+	free(dataPtr);
 	return 0;
 }
 
-byte *additionalBgTable[9] = { page2Raw, NULL, NULL, NULL, NULL, NULL, NULL, NULL, page3Raw };
+byte *additionalBgTable[9];
 byte currentAdditionalBgIdx = 0;
 byte currentAdditionalBgIdx2 = 0;
 
 void addBackground(const char *bgName, uint16 bgIdx) {
+	byte *ptr, *dataPtr;
+
 	strcpy(currentBgName[bgIdx], bgName);
 
 	byte fileIdx = findFileInBundle(bgName);
-	byte *ptr = readBundleFile(fileIdx);
+	ptr = dataPtr = readBundleFile(fileIdx);
 
 	additionalBgTable[bgIdx] = (byte *) malloc(320 * 200);
 
@@ -125,6 +135,7 @@ void addBackground(const char *bgName, uint16 bgIdx) {
 		ptr += 32;
 		gfxConvertSpriteToRaw(additionalBgTable[bgIdx], ptr, 160, 200);
 	}
+	free(dataPtr);
 }
 
 } // End of namespace Cine
