@@ -1,7 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
  *
- * cinE Engine is (C) 2004-2005 by CinE Team
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -17,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/cine/cine.cpp $
- * $Id: cine.cpp 22639 2006-05-25 22:51:42Z eriktorbjorn $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/cine/cine.cpp $
+ * $Id: cine.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -28,99 +29,26 @@
 #include "common/config-manager.h"
 #include "common/system.h"
 
-#include "base/plugins.h"
-
-#include "backends/fs/fs.h"
-
 #include "graphics/cursorman.h"
 
 #include "sound/mididrv.h"
 #include "sound/mixer.h"
 
 #include "cine/cine.h"
+#include "cine/bg_list.h"
 #include "cine/main_loop.h"
 #include "cine/object.h"
-#include "cine/sfx_player.h"
-#include "cine/sound_driver.h"
+#include "cine/texte.h"
+#include "cine/sound.h"
 #include "cine/various.h"
 
 
 namespace Cine {
 
-SoundDriver *g_soundDriver;
-SfxPlayer *g_sfxPlayer;
+Sound *g_sound;
 Common::SaveFileManager *g_saveFileMan;
 
-static void initialize();
-
-struct GameSettings {
-	const char *gameid;
-	const char *description;
-	byte id;
-	uint32 features;
-	const char *detectname;
-};
-
-static const GameSettings cine_settings[] = {
-	{"fw", "Future Wars", Cine::GID_FW, MDT_ADLIB, "AUTO00.PRC"},
-	{"os", "Operation Stealth", Cine::GID_OS, MDT_ADLIB, "PROCS00"},
-	{NULL, NULL, 0, 0, NULL}
-};
-
-} // End of namespace Cine
-
-
-GameList Engine_CINE_gameIDList() {
-	GameList games;
-	const Cine::GameSettings *g = Cine::cine_settings;
-
-	while (g->gameid) {
-		games.push_back(*g);
-		g++;
-	}
-
-	return games;
-}
-
-GameDescriptor Engine_CINE_findGameID(const char *gameid) {
-	const Cine::GameSettings *g = Cine::cine_settings;
-	while (g->gameid) {
-		if (0 == scumm_stricmp(gameid, g->gameid))
-			break;
-		g++;
-	}
-	return *g;
-}
-
-DetectedGameList Engine_CINE_detectGames(const FSList &fslist) {
-	DetectedGameList detectedGames;
-	const Cine::GameSettings *g;
-
-	for (g = Cine::cine_settings; g->gameid; ++g) {
-		// Iterate over all files in the given directory
-		for (FSList::const_iterator file = fslist.begin();
-		    file != fslist.end(); ++file) {
-			const char *gameName = file->displayName().c_str();
-
-			if (0 == scumm_stricmp(g->detectname, gameName)) {
-				// Match found, add to list of candidates, then abort inner loop.
-				detectedGames.push_back(*g);
-				break;
-			}
-		}
-	}
-	return detectedGames;
-}
-
-PluginError Engine_CINE_create(OSystem *syst, Engine **engine) {
-	assert(engine);
-	*engine = new Cine::CineEngine(syst);
-	return kNoError;
-}
-
-REGISTER_PLUGIN(CINE, "CINE Engine");
-
-namespace Cine {
+CineEngine *g_cine;
 
 CineEngine::CineEngine(OSystem *syst) : Engine(syst) {
 	Common::addSpecialDebugLevel(kCineDebugScript, "Script", "Script debug level");
@@ -133,38 +61,35 @@ CineEngine::CineEngine(OSystem *syst) : Engine(syst) {
 	_mixer->setVolumeForSoundType(Audio::Mixer::kSFXSoundType, ConfMan.getInt("sfx_volume"));
 	_mixer->setVolumeForSoundType(Audio::Mixer::kMusicSoundType, ConfMan.getInt("music_volume"));
 
-	const Cine::GameSettings *g;
-
-	const char *gameid = ConfMan.get("gameid").c_str();
-	for (g = Cine::cine_settings; g->gameid; ++g)
-		if (!scumm_stricmp(g->gameid, gameid)) {
-			_gameId = g->id;
-			break;
-		}
-
-	gameType = _gameId;
+	g_cine = this;
 }
 
 CineEngine::~CineEngine() {
-}
-
-void CineEngine::errorString(const char *buf1, char *buf2) {
-	strcpy(buf2, buf1);
+	if (g_cine->getGameType() == Cine::GType_OS) {
+		freePoldatDat();
+		freeErrmessDat();
+	}
 }
 
 int CineEngine::init() {
+	// Detect game
+	if (!initGame()) {
+		GUIErrorMessage("No valid games were found in the specified directory.");
+		return -1;
+	}
+
 	// Initialize backend
 	_system->beginGFXTransaction();
 	initCommonGFX(false);
 	_system->initSize(320, 200);
 	_system->endGFXTransaction();
 
-	if (gameType == GID_FW) {
-		g_soundDriver = new AdlibSoundDriverINS(_mixer);
+	if (g_cine->getPlatform() == Common::kPlatformPC) {
+		g_sound = new PCSound(_mixer, this);
 	} else {
-		g_soundDriver = new AdlibSoundDriverADL(_mixer);
+		// Paula chipset for Amiga and Atari versions
+		g_sound = new PaulaSound(_mixer, this);
 	}
-	g_sfxPlayer = new SfxPlayer(g_soundDriver);
 	g_saveFileMan = _saveFileMan;
 
 	initialize();
@@ -177,24 +102,18 @@ int CineEngine::go() {
 
 	mainLoop(1);
 
-	if (gameType == Cine::GID_FW)
-		snd_clearBasesonEntries();
-
-	delete g_sfxPlayer;
-	delete g_soundDriver;
+	delete g_sound;
 	return 0;
 }
 
 
-int gameType;
-
-static void initialize() {
+void CineEngine::initialize() {
 	uint16 i;
 
 	setupOpcodes();
 
-	initLanguage(Common::parseLanguage(ConfMan.get("language")));
-	init_video();
+	initLanguage(g_cine->getLanguage());
+	gfxInit();
 
 	textDataPtr = (byte *)malloc(8000);
 
@@ -204,15 +123,9 @@ static void initialize() {
 	
 	loadTextData("texte.dat", textDataPtr);
 
-	switch (gameType) {
-	case Cine::GID_FW:
-		snd_loadBasesonEntries("BASESON.SND");
-		break;
-	case Cine::GID_OS:
-		// TODO
-		// load POLDAT.DAT
-		// load ERRMESS.DAT (default responses to actions)
-		break;
+	if (g_cine->getGameType() == Cine::GType_OS && !(g_cine->getFeatures() & GF_DEMO)) {
+		loadPoldatDat("poldat.dat");
+		loadErrmessDat("errmess.dat");
 	}
 
 	for (i = 0; i < NUM_MAX_OBJECT; i++) {
@@ -224,11 +137,6 @@ static void initialize() {
 		globalVars[i] = 0;
 	}
 
-	// bypass protection
-	if (gameType == GID_OS && !ConfMan.getBool("copy_protection")) {
-		globalVars[255] = 1;
-	}
-
 	for (i = 0; i < NUM_MAX_SCRIPT; i++) {
 		scriptTable[i].ptr = NULL;
 		scriptTable[i].size = 0;
@@ -256,7 +164,7 @@ static void initialize() {
 	overlayHead.previous = NULL;
 
 	var8 = 0;
-	var9 = NULL;
+	bgIncrustList = NULL;
 
 	objScriptList.next = NULL;
 	globalScriptsHead.next = NULL;
@@ -271,10 +179,23 @@ static void initialize() {
 
 	freePrcLinkedList();
 
-	loadPrc(BOOT_PRC_NAME);
-	strcpy(currentPrcName, BOOT_PRC_NAME);
+	_preLoad = false;
+	if (ConfMan.hasKey("save_slot")) {
+		char saveNameBuffer[256];
 
-	setMouseCursor(MOUSE_CURSOR_NORMAL);
+		sprintf(saveNameBuffer, "%s.%1d", _targetName.c_str(), ConfMan.getInt("save_slot"));
+
+		bool res = makeLoad(saveNameBuffer);
+
+		if (res)
+			_preLoad = true;
+	}
+	
+	if (!_preLoad) {
+		loadPrc(BOOT_PRC_NAME);
+		strcpy(currentPrcName, BOOT_PRC_NAME);
+		setMouseCursor(MOUSE_CURSOR_NORMAL);
+	}
 }
 
 } // End of namespace Cine
