@@ -1,7 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
  *
- * cinE Engine is (C) 2004-2005 by CinE Team
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -17,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/cine/cine.h $
- * $Id: cine.h 21916 2006-04-15 20:36:41Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/cine/cine.h $
+ * $Id: cine.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -27,16 +28,16 @@
 
 #include "common/stdafx.h"
 #include "common/scummsys.h"
+#include "common/file.h"
 #include "common/util.h"
 
-#include "base/engine.h"
+#include "engines/engine.h"
 
 #include "cine/texte.h"
 #include "cine/rel.h"
 #include "cine/script.h"
 #include "cine/part.h"
 #include "cine/prc.h"
-#include "cine/resource.h"
 #include "cine/msg.h"
 #include "cine/bg.h"
 #include "cine/pal.h"
@@ -47,29 +48,53 @@
 
 namespace Cine {
 
-enum CineGameId {
-	GID_FW = 1,
-	GID_OS
+enum CineGameType {
+	GType_FW = 1,
+	GType_OS
 };
 
-class CineEngine : public Engine {
-	int _gameId;
+enum CineGameFeatures {
+	GF_CD =   1 << 0,
+	GF_DEMO = 1 << 1,
+	GF_ALT_FONT = 1 << 2
+};
 
-	void errorString(const char *buf_input, char *buf_output);
+struct CINEGameDescription;
+
+class CineEngine : public Engine {
 
 protected:
 	int init();
 	int go();
 	void shutdown();
+	
+	bool initGame();
 
 public:
 	CineEngine(OSystem *syst);
 	virtual ~CineEngine();
-	int getGameId() {
-		return _gameId;
-	}
+
+	int getGameType() const;
+	uint32 getFeatures() const;
+	Common::Language getLanguage() const;
+	Common::Platform getPlatform() const;
+
+	bool loadSaveDirectory(void);
+	void makeSystemMenu(void);
+
+	const CINEGameDescription *_gameDescription;
+	Common::File _partFileHandle;
+
+private:
+	void initialize(void);
+	bool makeLoad(char *saveName);
+	void mainLoop(int bootScriptIdx);
+
+	bool _preLoad;
 };
 
+extern CineEngine *g_cine;
+
 #define BOOT_PRC_NAME "AUTO00.PRC"
 
 enum {
