@@ -1,7 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
  *
- * cinE Engine is (C) 2004-2005 by CinE Team
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -17,12 +18,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/cine/gfx.cpp $
- * $Id: gfx.cpp 22639 2006-05-25 22:51:42Z eriktorbjorn $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/cine/gfx.cpp $
+ * $Id: gfx.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
 #include "cine/cine.h"
+#include "cine/bg.h"
 #include "cine/various.h"
 
 #include "common/system.h"
@@ -31,18 +33,12 @@
 
 namespace Cine {
 
-byte *screenBuffer;
-
 uint16 c_palette[256];
 
-byte *page0;
-byte *page1;
-byte *page2;
-byte *page3;
-
-byte page1Raw[320 * 200];
-byte page2Raw[320 * 200];
-byte page3Raw[320 * 200];
+byte *screenBuffer;
+byte *page1Raw;
+byte *page2Raw;
+byte *page3Raw;
 
 static const byte mouseCursorNormal[] = {
 	0x00, 0x00, 0x40, 0x00, 0x60, 0x00, 0x70, 0x00,
@@ -87,14 +83,26 @@ static const struct MouseCursor {
 	{ 7, 7, mouseCursorCross }
 };
 
-void init_video() {
-	screenBuffer = (byte *)malloc(320 * 200 * 3);
-	assert(screenBuffer);
+static const byte cursorPalette[] = {
+	0, 0, 0, 0xff,
+	0xff, 0xff, 0xff, 0xff
+};
 
-	page0 = (byte *)malloc(0x8000);
-	page1 = (byte *)malloc(0x8000);
-	page2 = (byte *)malloc(0x8000);
-	page3 = (byte *)malloc(0x8000);
+void gfxInit() {
+	screenBuffer = (byte *)malloc(320 * 200);
+	page1Raw = (byte *)malloc(320 * 200);
+	page2Raw = (byte *)malloc(320 * 200);
+	page3Raw = (byte *)malloc(320 * 200);
+	if (!screenBuffer || !page1Raw || !page2Raw || !page3Raw) {
+		error("Unable to allocate offscreen buffers");
+	}
+	memset(page1Raw, 0, 320 * 200);
+	memset(page2Raw, 0, 320 * 200);
+	memset(page3Raw, 0, 320 * 200);
+	
+	memset(additionalBgTable, 0, sizeof(additionalBgTable));
+	additionalBgTable[0] = page2Raw;
+	additionalBgTable[8] = page3Raw;
 }
 
 void setMouseCursor(int cursor) {
@@ -108,7 +116,7 @@ void setMouseCursor(int cursor) {
 			int offs = i * 8;
 			for (byte mask = 0x80; mask != 0; mask >>= 1) {
 				if (src[0] & mask) {
-					mouseCursor[offs] = 2;
+					mouseCursor[offs] = 1;
 				} else if (src[32] & mask) {
 					mouseCursor[offs] = 0;
 				} else {
@@ -119,11 +127,12 @@ void setMouseCursor(int cursor) {
 			++src;
 		}
 		CursorMan.replaceCursor(mouseCursor, 16, 16, mc->hotspotX, mc->hotspotY);
+		CursorMan.replaceCursorPalette(cursorPalette, 0, 2);
 		currentMouseCursor = cursor;
 	}
 }
 
-uint16 transformColor(uint16 baseColor, int8 r, int8 g, int8 b) {
+static uint16 transformColor(uint16 baseColor, int8 r, int8 g, int8 b) {
 	int8 oriR = (baseColor & 0x7);
 	int8 oriG = (baseColor & 0x70) >> 4;
 	int8 oriB = (baseColor & 0x700) >> 8;
@@ -159,7 +168,7 @@ void transformPaletteRange(byte startColor, byte stopColor, int8 r, int8 g, int8
 	//gfxFlipPage(page2);
 }
 
-void gfxFillSprite(byte *spritePtr, uint16 width, uint16 height, byte *page, int16 x, int16 y) {
+void gfxFillSprite(byte *spritePtr, uint16 width, uint16 height, byte *page, int16 x, int16 y, uint8 fillColor) {
 	int16 i;
 	int16 j;
 
@@ -171,7 +180,7 @@ void gfxFillSprite(byte *spritePtr, uint16 width, uint16 height, byte *page, int
 			if (x + j >= 0 && x + j < 320 && i + y >= 0
 			    && i + y < 200) {
 				if (!*(spritePtr++)) {
-					*(destPtr++) = 0;
+					*(destPtr++) = fillColor;
 				} else {
 					destPtr++;
 				}
@@ -183,6 +192,84 @@ void gfxFillSprite(byte *spritePtr, uint16 width, uint16 height, byte *page, int
 	}
 }
 
+// gfxDrawMaskedSprite
+void gfxSpriteFunc1(byte *spritePtr, byte *maskPtr, uint16 width, uint16 height, byte *page, int16 x, int16 y) {
+	int16 i, j;
+
+	for (i = 0; i < height; i++) {
+		byte *destPtr = page + x + y * 320;
+		destPtr += i * 320;
+
+		for (j = 0; j < width * 8; j++) {
+			if (x + j >= 0 && x + j < 320 && i + y >= 0 && i + y < 200 && *maskPtr == 0) {
+				*destPtr = *spritePtr;
+			}
+			++destPtr;
+			++spritePtr;
+			++maskPtr;
+		}
+	}
+}
+
+// gfxUpdateSpriteMask
+void gfxSpriteFunc2(byte *spritePtr, byte *spriteMskPtr, int16 width, int16 height, byte *maskPtr,
+	int16 maskWidth, int16 maskHeight, byte *bufferSprPtr, byte *bufferMskPtr, int16 xs, int16 ys, int16 xm, int16 ym, byte maskIdx) {
+	int16 i, j, d, spritePitch, maskPitch;
+        
+	width *= 8;
+	maskWidth *= 8;
+
+	spritePitch = width;
+	maskPitch = maskWidth;
+
+	if (maskIdx == 0) {
+		memcpy(bufferSprPtr, spritePtr, spritePitch * height);
+		memcpy(bufferMskPtr, spriteMskPtr, spritePitch * height);
+	}
+    
+	if (ys > ym) {
+		d = ys - ym;
+		maskPtr += d * maskPitch;
+		maskHeight -= d;
+	}
+	if (maskHeight <= 0) {
+		return;
+	}
+	if (xs > xm) {
+		d = xs - xm;
+		maskPtr += d;
+		maskWidth -= d;
+	}
+	if (maskWidth <= 0) {
+		return;
+	}
+	if (ys < ym) {
+		d = ym - ys;
+		spriteMskPtr += d * spritePitch;
+		bufferMskPtr += d * spritePitch;
+		height -= d;
+	}
+	if (height <= 0) {
+		return;
+	}
+	if (xs < xm) {
+		d = xm - xs;
+		spriteMskPtr += d;
+		bufferMskPtr += d;
+		width -= d;
+	}
+	if (width <= 0) {
+		return;
+	}
+	for (j = 0; j < MIN(maskHeight, height); ++j) {
+		for (i = 0; i < MIN(maskWidth, width); ++i) {
+			bufferMskPtr[i] |= maskPtr[i] ^ 1;
+		}
+		bufferMskPtr += spritePitch;
+		maskPtr += maskPitch;
+	}
+}
+
 void gfxDrawLine(int16 x1, int16 y1, int16 x2, int16 y2, byte color, byte *page) {
 	if (x1 == x2) {
 		if (y1 > y2) {
@@ -335,7 +422,7 @@ void drawSpriteRaw(byte *spritePtr, byte *maskPtr, int16 width, int16 height,
 		destPtr += i * 320;
 
 		for (j = 0; j < width * 8; j++) {
-			if (((gameType == Cine::GID_FW && (!maskPtr || !(*maskPtr))) || (gameType == Cine::GID_OS)) && (x + j >= 0
+			if (((g_cine->getGameType() == Cine::GType_FW && (!maskPtr || !(*maskPtr))) || (g_cine->getGameType() == Cine::GType_OS)) && (x + j >= 0
 					&& x + j < 320 && i + y >= 0 && i + y < 200)) {
 				*(destPtr++) = *(spritePtr++);
 			} else {
@@ -369,4 +456,38 @@ void drawSpriteRaw2(byte *spritePtr, byte transColor, int16 width, int16 height,
 	}
 }
 
+void fadeToBlack() {
+	for (int i = 0; i < 8; i++) {
+		if (colorMode256) {
+			for (int j = 0; j < 256; j++) {
+				palette256[j] = transformColor(palette256[j], -1, -1, -1);
+			}
+		} else {
+			for (int j = 0; j < 16; j++) {
+				c_palette[j] = transformColor(c_palette[j], -1, -1, -1);
+			}
+		}
+		gfxFlipRawPage(page1Raw);
+		g_system->updateScreen();
+		g_system->delayMillis(50);
+	}
+}
+
+void gfxFuncGen1(byte *param1, byte *param2, byte *param3, byte *param4, int16 param5) {
+}
+
+void ptrGfxFunc13(void) {
+}
+
+void gfxFuncGen2(void) {
+}
+
+void blitRawScreen(byte *frontBuffer) {
+	gfxFlipRawPage(frontBuffer);
+}
+
+void flip(void) {
+	blitRawScreen(page1Raw);
+}
+
 } // End of namespace Cine
