@@ -1,7 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
  *
- * cinE Engine is (C) 2004-2005 by CinE Team
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -17,50 +18,70 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/cine/main_loop.cpp $
- * $Id: main_loop.cpp 22758 2006-05-29 23:25:50Z joostp $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/cine/main_loop.cpp $
+ * $Id: main_loop.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
 #include "common/stdafx.h"
 #include "common/scummsys.h"
+#include "common/events.h"
 #include "common/system.h"
 
 #include "cine/main_loop.h"
 #include "cine/object.h"
-#include "cine/sfx_player.h"
 #include "cine/various.h"
+#include "cine/bg_list.h"
+#include "cine/sound.h"
 
 namespace Cine {
 
+struct mouseStatusStruct {
+	int left;
+	int right;
+};
+
 mouseStatusStruct mouseData;
 
 uint16 mouseRight = 0;
 uint16 mouseLeft = 0;
 
+int lastKeyStroke = 0;
+
 uint16 mouseUpdateStatus;
 uint16 dummyU16;
 
 void manageEvents(int count) {
-	OSystem::Event event;
+	Common::Event event;
 
-	while (g_system->pollEvent(event)) {
+	Common::EventManager *eventMan = g_system->getEventManager();
+	while (eventMan->pollEvent(event)) {
 		switch (event.type) {
-		case OSystem::EVENT_LBUTTONDOWN:
+		case Common::EVENT_LBUTTONDOWN:
 			mouseLeft = 1;
 			break;
-		case OSystem::EVENT_RBUTTONDOWN:
+		case Common::EVENT_RBUTTONDOWN:
 			mouseRight = 1;
 			break;
-		case OSystem::EVENT_MOUSEMOVE:
-			mouseData.X = event.mouse.x;
-			mouseData.Y = event.mouse.y;
+		case Common::EVENT_MOUSEMOVE:
 			break;
-		case OSystem::EVENT_QUIT:
+		case Common::EVENT_QUIT:
 			g_system->quit();
 			break;
-		case OSystem::EVENT_KEYDOWN:
+		case Common::EVENT_KEYDOWN:
 			switch (event.kbd.keycode) {
+			case '\n':
+			case '\r':
+			case 261: // Keypad 5
+				if (allowPlayerInput) {
+					mouseLeft = 1;
+				}
+				break;
+			case 27:  // ESC
+				if (allowPlayerInput) {
+					mouseRight = 1;
+				}
+				break;
 			case 282: // F1
 				if (allowPlayerInput) {
 					playerCommand = 0; // EXAMINE
@@ -98,17 +119,18 @@ void manageEvents(int count) {
 				}
 				break;
 			case 290: // F9
-				if (allowPlayerInput) {
+				if (allowPlayerInput && !inMenu) {
 					makeActionMenu();
 					makeCommandLine();
 				}
 				break;
 			case 291: // F10
-				if (allowPlayerInput) {
-					makeSystemMenu();
+				if (!disableSystemMenu && !inMenu) {
+					g_cine->makeSystemMenu();
 				}
 				break;
 			default:
+				lastKeyStroke = event.kbd.keycode;
 				break;
 			}
 			break;
@@ -125,7 +147,6 @@ void manageEvents(int count) {
 	}
 
 	int i;
-
 	for (i = 0; i < count; i++) {
 		//FIXME(?): Maybe there's a better way to "fix" this?
 		//
@@ -135,13 +156,15 @@ void manageEvents(int count) {
 		if (i % 2)
 			g_system->updateScreen();
 		g_system->delayMillis(10);
+		g_sound->update();
 		manageEvents(0);
 	}
 }
 
 void getMouseData(uint16 param, uint16 *pButton, uint16 *pX, uint16 *pY) {
-	*pX = mouseData.X;
-	*pY = mouseData.Y;
+	Common::Point mouse = g_system->getEventManager()->getMousePos();
+	*pX = mouse.x;
+	*pY = mouse.y;
 
 	*pButton = 0;
 
@@ -154,65 +177,77 @@ void getMouseData(uint16 param, uint16 *pButton, uint16 *pX, uint16 *pY) {
 	}
 }
 
-void mainLoop(int bootScriptIdx) {
+int getKeyData() {
+	int k = lastKeyStroke;
+
+	lastKeyStroke = -1;
+
+	return k;
+}
+
+void CineEngine::mainLoop(int bootScriptIdx) {
 	uint16 var_6;
 	uint16 quitFlag;
 	uint16 i;
 	byte di;
 	uint16 mouseButton;
 
-	freeAnimDataTable();
-	resetMessageHead();
-	resetSeqList();
-	resetglobalScriptsHead();
-	resetObjectScriptHead();
-	mainLoopSub1();
-
-	setTextWindow(0, 0, 20, 200);
-
-	errorVar = 0;
-
-	addScriptToList0(bootScriptIdx);
-
-	menuVar = 0;
-
-	gfxFuncGen1(page0c, page0, page0c, page0, -1);
-
-	ptrGfxFunc13();
-
-	gfxFuncGen2();
-
 	quitFlag = 0;
-	allowPlayerInput = 0;
-	checkForPendingDataLoadSwitch = 0;
 
-	fadeRequired = 0;
-	isDrawCommandEnabled = 0;
-	waitForPlayerClick = 0;
-	menuCommandLen = 0;
+	if (_preLoad == false) {
+		freeAnimDataTable();
+		resetMessageHead();
+		resetSeqList();
+		resetglobalScriptsHead();
+		resetObjectScriptHead();
+		resetBgIncrustList();
 
-	playerCommand = -1;
-	strcpy(commandBuffer, "");
+		setTextWindow(0, 0, 20, 200);
 
-	globalVars[0x1F2] = 0;
-	globalVars[0x1F4] = 0;
+		errorVar = 0;
 
-	for (i = 0; i < 16; i++) {
-		c_palette[i] = 0;
+		addScriptToList0(bootScriptIdx);
+
+		menuVar = 0;
+
+//		gfxFuncGen1(page0c, page0, page0c, page0, -1);
+
+		ptrGfxFunc13();
+
+		gfxFuncGen2();
+
+		inMenu = false;
+		allowPlayerInput = 0;
+		checkForPendingDataLoadSwitch = 0;
+
+		fadeRequired = 0;
+		isDrawCommandEnabled = 0;
+		waitForPlayerClick = 0;
+		menuCommandLen = 0;
+
+		playerCommand = -1;
+		strcpy(commandBuffer, "");
+
+		globalVars[0x1F2] = 0;
+		globalVars[0x1F4] = 0;
+
+		for (i = 0; i < 16; i++) {
+			c_palette[i] = 0;
+		}
+
+		var17 = 1;
+
+		strcpy(newPrcName, "");
+		strcpy(newRelName, "");
+		strcpy(newObjectName, "");
+		strcpy(newMsgName, "");
+		strcpy(currentBgName[0], "");
+		strcpy(currentCtName, "");
+		strcpy(currentPartName, "");
+
+		g_sound->stopMusic();
 	}
 
-	var17 = 1;
-
-	strcpy(newPrcName, "");
-	strcpy(newRelName, "");
-	strcpy(newObjectName, "");
-	strcpy(newMsgName, "");
-	strcpy(currentBgName[0], "");
-	strcpy(currentCtName, "");
-	strcpy(currentPartName, "");
-
-	g_sfxPlayer->stop();
-
 	do {
 		mainLoopSub3();
 		di = executePlayerInput();
@@ -299,15 +334,15 @@ void mainLoop(int bootScriptIdx) {
 	} while (!exitEngine && !quitFlag && var21 != 7);
 
 	hideMouse();
-	g_sfxPlayer->stop();
+	g_sound->stopMusic();
 	freeAnimDataTable();
 	unloadAllMasks();
 	freePrcLinkedList();
 	releaseObjectScripts();
-	// if (gameType == Cine::GID_OS) {
+	// if (g_cine->getGameType() == Cine::GType_OS) {
 	// 	freeUnkList();
 	// }
-	closeEngine7();
+	freeBgIncrustList();
 	closePart();
 }
 
