@@ -1,7 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
  *
- * cinE Engine is (C) 2004-2005 by CinE Team
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -17,13 +18,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/cine/part.h $
- * $Id: part.h 22651 2006-05-26 09:36:41Z eriktorbjorn $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/cine/part.h $
+ * $Id: part.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
-#ifndef CINE_PART_H_
-#define CINE_PART_H_
+#ifndef CINE_PART_H
+#define CINE_PART_H
 
 namespace Cine {
 
@@ -57,7 +58,6 @@ extern AnimData *animDataTable;
 extern PartBuffer *partBuffer;
 
 void loadPart(const char *partName);
-void freePartRange(byte startIdx, byte numIdx);
 void closePart(void);
 
 int16 findFileInBundle(const char *fileName);
@@ -65,8 +65,13 @@ int16 findFileInBundle(const char *fileName);
 void readFromPart(int16 idx, byte *dataPtr);
 
 byte *readBundleFile(int16 foundFileIdx);
+byte *readBundleSoundFile(const char *entryName, uint32 *size = 0);
 byte *readFile(const char *filename);
 
+void checkDataDisk(int16 param);
+
+void dumpBundle(const char *filename);
+
 } // End of namespace Cine
 
 #endif
