@@ -1,7 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
  *
- * cinE Engine is (C) 2004-2005 by CinE Team
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -17,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/cine/prc.cpp $
- * $Id: prc.cpp 21500 2006-03-29 15:59:37Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/cine/prc.cpp $
+ * $Id: prc.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -26,7 +27,6 @@
 #include "common/endian.h"
 
 #include "cine/cine.h"
-#include "cine/auto00.h"
 #include "cine/various.h"
 
 #include "common/config-manager.h"
@@ -64,7 +64,7 @@ void freePrcLinkedList(void) {
 void loadPrc(const char *pPrcName) {
 	byte i;
 	uint16 numScripts;
-	const byte *scriptPtr;
+	byte *scriptPtr, *dataPtr;
 
 	assert(pPrcName);
 
@@ -77,20 +77,26 @@ void loadPrc(const char *pPrcName) {
 		}
 	}
 
-	checkDataDisk(-1);
-	if ((gameType == Cine::GID_FW) && (!strcmp(pPrcName, "AUTO00.PRC"))) {
-		if (!ConfMan.getBool("copy_protection"))
-			scriptPtr = AUT000;
-		else
-			scriptPtr = readFile(pPrcName);
-	} else {
-		scriptPtr = readBundleFile(findFileInBundle(pPrcName));
-		assert(scriptPtr);
+	// This is copy protection. Used to hang the machine
+	if (!scumm_stricmp(pPrcName, "L201.ANI")) {
+		exitEngine = 1;
+		return;
 	}
 
+	checkDataDisk(-1);
+	if ((g_cine->getGameType() == Cine::GType_FW) &&
+		(!scumm_stricmp(pPrcName, BOOT_PRC_NAME) || !scumm_stricmp(pPrcName, "demo.prc"))) {
+		scriptPtr = dataPtr = readFile(pPrcName);
+	} else {
+		scriptPtr = dataPtr = readBundleFile(findFileInBundle(pPrcName));
+	}
+
+	assert(scriptPtr);
+
 	setMouseCursor(MOUSE_CURSOR_DISK);
 
-	numScripts = READ_BE_UINT16(scriptPtr); scriptPtr += 2;
+	numScripts = READ_BE_UINT16(scriptPtr);	
+	scriptPtr += 2;
 	assert(numScripts <= NUM_MAX_SCRIPT);
 
 	for (i = 0; i < numScripts; i++) {
@@ -107,6 +113,25 @@ void loadPrc(const char *pPrcName) {
 			computeScriptStack(scriptTable[i].ptr, scriptTable[i].stack, size);
 		}
 	}
+
+	free(dataPtr);
+
+#ifdef DUMP_SCRIPTS
+
+	{
+		uint16 s;
+		char buffer[256];
+
+		for (s = 0; s < numScripts; s++) {
+			if (scriptTable[s].size) {
+				sprintf(buffer, "%s_%03d.txt", pPrcName, s);
+
+				decompileScript(scriptTable[s].ptr, scriptTable[s].stack, scriptTable[s].size, s);
+				dumpScript(buffer);
+			}
+		}
+	}
+#endif
 }
 
 } // End of namespace Cine
