@@ -1,7 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
  *
- * cinE Engine is (C) 2004-2005 by CinE Team
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -17,13 +18,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/cine/script.h $
- * $Id: script.h 21699 2006-04-08 19:26:15Z eriktorbjorn $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/cine/script.h $
+ * $Id: script.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
-#ifndef CINE_SCRIPT_H_
-#define CINE_SCRIPT_H_
+#ifndef CINE_SCRIPT_H
+#define CINE_SCRIPT_H
 
 namespace Cine {
 
@@ -139,14 +140,15 @@ void o1_op72();
 void o1_op73();
 void o1_playSample();
 void o1_playSample();
-void o1_allowSystemMenu();
+void o1_disableSystemMenu();
 void o1_loadMask5();
 void o1_unloadMask5();
 
 void o2_loadPart();
 void o2_addSeqListElement();
 void o2_removeSeq();
-void o2_op78(); 
+void o2_playSample();
+void o2_playSampleAlt();
 void o2_op81();
 void o2_op82();
 void o2_isSeqRunning();
