@@ -1,7 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
  *
- * cinE Engine is (C) 2004-2005 by CinE Team
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -17,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/cine/unpack.cpp $
- * $Id: unpack.cpp 21500 2006-03-29 15:59:37Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/cine/unpack.cpp $
+ * $Id: unpack.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -29,6 +30,14 @@
 
 namespace Cine {
 
+struct UnpackCtx {
+	int size, datasize;
+	uint32 crc;
+	uint32 chk;
+	byte *dst;
+	const byte *src;	
+};
+
 static int rcr(UnpackCtx *uc, int CF) {
 	int rCF = (uc->chk & 1);
 	uc->chk >>= 1;
