@@ -1,7 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
  *
- * cinE Engine is (C) 2004-2005 by CinE Team
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -17,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/cine/various.cpp $
- * $Id: various.cpp 22795 2006-05-31 10:55:09Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/cine/various.cpp $
+ * $Id: various.cpp 27208 2007-06-08 19:30:22Z eriktorbjorn $
  *
  */
 
@@ -27,15 +28,16 @@
 #include "common/savefile.h"
 
 #include "cine/cine.h"
-#include "cine/font.h"
 #include "cine/main_loop.h"
 #include "cine/object.h"
-#include "cine/sfx_player.h"
+#include "cine/sound.h"
+#include "cine/bg_list.h"
 #include "cine/various.h"
 
 namespace Cine {
 
-int16 allowSystemMenu = 0;
+bool disableSystemMenu = false;
+bool inMenu;
 
 int16 commandVar3[4];
 int16 commandVar1;
@@ -44,8 +46,6 @@ int16 commandVar2;
 unk1Struct messageTable[NUM_MAX_MESSAGE];
 
 uint32 var6;
-uint32 var8;
-byte *var9;
 
 uint16 var2;
 uint16 var3;
@@ -57,38 +57,15 @@ int16 buildObjectListCommand(void);
 void drawString(const char *string, byte param) {
 }
 
-void blitRawScreen(byte *frontBuffer) {
-	gfxFlipRawPage(frontBuffer);
-}
-
-Common::File partFileHandle;
-
 void waitPlayerInput(void) {
 }
 
-void freeAnimDataTable(void) {
-}
-
-void mainLoopSub1(void) {
-}
-
 void setTextWindow(uint16 param1, uint16 param2, uint16 param3, uint16 param4) {
 }
 
 uint16 errorVar;
 byte menuVar;
 
-void gfxFuncGen1(byte *param1, byte *param2, byte *param3, byte *param4, int16 param5) {
-}
-
-byte *page0c;
-
-void ptrGfxFunc13(void) {
-}
-
-void gfxFuncGen2(void) {
-}
-
 uint16 allowPlayerInput;
 
 uint16 checkForPendingDataLoadSwitch;
@@ -132,148 +109,8 @@ byte inputVar1 = 0;
 uint16 inputVar2;
 uint16 inputVar3;
 
-const char **failureMessages;
-const commandeType *defaultActionCommand;
-const commandeType *systemMenu;
-const commandeType *confirmMenu;
-const char *commandPrepositionOn;
-
 selectedObjStruct currentSelectedObject;
 
-void initLanguage(Common::Language lang) {
-	static const char *failureMessages_EN[] = {
-		// EXAMINE
-		"I don't see anything unusual.",
-		"There's nothing of interest here.",
-		"This isn't particularly interesting.",
-		"You won't find anything.",
-		// TAKE
-		"I can't take that.",
-		"I find it difficult.",
-		"I don't see what I am supposed to take.",
-		"I have difficulty in following you.",
-		// INVENTORY (???)
-		"There's no point.",
-		"You have better things to do.",
-		"Come on, don't let's waste any time.",
-		"That doesn't seem to me to be a good idea.",
-		// USE
-		"I don't see why I should do that.",
-		"It's had no effect whatsoever.",
-		"It won't produce any results.",
-		"Try and find something else.",
-		// OPERATE
-		"It doesn't work.",
-		"Let suppose you are trying and don't let's mention it again.",
-		"Nothing happens.",
-		"You have better things to do.",
-		// SPEAK
-		"No answer.",
-		"More action , less talking !",
-		"I'd be very surprised if you got an answer",
-		"A wall of silence ..."
-	};
-
-	static const commandeType defaultActionCommand_EN[] = {
-		"EXAMINE",
-		"TAKE",
-		"INVENTORY",
-		"USE",
-		"OPERATE",
-		"SPEAK",
-		"NOACTION"
-	};
-
-	static const commandeType systemMenu_EN[] = {
-		"Pause",
-		"Restart Game",
-		"Quit",
-		"Backup Drive is A:",
-		"Restore game",
-		"Save game"
-	};
-
-	static const commandeType confirmMenu_EN[] = {
-		"Ok, go ahead ...",
-		"Absolutely Not!"
-	};
-
-	// \x82 == , \x85 == , \x87 == , \x88 == 
-	static const char *failureMessages_FR[] = {
-		// EXAMINER
-		"Je ne vois rien de special.",
-		"Il n'y a rien d'int\x82ressant.",
-		"Cela pr\x82sente peu d'int\x82r\x88ts.",
-		"Vous ne trouvez rien.",
-		// PRENDRE
-		"Je ne peux pas prendre cela.",
-		"Cela me semble difficile",
-		"Je ne vois pas ce qu'il y a \x85 prendre",
-		"j'ai du mal \x85 vous suivre.",
-		// INVENTAIRE (???)
-		"C'est inutile",
-		"Vous avez mieux \x85 faire",
-		"Allons, ne perdons pas de temps",
-		"\x87""a ne me semble pas \x88tre une bonne id\x82""e",
-		// UTILISER
-		"Je ne vois pas pourquoi je ferais cela.",
-		"C'est absolument sans effets",
-		"Cela n'amenerait \x85 rien",
-		"Essayez de trouver autre chose.",
-		// ACTIONNER
-		"Ca ne marche pas",
-		"Supposons que vous essayez et n'en parlons plus.",
-		"Rien n'y fait.",
-		"Vous avez mieux \x85 faire.",
-		// PARLER
-		"Vous lui parlez . Sans r\x82ponse.",
-		"Plus d'actes et moins de Paroles !",
-		"Je serais bien surpris si vous obteniez une r\x82ponse.",
-		"Un mur de silence ..."
-	};
-
-	static const commandeType defaultActionCommand_FR[] = {
-		"EXAMINER",
-		"PRENDRE",
-		"INVENTAIRE",
-		"UTILISER",
-		"ACTIONNER",
-		"PARLER",
-		"NOACTION"
-	};
-
-	static const commandeType systemMenu_FR[] = {
-		"Pause",
-		"Nouvelle partie",
-		"Quitter",
-		"Lecteur de Svg. A:",
-		"Charger une partie",
-		"Sauver la partie"
-	};
-
-	static const commandeType confirmMenu_FR[] = {
-		"Ok , Vas-y ...",
-		"Surtout Pas !"
-	};
-
-	switch (lang) {
-	case Common::FR_FRA:
-		failureMessages = failureMessages_FR;
-		defaultActionCommand = defaultActionCommand_FR;
-		systemMenu = systemMenu_FR;
-		confirmMenu = confirmMenu_FR;
-		commandPrepositionOn = "sur";
-		break;
-	default:
-		failureMessages = failureMessages_EN;
-		defaultActionCommand = defaultActionCommand_EN;
-		systemMenu = systemMenu_EN;
-		confirmMenu = confirmMenu_EN;
-		commandPrepositionOn = "on";
-		break;
-	}
-}
-
 void mainLoopSub3(void) {
 }
 
@@ -421,7 +258,7 @@ int16 getObjectUnderCursor(uint16 x, uint16 y) {
 
 				if ((xdif >= 0) && ((treshold << 4) > xdif) && (ydif > 0) && (ydif < height)) {
 					if (animDataTable[frame].ptr1) {
-						if (gameType == Cine::GID_OS)
+						if (g_cine->getGameType() == Cine::GType_OS)
 							return currentHead->objIdx;
 
 						if (currentHead->type == 0)	{ // use generated mask
@@ -446,22 +283,21 @@ int16 getObjectUnderCursor(uint16 x, uint16 y) {
 
 static commandeType currentSaveName[10];
 
-int16 loadSaveDirectory(void) {
+bool CineEngine::loadSaveDirectory(void) {
 	Common::InSaveFile *fHandle;
+	char tmp[80];
 
-	if (gameType == Cine::GID_FW)
-		fHandle = g_saveFileMan->openForLoading("FW.DIR");
-	else
-		fHandle = g_saveFileMan->openForLoading("OS.DIR");
+	snprintf(tmp, 80, "%s.dir", _targetName.c_str());
+	fHandle = g_saveFileMan->openForLoading(tmp);
 
 	if (!fHandle) {
-		return 0;
+		return false;
 	}
 
 	fHandle->read(currentSaveName, 10 * 20);
 	delete fHandle;
 
-	return 1;
+	return true;
 }
 
 int16 currentDisk;
@@ -583,7 +419,7 @@ void setupObjectScriptList(void) {
 	}
 }
 
-int16 makeLoad(char *saveName) {
+bool CineEngine::makeLoad(char *saveName) {
 	int16 i;
 	int16 size;
 	Common::InSaveFile *fHandle;
@@ -591,22 +427,22 @@ int16 makeLoad(char *saveName) {
 	fHandle = g_saveFileMan->openForLoading(saveName);
 
 	if (!fHandle) {
-		drawString("Cette sauvegarde n'existe pas ...", 0);
+		drawString(otherMessages[0], 0);
 		waitPlayerInput();
 		// restoreScreen();
 		checkDataDisk(-1);
-		return -1;
+		return false;
 	}
 
-	g_sfxPlayer->stop();
+	g_sound->stopMusic();
 	freeAnimDataTable();
 	unloadAllMasks();
-	// if (gameType == Cine::GID_OS) {
+	// if (g_cine->getGameType() == Cine::GType_OS) {
 	// 	freeUnkList();
 	// }
 	freePrcLinkedList();
 	releaseObjectScripts();
-	closeEngine7();
+	freeBgIncrustList();
 	closePart();
 
 	for (i = 0; i < NUM_MAX_REL; i++) {
@@ -781,7 +617,7 @@ int16 makeLoad(char *saveName) {
 
 	size = fHandle->readSint16BE();
 	for (i = 0; i < size; i++) {
-		// loadBgIncrustFromSave(fHandle);
+		loadBgIncrustFromSave(fHandle);
 	}
 
 	delete fHandle;
@@ -815,7 +651,7 @@ int16 makeLoad(char *saveName) {
 	}
 
 	loadResourcesFromSave();
-	//reincrustAllBg();
+	reincrustAllBg();
 
 	setMouseCursor(MOUSE_CURSOR_NORMAL);
 
@@ -828,7 +664,7 @@ int16 makeLoad(char *saveName) {
 		}*/
 	}
 
-	return 0;
+	return true;
 }
 
 void makeSave(char *saveFileName) {
@@ -838,7 +674,7 @@ void makeSave(char *saveFileName) {
 	fHandle = g_saveFileMan->openForSaving(saveFileName);
 
 	if (!fHandle) {
-		drawString("Could not create save file ...", 0);
+		drawString(otherMessages[1], 0);
 		waitPlayerInput();
 		// restoreScreen();
 		checkDataDisk(-1);
@@ -1021,22 +857,44 @@ void makeSave(char *saveFileName) {
 		}
 	}
 
-	// This corresponds to the loadBgIncrustFromSave() handling, I think.
-	fHandle->writeUint16BE(0);
+	int numBgIncrustList = 0;
+	BGIncrustList *bgIncrustPtr = bgIncrustList;
+
+	while (bgIncrustPtr) {
+		numBgIncrustList++;
+		bgIncrustPtr = bgIncrustPtr->next;
+	}
+
+	fHandle->writeUint16BE(numBgIncrustList);
+	bgIncrustPtr = bgIncrustList;
+	while (bgIncrustPtr) {
+		fHandle->writeUint32BE(0); // next
+		fHandle->writeUint32BE(0); // unkPtr
+		fHandle->writeUint16BE(bgIncrustPtr->objIdx);
+		fHandle->writeUint16BE(bgIncrustPtr->param);
+		fHandle->writeUint16BE(bgIncrustPtr->x);
+		fHandle->writeUint16BE(bgIncrustPtr->y);
+		fHandle->writeUint16BE(bgIncrustPtr->frame);
+		fHandle->writeUint16BE(bgIncrustPtr->part);
+
+		bgIncrustPtr = bgIncrustPtr->next;
+	}
 
 	delete fHandle;
 
 	setMouseCursor(MOUSE_CURSOR_NORMAL);
 }
 
-void makeSystemMenu(void) {
+void CineEngine::makeSystemMenu(void) {
 	int16 numEntry;
 	int16 mouseButton;
 	int16 mouseX;
 	int16 mouseY;
 	int16 systemCommand;
 
-	if (!allowSystemMenu) {
+	if (!disableSystemMenu) {
+		inMenu = true;
+
 		manageEvents();
 		getMouseData(mouseUpdateStatus, (uint16 *)&mouseButton,(uint16 *)&mouseX, (uint16 *)&mouseY);
 
@@ -1056,7 +914,7 @@ void makeSystemMenu(void) {
 		switch (systemCommand) {
 		case 0:
 			{
-				drawString("PAUSE", 0);
+				drawString(otherMessages[2], 0);
 				waitPlayerInput();
 				break;
 			}
@@ -1090,31 +948,28 @@ void makeSystemMenu(void) {
 
 					if (selectedSave >= 0) {
 						char saveNameBuffer[256];
-						if (gameType == Cine::GID_FW)
-							sprintf(saveNameBuffer, "FW.%1d", selectedSave);
-						else
-							sprintf(saveNameBuffer, "OS.%1d", selectedSave);
+						sprintf(saveNameBuffer, "%s.%1d", _targetName.c_str(), selectedSave);
 
 						getMouseData(mouseUpdateStatus, (uint16 *)&mouseButton, (uint16 *)&mouseX, (uint16 *)&mouseY);
 						if (!makeMenuChoice(confirmMenu, 2, mouseX, mouseY + 8, 100)) {
 							char loadString[256];
 
-							sprintf(loadString, "Chargement de | %s", currentSaveName[selectedSave]);
+							sprintf(loadString, otherMessages[3], currentSaveName[selectedSave]);
 							drawString(loadString, 0);
 
 							makeLoad(saveNameBuffer);
 						} else {
-							drawString("Chargement Annul ...", 0);
+							drawString(otherMessages[4], 0);
 							waitPlayerInput();
 							checkDataDisk(-1);
 						}
 					} else {
-						drawString("Chargement Annul ...", 0);
+						drawString(otherMessages[4], 0);
 						waitPlayerInput();
 						checkDataDisk(-1);
 					}
 				} else {
-					drawString("Aucune sauvegarde dans le lecteur ...", 0);
+					drawString(otherMessages[5], 0);
 					waitPlayerInput();
 					checkDataDisk(-1);
 				}
@@ -1129,36 +984,39 @@ void makeSystemMenu(void) {
 
 				if (selectedSave >= 0) {
 					char saveFileName[256];
-					//makeTextEntryMenu("Veuillez entrer le Nom de la Sauvegarde .", &currentSaveName[selectedSave], 120);
-					sprintf(currentSaveName[selectedSave], "temporary save name");
+					char saveName[20];
+					saveName[0] = 0;
 
-					if (gameType == Cine::GID_FW)
-						sprintf(saveFileName, "FW.%1d", selectedSave);
-					else
-						sprintf(saveFileName, "OS.%1d", selectedSave);
+					if (!makeTextEntryMenu(otherMessages[6], saveName, 20, 120))
+						break;
+
+					strncpy(currentSaveName[selectedSave], saveName, 20);
+
+					sprintf(saveFileName, "%s.%1d", _targetName.c_str(), selectedSave);
 
 					getMouseData(mouseUpdateStatus, (uint16 *)&mouseButton, (uint16 *)&mouseX, (uint16 *)&mouseY);
 					if (!makeMenuChoice(confirmMenu, 2, mouseX, mouseY + 8, 100)) {
-						char saveString[256];
+						char saveString[256], tmp[80];
 
-						Common::OutSaveFile *fHandle;
-
-						if (gameType == Cine::GID_FW)
-							fHandle = g_saveFileMan->openForSaving("FW.DIR");
-						else
-							fHandle = g_saveFileMan->openForSaving("OS.DIR");
+						snprintf(tmp, 80, "%s.dir", _targetName.c_str());
 
+						Common::OutSaveFile *fHandle = g_saveFileMan->openForSaving(tmp);
+						if (!fHandle) {
+							warning("Unable to open file %s for saving", tmp);
+							break;
+						}
+						
 						fHandle->write(currentSaveName, 200);
 						delete fHandle;
 
-						sprintf(saveString, "Sauvegarde de |%s", currentSaveName[selectedSave]);
+						sprintf(saveString, otherMessages[3], currentSaveName[selectedSave]);
 						drawString(saveString, 0);
 
 						makeSave(saveFileName);
 
 						checkDataDisk(-1);
 					} else {
-						drawString("Sauvegarde Annule ...", 0);
+						drawString(otherMessages[4], 0);
 						waitPlayerInput();
 						checkDataDisk(-1);
 					}
@@ -1166,10 +1024,12 @@ void makeSystemMenu(void) {
 				break;
 			}
 		}
+
+		inMenu = false;
 	}
 }
 
-const int16 choiceResultTable[] = {
+static const int16 choiceResultTable[] = {
 	1,
 	1,
 	1,
@@ -1179,7 +1039,7 @@ const int16 choiceResultTable[] = {
 	1
 };
 
-const int16 subObjectUseTable[] = {
+static const int16 subObjectUseTable[] = {
 	3,
 	3,
 	3,
@@ -1189,7 +1049,7 @@ const int16 subObjectUseTable[] = {
 	0
 };
 
-const int16 canUseOnItemTable[] = {
+static const int16 canUseOnItemTable[] = {
 	1,
 	0,
 	0,
@@ -1285,7 +1145,7 @@ int16 buildObjectListCommand(void) {
 	int16 i;
 	int16 j;
 
-	assert(gameType == Cine::GID_FW);
+	assert(g_cine->getGameType() == Cine::GType_FW);
 
 	for (i = 0; i < 20; i++) {
 		objectListCommand[i][0] = 0;
@@ -1308,7 +1168,7 @@ int16 buildObjectListCommand2(int16 param) {
 	int16 i;
 	int16 j;
 
-	assert(gameType == Cine::GID_OS);
+	assert(g_cine->getGameType() == Cine::GType_OS);
 
 	for (i = 0; i < 20; i++) {
 		objectListCommand[i][0] = 0;
@@ -1383,7 +1243,7 @@ void makeCommandLine(void) {
 
 		getMouseData(mouseUpdateStatus, &dummyU16, &x, &y);
 
-		if (gameType == Cine::GID_FW) {
+		if (g_cine->getGameType() == Cine::GType_FW) {
 			si = selectSubObject(x, y + 8);
 		} else {
 			si = selectSubObject2(x, y + 8, -subObjectUseTable[playerCommand]);
@@ -1393,7 +1253,7 @@ void makeCommandLine(void) {
 			playerCommand = -1;
 			strcpy(commandBuffer, "");
 		} else {
-			if (gameType == Cine::GID_OS) {
+			if (g_cine->getGameType() == Cine::GType_OS) {
 				if (si >= 8000) {
 					si -= 8000;
 					canUseOnObject = canUseOnItemTable[playerCommand];
@@ -1420,7 +1280,7 @@ void makeCommandLine(void) {
 		}
 	}
 
-	if (gameType == Cine::GID_OS) {
+	if (g_cine->getGameType() == Cine::GType_OS) {
 		if (playerCommand != -1 && canUseOnObject != 0)	{ // call use on sub object
 			int16 si;
 
@@ -1455,7 +1315,7 @@ void makeCommandLine(void) {
 		}
 	}
 
-	if (allowSystemMenu == 0) {
+	if (!disableSystemMenu) {
 		isDrawCommandEnabled = 1;
 	}
 }
@@ -1485,7 +1345,7 @@ int16 makeMenuChoice(const commandeType commandList[], uint16 height, uint16 X,
 	int16 oldSelection;
 	int16 var_4;
 
-	if (allowSystemMenu)
+	if (disableSystemMenu)
 		return -1;
 
 	paramY = (height * 9) + 10;
@@ -1717,7 +1577,7 @@ int16 makeMenuChoice2(const commandeType commandList[], uint16 height, uint16 X,
 	int16 oldSelection;
 	int16 var_4;
 
-	if (allowSystemMenu)
+	if (disableSystemMenu)
 		return -1;
 
 	paramY = (height * 9) + 10;
@@ -1969,9 +1829,11 @@ void makeActionMenu(void) {
 	uint16 mouseX;
 	uint16 mouseY;
 
+	inMenu = true;
+
 	getMouseData(mouseUpdateStatus, &mouseButton, &mouseX, &mouseY);
 
-	if (gameType == Cine::GID_OS) {
+	if (g_cine->getGameType() == Cine::GType_OS) {
 		playerCommand = makeMenuChoice2(defaultActionCommand, 6, mouseX, mouseY, 70);
 
 		if (playerCommand >= 8000) {
@@ -1979,8 +1841,10 @@ void makeActionMenu(void) {
 			canUseOnObject = 1;
 		}
 	} else {
-		playerCommand =  makeMenuChoice(defaultActionCommand, 6, mouseX, mouseY, 70);
+		playerCommand = makeMenuChoice(defaultActionCommand, 6, mouseX, mouseY, 70);
 	}
+
+	inMenu = false;
 }
 
 uint16 executePlayerInput(void) {
@@ -1993,7 +1857,7 @@ uint16 executePlayerInput(void) {
 	canUseOnObject = 0;
 
 	if (isInPause) {
-		drawString("PAUSE", 0);
+		drawString(otherMessages[2], 0);
 		waitPlayerInput();
 		isInPause = 0;
 	}
@@ -2030,7 +1894,7 @@ uint16 executePlayerInput(void) {
 		if (playerCommand != -1) {
 			if (mouseButton & 1) {
 				if (mouseButton & 2) {
-					makeSystemMenu();
+					g_cine->makeSystemMenu();
 				} else {
 					int16 si;
 					do {
@@ -2079,7 +1943,7 @@ uint16 executePlayerInput(void) {
 			} else {
 				if (mouseButton & 2) {
 					if (mouseButton & 1) {
-						makeSystemMenu();
+						g_cine->makeSystemMenu();
 					}
 
 					makeActionMenu();
@@ -2109,7 +1973,7 @@ uint16 executePlayerInput(void) {
 		} else {
 			if (mouseButton & 2) {
 				if (!(mouseButton & 1)) {
-					if (gameType == Cine::GID_OS) {
+					if (g_cine->getGameType() == Cine::GType_OS) {
 						playerCommand = makeMenuChoice2(defaultActionCommand, 6, mouseX, mouseY, 70);
 
 						if (playerCommand >= 8000) {
@@ -2122,7 +1986,7 @@ uint16 executePlayerInput(void) {
 
 					makeCommandLine();
 				} else {
-					makeSystemMenu();
+					g_cine->makeSystemMenu();
 				}
 			} else {
 				if (mouseButton & 1) {
@@ -2150,7 +2014,7 @@ uint16 executePlayerInput(void) {
 							}
 						}
 					} else {
-						makeSystemMenu();
+						g_cine->makeSystemMenu();
 					}
 				}
 			}
@@ -2177,7 +2041,7 @@ uint16 executePlayerInput(void) {
 		}
 
 		if ((mouseButton & 1) && (mouseButton & 2)) {
-			makeSystemMenu();
+			g_cine->makeSystemMenu();
 		}
 	}
 
@@ -2291,10 +2155,10 @@ uint16 executePlayerInput(void) {
 			break;
 		case 9:
 		case 24:
-			makeSystemMenu();
+			g_cine->makeSystemMenu();
 			break;
 		default:
-			//  printf("Unhandled case %d in last part of executePLayerInput\n",var2-59);
+			//  printf("Unhandled case %d in last part of executePlayerInput\n",var2-59);
 			break;
 		}
 	}
@@ -2304,13 +2168,18 @@ uint16 executePlayerInput(void) {
 
 void drawSprite(overlayHeadElement *currentOverlay, byte *spritePtr,
 				byte *maskPtr, uint16 width, uint16 height, byte *page, int16 x, int16 y) {
-#if 0
 	byte *ptr = NULL;
+	byte *msk = NULL;
 	byte i = 0;
 	uint16 si = 0;
 	overlayHeadElement *pCurrentOverlay = currentOverlay;
 
-	while (pCurrentOverlay) { // unfinished, probably for mask handling..
+	if (g_cine->getGameType() == Cine::GType_OS) {
+		drawSpriteRaw2(spritePtr, objectTable[currentOverlay->objIdx].part, width, height, page, x, y);
+		return;
+	}
+
+	while (pCurrentOverlay) {
 		if (pCurrentOverlay->type == 5) {
 			int16 maskX;
 			int16 maskY;
@@ -2319,7 +2188,8 @@ void drawSprite(overlayHeadElement *currentOverlay, byte *spritePtr,
 			uint16 maskSpriteIdx;
 	 
 			if (!si) {
-				ptr = (byte *)malloc(width * height);
+				ptr = (byte *)malloc(width * 8 * height);
+				msk = (byte *)malloc(width * 8 * height);
 				si = 1;
 			}
 	 
@@ -2330,25 +2200,22 @@ void drawSprite(overlayHeadElement *currentOverlay, byte *spritePtr,
 	 
 			maskWidth = animDataTable[maskSpriteIdx].width / 2;
 			maskHeight = animDataTable[maskSpriteIdx].height;
-	 
-			gfxSpriteFunc2(spritePtr, width, height, animDataTable[maskSpriteIdx].ptr1, maskWidth, maskHeight, ptr, maskX - x,maskY - y, i++);
+			gfxSpriteFunc2(spritePtr, maskPtr, width, height, animDataTable[maskSpriteIdx].ptr1, maskWidth, maskHeight, ptr, msk, x, y, maskX, maskY, i++);
+#ifdef DEBUG_SPRITE_MASK
+			gfxFillSprite(animDataTable[maskSpriteIdx].ptr1, maskWidth, maskHeight, page, maskX, maskY, 1);
+#endif
 		}
 	 
 		pCurrentOverlay = pCurrentOverlay->next;
 	} 
 	 
 	if (si) {
-		gfxSpriteFunc1(ptr, width, height, page, x, y);
+		gfxSpriteFunc1(ptr, msk, width, height, page, x, y);
 		free(ptr);
-	} else
-#endif
-
-	if (gameType == Cine::GID_OS) {
-		drawSpriteRaw2(spritePtr, objectTable[currentOverlay->objIdx].part, width, height, page, x, y);
+		free(msk);
 	} else {
-		drawSpriteRaw(spritePtr, maskPtr, width, height, page, x, y);
+		gfxSpriteFunc1(spritePtr, maskPtr, width, height, page, x, y);
 	}
-
 }
 
 int16 additionalBgVScroll = 0;
@@ -2379,67 +2246,6 @@ void backupOverlayPage(void) {
 	}
 }
 
-uint16 computeMessageLength(const byte *ptr, uint16 width, uint16 *numWords, uint16 *messageWidth, uint16 *lineResult) {
-	const byte *localPtr = ptr;
-
-	uint16 var_2 = 0;
-	uint16 localLineResult = 0;
-	uint16 var_6 = 0;
-	uint16 var_8 = 0;
-	uint16 localMessageWidth = 0;
-	uint16 var_16 = 0;
-	uint16 finished = 0;
-	uint16 si = 0;
-	uint16 di = 0;
-
-	while (!finished) {
-		byte character = *(localPtr++);
-
-		if (character == ' ') {
-			var_8 = var_16;
-			var_6 = localMessageWidth;
-			localLineResult = si;
-			var_2 = di;
-
-			if (si + 5 < width) {
-				var_16++;
-				si += 5;
-			} else {
-				finished = 1;
-			}
-		} else if (character == 0x7C || character == 0) {
-			finished = 1;
-			si = 0;
-		} else {
-			if (fontParamTable[character].characterWidth) {
-				uint16 var_C = fontParamTable[character].characterWidth + 1;
-
-				if (si + var_C < width) {
-					si += var_C;
-					localMessageWidth += var_C;
-				} else {
-					finished = 1;
-
-					if (localLineResult) {
-						var_16 = var_8;
-						localMessageWidth = var_6;
-						si = localLineResult;
-						di = var_2;
-					}
-				}
-			}
-		}
-
-		di++;
-	}
-
-	*numWords = var_16;
-	*messageWidth = localMessageWidth;
-	*lineResult = si;
-
-	return di;
-}
-
 void drawDialogueMessage(byte msgIdx, int16 x, int16 y, int16 width, int16 color) {
 	byte color2 = 2;
 	byte endOfMessageReached = 0;
@@ -2588,7 +2394,7 @@ void drawFailureMessage(byte cmd) {
 
 		messagePtr += messageLength;
 
-		messageLength = computeMessageLength((const byte *) messagePtr, localWidth, &numWords, &messageWidth, &lineResult);
+		messageLength = computeMessageLength((const byte *)messagePtr, localWidth, &numWords, &messageWidth, &lineResult);
 
 		endOfMessagePtr = messagePtr + messageLength;
 
@@ -2676,7 +2482,7 @@ void drawOverlays(void) {
 				y = objPtr->y;
 
 				if (objPtr->frame >= 0) {
-					if (gameType == Cine::GID_OS) {
+					if (g_cine->getGameType() == Cine::GType_OS) {
 						uint16 partVar1;
 						uint16 partVar2;
 						AnimData *pPart;
@@ -2827,10 +2633,6 @@ void drawOverlays(void) {
 	}
 }
 
-void flip(void) {
-	blitRawScreen(page1Raw);
-}
-
 uint16 processKeyboard(uint16 param) {
 	return 0;
 }
@@ -2884,9 +2686,6 @@ uint16 exitEngine;
 void hideMouse(void) {
 }
 
-void closeEngine7(void) {
-}
-
 void removeExtention(char *dest, const char *source) {
 	byte *ptr;
 
@@ -2899,8 +2698,6 @@ void removeExtention(char *dest, const char *source) {
 	}
 }
 
-uint16 var22;
-
 uint16 defaultMenuBoxColor2;
 
 uint16 zoneData[NUM_MAX_ZONE];
@@ -2917,7 +2714,7 @@ void addMessage(byte param1, int16 param2, int16 param3, int16 param4, int16 par
 		currentHead = tempHead->next;
 	}
 
-	newElement = (overlayHeadElement *) malloc(sizeof(overlayHeadElement));
+	newElement = (overlayHeadElement *)malloc(sizeof(overlayHeadElement));
 
 	newElement->next = tempHead->next;
 	tempHead->next = newElement;
@@ -3161,7 +2958,13 @@ void processSeqListElement(SeqListElement *element) {
 		param1 = ptr1[1];
 		param2 = ptr1[2];
 
-		assert(element->varC == 255);
+		if (element->varC != 255) {
+			// FIXME: Why is this here? Fingolfin gets lots of these
+			// in his copy of Operation Stealth (value 0 or 236) under
+			// Mac OS X. Maybe it's a endian issue? At least the graphics
+			// in the copy protection screen are partially messed up.
+			warning("processSeqListElement: varC = %d", element->varC);
+		}
 
 		if (globalVars[VAR_MOUSE_X_POS] || globalVars[VAR_MOUSE_Y_POS]) {
 			computeMove1(element, ptr1[4] + x, ptr1[5] + y, param1, param2, globalVars[VAR_MOUSE_X_POS], globalVars[VAR_MOUSE_Y_POS]);
@@ -3239,4 +3042,242 @@ void processSeqList(void) {
 	}
 }
 
+
+bool makeTextEntryMenu(const char *messagePtr, char *inputString, int stringMaxLength, int y) {
+	byte color2 = defaultMenuBoxColor2;
+	byte endOfMessageReached = 0;
+	int16 localX;
+	int16 localY;
+	int16 localWidth;
+	int margins = 16;
+
+	int len = strlen(messagePtr);
+
+	int16 width = 6 * len + 20;
+
+	if (width > 250)
+		width = 250;
+	
+	if (width < 180)
+		width = 180;
+	
+	int16 x = (320 - width) / 2;
+	int16 color = 2;
+
+	gfxDrawPlainBoxRaw(x - margins, y, x + width + margins, y + 4, color2, page1Raw);
+
+	localX = x + 4;
+	localY = y + 4;
+	localWidth = width;
+
+	getKeyData(); // clear input key
+
+	do {
+		uint16 messageLength = 0;
+		uint16 numWords;
+		uint16 messageWidth;
+		uint16 lineResult;
+		const char *endOfMessagePtr;
+		uint16 fullLineWidth;
+		uint16 interWordSize;
+		uint16 interWordSizeRemain;
+		byte currentChar;
+		byte characterWidth;
+
+		while (messagePtr[messageLength] == ' ') {
+			messageLength++;
+		}
+
+		messagePtr += messageLength;
+
+		messageLength = computeMessageLength((const byte *)messagePtr, localWidth, &numWords, &messageWidth, &lineResult);
+
+		endOfMessagePtr = messagePtr + messageLength;
+
+		if (lineResult) {
+			fullLineWidth = localWidth - messageWidth;
+
+			if (numWords) {
+				interWordSize = fullLineWidth / numWords;
+				interWordSizeRemain = fullLineWidth % numWords;
+			} else {
+				interWordSize = 5;
+				interWordSizeRemain = 0;
+			}
+		} else {
+			interWordSize = 5;
+			interWordSizeRemain = 0;
+		}
+
+		gfxDrawPlainBoxRaw(x - margins, localY, x + width + margins, localY + 9, color2, page1Raw);
+
+		do {
+			currentChar = *(messagePtr++);
+
+			if (currentChar == 0) {
+				endOfMessageReached = 1;
+			} else if (currentChar == ' ') {
+				localX += interWordSizeRemain + interWordSize;
+
+				if (interWordSizeRemain)
+					interWordSizeRemain = 0;
+			} else {
+				characterWidth = fontParamTable[currentChar].characterWidth;
+
+				if (characterWidth) {
+					byte characterIdx = fontParamTable[currentChar].characterIdx;
+					drawSpriteRaw(textTable[characterIdx][0], textTable[characterIdx][1], 2, 8, page1Raw, localX, localY);
+					localX += characterWidth + 1;
+				}
+			}
+		} while ((messagePtr < endOfMessagePtr) && !endOfMessageReached);
+
+		localX = x + 4;
+		localY += 9;
+	} while (!endOfMessageReached);
+
+	// Input string
+	gfxDrawPlainBoxRaw(x - margins, localY, x + width + margins, localY + 9, color2, page1Raw);
+	localY += 9;
+
+	x -= margins;
+	width += margins * 2;
+
+	gfxDrawPlainBoxRaw(x, localY, x + width, localY + 4, color2, page1Raw);
+
+	gfxDrawLine(x + 1, y + 1, x + width - 1, y + 1, 0, page1Raw);	// top
+	gfxDrawLine(x + 1, localY + 3, x + width - 1, localY + 3, 0, page1Raw);	// bottom
+	gfxDrawLine(x + 1, y + 1, x + 1, localY + 3, 0, page1Raw);	// left
+	gfxDrawLine(x + width - 1, y + 1, x + width - 1, localY + 3, 0, page1Raw);	// right
+
+	gfxDrawLine(x, y, x + width, y, color, page1Raw);
+	gfxDrawLine(x, localY + 4, x + width, localY + 4, color, page1Raw);
+	gfxDrawLine(x, y, x, localY + 4, color, page1Raw);
+	gfxDrawLine(x + width, y, x + width, localY + 4, color, page1Raw);
+
+	x += margins;
+	width -= margins * 2;
+	localY -= 9;
+
+
+	int quit = 0;
+	bool redraw = true;
+	commandeType tempString;
+	int inputLength = strlen(inputString);
+	int inputPos = inputLength + 1;
+
+	while (!quit) {
+		if (redraw) {
+			gfxDrawPlainBoxRaw(x, localY - 1, x + width, localY + 8, 0, page1Raw);
+
+			int currentX = x + 4;
+
+			for (uint j = 0; j < strlen(inputString); j++) {
+				char currentChar = inputString[j];
+
+				if (currentChar == ' ') {
+					currentX += 5;
+				} else {
+					byte characterWidth = fontParamTable[currentChar].characterWidth;
+
+					if (characterWidth) {
+						byte characterIdx = fontParamTable[currentChar].characterIdx;
+						drawSpriteRaw(textTable[characterIdx][0], textTable[characterIdx][1], 2, 8, page1Raw, currentX, localY);
+						currentX += characterWidth + 1;
+					}
+				}
+				// draw cursor here
+				if (inputPos == (int)(j + 2))
+					gfxDrawLine(currentX, localY - 1, currentX, localY + 8, color, page1Raw);
+
+			}
+
+			if (strlen(inputString) == 0 || inputPos == 1) // cursor wasn't yet drawn
+				gfxDrawLine(x + 4, localY - 1, x + 4, localY + 8, color, page1Raw);
+
+			blitRawScreen(page1Raw);
+			redraw = false;
+		}
+
+		char ch[2];
+		memset(tempString, 0, stringMaxLength);
+		ch[1] = 0;
+
+		manageEvents();
+		int ascii = getKeyData();
+		uint16 mouseButton, mouseX, mouseY;
+
+		getMouseData(0, &mouseButton, &mouseX, &mouseY);
+
+		if (mouseButton & 2)
+			quit = 2;
+		else if (mouseButton & 1)
+			quit = 1;
+
+		switch (ascii) {
+		case 8: // backspace
+			if (inputPos <= 1) {
+				break;
+			}
+			inputPos--;
+			redraw = true;
+		case 127: // del
+			if (inputPos <= inputLength) {
+				if (inputPos != 1) {
+					strncpy(tempString, inputString, inputPos - 1);
+				}
+				if (inputPos != inputLength) {
+					strncat(tempString, &inputString[inputPos], inputLength - inputPos);
+				}
+				strcpy(inputString, tempString);
+				inputLength = strlen(inputString);
+				redraw = true;
+			}
+			break;
+		case 276: // left
+			if (inputPos > 1) {
+				inputPos--;
+				redraw = true;
+			}
+			break;
+		case 275: // right
+			if (inputPos <= inputLength) {
+				inputPos++;
+				redraw = true;
+			}
+			break;
+		default:
+			if (((ascii >= 'a') && (ascii <='z')) ||
+				((ascii >= '0') && (ascii <='9')) ||
+				((ascii >= 'A') && (ascii <='Z')) ||
+				(ascii == ' ')) {
+				if (inputLength < stringMaxLength - 1) {
+					ch[0] = ascii;
+					if (inputPos != 1) {
+						strncpy(tempString, inputString, inputPos - 1);
+						strcat(tempString, ch);
+					}
+					if ((inputLength == 0) || (inputPos == 1)) {
+						strcpy(tempString, ch);
+					}
+					if ((inputLength != 0) && (inputPos != inputLength)) {
+						strncat(tempString, &inputString[inputPos - 1], inputLength - inputPos + 1);
+					}
+					
+					strcpy(inputString, tempString);
+					inputLength = strlen(inputString);
+					inputPos++;
+					redraw = true;
+				}
+			}
+			break;
+		}
+	}
+
+	if (quit == 2)
+		return false;
+
+	return true;
+}
+
 } // End of namespace Cine
