@@ -1,7 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
  *
- * cinE Engine is (C) 2004-2005 by CinE Team
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -17,13 +18,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/cine/various.h $
- * $Id: various.h 22651 2006-05-26 09:36:41Z eriktorbjorn $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/cine/various.h $
+ * $Id: various.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
-#ifndef CINE_VARIOUS_H_
-#define CINE_VARIOUS_H_
+#ifndef CINE_VARIOUS_H
+#define CINE_VARIOUS_H
 
 #include "common/stdafx.h"
 #include "common/file.h"
@@ -32,19 +33,15 @@
 
 namespace Cine {
 
-extern int gameType;
-
-typedef char commandeType[20];
-
 void initLanguage(Common::Language lang);
 
 int16 makeMenuChoice(const commandeType commandList[], uint16 height, uint16 X, uint16 Y, uint16 width);
 int16 makeMenuChoice2(const commandeType commandList[], uint16 height, uint16 X, uint16 Y, uint16 width);
 void makeCommandLine(void);
-void makeSystemMenu(void);
 void makeActionMenu(void);
 
-extern int16 allowSystemMenu;
+extern bool disableSystemMenu;
+extern bool inMenu; 
 
 struct unk1Struct {
 	byte *ptr;
@@ -77,31 +74,18 @@ extern SeqListElement seqList;
 
 extern uint32 var6;
 extern uint32 var8;
-extern byte *var9;
 
 extern uint16 var2;
 extern uint16 var3;
 extern uint16 var4;
 extern uint16 var5;
 
-extern Common::File palFileHandle;
-extern Common::File partFileHandle;
-
-void freeAnimDataTable(void);
 void mainLoopSub1(void);
 void setTextWindow(uint16 param1, uint16 param2, uint16 param3, uint16 param4);
 
 extern uint16 errorVar;
 extern byte menuVar;
 
-void gfxFuncGen1(byte *param1, byte *param2, byte *param3, byte *param4, int16 param5);
-
-extern byte *page0;
-extern byte *page0c;
-
-void ptrGfxFunc13(void);
-void gfxFuncGen2(void);
-
 extern uint16 allowPlayerInput;
 
 extern uint16 checkForPendingDataLoadSwitch;
@@ -138,12 +122,12 @@ void mainLoopSub3(void);
 uint16 executePlayerInput(void);
 
 void drawOverlays(void);
-void flip(void);
 
 extern uint16 mouseUpdateStatus;
 extern uint16 dummyU16;
 
 void getMouseData(uint16 param, uint16 *pButton, uint16 *pX, uint16 *pY);
+int getKeyData(void);
 
 uint16 processKeyboard(uint16 param);
 
@@ -154,9 +138,6 @@ void checkForPendingDataLoad(void);
 extern uint16 exitEngine;
 
 void hideMouse(void);
-void closeEngine7(void);
-
-extern uint16 var22;
 
 void removeExtention(char *dest, const char *source);
 
@@ -173,21 +154,14 @@ extern uint16 zoneData[NUM_MAX_ZONE];
 
 void addMessage(byte param1, int16 param2, int16 param3, int16 param4, int16 param5);
 
-void blitScreen(byte *frontBuffer, byte *backbuffer);
-
-struct mouseStatusStruct {
-	int left;
-	int right;
-	int X;
-	int Y;
-};
-
 extern int16 additionalBgVScroll;
 
 void addSeqListElement(int16 param0, int16 param1, int16 param2, int16 param3, int16 param4, int16 param5, int16 param6, int16 param7, int16 param8);
 void resetSeqList();
 void processSeqList(void);
 
+bool makeTextEntryMenu(const char *caption, char *string, int strLen, int y);
+
 } // End of namespace Cine
 
 #endif
