@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2005-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,26 +18,27 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/gob/cdrom.cpp $
- * $Id: cdrom.cpp 21500 2006-03-29 15:59:37Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/gob/cdrom.cpp $
+ * $Id: cdrom.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
 #include "common/stdafx.h"
 #include "common/endian.h"
+#include "sound/audiocd.h"
 
 #include "gob/gob.h"
 #include "gob/cdrom.h"
-#include "gob/dataio.h"
-#include "gob/game.h"
 #include "gob/global.h"
 #include "gob/util.h"
-#include "sound/audiocd.h"
+#include "gob/dataio.h"
+#include "gob/game.h"
 
 namespace Gob {
 
 CDROM::CDROM(GobEngine *vm) : _vm(vm) {
 	_cdPlaying = false;
+
 	_LICbuffer = 0;
 	for (int i = 0; i < 16; i++)
 		_curTrack[i] = 0;
@@ -52,52 +56,44 @@ void CDROM::readLIC(const char *fname) {
 
 	*_curTrack = 0;
 
-	strcpy(tmp, fname);
+	strncpy0(tmp, fname, 79);
 
-	handle = _vm->_dataio->openData(tmp);
+	handle = _vm->_dataIO->openData(tmp);
 
 	if (handle == -1)
 		return;
 
-	_vm->_dataio->closeData(handle);
+	_vm->_dataIO->closeData(handle);
 
-	_vm->_dataio->getUnpackedData(tmp);
+	_vm->_dataIO->getUnpackedData(tmp);
 
-	handle = _vm->_dataio->openData(tmp);
+	handle = _vm->_dataIO->openData(tmp);
 
-	_vm->_dataio->readData(handle, (char *)&version, 2);
-	version = READ_LE_UINT16(&version);
+	version = _vm->_dataIO->readUint16(handle);
+	startChunk = _vm->_dataIO->readUint16(handle);
+	_numTracks = _vm->_dataIO->readUint16(handle);
 
-	_vm->_dataio->readData(handle, (char *)&startChunk, 2);
-	startChunk = READ_LE_UINT16(&startChunk);
+	if (version != 3)
+		error("%s: Unknown version %d", fname, version);
 
-	_vm->_dataio->readData(handle, (char *)&_numTracks, 2);
-	_numTracks = READ_LE_UINT16(&_numTracks);
-
-	if (version != 3) {
-		error("Wrong file %s (%d)", fname, version);
-		return;
-	}
-
-	_vm->_dataio->seekData(handle, 50, SEEK_SET);
+	_vm->_dataIO->seekData(handle, 50, SEEK_SET);
 
 	for (int i = 0; i < startChunk; i++) {
-		_vm->_dataio->readData(handle, (char *)&pos, 2);
-		pos = READ_LE_UINT16(&pos);
+		pos = _vm->_dataIO->readUint16(handle);
 
 		if (!pos)
 			break;
 
-		_vm->_dataio->seekData(handle, pos, SEEK_CUR);
+		_vm->_dataIO->seekData(handle, pos, SEEK_CUR);
 	}
 
 	_LICbuffer = new byte[_numTracks * 22];
-	_vm->_dataio->readData(handle, (char *)_LICbuffer, _numTracks * 22);
+	_vm->_dataIO->readData(handle, _LICbuffer, _numTracks * 22);
 
-	_vm->_dataio->closeData(handle);
+	_vm->_dataIO->closeData(handle);
 }
 
-void CDROM::freeLICbuffer(void) {
+void CDROM::freeLICbuffer() {
 	delete[] _LICbuffer;
 	_LICbuffer = 0;
 }
@@ -145,10 +141,11 @@ void CDROM::playMultMusic() {
 		{"avt022.tot", "fra6", "all6", "ang6", "esp6", "ita6"}
 	};
 
+	// Default to "ang?" for other languages (including EN_USA)
+	int language = _vm->_global->_language <= 4 ? _vm->_global->_language : 2;
 	for (int i = 0; i < ARRAYSIZE(tracks); i++)
 		if (!scumm_stricmp(_vm->_game->_curTotFile, tracks[i][0])) {
-			_cdPlaying = true;
-			startTrack(tracks[i][_vm->_global->_language + 1]);
+			startTrack(tracks[i][language + 1]);
 			break;
 		}
 }
@@ -159,7 +156,7 @@ void CDROM::startTrack(const char *trackname) {
 	if (!_LICbuffer)
 		return;
 
-	debugC(1, DEBUG_MUSIC, "startTrack(%s)", trackname);
+	debugC(1, kDebugMusic, "CDROM::startTrack(%s)", trackname);
 
 	matchPtr = 0;
 	curPtr = _LICbuffer;
@@ -173,11 +170,11 @@ void CDROM::startTrack(const char *trackname) {
 	}
 
 	if (!matchPtr) {
-		error("Track %s not found", trackname);
+		warning("Track \"%s\" not found", trackname);
 		return;
 	}
 
-	strcpy(_curTrack, trackname);
+	strncpy0(_curTrack, trackname, 15);
 
 	stopPlaying();
 
@@ -201,30 +198,36 @@ void CDROM::play(uint32 from, uint32 to) {
 	// HSG encodes frame information into a double word:
 	// minute multiplied by 4500, plus second multiplied by 75,
 	// plus frame, minus 150
-	debugC(1, DEBUG_MUSIC, "play(%d, %d)", from, to);
+	debugC(1, kDebugMusic, "CDROM::play(%d, %d)", from, to);
 
-	AudioCD.play(1, 0, from, to - from + 1);
+	AudioCD.play(1, 1, from, to - from + 1);
+	_cdPlaying = true;
 }
 
-int32 CDROM::getTrackPos(void) {
+int32 CDROM::getTrackPos() {
 	uint32 curPos = _vm->_util->getTimeKey() - _startTime;
 
-	if (AudioCD.isPlaying() && (_vm->_util->getTimeKey() < _trackStop))
+	if (_cdPlaying && (_vm->_util->getTimeKey() < _trackStop))
 		return curPos * 3 / 40;
 	else
 		return -1;
 }
 
-void CDROM::stopPlaying(void) {
+const char *CDROM::getCurTrack() {
+	return _curTrack;
+}
+
+void CDROM::stopPlaying() {
 	stop();
 
 	while (getTrackPos() != -1);
 }
 
-void CDROM::stop(void) {
-	debugC(1, DEBUG_MUSIC, "stop()");
+void CDROM::stop() {
+	debugC(1, kDebugMusic, "CDROM::stop()");
 
 	AudioCD.stop();
+	_cdPlaying = false;
 }
 
 void CDROM::testCD(int trySubst, const char *label) {
