@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2005-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,11 +18,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/gob/cdrom.h $
- * $Id: cdrom.h 20520 2006-02-11 10:11:37Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/gob/cdrom.h $
+ * $Id: cdrom.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
-#include "gob/gob.h"
+
+#ifndef GOB_CDROM_H
+#define GOB_CDROM_H
 
 namespace Gob {
 
@@ -28,15 +33,16 @@ public:
 	bool _cdPlaying;
 
 	void readLIC(const char *fname);
-	void freeLICbuffer(void);
+	void freeLICbuffer();
 
 	void startTrack(const char *s);
 	void playBgMusic();
 	void playMultMusic();
 	void play(uint32 from, uint32 to);
-	int32 getTrackPos(void);
-	void stopPlaying(void);
-	void stop(void);
+	int32 getTrackPos();
+	const char *getCurTrack();
+	void stopPlaying();
+	void stop();
 	void testCD(int trySubst, const char *label);
 
 	CDROM(GobEngine *vm);
@@ -51,3 +57,5 @@ protected:
 };
 
 } // End of namespace Gob
+
+#endif // GOB_CDROM_H
