@@ -1,6 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2004 Ivan Dubrov
- * Copyright (C) 2004-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -16,10 +18,11 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/gob/draw.h $
- * $Id: draw.h 23012 2006-06-10 14:37:48Z drmccoy $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/gob/draw.h $
+ * $Id: draw.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
+
 #ifndef GOB_DRAW_H
 #define GOB_DRAW_H
 
@@ -27,10 +30,15 @@
 
 namespace Gob {
 
-#define RENDERFLAG_NOINVALIDATE	1
-#define RENDERFLAG_CAPTUREPUSH	2
-#define RENDERFLAG_CAPTUREPOP	8
-#define RENDERFLAG_USEDELTAS 	0x10
+#define SPRITES_COUNT 50
+
+#define RENDERFLAG_NOINVALIDATE      0x001
+#define RENDERFLAG_CAPTUREPUSH       0x002
+#define RENDERFLAG_COLLISIONS        0x004
+#define RENDERFLAG_CAPTUREPOP        0x008
+#define RENDERFLAG_USEDELTAS         0x010
+#define RENDERFLAG_NOBLITINVALIDATED 0x200
+#define RENDERFLAG_SKIPOPTIONALTEXT  0x400
 
 class Draw {
 public:
@@ -39,9 +47,11 @@ public:
 		int8 base;
 		int8 width;
 		int8 height;
-		FontToSprite() : sprite(0), base(0), width(0), height() {}
+		FontToSprite() : sprite(0), base(0), width(0), height(0) {}
 	};
 
+	int16 _renderFlags;
+
 	int16 _fontIndex;
 	int16 _spriteLeft;
 	int16 _spriteTop;
@@ -51,19 +61,21 @@ public:
 	int16 _destSpriteY;
 	int16 _backColor;
 	int16 _frontColor;
-	char _letterToPrint;
-	FontToSprite _fontToSprite[4];
-	int16 _destSurface;
+	int16 _transparency;
+
 	int16 _sourceSurface;
-	int16 _renderFlags;
+	int16 _destSurface;
+
+	char _letterToPrint;
+	char *_textToPrint;
+
 	int16 _backDeltaX;
 	int16 _backDeltaY;
+
+	FontToSprite _fontToSprite[4];
 	Video::FontDesc *_fonts[8];
-	char *_textToPrint;
-	int16 _transparency;
-	Video::SurfaceDesc *_spritesArray[50];
-	Video::SurfaceDesc *_bigSpritesParts[50][3];
-	uint16 _spritesHeights[50];
+
+	SurfaceDesc::Ptr _spritesArray[SPRITES_COUNT];
 
 	int16 _invalidatedCount;
 	int16 _invalidatedTops[30];
@@ -71,73 +83,83 @@ public:
 	int16 _invalidatedRights[30];
 	int16 _invalidatedBottoms[30];
 
-	int8 _noInvalidated;
-//	int8 doFullFlip; // Never used?!?
-	int8 _paletteCleared;
+	bool _noInvalidated;
+	// Don't blit invalidated rects when in video mode 5 or 7
+	bool _noInvalidated57;
+	bool _paletteCleared;
+	bool _applyPal;
 
-	int16 _cursorIndex;
-	int16 _transparentCursor;
-	uint32 _cursorTimeKey;
-
-	Video::SurfaceDesc *_backSurface;
-	Video::SurfaceDesc *_frontSurface;
+	SurfaceDesc::Ptr _backSurface;
+	SurfaceDesc::Ptr _frontSurface;
 
 	int16 _unusedPalette1[18];
 	int16 _unusedPalette2[16];
 	Video::Color _vgaPalette[256];
 	Video::Color _vgaSmallPalette[16];
 
+	// 0 (00b): No cursor
+	// 1 (01b): Cursor would be on _backSurface
+	// 2 (10b): Cursor would be on _frontSurface
+	// 3 (11b): Cursor would be on _backSurface and _frontSurface
+	uint8 _showCursor;
+	int16 _cursorIndex;
+	int16 _transparentCursor;
+	uint32 _cursorTimeKey;
+
 	int16 _cursorX;
 	int16 _cursorY;
 	int16 _cursorWidth;
 	int16 _cursorHeight;
 
-	int16 _cursorXDeltaVar;
-	int16 _cursorYDeltaVar;
+	int16 _cursorHotspotXVar;
+	int16 _cursorHotspotYVar;
+
+	SurfaceDesc::Ptr _cursorSprites;
+	SurfaceDesc::Ptr _cursorSpritesBack;
+	SurfaceDesc::Ptr _scummvmCursor;
 
-	Video::SurfaceDesc *_cursorSprites;
-	Video::SurfaceDesc *_cursorSpritesBack;
-	Video::SurfaceDesc *_cursorBack;
 	int16 _cursorAnim;
 	int8 _cursorAnimLow[40];
 	int8 _cursorAnimHigh[40];
 	int8 _cursorAnimDelays[40];
-	int8 _applyPal;
 
 	int16 _palLoadData1[4];
 	int16 _palLoadData2[4];
-		
-	int16 _word_2E8E2;
+
+	int16 _needAdjust;
+	int16 _scrollOffsetY;
+	int16 _scrollOffsetX;
 
 	void invalidateRect(int16 left, int16 top, int16 right, int16 bottom);
-	void blitInvalidated(void);
-	void setPalette(void);
-	void clearPalette(void);
-	void blitCursor(void);
+	void blitInvalidated();
+	void setPalette();
+	void clearPalette();
 
-	void animateCursor(int16 cursor);
-
-	void freeSprite(int16 index);
+	void initSpriteSurf(int16 index, int16 width, int16 height, int16 flags);
+	void freeSprite(int16 index) {
+		assert(index < SPRITES_COUNT);
+		_spritesArray[index] = 0;
+	}
 	void adjustCoords(char adjust, int16 *coord1, int16 *coord2);
-	void fillRect(int16 index, int16 left, int16 top, int16 right,
-		int16 bottom, int16 color);
-	void drawSprite(int16 source, int16 dest, int16 left,
-			int16 top, int16 right, int16 bottom, int16 x, int16 y, int16 transp);
-	void drawSprite(Video::SurfaceDesc * source, int16 dest, int16 left,
-			int16 top, int16 right, int16 bottom, int16 x, int16 y, int16 transp);
-	void drawSprite(int16 source, Video::SurfaceDesc * dest, int16 left,
-			int16 top, int16 right, int16 bottom, int16 x, int16 y, int16 transp);
+	void adjustCoords(char adjust, uint16 *coord1, uint16 *coord2) {
+		adjustCoords(adjust, (int16 *) coord1, (int16 *) coord2);
+	}
 	void drawString(char *str, int16 x, int16 y, int16 color1, int16 color2,
-		int16 transp, Video::SurfaceDesc *dest, Video::FontDesc *font);
-	void printTextCentered(int16 arg_0, int16 left, int16 top, int16 right,
+			int16 transp, SurfaceDesc *dest, Video::FontDesc *font);
+	void printTextCentered(int16 id, int16 left, int16 top, int16 right,
 			int16 bottom, char *str, int16 fontIndex, int16 color);
+	int32 getSpriteRectSize(int16 index);
+	void forceBlit(bool backwards = false);
 
-	virtual void initBigSprite(int16 index, int16 width, int16 height, int16 flags) = 0;
-	virtual void printText(void) = 0;
+	virtual void initScreen() = 0;
+	virtual void closeScreen() = 0;
+	virtual void blitCursor() = 0;
+	virtual void animateCursor(int16 cursor) = 0;
+	virtual void printTotText(int16 id) = 0;
 	virtual void spriteOperation(int16 operation) = 0;
 
 	Draw(GobEngine *vm);
-	virtual ~Draw() {};
+	virtual ~Draw() {}
 
 protected:
 	GobEngine *_vm;
@@ -145,22 +167,36 @@ protected:
 
 class Draw_v1 : public Draw {
 public:
-	virtual void initBigSprite(int16 index, int16 width, int16 height, int16 flags);
-	virtual void printText(void);
+	virtual void initScreen();
+	virtual void closeScreen();
+	virtual void blitCursor();
+	virtual void animateCursor(int16 cursor);
+	virtual void printTotText(int16 id);
 	virtual void spriteOperation(int16 operation);
 
 	Draw_v1(GobEngine *vm);
-	virtual ~Draw_v1() {};
+	virtual ~Draw_v1() {}
 };
 
 class Draw_v2 : public Draw_v1 {
 public:
-	virtual void initBigSprite(int16 index, int16 width, int16 height, int16 flags);
-	virtual void printText(void);
+	virtual void initScreen();
+	virtual void closeScreen();
+	virtual void blitCursor();
+	virtual void animateCursor(int16 cursor);
+	virtual void printTotText(int16 id);
 	virtual void spriteOperation(int16 operation);
 
 	Draw_v2(GobEngine *vm);
-	virtual ~Draw_v2() {};
+	virtual ~Draw_v2() {}
+};
+
+class Draw_Bargon: public Draw_v2 {
+public:
+	virtual void initScreen();
+
+	Draw_Bargon(GobEngine *vm);
+	virtual ~Draw_Bargon() {}
 };
 
 // Draw operations
@@ -177,6 +213,6 @@ public:
 #define DRAW_FILLRECTABS 9
 #define DRAW_DRAWLETTER	10
 
-}				// End of namespace Gob
+} // End of namespace Gob
 
-#endif	/* __DRAW_H */
+#endif // GOB_DRAW_H
