@@ -1,6 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2004 Ivan Dubrov
- * Copyright (C) 2004-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -16,14 +18,14 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/gob/goblin.h $
- * $Id: goblin.h 22979 2006-06-07 18:49:20Z drmccoy $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/gob/goblin.h $
+ * $Id: goblin.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
+
 #ifndef GOB_GOBLIN_H
 #define GOB_GOBLIN_H
 
-#include "gob/gob.h"
 #include "gob/util.h"
 #include "gob/sound.h"
 #include "gob/mult.h"
@@ -37,72 +39,74 @@ namespace Gob {
 class Goblin {
 
 public:
-#pragma START_PACK_STRUCTS
+
+#include "common/pack-start.h"	// START STRUCT PACKING
+
 	struct Gob_State {
-		int16 animation;// +0h
-		int16 layer;	// +2h
-		int16 unk0;		// +4h
-		int16 unk1;		// +6h
-		int16 sndItem;	// +8h, high/low byte - sound sample index
-		int16 freq;		// +Ah, high/low byte * 100 - frequency
-		int16 repCount;	// +Ch high/low byte - repeat count
-		int16 sndFrame;		// +Eh
-	} GCC_PACK;
+		int16 animation;
+		int16 layer;
+		int16 unk0;
+		int16 unk1;
+		int16 sndItem;  // high/low byte - sound sample index
+		int16 freq;     // high/low byte * 100 - frequency
+		int16 repCount;	// high/low byte - repeat count
+		int16 sndFrame;
+	};
 
 	typedef Gob_State *Gob_PState;
 
 	typedef Gob_PState Gob_StateLine[6];
 
 	struct Gob_Object {
-		int16 animation;	// +0h
-		int16 state;		// +2h
-		int16 stateColumn;	// +4h
-		int16 curFrame;		// +6h
-		int16 xPos;			// +8h
-		int16 yPos;			// +Ah
-		int16 dirtyLeft;	// +Ch
-		int16 dirtyTop;		// +Eh
-		int16 dirtyRight;	// +10h
-		int16 dirtyBottom;	// +12h
-		int16 left;			// +14h
-		int16 top;			// +16h
-		int16 right;		// +18h
-		int16 bottom;		// +1ah
-		int16 nextState;	// +1ch
-		int16 multState;	// +1eh
-		int16 actionStartState;	// +20h
-		int16 curLookDir;	// +22h
-		int16 pickable;		// +24h
-		int16 relaxTime;	// +26h
-		Gob_StateLine *stateMach;	// +28h
-		Gob_StateLine *realStateMach;	// +2ch
-		char doAnim;		// +30h
-		char order;			// +31h
-		char noTick;		// +32h
-		char toRedraw;		// +33h
-		char type;			// +34h
-		char maxTick;		// +35h
-		char tick;			// +36h
-		char multObjIndex;	// +37h, from which play mult animations
-		char unk14;			// +38h
-		char visible;		// +39h
-	} GCC_PACK;
+		int16 animation;
+		int16 state;
+		int16 stateColumn;
+		int16 curFrame;
+		int16 xPos;
+		int16 yPos;
+		int16 dirtyLeft;
+		int16 dirtyTop;
+		int16 dirtyRight;
+		int16 dirtyBottom;
+		int16 left;
+		int16 top;
+		int16 right;
+		int16 bottom;
+		int16 nextState;
+		int16 multState;
+		int16 actionStartState;
+		int16 curLookDir;
+		int16 pickable;
+		int16 relaxTime;
+		Gob_StateLine *stateMach;
+		Gob_StateLine *realStateMach;
+		char doAnim;
+		int8 order;
+		char noTick;
+		char toRedraw;
+		char type;
+		char maxTick;
+		char tick;
+		char multObjIndex;
+		char unk14;
+		char visible;
+	};
 
 	struct Gob_Pos {
 		char x;
 		char y;
-	} GCC_PACK;
-#pragma END_PACK_STRUCTS
+	};
+
+#include "common/pack-end.h"	// END STRUCT PACKING
 
-	Util::List *_objList;
 	Gob_Object *_goblins[4];
 	int16 _currentGoblin;
-	Snd::SoundDesc *_soundData[16];
+	SoundDesc _soundData[16];
 	int16 _gobStateLayer;
 	char _goesAtTarget;
 	char _readyToAct;
 	int16 _gobAction;	// 0 - move, 3 - do action, 4 - pick
-						// goblins  0 - picker, 1 - fighter, 2 - mage
+	// goblins: 0 - picker, 1 - fighter, 2 - mage
 	Gob_Pos _gobPositions[3];
 	int16 _gobDestX;
 	int16 _gobDestY;
@@ -180,30 +184,30 @@ public:
 	int16 _positionedGob;
 	char _noPick;
 
-	// GOB2:
+	// Gob2:
 	int16 _soundSlotsCount;
 	int16 _soundSlots[60];
-	int16 _word_2F9C0;
-	int16 _word_2F9BE;
-	int16 _word_2F9BC;
-	int16 _word_2F9BA;
-	int16 _dword_2F9B6; // index into the variables array
-	int16 _dword_2F9B2; // index into the variables array
+	bool _gob1Busy;
+	bool _gob2Busy;
+	int16 _gob1RelaxTimeVar;
+	int16 _gob2RelaxTimeVar;
+	bool _gob1NoTurn;
+	bool _gob2NoTurn;
 
 	// Functions
 	char rotateState(int16 from, int16 to);
-	void playSound(Snd::SoundDesc * snd, int16 repCount, int16 freq);
+	void playSound(SoundDesc &snd, int16 repCount, int16 freq);
 	void drawObjects(void);
 	void animateObjects(void);
 	int16 getObjMaxFrame(Gob_Object * obj);
-	int16 objIntersected(Gob_Object * obj1, Gob_Object * obj2);
+	bool objIntersected(Gob_Object * obj1, Gob_Object * obj2);
 	void setMultStates(Gob_Object * gobDesc);
 	int16 nextLayer(Gob_Object * gobDesc);
 	void showBoredom(int16 gobIndex);
 	void switchGoblin(int16 index);
 	void zeroObjects(void);
 	void freeAllObjects(void);
-	void loadObjects(char *source);
+	void loadObjects(const char *source);
 	void initVarPointers(void);
 	void saveGobDataToVars(int16 xPos, int16 yPos, int16 someVal);
 	void loadGobDataFromVars(void);
@@ -214,10 +218,13 @@ public:
 	int16 treatItem(int16 action);
 	int16 doMove(Gob_Object *gobDesc, int16 cont, int16 action);
 
-	void sub_19BD3(void);
-	void sub_195C7(int16 index, int16 state);
-	void sub_11984(Mult::Mult_Object *obj);
+	void setState(int16 index, int16 state);
+	void updateLayer1(Mult::Mult_AnimData *animData);
+	void updateLayer2(Mult::Mult_AnimData *animData);
+	void move(int16 destX, int16 destY, int16 objIndex);
+	void animate(Mult::Mult_Object *obj);
 
+	virtual void handleGoblins(void) = 0;
 	virtual void placeObject(Gob_Object * objDesc, char animated,
 			int16 index, int16 x, int16 y, int16 state) = 0;
 	virtual void freeObjects(void) = 0;
@@ -229,7 +236,9 @@ public:
 	virtual ~Goblin();
 
 protected:
+	Util::List *_objList;
 	int16 _rotStates[4][4];
+
 	GobEngine *_vm;
 
 	int16 peekGoblin(Gob_Object *curGob);
@@ -240,17 +249,22 @@ protected:
 	void targetDummyItem(Gob_Object *gobDesc);
 	void targetItem(void);
 	void moveFindItem(int16 posX, int16 posY);
-	void moveCheckSelect(int16 framesCount, Gob_Object * gobDesc, int16 *pGobIndex, int16 *nextAct);
+	void moveCheckSelect(int16 framesCount, Gob_Object * gobDesc,
+			int16 *pGobIndex, int16 *nextAct);
 	void moveInitStep(int16 framesCount, int16 action, int16 cont,
 					  Gob_Object *gobDesc, int16 *pGobIndex, int16 *pNextAct);
 	void moveTreatRopeStairs(Gob_Object *gobDesc);
 	void playSounds(Mult::Mult_Object *obj);
 
-	virtual void movePathFind(Mult::Mult_Object *obj, Gob_Object *gobDesc, int16 nextAct) = 0;
+	virtual bool isMovement(int8 state) = 0;
+	virtual void advMovement(Mult::Mult_Object *obj, int8 state) = 0;
+	virtual void movePathFind(Mult::Mult_Object *obj,
+			Gob_Object *gobDesc, int16 nextAct) = 0;
 };
 
 class Goblin_v1 : public Goblin {
 public:
+	virtual void handleGoblins(void) {}
 	virtual void placeObject(Gob_Object * objDesc, char animated,
 			int16 index, int16 x, int16 y, int16 state);
 	virtual void freeObjects(void);
@@ -259,14 +273,18 @@ public:
 			int16 nextAct, int16 framesCount);
 
 	Goblin_v1(GobEngine *vm);
-	virtual ~Goblin_v1() {};
+	virtual ~Goblin_v1() {}
 
 protected:
-	virtual void movePathFind(Mult::Mult_Object *obj, Gob_Object *gobDesc, int16 nextAct);
+	virtual bool isMovement(int8 state) { return false; }
+	virtual void advMovement(Mult::Mult_Object *obj, int8 state) {}
+	virtual void movePathFind(Mult::Mult_Object *obj,
+			Gob_Object *gobDesc, int16 nextAct);
 };
 
 class Goblin_v2 : public Goblin_v1 {
 public:
+	virtual void handleGoblins(void);
 	virtual void placeObject(Gob_Object * objDesc, char animated,
 			int16 index, int16 x, int16 y, int16 state);
 	virtual void freeObjects(void);
@@ -275,12 +293,28 @@ public:
 			int16 nextAct, int16 framesCount);
 
 	Goblin_v2(GobEngine *vm);
-	virtual ~Goblin_v2() {};
+	virtual ~Goblin_v2() {}
 
 protected:
-	virtual void movePathFind(Mult::Mult_Object *obj, Gob_Object *gobDesc, int16 nextAct);
+	virtual bool isMovement(int8 state);
+	virtual void advMovement(Mult::Mult_Object *obj, int8 state);
+	virtual void movePathFind(Mult::Mult_Object *obj,
+			Gob_Object *gobDesc, int16 nextAct);
 };
 
-}				// End of namespace Gob
+class Goblin_v3 : public Goblin_v2 {
+public:
+	virtual void placeObject(Gob_Object * objDesc, char animated,
+			int16 index, int16 x, int16 y, int16 state);
 
-#endif	/* __GOBLIN_H */
+	Goblin_v3(GobEngine *vm);
+	virtual ~Goblin_v3() {}
+
+protected:
+	virtual bool isMovement(int8 state);
+	virtual void advMovement(Mult::Mult_Object *obj, int8 state);
+};
+
+} // End of namespace Gob
+
+#endif // GOB_GOBLIN_H
