@@ -1,6 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2004 Ivan Dubrov
- * Copyright (C) 2004-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -16,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/gob/goblin_v2.cpp $
- * $Id: goblin_v2.cpp 23065 2006-06-12 21:33:30Z drmccoy $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/gob/goblin_v2.cpp $
+ * $Id: goblin_v2.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -26,10 +28,12 @@
 
 #include "gob/gob.h"
 #include "gob/goblin.h"
-#include "gob/mult.h"
+#include "gob/global.h"
+#include "gob/util.h"
 #include "gob/game.h"
-#include "gob/scenery.h"
 #include "gob/map.h"
+#include "gob/mult.h"
+#include "gob/scenery.h"
 
 namespace Gob {
 
@@ -42,19 +46,16 @@ Goblin_v2::Goblin_v2(GobEngine *vm) : Goblin_v1(vm) {
 }
 
 void Goblin_v2::freeObjects(void) {
-	int i;
-
 	if (_gobsCount < 0)
 		return;
 
-	for (i = 0; i < _gobsCount; i++) {
+	for (int i = 0; i < _gobsCount; i++) {
 		delete[] _vm->_mult->_objects[i].goblinStates[0];
 		delete[] _vm->_mult->_objects[i].goblinStates;
 	}
-	for (i = 0; i < _vm->_goblin->_soundSlotsCount; i++)
-		if ((_vm->_goblin->_soundSlots[i] & 0x8000) == 0)
-			_vm->_game->freeSoundSlot(_vm->_goblin->_soundSlots[i]);
-//	delete[] off_2F2AB;
+	for (int i = 0; i < _soundSlotsCount; i++)
+		if ((_soundSlots[i] & 0x8000) == 0)
+			_vm->_game->freeSoundSlot(_soundSlots[i]);
 	_gobsCount = -1;
 }
 
@@ -83,8 +84,8 @@ void Goblin_v2::placeObject(Gob_Object *objDesc, char animated,
 			*obj->pPosY = (y + 1) * _vm->_map->_tilesHeight
 				- (_vm->_scenery->_animBottom - _vm->_scenery->_animTop);
 		else
-			*obj->pPosY = ((y + 1) / 2) * _vm->_map->_tilesHeight
-				- (_vm->_scenery->_animBottom - _vm->_scenery->_animTop);
+			*obj->pPosY = ((y + 1) * _vm->_map->_tilesHeight) -
+				(_vm->_scenery->_animBottom - _vm->_scenery->_animTop) - (y + 1) / 2;
 		*obj->pPosX = x * _vm->_map->_tilesWidth;
 	} else {
 		if (obj->goblinStates[state] != 0) {
@@ -96,14 +97,14 @@ void Goblin_v2::placeObject(Gob_Object *objDesc, char animated,
 			objAnim->frame = 0;
 			objAnim->isPaused = 0;
 			objAnim->isStatic = 0;
-			objAnim->newCycle = _vm->_scenery->_animations[animation].layers[layer].framesCount;
+			objAnim->newCycle = _vm->_scenery->getAnimLayer(animation, layer)->framesCount;
 			_vm->_scenery->updateAnim(layer, 0, animation, 0, *obj->pPosX, *obj->pPosY, 0);
 			if (!_vm->_map->_bigTiles)
 				*obj->pPosY = (y + 1) * _vm->_map->_tilesHeight
 					- (_vm->_scenery->_animBottom - _vm->_scenery->_animTop);
 			else
-				*obj->pPosY = ((y + 1) / 2) * _vm->_map->_tilesHeight
-					- (_vm->_scenery->_animBottom - _vm->_scenery->_animTop);
+				*obj->pPosY = ((y + 1) * _vm->_map->_tilesHeight) -
+					(_vm->_scenery->_animBottom - _vm->_scenery->_animTop) - (y + 1) / 2;
 			*obj->pPosX = x * _vm->_map->_tilesWidth;
 			initiateMove(obj);
 		} else
@@ -138,15 +139,15 @@ void Goblin_v2::movePathFind(Mult::Mult_Object *obj, Gob_Object *gobDesc, int16
 
 	dir = 0;
 	animData = obj->pAnimData;
-	framesCount =
-		_vm->_scenery->_animations[(int)animData->animation].layers[animData->layer].framesCount;
+	framesCount = _vm->_scenery->getAnimLayer(animData->animation, animData->layer)->framesCount;
+	animData->newCycle = framesCount;
 	gobX = obj->goblinX;
 	gobY = obj->goblinY;
 	animData->order = gobY;
 	gobDestX = obj->gobDestX;
 	gobDestY = obj->gobDestY;
-	animData->field_13 = gobDestX;
-	animData->field_14 = gobDestY;
+	animData->destX = gobDestX;
+	animData->destY = gobDestY;
 	destX = obj->destX;
 	destY = obj->destY;
 
@@ -174,7 +175,7 @@ void Goblin_v2::movePathFind(Mult::Mult_Object *obj, Gob_Object *gobDesc, int16
 						}
 						if (obj->nearestWayPoint > obj->nearestDest)
 							obj->nearestWayPoint--;
-					} else if (obj->nearestWayPoint < obj->nearestDest) { // loc_10E96
+					} else if (obj->nearestWayPoint < obj->nearestDest) {
 						_vm->_map->optimizePoints(obj, gobX, gobY);
 						destX = _vm->_map->_wayPoints[obj->nearestWayPoint].x;
 						destY = _vm->_map->_wayPoints[obj->nearestWayPoint].y;
@@ -224,7 +225,8 @@ void Goblin_v2::movePathFind(Mult::Mult_Object *obj, Gob_Object *gobDesc, int16
 		break;
 
 	case Map::kDirN:
-		animData->nextState = animData->curLookDir == 2 ? 2 : rotateState(2, animData->curLookDir);
+		animData->nextState =
+			(animData->curLookDir == 2) ? 2 : rotateState(animData->curLookDir, 2);
 		if (_vm->_map->_screenWidth == 640) {
 			if (_vm->_map->getPass(obj->goblinX, obj->goblinY) == 10) {
 				if (_vm->_map->getPass(obj->goblinX - 1, obj->goblinY - 2) != 10) {
@@ -252,11 +254,11 @@ void Goblin_v2::movePathFind(Mult::Mult_Object *obj, Gob_Object *gobDesc, int16
 		break;
 
 	case Map::kDirW:
-		animData->nextState = rotateState(0, animData->curLookDir);
+		animData->nextState = rotateState(animData->curLookDir, 0);
 		break;
 
 	case Map::kDirE:
-		animData->nextState = rotateState(4, animData->curLookDir);
+		animData->nextState = rotateState(animData->curLookDir, 4);
 		break;
 
 	case Map::kDirSW:
@@ -270,7 +272,8 @@ void Goblin_v2::movePathFind(Mult::Mult_Object *obj, Gob_Object *gobDesc, int16
 		break;
 
 	case Map::kDirS:
-		animData->nextState = animData->curLookDir == 6 ? 6 : rotateState(6, animData->curLookDir);
+		animData->nextState =
+			(animData->curLookDir == 6) ? 6 : rotateState(animData->curLookDir, 6);
 		if (_vm->_map->_screenWidth == 640) {
 			if (_vm->_map->getPass(obj->goblinX, obj->goblinY) == 20)
 				animData->nextState = 39;
@@ -316,13 +319,12 @@ void Goblin_v2::moveAdvance(Mult::Mult_Object *obj, Gob_Object *gobDesc,
 
 	animData = obj->pAnimData;
 
-	framesCount =
-		_vm->_scenery->_animations[(int)animData->animation].layers[animData->layer].framesCount;
+	framesCount = _vm->_scenery->getAnimLayer(animData->animation, animData->layer)->framesCount;
 
 	if (animData->isPaused == 0)
 		animData->frame++;
 
-	switch (animData->field_16) {
+	switch (animData->stateType) {
 	case 0:
 	case 1:
 		animData->isPaused = 0;
@@ -339,14 +341,13 @@ void Goblin_v2::moveAdvance(Mult::Mult_Object *obj, Gob_Object *gobDesc,
 		break;
 	}
 
-	switch(animData->state) {
+	switch (animData->state) {
 	case 0:
 	case 1:
 	case 7:
 	case 13:
 	case 16:
 	case 23:
-		// loc_11452
 		animData->curLookDir = 0;
 		break;
 
@@ -382,66 +383,31 @@ void Goblin_v2::moveAdvance(Mult::Mult_Object *obj, Gob_Object *gobDesc,
 		break;
 	}
 
-	if ((animData->field_F != -1) && (animData->frame == framesCount) &&
-			(animData->field_F != animData->state)) { // loc_114B6
-		animData->nextState = animData->field_F;
-		animData->field_F = -1;
+	if ((animData->newState != -1) && (animData->frame == framesCount) &&
+			(animData->newState != animData->state)) {
+		animData->nextState = animData->newState;
+		animData->newState = -1;
 		animData->state = animData->nextState;
-		*obj->pPosX +=
-			_vm->_scenery->_animations[animData->animation].layers[animData->layer].animDeltaX;
-		*obj->pPosY +=
-			_vm->_scenery->_animations[animData->animation].layers[animData->layer].animDeltaY;
+
+		Scenery::AnimLayer *animLayer =
+			_vm->_scenery->getAnimLayer(animData->animation, animData->layer);
+		*obj->pPosX += animLayer->animDeltaX;
+		*obj->pPosY += animLayer->animDeltaY;
+
 		animation = obj->goblinStates[animData->nextState][0].animation;
 		layer = obj->goblinStates[animData->nextState][0].layer;
 		animData->layer = layer;
 		animData->animation = animation;
 		animData->frame = 0;
 	} else {
-		if (((animData->state >= 0) && (animData->state < 8)) ||
-				(animData->state == 38) || (animData->state == 39)) { // loc_115C4
+		if (isMovement(animData->state)) {
 			state = animData->nextState;
 			if (animData->frame == ((framesCount + 1) / 2)) {
 				gobX = obj->goblinX;
 				gobY = obj->goblinY;
-				switch (state) {
-				case 0:
-					obj->goblinX--;
-					break;
 
-				case 1:
-					obj->goblinX--;
-					obj->goblinY--;
-					break;
+				advMovement(obj, state);
 
-				case 2:
-				case 38:
-					obj->goblinY--;
-					break;
-
-				case 3:
-					obj->goblinX++;
-					obj->goblinY--;
-					break;
-
-				case 4:
-					obj->goblinX++;
-					break;
-
-				case 5:
-					obj->goblinX++;
-					obj->goblinY++;
-					break;
-
-				case 6:
-				case 39:
-					obj->goblinY++;
-					break;
-
-				case 7:
-					obj->goblinX--;
-					obj->goblinY++;
-					break;
-				}
 				if (animData->state != state) {
 					animation = obj->goblinStates[state][0].animation;
 					layer = obj->goblinStates[state][0].layer;
@@ -471,45 +437,9 @@ void Goblin_v2::moveAdvance(Mult::Mult_Object *obj, Gob_Object *gobDesc,
 			animData->state = state;
 			gobX = obj->goblinX;
 			gobY = obj->goblinY;
-			switch (state) {
-			case 0:
-				obj->goblinX--;
-				break;
 
-			case 1:
-				obj->goblinX--;
-				obj->goblinY--;
-				break;
+			advMovement(obj, state);
 
-			case 2:
-			case 38:
-				obj->goblinY--;
-				break;
-
-			case 3:
-				obj->goblinX++;
-				obj->goblinY--;
-				break;
-
-			case 4:
-				obj->goblinX++;
-				break;
-
-			case 5:
-				obj->goblinX++;
-				obj->goblinY++;
-				break;
-
-			case 6:
-			case 39:
-				obj->goblinY++;
-				break;
-
-			case 7:
-				obj->goblinX--;
-				obj->goblinY++;
-				break;
-			}
 			_vm->_scenery->updateAnim(layer, 0, animation, 0, *obj->pPosX, *obj->pPosY, 0);
 			if (_vm->_map->_bigTiles)
 				*obj->pPosY = ((gobY + 1) * _vm->_map->_tilesHeight) -
@@ -522,4 +452,216 @@ void Goblin_v2::moveAdvance(Mult::Mult_Object *obj, Gob_Object *gobDesc,
 	}
 }
 
+void Goblin_v2::handleGoblins(void) {
+	Mult::Mult_Object *obj0, *obj1;
+	Mult::Mult_AnimData *anim0, *anim1;
+	int16 pass;
+	int16 gob1State, gob2State;
+	int16 gob1X, gob2X;
+	int16 gob1Y, gob2Y;
+	int16 gob1DestX, gob2DestX;
+	int16 gob1DestY, gob2DestY;
+
+	obj0 = &_vm->_mult->_objects[0];
+	obj1 = &_vm->_mult->_objects[1];
+	anim0 = obj0->pAnimData;
+	anim1 = obj1->pAnimData;
+
+	gob1State = anim0->state;
+	gob2State = anim1->state;
+
+	if (!anim0->isBusy) {
+		if (!_gob1Busy && (anim0->isStatic == 0)) {
+			if ((VAR(_gob1RelaxTimeVar) == 0) && (gob1State == 28)) {
+				// Goblin 1 showing boredom
+				gob1State = _vm->_util->getRandom(3) + 24;
+				setState(0, gob1State);
+				WRITE_VAR(_gob1RelaxTimeVar, 100);
+			} else
+				WRITE_VAR(_gob1RelaxTimeVar, VAR(_gob1RelaxTimeVar) - 1);
+		}
+		if ((gob1State == 8) || (gob1State == 9) || (gob1State == 29))
+			anim0->curLookDir = 6;
+	}
+	if (!anim1->isBusy) {
+		if (!_gob2Busy && (anim1->isStatic == 0)) {
+			if ((VAR(_gob2RelaxTimeVar) == 0) && (gob2State == 28)) {
+				// Goblin 2 showing boredom
+				gob2State = _vm->_util->getRandom(3) + 24;
+				setState(1, gob2State);
+				WRITE_VAR(_gob2RelaxTimeVar, 100);
+			} else
+				WRITE_VAR(_gob2RelaxTimeVar, VAR(_gob2RelaxTimeVar) - 1);
+		}
+		if ((gob2State == 8) || (gob2State == 9) || (gob2State == 29))
+			anim1->curLookDir = 6;
+	}
+
+	if ((anim0->isBusy == 1) && (anim0->isStatic == 0) &&
+			((anim0->state == 28) || (anim0->state == 29)))
+		anim0->curLookDir = 0;
+	if ((anim1->isBusy == 1) && (anim1->isStatic == 0) &&
+			((anim1->state == 28) || (anim1->state == 29)))
+		anim1->curLookDir = 0;
+
+	if (VAR(18) != ((uint32) -1)) {
+		if (anim0->layer == 44)
+			anim0->curLookDir = 4;
+		else if (anim0->layer == 45)
+			anim0->curLookDir = 0;
+		if (anim0->isBusy == 0)
+			anim0->curLookDir = 6;
+	}
+	if (VAR(19) != ((uint32) -1)) {
+		if (anim1->layer == 48)
+			anim1->curLookDir = 4;
+		else if (anim1->layer == 49)
+			anim1->curLookDir = 0;
+		if (anim1->isBusy == 0)
+			anim1->curLookDir = 6;
+	}
+
+	if ((anim0->layer == 45) && (anim0->curLookDir == 4) && (anim0->pathExistence == 5) &&
+			(VAR(18) == ((uint32) -1)) && !_gob1NoTurn) {
+		setState(0, 19); // Turning right->left
+	}
+	if ((anim0->layer == 44) && (anim0->curLookDir == 0) && (anim0->pathExistence == 5) &&
+			(VAR(18) == ((uint32) -1)) && !_gob1NoTurn) {
+		setState(0, 16); // Turning left->right
+	}
+	if ((anim1->layer == 49) && (anim1->curLookDir == 4) && (anim1->pathExistence == 5) &&
+			(VAR(19) == ((uint32) -1)) && !_gob2NoTurn) {
+		setState(1, 19); // Turning right->left
+	}
+	if ((anim1->layer == 48) && (anim1->curLookDir == 0) && (anim1->pathExistence == 5) &&
+			(VAR(19) == ((uint32) -1)) && !_gob2NoTurn) {
+		setState(1, 16); // Turning left->right
+	}
+
+	gob1X = obj0->goblinX;
+	gob2X = obj1->goblinX;
+	gob1Y = obj0->goblinY;
+	gob2Y = obj1->goblinY;
+	gob1DestX = anim0->destX;
+	gob2DestX = anim1->destX;
+	gob1DestY = anim0->destY;
+	gob2DestY = anim1->destY;
+
+	pass = _vm->_map->getPass(gob1X, gob1Y);
+	if ((pass > 17) && (pass < 21)) // Ladders, ropes, stairs
+		updateLayer1(anim0);
+	pass = _vm->_map->getPass(gob2X, gob2Y);
+	if ((pass > 17) && (pass < 21)) // Ladders, ropes, stairs
+		updateLayer2(anim1);
+
+	if ((gob1DestX < 0) || (gob1DestX > 39) || (gob1DestY < 0) || (gob1DestY > 39))
+		return;
+
+	if (gob1Y > gob1DestY) {
+		if (_vm->_map->getPass(gob1DestX, gob1DestY) > 17) {
+			do {
+				gob1DestY--;
+			} while (_vm->_map->getPass(gob1DestX, gob1DestY) > 17);
+			gob1DestY++;
+			if (_vm->_map->getPass(gob1DestX - 1, gob1DestY) == 0) {
+				if (_vm->_map->getPass(gob1DestX + 1, gob1DestY) != 0)
+					gob1DestX++;
+			} else
+				gob1DestX--;
+			move(gob1DestX, gob1DestY, 0);
+		}
+	} else {
+		if (_vm->_map->getPass(gob1DestX, gob1DestY) > 17) {
+			do {
+				gob1DestY++;
+			} while (_vm->_map->getPass(gob1DestX, gob1DestY) > 17);
+			gob1DestY--;
+			if (_vm->_map->getPass(gob1DestX - 1, gob1DestY) == 0) {
+				if (_vm->_map->getPass(gob1DestX + 1, gob1DestY) != 0)
+					gob1DestX++;
+			} else
+				gob1DestX--;
+			move(gob1DestX, gob1DestY, 0);
+		}
+	}
+	if (gob2Y > gob2DestY) {
+		if (_vm->_map->getPass(gob2DestX, gob2DestY) > 17) {
+			do {
+				gob2DestY--;
+			} while (_vm->_map->getPass(gob2DestX, gob2DestY) > 17);
+			gob2DestY++;
+			if (_vm->_map->getPass(gob2DestX - 1, gob2DestY) == 0) {
+				if (_vm->_map->getPass(gob2DestX + 1, gob2DestY) != 0)
+					gob2DestX++;
+			} else
+				gob2DestX--;
+			move(gob2DestX, gob2DestY, 1);
+		}
+	} else {
+		if (_vm->_map->getPass(gob2DestX, gob2DestY) > 17) {
+			do {
+				gob2DestY++;
+			} while (_vm->_map->getPass(gob2DestX, gob2DestY) > 17);
+			gob2DestY--;
+			if (_vm->_map->getPass(gob2DestX - 1, gob2DestY) == 0) {
+				if (_vm->_map->getPass(gob2DestX + 1, gob2DestY) != 0)
+					gob2DestX++;
+			} else
+				gob2DestX--;
+			move(gob2DestX, gob2DestY, 1);
+		}
+	}
+}
+
+bool Goblin_v2::isMovement(int8 state) {
+	if ((state >= 0) && (state < 8))
+		return true;
+	if ((state == 38) || (state == 39))
+		return true;
+
+	return false;
+}
+
+void Goblin_v2::advMovement(Mult::Mult_Object *obj, int8 state) {
+	switch (state) {
+	case 0:
+		obj->goblinX--;
+		break;
+
+	case 1:
+		obj->goblinX--;
+		obj->goblinY--;
+		break;
+
+	case 2:
+	case 38:
+		obj->goblinY--;
+		break;
+
+	case 3:
+		obj->goblinX++;
+		obj->goblinY--;
+		break;
+
+	case 4:
+		obj->goblinX++;
+		break;
+
+	case 5:
+		obj->goblinX++;
+		obj->goblinY++;
+		break;
+
+	case 6:
+	case 39:
+		obj->goblinY++;
+		break;
+
+	case 7:
+		obj->goblinX--;
+		obj->goblinY++;
+		break;
+	}
+}
+
 } // End of namespace Gob
