@@ -1,6 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2004 Ivan Dubrov
- * Copyright (C) 2004-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -16,10 +18,11 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/gob/init.h $
- * $Id: init.h 22254 2006-05-01 12:43:50Z drmccoy $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/gob/init.h $
+ * $Id: init.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
+
 #ifndef GOB_INIT_H
 #define GOB_INIT_H
 
@@ -29,13 +32,12 @@ namespace Gob {
 
 class Init {
 public:
-	void findBestCfg(void);
-	void initGame(char *totFile);
+	void initGame(const char *totFile);
 
-	virtual void soundVideo(int32 smallHeapSize, int16 flag) = 0;
+	virtual void initVideo() = 0;
 
 	Init(GobEngine *vm);
-	virtual ~Init() {};
+	virtual ~Init() {}
 
 protected:
 	Video::PalDesc *_palDesc;
@@ -47,20 +49,28 @@ protected:
 
 class Init_v1 : public Init {
 public:
-	virtual void soundVideo(int32 smallHeapSize, int16 flag);
+	virtual void initVideo();
 
 	Init_v1(GobEngine *vm);
-	virtual ~Init_v1() {};
+	virtual ~Init_v1() {}
 };
 
 class Init_v2 : public Init_v1 {
 public:
-	virtual void soundVideo(int32 smallHeapSize, int16 flag);
+	virtual void initVideo();
 
 	Init_v2(GobEngine *vm);
-	virtual ~Init_v2() {};
+	virtual ~Init_v2() {}
 };
 
-}				// End of namespace Gob
+class Init_v3 : public Init_v2 {
+public:
+	virtual void initVideo();
 
-#endif
+	Init_v3(GobEngine *vm);
+	virtual ~Init_v3() {}
+};
+
+} // End of namespace Gob
+
+#endif // GOB_INIT_H
