@@ -1,6 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2004 Ivan Dubrov
- * Copyright (C) 2004-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -16,14 +18,14 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/gob/map.h $
- * $Id: map.h 23012 2006-06-10 14:37:48Z drmccoy $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/gob/map.h $
+ * $Id: map.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
+
 #ifndef GOB_MAP_H
 #define GOB_MAP_H
 
-#include "gob/util.h"
 #include "gob/mult.h"
 
 namespace Gob {
@@ -36,30 +38,30 @@ public:
 		kDirNW = 0x4700,
 		kDirN  = 0x4800,
 		kDirNE = 0x4900,
-		kDirW  = 0x4b00,
-		kDirE  = 0x4d00,
-		kDirSW = 0x4f00,
+		kDirW  = 0x4B00,
+		kDirE  = 0x4D00,
+		kDirSW = 0x4F00,
 		kDirS  = 0x5000,
 		kDirSE = 0x5100
 	};
 
-#pragma START_PACK_STRUCTS
+#include "common/pack-start.h"	// START STRUCT PACKING
 
 	struct Point {
 		int16 x;
 		int16 y;
-		int16 field_2; // Gob2
-	} GCC_PACK;
+		int16 notWalkable;
+	};
 
 #define szMap_ItemPos 3
 
 	struct ItemPos {
 		int8 x;
 		int8 y;
-		int8 orient;		// ??
-	} GCC_PACK;
+		int8 orient;
+	};
 
-#pragma END_PACK_STRUCTS
+#include "common/pack-end.h"	// END STRUCT PACKING
 
 	int16 _mapWidth;
 	int16 _mapHeight;
@@ -80,81 +82,96 @@ public:
 	int16 _curGoblinY;
 	int16 _destX;
 	int16 _destY;
-	int8 _loadFromAvo;
 
 	ItemPos _itemPoses[40];
 	char _sourceFile[15];
 
+	void findNearestWalkable(int16 &gobDestX, int16 &gobDestY,
+		int16 mouseX, int16 mouseY);
+
 	void placeItem(int16 x, int16 y, int16 id);
 
 	int16 getDirection(int16 x0, int16 y0, int16 x1, int16 y1);
-	int16 checkDirectPath(Mult::Mult_Object *obj, int16 x0, int16 y0, int16 x1, int16 y1);
-	int16 checkLongPath(int16 x0, int16 y0, int16 x1, int16 y1, int16 i0, int16 i1);
-	void loadItemToObject(void);
-	void loadDataFromAvo(char *dest, int16 size);
+	int16 checkDirectPath(Mult::Mult_Object *obj, int16 x0,
+			int16 y0, int16 x1, int16 y1);
+	int16 checkLongPath(int16 x0, int16 y0,
+			int16 x1, int16 y1, int16 i0, int16 i1);
+
 	void loadMapsInitGobs(void);
 
 	virtual int8 getPass(int x, int y, int heightOff = -1) = 0;
 	virtual void setPass(int x, int y, int8 pass, int heightOff = -1) = 0;
 
-	virtual void loadMapObjects(char *avjFile) = 0;
+	virtual void loadMapObjects(const char *avjFile) = 0;
 	virtual void findNearestToGob(Mult::Mult_Object *obj) = 0;
 	virtual void findNearestToDest(Mult::Mult_Object *obj) = 0;
 	virtual void optimizePoints(Mult::Mult_Object *obj, int16 x, int16 y) = 0;
 
+	virtual void init(void) = 0;
 	Map(GobEngine *vm);
-	virtual ~Map() {};
+	virtual ~Map();
 
 protected:
-	char *_avoDataPtr;
+	bool _loadFromAvo;
+
 	GobEngine *_vm;
 
 	int16 findNearestWayPoint(int16 x, int16 y);
-	uint16 loadFromAvo_LE_UINT16();
 };
 
 class Map_v1 : public Map {
 public:
-	virtual void loadMapObjects(char *avjFile);
+	virtual void loadMapObjects(const char *avjFile);
 	virtual void findNearestToGob(Mult::Mult_Object *obj);
 	virtual void findNearestToDest(Mult::Mult_Object *obj);
 	virtual void optimizePoints(Mult::Mult_Object *obj, int16 x, int16 y);
 
-	virtual inline int8 getPass(int x, int y, int heightOff = -1) {
+	virtual int8 getPass(int x, int y, int heightOff = -1) {
 		return _passMap[y * _mapWidth + x];
 	}
 	
-	virtual inline void setPass(int x, int y, int8 pass, int heightOff = -1) {
+	virtual void setPass(int x, int y, int8 pass, int heightOff = -1) {
 		_passMap[y * _mapWidth + x] = pass;
 	}
 
+	virtual void init(void);
 	Map_v1(GobEngine *vm);
 	virtual ~Map_v1();
+
+protected:
+	void loadSounds(Common::SeekableReadStream &data);
+	void loadGoblins(Common::SeekableReadStream &data, uint32 gobsPos);
+	void loadObjects(Common::SeekableReadStream &data, uint32 objsPos);
+	void loadItemToObject(Common::SeekableReadStream &data);
 };
 
 class Map_v2 : public Map_v1 {
 public:
-	virtual void loadMapObjects(char *avjFile);
+	virtual void loadMapObjects(const char *avjFile);
 	virtual void findNearestToGob(Mult::Mult_Object *obj);
 	virtual void findNearestToDest(Mult::Mult_Object *obj);
 	virtual void optimizePoints(Mult::Mult_Object *obj, int16 x, int16 y);
 
-	virtual inline int8 getPass(int x, int y, int heightOff = -1) {
+	virtual int8 getPass(int x, int y, int heightOff = -1) {
 		if (heightOff == -1)
 			heightOff = _passWidth;
-		return _vm->_util->readVariableByte((char *) (_passMap + y * heightOff + x));
+		return _passMap[y * heightOff + x];
 	}
 	
-	virtual inline void setPass(int x, int y, int8 pass, int heightOff = -1) {
+	virtual void setPass(int x, int y, int8 pass, int heightOff = -1) {
 		if (heightOff == -1)
 			heightOff = _passWidth;
-		_vm->_util->writeVariableByte((char *) (_passMap + y * heightOff + x) , pass);
+		_passMap[y * heightOff + x] = pass;
 	}
 
+	virtual void init(void);
 	Map_v2(GobEngine *vm);
 	virtual ~Map_v2();
+
+protected:
+	void loadGoblinStates(Common::SeekableReadStream &data, int index);
 };
 
-}				// End of namespace Gob
+} // End of namespace Gob
 
-#endif	/* __MAP_H */
+#endif // GOB_MAP_H
