@@ -1,6 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2006 The ScummVM project
- * Original ADL-Player source Copyright (C) 2004 by Patrick Combet aka Dorian Gray
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -16,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/gob/music.cpp $
- * $Id: music.cpp 23061 2006-06-12 13:54:07Z drmccoy $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/gob/music.cpp $
+ * $Id: music.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -25,13 +27,14 @@
 #include "common/stdafx.h"
 #include "common/endian.h"
 
-#include "gob/music.h"
 #include "gob/gob.h"
+#include "gob/music.h"
 #include "gob/game.h"
+#include "gob/util.h"
 
 namespace Gob {
 
-const char *Music::_tracks[][2] = {
+const char *Adlib::_tracks[][2] = {
 	{"avt00.tot",  "mine"},
 	{"avt001.tot", "nuit"},
 	{"avt002.tot", "campagne"},
@@ -56,32 +59,31 @@ const char *Music::_tracks[][2] = {
 	{"avt022.tot", "zombie"}
 };
 
-const char *Music::_tracksToFiles[][2] = {
-	{"campagne", "Musmac2.adl"},
-	{"extsor1", "Musmac3.adl"},
-	{"interieure", "Musmac4.adl"},
-	{"mine", "Musmac5.adl"},
-	{"nuit", "Musmac6.adl"},
-	{"statue", "Musmac2.adl"},
-	{"zombie", "Musmac3.adl"}
+const char *Adlib::_trackFiles[] = {
+//	"musmac1.adl", // TODO: This track isn't played correctly at all yet
+	"musmac2.adl",
+	"musmac3.adl",
+	"musmac4.adl",
+	"musmac5.adl",
+	"musmac6.adl"
 };
 
-const unsigned char Music::_operators[] = {0, 1, 2, 8, 9, 10, 16, 17, 18};
-const unsigned char Music::_volRegNums[] = { 
+const unsigned char Adlib::_operators[] = {0, 1, 2, 8, 9, 10, 16, 17, 18};
+const unsigned char Adlib::_volRegNums[] = { 
 	3,  4,  5,
 	11, 12, 13,
 	19, 20, 21
 };
 
-Music::Music(GobEngine *vm) : _vm(vm) {
+Adlib::Adlib(GobEngine *vm) : _vm(vm) {
 	int i;
 
+	_index = -1;
 	_data = 0;
 	_playPos = 0;
 	_dataSize = 0;
 	_rate = _vm->_mixer->getOutputRate();
 	_opl = makeAdlibOPL(_rate);
-	_vm->_mixer->setupPremix(this, Audio::Mixer::kMusicSoundType);
 	_first = true;
 	_ended = false;
 	_playing = false;
@@ -91,81 +93,79 @@ Music::Music(GobEngine *vm) : _vm(vm) {
 
 	for (i = 0; i < 16; i ++)
 		_pollNotes[i] = 0;
-
 	setFreqs();
+
+	_vm->_mixer->playInputStream(Audio::Mixer::kMusicSoundType, &_handle,
+			this, -1, 255, 0, false, true);
 }
 
-Music::~Music(void) {
+Adlib::~Adlib() {
+	Common::StackLock slock(_mutex);
+
+	_vm->_mixer->stopHandle(_handle);
 	OPLDestroy(_opl);
 	if (_data && _needFree)
 		delete[] _data;
-	_vm->_mixer->setupPremix(0);
 }
 
-void Music::premixerCall(int16 *buf, uint len) {
-	_mutex.lock();
-	if (!_playing) {
-		memset(buf, 0, 2 * len * sizeof(int16));
-		_mutex.unlock();
-		return;
+int Adlib::readBuffer(int16 *buffer, const int numSamples) {
+	Common::StackLock slock(_mutex);
+	int samples;
+	int render;
+
+	if (!_playing || (numSamples < 0)) {
+		memset(buffer, 0, numSamples * sizeof(int16));
+		return numSamples;
 	}
-	else {
-		if (_first) {
-			memset(buf, 0, 2 * len * sizeof(int16));
+	if (_first) {
+		memset(buffer, 0, numSamples * sizeof(int16));
+		pollMusic();
+		return numSamples;
+	}
+
+	samples = numSamples;
+	while (samples && _playing) {
+		if (_samplesTillPoll) {
+			render = (samples > _samplesTillPoll) ?  (_samplesTillPoll) : (samples);
+			samples -= render;
+			_samplesTillPoll -= render;
+			YM3812UpdateOne(_opl, buffer, render);
+			buffer += render;
+		} else {
 			pollMusic();
-			_mutex.unlock();
-			return;
-		}
-		else {
-			uint32 render;
-			int16 *data = buf;
-			uint datalen = len;
-			while (datalen && _playing) {
-				if (_samplesTillPoll) {
-					render = (datalen > _samplesTillPoll) ? (_samplesTillPoll) : (datalen);
-					datalen -= render;
-					_samplesTillPoll -= render;
-					YM3812UpdateOne(_opl, data, render);
-					data += render;
-				} else {
-					pollMusic();
-					if (_ended) {
-						memset(data, 0, datalen * sizeof(int16));
-						datalen = 0;
-					}
-				}
+			if (_ended) {
+				memset(buffer, 0, samples * sizeof(int16));
+				samples = 0;
 			}
 		}
-		if (_ended) {
-			_first = true;
-			_ended = false;
-			_playPos = _data + 3 + (_data[1] + 1) * 0x38;
-			_samplesTillPoll = 0;
-			if (_repCount == -1) {
-				reset();
-				setVoices();
-			} else if (_repCount > 0) {
-				_repCount--;
-				reset();
-				setVoices();
-			}
-			else
-				_playing = false;
-		}
-		// Convert mono data to stereo
-		for (int i = (len - 1); i >= 0; i--) {
-			buf[2 * i] = buf[2 * i + 1] = buf[i];
-		}
 	}
-	_mutex.unlock();
+
+	if (_ended) {
+		_first = true;
+		_ended = false;
+		_playPos = _data + 3 + (_data[1] + 1) * 0x38;
+		_samplesTillPoll = 0;
+		if (_repCount == -1) {
+			reset();
+			setVoices();
+		} else if (_repCount > 0) {
+			_repCount--;
+			reset();
+			setVoices();
+		}
+		else
+			_playing = false;
+	}
+
+	return numSamples;
 }
 
-void Music::writeOPL(byte reg, byte val) {
-	debugC(6, DEBUG_MUSIC, "writeOPL(%02X, %02X)", reg, val);
+void Adlib::writeOPL(byte reg, byte val) {
+	debugC(6, kDebugMusic, "writeOPL(%02X, %02X)", reg, val);
 	OPLWriteReg(_opl, reg, val);
 }
 
-void Music::setFreqs(void) {
+void Adlib::setFreqs() {
 	byte lin;
 	byte col;
 	long val = 0;
@@ -183,15 +183,16 @@ void Music::setFreqs(void) {
 		// Run through the 12 columns
 		for (col = 0; col < 12; col ++) {
 			if (!col)
-				val = (((0x2710L + lin * 0x18) * 0xCB78 / 0x3D090) << 0xE) * 9 / 0x1B503;
+				val = (((0x2710L + lin * 0x18) * 0xCB78 / 0x3D090) << 0xE) *
+					9 / 0x1B503;
 			_freqs[lin][col] = (short)((val + 4) >> 3);
 			val = val * 0x6A / 0x64;
-	//      val = val *  392 / 370;
 		} 
 	}
 }
 
-void Music::reset() {
+void Adlib::reset() {
+	_first = true;
 	OPLResetChip(_opl);
 	_samplesTillPoll = 0;
 
@@ -208,13 +209,13 @@ void Music::reset() {
 	writeOPL(0x01, 0x20);
 }
 
-void Music::setVoices() {
+void Adlib::setVoices() {
 	// Definitions of the 9 instruments
 	for (int i = 0; i < 9; i++)
 		setVoice(i, i, true);
 }
 
-void Music::setVoice(byte voice, byte instr, bool set) {
+void Adlib::setVoice(byte voice, byte instr, bool set) {
 	int i;
 	int j;
 	uint16 strct[27];
@@ -234,7 +235,8 @@ void Music::setVoice(byte voice, byte instr, bool set) {
 		writeOPL(0x08, 0x00);
 		writeOPL(0x40 | channel, ((strct[0] & 3) << 6) | (strct[8] & 0x3F));
 		if (!i)
-			writeOPL(0xC0 | voice  , ((strct[2] & 7) << 1) | (1 - (strct[12] & 1)));
+			writeOPL(0xC0 | voice,
+					((strct[2] & 7) << 1) | (1 - (strct[12] & 1)));
 		writeOPL(0x60 | channel, ((strct[3] & 0xF) << 4) | (strct[6] & 0xF));
 		writeOPL(0x80 | channel, ((strct[4] & 0xF) << 4) | (strct[7] & 0xF));
 		writeOPL(0x20 | channel, ((strct[9] & 1) << 7) |
@@ -249,7 +251,7 @@ void Music::setVoice(byte voice, byte instr, bool set) {
 	}
 }
 
-void Music::setKey(byte voice, byte note, bool on, bool spec) {
+void Adlib::setKey(byte voice, byte note, bool on, bool spec) {
 	short freq = 0;
 	short octa = 0;
 
@@ -304,30 +306,30 @@ void Music::setKey(byte voice, byte note, bool on, bool spec) {
 
 	_notes[voice] = note;
 	note += _notCol[voice];
-	note = MIN(0x5F, (int)note);
+	note = MIN((byte) 0x5F, note);
 	octa = note / 12;
 	freq = _freqs[_notLin[voice]][note - octa * 12];
 
-	writeOPL(0xA0 + voice,  freq & 0xff);
+	writeOPL(0xA0 + voice,  freq & 0xFF);
 	writeOPL(0xB0 + voice, (freq >> 8) | (octa << 2) | 0x20 * on);
 
 	if (!freq)
 		warning("Voice %d, note %02X unknown\n", voice, note);
 }
 
-void Music::setVolume(byte voice, byte volume) {
+void Adlib::setVolume(byte voice, byte volume) {
 	volume = 0x3F - (volume * 0x7E + 0x7F) / 0xFE;
 	writeOPL(0x40 + _volRegNums[voice], volume);
 }
 
-void Music::pollMusic(void) {
+void Adlib::pollMusic() {
 	unsigned char instr;
 	byte channel;
 	byte note;
 	byte volume;
 	uint16 tempo;
 
-	if ((_playPos > (_data + _dataSize)) && (_dataSize != (uint32) -1)) {
+	if ((_playPos > (_data + _dataSize)) && (_dataSize != 0xFFFFFFFF)) {
 		_ended = true;
 		return;
 	}
@@ -353,12 +355,6 @@ void Music::pollMusic(void) {
 			setVolume(channel, *(_playPos++));
 			setKey(channel, note, true, false);
 			break;
-		case 0x10:
-//			warning("GOB2 Stub! ADL command 0x10");
-			break;
-		case 0x50:
-//			warning("GOB2 Stub! ADL command 0x50");
-			break;
 		// Note on
 		case 0x90:
 			note = *(_playPos++);
@@ -391,16 +387,17 @@ void Music::pollMusic(void) {
 				_ended = true;
 				_samplesTillPoll = 0;
 				return;
-				break;
 			default:
-				warning("Unknown special command in ADL, stopping playback: %X", instr & 0x0F);
+				warning("Unknown special command in ADL, stopping playback: %X",
+						instr & 0x0F);
 				_repCount = 0;
 				_ended = true;
 				break;
 			}
 			break;
 		default:
-			warning("Unknown command in ADL, stopping playback: %X", instr & 0xF0);
+			warning("Unknown command in ADL, stopping playback: %X",
+					instr & 0xF0);
 			_repCount = 0;
 			_ended = true;
 			break;
@@ -422,14 +419,7 @@ void Music::pollMusic(void) {
 	_samplesTillPoll = tempo * (_rate / 1000);
 }
 
-void Music::startPlay(void) {
-	if (!_data)
-		return;
-	
-	_playing = true;
-}
-
-void Music::playBgMusic(void) {
+void Adlib::playBgMusic() {
 	for (int i = 0; i < ARRAYSIZE(_tracks); i++)
 		if (!scumm_stricmp(_vm->_game->_curTotFile, _tracks[i][0])) {
 			playTrack(_tracks[i][1]);
@@ -437,23 +427,20 @@ void Music::playBgMusic(void) {
 		}
 }
 
-void Music::playTrack(const char *trackname) {
+void Adlib::playTrack(const char *trackname) {
 	if (_playing) return;
 	
-	debugC(1, DEBUG_MUSIC, "Music::playTrack(%s)", trackname);
-	unloadMusic();
-	for (int i = 0; i < ARRAYSIZE(_tracksToFiles); i++)
-		if (!scumm_stricmp(trackname, _tracksToFiles[i][0])) {
-			loadMusic(_tracksToFiles[i][1]);
-			startPlay();
-			break;
-		}
+	debugC(1, kDebugMusic, "Adlib::playTrack(%s)", trackname);
+	unload();
+	load(_trackFiles[_vm->_util->getRandom(ARRAYSIZE(_trackFiles))]);
+	startPlay();
 }
 
-bool Music::loadMusic(const char *filename) {
+bool Adlib::load(const char *fileName) {
 	Common::File song;
 
-	song.open(filename);
+	unload();
+	song.open(fileName);
 	if (!song.isOpen())
 		return false;
 
@@ -470,20 +457,22 @@ bool Music::loadMusic(const char *filename) {
 	return true;
 }
 
-void Music::loadFromMemory(byte *data) {
-	unloadMusic();
+void Adlib::load(byte *data, uint32 size, int index) {
+	unload();
 	_repCount = 0;
 
-	_dataSize = (uint32) -1;
+	_dataSize = size;
 	_data = data;
+	_index = index;
 
 	reset();
 	setVoices();
 	_playPos = _data + 3 + (_data[1] + 1) * 0x38;
 }
 
-void Music::unloadMusic(void) {
+void Adlib::unload() {
 	_playing = false;
+	_index = -1;
 
 	if (_data && _needFree)
 		delete[] _data;
