@@ -1,6 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2006 The ScummVM project
- * Original ADL-Player source Copyright (C) 2004 by Patrick Combet aka Dorian Gray
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -16,58 +18,60 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/gob/music.h $
- * $Id: music.h 22790 2006-05-31 08:44:14Z drmccoy $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/gob/music.h $
+ * $Id: music.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
 #ifndef GOB_MUSIC_H
 #define GOB_MUSIC_H
 
+#include "common/mutex.h"
 #include "sound/audiostream.h"
 #include "sound/mixer.h"
 #include "sound/fmopl.h"
-#include "common/mutex.h"
-
-#include "gob/gob.h"
 
 namespace Gob {
 
 class GobEngine;
 
-class Music : public Audio::AudioStream {
+class Adlib : public Audio::AudioStream {
 public:
-	Music(GobEngine *vm);
-	virtual ~Music();
+	Adlib(GobEngine *vm);
+	~Adlib();
 
 	void lock() { _mutex.lock(); }
 	void unlock() { _mutex.unlock(); }
-	bool playing() { return _playing; }
-	bool getRepeating(void) { return _repCount != 0; }
+	bool playing() const { return _playing; }
+	bool getRepeating() const { return _repCount != 0; }
 	void setRepeating (int32 repCount) { _repCount = repCount; }
-	virtual void startPlay(void);
-	virtual void stopPlay(void) { _mutex.lock(); _playing = false; _mutex.unlock(); }
-	virtual void playTrack(const char *trackname);
-	virtual void playBgMusic(void);
-	virtual bool loadMusic(const char *filename);
-	virtual void loadFromMemory(byte *data);
-	virtual void unloadMusic(void);
+	int getIndex() const { return _index; }
+	void startPlay() { if (_data) _playing = true; }
+	void stopPlay() {
+		Common::StackLock slock(_mutex);
+		_playing = false;
+	}
+	void playTrack(const char *trackname);
+	void playBgMusic();
+	bool load(const char *fileName);
+	void load(byte *data, uint32 size, int index = -1);
+	void unload();
 
 // AudioStream API
-	int readBuffer(int16 *buffer, const int numSamples) {
-		premixerCall(buffer, numSamples / 2);
-		return numSamples;
-	}
-	bool isStereo() const { return true; }
-	bool endOfData() const { return false; }
+	int readBuffer(int16 *buffer, const int numSamples);
+	bool isStereo() const { return false; }
+	bool endOfData() const { return !_playing; }
+	bool endOfStream() const { return false; }
 	int getRate() const { return _rate; }
 	
 protected:
 	static const char *_tracks[][2];
-	static const char *_tracksToFiles[][2];
+	static const char *_trackFiles[];
 	static const unsigned char _operators[];
 	static const unsigned char _volRegNums [];
+	Audio::SoundHandle _handle;
 	FM_OPL *_opl;
+	int _index;
 	byte *_data;
 	byte *_playPos;
 	uint32 _dataSize;
@@ -78,7 +82,7 @@ protected:
 	byte _notLin[11];
 	bool _notOn[11];
 	byte _pollNotes[16];
-	uint32 _samplesTillPoll;
+	int _samplesTillPoll;
 	int32 _repCount;
 	bool _playing;
 	bool _first;
@@ -87,35 +91,16 @@ protected:
 	Common::Mutex _mutex;
 	GobEngine *_vm;
 
-	void premixerCall(int16 *buf, uint len);
 	void writeOPL(byte reg, byte val);
-	void setFreqs(void);
-	void reset(void);
+	void setFreqs();
+	void reset();
 	void setVoices();
 	void setVoice(byte voice, byte instr, bool set);
 	void setKey(byte voice, byte note, bool on, bool spec);
 	void setVolume(byte voice, byte volume);
-	void pollMusic(void);
-};
-
-class Music_Dummy: public Music {
-public:
-	Music_Dummy(GobEngine *vm) : Music(vm) {
-		_vm->_mixer->setupPremix(0);
-		OPLDestroy(_opl);
-	}
-
-	virtual void startPlay(void) {};
-	virtual void stopPlay(void) {};
-	virtual void playTrack(const char *trackname) {};
-	virtual void playBgMusic(void) {};
-	virtual bool loadMusic(const char *filename) { return true; };
-	virtual void loadFromMemory(byte *data) {};
-	virtual void unloadMusic(void) {};
-
-	virtual ~Music_Dummy() {};
+	void pollMusic();
 };
 
 } // End of namespace Gob
 
-#endif
+#endif // GOB_MUSIC_H
