@@ -1,6 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2004 Ivan Dubrov
- * Copyright (C) 2004-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -16,19 +18,21 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/gob/palanim.h $
- * $Id: palanim.h 20520 2006-02-11 10:11:37Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/gob/palanim.h $
+ * $Id: palanim.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
+
 #ifndef GOB_PALANIM_H
 #define GOB_PALANIM_H
 
+#include "gob/video.h"
+
 namespace Gob {
 
 class PalAnim {
 public:
-	char fadeColor(char from, char to);
-	char fadeStep(int16 oper);	// oper == 0 - fade all colors, 1, 2, 3 - red,green, blue
+	bool fadeStep(int16 oper); // 0: all colors, 1: red, 2: green, 3: blue
 	void fade(Video::PalDesc * palDesc, int16 fade, int16 all);
 
 	PalAnim(GobEngine *vm);
@@ -39,9 +43,16 @@ protected:
 	byte _toFadeRed[256];
 	byte _toFadeGreen[256];
 	byte _toFadeBlue[256];
+
+	char *_palArray[3];
+	byte *_fadeArray[3];
+
 	GobEngine *_vm;
+
+	bool fadeStepColor(int color);
+	char fadeColor(int16 from, int16 to);
 };
 
-}				// End of namespace Gob
+} // End of namespace Gob
 
-#endif	/* __PALANIM_H */
+#endif // GOB_PALANIM_H
