@@ -1,6 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2004 Ivan Dubrov
- * Copyright (C) 2004-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -16,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/gob/parse.cpp $
- * $Id: parse.cpp 22568 2006-05-21 21:18:23Z wjpalenstijn $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/gob/parse.cpp $
+ * $Id: parse.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -25,10 +27,9 @@
 #include "common/endian.h"
 
 #include "gob/gob.h"
+#include "gob/parse.h"
 #include "gob/global.h"
 #include "gob/game.h"
-#include "gob/parse.h"
-#include "gob/util.h"
 #include "gob/inter.h"
 
 namespace Gob {
@@ -36,7 +37,7 @@ namespace Gob {
 Parse::Parse(GobEngine *vm) : _vm(vm) {
 }
 
-int32 Parse::encodePtr(char *ptr, int type) {
+int32 Parse::encodePtr(byte *ptr, int type) {
 	int32 offset = 0;
 
 	switch (type) {
@@ -47,17 +48,17 @@ int32 Parse::encodePtr(char *ptr, int type) {
 		offset = ptr - _vm->_global->_inter_variables;
 		break;
 	case kResStr:
-		offset = ptr - _vm->_global->_inter_resStr;
+		offset = ptr - ((byte *) _vm->_global->_inter_resStr);
 		break;
 	default:
-		error("encodePtr: Unknown pointer type");
+		error("Parse::encodePtr(): Unknown pointer type");
 	}
 	assert((offset & 0xF0000000) == 0);
 	return (type << 28) | offset;
 }
 
-char *Parse::decodePtr(int32 n) {
-	char *ptr;
+byte *Parse::decodePtr(int32 n) {
+	byte *ptr;
 
 	switch (n >> 28) {
 	case kExecPtr:
@@ -67,17 +68,17 @@ char *Parse::decodePtr(int32 n) {
 		ptr = _vm->_global->_inter_variables;
 		break;
 	case kResStr:
-		ptr = _vm->_global->_inter_resStr;
+		ptr = (byte *) _vm->_global->_inter_resStr;
 		break;
 	default:
-		error("decodePtr: Unknown pointer type");
+		error("Parse::decodePtr(): Unknown pointer type");
 	}
 	return ptr + (n & 0x0FFFFFFF);
 }
 
 void Parse::skipExpr(char stopToken) {
 	int16 dimCount;
-	char operation;
+	byte operation;
 	int16 num;
 	int16 dim;
 
@@ -85,7 +86,7 @@ void Parse::skipExpr(char stopToken) {
 	while (1) {
 		operation = *_vm->_global->_inter_execPtr++;
 
-		if (operation >= 16 && operation <= 29) {
+		if ((operation >= 16) && (operation <= 29)) {
 			switch (operation) {
 			case 17:
 			case 18:
@@ -104,7 +105,8 @@ void Parse::skipExpr(char stopToken) {
 				break;
 
 			case 22:
-				_vm->_global->_inter_execPtr += strlen(_vm->_global->_inter_execPtr) + 1;
+				_vm->_global->_inter_execPtr +=
+					strlen((char *) _vm->_global->_inter_execPtr) + 1;
 				break;
 
 			case 25:
@@ -126,7 +128,7 @@ void Parse::skipExpr(char stopToken) {
 				for (dim = 0; dim < dimCount; dim++)
 					skipExpr(12);
 
-				if (operation == 28 && *_vm->_global->_inter_execPtr == 13) {
+				if ((operation == 28) && (*_vm->_global->_inter_execPtr == 13)) {
 					_vm->_global->_inter_execPtr++;
 					skipExpr(12);
 				}
@@ -137,17 +139,17 @@ void Parse::skipExpr(char stopToken) {
 				skipExpr(10);
 			}
 			continue;
-		}		// if (operation >= 16 && operation <= 29)
+		} // if ((operation >= 16) && (operation <= 29))
 
 		if (operation == 9) {
 			num++;
 			continue;
 		}
 
-		if (operation == 11 || (operation >= 1 && operation <= 8))
+		if ((operation == 11) || ((operation >= 1) && (operation <= 8)))
 			continue;
 
-		if (operation >= 30 && operation <= 37)
+		if ((operation >= 30) && (operation <= 37))
 			continue;
 
 		if (operation == 10)
@@ -156,7 +158,7 @@ void Parse::skipExpr(char stopToken) {
 		if (operation != stopToken)
 			continue;
 
-		if (stopToken != 10 || num < 0)
+		if ((stopToken != 10) || (num < 0))
 			return;
 	}
 }
@@ -165,7 +167,7 @@ void Parse::printExpr(char stopToken) {
 	// Expression printing disabled by default
 	return;
 
-	char* savedPos = _vm->_global->_inter_execPtr;
+	byte *savedPos = _vm->_global->_inter_execPtr;
 	printExpr_internal(stopToken);
 
 	// restore IP to start of expression
@@ -174,17 +176,17 @@ void Parse::printExpr(char stopToken) {
 
 void Parse::printExpr_internal(char stopToken) {
 	int16 dimCount;
-	char operation;
+	byte operation;
 	int16 num;
 	int16 dim;
-	char *arrDesc;
-	char func;
+	byte *arrDesc;
+	byte func;
 
 	num = 0;
 	while (1) {
 		operation = *_vm->_global->_inter_execPtr++;
 
-		if (operation >= 16 && operation <= 29) {
+		if ((operation >= 16) && (operation <= 29)) {
 			// operands
 
 			switch (operation) {
@@ -196,22 +198,23 @@ void Parse::printExpr_internal(char stopToken) {
 				debugN(5, "var8_%d", _vm->_inter->load16());
 				break;
 
-			case 19: // uint32 immediate
+			case 19: // int32/uint32 immediate
 				debugN(5, "%d", READ_LE_UINT32(_vm->_global->_inter_execPtr));
 				_vm->_global->_inter_execPtr += 4;
 				break;
 
-			case 20: // uint16 immediate
+			case 20: // int16 immediate
 				debugN(5, "%d", _vm->_inter->load16());
 				break;
 
-			case 21: // uint8 immediate
-				debugN(5, "%d",  *_vm->_global->_inter_execPtr++);
+			case 21: // int8 immediate
+				debugN(5, "%d", (int8) *_vm->_global->_inter_execPtr++);
 				break;
 
 			case 22: // string immediate
 				debugN(5, "\42%s\42", _vm->_global->_inter_execPtr);
-				_vm->_global->_inter_execPtr += strlen(_vm->_global->_inter_execPtr) + 1;
+				_vm->_global->_inter_execPtr +=
+					strlen((char *) _vm->_global->_inter_execPtr) + 1;
 				break;
 
 			case 23: // uint32 variable load
@@ -242,7 +245,7 @@ void Parse::printExpr_internal(char stopToken) {
 				_vm->_global->_inter_execPtr += dimCount;
 				for (dim = 0; dim < dimCount; dim++) {
 					printExpr_internal(12);
-					debugN(5, " of %d", (int16)arrDesc[dim]);
+					debugN(5, " of %d", (int16) arrDesc[dim]);
 					if (dim != dimCount - 1)
 						debugN(5, ",");
 				}
@@ -250,7 +253,7 @@ void Parse::printExpr_internal(char stopToken) {
 				if (operation == 28)
 					debugN(5, ")");
 
-				if (operation == 28 && *_vm->_global->_inter_execPtr == 13) {
+				if ((operation == 28) && (*_vm->_global->_inter_execPtr == 13)) {
 					_vm->_global->_inter_execPtr++;
 					debugN(5, "{");
 					printExpr_internal(12); // this also prints the closing }
@@ -265,7 +268,7 @@ void Parse::printExpr_internal(char stopToken) {
 					debugN(5, "rand(");
 				else if (func == 7)
 					debugN(5, "abs(");
-				else if (func == 0 || func == 1 || func == 6)
+				else if ((func == 0) || (func == 1) || (func == 6))
 					debugN(5, "sqrt(");
 				else
 					debugN(5, "id(");
@@ -273,7 +276,7 @@ void Parse::printExpr_internal(char stopToken) {
 				break;
 			}
 			continue;
-		}		// if (operation >= 16 && operation <= 29)
+		}		// if ((operation >= 16) && (operation <= 29))
 
 		// operators
 		switch (operation) {
@@ -365,8 +368,8 @@ void Parse::printExpr_internal(char stopToken) {
 			break;
 
 		default:
-			debugN(5, "<%d>", (int16)operation);
-			error("printExpr: invalid operator in expression");
+			debugN(5, "<%d>", (int16) operation);
+			error("Parse::printExpr(): invalid operator in expression");
 			break;
 		}
 
@@ -375,17 +378,17 @@ void Parse::printExpr_internal(char stopToken) {
 			continue;
 		}
 
-		if (operation == 11 || (operation >= 1 && operation <= 8))
+		if ((operation == 11) || ((operation >= 1) && (operation <= 8)))
 			continue;
 
-		if (operation >= 30 && operation <= 37)
+		if ((operation >= 30) && (operation <= 37))
 			continue;
 
 		if (operation == 10)
 			num--;
 
 		if (operation == stopToken) {
-			if (stopToken != 10 || num < 0) {
+			if ((stopToken != 10) || (num < 0)) {
 				return;
 			}
 		}
@@ -394,13 +397,13 @@ void Parse::printExpr_internal(char stopToken) {
 
 
 void Parse::printVarIndex() {
-	char *arrDesc;
+	byte *arrDesc;
 	int16 dim;
 	int16 dimCount;
 	int16 operation;
 	int16 temp;
 
-	char *pos = _vm->_global->_inter_execPtr;
+	byte *pos = _vm->_global->_inter_execPtr;
 
 	operation = *_vm->_global->_inter_execPtr++;
 	switch (operation) {
@@ -408,7 +411,7 @@ void Parse::printVarIndex() {
 	case 25:
 		temp = _vm->_inter->load16() * 4;
 		debugN(5, "&var_%d", temp);
-		if (operation == 25 && *_vm->_global->_inter_execPtr == 13) {
+		if ((operation == 25) && (*_vm->_global->_inter_execPtr == 13)) {
 			_vm->_global->_inter_execPtr++;
 			debugN(5, "+");
 			printExpr(12);
@@ -423,13 +426,13 @@ void Parse::printVarIndex() {
 		_vm->_global->_inter_execPtr += dimCount;
 		for (dim = 0; dim < dimCount; dim++) {
 			printExpr(12);
-			debugN(5, " of %d", (int16)arrDesc[dim]);
+			debugN(5, " of %d", (int16) arrDesc[dim]);
 			if (dim != dimCount - 1)
 				debugN(5, ",");
 		}
 		debugN(5, "]");
 
-		if (operation == 28 && *_vm->_global->_inter_execPtr == 13) {
+		if ((operation == 28) && (*_vm->_global->_inter_execPtr == 13)) {
 			_vm->_global->_inter_execPtr++;
 			debugN(5, "+");
 			printExpr(12);
