@@ -1,6 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2004 Ivan Dubrov
- * Copyright (C) 2004-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -16,10 +18,11 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/gob/parse.h $
- * $Id: parse.h 22568 2006-05-21 21:18:23Z wjpalenstijn $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/gob/parse.h $
+ * $Id: parse.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
+
 #ifndef GOB_PARSE_H
 #define GOB_PARSE_H
 
@@ -31,11 +34,11 @@ public:
 	void printExpr(char stopToken);
 	void printVarIndex(void);
 	virtual int16 parseVarIndex(void) = 0;
-	virtual int16 parseValExpr(unsigned stopToken=99) = 0;
-	virtual int16 parseExpr(char stopToken, byte *resultPtr) = 0;
+	virtual int16 parseValExpr(byte stopToken = 99) = 0;
+	virtual int16 parseExpr(byte stopToken, byte *resultPtr) = 0;
 	
 	Parse(GobEngine *vm);
-	virtual ~Parse() {};
+	virtual ~Parse() {}
 
 protected:
 	enum PointerType {
@@ -46,8 +49,8 @@ protected:
 
 	GobEngine *_vm;
 
-	int32 encodePtr(char *ptr, int type);
-	char *decodePtr(int32 n);
+	int32 encodePtr(byte *ptr, int type);
+	byte *decodePtr(int32 n);
 
 	void printExpr_internal(char stopToken);
 };
@@ -55,23 +58,23 @@ protected:
 class Parse_v1 : public Parse {
 public:
 	Parse_v1(GobEngine *vm);
-	virtual ~Parse_v1() {};
+	virtual ~Parse_v1() {}
 
 	virtual int16 parseVarIndex(void);
-	virtual int16 parseValExpr(unsigned stopToken=99);
-	virtual int16 parseExpr(char stopToken, byte *resultPtr);
+	virtual int16 parseValExpr(byte stopToken = 99);
+	virtual int16 parseExpr(byte stopToken, byte *resultPtr);
 };
 
 class Parse_v2 : public Parse_v1 {
 public:
 	Parse_v2(GobEngine *vm);
-	virtual ~Parse_v2() {};
+	virtual ~Parse_v2() {}
 
 	virtual int16 parseVarIndex(void);
-	virtual int16 parseValExpr(unsigned stopToken=99);
-	virtual int16 parseExpr(char stopToken, byte *resultPtr);
+	virtual int16 parseValExpr(byte stopToken = 99);
+	virtual int16 parseExpr(byte stopToken, byte *resultPtr);
 };
 
-}				// End of namespace Gob
+} // End of namespace Gob
 
-#endif
+#endif // GOB_PARSE_H
