@@ -1,6 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2004 Ivan Dubrov
- * Copyright (C) 2004-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -16,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/gob/parse_v1.cpp $
- * $Id: parse_v1.cpp 21500 2006-03-29 15:59:37Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/gob/parse_v1.cpp $
+ * $Id: parse_v1.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -26,8 +28,8 @@
 
 #include "gob/gob.h"
 #include "gob/parse.h"
-#include "gob/inter.h"
 #include "gob/global.h"
+#include "gob/inter.h"
 
 namespace Gob {
 
@@ -36,7 +38,7 @@ Parse_v1::Parse_v1(GobEngine *vm) : Parse(vm) {
 
 int16 Parse_v1::parseVarIndex() {
 	int16 temp2;
-	char *arrDesc;
+	byte *arrDesc;
 	int16 dim;
 	int16 dimCount;
 	int16 operation;
@@ -45,17 +47,19 @@ int16 Parse_v1::parseVarIndex() {
 	int16 val;
 
 	operation = *_vm->_global->_inter_execPtr++;
-	debugC(5, DEBUG_PARSER, "var parse = %d", operation);
+	debugC(5, kDebugParser, "var parse = %d", operation);
 	switch (operation) {
 	case 23:
 	case 25:
 		temp = _vm->_inter->load16() * 4;
-		debugC(5, DEBUG_PARSER, "oper = %d", (int16)*_vm->_global->_inter_execPtr);
-		if (operation == 25 && *_vm->_global->_inter_execPtr == 13) {
+		debugC(5, kDebugParser, "oper = %d",
+				(int16) *_vm->_global->_inter_execPtr);
+
+		if ((operation == 25) && (*_vm->_global->_inter_execPtr == 13)) {
 			_vm->_global->_inter_execPtr++;
 			val = parseValExpr(12);
 			temp += val;
-			debugC(5, DEBUG_PARSER, "parse subscript = %d", val);
+			debugC(5, kDebugParser, "parse subscript = %d", val);
 		}
 		return temp;
 
@@ -85,13 +89,13 @@ int16 Parse_v1::parseVarIndex() {
 	}
 }
 
-int16 Parse_v1::parseValExpr(unsigned stopToken) {
+int16 Parse_v1::parseValExpr(byte stopToken) {
 	int16 values[20];
 	byte operStack[20];
 	int16 *valPtr;
 	byte *operPtr;
 	byte *arrDesc;
-	unsigned operation;
+	byte operation;
 	int16 temp2;
 	int16 dim;
 	int16 dimCount;
@@ -118,7 +122,7 @@ int16 Parse_v1::parseValExpr(unsigned stopToken) {
 		valPtr++;
 
 		operation = *_vm->_global->_inter_execPtr++;
-		if (operation >= 19 && operation <= 29) {
+		if ((operation >= 19) && (operation <= 29)) {
 			*operPtr = 20;
 			switch (operation) {
 			case 19:
@@ -131,21 +135,21 @@ int16 Parse_v1::parseValExpr(unsigned stopToken) {
 				break;
 
 			case 23:
-				*valPtr = (uint16)VAR(_vm->_inter->load16());
+				*valPtr = (uint16) VAR(_vm->_inter->load16());
 				break;
 
 			case 25:
 				temp = _vm->_inter->load16() * 4;
 				_vm->_global->_inter_execPtr++;
 				temp += parseValExpr(12);
-				*valPtr = (uint8)*(_vm->_global->_inter_variables + temp);
+				*valPtr = READ_VARO_UINT8(temp);
 				break;
 
 			case 26:
 			case 28:
 				temp = _vm->_inter->load16();
 				dimCount = *_vm->_global->_inter_execPtr++;
-				arrDesc = (byte*)_vm->_global->_inter_execPtr;
+				arrDesc = _vm->_global->_inter_execPtr;
 				_vm->_global->_inter_execPtr += dimCount;
 				offset = 0;
 				for (dim = 0; dim < dimCount; dim++) {
@@ -153,11 +157,12 @@ int16 Parse_v1::parseValExpr(unsigned stopToken) {
 					offset = arrDesc[dim] * offset + temp2;
 				}
 				if (operation == 26) {
-					*valPtr = (uint16)VAR(temp + offset);
+					*valPtr = (uint16) VAR(temp + offset);
 				} else {
 					_vm->_global->_inter_execPtr++;
 					temp2 = parseValExpr(12);
-					*valPtr = (uint8)*(_vm->_global->_inter_variables + temp * 4 + offset * 4 * _vm->_global->_inter_animDataSize + temp2);
+					*valPtr = READ_VARO_UINT8(temp * 4 + offset * 4 *
+							_vm->_global->_inter_animDataSize + temp2);
 				}
 				break;
 
@@ -166,18 +171,20 @@ int16 Parse_v1::parseValExpr(unsigned stopToken) {
 				parseExpr(10, 0);
 
 				if (operation == 5) {
-					_vm->_global->_inter_resVal = _vm->_global->_inter_resVal * _vm->_global->_inter_resVal;
+					_vm->_global->_inter_resVal =
+						_vm->_global->_inter_resVal * _vm->_global->_inter_resVal;
 				} else if (operation == 7) {
 					if (_vm->_global->_inter_resVal < 0)
 						_vm->_global->_inter_resVal = -_vm->_global->_inter_resVal;
 				} else if (operation == 10) {
-					_vm->_global->_inter_resVal = _vm->_util->getRandom(_vm->_global->_inter_resVal);
+					_vm->_global->_inter_resVal =
+						_vm->_util->getRandom(_vm->_global->_inter_resVal);
 				}
 				*valPtr = _vm->_global->_inter_resVal;
 				break;
 
 			}	// switch
-			if (stkPos > 0 && operPtr[-1] == 1) {
+			if ((stkPos > 0) && (operPtr[-1] == 1)) {
 				stkPos--;
 				operPtr--;
 				valPtr--;
@@ -185,7 +192,7 @@ int16 Parse_v1::parseValExpr(unsigned stopToken) {
 				valPtr[0] = -valPtr[1];
 			}
 
-			if (stkPos > 0 && operPtr[-1] > 4 && operPtr[-1] < 9) {
+			if ((stkPos > 0) && (operPtr[-1] > 4) && (operPtr[-1] < 9)) {
 				stkPos -= 2;
 				operPtr -= 2;
 				valPtr -= 2;
@@ -207,11 +214,11 @@ int16 Parse_v1::parseValExpr(unsigned stopToken) {
 					valPtr[0] &= valPtr[2];
 					break;
 				}
-			}	// if (stkPos > 0 && cmdPtr[-1] > 4 && cmdPtr[-1] < 9)
+			}	// if ((stkPos > 0) && (cmdPtr[-1] > 4) && (cmdPtr[-1] < 9))
 			continue;
 		}
 
-		if (operation >= 1 && operation <= 9) {
+		if ((operation >= 1) && (operation <= 9)) {
 			*operPtr = operation;
 			continue;
 		}
@@ -224,7 +231,7 @@ int16 Parse_v1::parseValExpr(unsigned stopToken) {
 
 				operPtr[-1] = operPtr[0];
 				valPtr[-1] = valPtr[0];
-				if (stkPos > 1 && operPtr[-2] == 1) {
+				if ((stkPos > 1) && (operPtr[-2] == 1)) {
 					valPtr[-2] = 20;
 					valPtr[-2] = -valPtr[-1];
 
@@ -233,8 +240,7 @@ int16 Parse_v1::parseValExpr(unsigned stopToken) {
 					valPtr--;
 				}
 
-				if (stkPos > 2 && operPtr[-2] > 4
-				    && operPtr[-2] < 9) {
+				if ((stkPos > 2) && (operPtr[-2] > 4) && (operPtr[-2] < 9)) {
 					stkPos -= 2;
 					operPtr -= 2;
 					valPtr -= 2;
@@ -260,15 +266,14 @@ int16 Parse_v1::parseValExpr(unsigned stopToken) {
 					break;
 			}	// operPtr[-2] == 9
 
-			for (brackPos = stkPos - 2; brackPos > 0 &&
-			    operStack[brackPos] < 30
-			    && operStack[brackPos] != 9; brackPos--);
+			for (brackPos = (stkPos - 2); (brackPos > 0) &&
+			    (operStack[brackPos] < 30) && (operStack[brackPos] != 9);
+					brackPos--);
 
-			if (operStack[brackPos] >= 30
-			    || operStack[brackPos] == 9)
+			if ((operStack[brackPos] >= 30) || (operStack[brackPos] == 9))
 				brackPos++;
 
-			if (operPtr[-2] < 2 || operPtr[-2] > 8)
+			if ((operPtr[-2] < 2) || (operPtr[-2] > 8))
 				break;
 
 			stkPos -= 2;
@@ -300,9 +305,10 @@ int16 Parse_v1::parseValExpr(unsigned stopToken) {
 		}
 
 		if (operation != 10) {
-			if (operation != stopToken) {
-				debugC(5, DEBUG_PARSER, "stoptoken error: %d != %d", operation, stopToken);
-			}
+			if (operation != stopToken)
+				debugC(5, kDebugParser, "stoptoken error: %d != %d",
+						operation, stopToken);
+
 			flag = oldflag;
 			return values[0];
 		}
@@ -313,28 +319,28 @@ int16 Parse_v1::parseValExpr(unsigned stopToken) {
 	}
 }
 
-int16 Parse_v1::parseExpr(char stopToken, byte *arg_2) {
+int16 Parse_v1::parseExpr(byte stopToken, byte *arg_2) {
 	int32 values[20];
 	byte operStack[20];
 	int32 prevPrevVal;
 	int32 prevVal;
 	int32 curVal;
 	int32 *valPtr;
-	char *operPtr;
+	byte *operPtr;
 	byte *arrDescPtr;
-	char var_C;
+	byte var_C;
 	byte operation;
 	int16 dimCount;
 	int16 temp;
 	int16 temp2;
 	uint16 offset;
 	int16 dim;
-	char var_1A;
+	bool var_1A;
 	int16 stkPos;
 	int16 brackStart;
 
 	stkPos = -1;
-	operPtr = (char *)(operStack - 1);
+	operPtr = operStack - 1;
 	valPtr = values - 1;
 
 	while (1) {
@@ -342,7 +348,7 @@ int16 Parse_v1::parseExpr(char stopToken, byte *arg_2) {
 		operPtr++;
 		valPtr++;
 		operation = *_vm->_global->_inter_execPtr++;
-		if (operation >= 19 && operation <= 29) {
+		if ((operation >= 19) && (operation <= 29)) {
 			switch (operation) {
 			case 19:
 				*operPtr = 20;
@@ -358,7 +364,8 @@ int16 Parse_v1::parseExpr(char stopToken, byte *arg_2) {
 			case 22:
 				*operPtr = 22;
 				*valPtr = encodePtr(_vm->_global->_inter_execPtr, kExecPtr);
-				_vm->_global->_inter_execPtr += strlen(_vm->_global->_inter_execPtr) + 1;
+				_vm->_global->_inter_execPtr +=
+					strlen((char *) _vm->_global->_inter_execPtr) + 1;
 				break;
 
 			case 23:
@@ -369,12 +376,13 @@ int16 Parse_v1::parseExpr(char stopToken, byte *arg_2) {
 			case 25:
 				*operPtr = 22;
 				temp = _vm->_inter->load16() * 4;
-				*valPtr = encodePtr(_vm->_global->_inter_variables + temp, kInterVar);
+				*valPtr = encodePtr(_vm->_global->_inter_variables + temp,
+						kInterVar);
 				if (*_vm->_global->_inter_execPtr == 13) {
 					_vm->_global->_inter_execPtr++;
 					temp += parseValExpr(12);
 					*operPtr = 20;
-					*valPtr = (uint8)*(_vm->_global->_inter_variables + temp);
+					*valPtr = READ_VARO_UINT8(temp);
 				}
 				break;
 
@@ -383,7 +391,7 @@ int16 Parse_v1::parseExpr(char stopToken, byte *arg_2) {
 				*operPtr = operation - 6;
 				temp = _vm->_inter->load16();
 				dimCount = *_vm->_global->_inter_execPtr++;
-				arrDescPtr = (byte *)_vm->_global->_inter_execPtr;
+				arrDescPtr = _vm->_global->_inter_execPtr;
 				_vm->_global->_inter_execPtr += dimCount;
 				offset = 0;
 				dim = 0;
@@ -396,12 +404,15 @@ int16 Parse_v1::parseExpr(char stopToken, byte *arg_2) {
 					*valPtr = VAR(temp + offset);
 					break;
 				}
-				*valPtr = encodePtr(_vm->_global->_inter_variables + temp * 4 + offset * _vm->_global->_inter_animDataSize * 4, kInterVar);
+				*valPtr = encodePtr(_vm->_global->_inter_variables +
+						temp * 4 + offset * _vm->_global->_inter_animDataSize * 4,
+						kInterVar);
 				if (*_vm->_global->_inter_execPtr == 13) {
 					_vm->_global->_inter_execPtr++;
 					temp2 = parseValExpr(12);
 					*operPtr = 20;
-					*valPtr = (uint8)*(_vm->_global->_inter_variables + temp * 4 + offset * 4 * _vm->_global->_inter_animDataSize + temp2);
+					*valPtr = READ_VARO_UINT8(temp * 4 +
+							offset * 4 * _vm->_global->_inter_animDataSize + temp2);
 				}
 				break;
 
@@ -411,7 +422,8 @@ int16 Parse_v1::parseExpr(char stopToken, byte *arg_2) {
 
 				switch (operation) {
 				case 5:
-					_vm->_global->_inter_resVal = _vm->_global->_inter_resVal * _vm->_global->_inter_resVal;
+					_vm->_global->_inter_resVal =
+						_vm->_global->_inter_resVal * _vm->_global->_inter_resVal;
 					break;
 
 				case 0:
@@ -424,12 +436,13 @@ int16 Parse_v1::parseExpr(char stopToken, byte *arg_2) {
 						prevPrevVal = prevVal;
 						prevVal = curVal;
 						curVal = (curVal + _vm->_global->_inter_resVal / curVal) / 2;
-					} while (curVal != prevVal && curVal != prevPrevVal);
+					} while ((curVal != prevVal) && (curVal != prevPrevVal));
 					_vm->_global->_inter_resVal = curVal;
 					break;
 
 				case 10:
-					_vm->_global->_inter_resVal = _vm->_util->getRandom(_vm->_global->_inter_resVal);
+					_vm->_global->_inter_resVal =
+						_vm->_util->getRandom(_vm->_global->_inter_resVal);
 					break;
 
 				case 7:
@@ -441,7 +454,7 @@ int16 Parse_v1::parseExpr(char stopToken, byte *arg_2) {
 				*valPtr = _vm->_global->_inter_resVal;
 			}
 
-			if (stkPos > 0 && (operPtr[-1] == 1 || operPtr[-1] == 11)) {
+			if ((stkPos > 0) && ((operPtr[-1] == 1) || (operPtr[-1] == 11))) {
 				stkPos--;
 				operPtr--;
 				valPtr--;
@@ -463,11 +476,11 @@ int16 Parse_v1::parseExpr(char stopToken, byte *arg_2) {
 			switch (operPtr[-1]) {
 			case 2:
 				if (operPtr[-2] == 22) {
-					if (decodePtr(valPtr[-2]) != _vm->_global->_inter_resStr) {
-						strcpy(_vm->_global->_inter_resStr, decodePtr(valPtr[-2]));
-						valPtr[-2] = encodePtr(_vm->_global->_inter_resStr, kResStr);
+					if ((char *) decodePtr(valPtr[-2]) != _vm->_global->_inter_resStr) {
+						strcpy(_vm->_global->_inter_resStr, (char *) decodePtr(valPtr[-2]));
+						valPtr[-2] = encodePtr((byte *) _vm->_global->_inter_resStr, kResStr);
 					}
-					strcat(_vm->_global->_inter_resStr, decodePtr(valPtr[0]));
+					strcat(_vm->_global->_inter_resStr, (char *) decodePtr(valPtr[0]));
 					stkPos -= 2;
 					operPtr -= 2;
 					valPtr -= 2;
@@ -503,14 +516,16 @@ int16 Parse_v1::parseExpr(char stopToken, byte *arg_2) {
 				break;
 			}
 			continue;
-		}		// op>= 19 && op <= 29
+		} // (op >= 19) && (op <= 29)
 
-		if (operation == stopToken || operation == 30 || operation == 31 || operation == 10) {
+		if ((operation == stopToken) || (operation == 30) ||
+		    (operation == 31) || (operation == 10)) {
 			while (stkPos >= 2) {
-				var_1A = 0;
-				if (operPtr[-2] == 9 && (operation == 10 || operation == stopToken)) {
+				var_1A = false;
+				if ((operPtr[-2] == 9) &&
+						((operation == 10) || (operation == stopToken))) {
 					operPtr[-2] = operPtr[-1];
-					if (operPtr[-2] == 20 || operPtr[-2] == 22)
+					if ((operPtr[-2] == 20) || (operPtr[-2] == 22))
 						valPtr[-2] = valPtr[-1];
 
 					stkPos--;
@@ -569,13 +584,13 @@ int16 Parse_v1::parseExpr(char stopToken, byte *arg_2) {
 
 					if (operation != stopToken)
 						break;
-				}	// if (operPtr[-2] == 9 && ...)
+				}	// if ((operPtr[-2] == 9) && ...)
 
-				for (brackStart = stkPos - 2; brackStart > 0 &&
-				    operStack[brackStart] < 30 &&
-				    operStack[brackStart] != 9; brackStart--);
+				for (brackStart = (stkPos - 2); (brackStart > 0) &&
+				    (operStack[brackStart] < 30) && (operStack[brackStart] != 9);
+						brackStart--);
 
-				if (operStack[brackStart] >= 30 || operStack[brackStart] == 9)
+				if ((operStack[brackStart] >= 30) || (operStack[brackStart] == 9))
 					brackStart++;
 
 				switch (operPtr[-2]) {
@@ -583,11 +598,12 @@ int16 Parse_v1::parseExpr(char stopToken, byte *arg_2) {
 					if (operStack[brackStart] == 20) {
 						values[brackStart] += valPtr[-1];
 					} else if (operStack[brackStart] == 22) {
-						if (decodePtr(values[brackStart]) != _vm->_global->_inter_resStr) {
-							strcpy(_vm->_global->_inter_resStr, decodePtr(values[brackStart]));
-							values[brackStart] = encodePtr(_vm->_global->_inter_resStr, kResStr);
+						if ((char *) decodePtr(values[brackStart]) != _vm->_global->_inter_resStr) {
+							strcpy(_vm->_global->_inter_resStr, (char *) decodePtr(values[brackStart]));
+							values[brackStart] =
+								encodePtr((byte *) _vm->_global->_inter_resStr, kResStr);
 						}
-						strcat(_vm->_global->_inter_resStr, decodePtr(valPtr[-1]));
+						strcat(_vm->_global->_inter_resStr, (char *) decodePtr(valPtr[-1]));
 					}
 					stkPos -= 2;
 					operPtr -= 2;
@@ -659,11 +675,11 @@ int16 Parse_v1::parseExpr(char stopToken, byte *arg_2) {
 						if (valPtr[-3] < valPtr[-1])
 							operPtr[-3] = 24;
 					} else if (var_C == 22) {
-						if (decodePtr(valPtr[-3]) != _vm->_global->_inter_resStr) {
-							strcpy(_vm->_global->_inter_resStr, decodePtr(valPtr[-3]));
-							valPtr[-3] = encodePtr(_vm->_global->_inter_resStr, kResStr);
+						if ((char *) decodePtr(valPtr[-3]) != _vm->_global->_inter_resStr) {
+							strcpy(_vm->_global->_inter_resStr, (char *) decodePtr(valPtr[-3]));
+							valPtr[-3] = encodePtr((byte *) _vm->_global->_inter_resStr, kResStr);
 						}
-						if (strcmp(_vm->_global->_inter_resStr, decodePtr(valPtr[-1])) < 0)
+						if (strcmp(_vm->_global->_inter_resStr, (char *) decodePtr(valPtr[-1])) < 0)
 							operPtr[-3] = 24;
 					}
 					stkPos -= 2;
@@ -678,11 +694,11 @@ int16 Parse_v1::parseExpr(char stopToken, byte *arg_2) {
 						if (valPtr[-3] <= valPtr[-1])
 							operPtr[-3] = 24;
 					} else if (var_C == 22) {
-						if (decodePtr(valPtr[-3]) != _vm->_global->_inter_resStr) {
-							strcpy(_vm->_global->_inter_resStr, decodePtr(valPtr[-3]));
-							valPtr[-3] = encodePtr(_vm->_global->_inter_resStr, kResStr);
+						if ((char *) decodePtr(valPtr[-3]) != _vm->_global->_inter_resStr) {
+							strcpy(_vm->_global->_inter_resStr, (char *) decodePtr(valPtr[-3]));
+							valPtr[-3] = encodePtr((byte *) _vm->_global->_inter_resStr, kResStr);
 						}
-						if (strcmp(_vm->_global->_inter_resStr, decodePtr(valPtr[-1])) <= 0)
+						if (strcmp(_vm->_global->_inter_resStr, (char *) decodePtr(valPtr[-1])) <= 0)
 							operPtr[-3] = 24;
 					}
 					stkPos -= 2;
@@ -697,11 +713,11 @@ int16 Parse_v1::parseExpr(char stopToken, byte *arg_2) {
 						if (valPtr[-3] > valPtr[-1])
 							operPtr[-3] = 24;
 					} else if (var_C == 22) {
-						if (decodePtr(valPtr[-3]) != _vm->_global->_inter_resStr) {
-							strcpy(_vm->_global->_inter_resStr, decodePtr(valPtr[-3]));
-							valPtr[-3] = encodePtr(_vm->_global->_inter_resStr, kResStr);
+						if ((char *) decodePtr(valPtr[-3]) != _vm->_global->_inter_resStr) {
+							strcpy(_vm->_global->_inter_resStr, (char *) decodePtr(valPtr[-3]));
+							valPtr[-3] = encodePtr((byte *) _vm->_global->_inter_resStr, kResStr);
 						}
-						if (strcmp(_vm->_global->_inter_resStr, decodePtr(valPtr[-1])) > 0)
+						if (strcmp(_vm->_global->_inter_resStr, (char *) decodePtr(valPtr[-1])) > 0)
 							operPtr[-3] = 24;
 					}
 					stkPos -= 2;
@@ -716,11 +732,11 @@ int16 Parse_v1::parseExpr(char stopToken, byte *arg_2) {
 						if (valPtr[-3] >= valPtr[-1])
 							operPtr[-3] = 24;
 					} else if (var_C == 22) {
-						if (decodePtr(valPtr[-3]) != _vm->_global->_inter_resStr) {
-							strcpy(_vm->_global->_inter_resStr, decodePtr(valPtr[-3]));
-							valPtr[-3] = encodePtr(_vm->_global->_inter_resStr, kResStr);
+						if ((char *) decodePtr(valPtr[-3]) != _vm->_global->_inter_resStr) {
+							strcpy(_vm->_global->_inter_resStr, (char *) decodePtr(valPtr[-3]));
+							valPtr[-3] = encodePtr((byte *) _vm->_global->_inter_resStr, kResStr);
 						}
-						if (strcmp(_vm->_global->_inter_resStr, decodePtr(valPtr[-1])) >= 0)
+						if (strcmp(_vm->_global->_inter_resStr, (char *) decodePtr(valPtr[-1])) >= 0)
 							operPtr[-3] = 24;
 					}
 					stkPos -= 2;
@@ -735,11 +751,11 @@ int16 Parse_v1::parseExpr(char stopToken, byte *arg_2) {
 						if (valPtr[-3] == valPtr[-1])
 							operPtr[-3] = 24;
 					} else if (var_C == 22) {
-						if (decodePtr(valPtr[-3]) != _vm->_global->_inter_resStr) {
-							strcpy(_vm->_global->_inter_resStr, decodePtr(valPtr[-3]));
-							valPtr[-3] = encodePtr(_vm->_global->_inter_resStr, kResStr);
+						if ((char *) decodePtr(valPtr[-3]) != _vm->_global->_inter_resStr) {
+							strcpy(_vm->_global->_inter_resStr, (char *) decodePtr(valPtr[-3]));
+							valPtr[-3] = encodePtr((byte *) _vm->_global->_inter_resStr, kResStr);
 						}
-						if (strcmp(_vm->_global->_inter_resStr, decodePtr(valPtr[-1])) == 0)
+						if (strcmp(_vm->_global->_inter_resStr, (char *) decodePtr(valPtr[-1])) == 0)
 							operPtr[-3] = 24;
 					}
 					stkPos -= 2;
@@ -754,11 +770,11 @@ int16 Parse_v1::parseExpr(char stopToken, byte *arg_2) {
 						if (valPtr[-3] != valPtr[-1])
 							operPtr[-3] = 24;
 					} else if (var_C == 22) {
-						if (decodePtr(valPtr[-3]) != _vm->_global->_inter_resStr) {
-							strcpy(_vm->_global->_inter_resStr, decodePtr(valPtr[-3]));
-							valPtr[-3] = encodePtr(_vm->_global->_inter_resStr, kResStr);
+						if ((char *) decodePtr(valPtr[-3]) != _vm->_global->_inter_resStr) {
+							strcpy(_vm->_global->_inter_resStr, (char *) decodePtr(valPtr[-3]));
+							valPtr[-3] = encodePtr((byte *) _vm->_global->_inter_resStr, kResStr);
 						}
-						if (strcmp(_vm->_global->_inter_resStr, decodePtr(valPtr[-1])) != 0)
+						if (strcmp(_vm->_global->_inter_resStr, (char *) decodePtr(valPtr[-1])) != 0)
 							operPtr[-3] = 24;
 					}
 					stkPos -= 2;
@@ -767,15 +783,15 @@ int16 Parse_v1::parseExpr(char stopToken, byte *arg_2) {
 					break;
 
 				default:
-					var_1A = 1;
+					var_1A = true;
 					break;
 				}	// switch
 
-				if (var_1A != 0)
+				if (var_1A)
 					break;
 			}	// while (stkPos >= 2)
 
-			if (operation == 30 || operation == 31) {
+			if ((operation == 30) || (operation == 31)) {
 				if (operPtr[-1] == 20) {
 					if (valPtr[-1] != 0)
 						operPtr[-1] = 24;
@@ -783,19 +799,19 @@ int16 Parse_v1::parseExpr(char stopToken, byte *arg_2) {
 						operPtr[-1] = 23;
 				}
 
-				if ((operation == 30 && operPtr[-1] == 24) ||
-				    (operation == 31 && operPtr[-1] == 23)) {
-					if (stkPos > 1 && operPtr[-2] == 9) {
+				if (((operation == 30) && (operPtr[-1] == 24)) ||
+				    ((operation == 31) && (operPtr[-1] == 23))) {
+					if ((stkPos > 1) && (operPtr[-2] == 9)) {
 						skipExpr(10);
 						operPtr[-2] = operPtr[-1];
 						stkPos -= 2;
 						operPtr -= 2;
 						valPtr -= 2;
-					} else {
+					} else
 						skipExpr(stopToken);
-					}
+
 					operation = _vm->_global->_inter_execPtr[-1];
-					if (stkPos > 0 && operPtr[-1] == 11) {
+					if ((stkPos > 0) && (operPtr[-1] == 11)) {
 						if (operPtr[0] == 23)
 							operPtr[-1] = 24;
 						else
@@ -825,8 +841,8 @@ int16 Parse_v1::parseExpr(char stopToken, byte *arg_2) {
 				break;
 
 			case 22:
-				if (decodePtr(values[0]) != _vm->_global->_inter_resStr)
-					strcpy(_vm->_global->_inter_resStr, decodePtr(values[0]));
+				if ((char *) decodePtr(values[0]) != _vm->_global->_inter_resStr)
+					strcpy(_vm->_global->_inter_resStr, (char *) decodePtr(values[0]));
 				break;
 
 			case 11:
@@ -845,7 +861,7 @@ int16 Parse_v1::parseExpr(char stopToken, byte *arg_2) {
 				break;
 			}
 			return 0;
-		}		// operation == stopToken || operation == 30 || operation == 31 || operation == 10
+		}		// (operation == stopToken) || (operation == 30) || (operation == 31) || (operation == 10)
 
 		if (operation < 1 || operation > 11) {
 			if (operation < 32 || operation > 37)
@@ -856,11 +872,11 @@ int16 Parse_v1::parseExpr(char stopToken, byte *arg_2) {
 					if (operPtr[-3] == 20) {
 						valPtr[-3] += valPtr[-1];
 					} else if (operPtr[-3] == 22) {
-						if (decodePtr(valPtr[-3]) != _vm->_global->_inter_resStr) {
-							strcpy(_vm->_global->_inter_resStr, decodePtr(valPtr[-3]));
-							valPtr[-3] = encodePtr(_vm->_global->_inter_resStr, kResStr);
+						if ((char *) decodePtr(valPtr[-3]) != _vm->_global->_inter_resStr) {
+							strcpy(_vm->_global->_inter_resStr, (char *) decodePtr(valPtr[-3]));
+							valPtr[-3] = encodePtr((byte *) _vm->_global->_inter_resStr, kResStr);
 						}
-						strcat(_vm->_global->_inter_resStr, decodePtr(valPtr[-1]));
+						strcat(_vm->_global->_inter_resStr, (char *) decodePtr(valPtr[-1]));
 					}
 					stkPos -= 2;
 					operPtr -= 2;
