@@ -1,6 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2004 Ivan Dubrov
- * Copyright (C) 2004-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -16,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/gob/parse_v2.cpp $
- * $Id: parse_v2.cpp 22917 2006-06-04 17:24:03Z drmccoy $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/gob/parse_v2.cpp $
+ * $Id: parse_v2.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -26,8 +28,8 @@
 
 #include "gob/gob.h"
 #include "gob/parse.h"
-#include "gob/inter.h"
 #include "gob/global.h"
+#include "gob/inter.h"
 
 namespace Gob {
 
@@ -36,7 +38,7 @@ Parse_v2::Parse_v2(GobEngine *vm) : Parse_v1(vm) {
 
 int16 Parse_v2::parseVarIndex() {
 	int16 temp2;
-	char *arrDesc;
+	byte *arrDesc;
 	int16 dim;
 	int16 dimCount;
 	int16 operation;
@@ -45,7 +47,7 @@ int16 Parse_v2::parseVarIndex() {
 	int16 val;
 
 	operation = *_vm->_global->_inter_execPtr++;
-	debugC(5, DEBUG_PARSER, "var parse = %d", operation);
+	debugC(5, kDebugParser, "var parse = %d", operation);
 	switch (operation) {
 	case 16:
 	case 26:
@@ -84,12 +86,13 @@ int16 Parse_v2::parseVarIndex() {
 	case 24:
 	case 25:
 		temp = _vm->_inter->load16() * 4;
-		debugC(5, DEBUG_PARSER, "oper = %d", (int16)*_vm->_global->_inter_execPtr);
-		if (operation == 25 && *_vm->_global->_inter_execPtr == 13) {
+		debugC(5, kDebugParser, "oper = %d",
+				(int16) *_vm->_global->_inter_execPtr);
+		if ((operation == 25) && (*_vm->_global->_inter_execPtr == 13)) {
 			_vm->_global->_inter_execPtr++;
 			val = parseValExpr(12);
 			temp += val;
-			debugC(5, DEBUG_PARSER, "parse subscript = %d", val);
+			debugC(5, kDebugParser, "parse subscript = %d", val);
 		}
 		return temp;
 
@@ -98,13 +101,13 @@ int16 Parse_v2::parseVarIndex() {
 	}
 }
 
-int16 Parse_v2::parseValExpr(unsigned stopToken) {
+int16 Parse_v2::parseValExpr(byte stopToken) {
 	int16 values[20];
 	byte operStack[20];
 	int16 *valPtr;
 	byte *operPtr;
 	byte *arrDesc;
-	unsigned operation;
+	byte operation;
 	int16 temp2;
 	int16 dim;
 	int16 dimCount;
@@ -115,6 +118,8 @@ int16 Parse_v2::parseValExpr(unsigned stopToken) {
 	static int16 flag = 0;
 	int16 oldflag;
 
+	memset(values, 0, 20 * sizeof(int16));
+
 	oldflag = flag;
 	if (flag == 0) {
 		flag = 1;
@@ -131,7 +136,7 @@ int16 Parse_v2::parseValExpr(unsigned stopToken) {
 		valPtr++;
 
 		operation = *_vm->_global->_inter_execPtr++;
-		if (operation >= 16 && operation <= 29) {
+		if ((operation >= 16) && (operation <= 29)) {
 			*operPtr = 20;
 			switch (operation) {
 			case 16:
@@ -140,7 +145,7 @@ int16 Parse_v2::parseValExpr(unsigned stopToken) {
 			case 28:
 				temp = _vm->_inter->load16();
 				dimCount = *_vm->_global->_inter_execPtr++;
-				arrDesc = (byte*)_vm->_global->_inter_execPtr;
+				arrDesc = _vm->_global->_inter_execPtr;
 				_vm->_global->_inter_execPtr += dimCount;
 				offset = 0;
 				for (dim = 0; dim < dimCount; dim++) {
@@ -148,29 +153,30 @@ int16 Parse_v2::parseValExpr(unsigned stopToken) {
 					offset = arrDesc[dim] * offset + temp2;
 				}
 				if (operation == 16)
-					*valPtr = *(_vm->_global->_inter_variables + temp + offset);
+					*valPtr = (int8) READ_VARO_UINT8(temp + offset);
 				else if (operation == 26)
-					*valPtr = *(uint16*)(_vm->_global->_inter_variables + temp * 4 + offset * 4);
+					*valPtr = (uint16) READ_VARO_UINT32(temp * 4 + offset * 4);
 				else if (operation == 27)
-					*valPtr = *(uint16*)(_vm->_global->_inter_variables + temp * 2 + offset * 2);
+					*valPtr = READ_VARO_UINT16(temp * 2 + offset * 2);
 				else if (operation == 28) {
 					_vm->_global->_inter_execPtr++;
 					temp2 = parseValExpr(12);
-					*valPtr = (uint8)*(_vm->_global->_inter_variables + temp * 4 + offset * 4 * _vm->_global->_inter_animDataSize + temp2);
+					*valPtr = READ_VARO_UINT8(temp * 4 +
+							offset * 4 * _vm->_global->_inter_animDataSize + temp2);
 				}
 				break;
 
 			case 17:
-				*valPtr = *(uint16*)(_vm->_global->_inter_variables + _vm->_inter->load16() * 2);
+				*valPtr = READ_VARO_UINT16(_vm->_inter->load16() * 2);
 				break;
 
 			case 18:
-				*valPtr = *(_vm->_global->_inter_variables + _vm->_inter->load16());
+				*valPtr = (int8) READ_VARO_UINT8(_vm->_inter->load16());
 				break;
 
 			case 19:
-				*valPtr = _vm->_inter->load16();
-				_vm->_global->_inter_execPtr += 2;
+				*valPtr = (uint16) READ_LE_UINT32(_vm->_global->_inter_execPtr);
+				_vm->_global->_inter_execPtr += 4;
 				break;
 
 			case 20:
@@ -178,19 +184,22 @@ int16 Parse_v2::parseValExpr(unsigned stopToken) {
 				break;
 
 			case 21:
-				*valPtr = *((int8 *) _vm->_global->_inter_execPtr++);
+				*valPtr = (int8) *_vm->_global->_inter_execPtr++;
 				break;
 
 			case 23:
+				*valPtr = (uint16) VAR(_vm->_inter->load16());
+				break;
+
 			case 24:
-				*valPtr = VAR(_vm->_inter->load16());
+				*valPtr = READ_VARO_UINT16(_vm->_inter->load16() * 4);
 				break;
 
 			case 25:
 				temp = _vm->_inter->load16() * 4;
 				_vm->_global->_inter_execPtr++;
 				temp += parseValExpr(12);
-				*valPtr = *(_vm->_global->_inter_variables + temp);
+				*valPtr = READ_VARO_UINT8(temp);
 				break;
 
 			case 29:
@@ -198,18 +207,20 @@ int16 Parse_v2::parseValExpr(unsigned stopToken) {
 				parseExpr(10, 0);
 
 				if (operation == 5) {
-					_vm->_global->_inter_resVal = _vm->_global->_inter_resVal * _vm->_global->_inter_resVal;
+					_vm->_global->_inter_resVal =
+						_vm->_global->_inter_resVal * _vm->_global->_inter_resVal;
 				} else if (operation == 7) {
 					if (_vm->_global->_inter_resVal < 0)
 						_vm->_global->_inter_resVal = -_vm->_global->_inter_resVal;
 				} else if (operation == 10) {
-					_vm->_global->_inter_resVal = _vm->_util->getRandom(_vm->_global->_inter_resVal);
+					_vm->_global->_inter_resVal =
+						_vm->_util->getRandom(_vm->_global->_inter_resVal);
 				}
 				*valPtr = _vm->_global->_inter_resVal;
 				break;
 
 			}	// switch
-			if (stkPos > 0 && operPtr[-1] == 1) {
+			if ((stkPos > 0) && (operPtr[-1] == 1)) {
 				stkPos--;
 				operPtr--;
 				valPtr--;
@@ -217,7 +228,7 @@ int16 Parse_v2::parseValExpr(unsigned stopToken) {
 				valPtr[0] = -valPtr[1];
 			}
 
-			if (stkPos > 0 && operPtr[-1] > 4 && operPtr[-1] < 9) {
+			if ((stkPos > 0) && (operPtr[-1] > 4) && (operPtr[-1] < 9)) {
 				stkPos -= 2;
 				operPtr -= 2;
 				valPtr -= 2;
@@ -239,11 +250,11 @@ int16 Parse_v2::parseValExpr(unsigned stopToken) {
 					valPtr[0] &= valPtr[2];
 					break;
 				}
-			}	// if (stkPos > 0 && cmdPtr[-1] > 4 && cmdPtr[-1] < 9)
+			}	// if ((stkPos > 0) && (cmdPtr[-1] > 4) && (cmdPtr[-1] < 9))
 			continue;
 		}
 
-		if (operation >= 1 && operation <= 9) {
+		if ((operation >= 1) && (operation <= 9)) {
 			*operPtr = operation;
 			continue;
 		}
@@ -256,8 +267,8 @@ int16 Parse_v2::parseValExpr(unsigned stopToken) {
 
 				operPtr[-1] = operPtr[0];
 				valPtr[-1] = valPtr[0];
-				if (stkPos > 1 && operPtr[-2] == 1) {
-					valPtr[-2] = 20;
+				if ((stkPos > 1) && (operPtr[-2] == 1)) {
+					operPtr[-2] = 20;
 					valPtr[-2] = -valPtr[-1];
 
 					stkPos--;
@@ -265,26 +276,25 @@ int16 Parse_v2::parseValExpr(unsigned stopToken) {
 					valPtr--;
 				}
 
-				if (stkPos > 2 && operPtr[-2] > 4
-				    && operPtr[-2] < 9) {
+				if ((stkPos > 2) && (operPtr[-2] > 4) && (operPtr[-2] < 9)) {
 					stkPos -= 2;
 					operPtr -= 2;
 					valPtr -= 2;
 					switch (operPtr[0]) {
 					case 5:
-						operPtr[-1] *= operPtr[1];
+						valPtr[-1] *= valPtr[1];
 						break;
 
 					case 6:
-						operPtr[-1] /= operPtr[1];
+						valPtr[-1] /= valPtr[1];
 						break;
 
 					case 7:
-						operPtr[-1] %= operPtr[1];
+						valPtr[-1] %= valPtr[1];
 						break;
 
 					case 8:
-						operPtr[-1] &= operPtr[1];
+						valPtr[-1] &= valPtr[1];
 						break;
 					}
 				}
@@ -292,15 +302,14 @@ int16 Parse_v2::parseValExpr(unsigned stopToken) {
 					break;
 			}	// operPtr[-2] == 9
 
-			for (brackPos = stkPos - 2; brackPos > 0 &&
-			    operStack[brackPos] < 30
-			    && operStack[brackPos] != 9; brackPos--);
+			for (brackPos = (stkPos - 2); (brackPos > 0) &&
+			    (operStack[brackPos] < 30) && (operStack[brackPos] != 9);
+					brackPos--);
 
-			if (operStack[brackPos] >= 30
-			    || operStack[brackPos] == 9)
+			if ((operStack[brackPos] >= 30) || (operStack[brackPos] == 9))
 				brackPos++;
 
-			if (operPtr[-2] < 2 || operPtr[-2] > 8)
+			if ((operPtr[-2] < 2) || (operPtr[-2] > 8))
 				break;
 
 			stkPos -= 2;
@@ -333,7 +342,8 @@ int16 Parse_v2::parseValExpr(unsigned stopToken) {
 
 		if (operation != 10) {
 			if (operation != stopToken) {
-				debugC(5, DEBUG_PARSER, "stoptoken error: %d != %d", operation, stopToken);
+				debugC(5, kDebugParser, "stoptoken error: %d != %d",
+						operation, stopToken);
 			}
 			flag = oldflag;
 			return values[0];
@@ -345,30 +355,30 @@ int16 Parse_v2::parseValExpr(unsigned stopToken) {
 	}
 }
 
-int16 Parse_v2::parseExpr(char stopToken, byte *arg_2) {
+int16 Parse_v2::parseExpr(byte stopToken, byte *arg_2) {
 	int32 values[20];
 	byte operStack[20];
 	int32 prevPrevVal;
 	int32 prevVal;
 	int32 curVal;
 	int32 *valPtr;
-	char *operPtr;
+	byte *operPtr;
 	byte *arrDescPtr;
-	char var_C;
+	byte var_C;
 	byte operation;
 	int16 dimCount;
 	int16 temp;
 	int16 temp2;
-	uint16 offset;
+	int16 offset;
 	int16 dim;
-	char var_1A;
+	bool var_1A;
 	int16 stkPos;
 	int16 brackStart;
 
 	memset(operStack, 0, 20);
 
 	stkPos = -1;
-	operPtr = (char *)(operStack - 1);
+	operPtr = operStack - 1;
 	valPtr = values - 1;
 
 	while (1) {
@@ -376,19 +386,16 @@ int16 Parse_v2::parseExpr(char stopToken, byte *arg_2) {
 		operPtr++;
 		valPtr++;
 		operation = *_vm->_global->_inter_execPtr++;
-		if (operation >= 19 && operation <= 29) {
+		if ((operation >= 16) && (operation <= 29)) {
 			switch (operation) {
 			case 16:
 			case 26:
 			case 27:
 			case 28:
-				if ((operation == 27) || (operation == 16))
-					*operPtr = 20;
-				else
-					*operPtr = operation - 6;
+				*operPtr = (operation == 28) ? 22 : 20;
 				temp = _vm->_inter->load16();
 				dimCount = *_vm->_global->_inter_execPtr++;
-				arrDescPtr = (byte *)_vm->_global->_inter_execPtr;
+				arrDescPtr = _vm->_global->_inter_execPtr;
 				_vm->_global->_inter_execPtr += dimCount;
 				offset = 0;
 				for (dim = 0; dim < dimCount; dim++) {
@@ -396,30 +403,33 @@ int16 Parse_v2::parseExpr(char stopToken, byte *arg_2) {
 					offset = offset * arrDescPtr[dim] + temp2;
 				}
 				if (operation == 16)
-					*valPtr = *(_vm->_global->_inter_variables + temp + offset);
+					*valPtr = (int8) READ_VARO_UINT8(temp + offset);
 				else if (operation == 26)
-					*valPtr = *(uint32*)(_vm->_global->_inter_variables + temp * 4 + offset * 4);
+					*valPtr = READ_VARO_UINT32(temp * 4 + offset * 4);
 				else if (operation == 27)
-					*valPtr = *(uint16*)(_vm->_global->_inter_variables + temp * 2 + offset * 2);
+					*valPtr = (int16) READ_VARO_UINT16(temp * 2 + offset * 2);
 				else if (operation == 28) {
-					*valPtr = encodePtr(_vm->_global->_inter_variables + temp * 4 + offset * _vm->_global->_inter_animDataSize * 4, kInterVar);
+					*valPtr = encodePtr(_vm->_global->_inter_variables +
+							temp * 4 + offset * _vm->_global->_inter_animDataSize * 4,
+							kInterVar);
 					if (*_vm->_global->_inter_execPtr == 13) {
 						_vm->_global->_inter_execPtr++;
 						temp2 = parseValExpr(12);
 						*operPtr = 20;
-						*valPtr = (uint8)*(_vm->_global->_inter_variables + temp * 4 + offset * 4 * _vm->_global->_inter_animDataSize + temp2);
+						*valPtr = READ_VARO_UINT8(temp * 4 +
+								offset * 4 * _vm->_global->_inter_animDataSize + temp2);
 					}
 				}
 				break;
 
 			case 17:
 				*operPtr = 20;
-				*valPtr = *(uint16*)(_vm->_global->_inter_variables + _vm->_inter->load16() * 2);
+				*valPtr = (int16) READ_VARO_UINT16(_vm->_inter->load16() * 2);
 				break;
 
 			case 18:
 				*operPtr = 20;
-				*valPtr = *(_vm->_global->_inter_variables + _vm->_inter->load16());
+				*valPtr = (int8) READ_VARO_UINT8(_vm->_inter->load16());
 				break;
 
 			case 19:
@@ -435,21 +445,26 @@ int16 Parse_v2::parseExpr(char stopToken, byte *arg_2) {
 
 			case 21:
 				*operPtr = 20;
-				*valPtr = *((int8 *) _vm->_global->_inter_execPtr++);
+				*valPtr = (int8) *_vm->_global->_inter_execPtr++;
 				break;
 
 			case 22:
 				*operPtr = 22;
 				*valPtr = encodePtr(_vm->_global->_inter_execPtr, kExecPtr);
-				_vm->_global->_inter_execPtr += strlen(_vm->_global->_inter_execPtr) + 1;
+				_vm->_global->_inter_execPtr +=
+					strlen((char *) _vm->_global->_inter_execPtr) + 1;
 				break;
 
 			case 23:
-			case 24:
 				*operPtr = 20;
 				*valPtr = VAR(_vm->_inter->load16());
 				break;
 
+			case 24:
+				*operPtr = 20;
+				*valPtr = (int16) READ_VARO_UINT16(_vm->_inter->load16() * 4);
+				break;
+
 			case 25:
 				*operPtr = 22;
 				temp = _vm->_inter->load16() * 4;
@@ -458,7 +473,7 @@ int16 Parse_v2::parseExpr(char stopToken, byte *arg_2) {
 					_vm->_global->_inter_execPtr++;
 					temp += parseValExpr(12);
 					*operPtr = 20;
-					*valPtr = (uint8)*(_vm->_global->_inter_variables + temp);
+					*valPtr = READ_VARO_UINT8(temp);
 				}
 				break;
 
@@ -467,10 +482,6 @@ int16 Parse_v2::parseExpr(char stopToken, byte *arg_2) {
 				parseExpr(10, 0);
 
 				switch (operation) {
-				case 5:
-					_vm->_global->_inter_resVal = _vm->_global->_inter_resVal * _vm->_global->_inter_resVal;
-					break;
-
 				case 0:
 				case 1:
 				case 6:
@@ -481,24 +492,32 @@ int16 Parse_v2::parseExpr(char stopToken, byte *arg_2) {
 						prevPrevVal = prevVal;
 						prevVal = curVal;
 						curVal = (curVal + _vm->_global->_inter_resVal / curVal) / 2;
-					} while (curVal != prevVal && curVal != prevPrevVal);
+					} while ((curVal != prevVal) && (curVal != prevPrevVal));
 					_vm->_global->_inter_resVal = curVal;
 					break;
 
-				case 10:
-					_vm->_global->_inter_resVal = _vm->_util->getRandom(_vm->_global->_inter_resVal);
+				case 5:
+					_vm->_global->_inter_resVal =
+						_vm->_global->_inter_resVal * _vm->_global->_inter_resVal;
 					break;
 
 				case 7:
 					if (_vm->_global->_inter_resVal < 0)
 						_vm->_global->_inter_resVal = -_vm->_global->_inter_resVal;
 					break;
+
+				case 10:
+					_vm->_global->_inter_resVal =
+						_vm->_util->getRandom(_vm->_global->_inter_resVal);
+					break;
 				}
+
 				*operPtr = 20;
 				*valPtr = _vm->_global->_inter_resVal;
 				break;
 			}
-			if (stkPos > 0 && (operPtr[-1] == 1 || operPtr[-1] == 11)) {
+
+			if ((stkPos > 0) && ((operPtr[-1] == 1) || (operPtr[-1] == 11))) {
 				stkPos--;
 				operPtr--;
 				valPtr--;
@@ -506,12 +525,8 @@ int16 Parse_v2::parseExpr(char stopToken, byte *arg_2) {
 				if (*operPtr == 1) {
 					*operPtr = 20;
 					valPtr[0] = -valPtr[1];
-				} else if (*operPtr == 11) {
-					if (operPtr[1] == 23)
-						*operPtr = 24;
-					else
-						*operPtr = 23;
-				}
+				} else
+					*operPtr = (operPtr[1] == 23) ? 24 : 23;
 			}
 
 			if (stkPos <= 0)
@@ -520,11 +535,11 @@ int16 Parse_v2::parseExpr(char stopToken, byte *arg_2) {
 			switch (operPtr[-1]) {
 			case 2:
 				if (operPtr[-2] == 22) {
-					if (decodePtr(valPtr[-2]) != _vm->_global->_inter_resStr) {
-						strcpy(_vm->_global->_inter_resStr, decodePtr(valPtr[-2]));
-						valPtr[-2] = encodePtr(_vm->_global->_inter_resStr, kResStr);
+					if ((char *) decodePtr(valPtr[-2]) != _vm->_global->_inter_resStr) {
+						strcpy(_vm->_global->_inter_resStr, (char *) decodePtr(valPtr[-2]));
+						valPtr[-2] = encodePtr((byte *) _vm->_global->_inter_resStr, kResStr);
 					}
-					strcat(_vm->_global->_inter_resStr, decodePtr(valPtr[0]));
+					strcat(_vm->_global->_inter_resStr, (char *) decodePtr(valPtr[0]));
 					stkPos -= 2;
 					operPtr -= 2;
 					valPtr -= 2;
@@ -560,14 +575,16 @@ int16 Parse_v2::parseExpr(char stopToken, byte *arg_2) {
 				break;
 			}
 			continue;
-		}		// op>= 19 && op <= 29
+		} // (op >= 16) && (op <= 29)
 
-		if (operation == stopToken || operation == 30 || operation == 31 || operation == 10) {
+		if ((operation == stopToken) || (operation == 30) ||
+				(operation == 31) || (operation == 10)) {
 			while (stkPos >= 2) {
-				var_1A = 0;
-				if (operPtr[-2] == 9 && (operation == 10 || operation == stopToken)) {
+				var_1A = false;
+				if ((operPtr[-2] == 9) &&
+						((operation == 10) || (operation == stopToken))) {
 					operPtr[-2] = operPtr[-1];
-					if (operPtr[-2] == 20 || operPtr[-2] == 22)
+					if ((operPtr[-2] == 20) || (operPtr[-2] == 22))
 						valPtr[-2] = valPtr[-1];
 
 					stkPos--;
@@ -582,11 +599,7 @@ int16 Parse_v2::parseExpr(char stopToken, byte *arg_2) {
 							operPtr--;
 							valPtr--;
 						} else if (operPtr[-2] == 11) {
-							if (operPtr[-1] == 23)
-								operPtr[-2] = 24;
-							else
-								operPtr[-2] = 23;
-
+							operPtr[-2] = (operPtr[-1] == 23) ? 24 : 23;
 							stkPos--;
 							operPtr--;
 							valPtr--;
@@ -601,6 +614,7 @@ int16 Parse_v2::parseExpr(char stopToken, byte *arg_2) {
 							operPtr -= 2;
 							valPtr -= 2;
 							break;
+
 						case 6:
 							valPtr[-3] /= valPtr[-1];
 							stkPos -= 2;
@@ -626,13 +640,13 @@ int16 Parse_v2::parseExpr(char stopToken, byte *arg_2) {
 
 					if (operation != stopToken)
 						break;
-				}	// if (operPtr[-2] == 9 && ...)
+				}	// if ((operPtr[-2] == 9) && ...)
 
-				for (brackStart = stkPos - 2; brackStart > 0 &&
-				    operStack[brackStart] < 30 &&
-				    operStack[brackStart] != 9; brackStart--);
+				for (brackStart = (stkPos - 2); (brackStart > 0) &&
+				    (operStack[brackStart] < 30) && (operStack[brackStart] != 9);
+						brackStart--);
 
-				if (operStack[brackStart] >= 30 || operStack[brackStart] == 9)
+				if ((operStack[brackStart] >= 30) || (operStack[brackStart] == 9))
 					brackStart++;
 
 				switch (operPtr[-2]) {
@@ -640,11 +654,12 @@ int16 Parse_v2::parseExpr(char stopToken, byte *arg_2) {
 					if (operStack[brackStart] == 20) {
 						values[brackStart] += valPtr[-1];
 					} else if (operStack[brackStart] == 22) {
-						if (decodePtr(values[brackStart]) != _vm->_global->_inter_resStr) {
-							strcpy(_vm->_global->_inter_resStr, decodePtr(values[brackStart]));
-							values[brackStart] = encodePtr(_vm->_global->_inter_resStr, kResStr);
+						if ((char *) decodePtr(values[brackStart]) != _vm->_global->_inter_resStr) {
+							strcpy(_vm->_global->_inter_resStr, (char *) decodePtr(values[brackStart]));
+							values[brackStart] =
+								encodePtr((byte *) _vm->_global->_inter_resStr, kResStr);
 						}
-						strcat(_vm->_global->_inter_resStr, decodePtr(valPtr[-1]));
+						strcat(_vm->_global->_inter_resStr, (char *) decodePtr(valPtr[-1]));
 					}
 					stkPos -= 2;
 					operPtr -= 2;
@@ -716,11 +731,11 @@ int16 Parse_v2::parseExpr(char stopToken, byte *arg_2) {
 						if (valPtr[-3] < valPtr[-1])
 							operPtr[-3] = 24;
 					} else if (var_C == 22) {
-						if (decodePtr(valPtr[-3]) != _vm->_global->_inter_resStr) {
-							strcpy(_vm->_global->_inter_resStr, decodePtr(valPtr[-3]));
-							valPtr[-3] = encodePtr(_vm->_global->_inter_resStr, kResStr);
+						if ((char *) decodePtr(valPtr[-3]) != _vm->_global->_inter_resStr) {
+							strcpy(_vm->_global->_inter_resStr, (char *) decodePtr(valPtr[-3]));
+							valPtr[-3] = encodePtr((byte *) _vm->_global->_inter_resStr, kResStr);
 						}
-						if (strcmp(_vm->_global->_inter_resStr, decodePtr(valPtr[-1])) < 0)
+						if (strcmp(_vm->_global->_inter_resStr, (char *) decodePtr(valPtr[-1])) < 0)
 							operPtr[-3] = 24;
 					}
 					stkPos -= 2;
@@ -735,11 +750,11 @@ int16 Parse_v2::parseExpr(char stopToken, byte *arg_2) {
 						if (valPtr[-3] <= valPtr[-1])
 							operPtr[-3] = 24;
 					} else if (var_C == 22) {
-						if (decodePtr(valPtr[-3]) != _vm->_global->_inter_resStr) {
-							strcpy(_vm->_global->_inter_resStr, decodePtr(valPtr[-3]));
-							valPtr[-3] = encodePtr(_vm->_global->_inter_resStr, kResStr);
+						if ((char *) decodePtr(valPtr[-3]) != _vm->_global->_inter_resStr) {
+							strcpy(_vm->_global->_inter_resStr, (char *) decodePtr(valPtr[-3]));
+							valPtr[-3] = encodePtr((byte *) _vm->_global->_inter_resStr, kResStr);
 						}
-						if (strcmp(_vm->_global->_inter_resStr, decodePtr(valPtr[-1])) <= 0)
+						if (strcmp(_vm->_global->_inter_resStr, (char *) decodePtr(valPtr[-1])) <= 0)
 							operPtr[-3] = 24;
 					}
 					stkPos -= 2;
@@ -754,11 +769,11 @@ int16 Parse_v2::parseExpr(char stopToken, byte *arg_2) {
 						if (valPtr[-3] > valPtr[-1])
 							operPtr[-3] = 24;
 					} else if (var_C == 22) {
-						if (decodePtr(valPtr[-3]) != _vm->_global->_inter_resStr) {
-							strcpy(_vm->_global->_inter_resStr, decodePtr(valPtr[-3]));
-							valPtr[-3] = encodePtr(_vm->_global->_inter_resStr, kResStr);
+						if ((char *) decodePtr(valPtr[-3]) != _vm->_global->_inter_resStr) {
+							strcpy(_vm->_global->_inter_resStr, (char *) decodePtr(valPtr[-3]));
+							valPtr[-3] = encodePtr((byte *) _vm->_global->_inter_resStr, kResStr);
 						}
-						if (strcmp(_vm->_global->_inter_resStr, decodePtr(valPtr[-1])) > 0)
+						if (strcmp(_vm->_global->_inter_resStr, (char *) decodePtr(valPtr[-1])) > 0)
 							operPtr[-3] = 24;
 					}
 					stkPos -= 2;
@@ -773,11 +788,11 @@ int16 Parse_v2::parseExpr(char stopToken, byte *arg_2) {
 						if (valPtr[-3] >= valPtr[-1])
 							operPtr[-3] = 24;
 					} else if (var_C == 22) {
-						if (decodePtr(valPtr[-3]) != _vm->_global->_inter_resStr) {
-							strcpy(_vm->_global->_inter_resStr, decodePtr(valPtr[-3]));
-							valPtr[-3] = encodePtr(_vm->_global->_inter_resStr, kResStr);
+						if ((char *) decodePtr(valPtr[-3]) != _vm->_global->_inter_resStr) {
+							strcpy(_vm->_global->_inter_resStr, (char *) decodePtr(valPtr[-3]));
+							valPtr[-3] = encodePtr((byte *) _vm->_global->_inter_resStr, kResStr);
 						}
-						if (strcmp(_vm->_global->_inter_resStr, decodePtr(valPtr[-1])) >= 0)
+						if (strcmp(_vm->_global->_inter_resStr, (char *) decodePtr(valPtr[-1])) >= 0)
 							operPtr[-3] = 24;
 					}
 					stkPos -= 2;
@@ -792,11 +807,11 @@ int16 Parse_v2::parseExpr(char stopToken, byte *arg_2) {
 						if (valPtr[-3] == valPtr[-1])
 							operPtr[-3] = 24;
 					} else if (var_C == 22) {
-						if (decodePtr(valPtr[-3]) != _vm->_global->_inter_resStr) {
-							strcpy(_vm->_global->_inter_resStr, decodePtr(valPtr[-3]));
-							valPtr[-3] = encodePtr(_vm->_global->_inter_resStr, kResStr);
+						if ((char *) decodePtr(valPtr[-3]) != _vm->_global->_inter_resStr) {
+							strcpy(_vm->_global->_inter_resStr, (char *) decodePtr(valPtr[-3]));
+							valPtr[-3] = encodePtr((byte *) _vm->_global->_inter_resStr, kResStr);
 						}
-						if (strcmp(_vm->_global->_inter_resStr, decodePtr(valPtr[-1])) == 0)
+						if (strcmp(_vm->_global->_inter_resStr, (char *) decodePtr(valPtr[-1])) == 0)
 							operPtr[-3] = 24;
 					}
 					stkPos -= 2;
@@ -811,11 +826,12 @@ int16 Parse_v2::parseExpr(char stopToken, byte *arg_2) {
 						if (valPtr[-3] != valPtr[-1])
 							operPtr[-3] = 24;
 					} else if (var_C == 22) {
-						if (decodePtr(valPtr[-3]) != _vm->_global->_inter_resStr) {
-							strcpy(_vm->_global->_inter_resStr, decodePtr(valPtr[-3]));
-							valPtr[-3] = encodePtr(_vm->_global->_inter_resStr, kResStr);
+						if ((char *) decodePtr(valPtr[-3]) != _vm->_global->_inter_resStr) {
+							strcpy(_vm->_global->_inter_resStr, (char *) decodePtr(valPtr[-3]));
+							valPtr[-3] = encodePtr((byte *) _vm->_global->_inter_resStr, kResStr);
 						}
-						if (strcmp(_vm->_global->_inter_resStr, decodePtr(valPtr[-1])) != 0)
+						if (scumm_stricmp(_vm->_global->_inter_resStr,
+									(char *) decodePtr(valPtr[-1])) != 0)
 							operPtr[-3] = 24;
 					}
 					stkPos -= 2;
@@ -824,15 +840,15 @@ int16 Parse_v2::parseExpr(char stopToken, byte *arg_2) {
 					break;
 
 				default:
-					var_1A = 1;
+					var_1A = true;
 					break;
 				}	// switch
 
-				if (var_1A != 0)
+				if (var_1A)
 					break;
 			}	// while (stkPos >= 2)
 
-			if (operation == 30 || operation == 31) {
+			if ((operation == 30) || (operation == 31)) {
 				if (operPtr[-1] == 20) {
 					if (valPtr[-1] != 0)
 						operPtr[-1] = 24;
@@ -840,9 +856,9 @@ int16 Parse_v2::parseExpr(char stopToken, byte *arg_2) {
 						operPtr[-1] = 23;
 				}
 
-				if ((operation == 30 && operPtr[-1] == 24) ||
-				    (operation == 31 && operPtr[-1] == 23)) {
-					if (stkPos > 1 && operPtr[-2] == 9) {
+				if (((operation == 30) && (operPtr[-1] == 24)) ||
+				    ((operation == 31) && (operPtr[-1] == 23))) {
+					if ((stkPos > 1) && (operPtr[-2] == 9)) {
 						skipExpr(10);
 						operPtr[-2] = operPtr[-1];
 						stkPos -= 2;
@@ -852,7 +868,7 @@ int16 Parse_v2::parseExpr(char stopToken, byte *arg_2) {
 						skipExpr(stopToken);
 					}
 					operation = _vm->_global->_inter_execPtr[-1];
-					if (stkPos > 0 && operPtr[-1] == 11) {
+					if ((stkPos > 0) && (operPtr[-1] == 11)) {
 						if (operPtr[0] == 23)
 							operPtr[-1] = 24;
 						else
@@ -877,18 +893,18 @@ int16 Parse_v2::parseExpr(char stopToken, byte *arg_2) {
 				*arg_2 = operStack[0];
 
 			switch (operStack[0]) {
+			case 11:
+				if (arg_2 != 0)
+					*arg_2 ^= 1;
+				break;
+
 			case 20:
 				_vm->_global->_inter_resVal = values[0];
 				break;
 
 			case 22:
-				if (decodePtr(values[0]) != _vm->_global->_inter_resStr)
-					strcpy(_vm->_global->_inter_resStr, decodePtr(values[0]));
-				break;
-
-			case 11:
-				if (arg_2 != 0)
-					*arg_2 ^= 1;
+				if ((char *) decodePtr(values[0]) != _vm->_global->_inter_resStr)
+					strcpy(_vm->_global->_inter_resStr, (char *) decodePtr(values[0]));
 				break;
 
 			case 23:
@@ -902,10 +918,10 @@ int16 Parse_v2::parseExpr(char stopToken, byte *arg_2) {
 				break;
 			}
 			return 0;
-		}		// operation == stopToken || operation == 30 || operation == 31 || operation == 10
+		}		// (operation == stopToken) || (operation == 30) || (operation == 31) || (operation == 10)
 
-		if (operation < 1 || operation > 11) {
-			if (operation < 32 || operation > 37)
+		if ((operation < 1) || (operation > 11)) {
+			if ((operation < 32) || (operation > 37))
 				continue;
 
 			if (stkPos > 2) {
@@ -913,11 +929,11 @@ int16 Parse_v2::parseExpr(char stopToken, byte *arg_2) {
 					if (operPtr[-3] == 20) {
 						valPtr[-3] += valPtr[-1];
 					} else if (operPtr[-3] == 22) {
-						if (decodePtr(valPtr[-3]) != _vm->_global->_inter_resStr) {
-							strcpy(_vm->_global->_inter_resStr, decodePtr(valPtr[-3]));
-							valPtr[-3] = encodePtr(_vm->_global->_inter_resStr, kResStr);
+						if ((char *) decodePtr(valPtr[-3]) != _vm->_global->_inter_resStr) {
+							strcpy(_vm->_global->_inter_resStr, (char *) decodePtr(valPtr[-3]));
+							valPtr[-3] = encodePtr((byte *) _vm->_global->_inter_resStr, kResStr);
 						}
-						strcat(_vm->_global->_inter_resStr, decodePtr(valPtr[-1]));
+						strcat(_vm->_global->_inter_resStr, (char *) decodePtr(valPtr[-1]));
 					}
 					stkPos -= 2;
 					operPtr -= 2;
