@@ -1,6 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2004 Ivan Dubrov
- * Copyright (C) 2004-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -16,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/gob/scenery.cpp $
- * $Id: scenery.cpp 23088 2006-06-13 17:02:53Z drmccoy $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/gob/scenery.cpp $
+ * $Id: scenery.cpp 27486 2007-06-16 20:29:39Z chrilith $
  *
  */
 
@@ -27,29 +29,20 @@
 
 #include "gob/gob.h"
 #include "gob/scenery.h"
-#include "gob/inter.h"
-#include "gob/video.h"
+#include "gob/global.h"
 #include "gob/draw.h"
 #include "gob/game.h"
-#include "gob/global.h"
-#include "gob/util.h"
-#include "gob/parse.h"
-#include "gob/cdrom.h"
+#include "gob/inter.h"
 
 namespace Gob {
 
 Scenery::Scenery(GobEngine *vm) : _vm(vm) {
-	int i;
-
-	for (i = 0; i < 20; i++) {
+	for (int i = 0; i < 20; i++) {
 		_spriteRefs[i] = 0;
 		_spriteResId[i] = 0;
 	}
-	for (i = 0; i < 70; i++ ) {
-		_staticPictToSprite[i] = 0;
-		_animPictToSprite[i] = 0;
-	}
-	for (i = 0; i < 10; i++) {
+
+	for (int i = 0; i < 10; i++) {
 		_staticPictCount[i] = 0;
 		_staticResId[i] = 0;
 		_animPictCount[i] = 0;
@@ -68,35 +61,63 @@ Scenery::Scenery(GobEngine *vm) : _vm(vm) {
 	_animLeft = 0;
 
 	_pCaptureCounter = 0;
+
+	for (int i = 0; i < 70; i++ ) {
+		_staticPictToSprite[i] = 0;
+		_animPictToSprite[i] = 0;
+	}
+}
+
+Scenery::~Scenery() {
+	for (int i = 0; i < 10; i++) {
+		freeStatic(i);
+		freeAnim(i);
+	}
+}
+
+void Scenery::init() {
+	for (int i = 0; i < 10; i++) {
+		_animPictCount[i] = 0;
+		_staticPictCount[i] = -1;
+	}
+
+	for (int i = 0; i < 20; i++) {
+		_spriteRefs[i] = 0;
+		_spriteResId[i] = -1;
+	}
+
+	_curStaticLayer = -1;
+	_curStatic = -1;
 }
 
 int16 Scenery::loadStatic(char search) {
-	int16 tmp;
+	int16 size;
 	int16 *backsPtr;
 	int16 picsCount;
 	int16 resId;
-	int16 i;
 	int16 sceneryIndex;
-	char *extData;
-	char *dataPtr;
+	byte *extData = 0;
+	byte *dataPtr;
 	Static *ptr;
-	int16 offset;
 	int16 pictDescId;
 	int16 width;
 	int16 height;
 	int16 sprResId;
 	int16 sprIndex;
 
-	extData = 0;
 	_vm->_inter->evalExpr(&sceneryIndex);
-	tmp = _vm->_inter->load16();
-	backsPtr = (int16 *)_vm->_global->_inter_execPtr;
-	_vm->_global->_inter_execPtr += tmp * 2;
+
+	size = _vm->_inter->load16();
+	backsPtr = (int16 *) _vm->_global->_inter_execPtr;
+	_vm->_global->_inter_execPtr += size * 2;
 	picsCount = _vm->_inter->load16();
 	resId = _vm->_inter->load16();
+
 	if (search) {
+		int i;
+
 		for (i = 0; i < 10; i++) {
-			if (_staticPictCount[i] != -1 && _staticResId[i] == resId) {
+			if ((_staticPictCount[i] != -1) && (_staticResId[i] == resId)) {
 				_vm->_global->_inter_execPtr += 8 * _staticPictCount[i];
 				return i;
 			}
@@ -117,16 +138,16 @@ int16 Scenery::loadStatic(char search) {
 
 	ptr = &_statics[sceneryIndex];
 
-	ptr->layersCount = (int16)READ_LE_UINT16(dataPtr);
+	ptr->layersCount = (int16) READ_LE_UINT16(dataPtr);
 	dataPtr += 2;
 
 	ptr->layers = new StaticLayer[ptr->layersCount];
 	ptr->pieces = new PieceDesc*[picsCount];
-	ptr->piecesFromExt = new int8[picsCount];
+	ptr->piecesFromExt = new bool[picsCount];
 
-	for (i = 0; i < ptr->layersCount; i++) {
-		offset = (int16)READ_LE_UINT16(&((int16 *)dataPtr)[i]);
-		Common::MemoryReadStream layerData((byte *) (dataPtr + offset), 4294967295U);
+	for (int i = 0; i < ptr->layersCount; i++) {
+		int16 offset = READ_LE_UINT16(dataPtr + i * 2);
+		Common::MemoryReadStream layerData(dataPtr + offset, 4294967295U);
 
 		ptr->layers[i].planeCount = layerData.readSint16LE();
 
@@ -140,22 +161,21 @@ int16 Scenery::loadStatic(char search) {
 			ptr->layers[i].planes[j].transp = layerData.readSByte();
 		}
 
-		ptr->layers[i].backResId = (int16)READ_LE_UINT16(backsPtr);
+		ptr->layers[i].backResId = (int16) READ_LE_UINT16(backsPtr);
 		backsPtr++;
 	}
 
-	for (i = 0; i < picsCount; i++) {
+	for (int i = 0; i < picsCount; i++) {
 		pictDescId = _vm->_inter->load16();
+
 		if (pictDescId >= 30000) {
 			ptr->pieces[i] =
-			    (PieceDesc *) _vm->_game->loadExtData(pictDescId, 0,
-			    0);
-			ptr->piecesFromExt[i] = 1;
+				(PieceDesc *) _vm->_game->loadExtData(pictDescId, 0, 0);
+			ptr->piecesFromExt[i] = true;
 		} else {
 			ptr->pieces[i] =
-			    (PieceDesc *)
-			    _vm->_game->loadTotResource(pictDescId);
-			ptr->piecesFromExt[i] = 0;
+				(PieceDesc *) _vm->_game->loadTotResource(pictDescId);
+			ptr->piecesFromExt[i] = false;
 		}
 
 		width = _vm->_inter->load16();
@@ -167,19 +187,16 @@ int16 Scenery::loadStatic(char search) {
 		}
 
 		if (sprIndex < 20) {
-			_staticPictToSprite[7 * sceneryIndex + i] =
-			    sprIndex;
+			_staticPictToSprite[7 * sceneryIndex + i] = sprIndex;
 			_spriteRefs[sprIndex]++;
 		} else {
 			for (sprIndex = 19; _vm->_draw->_spritesArray[sprIndex] != 0;
-			    sprIndex--);
+				sprIndex--);
 
-			_staticPictToSprite[7 * sceneryIndex + i] =
-			    sprIndex;
+			_staticPictToSprite[7 * sceneryIndex + i] = sprIndex;
 			_spriteRefs[sprIndex] = 1;
 			_spriteResId[sprIndex] = sprResId;
-			_vm->_draw->_spritesArray[sprIndex] =
-			    _vm->_video->initSurfDesc(_vm->_global->_videoMode, width, height, 2);
+			_vm->_draw->initSpriteSurf(sprIndex, width, height, 2);
 
 			_vm->_video->clearSurf(_vm->_draw->_spritesArray[sprIndex]);
 			_vm->_draw->_destSurface = sprIndex;
@@ -190,13 +207,13 @@ int16 Scenery::loadStatic(char search) {
 			_vm->_draw->spriteOperation(DRAW_LOADSPRITE);
 		}
 	}
-	if (extData != 0)
-		delete[] extData;
+
+	delete[] extData;
+
 	return sceneryIndex + 100;
 }
 
 void Scenery::freeStatic(int16 index) {
-	int16 i;
 	int16 spr;
 
 	if (index == -1)
@@ -205,25 +222,25 @@ void Scenery::freeStatic(int16 index) {
 	if (_staticPictCount[index] == -1)
 		return;
 
-	for (i = 0; i < _staticPictCount[index]; i++) {
-		if (_statics[index].piecesFromExt[i] == 1)
+	for (int i = 0; i < _staticPictCount[index]; i++) {
+		if (_statics[index].piecesFromExt[i])
 			delete[] _statics[index].pieces[i];
 
 		spr = _staticPictToSprite[index * 7 + i];
 		_spriteRefs[spr]--;
 		if (_spriteRefs[spr] == 0) {
-			_vm->_video->freeSurfDesc(_vm->_draw->_spritesArray[spr]);
-			_vm->_draw->_spritesArray[spr] = 0;
+			_vm->_draw->freeSprite(spr);
 			_spriteResId[spr] = -1;
 		}
 	}
 
-	for (i = 0; i < _statics[index].layersCount; i++)
+	for (int i = 0; i < _statics[index].layersCount; i++)
 		delete[] _statics[index].layers[i].planes;
 	delete[] _statics[index].layers;
 	delete[] _statics[index].pieces;
 	delete[] _statics[index].piecesFromExt;
 
+	_statics[index].layersCount = 0;
 	_staticPictCount[index] = -1;
 }
 
@@ -259,7 +276,7 @@ void Scenery::renderStatic(int16 scenery, int16 layer) {
 	}
 
 	planeCount = layerPtr->planeCount;
-	for (order = 0; order < 10; order++) {
+	for (order = 0; order < 40; order++) {
 		for (plane = 0, planePtr = layerPtr->planes;
 		    plane < planeCount; plane++, planePtr++) {
 			if (planePtr->drawOrder != order)
@@ -288,7 +305,7 @@ void Scenery::renderStatic(int16 scenery, int16 layer) {
 	}
 }
 
-void Scenery::updateStatic(int16 orderFrom) {
+void Scenery::updateStatic(int16 orderFrom, byte index, byte layer) {
 	StaticLayer *layerPtr;
 	PieceDesc **pictPtr;
 	StaticPlane *planePtr;
@@ -303,18 +320,15 @@ void Scenery::updateStatic(int16 orderFrom) {
 	int16 top;
 	int16 bottom;
 
-	if (_curStatic == -1)
+	if (layer >= _statics[index].layersCount)
 		return;
 
-	if (_curStaticLayer >= _statics[_curStatic].layersCount)
-		return;
-
-	layerPtr = &_statics[_curStatic].layers[_curStaticLayer];
-	pictPtr = _statics[_curStatic].pieces;
+	layerPtr = &_statics[index].layers[layer];
+	pictPtr = _statics[index].pieces;
 
 	planeCount = layerPtr->planeCount;
 
-	for (order = orderFrom; order < 10; order++) {
+	for (order = orderFrom; order < 40; order++) {
 		for (planePtr = layerPtr->planes, plane = 0;
 		    plane < planeCount; plane++, planePtr++) {
 			if (planePtr->drawOrder != order)
@@ -351,22 +365,22 @@ void Scenery::updateStatic(int16 orderFrom) {
 			_vm->_draw->_spriteRight = right - left + 1;
 			_vm->_draw->_spriteBottom = bottom - top + 1;
 
-			if (_vm->_draw->_spriteRight <= 0 || _vm->_draw->_spriteBottom <= 0)
+			if ((_vm->_draw->_spriteRight <= 0) ||
+			    (_vm->_draw->_spriteBottom <= 0))
 				continue;
 
-			if (_vm->_draw->_destSpriteX + _vm->_draw->_spriteRight - 1 >
+			if ((_vm->_draw->_destSpriteX + _vm->_draw->_spriteRight - 1) >
 			    _toRedrawRight)
 				_vm->_draw->_spriteRight =
 				    _toRedrawRight - _vm->_draw->_destSpriteX + 1;
 
-			if (_vm->_draw->_destSpriteY + _vm->_draw->_spriteBottom - 1 >
+			if ((_vm->_draw->_destSpriteY + _vm->_draw->_spriteBottom - 1) >
 			    _toRedrawBottom)
 				_vm->_draw->_spriteBottom =
 				    _toRedrawBottom - _vm->_draw->_destSpriteY + 1;
 
 			_vm->_draw->_sourceSurface =
-			    _staticPictToSprite[_curStatic * 7 +
-			    pictIndex];
+			    _staticPictToSprite[index * 7 + pictIndex];
 			_vm->_draw->_destSurface = 21;
 			_vm->_draw->_transparency = planePtr->transp ? 3 : 0;
 			_vm->_draw->spriteOperation(DRAW_BLITSURF);
@@ -374,6 +388,17 @@ void Scenery::updateStatic(int16 orderFrom) {
 	}
 }
 
+void Scenery::updateStatic(int16 orderFrom) {
+	if (_curStatic == -1)
+		return;
+
+	updateStatic(orderFrom, _curStatic & 0xFF, _curStaticLayer & 0xFF);
+
+	if (_curStatic & 0xFF00)
+		updateStatic(orderFrom, ((_curStatic >> 8) & 0xFF) - 1,
+				(_curStaticLayer >> 8) & 0xFF);
+}
+
 int16 Scenery::loadAnim(char search) {
 	int16 picsCount;
 	int16 resId;
@@ -381,10 +406,9 @@ int16 Scenery::loadAnim(char search) {
 	int16 j;
 	int16 sceneryIndex;
 	int16 framesCount;
-	char *extData;
-	char *dataPtr;
+	byte *extData;
+	byte *dataPtr;
 	Animation *ptr;
-	int16 offset;
 	int16 pictDescId;
 	int16 width;
 	int16 height;
@@ -393,26 +417,18 @@ int16 Scenery::loadAnim(char search) {
 	uint32 layerPos;
 
 	extData = 0;
-	if (_vm->_cdrom->_cdPlaying) {
-		while (_vm->_cdrom->getTrackPos() != -1)
-		    _vm->_util->longDelay(50);
-
-		_vm->_cdrom->_cdPlaying = false;
-	}
-
 	_vm->_inter->evalExpr(&sceneryIndex);
 	picsCount = _vm->_inter->load16();
 	resId = _vm->_inter->load16();
 
 	if (search) {
 		for (i = 0; i < 10; i++) {
-			if (_animPictCount[i] != 0
-			    && _animResId[i] == resId) {
+			if ((_animPictCount[i] != 0) && (_animResId[i] == resId)) {
 				_vm->_global->_inter_execPtr += 8 * _animPictCount[i];
 				return i;
 			}
 
-			if (_animPictCount[i] == 0 && i < sceneryIndex)
+			if ((_animPictCount[i] == 0) && (i < sceneryIndex))
 				sceneryIndex = i;
 		}
 	}
@@ -433,11 +449,11 @@ int16 Scenery::loadAnim(char search) {
 
 	ptr->layers = new AnimLayer[ptr->layersCount];
 	ptr->pieces = new PieceDesc*[picsCount];
-	ptr->piecesFromExt = new int8[picsCount];
+	ptr->piecesFromExt = new bool[picsCount];
 
 	for (i = 0; i < ptr->layersCount; i++) {
-		offset = (int16)READ_LE_UINT16(&((int16 *)dataPtr)[i]);
-		Common::MemoryReadStream layerData((byte *) (dataPtr + offset - 2), 4294967295U);
+		int16 offset = READ_LE_UINT16(dataPtr + i * 2);
+		Common::MemoryReadStream layerData(dataPtr + offset - 2, 4294967295U);
 
 		ptr->layers[i].unknown0 = layerData.readSint16LE();
 		ptr->layers[i].posX = layerData.readSint16LE();
@@ -450,8 +466,9 @@ int16 Scenery::loadAnim(char search) {
 		layerPos = layerData.pos();
 		framesCount = 0;
 		layerData.seek(4, SEEK_CUR);
-		for (j = 0; j < ptr->layers[i].framesCount; j++, framesCount++, layerData.seek(4, SEEK_CUR)) {
-			while(layerData.readByte() == 1) {
+		for (j = 0; j < ptr->layers[i].framesCount;
+				j++, framesCount++, layerData.seek(4, SEEK_CUR)) {
+			while (layerData.readByte() == 1) {
 				framesCount++;
 				layerData.seek(4, SEEK_CUR);
 			}
@@ -472,35 +489,32 @@ int16 Scenery::loadAnim(char search) {
 		pictDescId = _vm->_inter->load16();
 		if (pictDescId >= 30000) {
 			ptr->pieces[i] =
-			    (PieceDesc *) _vm->_game->loadExtData(pictDescId, 0,
-			    0);
-			ptr->piecesFromExt[i] = 1;
+				(PieceDesc *) _vm->_game->loadExtData(pictDescId, 0, 0);
+			ptr->piecesFromExt[i] = true;
 		} else {
 			ptr->pieces[i] =
-			    (PieceDesc *)
-			    _vm->_game->loadTotResource(pictDescId);
-			ptr->piecesFromExt[i] = 0;
+				(PieceDesc *) _vm->_game->loadTotResource(pictDescId);
+			ptr->piecesFromExt[i] = false;
 		}
 
 		width = _vm->_inter->load16();
 		height = _vm->_inter->load16();
 		sprResId = _vm->_inter->load16();
-		for (sprIndex = 0; sprIndex < 20; sprIndex++) {
+		for (sprIndex = 0; sprIndex < 20; sprIndex++)
 			if (_spriteResId[sprIndex] == sprResId)
 				break;
-		}
 
 		if (sprIndex < 20) {
 			_animPictToSprite[7 * sceneryIndex + i] = sprIndex;
 			_spriteRefs[sprIndex]++;
 		} else {
 			for (sprIndex = 19; _vm->_draw->_spritesArray[sprIndex] != 0;
-			    sprIndex--);
+				sprIndex--);
 
 			_animPictToSprite[7 * sceneryIndex + i] = sprIndex;
 			_spriteRefs[sprIndex] = 1;
 			_spriteResId[sprIndex] = sprResId;
-			_vm->_draw->initBigSprite(sprIndex, width, height, 2);
+			_vm->_draw->initSpriteSurf(sprIndex, width, height, 2);
 
 			_vm->_video->clearSurf(_vm->_draw->_spritesArray[sprIndex]);
 			_vm->_draw->_destSurface = sprIndex;
@@ -511,68 +525,269 @@ int16 Scenery::loadAnim(char search) {
 			_vm->_draw->spriteOperation(DRAW_LOADSPRITE);
 		}
 	}
-	if (extData != 0)
-		delete[] extData;
+
+	delete[] extData;
+
 	return sceneryIndex + 100;
 }
 
-void Scenery::freeAnim(int16 animation) {
-	int16 i;
+void Scenery::freeAnim(int16 index) {
 	int16 spr;
 
-	if (animation == -1)
-		_vm->_inter->evalExpr(&animation);
+	if (index == -1)
+		_vm->_inter->evalExpr(&index);
 
-	if (_animPictCount[animation] == 0)
+	if (_animPictCount[index] == 0)
 		return;
 
-	for (i = 0; i < _animPictCount[animation]; i++) {
-		if (_animations[animation].piecesFromExt[i] == 1)
-			delete[] _animations[animation].pieces[i];
+	for (int i = 0; i < _animPictCount[index]; i++) {
+		if (_animations[index].piecesFromExt[i])
+			delete[] _animations[index].pieces[i];
 
-		spr = _animPictToSprite[animation * 7 + i];
+		spr = _animPictToSprite[index * 7 + i];
 		_spriteRefs[spr]--;
 		if (_spriteRefs[spr] == 0) {
-			_vm->_video->freeSurfDesc(_vm->_draw->_spritesArray[spr]);
-
-			_vm->_draw->_spritesArray[spr] = 0;
+			_vm->_draw->freeSprite(spr);
 			_spriteResId[spr] = -1;
 		}
 	}
-
 	
-	for (i = 0; i < _animations[animation].layersCount; i++)
-		delete[] _animations[animation].layers[i].frames;
-	delete[] _animations[animation].layers;
-	delete[] _animations[animation].pieces;
-	delete[] _animations[animation].piecesFromExt;
+	for (int i = 0; i < _animations[index].layersCount; i++)
+		delete[] _animations[index].layers[i].frames;
+	delete[] _animations[index].layers;
+	delete[] _animations[index].pieces;
+	delete[] _animations[index].piecesFromExt;
 
-	_animPictCount[animation] = 0;
+	_animPictCount[index] = 0;
 }
 
-void Scenery::interStoreParams(void) {
+// flags & 1 - do capture all area animation is occupying
+// flags & 4 == 0 - calculate animation final size
+// flags & 2 != 0 - don't check with "toRedraw"'s
+// flags & 4 != 0 - checkk view toRedraw
+void Scenery::updateAnim(int16 layer, int16 frame, int16 animation, int16 flags,
+	    int16 drawDeltaX, int16 drawDeltaY, char doDraw) {
 	AnimLayer *layerPtr;
-	int16 animation;
-	int16 layer;
-	int16 var;
+	PieceDesc **pictPtr;
+	AnimFramePiece *framePtr;
 
-	warning("interStoreParams: Storing...");
+	uint16 pieceIndex;
+	uint16 pictIndex;
+
+	int16 left;
+	int16 right;
+	int16 top;
+	int16 bottom;
+
+	byte highX;
+	byte highY;
+
+	int16 i;
+	int16 transp;
+
+	int16 destX;
+	int16 destY;
+
+	if ((_animPictCount[animation] == 0) || (layer < 0))
+		return;
+	if (layer >= _animations[animation].layersCount)
+		return;
 
-	_vm->_inter->evalExpr(&animation);
-	_vm->_inter->evalExpr(&layer);
 	layerPtr = &_animations[animation].layers[layer];
 
-	var = _vm->_parse->parseVarIndex();
-	WRITE_VAR_OFFSET(var, layerPtr->animDeltaX);
+	if (frame >= layerPtr->framesCount)
+		return;
 
-	var = _vm->_parse->parseVarIndex();
-	WRITE_VAR_OFFSET(var, layerPtr->animDeltaY);
+	if (flags & 1) { // Do capture
+		updateAnim(layer, frame, animation, 0, drawDeltaX, drawDeltaY, 0);
 
-	var = _vm->_parse->parseVarIndex();
-	WRITE_VAR_OFFSET(var, layerPtr->unknown0);
+		if (_toRedrawLeft == -12345)
+			return;
 
-	var = _vm->_parse->parseVarIndex();
-	WRITE_VAR_OFFSET(var, layerPtr->framesCount);
+		_vm->_game->capturePush(_toRedrawLeft, _toRedrawTop,
+		    _toRedrawRight - _toRedrawLeft + 1,
+		    _toRedrawBottom - _toRedrawTop + 1);
+
+		*_pCaptureCounter = *_pCaptureCounter + 1;
+	}
+
+	pictPtr = _animations[animation].pieces;
+	framePtr = layerPtr->frames;
+
+	for (i = 0; i < frame; i++, framePtr++)
+		while (framePtr->notFinal == 1)
+			framePtr++;
+
+	if (flags & 4) {
+		_toRedrawLeft = MAX(_toRedrawLeft, _vm->_mult->_animLeft);
+		_toRedrawTop = MAX(_toRedrawTop, _vm->_mult->_animTop);
+		_toRedrawRight = MIN(_toRedrawRight,
+		    (int16)(_vm->_mult->_animLeft + _vm->_mult->_animWidth - 1));
+		_toRedrawBottom = MIN(_toRedrawBottom,
+		    (int16)(_vm->_mult->_animTop + _vm->_mult->_animHeight - 1));
+	} else
+		_toRedrawLeft = -12345;
+
+	transp = layerPtr->transp ? 3 : 0;
+
+	framePtr--;
+	do {
+		framePtr++;
+
+		pieceIndex = framePtr->pieceIndex;
+		pictIndex = framePtr->pictIndex;
+
+		destX = framePtr->destX;
+		destY = framePtr->destY;
+
+		highX = pictIndex & 0xC0;
+		highY = pictIndex & 0x30;
+		highX >>= 6;
+		highY >>= 4;
+		if (destX >= 0)
+			destX += ((uint16)highX) << 7;
+		else
+			destX -= ((uint16)highX) << 7;
+
+		if (destY >= 0)
+			destY += ((uint16)highY) << 7;
+		else
+			destY -= ((uint16)highY) << 7;
+
+		if (drawDeltaX == 1000)
+			destX += layerPtr->posX;
+		else
+			destX += drawDeltaX;
+
+		if (drawDeltaY == 1000)
+			destY += layerPtr->posY;
+		else
+			destY += drawDeltaY;
+
+		pictIndex = (pictIndex & 15) - 1;
+
+		left = READ_LE_UINT16(&pictPtr[pictIndex][pieceIndex].left);
+		right = READ_LE_UINT16(&pictPtr[pictIndex][pieceIndex].right);
+		top = READ_LE_UINT16(&pictPtr[pictIndex][pieceIndex].top);
+		bottom = READ_LE_UINT16(&pictPtr[pictIndex][pieceIndex].bottom);
+
+		if (flags & 2) {
+			if (destX < _vm->_mult->_animLeft) {
+				left += _vm->_mult->_animLeft - destX;
+				destX = _vm->_mult->_animLeft;
+			}
+
+			if ((left <= right) && ((destX + right - left) >=
+			    (_vm->_mult->_animLeft + _vm->_mult->_animWidth)))
+				right -= (destX + right - left) -
+					(_vm->_mult->_animLeft + _vm->_mult->_animWidth) + 1;
+
+			if (destY < _vm->_mult->_animTop) {
+				top += _vm->_mult->_animTop - destY;
+				destY = _vm->_mult->_animTop;
+			}
+
+			if ((top <= bottom) && ((destY + bottom - top) >=
+						(_vm->_mult->_animTop + _vm->_mult->_animHeight)))
+				bottom -= (destY + bottom - top) -
+					(_vm->_mult->_animTop + _vm->_mult->_animHeight) + 1;
+
+		} else if (flags & 4) {
+			if (destX < _toRedrawLeft) {
+				left += _toRedrawLeft - destX;
+				destX = _toRedrawLeft;
+			}
+
+			if ((left <= right) && ((destX + right - left) > _toRedrawRight))
+				right -= destX + right - left - _toRedrawRight;
+
+			if (destY < _toRedrawTop) {
+				top += _toRedrawTop - destY;
+				destY = _toRedrawTop;
+			}
+
+			if ((top <= bottom) && ((destY + bottom - top) > _toRedrawBottom))
+				bottom -= destY + bottom - top - _toRedrawBottom;
+		}
+
+		if ((left > right) || (top > bottom))
+			continue;
+
+		if (doDraw) {
+			_vm->_draw->_sourceSurface =
+			    _animPictToSprite[animation * 7 + pictIndex];
+			_vm->_draw->_destSurface = 21;
+
+			_vm->_draw->_spriteLeft = left;
+			_vm->_draw->_spriteTop = top;
+			_vm->_draw->_spriteRight = right - left + 1;
+			_vm->_draw->_spriteBottom = bottom - top + 1;
+			_vm->_draw->_destSpriteX = destX;
+			_vm->_draw->_destSpriteY = destY;
+			_vm->_draw->_transparency = transp;
+			_vm->_draw->spriteOperation(DRAW_BLITSURF);
+		}
+
+		if (!(flags & 4)) {
+			if (_toRedrawLeft == -12345) {
+				_toRedrawLeft = destX;
+				_animLeft = destX;
+				_toRedrawTop = destY;
+				_animTop = destY;
+				_toRedrawRight = destX + right - left;
+				_animRight = destX + right - left;
+				_toRedrawBottom = destY + bottom - top;
+				_animBottom = destY + bottom - top;
+			} else {
+				_toRedrawLeft = MIN(_toRedrawLeft, destX);
+				_toRedrawTop = MIN(_toRedrawTop, destY);
+				_toRedrawRight =
+					MAX(_toRedrawRight, (int16)(destX + right - left));
+				_toRedrawBottom =
+					MAX(_toRedrawBottom, (int16)(destY + bottom - top));
+			}
+		}
+
+	} while (framePtr->notFinal == 1);
 }
 
-}				// End of namespace Gob
+void Scenery::writeAnimLayerInfo(uint16 index, uint16 layer,
+		int16 varDX, int16 varDY, int16 varUnk0, int16 varFrames) {
+
+	assert(index < 10);
+	assert(layer < _animations[index].layersCount);
+
+	AnimLayer &animLayer = _animations[index].layers[layer];
+	WRITE_VAR_OFFSET(varDX, animLayer.animDeltaX);
+	WRITE_VAR_OFFSET(varDY, animLayer.animDeltaY);
+	WRITE_VAR_OFFSET(varUnk0, animLayer.unknown0);
+	WRITE_VAR_OFFSET(varFrames, animLayer.framesCount);
+}
+
+int16 Scenery::getStaticLayersCount(uint16 index) {
+	assert(index < 10);
+
+	return _statics[index].layersCount;
+}
+
+int16 Scenery::getAnimLayersCount(uint16 index) {
+	assert(index < 10);
+
+	return _animations[index].layersCount;
+}
+
+Scenery::StaticLayer *Scenery::getStaticLayer(uint16 index, uint16 layer) {
+	assert(index < 10);
+	assert(layer < _statics[index].layersCount);
+
+	return &_statics[index].layers[layer];
+}
+
+Scenery::AnimLayer *Scenery::getAnimLayer(uint16 index, uint16 layer) {
+	assert(index < 10);
+	assert(layer < _animations[index].layersCount);
+
+	return &_animations[index].layers[layer];
+}
+
+} // End of namespace Gob
