@@ -1,6 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2004 Ivan Dubrov
- * Copyright (C) 2004-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -16,10 +18,11 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/gob/scenery.h $
- * $Id: scenery.h 23079 2006-06-13 08:10:54Z drmccoy $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/gob/scenery.h $
+ * $Id: scenery.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
+
 #ifndef GOB_SCENERY_H
 #define GOB_SCENERY_H
 
@@ -27,13 +30,15 @@ namespace Gob {
 
 class Scenery {
 public:
-#pragma START_PACK_STRUCTS
+
+#include "common/pack-start.h"	// START STRUCT PACKING
+
 	struct PieceDesc {
 		int16 left;		//NOTE:
 		int16 right;		//These are stored in Little Endian format
 		int16 top;		//And should be converted by client code when accessed
 		int16 bottom;		//i.e. use FROM_LE_16()
-	} GCC_PACK;
+	};
 
 	struct StaticPlane {
 		int8 pictIndex;
@@ -42,13 +47,13 @@ public:
 		int16 destX;
 		int16 destY;
 		int8 transp;
-	} GCC_PACK;
+	};
 
 	struct StaticLayer {
 		int16 backResId;
 		int16 planeCount;
 		StaticPlane *planes;
-	} GCC_PACK;
+	};
 
 	// Animations
 
@@ -58,7 +63,7 @@ public:
 		int8 destX;
 		int8 destY;
 		int8 notFinal;
-	} GCC_PACK;
+	};
 
 	struct AnimLayer {
 		int16 unknown0;
@@ -69,14 +74,15 @@ public:
 		int8 transp;
 		int16 framesCount;
 		AnimFramePiece *frames;
-	} GCC_PACK;
-#pragma END_PACK_STRUCTS
+	};
+
+#include "common/pack-end.h"	// END STRUCT PACKING
 
 	struct Static {
 		int16 layersCount;
 		StaticLayer *layers;
 		PieceDesc **pieces;
-		int8 *piecesFromExt;
+		bool *piecesFromExt;
 		Static() : layersCount(0), layers(0), pieces(0),
 				   piecesFromExt(0) {}
 	};
@@ -85,26 +91,11 @@ public:
 		int16 layersCount;
 		AnimLayer *layers;
 		PieceDesc **pieces;
-		int8 *piecesFromExt;
+		bool *piecesFromExt;
 		Animation() : layersCount(0), layers(0), pieces(0),
 			              piecesFromExt(0) {}
 	};
 
-	// Global variables
-
-	char _spriteRefs[20];
-	int16 _spriteResId[20];
-
-	char _staticPictToSprite[70];
-	int16 _staticPictCount[10];
-	Static _statics[10];
-	int16 _staticResId[10];
-
-	char _animPictToSprite[70];
-	int16 _animPictCount[10];
-	Animation _animations[10];
-	int16 _animResId[10];
-
 	int16 _curStatic;
 	int16 _curStaticLayer;
 
@@ -120,44 +111,64 @@ public:
 
 	int16 *_pCaptureCounter;
 
-	// Functions
-
+	void init();
 	int16 loadStatic(char search);
 	void freeStatic(int16 index);
 	void renderStatic(int16 scenery, int16 layer);
 	void updateStatic(int16 orderFrom);
-	int16 loadAnim(char search);
-	void freeAnim(int16 animation);
-	void interStoreParams(void);
+	void freeAnim(int16 index);
+	void updateAnim(int16 layer, int16 frame, int16 animation,
+			int16 flags, int16 drawDeltaX, int16 drawDeltaY, char doDraw);
 
-	virtual void updateAnim(int16 layer, int16 frame, int16 animation,
-			int16 flags, int16 drawDeltaX, int16 drawDeltaY, char doDraw) = 0;
+	void writeAnimLayerInfo(uint16 index, uint16 layer, 
+			int16 varDX, int16 varDY, int16 varUnk0, int16 varFrames);
+	int16 getStaticLayersCount(uint16 index);
+	int16 getAnimLayersCount(uint16 index);
+	StaticLayer *getStaticLayer(uint16 index, uint16 layer);
+	AnimLayer *getAnimLayer(uint16 index, uint16 layer);
+	
+	virtual int16 loadAnim(char search);
 
 	Scenery(GobEngine *vm);
-	virtual ~Scenery() {};
+	virtual ~Scenery();
 
 protected:
+	char _spriteRefs[20];
+	int16 _spriteResId[20];
+
+	int16 _staticPictCount[10];
+	int16 _staticResId[10];
+
+	int16 _animPictCount[10];
+	int16 _animResId[10];
+
+	char _staticPictToSprite[70];
+	char _animPictToSprite[70];
+
+	Static _statics[10];
+	Animation _animations[10];
+
 	GobEngine *_vm;
+
+	void updateStatic(int16 orderFrom, byte index, byte layer);
 };
 
 class Scenery_v1 : public Scenery {
 public:
-	virtual void updateAnim(int16 layer, int16 frame, int16 animation,
-			int16 flags, int16 drawDeltaX, int16 drawDeltaY, char doDraw);
+	virtual int16 loadAnim(char search);
 
 	Scenery_v1(GobEngine *vm);
-	virtual ~Scenery_v1() {};
+	virtual ~Scenery_v1() {}
 };
 
 class Scenery_v2 : public Scenery_v1 {
 public:
-	virtual void updateAnim(int16 layer, int16 frame, int16 animation,
-			int16 flags, int16 drawDeltaX, int16 drawDeltaY, char doDraw);
+	virtual int16 loadAnim(char search);
 
 	Scenery_v2(GobEngine *vm);
-	virtual ~Scenery_v2() {};
+	virtual ~Scenery_v2() {}
 };
 
-}				// End of namespace Gob
+} // End of namespace Gob
 
-#endif	/* __SCENERY_H */
+#endif // GOB_SCENERY_H
