@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2005-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,13 +18,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/kyra/animator.h $
- * $Id: animator.h 20493 2006-02-11 08:31:13Z vinterstum $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/kyra/animator.h $
+ * $Id: animator.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
-#ifndef KYRAANIMATOR_H
-#define KYRAANIMATOR_H
+#ifndef KYRA_ANIMATOR_H
+#define KYRA_ANIMATOR_H
 
 namespace Kyra {
 class KyraEngine;
@@ -109,6 +112,8 @@ protected:
 	AnimObject *_items;
 	AnimObject *_sprites;
 
+	uint8 *_actorBkgBackUp[2];
+
 	AnimObject *objectRemoveQueue(AnimObject *queue, AnimObject *rem);
 	AnimObject *objectAddHead(AnimObject *queue, AnimObject *head);
 	AnimObject *objectQueue(AnimObject *queue, AnimObject *add);
@@ -124,3 +129,4 @@ protected:
 } // end of namespace Kyra
 
 #endif
+
