@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2005-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,14 +18,14 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/kyra/debugger.cpp $
- * $Id: debugger.cpp 21125 2006-03-07 14:16:52Z vinterstum $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/kyra/debugger.cpp $
+ * $Id: debugger.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
 #include "common/stdafx.h"
 #include "common/config-manager.h"
-#include "common/debugger.cpp"
+#include "common/system.h"
 #include "kyra/debugger.h"
 #include "kyra/kyra.h"
 #include "kyra/screen.h"
@@ -30,21 +33,18 @@
 namespace Kyra {
 
 Debugger::Debugger(KyraEngine *vm)
-	: Common::Debugger<Debugger>() {
+	: GUI::Debugger() {
 	_vm = vm;
 
-	DCmd_Register("continue",			&Debugger::cmd_exit);
-	DCmd_Register("exit",				&Debugger::cmd_exit);
-	DCmd_Register("help",				&Debugger::cmd_help);
-	DCmd_Register("quit",				&Debugger::cmd_exit);
-	DCmd_Register("enter",				&Debugger::cmd_enterRoom);
-	DCmd_Register("rooms",				&Debugger::cmd_listRooms);
-	DCmd_Register("flags",				&Debugger::cmd_listFlags);
-	DCmd_Register("toggleflag",			&Debugger::cmd_toggleFlag);
-	DCmd_Register("queryflag",			&Debugger::cmd_queryFlag);
-	DCmd_Register("timers",				&Debugger::cmd_listTimers);
-	DCmd_Register("settimercountdown",	&Debugger::cmd_setTimerCountdown);
-	DCmd_Register("give",				&Debugger::cmd_giveItem);
+	DCmd_Register("continue",			WRAP_METHOD(Debugger, Cmd_Exit));
+	DCmd_Register("enter",				WRAP_METHOD(Debugger, cmd_enterRoom));
+	DCmd_Register("rooms",				WRAP_METHOD(Debugger, cmd_listRooms));
+	DCmd_Register("flags",				WRAP_METHOD(Debugger, cmd_listFlags));
+	DCmd_Register("toggleflag",			WRAP_METHOD(Debugger, cmd_toggleFlag));
+	DCmd_Register("queryflag",			WRAP_METHOD(Debugger, cmd_queryFlag));
+	DCmd_Register("timers",				WRAP_METHOD(Debugger, cmd_listTimers));
+	DCmd_Register("settimercountdown",	WRAP_METHOD(Debugger, cmd_setTimerCountdown));
+	DCmd_Register("give",				WRAP_METHOD(Debugger, cmd_giveItem));
 }
 
 void Debugger::preEnter() {
@@ -93,32 +93,6 @@ bool Debugger::cmd_enterRoom(int argc, const char **argv) {
 	return true;
 }
 
-bool Debugger::cmd_exit(int argc, const char **argv) {
-	_detach_now = true;
-	return false;
-}
-
-bool Debugger::cmd_help(int argc, const char **argv) {
-	// console normally has 39 line width
-	// wrap around nicely
-	int width = 0, size, i;
-
-	DebugPrintf("Commands are:\n");
-	for (i = 0 ; i < _dcmd_count ; i++) {
-		size = strlen(_dcmds[i].name) + 1;
-
-		if ((width + size) >= 39) {
-			DebugPrintf("\n");
-			width = size;
-		} else
-			width += size;
-
-		DebugPrintf("%s ", _dcmds[i].name);
-	}
-	DebugPrintf("\n");
-	return true;
-}
-
 bool Debugger::cmd_listRooms(int argc, const char **argv) {
 	for (int i = 0; i < _vm->_roomTableSize; i++) {
 		DebugPrintf("%-3i: %-10s", i, _vm->_roomFilenameTable[_vm->_roomTable[i].nameIndex]);
@@ -148,8 +122,9 @@ bool Debugger::cmd_toggleFlag(int argc, const char **argv) {
 		else
 			_vm->setGameFlag(flag);
 		DebugPrintf("Flag %i is now %i\n", flag, _vm->queryGameFlag(flag)); 
-	} else
+	} else {
 		DebugPrintf("Syntax: toggleflag <flag>\n");
+	}
 
 	return true;
 }
@@ -158,8 +133,9 @@ bool Debugger::cmd_queryFlag(int argc, const char **argv) {
 	if (argc > 1) {
 		uint flag = atoi(argv[1]);
 		DebugPrintf("Flag %i is %i\n", flag, _vm->queryGameFlag(flag)); 
-	} else
+	} else {
 		DebugPrintf("Syntax: queryflag <flag>\n");
+	}
 
 	return true;
 }
@@ -177,8 +153,9 @@ bool Debugger::cmd_setTimerCountdown(int argc, const char **argv) {
 		uint countdown = atoi(argv[2]);
 		_vm->setTimerCountdown(timer, countdown);	
 		DebugPrintf("Timer %i now has countdown %i\n", timer, _vm->_timers[timer].countdown); 
-	} else
+	} else {
 		DebugPrintf("Syntax: settimercountdown <timer> <countdown>\n");
+	}
 
 	return true;
 }
@@ -195,9 +172,11 @@ bool Debugger::cmd_giveItem(int argc, const char **argv) {
 
 		_vm->setMouseItem(item);
 		_vm->_itemInHand = item;
-	} else
+	} else {
 		DebugPrintf("Syntax: give <itemid>\n");
+	}
 		
 	return true;
 }
 } // End of namespace Kyra
+
