@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2005-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,21 +18,21 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/kyra/debugger.h $
- * $Id: debugger.h 20695 2006-02-14 23:31:25Z lordhoto $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/kyra/debugger.h $
+ * $Id: debugger.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
 #ifndef KYRA_DEBUGGER_H
 #define KYRA_DEBUGGER_H
 
-#include "common/debugger.h"
+#include "gui/debugger.h"
 
 namespace Kyra {
 
 class KyraEngine;
 
-class Debugger : public Common::Debugger<Debugger> {
+class Debugger : public GUI::Debugger {
 public:
 	Debugger(KyraEngine *vm);
 	virtual ~Debugger() {}  // we need this for __SYMBIAN32__ archaic gcc/UIQ
@@ -40,8 +43,6 @@ protected:
 	virtual void preEnter();
 	virtual void postEnter();
 
-	bool cmd_exit(int argc, const char **argv);
-	bool cmd_help(int argc, const char **argv);
 	bool cmd_enterRoom(int argc, const char **argv);
 	bool cmd_listRooms(int argc, const char **argv);
 	bool cmd_listFlags(int argc, const char **argv);
@@ -55,3 +56,4 @@ protected:
 } // End of namespace Kyra
 
 #endif
+
