@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2005-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/kyra/gui.cpp $
- * $Id: gui.cpp 23168 2006-06-18 10:38:24Z vinterstum $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/kyra/gui.cpp $
+ * $Id: gui.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -29,6 +32,7 @@
 
 #include "common/config-manager.h"
 #include "common/savefile.h"
+#include "common/events.h"
 #include "common/system.h"
 
 namespace Kyra {
@@ -37,6 +41,7 @@ void KyraEngine::registerDefaultSettings() {
 	// Most settings already have sensible defaults. This one, however, is
 	// specific to the Kyra engine.
 	ConfMan.registerDefault("walkspeed", 2);
+	ConfMan.registerDefault("cdaudio", _flags.platform == Common::kPlatformFMTowns);
 }
 
 void KyraEngine::readSettings() {
@@ -54,7 +59,7 @@ void KyraEngine::readSettings() {
 		_configTextspeed = 2;	// Fast
 
 	_configWalkspeed = ConfMan.getInt("walkspeed");
-	_configMusic = ConfMan.getBool("music_mute") ? 0 : 1;
+	_configMusic = ConfMan.getBool("music_mute") ? 0 : ((ConfMan.getBool("cdaudio") && _flags.platform == Common::kPlatformFMTowns) ? 2 : 1);	
 	_configSounds = ConfMan.getBool("sfx_mute") ? 0 : 1;
 
 	_sound->enableMusic(_configMusic);
@@ -95,6 +100,7 @@ void KyraEngine::writeSettings() {
 	ConfMan.setInt("talkspeed", talkspeed);
 	ConfMan.setInt("walkspeed", _configWalkspeed);
 	ConfMan.setBool("music_mute", _configMusic == 0);
+	ConfMan.setBool("cdaudio", _configMusic == 2);
 	ConfMan.setBool("sfx_mute", _configSounds == 0);
 
 	switch (_configVoice) {
@@ -127,9 +133,8 @@ void KyraEngine::writeSettings() {
 void KyraEngine::initMainButtonList() {
 	_haveScrollButtons = false;
 	_buttonList = &_buttonData[0];
-	for (int i = 0; _buttonDataListPtr[i]; ++i) {
+	for (int i = 0; _buttonDataListPtr[i]; ++i)
 		_buttonList = initButton(_buttonList, _buttonDataListPtr[i]);
-	}
 }
 
 Button *KyraEngine::initButton(Button *list, Button *newButton) {
@@ -138,12 +143,13 @@ Button *KyraEngine::initButton(Button *list, Button *newButton) {
 	if (!list)
 		return newButton;
 	Button *cur = list;
+
 	while (true) {
-		if (!cur->nextButton) {
+		if (!cur->nextButton)
 			break;
-		}
 		cur = cur->nextButton;
 	}
+
 	cur->nextButton = newButton;
 	return list;
 }
@@ -170,7 +176,7 @@ int KyraEngine::buttonInventoryCallback(Button *caller) {
 			snd_playSoundEffect(0x35);
 			_screen->hideMouse();
 			_screen->fillRect(_itemPosX[itemOffset], _itemPosY[itemOffset], _itemPosX[itemOffset] + 15, _itemPosY[itemOffset] + 15, 12);
-			_screen->drawShape(0, _shapes[220+_itemInHand], _itemPosX[itemOffset], _itemPosY[itemOffset], 0, 0);
+			_screen->drawShape(0, _shapes[216+_itemInHand], _itemPosX[itemOffset], _itemPosY[itemOffset], 0, 0);
 			setMouseItem(inventoryItem);
 			updateSentenceCommand(_itemList[inventoryItem], _takenList[1], 179);
 			_screen->showMouse();
@@ -179,8 +185,8 @@ int KyraEngine::buttonInventoryCallback(Button *caller) {
 		} else {
 			snd_playSoundEffect(0x32);
 			_screen->hideMouse();
-			_screen->drawShape(0, _shapes[220+_itemInHand], _itemPosX[itemOffset], _itemPosY[itemOffset], 0, 0);
-			_screen->setMouseCursor(1, 1, _shapes[4]);
+			_screen->drawShape(0, _shapes[216+_itemInHand], _itemPosX[itemOffset], _itemPosY[itemOffset], 0, 0);
+			_screen->setMouseCursor(1, 1, _shapes[0]);
 			updateSentenceCommand(_itemList[_itemInHand], _placedList[0], 179);
 			_screen->showMouse();
 			_currentCharacter->inventoryItems[itemOffset] = _itemInHand;
@@ -204,34 +210,19 @@ int KyraEngine::buttonAmuletCallback(Button *caller) {
 		return 1;
 	if (_itemInHand != -1) {
 		assert(_putDownFirst);
-		if (speechEnabled()) {
-			snd_voiceWaitForFinish();
-			snd_playVoiceFile(2000);
-		}
-		if (textEnabled())	
-			characterSays(_putDownFirst[0], 0, -2);
+		characterSays(2000, _putDownFirst[0], 0, -2);
 		return 1;
 	}
 	if (queryGameFlag(0xF1)) {
 		assert(_waitForAmulet);
-		if (speechEnabled()) {
-			snd_voiceWaitForFinish();
-			snd_playVoiceFile(2001);
-		}
-		if (textEnabled())	
-			characterSays(_waitForAmulet[0], 0, -2);	
+		characterSays(2001, _waitForAmulet[0], 0, -2);	
 		return 1;
 	}
 	if (!queryGameFlag(0x55+jewel)) {
 		assert(_blackJewel);
 		_animator->makeBrandonFaceMouse();
 		drawJewelPress(jewel, 1);
-		if (speechEnabled()) {
-			snd_voiceWaitForFinish();
-			snd_playVoiceFile(2002);
-		}
-		if (textEnabled())	
-			characterSays(_blackJewel[0], 0, -2);
+		characterSays(2002, _blackJewel[0], 0, -2);
 		return 1;
 	}
 	drawJewelPress(jewel, 0);
@@ -243,9 +234,8 @@ int KyraEngine::buttonAmuletCallback(Button *caller) {
 	_scriptClick->variables[6] = jewel;
 	_scriptInterpreter->startScript(_scriptClick, 4);
 	
-	while (_scriptInterpreter->validScript(_scriptClick)) {
+	while (_scriptInterpreter->validScript(_scriptClick))
 		_scriptInterpreter->runScript(_scriptClick);
-	}
 	
 	if (_scriptClick->variables[3])
 		return 1;
@@ -258,12 +248,7 @@ int KyraEngine::buttonAmuletCallback(Button *caller) {
 		} else if (_brandonStatusBit == 0) {
 			seq_brandonHealing();
 			assert(_healingTip);
-			if (speechEnabled()) {
-				snd_voiceWaitForFinish();
-				snd_playVoiceFile(2003);
-			}
-			if (textEnabled())	
-				characterSays(_healingTip[0], 0, -2);
+			characterSays(2003, _healingTip[0], 0, -2);
 		}
 		break;
 		
@@ -274,12 +259,7 @@ int KyraEngine::buttonAmuletCallback(Button *caller) {
 	case 2:
 		if (_brandonStatusBit & 1) {
 			assert(_wispJewelStrings);
-			if (speechEnabled()) {
-				snd_voiceWaitForFinish();
-				snd_playVoiceFile(2004);
-			}
-			if (textEnabled())	
-				characterSays(_wispJewelStrings[0], 0, -2);
+			characterSays(2004, _wispJewelStrings[0], 0, -2);
 		} else {
 			if (_brandonStatusBit & 2) {
 				// XXX
@@ -303,12 +283,7 @@ int KyraEngine::buttonAmuletCallback(Button *caller) {
 	case 3:
 		seq_dispelMagicAnimation();
 		assert(_magicJewelString);
-		if (speechEnabled()) {
-			snd_voiceWaitForFinish();
-			snd_playVoiceFile(2007);
-		}
-		if (textEnabled())	
-			characterSays(_magicJewelString[0], 0, -2);
+		characterSays(2007, _magicJewelString[0], 0, -2);
 		break;
 		
 	default:
@@ -345,7 +320,8 @@ void KyraEngine::processButtonList(Button *list) {
 		}
 		y += _screen->_screenDimTable[list->dimTableIndex].sy;
 		
-		if (_mouseX >= x && _mouseY >= y && x + list->width >= _mouseX && y + list->height >= _mouseY) {
+		Common::Point mouse = getMousePos();
+		if (mouse.x >= x && mouse.y >= y && x + list->width >= mouse.x && y + list->height >= mouse.y) {
 			int processMouseClick = 0;
 			if (list->flags & 0x400) {
 				if (_mousePressFlag) {
@@ -402,43 +378,37 @@ void KyraEngine::processButton(Button *button) {
 	int flags = (button->flags2 & 5);
 	if (flags == 1) {
 		processType = button->process2;
-		if (processType == 1) {
+		if (processType == 1)
 			shape = button->process2PtrShape;
-		} else if (processType == 4) {
+		else if (processType == 4)
 			callback = button->process2PtrCallback;
-		}
 	} else if (flags == 4 || flags == 5) {
 		processType = button->process1;
-		if (processType == 1) {
+		if (processType == 1)
 			shape = button->process1PtrShape;
-		} else if (processType == 4) {
+		else if (processType == 4)
 			callback = button->process1PtrCallback;
-		}
 	} else {
 		processType = button->process0;
-		if (processType == 1) {
+		if (processType == 1)
 			shape = button->process0PtrShape;
-		} else if (processType == 4) {
+		else if (processType == 4)
 			callback = button->process0PtrCallback;
-		}
 	}
 	
 	int x = button->x;
 	int y = button->y;
 	assert(button->dimTableIndex < _screen->_screenDimTableCount);
-	if (x < 0) {
+	if (x < 0)
 		x += _screen->_screenDimTable[button->dimTableIndex].w << 3;
-	}
 	
-	if (y < 0) {
+	if (y < 0)
 		y += _screen->_screenDimTable[button->dimTableIndex].h;
-	}
 	
-	if (processType == 1 && shape) {
+	if (processType == 1 && shape)
 		_screen->drawShape(_screen->_curPage, shape, x, y, button->dimTableIndex, 0x10);
-	} else if (processType == 4 && callback) {
+	else if (processType == 4 && callback)
 		(this->*callback)(button);
-	}
 }
 
 void KyraEngine::processAllMenuButtons() {
@@ -447,9 +417,8 @@ void KyraEngine::processAllMenuButtons() {
 
 	Button *cur = _menuButtonList;
 	while (true) {
-		if (!cur->nextButton) {
+		if (!cur->nextButton)
 			break;
-		}
 		processMenuButton(cur);
 		cur = cur->nextButton;
 	}
@@ -460,7 +429,7 @@ void KyraEngine::processMenuButton(Button *button) {
 	if (!_displayMenu)
 		return;
 
-	if ( !button || (button->flags & 8))
+	if (!button || (button->flags & 8))
 		return;
 
 	if (button->flags2 & 1)
@@ -492,7 +461,6 @@ int KyraEngine::drawBoxCallback(Button *button) {
 }
 
 int KyraEngine::drawShadedBoxCallback(Button *button) {
-
 	if (!_displayMenu)
 		return 0;
 	
@@ -505,23 +473,34 @@ int KyraEngine::drawShadedBoxCallback(Button *button) {
 
 void KyraEngine::setGUILabels() {
 	int offset = 0;
+	int offsetOptions = 0;
+	int offsetMainMenu = 0;
+	int offsetOn = 0;
+
 	int walkspeedGarbageOffset = 36;
 	int menuLabelGarbageOffset = 0;
 	
-	if (_features & GF_TALKIE) {
-		if (_features & GF_ENGLISH) {
+	if (_flags.isTalkie) {
+		if (_flags.lang == Common::EN_ANY)
 			offset = 52;
-		} else if (_features & GF_GERMAN) {
+		else if (_flags.lang == Common::DE_DEU)
 			offset = 30;
-		} else if (_features & GF_FRENCH) {
+		else if (_flags.lang == Common::FR_FRA)
 			offset = 6;
-		}
+		offsetOn = offsetMainMenu = offsetOptions = offset;
 		walkspeedGarbageOffset = 48;
-	} else if (_features & GF_SPANISH) {
-		offset = -4;
+	} else if (_flags.lang == Common::ES_ESP) {
+		offsetOn = offsetMainMenu = offsetOptions = offset = -4;
 		menuLabelGarbageOffset = 72;
+	} else if (_flags.lang == Common::DE_DEU) {
+		offset = offsetMainMenu = offsetOn = offsetOptions = 24;
+	} else if (_flags.platform == Common::kPlatformFMTowns) {
+		offset = 1;
+		offsetOptions = 10;
+		offsetOn = 0;
+		walkspeedGarbageOffset = 0;
 	}
-	
+
 	assert(offset + 27 < _guiStringsSize);
 		
 	// The Legend of Kyrandia
@@ -562,23 +541,24 @@ void KyraEngine::setGUILabels() {
 	_menu[1].item[1].itemString = _guiStrings[23 + offset];
 		
 	// Music is
-	_menu[5].item[0].labelString = _guiStrings[26 + offset];
+	_menu[5].item[0].labelString = _guiStrings[26 + offsetOptions];
 	// Sounds are
-	_menu[5].item[1].labelString = _guiStrings[27 + offset];
+	_menu[5].item[1].labelString = _guiStrings[27 + offsetOptions];
 	// Walk speed
-	_menu[5].item[2].labelString = &_guiStrings[24 + offset][walkspeedGarbageOffset];
+	_menu[5].item[2].labelString = &_guiStrings[24 + offsetOptions][walkspeedGarbageOffset];
 	// Text speed
-	_menu[5].item[4].labelString = _guiStrings[25 + offset];
+	_menu[5].item[4].labelString = _guiStrings[25 + offsetOptions];
 	// Main Menu
-	_menu[5].item[5].itemString = &_guiStrings[19 + offset][menuLabelGarbageOffset];
+	_menu[5].item[5].itemString = &_guiStrings[19 + offsetMainMenu][menuLabelGarbageOffset];
 	
-	if (_features & GF_TALKIE) {
+	if (_flags.isTalkie)
 		// Text & Voice
 		_voiceTextString = _guiStrings[28 + offset];
-	}
-	_textSpeedString = _guiStrings[25 + offset];
-	_onString =  _guiStrings[20 + offset];
+
+	_textSpeedString = _guiStrings[25 + offsetOptions];
+	_onString =  _guiStrings[20 + offsetOn];
 	_offString =  _guiStrings[21 + offset];
+	_onCDString = _guiStrings[21];
 }
 
 int KyraEngine::buttonMenuCallback(Button *caller) {
@@ -616,14 +596,13 @@ int KyraEngine::buttonMenuCallback(Button *caller) {
 		calcCoords(_menu[i]);
 
 	_menuRestoreScreen = true;
-	_keyboardEvent.pending = 0;
-	_keyboardEvent.repeat = 0;
+	_keyPressed = 0;
 	_mousePressFlag = false;
 	
 	_toplevelMenu = 0;
-	if (_menuDirectlyToLoad)
+	if (_menuDirectlyToLoad) {
 		gui_loadGameMenu(0);
-	else {
+	} else {
 		if (!caller)
 			_toplevelMenu = 4;
 
@@ -641,9 +620,9 @@ int KyraEngine::buttonMenuCallback(Button *caller) {
 		gui_restorePalette();
 		_screen->loadPageFromDisk("SEENPAGE.TMP", 0);
 		_animator->_updateScreen = true;
-	}
-	else
+	} else {
 		_screen->deletePageFromDisk(0);
+	}
 
 	return 0;
 }
@@ -677,18 +656,6 @@ void KyraEngine::initMenu(Menu &menu) {
 		if (!menu.item[i].enabled)
 			continue;
 
-
-		if (menu.item[i].itemString) {
-			int textWidth = _screen->getTextWidth(menu.item[i].itemString) - 18;
-
-			if (menu.item[i].width < textWidth) {
-				menu.item[i].width = textWidth;
-				
-				if ( menu.x + menu.item[i].x + menu.item[i].width > menu_x2)
-					menu.item[i].x -= (menu.x + menu.item[i].x + menu.item[i].width) - menu_x2 + 10;
-			}
-		}	
-
 		x1 = menu.x + menu.item[i].x;
 		y1 = menu.y + menu.item[i].y;
 
@@ -715,7 +682,7 @@ void KyraEngine::initMenu(Menu &menu) {
 		_screen->drawShadedBox(x1, y1, x2, y2, menu.item[i].color1, menu.item[i].color2);
 
 		if (menu.item[i].itemString) {
-			if (menu.item[i].field_12 != -1)
+			if (menu.item[i].field_12 != -1 && _flags.lang == Common::EN_ANY)
 				textX = x1 + menu.item[i].field_12 + 3;
 			else
 				textX = _text->getCenterStringX(menu.item[i].itemString, x1, x2);
@@ -751,8 +718,9 @@ void KyraEngine::initMenu(Menu &menu) {
 		_scrollDownButton.nextButton = 0;
 		_menuButtonList = initButton(_menuButtonList, &_scrollDownButton);
 		processMenuButton(&_scrollDownButton);
-	} else
+	} else {
 		_haveScrollButtons = false;
+	}
 
 	_screen->showMouse();
 	_screen->updateScreen();
@@ -761,85 +729,104 @@ void KyraEngine::initMenu(Menu &menu) {
 void KyraEngine::calcCoords(Menu &menu) {
 	assert(menu.nrOfItems < 7);
 
+	int widthBackup = _screen->_charWidth;
+	_screen->_charWidth = -2;
+	
+	menu.x = (320 - menu.width)/2;
+
+	int menu_x2 = menu.width  + menu.x - 1;	
 	int maxOffset = 0;
 	int x1, x2, y1, y2;	
+
 	for (int i = 0; i < menu.nrOfItems; i++) {
 		if (menu.item[i].x == -1)
 			menu.item[i].x = (menu.width - menu.item[i].width)/2;
 
 		if (menu.item[i].labelString) {
-			x1 = (320 - menu.width)/2 + menu.item[i].x;
+			x1 = menu.x + menu.item[i].x + 25;
 			y1 = (200 - menu.height)/2 + menu.item[i].y;
 
-			x2 = x1 + menu.item[i].width - 1;
-			y2 = y1 + menu.item[i].height - 1;
+			x2 = x1 + menu.item[i].width;
+			y2 = y1 + menu.item[i].height;
 
-			int textWidth = _screen->getTextWidth(menu.item[i].labelString);
-			int menu_x1 = (320 - menu.width) / 2;
-			int textX = menu.item[i].labelX + menu_x1;
-			int offset;
+			int textWidth = _screen->getTextWidth(menu.item[i].labelString) + 25;
+			int textX = menu.item[i].labelX + menu.x;
 
-			if (textWidth/2 + textX >= x1) {
-				offset = ((textWidth + textX) - x1) / 2;
+			if (textWidth + textX > x1) {
+				int offset = ((textWidth + textX) - x1);
 				if (maxOffset < offset)
 					maxOffset = offset;
 			}
 		}
+		
+		if (menu.item[i].itemString) {
+			int textWidth = _screen->getTextWidth(menu.item[i].itemString) + 15;
+
+			if (menu.item[i].width < textWidth) {
+				menu.item[i].width = textWidth;
+				
+				if ( menu.x + menu.item[i].x + menu.item[i].width > menu_x2)
+					menu.item[i].x -= (menu.x + menu.item[i].x + menu.item[i].width) - menu_x2 + 10;
+			}
+		}	
+		
 	}
 
 	if (maxOffset > 0) {
-		for (int i = 0; i < menu.nrOfItems; i++)
-			menu.item[i].x += maxOffset + 20;
-		menu.width += maxOffset + 20;	
+		maxOffset = maxOffset/2;
+		for (int i = 0; i < menu.nrOfItems; i++) {
+			menu.item[i].x += maxOffset + 10;
+			menu.item[i].labelX -= maxOffset;
+		}
+		menu.width += maxOffset;
 	}
-			
+	
+	if (menu.menuName != 0) {
+		int menuNameLength = _screen->getTextWidth(menu.menuName);
+		if (menuNameLength  > menu.width)
+			menu.width = menuNameLength;
+	}
+
 	if (menu.width > 310)
 		menu.width = 310;
-		
-	if (menu.x == -1)
-		menu.x = (320 - menu.width)/2;
 
+	menu.x = (320 - menu.width)/2;
+			
 	if (menu.y == -1)
 		menu.y = (200 - menu.height)/2;
-
+		
+	_screen->_charWidth = widthBackup;
 }
 
 void KyraEngine::gui_getInput() {
-	OSystem::Event event;
+	Common::Event event;
 	static uint32 lastScreenUpdate = 0;
 	uint32 now = _system->getMillis();
 
 	_mouseWheel = 0;
-	while (_system->pollEvent(event)) {
+	while (_eventMan->pollEvent(event)) {
 		switch (event.type) {
-		case OSystem::EVENT_QUIT:
+		case Common::EVENT_QUIT:
 			quitGame();
 			break;
-		case OSystem::EVENT_LBUTTONDOWN:
+		case Common::EVENT_LBUTTONDOWN:
 			_mousePressFlag = true;
 			break;
-		case OSystem::EVENT_LBUTTONUP:
+		case Common::EVENT_LBUTTONUP:
 			_mousePressFlag = false;
 			break;
-		case OSystem::EVENT_MOUSEMOVE:
-			_mouseX = event.mouse.x;
-			_mouseY = event.mouse.y;
+		case Common::EVENT_MOUSEMOVE:
 			_system->updateScreen();
 			lastScreenUpdate = now;
 			break;
-		case OSystem::EVENT_WHEELUP:
+		case Common::EVENT_WHEELUP:
 			_mouseWheel = -1;
 			break;
-		case OSystem::EVENT_WHEELDOWN:
+		case Common::EVENT_WHEELDOWN:
 			_mouseWheel = 1;
 			break;
-		case OSystem::EVENT_KEYDOWN:
-			_keyboardEvent.pending = true;
-			_keyboardEvent.repeat = now + 400;
-			_keyboardEvent.ascii = event.kbd.ascii;
-			break;
-		case OSystem::EVENT_KEYUP:
-			_keyboardEvent.repeat = 0;
+		case Common::EVENT_KEYDOWN:
+			_keyPressed = event.kbd.ascii;
 			break;
 		default:
 			break;
@@ -851,10 +838,6 @@ void KyraEngine::gui_getInput() {
 		lastScreenUpdate = now;
 	}
 
-	if (!_keyboardEvent.pending && _keyboardEvent.repeat && now >= _keyboardEvent.repeat) {
-		_keyboardEvent.pending = true;
-		_keyboardEvent.repeat = now + 100;
-	}
 	_system->delayMillis(3);
 }
 
@@ -877,11 +860,10 @@ int KyraEngine::getNextSavegameSlot() {
 	Common::InSaveFile *in;
 
 	for (int i = 1; i < 1000; i++) {
-		if ((in = _saveFileMan->openForLoading(getSavegameFilename(i)))) {
+		if ((in = _saveFileMan->openForLoading(getSavegameFilename(i))))
 			delete in;
-		} else {
+		else
 			return i;
-		}
 	}
 	warning("Didn't save: Ran out of savegame filenames");
 	return 0;
@@ -898,8 +880,9 @@ void KyraEngine::setupSavegames(Menu &menu, int num) {
 		menu.item[0].enabled = 1;
 		menu.item[0].field_1b = 0;
 		startSlot = 1;
-	} else
+	} else {
 		startSlot = 0;
+	}
 
 	for (int i = startSlot; i < num; i++) {
 		if ((in = _saveFileMan->openForLoading(getSavegameFilename(i + _savegameOffset)))) {
@@ -959,9 +942,9 @@ int KyraEngine::gui_saveGameMenu(Button *button) {
 
 int KyraEngine::gui_loadGameMenu(Button *button) {
 	debugC(9, kDebugLevelGUI, "KyraEngine::gui_loadGameMenu()");
-	if (_menuDirectlyToLoad)
+	if (_menuDirectlyToLoad) {
 		_menu[2].item[5].enabled = false;
-	else {
+	} else {
 		processMenuButton(button);
 		_menu[2].item[5].enabled = true;
 	}
@@ -1019,26 +1002,26 @@ void KyraEngine::gui_redrawTextfield() {
 void KyraEngine::gui_updateSavegameString() {
 	int length;
 
-	if (_keyboardEvent.pending && _keyboardEvent.ascii) {
+	if (_keyPressed) {
 		length = strlen(_savegameName);
 
-		if (_keyboardEvent.ascii > 31 && _keyboardEvent.ascii < 127) {
+		if (_keyPressed > 31 && _keyPressed < 127) {
 			if (length < 31) {
-				_savegameName[length] = _keyboardEvent.ascii;
+				_savegameName[length] = _keyPressed;
 				_savegameName[length+1] = 0;
 				gui_redrawTextfield();
 			}
-		} else if (_keyboardEvent.ascii == 8 ||_keyboardEvent.ascii == 127) {
+		} else if (_keyPressed == 8 ||_keyPressed == 127) {
 			if (length > 0) {
 				_savegameName[length-1] = 0;
 				gui_redrawTextfield();
 			}
-		} else if (_keyboardEvent.ascii == 13) {
+		} else if (_keyPressed == 13) {
 			_displaySubMenu = false;
 		}
 	}
 
-	_keyboardEvent.pending = false;
+	_keyPressed = 0;
 }
 
 int KyraEngine::gui_saveGame(Button *button) {
@@ -1119,9 +1102,9 @@ int KyraEngine::gui_quitPlaying(Button *button) {
 	debugC(9, kDebugLevelGUI, "KyraEngine::gui_quitPlaying()");
 	processMenuButton(button);
 
-	if (gui_quitConfirm(_guiStrings[14])) // Are you sure you want to quit playing?
+	if (gui_quitConfirm(_guiStrings[14])) { // Are you sure you want to quit playing?
 		quitGame();
-	else {
+	} else {
 		initMenu(_menu[_toplevelMenu]);
 		processAllMenuButtons();
 	}
@@ -1136,8 +1119,9 @@ bool KyraEngine::gui_quitConfirm(const char *str) {
 	_screen->savePageToDisk("SEENPAGE.TMP", 0);
 
 	_menu[1].menuName = str;
+	calcCoords(_menu[1]);
 	initMenu(_menu[1]);
-
+	
 	_displaySubMenu = true;
 	_cancelSubMenu = true;
 
@@ -1179,27 +1163,26 @@ int KyraEngine::gui_gameControlsMenu(Button *button) {
 	_screen->loadPageFromDisk("SEENPAGE.TMP", 0);
 	_screen->savePageToDisk("SEENPAGE.TMP", 0);
 
-	if (_features & GF_TALKIE) {
+	if (_flags.isTalkie) {
 		//_menu[5].width = 230;
 
 		for (int i = 0; i < 5; i++) {
 			//_menu[5].item[i].labelX = 24;
 			//_menu[5].item[i].x = 115;
-			_menu[5].item[i].width = 94;
+			//_menu[5].item[i].width = 94;
 		}
 
 		_menu[5].item[3].labelString = _voiceTextString; //"Voice / Text "
 		_menu[5].item[3].callback = &KyraEngine::gui_controlsChangeVoice;
 
 	} else {
-		_menu[5].height = 136;
-		_menu[5].item[5].y = 110;
+		//_menu[5].height = 136;
+		//_menu[5].item[5].y = 110;
 		_menu[5].item[4].enabled = 0;
 		_menu[5].item[3].labelString = _textSpeedString; // "Text speed "
 		_menu[5].item[3].callback = &KyraEngine::gui_controlsChangeText;
 	}
 
-	calcCoords(_menu[5]);
 	gui_setupControls(_menu[5]);
 
 	processAllMenuButtons();
@@ -1226,10 +1209,17 @@ int KyraEngine::gui_gameControlsMenu(Button *button) {
 void KyraEngine::gui_setupControls(Menu &menu) {
 	debugC(9, kDebugLevelGUI, "KyraEngine::gui_setupControls()");
 
-	if (_configMusic)
-		menu.item[0].itemString = _onString; //"On"
-	else
-		menu.item[0].itemString = _offString; //"Off"
+	switch (_configMusic) {
+		case 0:
+			menu.item[0].itemString = _offString; //"Off"
+			break;
+		case 1:
+			menu.item[0].itemString = _onString; //"On"
+			break;
+		case 2:
+			menu.item[0].itemString = _onCDString; //"On + CD"
+			break;
+	}
 
 	if (_configSounds)
 		menu.item[1].itemString = _onString; //"On"
@@ -1260,7 +1250,7 @@ void KyraEngine::gui_setupControls(Menu &menu) {
 
 	int textControl = 3;
 	int clickableOffset = 8;
-	if (_features & GF_TALKIE) {
+	if (_flags.isTalkie) {
 		textControl = 4;
 		clickableOffset = 11;
 		
@@ -1303,7 +1293,7 @@ void KyraEngine::gui_setupControls(Menu &menu) {
 		break;
 	}
 
-
+	calcCoords(menu);
 	initMenu(menu);
 }
 
@@ -1311,7 +1301,7 @@ int KyraEngine::gui_controlsChangeMusic(Button *button) {
 	debugC(9, kDebugLevelGUI, "KyraEngine::gui_controlsChangeMusic()");
 	processMenuButton(button);
 
-	_configMusic = !_configMusic;
+	_configMusic = ++_configMusic % (_flags.platform == Common::kPlatformFMTowns ? 3 : 2);
 	gui_setupControls(_menu[5]);
 	return 0;
 }
@@ -1385,6 +1375,7 @@ int KyraEngine::gui_scrollDown(Button *button) {
 void KyraEngine::gui_processHighlights(Menu &menu) {
 	int x1, y1, x2, y2;
 
+	Common::Point mouse = getMousePos();
 	for (int i = 0; i < menu.nrOfItems; i++) {
 		if (!menu.item[i].enabled)
 			continue;
@@ -1395,8 +1386,8 @@ void KyraEngine::gui_processHighlights(Menu &menu) {
 		x2 = x1 + menu.item[i].width;
 		y2 = y1 + menu.item[i].height;
 
-		if (_mouseX > x1 && _mouseX < x2 &&
-			_mouseY > y1 && _mouseY < y2) {
+		if (mouse.x > x1 && mouse.x < x2 &&
+			mouse.y > y1 && mouse.y < y2) {
 			
 			if (menu.highlightedItem != i) {
 				if (menu.item[menu.highlightedItem].enabled )
@@ -1419,7 +1410,7 @@ void KyraEngine::gui_redrawText(Menu menu) {
 
 	int x2 = x1 + menu.item[i].width - 1;
 
-	if (menu.item[i].field_12 != -1)
+	if (menu.item[i].field_12 != -1 &&_flags.lang == Common::EN_ANY)
 		textX = x1 + menu.item[i].field_12 + 3;
 	else
 		textX = _text->getCenterStringX(menu.item[i].itemString, x1, x2);
@@ -1438,7 +1429,7 @@ void KyraEngine::gui_redrawHighlight(Menu menu) {
 
 	int x2 = x1 + menu.item[i].width - 1;
 
-	if (menu.item[i].field_12 != -1)
+	if (menu.item[i].field_12 != -1 &&_flags.lang == Common::EN_ANY)
 		textX = x1 + menu.item[i].field_12 + 3;
 	else
 		textX = _text->getCenterStringX(menu.item[i].itemString, x1, x2);
@@ -1449,16 +1440,18 @@ void KyraEngine::gui_redrawHighlight(Menu menu) {
 }
 
 void KyraEngine::gui_fadePalette() {
+	if (_flags.platform == Common::kPlatformAmiga)
+		return;
+
 	static int16 menuPalIndexes[] = {248, 249, 250, 251, 252, 253, 254, -1};
 	int index = 0;
 
 	memcpy(_screen->getPalette(2), _screen->_currentPalette, 768);
 
-	for (int i = 0; i < 768; i++) {
-		_screen->_currentPalette[i] /= 2;
-	}
+	for (int i = 0; i < 768; i++)
+		_screen->_currentPalette[i] >>= 1;
 
-	while( menuPalIndexes[index] != -1) {
+	while (menuPalIndexes[index] != -1) {
 		memcpy(&_screen->_currentPalette[menuPalIndexes[index]*3], &_screen->getPalette(2)[menuPalIndexes[index]*3], 3);
 		index++;
 	}
@@ -1467,10 +1460,179 @@ void KyraEngine::gui_fadePalette() {
 }
 
 void KyraEngine::gui_restorePalette() {
+	if (_flags.platform == Common::kPlatformAmiga)
+		return;
+
 	memcpy(_screen->_currentPalette, _screen->getPalette(2), 768);
 	_screen->fadePalette(_screen->_currentPalette, 2);
 }
 
+#pragma mark -
+
+// Kyra 2 and 3 main menu
+
+void KyraEngine::gui_updateMainMenuAnimation() {
+	_screen->updateScreen();
+}
+
+bool KyraEngine::gui_mainMenuGetInput() {
+	Common::Event event;
+
+	while (_eventMan->pollEvent(event)) {
+		switch (event.type) {
+		case Common::EVENT_QUIT:
+			quitGame();
+			break;
+		case Common::EVENT_LBUTTONUP:
+			return true;
+		default:
+			break;
+		}
+	}
+	return false;
+}
+
+int KyraEngine::gui_handleMainMenu() {
+	debugC(9, kDebugLevelMain, "KyraEngine::gui_handleMainMenu()");
+	int command = -1;
+	
+	uint8 colorMap[16];
+	memset(colorMap, 0, sizeof(colorMap));
+	_screen->setTextColorMap(colorMap);
+	
+	const char * const *strings = &_mainMenuStrings[_lang << 2];
+	Screen::FontId oldFont = _screen->setFont(Screen::FID_8_FNT);
+	int charWidthBackUp = _screen->_charWidth;
+	
+	_screen->_charWidth = -2;
+	_screen->setScreenDim(3);
+	int backUpX = _screen->_curDim->sx;
+	int backUpY = _screen->_curDim->sy;
+	int backUpWidth = _screen->_curDim->w;
+	int backUpHeight = _screen->_curDim->h;
+	_screen->copyRegion(backUpX, backUpY, backUpX, backUpY, backUpWidth, backUpHeight, 0, 3);
+
+	int x = _screen->_curDim->sx << 3;
+	int y = _screen->_curDim->sy;
+	int width = _screen->_curDim->w << 3;
+	int height =  _screen->_curDim->h;
+
+	gui_drawMainBox(x, y, width, height, 1);
+	gui_drawMainBox(x + 1, y + 1, width - 2, height - 2, 0);
+	
+	int selected = 0;
+	
+	gui_drawMainMenu(strings, selected);
+
+	_screen->showMouse();
+
+	int fh = _screen->getFontHeight();
+	int textPos = ((_screen->_curDim->w >> 1) + _screen->_curDim->sx) << 3;
+
+	Common::Rect menuRect(x + 16, y + 4, x + width - 16, y + 4 + fh * 4);
+	
+	while (!_quitFlag) {
+		gui_updateMainMenuAnimation();
+		bool mousePressed = gui_mainMenuGetInput();
+
+		Common::Point mouse = getMousePos();
+		if (menuRect.contains(mouse)) {
+			int item = (mouse.y - menuRect.top) / fh;
+
+			if (item != selected) {
+				gui_printString(strings[selected], textPos, menuRect.top + selected * fh, 0x80, 0, 5);
+				gui_printString(strings[item], textPos, menuRect.top + item * fh, 0xFF, 0, 5);
+
+				selected = item;
+			}
+
+			if (mousePressed) {
+				// TODO: Flash the text
+				command = item;
+				break;
+			}
+		}
+		_system->delayMillis(10);
+	}
+	
+	if (_quitFlag)
+		command = -1;
+	
+	_screen->copyRegion(backUpX, backUpY, backUpX, backUpY, backUpWidth, backUpHeight, 3, 0);
+	_screen->_charWidth = charWidthBackUp;
+	_screen->setFont(oldFont);
+	
+	return command;
+}
+
+void KyraEngine::gui_drawMainMenu(const char * const *strings, int select) {
+	debugC(9, kDebugLevelMain, "KyraEngine::gui_drawMainMenu(%p)", (const void*)strings);
+	static const uint16 menuTable[] = { 0x01, 0x04, 0x0C, 0x04, 0x00, 0x80, 0xFF, 0x00, 0x01, 0x02, 0x03 };
+	
+	int top = _screen->_curDim->sy;
+	top += menuTable[1];
+	
+	for (int i = 0; i < menuTable[3]; ++i) {
+		int curY = top + i * _screen->getFontHeight();
+		int color = (i == select) ? menuTable[6] : menuTable[5];
+		gui_printString(strings[i], ((_screen->_curDim->w >> 1) + _screen->_curDim->sx) << 3, curY, color, 0, 5);
+	}
+}
+
+void KyraEngine::gui_drawMainBox(int x, int y, int w, int h, int fill) {
+	debugC(9, kDebugLevelMain, "KyraEngine::gui_drawMainBox(%d, %d, %d, %d, %d)", x, y, w, h, fill);
+	static const uint8 kyra3ColorTable[] = { 0x16, 0x19, 0x1A, 0x16 };
+	static const uint8 kyra2ColorTable[] = { 0x0, 0x19, 0x28, 0xc8 };
+	
+	const uint8 *colorTable;
+	if (_flags.gameID == GI_KYRA3)
+		colorTable = kyra3ColorTable;
+	else
+		colorTable = kyra2ColorTable;
+		
+	--w; --h;
+
+	if (fill)
+		_screen->fillRect(x, y, x+w, y+h, colorTable[0]);
+	
+	_screen->drawClippedLine(x, y+h, x+w, y+h, colorTable[1]);
+	_screen->drawClippedLine(x+w, y, x+w, y+h, colorTable[1]);
+	_screen->drawClippedLine(x, y, x+w, y, colorTable[2]);
+	_screen->drawClippedLine(x, y, x, y+h, colorTable[2]);
+	
+	_screen->setPagePixel(_screen->_curPage, x, y+h, colorTable[3]);
+	_screen->setPagePixel(_screen->_curPage, x+w, y, colorTable[3]);
+}
+
+void KyraEngine::gui_printString(const char *format, int x, int y, int col1, int col2, int flags, ...) {
+	debugC(9, kDebugLevelMain, "KyraEngine::gui_printString('%s', %d, %d, %d, %d, %d, ...)", format, x, y, col1, col2, flags);
+	if (!format)
+		return;
+	
+	char string[512];
+	va_list vaList;
+	va_start(vaList, flags);
+	vsprintf(string, format, vaList);
+	va_end(vaList);
+	
+	if (flags & 1)
+		x -= _screen->getTextWidth(string) >> 1;
+	
+	if (flags & 2)
+		x -= _screen->getTextWidth(string);
+	
+	if (flags & 4) {
+		_screen->printText(string, x - 1, y, 240, col2);
+		_screen->printText(string, x, y + 1, 240, col2);
+	}
+	
+	if (flags & 8) {
+		_screen->printText(string, x - 1, y, 227, col2);
+		_screen->printText(string, x, y + 1, 227, col2);
+	}
+	
+	_screen->printText(string, x, y, col1, col2);
+}
 
 } // end of namespace Kyra
  
