@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2005-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/kyra/items.cpp $
- * $Id: items.cpp 22393 2006-05-08 15:40:30Z eriktorbjorn $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/kyra/items.cpp $
+ * $Id: items.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -148,9 +151,8 @@ void KyraEngine::placeItemInGenericMapScene(int item, int index) {
 			break;
 			
 		case 51:
-			if (room != 46) {
+			if (room != 46)
 				placeItem = true;
-			}
 			break;
 
 		default:
@@ -159,11 +161,10 @@ void KyraEngine::placeItemInGenericMapScene(int item, int index) {
 		
 		if (placeItem) {
 			Room *roomPtr = &_roomTable[room];
-			if (roomPtr->northExit == 0xFFFF && roomPtr->eastExit == 0xFFFF && roomPtr->southExit == 0xFFFF && roomPtr->westExit == 0xFFFF) {
+			if (roomPtr->northExit == 0xFFFF && roomPtr->eastExit == 0xFFFF && roomPtr->southExit == 0xFFFF && roomPtr->westExit == 0xFFFF)
 				placeItem = false;
-			} else if (_currentCharacter->sceneId == room) {
+			else if (_currentCharacter->sceneId == room)
 				placeItem = false;
-			}
 		}
 		
 		if (placeItem) {
@@ -185,18 +186,17 @@ void KyraEngine::createMouseItem(int item) {
 void KyraEngine::destroyMouseItem() {
 	debugC(9, kDebugLevelMain, "KyraEngine::destroyMouseItem()");
 	_screen->hideMouse();
-	_screen->setMouseCursor(1, 1, _shapes[4]);
+	_screen->setMouseCursor(1, 1, _shapes[0]);
 	_itemInHand = -1;
 	_screen->showMouse();
 }
 
 void KyraEngine::setMouseItem(int item) {
 	debugC(9, kDebugLevelMain, "KyraEngine::setMouseItem(%d)", item);
-	if (item == -1) {
-		_screen->setMouseCursor(1, 1, _shapes[10]);
-	} else {
-		_screen->setMouseCursor(8, 15, _shapes[220+item]);
-	}
+	if (item == -1)
+		_screen->setMouseCursor(1, 1, _shapes[6]);
+	else
+		_screen->setMouseCursor(8, 15, _shapes[216+item]);
 }
 
 void KyraEngine::wipeDownMouseItem(int xpos, int ypos) {
@@ -206,22 +206,22 @@ void KyraEngine::wipeDownMouseItem(int xpos, int ypos) {
 	xpos -= 8;
 	ypos -= 15;
 	_screen->hideMouse();
-	_screen->backUpRect1(xpos, ypos);
+	backUpItemRect1(xpos, ypos);
 	int y = ypos;
 	int height = 16;
 	
 	while (height >= 0) {
-		_screen->restoreRect1(xpos, ypos);
-		_screen->setNewShapeHeight(_shapes[220+_itemInHand], height);
+		restoreItemRect1(xpos, ypos);
+		_screen->setNewShapeHeight(_shapes[216+_itemInHand], height);
 		uint32 nextTime = _system->getMillis() + 1 * _tickLength;
-		_screen->drawShape(0, _shapes[220+_itemInHand], xpos, y, 0, 0);
+		_screen->drawShape(0, _shapes[216+_itemInHand], xpos, y, 0, 0);
 		_screen->updateScreen();
 		y += 2;
 		height -= 2;
 		delayUntil(nextTime);
 	}	
-	_screen->restoreRect1(xpos, ypos);
-	_screen->resetShapeHeight(_shapes[220+_itemInHand]);
+	restoreItemRect1(xpos, ypos);
+	_screen->resetShapeHeight(_shapes[216+_itemInHand]);
 	destroyMouseItem();
 	_screen->showMouse();
 }
@@ -233,9 +233,8 @@ void KyraEngine::setupSceneItems() {
 	Room *currentRoom = &_roomTable[sceneId];
 	for (int i = 0; i < 12; ++i) {
 		uint8 item = currentRoom->itemsTable[i];
-		if (item == 0xFF || !currentRoom->needInit[i]) {
+		if (item == 0xFF || !currentRoom->needInit[i])
 			continue;
-		}
 		
 		int xpos = 0;
 		int ypos = 0;
@@ -256,9 +255,8 @@ void KyraEngine::setupSceneItems() {
 			if (!stop) {
 				xpos = currentRoom->itemsXPos[i] = _rnd.getRandomNumberRng(24, 296);
 				ypos = currentRoom->itemsYPos[i] = _rnd.getRandomNumberRng(_northExitHeight & 0xFF, 130);
-				if (countItemsInScene(sceneId) >= 12) {
+				if (countItemsInScene(sceneId) >= 12)
 					break;
-				}
 			} else {
 				currentRoom->needInit[i] = 0;
 			}
@@ -274,9 +272,8 @@ int KyraEngine::countItemsInScene(uint16 sceneId) {
 	int items = 0;
 	
 	for (int i = 0; i < 12; ++i) {
-		if (currentRoom->itemsTable[i] != 0xFF) {
+		if (currentRoom->itemsTable[i] != 0xFF)
 			++items;
-		}
 	}
 	
 	return items;
@@ -286,9 +283,8 @@ int KyraEngine::processItemDrop(uint16 sceneId, uint8 item, int x, int y, int un
 	debugC(9, kDebugLevelMain, "KyraEngine::processItemDrop(%d, %d, %d, %d, %d, %d)", sceneId, item, x, y, unk1, unk2);
 	int freeItem = -1;
 	uint8 itemIndex = findItemAtPos(x, y);
-	if (unk1) {
+	if (unk1)
 		itemIndex = 0xFF;
-	}
 	
 	if (itemIndex != 0xFF) {
 		exchangeItemWithMouseItem(sceneId, itemIndex);
@@ -309,9 +305,8 @@ int KyraEngine::processItemDrop(uint16 sceneId, uint8 item, int x, int y, int un
 		freeItem = _lastProcessedItem;
 	}
 
-	if (freeItem == -1) {
+	if (freeItem == -1)
 		return 0;
-	}
 	
 	if (sceneId != _currentCharacter->sceneId) {
 		addItemToRoom(sceneId, item, freeItem, x, y);
@@ -337,21 +332,18 @@ int KyraEngine::processItemDrop(uint16 sceneId, uint8 item, int x, int y, int un
 			bool running2 = true;
 			
 			if (_screen->getDrawLayer(xpos, ypos) > 1) {
-				if (((_northExitHeight >> 8) & 0xFF) != ypos) {
+				if (((_northExitHeight >> 8) & 0xFF) != ypos)
 					running2 = false;
-				}
 			}
 			
 			if (_screen->getDrawLayer2(xpos, ypos, itemHeight) > 1) {
-				if (((_northExitHeight >> 8) & 0xFF) != ypos) {
+				if (((_northExitHeight >> 8) & 0xFF) != ypos)
 					running2 = false;
-				}
 			}
 			
 			if (!isDropable(xpos, ypos)) {
-				if (((_northExitHeight >> 8) & 0xFF) != ypos) {
+				if (((_northExitHeight >> 8) & 0xFF) != ypos)
 					running2 = false;
-				}
 			}
 			
 			int xpos2 = xpos;
@@ -384,14 +376,12 @@ int KyraEngine::processItemDrop(uint16 sceneId, uint8 item, int x, int y, int un
 					continue;
 				
 				xpos2 -= 2;
-				if (xpos2 < 16) {
+				if (xpos2 < 16)
 					xpos2 = 16;
-				}
 				
 				xpos3 += 2;
-				if (xpos3 > 304) {
+				if (xpos3 > 304)
 					xpos3 = 304;
-				}
 				
 				if (xpos2 > 16)
 					continue;
@@ -405,23 +395,20 @@ int KyraEngine::processItemDrop(uint16 sceneId, uint8 item, int x, int y, int un
 			running = 0;
 			destY -= _rnd.getRandomNumberRng(0, 3);
 			
-			if ((_northExitHeight & 0xFF) < destY) {
+			if ((_northExitHeight & 0xFF) < destY)
 				continue;
-			}
 			
 			destY = (_northExitHeight & 0xFF) + 1;
 			continue;
 		}		
 		ypos += 2;
-		if (((_northExitHeight >> 8) & 0xFF) >= ypos) {
+		if (((_northExitHeight >> 8) & 0xFF) >= ypos)
 			continue;
-		}
 		ypos = (_northExitHeight >> 8) & 0xFF;
 	}
 	
-	if (destX == -1 || destY == -1) {
+	if (destX == -1 || destY == -1)
 		return 0;
-	}
 	
 	if (unk1 == 3) {
 		currentRoom->itemsXPos[freeItem] = destX;
@@ -429,13 +416,11 @@ int KyraEngine::processItemDrop(uint16 sceneId, uint8 item, int x, int y, int un
 		return 1;
 	}
 	
-	if (unk1 == 2) {
+	if (unk1 == 2)
 		itemSpecialFX(x, y, item);
-	}
 	
-	if (unk1 == 0) {
+	if (unk1 == 0)
 		destroyMouseItem();
-	}
 	
 	itemDropDown(x, y, destX, destY, freeItem, item);
 	
@@ -479,17 +464,14 @@ void KyraEngine::addItemToRoom(uint16 sceneId, uint8 item, int itemIndex, int x,
 
 int KyraEngine::checkNoDropRects(int x, int y) {
 	debugC(9, kDebugLevelMain, "KyraEngine::checkNoDropRects(%d, %d)", x, y);
-	if (_lastProcessedItemHeight < 1 || _lastProcessedItemHeight > 16) {
+	if (_lastProcessedItemHeight < 1 || _lastProcessedItemHeight > 16)
 		_lastProcessedItemHeight = 16;
-	}
-	if (_noDropRects[0].x == -1) {
+	if (_noDropRects[0].x == -1)
 		return 0;
-	}
 	
 	for (int i = 0; i < 11; ++i) {
-		if (_noDropRects[i].x == -1) {
+		if (_noDropRects[i].x == -1)
 			break;
-		}
 		
 		int xpos = _noDropRects[i].x;
 		int ypos = _noDropRects[i].y;
@@ -515,14 +497,12 @@ int KyraEngine::isDropable(int x, int y) {
 	x -= 8;
 	y -= 1;
 	
-	if (checkNoDropRects(x, y)) {
+	if (checkNoDropRects(x, y))
 		return 0;
-	}
 	
 	for (int xpos = x; xpos < x + 16; ++xpos) {
-		if (_screen->getShapeFlag1(xpos, y) == 0) {
+		if (_screen->getShapeFlag1(xpos, y) == 0)
 			return 0;
-		}
 	}	
 	return 1;
 }
@@ -546,28 +526,26 @@ void KyraEngine::itemDropDown(int x, int y, int destX, int destY, byte freeItem,
 		int drawX = x - 8;
 		int drawY = 0;
 		
-		_screen->backUpRect0(drawX, y - 16);
+		backUpItemRect0(drawX, y - 16);
 		
 		while (tempY < destY) {
-			_screen->restoreRect0(drawX, tempY - 16);
+			restoreItemRect0(drawX, tempY - 16);
 			tempY += addY;
-			if (tempY > destY) {
+			if (tempY > destY)
 				tempY = destY;
-			}
 			++addY;
 			drawY = tempY - 16;
-			_screen->backUpRect0(drawX, drawY);
+			backUpItemRect0(drawX, drawY);
 			uint32 nextTime = _system->getMillis() + 1 * _tickLength;
-			_screen->drawShape(0, _shapes[220+item], drawX, drawY, 0, 0);
+			_screen->drawShape(0, _shapes[216+item], drawX, drawY, 0, 0);
 			_screen->updateScreen();
 			delayUntil(nextTime);
 		}
 		
 		bool skip = false;
 		if (x == destX) {
-			if (destY - y <= 16) {
+			if (destY - y <= 16)
 				skip = true;
-			}
 		}
 		
 		if (!skip) {
@@ -579,32 +557,30 @@ void KyraEngine::itemDropDown(int x, int y, int destX, int destY, byte freeItem,
 			xDiff /= addY;
 			int startAddY = addY;
 			addY >>= 1;
-			if (destY - y <= 8) {
+			if (destY - y <= 8)
 				addY >>= 1;
-			}
 			addY = -addY;
 			int unkX = x << 4;
 			while (--startAddY) {
 				drawX = (unkX >> 4) - 8;
 				drawY = tempY - 16;
-				_screen->restoreRect0(drawX, drawY);
+				restoreItemRect0(drawX, drawY);
 				tempY += addY;
 				unkX += xDiff;
-				if (tempY > destY) {
+				if (tempY > destY)
 					tempY = destY;
-				}
 				++addY;
 				drawX = (unkX >> 4) - 8;
 				drawY = tempY - 16;
-				_screen->backUpRect0(drawX, drawY);
+				backUpItemRect0(drawX, drawY);
 				uint32 nextTime = _system->getMillis() + 1 * _tickLength;
-				_screen->drawShape(0, _shapes[220+item], drawX, drawY, 0, 0);
+				_screen->drawShape(0, _shapes[216+item], drawX, drawY, 0, 0);
 				_screen->updateScreen();
 				delayUntil(nextTime);
 			}
-			_screen->restoreRect0(drawX, drawY);
+			restoreItemRect0(drawX, drawY);
 		} else {
-			_screen->restoreRect0(drawX, tempY - 16);
+			restoreItemRect0(drawX, tempY - 16);
 		}
 	}
 	currentRoom->itemsXPos[freeItem] = destX;
@@ -620,42 +596,39 @@ void KyraEngine::dropItem(int unk1, int item, int x, int y, int unk2) {
 	if (processItemDrop(_currentCharacter->sceneId, item, x, y, unk1, unk2))
 		return;
 	snd_playSoundEffect(54);
-	if (12 == countItemsInScene(_currentCharacter->sceneId)) {
-		assert(_noDropList);
+	assert(_noDropList);
+	if (12 == countItemsInScene(_currentCharacter->sceneId))
 		drawSentenceCommand(_noDropList[0], 6);
-	} else {
-		assert(_noDropList);
+	else
 		drawSentenceCommand(_noDropList[1], 6);
-	}
 }
 
 void KyraEngine::itemSpecialFX(int x, int y, int item) {
 	debugC(9, kDebugLevelMain, "KyraEngine::itemSpecialFX(%d, %d, %d)", x, y, item);
-	if (item == 41) {
+	if (item == 41)
 		itemSpecialFX1(x, y, item);
-	} else {
+	else
 		itemSpecialFX2(x, y, item);
-	}
 }
 
 void KyraEngine::itemSpecialFX1(int x, int y, int item) {
 	debugC(9, kDebugLevelMain, "KyraEngine::itemSpecialFX1(%d, %d, %d)", x, y, item);
-	uint8 *shape = _shapes[220+item];
+	uint8 *shape = _shapes[216+item];
 	x -= 8;
 	int startY = y;
 	y -= 15;
 	_screen->hideMouse();
-	_screen->backUpRect0(x, y);
+	backUpItemRect0(x, y);
 	for (int i = 1; i <= 16; ++i) {
 		_screen->setNewShapeHeight(shape, i);
 		--startY;
-		_screen->restoreRect0(x, y);
+		restoreItemRect0(x, y);
 		uint32 nextTime = _system->getMillis() + 1 * _tickLength;
 		_screen->drawShape(0, shape, x, startY, 0, 0);
 		_screen->updateScreen();
 		delayUntil(nextTime);
 	}
-	_screen->restoreRect0(x, y);
+	restoreItemRect0(x, y);
 	_screen->showMouse();
 }
 
@@ -664,28 +637,27 @@ void KyraEngine::itemSpecialFX2(int x, int y, int item) {
 	x -= 8;
 	y -= 15;
 	int yAdd = (int8)(((16 - _itemTable[item].height) >> 1) & 0xFF);
-	_screen->backUpRect0(x, y);
-	if (item >= 80 && item <= 89) {
+	backUpItemRect0(x, y);
+	if (item >= 80 && item <= 89)
 		snd_playSoundEffect(55);
-	}
 	
 	for (int i = 201; i <= 205; ++i) {
-		_screen->restoreRect0(x, y);
+		restoreItemRect0(x, y);
 		uint32 nextTime = _system->getMillis() + 3 * _tickLength;
-		_screen->drawShape(0, _shapes[4+i], x, y + yAdd, 0, 0);
+		_screen->drawShape(0, _shapes[i], x, y + yAdd, 0, 0);
 		_screen->updateScreen();
 		delayUntil(nextTime);
 	}
 	
 	for (int i = 204; i >= 201; --i) {
-		_screen->restoreRect0(x, y);
+		restoreItemRect0(x, y);
 		uint32 nextTime = _system->getMillis() + 3 * _tickLength;
-		_screen->drawShape(0, _shapes[220+item], x, y, 0, 0);
-		_screen->drawShape(0, _shapes[4+i], x, y + yAdd, 0, 0);
+		_screen->drawShape(0, _shapes[216+item], x, y, 0, 0);
+		_screen->drawShape(0, _shapes[i], x, y + yAdd, 0, 0);
 		_screen->updateScreen();
 		delayUntil(nextTime);
 	}
-	_screen->restoreRect0(x, y);
+	restoreItemRect0(x, y);
 }
 
 void KyraEngine::magicOutMouseItem(int animIndex, int itemPos) {
@@ -694,16 +666,16 @@ void KyraEngine::magicOutMouseItem(int animIndex, int itemPos) {
 	_screen->_curPage = 0;
 	int x = 0, y = 0;
 	if (itemPos == -1) {
-		x = _mouseX - 12;
-		y = _mouseY - 18;
+		Common::Point mouse = getMousePos();
+		x = mouse.x - 12;
+		y = mouse.y - 18;
 	} else {
 		x = _itemPosX[itemPos] - 4;
 		y = _itemPosY[itemPos] - 3;
 	}
 	
-	if (_itemInHand == -1 && itemPos == -1) {
+	if (_itemInHand == -1 && itemPos == -1)
 		return;
-	}
 	
 	int tableIndex = 0, loopStart = 0, maxLoops = 0;
 	if (animIndex == 0) {
@@ -722,48 +694,45 @@ void KyraEngine::magicOutMouseItem(int animIndex, int itemPos) {
 		tableIndex = -1;
 	}
 	
-	if (animIndex == 2) {
+	if (animIndex == 2)
 		snd_playSoundEffect(0x5E);
-	} else {
+	else
 		snd_playSoundEffect(0x37);
-	}
 	_screen->hideMouse();
-	_screen->backUpRect1(x, y);
+	backUpItemRect1(x, y);
 
 	for (int shape = _magicMouseItemStartFrame[animIndex]; shape <= _magicMouseItemEndFrame[animIndex]; ++shape) {
-		_screen->restoreRect1(x, y);
+		restoreItemRect1(x, y);
 		uint32 nextTime = _system->getMillis() + 4 * _tickLength;
-		_screen->drawShape(0, _shapes[220+_itemInHand], x + 4, y + 3, 0, 0);
-		if (tableIndex == -1) {
-			_screen->drawShape(0, _shapes[4+shape], x, y, 0, 0);
-		} else {
+		_screen->drawShape(0, _shapes[216+_itemInHand], x + 4, y + 3, 0, 0);
+		if (tableIndex == -1)
+			_screen->drawShape(0, _shapes[shape], x, y, 0, 0);
+		else
 			specialMouseItemFX(shape, x, y, animIndex, tableIndex, loopStart, maxLoops);
-		}
 		_screen->updateScreen();
 		delayUntil(nextTime);
 	}
 	
 	if (itemPos != -1) {
-		_screen->restoreRect1(x, y);
+		restoreItemRect1(x, y);
 		_screen->fillRect(_itemPosX[itemPos], _itemPosY[itemPos], _itemPosX[itemPos] + 15, _itemPosY[itemPos] + 15, 12, 0);
-		_screen->backUpRect1(x, y);
+		backUpItemRect1(x, y);
 	}
 	
 	for (int shape = _magicMouseItemStartFrame2[animIndex]; shape <= _magicMouseItemEndFrame2[animIndex]; ++shape) {
-		_screen->restoreRect1(x, y);
+		restoreItemRect1(x, y);
 		uint32 nextTime = _system->getMillis() + 4 * _tickLength;
-		_screen->drawShape(0, _shapes[220+_itemInHand], x + 4, y + 3, 0, 0);
-		if (tableIndex == -1) {
-			_screen->drawShape(0, _shapes[4+shape], x, y, 0, 0);
-		} else {
+		_screen->drawShape(0, _shapes[216+_itemInHand], x + 4, y + 3, 0, 0);
+		if (tableIndex == -1)
+			_screen->drawShape(0, _shapes[shape], x, y, 0, 0);
+		else
 			specialMouseItemFX(shape, x, y, animIndex, tableIndex, loopStart, maxLoops);
-		}
 		_screen->updateScreen();
 		delayUntil(nextTime);
 	}
-	_screen->restoreRect1(x, y);
+	restoreItemRect1(x, y);
 	if (itemPos == -1) {
-		_screen->setMouseCursor(1, 1, _shapes[4]);
+		_screen->setMouseCursor(1, 1, _shapes[0]);
 		_itemInHand = -1;
 	} else {
 		_characterList[0].inventoryItems[itemPos] = 0xFF;
@@ -781,8 +750,9 @@ void KyraEngine::magicInMouseItem(int animIndex, int item, int itemPos) {
 	_screen->_curPage = 0;
 	int x = 0, y = 0;
 	if (itemPos == -1) {
-		x = _mouseX - 12;
-		y = _mouseY - 18;
+		Common::Point mouse = getMousePos();
+		x = mouse.x - 12;
+		y = mouse.y - 18;
 	} else {
 		x = _itemPosX[itemPos] - 4;
 		y = _itemPosX[itemPos] - 3;
@@ -806,44 +776,41 @@ void KyraEngine::magicInMouseItem(int animIndex, int item, int itemPos) {
 	}
 	
 	_screen->hideMouse();
-	_screen->backUpRect1(x, y);
-	if (animIndex == 2) {
+	backUpItemRect1(x, y);
+	if (animIndex == 2)
 		snd_playSoundEffect(0x5E);
-	} else {
+	else
 		snd_playSoundEffect(0x37);
-	}
 	
 	for (int shape = _magicMouseItemStartFrame[animIndex]; shape <= _magicMouseItemEndFrame[animIndex]; ++shape) {
-		_screen->restoreRect1(x, y);
+		restoreItemRect1(x, y);
 		uint32 nextTime = _system->getMillis() + 4 * _tickLength;
-		if (tableIndex == -1) {
-			_screen->drawShape(0, _shapes[4+shape], x, y, 0, 0);
-		} else {
+		if (tableIndex == -1)
+			_screen->drawShape(0, _shapes[shape], x, y, 0, 0);
+		else
 			specialMouseItemFX(shape, x, y, animIndex, tableIndex, loopStart, maxLoops);
-		}
 		_screen->updateScreen();
 		delayUntil(nextTime);
 	}
 	
 	for (int shape = _magicMouseItemStartFrame2[animIndex]; shape <= _magicMouseItemEndFrame2[animIndex]; ++shape) {
-		_screen->restoreRect1(x, y);
+		restoreItemRect1(x, y);
 		uint32 nextTime = _system->getMillis() + 4 * _tickLength;
-		if (tableIndex == -1) {
-			_screen->drawShape(0, _shapes[4+shape], x, y, 0, 0);
-		} else {
+		if (tableIndex == -1)
+			_screen->drawShape(0, _shapes[shape], x, y, 0, 0);
+		else
 			specialMouseItemFX(shape, x, y, animIndex, tableIndex, loopStart, maxLoops);
-		}
 		_screen->updateScreen();
 		delayUntil(nextTime);
 	}
-	_screen->restoreRect1(x, y);
+	restoreItemRect1(x, y);
 	if (itemPos == -1) {
-		_screen->setMouseCursor(8, 15, _shapes[220+item]);
+		_screen->setMouseCursor(8, 15, _shapes[216+item]);
 		_itemInHand = item;
 	} else {
 		_characterList[0].inventoryItems[itemPos] = item;
 		_screen->hideMouse();
-		_screen->drawShape(0, _shapes[220+item], _itemPosX[itemPos], _itemPosY[itemPos], 0, 0);
+		_screen->drawShape(0, _shapes[216+item], _itemPosX[itemPos], _itemPosY[itemPos], 0, 0);
 		_screen->showMouse();
 	}
 	_screen->showMouse();
@@ -862,35 +829,34 @@ void KyraEngine::specialMouseItemFX(int shape, int x, int y, int animIndex, int
 		0x7C, 0xD0, 0x74, 0x84, 0x87, 0x00, 0x00, 0x00
 	};
 	int tableValue = 0;
-	if (animIndex == 0) {
+	if (animIndex == 0)
 		tableValue = table1[tableIndex];
-	} else if (animIndex == 1) {
+	else if (animIndex == 1)
 		tableValue = table2[tableIndex];
-	} else if (animIndex == 2) {
+	else if (animIndex == 2)
 		tableValue = table3[tableIndex];
-	} else {
+	else
 		return;
-	}
 	processSpecialMouseItemFX(shape, x, y, tableValue, loopStart, maxLoops);
 }
 
 void KyraEngine::processSpecialMouseItemFX(int shape, int x, int y, int tableValue, int loopStart, int maxLoops) {
 	debugC(9, kDebugLevelMain, "KyraEngine::processSpecialMouseItemFX(%d, %d, %d, %d, %d, %d)", shape, x, y, tableValue, loopStart, maxLoops);
 	uint8 shapeColorTable[16];
-	uint8 *shapePtr = _shapes[4+shape] + 10;
-	if (_features & GF_TALKIE)
+	uint8 *shapePtr = _shapes[shape] + 10;
+	if (_flags.useAltShapeHeader)
 		shapePtr += 2;
-	for (int i = 0; i < 16; ++i) {
+
+	for (int i = 0; i < 16; ++i)
 		shapeColorTable[i] = shapePtr[i];
-	}
+
 	for (int i = loopStart; i < loopStart + maxLoops; ++i) {
 		for (int i2 = 0; i2 < 16; ++i2) {
-			if (shapePtr[i2] == i) {
+			if (shapePtr[i2] == i)
 				shapeColorTable[i2] = (i + tableValue) - loopStart;
-			}
 		}
 	}
-	_screen->drawShape(0, _shapes[4+shape], x, y, 0, 0x8000, shapeColorTable);
+	_screen->drawShape(0, _shapes[shape], x, y, 0, 0x8000, shapeColorTable);
 }
 
 void KyraEngine::updatePlayerItemsForScene() {
@@ -900,7 +866,7 @@ void KyraEngine::updatePlayerItemsForScene() {
 		if (_itemInHand > 33)
 			_itemInHand = 33;
 		_screen->hideMouse();
-		_screen->setMouseCursor(8, 15, _shapes[220+_itemInHand]);
+		_screen->setMouseCursor(8, 15, _shapes[216+_itemInHand]);
 		_screen->showMouse();
 	}
 	
@@ -922,16 +888,14 @@ void KyraEngine::updatePlayerItemsForScene() {
 		_screen->showMouse();
 	}
 	
-	if (_itemInHand == 33) {
+	if (_itemInHand == 33)
 		magicOutMouseItem(2, -1);
-	}
 	
 	_screen->hideMouse();
 	for (int i = 0; i < 10; ++i) {
 		uint8 item = _currentCharacter->inventoryItems[i];
-		if (item == 33) {
+		if (item == 33)
 			magicOutMouseItem(2, i);
-		}
 	}
 	_screen->showMouse();
 }
@@ -944,7 +908,7 @@ void KyraEngine::redrawInventory(int page) {
 		_screen->fillRect(_itemPosX[i], _itemPosY[i], _itemPosX[i] + 15, _itemPosY[i] + 15, 12, page);
 		if (_currentCharacter->inventoryItems[i] != 0xFF) {
 			uint8 item = _currentCharacter->inventoryItems[i];
-			_screen->drawShape(page, _shapes[220+item], _itemPosX[i], _itemPosY[i], 0, 0);
+			_screen->drawShape(page, _shapes[216+item], _itemPosX[i], _itemPosY[i], 0, 0);
 		}
 	}
 	_screen->showMouse();
@@ -952,4 +916,29 @@ void KyraEngine::redrawInventory(int page) {
 	_screen->updateScreen();
 }
 
+void KyraEngine::backUpItemRect0(int xpos, int ypos) {
+	debugC(9, kDebugLevelMain, "KyraEngine::backUpItemRect0(%d, %d)", xpos, ypos);
+	_screen->rectClip(xpos, ypos, 3<<3, 24);
+	_screen->copyRegionToBuffer(_screen->_curPage, xpos, ypos, 3<<3, 24, _itemBkgBackUp[0]);
+}
+
+void KyraEngine::restoreItemRect0(int xpos, int ypos) {
+	debugC(9, kDebugLevelMain, "KyraEngine::restoreItemRect0(%d, %d)", xpos, ypos);
+	_screen->rectClip(xpos, ypos, 3<<3, 24);
+	_screen->copyBlockToPage(_screen->_curPage, xpos, ypos, 3<<3, 24, _itemBkgBackUp[0]);
+}
+
+void KyraEngine::backUpItemRect1(int xpos, int ypos) {
+	debugC(9, kDebugLevelMain, "KyraEngine::backUpItemRect1(%d, %d)", xpos, ypos);
+	_screen->rectClip(xpos, ypos, 4<<3, 32);
+	_screen->copyRegionToBuffer(_screen->_curPage, xpos, ypos, 4<<3, 32, _itemBkgBackUp[1]);
+}
+
+void KyraEngine::restoreItemRect1(int xpos, int ypos) {
+	debugC(9, kDebugLevelMain, "KyraEngine::restoreItemRect1(%d, %d)", xpos, ypos);
+	_screen->rectClip(xpos, ypos, 4<<3, 32);
+	_screen->copyBlockToPage(_screen->_curPage, xpos, ypos, 4<<3, 32, _itemBkgBackUp[1]);
+}
+
 } // end of namespace Kyra
+
