@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2004-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/kyra/kyra.cpp $
- * $Id: kyra.cpp 23907 2006-09-17 09:57:07Z lordhoto $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/kyra/kyra.cpp $
+ * $Id: kyra.cpp 27224 2007-06-08 23:05:24Z lordhoto $
  *
  */
 
@@ -24,6 +27,7 @@
 
 #include "common/config-manager.h"
 #include "common/file.h"
+#include "common/events.h"
 #include "common/system.h"
 #include "common/savefile.h"
 
@@ -46,11 +50,9 @@
 
 namespace Kyra {
 
-KyraEngine::KyraEngine(OSystem *system)
+KyraEngine::KyraEngine(OSystem *system, const GameFlags &flags)
 	: Engine(system) {
-	setupOpcodeTable();
-	setupButtonData();
-	setupMenu();
+	_flags = flags;
 
 	_seq_Forest = _seq_KallakWriting = _seq_KyrandiaLogo = _seq_KallakMalcolm =
 	_seq_MalcolmTree = _seq_WestwoodLogo = _seq_Demo1 = _seq_Demo2 = _seq_Demo3 =
@@ -103,28 +105,12 @@ KyraEngine::KyraEngine(OSystem *system)
 	_scrollUpButton.process0PtrShape = _scrollUpButton.process1PtrShape = _scrollUpButton.process2PtrShape = 0;
 	_scrollDownButton.process0PtrShape = _scrollDownButton.process1PtrShape = _scrollDownButton.process2PtrShape = 0;
 	memset(_sceneAnimTable, 0, sizeof(_sceneAnimTable));
-	_features = 0;
 	_quitFlag = false;
 	_currHeadShape = 0;
-}
 
-KyraEngine_v1::KyraEngine_v1(OSystem *system)
-	: KyraEngine(system) {
-}
+	_curSfxFile = _curMusicTheme = 0;
 
-KyraEngine_v2::KyraEngine_v2(OSystem *system)
-	: KyraEngine(system) {
-}
-
-int KyraEngine::init() {
-	// Setup mixer
-	if (!_mixer->isReady()) {
-		warning("Sound initialization failed.");
-	}
-
-	_mixer->setVolumeForSoundType(Audio::Mixer::kSFXSoundType, ConfMan.getInt("sfx_volume"));
-	_mixer->setVolumeForSoundType(Audio::Mixer::kMusicSoundType, ConfMan.getInt("music_volume"));
-	_mixer->setVolumeForSoundType(Audio::Mixer::kSpeechSoundType, ConfMan.getInt("speech_volume"));
+	memset(&_itemBkgBackUp, 0, sizeof(_itemBkgBackUp));
 
 	// sets up all engine specific debug levels
 	Common::addSpecialDebugLevel(kDebugLevelScriptFuncs, "ScriptFuncs", "Script function debug level");
@@ -137,48 +123,64 @@ int KyraEngine::init() {
 	Common::addSpecialDebugLevel(kDebugLevelGUI, "GUI", "GUI debug level");
 	Common::addSpecialDebugLevel(kDebugLevelSequence, "Sequence", "Sequence debug level");
 	Common::addSpecialDebugLevel(kDebugLevelMovie, "Movie", "Movie debug level");
+}
+
+int KyraEngine::init() {
+	// Setup mixer
+	if (!_mixer->isReady())
+		warning("Sound initialization failed.");
+
+	_mixer->setVolumeForSoundType(Audio::Mixer::kSFXSoundType, ConfMan.getInt("sfx_volume"));
+	_mixer->setVolumeForSoundType(Audio::Mixer::kMusicSoundType, ConfMan.getInt("music_volume"));
+	_mixer->setVolumeForSoundType(Audio::Mixer::kSpeechSoundType, ConfMan.getInt("speech_volume"));
+
+	_screen = new Screen(this, _system);
+	assert(_screen);
+	if (!_screen->init())
+		error("_screen->init() failed");
 
 	// for now we prefer Adlib over native MIDI
 	int midiDriver = MidiDriver::detectMusicDriver(MDT_MIDI | MDT_ADLIB/* | MDT_PREFER_MIDI*/);
 
-	if (midiDriver == MD_ADLIB) {
-		_sound = new SoundAdlibPC(_mixer, this);
+	// TODO: We should play the native Kyra 2 Adlib music, but until that
+	//       is support, we'll use the automagic MIDI -> Adlib converter.
+
+	if (_flags.platform == Common::kPlatformFMTowns) {
+		// no sfx enabled for CD audio music atm
+		// later on here should be a usage of MixedSoundDriver
+		_sound = new SoundTowns(this, _mixer);
+	} else if (midiDriver == MD_ADLIB) {
+		_sound = new SoundAdlibPC(this, _mixer);
 		assert(_sound);
 	} else {
 		bool native_mt32 = ((midiDriver == MD_MT32) || ConfMan.getBool("native_mt32"));
 
 		MidiDriver *driver = MidiDriver::createMidi(midiDriver);
 		assert(driver);
-		if (native_mt32) {
+		if (native_mt32)
 			driver->property(MidiDriver::PROP_CHANNEL_MASK, 0x03FE);
-		}
 
-		SoundMidiPC *soundMidiPc = new SoundMidiPC(driver, _mixer, this);
+		SoundMidiPC *soundMidiPc = new SoundMidiPC(this, _mixer, driver);
 		_sound = soundMidiPc;
 		assert(_sound);
 		soundMidiPc->hasNativeMT32(native_mt32);
+
+		// C55 appears to be XMIDI for General MIDI instruments
+		soundMidiPc->setUseC55(_flags.gameID == GI_KYRA2 && !native_mt32);
 		
 		// Unlike some SCUMM games, it's not that the MIDI sounds are
 		// missing. It's just that at least at the time of writing they
-		// decidedly inferior to the Adlib ones.
+		// are decidedly inferior to the Adlib ones.
 
-		if (ConfMan.getBool("multi_midi")) {
-			SoundAdlibPC *adlib = new SoundAdlibPC(_mixer, this);
+		if (midiDriver != MD_ADLIB && ConfMan.getBool("multi_midi")) {
+			SoundAdlibPC *adlib = new SoundAdlibPC(this, _mixer);
 			assert(adlib);
 			
 			_sound = new MixedSoundDriver(this, _mixer, soundMidiPc, adlib);
 			assert(_sound);
 		}
 	}
-	if (!_sound->init()) {
-		error("Couldn't init sound");
-	}
-	_sound->setVolume(255);
-	
-	_screen = new Screen(this, _system);
-	assert(_screen);
-	if (!_screen->init())
-		error("_screen->init() failed");
+
 	_res = new Resource(this);
 	assert(_res);
 	_sprites = new Sprites(this, _system);
@@ -189,7 +191,7 @@ int KyraEngine::init() {
 	assert(_animator);
 	_animator->init(5, 11, 12);
 	assert(*_animator);
-	_text = new TextDisplayer(_screen);
+	_text = new TextDisplayer(this, _screen);
 	assert(_text);
 
 	_staticres = new StaticResource(this);
@@ -199,14 +201,30 @@ int KyraEngine::init() {
 	
 	initStaticResource();
 	
+	if (!_sound->init())
+		error("Couldn't init sound");
+
+	if (_flags.platform == Common::kPlatformFMTowns)
+		_sound->setSoundFileList(_soundFilesTowns, _soundFilesTownsCount);
+	else
+		_sound->setSoundFileList(_soundFiles, _soundFilesCount);
+
+	_sound->setVolume(255);
+	_sound->loadSoundFile(0);
+
+	setupOpcodeTable();
+	setupButtonData();
+	setupMenu();
+
 	_paletteChanged = 1;
 	_currentCharacter = 0;
 	_characterList = new Character[11];
 	assert(_characterList);
-	for (int i = 0; i < 11; ++i) {
-		memset(&_characterList[i], 0, sizeof(Character));
+	memset(_characterList, 0, sizeof(Character)*11);
+
+	for (int i = 0; i < 11; ++i)
 		memset(_characterList[i].inventoryItems, 0xFF, sizeof(_characterList[i].inventoryItems));
-	}
+
 	_characterList[0].sceneId = 5;
 	_characterList[0].height = 48;
 	_characterList[0].facing = 3;
@@ -237,9 +255,8 @@ int KyraEngine::init() {
 	assert(_debugger);	
 	memset(_shapes, 0, sizeof(_shapes));
 
-	for (int i = 0; i < ARRAYSIZE(_movieObjects); ++i) {
+	for (int i = 0; i < ARRAYSIZE(_movieObjects); ++i)
 		_movieObjects[i] = createWSAMovie();
-	}
 
 	memset(_flagsTable, 0, sizeof(_flagsTable));
 
@@ -247,7 +264,6 @@ int KyraEngine::init() {
 	_abortWalkFlag2 = false;
 	_talkingCharNum = -1;
 	_charSayUnk3 = -1;
-	_mouseX = _mouseY = -1;
 	memset(_currSentenceColor, 0, 3);
 	_startSentencePalIndex = -1;
 	_fadeText = false;
@@ -317,6 +333,32 @@ int KyraEngine::init() {
 	_gameSpeed = 60;
 	_tickLength = (uint8)(1000.0 / _gameSpeed);
 
+	_lang = 0;
+	Common::Language lang = Common::parseLanguage(ConfMan.get("language"));
+
+	if (_flags.gameID == GI_KYRA2 || _flags.gameID == GI_KYRA3) {
+		switch (lang) {
+		case Common::EN_ANY:
+		case Common::EN_USA:
+		case Common::EN_GRB:
+			_lang = 0;
+			break;
+
+		case Common::FR_FRA:
+			_lang = 1;
+			break;
+
+		case Common::DE_DEU:
+			_lang = 2;
+			break;
+
+		default:
+			warning("unsupported language, switching back to English");
+			_lang = 0;
+			break;
+		}
+	}
+
 	return 0;
 }
 
@@ -356,16 +398,19 @@ KyraEngine::~KyraEngine() {
 	
 	delete [] _movFacingTable;
 
-	free(_scrollUpButton.process0PtrShape);
-	free(_scrollUpButton.process1PtrShape);
-	free(_scrollUpButton.process2PtrShape);
-	free(_scrollDownButton.process0PtrShape);
-	free(_scrollDownButton.process1PtrShape);
-	free(_scrollDownButton.process2PtrShape);
-		
+	delete [] _scrollUpButton.process0PtrShape;
+	delete [] _scrollUpButton.process1PtrShape;
+	delete [] _scrollUpButton.process2PtrShape;
+	delete [] _scrollDownButton.process0PtrShape;
+	delete [] _scrollDownButton.process1PtrShape;
+	delete [] _scrollDownButton.process2PtrShape;
+
+	delete [] _itemBkgBackUp[0];
+	delete [] _itemBkgBackUp[1];	
+
 	for (int i = 0; i < ARRAYSIZE(_shapes); ++i) {
 		if (_shapes[i] != 0) {
-			free(_shapes[i]);
+			delete [] _shapes[i];
 			for (int i2 = 0; i2 < ARRAYSIZE(_shapes); i2++) {
 				if (_shapes[i2] == _shapes[i] && i2 != i) {
 					_shapes[i2] = 0;
@@ -374,37 +419,20 @@ KyraEngine::~KyraEngine() {
 			_shapes[i] = 0;
 		}
 	}
-	for (int i = 0; i < ARRAYSIZE(_sceneAnimTable); ++i) {
-		free(_sceneAnimTable[i]);
-	}
-}
 
-KyraEngine_v1::~KyraEngine_v1() {
-	// FIXME: I am leaking badly!
-
-}
-
-KyraEngine_v2::~KyraEngine_v2() {
-	// FIXME: I am leaking badly!
-}
-
-void KyraEngine::errorString(const char *buf1, char *buf2) {
-	strcpy(buf2, buf1);
+	for (int i = 0; i < ARRAYSIZE(_sceneAnimTable); ++i)
+		delete [] _sceneAnimTable[i];
 }
 
 int KyraEngine::go() {
-	_quitFlag = false;
-	uint32 sz;
-
-	if (_features & GF_FLOPPY && !(_features & GF_AMIGA)) {
-		_screen->loadFont(Screen::FID_6_FNT, _res->fileData("6.FNT", &sz));
-	}
-	_screen->loadFont(Screen::FID_8_FNT, _res->fileData("8FAT.FNT", &sz));
+	if (_res->getFileSize("6.FNT"))
+		_screen->loadFont(Screen::FID_6_FNT, "6.FNT");
+	_screen->loadFont(Screen::FID_8_FNT, "8FAT.FNT");
 	_screen->setScreenDim(0);
 
 	_abortIntroFlag = false;
 
-	if (_features & GF_DEMO) {
+	if (_flags.isDemo) {
 		seq_demo();
 	} else {
 		setGameFlag(0xF3);
@@ -422,15 +450,6 @@ int KyraEngine::go() {
 	return 0;
 }
 
-int KyraEngine_v2::go() {
-	// Kyra2 goes here :)
-	loadPalette("palette.col", _screen->_currentPalette);
-	_screen->setScreenPalette(_screen->_currentPalette);
-	_screen->loadBitmap("_playfld.cps", 0, 0, 0);
-	_screen->updateScreen();
-	waitForEvent();
-	return 0;
-}
 
 void KyraEngine::startup() {
 	debugC(9, kDebugLevelMain, "KyraEngine::startup()");
@@ -449,18 +468,16 @@ void KyraEngine::startup() {
 	resetBrandonPoisonFlags();
 	_screen->_curPage = 0;
 	// XXX
-	for (int i = 0; i < 0x0C; ++i) {
+	for (int i = 0; i < 12; ++i) {
 		int size = _screen->getRectSize(3, 24);
-		_shapes[365+i] = (byte*)malloc(size);
+		_shapes[361+i] = new byte[size];
 	}
-	_shapes[0] = (uint8*)malloc(_screen->getRectSize(3, 24));
-	memset(_shapes[0], 0, _screen->getRectSize(3, 24));
-	_shapes[1] = (uint8*)malloc(_screen->getRectSize(4, 32));
-	memset(_shapes[1], 0, _screen->getRectSize(4, 32));
-	_shapes[2] = (uint8*)malloc(_screen->getRectSize(8, 69));
-	memset(_shapes[2], 0, _screen->getRectSize(8, 69));
-	_shapes[3] = (uint8*)malloc(_screen->getRectSize(8, 69));
-	memset(_shapes[3], 0, _screen->getRectSize(8, 69));
+
+	_itemBkgBackUp[0] = new uint8[_screen->getRectSize(3, 24)];
+	memset(_itemBkgBackUp[0], 0, _screen->getRectSize(3, 24));
+	_itemBkgBackUp[1] = new uint8[_screen->getRectSize(4, 32)];
+	memset(_itemBkgBackUp[1], 0, _screen->getRectSize(4, 32));
+
 	for (int i = 0; i < _roomTableSize; ++i) {
 		for (int item = 0; item < 12; ++item) {
 			_roomTable[i].itemsTable[item] = 0xFF;
@@ -469,6 +486,7 @@ void KyraEngine::startup() {
 			_roomTable[i].needInit[item] = 0;
 		}
 	}
+
 	loadCharacterShapes();
 	loadSpecialEffectShapes();
 	loadItems();
@@ -476,34 +494,33 @@ void KyraEngine::startup() {
 	initMainButtonList();
 	loadMainScreen();
 	setupTimers();
-	loadPalette("PALETTE.COL", _screen->_currentPalette);
+	_screen->loadPalette("PALETTE.COL", _screen->_currentPalette);
 
 	// XXX
 	_animator->initAnimStateList();
 	setCharactersInDefaultScene();
 
-	if (!_scriptInterpreter->loadScript("_STARTUP.EMC", _npcScriptData, _opcodeTable, _opcodeTableSize, 0)) {
+	if (!_scriptInterpreter->loadScript("_STARTUP.EMC", _npcScriptData, &_opcodes))
 		error("Could not load \"_STARTUP.EMC\" script");
-	}
 	_scriptInterpreter->initScript(_scriptMain, _npcScriptData);
-	if (!_scriptInterpreter->startScript(_scriptMain, 0)) {
+
+	if (!_scriptInterpreter->startScript(_scriptMain, 0))
 		error("Could not start script function 0 of script \"_STARTUP.EMC\"");
-	}
-	while (_scriptInterpreter->validScript(_scriptMain)) {
+
+	while (_scriptInterpreter->validScript(_scriptMain))
 		_scriptInterpreter->runScript(_scriptMain);
-	}
 	
 	_scriptInterpreter->unloadScript(_npcScriptData);
-	if (!_scriptInterpreter->loadScript("_NPC.EMC", _npcScriptData, _opcodeTable, _opcodeTableSize, 0)) {
+
+	if (!_scriptInterpreter->loadScript("_NPC.EMC", _npcScriptData, &_opcodes))
 		error("Could not load \"_NPC.EMC\" script");
-	}
 	
 	snd_playTheme(1);
 	enterNewScene(_currentCharacter->sceneId, _currentCharacter->facing, 0, 0, 1);
 	
 	if (_abortIntroFlag && _skipIntroFlag) {
 		_menuDirectlyToLoad = true;
-		_screen->setMouseCursor(1, 1, _shapes[4]);
+		_screen->setMouseCursor(1, 1, _shapes[0]);
 		_screen->showMouse();
 		buttonMenuCallback(0);
 		_menuDirectlyToLoad = false;
@@ -520,32 +537,27 @@ void KyraEngine::mainLoop() {
 
 		if (_currentCharacter->sceneId == 210) {
 			updateKyragemFading();
-			if (seq_playEnd()) {
-				if (_deathHandler != 8)
-					break;
-			}
+			if (seq_playEnd() && _deathHandler != 8)
+				break;
 		}
 		
 		if (_deathHandler != 0xFF) {
 			snd_playWanderScoreViaMap(0, 1);
 			snd_playSoundEffect(49);
 			_screen->hideMouse();
-			_screen->setMouseCursor(1, 1, _shapes[4]);
+			_screen->setMouseCursor(1, 1, _shapes[0]);
 			destroyMouseItem();
 			_screen->showMouse();
 			buttonMenuCallback(0);
 			_deathHandler = 0xFF;
 		}
 		
-		if (_brandonStatusBit & 2) {
-			if (_brandonStatusBit0x02Flag)
-				_animator->animRefreshNPC(0);
-		}
-		if (_brandonStatusBit & 0x20) {
-			if (_brandonStatusBit0x20Flag) {
-				_animator->animRefreshNPC(0);
-				_brandonStatusBit0x20Flag = 0;
-			}
+		if ((_brandonStatusBit & 2) && _brandonStatusBit0x02Flag)
+			_animator->animRefreshNPC(0);
+
+		if ((_brandonStatusBit & 0x20) && _brandonStatusBit0x20Flag) {
+			_animator->animRefreshNPC(0);
+			_brandonStatusBit0x20Flag = 0;
 		}
 		
 		_screen->showMouse();
@@ -573,31 +585,32 @@ void KyraEngine::delayUntil(uint32 timestamp, bool updateTimers, bool update, bo
 	while (_system->getMillis() < timestamp && !_quitFlag) {
 		if (updateTimers)
 			updateGameTimers();
+
 		if (timestamp - _system->getMillis() >= 10)
 			delay(10, update, isMainLoop);
 	}
 }
 
 void KyraEngine::delay(uint32 amount, bool update, bool isMainLoop) {
-	OSystem::Event event;
+	Common::Event event;
 	char saveLoadSlot[20];
 	char savegameName[14];
 
 	uint32 start = _system->getMillis();
 	do {
-		while (_system->pollEvent(event)) {
+		while (_eventMan->pollEvent(event)) {
 			switch (event.type) {
-			case OSystem::EVENT_KEYDOWN:
+			case Common::EVENT_KEYDOWN:
 				if (event.kbd.keycode >= '1' && event.kbd.keycode <= '9' && 
-						(event.kbd.flags == OSystem::KBD_CTRL || event.kbd.flags == OSystem::KBD_ALT) && isMainLoop) {
+						(event.kbd.flags == Common::KBD_CTRL || event.kbd.flags == Common::KBD_ALT) && isMainLoop) {
 					sprintf(saveLoadSlot, "%s.00%d", _targetName.c_str(), event.kbd.keycode - '0');
-					if (event.kbd.flags == OSystem::KBD_CTRL)
+					if (event.kbd.flags == Common::KBD_CTRL)
 						loadGame(saveLoadSlot);
 					else {
 						sprintf(savegameName, "Quicksave %d",  event.kbd.keycode - '0');
 						saveGame(saveLoadSlot, savegameName);
 					}
-				} else if (event.kbd.flags == OSystem::KBD_CTRL) {
+				} else if (event.kbd.flags == Common::KBD_CTRL) {
 					if (event.kbd.keycode == 'd')
 						_debugger->attach();
 					else if (event.kbd.keycode == 'q')
@@ -610,37 +623,34 @@ void KyraEngine::delay(uint32 amount, bool update, bool isMainLoop) {
 				}
 
 				break;
-			case OSystem::EVENT_MOUSEMOVE:
-				_mouseX = event.mouse.x;
-				_mouseY = event.mouse.y;
-				_system->updateScreen();
+			case Common::EVENT_MOUSEMOVE:
+				_animator->_updateScreen = true;
 				break;
-			case OSystem::EVENT_QUIT:
+			case Common::EVENT_QUIT:
 				quitGame();
 				break;
-			case OSystem::EVENT_LBUTTONDOWN:
+			case Common::EVENT_LBUTTONDOWN:
 				_mousePressFlag = true;
 				break;
-			case OSystem::EVENT_LBUTTONUP:
+			case Common::EVENT_LBUTTONUP:
 				_mousePressFlag = false;
-				if (_abortWalkFlag2) {
+
+				if (_abortWalkFlag2) 
 					_abortWalkFlag = true;
-					_mouseX = event.mouse.x;
-					_mouseY = event.mouse.y;
-				}
+
 				if (_handleInput) {
-					_mouseX = event.mouse.x;
-					_mouseY = event.mouse.y;
 					_handleInput = false;
-					processInput(_mouseX, _mouseY);
+					processInput();
 					_handleInput = true;
 				} else
 					_skipFlag = true;
+
 				break;
 			default:
 				break;
 			}
 		}
+
 		if (_debugger->isAttached())
 			_debugger->onFrame();
 
@@ -651,41 +661,43 @@ void KyraEngine::delay(uint32 amount, bool update, bool isMainLoop) {
 			updateMousePointer();
 		}
 
-		if (_currentCharacter && _currentCharacter->sceneId == 210 && update) {
+		if (_currentCharacter && _currentCharacter->sceneId == 210 && update)
 			updateKyragemFading();
-		}
 
 		if (_skipFlag && !_abortIntroFlag && !queryGameFlag(0xFE))
 			_skipFlag = false;
 			
-		if (amount > 0 && !_skipFlag && !_quitFlag) {
-			_system->delayMillis((amount > 10) ? 10 : amount);
-		}
+		if (amount > 0 && !_skipFlag && !_quitFlag)
+			_system->delayMillis(10);
 
-		if (_skipFlag) {
+		if (_skipFlag)
 			_sound->voiceStop();
-		}
 	} while (!_skipFlag && _system->getMillis() < start + amount && !_quitFlag);
-	
+}
+
+Common::Point KyraEngine::getMousePos() const {
+	Common::Point mouse = _eventMan->getMousePos();
+	if (_flags.useHiResOverlay) {
+		mouse.x >>= 1;
+		mouse.y >>= 1;
+	}
+	return mouse;
 }
 
 void KyraEngine::waitForEvent() {
 	bool finished = false;
-	OSystem::Event event;
-	while (!finished) {
-		while (_system->pollEvent(event)) {
+	Common::Event event;
+
+	while (!finished && !_quitFlag) {
+		while (_eventMan->pollEvent(event)) {
 			switch (event.type) {
-			case OSystem::EVENT_KEYDOWN:
+			case Common::EVENT_KEYDOWN:
 				finished = true;
 				break;
-			case OSystem::EVENT_MOUSEMOVE:
-				_mouseX = event.mouse.x;
-				_mouseY = event.mouse.y;
-				break;
-			case OSystem::EVENT_QUIT:
+			case Common::EVENT_QUIT:
 				quitGame();
 				break;
-			case OSystem::EVENT_LBUTTONDOWN:
+			case Common::EVENT_LBUTTONDOWN:
 				finished = true;
 				_skipFlag = true;
 				break;
@@ -703,15 +715,19 @@ void KyraEngine::waitForEvent() {
 
 void KyraEngine::delayWithTicks(int ticks) {
 	uint32 nextTime = _system->getMillis() + ticks * _tickLength;
+
 	while (_system->getMillis() < nextTime) {
 		_sprites->updateSceneAnims();
 		_animator->updateAllObjectShapes();
+
 		if (_currentCharacter->sceneId == 210) {
 			updateKyragemFading();
 			seq_playEnd();
 		}
+
 		if (_skipFlag)
 			break;
+
 		if (nextTime - _system->getMillis() >= 10)
 			delay(10);
 	}
@@ -723,9 +739,10 @@ void KyraEngine::delayWithTicks(int ticks) {
 
 void KyraEngine::setupShapes123(const Shape *shapeTable, int endShape, int flags) {
 	debugC(9, kDebugLevelMain, "KyraEngine::setupShapes123(%p, %d, %d)", (const void *)shapeTable, endShape, flags);
-	for (int i = 123; i <= 172; ++i) {
-		_shapes[4+i] = NULL;
-	}
+
+	for (int i = 123; i <= 172; ++i)
+		_shapes[i] = 0;
+
 	uint8 curImage = 0xFF;
 	int curPageBackUp = _screen->_curPage;
 	_screen->_curPage = 8;	// we are using page 8 here in the original page 2 was backuped and then used for this stuff
@@ -739,7 +756,7 @@ void KyraEngine::setupShapes123(const Shape *shapeTable, int endShape, int flags
 			_screen->loadBitmap(_characterImageTable[newImage], 8, 8, 0);
 			curImage = newImage;
 		}
-		_shapes[4+i] = _screen->encodeShape(shapeTable[i-123].x<<3, shapeTable[i-123].y, shapeTable[i-123].w<<3, shapeTable[i-123].h, shapeFlags);
+		_shapes[i] = _screen->encodeShape(shapeTable[i-123].x<<3, shapeTable[i-123].y, shapeTable[i-123].w<<3, shapeTable[i-123].h, shapeFlags);
 		assert(i-7 < _defaultShapeTableSize);
 		_defaultShapeTable[i-7].xOffset = shapeTable[i-123].xOffset;
 		_defaultShapeTable[i-7].yOffset = shapeTable[i-123].yOffset;
@@ -751,9 +768,10 @@ void KyraEngine::setupShapes123(const Shape *shapeTable, int endShape, int flags
 
 void KyraEngine::freeShapes123() {
 	debugC(9, kDebugLevelMain, "KyraEngine::freeShapes123()");
+
 	for (int i = 123; i <= 172; ++i) {
-		free(_shapes[4+i]);
-		_shapes[4+i] = NULL;
+		delete [] _shapes[i];
+		_shapes[i] = 0;
 	}
 }
 
@@ -762,7 +780,9 @@ void KyraEngine::freeShapes123() {
 #pragma mark -
 
 Movie *KyraEngine::createWSAMovie() {
-	// for kyra2 here could be added then WSAMovieV2
+	if (_flags.platform == Common::kPlatformAmiga)
+		return new WSAMovieAmiga(this);
+
 	return new WSAMovieV1(this);
 }
 
@@ -783,11 +803,13 @@ int KyraEngine::resetGameFlag(int flag) {
 void KyraEngine::setBrandonPoisonFlags(int reset) {
 	debugC(9, kDebugLevelMain, "KyraEngine::setBrandonPoisonFlags(%d)", reset);
 	_brandonStatusBit |= 1;
+
 	if (reset)
 		_poisonDeathCounter = 0;
-	for (int i = 0; i < 0x100; ++i) {
+
+	for (int i = 0; i < 0x100; ++i)
 		_brandonPoisonFlagsGFX[i] = i;
-	}
+
 	_brandonPoisonFlagsGFX[0x99] = 0x34;
 	_brandonPoisonFlagsGFX[0x9A] = 0x35;
 	_brandonPoisonFlagsGFX[0x9B] = 0x37;
@@ -798,22 +820,26 @@ void KyraEngine::setBrandonPoisonFlags(int reset) {
 void KyraEngine::resetBrandonPoisonFlags() {
 	debugC(9, kDebugLevelMain, "KyraEngine::resetBrandonPoisonFlags()");
 	_brandonStatusBit = 0;
-	for (int i = 0; i < 0x100; ++i) {
+
+	for (int i = 0; i < 0x100; ++i)
 		_brandonPoisonFlagsGFX[i] = i;
-	}
 }
 
 #pragma mark -
 #pragma mark - Input
 #pragma mark -
 
-void KyraEngine::processInput(int xpos, int ypos) {
+void KyraEngine::processInput() {
+	Common::Point mouse = getMousePos();
+	int xpos = mouse.x;
+	int ypos = mouse.y;
+
 	debugC(9, kDebugLevelMain, "KyraEngine::processInput(%d, %d)", xpos, ypos);
 	_abortWalkFlag2 = false;
 
-	if (processInputHelper(xpos, ypos)) {
+	if (processInputHelper(xpos, ypos))
 		return;
-	}
+
 	uint8 item = findItemAtPos(xpos, ypos);
 	if (item == 0xFF) {
 		_changedScene = false;
@@ -903,9 +929,8 @@ int KyraEngine::clickEventHandler(int xpos, int ypos) {
 	_scriptClick->variables[4] = _itemInHand;
 	_scriptInterpreter->startScript(_scriptClick, 1);
 
-	while (_scriptInterpreter->validScript(_scriptClick)) {
+	while (_scriptInterpreter->validScript(_scriptClick))
 		_scriptInterpreter->runScript(_scriptClick);
-	}
 
 	return _scriptClick->variables[3];
 }
@@ -916,9 +941,10 @@ void KyraEngine::updateMousePointer(bool forceUpdate) {
 	int newMouseState = 0;
 	int newX = 0;
 	int newY = 0;
-	if (_mouseY <= 158) {
-		if (_mouseX >= 12) {
-			if (_mouseX >= 308) {
+	Common::Point mouse = getMousePos();
+	if (mouse.y <= 158) {
+		if (mouse.x >= 12) {
+			if (mouse.x >= 308) {
 				if (_walkBlockEast == 0xFFFF) {
 					newMouseState = -2;
 				} else {
@@ -927,7 +953,7 @@ void KyraEngine::updateMousePointer(bool forceUpdate) {
 					newX = 7;
 					newY = 5;
 				}
-			} else if (_mouseY >= 136) {
+			} else if (mouse.y >= 136) {
 				if (_walkBlockSouth == 0xFFFF) {
 					newMouseState = -2;
 				} else {
@@ -936,7 +962,7 @@ void KyraEngine::updateMousePointer(bool forceUpdate) {
 					newX = 5;
 					newY = 7;
 				}
-			} else if (_mouseY < 12) {
+			} else if (mouse.y < 12) {
 				if (_walkBlockNorth == 0xFFFF) {
 					newMouseState = -2;
 				} else {
@@ -957,8 +983,8 @@ void KyraEngine::updateMousePointer(bool forceUpdate) {
 		}
 	}
 	
-	if (_mouseX >= _entranceMouseCursorTracks[0] && _mouseY >= _entranceMouseCursorTracks[1]
-		&& _mouseX <= _entranceMouseCursorTracks[2] && _mouseY <= _entranceMouseCursorTracks[3]) {
+	if (mouse.x >= _entranceMouseCursorTracks[0] && mouse.y >= _entranceMouseCursorTracks[1]
+		&& mouse.x <= _entranceMouseCursorTracks[2] && mouse.y <= _entranceMouseCursorTracks[3]) {
 		switch (_entranceMouseCursorTracks[4]) {
 		case 0:
 			newMouseState = -6;
@@ -1002,19 +1028,19 @@ void KyraEngine::updateMousePointer(bool forceUpdate) {
 	if ((newMouseState && _mouseState != newMouseState) || (newMouseState && forceUpdate)) {
 		_mouseState = newMouseState;
 		_screen->hideMouse();
-		_screen->setMouseCursor(newX, newY, _shapes[4+shape]);
+		_screen->setMouseCursor(newX, newY, _shapes[shape]);
 		_screen->showMouse();
 	}
 	
 	if (!newMouseState) {
 		if (_mouseState != _itemInHand || forceUpdate) {
-			if (_mouseY > 158 || (_mouseX >= 12 && _mouseX < 308 && _mouseY < 136 && _mouseY >= 12) || forceUpdate) {
+			if (mouse.y > 158 || (mouse.x >= 12 && mouse.x < 308 && mouse.y < 136 && mouse.y >= 12) || forceUpdate) {
 				_mouseState = _itemInHand;
 				_screen->hideMouse();
 				if (_itemInHand == -1) {
-					_screen->setMouseCursor(1, 1, _shapes[4]);
+					_screen->setMouseCursor(1, 1, _shapes[0]);
 				} else {
-					_screen->setMouseCursor(8, 15, _shapes[220+_itemInHand]);
+					_screen->setMouseCursor(8, 15, _shapes[216+_itemInHand]);
 				}
 				_screen->showMouse();
 			}
@@ -1024,29 +1050,32 @@ void KyraEngine::updateMousePointer(bool forceUpdate) {
 
 bool KyraEngine::hasClickedOnExit(int xpos, int ypos) {
 	debugC(9, kDebugLevelMain, "KyraEngine::hasClickedOnExit(%d, %d)", xpos, ypos);
-	if (xpos < 16 || xpos >= 304) {
+	if (xpos < 16 || xpos >= 304)
 		return true;
-	}
+
 	if (ypos < 8)
 		return true;
-	if (ypos < 136 || ypos > 155) {
+
+	if (ypos < 136 || ypos > 155)
 		return false;
-	}
+
 	return true;
 }
 
 void KyraEngine::clickEventHandler2() {
 	debugC(9, kDebugLevelMain, "KyraEngine::clickEventHandler2()");
+
+	Common::Point mouse = getMousePos();
+
 	_scriptInterpreter->initScript(_scriptClick, _scriptClickData);
 	_scriptClick->variables[0] = _currentCharacter->sceneId;
-	_scriptClick->variables[1] = _mouseX;
-	_scriptClick->variables[2] = _mouseY;
+	_scriptClick->variables[1] = mouse.x;
+	_scriptClick->variables[2] = mouse.y;
 	_scriptClick->variables[4] = _itemInHand;
 	_scriptInterpreter->startScript(_scriptClick, 6);
 	
-	while (_scriptInterpreter->validScript(_scriptClick)) {
+	while (_scriptInterpreter->validScript(_scriptClick))
 		_scriptInterpreter->runScript(_scriptClick);
-	}
 }
 
 int KyraEngine::checkForNPCScriptRun(int xpos, int ypos) {
@@ -1064,13 +1093,11 @@ int KyraEngine::checkForNPCScriptRun(int xpos, int ypos) {
 	charTop = currentChar->y1 - addY;
 	charBottom = currentChar->y1;
 	
-	if (xpos >= charLeft && charRight >= xpos && charTop <= ypos && charBottom >= ypos) {
+	if (xpos >= charLeft && charRight >= xpos && charTop <= ypos && charBottom >= ypos)
 		return 0;
-	}
 	
-	if (xpos > 304 || xpos < 16) {
+	if (xpos > 304 || xpos < 16)
 		return -1;
-	}
 	
 	for (int i = 1; i < 5; ++i) {
 		currentChar = &_characterList[i];
@@ -1087,14 +1114,12 @@ int KyraEngine::checkForNPCScriptRun(int xpos, int ypos) {
 		charBottom = currentChar->y1;
 		// }
 		
-		if (xpos < charLeft || xpos > charRight || ypos < charTop || charBottom < ypos) {
+		if (xpos < charLeft || xpos > charRight || ypos < charTop || charBottom < ypos)
 			continue;
-		}
 		
 		if (returnValue != -1) {
-			if (currentChar->y1 >= _characterList[returnValue].y1) {
+			if (currentChar->y1 >= _characterList[returnValue].y1)
 				returnValue = i;
-			}
 		} else {
 			returnValue = i;
 		}
@@ -1111,9 +1136,9 @@ void KyraEngine::runNpcScript(int func) {
 	_npcScript->variables[4] = _itemInHand;
 	_npcScript->variables[5] = func;
 	
-	while (_scriptInterpreter->validScript(_npcScript)) {
+	while (_scriptInterpreter->validScript(_npcScript))
 		_scriptInterpreter->runScript(_npcScript);
-	}
 }
 
 } // End of namespace Kyra
+
