@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2004-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,16 +18,17 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/kyra/kyra.h $
- * $Id: kyra.h 23950 2006-09-20 20:19:07Z lordhoto $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/kyra/kyra.h $
+ * $Id: kyra.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
-#ifndef KYRA_H
-#define KYRA_H
+#ifndef KYRA_KYRA_H
+#define KYRA_KYRA_H
 
-#include "base/engine.h"
+#include "engines/engine.h"
 #include "common/rect.h"
+#include "common/array.h"
 
 namespace Kyra {
 
@@ -43,22 +47,18 @@ class TextDisplayer;
 class KyraEngine;
 class StaticResource;
 
+struct Opcode;
 struct ScriptState;
 struct ScriptData;
 
-enum {
-	GF_FLOPPY	= 1 <<  0,
-	GF_TALKIE	= 1 <<  1,
-	GF_AUDIOCD	= 1 <<  2,  // FM-Towns versions seems to use audio CD
-	GF_DEMO		= 1 <<  3,
-	GF_ENGLISH	= 1 <<  4,
-	GF_FRENCH	= 1 <<  5,
-	GF_GERMAN	= 1 <<  6,
-	GF_SPANISH	= 1 <<  7,
-	GF_ITALIAN	= 1 <<  8,
-	// other languages here
-	GF_LNGUNK	= 1 << 16,	// also used for multi language in kyra3
-	GF_AMIGA	= 1 << 17	// this is no special version flag yet!
+struct GameFlags {
+	Common::Language lang;
+	Common::Platform platform;
+	bool isDemo;
+	bool useAltShapeHeader;	// alternative shape header (uses 2 bytes more, those are unused though)
+	bool isTalkie;
+	bool useHiResOverlay;
+	byte gameID;
 };
 
 enum {
@@ -70,7 +70,7 @@ enum {
 // TODO: this is just the start of makeing the debug output of the kyra engine a bit more useable
 // in the future we maybe merge some flags  and/or create new ones
 enum kDebugLevels {
-	kDebugLevelScriptFuncs = 1 << 0,		// prints debug output of cmd_* functions
+	kDebugLevelScriptFuncs = 1 << 0,		// prints debug output of o1_* functions
 	kDebugLevelScript = 1 << 1,				// prints debug output of "ScriptHelper" functions
 	kDebugLevelSprites = 1 << 2,			// prints debug output of "Sprites" functions
 	kDebugLevelScreen = 1 << 3,				// prints debug output of "Screen" functions
@@ -232,29 +232,14 @@ struct Menu {
 	MenuItem item[6];
 };
 
-struct KeyboardEvent {
-	bool pending;
-	uint32 repeat;
-	uint8 ascii;
-};
-
 class KyraEngine : public Engine {
 	friend class MusicPlayer;
 	friend class Debugger;
 	friend class ScreenAnimator;
 public:
-
-	enum {
-		MUSIC_INTRO = 0
-	};
-
-	KyraEngine(OSystem *system);
+	KyraEngine(OSystem *system, const GameFlags &flags);
 	virtual ~KyraEngine();
 
-	virtual int setupGameFlags() = 0;
-	
-	void errorString(const char *buf_input, char *buf_output);
-
 	Resource *resource() { return _res; }
 	Screen *screen() { return _screen; }
 	ScreenAnimator *animator() { return _animator; }
@@ -264,8 +249,8 @@ public:
 	uint32 tickLength() const { return _tickLength; }
 	virtual Movie *createWSAMovie();
 
-	uint8 game() const { return _game; }
-	uint32 features() const { return _features; }
+	uint8 game() const { return _flags.gameID; }
+	const GameFlags &gameFlags() const { return _flags; }
 
 	uint8 **shapes() { return _shapes; }
 	Character *currentCharacter() { return _currentCharacter; }
@@ -282,8 +267,8 @@ public:
 	int16 _northExitHeight;
 
 	typedef void (KyraEngine::*IntroProc)();
-	typedef int (KyraEngine::*OpcodeProc)(ScriptState *script);
 
+	// static data access
 	const char * const*seqWSATable() { return _seq_WSATable; }
 	const char * const*seqCPSTable() { return _seq_CPSTable; }
 	const char * const*seqCOLTable() { return _seq_COLTable; }
@@ -292,301 +277,23 @@ public:
 	const uint8 * const*palTable1() { return &_specialPalettes[0]; }
 	const uint8 * const*palTable2() { return &_specialPalettes[29]; }
 
+	// sequences
+	// -> misc
 	bool seq_skipSequence() const;
-	void delayUntil(uint32 timestamp, bool updateGameTimers = false, bool update = false, bool isMainLoop = false);
-	void delay(uint32 millis, bool update = false, bool isMainLoop = false);
-	void quitGame();
-
-	void registerDefaultSettings();
-	void readSettings();
-	void writeSettings();
-
-	void snd_playTheme(int file, int track = 0);
-	void snd_playVoiceFile(int id);
-	void snd_voiceWaitForFinish(bool ingame = true);
-	void snd_playSoundEffect(int track);
-	void snd_playWanderScoreViaMap(int command, int restart);
-
-	bool speechEnabled();
-	bool textEnabled();
-	
-	void drawSentenceCommand(const char *sentence, int unk1);
-	void updateSentenceCommand(const char *str1, const char *str2, int unk1);
-	void updateTextFade();
-
-	void updateGameTimers();
-	void clearNextEventTickCount();
-	void setTimerCountdown(uint8 timer, int32 countdown);
-	void setTimerDelay(uint8 timer, int32 countdown);
-	int16 getTimerDelay(uint8 timer);
-	void enableTimer(uint8 timer);
-	void disableTimer(uint8 timer);
-
-	void delayWithTicks(int ticks);
-	
-	void saveGame(const char *fileName, const char *saveName);
-	void loadGame(const char *fileName);
-
-	int mouseX() { return _mouseX; }
-	int mouseY() { return _mouseY; }
-	
-	// all opcode procs (maybe that is somehow useless atm)
-	int cmd_magicInMouseItem(ScriptState *script);
-	int cmd_characterSays(ScriptState *script);
-	int cmd_pauseTicks(ScriptState *script);
-	int cmd_drawSceneAnimShape(ScriptState *script);
-	int cmd_queryGameFlag(ScriptState *script);
-	int cmd_setGameFlag(ScriptState *script);
-	int cmd_resetGameFlag(ScriptState *script);
-	int cmd_runNPCScript(ScriptState *script);
-	int cmd_setSpecialExitList(ScriptState *script);
-	int cmd_blockInWalkableRegion(ScriptState *script);
-	int cmd_blockOutWalkableRegion(ScriptState *script);
-	int cmd_walkPlayerToPoint(ScriptState *script);
-	int cmd_dropItemInScene(ScriptState *script);
-	int cmd_drawAnimShapeIntoScene(ScriptState *script);
-	int cmd_createMouseItem(ScriptState *script);
-	int cmd_savePageToDisk(ScriptState *script);
-	int cmd_sceneAnimOn(ScriptState *script);
-	int cmd_sceneAnimOff(ScriptState *script);
-	int cmd_getElapsedSeconds(ScriptState *script);
-	int cmd_mouseIsPointer(ScriptState *script);
-	int cmd_destroyMouseItem(ScriptState *script);
-	int cmd_runSceneAnimUntilDone(ScriptState *script);
-	int cmd_fadeSpecialPalette(ScriptState *script);
-	int cmd_playAdlibSound(ScriptState *script);
-	int cmd_playAdlibScore(ScriptState *script);
-	int cmd_phaseInSameScene(ScriptState *script);
-	int cmd_setScenePhasingFlag(ScriptState *script);
-	int cmd_resetScenePhasingFlag(ScriptState *script);
-	int cmd_queryScenePhasingFlag(ScriptState *script);
-	int cmd_sceneToDirection(ScriptState *script);
-	int cmd_setBirthstoneGem(ScriptState *script);
-	int cmd_placeItemInGenericMapScene(ScriptState *script);
-	int cmd_setBrandonStatusBit(ScriptState *script);
-	int cmd_pauseSeconds(ScriptState *script);
-	int cmd_getCharactersLocation(ScriptState *script);
-	int cmd_runNPCSubscript(ScriptState *script);
-	int cmd_magicOutMouseItem(ScriptState *script);
-	int cmd_internalAnimOn(ScriptState *script);
-	int cmd_forceBrandonToNormal(ScriptState *script);
-	int cmd_poisonDeathNow(ScriptState *script);
-	int cmd_setScaleMode(ScriptState *script);
-	int cmd_openWSAFile(ScriptState *script);
-	int cmd_closeWSAFile(ScriptState *script);
-	int cmd_runWSAFromBeginningToEnd(ScriptState *script);
-	int cmd_displayWSAFrame(ScriptState *script);
-	int cmd_enterNewScene(ScriptState *script);
-	int cmd_setSpecialEnterXAndY(ScriptState *script);
-	int cmd_runWSAFrames(ScriptState *script);
-	int cmd_popBrandonIntoScene(ScriptState *script);
-	int cmd_restoreAllObjectBackgrounds(ScriptState *script);
-	int cmd_setCustomPaletteRange(ScriptState *script);
-	int cmd_loadPageFromDisk(ScriptState *script);
-	int cmd_customPrintTalkString(ScriptState *script);
-	int cmd_restoreCustomPrintBackground(ScriptState *script);
-	int cmd_hideMouse(ScriptState *script);
-	int cmd_showMouse(ScriptState *script);
-	int cmd_getCharacterX(ScriptState *script);
-	int cmd_getCharacterY(ScriptState *script);
-	int cmd_changeCharactersFacing(ScriptState *script);
-	int cmd_copyWSARegion(ScriptState *script);
-	int cmd_printText(ScriptState *script);
-	int cmd_random(ScriptState *script);
-	int cmd_loadSoundFile(ScriptState *script);
-	int cmd_displayWSAFrameOnHidPage(ScriptState *script);
-	int cmd_displayWSASequentialFrames(ScriptState *script);
-	int cmd_drawCharacterStanding(ScriptState *script);
-	int cmd_internalAnimOff(ScriptState *script);
-	int cmd_changeCharactersXAndY(ScriptState *script);
-	int cmd_clearSceneAnimatorBeacon(ScriptState *script);
-	int cmd_querySceneAnimatorBeacon(ScriptState *script);
-	int cmd_refreshSceneAnimator(ScriptState *script);
-	int cmd_placeItemInOffScene(ScriptState *script);
-	int cmd_wipeDownMouseItem(ScriptState *script);
-	int cmd_placeCharacterInOtherScene(ScriptState *script);
-	int cmd_getKey(ScriptState *script);
-	int cmd_specificItemInInventory(ScriptState *script);
-	int cmd_popMobileNPCIntoScene(ScriptState *script);
-	int cmd_mobileCharacterInScene(ScriptState *script);
-	int cmd_hideMobileCharacter(ScriptState *script);
-	int cmd_unhideMobileCharacter(ScriptState *script);
-	int cmd_setCharactersLocation(ScriptState *script);
-	int cmd_walkCharacterToPoint(ScriptState *script);
-	int cmd_specialEventDisplayBrynnsNote(ScriptState *script);
-	int cmd_specialEventRemoveBrynnsNote(ScriptState *script);
-	int cmd_setLogicPage(ScriptState *script);
-	int cmd_fatPrint(ScriptState *script);
-	int cmd_preserveAllObjectBackgrounds(ScriptState *script);
-	int cmd_updateSceneAnimations(ScriptState *script);
-	int cmd_sceneAnimationActive(ScriptState *script);
-	int cmd_setCharactersMovementDelay(ScriptState *script);
-	int cmd_getCharactersFacing(ScriptState *script);
-	int cmd_bkgdScrollSceneAndMasksRight(ScriptState *script);
-	int cmd_dispelMagicAnimation(ScriptState *script);
-	int cmd_findBrightestFireberry(ScriptState *script);
-	int cmd_setFireberryGlowPalette(ScriptState *script);
-	int cmd_setDeathHandlerFlag(ScriptState *script);
-	int cmd_drinkPotionAnimation(ScriptState *script);
-	int cmd_makeAmuletAppear(ScriptState *script);
-	int cmd_drawItemShapeIntoScene(ScriptState *script);
-	int cmd_setCharactersCurrentFrame(ScriptState *script);
-	int cmd_waitForConfirmationMouseClick(ScriptState *script);
-	int cmd_pageFlip(ScriptState *script);
-	int cmd_setSceneFile(ScriptState *script);
-	int cmd_getItemInMarbleVase(ScriptState *script);
-	int cmd_setItemInMarbleVase(ScriptState *script);
-	int cmd_addItemToInventory(ScriptState *script);
-	int cmd_intPrint(ScriptState *script);
-	int cmd_shakeScreen(ScriptState *script);
-	int cmd_createAmuletJewel(ScriptState *script);
-	int cmd_setSceneAnimCurrXY(ScriptState *script);
-	int cmd_poisonBrandonAndRemaps(ScriptState *script);
-	int cmd_fillFlaskWithWater(ScriptState *script);
-	int cmd_getCharactersMovementDelay(ScriptState *script);
-	int cmd_getBirthstoneGem(ScriptState *script);
-	int cmd_queryBrandonStatusBit(ScriptState *script);
-	int cmd_playFluteAnimation(ScriptState *script);
-	int cmd_playWinterScrollSequence(ScriptState *script);
-	int cmd_getIdolGem(ScriptState *script);
-	int cmd_setIdolGem(ScriptState *script);
-	int cmd_totalItemsInScene(ScriptState *script);
-	int cmd_restoreBrandonsMovementDelay(ScriptState *script);
-	int cmd_setMousePos(ScriptState *script);
-	int cmd_getMouseState(ScriptState *script);
-	int cmd_setEntranceMouseCursorTrack(ScriptState *script);
-	int cmd_itemAppearsOnGround(ScriptState *script);
-	int cmd_setNoDrawShapesFlag(ScriptState *script);
-	int cmd_fadeEntirePalette(ScriptState *script);
-	int cmd_itemOnGroundHere(ScriptState *script);
-	int cmd_queryCauldronState(ScriptState *script);
-	int cmd_setCauldronState(ScriptState *script);
-	int cmd_queryCrystalState(ScriptState *script);
-	int cmd_setCrystalState(ScriptState *script);
-	int cmd_setPaletteRange(ScriptState *script);
-	int cmd_shrinkBrandonDown(ScriptState *script);
-	int cmd_growBrandonUp(ScriptState *script);
-	int cmd_setBrandonScaleXAndY(ScriptState *script);
-	int cmd_resetScaleMode(ScriptState *script);
-	int cmd_getScaleDepthTableValue(ScriptState *script);
-	int cmd_setScaleDepthTableValue(ScriptState *script);
-	int cmd_message(ScriptState *script);
-	int cmd_checkClickOnNPC(ScriptState *script);
-	int cmd_getFoyerItem(ScriptState *script);
-	int cmd_setFoyerItem(ScriptState *script);
-	int cmd_setNoItemDropRegion(ScriptState *script);
-	int cmd_walkMalcolmOn(ScriptState *script);
-	int cmd_passiveProtection(ScriptState *script);
-	int cmd_setPlayingLoop(ScriptState *script);
-	int cmd_brandonToStoneSequence(ScriptState *script);
-	int cmd_brandonHealingSequence(ScriptState *script);
-	int cmd_protectCommandLine(ScriptState *script);
-	int cmd_pauseMusicSeconds(ScriptState *script);
-	int cmd_resetMaskRegion(ScriptState *script);
-	int cmd_setPaletteChangeFlag(ScriptState *script);
-	int cmd_fillRect(ScriptState *script);
-	int cmd_dummy(ScriptState *script);
-	int cmd_vocUnload(ScriptState *script);
-	int cmd_vocLoad(ScriptState *script);
 
 protected:
-
-	virtual int go();
-	virtual int init();
-
-	void startup();
-	void mainLoop();
-	int initCharacterChat(int8 charNum);
-	int8 getChatPartnerNum();
-	void backupChatPartnerAnimFrame(int8 charNum);
-	void restoreChatPartnerAnimFrame(int8 charNum);
-	void endCharacterChat(int8 charNum, int16 arg_4);
-	void waitForChatToFinish(int16 chatDuration, const char *str, uint8 charNum);
-	void characterSays(const char *chatStr, int8 charNum, int8 chatDuration);
-
-	void setCharactersPositions(int character);
-	int setGameFlag(int flag);
-	int queryGameFlag(int flag);
-	int resetGameFlag(int flag);
-
-	void setupSceneResource(int sceneId);
-	
-	void enterNewScene(int sceneId, int facing, int unk1, int unk2, int brandonAlive);
-	void transcendScenes(int roomIndex, int roomName);
-	void setSceneFile(int roomIndex, int roomName);
-	void moveCharacterToPos(int character, int facing, int xpos, int ypos);
-	void setCharacterPositionWithUpdate(int character);
-	int setCharacterPosition(int character, int *facingTable);
-	void setCharacterPositionHelper(int character, int *facingTable);
-	int getOppositeFacingDirection(int dir);
-	void loadSceneMSC();
-	void startSceneScript(int brandonAlive);
-	void setupSceneItems();
-	void initSceneData(int facing, int unk1, int brandonAlive);
-	void clearNoDropRects();
-	void addToNoDropRects(int x, int y, int w, int h);
-	byte findFreeItemInScene(int scene);
-	byte findItemAtPos(int x, int y);
-	void placeItemInGenericMapScene(int item, int index);
-	void initSceneObjectList(int brandonAlive);
-	void initSceneScreen(int brandonAlive);
-	int findDuplicateItemShape(int shape);
-	int findWay(int x, int y, int toX, int toY, int *moveTable, int moveTableSize);
-	int findSubPath(int x, int y, int toX, int toY, int *moveTable, int start, int end);
-	int getFacingFromPointToPoint(int x, int y, int toX, int toY);
-	void changePosTowardsFacing(int &x, int &y, int facing);
-	bool lineIsPassable(int x, int y);
-	int getMoveTableSize(int *moveTable);
-	int handleSceneChange(int xpos, int ypos, int unk1, int frameReset);
-	int processSceneChange(int *table, int unk1, int frameReset);
-	int changeScene(int facing);
-	void createMouseItem(int item);
-	void destroyMouseItem();
-	void setMouseItem(int item);
-	void wipeDownMouseItem(int xpos, int ypos);
-	void setBrandonPoisonFlags(int reset);
-	void resetBrandonPoisonFlags();
-
-	void processInput(int xpos, int ypos);
-	int processInputHelper(int xpos, int ypos);
-	int clickEventHandler(int xpos, int ypos);
-	void clickEventHandler2();
-	void updateMousePointer(bool forceUpdate = false);
-	bool hasClickedOnExit(int xpos, int ypos);
-	int checkForNPCScriptRun(int xpos, int ypos);
-	void runNpcScript(int func);
-	
-	int countItemsInScene(uint16 sceneId);
-	int processItemDrop(uint16 sceneId, uint8 item, int x, int y, int unk1, int unk2);
-	void exchangeItemWithMouseItem(uint16 sceneId, int itemIndex);
-	void addItemToRoom(uint16 sceneId, uint8 item, int itemIndex, int x, int y);
-	int checkNoDropRects(int x, int y);
-	int isDropable(int x, int y);
-	void itemDropDown(int x, int y, int destX, int destY, byte freeItem, int item);
-	void dropItem(int unk1, int item, int x, int y, int unk2);
-	void itemSpecialFX(int x, int y, int item);
-	void itemSpecialFX1(int x, int y, int item);
-	void itemSpecialFX2(int x, int y, int item);
-	void magicOutMouseItem(int animIndex, int itemPos);
-	void magicInMouseItem(int animIndex, int item, int itemPos);
-	void specialMouseItemFX(int shape, int x, int y, int animIndex, int tableIndex, int loopStart, int maxLoops);
-	void processSpecialMouseItemFX(int shape, int x, int y, int tableValue, int loopStart, int maxLoops);
-	void updatePlayerItemsForScene();
-	void redrawInventory(int page);
-	
-	void drawJewelPress(int jewel, int drawSpecial);
-	void drawJewelsFadeOutStart();
-	void drawJewelsFadeOutEnd(int jewel);
-	void setupShapes123(const Shape *shapeTable, int endShape, int flags);
-	void freeShapes123();
-
+	// -> demo
 	void seq_demo();
+
+	// -> intro
 	void seq_intro();
 	void seq_introLogos();
 	void seq_introStory();
 	void seq_introMalcolmTree();
 	void seq_introKallakWriting();
 	void seq_introKallakMalcolm();
+
+	// -> ingame animations
 	void seq_createAmuletJewel(int jewel, int page, int noSound, int drawOnly);
 	void seq_brandonHealing();
 	void seq_brandonHealing2();
@@ -602,33 +309,220 @@ protected:
 	void seq_dispelMagicAnimation();
 	void seq_fillFlaskWithWater(int item, int type);
 	void seq_playDrinkPotionAnim(int item, int unk2, int flags);
-	int seq_playEnd();
 	void seq_brandonToStone();
+
+	// -> end fight
+	int seq_playEnd();
 	void seq_playEnding();
+
+	int handleMalcolmFlag();
+	int handleBeadState();
+	void initBeadState(int x, int y, int x2, int y2, int unk1, BeadState *ptr);
+	int processBead(int x, int y, int &x2, int &y2, BeadState *ptr);
+
+	// -> credits
 	void seq_playCredits();
-	void updateKyragemFading();
-	
-	void setupOpcodeTable();
-	OpcodeProc *_opcodeTable;
-	int _opcodeTableSize;
-	
+
+public:
+	// delay
+	void delayUntil(uint32 timestamp, bool updateGameTimers = false, bool update = false, bool isMainLoop = false);
+	void delay(uint32 millis, bool update = false, bool isMainLoop = false);
+	void delayWithTicks(int ticks);
 	void waitForEvent();
-	void loadPalette(const char *filename, uint8 *palData);
+
+	// TODO
+	void quitGame();
+
+	void registerDefaultSettings();
+	void readSettings();
+	void writeSettings();
+
+	void snd_playTheme(int file, int track = 0);
+	void snd_playVoiceFile(int id);
+	void snd_voiceWaitForFinish(bool ingame = true);
+	bool snd_voiceIsPlaying();
+	void snd_stopVoice();
+	void snd_playSoundEffect(int track);
+	void snd_playWanderScoreViaMap(int command, int restart);
+
+	bool speechEnabled();
+	bool textEnabled();
+	
+	void updateGameTimers();
+	void clearNextEventTickCount();
+	void setTimerCountdown(uint8 timer, int32 countdown);
+	void setTimerDelay(uint8 timer, int32 countdown);
+	int16 getTimerDelay(uint8 timer);
+	void enableTimer(uint8 timer);
+	void disableTimer(uint8 timer);
+
+	void saveGame(const char *fileName, const char *saveName);
+	void loadGame(const char *fileName);
+
+	Common::Point getMousePos() const;
+
+	int setGameFlag(int flag);
+	int queryGameFlag(int flag);
+	int resetGameFlag(int flag);
+protected:
+	virtual int go();
+	virtual int init();
+
+	// input
+	void processInput();
+	int processInputHelper(int xpos, int ypos);
+	int clickEventHandler(int xpos, int ypos);
+	void clickEventHandler2();
+	void updateMousePointer(bool forceUpdate = false);
+	bool hasClickedOnExit(int xpos, int ypos);
+
+	// scene
+	// -> init
+	void loadSceneMsc();
+	void startSceneScript(int brandonAlive);
+	void setupSceneItems();
+	void initSceneData(int facing, int unk1, int brandonAlive);
+	void initSceneObjectList(int brandonAlive);
+	void initSceneScreen(int brandonAlive);
+	void setupSceneResource(int sceneId);
+
+	// -> process
+	void enterNewScene(int sceneId, int facing, int unk1, int unk2, int brandonAlive);
+	int handleSceneChange(int xpos, int ypos, int unk1, int frameReset);
+	int processSceneChange(int *table, int unk1, int frameReset);
+	int changeScene(int facing);
+
+	// -> modification
+	void transcendScenes(int roomIndex, int roomName);
+	void setSceneFile(int roomIndex, int roomName);
+
+	// -> pathfinder
+	int findWay(int x, int y, int toX, int toY, int *moveTable, int moveTableSize);
+	int findSubPath(int x, int y, int toX, int toY, int *moveTable, int start, int end);
+	int getFacingFromPointToPoint(int x, int y, int toX, int toY);
+	void changePosTowardsFacing(int &x, int &y, int facing);
+	bool lineIsPassable(int x, int y);
+	int getMoveTableSize(int *moveTable);
+
+	// -> item handling 
+	// --> misc
+	void addItemToRoom(uint16 sceneId, uint8 item, int itemIndex, int x, int y);
+
+	// --> drop handling
+	void itemDropDown(int x, int y, int destX, int destY, byte freeItem, int item);
+	int processItemDrop(uint16 sceneId, uint8 item, int x, int y, int unk1, int unk2);
+	void dropItem(int unk1, int item, int x, int y, int unk2);
+
+	// --> dropped item handling
+	int countItemsInScene(uint16 sceneId);
+	void exchangeItemWithMouseItem(uint16 sceneId, int itemIndex);
+	byte findFreeItemInScene(int scene);
+	byte findItemAtPos(int x, int y);
+
+	// --> drop area handling
+	void addToNoDropRects(int x, int y, int w, int h);
+	void clearNoDropRects();
+	int isDropable(int x, int y);
+	int checkNoDropRects(int x, int y);
+
+	// --> player items handling
+	void updatePlayerItemsForScene();
+
+	// --> item GFX handling
+	void backUpItemRect0(int xpos, int ypos);
+	void restoreItemRect0(int xpos, int ypos);
+	void backUpItemRect1(int xpos, int ypos);
+	void restoreItemRect1(int xpos, int ypos);
+
+	// items
+	// -> misc
+	void placeItemInGenericMapScene(int item, int index);
+
+	// -> mouse item
+	void createMouseItem(int item);
+	void destroyMouseItem();
+	void setMouseItem(int item);
+
+	// -> graphics effects
+	void wipeDownMouseItem(int xpos, int ypos);
+	void itemSpecialFX(int x, int y, int item);
+	void itemSpecialFX1(int x, int y, int item);
+	void itemSpecialFX2(int x, int y, int item);
+	void magicOutMouseItem(int animIndex, int itemPos);
+	void magicInMouseItem(int animIndex, int item, int itemPos);
+	void specialMouseItemFX(int shape, int x, int y, int animIndex, int tableIndex, int loopStart, int maxLoops);
+	void processSpecialMouseItemFX(int shape, int x, int y, int tableValue, int loopStart, int maxLoops);
+
+	// character
+	// -> movement
+	void moveCharacterToPos(int character, int facing, int xpos, int ypos);
+	void setCharacterPositionWithUpdate(int character);
+	int setCharacterPosition(int character, int *facingTable);
+	void setCharacterPositionHelper(int character, int *facingTable);
+	int getOppositeFacingDirection(int dir);
+	void setCharactersPositions(int character);
+
+	// -> brandon
+	void setBrandonPoisonFlags(int reset);
+	void resetBrandonPoisonFlags();
+
+	// chat
+	// -> process
+	void characterSays(int vocFile, const char *chatStr, int8 charNum, int8 chatDuration);
+	void waitForChatToFinish(int vocFile, int16 chatDuration, const char *str, uint8 charNum);
+
+	// -> initialization
+	int initCharacterChat(int8 charNum);
+	void backupChatPartnerAnimFrame(int8 charNum);
+	void restoreChatPartnerAnimFrame(int8 charNum);
+	int8 getChatPartnerNum();
+
+	// -> deinitialization
+	void endCharacterChat(int8 charNum, int16 arg_4);
+
+	// graphics
+	// -> misc
+	int findDuplicateItemShape(int shape);
+	void updateKyragemFading();
+
+	// -> interface
+	void loadMainScreen(int page = 3);
+	void redrawInventory(int page);
+public:
+	void drawSentenceCommand(const char *sentence, int unk1);
+	void updateSentenceCommand(const char *str1, const char *str2, int unk1);
+	void updateTextFade();
+
+protected:
+	// -> amulet
+	void drawJewelPress(int jewel, int drawSpecial);
+	void drawJewelsFadeOutStart();
+	void drawJewelsFadeOutEnd(int jewel);
+
+	// -> shape handling
+	void setupShapes123(const Shape *shapeTable, int endShape, int flags);
+	void freeShapes123();
+
+	// misc (TODO)
+	void startup();
+	void mainLoop();
+	
+	int checkForNPCScriptRun(int xpos, int ypos);
+	void runNpcScript(int func);
+	
+	virtual void setupOpcodeTable() = 0;
+	Common::Array<const Opcode*> _opcodes; 
+	
 	void loadMouseShapes();
 	void loadCharacterShapes();
 	void loadSpecialEffectShapes();
 	void loadItems();
 	void loadButtonShapes();
 	void initMainButtonList();
-	void loadMainScreen(int page = 3);
 	void setCharactersInDefaultScene();
 	void setupPanPages();
 	void freePanPages();
 	void closeFinalWsa();
-	int handleMalcolmFlag();
-	int handleBeadState();
-	void initBeadState(int x, int y, int x2, int y2, int unk1, BeadState *ptr);
-	int processBead(int x, int y, int &x2, int &y2, BeadState *ptr);
 	
 	void setTimer19();
 	void setupTimers();
@@ -697,7 +591,19 @@ protected:
 	void gui_restorePalette();
 	void gui_setupControls(Menu &menu);
 
-	uint8 _game;
+	// Kyra 2 and 3 main menu
+
+	static const char *_mainMenuStrings[];
+	virtual void gui_initMainMenu() {}
+	int gui_handleMainMenu();
+	virtual void gui_updateMainMenuAnimation();
+	void gui_drawMainMenu(const char * const *strings, int select);
+	void gui_drawMainBox(int x, int y, int w, int h, int fill);
+	bool gui_mainMenuGetInput();
+	
+	void gui_printString(const char *string, int x, int y, int col1, int col2, int flags, ...);
+
+	GameFlags _flags;
 	bool _quitFlag;
 	bool _skipFlag;
 	bool _skipIntroFlag;
@@ -707,12 +613,12 @@ protected:
 	bool _abortWalkFlag2;
 	bool _mousePressFlag;
 	int8 _mouseWheel;
-	uint8 _flagsTable[53];
-	uint8 *_shapes[377];
+	uint8 _flagsTable[69];
+	uint8 *_itemBkgBackUp[2];
+	uint8 *_shapes[373];
 	uint16 _gameSpeed;
 	uint16 _tickLength;
-	uint32 _features;
-	int _mouseX, _mouseY;
+	int _lang;
 	int8 _itemInHand;
 	int _mouseState;
 	bool _handleInput;
@@ -797,12 +703,12 @@ protected:
 
 	uint8 _configTextspeed;
 	uint8 _configWalkspeed;
-	bool _configMusic;
+	int _configMusic;
 	bool _configSounds;
 	uint8 _configVoice;
 
 	int _curMusicTheme;
-	int _newMusicTheme;
+	int _curSfxFile;
 	int16 _lastMusicCommand;
 
 	Resource *_res;
@@ -838,7 +744,7 @@ protected:
 	int _gameToLoad;
 	char _savegameName[31];
 	const char *_specialSavegameString;
-	KeyboardEvent _keyboardEvent;
+	uint8 _keyPressed;
 
 	struct KyragemState {
 		uint16 nextOperation;
@@ -898,6 +804,7 @@ protected:
 	const char *_textSpeedString;
 	const char *_onString;
 	const char *_offString;
+	const char *_onCDString;
 		
 	int _itemList_Size;
 	int _takenList_Size;
@@ -971,9 +878,12 @@ protected:
 	const uint8 * const*_specialPalettes;
 
 	Timer _timers[34];
-	uint32 _timerNextRun;	
-	static const char *_musicFiles[];
-	static const int _musicFilesCount;
+	uint32 _timerNextRun;
+
+	static const char *_soundFiles[];
+	static const int _soundFilesCount;
+	static const char *_soundFilesTowns[];
+	static const int _soundFilesTownsCount;
 	
 	static const int8 _charXPosTable[];
 	static const int8 _addXPosTable[];
@@ -1007,65 +917,7 @@ protected:
 	static const uint16 _amuletY2[];
 };
 
-class KyraEngine_v1 : public KyraEngine {
-public:
-	KyraEngine_v1(OSystem *system);
-	~KyraEngine_v1();
-
-	int setupGameFlags();
-};
-
-class KyraEngine_v2 : public KyraEngine {
-public:
-	KyraEngine_v2(OSystem *system);
-	~KyraEngine_v2();
-
-	int setupGameFlags() { _game = GI_KYRA2; return 0; }
-	
-	int go();
-};
-
-// maybe subclass KyraEngine_v2 later
-class WSAMovieV3;
-
-class KyraEngine_v3 : public KyraEngine {
-public:
-	KyraEngine_v3(OSystem *system);
-	~KyraEngine_v3();
-	
-	Movie *createWSAMovie();
-	
-	SoundDigital *soundDigital() { return _soundDigital; }
-
-	int setupGameFlags();
-	
-	int go();
-
-	void playVQA(const char *name);
-private:
-	int init();
-
-	SoundDigital *_soundDigital;
-	
-	int _lang;
-	
-	// sound specific
-private:
-	void playMenuAudioFile();
-	
-	int _musicSoundChannel;
-	const char *_menuAudioFile;
-
-	// gui/menu specific
-private:
-	static const char *_mainMenuStrings[];
-	int handleMainMenu(WSAMovieV3 *logo);
-	void drawMainMenu(const char * const *strings, int select);
-	void drawMainBox(int x, int y, int w, int h, int fill);
-	
-	void gui_printString(const char *string, int x, int y, int col1, int col2, int flags, ...);
-};
-
 } // End of namespace Kyra
 
 #endif
+
