@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2004-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,13 +18,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/kyra/resource.h $
- * $Id: resource.h 23950 2006-09-20 20:19:07Z lordhoto $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/kyra/resource.h $
+ * $Id: resource.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
-#ifndef RESOURCE_H
-#define RESOURCE_H
+#ifndef KYRA_RESOURCE_H
+#define KYRA_RESOURCE_H
 
 #include "common/stdafx.h"
 #include "common/scummsys.h"
@@ -33,63 +36,105 @@
 
 namespace Kyra {
 
+class ResourceFile {
+public:
+	ResourceFile() : _open(false), _protected(false), _filename() {}
+	virtual ~ResourceFile() {}
+
+	virtual uint8 *getFile(uint file) const = 0;
+	virtual bool getFileHandle(uint file, Common::File &filehandle) const = 0;
+	virtual uint32 getFileSize(uint file) const = 0;
+
+	uint filename() const { return _filename; }
+
+	virtual bool isValid(void) const { return (_filename != 0); }
+	bool isOpen(void) const { return _open; }
+
+	virtual void close() { if (!_protected) _open = false; }
+	virtual void protect(const bool prot = true) { _protected = prot; }
+	virtual void open() { _open = true; }
+protected:
+	bool _open;
+	bool _protected;
+	uint _filename;
+};
+
 // standard Package format for Kyrandia games
-class PAKFile {
+class PAKFile : public ResourceFile {
 	struct PakChunk {
-		Common::String _name;
+		uint _name;
 		uint32 _start;
 		uint32 _size;
+		
+		operator uint() const { return _name; }
 	};
 
 public:
-	PAKFile(const char *file, bool isAmiga = false);
+	PAKFile(const char *file, const char *physfile, Common::File &pakfile, bool isAmiga = false);
 	~PAKFile();
 
-	uint8 *getFile(const char *file);
-	bool getFileHandle(const char *file, Common::File &filehandle);
-	uint32 getFileSize(const char *file);
-
-	bool isValid(void) const { return !(_filename.empty()); }
-	bool isOpen(void) const { return _open; }
-
-	void open() { _open = true; }
-	void close() { _open = false; }
+	uint8 *getFile(uint file) const;
+	bool getFileHandle(uint file, Common::File &filehandle) const;
+	uint32 getFileSize(uint file) const;
 private:
-	bool _open;
-	bool _isAmiga;
-	Common::String _filename;
+	bool openFile(Common::File &filehandle) const;
+
+	Common::String _physfile;
+	uint32 _physOffset;
+
+	typedef Common::List<PakChunk>::iterator PakIterator;
+	typedef Common::List<PakChunk>::const_iterator ConstPakIterator;
 	Common::List<PakChunk> _files; // the entries
 };
 
+// installation file packages for (Kyra2/)Kyra3
+class INSFile : public ResourceFile {
+	struct FileEntry {
+		uint _name;
+		uint32 _start;
+		uint32 _size;
+		
+		operator uint() const { return _name; }
+	};
+public:
+	INSFile(const char *file);
+	~INSFile();
+
+	uint8 *getFile(uint file) const;
+	bool getFileHandle(uint file, Common::File &filehandle) const;
+	uint32 getFileSize(uint file) const;
+protected:
+	typedef Common::List<FileEntry>::iterator FileIterator;
+	typedef Common::List<FileEntry>::const_iterator ConstFileIterator;
+	Common::List<FileEntry> _files; // the entries
+
+	Common::String _physfile;
+};
+
 class Resource {
 public:
-	Resource(KyraEngine *engine);
+	Resource(KyraEngine *vm);
 	~Resource();
 	
 	bool loadPakFile(const Common::String &filename);
 	void unloadPakFile(const Common::String &filename);
-	bool isInPakList(const Common::String &filename);
+	bool isInPakList(const Common::String &filename) const;
 
-	uint8* fileData(const char *file, uint32 *size);
+	uint32 getFileSize(const char *file) const;
+	uint8* fileData(const char *file, uint32 *size) const;
 	// it gives back a file handle (used for the speech player)
 	// it could be that the needed file is embedded in the returned
 	// handle
-	bool fileHandle(const char *file, uint32 *size, Common::File &filehandle);
+	bool getFileHandle(const char *file, uint32 *size, Common::File &filehandle);
+
+	bool loadFileToBuf(const char *file, void *buf, uint32 maxSize); 
 
 protected:
-	class PakFileEntry {
-	public:
-		PakFileEntry(PAKFile *file, const Common::String str) : _file(file), _filename(str) {}
-		PakFileEntry(const PakFileEntry &c) : _file(c._file), _filename(c._filename) {}
+	typedef Common::List<ResourceFile*>::iterator ResIterator;
+	typedef Common::List<ResourceFile*>::const_iterator ConstResIterator;
 
-		PAKFile *_file;
-		const Common::String _filename;
-	private:
-		PakFileEntry &operator =(const PakFileEntry &c) { return *this; }
-	};
-
-	KyraEngine *_engine;
-	Common::List<PakFileEntry> _pakfiles;
+	KyraEngine *_vm;
+	Common::List<ResourceFile*> _pakfiles;
 };
 
 // TODO?: maybe prefix all things here with 'kKyra1' instead of 'k'
@@ -165,6 +210,9 @@ enum kKyraResources {
 	kGUIStrings,
 	kConfigStrings,
 	
+	kKyra1TownsSFXTable,
+	kCreditsStrings,
+	
 	kMaxResIDs
 };
 
@@ -173,7 +221,7 @@ struct Room;
 
 class StaticResource {
 public:
-	StaticResource(KyraEngine *engine) : _engine(engine), _resList(), _fileLoader(0), _builtIn(0), _filenameTable(0) {}
+	StaticResource(KyraEngine *vm) : _vm(vm), _resList(), _fileLoader(0), _builtIn(0), _filenameTable(0) {}
 	~StaticResource() { deinit(); }
 	
 	static bool checkKyraDat();
@@ -194,7 +242,7 @@ public:
 	bool prefetchId(int id);
 	void unloadId(int id);
 private:
-	KyraEngine *_engine;
+	KyraEngine *_vm;
 
 	struct FilenameTable;
 	struct ResData;
@@ -269,3 +317,4 @@ private:
 } // end of namespace Kyra
 
 #endif
+
