@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2005-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/kyra/saveload.cpp $
- * $Id: saveload.cpp 22904 2006-06-03 18:38:36Z lordhoto $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/kyra/saveload.cpp $
+ * $Id: saveload.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -29,8 +32,16 @@
 #include "kyra/animator.h"
 #include "kyra/screen.h"
 #include "kyra/resource.h"
+#include "kyra/sound.h"
 
-#define CURRENT_VERSION 5
+#define CURRENT_VERSION 7
+
+// TODO: our current savefiles still use the old
+// flag system to check the version, we should
+// change that some day, but for now it works
+#define GF_FLOPPY (1 <<  0)
+#define GF_TALKIE (1 <<  1)
+#define GF_FMTOWNS (1 <<  2)
 
 namespace Kyra {
 void KyraEngine::loadGame(const char *fileName) {
@@ -63,25 +74,29 @@ void KyraEngine::loadGame(const char *fileName) {
 	in->read(saveName, 31);
 
 	if (version >= 2) {
-		uint32 gameFlags = in->readUint32BE();
-		if ((gameFlags & GF_FLOPPY) && !(_features & GF_FLOPPY)) {
-			warning("can not load floppy savefile for this (non floppy) gameversion");
+		uint32 flags = in->readUint32BE();
+		if ((flags & GF_FLOPPY) && _flags.isTalkie) {
+			warning("Can not load floppy savefile for this (non floppy) gameversion");
 			delete in;
 			return;
-		} else if ((gameFlags & GF_TALKIE) && !(_features & GF_TALKIE)) {
-			warning("can not load cdrom savefile for this (non cdrom) gameversion");
+		} else if ((flags & GF_TALKIE) && !(_flags.isTalkie)) {
+			warning("Can not load cdrom savefile for this (non cdrom) gameversion");
+			delete in;
+			return;
+		} else if ((flags & GF_FMTOWNS) && !(_flags.platform == Common::kPlatformFMTowns)) {
+			warning("can not load FM-Towns savefile for this (non FM-Towns) gameversion");
 			delete in;
 			return;
 		}
 	} else {
 		warning("Make sure your savefile was from this version! (too old savefile version to detect that)");
 	}
-	
+
 	snd_playSoundEffect(0x0A);
 	snd_playWanderScoreViaMap(0, 1);
 
 	// unload the current voice file should fix some problems with voices
-	if (_currentRoom != 0xFFFF && (_features & GF_TALKIE)) {
+	if (_currentRoom != 0xFFFF && _flags.isTalkie) {
 		char file[32];
 		assert(_currentRoom < _roomTableSize);
 		int tableId = _roomTable[_currentRoom].nameIndex;
@@ -114,19 +129,15 @@ void KyraEngine::loadGame(const char *fileName) {
 	_marbleVaseItem = in->readSint16BE();
 	_itemInHand = in->readByte();
 	
-	for (int i = 0; i < 4; ++i) {
+	for (int i = 0; i < 4; ++i)
 		_birthstoneGemTable[i] = in->readByte();
-	}
-	for (int i = 0; i < 3; ++i) {
+	for (int i = 0; i < 3; ++i)
 		_idolGemsTable[i] = in->readByte();
-	}
-	for (int i = 0; i < 3; ++i) {
+	for (int i = 0; i < 3; ++i)
 		_foyerItemTable[i] = in->readByte();
-	}
 	_cauldronState = in->readByte();
-	for (int i = 0; i < 2; ++i) {
+	for (int i = 0; i < 2; ++i)
 		_crystalState[i] = in->readByte();
-	}
 	
 	_brandonStatusBit = in->readUint16BE();
 	_brandonStatusBit0x02Flag = in->readByte();
@@ -145,8 +156,9 @@ void KyraEngine::loadGame(const char *fileName) {
 	}
 	_timerNextRun = 0;
 
+	memset(_flagsTable, 0, sizeof(_flagsTable));
 	uint32 flagsSize = in->readUint32BE();
-	assert(flagsSize == sizeof(_flagsTable));
+	assert(flagsSize <= sizeof(_flagsTable));
 	in->read(_flagsTable, flagsSize);
 
 	for (int i = 0; i < _roomTableSize; ++i) {
@@ -191,21 +203,34 @@ void KyraEngine::loadGame(const char *fileName) {
 		in->readByte(); // Voice
 	}
 
+	if (version >= 7) {
+		_curSfxFile = in->readByte();
+
+		// In the first version there this entry was introduced,
+		// it wasn't made sure that _curSfxFile was initialized
+		// so if it's out of bounds we just set it to 0.
+		if (_curSfxFile >= _soundFilesTownsCount || _curSfxFile < 0)
+			_curSfxFile = 0;
+		
+		if (_flags.platform == Common::kPlatformFMTowns)
+			_sound->loadSoundFile(_curSfxFile);
+	}
+
+	_screen->_disableScreen = true;
 	loadMainScreen(8);
 
 	if (queryGameFlag(0x2D)) {
 		_screen->loadBitmap("AMULET3.CPS", 10, 10, 0);
 		if (!queryGameFlag(0xF1)) {
 			for (int i = 0x55; i <= 0x5A; ++i) {
-				if (queryGameFlag(i)) {
+				if (queryGameFlag(i))
 					seq_createAmuletJewel(i-0x55, 10, 1, 1);
-				}
 			}
 		}
 		_screen->copyRegion(0, 0, 0, 0, 320, 200, 10, 8);
 		_screen->copyRegion(0, 0, 0, 0, 320, 200, 8, 0);
 	}
-	
+
 	createMouseItem(_itemInHand);
 	_animator->setBrandonAnimSeqSize(3, 48);
 	redrawInventory(0);
@@ -221,12 +246,12 @@ void KyraEngine::loadGame(const char *fileName) {
 	_animator->prepDrawAllObjects();
 	_animator->copyChangedObjectsForward(0);
 	_screen->copyRegion(8, 8, 8, 8, 304, 128, 2, 0);
+	_screen->_disableScreen = false;
+	_screen->updateScreen();
 	
 	_abortWalkFlag = true;
 	_abortWalkFlag2 = false;
 	_mousePressFlag = false;
-	_mouseX = brandonX;
-	_mouseY = brandonY;
 	_system->warpMouse(brandonX, brandonY);
 
 	if (in->ioFailed())
@@ -255,7 +280,12 @@ void KyraEngine::saveGame(const char *fileName, const char *saveName) {
 	out->writeUint32BE(MKID_BE('KYRA'));
 	out->writeUint32BE(CURRENT_VERSION);
 	out->write(saveName, 31);
-	out->writeUint32BE(_features);
+	if (_flags.isTalkie)
+		out->writeUint32BE(GF_TALKIE);
+	else if (_flags.platform == Common::kPlatformFMTowns)
+		out->writeUint32BE(GF_FMTOWNS);
+	else
+		out->writeUint32BE(GF_FLOPPY);
 
 	for (int i = 0; i < 11; i++) {
 		out->writeUint16BE(_characterList[i].sceneId);
@@ -275,19 +305,15 @@ void KyraEngine::saveGame(const char *fileName, const char *saveName) {
 	out->writeSint16BE(_marbleVaseItem);
 	out->writeByte(_itemInHand);
 	
-	for (int i = 0; i < 4; ++i) {
+	for (int i = 0; i < 4; ++i)
 		out->writeByte(_birthstoneGemTable[i]);
-	}
-	for (int i = 0; i < 3; ++i) {
+	for (int i = 0; i < 3; ++i)
 		out->writeByte(_idolGemsTable[i]);
-	}
-	for (int i = 0; i < 3; ++i) {
+	for (int i = 0; i < 3; ++i)
 		out->writeByte(_foyerItemTable[i]);
-	}
 	out->writeByte(_cauldronState);
-	for (int i = 0; i < 2; ++i) {
+	for (int i = 0; i < 2; ++i)
 		out->writeByte(_crystalState[i]);
-	}
 	
 	out->writeUint16BE(_brandonStatusBit);
 	out->writeByte(_brandonStatusBit0x02Flag);
@@ -300,11 +326,10 @@ void KyraEngine::saveGame(const char *fileName, const char *saveName) {
 	for (int i = 0; i < 32; i++) {
 		out->writeByte(_timers[i].active);
 		out->writeSint32BE(_timers[i].countdown);
-		if (_system->getMillis() >= _timers[i].nextRun) {
+		if (_system->getMillis() >= _timers[i].nextRun)
 			out->writeUint32BE(0);
-		} else {
+		else
 			out->writeUint32BE(_timers[i].nextRun - _system->getMillis());
-		}
 	}
 
 	out->writeUint32BE(sizeof(_flagsTable));
@@ -325,7 +350,9 @@ void KyraEngine::saveGame(const char *fileName, const char *saveName) {
 	
 	out->writeSint16BE(_lastMusicCommand);
 
-	out->flush();
+	out->writeByte(_curSfxFile);
+
+	out->finalize();
 
 	// check for errors
 	if (out->ioFailed())
@@ -336,3 +363,4 @@ void KyraEngine::saveGame(const char *fileName, const char *saveName) {
 	delete out;
 }
 } // end of namespace Kyra
+
