@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2005-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/kyra/scene.cpp $
- * $Id: scene.cpp 23950 2006-09-20 20:19:07Z lordhoto $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/kyra/scene.cpp $
+ * $Id: scene.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -43,25 +46,29 @@ void KyraEngine::enterNewScene(int sceneId, int facing, int unk1, int unk2, int
 	_handleInput = false;
 	_abortWalkFlag = false;
 	_abortWalkFlag2 = false;
-	// just used for cd audio version, it should only load the sfx music file there
-	/*if (_currentCharacter->sceneId == 7 && sceneId == 24) {
-		_newMusicTheme = 3;
-	} else if (_currentCharacter->sceneId == 25 && sceneId == 109) {
-		_newMusicTheme = 4;
-	} else if (_currentCharacter->sceneId == 120 && sceneId == 37) {
-		_newMusicTheme = 5;
-	} else if (_currentCharacter->sceneId == 52 && sceneId == 199) {
-		_newMusicTheme = 6;
-	} else if (_currentCharacter->sceneId == 37 && sceneId == 120) {
-		_newMusicTheme = 4;
-	} else if (_currentCharacter->sceneId == 109 && sceneId == 25) {
-		_newMusicTheme = 3;
-	} else if (_currentCharacter->sceneId == 24 && sceneId == 7) {
-		_newMusicTheme = 2;
+
+	if (_flags.platform == Common::kPlatformFMTowns) {
+		int newSfxFile = -1;
+		if (_currentCharacter->sceneId == 7 && sceneId == 24)
+			newSfxFile = 2;
+		else if (_currentCharacter->sceneId == 25 && sceneId == 109)
+			newSfxFile = 3;
+		else if (_currentCharacter->sceneId == 120 && sceneId == 37)
+			newSfxFile = 4;
+		else if (_currentCharacter->sceneId == 52 && sceneId == 199)
+			newSfxFile = 5;
+		else if (_currentCharacter->sceneId == 37 && sceneId == 120)
+			newSfxFile = 3;
+		else if (_currentCharacter->sceneId == 109 && sceneId == 25)
+			newSfxFile = 2;
+		else if (_currentCharacter->sceneId == 24 && sceneId == 7)
+			newSfxFile = 1;
+		
+		if (newSfxFile != -1) {
+			_curSfxFile = newSfxFile;
+			_sound->loadSoundFile(_curSfxFile);
+		}
 	}
-	if (_newMusicTheme != _curMusicTheme) {
-		snd_playTheme(_newMusicTheme);
-	}*/
 	
 	switch (_currentCharacter->sceneId) {
 	case 1:
@@ -123,16 +130,14 @@ void KyraEngine::enterNewScene(int sceneId, int facing, int unk1, int unk2, int
 		moveCharacterToPos(0, facing, xpos, ypos);
 	}
 	
-	for (int i = 0; i < ARRAYSIZE(_movieObjects); ++i) {
+	for (int i = 0; i < ARRAYSIZE(_movieObjects); ++i)
 		_movieObjects[i]->close();
-	}
 	
 	if (!brandonAlive) {
 		_scriptInterpreter->initScript(_scriptClick, _scriptClickData);
 		_scriptInterpreter->startScript(_scriptClick, 5);
-		while (_scriptInterpreter->validScript(_scriptClick)) {
+		while (_scriptInterpreter->validScript(_scriptClick))
 			_scriptInterpreter->runScript(_scriptClick);
-		}
 	}
 	
 	memset(_entranceMouseCursorTracks, 0xFFFF, sizeof(uint16)*4);
@@ -144,39 +149,34 @@ void KyraEngine::enterNewScene(int sceneId, int facing, int unk1, int unk2, int
 	Room *currentRoom = &_roomTable[sceneId];
 	
 	setupSceneResource(sceneId);
-
+	
 	_currentRoom = sceneId;
 	
-	int tableId = _roomTable[_currentCharacter->sceneId].nameIndex;
+	int tableId = _roomTable[sceneId].nameIndex;
 	char fileNameBuffer[32];
 	strcpy(fileNameBuffer, _roomFilenameTable[tableId]);
 	strcat(fileNameBuffer, ".DAT");
-	_sprites->loadDAT(fileNameBuffer, _sceneExits);
+	_sprites->loadDat(fileNameBuffer, _sceneExits);
 	_sprites->setupSceneAnims();
 	_scriptInterpreter->unloadScript(_scriptClickData);
-	loadSceneMSC();
+	loadSceneMsc();
 	
 	_walkBlockNorth = currentRoom->northExit;
 	_walkBlockEast = currentRoom->eastExit;
 	_walkBlockSouth = currentRoom->southExit;
 	_walkBlockWest = currentRoom->westExit;
 	
-	if (_walkBlockNorth == 0xFFFF) {
+	if (_walkBlockNorth == 0xFFFF)
 		_screen->blockOutRegion(0, 0, 320, (_northExitHeight & 0xFF)+3);
-	}
-	if (_walkBlockEast == 0xFFFF) {
+	if (_walkBlockEast == 0xFFFF)
 		_screen->blockOutRegion(312, 0, 8, 139);
-	}
-	if (_walkBlockSouth == 0xFFFF) {
+	if (_walkBlockSouth == 0xFFFF)
 		_screen->blockOutRegion(0, 135, 320, 8);
-	}
-	if (_walkBlockWest == 0xFFFF) {
+	if (_walkBlockWest == 0xFFFF)
 		_screen->blockOutRegion(0, 0, 8, 139);
-	}
 	
-	if (!brandonAlive) {
+	if (!brandonAlive)
 		updatePlayerItemsForScene();
-	}
 
 	startSceneScript(brandonAlive);
 	setupSceneItems();
@@ -185,9 +185,8 @@ void KyraEngine::enterNewScene(int sceneId, int facing, int unk1, int unk2, int
 	
 	_loopFlag2 = 0;
 	_screen->showMouse();
-	if (!brandonAlive) {
+	if (!brandonAlive)
 		seq_poisonDeathNow(0);
-	}
 	updateMousePointer(true);
 	_changedScene = true;
 }
@@ -195,7 +194,8 @@ void KyraEngine::enterNewScene(int sceneId, int facing, int unk1, int unk2, int
 void KyraEngine::transcendScenes(int roomIndex, int roomName) {
 	debugC(9, kDebugLevelMain, "KyraEngine::transcendScenes(%d, %d)", roomIndex, roomName);
 	assert(roomIndex < _roomTableSize);
-	if (_features & GF_TALKIE) {
+
+	if (_flags.isTalkie) {
 		char file[32];
 		assert(roomIndex < _roomTableSize);
 		int tableId = _roomTable[roomIndex].nameIndex;
@@ -204,6 +204,7 @@ void KyraEngine::transcendScenes(int roomIndex, int roomName) {
 		strcat(file, ".VRM");
 		_res->unloadPakFile(file);
 	}
+
 	_roomTable[roomIndex].nameIndex = roomName;
 	_unkScreenVar2 = 1;
 	_unkScreenVar3 = 1;
@@ -233,6 +234,7 @@ void KyraEngine::moveCharacterToPos(int character, int facing, int xpos, int ypo
 	disableTimer(14);
 	disableTimer(18);
 	uint32 nextFrame = 0;
+
 	switch (facing) {
 	case 0:
 		while (ypos < ch->y1) {
@@ -269,6 +271,7 @@ void KyraEngine::moveCharacterToPos(int character, int facing, int xpos, int ypo
 	default:
 		break;
 	}
+
 	enableTimer(19);
 	enableTimer(14);
 	enableTimer(18);
@@ -283,13 +286,13 @@ void KyraEngine::setCharacterPositionWithUpdate(int character) {
 	_animator->updateAllObjectShapes();
 	updateTextFade();
 
-	if (_currentCharacter->sceneId == 210) {
+	if (_currentCharacter->sceneId == 210)
 		updateKyragemFading();
-	}
 }
 
 int KyraEngine::setCharacterPosition(int character, int *facingTable) {
 	debugC(9, kDebugLevelMain, "KyraEngine::setCharacterPosition(%d, %p)", character, (const void *)facingTable);
+
 	if (character == 0) {
 		_currentCharacter->x1 += _charXPosTable[_currentCharacter->facing];
 		_currentCharacter->y1 += _charYPosTable[_currentCharacter->facing];
@@ -298,9 +301,8 @@ int KyraEngine::setCharacterPosition(int character, int *facingTable) {
 	} else {
 		_characterList[character].x1 += _charXPosTable[_characterList[character].facing];
 		_characterList[character].y1 += _charYPosTable[_characterList[character].facing];
-		if (_characterList[character].sceneId == _currentCharacter->sceneId) {
+		if (_characterList[character].sceneId == _currentCharacter->sceneId)
 			setCharacterPositionHelper(character, 0);
-		}
 	}
 	return 0;
 }
@@ -330,24 +332,21 @@ void KyraEngine::setCharacterPositionHelper(int character, int *facingTable) {
 			if (facing - 1 != 0) {
 				if (facing != 4) {
 					if (facing == 3 || facing == 5) {
-						if (facingIsFour[character] > 2) {
+						if (facingIsFour[character] > 2)
 							facing = 4;
-						}
 						resetTables = true;
 					}
 				} else {
 					++facingIsFour[character];
 				}
 			} else {
-				if (facingIsZero[character] > 2) {
+				if (facingIsZero[character] > 2)
 					facing = 0;
-				}
 				resetTables = true;
 			}
 		} else {
-			if (facingIsZero[character] > 2) {
+			if (facingIsZero[character] > 2)
 				facing = 0;
-			}
 			resetTables = true;
 		}
 		
@@ -368,35 +367,26 @@ void KyraEngine::setCharacterPositionHelper(int character, int *facingTable) {
 	};
 	
 	if (facing == 0) {
-		if (maxAnimationFrame[36+character] > ch->currentAnimFrame) {
+		if (maxAnimationFrame[36+character] > ch->currentAnimFrame)
 			ch->currentAnimFrame = maxAnimationFrame[36+character];
-		}
-		if (maxAnimationFrame[30+character] < ch->currentAnimFrame) {
+		if (maxAnimationFrame[30+character] < ch->currentAnimFrame)
 			ch->currentAnimFrame = maxAnimationFrame[36+character];
-		}
 	} else if (facing == 4) {
-		if (maxAnimationFrame[18+character] > ch->currentAnimFrame) {
+		if (maxAnimationFrame[18+character] > ch->currentAnimFrame)
 			ch->currentAnimFrame = maxAnimationFrame[18+character];
-		}
-		if (maxAnimationFrame[12+character] < ch->currentAnimFrame) {
+		if (maxAnimationFrame[12+character] < ch->currentAnimFrame)
 			ch->currentAnimFrame = maxAnimationFrame[18+character];
-		}
 	} else {
-		if (maxAnimationFrame[18+character] < ch->currentAnimFrame) {
+		if (maxAnimationFrame[18+character] < ch->currentAnimFrame)
 			ch->currentAnimFrame = maxAnimationFrame[30+character];
-		}
-		if (maxAnimationFrame[character] == ch->currentAnimFrame) {
+		if (maxAnimationFrame[character] == ch->currentAnimFrame)
 			ch->currentAnimFrame = maxAnimationFrame[6+character];
-		}
-		if (maxAnimationFrame[character] < ch->currentAnimFrame) {
+		if (maxAnimationFrame[character] < ch->currentAnimFrame)
 			ch->currentAnimFrame = maxAnimationFrame[6+character]+2;
-		}
 	}
 	
-	if (character == 0) {
-		if (_brandonStatusBit & 0x10)
-			ch->currentAnimFrame = 88;
-	}
+	if (character == 0 && (_brandonStatusBit & 0x10))
+		ch->currentAnimFrame = 88;
 	
 	_animator->animRefreshNPC(character);
 }
@@ -424,7 +414,7 @@ int KyraEngine::getOppositeFacingDirection(int dir) {
 	return 0;
 }
 
-void KyraEngine::loadSceneMSC() {
+void KyraEngine::loadSceneMsc() {
 	assert(_currentCharacter->sceneId < _roomTableSize);
 	int tableId = _roomTable[_currentCharacter->sceneId].nameIndex;
 	assert(tableId < _roomFilenameTableSize);
@@ -443,28 +433,28 @@ void KyraEngine::startSceneScript(int brandonAlive) {
 	char fileNameBuffer[32];
 	strcpy(fileNameBuffer, _roomFilenameTable[tableId]);
 	strcat(fileNameBuffer, ".CPS");
-	_screen->loadBitmap(fileNameBuffer, 3, 3, 0);
+	_screen->clearPage(3);
+	// FIXME: check this hack for amiga version
+	_screen->loadBitmap(fileNameBuffer, 3, 3, (_flags.platform == Common::kPlatformAmiga ? _screen->getPalette(0) : 0));
 	_sprites->loadSceneShapes();
 	_exitListPtr = 0;
 	
 	_scaleMode = 1;	
-	for (int i = 0; i < 145; ++i) {
+	for (int i = 0; i < 145; ++i)
 		_scaleTable[i] = 256;
-	}
 	
 	clearNoDropRects();
 	_scriptInterpreter->initScript(_scriptClick, _scriptClickData);
 	strcpy(fileNameBuffer, _roomFilenameTable[tableId]);
 	strcat(fileNameBuffer, ".EMC");
 	_scriptInterpreter->unloadScript(_scriptClickData);
-	_scriptInterpreter->loadScript(fileNameBuffer, _scriptClickData, _opcodeTable, _opcodeTableSize, 0);
+	_scriptInterpreter->loadScript(fileNameBuffer, _scriptClickData, &_opcodes);
 	_scriptInterpreter->startScript(_scriptClick, 0);
 	_scriptClick->variables[0] = _currentCharacter->sceneId;
 	_scriptClick->variables[7] = brandonAlive;
 	
-	while (_scriptInterpreter->validScript(_scriptClick)) {
+	while (_scriptInterpreter->validScript(_scriptClick))
 		_scriptInterpreter->runScript(_scriptClick);
-	}
 }
 
 void KyraEngine::initSceneData(int facing, int unk1, int brandonAlive) {
@@ -505,26 +495,20 @@ void KyraEngine::initSceneData(int facing, int unk1, int brandonAlive) {
 			break;
 		}
 		
-		if ((uint8)(_northExitHeight & 0xFF) + 2 >= ypos) {
+		if ((uint8)(_northExitHeight & 0xFF) + 2 >= ypos)
 			ypos = (_northExitHeight & 0xFF) + 4;
-		}
-		if (xpos >= 308) {
+		if (xpos >= 308)
 			xpos = 304;
-		}
-		if ((uint8)(_northExitHeight >> 8) - 2 <= ypos) {
+		if ((uint8)(_northExitHeight >> 8) - 2 <= ypos)
 			ypos = (_northExitHeight >> 8) - 4;
-		}
-		if (xpos <= 12) {
+		if (xpos <= 12)
 			xpos = 16;
-		}
 	}
 	
-	if (_brandonPosX > -1) {
+	if (_brandonPosX > -1)
 		xpos = _brandonPosX;
-	}
-	if (_brandonPosY > -1) {
+	if (_brandonPosY > -1)
 		ypos = _brandonPosY;
-	}
 	
 	int16 ypos2 = 0;
 	if (_brandonPosX > -1 && _brandonPosY > -1) {
@@ -633,34 +617,32 @@ void KyraEngine::initSceneData(int facing, int unk1, int brandonAlive) {
 	
 	initSceneObjectList(brandonAlive);
 	
-	if (unk1 && brandonAlive == 0) {
+	if (unk1 && brandonAlive == 0)
 		moveCharacterToPos(0, facing, xpos2, ypos2);
-	}
 	
 	_scriptClick->variables[4] = _itemInHand;
 	_scriptClick->variables[7] = brandonAlive;
 	_scriptInterpreter->startScript(_scriptClick, 3);
-	while (_scriptInterpreter->validScript(_scriptClick)) {
+	while (_scriptInterpreter->validScript(_scriptClick))
 		_scriptInterpreter->runScript(_scriptClick);
-	}
 }
 
 void KyraEngine::initSceneObjectList(int brandonAlive) {
 	debugC(9, kDebugLevelMain, "KyraEngine::initSceneObjectList(%d)", brandonAlive);
-	for (int i = 0; i < 28; ++i) {
+	for (int i = 0; i < 28; ++i)
 		_animator->actors()[i].active = 0;
-	}
 	
 	int startAnimFrame = 0;
 	
 	AnimObject *curAnimState = _animator->actors();
 	curAnimState->active = 1;
 	curAnimState->drawY = _currentCharacter->y1;
-	curAnimState->sceneAnimPtr = _shapes[4+_currentCharacter->currentAnimFrame];
+	curAnimState->sceneAnimPtr = _shapes[_currentCharacter->currentAnimFrame];
 	curAnimState->animFrameNumber = _currentCharacter->currentAnimFrame;
 	startAnimFrame = _currentCharacter->currentAnimFrame-7;
 	int xOffset = _defaultShapeTable[startAnimFrame].xOffset;
 	int yOffset = _defaultShapeTable[startAnimFrame].yOffset;
+
 	if (_scaleMode) {
 		curAnimState->x1 = _currentCharacter->x1;
 		curAnimState->y1 = _currentCharacter->y1;
@@ -674,6 +656,7 @@ void KyraEngine::initSceneObjectList(int brandonAlive) {
 		curAnimState->x1 = _currentCharacter->x1 + xOffset;
 		curAnimState->y1 = _currentCharacter->y1 + yOffset;
 	}
+
 	curAnimState->x2 = curAnimState->x1;
 	curAnimState->y2 = curAnimState->y1;
 	curAnimState->refreshFlag = 1;
@@ -696,7 +679,7 @@ void KyraEngine::initSceneObjectList(int brandonAlive) {
 		}
 		
 		curAnimState->drawY = ch->y1;
-		curAnimState->sceneAnimPtr = _shapes[4+ch->currentAnimFrame];
+		curAnimState->sceneAnimPtr = _shapes[ch->currentAnimFrame];
 		curAnimState->animFrameNumber = ch->currentAnimFrame;
 		startAnimFrame = ch->currentAnimFrame-7;
 		xOffset = _defaultShapeTable[startAnimFrame].xOffset;
@@ -720,11 +703,10 @@ void KyraEngine::initSceneObjectList(int brandonAlive) {
 		curAnimState->refreshFlag = 1;
 		curAnimState->bkgdChangeFlag = 1;
 		
-		if (ch->facing >= 1 && ch->facing <= 3) {
+		if (ch->facing >= 1 && ch->facing <= 3)
 			curAnimState->flags |= 1;
-		} else if (ch->facing >= 5 && ch->facing <= 7) {
+		else if (ch->facing >= 5 && ch->facing <= 7)
 			curAnimState->flags &= 0xFFFFFFFE;
-		}
 		
 		_animator->addObjectToQueue(curAnimState);
 		
@@ -775,7 +757,7 @@ void KyraEngine::initSceneObjectList(int brandonAlive) {
 		byte curItem = curRoom->itemsTable[i];
 		if (curItem != 0xFF) {
 			curAnimState->drawY = curRoom->itemsYPos[i];
-			curAnimState->sceneAnimPtr = _shapes[220+curItem];
+			curAnimState->sceneAnimPtr = _shapes[216+curItem];
 			curAnimState->animFrameNumber = (int16)0xFFFF;
 			curAnimState->y1 = curRoom->itemsYPos[i];
 			curAnimState->x1 = curRoom->itemsXPos[i];
@@ -817,20 +799,53 @@ void KyraEngine::initSceneObjectList(int brandonAlive) {
 }
 
 void KyraEngine::initSceneScreen(int brandonAlive) {
-	// XXX Palette stuff
-	if (_unkScreenVar2 == 1) {
-		_screen->shuffleScreen(8, 8, 304, 128, 2, 0, _unkScreenVar3, false);
+	if (_flags.platform == Common::kPlatformAmiga) {
+		if (_unkScreenVar1 && !queryGameFlag(0xF0)) {
+			memset(_screen->getPalette(2), 0, 32*3);
+			if (_currentCharacter->sceneId != 117 || !queryGameFlag(0xB3))
+				_screen->setScreenPalette(_screen->getPalette(2));
+		}
+
+		if (_unkScreenVar2 == 1)
+			_screen->shuffleScreen(8, 8, 304, 128, 2, 0, _unkScreenVar3, false);
+		else
+			_screen->copyRegion(8, 8, 8, 8, 304, 128, 2, 0);
+
+		if (_unkScreenVar1 && !queryGameFlag(0xA0)) {
+			if (_currentCharacter->sceneId == 45 && _paletteChanged)
+				memcpy(_screen->getPalette(0) + 12*3, _screen->getPalette(4) + 12*3, 2);
+
+			if (_currentCharacter->sceneId >= 229 && _currentCharacter->sceneId <= 245 && (_brandonStatusBit & 1))
+				memcpy(_screen->getPalette(0), _screen->getPalette(0) + 320*3, 64);
+
+			_screen->setScreenPalette(_screen->getPalette(0));
+		}
 	} else {
-		_screen->copyRegion(8, 8, 8, 8, 304, 128, 2, 0);
-	}
-	if (_unkScreenVar1 && _paletteChanged) {
-		if (!queryGameFlag(0xA0)) {
-			// XXX Palette stuff
-			_screen->setScreenPalette(_screen->_currentPalette);
-		} else {
-			// XXX Palette stuff
+		if (_unkScreenVar1 && !queryGameFlag(0xA0)) {
+			for (int i = 0; i < 60; ++i) {
+				uint16 col = _screen->getPalette(0)[684+i];
+				col += _screen->getPalette(1)[684+i] << 1;
+				col >>= 2;
+				_screen->getPalette(0)[684+i] = col;
+			}
+			_screen->setScreenPalette(_screen->getPalette(0));
+		}
+
+		if (_unkScreenVar2 == 1)
+			_screen->shuffleScreen(8, 8, 304, 128, 2, 0, _unkScreenVar3, false);
+		else
+			_screen->copyRegion(8, 8, 8, 8, 304, 128, 2, 0);
+
+		if (_unkScreenVar1 && _paletteChanged) {
+			if (!queryGameFlag(0xA0)) {
+				memcpy(_screen->getPalette(0) + 684, _screen->getPalette(1) + 684, 60);
+				_screen->setScreenPalette(_screen->getPalette(0));
+			} else {
+				memset(_screen->getPalette(0), 0, 768);
+			}
 		}
 	}
+
 	// really call this here?
 	_screen->updateScreen();
 
@@ -858,18 +873,19 @@ void KyraEngine::initSceneScreen(int brandonAlive) {
 
 int KyraEngine::handleSceneChange(int xpos, int ypos, int unk1, int frameReset) {
 	debugC(9, kDebugLevelMain, "KyraEngine::handleSceneChange(%d, %d, %d, %d)", xpos, ypos, unk1, frameReset);
-	if (queryGameFlag(0xEF)) {
+	if (queryGameFlag(0xEF))
 		unk1 = 0;
-	}
+
 	int sceneId = _currentCharacter->sceneId;
 	_pathfinderFlag = 0;
+
 	if (xpos < 12) {
 		if (_roomTable[sceneId].westExit != 0xFFFF) {
 			xpos = 12;
 			ypos = _sceneExits.westYPos;
 			_pathfinderFlag = 7;
 		}
-	} else if(xpos >= 308) {
+	} else if (xpos >= 308) {
 		if (_roomTable[sceneId].eastExit != 0xFFFF) {
 			xpos = 307;
 			ypos = _sceneExits.eastYPos;
@@ -894,31 +910,32 @@ int KyraEngine::handleSceneChange(int xpos, int ypos, int unk1, int frameReset)
 	int temp = xpos - _currentCharacter->x1;
 	if (ABS(temp) < 4) {
 		temp = ypos - _currentCharacter->y1;
-		if (ABS(temp) < 2) {
+		if (ABS(temp) < 2)
 			return 0;
-		}
 	}
 	
 	int x = (int16)(_currentCharacter->x1 & 0xFFFC);
 	int y = (int16)(_currentCharacter->y1 & 0xFFFE);
 	xpos = (int16)(xpos & 0xFFFC);
 	ypos = (int16)(ypos & 0xFFFE);
+	
 	int ret = findWay(x, y, xpos, ypos, _movFacingTable, 150);
 	_pathfinderFlag = 0;
-	if (ret >= _lastFindWayRet) {
+
+	if (ret >= _lastFindWayRet)
 		_lastFindWayRet = ret;
-	}
-	if (ret == 0x7D00 || ret == 0) {
+
+	if (ret == 0x7D00 || ret == 0)
 		return 0;
-	}
+
 	return processSceneChange(_movFacingTable, unk1, frameReset);
 }
 
 int KyraEngine::processSceneChange(int *table, int unk1, int frameReset) {
 	debugC(9, kDebugLevelMain, "KyraEngine::processSceneChange(%p, %d, %d)", (const void *)table, unk1, frameReset);
-	if (queryGameFlag(0xEF)) {
+	if (queryGameFlag(0xEF))
 		unk1 = 0;
-	}
+
 	int *tableStart = table;
 	_sceneChangeState = 0;
 	_loopFlag2 = 0;
@@ -934,7 +951,7 @@ int KyraEngine::processSceneChange(int *table, int unk1, int frameReset) {
 			_currentCharacter->currentAnimFrame = 7;
 			_animator->animRefreshNPC(0);
 			_animator->updateAllObjectShapes();
-			processInput(_mouseX, _mouseY);
+			processInput();
 			return 0;
 		}
 		bool forceContinue = false;
@@ -969,23 +986,22 @@ int KyraEngine::processSceneChange(int *table, int unk1, int frameReset) {
 			}
 		}
 		
-		if (forceContinue || !running) {
+		if (forceContinue || !running)
 			continue;
-		}
 		
 		int temp = 0;
-		if (table == tableStart || table[1] == 8) {
+		if (table == tableStart || table[1] == 8)
 			temp = setCharacterPosition(0, 0);
-		} else {
+		else
 			temp = setCharacterPosition(0, table);
-		}
-		if (temp) {
+
+		if (temp)
 			++table;
-		}
 		
 		nextFrame = getTimerDelay(5) * _tickLength + _system->getMillis();
 		while (_system->getMillis() < nextFrame) {
 			updateGameTimers();
+
 			if (_currentCharacter->sceneId == 210) {
 				updateKyragemFading();
 				if (seq_playEnd() || _beadStateVar == 4 || _beadStateVar == 5) {
@@ -994,14 +1010,15 @@ int KyraEngine::processSceneChange(int *table, int unk1, int frameReset) {
 					break;
 				}
 			}
+
 			if ((nextFrame - _system->getMillis()) >= 10)
 				delay(10, true);
 		}
 	}
 	
-	if (frameReset && !(_brandonStatusBit & 2)) {
+	if (frameReset && !(_brandonStatusBit & 2))
 		_currentCharacter->currentAnimFrame = 7;
-	}
+
 	_animator->animRefreshNPC(0);
 	_animator->updateAllObjectShapes();
 	return returnValue;
@@ -1010,9 +1027,8 @@ int KyraEngine::processSceneChange(int *table, int unk1, int frameReset) {
 int KyraEngine::changeScene(int facing) {
 	debugC(9, kDebugLevelMain, "KyraEngine::changeScene(%d)", facing);
 	if (queryGameFlag(0xEF)) {
-		if (_currentCharacter->sceneId == 5) {
+		if (_currentCharacter->sceneId == 5)
 			return 0;
-		}
 	}
 	
 	int xpos = _charXPosTable[facing] + _currentCharacter->x1;
@@ -1025,7 +1041,7 @@ int KyraEngine::changeScene(int facing) {
 	
 	if (_exitListPtr) {
 		int16 *ptr = _exitListPtr;
-		// this loop should be only entered on time, seems to be some hack in the original
+		// this loop should be only entered one time, seems to be some hack in the original
 		while (true) {
 			if (*ptr == -1)
 				break;
@@ -1034,12 +1050,14 @@ int KyraEngine::changeScene(int facing) {
 				ptr += 10;
 				break;
 			}
+
 			_brandonPosX = ptr[6];
 			_brandonPosY = ptr[7];
 			uint16 sceneId = ptr[5];
 			facing = ptr[4];
 			int unk1 = ptr[8];
 			int unk2 = ptr[9];
+
 			if (sceneId == 0xFFFF) {
 				switch (facing) {
 				case 0:
@@ -1135,12 +1153,14 @@ void KyraEngine::setCharactersInDefaultScene() {
 	for (int i = 1; i < 5; ++i) {
 		Character *cur = &_characterList[i];
 		//cur->field_20 = 0;
+
 		const uint32 *curTable = defaultSceneTable[i-1];
 		cur->sceneId = curTable[0];
-		if (cur->sceneId == _currentCharacter->sceneId) {
+
+		if (cur->sceneId == _currentCharacter->sceneId)
 			//++cur->field_20;
 			cur->sceneId = curTable[1/*cur->field_20*/];
-		}
+
 		//cur->field_23 = curTable[cur->field_20+1];
 	}
 }
@@ -1155,6 +1175,7 @@ void KyraEngine::setCharactersPositions(int character) {
 		0x67, 0x67, 0x60, 0x5A, 0x71,
 		0x76
 	};
+
 	assert(character < ARRAYSIZE(initXPosTable));
 	Character *edit = &_characterList[character];
 	edit->x1 = edit->x2 = initXPosTable[character];
@@ -1247,9 +1268,8 @@ int KyraEngine::findWay(int x, int y, int toX, int toY, int *moveTable, int move
 				}
 			}
 			
-			if (temp != 0x7D00 || tempValue != 0x7D00) {
+			if (temp != 0x7D00 || tempValue != 0x7D00)
 				break;
-			}
 		}
 		
 		if (temp < tempValue) {
@@ -1271,10 +1291,10 @@ int KyraEngine::findWay(int x, int y, int toX, int toY, int *moveTable, int move
 		}
 		x = curX;
 		y = curY;
-		if (curX == toX && curY == toY) {
+		if (curX == toX && curY == toY)
 			break;
-		}
 	}
+
 	delete [] pathTable1;
 	delete [] pathTable2;
 	moveTable[lastUsedEntry] = 8;
@@ -1307,9 +1327,8 @@ int KyraEngine::findSubPath(int x, int y, int toX, int toY, int *moveTable, int
 		while (true) {
 			changePosTowardsFacing(xpos1, ypos1, facingTable1[start*8 + newFacing2]);
 			if (!lineIsPassable(xpos1, ypos1)) {
-				if (facingTable1[start*8 + newFacing2] == newFacing) {
+				if (facingTable1[start*8 + newFacing2] == newFacing)
 					return 0x7D00;
-				}
 				newFacing2 = facingTable1[start*8 + newFacing2];
 				xpos1 = x;
 				ypos1 = y;
@@ -1334,19 +1353,20 @@ int KyraEngine::findSubPath(int x, int y, int toX, int toY, int *moveTable, int
 				}
 			}
 		}
+
 		moveTable[position++] = newFacing;
 		x = xpos1;
 		y = ypos1;
-		if (x == toX && y == toY) {
+
+		if (x == toX && y == toY)
 			return position;
-		}
 		
-		if (xpos1 == xpos2 && ypos1 == ypos2) {
+		if (xpos1 == xpos2 && ypos1 == ypos2)
 			break;
-		}
 		
 		newFacing = facingTable3[start*8 + newFacing];
 	}
+
 	return 0x7D00;
 }
 
@@ -1388,6 +1408,7 @@ int KyraEngine::getFacingFromPointToPoint(int x, int y, int toX, int toY) {
 	} else {
 		facingEntry <<= 1;
 	}
+
 	assert(facingEntry < ARRAYSIZE(facingTable));
 	return facingTable[facingEntry];
 }
@@ -1427,13 +1448,11 @@ bool KyraEngine::lineIsPassable(int x, int y) {
 			return true;
 	}
 	
-	if (y > 137) {
+	if (y > 137)
 		return false;
-	}
 
-	if (y < 0) {
+	if (y < 0)
 		y = 0;
-	}
 	
 	int ypos = 8;
 	if (_scaleMode) {
@@ -1497,19 +1516,19 @@ int KyraEngine::getMoveTableSize(int *moveTable) {
 			}
 			
 			if (tempPosition == moveTable && *tempPosition == 9) {
-				while (*tempPosition != 8 && *tempPosition == 9) {
+				while (*tempPosition != 8 && *tempPosition == 9)
 					++tempPosition;
-				}
-				if (*tempPosition == 8) {
+
+				if (*tempPosition == 8)
 					return 0;
-				}
 			}
 			
 			oldPosition = tempPosition;
 			curPosition = oldPosition+1;
-			while (*curPosition != 8 && *curPosition == 9) {
+
+			while (*curPosition != 8 && *curPosition == 9)
 				++curPosition;
-			}
+
 			continue;
 		}
 		
@@ -1522,20 +1541,19 @@ int KyraEngine::getMoveTableSize(int *moveTable) {
 				curPosition = oldPosition;
 				oldPosition = tempPosition;
 				while (true) {
-					if (tempPosition == moveTable) {
+					if (tempPosition == moveTable)
 						break;
-					}
+
 					--tempPosition;
-					if (*tempPosition != 9) {
+					if (*tempPosition != 9)
 						break;
-					}
+
 				}
 			} else {
 				while (true) {
 					++curPosition;
-					if (*curPosition != 9) {
+					if (*curPosition != 9)
 						break;
-					}
 				}
 			}
 			continue;
@@ -1544,11 +1562,11 @@ int KyraEngine::getMoveTableSize(int *moveTable) {
 		tempPosition = oldPosition;
 		oldPosition = curPosition;
 		++retValue;
+
 		while (true) {
 			++curPosition;
-			if (*curPosition != 9) {
+			if (*curPosition != 9)
 				break;
-			}
 		}
 	}
 
@@ -1557,7 +1575,7 @@ int KyraEngine::getMoveTableSize(int *moveTable) {
 
 void KyraEngine::setupSceneResource(int sceneId) {
 	debugC(9, kDebugLevelMain, "KyraEngine::setupSceneResource(%d)", sceneId);
-	if (_features & GF_FLOPPY)
+	if (!_flags.isTalkie)
 		return;
 
 	if (_currentRoom != 0xFFFF) {
@@ -1570,28 +1588,32 @@ void KyraEngine::setupSceneResource(int sceneId) {
 		strcpy(file, _roomFilenameTable[tableId]);
 		strcat(file, ".VRM");
 		_res->unloadPakFile(file);
+
 		strcpy(file, _roomFilenameTable[tableId]);
 		strcat(file, ".PAK");
-		_res->unloadPakFile(file);	
+		_res->unloadPakFile(file);
+
 		strcpy(file, _roomFilenameTable[tableId]);
 		strcat(file, ".APK");
 		_res->unloadPakFile(file);
 	}
 
-   	assert(sceneId < _roomTableSize);
+	assert(sceneId < _roomTableSize);
 	int tableId = _roomTable[sceneId].nameIndex;
 	assert(tableId < _roomFilenameTableSize);
 
-   	// load our new room
-   	char file[64];
+	// load our new room
+	char file[64];
 	strcpy(file, _roomFilenameTable[tableId]);
 	strcat(file, ".VRM");
 	if (Common::File::exists(file))
 		_res->loadPakFile(file);
+
 	strcpy(file, _roomFilenameTable[tableId]);
 	strcat(file, ".PAK");
 	if (Common::File::exists(file))
 		_res->loadPakFile(file);
+
 	strcpy(file, _roomFilenameTable[tableId]);
 	strcat(file, ".APK");
 	if (Common::File::exists(file))
@@ -1599,3 +1621,4 @@ void KyraEngine::setupSceneResource(int sceneId) {
 }
 
 } // end of namespace Kyra
+
