@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2004-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,13 +18,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/kyra/screen.h $
- * $Id: screen.h 23906 2006-09-17 09:54:42Z lordhoto $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/kyra/screen.h $
+ * $Id: screen.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
-#ifndef KYRASCREEN_H
-#define KYRASCREEN_H
+#ifndef KYRA_SCREEN_H
+#define KYRA_SCREEN_H
 
 #include "common/util.h"
 
@@ -61,7 +64,9 @@ public:
 		SCREEN_W = 320,
 		SCREEN_H = 200,
 		SCREEN_PAGE_SIZE = 320 * 200 + 1024,
-		SCREEN_PAGE_NUM  = 16
+		SCREEN_OVL_SJIS_SIZE = 640 * 400,
+		SCREEN_PAGE_NUM = 16,
+		SCREEN_OVLS_NUM = 3
 	};
 
 	enum CopyRegionFlags {
@@ -83,6 +88,8 @@ public:
 		FID_8_FNT,
 		FID_CRED6_FNT,
 		FID_CRED8_FNT,
+		FID_BOOKFONT_FNT,
+		FID_GOLDFONT_FNT,
 		FID_NUM
 	};
 	
@@ -92,68 +99,104 @@ public:
 	bool init();
 
 	void updateScreen();
-	const uint8 *getCPagePtr(int pageNum) const;
-	uint8 *getPageRect(int pageNum, int x, int y, int w, int h);
-	void clearPage(int pageNum);
+
+	// page cur. functions
 	int setCurPage(int pageNum);
-	void clearCurPage();
-	uint8 getPagePixel(int pageNum, int x, int y);
-	void setPagePixel(int pageNum, int x, int y, uint8 color);
-	void fadeFromBlack();
-	void fadeToBlack();
-	void fadeSpecialPalette(int palIndex, int startIndex, int size, int fadeTime);
-	void fadePalette(const uint8 *palData, int delay);
-	void setPaletteIndex(uint8 index, uint8 red, uint8 green, uint8 blue);
-	void setScreenPalette(const uint8 *palData);
-	void copyToPage0(int y, int h, uint8 page, uint8 *seqBuf);
-	void copyRegion(int x1, int y1, int x2, int y2, int w, int h, int srcPage, int dstPage, int flags=0);
-	void copyBlockToPage(int pageNum, int x, int y, int w, int h, const uint8 *src);
+
 	void copyFromCurPageBlock(int x, int y, int w, int h, const uint8 *src);
 	void copyCurPageBlock(int x, int y, int w, int h, uint8 *dst);
+
+	void clearCurPage();
+
+	// page 0 functions
+	void copyToPage0(int y, int h, uint8 page, uint8 *seqBuf);
+	void shakeScreen(int times);
+
+	// page functions
+	void copyRegion(int x1, int y1, int x2, int y2, int w, int h, int srcPage, int dstPage, int flags=0);
+	void copyPage(uint8 srcPage, uint8 dstPage);
+
+	void copyRegionToBuffer(int pageNum, int x, int y, int w, int h, uint8 *dest);
+	void copyBlockToPage(int pageNum, int x, int y, int w, int h, const uint8 *src);
+
 	void shuffleScreen(int sx, int sy, int w, int h, int srcPage, int dstPage, int ticks, bool transparent);
 	void fillRect(int x1, int y1, int x2, int y2, uint8 color, int pageNum = -1);
+
+	void clearPage(int pageNum);
+
+	uint8 getPagePixel(int pageNum, int x, int y);
+	void setPagePixel(int pageNum, int x, int y, uint8 color);
+
+	const uint8 *getCPagePtr(int pageNum) const;
+	uint8 *getPageRect(int pageNum, int x, int y, int w, int h);
+
+	// palette handling
+	void fadeFromBlack(int delay=0x54);
+	void fadeToBlack(int delay=0x54);
+
+	void fadeSpecialPalette(int palIndex, int startIndex, int size, int fadeTime);
+	void fadePalette(const uint8 *palData, int delay);
+
+	void setPaletteIndex(uint8 index, uint8 red, uint8 green, uint8 blue);
+	void setScreenPalette(const uint8 *palData);
+	uint8 *getPalette(int num);
+
+	// gui specific (processing on _curPage)
 	void drawLine(bool vertical, int x, int y, int length, int color);
 	void drawClippedLine(int x1, int y1, int x2, int y2, int color);
 	void drawShadedBox(int x1, int y1, int x2, int y2, int color1, int color2);
 	void drawBox(int x1, int y1, int x2, int y2, int color);
-	void setAnimBlockPtr(int size);
-	void setTextColorMap(const uint8 *cmap);
-	void setTextColor(const uint8 *cmap, int a, int b);
-	void loadFont(FontId fontId, uint8 *fontData);
+
+	// font/text handling
+	bool loadFont(FontId fontId, const char *filename);
 	FontId setFont(FontId fontId);
+
 	int getFontHeight() const;
 	int getFontWidth() const;
-	int getCharWidth(uint8 c) const;
+
+	int getCharWidth(uint16 c) const;
 	int getTextWidth(const char *str) const;
+
 	void printText(const char *str, int x, int y, uint8 color1, uint8 color2);
-	void drawChar(uint8 c, int x, int y);
+
+	void setTextColorMap(const uint8 *cmap);
+	void setTextColor(const uint8 *cmap, int a, int b);
+
 	void setScreenDim(int dim);
-	void drawShapePlotPixelCallback1(uint8 *dst, uint8 color);
-	void drawShape(uint8 pageNum, const uint8 *shapeData, int x, int y, int sd, int flags, ...);
-	static void decodeFrame3(const uint8 *src, uint8 *dst, uint32 size);
-	static void decodeFrame4(const uint8 *src, uint8 *dst, uint32 dstSize);
-	static void decodeFrameDelta(uint8 *dst, const uint8 *src);
-	static void decodeFrameDeltaPage(uint8 *dst, const uint8 *src, int pitch, int noXor);
+
+	// shape handling
 	uint8 *encodeShape(int x, int y, int w, int h, int flags);
-	void copyRegionToBuffer(int pageNum, int x, int y, int w, int h, uint8 *dest);
-	void loadBitmap(const char *filename, int tempPage, int dstPage, uint8 *palData);
 
-	void shakeScreen(int times);
+	int setNewShapeHeight(uint8 *shape, int height);
+	int resetShapeHeight(uint8 *shape);
 
-	int getRectSize(int x, int y);
+	void drawShape(uint8 pageNum, const uint8 *shapeData, int x, int y, int sd, int flags, ...);
+
+	// mouse handling
 	void hideMouse();
 	void showMouse();
+	void setMouseCursor(int x, int y, byte *shape);	
+
+	// rect handling
+	int getRectSize(int w, int h);
+
+	void rectClip(int &x, int &y, int w, int h);
+
+	void addBitBlitRect(int x, int y, int w, int h);
+	void bitBlitRects();
+
+	// misc	
+	void loadBitmap(const char *filename, int tempPage, int dstPage, uint8 *palData);
+
+	void loadPalette(const char *filename, uint8 *palData);
+	void loadPalette(const byte *data, uint8 *palData, int bytes);
+
+	void setAnimBlockPtr(int size);
+
 	void setShapePages(int page1, int page2);
-	void setMouseCursor(int x, int y, byte *shape);
-	uint8 *getPalette(int num);
 	
 	byte getShapeFlag1(int x, int y);
 	byte getShapeFlag2(int x, int y);
-	int setNewShapeHeight(uint8 *shape, int height);
-	int resetShapeHeight(uint8 *shape);
-	
-	void addBitBlitRect(int x, int y, int w, int h);
-	void bitBlitRects();
 	
 	void savePageToDisk(const char *file, int page);
 	void loadPageFromDisk(const char *file, int page);
@@ -162,13 +205,9 @@ public:
 	void blockInRegion(int x, int y, int width, int height);
 	void blockOutRegion(int x, int y, int width, int height);
 
-	void backUpRect0(int xpos, int ypos);
-	void restoreRect0(int xpos, int ypos);
-	void backUpRect1(int xpos, int ypos);
-	void restoreRect1(int xpos, int ypos);
 	void copyBackgroundBlock(int x, int page, int flag);
 	void copyBackgroundBlock2(int x);
-	void rectClip(int &x, int &y, int w, int h);
+
 	int getDrawLayer(int x, int y);
 	int getDrawLayer2(int x, int y, int height);
 
@@ -178,17 +217,52 @@ public:
 	uint8 *_currentPalette;
 	uint8 *_shapePages[2];
 	FontId _currentFont;
+	bool _disableScreen;
 
 	const ScreenDim *_curDim;
 	
 	static const ScreenDim _screenDimTable[];
 	static const int _screenDimTableCount;
 
+	// decoding functions
+	static void decodeFrame3(const uint8 *src, uint8 *dst, uint32 size);
+	static uint decodeFrame4(const uint8 *src, uint8 *dst, uint32 dstSize);
+	static void decodeFrameDelta(uint8 *dst, const uint8 *src, bool noXor = false);
+	static void decodeFrameDeltaPage(uint8 *dst, const uint8 *src, const int pitch, bool noXor);
+	static void convertAmigaGfx(uint8 *data, int w, int h, bool offscreen = true);
+	static void convertAmigaMsc(uint8 *data);
+
 	// maybe subclass screen for kyra3
 	static const ScreenDim _screenDimTableK3[];
 	static const int _screenDimTableCountK3;
+
+	uint8 *getPtrToShape(uint8 *shpFile, int shape);
+	const uint8 *getPtrToShape(const uint8 *shpFile, int shape);
+
+	uint16 getShapeSize(const uint8 *shp);
+
 private:
 	uint8 *getPagePtr(int pageNum);
+	void updateDirtyRects();
+	void updateDirtyRectsOvl();
+
+	void scale2x(byte *dst, int dstPitch, const byte *src, int srcPitch, int w, int h);
+	void mergeOverlay(int x, int y, int w, int h);
+
+	// overlay specific
+	byte *getOverlayPtr(int pageNum);
+	void clearOverlayPage(int pageNum);
+	void clearOverlayRect(int pageNum, int x, int y, int w, int h);
+	void copyOverlayRegion(int x, int y, int x2, int y2, int w, int h, int srcPage, int dstPage);
+
+	// font/text specific
+	void drawCharANSI(uint8 c, int x, int y);
+	void drawCharSJIS(uint16 c, int x, int y);
+
+	enum {
+		SJIS_CHARSIZE = 18,
+		SJIS_CHARS = 8192
+	};
 
 	int16 encodeShapeAndCalculateSize(uint8 *from, uint8 *to, int size);
 	void restoreMouseRect();
@@ -196,16 +270,35 @@ private:
 	void copyScreenFromRect(int x, int y, int w, int h, const uint8 *ptr);
 	void copyScreenToRect(int x, int y, int w, int h, uint8 *ptr);
 
+	template<bool noXor> static void wrapped_decodeFrameDelta(uint8 *dst, const uint8 *src);
+	template<bool noXor> static void wrapped_decodeFrameDeltaPage(uint8 *dst, const uint8 *src, const int pitch);
+
 	uint8 *_pagePtrs[16];
+	uint8 *_sjisOverlayPtrs[SCREEN_OVLS_NUM];
+
+	bool _useOverlays;
+	bool _useSJIS;
+
+	uint8 *_sjisFontData;
+	uint8 *_sjisTempPage;
+	uint8 *_sjisTempPage2;
+	uint8 *_sjisSourceChar;
+
 	uint8 *_saveLoadPage[8];
+	uint8 *_saveLoadPageOvl[8];
+
 	uint8 *_screenPalette;
-	uint8 *_palettes[3];
+	uint8 *_palettes[6];
+
 	Font _fonts[FID_NUM];
 	uint8 _textColorsMap[16];
+
 	uint8 *_decodeShapeBuffer;
 	int _decodeShapeBufferSize;
+
 	uint8 *_animBlockPtr;
 	int _animBlockSize;
+
 	int _mouseLockCount;
 	
 	Rect *_bitBlitRects;
@@ -229,3 +322,4 @@ private:
 } // End of namespace Kyra
 
 #endif
+
