@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2004-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,30 +18,55 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/kyra/script.h $
- * $Id: script.h 20450 2006-02-09 22:50:59Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/kyra/script.h $
+ * $Id: script.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
-#ifndef KYRASCRIPT_H
-#define KYRASCRIPT_H
+#ifndef KYRA_SCRIPT_H
+#define KYRA_SCRIPT_H
 
 #include "kyra/kyra.h"
 
 namespace Kyra {
+
+struct ScriptState;
+
+struct Opcode {
+	virtual ~Opcode() {}
+
+	virtual operator bool() const = 0;
+
+	virtual int operator()(ScriptState*) const = 0;
+};
+
+template<class T>
+struct OpcodeImpl : public Opcode {
+	T *vm;
+	typedef int (T::*Callback)(ScriptState*);
+	Callback callback;
+
+	OpcodeImpl(T *v, Callback c) : Opcode(), vm(v), callback(c) {} 
+
+	operator bool() const { return callback != 0; }
+
+	int operator()(ScriptState *state) const {
+		return (vm->*callback)(state);
+	}
+};
+
 struct ScriptData {
 	byte *text;
-	byte *data;
-	byte *ordr;
+	uint16 *data;
+	uint16 *ordr;
 	uint16 dataSize;
-	KyraEngine::OpcodeProc *opcodes;
-	int opcodeSize;
-	uint16 mustBeFreed;
+
+	const Common::Array<const Opcode*> *opcodes;
 };
 
 struct ScriptState {
-	byte *ip;
-	ScriptData *dataPtr;
+	uint16 *ip;
+	const ScriptData *dataPtr;
 	int16 retValue;
 	uint16 bp;
 	uint16 sp;
@@ -46,28 +74,23 @@ struct ScriptState {
 	int16 stack[61];
 };
 
-enum {
-	SCRIPT_INIT = 0
-};
-
 class ScriptHelper {
 public:
 	ScriptHelper(KyraEngine *vm);
-	virtual ~ScriptHelper();
 	
-	bool loadScript(const char *filename, ScriptData *data, KyraEngine::OpcodeProc *opcodes, int opcodeSize, byte *specialPtr = 0);
+	bool loadScript(const char *filename, ScriptData *data, const Common::Array<const Opcode*> *opcodes);
 	void unloadScript(ScriptData *data);
 	
-	void initScript(ScriptState *scriptState, ScriptData *data);
+	void initScript(ScriptState *scriptState, const ScriptData *data);
 	bool startScript(ScriptState *script, int function);
 	
 	bool validScript(ScriptState *script);
 	
 	bool runScript(ScriptState *script);
 protected:
-	uint32 getFORMBlockSize(byte *&data) const;
-	uint32 getIFFBlockSize(byte *start, byte *&data, uint32 maxSize, const uint32 chunk) const;
-	bool loadIFFBlock(byte *start, byte *&data, uint32 maxSize, const uint32 chunk, byte *loadTo, uint32 ptrSize) const;
+	uint32 getFORMBlockSize(const byte *&data) const;
+	uint32 getIFFBlockSize(const byte *start, const byte *&data, uint32 maxSize, const uint32 chunk) const;
+	bool loadIFFBlock(const byte *start, const byte *&data, uint32 maxSize, const uint32 chunk, void *loadTo, uint32 ptrSize) const;
 	
 	KyraEngine *_vm;
 	int16 _parameter;
@@ -104,3 +127,4 @@ private:
 } // end of namespace Kyra
 
 #endif
+
