@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2004-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,15 +18,15 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/kyra/seqplayer.cpp $
- * $Id: seqplayer.cpp 23595 2006-07-26 05:29:24Z lordhoto $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/kyra/seqplayer.cpp $
+ * $Id: seqplayer.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
 #include "common/stdafx.h"
 #include "common/system.h"
 
-#include "base/engine.h"
+#include "engines/engine.h"
 
 #include "kyra/kyra.h"
 #include "kyra/resource.h"
@@ -68,43 +71,57 @@ SeqPlayer::~SeqPlayer() {
 }
 
 uint8 *SeqPlayer::setPanPages(int pageNum, int shape) {
-	debugC( 9, kDebugLevelSequence, "SeqPlayer::setPanPages(%d, %d)", pageNum, shape);
+	debugC(9, kDebugLevelSequence, "SeqPlayer::setPanPages(%d, %d)", pageNum, shape);
 	uint8 *panPage = 0;
 	const uint8 *data = _screen->getCPagePtr(pageNum);
 	uint16 numShapes = READ_LE_UINT16(data);
 	if (shape < numShapes) {
 		uint32 offs = 0;
-		if (_vm->features() & GF_TALKIE) {
+		if (_vm->gameFlags().useAltShapeHeader)
 			offs = READ_LE_UINT32(data + 2 + shape * 4);
-		} else {
+		else
 			offs = READ_LE_UINT16(data + 2 + shape * 2);
-		}
+
 		if (offs != 0) {
 			data += offs;
 			uint16 sz = READ_LE_UINT16(data + 6);
-			panPage = (uint8 *)malloc(sz);
-			if (panPage) {
-				memcpy(panPage, data, sz);
-			}
+			panPage = new uint8[sz];
+			assert(panPage);	
+			memcpy(panPage, data, sz);
 		}
 	}
 	return panPage;
 }
 
 void SeqPlayer::makeHandShapes() {
-	debugC( 9, kDebugLevelSequence, "SeqPlayer::makeHandShapes()");
-	_screen->loadBitmap("WRITING.CPS", 3, 3, 0);
-	for (int i = 0; i < ARRAYSIZE(_handShapes); ++i) {
-		if (_handShapes[i])
-			free(_handShapes[i]);
-		_handShapes[i] = setPanPages(3, i);
+	debugC(9, kDebugLevelSequence, "SeqPlayer::makeHandShapes()");
+	_screen->loadBitmap("WRITING.CPS", 3, 3, _screen->_currentPalette);
+	if (_vm->gameFlags().platform == Common::kPlatformMacintosh || _vm->gameFlags().platform == Common::kPlatformAmiga) {
+		freeHandShapes();
+		
+		int pageBackUp = _screen->_curPage;
+		_screen->_curPage = 2;
+		_handShapes[0] = _screen->encodeShape(0, 0, 88, 122, 0);
+		assert(_handShapes[0]);
+		_handShapes[1] = _screen->encodeShape(88, 0, 80, 117, 0);
+		assert(_handShapes[1]);
+		_handShapes[2] = _screen->encodeShape(168, 0, 117, 124, 0);
+		assert(_handShapes[2]);
+		_screen->_curPage = pageBackUp;
+	} else {
+		for (int i = 0; i < ARRAYSIZE(_handShapes); ++i) {
+			if (_handShapes[i])
+				delete [] _handShapes[i];
+			_handShapes[i] = setPanPages(3, i);
+			assert(_handShapes[i]);
+		}
 	}
 }
 
 void SeqPlayer::freeHandShapes() {
-	debugC( 9, kDebugLevelSequence, "SeqPlayer::freeHandShapes()");
+	debugC(9, kDebugLevelSequence, "SeqPlayer::freeHandShapes()");
 	for (int i = 0; i < ARRAYSIZE(_handShapes); ++i) {
-		free(_handShapes[i]);
+		delete [] _handShapes[i];
 		_handShapes[i] = 0;
 	}
 }
@@ -125,9 +142,8 @@ void SeqPlayer::s1_wsaOpen() {
 void SeqPlayer::s1_wsaClose() {
 	uint8 wsaObj = *_seqData++;
 	assert(wsaObj < ARRAYSIZE(_seqMovies));
-	if (_seqMovies[wsaObj].movie) {
+	if (_seqMovies[wsaObj].movie)
 		_seqMovies[wsaObj].movie->close();
-	}
 }
 
 void SeqPlayer::s1_wsaPlayFrame() {
@@ -193,23 +209,21 @@ void SeqPlayer::s1_shuffleScreen() {
 
 void SeqPlayer::s1_copyView() {
 	int y = 128;
-	if (!_copyViewOffs) {
+	if (!_copyViewOffs)
 		y -= 8;
-	}
-	if (_specialBuffer && !_copyViewOffs) {
+
+	if (_specialBuffer && !_copyViewOffs)
 		_screen->copyToPage0(16, y, 3, _specialBuffer);
-	} else {
+	else
 		_screen->copyRegion(0, 16, 0, 16, 320, y, 2, 0);
-	}
 }
 
 void SeqPlayer::s1_loopInit() {
 	uint8 seqLoop = *_seqData++;
-	if (seqLoop < ARRAYSIZE(_seqLoopTable)) {
+	if (seqLoop < ARRAYSIZE(_seqLoopTable))
 		_seqLoopTable[seqLoop].ptr = _seqData;
-	} else {
+	else
 		_seqQuitFlag = true;
-	}
 }
 
 void SeqPlayer::s1_loopInc() {
@@ -234,16 +248,28 @@ void SeqPlayer::s1_skip() {
 
 void SeqPlayer::s1_loadPalette() {
 	uint8 colNum = *_seqData++;
-	uint32 fileSize;
-	uint8 *srcData;
-	srcData = _res->fileData(_vm->seqCOLTable()[colNum], &fileSize);
-	memcpy(_screen->_currentPalette, srcData, fileSize);
-	delete[] srcData;
+
+	if (_vm->gameFlags().platform == Common::kPlatformAmiga) {
+		if (!colNum)
+			memcpy(_screen->_currentPalette, _screen->_currentPalette + 576, 3*32);
+		else if (colNum == 3)
+			memcpy(_screen->_currentPalette, _screen->_currentPalette + 672, 3*32);
+		else if (colNum == 4)
+			memcpy(_screen->_currentPalette, _screen->_currentPalette + 288, 3*32);
+
+		_screen->setScreenPalette(_screen->_currentPalette);
+	} else {
+		uint32 fileSize;
+		uint8 *srcData;
+		srcData = _res->fileData(_vm->seqCOLTable()[colNum], &fileSize);
+		memcpy(_screen->_currentPalette, srcData, fileSize);
+		delete [] srcData;
+	}
 }
 
 void SeqPlayer::s1_loadBitmap() {
 	uint8 cpsNum = *_seqData++;
-	_screen->loadBitmap(_vm->seqCPSTable()[cpsNum], 3, 3, 0);
+	_screen->loadBitmap(_vm->seqCPSTable()[cpsNum], 3, 3, _screen->_currentPalette);
 }
 
 void SeqPlayer::s1_fadeToBlack() {
@@ -253,14 +279,17 @@ void SeqPlayer::s1_fadeToBlack() {
 void SeqPlayer::s1_printText() {
 	static const uint8 colorMap[] = { 0, 0, 0, 0, 12, 12, 12, 0, 0, 0, 0, 0 };
 	uint8 txt = *_seqData++;
-	_screen->fillRect(0, 180, 319, 195, 12);
+	if (_vm->gameFlags().platform == Common::kPlatformAmiga)
+		_screen->fillRect(0, 180, 319, 195, 0);
+	else
+		_screen->fillRect(0, 180, 319, 195, 12);
 	_screen->setTextColorMap(colorMap);
 	if (!_seqDisplayTextFlag) {
 		const char *str = _vm->seqTextsTable()[txt];
 		int x = (Screen::SCREEN_W - _screen->getTextWidth(str)) / 2;
 		_screen->printText(str, x, 180, 0xF, 0xC);
 	} else {
-		_seqDisplayedTextTimer = _system->getMillis() + 1000 / ((_vm->features() & GF_FRENCH) ? 120 : 60);
+		_seqDisplayedTextTimer = _system->getMillis() + 1000 / ((_vm->gameFlags().lang == Common::FR_FRA) ? 120 : 60);
 		_seqDisplayedText = txt;
 		_seqDisplayedChar = 0;
 		const char *str = _vm->seqTextsTable()[_seqDisplayedText];
@@ -275,38 +304,38 @@ void SeqPlayer::s1_printTalkText() {
 	uint8 fillColor = *_seqData++;
 	int b;
 	if (_seqTalkTextPrinted && !_seqTalkTextRestored) {
-		if (_seqWsaCurDecodePage != 0 && !_specialBuffer) {
+		if (_seqWsaCurDecodePage != 0 && !_specialBuffer)
 			b = 2;
-		} else {
+		else
 			b = 0;
-		}
 		_vm->text()->restoreTalkTextMessageBkgd(2, b);
 	}
 	_seqTalkTextPrinted = true;
 	_seqTalkTextRestored = false;
-	if (_seqWsaCurDecodePage != 0 && !_specialBuffer) {
+	if (_seqWsaCurDecodePage != 0 && !_specialBuffer)
 		b = 2;
-	} else {
+	else
 		b = 0;
-	}
 	_vm->text()->printTalkTextMessage(_vm->seqTextsTable()[txt], x, y, fillColor, b, 2);
 }
 
 void SeqPlayer::s1_restoreTalkText() {
 	if (_seqTalkTextPrinted && !_seqTalkTextRestored) {
 		int b;
-		if (_seqWsaCurDecodePage != 0 && !_specialBuffer) {
+		if (_seqWsaCurDecodePage != 0 && !_specialBuffer)
 			b = 2;
-		} else {
+		else
 			b = 0;
-		}
 		_vm->text()->restoreTalkTextMessageBkgd(2, b);
 		_seqTalkTextRestored = true;
 	}
 }
 
 void SeqPlayer::s1_clearCurrentScreen() {
-	_screen->fillRect(10, 180, 319, 196, 0xC);
+	if (_vm->gameFlags().platform == Common::kPlatformAmiga)
+		_screen->fillRect(10, 180, 319, 195, 0);
+	else
+		_screen->fillRect(10, 180, 319, 196, 0xC);
 }
 
 void SeqPlayer::s1_break() {
@@ -326,25 +355,37 @@ void SeqPlayer::s1_copyRegion() {
 void SeqPlayer::s1_copyRegionSpecial() {
 	static const uint8 colorMap[] = { 0, 0, 0, 0, 0, 12, 12, 0, 0, 0, 0, 0 };
 	const char *copyStr = 0;
-	if (_vm->features() & GF_FLOPPY || _vm->features() & GF_DEMO) {
+	if (!_vm->gameFlags().isTalkie)
 		copyStr = "Copyright (c) 1992 Westwood Studios";
-	} else if (_vm->features() & GF_TALKIE) {
+	else
 		copyStr = "Copyright (c) 1992,1993 Westwood Studios";
-	}
-	
+
 	uint8 so = *_seqData++;
 	switch (so) {
 	case 0:
-		_screen->copyRegion(0, 0, 0, 47, 320, 77, 2, 0);
+		if (_vm->gameFlags().platform == Common::kPlatformAmiga)
+			_screen->copyRegion(0, 0, 0, 47, 312, 76, 2, 0);
+		else
+			_screen->copyRegion(0, 0, 0, 47, 320, 77, 2, 0);
 		break;
 	case 1:
-		_screen->copyRegion(0, 0, 0, 47, 320, 56, 2, 0);
+		if (_vm->gameFlags().platform == Common::kPlatformAmiga)
+			_screen->copyRegion(0, 0, 8, 47, 312, 55, 2, 0);
+		else
+			_screen->copyRegion(0, 0, 0, 47, 320, 56, 2, 0);
 		break;
 	case 2:
-		_screen->copyRegion(107, 72, 107, 72, 43, 87, 2, 0);
-		_screen->copyRegion(130, 159, 130, 159, 35, 17, 2, 0);
-		_screen->copyRegion(165, 105, 165, 105, 32, 9, 2, 0);
-		_screen->copyRegion(206, 83, 206, 83, 94, 93, 2, 0);
+		if (_vm->gameFlags().platform == Common::kPlatformAmiga) {
+			_screen->copyRegion(104, 72, 104, 72, 40, 87, 2, 0);
+			_screen->copyRegion(128, 159, 128, 159, 32, 17, 2, 0);
+			_screen->copyRegion(160, 105, 160, 105, 32, 9, 2, 0);
+			_screen->copyRegion(200, 83, 200, 83, 88, 93, 2, 0);
+		} else {
+			_screen->copyRegion(107, 72, 107, 72, 43, 87, 2, 0);
+			_screen->copyRegion(130, 159, 130, 159, 35, 17, 2, 0);
+			_screen->copyRegion(165, 105, 165, 105, 32, 9, 2, 0);
+			_screen->copyRegion(206, 83, 206, 83, 94, 93, 2, 0);
+		}
 		break;
 	case 3:
 		_screen->copyRegion(152, 56, 152, 56, 48, 48, 2, 0);
@@ -379,47 +420,30 @@ void SeqPlayer::s1_fillRect() {
 
 void SeqPlayer::s1_playEffect() {
 	uint8 track = *_seqData++;
+	if (_vm->gameFlags().platform == Common::kPlatformAmiga)
+		return;
 	_vm->delay(3 * _vm->tickLength());
 	_sound->playSoundEffect(track);
 }
 
 void SeqPlayer::s1_playTrack() {
 	uint8 msg = *_seqData++;
-/*	
-	// we do not have audio cd support for now
-	if (_vm->features() & GF_AUDIOCD) {
-		switch (msg) {
-		case 0:
-			// nothing to do here...
-			break;
-		case 1:
-			_sound->beginFadeOut();
-			break;
-		case 56:
-			_vm->snd_playTheme(KyraEngine::MUSIC_INTRO, 3);
-			break;
-		case 57:
-			_vm->snd_playTheme(KyraEngine::MUSIC_INTRO, 4);
-			break;
-		case 58:
-			_vm->snd_playTheme(KyraEngine::MUSIC_INTRO, 5);
-			break;
-		default:
-			warning("Unknown seq. message: %.02d", msg);
-			break;
-		}
-	} else {*/
-		if (msg == 1) {
-			_sound->beginFadeOut();
-		} else {
-			_sound->haltTrack();
-			_sound->playTrack(msg);
-		}
-//	}
+
+	if (_vm->gameFlags().platform == Common::kPlatformAmiga)
+		return;
+
+	if (msg == 1) {
+		_sound->beginFadeOut();
+	} else {
+		_sound->haltTrack();
+		if (_vm->gameFlags().platform == Common::kPlatformFMTowns)
+			msg += 2;
+		_sound->playTrack(msg);
+	}
 }
 
 void SeqPlayer::s1_allocTempBuffer() {
-	if (_vm->features() & GF_DEMO) {
+	if (_vm->gameFlags().isDemo) {
 		_seqQuitFlag = true;
 	} else {
 		if (!_specialBuffer && !_copyViewOffs) {
@@ -465,7 +489,7 @@ void SeqPlayer::s1_prefetchVocFile() {
 }
 
 bool SeqPlayer::playSequence(const uint8 *seqData, bool skipSeq) {
-	debugC( 9, kDebugLevelSequence, "SeqPlayer::seq_playSequence(%p, %d)", (const void *)seqData, skipSeq);
+	debugC(9, kDebugLevelSequence, "SeqPlayer::seq_playSequence(%p, %d)", (const void *)seqData, skipSeq);
 	assert(seqData);
 	
 	static SeqEntry floppySeqProcs[] = {
@@ -562,14 +586,12 @@ bool SeqPlayer::playSequence(const uint8 *seqData, bool skipSeq) {
 	const SeqEntry* commands;
 	int numCommands;
 
-	if (_vm->features() & GF_FLOPPY || _vm->features() & GF_DEMO) {
-		commands = floppySeqProcs;
-		numCommands = ARRAYSIZE(floppySeqProcs);
-	} else if (_vm->features() & GF_TALKIE) {
+	if (_vm->gameFlags().isTalkie) {
 		commands = cdromSeqProcs;
 		numCommands = ARRAYSIZE(cdromSeqProcs);
 	} else {
-		error("No commandlist found");
+		commands = floppySeqProcs;
+		numCommands = ARRAYSIZE(floppySeqProcs);
 	}
 
 	bool seqSkippedFlag = false;
@@ -599,9 +621,9 @@ bool SeqPlayer::playSequence(const uint8 *seqData, bool skipSeq) {
 		if (skipSeq && _vm->seq_skipSequence()) {
 			while (1) {
 				uint8 code = *_seqData;
-				if (commands[code].proc == &SeqPlayer::s1_endOfScript || commands[code].proc == &SeqPlayer::s1_break) {
+				if (commands[code].proc == &SeqPlayer::s1_endOfScript || commands[code].proc == &SeqPlayer::s1_break)
 					break;
-				}
+
 				_seqData += commands[code].len;
 			}
 			skipSeq = false;
@@ -610,17 +632,19 @@ bool SeqPlayer::playSequence(const uint8 *seqData, bool skipSeq) {
 		// used in Kallak writing intro
 		if (_seqDisplayTextFlag && _seqDisplayedTextTimer != 0xFFFFFFFF) {
 			if (_seqDisplayedTextTimer < _system->getMillis()) {
-				char charStr[2];
+				char charStr[3];
 				charStr[0] = _vm->seqTextsTable()[_seqDisplayedText][_seqDisplayedChar];
-				charStr[1] = '\0';
+				charStr[1] = charStr[2] = '\0';
+				if (_vm->gameFlags().lang == Common::JA_JPN)
+					charStr[1] = _vm->seqTextsTable()[_seqDisplayedText][++_seqDisplayedChar];
 				_screen->printText(charStr, _seqDisplayedTextX, 180, 0xF, 0xC);
 				_seqDisplayedTextX += _screen->getCharWidth(charStr[0]);
 				++_seqDisplayedChar;
-				if (_vm->seqTextsTable()[_seqDisplayedText][_seqDisplayedChar] == '\0') {
+
+				if (_vm->seqTextsTable()[_seqDisplayedText][_seqDisplayedChar] == '\0')
 					_seqDisplayedTextTimer = 0xFFFFFFFF;
-				} else {
-					_seqDisplayedTextTimer = _system->getMillis() + 1000 / ((_vm->features() & GF_FRENCH) ? 120 : 60);
-				}
+				else
+					_seqDisplayedTextTimer = _system->getMillis() + 1000 / ((_vm->gameFlags().lang == Common::FR_FRA) ? 120 : 60);
 			}
 		}
 
@@ -642,3 +666,4 @@ bool SeqPlayer::playSequence(const uint8 *seqData, bool skipSeq) {
 
 
 } // End of namespace Kyra
+
