@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2005-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/kyra/sequences_v1.cpp $
- * $Id: sequences_v1.cpp 23812 2006-08-31 17:29:26Z eriktorbjorn $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/kyra/sequences_v1.cpp $
+ * $Id: sequences_v1.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -30,6 +33,7 @@
 #include "kyra/animator.h"
 #include "kyra/text.h"
 
+#include "common/events.h"
 #include "common/system.h"
 #include "common/savefile.h"
 
@@ -38,7 +42,7 @@ namespace Kyra {
 void KyraEngine::seq_demo() {
 	debugC(9, kDebugLevelMain, "KyraEngine::seq_demo()");
 
-	snd_playTheme(MUSIC_INTRO, 2);
+	snd_playTheme(0, 2);
 
 	_screen->loadBitmap("START.CPS", 7, 7, _screen->_currentPalette);
 	_screen->copyRegion(0, 0, 0, 0, 320, 200, 6, 0);
@@ -88,9 +92,8 @@ void KyraEngine::seq_demo() {
 void KyraEngine::seq_intro() {
 	debugC(9, kDebugLevelMain, "KyraEngine::seq_intro()");
 
-	if (_features & GF_TALKIE) {
+	if (_flags.isTalkie)
 		_res->loadPakFile("INTRO.VRM");
-	}
 	
 	static const IntroProc introProcTable[] = {
 		&KyraEngine::seq_introLogos,
@@ -109,29 +112,54 @@ void KyraEngine::seq_intro() {
 
 	_seq->setCopyViewOffs(true);
 	_screen->setFont(Screen::FID_8_FNT);
-	snd_playTheme(MUSIC_INTRO, 2);
+	if (_flags.platform != Common::kPlatformFMTowns)
+		snd_playTheme(0, 2);
 	_text->setTalkCoords(144);
-	for (int i = 0; i < ARRAYSIZE(introProcTable) && !seq_skipSequence(); ++i) {
+
+	for (int i = 0; i < ARRAYSIZE(introProcTable) && !seq_skipSequence(); ++i)
 		(this->*introProcTable[i])();
-	}
+
 	_text->setTalkCoords(136);
 	delay(30 * _tickLength);
 	_seq->setCopyViewOffs(false);
 	_sound->haltTrack();
 	_sound->voiceStop();
-	if (_features & GF_TALKIE) {
+
+	if (_flags.isTalkie)
 		_res->unloadPakFile("INTRO.VRM");
-	}
 }
 
 void KyraEngine::seq_introLogos() {
 	debugC(9, kDebugLevelMain, "KyraEngine::seq_introLogos()");
+
+	if (_flags.platform == Common::kPlatformFMTowns) {
+		_screen->loadBitmap("LOGO.CPS", 3, 3, _screen->_currentPalette);
+		_screen->copyRegion(0, 0, 0, 0, 320, 200, 2, 0);
+		_screen->updateScreen();
+		_screen->fadeFromBlack();
+		delay(90 * _tickLength);
+		_screen->fadeToBlack();
+		if (!_abortIntroFlag)
+			snd_playWanderScoreViaMap(57, 0);
+	}
+
 	_screen->clearPage(0);
-	_screen->loadBitmap("TOP.CPS", 7, 7, NULL);
-	_screen->loadBitmap("BOTTOM.CPS", 5, 5, _screen->_currentPalette);
+	
+	if (_flags.platform == Common::kPlatformAmiga) {
+		_screen->loadPalette("INTRO.PAL", _screen->_currentPalette);
+		_screen->loadBitmap("BOTTOM.CPS", 3, 5, 0);
+		_screen->loadBitmap("TOP.CPS", 3, 3, 0);
+		_screen->copyRegion(0, 0, 0, 111, 320, 64, 2, 0);
+		_screen->copyRegion(0, 91, 0, 8, 320, 109, 2, 0);
+		_screen->copyRegion(0, 0, 0, 0, 320, 190, 0, 2);
+	} else {
+		_screen->loadBitmap("TOP.CPS", 7, 7, 0);
+		_screen->loadBitmap("BOTTOM.CPS", 5, 5, _screen->_currentPalette);
+		_screen->copyRegion(0, 91, 0, 8, 320, 103, 6, 0);
+		_screen->copyRegion(0, 0, 0, 111, 320, 64, 6, 0);
+	}
+
 	_screen->_curPage = 0;
-	_screen->copyRegion(0, 91, 0, 8, 320, 103, 6, 0);
-	_screen->copyRegion(0, 0, 0, 111, 320, 64, 6, 0);
 	_screen->updateScreen();
 	_screen->fadeFromBlack();
 	
@@ -141,6 +169,12 @@ void KyraEngine::seq_introLogos() {
 		return;
 	}
 	delay(60 * _tickLength);
+
+	if (_flags.platform == Common::kPlatformAmiga) {
+		memcpy(_screen->_currentPalette, _screen->_currentPalette + 3*32, 3*32);
+		_screen->setScreenPalette(_screen->_currentPalette);
+	}
+
 	if (_seq->playSequence(_seq_KyrandiaLogo, _skipFlag) && !seq_skipSequence() || _quitFlag) {
 		_screen->fadeToBlack();
 		_screen->clearPage(0);
@@ -151,38 +185,45 @@ void KyraEngine::seq_introLogos() {
 	if (_quitFlag)
 		return;
 
-	_screen->copyRegion(0, 91, 0, 8, 320, 104, 6, 2);
-	_screen->copyRegion(0, 0, 0, 112, 320, 64, 6, 2);
+	if (_flags.platform == Common::kPlatformAmiga) {
+		memcpy(_screen->_currentPalette, _screen->_currentPalette + 3*64, 3*32);
+		_screen->fadeToBlack();
+		_screen->copyRegion(0, 0, 0, 0, 320, 200, 4, 0);
+		_screen->fadeFromBlack();
+	} else {
+		_screen->copyRegion(0, 91, 0, 8, 320, 104, 6, 2);
+		_screen->copyRegion(0, 0, 0, 112, 320, 64, 6, 2);
 
-	uint32 start = _system->getMillis();
-	bool doneFlag = false;
-	int oldDistance = 0;
+		uint32 start = _system->getMillis();
+		bool doneFlag = false;
+		int oldDistance = 0;
 
-	do {
-		uint32 now = _system->getMillis();
+		do {
+			uint32 now = _system->getMillis();
 
-		// The smallest y2 we ever draw the screen for is 65.
-		int distance = (now - start) / _tickLength;
-		if (distance > 112) {
-			distance = 112;
-			doneFlag = true;
-		}
+			// The smallest y2 we ever draw the screen for is 65.
+			int distance = (now - start) / _tickLength;
+			if (distance > 112) {
+				distance = 112;
+				doneFlag = true;
+			}
 
-		if (distance > oldDistance) {
-			int y1 = 8 + distance;
-			int h1 = 168 - distance;
-			int y2 = 176 - distance;
-			int h2 = distance;
+			if (distance > oldDistance) {
+				int y1 = 8 + distance;
+				int h1 = 168 - distance;
+				int y2 = 176 - distance;
+				int h2 = distance;
 
-			_screen->copyRegion(0, y1, 0, 8, 320, h1, 2, 0);
-			if (h2 > 0)
-				_screen->copyRegion(0, 64, 0, y2, 320, h2, 4, 0);
-			_screen->updateScreen();
-		}
+				_screen->copyRegion(0, y1, 0, 8, 320, h1, 2, 0);
+				if (h2 > 0)
+					_screen->copyRegion(0, 64, 0, y2, 320, h2, 4, 0);
+				_screen->updateScreen();
+			}
 
-		oldDistance = distance;
-		delay(10);
-	} while (!doneFlag && !_quitFlag && !_abortIntroFlag);
+			oldDistance = distance;
+			delay(10);
+		} while (!doneFlag && !_quitFlag && !_abortIntroFlag);
+	}
 
 	if (_quitFlag)
 		return;
@@ -194,22 +235,39 @@ void KyraEngine::seq_introStory() {
 	debugC(9, kDebugLevelMain, "KyraEngine::seq_introStory()");
 	_screen->clearPage(3);
 	_screen->clearPage(0);
-	if (_features & GF_TALKIE) {
+
+	if (_flags.isTalkie)
 		return;
-	} else if (_features & GF_ENGLISH) {
-		_screen->loadBitmap("TEXT.CPS", 3, 3, 0);
-	} else if (_features & GF_GERMAN) {
-		_screen->loadBitmap("TEXT_GER.CPS", 3, 3, 0);
-	} else if (_features & GF_FRENCH) {
-		_screen->loadBitmap("TEXT_FRE.CPS", 3, 3, 0);
-	} else if (_features & GF_SPANISH) {
-		_screen->loadBitmap("TEXT_SPA.CPS", 3, 3, 0);
-	} else if (_features & GF_ITALIAN) {
-		_screen->loadBitmap("TEXT_ITA.CPS", 3, 3, 0);
-	} else {
+	else if (_flags.lang == Common::EN_ANY && (_flags.platform == Common::kPlatformPC || _flags.platform == Common::kPlatformAmiga))
+		_screen->loadBitmap("TEXT.CPS", 3, 3, _screen->_currentPalette);
+	else if (_flags.lang == Common::EN_ANY || _flags.lang == Common::JA_JPN)
+		_screen->loadBitmap("TEXT_ENG.CPS", 3, 3, _screen->_currentPalette);
+	else if (_flags.lang == Common::DE_DEU)
+		_screen->loadBitmap("TEXT_GER.CPS", 3, 3, _screen->_currentPalette);
+	else if (_flags.lang == Common::FR_FRA)
+		_screen->loadBitmap("TEXT_FRE.CPS", 3, 3, _screen->_currentPalette);
+	else if (_flags.lang == Common::ES_ESP)
+		_screen->loadBitmap("TEXT_SPA.CPS", 3, 3, _screen->_currentPalette);
+	else if (_flags.lang == Common::IT_ITA)
+		_screen->loadBitmap("TEXT_ITA.CPS", 3, 3, _screen->_currentPalette);
+	else
 		warning("no story graphics file found");
-	}
+	_screen->setScreenPalette(_screen->_currentPalette);
 	_screen->copyRegion(0, 0, 0, 0, 320, 200, 3, 0);
+
+	if (_flags.lang == Common::JA_JPN) {
+		const int x1 = (Screen::SCREEN_W - _screen->getTextWidth(_seq_textsTable[18])) / 2;
+		const int x2 = (Screen::SCREEN_W - _screen->getTextWidth(_seq_textsTable[19])) / 2;
+		const int y1 = 175;
+		const int y2 = 184;
+		
+		uint8 colorMap[] = { 0, 15, 12, 12 };
+		_screen->setTextColor(colorMap, 0, 3);
+
+		_screen->printText(_seq_textsTable[18], x1, y1, 5, 8);
+		_screen->printText(_seq_textsTable[19], x2, y2, 5, 8);
+	}
+
 	_screen->updateScreen();
 	//debugC(0, kDebugLevelMain, "skipFlag %i, %i", _skipFlag, _tickLength);
 	delay(360 * _tickLength);
@@ -259,7 +317,7 @@ void KyraEngine::seq_createAmuletJewel(int jewel, int page, int noSound, int dra
 	_screen->hideMouse();
 	if (!drawOnly) {
 		for (int i = 0; specialJewelTable[i] != 0xFFFF; ++i) {
-			_screen->drawShape(page, _shapes[4+specialJewelTable[i]], _amuletX2[jewel], _amuletY2[jewel], 0, 0);
+			_screen->drawShape(page, _shapes[specialJewelTable[i]], _amuletX2[jewel], _amuletY2[jewel], 0, 0);
 			_screen->updateScreen();
 			delayWithTicks(3);
 		}
@@ -285,13 +343,13 @@ void KyraEngine::seq_createAmuletJewel(int jewel, int page, int noSound, int dra
 		
 		if (opcodes) {
 			for (int i = 0; opcodes[i] != 0xFFFF; ++i) {
-				_screen->drawShape(page, _shapes[4+opcodes[i]], _amuletX2[jewel], _amuletY2[jewel], 0, 0);
+				_screen->drawShape(page, _shapes[opcodes[i]], _amuletX2[jewel], _amuletY2[jewel], 0, 0);
 				_screen->updateScreen();
 				delayWithTicks(3);
 			}
 		}
 	}
-	_screen->drawShape(page, _shapes[327+jewel], _amuletX2[jewel], _amuletY2[jewel], 0, 0);
+	_screen->drawShape(page, _shapes[323+jewel], _amuletX2[jewel], _amuletY2[jewel], 0, 0);
 	_screen->updateScreen();
 	_screen->showMouse();
 	setGameFlag(0x55+jewel);
@@ -348,18 +406,8 @@ void KyraEngine::seq_brandonHealing2() {
 	freeShapes123();
 	_screen->showMouse();
 	assert(_poisonGone);
-	if (speechEnabled()) {
-		snd_voiceWaitForFinish();
-		snd_playVoiceFile(2010);
-	}
-	if (textEnabled())
-		characterSays(_poisonGone[0], 0, -2);
-	if (speechEnabled()) {
-		snd_voiceWaitForFinish();
-		snd_playVoiceFile(2011);
-	}
-	if (textEnabled())
-		characterSays(_poisonGone[1], 0, -2);
+	characterSays(2010, _poisonGone[0], 0, -2);
+	characterSays(2011, _poisonGone[1], 0, -2);
 }
 
 void KyraEngine::seq_poisonDeathNow(int now) {
@@ -372,34 +420,14 @@ void KyraEngine::seq_poisonDeathNow(int now) {
 	if (_poisonDeathCounter >= 2) {
 		snd_playWanderScoreViaMap(1, 1);
 		assert(_thePoison);
-		if (speechEnabled()) {
-			snd_voiceWaitForFinish();
-			snd_playVoiceFile(7000);
-		}
-		if (textEnabled())
-			characterSays(_thePoison[0], 0, -2);
-		if (speechEnabled()) {
-			snd_voiceWaitForFinish();
-			snd_playVoiceFile(7001);
-		}
-		if (textEnabled())
-			characterSays(_thePoison[1], 0, -2);
+		characterSays(7000, _thePoison[0], 0, -2);
+		characterSays(7001, _thePoison[1], 0, -2);
 		seq_poisonDeathNowAnim();
 		_deathHandler = 3;
 	} else {
 		assert(_thePoison);
-		if (speechEnabled()) {
-			snd_voiceWaitForFinish();
-			snd_playVoiceFile(7002);
-		}
-		if (textEnabled())
-			characterSays(_thePoison[2], 0, -2);
-		if (speechEnabled()) {
-			snd_voiceWaitForFinish();
-			snd_playVoiceFile(7004);
-		}
-		if (textEnabled())
-			characterSays(_thePoison[3], 0, -2);
+		characterSays(7002, _thePoison[2], 0, -2);
+		characterSays(7004, _thePoison[3], 0, -2);
 	}
 }
 
@@ -491,20 +519,10 @@ void KyraEngine::seq_playFluteAnimation() {
 	
 	if (soundType == 1) {
 		assert(_fluteString);
-		if (speechEnabled()) {
-			snd_voiceWaitForFinish();
-			snd_playVoiceFile(1000);
-		}
-		if (textEnabled())
-			characterSays(_fluteString[0], 0, -2);
+		characterSays(1000, _fluteString[0], 0, -2);
 	} else if (soundType == 2) {
 		assert(_fluteString);
-		if (speechEnabled()) {
-			snd_voiceWaitForFinish();
-			snd_playVoiceFile(1001);
-		}
-		if (textEnabled())
-			characterSays(_fluteString[1], 0, -2);
+		characterSays(1001, _fluteString[1], 0, -2);
 	}
 }
 
@@ -528,7 +546,7 @@ void KyraEngine::seq_winterScroll1() {
 	snd_playSoundEffect(0x20);
 
 	uint8 numFrames, midpoint;
-	if (_features & GF_TALKIE) {
+	if (_flags.isTalkie) {
 		numFrames = 18;
 		midpoint = 136;
 	} else {
@@ -676,11 +694,12 @@ void KyraEngine::seq_makeBrandonNormal2() {
 	_animator->setBrandonAnimSeqSize(4, 48);
 	_currentCharacter->currentAnimFrame = 7;
 	_animator->animRefreshNPC(0);
-	if (_currentCharacter->sceneId >= 229 && _currentCharacter->sceneId <= 245) {
+
+	if (_currentCharacter->sceneId >= 229 && _currentCharacter->sceneId <= 245)
 		_screen->fadeSpecialPalette(31, 234, 13, 4);
-	} else if (_currentCharacter->sceneId >= 118 && _currentCharacter->sceneId <= 186) {
+	else if (_currentCharacter->sceneId >= 118 && _currentCharacter->sceneId <= 186)
 		_screen->fadeSpecialPalette(14, 228, 15, 4);
-	}
+
 	freeShapes123();
 	_screen->showMouse();
 }
@@ -706,21 +725,23 @@ void KyraEngine::seq_makeBrandonWisp() {
 		delayWithTicks(8);
 	}
 	_brandonStatusBit |= 2;
-	if (_currentCharacter->sceneId >= 109 && _currentCharacter->sceneId <= 198) {
+
+	if (_currentCharacter->sceneId >= 109 && _currentCharacter->sceneId <= 198)
 		setTimerCountdown(14, 18000);
-	} else {
+	else
 		setTimerCountdown(14, 7200);
-	}
+
 	_animator->_brandonDrawFrame = 113;
 	_brandonStatusBit0x02Flag = 1;
 	_currentCharacter->currentAnimFrame = 113;
 	_animator->animRefreshNPC(0);
 	_animator->updateAllObjectShapes();
-	if (_currentCharacter->sceneId >= 229 && _currentCharacter->sceneId <= 245) {
+
+	if (_currentCharacter->sceneId >= 229 && _currentCharacter->sceneId <= 245)
 		_screen->fadeSpecialPalette(30, 234, 13, 4);
-	} else if (_currentCharacter->sceneId >= 118 && _currentCharacter->sceneId <= 186) {
+	else if (_currentCharacter->sceneId >= 118 && _currentCharacter->sceneId <= 186)
 		_screen->fadeSpecialPalette(14, 228, 15, 4);
-	}
+
 	freeShapes123();
 	_screen->showMouse();
 }
@@ -775,12 +796,7 @@ void KyraEngine::seq_fillFlaskWithWater(int item, int type) {
 	
 	if (item >= 60 && item <= 77) {
 		assert(_flaskFull);
-		if (speechEnabled()) {
-			snd_voiceWaitForFinish();
-			snd_playVoiceFile(8006);
-		}
-		if (textEnabled())
-			characterSays(_flaskFull[0], 0, -2);
+		characterSays(8006, _flaskFull[0], 0, -2);
 	} else if (item == 78) {
 		assert(type >= 0 && type < ARRAYSIZE(flaskTable1));
 		newItem = flaskTable1[type];
@@ -798,16 +814,11 @@ void KyraEngine::seq_fillFlaskWithWater(int item, int type) {
 	_itemInHand = newItem;
 	assert(_fullFlask);
 	assert(type < _fullFlask_Size && type >= 0);
-	if (speechEnabled()) {
-		snd_voiceWaitForFinish();
-		static const uint16 voiceEntries[] = {
-			0x1F40, 0x1F41, 0x1F42, 0x1F45
-		};
-		assert(type < ARRAYSIZE(voiceEntries));
-		snd_playVoiceFile(voiceEntries[type]);
-	}
-	if (textEnabled())
-		characterSays(_fullFlask[type], 0, -2);
+	static const uint16 voiceEntries[] = {
+		0x1F40, 0x1F41, 0x1F42, 0x1F45
+	};
+	assert(type < ARRAYSIZE(voiceEntries));
+	characterSays(voiceEntries[type], _fullFlask[type], 0, -2);
 }
 
 void KyraEngine::seq_playDrinkPotionAnim(int item, int unk2, int flags) {
@@ -904,12 +915,12 @@ void KyraEngine::seq_playDrinkPotionAnim(int item, int unk2, int flags) {
 
 int KyraEngine::seq_playEnd() {
 	debugC(9, kDebugLevelMain, "KyraEngine::seq_playEnd()");
-	if (_endSequenceSkipFlag) {
+	if (_endSequenceSkipFlag)
 		return 0;
-	}
-	if (_deathHandler == 8) {
+
+	if (_deathHandler == 8)
 		return 0;
-	}
+
 	_screen->_curPage = 2;
 	if (_endSequenceNeedLoading) {
 		snd_playWanderScoreViaMap(50, 1);
@@ -954,9 +965,8 @@ int KyraEngine::seq_playEnd() {
 			return 1;
 		} else {
 			_endSequenceSkipFlag = 1;
-			if (_text->printed()) {
+			if (_text->printed())
 				_text->restoreTalkTextMessageBkgd(2, 0);
-			}
 			_screen->_curPage = 0;
 			_screen->hideMouse();
 			_screen->fadeSpecialPalette(32, 228, 20, 60);
@@ -974,11 +984,10 @@ int KyraEngine::seq_playEnd() {
 			snd_playSoundEffect(0x40);
 			for (int i = 0; i < 22; ++i) {
 				delayUntil(nextTime);
-				if (i == 4) {
+				if (i == 4)
 					snd_playSoundEffect(0x3E);
-				} else if (i == 20) {
+				else if (i == 20)
 					snd_playSoundEffect(0x0E);
-				}
 				nextTime = _system->getMillis() + 8 * _tickLength;
 				_finalA->displayFrame(i);
 				_screen->updateScreen();
@@ -1026,10 +1035,13 @@ void KyraEngine::seq_playEnding() {
 	// XXX
 	assert(_homeString);
 	drawSentenceCommand(_homeString[0], 179);
-	memset(_screen->getPalette(0), 0, sizeof(uint8)*768);
-	_screen->setScreenPalette(_screen->getPalette(0));
+
+	memset(_screen->getPalette(2), 0, sizeof(uint8)*768);
+	_screen->setScreenPalette(_screen->getPalette(2));
+
 	_seq->playSequence(_seq_Reunion, false);
 	_screen->fadeToBlack();
+
 	_screen->showMouse();
 	seq_playCredits();
 }
@@ -1051,28 +1063,38 @@ void KyraEngine::seq_playCredits() {
 	memset(strings, 0, sizeof(strings));
 	
 	_screen->hideMouse();
-	uint32 sz = 0;
-	if (_features & GF_FLOPPY) {
-		_screen->loadFont(Screen::FID_CRED6_FNT, _res->fileData("CREDIT6.FNT", &sz));
-		_screen->loadFont(Screen::FID_CRED8_FNT, _res->fileData("CREDIT8.FNT", &sz));
+	if (!_flags.isTalkie) {
+		_screen->loadFont(Screen::FID_CRED6_FNT, "CREDIT6.FNT");
+		_screen->loadFont(Screen::FID_CRED8_FNT, "CREDIT8.FNT");
 	} else
 		_screen->setFont(Screen::FID_8_FNT);
 	
 	_screen->loadBitmap("CHALET.CPS", 4, 4, _screen->_currentPalette);
-	_screen->setScreenPalette(_screen->_currentPalette);
 	
 	_screen->setCurPage(0);
 	_screen->clearCurPage();
 	_screen->setTextColorMap(colorMap);
 	_screen->_charWidth = -1;
-	// we don't need that one for midi or adlib
-	//snd_playWanderScoreViaMap(53, 1);
+
+	// we only need this for the fm-towns version
+	if (_flags.platform == Common::kPlatformFMTowns)
+		snd_playWanderScoreViaMap(53, 1);
 
 	uint8 *buffer = 0;
-	uint32 size;
+	uint32 size = 0;
 	
-	buffer = _res->fileData("CREDITS.TXT", &size);
-	assert(buffer);
+	if (_flags.platform == Common::kPlatformFMTowns) {
+		int sizeTmp = 0;
+		const uint8 *bufferTmp = _staticres->loadRawData(kCreditsStrings, sizeTmp);
+		buffer = new uint8[sizeTmp];
+		assert(buffer);
+		memcpy(buffer, bufferTmp, sizeTmp);
+		size = sizeTmp;
+		_staticres->unloadId(kCreditsStrings);
+	} else {
+		buffer = _res->fileData("CREDITS.TXT", &size);
+		assert(buffer);
+	}
 
 	uint8 *nextString = buffer;
 	uint8 *currentString = buffer;
@@ -1099,12 +1121,12 @@ void KyraEngine::seq_playCredits() {
 		
 		if (*currentString == 1) {
 		   	currentString++;
-			if (_features & GF_FLOPPY)
+			if (!_flags.isTalkie)
 				_screen->setFont(Screen::FID_CRED6_FNT);
 		} else {
 			if (*currentString == 2)
 				currentString++;
-			if (_features & GF_FLOPPY)
+			if (!_flags.isTalkie)
 				_screen->setFont(Screen::FID_CRED8_FNT);
 		}
 		strings[i].font = _screen->_currentFont;
@@ -1124,8 +1146,13 @@ void KyraEngine::seq_playCredits() {
 	}
 
 	_screen->setCurPage(2);
+	
+	memset(_screen->getPalette(2), 0, sizeof(uint8)*768);
+	_screen->setScreenPalette(_screen->getPalette(2));
+	_screen->copyRegion(8, 32, 8, 32, 312, 128, 4, 0, Screen::CR_NO_P_CHECK);
+	_screen->fadePalette(_screen->_currentPalette, 0x5A);
 
-	OSystem::Event event;	
+	Common::Event event;	
 	bool finished = false;
 	int bottom = 201;
 	while (!finished) {
@@ -1148,12 +1175,12 @@ void KyraEngine::seq_playCredits() {
 			_screen->updateScreen();
 		}
 
-		while (_system->pollEvent(event)) {
+		while (_eventMan->pollEvent(event)) {
 			switch (event.type) {
-			case OSystem::EVENT_KEYDOWN:
+			case Common::EVENT_KEYDOWN:
 				finished = true;
 				break;
-			case OSystem::EVENT_QUIT:
+			case Common::EVENT_QUIT:
 				quitGame();
 				finished = true;
 				break;
@@ -1168,8 +1195,8 @@ void KyraEngine::seq_playCredits() {
 		if (nextLoop > now)
 			_system->delayMillis(nextLoop - now);
 	}
-	
-	delete[] buffer;
+
+	delete [] buffer;
 	
 	_screen->fadeToBlack();
 	_screen->clearCurPage();
@@ -1429,10 +1456,13 @@ int KyraEngine::handleBeadState() {
 					beadState1.y = y;
 				}
 			}
+
 			_screen->copyCurPageBlock(x >> 3, y, beadState1.width, beadState1.height, _endSequenceBackUpRect);
 			_screen->drawShape(2, _panPagesTable[_lastDisplayedPanPage++], x, y, 0, 0);
+
 			if (_lastDisplayedPanPage > 17)
 				_lastDisplayedPanPage = 0;
+
 			_screen->addBitBlitRect(x, y, beadState1.width2, beadState1.height);
 		}
 		break;
@@ -1442,19 +1472,21 @@ int KyraEngine::handleBeadState() {
 			timer1 = _system->getMillis() + 4 * _tickLength;
 			_screen->copyFromCurPageBlock(beadState1.x >> 3, beadState1.y, beadState1.width, beadState1.height, _endSequenceBackUpRect);
 			_screen->addBitBlitRect(beadState1.x, beadState1.y, beadState1.width2, beadState1.height);
+
 			beadState1.x = beadState1.dstX + table1[beadState1.tableIndex];
 			beadState1.y = beadState1.dstY + table2[beadState1.tableIndex];
 			_screen->copyCurPageBlock(beadState1.x >> 3, beadState1.y, beadState1.width, beadState1.height, _endSequenceBackUpRect);
+
 			_screen->drawShape(2, _panPagesTable[_lastDisplayedPanPage++], beadState1.x, beadState1.y, 0, 0);
-			if (_lastDisplayedPanPage >= 17) {
+			if (_lastDisplayedPanPage >= 17)
 				_lastDisplayedPanPage = 0;
-			}
+
 			_screen->addBitBlitRect(beadState1.x, beadState1.y, beadState1.width2, beadState1.height);
+
 			++beadState1.tableIndex;
-			if (beadState1.tableIndex > 24) {
+			if (beadState1.tableIndex > 24)
 				beadState1.tableIndex = 0;
 				_unkEndSeqVar4 = 1;
-			}
 			if (_system->getMillis() > timer2 && _malcolmFlag == 7 && !_unkAmuletVar && !_text->printed()) {
 				snd_playSoundEffect(0x0B);
 				if (_currentCharacter->x1 > 233 && _currentCharacter->x1 < 305 && _currentCharacter->y1 > 85 && _currentCharacter->y1 < 105 &&
@@ -1468,9 +1500,9 @@ int KyraEngine::handleBeadState() {
 					beadState1.unk9 = _currentCharacter->y1 - 30;
 				}
 					
-				if (_text->printed()) {
+				if (_text->printed())
 					_text->restoreTalkTextMessageBkgd(2, 0);
-				}
+
 				initBeadState(beadState1.x, beadState1.y, beadState1.unk8, beadState1.unk9, 12, &beadState2);
 				_lastDisplayedPanPage = 18;
 			}
@@ -1546,9 +1578,8 @@ int KyraEngine::handleBeadState() {
 				beadState1.y = y;
 				_screen->copyCurPageBlock(beadState1.x >> 3, beadState1.y, beadState1.width, beadState1.height, _endSequenceBackUpRect);
 				_screen->drawShape(2, _panPagesTable[_lastDisplayedPanPage++], x, y, 0, 0);
-				if (_lastDisplayedPanPage > 17) {
+				if (_lastDisplayedPanPage > 17)
 					_lastDisplayedPanPage = 0;
-				}
 				_screen->addBitBlitRect(beadState1.x, beadState1.y, beadState1.width2, beadState1.height);
 			}
 		}
@@ -1572,21 +1603,20 @@ void KyraEngine::initBeadState(int x, int y, int x2, int y2, int unk, BeadState
 	int xDiff = x2 - x;
 	int yDiff = y2 - y;
 	int unk1 = 0, unk2 = 0;
-	if (xDiff > 0) {
+	if (xDiff > 0)
 		unk1 = 1;
-	} else if (xDiff == 0) {
+	else if (xDiff == 0)
 		unk1 = 0;
-	} else {
+	else
 		unk1 = -1;
-	}
 	
-	if (yDiff > 0) {
+	
+	if (yDiff > 0)
 		unk2 = 1;
-	} else if (yDiff == 0) {
+	else if (yDiff == 0)
 		unk2 = 0;
-	} else {
+	else
 		unk2 = -1;
-	}
 	
 	xDiff = ABS(xDiff);
 	yDiff = ABS(yDiff);
@@ -1603,9 +1633,8 @@ void KyraEngine::initBeadState(int x, int y, int x2, int y2, int unk, BeadState
 
 int KyraEngine::processBead(int x, int y, int &x2, int &y2, BeadState *ptr) {
 	debugC(9, kDebugLevelMain, "KyraEngine::processBead(%d, %d, %p, %p, %p)", x, y, (const void *)&x2, (const void *)&y2, (const void *)ptr);
-	if (x == ptr->dstX && y == ptr->dstY) {
+	if (x == ptr->dstX && y == ptr->dstY)
 		return 1;
-	}
 	
 	int xPos = x, yPos = y;
 	if (ptr->width >= ptr->height) {
@@ -1629,13 +1658,11 @@ int KyraEngine::processBead(int x, int y, int &x2, int &y2, BeadState *ptr) {
 	}
 	
 	int temp = ABS(x - ptr->dstX);
-	if (ptr->unk9 > temp) {
+	if (ptr->unk9 > temp)
 		xPos = ptr->dstX;
-	}
 	temp = ABS(y - ptr->dstY);
-	if (ptr->unk9 > temp) {
+	if (ptr->unk9 > temp)
 		yPos = ptr->dstY;
-	}
 	x2 = xPos;
 	y2 = yPos;
 	return 0;
@@ -1644,8 +1671,29 @@ int KyraEngine::processBead(int x, int y, int &x2, int &y2, BeadState *ptr) {
 void KyraEngine::setupPanPages() {
 	debugC(9, kDebugLevelMain, "KyraEngine::setupPanPages()");
 	_screen->loadBitmap("BEAD.CPS", 3, 3, 0);
-	for (int i = 0; i <= 19; ++i) {
-		_panPagesTable[i] = _seq->setPanPages(3, i);
+	if (_flags.platform == Common::kPlatformMacintosh || _flags.platform == Common::kPlatformAmiga) {
+		int pageBackUp = _screen->_curPage;
+		_screen->_curPage = 2;
+
+		delete [] _panPagesTable[19];
+		_panPagesTable[19] = _screen->encodeShape(0, 0, 16, 9, 0);
+		assert(_panPagesTable[19]);
+		
+		int curX = 16;
+		for (int i = 0; i < 19; ++i) {
+			delete [] _panPagesTable[i];
+			_panPagesTable[i] = _screen->encodeShape(curX, 0, 8, 5, 0);
+			assert(_panPagesTable[i]);
+			curX += 8;
+		}
+
+		_screen->_curPage = pageBackUp;
+	} else {
+		for (int i = 0; i <= 19; ++i) {
+			delete [] _panPagesTable[i];
+			_panPagesTable[i] = _seq->setPanPages(3, i);
+			assert(_panPagesTable[i]);
+		}
 	}
 }
 
@@ -1654,8 +1702,8 @@ void KyraEngine::freePanPages() {
 	delete _endSequenceBackUpRect;
 	_endSequenceBackUpRect = 0;
 	for (int i = 0; i <= 19; ++i) {
-		free(_panPagesTable[i]);
-		_panPagesTable[i] = NULL;
+		delete [] _panPagesTable[i];
+		_panPagesTable[i] = 0;
 	}
 }
 
@@ -1745,21 +1793,23 @@ void KyraEngine::drawJewelPress(int jewel, int drawSpecial) {
 	debugC(9, kDebugLevelMain, "KyraEngine::drawJewelPress(%d, %d)", jewel, drawSpecial);
 	_screen->hideMouse();
 	int shape = 0;
-	if (drawSpecial) {
+
+	if (drawSpecial)
 		shape = 0x14E;
-	} else {
+	else
 		shape = jewel + 0x149;
-	}
+
 	snd_playSoundEffect(0x45);
-	_screen->drawShape(0, _shapes[4+shape], _amuletX2[jewel], _amuletY2[jewel], 0, 0);
+	_screen->drawShape(0, _shapes[shape], _amuletX2[jewel], _amuletY2[jewel], 0, 0);
 	_screen->updateScreen();
 	delayWithTicks(2);
-	if (drawSpecial) {
+
+	if (drawSpecial)
 		shape = 0x148;
-	} else {
+	else
 		shape = jewel + 0x143;
-	}
-	_screen->drawShape(0, _shapes[4+shape], _amuletX2[jewel], _amuletY2[jewel], 0, 0);
+
+	_screen->drawShape(0, _shapes[shape], _amuletX2[jewel], _amuletY2[jewel], 0, 0);
 	_screen->updateScreen();
 	_screen->showMouse();
 }
@@ -1771,18 +1821,14 @@ void KyraEngine::drawJewelsFadeOutStart() {
 	static const uint16 jewelTable3[] = { 0x166, 0x160, 0x15C, 0x157, 0x152, 0xFFFF };
 	static const uint16 jewelTable4[] = { 0x165, 0x161, 0x15B, 0x156, 0x151, 0xFFFF };
 	for (int i = 0; jewelTable1[i] != 0xFFFF; ++i) {
-		if (queryGameFlag(0x57)) {
-			_screen->drawShape(0, _shapes[4+jewelTable1[i]], _amuletX2[2], _amuletY2[2], 0, 0);
-		}
-		if (queryGameFlag(0x59)) {
-			_screen->drawShape(0, _shapes[4+jewelTable3[i]], _amuletX2[4], _amuletY2[4], 0, 0);
-		}
-		if (queryGameFlag(0x56)) {
-			_screen->drawShape(0, _shapes[4+jewelTable2[i]], _amuletX2[1], _amuletY2[1], 0, 0);
-		}
-		if (queryGameFlag(0x58)) {
-			_screen->drawShape(0, _shapes[4+jewelTable4[i]], _amuletX2[3], _amuletY2[3], 0, 0);
-		}
+		if (queryGameFlag(0x57))
+			_screen->drawShape(0, _shapes[jewelTable1[i]], _amuletX2[2], _amuletY2[2], 0, 0);
+		if (queryGameFlag(0x59))
+			_screen->drawShape(0, _shapes[jewelTable3[i]], _amuletX2[4], _amuletY2[4], 0, 0);
+		if (queryGameFlag(0x56))
+			_screen->drawShape(0, _shapes[jewelTable2[i]], _amuletX2[1], _amuletY2[1], 0, 0);
+		if (queryGameFlag(0x58))
+			_screen->drawShape(0, _shapes[jewelTable4[i]], _amuletX2[3], _amuletY2[3], 0, 0);
 		_screen->updateScreen();
 		delayWithTicks(3);
 	}
@@ -1792,36 +1838,34 @@ void KyraEngine::drawJewelsFadeOutEnd(int jewel) {
 	debugC(9, kDebugLevelMain, "KyraEngine::drawJewelsFadeOutEnd(%d)", jewel);
 	static const uint16 jewelTable[] = { 0x153, 0x158, 0x15D, 0x162, 0x148, 0xFFFF };
 	int newDelay = 0;
+
 	switch (jewel - 1) {
 	case 2:
-		if (_currentCharacter->sceneId >= 109 && _currentCharacter->sceneId <= 198) {
+		if (_currentCharacter->sceneId >= 109 && _currentCharacter->sceneId <= 198)
 			newDelay = 18900;
-		} else {
+		else
 			newDelay = 8100;
-		}
 		break;
 			
 	default:
 		newDelay = 3600;
 		break;
 	}
+
 	setGameFlag(0xF1);
 	setTimerCountdown(19, newDelay);
 	_screen->hideMouse();
 	for (int i = 0; jewelTable[i] != 0xFFFF; ++i) {
 		uint16 shape = jewelTable[i];
-		if (queryGameFlag(0x57)) {
-			_screen->drawShape(0, _shapes[4+shape], _amuletX2[2], _amuletY2[2], 0, 0);
-		}
-		if (queryGameFlag(0x59)) {
-			_screen->drawShape(0, _shapes[4+shape], _amuletX2[4], _amuletY2[4], 0, 0);
-		}
-		if (queryGameFlag(0x56)) {
-			_screen->drawShape(0, _shapes[4+shape], _amuletX2[1], _amuletY2[1], 0, 0);
-		}
-		if (queryGameFlag(0x58)) {
-			_screen->drawShape(0, _shapes[4+shape], _amuletX2[3], _amuletY2[3], 0, 0);
-		}
+		if (queryGameFlag(0x57))
+			_screen->drawShape(0, _shapes[shape], _amuletX2[2], _amuletY2[2], 0, 0);
+		if (queryGameFlag(0x59))
+			_screen->drawShape(0, _shapes[shape], _amuletX2[4], _amuletY2[4], 0, 0);
+		if (queryGameFlag(0x56))
+			_screen->drawShape(0, _shapes[shape], _amuletX2[1], _amuletY2[1], 0, 0);
+		if (queryGameFlag(0x58))
+			_screen->drawShape(0, _shapes[shape], _amuletX2[3], _amuletY2[3], 0, 0);
+
 		_screen->updateScreen();
 		delayWithTicks(3);
 	}
@@ -1829,3 +1873,4 @@ void KyraEngine::drawJewelsFadeOutEnd(int jewel) {
 }
 
 } // end of namespace Kyra
+
