@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2004-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/kyra/sound.cpp $
- * $Id: sound.cpp 23606 2006-07-26 22:25:55Z lordhoto $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/kyra/sound.cpp $
+ * $Id: sound.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -35,9 +38,9 @@
 
 namespace Kyra {
 
-Sound::Sound(KyraEngine *engine, Audio::Mixer *mixer)
-	: _engine(engine), _mixer(mixer), _currentVocFile(0), _vocHandle(), _compressHandle(),
-	_musicEnabled(true), _sfxEnabled(true) {
+Sound::Sound(KyraEngine *vm, Audio::Mixer *mixer)
+	: _vm(vm), _mixer(mixer), _currentVocFile(0), _vocHandle(), _compressHandle(),
+	_musicEnabled(1), _sfxEnabled(true), _soundFileList(0), _soundFileListSize(0) {
 }
 
 Sound::~Sound() {
@@ -54,7 +57,7 @@ void Sound::voicePlay(const char *file) {
 		strcat(filenamebuffer, _supportedCodes[i].fileext);
 
 		_compressHandle.close();
-		_engine->resource()->fileHandle(filenamebuffer, &fileSize, _compressHandle);
+		_vm->resource()->getFileHandle(filenamebuffer, &fileSize, _compressHandle);
 		if (!_compressHandle.isOpen())
 			continue;
 		
@@ -67,17 +70,18 @@ void Sound::voicePlay(const char *file) {
 		strcpy(filenamebuffer, file);
 		strcat(filenamebuffer, ".VOC");
 		
-		fileData = _engine->resource()->fileData(filenamebuffer, &fileSize);
+		fileData = _vm->resource()->fileData(filenamebuffer, &fileSize);
 		if (!fileData)
 			return;
 
 		Common::MemoryReadStream vocStream(fileData, fileSize);
-		_mixer->stopHandle(_vocHandle);
 		_currentVocFile = Audio::makeVOCStream(vocStream);
 	}
 
-	if (_currentVocFile)
+	if (_currentVocFile) {
+		_mixer->stopHandle(_vocHandle);
 		_mixer->playInputStream(Audio::Mixer::kSpeechSoundType, &_vocHandle, _currentVocFile);
+	}
 	delete [] fileData;
 	fileSize = 0;
 }
@@ -93,28 +97,26 @@ bool Sound::voiceIsPlaying() {
 
 #pragma mark -
 
-SoundMidiPC::SoundMidiPC(MidiDriver *driver, Audio::Mixer *mixer, KyraEngine *engine) : Sound(engine, mixer) {
+SoundMidiPC::SoundMidiPC(KyraEngine *vm, Audio::Mixer *mixer, MidiDriver *driver) : Sound(vm, mixer) {
 	_driver = driver;
 	_passThrough = false;
 	_eventFromMusic = false;
 	_fadeMusicOut = _sfxIsPlaying = false;
 	_fadeStartTime = 0;
-	_isPlaying = _nativeMT32 = false;
+	_isPlaying = _nativeMT32 = _useC55 = false;
 	_soundEffect = _parser = 0;
 	_soundEffectSource = _parserSource = 0;
 
 	memset(_channel, 0, sizeof(MidiChannel*) * 32);
 	memset(_channelVolume, 50, sizeof(uint8) * 16);
 	_channelVolume[10] = 100;
-	for (int i = 0; i < 16; ++i) {
+	for (int i = 0; i < 16; ++i)
 		_virChannel[i] = i;
-	}
 	_volume = 0;
 
 	int ret = open();
-	if (ret != MERR_ALREADY_OPEN && ret != 0) {
+	if (ret != MERR_ALREADY_OPEN && ret != 0)
 		error("couldn't open midi driver");
-	}
 }
 
 SoundMidiPC::~SoundMidiPC() {
@@ -123,7 +125,7 @@ SoundMidiPC::~SoundMidiPC() {
 
 	Common::StackLock lock(_mutex);
 
-	_driver->setTimerCallback(NULL, NULL);
+	_driver->setTimerCallback(0, 0);
 	close();
 }
 
@@ -168,6 +170,21 @@ void SoundMidiPC::close() {
 }
 
 void SoundMidiPC::send(uint32 b) {
+	// HACK: For Kyrandia, we make the simplifying assumption that a song
+	// either loops in its entirety, or not at all. So if we see a FOR_LOOP
+	// controller event, we turn on looping even if there isn't any
+	// corresponding NEXT_BREAK event.
+	//
+	// This is a gross over-simplification of how XMIDI handles loops. If
+	// anyone feels like doing a proper implementation, please refer to
+	// the Exult project, and do it in midiparser_xmidi.cpp
+
+	if ((b & 0xFFF0) == 0x74B0) {
+		debugC(9, kDebugLevelMain | kDebugLevelSound, "SoundMidiPC: Looping song");
+		_parser->property(MidiParser::mpAutoLoop, true);
+		return;
+	}
+
 	if (_passThrough) {
 		if ((b & 0xFFF0) == 0x007BB0)
 			return;
@@ -193,7 +210,7 @@ void SoundMidiPC::send(uint32 b) {
 		_channelVolume[channel] = volume;
 		volume = volume * _volume / 255;
 		b = (b & 0xFF00FFFF) | (volume << 16);
-	} else if ((b & 0xF0) == 0xC0 && !_nativeMT32) {
+	} else if ((b & 0xF0) == 0xC0 && !_nativeMT32 && !_useC55) {
 		b = (b & 0xFFFF00FF) | MidiDriver::_mt32ToGm[(b >> 8) & 0xFF] << 8;
 	} else if ((b & 0xFFF0) == 0x007BB0) {
 		//Only respond to All Notes Off if this channel
@@ -229,12 +246,12 @@ void SoundMidiPC::metaEvent(byte type, byte *data, uint16 length) {
 	}
 }
 
-void SoundMidiPC::loadMusicFile(const char *file) {
+void SoundMidiPC::loadSoundFile(uint file) {
 	char filename[25];
-	sprintf(filename, "%s.XMI", file);
+	sprintf(filename, "%s.%s", soundFilename(file), _useC55 ? "C55" : "XMI");
 
 	uint32 size;
-	uint8 *data = (_engine->resource())->fileData(filename, &size);
+	uint8 *data = (_vm->resource())->fileData(filename, &size);
 
 	if (!data) {
 		warning("couldn't load '%s'", filename);
@@ -264,15 +281,14 @@ void SoundMidiPC::playMusic(uint8 *data, uint32 size) {
 	_parser->setMidiDriver(this);
 	_parser->setTimerRate(getBaseTempo());
 	_parser->setTempo(0);
-	_parser->property(MidiParser::mpAutoLoop, true);
 }
 
-void SoundMidiPC::loadSoundEffectFile(const char *file) {
+void SoundMidiPC::loadSoundEffectFile(uint file) {
 	char filename[25];
-	sprintf(filename, "%s.XMI", file);
+	sprintf(filename, "%s.%s", soundFilename(file), _useC55 ? "C55" : "XMI");
 
 	uint32 size;
-	uint8 *data = (_engine->resource())->fileData(filename, &size);
+	uint8 *data = (_vm->resource())->fileData(filename, &size);
 
 	if (!data) {
 		warning("couldn't load '%s'", filename);
@@ -341,11 +357,10 @@ void SoundMidiPC::onTimer(void *refCon) {
 	// this should be set to the fadeToBlack value
 	static const uint32 musicFadeTime = 2 * 1000;
 
-	if (music->_fadeMusicOut && music->_fadeStartTime + musicFadeTime > music->_engine->_system->getMillis()) {
-		byte volume = (byte)((musicFadeTime - (music->_engine->_system->getMillis() - music->_fadeStartTime)) * 255 / musicFadeTime);
+	if (music->_fadeMusicOut && music->_fadeStartTime + musicFadeTime > music->_vm->_system->getMillis()) {
+		byte volume = (byte)((musicFadeTime - (music->_vm->_system->getMillis() - music->_fadeStartTime)) * 255 / musicFadeTime);
 		music->setVolume(volume);
 	} else if (music->_fadeStartTime) {
-		music->setVolume(255);
 		music->_fadeStartTime = 0;
 		music->_fadeMusicOut = false;
 		music->_isPlaying = false;
@@ -357,9 +372,9 @@ void SoundMidiPC::onTimer(void *refCon) {
 				music->send(0x80 | j | i << 8);
 			}
 		}
-		for (int i = 0; i < 16; ++i) {
+
+		for (int i = 0; i < 16; ++i)
 			music->send(0x007BB0 | i);
-		}
 	}
 
 	if (music->_isPlaying) {
@@ -386,6 +401,7 @@ void SoundMidiPC::playTrack(uint8 track) {
 		_parser->setTrack(track);
 		_parser->jumpToTick(0);
 		_parser->setTempo(1);
+		_parser->property(MidiParser::mpAutoLoop, false);
 	}
 }
 
@@ -413,74 +429,90 @@ void SoundMidiPC::playSoundEffect(uint8 track) {
 
 void SoundMidiPC::beginFadeOut() {
 	_fadeMusicOut = true;
-	_fadeStartTime = _engine->_system->getMillis();
+	_fadeStartTime = _vm->_system->getMillis();
 }
 
 #pragma mark -
 
-
 bool KyraEngine::speechEnabled() {
-	return _features & GF_TALKIE && (_configVoice == 1 || _configVoice == 2);
+	return _flags.isTalkie && (_configVoice == 1 || _configVoice == 2);
 }
 
 bool KyraEngine::textEnabled() {
-	return _features & GF_FLOPPY || (_configVoice == 0 || _configVoice == 2);
+	return !_flags.isTalkie || (_configVoice == 0 || _configVoice == 2);
 }
 
 void KyraEngine::snd_playTheme(int file, int track) {
 	debugC(9, kDebugLevelMain | kDebugLevelSound, "KyraEngine::snd_playTheme(%d)", file);
-	assert(file < _musicFilesCount);
-	_curMusicTheme = _newMusicTheme = file;
-	_sound->loadMusicFile(_musicFiles[file]);
+	_curSfxFile = _curMusicTheme = file;
+	_sound->loadSoundFile(_curMusicTheme);
 	_sound->playTrack(track);
 }
 
 void KyraEngine::snd_playSoundEffect(int track) {
 	debugC(9, kDebugLevelMain | kDebugLevelSound, "KyraEngine::snd_playSoundEffect(%d)", track);
+	if (_flags.platform == Common::kPlatformFMTowns && track == 49) {
+		snd_playWanderScoreViaMap(56, 1);
+		return;
+	}
 	_sound->playSoundEffect(track);
 }
 
 void KyraEngine::snd_playWanderScoreViaMap(int command, int restart) {
 	debugC(9, kDebugLevelMain | kDebugLevelSound, "KyraEngine::snd_playWanderScoreViaMap(%d, %d)", command, restart);
-	static const int8 soundTable[] = {
-		-1,   0,  -1,   1,   0,   3,   0,   2,
-		 0,   4,   1,   2,   1,   3,   1,   4,
-		 1,  92,   1,   6,   1,   7,   2,   2,
-		 2,   3,   2,   4,   2,   5,   2,   6,
-		 2,   7,   3,   3,   3,   4,   1,   8,
-		 1,   9,   4,   2,   4,   3,   4,   4,
-		 4,   5,   4,   6,   4,   7,   4,   8,
-		 1,  11,   1,  12,   1,  14,   1,  13,
-		 4,   9,   5,  12,   6,   2,   6,   6,
-		 6,   7,   6,   8,   6,   9,   6,   3,
-		 6,   4,   6,   5,   7,   2,   7,   3,
-		 7,   4,   7,   5,   7,   6,   7,   7,
-		 7,   8,   7,   9,   8,   2,   8,   3,
-		 8,   4,   8,   5,   6,  11,   5,  11
-	};
-	//if (!_disableSound) {
-	//	XXX
-	//}
-	assert(command*2+1 < ARRAYSIZE(soundTable));
-	if (_curMusicTheme != soundTable[command*2]+1) {
-		if (soundTable[command*2] != -1) {
-			snd_playTheme(soundTable[command*2]+1);
-		}
-	}
-	
 	if (restart)
 		_lastMusicCommand = -1;
-	
-	if (command != 1) {
-		if (_lastMusicCommand != command) {
-			_lastMusicCommand = command;
+
+	if (_flags.platform == Common::kPlatformFMTowns) {
+		if (command == 1) {
+			_sound->beginFadeOut();
+		} else if (command >= 35 && command <= 38) {
+			snd_playSoundEffect(command-20);
+		} else if (command >= 2) {
+			if (_lastMusicCommand != command) {
+				// the original does -2 here we handle this inside _sound->playTrack()
+				_sound->playTrack(command);
+			}
+		} else {
 			_sound->haltTrack();
-			_sound->playTrack(soundTable[command*2+1]);
 		}
 	} else {
-		_lastMusicCommand = 1;
-		_sound->beginFadeOut();
+		static const int8 soundTable[] = {
+			-1,   0,  -1,   1,   0,   3,   0,   2,
+			 0,   4,   1,   2,   1,   3,   1,   4,
+			 1,  92,   1,   6,   1,   7,   2,   2,
+			 2,   3,   2,   4,   2,   5,   2,   6,
+			 2,   7,   3,   3,   3,   4,   1,   8,
+			 1,   9,   4,   2,   4,   3,   4,   4,
+			 4,   5,   4,   6,   4,   7,   4,   8,
+			 1,  11,   1,  12,   1,  14,   1,  13,
+			 4,   9,   5,  12,   6,   2,   6,   6,
+			 6,   7,   6,   8,   6,   9,   6,   3,
+			 6,   4,   6,   5,   7,   2,   7,   3,
+			 7,   4,   7,   5,   7,   6,   7,   7,
+			 7,   8,   7,   9,   8,   2,   8,   3,
+			 8,   4,   8,   5,   6,  11,   5,  11
+		};
+		//if (!_disableSound) {
+		//	XXX
+		//}
+		assert(command*2+1 < ARRAYSIZE(soundTable));
+		if (_curMusicTheme != soundTable[command*2]+1) {
+			if (soundTable[command*2] != -1)
+				snd_playTheme(soundTable[command*2]+1);
+		}
+	
+		if (command != 1) {
+			if (_lastMusicCommand != command) {
+				_sound->haltTrack();
+				_sound->playTrack(soundTable[command*2+1]);
+			}
+		} else {
+			_sound->beginFadeOut();
+		}
 	}
+
+	_lastMusicCommand = command;
 }
 
 void KyraEngine::snd_playVoiceFile(int id) {
@@ -494,27 +526,37 @@ void KyraEngine::snd_playVoiceFile(int id) {
 void KyraEngine::snd_voiceWaitForFinish(bool ingame) {
 	debugC(9, kDebugLevelMain | kDebugLevelSound, "KyraEngine::snd_voiceWaitForFinish(%d)", ingame);
 	while (_sound->voiceIsPlaying() && !_skipFlag) {
-		if (ingame) {
+		if (ingame)
 			delay(10, true);
-		} else {
+		else
 			_system->delayMillis(10);
-		}
 	}
 }
 
+void KyraEngine::snd_stopVoice() {
+	debugC(9, kDebugLevelMain | kDebugLevelSound, "KyraEngine::snd_stopVoice()");
+	_sound->voiceStop();
+}
+
+bool KyraEngine::snd_voiceIsPlaying() {
+	debugC(9, kDebugLevelMain | kDebugLevelSound, "KyraEngine::snd_voiceIsPlaying()");
+	return _sound->voiceIsPlaying();
+}
+
 // static res
 
 const Sound::SpeechCodecs Sound::_supportedCodes[] = {
-#ifdef USE_MAD
-	{ ".VO3", Audio::makeMP3Stream },
-#endif // USE_MAD
-#ifdef USE_VORBIS
-	{ ".VOG", Audio::makeVorbisStream },
-#endif // USE_VORBIS
 #ifdef USE_FLAC
 	{ ".VOF", Audio::makeFlacStream },
 #endif // USE_FLAC
+#ifdef USE_VORBIS
+	{ ".VOG", Audio::makeVorbisStream },
+#endif // USE_VORBIS
+#ifdef USE_MAD
+	{ ".VO3", Audio::makeMP3Stream },
+#endif // USE_MAD
 	{ 0, 0 }
 };
 
 } // end of namespace Kyra
+
