@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,15 +18,30 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/kyra/sound_adlib.cpp $
- * $Id: sound_adlib.cpp 22844 2006-06-02 22:57:02Z lordhoto $
+ * LPGL License
+ *
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
+
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
+
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+ *
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/kyra/sound_adlib.cpp $
+ * $Id: sound_adlib.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
 #include "common/stdafx.h"
 #include "common/system.h"
 #include "common/mutex.h"
-#include "common/timer.h"
 #include "kyra/resource.h"
 #include "kyra/sound.h"
 
@@ -40,7 +58,7 @@ namespace Kyra {
 
 class AdlibDriver : public Audio::AudioStream {
 public:
-	AdlibDriver(Audio::Mixer *mixer);
+	AdlibDriver(Audio::Mixer *mixer, bool v2);
 	~AdlibDriver();
 
 	int callback(int opcode, ...);
@@ -74,6 +92,8 @@ public:
 	bool endOfData() const { return false; }
 	int getRate() const { return _mixer->getOutputRate(); }
 
+	void setSyncJumpMask(uint16 mask) { _syncJumpMask = mask; }
+
 private:
 	struct OpcodeEntry {
 		typedef int (AdlibDriver::*DriverOpcode)(va_list &list);
@@ -128,6 +148,7 @@ private:
 	// unk41 - Sound-effect. Used for primaryEffect2()
 
 	struct Channel {
+		bool lock;	// New to ScummVM
 		uint8 opExtraLevel2;
 		uint8 *dataptr;
 		uint8 duration;
@@ -218,7 +239,7 @@ private:
 	}
 
 	uint8 *getInstrument(int instrumentId) {
-		return _soundData + READ_LE_UINT16(_soundData + 500 + 2 * instrumentId);
+		return _soundData + READ_LE_UINT16(_soundData + (_v2 ? 1000 : 500) + 2 * instrumentId);
 	}
 
 	void setupPrograms();
@@ -379,17 +400,24 @@ private:
 	static const uint8 _unkTable2_3[];
 	static const uint8 _unkTables[][32];
 
+	uint16 _syncJumpMask;
+
 	Common::Mutex _mutex;
 	Audio::Mixer *_mixer;
+	Audio::SoundHandle _soundHandle;
+
+	bool _v2;
 
 	void lock() { _mutex.lock(); }
 	void unlock() { _mutex.unlock(); }
 };
 
-AdlibDriver::AdlibDriver(Audio::Mixer *mixer) {
+AdlibDriver::AdlibDriver(Audio::Mixer *mixer, bool v2) {
 	setupOpcodeList();
 	setupParserOpcodeTable();
 
+	_v2 = v2;
+
 	_mixer = mixer;
 
 	_flags = 0;
@@ -416,16 +444,18 @@ AdlibDriver::AdlibDriver(Audio::Mixer *mixer) {
 
 	_tablePtr1 = _tablePtr2 = 0;
 
-	_mixer->setupPremix(this);
+	_mixer->playInputStream(Audio::Mixer::kPlainSoundType, &_soundHandle, this, -1, Audio::Mixer::kMaxChannelVolume, 0, false, true);
 
 	_samplesPerCallback = getRate() / CALLBACKS_PER_SECOND;
 	_samplesPerCallbackRemainder = getRate() % CALLBACKS_PER_SECOND;
 	_samplesTillCallback = 0;
 	_samplesTillCallbackRemainder = 0;
+
+	_syncJumpMask = 0;
 }
 
 AdlibDriver::~AdlibDriver() {
-	_mixer->setupPremix(0);
+	_mixer->stopHandle(_soundHandle);
 	OPLDestroy(_adlib);
 	_adlib = 0;
 }
@@ -526,9 +556,8 @@ int AdlibDriver::snd_unkOpcode3(va_list &list) {
 		Channel &channel = _channels[_curChannel];
 		channel.priority = 0;
 		channel.dataptr = 0;
-		if (value != 9) {
+		if (value != 9)
 			noteOff(channel);
-		}
 		++value;
 	}
 
@@ -629,6 +658,11 @@ void AdlibDriver::setupPrograms() {
 			unkOutput2(chan);
 		}
 
+		// What we have set up now is, probably, the controlling
+		// channel for the sound. It is assumed that this program will
+		// set up all the other channels it needs, clearing their locks
+		// along the way.
+
 		++_lastProcessed;
 		_lastProcessed &= 0x0F;
 	}
@@ -671,21 +705,43 @@ void AdlibDriver::setupPrograms() {
 void AdlibDriver::executePrograms() {
 	// Each channel runs its own program. There are ten channels: One for
 	// each Adlib channel (0-8), plus one "control channel" (9) which is
-	// the one that tells the other channels what to do. 
+	// the one that tells the other channels what to do.
+
+	// This is where we ensure that channels that are made to jump "in
+	// sync" do so.
+
+	if (_syncJumpMask) {
+		bool forceUnlock = true;
+
+		for (_curChannel = 9; _curChannel >= 0; --_curChannel) {
+			if ((_syncJumpMask & (1 << _curChannel)) == 0)
+				continue;
+
+			if (_channels[_curChannel].dataptr && !_channels[_curChannel].lock)
+				forceUnlock = false;
+		}
+
+		if (forceUnlock) {
+			for (_curChannel = 9; _curChannel >= 0; --_curChannel)
+				if (_syncJumpMask & (1 << _curChannel))
+					_channels[_curChannel].lock = false;
+		}
+	}
 
 	for (_curChannel = 9; _curChannel >= 0; --_curChannel) {
 		int result = 1;
 
-		if (!_channels[_curChannel].dataptr) {
+		if (!_channels[_curChannel].dataptr)
 			continue;
-		}
-	
+
+		if (_channels[_curChannel].lock && (_syncJumpMask & (1 << _curChannel)))
+			continue;
+
 		Channel &channel = _channels[_curChannel];
 		_curRegOffset = _regOffset[_curChannel];
 
-		if (channel.tempoReset) {
+		if (channel.tempoReset)
 			channel.tempo = _tempo;
-		}
 
 		uint8 backup = channel.position;
 		channel.position += channel.tempo;
@@ -785,6 +841,7 @@ void AdlibDriver::initChannel(Channel &channel) {
 	channel.primaryEffect = 0;
 	channel.secondaryEffect = 0;
 	channel.spacing1 = 1;
+	channel.lock = false;
 }
 
 void AdlibDriver::noteOff(Channel &channel) {
@@ -872,9 +929,8 @@ void AdlibDriver::setupDuration(uint8 duration, Channel &channel) {
 		channel.duration = duration + (getRandomNr() & channel.durationRandomness);
 		return;
 	}
-	if (channel.fractionalSpacing) {
+	if (channel.fractionalSpacing)
 		channel.spacing2 = (duration >> 3) * channel.fractionalSpacing;
-	}
 	channel.duration = duration;
 }
 
@@ -1164,9 +1220,8 @@ void AdlibDriver::secondaryEffect1(Channel &channel) {
 	uint8 temp = channel.unk18;
 	channel.unk18 += channel.unk19;
 	if (channel.unk18 < temp) {
-		if (--channel.unk21 < 0) {
+		if (--channel.unk21 < 0)
 			channel.unk21 = channel.unk20;
-		}
 		writeOPL(channel.unk22 + _curRegOffset, _soundData[channel.offset + channel.unk21]);
 	}
 }
@@ -1247,6 +1302,8 @@ int AdlibDriver::update_jump(uint8 *&dataptr, Channel &channel, uint8 value) {
 	--dataptr;
 	int16 add = READ_LE_UINT16(dataptr); dataptr += 2;
 	dataptr += add;
+	if (_syncJumpMask & (1 << (&channel - _channels)))
+		channel.lock = true;
 	return 0;
 }
 
@@ -1270,9 +1327,8 @@ int AdlibDriver::update_setBaseOctave(uint8 *&dataptr, Channel &channel, uint8 v
 
 int AdlibDriver::update_stopChannel(uint8 *&dataptr, Channel &channel, uint8 value) {
 	channel.priority = 0;
-	if (_curChannel != 9) {
+	if (_curChannel != 9)
 		noteOff(channel);
-	}
 	dataptr = 0;
 	return 2;
 }
@@ -1322,9 +1378,8 @@ int AdlibDriver::update_waitForEndOfProgram(uint8 *&dataptr, Channel &channel, u
 	uint8 *ptr = getProgram(value);
 	uint8 chan = *ptr;
 
-	if (!_channels[chan].dataptr) {
+	if (!_channels[chan].dataptr)
 		return 0;
-	}
 
 	dataptr -= 2;
 	return 2;
@@ -1388,9 +1443,8 @@ int AdlibDriver::updateCallback24(uint8 *&dataptr, Channel &channel, uint8 value
 		}
 	}
 
-	if (!(value & _unkValue4)) {
+	if (!(value & _unkValue4))
 		++_unkValue5;
-	}
 
 	dataptr -= 2;
 	channel.duration = 1;
@@ -2136,8 +2190,10 @@ const uint8 AdlibDriver::_unkTables[][32] = {
 
 #pragma mark -
 
-// At the time of writing, the only known case where Kyra 1 uses sound triggers
-// is in the castle, to cycle between three different songs.
+// Kyra 1 sound triggers. Most noticeably, these are used towards the end of
+// the game, in the castle, to cycle between different songs. The same music is
+// used in other places throughout the game, but the player is less likely to
+// spend enough time there to notice.
 
 const int SoundAdlibPC::_kyra1SoundTriggers[] = {
 	0, 4, 5, 3
@@ -2145,17 +2201,24 @@ const int SoundAdlibPC::_kyra1SoundTriggers[] = {
 
 const int SoundAdlibPC::_kyra1NumSoundTriggers = ARRAYSIZE(SoundAdlibPC::_kyra1SoundTriggers);
 
-SoundAdlibPC::SoundAdlibPC(Audio::Mixer *mixer, KyraEngine *engine)
-	: Sound(engine, mixer), _driver(0), _trackEntries(), _soundDataPtr(0) {
+SoundAdlibPC::SoundAdlibPC(KyraEngine *vm, Audio::Mixer *mixer)
+	: Sound(vm, mixer), _driver(0), _trackEntries(), _soundDataPtr(0) {
 	memset(_trackEntries, 0, sizeof(_trackEntries));
-	_driver = new AdlibDriver(mixer);
+	_v2 = (_vm->gameFlags().gameID == GI_KYRA2);
+	_driver = new AdlibDriver(mixer, _v2);
 	assert(_driver);
 
 	_sfxPlayingSound = -1;
-	_soundFileLoaded = "";
+	_soundFileLoaded = (uint)-1;
 
-	_soundTriggers = _kyra1SoundTriggers;
-	_numSoundTriggers = _kyra1NumSoundTriggers;
+	if (_v2) {
+		// TODO: Figure out if Kyra 2 uses sound triggers at all.
+		_soundTriggers = 0;
+		_numSoundTriggers = 0;
+	} else {
+		_soundTriggers = _kyra1SoundTriggers;
+		_numSoundTriggers = _kyra1NumSoundTriggers;
+	}
 }
 
 SoundAdlibPC::~SoundAdlibPC() {
@@ -2175,9 +2238,8 @@ void SoundAdlibPC::process() {
 	if (trigger < _numSoundTriggers) {
 		int soundId = _soundTriggers[trigger];
 
-		if (soundId) {
+		if (soundId)
 			playTrack(soundId);
-		}
 	} else {
 		warning("Unknown sound trigger %d", trigger);
 		// TODO: At this point, we really want to clear the trigger...
@@ -2191,19 +2253,25 @@ int SoundAdlibPC::getVolume() {
 	return 0;
 }
 
-void SoundAdlibPC::loadMusicFile(const char *file) {
-	loadSoundFile(file);
-}
-
 void SoundAdlibPC::playTrack(uint8 track) {
-	if (_musicEnabled)
+	if (_musicEnabled) {
+		// WORKAROUND: There is a bug in the Kyra 1 "Pool of Sorrow"
+		// music which causes the channels to get progressively out of
+		// sync for each loop. To avoid that, we declare that all four
+		// of the song channels have to jump "in sync".
+
+		if (track == 4 && scumm_stricmp(soundFilename(_soundFileLoaded), "KYRA1B") == 0)
+			_driver->setSyncJumpMask(0x000F);
+		else
+			_driver->setSyncJumpMask(0);
 		play(track);
+	}
 }
 
 void SoundAdlibPC::haltTrack() {
 	unk1();
 	unk2();
-	//_engine->_system->delayMillis(3 * 60);
+	//_vm->_system->delayMillis(3 * 60);
 }
 
 void SoundAdlibPC::playSoundEffect(uint8 track) {
@@ -2212,14 +2280,21 @@ void SoundAdlibPC::playSoundEffect(uint8 track) {
 }
 
 void SoundAdlibPC::play(uint8 track) {
-	uint8 soundId = _trackEntries[track];
-	if ((int8)soundId == -1 || !_soundDataPtr)
+	uint16 soundId = 0;
+	
+	if (_v2)
+		soundId = READ_LE_UINT16(&_trackEntries[track<<1]);
+	else
+		soundId = _trackEntries[track];
+
+	if ((soundId == 0xFFFF && _v2) || (soundId == 0xFF && !_v2) || !_soundDataPtr)
 		return;
-	soundId &= 0xFF;
+
 	while ((_driver->callback(16, 0) & 8)) {
 		// We call the system delay and not the game delay to avoid concurrency issues.
-		_engine->_system->delayMillis(10);
+		_vm->_system->delayMillis(10);
 	}
+
 	if (_sfxPlayingSound != -1) {
 		// Restore the sounds's normal values.
 		_driver->callback(10, _sfxPlayingSound, int(1), int(_sfxPriority));
@@ -2259,20 +2334,19 @@ void SoundAdlibPC::beginFadeOut() {
 	playSoundEffect(1);
 }
 
-void SoundAdlibPC::loadSoundFile(const char *file) {
+void SoundAdlibPC::loadSoundFile(uint file) {
 	if (_soundFileLoaded == file)
 		return;
 
-	if (_soundDataPtr) {
+	if (_soundDataPtr)
 		haltTrack();
-	}
 
 	uint8 *file_data = 0; uint32 file_size = 0;
 
 	char filename[25];
-	sprintf(filename, "%s.ADL", file);
+	sprintf(filename, "%s.ADL", soundFilename(file));
 
-	file_data = _engine->resource()->fileData(filename, &file_size);
+	file_data = _vm->resource()->fileData(filename, &file_size);
 	if (!file_data) {
 		warning("Couldn't find music file: '%s'", filename);
 		return;
@@ -2284,11 +2358,18 @@ void SoundAdlibPC::loadSoundFile(const char *file) {
 	_driver->callback(8, int(-1));
 	_soundDataPtr = 0;
 
+	int soundDataSize = file_size;
 	uint8 *p = file_data;
-	memcpy(_trackEntries, p, 120*sizeof(uint8));
-	p += 120;
 
-	int soundDataSize = file_size - 120;
+	if (_v2) {
+		memcpy(_trackEntries, p, 500);
+		p += 500;
+		soundDataSize -= 500;
+	} else {
+		memcpy(_trackEntries, p, 120);
+		p += 120;
+		soundDataSize -= 120;
+	}
 
 	_soundDataPtr = new uint8[soundDataSize];
 	assert(_soundDataPtr);
@@ -2306,7 +2387,7 @@ void SoundAdlibPC::loadSoundFile(const char *file) {
 
 void SoundAdlibPC::unk1() {
 	playSoundEffect(0);
-	//_engine->_system->delayMillis(5 * 60);
+	//_vm->_system->delayMillis(5 * 60);
 }
 
 void SoundAdlibPC::unk2() {
@@ -2314,3 +2395,4 @@ void SoundAdlibPC::unk2() {
 }
 
 } // end of namespace Kyra
+
