@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/kyra/sound_digital.cpp $
- * $Id: sound_digital.cpp 22807 2006-05-31 19:55:29Z anotherguest $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/kyra/sound_digital.cpp $
+ * $Id: sound_digital.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -79,20 +82,9 @@ const int8 AUDStream::WSTable4Bit[] = {
 	 0,  1,  2,  3,  4,  5,  6,  8
 };
 
-AUDStream::AUDStream(Common::File *file, bool loop) : _file(0), _endOfData(true), _rate(0),
+AUDStream::AUDStream(Common::File *file, bool loop) : _file(file), _endOfData(true), _rate(0),
 	_processedSize(0), _totalSize(0), _bytesLeft(0), _outBuffer(0),
 	_outBufferOffset(0), _outBufferSize(0), _inBuffer(0), _inBufferSize(0) {
-#if defined(__SYMBIAN32__)
-	// Symbian can't share filehandles between different threads.
-	// So create a new file and seek that to the other filehandle's
-	// position
-	_file = new Common::File;
-	_file->open(file->name());
-	_file->seek(file->pos());
-#else
-	_file = file;
-#endif
-	_file->incRef();
 	
 	_rate = _file->readUint16LE();
 	_totalSize = _file->readUint32LE();
@@ -119,12 +111,7 @@ AUDStream::AUDStream(Common::File *file, bool loop) : _file(0), _endOfData(true)
 AUDStream::~AUDStream() {
 	delete [] _outBuffer;
 	delete [] _inBuffer;
-
-	if (_file)
-		_file->decRef();
-#ifdef __SYMBIAN32__
 	delete _file;
-#endif
 }
 
 void AUDStream::beginFadeIn() {
@@ -337,9 +324,8 @@ SoundDigital::SoundDigital(KyraEngine *vm, Audio::Mixer *mixer) : _vm(vm), _mixe
 }
 
 SoundDigital::~SoundDigital() {
-	for (int i = 0; i < SOUND_STREAMS; ++i) {
+	for (int i = 0; i < SOUND_STREAMS; ++i)
 		stopSound(i);
-	}
 }
 
 bool SoundDigital::init() {
@@ -353,7 +339,7 @@ int SoundDigital::playSound(Common::File *fileHandle, bool loop, bool fadeIn, in
 		use = &_sounds[channel];
 	} else {
 		for (channel = 0; channel < SOUND_STREAMS; ++channel) {
-			if (!_sounds[channel].fileHandle) {
+			if (!_sounds[channel].stream) {
 				use = &_sounds[channel];
 				break;
 			}
@@ -361,6 +347,7 @@ int SoundDigital::playSound(Common::File *fileHandle, bool loop, bool fadeIn, in
 		
 		if (!use) {
 			warning("no free sound channel");
+			delete fileHandle;
 			return -1;
 		}
 	}
@@ -368,7 +355,7 @@ int SoundDigital::playSound(Common::File *fileHandle, bool loop, bool fadeIn, in
 	use->stream = new AUDStream(fileHandle, loop);
 	if (use->stream->endOfData()) {
 		delete use->stream;
-		delete fileHandle;
+		use->stream = 0;
 
 		return -1;
 	}
@@ -378,7 +365,6 @@ int SoundDigital::playSound(Common::File *fileHandle, bool loop, bool fadeIn, in
 	
 	// TODO: set correct sound type from channel id
 	_mixer->playInputStream(Audio::Mixer::kPlainSoundType, &use->handle, use->stream);
-	use->fileHandle = fileHandle;
 	
 	return use - _sounds;
 }
@@ -393,20 +379,15 @@ bool SoundDigital::isPlaying(int channel) {
 }
 
 void SoundDigital::stopSound(int channel) {
-	if (isPlaying(channel)) {
-		_mixer->stopHandle(_sounds[channel].handle);
-	}
-	
-	if (_sounds[channel].fileHandle) {
-		delete _sounds[channel].fileHandle;
-		_sounds[channel].fileHandle = 0;
-	}
+	assert(channel >= 0 && channel < SOUND_STREAMS);
+	_mixer->stopHandle(_sounds[channel].handle);
+	_sounds[channel].stream = 0;
 }
 
 void SoundDigital::beginFadeOut(int channel) {
-	if (isPlaying(channel)) {
+	if (isPlaying(channel))
 		_sounds[channel].stream->beginFadeOut();
-	}
 }
 
 } // end of namespace Kyra
+
