@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2004-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,13 +18,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/kyra/sprites.h $
- * $Id: sprites.h 20483 2006-02-11 00:17:20Z lordhoto $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/kyra/sprites.h $
+ * $Id: sprites.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
-#ifndef KYRASPRITES_H
-#define KYRASPRITES_H
+#ifndef KYRA_SPRITES_H
+#define KYRA_SPRITES_H
 
 namespace Kyra {
 
@@ -36,6 +39,7 @@ struct Sprite {
 
 struct Anim {
 	uint8 *script;
+	uint8 *curPos;
 	uint16 length;
 	int16 x;
 	int16 y;
@@ -43,8 +47,8 @@ struct Anim {
 	int8 sprite;
 	uint8 *loopStart;
 	uint16 loopsLeft;
-	uint8 *reentry;
 	uint32 nextRun;
+	uint32 lastRefresh;
 	bool play;
 	uint16 width;
 	uint16 height;
@@ -60,12 +64,12 @@ struct Anim {
 class Sprites {
 public:
 
-	Sprites(KyraEngine *engine, OSystem *system);
+	Sprites(KyraEngine *vm, OSystem *system);
 	~Sprites();
 
 	void updateSceneAnims();
 	void setupSceneAnims();
-	void loadDAT(const char *filename, SceneExits &exits);
+	void loadDat(const char *filename, SceneExits &exits);
 	void loadSceneShapes();
 	
 	Anim _anims[MAX_NUM_ANIMS];
@@ -79,13 +83,12 @@ public:
 protected:
 	void freeSceneShapes();
 
-	KyraEngine *_engine;
+	KyraEngine *_vm;
 	Resource *_res;
 	OSystem *_system;
 	Screen *_screen;
 	uint8 *_dat;
 	Common::RandomSource _rnd;
-	uint8 _animDelay;
 	uint8 *_spriteDefStart;
 	uint8 _drawLayerTable[8];
 };
@@ -93,3 +96,4 @@ protected:
 } // End of namespace Kyra
 
 #endif
+
