@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2004-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,47 +18,108 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/kyra/staticres.cpp $
- * $Id: staticres.cpp 24460 2006-10-22 20:06:52Z lordhoto $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/kyra/staticres.cpp $
+ * $Id: staticres.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
 #include "common/stdafx.h"
 #include "common/endian.h"
+#include "common/md5.h"
 #include "kyra/kyra.h"
+#include "kyra/kyra_v2.h"
+#include "kyra/kyra_v3.h"
 #include "kyra/screen.h"
 #include "kyra/resource.h"
 
 namespace Kyra {
 
-#define RESFILE_VERSION 12
-#define KYRADAT_FILESIZE 67227
+#define RESFILE_VERSION 17
 
 bool StaticResource::checkKyraDat() {
 	Common::File kyraDat;
 	if (!kyraDat.open("KYRA.DAT"))
 		return false;
 	
-	if (kyraDat.size() != KYRADAT_FILESIZE)
+	uint32 size = kyraDat.size() - 16;
+	uint8 digest[16];
+	kyraDat.seek(size, SEEK_SET);
+	if (kyraDat.read(digest, 16) != 16)
 		return false;
+	kyraDat.close();
 	
+	uint8 digestCalc[16];
+	if (!Common::md5_file("KYRA.DAT", digestCalc, size))
+		return false;
+
+	for (int i = 0; i < 16; ++i)
+		if (digest[i] != digestCalc[i])
+			return false;
 	return true;
 }
 
-#define GAME_FLAGS (GF_FLOPPY | GF_TALKIE | GF_DEMO | GF_AUDIOCD)
-#define LANGUAGE_FLAGS (GF_ENGLISH | GF_FRENCH | GF_GERMAN | GF_SPANISH | GF_ITALIAN | GF_LNGUNK)
+// used for the KYRA.DAT file which still uses
+// the old flag system, we just convert it, which
+// is less work than to change KYRA.DAT again
+enum {
+	GF_FLOPPY	= 1 <<  0,
+	GF_TALKIE	= 1 <<  1,
+	GF_FMTOWNS	= 1 <<  2,
+	GF_DEMO		= 1 <<  3,
+	GF_ENGLISH	= 1 <<  4,
+	GF_FRENCH	= 1 <<  5,
+	GF_GERMAN	= 1 <<  6,
+	GF_SPANISH	= 1 <<  7,
+	GF_ITALIAN	= 1 <<  8,
+	GF_JAPANESE = 1 <<  9,
+	// other languages here
+	GF_LNGUNK	= 1 << 16,	// also used for multi language in kyra3
+	GF_AMIGA	= 1 << 17
+};
+
+#define GAME_FLAGS (GF_FLOPPY | GF_TALKIE | GF_DEMO | GF_FMTOWNS | GF_AMIGA)
+#define LANGUAGE_FLAGS (GF_ENGLISH | GF_FRENCH | GF_GERMAN | GF_SPANISH | GF_ITALIAN | GF_JAPANESE | GF_LNGUNK)
+
+uint32 createFeatures(const GameFlags &flags) {
+	if (flags.isTalkie)
+		return GF_TALKIE;
+	if (flags.isDemo)
+		return GF_DEMO;
+	if (flags.platform == Common::kPlatformFMTowns)
+		return GF_FMTOWNS;
+	if (flags.platform == Common::kPlatformAmiga)
+		return GF_AMIGA;
+	return GF_FLOPPY;
+}
+
+uint32 createLanguage(const GameFlags &flags) {
+	if (flags.lang == Common::EN_ANY)
+		return GF_ENGLISH;
+	if (flags.lang == Common::DE_DEU)
+		return GF_GERMAN;
+	if (flags.lang == Common::FR_FRA)
+		return GF_FRENCH;
+	if (flags.lang == Common::ES_ESP)
+		return GF_SPANISH;
+	if (flags.lang == Common::IT_ITA)
+		return GF_ITALIAN;
+	if (flags.lang == Common::JA_JPN)
+		return GF_JAPANESE;
+	return GF_LNGUNK;
+}
 
 struct LanguageTypes {
 	uint32 flags;
 	const char *ext;
 };
 
-static LanguageTypes languages[] = {
+static const LanguageTypes languages[] = {
 	{ GF_ENGLISH, "ENG" },	// this is the default language
 	{ GF_FRENCH, "FRE" },
 	{ GF_GERMAN, "GER" },
 	{ GF_SPANISH, "SPA" },
 	{ GF_ITALIAN, "ITA" },
+	{ GF_JAPANESE, "JPN" },
 	{ 0, 0 }
 };
 
@@ -145,14 +209,18 @@ bool StaticResource::init() {
 
 		// PALETTE table
 		{ kPaletteList, kPaletteTable, "1 33 PALTABLE" },
+		
+		// FM-TOWNS specific
+		{ kKyra1TownsSFXTable, kRawData, "SFXTABLE" },
+		{ kCreditsStrings, kRawData, "CREDITS" },
 
 		{ 0, 0, 0 }
 	};
 
-	if (_engine->game() == GI_KYRA1) {
+	if (_vm->game() == GI_KYRA1) {
 		_builtIn = 0;
 		_filenameTable = kyra1StaticRes;
-	} else if (_engine->game() == GI_KYRA2 || _engine->game() == GI_KYRA3) {
+	} else if (_vm->game() == GI_KYRA2 || _vm->game() == GI_KYRA3) {
 		return true;
 	} else {
 		error("unknown game ID");
@@ -172,20 +240,18 @@ bool StaticResource::init() {
 	delete [] temp;
 	temp = 0;
 	
-	if (version < RESFILE_VERSION) {
+	if (version != RESFILE_VERSION)
 		error("invalid KYRA.DAT file version (%d, required %d)", version, RESFILE_VERSION);
-	}
-	if (gameID != _engine->game()) {
+	if (gameID != _vm->game())
 		error("invalid game id (%d)", gameID);
-	}
-	if ((featuresValue & GAME_FLAGS) != (_engine->features() & GAME_FLAGS)) {
-		error("your data file has a different game flags (0x%.08X has the data and your version has 0x%.08X)", (featuresValue & GAME_FLAGS), (_engine->features() & GAME_FLAGS));
-	}
+
+	uint32 gameFeatures = createFeatures(_vm->gameFlags());
+	if ((featuresValue & GAME_FLAGS) != gameFeatures)
+		error("your data file has a different game flags (0x%.08X has the data and your version has 0x%.08X)", (featuresValue & GAME_FLAGS), gameFeatures);
 
 	// load all tables for now
-	if (!prefetchId(-1)) {
+	if (!prefetchId(-1))
 		error("couldn't load all needed resources from 'KYRA.DAT'");
-	}
 	return true;
 }
 
@@ -218,21 +284,18 @@ const uint8 * const*StaticResource::loadPaletteTable(int id, int &entries) {
 
 bool StaticResource::prefetchId(int id) {
 	if (id == -1) {
-		for (int i = 0; _filenameTable[i].filename; ++i) {
+		for (int i = 0; _filenameTable[i].filename; ++i)
 			prefetchId(_filenameTable[i].id);
-		}
 		return true;
 	}
 	const void *ptr = 0;
 	int type = -1, size = -1;
 
-	if (checkResList(id, type, ptr, size)) {
+	if (checkResList(id, type, ptr, size))
 		return true;
-	}
 
-	if (checkForBuiltin(id, type, size)) {
+	if (checkForBuiltin(id, type, size))
 		return true;
-	}
 
 	const FilenameTable *filename = searchFile(id);
 	if (!filename)
@@ -244,9 +307,8 @@ bool StaticResource::prefetchId(int id) {
 	ResData data;
 	data.id = id;
 	data.type = filetype->type;
-	if (!(this->*(filetype->load))(filename->filename, data.data, data.size)) {
+	if (!(this->*(filetype->load))(filename->filename, data.data, data.size))
 		return false;
-	}
 	_resList.push_back(data);
 
 	return true;
@@ -309,9 +371,8 @@ const StaticResource::FileType *StaticResource::getFiletype(int type) {
 		return 0;
 
 	for (int i = 0; _fileLoader[i].load; ++i) {
-		if (_fileLoader[i].type == type) {
+		if (_fileLoader[i].type == type)
 			return &_fileLoader[i];
-		}
 	}
 
 	return 0;
@@ -349,9 +410,8 @@ const void *StaticResource::getData(int id, int requesttype, int &size) {
 bool StaticResource::loadLanguageTable(const char *filename, void *&ptr, int &size) {
 	char file[64];
 	for (int i = 0; languages[i].ext; ++i) {
-		if (languages[i].flags != (_engine->features() & LANGUAGE_FLAGS)) {
+		if (languages[i].flags != createLanguage(_vm->gameFlags()))
 			continue;
-		}
 			
 		strcpy(file, filename);
 		strcat(file, languages[i].ext);
@@ -365,7 +425,7 @@ bool StaticResource::loadLanguageTable(const char *filename, void *&ptr, int &si
 		static bool warned = false;
 		if (!warned) {
 			warned = true;
-			warning("coudln't find specific language table for your version, using English now");
+			warning("couldn't find specific language table for your version, using English now");
 		}
 		return true;
 	}
@@ -509,9 +569,8 @@ void StaticResource::freeRawData(void *&ptr, int &size) {
 
 void StaticResource::freeStringTable(void *&ptr, int &size) {
 	char **data = (char**)ptr;
-	while (size--) {
+	while (size--)
 		delete [] data[size];
-	}
 	ptr = 0;
 	size = 0;
 }
@@ -532,9 +591,8 @@ void StaticResource::freeRoomTable(void *&ptr, int &size) {
 
 void StaticResource::freePaletteTable(void *&ptr, int &size) {
 	uint8 **data = (uint8**)ptr;
-	while (size--) {
+	while (size--)
 		delete [] data[size];
-	}
 	ptr = 0;
 	size = 0;
 }
@@ -542,14 +600,17 @@ void StaticResource::freePaletteTable(void *&ptr, int &size) {
 uint8 *StaticResource::getFile(const char *name, int &size) {
 	char buffer[64];
 	const char *ext = "";
-	if (_engine->features() & GF_TALKIE) {
+	if (_vm->gameFlags().isTalkie)
 		ext = ".CD";
-	} else if (_engine->features() & GF_DEMO) {
+	else if (_vm->gameFlags().isDemo)
 		ext = ".DEM";
-	}
+	else if (_vm->gameFlags().platform == Common::kPlatformFMTowns)
+		ext = ".TNS";
+	else if (_vm->gameFlags().platform == Common::kPlatformAmiga)
+		ext = ".AMG";
 	snprintf(buffer, 64, "%s%s", name, ext);
 	uint32 tempSize = 0;
-	uint8 *data = _engine->resource()->fileData(buffer, &tempSize);
+	uint8 *data = _vm->resource()->fileData(buffer, &tempSize);
 	size = tempSize;
 	return data;
 }
@@ -650,16 +711,16 @@ void KyraEngine::initStaticResource() {
 void KyraEngine::loadMouseShapes() {
 	_screen->loadBitmap("MOUSE.CPS", 3, 3, 0);
 	_screen->_curPage = 2;
-	_shapes[4] = _screen->encodeShape(0, 0, 8, 10, 0);
-	_shapes[5] = _screen->encodeShape(0, 0x17, 0x20, 7, 0);
-	_shapes[6] = _screen->encodeShape(0x50, 0x12, 0x10, 9, 0);
-	_shapes[7] = _screen->encodeShape(0x60, 0x12, 0x10, 11, 0);
-	_shapes[8] = _screen->encodeShape(0x70, 0x12, 0x10, 9, 0);
-	_shapes[9] = _screen->encodeShape(0x80, 0x12, 0x10, 11, 0);
-	_shapes[10] = _screen->encodeShape(0x90, 0x12, 0x10, 10, 0);
-	_shapes[364] = _screen->encodeShape(0x28, 0, 0x10, 13, 0);
+	_shapes[0] = _screen->encodeShape(0, 0, 8, 10, 0);
+	_shapes[1] = _screen->encodeShape(0, 0x17, 0x20, 7, 0);
+	_shapes[2] = _screen->encodeShape(0x50, 0x12, 0x10, 9, 0);
+	_shapes[3] = _screen->encodeShape(0x60, 0x12, 0x10, 11, 0);
+	_shapes[4] = _screen->encodeShape(0x70, 0x12, 0x10, 9, 0);
+	_shapes[5] = _screen->encodeShape(0x80, 0x12, 0x10, 11, 0);
+	_shapes[6] = _screen->encodeShape(0x90, 0x12, 0x10, 10, 0);
+	_shapes[360] = _screen->encodeShape(0x28, 0, 0x10, 13, 0);
 	_screen->setMouseCursor(1, 1, 0);
-	_screen->setMouseCursor(1, 1, _shapes[4]);
+	_screen->setMouseCursor(1, 1, _shapes[0]);
 	_screen->setShapePages(5, 3);
 }
 
@@ -671,7 +732,7 @@ void KyraEngine::loadCharacterShapes() {
 		assert(i < _defaultShapeTableSize);
 		Shape *shape = &_defaultShapeTable[i];
 		if (shape->imageIndex == 0xFF) {
-			_shapes[i+7+4] = 0;
+			_shapes[i+7] = 0;
 			continue;
 		}
 		if (shape->imageIndex != curImage) {
@@ -679,7 +740,7 @@ void KyraEngine::loadCharacterShapes() {
 			_screen->loadBitmap(_characterImageTable[shape->imageIndex], 3, 3, 0);
 			curImage = shape->imageIndex;
 		}
-		_shapes[i+7+4] = _screen->encodeShape(shape->x<<3, shape->y, shape->w<<3, shape->h, 1);
+		_shapes[i+7] = _screen->encodeShape(shape->x<<3, shape->y, shape->w<<3, shape->h, 1);
 	}
 	_screen->_curPage = videoPage;
 }
@@ -690,16 +751,16 @@ void KyraEngine::loadSpecialEffectShapes() {
  
 	int currShape; 
 	for (currShape = 173; currShape < 183; currShape++)
-		_shapes[4 + currShape] = _screen->encodeShape((currShape-173) * 24, 0, 24, 24, 1);
+		_shapes[currShape] = _screen->encodeShape((currShape-173) * 24, 0, 24, 24, 1);
  
 	for (currShape = 183; currShape < 190; currShape++)
-		_shapes[4 + currShape] = _screen->encodeShape((currShape-183) * 24, 24, 24, 24, 1);
+		_shapes[currShape] = _screen->encodeShape((currShape-183) * 24, 24, 24, 24, 1);
  
 	for (currShape = 190; currShape < 201; currShape++)
-		_shapes[4 + currShape] = _screen->encodeShape((currShape-190) * 24, 48, 24, 24, 1);
+		_shapes[currShape] = _screen->encodeShape((currShape-190) * 24, 48, 24, 24, 1);
  
 	for (currShape = 201; currShape < 206; currShape++)
-		_shapes[4 + currShape] = _screen->encodeShape((currShape-201) * 16, 106, 16, 16, 1);
+		_shapes[currShape] = _screen->encodeShape((currShape-201) * 16, 106, 16, 16, 1);
 }
 
 void KyraEngine::loadItems() {
@@ -708,28 +769,28 @@ void KyraEngine::loadItems() {
 	_screen->loadBitmap("JEWELS3.CPS", 3, 3, 0);
 	_screen->_curPage = 2;
 
-	_shapes[327] = 0;
+	_shapes[323] = 0;
 
 	for (shape = 1; shape < 6; shape++ )
-		_shapes[327 + shape] = _screen->encodeShape((shape - 1) * 32, 0, 32, 17, 0);
+		_shapes[323 + shape] = _screen->encodeShape((shape - 1) * 32, 0, 32, 17, 0);
 
 	for (shape = 330; shape <= 334; shape++)
-		_shapes[4 + shape] = _screen->encodeShape((shape-330) * 32, 102, 32, 17, 0);
+		_shapes[shape] = _screen->encodeShape((shape-330) * 32, 102, 32, 17, 0);
 
 	for (shape = 335; shape <= 339; shape++)
-		_shapes[4 + shape] = _screen->encodeShape((shape-335) * 32, 17,  32, 17, 0);
+		_shapes[shape] = _screen->encodeShape((shape-335) * 32, 17,  32, 17, 0);
 
 	for (shape = 340; shape <= 344; shape++)
-		_shapes[4 + shape] = _screen->encodeShape((shape-340) * 32, 34,  32, 17, 0);
+		_shapes[shape] = _screen->encodeShape((shape-340) * 32, 34,  32, 17, 0);
 
 	for (shape = 345; shape <= 349; shape++)
-		_shapes[4 + shape] = _screen->encodeShape((shape-345) * 32, 51,  32, 17, 0);
+		_shapes[shape] = _screen->encodeShape((shape-345) * 32, 51,  32, 17, 0);
 
 	for (shape = 350; shape <= 354; shape++)
-		_shapes[4 + shape] = _screen->encodeShape((shape-350) * 32, 68,  32, 17, 0);
+		_shapes[shape] = _screen->encodeShape((shape-350) * 32, 68,  32, 17, 0);
 
 	for (shape = 355; shape <= 359; shape++)
-		_shapes[4 + shape] = _screen->encodeShape((shape-355) * 32, 85,  32, 17, 0);
+		_shapes[shape] = _screen->encodeShape((shape-355) * 32, 85,  32, 17, 0);
 
 
 	_screen->loadBitmap("ITEMS.CPS", 3, 3, 0);
@@ -739,9 +800,9 @@ void KyraEngine::loadItems() {
 		shape = findDuplicateItemShape(i);
 
 		if (shape != -1)
-			_shapes[220 + i] = _shapes[220 + shape];
+			_shapes[216 + i] = _shapes[216 + shape];
 		else
-			_shapes[220 + i] = _screen->encodeShape( (i % 20) * 16, i/20 * 16, 16, 16, 0);
+			_shapes[216 + i] = _screen->encodeShape( (i % 20) * 16, i/20 * 16, 16, 16, 0);
 	}
 
 	uint32 size;
@@ -771,21 +832,24 @@ void KyraEngine::loadButtonShapes() {
 void KyraEngine::loadMainScreen(int page) {
 	_screen->clearPage(page);
 
-	if ((_features & GF_ENGLISH) && (_features & GF_FLOPPY))
-		_screen->loadBitmap("MAIN15.CPS", page, page, 0);
-	else if ((_features & GF_ENGLISH) && (_features & GF_TALKIE)) 
+	if (_flags.lang == Common::EN_ANY && !_flags.isTalkie && (_flags.platform == Common::kPlatformPC || _flags.platform == Common::kPlatformAmiga))
+		_screen->loadBitmap("MAIN15.CPS", page, page, _screen->getPalette(0));
+	else if (_flags.lang == Common::EN_ANY || _flags.lang == Common::JA_JPN) 
 		_screen->loadBitmap("MAIN_ENG.CPS", page, page, 0);
-	else if(_features & GF_FRENCH)
+	else if (_flags.lang == Common::FR_FRA)
 		_screen->loadBitmap("MAIN_FRE.CPS", page, page, 0);
-	else if(_features & GF_GERMAN)
+	else if (_flags.lang == Common::DE_DEU)
 		_screen->loadBitmap("MAIN_GER.CPS", page, page, 0);
-	else if (_features & GF_SPANISH)
+	else if (_flags.lang == Common::ES_ESP)
 		_screen->loadBitmap("MAIN_SPA.CPS", page, page, 0);
-	else if (_features & GF_ITALIAN)
+	else if (_flags.lang == Common::IT_ITA)
 		_screen->loadBitmap("MAIN_ITA.CPS", page, page, 0);
 	else
 		warning("no main graphics file found");
 
+	if (_flags.platform == Common::kPlatformAmiga)
+		memcpy(_screen->getPalette(1), _screen->getPalette(0), 32*3);
+
 	_screen->copyRegion(0, 0, 0, 0, 320, 200, page, 0);
 }
 
@@ -814,213 +878,7 @@ const ScreenDim Screen::_screenDimTableK3[] = {
 
 const int Screen::_screenDimTableCountK3 = ARRAYSIZE(_screenDimTableK3);
 
-#define Opcode(x) &KyraEngine::x
-void KyraEngine::setupOpcodeTable() {
-	static OpcodeProc opcodeTable[] = {
-		// 0x00
-		Opcode(cmd_magicInMouseItem),
-		Opcode(cmd_characterSays),
-		Opcode(cmd_pauseTicks),
-		Opcode(cmd_drawSceneAnimShape),
-		// 0x04
-		Opcode(cmd_queryGameFlag),
-		Opcode(cmd_setGameFlag),
-		Opcode(cmd_resetGameFlag),
-		Opcode(cmd_runNPCScript),
-		// 0x08
-		Opcode(cmd_setSpecialExitList),
-		Opcode(cmd_blockInWalkableRegion),
-		Opcode(cmd_blockOutWalkableRegion),
-		Opcode(cmd_walkPlayerToPoint),
-		// 0x0c
-		Opcode(cmd_dropItemInScene),
-		Opcode(cmd_drawAnimShapeIntoScene),
-		Opcode(cmd_createMouseItem),
-		Opcode(cmd_savePageToDisk),
-		// 0x10
-		Opcode(cmd_sceneAnimOn),
-		Opcode(cmd_sceneAnimOff),
-		Opcode(cmd_getElapsedSeconds),
-		Opcode(cmd_mouseIsPointer),
-		// 0x14
-		Opcode(cmd_destroyMouseItem),
-		Opcode(cmd_runSceneAnimUntilDone),
-		Opcode(cmd_fadeSpecialPalette),
-		Opcode(cmd_playAdlibSound),
-		// 0x18
-		Opcode(cmd_playAdlibScore),
-		Opcode(cmd_phaseInSameScene),
-		Opcode(cmd_setScenePhasingFlag),
-		Opcode(cmd_resetScenePhasingFlag),
-		// 0x1c
-		Opcode(cmd_queryScenePhasingFlag),
-		Opcode(cmd_sceneToDirection),
-		Opcode(cmd_setBirthstoneGem),
-		Opcode(cmd_placeItemInGenericMapScene),
-		// 0x20
-		Opcode(cmd_setBrandonStatusBit),
-		Opcode(cmd_pauseSeconds),
-		Opcode(cmd_getCharactersLocation),
-		Opcode(cmd_runNPCSubscript),
-		// 0x24
-		Opcode(cmd_magicOutMouseItem),
-		Opcode(cmd_internalAnimOn),
-		Opcode(cmd_forceBrandonToNormal),
-		Opcode(cmd_poisonDeathNow),
-		// 0x28
-		Opcode(cmd_setScaleMode),
-		Opcode(cmd_openWSAFile),
-		Opcode(cmd_closeWSAFile),
-		Opcode(cmd_runWSAFromBeginningToEnd),
-		// 0x2c
-		Opcode(cmd_displayWSAFrame),
-		Opcode(cmd_enterNewScene),
-		Opcode(cmd_setSpecialEnterXAndY),
-		Opcode(cmd_runWSAFrames),
-		// 0x30
-		Opcode(cmd_popBrandonIntoScene),
-		Opcode(cmd_restoreAllObjectBackgrounds),
-		Opcode(cmd_setCustomPaletteRange),
-		Opcode(cmd_loadPageFromDisk),
-		// 0x34
-		Opcode(cmd_customPrintTalkString),
-		Opcode(cmd_restoreCustomPrintBackground),
-		Opcode(cmd_hideMouse),
-		Opcode(cmd_showMouse),
-		// 0x38
-		Opcode(cmd_getCharacterX),
-		Opcode(cmd_getCharacterY),
-		Opcode(cmd_changeCharactersFacing),
-		Opcode(cmd_copyWSARegion),
-		// 0x3c
-		Opcode(cmd_printText),
-		Opcode(cmd_random),
-		Opcode(cmd_loadSoundFile),
-		Opcode(cmd_displayWSAFrameOnHidPage),
-		// 0x40
-		Opcode(cmd_displayWSASequentialFrames),
-		Opcode(cmd_drawCharacterStanding),
-		Opcode(cmd_internalAnimOff),
-		Opcode(cmd_changeCharactersXAndY),
-		// 0x44
-		Opcode(cmd_clearSceneAnimatorBeacon),
-		Opcode(cmd_querySceneAnimatorBeacon),
-		Opcode(cmd_refreshSceneAnimator),
-		Opcode(cmd_placeItemInOffScene),
-		// 0x48
-		Opcode(cmd_wipeDownMouseItem),
-		Opcode(cmd_placeCharacterInOtherScene),
-		Opcode(cmd_getKey),
-		Opcode(cmd_specificItemInInventory),
-		// 0x4c
-		Opcode(cmd_popMobileNPCIntoScene),
-		Opcode(cmd_mobileCharacterInScene),
-		Opcode(cmd_hideMobileCharacter),
-		Opcode(cmd_unhideMobileCharacter),
-		// 0x50
-		Opcode(cmd_setCharactersLocation),
-		Opcode(cmd_walkCharacterToPoint),
-		Opcode(cmd_specialEventDisplayBrynnsNote),
-		Opcode(cmd_specialEventRemoveBrynnsNote),
-		// 0x54
-		Opcode(cmd_setLogicPage),
-		Opcode(cmd_fatPrint),
-		Opcode(cmd_preserveAllObjectBackgrounds),
-		Opcode(cmd_updateSceneAnimations),
-		// 0x58
-		Opcode(cmd_sceneAnimationActive),
-		Opcode(cmd_setCharactersMovementDelay),
-		Opcode(cmd_getCharactersFacing),
-		Opcode(cmd_bkgdScrollSceneAndMasksRight),
-		// 0x5c
-		Opcode(cmd_dispelMagicAnimation),
-		Opcode(cmd_findBrightestFireberry),
-		Opcode(cmd_setFireberryGlowPalette),
-		Opcode(cmd_setDeathHandlerFlag),
-		// 0x60
-		Opcode(cmd_drinkPotionAnimation),
-		Opcode(cmd_makeAmuletAppear),
-		Opcode(cmd_drawItemShapeIntoScene),
-		Opcode(cmd_setCharactersCurrentFrame),
-		// 0x64
-		Opcode(cmd_waitForConfirmationMouseClick),
-		Opcode(cmd_pageFlip),
-		Opcode(cmd_setSceneFile),
-		Opcode(cmd_getItemInMarbleVase),
-		// 0x68
-		Opcode(cmd_setItemInMarbleVase),
-		Opcode(cmd_addItemToInventory),
-		Opcode(cmd_intPrint),
-		Opcode(cmd_shakeScreen),
-		// 0x6c
-		Opcode(cmd_createAmuletJewel),
-		Opcode(cmd_setSceneAnimCurrXY),
-		Opcode(cmd_poisonBrandonAndRemaps),
-		Opcode(cmd_fillFlaskWithWater),
-		// 0x70
-		Opcode(cmd_getCharactersMovementDelay),
-		Opcode(cmd_getBirthstoneGem),
-		Opcode(cmd_queryBrandonStatusBit),
-		Opcode(cmd_playFluteAnimation),
-		// 0x74
-		Opcode(cmd_playWinterScrollSequence),
-		Opcode(cmd_getIdolGem),
-		Opcode(cmd_setIdolGem),
-		Opcode(cmd_totalItemsInScene),
-		// 0x78
-		Opcode(cmd_restoreBrandonsMovementDelay),
-		Opcode(cmd_setMousePos),
-		Opcode(cmd_getMouseState),
-		Opcode(cmd_setEntranceMouseCursorTrack),
-		// 0x7c
-		Opcode(cmd_itemAppearsOnGround),
-		Opcode(cmd_setNoDrawShapesFlag),
-		Opcode(cmd_fadeEntirePalette),
-		Opcode(cmd_itemOnGroundHere),
-		// 0x80
-		Opcode(cmd_queryCauldronState),
-		Opcode(cmd_setCauldronState),
-		Opcode(cmd_queryCrystalState),
-		Opcode(cmd_setCrystalState),
-		// 0x84
-		Opcode(cmd_setPaletteRange),
-		Opcode(cmd_shrinkBrandonDown),
-		Opcode(cmd_growBrandonUp),
-		Opcode(cmd_setBrandonScaleXAndY),
-		// 0x88
-		Opcode(cmd_resetScaleMode),
-		Opcode(cmd_getScaleDepthTableValue),
-		Opcode(cmd_setScaleDepthTableValue),
-		Opcode(cmd_message),
-		// 0x8c
-		Opcode(cmd_checkClickOnNPC),
-		Opcode(cmd_getFoyerItem),
-		Opcode(cmd_setFoyerItem),
-		Opcode(cmd_setNoItemDropRegion),
-		// 0x90
-		Opcode(cmd_walkMalcolmOn),
-		Opcode(cmd_passiveProtection),
-		Opcode(cmd_setPlayingLoop),
-		Opcode(cmd_brandonToStoneSequence),
-		// 0x94
-		Opcode(cmd_brandonHealingSequence),
-		Opcode(cmd_protectCommandLine),
-		Opcode(cmd_pauseMusicSeconds),
-		Opcode(cmd_resetMaskRegion),
-		// 0x98
-		Opcode(cmd_setPaletteChangeFlag),
-		Opcode(cmd_fillRect),
-		Opcode(cmd_vocUnload),
-		Opcode(cmd_vocLoad),
-		Opcode(cmd_dummy)
-	};
-	
-	_opcodeTable = opcodeTable;
-	_opcodeTableSize = ARRAYSIZE(opcodeTable);
-}
-#undef Opcode
-
-const char *KyraEngine::_musicFiles[] = {
+const char *KyraEngine::_soundFiles[] = {
 	"INTRO",
 	"KYRA1A",
 	"KYRA1B",
@@ -1033,7 +891,18 @@ const char *KyraEngine::_musicFiles[] = {
 	"KYRAMISC"
 };
 
-const int KyraEngine::_musicFilesCount = ARRAYSIZE(_musicFiles);
+const int KyraEngine::_soundFilesCount = ARRAYSIZE(KyraEngine::_soundFiles);
+
+const char *KyraEngine::_soundFilesTowns[] = {
+	"TW_INTRO.SFX",
+	"TW_SCEN1.SFX",
+	"TW_SCEN2.SFX",
+	"TW_SCEN3.SFX",
+	"TW_SCEN4.SFX",
+	"TW_SCEN5.SFX",
+};
+
+const int KyraEngine::_soundFilesTownsCount = ARRAYSIZE(KyraEngine::_soundFilesTowns);
 
 const int8 KyraEngine::_charXPosTable[] = {
 	 0,  4,  4,  4,  0, -4, -4, -4
@@ -1228,8 +1097,9 @@ const uint16 KyraEngine::_amuletY[] = { 170, 170, 159, 181 };
 const uint16 KyraEngine::_amuletX2[] = { 0x000, 0x0FD, 0x0E7, 0x0FD, 0x113, 0x000 };
 const uint16 KyraEngine::_amuletY2[] = { 0x000, 0x09F, 0x0AA, 0x0B5, 0x0AA, 0x000 };
 
-// kyra 3 static res
-const char *KyraEngine_v3::_mainMenuStrings[] = {
+// Kyra 2 and 3 main menu
+
+const char *KyraEngine::_mainMenuStrings[] = {
 	"Start a new game",
 	"Introduction",
 	"Load a game",
@@ -1245,4 +1115,144 @@ const char *KyraEngine_v3::_mainMenuStrings[] = {
 	0
 };
 
+// kyra 2 static res
+
+const char *KyraEngine_v2::_introStrings[] = {
+	"Kyrandia is disappearing!",
+	"Rock by rock...",
+	"...and tree by tree.",
+	"Kyrandia ceases to exist!",
+	"The Royal Mystics are baffled.",
+	"Every reference has been consulted.",
+	"Even Marko and his new valet have been allowed into the conference.",
+	"Luckily, the Hand was experienced in these matters.",
+	"And finally a plan was approved...",
+	"...that required a magic Anchor Stone...",
+	"...to be retrieved from the center of the world.",
+	"Zanthia, youngest of the Kyrandian Mystics, has been selected to retrieve the Stone.",
+	"Thank you for playing The Hand of Fate.",
+	"This should be enough blueberries to open a portal to the center of the world.",
+	" DUMMY STRING... ",
+	" DUMMY STRING... ",
+	"Hey! All my equipment has been stolen!",
+	" DUMMY STRING... ",
+	"If they think I'm going to walk all the way down there, they're nuts!",
+	" DUMMY STRING... ",
+	"Hurry up faun!"
+};
+
+const int KyraEngine_v2::_introStringsSize = ARRAYSIZE(KyraEngine_v2::_introStrings);
+
+const char *KyraEngine_v2::_introSoundList[] = {
+	"eintro1",
+	"eintro2",
+	"eintro3",
+	"eintro4",
+	"eintro5",
+	"eintro6",
+	"eintro7",
+	"eintro8",
+	"eintro9",
+	"eintro10",
+	"eintro11",
+	"eintro12",
+	"eglow",
+	"0000210",
+	"0000130",
+	"0000180",
+	"0000160",
+	"asong",
+	"crowcaw",
+	"eyerub2",
+	"pluck3",
+	"rodnreel",
+	"frog1",
+	"scavmov2",
+	"lambmom3",
+	"lambkid1",
+	"thunder2",
+	"tunder3",
+	"wind6",
+	"h2odrop2",
+	"gasleak",
+	"polgulp1",
+	"hndslap1",
+	"burp1",
+	"0000220",
+	"0000230",
+	"0000250",
+	"0000260",
+	"0000270",
+	"0000280",
+	"0000290",
+	"0000300",
+	"0000310",
+	"0000320",
+	"0000330",
+	"scream1",
+	"theend"
+};
+
+const int KyraEngine_v2::_introSoundListSize = ARRAYSIZE(KyraEngine_v2::_introSoundList);
+
+// kyra 3 static res
+
+const char *KyraEngine_v3::_soundList[] = {
+	"ARREST1.AUD",
+	"BATH1.AUD",
+	"OCEAN1.AUD",
+	"CLOWN1.AUD",
+	"DARM2.AUD",
+	"FALL1M.AUD",
+	"FALL2.AUD",
+	"FISH1.AUD",
+	"FISHWNDR.AUD",
+	"HERMAN1.AUD",
+	"JAIL1.AUD",
+	"JUNGLE1.AUD",
+	"KATHY1.AUD",
+	"NICESINE.AUD",
+	"PEGASUS1.AUD",
+	"PIRATE1.AUD",
+	"PIRATE2.AUD",
+	"PIRATE3.AUD",
+	"POP3.AUD",
+	"PORT1.AUD",
+	"QUEEN1.AUD",
+	"RUINS1.AUD",
+	"SNAKES1.AUD",
+	"SPRING1.AUD",
+	"STATUE1.AUD",
+	"STATUE2.AUD",
+	"TITLE1.AUD",
+	"UNDER1.AUD",
+	"WALKCHP1.AUD",
+	"YANK1.AUD",
+	"ZAN2.AUD",
+	"GROOVE2.AUD",
+	"GROOVE3.AUD",
+	"KING1.AUD",
+	"KING2.AUD",
+	"GROOVE1.AUD",
+	"JAIL2.AUD",
+	"SPIRIT1.AUD",
+	"SPRING1A.AUD",
+	"POP1.AUD",
+	"POP2.AUD",
+	"SQUIRL1.AUD"
+};
+
+const int KyraEngine_v3::_soundListSize = ARRAYSIZE(KyraEngine_v3::_soundList);
+
+const char *KyraEngine_v3::_languageExtension[] = {
+	"TRE",
+	"TRF",
+	"TRG"/*,
+	"TRI",		Italian and Spanish was never included
+	"TRS"*/
+};
+
+const int KyraEngine_v3::_languageExtensionSize = ARRAYSIZE(KyraEngine_v3::_languageExtension);
+
 } // End of namespace Kyra
+
