@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2005-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/kyra/text.cpp $
- * $Id: text.cpp 22769 2006-05-30 12:13:34Z eriktorbjorn $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/kyra/text.cpp $
+ * $Id: text.cpp 27083 2007-06-04 19:00:41Z lordhoto $
  *
  */
 
@@ -28,18 +31,21 @@
 #include "kyra/animator.h"
 #include "kyra/sprites.h"
 
+#include "common/events.h"
 #include "common/system.h"
+#include "common/endian.h"
 
 namespace Kyra {
 
-void KyraEngine::waitForChatToFinish(int16 chatDuration, const char *chatStr, uint8 charNum) {
+void KyraEngine::waitForChatToFinish(int vocFile, int16 chatDuration, const char *chatStr, uint8 charNum) {
 	debugC(9, kDebugLevelMain, "KyraEngine::waitForChatToFinish(%i, %s, %i)", chatDuration, chatStr, charNum); 
 	bool hasUpdatedNPCs = false;
 	bool runLoop = true;
+	bool drawText = textEnabled();
 	uint8 currPage;
-	OSystem::Event event;
+	Common::Event event;
 
-	//while( towns_isEscKeyPressed() )
+	//while (towns_isEscKeyPressed() )
 		//towns_getKey();
 
 	uint32 timeToEnd = strlen(chatStr) * 8 * _tickLength + _system->getMillis();
@@ -61,6 +67,11 @@ void KyraEngine::waitForChatToFinish(int16 chatDuration, const char *chatStr, ui
 	if (chatDuration != -1)
 		chatDuration *= _tickLength;
 
+	if (vocFile != -1) {
+		snd_voiceWaitForFinish();
+		snd_playVoiceFile(vocFile);
+	}
+
 	disableTimer(14);
 	disableTimer(18);
 	disableTimer(19);
@@ -93,32 +104,34 @@ void KyraEngine::waitForChatToFinish(int16 chatDuration, const char *chatStr, ui
 		_animator->preserveAnyChangedBackgrounds();
 		_animator->prepDrawAllObjects();
 
-		currPage = _screen->_curPage;
-		_screen->_curPage = 2;
-		_text->printCharacterText(chatStr, charNum, _characterList[charNum].x1);
-		_animator->_updateScreen = true;
-		_screen->_curPage = currPage;
+		if (drawText) {
+			currPage = _screen->_curPage;
+			_screen->_curPage = 2;
+			_text->printCharacterText(chatStr, charNum, _characterList[charNum].x1);
+			_animator->_updateScreen = true;
+			_screen->_curPage = currPage;
+		}
 
 		_animator->copyChangedObjectsForward(0);
 		updateTextFade();
 
-		if ((chatDuration < (int16)(_system->getMillis() - timeAtStart)) && chatDuration != -1)
+		if ((chatDuration < (int16)(_system->getMillis() - timeAtStart)) && chatDuration != -1 && (!drawText || !snd_voiceIsPlaying()))
 			break;
 
-		uint32 nextTime = loopStart + _gameSpeed;
+		uint32 nextTime = loopStart + _tickLength; 
 		
 		while (_system->getMillis() < nextTime) {
-			while (_system->pollEvent(event)) {
+			while (_eventMan->pollEvent(event)) {
 				switch (event.type) {
-				case OSystem::EVENT_KEYDOWN:
+				case Common::EVENT_KEYDOWN:
 					if (event.kbd.keycode == '.')
 						_skipFlag = true;
 					break;
-				case OSystem::EVENT_QUIT:
+				case Common::EVENT_QUIT:
 					quitGame();
 					runLoop = false;
 					break;
-				case OSystem::EVENT_LBUTTONDOWN:
+				case Common::EVENT_LBUTTONDOWN:
 					runLoop = false;
 					break;
 				default:
@@ -136,6 +149,9 @@ void KyraEngine::waitForChatToFinish(int16 chatDuration, const char *chatStr, ui
 			runLoop = false;
 	}
 
+	snd_voiceWaitForFinish();
+	snd_stopVoice();
+
 	enableTimer(14);
 	enableTimer(15);
 	enableTimer(18);
@@ -153,7 +169,8 @@ void KyraEngine::endCharacterChat(int8 charNum, int16 convoInitialized) {
 
 	if (convoInitialized != 0) {
 		_talkingCharNum = -1;
-		_currentCharacter->currentAnimFrame = 7;
+		if (_currentCharacter->currentAnimFrame != 88)
+			_currentCharacter->currentAnimFrame = 7;
 		_animator->animRefreshNPC(0);
 		_animator->updateAllObjectShapes();
 	}
@@ -167,7 +184,9 @@ void KyraEngine::restoreChatPartnerAnimFrame(int8 charNum) {
 		_animator->animRefreshNPC(charNum);
 	}
 
-	_currentCharacter->currentAnimFrame = 7;
+	if (_currentCharacter->currentAnimFrame != 88)
+		_currentCharacter->currentAnimFrame = 7;
+
 	_animator->animRefreshNPC(0);
 	_animator->updateAllObjectShapes();
 }
@@ -178,10 +197,11 @@ void KyraEngine::backupChatPartnerAnimFrame(int8 charNum) {
 	if (charNum < 5 && charNum > 0) 
 		_currentChatPartnerBackupFrame = _characterList[charNum].currentAnimFrame;
 
-	if (_scaleMode != 0)
-		_currentCharacter->currentAnimFrame = 7;
-	else
-		_currentCharacter->currentAnimFrame = _currentCharAnimFrame;
+	if (_currentCharacter->currentAnimFrame != 88) {
+		_currentCharacter->currentAnimFrame = 16;
+		if (_scaleMode != 0)
+			_currentCharacter->currentAnimFrame = 7;
+	}
 
 	_animator->animRefreshNPC(0);
 	_animator->updateAllObjectShapes();
@@ -210,13 +230,17 @@ int8 KyraEngine::getChatPartnerNum() {
 }
 
 int KyraEngine::initCharacterChat(int8 charNum) {
-	if (_talkingCharNum == -1) {
-		_talkingCharNum = 0;
+	int returnValue = 0;
 
-		if (_scaleMode != 0)
-			_currentCharacter->currentAnimFrame = 7;
-		else
+	if (_talkingCharNum == -1) {
+		returnValue = 1;
+		_talkingCharNum = 0;
+		
+		if (_currentCharacter->currentAnimFrame != 88) {
 			_currentCharacter->currentAnimFrame = 16;
+			if (_scaleMode != 0)
+				_currentCharacter->currentAnimFrame = 7;
+		}
 
 		_animator->animRefreshNPC(0);
 		_animator->updateAllObjectShapes();
@@ -236,10 +260,10 @@ int KyraEngine::initCharacterChat(int8 charNum) {
 	_animator->preserveAnyChangedBackgrounds();
 	_charSayUnk3 = charNum;
 
-	return 1;
+	return returnValue;
 }
 
-void KyraEngine::characterSays(const char *chatStr, int8 charNum, int8 chatDuration) {
+void KyraEngine::characterSays(int vocFile, const char *chatStr, int8 charNum, int8 chatDuration) {
 	debugC(9, kDebugLevelMain, "KyraEngine::characterSays('%s', %i, %d)", chatStr, charNum, chatDuration);
 	uint8 startAnimFrames[] =  { 0x10, 0x32, 0x56, 0x0, 0x0, 0x0 };
 
@@ -253,7 +277,7 @@ void KyraEngine::characterSays(const char *chatStr, int8 charNum, int8 chatDurat
 	convoInitialized = initCharacterChat(charNum);	
 	chatPartnerNum = getChatPartnerNum();
 
-	if (chatPartnerNum != -1 && chatPartnerNum < 5)
+	if (chatPartnerNum >= 0 && chatPartnerNum < 5)
 		backupChatPartnerAnimFrame(chatPartnerNum);
 
 	if (charNum < 5) {
@@ -279,32 +303,39 @@ void KyraEngine::characterSays(const char *chatStr, int8 charNum, int8 chatDurat
 
 	_text->_talkMessageY = yPos;
 	_text->_talkMessageH = lineNum * 10;
-	_animator->restoreAllObjectBackgrounds();
+	
+	if (textEnabled()) {
+		_animator->restoreAllObjectBackgrounds();
 
-	_screen->copyRegion(12, _text->_talkMessageY, 12, 136, 296, _text->_talkMessageH, 2, 2);
-	_screen->hideMouse();
+		_screen->copyRegion(12, _text->_talkMessageY, 12, 136, 308, _text->_talkMessageH, 2, 2);
+		_screen->hideMouse();
 
-	_text->printCharacterText(processedString, charNum, _characterList[charNum].x1);
-	_screen->showMouse();
+		_text->printCharacterText(processedString, charNum, _characterList[charNum].x1);
+		_screen->showMouse();
+	}
 
 	if (chatDuration == -2)
 		chatTicks = strlen(processedString) * 9;
 	else
 		chatTicks = chatDuration;
 
-	waitForChatToFinish(chatTicks, chatStr, charNum);
+	if (!speechEnabled())
+		vocFile = -1;
+	waitForChatToFinish(vocFile, chatTicks, chatStr, charNum);
 
-	_animator->restoreAllObjectBackgrounds();
+	if (textEnabled()) {
+		_animator->restoreAllObjectBackgrounds();
 
-	_screen->copyRegion(12, 136, 12, _text->_talkMessageY, 296, _text->_talkMessageH, 2, 2);
-	_animator->preserveAllBackgrounds();
-	_animator->prepDrawAllObjects();
-	_screen->hideMouse();
+		_screen->copyRegion(12, 136, 12, _text->_talkMessageY, 308, _text->_talkMessageH, 2, 2);
+		_animator->preserveAllBackgrounds();
+		_animator->prepDrawAllObjects();
+		_screen->hideMouse();
 
-	_screen->copyRegion(12, _text->_talkMessageY, 12, _text->_talkMessageY, 296, _text->_talkMessageH, 2, 0);
-	_screen->showMouse();
-	_animator->flagAllObjectsForRefresh();
-	_animator->copyChangedObjectsForward(0);
+		_screen->copyRegion(12, _text->_talkMessageY, 12, _text->_talkMessageY, 308, _text->_talkMessageH, 2, 0);
+		_screen->showMouse();
+		_animator->flagAllObjectsForRefresh();
+		_animator->copyChangedObjectsForward(0);
+	}
 
 	if (chatPartnerNum != -1 && chatPartnerNum < 5)
 		restoreChatPartnerAnimFrame(chatPartnerNum);
@@ -369,8 +400,9 @@ void KyraEngine::updateTextFade() {
 	}
 }
 
-TextDisplayer::TextDisplayer(Screen *screen) {
+TextDisplayer::TextDisplayer(KyraEngine *vm, Screen *screen) {
 	_screen = screen;
+	_vm = vm;
 
 	_talkCoords.y = 0x88;
 	_talkCoords.x = 0;
@@ -404,6 +436,12 @@ int TextDisplayer::getCharLength(const char *str, int len) {
 		Screen::FontId curFont = _screen->setFont(Screen::FID_8_FNT);
 		int i = 0;
 		while (i <= len && *str) {
+			uint c = *str++;
+			c &= 0xFF;
+			if (c >= 0x7F && _vm->gameFlags().lang == Common::JA_JPN) {
+				c = READ_LE_UINT16(str - 1);
+				++str;
+			}	
 			i += _screen->getCharWidth(*str++);
 			++charsCount;
 		}
@@ -551,12 +589,40 @@ void TextDisplayer::printTalkTextMessage(const char *text, int x, int y, uint8 c
 	_talkMessagePrinted = true;
 }
 
-void TextDisplayer::printText(const char *str, int x, int y, uint8 c0, uint8 c1, uint8 c2) {
+void TextDisplayer::printIntroTextMessage(const char *text, int x, int y, uint8 col1, uint8 col2, uint8 col3, int dstPage, Screen::FontId font) {
+	debugC(9, kDebugLevelMain, "TextDisplayer::printIntroTextMessage('%s', %d, %d, %d, %d, %d, %d, %d)",
+			text, x, y, col1, col2, col3, dstPage, font);
+	char *str = preprocessString(text);
+	int lineCount = buildMessageSubstrings(str);
+	int top = y - lineCount * 10;
+	if (top < 0) {
+		top = 0;
+	}
+	_talkMessageY = top;
+	_talkMessageH = lineCount * 10;
+	int w = getWidestLineWidth(lineCount);
+	int x1, x2;
+	calcWidestLineBounds(x1, x2, w, x);
+	_talkCoords.x = x1;
+	_talkCoords.w = w + 2;
+	int curPage = _screen->setCurPage(dstPage);
+
+	for (int i = 0; i < lineCount; ++i) {
+		top = i * 10 + _talkMessageY;
+		char *msg = &_talkSubstrings[i * TALK_SUBSTRING_LEN];
+		int left = getCenterStringX(msg, x1, x2);
+		printText(msg, left, top, col1, col2, col3, font);
+	}
+	_screen->_curPage = curPage;
+	_talkMessagePrinted = true;
+}
+
+void TextDisplayer::printText(const char *str, int x, int y, uint8 c0, uint8 c1, uint8 c2, Screen::FontId font) {
 	debugC(9, kDebugLevelMain, "TextDisplayer::printText('%s', %d, %d, %d, %d, %d)", str, x, y, c0, c1, c2);
 	uint8 colorMap[] = { 0, 15, 12, 12 };
 	colorMap[3] = c1;
 	_screen->setTextColor(colorMap, 0, 3);
-	Screen::FontId curFont = _screen->setFont(Screen::FID_8_FNT);
+	Screen::FontId curFont = _screen->setFont(font);
 	_screen->_charWidth = -2;
 	_screen->printText(str, x, y, c0, c2);
 	_screen->_charWidth = 0;
