@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2005-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,16 +18,18 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/kyra/text.h $
- * $Id: text.h 21365 2006-03-18 14:43:18Z lordhoto $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/kyra/text.h $
+ * $Id: text.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
-#ifndef KYRATEXT_H
-#define KYRATEXT_H
+#ifndef KYRA_TEXT_H
+#define KYRA_TEXT_H
 
 namespace Kyra {
 class Screen;
+class KyraEngine;
+class FontId;
 
 class TextDisplayer {
 	struct TalkCoords {
@@ -36,7 +41,7 @@ class TextDisplayer {
 		TALK_SUBSTRING_NUM = 3
 	};
 public:
-	TextDisplayer(Screen *screen);
+	TextDisplayer(KyraEngine *vm, Screen *screen);
 	~TextDisplayer() {}
 
 	void setTalkCoords(uint16 y);
@@ -49,7 +54,9 @@ public:
 	void calcWidestLineBounds(int &x1, int &x2, int w, int cx);
 	void restoreTalkTextMessageBkgd(int srcPage, int dstPage);
 	void printTalkTextMessage(const char *text, int x, int y, uint8 color, int srcPage, int dstPage);
-	void printText(const char *str, int x, int y, uint8 c0, uint8 c1, uint8 c2);
+	void printIntroTextMessage(const char *text, int x, int y, uint8 col1, uint8 col2, uint8 col3,
+			int dstPage, Screen::FontId font=Screen::FID_8_FNT);	
+	void printText(const char *str, int x, int y, uint8 c0, uint8 c1, uint8 c2, Screen::FontId font=Screen::FID_8_FNT);
 	void printCharacterText(const char *text, int8 charNum, int charX);
 
 	uint16 _talkMessageY;
@@ -57,6 +64,7 @@ public:
 	bool printed() const { return _talkMessagePrinted; }
 private:
 	Screen *_screen;
+	KyraEngine *_vm;
 
 	char _talkBuffer[300];
 	char _talkSubstrings[TALK_SUBSTRING_LEN * TALK_SUBSTRING_NUM];
