@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/kyra/vqa.cpp $
- * $Id: vqa.cpp 22569 2006-05-22 07:27:20Z eriktorbjorn $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/kyra/vqa.cpp $
+ * $Id: vqa.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -29,6 +32,7 @@
 // The jung2.vqa movie does work, but only thanks to a grotesque hack.
 
 #include "common/stdafx.h"
+#include "common/events.h"
 #include "common/system.h"
 #include "sound/audiostream.h"
 #include "sound/mixer.h"
@@ -51,7 +55,7 @@ VQAMovie::~VQAMovie() {
 
 void VQAMovie::initBuffers() {
 	for (int i = 0; i < ARRAYSIZE(_buffers); i++) {
-		_buffers[i].data = NULL;
+		_buffers[i].data = 0;
 		_buffers[i].size = 0;
 	}
 }
@@ -65,8 +69,8 @@ void *VQAMovie::allocBuffer(int num, uint32 size) {
 		 * We could use realloc() here, but we don't actually need the
 		 * old contents of the buffer.
 		 */
-		free(_buffers[num].data);
-		_buffers[num].data = malloc(size);
+		delete [] _buffers[num].data;
+		_buffers[num].data = new uint8[size];
 		_buffers[num].size = size;
 	}
 
@@ -77,7 +81,7 @@ void *VQAMovie::allocBuffer(int num, uint32 size) {
 
 void VQAMovie::freeBuffers() {
 	for (int i = 0; i < ARRAYSIZE(_buffers); i++) {
-		free(_buffers[i].data);
+		delete [] _buffers[i].data;
 		_buffers[i].data = NULL;
 		_buffers[i].size = 0;
 	}
@@ -96,77 +100,6 @@ uint32 VQAMovie::readTag() {
 	return tag;
 }
 
-// Chunk types ending in a 'Z' are decoded using this function.
-
-int VQAMovie::decodeFormat80(byte *inbuf, byte *outbuf) {
-	byte *src = inbuf;
-	byte *dst = outbuf;
-
-	while (1) {
-		int relPos, pos;
-		int count;
-		byte color;
-		int i;
-
-		byte command = *src++;
-
-		switch (command) {
-		case 0x80:
-			return dst - outbuf;
-
-		case 0xFF:
-			/* 11111111 <count> <pos> */
-			count = src[0] | (src[1] << 8);
-			pos = src[2] | (src[3] << 8);
-			src += 4;
-			for (i = 0; i < count; i++)
-				dst[i] = outbuf[i + pos];
-			break;
-
-		case 0xFE:
-			/* 11111110 <count> <color> */
-			count = src[0] | (src[1] << 8);
-			color = src[2];
-			src += 3;
-			memset(dst, color, count);
-			break;
-
-		default:
-			if (command >= 0xC0) {
-				/* 11 <count - 3> <pos> */
-				count = (command & 0x3F) + 3;
-				pos = src[0] | (src[1] << 8);
-				src += 2;
-				for (i = 0; i < count; i++)
-					dst[i] = outbuf[pos + i];
-			} else if (command >= 0x80) {
-				/* 10 <count> */
-				count = command & 0x3F;
-				memcpy(dst, src, count);
-				src += count;
-			} else {
-				/* 0 <count - 3> <relative pos> */
-				count = ((command & 0x70) >> 4) + 3;
-				relPos = ((command & 0x0F) << 8) | src[0];
-				src++;
-				for (i = 0; i < count; i++)
-					dst[i] = dst[i - relPos];
-			}
-			break;
-		}
-
-		dst += count;
-	}
-}
-
-inline int16 clip8BitSample(int16 sample) {
-	if (sample > 255)
-		return 255;
-	if (sample < 0)
-		return 0;
-	return sample;
-}
-
 void VQAMovie::decodeSND1(byte *inbuf, uint32 insize, byte *outbuf, uint32 outsize) {
 	const int8 WSTable2Bit[] = { -2, -1, 0, 1 };
 	const int8 WSTable4Bit[] = {
@@ -206,11 +139,11 @@ void VQAMovie::decodeSND1(byte *inbuf, uint32 insize, byte *outbuf, uint32 outsi
 				code = *inbuf++;
 
 				curSample += WSTable4Bit[code & 0x0f];
-				curSample = clip8BitSample(curSample);
+				curSample = CLIP<int16>(curSample, 0, 255);
 				*outbuf++ = curSample;
 
 				curSample += WSTable4Bit[code >> 4];
-				curSample = clip8BitSample(curSample);
+				curSample = CLIP<int16>(curSample, 0, 255);
 				*outbuf++ = curSample;
 
 				outsize -= 2;
@@ -221,19 +154,19 @@ void VQAMovie::decodeSND1(byte *inbuf, uint32 insize, byte *outbuf, uint32 outsi
 				code = *inbuf++;
 
 				curSample += WSTable2Bit[code & 0x03];
-				curSample = clip8BitSample(curSample);
+				curSample = CLIP<int16>(curSample, 0, 255);
 				*outbuf++ = curSample;
 
 				curSample += WSTable2Bit[(code >> 2) & 0x03];
-				curSample = clip8BitSample(curSample);
+				curSample = CLIP<int16>(curSample, 0, 255);
 				*outbuf++ = curSample;
 
 				curSample += WSTable2Bit[(code >> 4) & 0x03];
-				curSample = clip8BitSample(curSample);
+				curSample = CLIP<int16>(curSample, 0, 255);
 				*outbuf++ = curSample;
 
 				curSample += WSTable2Bit[(code >> 6) & 0x03];
-				curSample = clip8BitSample(curSample);
+				curSample = CLIP<int16>(curSample, 0, 255);
 				*outbuf++ = curSample;
 
 				outsize -= 4;
@@ -301,35 +234,30 @@ bool VQAMovie::open(const char *filename) {
 			_header.maxCBFZSize = _file.readUint32LE();
 			_header.unk5        = _file.readUint32LE();
 
-			// Version 1 VQA files have some implicit defaults
+			// Kyrandia 3 uses version 1 VQA files, and is the only
+			// known game to do so. This version of the format has
+			// some implicit default values.
 
 			if (_header.version == 1) {
-				if (_header.flags & 1) {
-					if (_header.freq == 0)
-						_header.freq = 22050;
-					if (_header.channels == 0)
-						_header.channels = 1;
-					if (_header.bits == 0)
-						_header.bits = 8;
-				}
+				if (_header.freq == 0)
+					_header.freq = 22050;
+				if (_header.channels == 0)
+					_header.channels = 1;
+				if (_header.bits == 0)
+					_header.bits = 8;
 			}
 
 			setX((Screen::SCREEN_W - _header.width) / 2);
 			setY((Screen::SCREEN_H - _header.height) / 2);
 
-			// HACK: I've only seen 8-bit mono audio in Kyra 3
-
-			assert(_header.bits == 8);
-			assert(_header.channels == 1);
-
 			_frameInfo = new uint32[_header.numFrames];
 			_frame = new byte[_header.width * _header.height];
 
-			size = 0xf00 * _header.blockW * _header.blockH;
-			_codeBook = new byte[size];
-			_partialCodeBook = new byte[size];
-			memset(_codeBook, 0, size);
-			memset(_partialCodeBook, 0, size);
+			_codeBookSize = 0xf00 * _header.blockW * _header.blockH;
+			_codeBook = new byte[_codeBookSize];
+			_partialCodeBook = new byte[_codeBookSize];
+			memset(_codeBook, 0, _codeBookSize);
+			memset(_partialCodeBook, 0, _codeBookSize);
 
 			_numVectorPointers = (_header.width / _header.blockW) * (_header.height * _header.blockH);
 			_vectorPointers = new uint16[_numVectorPointers];
@@ -339,8 +267,20 @@ bool VQAMovie::open(const char *filename) {
 			_numPartialCodeBooks = 0;
 
 			if (_header.flags & 1) {
-				// A 2-second buffer ought to be enough
-				_stream = Audio::makeAppendableAudioStream(_header.freq, Audio::Mixer::FLAG_UNSIGNED, 2 * _header.freq * _header.channels);
+				// This VQA movie has sound. Kyrandia 3 uses
+				// 8-bit sound, and so far testing indicates
+				// that it's all mono.
+				//
+				// This is good, because it means we won't have
+				// to worry about the confusing parts of the
+				// VQA spec, where 8- and 16-bit data have 
+				// different signedness and stereo sample
+				// layout varies between different games.
+
+				assert(_header.bits == 8);
+				assert(_header.channels == 1);
+
+				_stream = Audio::makeAppendableAudioStream(_header.freq, Audio::Mixer::FLAG_UNSIGNED);
 			} else {
 				_stream = NULL;
 			}
@@ -368,7 +308,8 @@ bool VQAMovie::open(const char *filename) {
 			// HACK: This flag is set in jung2.vqa, and its
 			// purpose, if it has one, is unknown. It can't be a
 			// general purpose flag, because in large movies the
-			// frame offsets can be large enough to set this flag.
+			// frame offsets can be large enough to set this flag,
+			// though of course never for the first frame.
 			//
 			// At least in my copy of Kyrandia 3, _frameInfo[0] is
 			// 0x81000098, and the desired index is 0x4716. So the
@@ -427,6 +368,7 @@ void VQAMovie::close() {
 
 		_frameInfo = NULL;
 		_frame = NULL;
+		_codeBookSize = 0;
 		_codeBook = NULL;
 		_partialCodeBook = NULL;
 		_vectorPointers = NULL;
@@ -469,9 +411,10 @@ void VQAMovie::displayFrame(uint frameNum) {
 		switch (tag) {
 		case MKID_BE('SND0'):	// Uncompressed sound
 			foundSound = true;
-			inbuf = (byte *)allocBuffer(0, size);
+			inbuf = new byte[size];
 			_file.read(inbuf, size);
-			_stream->append(inbuf, size);
+			assert(_stream);
+			_stream->queueBuffer(inbuf, size);
 			break;
 
 		case MKID_BE('SND1'):	// Compressed sound, almost like AUD
@@ -479,15 +422,18 @@ void VQAMovie::displayFrame(uint frameNum) {
 			outsize = _file.readUint16LE();
 			insize = _file.readUint16LE();
 
-			inbuf = (byte *)allocBuffer(0, insize);
+			inbuf = new byte[insize];
 			_file.read(inbuf, insize);
 
 			if (insize == outsize) {
-				_stream->append(inbuf, insize);
+				assert(_stream);
+				_stream->queueBuffer(inbuf, insize);
 			} else {
-				outbuf = (byte *)allocBuffer(1, outsize);
+				outbuf = new byte[outsize];
 				decodeSND1(inbuf, insize, outbuf, outsize);
-				_stream->append(outbuf, outsize);
+				assert(_stream);
+				_stream->queueBuffer(outbuf, outsize);
+				delete[] inbuf;
 			}
 			break;
 
@@ -513,7 +459,7 @@ void VQAMovie::displayFrame(uint frameNum) {
 				case MKID_BE('CBFZ'):	// Full codebook
 					inbuf = (byte *)allocBuffer(0, size);
 					_file.read(inbuf, size);
-					decodeFormat80(inbuf, _codeBook);
+					Screen::decodeFrame4(inbuf, _codeBook, _codeBookSize);
 					break;
 
 				case MKID_BE('CBP0'):	// Partial codebook
@@ -538,7 +484,7 @@ void VQAMovie::displayFrame(uint frameNum) {
 				case MKID_BE('CPLZ'):	// Palette
 					inbuf = (byte *)allocBuffer(0, size);
 					_file.read(inbuf, size);
-					size = decodeFormat80(inbuf, _vm->screen()->_currentPalette);
+					Screen::decodeFrame4(inbuf, _vm->screen()->_currentPalette, 768);
 					break;
 
 				case MKID_BE('VPT0'):	// Frame data
@@ -553,7 +499,7 @@ void VQAMovie::displayFrame(uint frameNum) {
 					outbuf = (byte *)allocBuffer(1, 2 * _numVectorPointers);
 
 					_file.read(inbuf, size);
-					size = decodeFormat80(inbuf, outbuf);
+					size = Screen::decodeFrame4(inbuf, outbuf, 2 * _numVectorPointers);
 
 					assert(size / 2 <= _numVectorPointers);
 
@@ -614,7 +560,7 @@ void VQAMovie::displayFrame(uint frameNum) {
 
 	if (_numPartialCodeBooks == _header.cbParts) {
 		if (_compressedCodeBook) {
-			decodeFormat80(_partialCodeBook, _codeBook);
+			Screen::decodeFrame4(_partialCodeBook, _codeBook, _codeBookSize);
 		} else {
 			memcpy(_codeBook, _partialCodeBook, _partialCodeBookSize);
 		}
@@ -633,7 +579,7 @@ void VQAMovie::play() {
 
 	startTick = _system->getMillis();
 
-	// First, handle any sound chunk that apears before the first frame.
+	// First, handle any sound chunk that appears before the first frame.
 	// At least in some versions, it will contain half a second of audio,
 	// presumably to lessen the risk of audio underflow.
 	//
@@ -659,24 +605,25 @@ void VQAMovie::play() {
 
 			switch (tag) {
 			case MKID_BE('SND0'):	// Uncompressed sound
-				inbuf = (byte *)allocBuffer(0, size);
+				inbuf = new byte[size];
 				_file.read(inbuf, size);
-				_stream->append(inbuf, size);
+				_stream->queueBuffer(inbuf, size);
 				break;
 
 			case MKID_BE('SND1'):	// Compressed sound
 				outsize = _file.readUint16LE();
 				insize = _file.readUint16LE();
 
-				inbuf = (byte *)allocBuffer(0, insize);
+				inbuf = new byte[insize];
 				_file.read(inbuf, insize);
 
 				if (insize == outsize) {
-					_stream->append(inbuf, insize);
+					_stream->queueBuffer(inbuf, insize);
 				} else {
-					outbuf = (byte *)allocBuffer(1, outsize);
+					outbuf = new byte[outsize];
 					decodeSND1(inbuf, insize, outbuf, outsize);
-					_stream->append(outbuf, outsize);
+					_stream->queueBuffer(outbuf, outsize);
+					delete[] inbuf;
 				}
 				break;
 
@@ -685,6 +632,11 @@ void VQAMovie::play() {
 				_file.seek(size, SEEK_CUR);
 				break;
 
+			case MKID_BE('CMDS'):	// Unused tag, always empty in kyra3
+				debugC(9, kDebugLevelMovie, "VQAMovie::play: skipping CMDS tag");
+				_file.seek(size, SEEK_CUR);
+				break;
+
 			default:
 				warning("VQAMovie::play: Unknown tag `%c%c%c%c'", (tag >> 24) & 0xFF, (tag >> 16) & 0xFF, (tag >> 8) & 0xFF, tag & 0xFF);
 				_file.seek(size, SEEK_CUR);
@@ -712,15 +664,16 @@ void VQAMovie::play() {
 			if (elapsedTime >= (i * 1000) / _header.frameRate)
 				break;
 
-			OSystem::Event event;
+			Common::Event event;
 
-			while (_system->pollEvent(event)) {
+			Common::EventManager *eventMan = _system->getEventManager();
+			while (eventMan->pollEvent(event)) {
 				switch (event.type) {
-				case OSystem::EVENT_KEYDOWN:
+				case Common::EVENT_KEYDOWN:
 					if (event.kbd.ascii == 27)
 						return;
 					break;
-				case OSystem::EVENT_QUIT:
+				case Common::EVENT_QUIT:
 					_vm->quitGame();
 					return;
 				default:
