@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,13 +18,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/kyra/vqa.h $
- * $Id: vqa.h 22569 2006-05-22 07:27:20Z eriktorbjorn $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/kyra/vqa.h $
+ * $Id: vqa.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
-#ifndef VQA_H
-#define VQA_H
+#ifndef KYRA_VQA_H
+#define KYRA_VQA_H
 
 class OSystem;
 
@@ -81,7 +84,7 @@ protected:
 	};
 
 	struct Buffer {
-		void *data;
+		uint8 *data;
 		uint32 size;
 	};
 
@@ -91,7 +94,6 @@ protected:
 	void *allocBuffer(int num, uint32 size);
 	void freeBuffers();
 
-	int decodeFormat80(byte *inbuf, byte *outbuf);
 	void decodeSND1(byte *inbuf, uint32 insize, byte *outbuf, uint32 outsize);
 
 	void displayFrame(uint frameNum);
@@ -100,6 +102,7 @@ protected:
 
 	VQAHeader _header;
 	uint32 *_frameInfo;
+	uint32 _codeBookSize;
 	byte *_codeBook;
 	byte *_partialCodeBook;
 	bool _compressedCodeBook;
