@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2004-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,13 +18,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/kyra/wsamovie.h $
- * $Id: wsamovie.h 22561 2006-05-21 18:16:34Z eriktorbjorn $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/kyra/wsamovie.h $
+ * $Id: wsamovie.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
-#ifndef WSAMOVIES_H
-#define WSAMOVIES_H
+#ifndef KYRA_WSAMOVIE_H
+#define KYRA_WSAMOVIE_H
 
 #include "kyra/resource.h"
 
@@ -40,7 +43,7 @@ public:
 
 	virtual bool opened() { return _opened; }
 
-	virtual void open(const char *filename, int offscreen, uint8 *palette) = 0;
+	virtual int open(const char *filename, int offscreen, uint8 *palette) = 0;
 	virtual void close() = 0;
 
 	virtual int frames() = 0;
@@ -63,21 +66,23 @@ public:
 	WSAMovieV1(KyraEngine *vm);
 	virtual ~WSAMovieV1();
 
-	virtual void open(const char *filename, int offscreen, uint8 *palette);
+	virtual int open(const char *filename, int offscreen, uint8 *palette);
 	virtual void close();
 
 	virtual int frames() { return _opened ? _numFrames : -1; }
 
 	virtual void displayFrame(int frameNum);
-protected:
-	virtual void processFrame(int frameNum, uint8 *dst);
 
 	enum WSAFlags {
 		WF_OFFSCREEN_DECODE = 0x10,
 		WF_NO_FIRST_FRAME = 0x40,
-		WF_HAS_PALETTE = 0x100
+		WF_HAS_PALETTE = 0x100,
+		WF_XOR = 0x200
 	};
 
+protected:
+	virtual void processFrame(int frameNum, uint8 *dst);
+
 	uint16 _currentFrame;
 	uint16 _numFrames;
 	uint16 _width;
@@ -90,15 +95,25 @@ protected:
 	uint8 *_frameData;
 };
 
-class KyraEngine_v3;
-
-// it could be possible that Kyrandia2 uses exactly the same format
-class WSAMovieV3 : public WSAMovieV1 {
+class WSAMovieAmiga : public WSAMovieV1 {
 public:
-	WSAMovieV3(KyraEngine_v3 *vm);
-	
-	void open(const char *filename, int unk1, uint8 *palette);
+	WSAMovieAmiga(KyraEngine *vm);
+	int open(const char *filename, int offscreen, uint8 *palette);
+	void close();
+
+	void displayFrame(int frameNum);
+private:	
+	void processFrame(int frameNum, uint8 *dst);
+
+	uint8 *_buffer;
+};
+
+class WSAMovieV2 : public WSAMovieV1 {
+public:
+	WSAMovieV2(KyraEngine *vm);
 	
+	int open(const char *filename, int unk1, uint8 *palette);
+
 	void setX(int x) { _x = x + _xAdd; }
 	void setY(int y) { _y = y + _yAdd; }
 	
@@ -108,7 +123,6 @@ public:
 	int width() const { return _width; }
 	int height() const { return _height; }
 protected:
-	KyraEngine_v3 *_vm3;
 	
 	int16 _xAdd;
 	int16 _yAdd;
@@ -117,3 +131,4 @@ protected:
 } // end of namespace Kyra
 
 #endif
+
