@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2005-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/lure/animseq.cpp $
- * $Id: animseq.cpp 21500 2006-03-29 15:59:37Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/lure/animseq.cpp $
+ * $Id: animseq.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -38,12 +41,12 @@ AnimAbortType AnimationSequence::delay(uint32 milliseconds) {
 
 	while (_system.getMillis() < delayCtr) {
 		while (events.pollEvent()) {
-			if (events.type() == OSystem::EVENT_KEYDOWN) {
+			if (events.type() == Common::EVENT_KEYDOWN) {
 				if (events.event().kbd.keycode == 27) return ABORT_END_INTRO;
 				else return ABORT_NEXT_SCENE;
-			} else if (events.type() == OSystem::EVENT_LBUTTONDOWN)
+			} else if (events.type() == Common::EVENT_LBUTTONDOWN)
 				return ABORT_NEXT_SCENE;
-			else if (events.type() == OSystem::EVENT_QUIT) 
+			else if (events.type() == Common::EVENT_QUIT) 
 				return ABORT_END_INTRO;
 		}
 
