@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2005-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/lure/debug-input.cpp $
- * $Id: debug-input.cpp 20578 2006-02-11 19:25:13Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/lure/debug-input.cpp $
+ * $Id: debug-input.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -53,7 +56,7 @@ bool get_string(char *buffer, uint32 maxSize, bool isNumeric, uint16 x, uint16 y
 		}
 
 		if (e.pollEvent()) {
-			if (e.type() == OSystem::EVENT_KEYDOWN) {
+			if (e.type() == Common::EVENT_KEYDOWN) {
 				char ch = e.event().kbd.ascii;
 				uint16 keycode = e.event().kbd.keycode;
 
