@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2005-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/lure/debug-methods.cpp $
- * $Id: debug-methods.cpp 20765 2006-02-19 04:19:09Z dreammaster $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/lure/debug-methods.cpp $
+ * $Id: debug-methods.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -72,7 +75,7 @@ void showActiveHotspots() {
 		++s;
 	}
 
-	Surface *surface = Surface::newDialog(300, numLines, lines);
+	Surface *surface = Surface::newDialog(300, numLines, (const char**)lines);
 	mouse.cursorOff();
 	surface->copyToScreen(10, 40);
 	events.waitForPress();
@@ -117,7 +120,7 @@ void showRoomHotspots() {
 		}
 	}
 
-	Surface *surface = Surface::newDialog(300, numLines, lines);
+	Surface *surface = Surface::newDialog(300, numLines, (const char**)lines);
 	mouse.cursorOff();
 	surface->copyToScreen(10, 40);
 	events.waitForPress();
