@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2005-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/lure/disk.cpp $
- * $Id: disk.cpp 22272 2006-05-01 22:27:56Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/lure/disk.cpp $
+ * $Id: disk.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -62,7 +65,10 @@ uint8 Disk::indexOf(uint16 id, bool suppressError) {
 	}
 	
 	if (suppressError) return 0xff;
-	error("Could not find entry Id #%d in file disk%d.vga", id, _fileNum);
+	if (_fileNum == 0) 
+		error("Could not find entry Id #%d in file %s", id, SUPPORT_FILENAME);
+	else
+		error("Could not find entry Id #%d in file disk%d.vga", id, _fileNum);
 }
 
 void Disk::openFile(uint8 fileNum) {
