@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2005-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,15 +18,17 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/lure/events.cpp $
- * $Id: events.cpp 22639 2006-05-25 22:51:42Z eriktorbjorn $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/lure/events.cpp $
+ * $Id: events.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
+#include "common/stdafx.h"
+#include "common/events.h"
+
 #include "graphics/cursorman.h"
 
 #include "lure/events.h"
-#include "lure/system.h"
 #include "lure/res.h"
 
 namespace Lure {
@@ -48,21 +53,21 @@ Mouse::Mouse() {
 Mouse::~Mouse() {
 }
 
-void Mouse::handleEvent(OSystem::Event event) {
+void Mouse::handleEvent(Common::Event event) {
 	_x = (int16) event.mouse.x;
 	_y = (int16) event.mouse.y;
 
 	switch (event.type) {
-	case OSystem::EVENT_LBUTTONDOWN:
+	case Common::EVENT_LBUTTONDOWN:
 		_lButton = true;
 		break;
-	case OSystem::EVENT_LBUTTONUP:
+	case Common::EVENT_LBUTTONUP:
 		_lButton = false;
 		break;
-	case OSystem::EVENT_RBUTTONDOWN:
+	case Common::EVENT_RBUTTONDOWN:
 		_rButton = true;
 		break;
-	case OSystem::EVENT_RBUTTONUP:
+	case Common::EVENT_RBUTTONUP:
 		_rButton = false;
 		break;
 	default:
@@ -97,17 +102,38 @@ void Mouse::setCursorNum(uint8 cursorNum, int hotspotX, int hotspotY) {
 	CursorMan.replaceCursor(cursorAddr, CURSOR_WIDTH, CURSOR_HEIGHT, hotspotX, hotspotY, 0);
 }
 
+void Mouse::pushCursorNum(uint8 cursorNum) {
+	int hotspotX = 7, hotspotY = 7;
+	if ((cursorNum == CURSOR_ARROW) || (cursorNum == CURSOR_MENUBAR)) {
+		hotspotX = 0; 
+		hotspotY = 0;
+	}
+
+	pushCursorNum(cursorNum, hotspotX, hotspotY);
+}
+
+void Mouse::pushCursorNum(uint8 cursorNum, int hotspotX, int hotspotY) {
+	Resources &res = Resources::getReference();
+
+	_cursorNum = cursorNum;
+	byte *cursorAddr = res.getCursor(cursorNum);
+	CursorMan.pushCursor(cursorAddr, CURSOR_WIDTH, CURSOR_HEIGHT, hotspotX, hotspotY, 0);
+}
+
+void Mouse::popCursor() {
+	CursorMan.popCursor();
+}
+
 void Mouse::setPosition(int newX, int newY) {
-	System::getReference().warpMouse(newX, newY);
+	g_system->warpMouse(newX, newY);
 }
 
 void Mouse::waitForRelease() {
-	OSystem &system = System::getReference();
 	Events &e = Events::getReference();
 
 	do {
 		e.pollEvent();
-		system.delayMillis(20);
+		g_system->delayMillis(20);
 	} while (!e.quitFlag && (lButton() || rButton()));
 }
 
@@ -126,21 +152,21 @@ Events &Events::getReference() {
 
 
 bool Events::pollEvent() {
-	if (!System::getReference().pollEvent(_event)) return false;
+	if (!g_system->getEventManager()->pollEvent(_event)) return false;
 
 	// Handle keypress
 	switch (_event.type) {
-	case OSystem::EVENT_QUIT:
+	case Common::EVENT_QUIT:
 		quitFlag = true;
 		break;
 
-	case OSystem::EVENT_LBUTTONDOWN:
-	case OSystem::EVENT_LBUTTONUP:
-	case OSystem::EVENT_RBUTTONDOWN:
-	case OSystem::EVENT_RBUTTONUP:
-	case OSystem::EVENT_MOUSEMOVE:
-	case OSystem::EVENT_WHEELUP:
-	case OSystem::EVENT_WHEELDOWN:
+	case Common::EVENT_LBUTTONDOWN:
+	case Common::EVENT_LBUTTONUP:
+	case Common::EVENT_RBUTTONDOWN:
+	case Common::EVENT_RBUTTONUP:
+	case Common::EVENT_MOUSEMOVE:
+	case Common::EVENT_WHEELUP:
+	case Common::EVENT_WHEELDOWN:
 		Mouse::getReference().handleEvent(_event);
 		break;
 
@@ -152,19 +178,18 @@ bool Events::pollEvent() {
 }
 
 void Events::waitForPress() {
-	OSystem &system = System::getReference();
 	bool keyButton = false;
 	while (!keyButton) {
 		if (pollEvent()) {
-			if (_event.type == OSystem::EVENT_QUIT) return;
-			else if (_event.type == OSystem::EVENT_KEYDOWN) keyButton = true;
-			else if ((_event.type == OSystem::EVENT_LBUTTONDOWN) ||
-				(_event.type == OSystem::EVENT_RBUTTONDOWN)) {
+			if (_event.type == Common::EVENT_QUIT) return;
+			else if (_event.type == Common::EVENT_KEYDOWN) keyButton = true;
+			else if ((_event.type == Common::EVENT_LBUTTONDOWN) ||
+				(_event.type == Common::EVENT_RBUTTONDOWN)) {
 				keyButton = true;
 				Mouse::getReference().waitForRelease();				
 			}
 		}
-		system.delayMillis(20);
+		g_system->delayMillis(20);
 	}
 }
 
