@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2005-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,15 +18,16 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/lure/events.h $
- * $Id: events.h 20914 2006-02-26 13:43:50Z eriktorbjorn $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/lure/events.h $
+ * $Id: events.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
-#ifndef __lure_events_h__
-#define __lure_events_h__
+#ifndef LURE_EVENTS_H
+#define LURE_EVENTS_H
 
 #include "common/stdafx.h"
+#include "common/events.h"
 #include "common/str.h"
 #include "lure/luredefs.h"
 #include "lure/disk.h"
@@ -39,7 +43,7 @@ public:
 	Mouse();
 	~Mouse();
 	static Mouse &getReference(); 
-	void handleEvent(OSystem::Event event);
+	void handleEvent(Common::Event event);
 
 	void cursorOn();
 	void cursorOff();
@@ -52,11 +56,14 @@ public:
 	bool lButton() { return _lButton; }
 	bool rButton() { return _rButton; }
 	void waitForRelease();
+	void pushCursorNum(uint8 cursorNum);
+	void pushCursorNum(uint8 cursorNum, int hotspotX, int hotspotY);	
+	void popCursor();
 };
 
 class Events {
 private:
-	OSystem::Event _event;
+	Common::Event _event;
 public:
 	bool quitFlag;
 
@@ -65,8 +72,8 @@ public:
 
 	bool pollEvent();
 	void waitForPress();
-	OSystem::Event event() { return _event; }
-	OSystem::EventType type() { return _event.type; }
+	Common::Event event() { return _event; }
+	Common::EventType type() { return _event.type; }
 };
 
 } // End of namespace Lure
