@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2005-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,13 +18,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/lure/hotspots.h $
- * $Id: hotspots.h 22731 2006-05-29 08:12:07Z dreammaster $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/lure/hotspots.h $
+ * $Id: hotspots.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
-#ifndef __lure_hotspots_h__
-#define __lure_hotspots_h__
+#ifndef LURE_HOTSPOTS_H
+#define LURE_HOTSPOTS_H
 
 #include "lure/luredefs.h"
 #include "lure/screen.h"
@@ -35,10 +38,12 @@ namespace Lure {
 class Hotspot;
 
 class Support {
+private:
+	static bool changeRoomCheckBumped(Hotspot &h);
 public:
 	static int findIntersectingCharacters(Hotspot &h, uint16 *charList); 
 	static bool checkForIntersectingCharacter(Hotspot &h);
-	static void checkRoomChange(Hotspot &h);
+	static bool checkRoomChange(Hotspot &h);
 	static void characterChangeRoom(Hotspot &h, uint16 roomNumber, 
 								  int16 newX, int16 newY, Direction dir);
 	static bool charactersIntersecting(HotspotData *hotspot1, HotspotData *hotspot2);
@@ -54,15 +59,32 @@ private:
 
 	// Handler methods
 	static void defaultHandler(Hotspot &h);
+	static void voiceBubbleAnimHandler(Hotspot &h);
 	static void standardAnimHandler(Hotspot &h);
+	static void standardAnimHandler2(Hotspot &h);
 	static void standardCharacterAnimHandler(Hotspot &h);
+	static void puzzledAnimHandler(Hotspot &h);
 	static void roomExitAnimHandler(Hotspot &h);
 	static void playerAnimHandler(Hotspot &h);
+	static void followerAnimHandler(Hotspot &h);
 	static void skorlAnimHandler(Hotspot &h);
+	static void sonicRatAnimHandler(Hotspot &h);
 	static void droppingTorchAnimHandler(Hotspot &h);
+	static void playerSewerExitAnimHandler(Hotspot &h);
 	static void fireAnimHandler(Hotspot &h);
+	static void sparkleAnimHandler(Hotspot &h);
+	static void teaAnimHandler(Hotspot &h);
+	static void goewinCaptiveAnimHandler(Hotspot &h);
+	static void prisonerAnimHandler(Hotspot &h);
+	static void catrionaAnimHandler(Hotspot &h);
+	static void morkusAnimHandler(Hotspot &h);
 	static void talkAnimHandler(Hotspot &h);
-	static void headAnimationHandler(Hotspot &h);
+	static void grubAnimHandler(Hotspot &h);
+	static void barmanAnimHandler(Hotspot &h);
+	static void skorlGaurdAnimHandler(Hotspot &h);
+	static void gargoyleAnimHandler(Hotspot &h);
+	static void skullAnimHandler(Hotspot &h);
+	static void rackSerfAnimHandler(Hotspot &h);
 
 public:
 	static HandlerMethodPtr getHandler(uint16 procOffset);
@@ -76,16 +98,13 @@ private:
 	CurrentAction _action;
 	CharacterScheduleEntry *_supportData;
 	uint16 _roomNumber;
+	bool _dynamicSupportData;
 public:
-	CurrentActionEntry(CurrentAction newAction, uint16 roomNum) {
-		_action = newAction; 
-		_supportData = NULL; 
-		_roomNumber = roomNum;
-	}
-	CurrentActionEntry(CurrentAction newAction, CharacterScheduleEntry *data, uint16 roomNum) { 
-		_action = newAction; 
-		_supportData = data; 
-		_roomNumber = roomNum;
+	CurrentActionEntry(CurrentAction newAction, uint16 roomNum);
+	CurrentActionEntry(CurrentAction newAction, CharacterScheduleEntry *data, uint16 roomNum);
+	CurrentActionEntry(Action newAction, uint16 roomNum, uint16 param1, uint16 param2);
+	virtual ~CurrentActionEntry() {
+		if (_dynamicSupportData) delete _supportData;
 	}
 
 	CurrentAction action() { return _action; }
@@ -97,7 +116,14 @@ public:
 	uint16 roomNumber() { return _roomNumber; }
 	void setAction(CurrentAction newAction) { _action = newAction; }
 	void setRoomNumber(uint16 roomNum) { _roomNumber = roomNum; }
-	void setSupportData(CharacterScheduleEntry *newRec) { _supportData = newRec; }
+	void setSupportData(CharacterScheduleEntry *newRec) { 
+		assert((newRec == NULL) || (newRec->parent() != NULL));
+		_supportData = newRec; 
+	}
+	void setSupportData(uint16 entryId);
+
+	void saveToStream(WriteStream *stream);
+	static CurrentActionEntry *loadFromStream(ReadStream *stream);
 };
 
 class CurrentActionStack {
@@ -112,7 +138,8 @@ public:
 	CurrentAction action() { return isEmpty() ? NO_ACTION : top().action(); }
 	void pop() { _actions.erase(_actions.begin()); }
 	int size() { return _actions.size(); }
-	void list();
+	void list(char *buffer);
+	void list() { list(NULL); }
 
 	void addBack(CurrentAction newAction, uint16 roomNum) {
 		_actions.push_back(new CurrentActionEntry(newAction, roomNum));
@@ -120,12 +147,21 @@ public:
 	void addBack(CurrentAction newAction, CharacterScheduleEntry *rec, uint16 roomNum) {
 		_actions.push_back(new CurrentActionEntry(newAction, rec, roomNum));
 	}
+	void addBack(Action newAction, uint16 roomNum, uint16 param1, uint16 param2) {
+		_actions.push_back(new CurrentActionEntry(newAction, roomNum, param1, param2));
+	}
 	void addFront(CurrentAction newAction, uint16 roomNum) {
 		_actions.push_front(new CurrentActionEntry(newAction, roomNum));
 	}
 	void addFront(CurrentAction newAction, CharacterScheduleEntry *rec, uint16 roomNum) {
 		_actions.push_front(new CurrentActionEntry(newAction, rec, roomNum));
 	}
+	void addFront(Action newAction, uint16 roomNum, uint16 param1, uint16 param2) {
+		_actions.push_front(new CurrentActionEntry(newAction, roomNum, param1, param2));
+	}
+
+	void saveToStream(WriteStream *stream);
+	void loadFromStream(ReadStream *stream);
 };
 
 class WalkingActionEntry {
@@ -133,17 +169,18 @@ private:
 	Direction _direction;
 	int _numSteps;
 public:
-	WalkingActionEntry(Direction dir, int steps): _direction(dir), _numSteps(steps) {};
+	WalkingActionEntry(Direction dir, int steps): _direction(dir), _numSteps(steps) {}
 	Direction direction() { return _direction; }
 	int &rawSteps() { return _numSteps; }
 	int numSteps();
 };
 
-enum PathFinderResult {PF_OK, PF_DEST_OCCUPIED, PF_NO_PATH, PF_NO_WALK};
+enum PathFinderResult {PF_UNFINISHED, PF_OK, PF_DEST_OCCUPIED, PF_PART_PATH, PF_NO_WALK};
 
 class PathFinder {
 private:
 	Hotspot *_hotspot;
+	bool _inUse;
 	ManagedList<WalkingActionEntry *> _list;
 	RoomPathsDecompressedData _layer;
 	int _stepCtr;
@@ -156,7 +193,6 @@ private:
 	int16 _xDestCurrent, _yDestCurrent;
 	bool _destOccupied;
 	bool _cellPopulated;
-	PathFinderResult _result;
 	uint16 *_pSrc, *_pDest;
 	int _xChangeInc, _xChangeStart;
 	int _yChangeInc, _yChangeStart;
@@ -176,29 +212,34 @@ public:
 	PathFinder(Hotspot *h);
 	void clear();
 	void reset(RoomPathsData &src);
-	bool process();
-	void list();
+	PathFinderResult process();
+	void list(char *buffer);
+	void list() { list(NULL); }
 
 	void pop() { _list.erase(_list.begin()); }
 	WalkingActionEntry &top() { return **_list.begin(); }
 	bool isEmpty() { return _list.empty(); }
 	int &stepCtr() { return _stepCtr; }
-	PathFinderResult result() { return _result; }
+
+	void saveToStream(Common::WriteStream *stream);
+	void loadFromStream(Common::ReadStream *stream);
 };
 
-enum HotspotPrecheckResult {PC_EXECUTE, PC_NOT_IN_ROOM, PC_UNKNOWN, PC_INITIAL, PC_EXCESS};
+enum HotspotPrecheckResult {PC_EXECUTE, PC_NOT_IN_ROOM, PC_FAILED, PC_WAIT, PC_EXCESS};
 
-enum BlockedState {BS_NONE, BS_INITIAL, BS_UNKNOWN};
+enum BarPlaceResult {BP_KEEP_TRYING, BP_GOT_THERE, BP_FAIL};
+
+#define MAX_NUM_FRAMES 16
 
 class Hotspot {
 private:
 	HotspotData *_data;
+	uint16 _animId;
 	HotspotAnimData *_anim;
-public:
 	HandlerMethodPtr _tickHandler;
-private:
 	Surface *_frames;
 	uint16 _hotspotId;
+	uint16 _originalId;
 	uint16 _roomNumber;
 	int16 _startX, _startY;
 	uint16 _height, _width;
@@ -210,33 +251,39 @@ private:
 	uint16 _frameNumber;
 	Direction _direction;
 	uint8 _layer;
-	uint16 _sequenceOffset;
+	uint16 _hotspotScriptOffset;
 	uint16 _tickCtr;
-	uint32 _actions;
 	uint8 _colourOffset;
 	bool _persistant;
 	HotspotOverrideData *_override;
 	bool _skipFlag;
 	CurrentActionStack _currentActions;
-	CharacterScheduleEntry _npcSupportData;
 	PathFinder _pathFinder;
+	uint16 _frameWidth;
+	bool _frameStartsUsed;
+	uint16 _frameStarts[MAX_NUM_FRAMES];
+	char _nameBuffer[MAX_HOTSPOT_NAME_SIZE];
 
+	// Runtime fields
 	uint16 _frameCtr;
-	uint8 _actionCtr;
+	uint8 _voiceCtr;
 	int16 _destX, _destY;
 	uint16 _destHotspotId;
 	uint16 _blockedOffset;
 	uint8 _exitCtr;
-	BlockedState _blockedState;
-	bool _unknownFlag;
-
+	bool _walkFlag;
+	uint16 _startRoomNumber;
+	uint16 _supportValue; 
+	
 	// Support methods
-	void startTalk(HotspotData *charHotspot);
+	uint16 getTalkId(HotspotData *charHotspot);
+	void startTalk(HotspotData *charHotspot, uint16 id);
+	void startTalkDialog();
 
 	// Action support methods
 	HotspotPrecheckResult actionPrecheck(HotspotData *hotspot);
-	HotspotPrecheckResult actionPrecheck2(HotspotData *hotspot);
-	void actionPrecheck3(HotspotData *hotspot);
+	BarPlaceResult getBarPlace();
+	bool findClearBarPlace();
 	bool characterWalkingCheck(HotspotData *hotspot);
 	bool doorCloseCheck(uint16 doorId);
 	void resetDirection();
@@ -263,16 +310,16 @@ private:
 	void doBribe(HotspotData *hotspot);
 	void doExamine(HotspotData *hotspot);
 	void npcSetRoomAndBlockedOffset(HotspotData *hotspot);
-	void npcUnknown1(HotspotData *hotspot); 
+	void npcHeySir(HotspotData *hotspot); 
 	void npcExecScript(HotspotData *hotspot); 
-	void npcUnknown2(HotspotData *hotspot); 
+	void npcResetPausedList(HotspotData *hotspot); 
 	void npcSetRandomDest(HotspotData *hotspot);
 	void npcWalkingCheck(HotspotData *hotspot); 
 	void npcSetSupportOffset(HotspotData *hotspot); 
 	void npcSupportOffsetConditional(HotspotData *hotspot);
 	void npcDispatchAction(HotspotData *hotspot); 
-	void npcUnknown3(HotspotData *hotspot); 
-	void npcUnknown4(HotspotData *hotspot); 
+	void npcTalkNpcToNpc(HotspotData *hotspot); 
+	void npcPause(HotspotData *hotspot); 
 	void npcStartTalking(HotspotData *hotspot);
 	void npcJumpAddress(HotspotData *hotspot);
 public:
@@ -283,12 +330,16 @@ public:
 	void setAnimation(uint16 newAnimId);
 	void setAnimation(HotspotAnimData *newRecord);
 	uint16 hotspotId() { return _hotspotId; }
+	uint16 originalId() { return _originalId; }
 	Surface &frames() { return *_frames; }
 	HotspotAnimData &anim() { return *_anim; }
 	HotspotData *resource() { return _data; }
 	uint16 numFrames() { return _numFrames; }
 	uint16 frameNumber() { return _frameNumber; }
-	void setFrameNumber(uint16 v) { _frameNumber = v; }
+	void setFrameNumber(uint16 frameNum) { 
+		assert(frameNum < _numFrames);
+		_frameNumber = frameNum; 
+	}
 	void incFrameNumber();
 	Direction direction() { return _direction; }
 	uint16 frameWidth() { return _width; }
@@ -301,8 +352,8 @@ public:
 	uint16 destHotspotId() { return _destHotspotId; }
 	uint16 blockedOffset() { return _blockedOffset; }
 	uint8 exitCtr() { return _exitCtr; }
-	BlockedState blockedState() { return _blockedState; }
-	bool unknownFlag() { return _unknownFlag; }
+	bool walkFlag() { return _walkFlag; }
+	uint16 startRoomNumber() { return _startRoomNumber; }
 	uint16 width() { return _width; }
 	uint16 height() { return _height; }
 	uint16 widthCopy() { return _widthCopy; }
@@ -310,7 +361,11 @@ public:
 	uint16 yCorrection() { return _yCorrection; }
 	uint16 charRectY() { return _charRectY; }
 	uint16 roomNumber() { return _roomNumber; }
-	uint16 script() { return _sequenceOffset; }
+	uint16 talkScript() { 
+		assert(_data);
+		return _data->talkScriptOffset; 
+	}
+	uint16 hotspotScript() { return _hotspotScriptOffset; }
 	uint8 layer() { return _layer; }
 	uint16 tickCtr() { return _tickCtr; }
 	bool skipFlag() { return _skipFlag; }
@@ -323,20 +378,52 @@ public:
 		if (_data) _data->roomNumber = roomNum;
 	}
 	uint16 nameId();
+	const char *getName();
 	bool isActiveAnimation();
 	void setPosition(int16 newX, int16 newY);
 	void setDestPosition(int16 newX, int16 newY) { _destX = newX; _destY = newY; }
 	void setDestHotspot(uint16 id) { _destHotspotId = id; }
 	void setExitCtr(uint8 value) { _exitCtr = value; }
-	void setBlockedState(BlockedState newState) { _blockedState = newState; }
-	void setUnknownFlag(bool value) { _unknownFlag = value; }
+	BlockedState blockedState() { 
+		assert(_data);
+		return _data->blockedState; 
+	}
+	void setBlockedState(BlockedState newState) { 
+		assert(_data);
+		_data->blockedState = newState; 
+	}
+	bool blockedFlag() {
+		assert(_data);
+		return _data->blockedFlag;
+	}
+	void setBlockedFlag(bool newValue) {
+		assert(_data);
+		_data->blockedFlag = newValue;
+	}
+	void setWalkFlag(bool value) { _walkFlag = value; }
+	void setStartRoomNumber(uint16 value) { _startRoomNumber = value; }
 	void setSize(uint16 newWidth, uint16 newHeight);
-	void setScript(uint16 offset) {
+	void setWidth(uint16 newWidth) {
+		_width = newWidth;
+		_frameWidth = newWidth;
+	}
+	void setHeight(uint16 newHeight) { 
+		_height = newHeight;
+	}
+	void setHotspotScript(uint16 offset) {
 		assert(_data != NULL);
-		_sequenceOffset = offset;
-		_data->sequenceOffset = offset; 
+		_hotspotScriptOffset = offset;
+		_data->hotspotScriptOffset = offset; 
+	}
+	void setLayer(uint8 newLayer) {
+		assert(_data != NULL);
+		_layer = newLayer;
+		_data->layer = newLayer;
+	}
+	void setActions(uint32 newActions) { 
+		assert(_data);
+		_data->actions = newActions;
 	}
-	void setActions(uint32 newActions) { _actions = newActions; }
 	void setCharRectY(uint16 value) { _charRectY = value; }
 	void setSkipFlag(bool value) { _skipFlag = value; }
 	CharacterMode characterMode() {
@@ -348,13 +435,47 @@ public:
 		_data->characterMode = value;
 	}
 	uint16 delayCtr() { 
-		assert(_data != NULL);
-		return _data->delayCtr;
+		assert(_data);
+		return _data->delayCtr; 
 	}
 	void setDelayCtr(uint16 value) { 
-		assert(_data != NULL);
-		_data->delayCtr = value;
+		assert(_data);
+		_data->delayCtr = value;	
 	}
+	uint16 pauseCtr() { 
+		assert(_data);
+		return _data->pauseCtr;
+	}
+	void setPauseCtr(uint16 value) { 
+		assert(_data);
+		_data->pauseCtr = value;
+	}
+	VariantBool coveredFlag() { 
+		assert(_data);
+		return _data->coveredFlag;
+	}
+	void setCoveredFlag(VariantBool value) { 
+		assert(_data);
+		_data->coveredFlag = value;
+	}
+	uint16 useHotspotId() { 
+		assert(_data);
+		return _data->useHotspotId; 
+	}
+	void setUseHotspotId(uint16 value) {
+		assert(_data);
+		_data->useHotspotId = value;
+	}
+	uint16 talkGate() {
+		assert(_data);
+		return _data->talkGate;
+	}
+	void setTalkGate(uint16 value) {
+		assert(_data);
+		_data->talkGate = value;
+	}
+	uint16 supportValue() { return _supportValue; }
+	void setSupportValue(uint16 value) { _supportValue = value; }
 
 	void copyTo(Surface *dest);
 	bool executeScript();
@@ -367,6 +488,7 @@ public:
 	void endAction();
 	void setDirection(Direction dir);
 	void faceHotspot(HotspotData *hotspot);
+	void faceHotspot(uint16 hotspotId);
 	void setRandomDest();
 	void setOccupied(bool occupiedFlag);
 	bool walkingStep();
@@ -374,20 +496,42 @@ public:
 	void updateMovement2(CharacterMode value);
 	void resetPosition();
 
-	// Actions
 	void doAction();
 	void doAction(Action action, HotspotData *hotspot);
 	CurrentActionStack &currentActions() { return _currentActions; }
-	CharacterScheduleEntry &npcSupportData() { return _npcSupportData; }
 	PathFinder &pathFinder() { return _pathFinder; }
 	uint16 frameCtr() { return _frameCtr; }
 	void setFrameCtr(uint16 value) { _frameCtr = value; }
 	void decrFrameCtr() { if (_frameCtr > 0) --_frameCtr; }
-	uint8 actionCtr() { return _actionCtr; }
-	void setActionCtr(uint8 v) { _actionCtr = v; }
+	uint8 actionCtr() { 
+		assert(_data);
+		return _data->actionCtr; 
+	}
+	void setActionCtr(uint8 v) { 
+		assert(_data);
+		_data->actionCtr = v; 
+	}
+	uint8 voiceCtr() { return _voiceCtr; }
+	void setVoiceCtr(uint8 v) { _voiceCtr = v; }
+
+	// Miscellaneous
+	void converse(uint16 destCharacterId, uint16 messageId, bool standStill);
+	void converse(uint16 destCharacterId, uint16 messageId) {
+		converse(destCharacterId, messageId, false);
+	}
+	void showMessage(uint16 messageId, uint16 destCharacterId = NOONE_ID);
+	void scheduleConverse(uint16 destHotspot, uint16 messageId);
+	void handleTalkDialog();
+
+	void saveToStream(Common::WriteStream *stream);
+	void loadFromStream(Common::ReadStream *stream);
 };
 
-typedef ManagedList<Hotspot *> HotspotList;
+class HotspotList: public ManagedList<Hotspot *> {
+public:
+	void saveToStream(WriteStream *stream);
+	void loadFromStream(ReadStream *stream);
+};
 
 } // End of namespace Lure
 
