@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2005-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,13 +18,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/lure/intro.h $
- * $Id: intro.h 20578 2006-02-11 19:25:13Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/lure/intro.h $
+ * $Id: intro.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
-#ifndef __lure_intro_h__
-#define __lure_intro_h__
+#ifndef LURE_INTRO_H
+#define LURE_INTRO_H
 
 #include "lure/screen.h"
 
@@ -35,7 +38,7 @@ private:
 	bool showScreen(uint16 screenId, uint16 paletteId, uint16 delaySize);
 	bool delay(uint32 milliseconds);
 public:
-	Introduction(Screen &screen, OSystem &system): _screen(screen), _system(system) {};
+	Introduction(Screen &screen, OSystem &system): _screen(screen), _system(system) {}
 
 	bool show();
 };
