@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2005-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,15 +18,15 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/lure/lure.h $
- * $Id: lure.h 21916 2006-04-15 20:36:41Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/lure/lure.h $
+ * $Id: lure.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
-#ifndef __LURE_H__
-#define __LURE_H__
+#ifndef LURE_H
+#define LURE_H
 
-#include "base/engine.h"
+#include "engines/engine.h"
 #include "common/rect.h"
 #include "sound/mixer.h"
 #include "common/file.h"
@@ -33,7 +36,6 @@
 #include "lure/screen.h"
 #include "lure/events.h"
 #include "lure/menu.h"
-#include "lure/system.h"
 #include "lure/strings.h"
 #include "lure/room.h"
 
@@ -43,29 +45,34 @@ class LureEngine : public Engine {
 private:
 	uint32 _features;
 	uint8 _game;
+	Common::Language _language;
 	Disk *_disk;
 	Resources *_resources;
 	Screen *_screen;
 	Mouse *_mouse;
 	Events *_events;
 	Menu *_menu;
-	System *_sys;
 	StringData *_strings;
 	Room *_room;
 
 	void detectGame();
+	const char *generateSaveName(int slotNumber);
 public:
 	LureEngine(OSystem *system);
 	~LureEngine();
+	static LureEngine &getReference();
 	
 	virtual int init();
 	virtual int go();
-	virtual void errorString(const char *buf_input, char *buf_output);
 	void quitGame();
 
 	uint32 features() { return _features; }
 	uint8 game() { return _game; }
 	Disk &disk() { return *_disk; }
+
+	bool loadGame(uint8 slotNumber);
+	bool saveGame(uint8 slotNumber, Common::String &caption);
+	Common::String *detectSave(int slotNumber);
 };
 
 } // End of namespace Lure
