@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2005-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,13 +18,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/lure/luredefs.h $
- * $Id: luredefs.h 22852 2006-06-03 03:38:24Z dreammaster $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/lure/luredefs.h $
+ * $Id: luredefs.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
-#ifndef __luredefs_h__
-#define __luredefs_h__
+#ifndef LUREDEFS_H
+#define LUREDEFS_H
 
 #include "common/stdafx.h"
 #include "common/scummsys.h"
@@ -29,11 +32,25 @@
 
 namespace Lure {
 
+#define SUPPORT_FILENAME "lure.dat"
+#define LURE_DAT_MAJOR 1
+#define LURE_DAT_MINOR 19
+
 #define LURE_DEBUG 1
 
 #define READ_LE_INT16(x) (int16) READ_LE_UINT16(x)
 #define READ_LE_INT32(x) (int32) READ_LE_UINT32(x)
 
+enum {
+	kLureDebugScripts = 1 << 0,
+	kLureDebugAnimations = 1 << 1,
+	kLureDebugHotspots = 1 << 2
+};
+
+#define ERROR_BASIC 1
+#define ERROR_INTERMEDIATE 2
+#define ERROR_DETAILED 3
+
 enum {
 	GF_FLOPPY	= 1 <<  0,
 	GF_LNGUNK	= 1 << 15
@@ -71,16 +88,16 @@ enum Action {
 	BRIBE = 24,
 	EXAMINE = 25,
 	NPC_SET_ROOM_AND_OFFSET = 28, 
-	NPC_UNKNOWN1 = 29, 
+	NPC_TALK_TO_PLAYER = 29, 
 	NPC_EXEC_SCRIPT = 30, 
-	NPC_UNKNOWN2 = 31,
+	NPC_RESET_PAUSED_LIST = 31,
 	NPC_SET_RAND_DEST = 32, 
 	NPC_WALKING_CHECK = 33, 
 	NPC_SET_SUPPORT_OFFSET = 34,
 	NPC_SUPPORT_OFFSET_COND = 35, 
 	NPC_DISPATCH_ACTION = 36, 
-	NPC_UNKNOWN3 = 37,
-	NPC_UNKNOWN4 = 38, 
+	NPC_TALK_NPC_TO_NPC = 37,
+	NPC_PAUSE = 38, 
 	NPC_START_TALKING = 39, 
 	NPC_JUMP_ADDRESS = 40
 };
@@ -91,10 +108,6 @@ enum Action {
 #define GAME_COLOURS 256
 #define SCREEN_SIZE (FULL_SCREEN_HEIGHT * FULL_SCREEN_WIDTH)
 
-#define SUPPORT_FILENAME "lure.dat"
-#define LURE_DAT_MAJOR 1
-#define LURE_DAT_MINOR 8
-
 // Some resources include multiple packed palettes of 64 entries each
 #define SUB_PALETTE_SIZE 64
 // Palette resources have 220 palette entries
@@ -105,6 +118,14 @@ enum Action {
 // Palette and animation for Skorl catching player
 #define SKORL_CATCH_PALETTE_ID 0x4060
 #define SKORL_CATCH_ANIM_ID 0x4061
+// Palette and animation for chute animation
+#define CHUTE_PALETTE_ID 0x404C
+#define CHUTE_ANIM_ID 0x404D
+#define CHUTE2_ANIM_ID 0x404f
+#define CHUTE3_ANIM_ID 0x4051
+// Palette and animation for hiding in barrel
+#define BARREL_PALETTE_ID 0xE9F0
+#define BARREL_ANIM_ID 0xE9F1
 
 // Specifies the maximum buffer sized allocated for decoding animation data
 #define MAX_ANIM_DECODER_BUFFER_SIZE 300000
@@ -163,6 +184,8 @@ enum Action {
 #define TALK_DIALOG_WIDTH 128
 #define TALK_DIALOG_EDGE_SIZE 3
 #define TALK_DIALOG_Y 33
+#define SAVE_DIALOG_X 69
+#define SAVE_DIALOG_Y 28
 
 // Strings defines
 #define STRINGS_RESOURCE_ID 0x10
@@ -193,6 +216,7 @@ enum Action {
 #define ROOM_PATHS_RESOURCE_ID 0x3f13
 #define EXIT_COORDINATES_RESOURCE_ID 0x3f14
 #define EXIT_HOTSPOT_ID_LIST 0x3f15
+#define STRING_LIST_RESOURCE_ID 0x3f16
 
 // Script constants
 #define STARTUP_SCRIPT 0x23FC
@@ -204,19 +228,33 @@ enum Action {
 #define PLAYER_ID 0x3E8
 #define RATPOUCH_ID 0x3E9
 #define SKORL_ID 0x3EA
+#define BLACKSMITH_ID 0x3EB
+#define GOEWIN_ID 0x3EF
 #define FIRST_NONCHARACTER_ID 0x408
 #define SACK_ID 0x40D
+#define PRISONER_ID 0x412
+#define SID_ID 0x420
+#define OIL_BURNER_ID 0x424
 #define TRANSFORM_ID 0x425
-
+#define NELLIE_ID 0x429
+#define EWAN_ID 0x436
+#define WAYNE_ID 0x3f1
+#define START_EXIT_ID 0x2710
+#define BOTTLE_HOTSPOT_ID 0x2710
+#define BRICKS_ID 0x2714
 #define START_NONVISUAL_HOTSPOT_ID 0x7530
 
 // Milliseconds delay between game frames
 #define GAME_FRAME_DELAY 80
 
 // Tick proc constants
-#define TALK_TICK_PROC_ID 0x8ABD
+#define STANDARD_CHARACTER_TICK_PROC 0x4f82 
 #define PLAYER_TICK_PROC_ID 0x5E44
 #define VOICE_TICK_PROC_ID 0x625E
+#define PUZZLED_TICK_PROC_ID 0x6571
+#define STANDARD_ANIM_2_TICK_PROC 0x7F37
+#define STANDARD_ANIM_TICK_PROC 0x7f3a
+#define TALK_TICK_PROC_ID 0x8ABD
 
 // String constants
 #define STRANGER_ID 0x17A
@@ -224,7 +262,22 @@ enum Action {
 #define TALK_RESPONSE_MAGIC_ID 0x1092
 
 // Misc constants
+#define GENERAL_MAGIC_ID 42
 #define VOICE_ANIM_ID 0x5810
+#define PUZZLED_ANIM_ID 0x8001
+#define EXCLAMATION_ANIM_ID 0x8002
+#define SERF_ANIM_ID 0x58A0
+#define BLACKSMITH_STANDARD 0x8a12
+#define BLACKSMITH_HAMMERING_ANIM_ID 0x9c11
+#define EWAN_ANIM_ID 0x59E4
+#define EWAN_ALT_ANIM_ID 0x59ED
+#define PLAYER_ANIM_ID 0x5C80
+#define SELENA_ANIM_ID 0x5CAA
+
+#define CONVERSE_COUNTDOWN_SIZE 40
+#define IDLE_COUNTDOWN_SIZE 15
+#define MAX_TELL_COMMANDS 8
+#define MAX_SAVEGAME_SLOTS 10
 
 // Countdown for # operations in path finder before breaking until next
 // tick - set it to 0 if you'd like all pathfinding to be done at once
@@ -234,9 +287,19 @@ enum Action {
 // Pixel record flags
 #define PIXELFLAG_HAS_TABLE 4
 
+// Hotspot flags
+#define HOTSPOTFLAG_FOUND 0x80
+#define HOTSPOTFLAG_SKIP 0x40
+#define HOTSPOTFLAG_20 0x20
+
 // Constants used to reference entries in the reworked support data entry lists
-#define RETURN_SUPPORT_ID 0x100
-#define EXIT_BLOCKED_SUPPORT_ID 0x200
+#define RETURN_SUPPORT_ID 0x400
+#define EXIT_BLOCKED_SUPPORT_ID 0x800
+#define JUMP_ADDR_2_SUPPORT_ID 0x1403
+
+// Constants used in animation Serf on the rack
+#define RACK_SERF_SCRIPT_ID_1 0x35C
+#define RACK_SERF_SCRIPT_ID_2 0x384
 
 } // End of namespace Lure
 
