@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2005-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/lure/memory.cpp $
- * $Id: memory.cpp 20756 2006-02-19 04:02:01Z dreammaster $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/lure/memory.cpp $
+ * $Id: memory.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -39,7 +42,7 @@ uint8 *Memory::alloc(uint32 size) {
 	return (uint8 *) malloc(size);
 }
 
-void Memory::dealloc(uint8 *block) {
+void Memory::dealloc(void *block) {
 	free(block);
 }
 
