@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2005-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,13 +18,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/lure/memory.h $
- * $Id: memory.h 20756 2006-02-19 04:02:01Z dreammaster $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/lure/memory.h $
+ * $Id: memory.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
-#ifndef __lure_memory_h__
-#define __lure_memory_h__
+#ifndef LURE_MEMORY_H
+#define LURE_MEMORY_H
 
 #include "common/stdafx.h"
 #include "common/system.h"
@@ -55,7 +58,7 @@ public:
 	static MemoryBlock *allocate(uint32 size);
 	static MemoryBlock *duplicate(MemoryBlock *src);
 	static uint8 *alloc(uint32 size);
-	static void dealloc(uint8 *block);
+	static void dealloc(void *block);
 };
 
 } // end of namspace Lure
