@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2005-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/lure/menu.cpp $
- * $Id: menu.cpp 21992 2006-04-17 23:37:22Z eriktorbjorn $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/lure/menu.cpp $
+ * $Id: menu.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -24,45 +27,32 @@
 #include "lure/luredefs.h"
 #include "lure/decode.h"
 #include "lure/surface.h"
-#include "lure/system.h"
 #include "lure/res_struct.h"
 #include "lure/res.h"
 #include "lure/strings.h"
+#include "lure/room.h"
+#include "lure/events.h"
 
 namespace Lure {
 
 MenuRecord::MenuRecord(uint16 hsxstartVal, uint16 hsxendVal, uint16 xstartVal, 
-					   uint16 widthVal, const char *strings) {
+					   uint16 widthVal, int numParams, ...) {
+	// Store list of pointers to strings
+	va_list params;
+
+	_numEntries = numParams;
+	_entries = (const char **) malloc(sizeof(const char *) * _numEntries);	
+
+	va_start(params, numParams);
+	for (int index = 0; index < _numEntries; ++index)
+		_entries[index] = va_arg(params, const char *);
+
+	// Store position data
 	_xstart = xstartVal; _width = widthVal;
 	_hsxstart = hsxstartVal; _hsxend = hsxendVal;
-
-	// Figure out the number of entries
-	const char *sPtr = strings;
-	_numEntries = 1;
-	while ((sPtr = strchr(sPtr, ',')) != NULL) {
-		++_numEntries;
-		++sPtr;
-	}
-
-	// Set up the list of entries
-	char *sCopy = strdup(strings);
-	char *s;
-	_entries = (char **) malloc(sizeof(char *) * _numEntries);
-	uint8 index = 0;
-	s = sCopy;
-	while (s != NULL) {
-		_entries[index++] = s;
-		s = strchr(s, ',');
-		if (s != NULL) *s++ = '\0'; // replace comma with NULL
-	}
 }
 
-MenuRecord::~MenuRecord() {
-	free(_entries[0]);	// Delete string data for all the menu items
-	free(_entries);		// Free the list
-}
-
-char *MenuRecord::getEntry(uint8 index) {
+const char *MenuRecord::getEntry(uint8 index) {
 	if (index >= _numEntries) error("Invalid menuitem index specified: %d", index);
 	return _entries[index];
 }
@@ -73,15 +63,19 @@ static Menu *int_menu = NULL;
 
 Menu::Menu() {
 	int_menu = this;
+	StringList &sl = Resources::getReference().stringList();
 
 	MemoryBlock *data = Disk::getReference().getEntry(MENU_RESOURCE_ID);
 	PictureDecoder decoder;
 	_menu = decoder.decode(data, SCREEN_SIZE);
 	delete data;
 
-	_menus[0] = new MenuRecord(40, 87, 20, 80, "Credits");
-	_menus[1] = new MenuRecord(127, 179, 100, 120, "Restart game,Save game,Restore game");
-	_menus[2] = new MenuRecord(224, 281, 210, 105, "Quit,Slow Text\x8b,Sound on ");
+	_menus[0] = new MenuRecord(40, 87, 20, 80, 1, sl.getString(S_CREDITS));
+	_menus[1] = new MenuRecord(127, 179, 100, 120, 3, 
+		sl.getString(S_RESTART_GAME), sl.getString(S_SAVE_GAME), sl.getString(S_RESTORE_GAME));
+	_menus[2] = new MenuRecord(224, 281, 210, 105, 3,
+		sl.getString(S_QUIT), sl.getString(S_SLOW_TEXT), sl.getString(S_SOUND_ON));
+
 	_selectedMenu = NULL;
 }
 
@@ -95,7 +89,7 @@ Menu &Menu::getReference() {
 }
 
 uint8 Menu::execute() {
-	OSystem &system = System::getReference();
+	OSystem &system = *g_system;
 	Mouse &mouse = Mouse::getReference();
 	Events &events = Events::getReference();
 	Screen &screen = Screen::getReference();
@@ -120,10 +114,7 @@ uint8 Menu::execute() {
 					// If necessary, remove prior menu
 					if (_selectedMenu) {
 						toggleHighlight(_selectedMenu);
-//						screen.updateArea(_selectedMenu->xstart(), MENUBAR_Y_SIZE,
-//							_surfaceMenu->width(), _surfaceMenu->height());
-						screen.updateArea(0, MENUBAR_Y_SIZE, FULL_SCREEN_WIDTH, 
-							_surfaceMenu->height());
+						screen.updateArea(0, 0, FULL_SCREEN_WIDTH, _surfaceMenu->height() + 8);
 						delete _surfaceMenu;
 						_surfaceMenu = NULL;
 						_selectedIndex = 0;
@@ -220,8 +211,8 @@ void Menu::toggleHighlight(MenuRecord *menuRec) {
 
 void Menu::toggleHighlightItem(uint8 index) {
 	byte *p = _surfaceMenu->data().data() + (DIALOG_EDGE_SIZE + 3 + 
-		((index - 1) * 8)) * _surfaceMenu->width();
-	uint32 numBytes = 8 * _surfaceMenu->width();
+		((index - 1) * (FONT_HEIGHT - 1))) * _surfaceMenu->width();
+	uint32 numBytes = (FONT_HEIGHT - 1) * _surfaceMenu->width();
 
 	while (numBytes-- > 0) {
 		if (*p == MENU_UNSELECTED_COLOUR) *p = MENU_SELECTED_COLOUR;
@@ -249,11 +240,11 @@ uint16 PopupMenu::ShowInventory() {
 		if (hotspot->roomNumber == PLAYER_ID) {
 			idList[itemCtr] = hotspot->hotspotId;
 			char *hotspotName = itemNames[itemCtr++] = (char *) malloc(MAX_HOTSPOT_NAME_SIZE);
-			strings.getString(hotspot->nameId, hotspotName, NULL, NULL);
+			strings.getString(hotspot->nameId, hotspotName);
 		}
 	}
 	
-	uint16 result = Show(numItems, (const char **) itemNames);
+	uint16 result = Show(numItems, const_cast<const char **>(itemNames));
 	if (result != 0xffff) result = idList[result];
 
 	for (itemCtr = 0; itemCtr < numItems; ++itemCtr)
@@ -264,43 +255,154 @@ uint16 PopupMenu::ShowInventory() {
 	return result;
 }
 
+#define MAX_NUM_DISPLAY_ITEMS 20
+
+uint16 PopupMenu::ShowItems(Action contextAction) {
+	Resources &res = Resources::getReference();
+	ValueTableData &fields = res.fieldList();
+	RoomDataList &rooms = res.roomData();
+	HotspotDataList &hotspots = res.hotspotData();
+	StringData &strings = StringData::getReference();
+	Room &room = Room::getReference();
+	Screen &screen = Screen::getReference();
+	Mouse &mouse = Mouse::getReference();
+	RoomDataList::iterator ir;
+	HotspotDataList::iterator ih;
+	uint16 entryIds[MAX_NUM_DISPLAY_ITEMS];
+	uint16 nameIds[MAX_NUM_DISPLAY_ITEMS];
+	char *entryNames[MAX_NUM_DISPLAY_ITEMS];
+	int numItems = 0;
+	int itemCtr;
+	Hotspot *player = res.getActiveHotspot(PLAYER_ID);
+	uint32 contextBitflag = 1 << (contextAction - 1);
+
+	// Loop for rooms
+	for (ir = rooms.begin(); ir != rooms.end(); ++ir) {
+		RoomData *roomData = *ir;
+		// Pre-condition checks for whether to skip room
+		if ((roomData->hdrFlags != 15) && ((roomData->hdrFlags & fields.hdrFlagMask()) == 0))
+			continue;
+		if (((roomData->flags & HOTSPOTFLAG_20) != 0) || ((roomData->flags & HOTSPOTFLAG_FOUND) == 0))
+			continue;
+		if ((roomData->actions & contextBitflag) == 0)
+			continue;
+
+		// Add room to list of entries to display
+		if (numItems == MAX_NUM_DISPLAY_ITEMS) error("Out of space in ask list");
+		entryIds[numItems] = roomData->roomNumber;
+		nameIds[numItems] = roomData->roomNumber;
+		entryNames[numItems] = (char *) Memory::alloc(MAX_HOTSPOT_NAME_SIZE);
+		strings.getString(roomData->roomNumber, entryNames[numItems]);
+		++numItems;
+	}
+
+	// Loop for hotspots
+	for (ih = hotspots.begin(); ih != hotspots.end(); ++ih) {
+		HotspotData *hotspot = *ih;
+
+		if ((hotspot->headerFlags != 15) && 
+			((hotspot->headerFlags & fields.hdrFlagMask()) == 0))
+			continue;
+
+		if (((hotspot->flags & HOTSPOTFLAG_20) != 0) || ((hotspot->flags & HOTSPOTFLAG_FOUND) == 0))
+			// Skip the current hotspot		
+			continue;
+
+		// Following checks are done for room list - still need to include check against [3350h]
+		if (((hotspot->flags & 0x10) != 0) && (hotspot->roomNumber != player->roomNumber()))
+			continue;
+
+		if ((hotspot->actions & contextBitflag) == 0)
+			// If hotspot does not allow action, then skip it
+			continue;
+
+		if ((hotspot->nameId == 0x17A) || (hotspot->nameId == 0x147))
+			// Special hotspot names to skip 
+			continue;
+
+		// Check if the hotspot's name is already used in an already set item
+		itemCtr = 0;
+		while ((itemCtr < numItems) && (nameIds[itemCtr] != hotspot->nameId))
+			++itemCtr;
+		if (itemCtr != numItems)
+			// Item's name is already present - skip hotspot
+			continue;
+
+		// Add hotspot to list of entries to display
+		if (numItems == MAX_NUM_DISPLAY_ITEMS) error("Out of space in ask list");
+		entryIds[numItems] = hotspot->hotspotId;
+		nameIds[numItems] = hotspot->nameId;
+		entryNames[numItems] = (char *) Memory::alloc(MAX_HOTSPOT_NAME_SIZE);
+		strings.getString(hotspot->nameId, entryNames[numItems]);
+		++numItems;
+	}
+
+	if (numItems == 0) 
+		// No items, so add a 'nothing' to the statusLine
+		strcat(room.statusLine(), "(nothing)");
+
+	room.update();
+	screen.update();
+	mouse.waitForRelease();
+
+	if (numItems == 0)
+		// Return flag for no items to ask for
+		return 0xfffe;
+
+	// Display items
+	uint16 result = Show(numItems, const_cast<const char **>(entryNames));
+	if (result != 0xffff) result = entryIds[result];
+
+	// Deallocate display strings
+	for (itemCtr = 0; itemCtr < numItems; ++itemCtr)
+		Memory::dealloc(entryNames[itemCtr]);
+
+	return result;
+}
+
 Action PopupMenu::Show(uint32 actionMask) {
+	StringList &stringList = Resources::getReference().stringList();
 	int numEntries = 0;
 	uint32 v = actionMask;
 	int index;
+	const Action *currentAction;
+	uint16 resultIndex;
+	Action resultAction;
 
 	for (index = 1; index <= EXAMINE; ++index, v >>= 1) {
 		if (v & 1) ++numEntries;
 	}
 
 	const char **strList = (const char **) Memory::alloc(sizeof(char *) * numEntries);
+	Action *actionSet = (Action *) Memory::alloc(sizeof(Action) * numEntries);
 
-	v = actionMask;
 	int strIndex = 0;
-	for (index=1; index<=EXAMINE; ++index, v >>= 1) {
-		if (v & 1) 
-			strList[strIndex++] = actionList[index];
+	for (currentAction = &sortedActions[0]; *currentAction != NONE; ++currentAction) {
+		if ((actionMask & (1 << (*currentAction - 1))) != 0) {
+			strList[strIndex] = stringList.getString(*currentAction);
+			actionSet[strIndex] = *currentAction;
+			++strIndex;
+		}
 	}
 
-	uint16 result = Show(numEntries, strList);
-	
-	if (result == 0xffff) return NONE;
+	resultIndex = Show(numEntries, strList);
 
-	v = actionMask;
-	for (index = 1; index <= EXAMINE; ++index, v >>= 1) {
-		if (v & 1) 
-			if (result-- == 0) return (Action) index;
-	}
+	if (resultIndex == 0xffff) 
+		resultAction = NONE;
+	else 
+		resultAction = actionSet[resultIndex];
 
-	delete strList;
-	return NONE;
+	Memory::dealloc(strList);
+	Memory::dealloc(actionSet);
+	return resultAction;
 }
 
 Action PopupMenu::Show(int numEntries, Action *actions) {
+	StringList &stringList = Resources::getReference().stringList();
 	const char **strList = (const char **) Memory::alloc(sizeof(char *) * numEntries);
 	Action *actionPtr = actions;
 	for (int index = 0; index < numEntries; ++index)
-		strList[index] = actionList[*actionPtr++];
+		strList[index] = stringList.getString(*actionPtr++);
 	uint16 result = Show(numEntries, strList);
 	
 	delete strList;
@@ -312,7 +414,7 @@ uint16 PopupMenu::Show(int numEntries, const char *actions[]) {
 	if (numEntries == 0) return 0xffff;
 	Events &e = Events::getReference();
 	Mouse &mouse = Mouse::getReference();
-	OSystem &system = System::getReference();
+	OSystem &system = *g_system;
 	Screen &screen = Screen::getReference();
 	Rect r;
 
@@ -352,7 +454,7 @@ uint16 PopupMenu::Show(int numEntries, const char *actions[]) {
 			s->fillRect(r, 0);
 
 			for (int index = 0; index < numLines; ++index) {
-				int actionIndex = selectedIndex - (numEntries / 2) + index;
+				int actionIndex = selectedIndex - (numLines / 2) + index;
 				if ((actionIndex >= 0) && (actionIndex < numEntries)) {
 					s->writeString(DIALOG_EDGE_SIZE, DIALOG_EDGE_SIZE + index * FONT_HEIGHT,
 						actions[actionIndex], true, 
@@ -372,7 +474,7 @@ uint16 PopupMenu::Show(int numEntries, const char *actions[]) {
 				goto bail_out;
 			}
 
-			else if (e.type() == OSystem::EVENT_KEYDOWN) {
+			else if (e.type() == Common::EVENT_KEYDOWN) {
 				byte ch = e.event().kbd.ascii;
 				uint16 keycode = e.event().kbd.keycode;
 
@@ -390,11 +492,11 @@ uint16 PopupMenu::Show(int numEntries, const char *actions[]) {
 					goto bail_out;
 				}
 
-			} else if (e.type() == OSystem::EVENT_LBUTTONDOWN) {
-				mouse.waitForRelease();
+			} else if (e.type() == Common::EVENT_LBUTTONDOWN) {
+				//mouse.waitForRelease();
 				goto bail_out;
 
-			} else if (e.type() == OSystem::EVENT_RBUTTONDOWN) {
+			} else if (e.type() == Common::EVENT_RBUTTONDOWN) {
 				mouse.waitForRelease();
 				selectedIndex = 0xffff;
 				goto bail_out;
