@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2005-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,13 +18,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/lure/menu.h $
- * $Id: menu.h 20757 2006-02-19 04:04:07Z dreammaster $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/lure/menu.h $
+ * $Id: menu.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
-#ifndef __lure_menu_h__
-#define __lure_menu_h__
+#ifndef LURE_MENU_H
+#define LURE_MENU_H
 
 #include "common/stdafx.h"
 #include "common/str.h"
@@ -39,20 +42,19 @@ class MenuRecord {
 private:
 	uint16 _xstart, _width;
 	uint16 _hsxstart, _hsxend;
-	char **_entries;
+	const char **_entries;
 	uint8 _numEntries;
 public:
-	MenuRecord(uint16 hsxstartVal, uint16 hsxendVal, uint16 xstartVal, 
-		uint16 widthVal, const char *strings); 
-	~MenuRecord();
+	MenuRecord(uint16 hsxstartVal, uint16 hsxendVal, uint16 xstartVal, uint16 widthVal, 
+		int numParams, ...); 
 
 	uint16 xstart() { return _xstart; }
 	uint16 width() { return _width; }
 	uint16 hsxstart() { return _hsxstart; }
 	uint16 hsxend() { return _hsxend; }
 	uint8 numEntries() { return _numEntries; }
-	char **entries() { return _entries; }
-	char *getEntry(uint8 index);
+	const char **entries() { return _entries; }
+	const char *getEntry(uint8 index);
 };
 
 class Menu {
@@ -81,6 +83,7 @@ public:
 	static Action Show(int numEntries, Action *actions);
 	static uint16 Show(int numEntries, const char *actions[]);
 	static uint16 ShowInventory();
+	static uint16 ShowItems(Action contextAction);
 };
 
 } // End of namespace Lure
