@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2005-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/lure/res.cpp $
- * $Id: res.cpp 22852 2006-06-03 03:38:24Z dreammaster $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/lure/res.cpp $
+ * $Id: res.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -28,6 +31,8 @@
 
 namespace Lure {
 
+using namespace Common;
+
 static Resources *int_resources = NULL;
 
 Resources &Resources::getReference() {
@@ -53,7 +58,10 @@ void Resources::freeData() {
 	_exitJoins.clear();
 	_delayList.clear();
 	_charSchedules.clear();
+	_randomActions.clear();
 	_indexedRoomExitHospots.clear();
+	_pausedList.clear();
+	_stringList.clear();
 
 	delete _paletteSubset;
 	delete _scriptData;
@@ -62,7 +70,6 @@ void Resources::freeData() {
 	delete _messagesData;
 	delete _cursors;
 	delete [] _charOffsets;
-	delete _playerSupportRecord;
 }
 
 struct AnimRecordTemp {
@@ -91,7 +98,7 @@ void Resources::reloadData() {
 		if (offsetVal != 0) {
 			// Get room resource
 			RoomResource *rec = (RoomResource *) (mb->data() + offsetVal);
-			
+
 			RoomData *newEntry = new RoomData(rec, paths);
 			_roomData.push_back(newEntry);
 
@@ -280,6 +287,14 @@ void Resources::reloadData() {
 	for (ctr = 0; ctr < numCharOffsets; ++ctr, ++offset) 
 		_charOffsets[ctr] = READ_LE_UINT16(offset);
 
+	// Next load up the list of random actions your follower can do in each room
+
+	++offset;
+	while (READ_LE_UINT16(offset) != 0xffff) {
+		RandomActionSet *actionSet = new RandomActionSet(offset);
+		_randomActions.push_back(actionSet);
+	}
+
 	// Loop through loading the schedules
 	ctr = 0;
 	while ((startOffset = READ_LE_UINT16(++offset)) != 0xffff) {
@@ -297,8 +312,13 @@ void Resources::reloadData() {
 		indexedRec++;
 	}
 
+	// Load the string list
+	mb = d.getEntry(STRING_LIST_RESOURCE_ID);
+	_stringList.load(mb);
+	delete mb;
+
 	// Initialise delay list
-	_delayList.clear();
+	_delayList.clear(true);
 
 	// Load miscellaneous data
 	_cursors = d.getEntry(CURSOR_RESOURCE_ID);
@@ -307,12 +327,13 @@ void Resources::reloadData() {
 	_messagesData = d.getEntry(MESSAGES_LIST_RESOURCE_ID);
 	_talkDialogData = d.getEntry(TALK_DIALOG_RESOURCE_ID);
 
-	_playerSupportRecord = new CharacterScheduleEntry();
 	_activeTalkData = NULL;
 	_currentAction = NONE;
 	_talkState = TALK_NONE;
 	_talkSelection = 0;
 	_talkStartEntry = 0;
+	_talkDetails.active = false;
+	_talkingCharacter = 0;
 }
 
 RoomExitJoinData *Resources::getExitJoin(uint16 hotspotId) {
@@ -346,7 +367,6 @@ RoomData *Resources::getRoom(uint16 roomNumber) {
 bool Resources::checkHotspotExtent(HotspotData *hotspot) {
 	uint16 roomNum = hotspot->roomNumber;
 	RoomData *room = getRoom(roomNum);
-	
 	return (hotspot->startX >= room->clippingXStart) && ((room->clippingXEnd == 0) || 
 			(hotspot->startX + 32 < room->clippingXEnd));
 }
@@ -420,16 +440,28 @@ HotspotActionList *Resources::getHotspotActions(uint16 actionsOffset) {
 	return _actionsList.getActions(actionsOffset);
 }
 
-void Resources::setTalkingCharacter(uint16 id) { 
-	if (_talkingCharacter != 0)
+void Resources::setTalkingCharacter(uint16 id) {
+	Resources &res = Resources::getReference();
+
+	if (_talkingCharacter != 0) {
 		deactivateHotspot(_talkingCharacter, true);
+		HotspotData *charHotspot = res.getHotspot(_talkingCharacter);
+		assert(charHotspot);
+		charHotspot->talkCountdown = 0;
+
+		if (charHotspot->talkDestCharacterId != 0) {
+			HotspotData *destHotspot = res.getHotspot(charHotspot->talkDestCharacterId);
+			if (destHotspot != NULL)
+				destHotspot->talkDestCharacterId = 0;
+		}
+		charHotspot->talkDestCharacterId = 0;
+	}
 
 	_talkingCharacter = id; 
 	
 	if (_talkingCharacter != 0) {
 		Hotspot *character = getActiveHotspot(id);
-		if (!character)
-			error("Set talking character to non-active hotspot id");
+		assert(character);
 
 		// Add the special "voice" animation above the character
 		Hotspot *hotspot = new Hotspot(character, VOICE_ANIM_ID);
@@ -437,23 +469,18 @@ void Resources::setTalkingCharacter(uint16 id) {
 	}
 }
 
-void Resources::activateHotspot(uint16 hotspotId) {
+Hotspot *Resources::activateHotspot(uint16 hotspotId) {
 	HotspotData *res = getHotspot(hotspotId);
-	if (!res) return;
+	if (!res) return NULL;
 	res->roomNumber &= 0x7fff; // clear any suppression bit in room #
 
 	// Make sure that the hotspot isn't already active
 	HotspotList::iterator i = _activeHotspots.begin();
-	bool found = false;
-
 	for (; i != _activeHotspots.end(); ++i) {
-		Hotspot &h = *i.operator*();
-		if (h.hotspotId() == res->hotspotId) {
-			found = true;
-			break;
-		}
+		Hotspot *h = *i;
+		if (h->hotspotId() == res->hotspotId) 
+			return h;
 	}
-	if (found) return;
 
 	// Check the script load flag
 	if (res->scriptLoadFlag) {
@@ -462,6 +489,7 @@ void Resources::activateHotspot(uint16 hotspotId) {
 	} else {
 		// Standard load
 		bool loadFlag = true;
+		uint16 talkIndex;
 
 		switch (res->loadOffset) {
 		case 0x3afe:
@@ -481,6 +509,17 @@ void Resources::activateHotspot(uint16 hotspotId) {
 			// Standard animation load
 			break;
 
+		case 0x8617:
+			// Custom loader used by the notice hotspot 42ah in room #20
+			talkIndex = _fieldList.getField(TALK_INDEX);
+			if ((talkIndex < 8) || (talkIndex >= 14))
+				// Don't load hotspot
+				loadFlag = false;
+			else
+				// Make the notice be on-screen
+				res->startY = 85;    
+			break;
+
 		case 0x88ac:
 			// Torch in room #1
 			loadFlag = _fieldList.getField(TORCH_HIDE) == 0;
@@ -494,14 +533,38 @@ void Resources::activateHotspot(uint16 hotspotId) {
 
 		if (loadFlag) {
 			Hotspot *hotspot = addHotspot(hotspotId);
+			assert(hotspot);
+
+			// Special post-load handling
 			if (res->loadOffset == 0x7167) hotspot->setPersistant(true);
+			if (res->loadOffset == 0x8617) hotspot->handleTalkDialog();
+			
+			// TODO: Figure out why there's a room set in the animation decode for a range of characters,
+			// particularly since it doesn't seem to match what happens in-game
+			/*
+			if ((hotspot->hotspotId() >= RATPOUCH_ID) &&
+				(hotspot->hotspotId() < FIRST_NONCHARACTER_ID) &&
+				(hotspot->roomNumber() < 42)) {
+				// Start wandering characters off in room 24
+				hotspot->setRoomNumber(24);
+				hotspot->setPosition(64, 116);
+				_fieldList.wanderingCharsLoaded() = true;
+			}
+			*/
+
+			return hotspot;
 		}
 	}
+
+	return NULL;
 }
 
 Hotspot *Resources::addHotspot(uint16 hotspotId) {
-	Hotspot *hotspot = new Hotspot(getHotspot(hotspotId));
+	HotspotData *hData = getHotspot(hotspotId);
+	assert(hData);
+	Hotspot *hotspot = new Hotspot(hData);
 	_activeHotspots.push_back(hotspot);
+
 	return hotspot;
 }
 
@@ -524,6 +587,20 @@ void Resources::deactivateHotspot(uint16 hotspotId, bool isDestId) {
 	}
 }
 
+void Resources::deactivateHotspot(Hotspot *hotspot) {
+	HotspotList::iterator i = _activeHotspots.begin();
+
+	while (i != _activeHotspots.end()) {
+		Hotspot *h = *i;
+		if (h == hotspot) {
+			_activeHotspots.erase(i);
+			break;
+		}
+		
+		i++;
+	}
+}
+
 uint16 Resources::numInventoryItems() {
 	uint16 numItems = 0;
 	HotspotDataList &list = _hotspotData;
@@ -568,4 +645,36 @@ void Resources::setTalkData(uint16 offset) {
 	error("Unknown talk entry offset %d requested", offset);
 }
 
+void Resources::saveToStream(Common::WriteStream *stream)
+{
+	_hotspotData.saveToStream(stream);
+	_activeHotspots.saveToStream(stream);
+	_fieldList.saveToStream(stream);
+	_randomActions.saveToStream(stream);
+	_barmanLists.saveToStream(stream);
+	_exitJoins.saveToStream(stream);
+	_roomData.saveToStream(stream);
+	_delayList.saveToStream(stream);
+}
+
+void Resources::loadFromStream(Common::ReadStream *stream) {
+	debugC(ERROR_DETAILED, kLureDebugScripts, "Loading hotspot data");
+	_hotspotData.loadFromStream(stream);
+	debugC(ERROR_DETAILED, kLureDebugScripts, "Loading active hotspots");
+	_activeHotspots.loadFromStream(stream);
+	debugC(ERROR_DETAILED, kLureDebugScripts, "Loading fields");
+	_fieldList.loadFromStream(stream);
+	debugC(ERROR_DETAILED, kLureDebugScripts, "Loading random actions");
+	_randomActions.loadFromStream(stream);
+	debugC(ERROR_DETAILED, kLureDebugScripts, "Loading barman lists");
+	_barmanLists.loadFromStream(stream);
+	debugC(ERROR_DETAILED, kLureDebugScripts, "Loading room exit joins");
+	_exitJoins.loadFromStream(stream);
+	debugC(ERROR_DETAILED, kLureDebugScripts, "Loading walkable paths");
+	_roomData.loadFromStream(stream);
+	debugC(ERROR_DETAILED, kLureDebugScripts, "Loading delay list");
+	_delayList.loadFromStream(stream); 
+	debugC(ERROR_DETAILED, kLureDebugScripts, "Finished loading");
+}
+
 } // end of namespace Lure
