@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2005-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,13 +18,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/lure/res.h $
- * $Id: res.h 22580 2006-05-23 12:39:39Z dreammaster $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/lure/res.h $
+ * $Id: res.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
-#ifndef __lure_res_h__
-#define __lure_res_h__
+#ifndef LURE_RES_H
+#define LURE_RES_H
 
 #include "lure/luredefs.h"
 #include "lure/memory.h"
@@ -29,15 +32,21 @@
 #include "lure/res_struct.h"
 #include "lure/hotspots.h"
 #include "lure/palette.h"
+#include "common/file.h"
 
 namespace Lure {
 
-enum TalkState {TALK_NONE, TALK_SELECT, TALK_RESPOND, TALK_RESPONSE_WAIT,
-	TALK_RESPOND_2};
+enum TalkState {TALK_NONE, TALK_START, TALK_SELECT, TALK_RESPOND, TALK_RESPONSE_WAIT,
+	TALK_RESPOND_2, TALK_RESPOND_3};
 
 #define MAX_TALK_SELECTIONS 4
 typedef TalkEntryData *TalkSelections[MAX_TALK_SELECTIONS];
 
+struct TalkDialogDetails {
+	Common::Rect bounds;
+	bool active;
+};
+
 class Resources {
 private:
 	Common::RandomSource _rnd;
@@ -58,21 +67,23 @@ private:
 	TalkHeaderList _talkHeaders;
 	TalkDataList _talkData;
 	SequenceDelayList _delayList;
-public: //**DEBUG**
 	Action _currentAction;
-private:
 	MemoryBlock *_talkDialogData;
 	RoomExitCoordinatesList _coordinateList;
 	CharacterScheduleList _charSchedules;
+	RandomActionList _randomActions;
 	RoomExitIndexedHotspotList _indexedRoomExitHospots;
+	PausedCharacterList _pausedList;
+	BarmanLists _barmanLists;
+	StringList _stringList;
 
 	int numCharOffsets;
 	uint16 *_charOffsets;
-	CharacterScheduleEntry *_playerSupportRecord;
 
 	TalkData *_activeTalkData;
 	TalkState _talkState;
 	TalkSelections _talkSelections;
+	TalkDialogDetails _talkDetails;
 	int _talkSelection;
 	int _talkStartEntry;
 	uint16 _talkingCharacter;
@@ -116,7 +127,11 @@ public:
 	MemoryBlock &getTalkDialogData() { return *_talkDialogData; }
 	RoomExitCoordinatesList &coordinateList() { return _coordinateList; }
 	CharacterScheduleList &charSchedules() { return _charSchedules; }
+	RandomActionList &randomActions() { return _randomActions; }
 	RoomExitIndexedHotspotList &exitHotspots() { return _indexedRoomExitHospots; }
+	PausedCharacterList &pausedList() { return _pausedList; }
+	BarmanLists &barmanLists() { return _barmanLists; }
+	StringList &stringList() { return _stringList; }
 	uint16 getCharOffset(int index) { 
 		if (index >= numCharOffsets) 
 			error("Invalid index %d passed to script engine support data offset list", index);
@@ -124,7 +139,6 @@ public:
 			error("support data list index #1 was referenced - special handlng TODO");
 		return _charOffsets[index]; 
 	}
-	CharacterScheduleEntry *playerSupportRecord() { return _playerSupportRecord; }
 	void copyCursorTo(Surface *s, uint8 cursorNum, int16 x, int16 y);
 
 	uint16 numInventoryItems();
@@ -133,6 +147,7 @@ public:
 	void setTalkState(TalkState state) { _talkState = state; }
 	TalkState getTalkState() { return _talkState; }
 	TalkSelections &getTalkSelections() { return _talkSelections; }	
+	TalkDialogDetails &getTalkDetails() { return _talkDetails; }
 	void setTalkSelection(int index) { _talkSelection = index; }
 	int getTalkSelection() { return _talkSelection; }
 	void setTalkStartEntry(int index) { _talkStartEntry = index; }
@@ -145,12 +160,15 @@ public:
 	const char *getCurrentActionStr() { 
 		if (_currentAction > EXAMINE) 
 			error("Invalid current action %d", _currentAction);
-		return actionList[_currentAction]; 
+		return _stringList.getString(_currentAction);
 	}
-	void activateHotspot(uint16 hotspotId);
+	Hotspot *activateHotspot(uint16 hotspotId);
 	Hotspot *addHotspot(uint16 hotspotId);
 	void addHotspot(Hotspot *hotspot);
 	void deactivateHotspot(uint16 hotspotId, bool isDestId = false);
+	void deactivateHotspot(Hotspot *hotspot);
+	void saveToStream(Common::WriteStream *stream);
+	void loadFromStream(Common::ReadStream *stream);
 };
 
 } // End of namespace Lure
