@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2005-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,37 +18,36 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/lure/res_struct.h $
- * $Id: res_struct.h 22731 2006-05-29 08:12:07Z dreammaster $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/lure/res_struct.h $
+ * $Id: res_struct.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
-#ifndef __lure_resstruct_h__
-#define __lure_resstruct_h__
+#ifndef LURE_RESSTRUCT_H
+#define LURE_RESSTRUCT_H
 
 #include "lure/luredefs.h"
 #include "common/list.h"
+#include "common/file.h"
 
 namespace Lure {
 
 using namespace Common;
 
-extern const char *actionList[];
+extern const Action sortedActions[];
 
 /*-------------------------------------------------------------------------*/
 /* Structure definitions                                                   */
 /*                                                                         */
 /*-------------------------------------------------------------------------*/
 
-#if !defined(__GNUC__)
-#pragma START_PACK_STRUCTS
-#endif
+#include "common/pack-start.h"	// START STRUCT PACKING
 
 struct VersionStructure {
 	uint16 id;
 	byte vMajor;
 	byte vMinor;
-} GCC_PACK;
+};
 
 struct FileEntry {
 	uint16 id;
@@ -53,7 +55,7 @@ struct FileEntry {
 	byte sizeExtension;
 	uint16 size;
 	uint16 offset;
-} GCC_PACK;
+};
 
 struct HotspotResource {
 	uint16 hotspotId;
@@ -79,12 +81,17 @@ struct HotspotResource {
 	int8 talkY;
 	uint16 colourOffset;
 	uint16 animRecordId;
-	uint16 sequenceOffset;
+	uint16 hotspotScriptOffset;
+	uint16 talkScriptOffset;
 	uint16 tickProcOffset;
 	uint16 tickTimeout;
-	uint16 tickSequenceOffset;
+	uint16 tickScriptOffset;
 	uint16 npcSchedule;
-} GCC_PACK;
+	uint16 characterMode;
+	uint16 delayCtr;
+	byte flags2;
+	byte hdrFlags;
+};
 
 struct HotspotAnimResource {
 	uint16 animRecordId;
@@ -98,13 +105,13 @@ struct HotspotAnimResource {
 	uint8 downFrame;
 	uint8 leftFrame;
 	uint8 rightFrame;
-} GCC_PACK;
+};
 
 struct MovementResource {
 	uint16 frameNumber;
 	int16 xChange;
 	int16 yChange;
-} GCC_PACK;
+};
 
 struct RoomRect {
 	int16 xs, xe;
@@ -113,6 +120,9 @@ struct RoomRect {
 
 struct RoomResource {
 	uint16 roomNumber;
+	uint8 hdrFlags;
+	uint8 unused;
+	uint32 actions;
 	uint16 descId;
 	uint16 numLayers;
 	uint16 layers[4];
@@ -121,7 +131,7 @@ struct RoomResource {
 	int16 clippingXEnd;
 	RoomRect walkBounds;
 	uint16 numExits;
-} GCC_PACK;
+};
 
 struct RoomExitResource {
 	int16 xs, xe, ys, ye;
@@ -129,12 +139,12 @@ struct RoomExitResource {
 	uint8 newRoom;
 	uint8 direction;
 	int16 newRoomX, newRoomY;
-} GCC_PACK;
+};
 
 struct HotspotOverrideResource {
 	uint16 hotspotId;
 	int16 xs, xe, ys, ye;
-} GCC_PACK;
+};
 
 struct RoomExitHotspotResource {
 	uint16 hotspotId;
@@ -142,7 +152,7 @@ struct RoomExitHotspotResource {
 	int16 ys, ye;
 	uint16 cursorNum;
 	uint16 destRoomNumber;
-} GCC_PACK;
+};
 
 struct RoomExitJoinResource {
 	uint16 hotspot1Id;
@@ -156,41 +166,41 @@ struct RoomExitJoinResource {
 	uint8 h2OpenSound;
 	uint8 h2CloseSound;
 	byte blocked;
-} GCC_PACK;
+};
 
 struct HotspotActionResource {
 	byte action;
 	uint16 sequenceOffset;
-} GCC_PACK;
+};
 
 struct TalkHeaderResource {
 	uint16 hotspotId;
 	uint16 offset;
-} GCC_PACK;
+};
 
 struct TalkDataHeaderResource {
 	uint16 recordId;
 	uint16 listOffset;
 	uint16 responsesOffset;
-} GCC_PACK;
+};
 
 struct TalkDataResource {
 	uint16 preSequenceId;
 	uint16 descId;
 	uint16 postSequenceId;
-} GCC_PACK;
+};
 
 struct TalkResponseResource {
 	uint16 sequenceId1;
 	uint16 sequenceId2;
 	uint16 sequenceId3;
-} GCC_PACK;
+};
 
 struct RoomExitCoordinateResource {
 	int16 x;
 	int16 y;
 	uint16 roomNumber;
-} GCC_PACK;
+};
 
 #define ROOM_EXIT_COORDINATES_NUM_ENTRIES 6
 #define ROOM_EXIT_COORDINATES_NUM_ROOMS 52
@@ -198,7 +208,7 @@ struct RoomExitCoordinateResource {
 struct RoomExitCoordinateEntryResource {
 	RoomExitCoordinateResource entries[ROOM_EXIT_COORDINATES_NUM_ENTRIES];
 	uint8 roomIndex[ROOM_EXIT_COORDINATES_NUM_ROOMS];
-} GCC_PACK;
+};
 
 #define MAX_SCHEDULE_ENTRY_PARAMS 5
 
@@ -213,15 +223,14 @@ struct RoomExitIndexedHotspotResource {
 	uint16 hotspotId;
 };
 
-#if !defined(__GNUC__)
-#pragma END_PACK_STRUCTS
-#endif
-
-// Class template for a derived list that destroys the contained
-// object when the record containing it is destroyed. It's not
-// perfect, since the underlying list doesn't have virtual
-// methods, but it's sufficient for my usage
+#include "common/pack-end.h"	// END STRUCT PACKING
 
+/**
+ * Class template for a derived list that destroys the contained
+ * object when the record containing it is destroyed. It's not
+ * perfect, since the underlying list doesn't have virtual
+ * methods, but it's sufficient for my usage.
+ */
 template <class T>
 class ManagedList: public Common::List<T> {
 	typedef typename Common::List<T> Common_List;
@@ -251,8 +260,7 @@ public:
 	}
 };
 
-// Enumeration used for direction facings
-
+/** Enumeration used for direction facings */
 enum Direction {UP, DOWN, LEFT, RIGHT, NO_DIRECTION};
 
 // Support classes to hold loaded resources
@@ -299,13 +307,15 @@ class RoomPathsData {
 private:
 	byte _data[ROOM_PATHS_HEIGHT * ROOM_PATHS_WIDTH];
 public:
-	RoomPathsData() {};
+	RoomPathsData() {}
 	RoomPathsData(byte *srcData) { load(srcData); }
 
 	void load(byte *srcData) {
 		memcpy(_data, srcData, ROOM_PATHS_SIZE);
 	}
+	const byte *data() { return _data; }
 	bool isOccupied(int x, int y);
+	bool isOccupied(int x, int y, int width);
 	void setOccupied(int x, int y, int width);
 	void clearOccupied(int x, int y, int width);
 	void decompress(RoomPathsDecompressedData &dataOut, int characterWidth);
@@ -318,6 +328,9 @@ public:
 	RoomData(RoomResource *rec, MemoryBlock *pathData);
 
 	uint16 roomNumber;
+	uint8 hdrFlags;
+	uint8 flags;
+	uint32 actions;
 	uint16 descId;
 	uint16 numLayers;
 	uint16 layers[MAX_NUM_LAYERS];
@@ -330,7 +343,11 @@ public:
 	RoomPathsData paths;
 };
 
-typedef ManagedList<RoomData *> RoomDataList;
+class RoomDataList: public ManagedList<RoomData *> {
+public:
+	void saveToStream(WriteStream *stream);
+	void loadFromStream(ReadStream *stream);
+};
 
 class RoomExitJoinData {
 public:
@@ -350,7 +367,11 @@ public:
 	uint32 unknown;
 };
 
-typedef ManagedList<RoomExitJoinData *> RoomExitJoinList;
+class RoomExitJoinList: public ManagedList<RoomExitJoinData *> {
+public:
+	void saveToStream(WriteStream *stream);
+	void loadFromStream(ReadStream *stream);
+};
 
 class HotspotActionData {
 public:
@@ -373,8 +394,13 @@ public:
 	HotspotActionList *getActions(uint16 recordId);
 };
 
-enum CharacterMode {CHARMODE_NONE, CHARMODE_1, CHARMODE_IDLE, CHARMODE_PAUSED,
-	CHARMODE_4, CHARMODE_5, CHARMODE_6, CHARMODE_7};
+enum CharacterMode {CHARMODE_NONE, CHARMODE_HESITATE, CHARMODE_IDLE, CHARMODE_PAUSED,
+	CHARMODE_WAIT_FOR_PLAYER, CHARMODE_CONVERSING, CHARMODE_PLAYER_WAIT, 
+	CHARMODE_WAIT_FOR_INTERACT, CHARMODE_INTERACTING, CHARMODE_SPECIAL_PLAYER};
+
+enum BlockedState {BS_NONE, BS_INITIAL, BS_FINAL};
+
+enum VariantBool {VB_INITIAL, VB_FALSE, VB_TRUE};
 
 class HotspotData {
 public:
@@ -404,22 +430,45 @@ public:
 	int8 talkY;
 	uint16 colourOffset;
 	uint16 animRecordId;
-	uint16 sequenceOffset;
+	uint16 hotspotScriptOffset;
+	uint16 talkScriptOffset;
 	uint16 tickProcOffset;
 	uint16 tickTimeout;
-	uint16 tickSequenceOffset;
+	uint16 tickScriptOffset;
 	uint16 npcSchedule;
-
-	uint16 delayCtr;
 	CharacterMode characterMode;
-	bool coveredFlag;
+	uint16 delayCtr;
+	uint8 flags2;
+	uint8 headerFlags;
+
+	// Runtime fields
+	uint16 actionCtr;
+	BlockedState blockedState;
+	bool blockedFlag;
+	VariantBool coveredFlag;
+	uint16 talkMessageId;
+	uint16 talkDestCharacterId;
+	uint16 talkCountdown;
+	uint16 pauseCtr;
+	uint16 useHotspotId;	
+	uint16 talkGate;
+	uint16 actionHotspotId;
+	uint16 talkOverride;
+
+	uint16 use2HotspotId;
 
 	void enable() { flags |= 0x80; }
 	void disable() { flags &= 0x7F; }
 	Direction nonVisualDirection() { return (Direction) scriptLoadFlag; }
+	void saveToStream(WriteStream *stream);
+	void loadFromStream(ReadStream *stream);
 };
 
-typedef ManagedList<HotspotData *> HotspotDataList;
+class HotspotDataList: public ManagedList<HotspotData *> {
+public:
+	void saveToStream(WriteStream *stream);
+	void loadFromStream(ReadStream *stream);
+};
 
 class HotspotOverrideData {
 public:
@@ -543,18 +592,25 @@ public:
 // The following classes hold any sequence offsets that are being delayed
 
 class SequenceDelayData {
-	friend class SequenceDelayList;
 private:
-	uint32 _timeoutCtr;
-	uint16 _sequenceOffset;
+	SequenceDelayData() {};
 public:
-	SequenceDelayData(uint16 delay, uint16 seqOffset);
+	SequenceDelayData(uint16 delay, uint16 seqOffset, bool canClearFlag);
+	static SequenceDelayData *load(uint32 delay, uint16 seqOffset, bool canClearFlag);
+
+	uint32 timeoutCtr;
+	uint16 sequenceOffset;
+	bool canClear;
 };
 
 class SequenceDelayList: public ManagedList<SequenceDelayData *> {
 public:
-	void addSequence(uint16 delay, uint16 seqOffset);
+	void add(uint16 delay, uint16 seqOffset, bool canClear);
 	void tick();
+	void clear(bool forceClear = false);
+
+	void saveToStream(WriteStream *stream);
+	void loadFromStream(ReadStream *stream);
 };
 
 // The following classes holds the data for NPC schedules
@@ -567,7 +623,8 @@ class CharacterScheduleEntry {
 private:
 	CharacterScheduleSet *_parent;
 	Action _action;
-	uint16 _params[MAX_SCHEDULE_ENTRY_PARAMS];
+	uint16 _params[MAX_TELL_COMMANDS * 3];
+	int _numParams;
 public:
 	CharacterScheduleEntry() { _action = NONE; _parent = NULL; }
 	CharacterScheduleEntry(Action theAction, ...);
@@ -575,9 +632,10 @@ public:
 		CharacterScheduleResource *&rec);
 
 	Action action() { return _action; }
-	int numParams();
+	int numParams() { return _numParams; }
 	uint16 param(int index);
 	void setDetails(Action theAction, ...);
+	void setDetails2(Action theAction, int numParamEntries, uint16 *paramList);
 	CharacterScheduleEntry *next();
 	CharacterScheduleSet *parent() { return _parent; }
 	uint16 id();
@@ -599,6 +657,122 @@ public:
 
 typedef List<uint16> CharacterScheduleOffsets;
 
+// The follow classes are used to store the NPC schedule Ids for the random actions a follower can do in each room
+
+enum RandomActionType {REPEATABLE, REPEAT_ONCE, REPEAT_ONCE_DONE};
+
+class RandomActionSet {
+private:
+	uint16 _roomNumber;
+	int _numActions;
+	RandomActionType *_types;
+	uint16 *_ids;
+public:
+	RandomActionSet(uint16 *&offset);
+	~RandomActionSet();
+
+	uint16 roomNumber() { return _roomNumber; }
+	int numActions() { return _numActions; }
+	void getEntry(int index, RandomActionType &actionType, uint16 &id) {
+		assert((index >= 0) && (index < _numActions));
+		actionType = _types[index];
+		id = _ids[index];
+	}
+	void setDone(int index) {
+		assert((index >= 0) && (index < _numActions));
+		assert(_types[index] == REPEAT_ONCE);
+		_types[index] = REPEAT_ONCE_DONE;
+	}
+	void saveToStream(Common::WriteStream *stream);
+	void loadFromStream(Common::ReadStream *stream);
+};
+
+class RandomActionList: public ManagedList<RandomActionSet *> {
+public:
+	RandomActionSet *getRoom(uint16 roomNumber);
+	void saveToStream(Common::WriteStream *stream);
+	void loadFromStream(Common::ReadStream *stream);
+};
+
+class PausedCharacter {
+public:
+	PausedCharacter(uint16 SrcCharId, uint16 DestCharId);
+
+	uint16 srcCharId;
+	uint16 destCharId;
+	uint16 counter;
+	HotspotData *charHotspot;
+};
+
+class Hotspot;
+
+class PausedCharacterList: public ManagedList<PausedCharacter *> {
+public:
+	void reset(uint16 hotspotId);
+	void countdown();
+	void scan(Hotspot &h);
+	int check(uint16 charId, int numImpinging, uint16 *impingingList);
+};
+
+struct ServeEntry {
+	uint16 hotspotId;
+	uint8 serveFlags;
+};
+
+#define NUM_SERVE_CUSTOMERS 4
+
+enum BarmanGraphicType {BG_RANDOM = 0, BG_BEER = 1, BG_EXTRA1 = 2, BG_EXTRA2 = 3};
+
+struct BarEntry {
+	uint16 roomNumber;
+	uint16 barmanId;
+	ServeEntry customers[NUM_SERVE_CUSTOMERS];
+	uint16 *graphics[4];
+	uint16 gridLine;
+	ServeEntry *currentCustomer;
+};
+
+class BarmanLists {
+public:
+	BarEntry &getDetails(uint16 roomNumber);
+	void saveToStream(Common::WriteStream *stream);
+	void loadFromStream(Common::ReadStream *stream);
+};
+
+enum BarmanAction {WALK_AROUND = 1, POLISH_BAR = 2, WAIT = 3, WAIT_DIALOG = 4, SERVE_BEER = 5};
+
+struct RoomTranslationRecord {
+	uint8 srcRoom;
+	uint8 destRoom;
+};
+
+extern RoomTranslationRecord roomTranslations[];
+
+enum StringEnum {S_CREDITS = 25, S_RESTART_GAME = 26, S_SAVE_GAME = 27, S_RESTORE_GAME = 28, 
+	S_QUIT = 29, S_FAST_TEXT = 30, S_SLOW_TEXT = 31, S_SOUND_ON = 32, S_SOUND_OFF = 33, 
+	S_NOTHING = 34, S_FOR = 35, S_TO = 36, S_ON = 37, S_AND_THEN = 38,
+	S_FINISH = 39};
+
+class StringList {
+private:
+	MemoryBlock *_data;
+	int _numEntries;
+	char **_entries;
+public:
+	StringList() { _numEntries = 0; }
+	~StringList() { clear(); }
+
+	void load(MemoryBlock *data);
+	void clear();
+	int count() { return _numEntries; }
+	const char *getString(int index) {
+		if ((index < 0) || (index >= _numEntries)) error("Invalid index specified to String List");
+		return _entries[index];
+	}
+	const char *getString(Action action) { return getString((int) action - 1); }
+	const char *getString(StringEnum sEnum) { return getString((int) sEnum); }
+};
+
 // The following class holds the field list used by the script engine as 
 // well as miscellaneous fields used by the game.                          
 
@@ -611,6 +785,7 @@ enum FieldName {
 	ACTIVE_HOTSPOT_ID = 3,
 	SEQUENCE_RESULT = 4,
 	GENERAL = 5,
+	GIVE_TALK_INDEX = 6,
 	NEW_ROOM_NUMBER = 7,
 	OLD_ROOM_NUMBER = 8,
 	CELL_DOOR_STATE = 9,
@@ -618,7 +793,19 @@ enum FieldName {
 	PRISONER_DEAD = 15,
 	BOTTLE_FILLED = 18,
 	TALK_INDEX = 19,
-	SACK_CUT = 20
+	SACK_CUT = 20,
+	ROOM_EXIT_ANIMATION = 76
+};
+
+enum GameFlags {
+	GAMEFLAG_1 = 1,
+	GAMEFLAG_2 = 2,
+	GAMEFLAG_4 = 4,
+	GAMEFLAG_8 = 8,
+	GAMEFLAG_10 = 0x10,
+	GAMEFLAG_20 = 0x20,
+	GAMEFLAG_40 = 0x40,
+	GAMEFLAG_FAST_TEXTSPEED = 0x80
 };
 
 struct PlayerNewPosition {
@@ -626,10 +813,18 @@ struct PlayerNewPosition {
 	uint16 roomNumber;
 };
 
+struct PlayerPendingPosition {
+	Point pos;
+	bool isSet;
+};
+
 class ValueTableData {
 private:
 	uint16 _numGroats;
 	PlayerNewPosition _playerNewPos;
+	PlayerPendingPosition _playerPendingPos;
+	uint8 _flags;
+	uint8 _hdrFlagMask;
 
 	uint16 _fieldList[NUM_VALUE_FIELDS];
 	bool isKnownField(uint16 fieldIndex);
@@ -640,8 +835,16 @@ public:
 
 	void setField(uint16 fieldIndex, uint16 value);
 	void setField(FieldName fieldName, uint16 value);
+	int size() { return NUM_VALUE_FIELDS; }
+
 	uint16 &numGroats() { return _numGroats; }
+	uint8 &flags() { return _flags; }
+	uint8 &hdrFlagMask() { return _hdrFlagMask; }
 	PlayerNewPosition &playerNewPos() { return _playerNewPos; }
+	PlayerPendingPosition &playerPendingPos() { return _playerPendingPos; }
+
+	void saveToStream(Common::WriteStream *stream);
+	void loadFromStream(Common::ReadStream *stream);
 };
 
 } // End of namespace Lure
