@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2005-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/lure/room.cpp $
- * $Id: room.cpp 22731 2006-05-29 08:12:07Z dreammaster $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/lure/room.cpp $
+ * $Id: room.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -78,6 +81,7 @@ Room::Room(): _screen(Screen::getReference()) {
 	_talkDialog = NULL;
 	_hotspotId = 0;
 	_hotspotName[0] = '\0';
+	_statusLine[0] = '\0';
 	for (int ctr = 0; ctr < MAX_NUM_LAYERS; ++ctr) _layers[ctr] = NULL;
 	_numLayers = 0;
 	_showInfo = false;
@@ -116,7 +120,7 @@ void Room::leaveRoom() {
 		}
 
 	// Scan through the hotspot list and remove any uneeded entries 
-//r.activeHotspots().clear();
+
 	HotspotList &list = r.activeHotspots();
 	HotspotList::iterator i = list.begin();
 	while (i != list.end()) {
@@ -144,54 +148,67 @@ void Room::loadRoomHotspots() {
 }
 
 void Room::checkRoomHotspots() {
+	uint16 rangeStart[4] = {0x408, 0x3e8, 0x7530, 0x2710};
+	uint16 rangeEnd[4] = {0x270f, 0x407, 0xffff, 0x752f};
+
 	Mouse &m = Mouse::getReference();
 	Resources &res = Resources::getReference();
 	HotspotDataList &list = res.hotspotData();
 	HotspotData *entry = NULL;
 	int16 currentX = m.x();
 	int16 currentY = m.y();
-
 	HotspotDataList::iterator i;
-	for (i = list.begin(); i != list.end(); ++i) {
-		entry = *i;
 
-		bool skipFlag = (entry->roomNumber != _roomNumber);
-		if (!skipFlag) {
-			skipFlag = (((entry->flags & 0x80) == 0) && 
-				        ((entry->flags & 0x40) != 0)) ||
-				       ((entry->flags & 0x20) != 0);
-		}
+	// Loop for each range of hotspot Ids
+	for (int ctr = 0; ctr < 4; ++ctr) {
+		for (i = list.begin(); i != list.end(); ++i) {
+			entry = *i;
+			if ((entry->hotspotId < rangeStart[ctr]) || (entry->hotspotId > rangeEnd[ctr]))
+				// Hotspot outside range, so skip it
+				continue;
 
-		if ((!skipFlag) && (entry->hotspotId < 0x409))
-			// For character hotspots, validate they're in clipping range
-			skipFlag = !res.checkHotspotExtent(entry);
-		
-		if (!skipFlag && (entry->hotspotId >= 0x2710) && (entry->hotspotId <= 0x27ff)) {
-			RoomExitJoinData *rec = res.getExitJoin(entry->hotspotId);
-			if ((rec) && (!rec->blocked))
-				// Hotspot is over a room exit, and it's not blocked, so don't 
-				// register it as an active hotspot
-				skipFlag = true;
-		}
+			bool skipFlag = (entry->roomNumber != _roomNumber);
+			if (!skipFlag) {
+				skipFlag = (((entry->flags & HOTSPOTFLAG_FOUND) == 0) && 
+							((entry->flags & HOTSPOTFLAG_SKIP) != 0)) ||
+						    ((entry->flags & HOTSPOTFLAG_20) != 0);
+			}
 
-		if (!skipFlag) {
-			// Check for a hotspot override
-			HotspotOverrideData *hsEntry = res.getHotspotOverride(entry->hotspotId);
+			if ((!skipFlag) && (entry->hotspotId < 0x409))
+				// For character hotspots, validate they're in clipping range
+				skipFlag = !res.checkHotspotExtent(entry);
+			
+			if (!skipFlag && (entry->hotspotId >= 0x2710) && (entry->hotspotId <= 0x27ff)) {
+				RoomExitJoinData *rec = res.getExitJoin(entry->hotspotId);
+				if ((rec) && (!rec->blocked))
+					// Hotspot is over a room exit, and it's not blocked, so don't 
+					// register it as an active hotspot
+					skipFlag = true;
+			}
 
-			if (hsEntry) {
-				// Check whether cursor is in override hotspot area
-				if ((currentX >= hsEntry->xs) && (currentX <= hsEntry->xe) &&
-					(currentY >= hsEntry->ys) && (currentY <= hsEntry->ye))
-					// Found to be in hotspot entry
-					break;
-			} else {
-				// Check whether cursor is in default hospot area
-				if ((currentX >= entry->startX) && (currentX < entry->startX + entry->width) && 
-					(currentY >= entry->startY) && (currentY < entry->startY + entry->height)) 
-					// Found hotspot entry
-					break;
+			if (!skipFlag) {
+				// Check for a hotspot override
+				HotspotOverrideData *hsEntry = res.getHotspotOverride(entry->hotspotId);
+
+				if (hsEntry) {
+					// Check whether cursor is in override hotspot area
+					if ((currentX >= hsEntry->xs) && (currentX <= hsEntry->xe) &&
+						(currentY >= hsEntry->ys) && (currentY <= hsEntry->ye))
+						// Found to be in hotspot entry
+						break;
+				} else {
+					// Check whether cursor is in default hospot area
+					if ((currentX >= entry->startX) && (currentY >= entry->startY) &&
+						(currentX < entry->startX + entry->widthCopy) && 
+						(currentY < entry->startY + entry->height)) 
+						// Found hotspot entry
+						break;
+				}
 			}
 		}
+
+		if (i != list.end()) 
+			break;
 	}
 
 	if (i == list.end()) {
@@ -204,6 +221,7 @@ void Room::checkRoomHotspots() {
 		_hotspot = entry;
 		_hotspotId = entry->hotspotId;
 		_isExit = false;
+		entry->flags |= HOTSPOTFLAG_FOUND;
 	}
 }
 
@@ -280,11 +298,9 @@ void Room::addAnimation(Hotspot &h) {
 	if (_showInfo) {
 		int16 x = h.x();
 		int16 y = h.y();
-		if ((x >= 0) && (x <= 319) && (y >= 0) && (y <= 200)) {
-			sprintf(buffer, "%x", h.hotspotId());
-			strcat(buffer, "h");
-			s.writeString(h.x(), h.y(), buffer, false);
-		}
+		if ((x >= 0) && (x < FULL_SCREEN_WIDTH) && (y >= 0) && (y < FULL_SCREEN_HEIGHT)) 
+			sprintf(buffer, "%xh", h.hotspotId());
+		
 	}
 }
 
@@ -298,7 +314,7 @@ void Room::addLayers(Hotspot &h) {
 	int16 yStart = hsY / RECT_SIZE;
 	int16 yEnd = (hsY + h.heightCopy() - 1) / RECT_SIZE;
 	int16 numY = yEnd - yStart + 1;
-
+	
 	for (int16 xCtr = 0; xCtr < numX; ++xCtr, ++xStart) {
 		int16 xs = xStart - 4;
 		if (xs < 0) continue;
@@ -346,7 +362,6 @@ void Room::addCell(int16 xp, int16 yp, int layerNum) {
 void Room::update() {
 	Surface &s = _screen.screen();
 	Resources &res = Resources::getReference();
-	StringData &strings = StringData::getReference();
 	HotspotList &hotspots = res.activeHotspots();
 	HotspotList::iterator i;
 
@@ -371,8 +386,8 @@ void Room::update() {
 	List<Hotspot *>::iterator iTemp;
 	for (i = hotspots.begin(); i != hotspots.end(); ++i) {
 		Hotspot *h = i.operator*();
-		if ((h->roomNumber() != _roomNumber) || !h->isActiveAnimation() 
-				|| (h->layer() != 1)) 
+		if ((h->layer() != 1) || (h->roomNumber() != _roomNumber) || 
+			h->skipFlag() || !h->isActiveAnimation())
 			continue;
 		int16 endY = h->y() + h->heightCopy();
 
@@ -385,7 +400,6 @@ void Room::update() {
 	}
 	for (iTemp = tempList.begin(); iTemp != tempList.end(); ++iTemp) {
 		Hotspot &h = *iTemp.operator*();
-
 		flagCoveredCells(h);
 		addAnimation(h);
 		addLayers(h);
@@ -417,62 +431,57 @@ void Room::update() {
 
 	// Show any active talk dialog 
 	if (_talkDialog)  {
-		_talkDialog->surface().copyTo(&s, _talkDialogX, _talkDialogY);
+		// Make sure the character is still active and in the viewing room
+		Hotspot *talkCharacter = res.getActiveHotspot(res.getTalkingCharacter());
+		if ((talkCharacter != NULL) && (talkCharacter->roomNumber() == _roomNumber)) 
+			_talkDialog->surface().copyTo(&s, _talkDialogX, _talkDialogY);
 	}
 
-	// Handle showing name of highlighted hotspot
-	if (_hotspotId != 0) {
-		Action action = res.getCurrentAction();
-		uint16 usedId = res.fieldList().getField(USE_HOTSPOT_ID);
-
-		if (action == NONE) {
+	// Handle showing the status line
+	if (!*_statusLine) {
+		// No current status action being display
+		if (_hotspotId != 0) 
 			s.writeString(0, 0, _hotspotName, false, DIALOG_TEXT_COLOUR);
-		} else {
-			char buffer[MAX_DESC_SIZE];
-			const char *actionStr = res.getCurrentActionStr();
-			strcpy(buffer, actionStr);
-
-			if (action != STATUS) {
-				strcat(buffer, " ");
-
-				if ((usedId != 0xffff) && (usedId != 0)) {
-					HotspotData *usedHotspot = res.getHotspot(usedId);
-
-					if (usedHotspot != NULL) 
-						strings.getString(usedHotspot->nameId, buffer + strlen(buffer), NULL, NULL);
-					else
-						strcat(buffer, "???");
-
-					if (action == GIVE) strcat(buffer, " to ");
-					else strcat(buffer, " on ");
-				}
-
-				strcat(buffer, _hotspotName);
-			}
-			s.writeString(0, 0, buffer, false, DIALOG_WHITE_COLOUR);
+	} else {
+		// Word wrap (if necessary) the status line and dispaly it
+		char *statusLineCopy = strdup(_statusLine);
+		char **lines;
+		uint8 numLines;
+		int16 yPos = 0;
+		s.wordWrap(statusLineCopy, s.width(), lines, numLines);
+		for (int lineNum = 0; lineNum < numLines; ++lineNum) {
+			s.writeString(0, yPos, lines[lineNum], false, DIALOG_WHITE_COLOUR);
+			yPos += FONT_HEIGHT;
 		}
+		Memory::dealloc(lines);
+		Memory::dealloc(statusLineCopy);
 	}
 
 	// If show information is turned on, show extra debugging information
 	if (_showInfo) {
 		char buffer[64];
-		Mouse &m = Mouse::getReference();
-		sprintf(buffer, "Room %d Pos (%d,%d)", _roomNumber, m.x(), m.y());
-		s.writeString(FULL_SCREEN_WIDTH / 2, 0, buffer, false, DIALOG_TEXT_COLOUR);
 
 		// Temporary display of pathfinding data
 		for (int yctr = 0; yctr < ROOM_PATHS_HEIGHT; ++yctr) {
 			for (int xctr = 0; xctr < ROOM_PATHS_WIDTH; ++xctr) {
+/*
+				if (_roomData->paths.isOccupied(xctr, yctr)) 
+					s.fillRect(Rect(xctr * 8, yctr * 8 + 8, xctr * 8 + 7, yctr * 8 + 15), 255);
+*/
 				uint16 v = tempLayer[(yctr + 1) * DECODED_PATHS_WIDTH + xctr + 1];
 				if ((v != 0) && (v < 100)) {
 					sprintf(buffer, "%d", v % 10);
-//					s.writeString(xctr * 8, yctr * 8 + 8, buffer, true);
+					s.writeString(xctr * 8, yctr * 8 + 8, buffer, true);
 //				} else if (v == 0xffff) {
 				} else if (_roomData->paths.isOccupied(xctr, yctr)) {
 					s.fillRect(Rect(xctr * 8, yctr * 8 + 8, xctr * 8 + 7, yctr * 8 + 15), 255);
 				}
 			}
 		}
+
+		Mouse &m = Mouse::getReference();
+		sprintf(buffer, "Room %d Pos (%d,%d)", _roomNumber, m.x(), m.y());
+		s.writeString(FULL_SCREEN_WIDTH / 2, 0, buffer, false, DIALOG_TEXT_COLOUR);
 	}
 }
 
@@ -484,17 +493,15 @@ void Room::setRoomNumber(uint16 newRoomNumber, bool showOverlay) {
 
 	_roomNumber = _roomData->roomNumber;
 	_descId = _roomData->descId;
-	
+
 	_screen.empty();
 	_screen.resetPalette();
 
 	if (_layers[0]) leaveRoom();
-
 	_numLayers = _roomData->numLayers;
 	if (showOverlay) ++_numLayers;
 
 	uint16 paletteId = (_roomData->layers[0] & 0xffe0) - 1;
-
 	for (uint8 layerNum = 0; layerNum < _numLayers; ++layerNum) 
 		_layers[layerNum] = new RoomLayer(_roomData->layers[layerNum],
 			layerNum == 0);
@@ -507,11 +514,14 @@ void Room::setRoomNumber(uint16 newRoomNumber, bool showOverlay) {
 	r.insertPaletteSubset(p);
 	_screen.setPalette(&p);
 
+	// Set the new room number
+	r.fieldList().setField(ROOM_NUMBER, newRoomNumber);
+
 	if (_roomData->sequenceOffset != 0xffff)
 		Script::execute(_roomData->sequenceOffset);
+
 	loadRoomHotspots();
 	checkCursor();
-
 	update();
 }
 
@@ -525,13 +535,14 @@ void Room::checkCursor() {
 	uint16 currentCursor = mouse.getCursorNum();
 	uint16 newCursor = currentCursor;
 	CurrentAction playerAction = res.getActiveHotspot(PLAYER_ID)->currentActions().action();
+	uint16 oldRoomNumber = res.fieldList().getField(OLD_ROOM_NUMBER);
 
 	if ((currentCursor >= CURSOR_TIME_START) && (currentCursor <= CURSOR_TIME_END) &&
 		((playerAction == START_WALKING) || (playerAction == PROCESSING_PATH))) {
 		// Animate the clock when processing the player path
 		++newCursor;
 		if (newCursor == CURSOR_CROSS) newCursor = CURSOR_TIME_START;
-	} else if (checkInTalkDialog()) {
+	} else if (checkInTalkDialog() && (oldRoomNumber == 0)) {
 		newCursor = CURSOR_TALK;
 	} else if (res.getTalkData()) {
 		newCursor = CURSOR_ARROW;
@@ -541,7 +552,6 @@ void Room::checkCursor() {
 		newCursor = CURSOR_ARROW;
 	} else if (mouse.y() < MENUBAR_Y_SIZE) {
 		// If viewing a room remotely, then don't change to the menu cursor
-		uint16 oldRoomNumber = res.fieldList().getField(OLD_ROOM_NUMBER);
 		if (oldRoomNumber != 0) return;
 		
 		newCursor = CURSOR_MENUBAR;
@@ -560,33 +570,33 @@ void Room::checkCursor() {
 		}
 
 		if (oldHotspotId != _hotspotId) 
-			StringData::getReference().getString(_hotspotNameId, _hotspotName, NULL, NULL);
+			StringData::getReference().getString(_hotspotNameId, _hotspotName);
 	}
 
 	if (mouse.getCursorNum() != newCursor) 
 		mouse.setCursorNum(newCursor);
 }
 
-void Room::setTalkDialog(uint16 characterId, uint16 stringId) {
+void Room::setTalkDialog(uint16 srcCharacterId, uint16 destCharacterId, uint16 usedId, uint16 stringId) {
+	debugC(ERROR_DETAILED, kLureDebugAnimations, "Room::setTalkDialog - char=%xh string=%d",
+		srcCharacterId, stringId);
+
 	if (_talkDialog) {
 		delete _talkDialog;
 		_talkDialog = NULL;
 	}
 
 	Resources &res = Resources::getReference();
-	res.setTalkingCharacter(characterId);
+	res.setTalkingCharacter(srcCharacterId);
 
-	if (characterId == 0) {
-		if (res.getTalkState() == TALK_RESPONSE_WAIT)
-			res.setTalkState(TALK_RESPOND_2);
+	if (srcCharacterId == 0)
 		return;
-	}
 
-	HotspotData *character = res.getHotspot(characterId);
+	HotspotData *character = res.getHotspot(srcCharacterId);
 	if (character->roomNumber != _roomNumber)
 		return;
 
-	_talkDialog = new TalkDialog(characterId, stringId);
+	_talkDialog = new TalkDialog(srcCharacterId, destCharacterId, usedId, stringId);
 	_talkDialogX = character->startX + (character->width / 2) - (TALK_DIALOG_WIDTH / 2);
 
 	if (_talkDialogX < 0) _talkDialogX = 0;
@@ -610,4 +620,20 @@ bool Room::checkInTalkDialog() {
 		(mouse.y() < _talkDialogY + _talkDialog->surface().height()));
 }
 
+void Room::saveToStream(Common::WriteStream *stream) {
+	stream->writeUint16LE(_roomNumber);
+	stream->writeUint16LE(_destRoomNumber);
+	stream->writeByte(_showInfo);
+	stream->writeUint16LE(_cursorState);
+}
+
+void Room::loadFromStream(Common::ReadStream *stream) {
+	int roomNum = stream->readUint16LE();
+	setRoomNumber(roomNum, false);
+
+	_destRoomNumber = stream->readUint16LE();
+	_showInfo = stream->readByte() != 0;
+	_cursorState = (CursorState) stream->readUint16LE();
+}
+
 } // end of namespace Lure
