@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2005-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,13 +18,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/lure/room.h $
- * $Id: room.h 22731 2006-05-29 08:12:07Z dreammaster $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/lure/room.h $
+ * $Id: room.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
-#ifndef __lure_room_h__
-#define __lure_room_h__
+#ifndef LURE_ROOM_H
+#define LURE_ROOM_H
 
 #include "common/stdafx.h"
 #include "common/scummsys.h"
@@ -64,6 +67,7 @@ private:
 	uint16 _destRoomNumber;
 	bool _isExit;
 	char _hotspotName[MAX_HOTSPOT_NAME_SIZE + MAX_ACTION_NAME_SIZE];
+	char _statusLine[MAX_DESC_SIZE];
 	HotspotData *_hotspot;
 	bool _showInfo;
 	uint8 _numLayers;
@@ -103,9 +107,13 @@ public:
 	bool showInfo() { return _showInfo; }
 	CursorState cursorState() { return _cursorState; }
 	void setShowInfo(bool value) { _showInfo = value; }
-	void setTalkDialog(uint16 characterId, uint16 descId);
+	void setTalkDialog(uint16 srcCharacterId, uint16 destCharacterId, uint16 usedId, uint16 stringId);
 	void setCursorState(CursorState state) { _cursorState = state; }
+	bool isDialogActive() { return _talkDialog != NULL; }
 	bool checkInTalkDialog();
+	char *statusLine() { return _statusLine; }
+	void saveToStream(Common::WriteStream *stream);
+	void loadFromStream(Common::ReadStream *stream);
 };
 
 } // end of namespace Lure
