@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2005-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,14 +18,15 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/lure/scripts.cpp $
- * $Id: scripts.cpp 22576 2006-05-23 12:24:50Z dreammaster $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/lure/scripts.cpp $
+ * $Id: scripts.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
 #include "lure/scripts.h"
 #include "lure/res.h"
 #include "lure/game.h"
+#include "lure/room.h"
 #include "common/stack.h"
 #include "common/endian.h"
 
@@ -54,10 +58,29 @@ void Script::activateHotspot(uint16 hotspotId, uint16 v2, uint16 v3) {
 // Sets a hotspot's animation script offset from a master table of offsets
 
 void Script::setHotspotScript(uint16 hotspotId, uint16 scriptIndex, uint16 v3) {
-	Resources &r = Resources::getReference();
-	uint16 offset = r.getHotspotScript(scriptIndex);
-	HotspotData *rsc = r.getHotspot(hotspotId);
-	rsc->sequenceOffset = offset;
+	Resources &res = Resources::getReference();
+	uint16 offset = res.getHotspotScript(scriptIndex);
+	Hotspot *hotspot = res.getActiveHotspot(hotspotId);
+
+	if (hotspot != NULL) {
+		hotspot->setHotspotScript(offset);
+	} else {
+		HotspotData *hs = res.getHotspot(hotspotId);
+		assert(hs);
+		hs->hotspotScriptOffset = offset;
+	}
+}
+
+void Script::method2(uint16 v1, uint16 v2, uint16 v3) {
+	// Not yet implemented
+}
+
+// Sets the bitmask indicating what group of rooms/hotspots to display when the
+// player is issuing commands via the "Tell" action
+
+void Script::setHotspotFlagMask(uint16 maskVal, uint16 v2, uint16 v3) {
+	ValueTableData &fields = Resources::getReference().fieldList();
+	fields.hdrFlagMask() = (uint8) maskVal;
 }
 
 // Clears the sequence delay list
@@ -80,6 +103,18 @@ void Script::deactivateHotspotSet(uint16 listIndex, uint16 v2, uint16 v3) {
 	}
 }
 
+// Resets the pathfinder
+
+void Script::resetPathfinder(uint16 v1, uint16 v2, uint16 v3) {
+	// Reset the pathfinding for any characters
+	Resources &res = Resources::getReference();
+
+	for (uint16 charId = PLAYER_ID + 1; charId < FIRST_NONCHARACTER_ID; ++charId) {
+		Hotspot *charHotspot = res.getActiveHotspot(charId);
+		if (charHotspot) charHotspot->pathFinder().clear();
+	}
+}
+
 // deactivates the specified hotspot from active animation
 
 void Script::deactivateHotspot(uint16 hotspotId, uint16 v2, uint16 v3) {
@@ -87,6 +122,7 @@ void Script::deactivateHotspot(uint16 hotspotId, uint16 v2, uint16 v3) {
 	if (hotspotId < START_NONVISUAL_HOTSPOT_ID) 
 		rsc.deactivateHotspot(hotspotId);
 	HotspotData *hs = rsc.getHotspot(hotspotId);
+	hs->roomNumber = 0xffff;
 	hs->flags |= 0x20;
 	if (hotspotId < START_NONVISUAL_HOTSPOT_ID) 
 		hs->layer = 0xff;
@@ -95,22 +131,18 @@ void Script::deactivateHotspot(uint16 hotspotId, uint16 v2, uint16 v3) {
 // Sets the offset for the table of action sequence offsets for the given
 // hotspot
 
-void Script::setActionsOffset(uint16 hotspotId, uint16 offset, uint16 v3) {
+void Script::setDesc(uint16 hotspotId, uint16 descId, uint16 v3) {
 	Resources &res = Resources::getReference();
 	HotspotData *hotspot = res.getHotspot(hotspotId);
-	
-	if (!res.getHotspotActions(offset))
-		warning("Hotspot %xh set to invalid actions offset %d",
-			hotspotId, offset);
-	
-	hotspot->actionsOffset = offset;
+	assert(hotspot);
+	hotspot->descId = descId;
 }
 
 // Add a sequence to be executed after a specified delay
 
-void Script::addDelayedSequence(uint16 seqOffset, uint16 delay, uint16 v3) {
+void Script::addDelayedSequence(uint16 seqOffset, uint16 delay, uint16 canClear) {
 	SequenceDelayList &list = Resources::getReference().delayList();
-	list.addSequence(delay, seqOffset);
+	list.add(delay, seqOffset, canClear != 0);
 }
 
 // Checks whether the given character is in the specified room, and stores
@@ -157,14 +189,17 @@ void Script::remoteRoomViewSetup(uint16 v1, uint16 v2, uint16 v3) {
 		player->roomNumber());
 }
 
-// Checks the status of the cell door, and starts music depending on it's state
+// Starts a character speaking out loud to themselves (ie. no destination character)
 
-void Script::checkCellDoor(uint16 v1, uint16 v2, uint16 v3) {
-	// In the original game, this method checks to see if the cell door
-	// is currently open, if it is, starts a music sequence. 
-	// TODO: Implement starting music if cell door is open
+void Script::startSpeakingToNoone(uint16 characterId, uint16 stringId, uint16 v3) {
+	Resources &res = Resources::getReference();
+	Hotspot *charHotspot = res.getActiveHotspot(characterId);
+	assert(charHotspot);
+
+	charHotspot->converse(NOONE_ID, stringId, false);
 }
 
+
 void Script::playMusic(uint16 musicNum, uint16 v2, uint16 v3) {
 	// TODO: Play a given music
 	warning("TODO: Play music #%d", musicNum);
@@ -188,6 +223,49 @@ void Script::isSkorlInCell(uint16 v1, uint16 v2, uint16 v3) {
 	res.fieldList().setField(GENERAL, v);
 }
 
+// Called by the script when Ratpouch is pushing the bricks in the Outer Cell
+
+void Script::ratpouchPushBricks(uint16 v1, uint16 v2, uint16 v3)
+{
+	Resources &res = Resources::getReference();
+
+	// Mark the bricks exit as now open
+	RoomExitJoinData *joinRec = res.getExitJoin(BRICKS_ID);
+	joinRec->blocked = 0;
+
+	// Set Ratpouch moving through the new exit to room #7
+	Hotspot *ratpouchHotspot = res.getActiveHotspot(RATPOUCH_ID);
+	ratpouchHotspot->setActions(0); // Make sure he can't be interrupted
+	ratpouchHotspot->currentActions().clear();
+	ratpouchHotspot->currentActions().addBack(DISPATCH_ACTION, 7);
+}
+
+// Causes the current character to change to the specified room and position
+
+void Script::characterChangeRoom(uint16 y, uint16 x, uint16 roomNumber) {
+	Resources &res = Resources::getReference();
+	ValueTableData &fields = res.fieldList();
+	Hotspot *charHotspot = res.getActiveHotspot(fields.getField(CHARACTER_HOTSPOT_ID));
+	assert(charHotspot);
+
+	uint16 newRoomNumber = roomNumber & 0xff;
+	Direction newDirection = (Direction)(roomNumber >> 8);
+
+	Support::characterChangeRoom(*charHotspot, newRoomNumber, 
+		(int16) (x - 0x80), (int16) (y - 0x80), newDirection);
+}
+
+// Pauses Ratpouch for a long period (as good as idefinite)
+
+void Script::pauseRatpouch(uint16 v1, uint16 v2, uint16 v3)
+{
+	Resources &res = Resources::getReference();
+	Hotspot *ratpouch = res.getActiveHotspot(RATPOUCH_ID);
+	assert(ratpouch);
+	ratpouch->setCharacterMode(CHARMODE_PAUSED);
+	ratpouch->setDelayCtr(0x7fff);
+}
+
 // Sets a character to a given hotspot script, and sets the character's current 
 // action to executing a script
 
@@ -196,7 +274,7 @@ void Script::setBlockingHotspotScript(uint16 charId, uint16 scriptIndex, uint16
 	uint16 offset = r.getHotspotScript(scriptIndex);
 
 	Hotspot *hs = r.getActiveHotspot(charId);
-	hs->setScript(offset);
+	hs->setHotspotScript(offset);
 	hs->currentActions().top().setAction(EXEC_HOTSPOT_SCRIPT);
 	hs->setOccupied(true);
 }
@@ -208,11 +286,25 @@ void Script::decrInventoryItems(uint16 v1, uint16 v2, uint16 v3) {
 	// inventory items, so don't do anything
 }
 
-// Sets the current frame number for the given hotspot
+void Script::setTalking(uint16 characterId, uint16 destHotspot, uint16 messageId) {
+	Hotspot *hotspot = Resources::getReference().getActiveHotspot(characterId);
+	if (hotspot)
+		hotspot->converse(destHotspot, messageId, false);	
+}
 
-void Script::setFrameNumber(uint16 hotspotId, uint16 frameNumber, uint16 v3) {
-	Hotspot *hotspot = Resources::getReference().getActiveHotspot(hotspotId);
-	hotspot->setFrameNumber(frameNumber);
+// Sets the current action ctr value for the given hotspot
+
+void Script::setActionCtr(uint16 hotspotId, uint16 value, uint16 v3) {
+	HotspotData *hotspot = Resources::getReference().getHotspot(hotspotId);
+	assert(hotspot);
+	hotspot->actionCtr = value;
+}
+
+// Starts the designated character speaking the given message to a given dest hotspot
+
+void Script::startSpeaking(uint16 characterId, uint16 destHotspot, uint16 messageId) {
+	Hotspot *hotspot = Resources::getReference().getActiveHotspot(characterId);
+	hotspot->converse(destHotspot, messageId, false);
 }
 
 // Disables the given hotspot from being highlighted by the cursor
@@ -248,6 +340,21 @@ void Script::enableHotspot(uint16 hotspotId, uint16 v2, uint16 v3) {
 	hotspot->flags = (hotspot->flags & 0xdf) | 0x80;
 }
 
+// Display a message
+
+void Script::displayMessage2(uint16 messageId, uint16 hotspotId, uint16 v3) {
+	Hotspot *hotspot = Resources::getReference().getActiveHotspot(hotspotId);
+	assert(hotspot);
+	hotspot->showMessage(messageId);
+}
+
+void Script::startOilBurner(uint16 v1, uint16 v2, uint16 v3) {
+	Hotspot *hotspot = Resources::getReference().getActiveHotspot(OIL_BURNER_ID);
+	assert(hotspot);
+	hotspot->setPosition(152, hotspot->y());
+	hotspot->setTickProc(STANDARD_ANIM_TICK_PROC);
+}
+
 // Transforms the player
 
 void Script::transformPlayer(uint16 v1, uint16 v2, uint16 v3) {
@@ -260,13 +367,13 @@ void Script::transformPlayer(uint16 v1, uint16 v2, uint16 v3) {
 	hotspot->startY = player->startY - 10;
 	
 	Hotspot *activeHotspot = res.addHotspot(TRANSFORM_ID);
-	activeHotspot->setFrameNumber(0);
-	activeHotspot->setScript(0x630);
+	activeHotspot->setActionCtr(0);
+	activeHotspot->setHotspotScript(0x630);
 }
 
-// Marks the door in room 14 for closing
+// Marks the jail door in room 14 for closing
 
-void Script::room14DoorClose(uint16 v1, uint16 v2, uint16 v3) {
+void Script::jailClose(uint16 v1, uint16 v2, uint16 v3) {
 	RoomExitJoinData *joinRec = Resources::getReference().getExitJoin(0x2719);
 	joinRec->blocked = 1;
 }
@@ -298,10 +405,21 @@ void Script::doorOpen(uint16 hotspotId, uint16 v2, uint16 v3) {
 	joinRec->blocked = 0;
 }
 
+// Makes the specified NPC wait for the player to walk to them
+
+void Script::npcWait(uint16 hotspotId, uint16 v2, uint16 v3) {
+       Hotspot *hotspot = Resources::getReference().getActiveHotspot(hotspotId);
+       assert(hotspot);
+       hotspot->setCharacterMode(CHARMODE_WAIT_FOR_INTERACT);
+       hotspot->setDelayCtr(130);
+}
+
 // Lookup the given message Id for the specified character and display in a dialog
 
 void Script::displayMessage(uint16 messageId, uint16 characterId, uint16 unknownVal) {
-	Dialog::showMessage(messageId, characterId);
+	Hotspot *hotspot = Resources::getReference().getActiveHotspot(characterId);
+	if (hotspot != NULL)
+		hotspot->showMessage(messageId);
 }
 
 // Creates a new dispatch action with the given support data entry
@@ -312,7 +430,9 @@ void Script::setNewSupportData(uint16 hotspotId, uint16 index, uint16 v3) {
 	CharacterScheduleEntry *entry = res.charSchedules().getEntry(dataId);
 
 	Hotspot *h = res.getActiveHotspot(hotspotId);
+	h->setBlockedFlag(false);
 	h->currentActions().addFront(DISPATCH_ACTION, entry, h->roomNumber());
+	h->setActionCtr(0);
 }
 
 // Replaces the existing current action with a new dispatch data entry
@@ -322,7 +442,10 @@ void Script::setSupportData(uint16 hotspotId, uint16 index, uint16 v3) {
 	uint16 dataId = res.getCharOffset(index);
 
 	CharacterScheduleEntry *entry = res.charSchedules().getEntry(dataId);
+	assert(entry != NULL);
 	Hotspot *h = res.getActiveHotspot(hotspotId);
+	assert(h != NULL);
+	assert(!h->currentActions().isEmpty());
 
 	h->currentActions().pop();
 	h->currentActions().addFront(DISPATCH_ACTION, entry, h->roomNumber());
@@ -343,6 +466,34 @@ void Script::decreaseNumGroats(uint16 characterId, uint16 numGroats, uint16 v3)
 	fields.numGroats() -= numGroats;
 }
 
+// Sets a character moving to the player's room (if they're not already there)
+
+void Script::moveCharacterToPlayer(uint16 characterId, uint16 v2, uint16 v3) {
+	Resources &res = Resources::getReference();
+	Hotspot *playerHotspot = res.getActiveHotspot(PLAYER_ID);
+	Hotspot *charHotspot = res.getActiveHotspot(characterId);
+	assert(charHotspot); 
+
+	// If character in same room as player, then no need to do anything
+	if (!charHotspot->currentActions().isEmpty() &&
+		(charHotspot->currentActions().top().roomNumber() == playerHotspot->roomNumber()))
+		return;
+
+	uint16 destRoom = playerHotspot->roomNumber();
+	RoomTranslationRecord *rec; 
+	for (rec = &roomTranslations[0]; rec->srcRoom != 0; ++rec) {
+		if (rec->srcRoom == destRoom) {
+			destRoom = rec->destRoom;
+			break;
+		}
+	}
+
+	if (charHotspot->currentActions().isEmpty())
+		charHotspot->currentActions().addFront(DISPATCH_ACTION,  destRoom);
+	else
+		charHotspot->currentActions().top().setRoomNumber(destRoom);
+}
+
 // Sets the tick handler for the village Skorl to an alternate handler
 
 void Script::setVillageSkorlTickProc(uint16 v1, uint16 v2, uint16 v3) {
@@ -350,6 +501,29 @@ void Script::setVillageSkorlTickProc(uint16 v1, uint16 v2, uint16 v3) {
 	hotspot->tickProcOffset = 0x7efa;
 }
 
+// Free Goewin from captivity
+
+void Script::freeGoewin(uint16 v1, uint16 v2, uint16 v3) {
+	HotspotData *goewin = Resources::getReference().getHotspot(GOEWIN_ID);
+	goewin->actions = 0x820C00;   // Enable Talk To, Give, Bribe, and Ask for
+	goewin->actionCtr = 1;
+}
+
+// Barman serving the player
+
+void Script::barmanServe(uint16 v1, uint16 v2, uint16 v3) {
+	Resources &res = Resources::getReference();
+	Hotspot *player = res.getActiveHotspot(PLAYER_ID);
+	BarEntry &barEntry = res.barmanLists().getDetails(player->roomNumber());
+	
+	for (int index = 0; index < NUM_SERVE_CUSTOMERS; ++index) {
+		if (barEntry.customers[index].hotspotId == PLAYER_ID) {
+			barEntry.customers[index].serveFlags |= 5;
+			break;
+		}
+	}
+}
+
 // Stores the current number of groats in the general field
 
 void Script::getNumGroats(uint16 v1, uint16 v2, uint16 v3) {
@@ -357,6 +531,23 @@ void Script::getNumGroats(uint16 v1, uint16 v2, uint16 v3) {
 	fields.setField(GENERAL, fields.numGroats());
 }
 
+// Enables the talk action on the two gargoyles
+
+void Script::enableGargoylesTalk(uint16 v1, uint16 v2, uint16 v3) {
+	Resources &res = Resources::getReference();
+	HotspotData *g1 = res.getHotspot(0x42C);
+	HotspotData *g2 = res.getHotspot(0x42D);
+	g1->actions = 1 << (TALK_TO - 1);
+	g2->actions = 1 << (TALK_TO - 1);
+}
+
+// Flags the player as dead
+
+void Script::killPlayer(uint16 v1, uint16 v2, uint16 v3) {
+	Game &game = Game::getReference();
+	game.setState(GS_RESTORE_RESTART);
+}
+
 // Loads the specified animation, completely bypassing the standard process
 // of checking for a load proc/sequence
 
@@ -371,6 +562,29 @@ void Script::addActions(uint16 hotspotId, uint16 actions, uint16 v3) {
 	hotspot->actions |= actions;
 }
 
+// Generates a random number and stores it in the general field
+
+void Script::randomToGeneral(uint16 maxVal, uint16 minVal, uint16 v3) {
+	Common::RandomSource rnd;
+	uint16 v = minVal + rnd.getRandomNumber(maxVal - minVal);
+	Resources::getReference().fieldList().setField(GENERAL, v);
+}
+
+// Checks the status of the cell door, and starts music depending on it's state
+
+void Script::checkCellDoor(uint16 v1, uint16 v2, uint16 v3) {
+	// In the original game, this method checks to see if the cell door
+	// is currently open, if it is, starts a music sequence. 
+	// TODO: Implement starting music if cell door is open
+}
+
+// Has something to do with music handling 
+
+void Script::method66(uint16 hotspotId, uint16 actions, uint16 v3) {
+	// For now, simply set the general value field so that the Skorl schedule
+	// will work properly
+	Resources::getReference().fieldList().setField(GENERAL, 0);
+}
 
 typedef void(*SequenceMethodPtr)(uint16, uint16, uint16);
 
@@ -382,44 +596,98 @@ struct SequenceMethodRecord {
 SequenceMethodRecord scriptMethods[] = {
 	{0, Script::activateHotspot}, 
 	{1, Script::setHotspotScript},
+	{3, Script::setHotspotFlagMask},
 	{4, Script::clearSequenceDelayList},
 	{5, Script::deactivateHotspotSet},
 	{6, Script::deactivateHotspot},
+	{7, Script::resetPathfinder},
 	{8, Script::addDelayedSequence},
 	{10, Script::characterInRoom},
-	{11, Script::setActionsOffset},
+	{11, Script::setDesc},
 	{12, Script::setHotspotName},
 	{13, Script::playSound},
 	{16, Script::displayDialog},
 	{18, Script::remoteRoomViewSetup},
+	{19, Script::startSpeakingToNoone},
 	{20, Script::checkCellDoor},
 	{21, Script::playMusic},
 	{22, Script::getDoorBlocked},
 	{23, Script::isSkorlInCell},
+	{24, Script::ratpouchPushBricks},
+	{25, Script::characterChangeRoom},
+	{26, Script::pauseRatpouch},
 	{27, Script::setBlockingHotspotScript},
 	{28, Script::decrInventoryItems},
-	{30, Script::setFrameNumber},
+	{29, Script::setTalking},
+	{30, Script::setActionCtr},
+	{31, Script::startSpeaking},
 	{32, Script::disableHotspot},
 	{33, Script::cutSack},
 	{34, Script::increaseNumGroats},
 	{35, Script::enableHotspot},
-	{37, Script::transformPlayer},
-	{39, Script::room14DoorClose},
+	{36, Script::displayMessage2},
+	{37, Script::startOilBurner},
+	{38, Script::transformPlayer},
+	{39, Script::jailClose},
 	{40, Script::checkDroppedDesc},
 	{42, Script::doorClose},
 	{44, Script::doorOpen},
+	{45, Script::npcWait},
 	{47, Script::displayMessage},
 	{48, Script::setNewSupportData},
 	{49, Script::setSupportData},
 	{50, Script::givePlayerItem},
 	{51, Script::decreaseNumGroats},
+	{53, Script::moveCharacterToPlayer},
 	{54, Script::setVillageSkorlTickProc},
+	{55, Script::freeGoewin},
+	{56, Script::barmanServe},
 	{57, Script::getNumGroats},
+	{59, Script::enableGargoylesTalk},
+	{61, Script::killPlayer},
 	{62, Script::animationLoad},
 	{63, Script::addActions},
+	{64, Script::randomToGeneral},
 	{65, Script::checkCellDoor},
+	{66, Script::method66},
 	{0xff, NULL}};
+
+const char *scriptOpcodes[] = {
+	"ABORT", "ADD", "SUBTRACT", "MULTIPLY", "DIVIDE", "EQUALS", "NOT_EQUALS",
+		"LT", "GT", "LTE", "GTE", "AND", "OR", "LOGICAL_AND", "LOGICAL_OR",
+		"GET_FIELD", "SET_FIELD", "PUSH", "SUBROUTINE", "EXEC", "END",
+		"COND_JUMP", "JUMP", "ABORT2", "ABORT3", "RANDOM"
+};
+
+const char *scriptMethodNames[67] = {
+	"ACTIVATE HOTSPOT", "SET HOTSPOT SCRIPT", NULL, "SET HOTSPOT FLAG MASK", 
+	"CLEAR SEQUENCE DELAY LIST", "DEACTIVATE HOTSPOT SET", "DEACTIVATE HOTSPOT", 
+	"RESET PATHFINDER",	"ADD DELAYED SCRIPT", NULL,
+
+	"IS CHARACTER IN ROOM", "SET HOTSPOT DESC", "SET HOTSPOT NAME",
+	"PLAY SOUND", NULL, NULL, "DISPLAY DIALOG", NULL, "REMOTE ROOM VIEW SETUP",
+	"SET CHAR SPEAKING TO ITSELF",
+
+	"CHECK CELL DOOR", "PLAY MUSIC", "IS DOOR BLOCKED", "IS SKORL IN CELL",
+	"PUSH BRICKS", "CHARACTER CHANGE ROOM", "PAUSE RATPOUCH", "SET BLOCKING HOTSPOT SCRIPT", 
+	"DECREMENT # INVENTORY ITEMS", "SET TALKING",
+
+	"SET ACTION CTR", "START SPEAKING", "DISABLE HOTSPOT", "CUT SACK",
+	"INCREASE # GROATS", "ENABLE HOTSPOT", "DISPLAY MESSAGE 2", "START OIL BURNER"
+	"TRANSFORM PLAYER", "JAIL CLOSE",
+
+	"CHECK DROPPED DESC", NULL, "CLOSE DOOR", NULL, "OPEN DOOR", "NPC WAIT", NULL,
+	"DISPLAY MESSAGE", "SET NEW ACTION SUPPORT DATA", "SET ACTION SUPPORT DATA",
 	
+	"GIVE PLAYER ITEM", "DECREASE # GROATS", NULL, NULL,
+	"SET VILLAGE SKORL TICK PROC", "FREE GOEWIN", "BARMAN SERVE", "GET # GROATS", 
+	NULL, "ENABLE GARGOYLE TALK",
+
+	NULL, "KILL PLAYER", "ANIMATION LOAD", "ADD ACTIONS", "RANDOM TO GENERAL", 
+	"CHECK CELL DOOR", "METHOD 66"
+};
+
+
 /*------------------------------------------------------------------------*/
 /*-  Script Execution                                                    -*/
 /*-                                                                      -*/
@@ -433,32 +701,82 @@ uint16 Script::execute(uint16 startOffset) {
 	Common::Stack<uint16> stack;
 	Common::Stack<uint16> methodStack;
 	byte opcode;
+	bool hasParam;
 	uint16 param, v1, v2;
 	uint16 param1, param2, param3;
 	uint16 fieldNum;
 	uint32 tempVal;
 	SequenceMethodPtr ptr;
 	SequenceMethodRecord *rec;
+	char debugInfo[MAX_DESC_SIZE];
 
 	uint16 offset = startOffset;
 	bool breakFlag = false;
+
 	param = 0;
 	fields.setField(SEQUENCE_RESULT, 0);
 
+	debugC(ERROR_BASIC, kLureDebugScripts, "Executing script %xh", startOffset);
+	strcpy(debugInfo, "");
+
 	while (!breakFlag) {
 		if (offset >= scriptData->size()) 
 			error("Script failure in script %d - invalid offset %d", startOffset, offset);
 
+		if (gDebugLevel >= ERROR_DETAILED)
+			sprintf(debugInfo, "%xh - ", offset);
+
+		// Get opcode byte and separate into opcode and has parameter bit flag
 		opcode = scripts[offset++];
-		if ((opcode & 1) != 0) {
+		hasParam = (opcode & 1) != 0;
+		opcode >>= 1;
+
+		if (gDebugLevel >= ERROR_DETAILED)
+			strcat(debugInfo, (opcode > S_OPCODE_RANDOM) ? "INVALID" :
+				scriptOpcodes[opcode]);
+
+		if (hasParam) {
 			// Flag to read next two bytes as active parameter
 			if (offset >= scriptData->size()-2) 
 				error("Script failure in script %d - invalid offset %d", startOffset, offset);
 
 			param = READ_LE_UINT16(scripts + offset);
 			offset += 2;
+
+			if (gDebugLevel >= ERROR_DETAILED)
+				sprintf(debugInfo + strlen(debugInfo), " [%d]", 
+					((opcode == S_OPCODE_GET_FIELD) || (opcode == S_OPCODE_SET_FIELD)) ?
+					param >> 1 : param);
+		}
+
+		if (gDebugLevel >= ERROR_DETAILED) {
+			switch (opcode) {
+			case S_OPCODE_ADD:
+			case S_OPCODE_SUBTRACT:
+			case S_OPCODE_MULTIPLY:
+			case S_OPCODE_DIVIDE:
+			case S_OPCODE_EQUALS:
+			case S_OPCODE_NOT_EQUALS:
+			case S_OPCODE_LT:
+			case S_OPCODE_GT:
+			case S_OPCODE_LTE:
+			case S_OPCODE_GTE:
+			case S_OPCODE_AND:
+			case S_OPCODE_OR:
+			case S_OPCODE_LOGICAL_AND:
+			case S_OPCODE_LOGICAL_OR:
+				sprintf(debugInfo + strlen(debugInfo), 
+					" %d, %d", stack[stack.size() - 1], stack[stack.size() - 2]);
+				break;
+			
+			case S_OPCODE_SET_FIELD:
+				sprintf(debugInfo + strlen(debugInfo), " <= ST (%d)", stack[stack.size() - 1]);
+				break;
+
+			default:
+				break;
+			}
 		}
-		opcode >>= 1;	// Discard param bit from opcode byte
 
 		switch (opcode) {
 		case S_OPCODE_ABORT:
@@ -490,28 +808,36 @@ uint16 Script::execute(uint16 startOffset) {
 			param = v2 % v1;      // remainder
 			break;
 
-		case S_OPCODE_NOT_EQUALS:
-			stack.push((stack.pop() != stack.pop()) ? 0 : 1);
-			break;
-
 		case S_OPCODE_EQUALS:
-			stack.push((stack.pop() == stack.pop()) ? 0 : 1);
+			stack.push((stack.pop() == stack.pop()) ? 1 : 0);
 			break;
 
-		case S_OPCODE_GT:
-			stack.push((stack.pop() > stack.pop()) ? 1 : 0);
+		case S_OPCODE_NOT_EQUALS:
+			stack.push((stack.pop() != stack.pop()) ? 1 : 0);
 			break;
 
 		case S_OPCODE_LT:
-			stack.push((stack.pop() < stack.pop()) ? 1 : 0);
+			v1 = stack.pop();
+			v2 = stack.pop();
+			stack.push(v2 < v1 ? 1 : 0);
 			break;
 
-		case S_OPCODE_LT2:
-			stack.push((stack.pop() < stack.pop()) ? 1 : 0);
+		case S_OPCODE_GT:
+			v1 = stack.pop();
+			v2 = stack.pop();
+			stack.push(v2 > v1 ? 1 : 0);
 			break;
 
-		case S_OPCODE_GT2:
-			stack.push((stack.pop() > stack.pop()) ? 1 : 0);
+		case S_OPCODE_LTE:
+			v1 = stack.pop();
+			v2 = stack.pop();
+			stack.push(v2 <= v1 ? 1 : 0);
+			break;
+
+		case S_OPCODE_GTE:
+			v1 = stack.pop();
+			v2 = stack.pop();
+			stack.push(v2 >= v1 ? 1 : 0);
 			break;
 
 		case S_OPCODE_AND:
@@ -523,22 +849,24 @@ uint16 Script::execute(uint16 startOffset) {
 			break;
 
 		case S_OPCODE_LOGICAL_AND:
-			stack.push(((stack.pop() != 0) && (stack.pop() != 0)) ? 1 : 0);
+			v1 = stack.pop();
+			v2 = stack.pop();
+			stack.push(((v1 != 0) && (v2 != 0)) ? 1 : 0);
 			break;
 
 		case S_OPCODE_LOGICAL_OR:
-			stack.push(((stack.pop() != 0) || (stack.pop() != 0)) ? 1 : 0);
+			v1 = stack.pop();
+			v2 = stack.pop();
+			stack.push(((v1 != 0) || (v2 != 0)) ? 1 : 0);
 			break;
 
 		case S_OPCODE_GET_FIELD:
-			// Opcode not yet fully implemented
 			fieldNum = param >> 1;
 			v1 = fields.getField(fieldNum);
 			stack.push(v1);
 			break;
 
 		case S_OPCODE_SET_FIELD:
-			// Opcode not yet fully implemented
 			fieldNum = param >> 1;
 			v1 = stack.pop();
 			fields.setField(fieldNum, v1);
@@ -554,15 +882,39 @@ uint16 Script::execute(uint16 startOffset) {
 			break;
 
 		case S_OPCODE_EXEC:
+			// Search for the method in the method list
+			rec = &scriptMethods[0];
+			while ((rec->methodIndex != 0xff) && (rec->methodIndex != param))
+				++rec;
+
+			if (gDebugLevel >= ERROR_DETAILED) {
+				// Set up the debug string for the method call
+				if (rec->methodIndex == 0xff) strcat(debugInfo, " INVALID INDEX");
+				else if (scriptMethodNames[param] == NULL) strcat(debugInfo, " UNKNOWN METHOD");
+				else {
+					strcat(debugInfo, " ");
+					strcat(debugInfo, scriptMethodNames[param]);
+				}
+
+				// Any params
+				if (stack.size() >= 3)
+					sprintf(debugInfo + strlen(debugInfo), " (%d,%d,%d)",
+						stack[stack.size()-1], stack[stack.size()-2], stack[stack.size()-3]);
+				if (stack.size() == 2)
+					sprintf(debugInfo + strlen(debugInfo), " (%d,%d)",
+						stack[stack.size()-1], stack[stack.size()-2]);
+				else if (stack.size() == 1)
+					sprintf(debugInfo + strlen(debugInfo), " (%d)", stack[stack.size()-1]);
+				strcat(debugInfo, ")");
+				
+				debugC(ERROR_DETAILED, kLureDebugScripts, debugInfo);
+			}
+
 			param1 = 0; param2 = 0; param3 = 0;
 			if (!stack.empty()) param1 = stack.pop();
 			if (!stack.empty()) param2 = stack.pop();
 			if (!stack.empty()) param3 = stack.pop();
 
-			rec = &scriptMethods[0];
-			while ((rec->methodIndex != 0xff) && (rec->methodIndex != param))
-				++rec;
-
 			if (rec->methodIndex == 0xff) 
 				warning("Undefined script method %d", param);
 			else {
@@ -596,9 +948,42 @@ uint16 Script::execute(uint16 startOffset) {
 			error("Unknown script opcode %d", opcode);
 			break;
 		}
+
+		if ((gDebugLevel >= ERROR_DETAILED) && (opcode != S_OPCODE_EXEC)) {
+			switch (opcode) {
+			case S_OPCODE_ADD:
+			case S_OPCODE_SUBTRACT:
+			case S_OPCODE_MULTIPLY:
+			case S_OPCODE_DIVIDE:
+			case S_OPCODE_NOT_EQUALS:
+			case S_OPCODE_EQUALS:
+			case S_OPCODE_GT:
+			case S_OPCODE_LT:
+			case S_OPCODE_LTE:
+			case S_OPCODE_GTE:
+			case S_OPCODE_AND:
+			case S_OPCODE_OR:
+			case S_OPCODE_LOGICAL_AND:
+			case S_OPCODE_LOGICAL_OR:
+			case S_OPCODE_GET_FIELD:
+				sprintf(debugInfo + strlen(debugInfo), " => ST (%d)", stack[stack.size()-1]);
+				break;
+
+			case S_OPCODE_PUSH:
+				strcat(debugInfo, " => ST");
+				break;
+
+			default:
+				break;
+			}
+
+			debugC(ERROR_DETAILED, kLureDebugScripts, debugInfo);
+		}
 	}
 
-	return fields.getField(SEQUENCE_RESULT);
+	uint16 result = fields.getField(SEQUENCE_RESULT);
+	debugC(ERROR_DETAILED, kLureDebugScripts, "Script result = %d", result); 
+	return result;
 }
 
 /*------------------------------------------------------------------------*/
@@ -618,74 +1003,100 @@ bool HotspotScript::execute(Hotspot *h)
 {
 	Resources &r = Resources::getReference();
 	MemoryBlock *scriptData = r.hotspotScriptData();
-	uint16 offset = h->script();
+	uint16 offset = h->hotspotScript();
 	int16 opcode = 0;
 	int16 param1, param2;
+	uint32 actions;
 	bool breakFlag = false;
 
+	debugC(ERROR_BASIC, kLureDebugScripts, "Executing hotspot %xh script pos=%xh", 
+		h->hotspotId(), offset);
+
 	while (!breakFlag) {
 		opcode = nextVal(scriptData, offset);
 
 		switch (opcode) {
-		case S2_OPCODE_TIMEOUT:
+		case S2_OPCODE_FRAME_CTR:
 			param1 = nextVal(scriptData, offset);
+			debugC(ERROR_DETAILED, kLureDebugScripts, "SET FRAME_CTR = %d", param1);
+
 			h->setTickCtr(param1);
-			h->setScript(offset);
+			h->setHotspotScript(offset);
 			breakFlag = true;
 			break;
 
 		case S2_OPCODE_POSITION:
 			param1 = nextVal(scriptData, offset);
 			param2 = nextVal(scriptData, offset);
+			debugC(ERROR_DETAILED, kLureDebugScripts, "SET POSITION = (%d,%d)", 
+				param1 - 0x80, param2 - 0x80);
+
 			h->setPosition(param1 - 0x80, param2 - 0x80);
 			break;
 			
 		case S2_OPCODE_CHANGE_POS:
 			param1 = nextVal(scriptData, offset);
 			param2 = nextVal(scriptData, offset);
+			debugC(ERROR_DETAILED, kLureDebugScripts, "CHANGE POSITION BY = (%d,%d)", 
+				param1, param2);
+
 			h->setPosition(h->x() + param1, h->y() + param2);
 			break;
 
 		case S2_OPCODE_UNLOAD:
+			debugC(ERROR_DETAILED, kLureDebugScripts, "UNLOAD HOTSPOT");
 			breakFlag = true;
 			break;
 
 		case S2_OPCODE_DIMENSIONS:
 			param1 = nextVal(scriptData, offset) << 4;
 			param2 = nextVal(scriptData, offset);
+			debugC(ERROR_DETAILED, kLureDebugScripts, "SET SIZE = (%d,%d)",
+				param1, param2);
+
 			h->setSize((uint16) param1, (uint16) param2);
 			break;
 
 		case S2_OPCODE_JUMP:
 			offset = (uint16) nextVal(scriptData, offset);
+			debugC(ERROR_DETAILED, kLureDebugScripts, "JUMP OFFSET = %xh", offset);
 			break;
 
 		case S2_OPCODE_ANIMATION:
 			param1 = nextVal(scriptData, offset);
+			debugC(ERROR_DETAILED, kLureDebugScripts, "SET ANIMATION ID = %xh", param1);
+
 			h->setAnimation(param1);
 			break;
 
-		case S2_OPCODE_UNKNOWN_247:
+		case S2_OPCODE_PLAY_SOUND:
 			param1 = nextVal(scriptData, offset);
 			param2 = nextVal(scriptData, offset);
+			debugC(ERROR_DETAILED, kLureDebugScripts, "PLAY_SOUND(%d,%d)", param1, param2);
 //			warning("UNKNOWN_247 stub called");
 			break;
 
-		case S2_OPCODE_UNKNOWN_258:
+		case S2_OPCODE_STOP_SOUND:
 			param1 = nextVal(scriptData, offset);
+			debugC(ERROR_DETAILED, kLureDebugScripts, "STOP_SOUND()");
 //			warning("UNKNOWN_258 stub called");
 			break;
 
 		case S2_OPCODE_ACTIONS:
-			param1 = nextVal(scriptData, offset) << 4;
+			param1 = nextVal(scriptData, offset);
 			param2 = nextVal(scriptData, offset);
-			h->setActions((uint32) param1 | ((uint32) param2 << 16));
+			actions = (uint32) param1 | ((uint32) param2 << 16);
+
+			debugC(ERROR_DETAILED, kLureDebugScripts, "SET ACTIONS = %xh", actions);
+			h->setActions(actions);
 			break;
 
 		default:
 			// Set the animation frame number
+			debugC(ERROR_DETAILED, kLureDebugScripts, "SET FRAME NUMBER = %d", opcode);
+
 			h->setFrameNumber(opcode);
-			h->setScript(offset);
+			h->setHotspotScript(offset);
 			breakFlag = true;
 			break;
 		}
