@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2005-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,13 +18,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/lure/scripts.h $
- * $Id: scripts.h 22576 2006-05-23 12:24:50Z dreammaster $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/lure/scripts.h $
+ * $Id: scripts.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
-#ifndef __lure_scripts_h__
-#define __lure_scripts_h__
+#ifndef LURE_SCRIPTS_H
+#define LURE_SCRIPTS_H
 
 #include "lure/luredefs.h"
 #include "lure/memory.h"
@@ -35,12 +38,12 @@ namespace Lure {
 #define S_OPCODE_SUBTRACT 2
 #define S_OPCODE_MULTIPLY 3
 #define S_OPCODE_DIVIDE 4
-#define S_OPCODE_NOT_EQUALS 5
-#define S_OPCODE_EQUALS 6
-#define S_OPCODE_GT 7
-#define S_OPCODE_LT 8
-#define S_OPCODE_LT2 9
-#define S_OPCODE_GT2 10
+#define S_OPCODE_EQUALS 5
+#define S_OPCODE_NOT_EQUALS 6
+#define S_OPCODE_LT 7
+#define S_OPCODE_GT 8
+#define S_OPCODE_LTE 9
+#define S_OPCODE_GTE 10
 #define S_OPCODE_AND 11
 #define S_OPCODE_OR 12
 #define S_OPCODE_LOGICAL_AND 13
@@ -57,15 +60,15 @@ namespace Lure {
 #define S_OPCODE_ABORT3 24
 #define S_OPCODE_RANDOM 25
 
-#define S2_OPCODE_TIMEOUT -1
+#define S2_OPCODE_FRAME_CTR -1
 #define S2_OPCODE_POSITION -2
 #define S2_OPCODE_CHANGE_POS -3
 #define S2_OPCODE_UNLOAD -4
 #define S2_OPCODE_DIMENSIONS -5
 #define S2_OPCODE_JUMP -6
 #define S2_OPCODE_ANIMATION -7
-#define S2_OPCODE_UNKNOWN_247 -8
-#define S2_OPCODE_UNKNOWN_258 -9
+#define S2_OPCODE_PLAY_SOUND -8
+#define S2_OPCODE_STOP_SOUND -9
 #define S2_OPCODE_ACTIONS -10
 
 
@@ -76,42 +79,60 @@ public:
 
 	static void activateHotspot(uint16 hotspotId, uint16 v2, uint16 v3);
 	static void setHotspotScript(uint16 hotspotId, uint16 scriptIndex, uint16 v3);
+	static void method2(uint16 v1, uint16 v2, uint16 v3);
+	static void setHotspotFlagMask(uint16 maskVal, uint16 v2, uint16 v3);
 	static void clearSequenceDelayList(uint16 v1, uint16 scriptIndex, uint16 v3);
 	static void deactivateHotspotSet(uint16 listIndex, uint16 v2, uint16 v3);
-	static void method2(uint16 v1, uint16 v2, uint16 v3);
 	static void deactivateHotspot(uint16 hotspotId, uint16 v2, uint16 v3);
-	static void setActionsOffset(uint16 hotspotId, uint16 offset, uint16 v3);
-	static void addDelayedSequence(uint16 seqOffset, uint16 delay, uint16 v3);
+	static void resetPathfinder(uint16 v1, uint16 v2, uint16 v3);
+	static void addDelayedSequence(uint16 seqOffset, uint16 delay, uint16 canClear);
 	static void characterInRoom(uint16 characterId, uint16 roomNumber, uint16 v3);
+	static void setDesc(uint16 hotspotId, uint16 descId, uint16 v3);
 	static void setHotspotName(uint16 hotspotId, uint16 nameId, uint16 v3);
 	static void playSound(uint16 v1, uint16 v2, uint16 v3);
 	static void displayDialog(uint16 stringId, uint16 v2, uint16 v3);
 	static void remoteRoomViewSetup(uint16 v1, uint16 v2, uint16 v3);
-	static void checkCellDoor(uint16 v1, uint16 v2, uint16 v3);
+	static void startSpeakingToNoone(uint16 characterId, uint16 stringId, uint16 v3);
 	static void playMusic(uint16 musicNum, uint16 v2, uint16 v3);
 	static void getDoorBlocked(uint16 hotspotId, uint16 v2, uint16 v3);
 	static void isSkorlInCell(uint16 v1, uint16 v2, uint16 v3);
+	static void ratpouchPushBricks(uint16 v1, uint16 v2, uint16 v3);
+	static void characterChangeRoom(uint16 y, uint16 x, uint16 roomNumber);
+	static void pauseRatpouch(uint16 v1, uint16 v2, uint16 v3);
 	static void setBlockingHotspotScript(uint16 charId, uint16 scriptIndex, uint16 v3);
 	static void decrInventoryItems(uint16 v1, uint16 v2, uint16 v3);
-	static void setFrameNumber(uint16 hotspotId, uint16 offset, uint16 v3);
+	static void setTalking(uint16 characterId, uint16 destHotspot, uint16 messageId);
+	static void setActionCtr(uint16 hotspotId, uint16 value, uint16 v3);
+	static void startSpeaking(uint16 characterId, uint16 destHotspot, uint16 messageId);
 	static void disableHotspot(uint16 hotspotId, uint16 v2, uint16 v3);
 	static void cutSack(uint16 hotspotId, uint16 v2, uint16 v3);
 	static void increaseNumGroats(uint16 characterId, uint16 numGroats, uint16 v3);
 	static void enableHotspot(uint16 hotspotId, uint16 v2, uint16 v3);
+	static void displayMessage2(uint16 messageId, uint16 hotspotId, uint16 v3);
+	static void startOilBurner(uint16 v1, uint16 v2, uint16 v3);
 	static void transformPlayer(uint16 v1, uint16 v2, uint16 v3);
-	static void room14DoorClose(uint16 v1, uint16 v2, uint16 v3);
+	static void jailClose(uint16 v1, uint16 v2, uint16 v3);
 	static void checkDroppedDesc(uint16 hotspotId, uint16 v2, uint16 v3);
 	static void doorClose(uint16 hotspotId, uint16 v2, uint16 v3);
 	static void displayMessage(uint16 messageId, uint16 characterId, uint16 unknownVal);
 	static void doorOpen(uint16 hotspotId, uint16 v2, uint16 v3);
+	static void npcWait(uint16 hotspotId, uint16 v2, uint16 v3);
 	static void setNewSupportData(uint16 hotspotId, uint16 index, uint16 v3);
 	static void setSupportData(uint16 hotspotId, uint16 index, uint16 v3);
 	static void givePlayerItem(uint16 hotspotId, uint16 v2, uint16 v3);
 	static void decreaseNumGroats(uint16 characterId, uint16 numGroats, uint16 v3);
+	static void moveCharacterToPlayer(uint16 characterId, uint16 v2, uint16 v3);
 	static void setVillageSkorlTickProc(uint16 v1, uint16 v2, uint16 v3);
+	static void freeGoewin(uint16 v1, uint16 v2, uint16 v3);
+	static void barmanServe(uint16 v1, uint16 v2, uint16 v3);
 	static void getNumGroats(uint16 v1, uint16 v2, uint16 v3);
+	static void enableGargoylesTalk(uint16 v1, uint16 v2, uint16 v3);
+	static void killPlayer(uint16 v1, uint16 v2, uint16 v3);
 	static void animationLoad(uint16 hotspotId, uint16 v2, uint16 v3);
 	static void addActions(uint16 hotspotId, uint16 actions, uint16 v3);
+	static void randomToGeneral(uint16 maxVal, uint16 minVal, uint16 v3);
+	static void checkCellDoor(uint16 v1, uint16 v2, uint16 v3);
+	static void method66(uint16 v1, uint16 v2, uint16 v3);
 };
 
 class HotspotScript {
