@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2005-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,13 +18,14 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/lure/strings.cpp $
- * $Id: strings.cpp 22820 2006-06-01 22:43:19Z anotherguest $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/lure/strings.cpp $
+ * $Id: strings.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
 #include "lure/strings.h"
 #include "lure/disk.h"
+#include "lure/res.h"
 #include "lure/room.h"
 #include "common/endian.h"
 
@@ -179,7 +183,7 @@ byte StringData::readBit() {
 	return result;
 }
 
-void StringData::initPosition(uint16 stringId) {
+bool StringData::initPosition(uint16 stringId) {
 	uint16 roomNumber = Room::getReference().roomNumber();
 	byte *stringTable;
 	
@@ -233,7 +237,7 @@ void StringData::initPosition(uint16 stringId) {
 		if (readBit() == 0) break;
 		_srcPos += 2;
 	}
-	readBit();
+	return readBit() != 0;
 }
 
 // readCharatcer
@@ -261,19 +265,29 @@ char StringData::readCharacter() {
 }
 
 void StringData::getString(uint16 stringId, char *dest, const char *hotspotName, 
-							  const char *actionName) {
+		const char *characterName, int hotspotArticle, int characterArticle) {
+	const char *articles[4] = {"the ", "a ", "an ", ""};
 	char ch;
+	strcpy(dest, "");
 	char *destPos = dest;
-	initPosition(stringId);
+	stringId &= 0x1fff;      // Strip off any article identifier
+	if (stringId == 0) return;
+
+	bool includeArticles = initPosition(stringId);
 
 	ch = readCharacter();
 	while (ch != '\0') {
 		if (ch == '%') {
 			// Copy over hotspot or action 
 			ch = readCharacter();
-			const char *p = (ch == '1') ? hotspotName : actionName;
-			strcpy(destPos, p);
-			destPos += strlen(p);
+			const char *p = (ch == '1') ? hotspotName : characterName;
+			int article = !includeArticles ? 3 : ((ch == 1) ? hotspotArticle : characterArticle);
+
+			if (p != NULL) {
+				strcpy(destPos, articles[article]);
+				strcat(destPos, p);
+				destPos += strlen(destPos);
+			}
 		} else if ((uint8) ch >= 0xa0) {
 			const char *p = getName((uint8) ch - 0xa0);
 			strcpy(destPos, p);
