@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2005-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,13 +18,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/lure/strings.h $
- * $Id: strings.h 20578 2006-02-11 19:25:13Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/lure/strings.h $
+ * $Id: strings.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
-#ifndef __lure_strings_h__
-#define __lure_strings_h__
+#ifndef LURE_STRINGS_H
+#define LURE_STRINGS_H
 
 #include "lure/luredefs.h"
 #include "lure/memory.h"
@@ -35,7 +38,7 @@ public:
 	char _ascii;
 
 	CharacterEntry(uint8 numBits, uint32 sequence, char ascii): _numBits(numBits),
-		_sequence(sequence), _ascii(ascii) {};
+		_sequence(sequence), _ascii(ascii) {}
 };
 
 #define MAX_NUM_CHARS 218
@@ -50,7 +53,7 @@ private:
 	byte _bitMask;
 
 	void add(const char *sequence, char ascii);
-	void initPosition(uint16 stringId);
+	bool initPosition(uint16 stringId);
 	char readCharacter();
 	byte readBit();
 public:
@@ -58,7 +61,11 @@ public:
 	~StringData();
 	static StringData &getReference();
 
-	void getString(uint16 stringId, char *dest, const char *hotspotName, const char *actionName);
+	void getString(uint16 stringId, char *dest, const char *hotspotName, const char *characterName,
+		int hotspotArticle = 3, int characterArticle = 3);
+	void getString(uint16 stringId, char *dest) {
+		getString(stringId, dest, NULL, NULL);
+	}
 	char *getName(uint8 nameIndex);
 };
 
