@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2005-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,16 +18,16 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/lure/surface.cpp $
- * $Id: surface.cpp 22407 2006-05-11 07:59:50Z eriktorbjorn $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/lure/surface.cpp $
+ * $Id: surface.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
 #include "lure/surface.h"
 #include "lure/decode.h"
-#include "lure/system.h"
 #include "lure/events.h"
 #include "lure/screen.h"
+#include "lure/lure.h"
 #include "lure/room.h"
 #include "lure/strings.h"
 #include "common/endian.h"
@@ -175,11 +178,15 @@ void Surface::copyTo(Surface *dest, uint16 x, uint16 y)
 
 void Surface::copyTo(Surface *dest, const Rect &srcBounds, 
 					 uint16 destX, uint16 destY, int transparentColour) {
+	int numBytes = srcBounds.right - srcBounds.left + 1;
+	if (destX + numBytes > dest->width())
+		numBytes = dest->width() - destX;
+	if (numBytes <= 0) return;
+
 	for (uint16 y=0; y<=(srcBounds.bottom-srcBounds.top); ++y) {
 		const uint32 srcPos = (srcBounds.top + y) * _width + srcBounds.left;
 		const uint32 destPos = (destY+y) * dest->width() + destX;
-		
-		uint16 numBytes = srcBounds.right-srcBounds.left+1;
+
 		if (transparentColour == -1) {
 			// No trnnsparent colour, so copy all the bytes of the line
 			dest->data().copyFrom(_data, srcPos, destPos, numBytes);
@@ -187,7 +194,8 @@ void Surface::copyTo(Surface *dest, const Rect &srcBounds,
 			byte *pSrc = _data->data() + srcPos;
 			byte *pDest = dest->data().data() + destPos;
 
-			while (numBytes-- > 0) {
+			int bytesCtr = numBytes;
+			while (bytesCtr-- > 0) {
 				if (*pSrc != (uint8) transparentColour)
 					*pDest = *pSrc;
 				++pSrc;
@@ -267,13 +275,13 @@ void Surface::createDialog(bool blackFlag) {
 }
 
 void Surface::copyToScreen(uint16 x, uint16 y) {
-	OSystem &system = System::getReference();
+	OSystem &system = *g_system;
 	system.copyRectToScreen(_data->data(), _width, x, y, _width, _height);
 	system.updateScreen();
 }
 
 void Surface::centerOnScreen() {
-	OSystem &system = System::getReference();
+	OSystem &system = *g_system;
 
 	system.copyRectToScreen(_data->data(), _width, 
 		(FULL_SCREEN_WIDTH - _width) / 2, (FULL_SCREEN_HEIGHT - _height) / 2,
@@ -323,10 +331,8 @@ void Surface::wordWrap(char *text, uint16 width, char **&lines, uint8 &numLines)
 			// Break word onto next line
 			*(wordStart - 1) = '\0';
 			++numLines;
-			if (newLine)
-				lineWidth = 0;
-			else
-				lineWidth = textWidth(wordStart, (int) (wordEnd - wordStart + 1));
+			lineWidth = 0;
+			wordEnd = wordStart;
 		} else if (newLine) {
 			// Break on newline
 			++numLines;
@@ -347,13 +353,13 @@ void Surface::wordWrap(char *text, uint16 width, char **&lines, uint8 &numLines)
 		lines[ctr] = strchr(lines[ctr-1], 0) + 1;
 }
 
-Surface *Surface::newDialog(uint16 width, uint8 numLines, char **lines, bool varLength, uint8 colour) {
+Surface *Surface::newDialog(uint16 width, uint8 numLines, const char **lines, bool varLength, uint8 colour) {
 	Surface *s = new Surface(width, (DIALOG_EDGE_SIZE + 3) * 2 + 
 		numLines * (FONT_HEIGHT - 1));
 	s->createDialog();
 
 	for (uint8 ctr = 0; ctr < numLines; ++ctr)
-		s->writeString(DIALOG_EDGE_SIZE + 3, DIALOG_EDGE_SIZE + 3 + 
+		s->writeString(DIALOG_EDGE_SIZE + 3, DIALOG_EDGE_SIZE + 2 + 
 			(ctr * (FONT_HEIGHT - 1)), lines[ctr], true, colour, varLength);
 	return s;
 }
@@ -364,9 +370,8 @@ Surface *Surface::newDialog(uint16 width, const char *line, uint8 colour) {
 	uint8 numLines;
 	wordWrap(lineCopy, width - (DIALOG_EDGE_SIZE + 3) * 2, lines, numLines);
 
-
 	// Create the dialog 
-	Surface *result = newDialog(width, numLines, lines, true, colour);
+	Surface *result = newDialog(width, numLines, const_cast<const char **>(lines), true, colour);
 
 	// Deallocate used resources
 	free(lines);
@@ -383,6 +388,84 @@ Surface *Surface::getScreen(uint16 resourceId) {
 	return new Surface(decodedData, FULL_SCREEN_WIDTH, decodedData->size() / FULL_SCREEN_WIDTH);
 }
 
+bool Surface::getString(Common::String &line, int maxSize, bool isNumeric, bool varLength, int16 x, int16 y) {
+	OSystem &system = *g_system;
+	Mouse &mouse = Mouse::getReference();
+	Events &events = Events::getReference();
+	Screen &screen = Screen::getReference();
+	uint8 bgColour = *(screen.screen().data().data() + (y * FULL_SCREEN_WIDTH) + x);
+	String newLine(line);
+	bool abortFlag = false;
+	bool refreshFlag = false;
+
+	mouse.cursorOff();
+
+	// Insert a cursor character at the end of the string
+	newLine.insertChar('_', newLine.size());
+
+	while (!abortFlag) {
+		// Display the string
+		screen.screen().writeString(x, y, newLine, true, DIALOG_TEXT_COLOUR, varLength);
+		screen.update();
+		int stringSize = textWidth(newLine.c_str());
+
+		// Loop until the input string changes
+		refreshFlag = false;
+		while (!refreshFlag && !abortFlag) {
+			abortFlag = events.quitFlag;
+			if (abortFlag) break;
+
+			if (events.pollEvent()) {
+				if (events.type() == Common::EVENT_KEYDOWN) {
+					char ch = events.event().kbd.ascii;
+					uint16 keycode = events.event().kbd.keycode;
+
+					if ((ch == 13) || (keycode == 0x10f)) {
+						// Return character
+						screen.screen().fillRect(
+							Rect(x, y, x + maxSize - 1, y + FONT_HEIGHT), bgColour);
+						screen.update();
+						newLine.deleteLastChar();
+						line = newLine;
+						mouse.cursorOn();
+						return true;
+					}
+					else if (ch == 27) {
+						// Escape character
+						screen.screen().fillRect(
+							Rect(x, y, x + maxSize - 1, y + FONT_HEIGHT), bgColour);
+						screen.update();
+						abortFlag = true;
+					} else if (ch == 8) {
+						// Delete the last character
+						if (newLine.size() == 1) continue;
+
+						screen.screen().fillRect(
+							Rect(x, y, x + maxSize - 1, y + FONT_HEIGHT), bgColour);
+						newLine.deleteChar(newLine.size() - 2);
+						refreshFlag = true;
+
+					} else if ((ch >= ' ') && (stringSize + 8 < maxSize)) {
+						if (((ch >= '0') && (ch <= '9')) || !isNumeric) {
+							screen.screen().fillRect(
+								Rect(x, y, x + maxSize - 1, y + FONT_HEIGHT), bgColour);
+							newLine.insertChar(ch, newLine.size() - 1);
+							refreshFlag = true;
+						}
+					}
+				}
+			}
+
+			system.updateScreen();
+			system.delayMillis(10);
+		}
+	}
+
+	mouse.cursorOn();
+	return false;
+}
+
+
 /*--------------------------------------------------------------------------*/
 
 void Dialog::show(const char *text) {
@@ -402,87 +485,53 @@ void Dialog::show(const char *text) {
 	mouse.cursorOn();
 }
 
-void Dialog::show(uint16 stringId) {
+void Dialog::show(uint16 stringId, const char *hotspotName, const char *characterName) {
 	char buffer[MAX_DESC_SIZE];
-	Resources &res = Resources::getReference();
-	Room &r = Room::getReference();
 	StringData &sl = StringData::getReference();
 
-	const char *actionName = res.getCurrentActionStr();
-	char hotspotName[MAX_HOTSPOT_NAME_SIZE];
-	if (r.hotspotId() == 0) 
-		strcpy(hotspotName, "");
-	else
-		sl.getString(r.hotspot().nameId, hotspotName, NULL, NULL);
-
-	sl.getString(stringId, buffer, hotspotName, actionName);
+	sl.getString(stringId, buffer, hotspotName, characterName);
 	show(buffer);
 }
 
-void Dialog::showMessage(uint16 messageId, uint16 characterId) {
-	MemoryBlock *data = Resources::getReference().messagesData();
-	uint16 *v = (uint16 *) data->data();
-	uint16 v2, idVal;
-	messageId &= 0x7fff;
-
-	// Skip through header to find table for given character
-	while (READ_LE_UINT16(v) != characterId) v += 2;
-
-	// Scan through secondary list
-	++v;
-	v = (uint16 *) (data->data() + READ_LE_UINT16(v));
-	v2 = 0;
-	while ((idVal = READ_LE_UINT16(v)) != 0xffff) {
-		++v;
-		if (READ_LE_UINT16(v) == messageId) break;
-		++v;
-	}
-
-	// default response if a specific response not found
-	if (idVal == 0xffff) idVal = 0x8c4; 
-	
-	if (idVal == 0x76) {
-		/* 
-		call    sub_154                 ; (64E7)
-		mov     ax,word ptr ds:[5813h]  ; (273F:5813=1BA3h)
-		mov     [bx+ANIM_SEGMENT],ax
-		mov     ax,word ptr ds:[5817h]  ; (273F:5817=0ED8Eh)
-		mov     [bx+ANIM_FRAME],ax
-		retn
-*/
-	} else if (idVal == 0x120) {
-		/* 
-		call    sub_154                 ; (64E7)
-		mov     ax,word ptr ds:[5813h]  ; (273F:5813=1BA3h)
-		mov     [bx+ANIM_SEGMENT],ax
-		mov     ax,word ptr ds:[5817h]  ; (273F:5817=0ED8Eh)
-		shl     ax,1
-		mov     [bx+ANIM_FRAME],ax
-*/
-	} else if (idVal >= 0x8000) {
-		// Handle string display
-		idVal &= 0x7fff;
-		Dialog::show(idVal);
-		
-	} else if (idVal != 0) {
-		// Handle message as a talking dialog
-		// TODO: show talk dialog
-		warning("Dialog style for message #%d not yet implemented", idVal);
-	}
+void Dialog::show(uint16 stringId) {
+	show(stringId, NULL, NULL);
 }
 
 /*--------------------------------------------------------------------------*/
 
-TalkDialog::TalkDialog(uint16 characterId, uint16 descId) {
+TalkDialog::TalkDialog(uint16 characterId, uint16 destCharacterId, uint16 activeItemId, uint16 descId) {
+	debugC(ERROR_DETAILED, kLureDebugAnimations, "TalkDialog(chars=%xh/%xh, item=%d, str=%d", 
+		characterId, destCharacterId, activeItemId, descId);
 	StringData &strings = StringData::getReference();
 	Resources &res = Resources::getReference();
-	HotspotData *character = res.getHotspot(characterId);
-	char charName[MAX_DESC_SIZE];
-	strings.getString(character->nameId, charName, NULL, NULL);
-	strings.getString(descId, _desc, NULL, NULL);
+	char srcCharName[MAX_DESC_SIZE];
+	char destCharName[MAX_DESC_SIZE];
+	char itemName[MAX_DESC_SIZE];
+	int characterArticle, hotspotArticle = 3;
+
+	HotspotData *talkingChar = res.getHotspot(characterId);
+	HotspotData *destCharacter = (destCharacterId == 0) ? NULL : 
+		res.getHotspot(destCharacterId);
+	HotspotData *itemHotspot = (activeItemId == 0) ? NULL :
+		res.getHotspot(activeItemId);
+	assert(talkingChar);
+
+	strings.getString(talkingChar->nameId & 0x1fff, srcCharName);
+	characterArticle = talkingChar->nameId >> 14;
+
+	strcpy(destCharName, "");
+	if (destCharacter != NULL)
+		strings.getString(destCharacter->nameId, destCharName);
+	strcpy(itemName, "");
+	if (itemHotspot != NULL) {
+		strings.getString(itemHotspot->nameId & 0x1fff, itemName);
+		hotspotArticle = itemHotspot->nameId >> 14;
+	}
+
+	strings.getString(descId, _desc, itemName, destCharName, hotspotArticle, characterArticle);
 
 	// Apply word wrapping to figure out the needed size of the dialog
-	Surface::wordWrap(_desc, TALK_DIALOG_WIDTH - TALK_DIALOG_EDGE_SIZE * 2 - 2,
+	Surface::wordWrap(_desc, TALK_DIALOG_WIDTH - (TALK_DIALOG_EDGE_SIZE + 3) * 2,
 		_lines, _numLines);
 
 	_surface = new Surface(TALK_DIALOG_WIDTH, 
@@ -539,9 +588,9 @@ TalkDialog::TalkDialog(uint16 characterId, uint16 descId) {
 	}
 
 	// Write out the character name
-	uint16 charWidth = Surface::textWidth(charName);
+	uint16 charWidth = Surface::textWidth(srcCharName);
 	_surface->writeString((TALK_DIALOG_WIDTH-charWidth)/2, TALK_DIALOG_EDGE_SIZE + 2,
-		charName, true, DIALOG_WHITE_COLOUR);
+		srcCharName, true, DIALOG_WHITE_COLOUR);
 
 	// TEMPORARY CODE - write out description. More properly, the text is meant to
 	// be displayed slowly, word by word
@@ -556,4 +605,177 @@ TalkDialog::~TalkDialog() {
 	delete _surface;
 }
 
+/*--------------------------------------------------------------------------*/
+
+#define SR_SEPARATOR_Y 21
+#define SR_SEPARATOR_X 5
+#define SR_SEPARATOR_HEIGHT 5
+#define SR_SAVEGAME_NAMES_Y (SR_SEPARATOR_Y + SR_SEPARATOR_HEIGHT + 1)
+
+void SaveRestoreDialog::toggleHightlight(int xs, int xe, int ys, int ye) {
+	Screen &screen = Screen::getReference();
+	byte *addr = screen.screen().data().data() + FULL_SCREEN_WIDTH * ys + xs;
+
+	for (int y = 0; y < ye - ys + 1; ++y, addr += FULL_SCREEN_WIDTH) {
+		for (int x = 0; x < xe - xs + 1; ++x) {
+			if (addr[x] == DIALOG_TEXT_COLOUR) addr[x] = DIALOG_WHITE_COLOUR;
+			else if (addr[x] == DIALOG_WHITE_COLOUR) addr[x] = DIALOG_TEXT_COLOUR;
+		}
+	}
+
+	screen.update();
+}
+
+bool SaveRestoreDialog::show(bool saveDialog) {
+	OSystem &system = *g_system;
+	Screen &screen = Screen::getReference();
+	Mouse &mouse = Mouse::getReference();
+	Events &events = Events::getReference();
+	Room &room = Room::getReference();
+	Resources &res = Resources::getReference();
+	LureEngine &engine = LureEngine::getReference();
+	int selectedLine = -1;
+	int index;
+
+	// Figure out a list of present savegames
+	String **saveNames = (String **)Memory::alloc(sizeof(String *) * MAX_SAVEGAME_SLOTS);
+	int numSaves = 0;
+	while ((numSaves < MAX_SAVEGAME_SLOTS) && 
+		((saveNames[numSaves] = engine.detectSave(numSaves + 1)) != NULL))
+		++numSaves;
+
+	// For the save dialog, if all the slots have not been used up, create a 
+	// blank entry for a new savegame
+	if (saveDialog && (numSaves < MAX_SAVEGAME_SLOTS))
+		saveNames[numSaves++] = new String();
+
+	// For the restore dialog, if there are no savegames, return immediately
+	if (!saveDialog && (numSaves == 0)) {
+		Memory::dealloc(saveNames);
+		return false;
+	}
+
+	room.update();
+	Surface *s = new Surface(INFO_DIALOG_WIDTH, SR_SAVEGAME_NAMES_Y + 
+		numSaves * FONT_HEIGHT + FONT_HEIGHT + 2);
+
+	// Create the outer dialog and dividing line
+	s->createDialog();
+	byte *pDest = s->data().data() + (s->width() * SR_SEPARATOR_Y) + SR_SEPARATOR_X;
+	uint8 rowColours[5] = {*(pDest-2), *(pDest-1), *(pDest-1), *(pDest-2), *(pDest+1)};
+	for (int y = 0; y < SR_SEPARATOR_HEIGHT; ++y, pDest += s->width())
+		memset(pDest, rowColours[y], s->width() - 12);
+
+	// Create title line
+	Common::String title(res.stringList().getString(
+		saveDialog ? S_SAVE_GAME : S_RESTORE_GAME));
+	s->writeString((s->width() - s->textWidth(title.c_str())) / 2, FONT_HEIGHT+2, title, true);
+
+	// Write out any existing save names
+	for (index = 0; index < numSaves; ++index)
+		s->writeString(DIALOG_EDGE_SIZE, SR_SAVEGAME_NAMES_Y + (index * 8), saveNames[index]->c_str(), true);
+
+	// Display the dialog
+	s->copyTo(&screen.screen(), SAVE_DIALOG_X, SAVE_DIALOG_Y);
+	screen.update();
+	mouse.pushCursorNum(CURSOR_ARROW);
+
+	bool abortFlag = false;
+	bool doneFlag = false;
+	while (!abortFlag && !doneFlag) {
+		// Provide highlighting of lines to select a save slot
+		while (!(mouse.lButton() && (selectedLine != -1)) && !mouse.rButton()) {
+			abortFlag = events.quitFlag;
+			if (abortFlag) break;
+
+			if (events.pollEvent()) {
+				if ((events.type() == Common::EVENT_KEYDOWN) &&
+					(events.event().kbd.ascii == 27)) {
+					abortFlag = true;
+					break;
+				}
+				if (events.type() == Common::EVENT_MOUSEMOVE) {
+					// Mouse movement
+					int lineNum;
+					if ((mouse.x() < (SAVE_DIALOG_X + DIALOG_EDGE_SIZE)) ||
+						(mouse.x() >= (SAVE_DIALOG_X + s->width() - DIALOG_EDGE_SIZE)) ||
+						(mouse.y() < SAVE_DIALOG_Y + SR_SAVEGAME_NAMES_Y) ||
+						(mouse.y() >= SAVE_DIALOG_Y + SR_SAVEGAME_NAMES_Y + numSaves * FONT_HEIGHT))
+						// Outside displayed lines
+						lineNum = -1;
+					else
+						lineNum = (mouse.y() - (SAVE_DIALOG_Y + SR_SAVEGAME_NAMES_Y)) / FONT_HEIGHT;
+
+					if (lineNum != selectedLine) {
+						if (selectedLine != -1)
+							// Deselect previously selected line
+							toggleHightlight(SAVE_DIALOG_X + DIALOG_EDGE_SIZE, 
+								SAVE_DIALOG_X + s->width() - DIALOG_EDGE_SIZE,
+								SAVE_DIALOG_Y + SR_SAVEGAME_NAMES_Y + selectedLine * FONT_HEIGHT,
+								SAVE_DIALOG_Y + SR_SAVEGAME_NAMES_Y + (selectedLine + 1) * FONT_HEIGHT - 1);
+
+						// Highlight new line
+						selectedLine = lineNum;
+						if (selectedLine != -1)
+							toggleHightlight(SAVE_DIALOG_X + DIALOG_EDGE_SIZE, 
+								SAVE_DIALOG_X + s->width() - DIALOG_EDGE_SIZE,
+								SAVE_DIALOG_Y + SR_SAVEGAME_NAMES_Y + selectedLine * FONT_HEIGHT,
+								SAVE_DIALOG_Y + SR_SAVEGAME_NAMES_Y + (selectedLine + 1) * FONT_HEIGHT - 1);
+					}
+				}
+			}
+
+			system.updateScreen();
+			system.delayMillis(10);
+		}
+
+		// Deselect selected row
+		if (selectedLine != -1) 
+			toggleHightlight(SAVE_DIALOG_X + DIALOG_EDGE_SIZE, 
+				SAVE_DIALOG_X + s->width() - DIALOG_EDGE_SIZE,
+				SAVE_DIALOG_Y + SR_SAVEGAME_NAMES_Y + selectedLine * FONT_HEIGHT,
+				SAVE_DIALOG_Y + SR_SAVEGAME_NAMES_Y + (selectedLine + 1) * FONT_HEIGHT - 1);
+
+		if (mouse.lButton() || mouse.rButton()) {
+			abortFlag = mouse.rButton();
+			mouse.waitForRelease();
+		}
+		if (abortFlag) break;
+
+		// If in save mode, allow the entry of a new savename
+		if (saveDialog) {
+			if (!screen.screen().getString(*saveNames[selectedLine], 
+				INFO_DIALOG_WIDTH - (DIALOG_EDGE_SIZE * 2), 
+				false, true, SAVE_DIALOG_X + DIALOG_EDGE_SIZE, 
+				SAVE_DIALOG_Y + SR_SAVEGAME_NAMES_Y + selectedLine * FONT_HEIGHT)) {
+				// Aborted out of name selection, so restore old name and 
+				// go back to slot selection
+				screen.screen().writeString(
+					SAVE_DIALOG_X + DIALOG_EDGE_SIZE, 
+					SAVE_DIALOG_Y + SR_SAVEGAME_NAMES_Y + selectedLine * FONT_HEIGHT,
+                    saveNames[selectedLine]->c_str(), true, DIALOG_TEXT_COLOUR, true);
+				selectedLine = -1;
+				continue;
+			}
+		}
+		doneFlag = true;
+	}
+
+	if (doneFlag) {
+		// Handle save or restore
+		if (saveDialog)
+			doneFlag = engine.saveGame(selectedLine + 1, *saveNames[selectedLine]);
+		else
+			doneFlag = engine.loadGame(selectedLine + 1);
+	}
+
+	mouse.popCursor();
+
+	// Free savegame caption list
+	for (index = 0; index < numSaves; ++index) delete saveNames[index];
+	Memory::dealloc(saveNames);
+
+	return doneFlag;
+}
+
 } // end of namespace Lure
