@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2005-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,13 +18,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/lure/surface.h $
- * $Id: surface.h 22809 2006-05-31 20:05:31Z anotherguest $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/lure/surface.h $
+ * $Id: surface.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
-#ifndef __lure_surface_h__
-#define __lure_surface_h__
+#ifndef LURE_SURFACE_H
+#define LURE_SURFACE_H
 
 #include "common/stdafx.h"
 #include "common/str.h"
@@ -67,16 +70,17 @@ public:
 
 	static uint16 textWidth(const char *s, int numChars = 0);
 	static void wordWrap(char *text, uint16 width, char **&lines, uint8 &numLines);
-	static Surface *newDialog(uint16 width, uint8 numLines, char **lines, bool varLength = true, uint8 colour = DIALOG_TEXT_COLOUR);
+	static Surface *newDialog(uint16 width, uint8 numLines, const char **lines, bool varLength = true, uint8 colour = DIALOG_TEXT_COLOUR);
 	static Surface *newDialog(uint16 width, const char *lines, uint8 colour = DIALOG_TEXT_COLOUR);
 	static Surface *getScreen(uint16 resourceId);
+	bool getString(Common::String &line, int maxSize, bool isNumeric, bool varLength, int16 x, int16 y);
 };
 
 class Dialog {
 public:
 	static void show(const char *text);
+	static void show(uint16 stringId, const char *hotspotName, const char *characterName);
 	static void show(uint16 stringId);
-	static void showMessage(uint16 messageId, uint16 characterId);
 };
 
 class TalkDialog {
@@ -86,13 +90,20 @@ private:
 	char **_lines;
 	uint8 _numLines;
 public:
-	TalkDialog(uint16 characterId, uint16 descId);
+	TalkDialog(uint16 characterId, uint16 destCharacterId, uint16 activeItemId, uint16 descId);
 	~TalkDialog();
 
 	char *desc() { return _desc; }
 	Surface &surface() { return *_surface; }
 };
 
+class SaveRestoreDialog {
+private:
+	static void toggleHightlight(int xs, int xe, int ys, int ye);
+public:
+	static bool show(bool saveDialog);
+};
+
 } // End of namespace Lure
 
 #endif
