@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2003-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,13 +18,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/queen/bankman.h $
- * $Id: bankman.h 20516 2006-02-11 10:07:12Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/queen/bankman.h $
+ * $Id: bankman.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
-#ifndef QUEENBANKMAN_H
-#define QUEENBANKMAN_H
+#ifndef QUEEN_BANKMAN_H
+#define QUEEN_BANKMAN_H
 
 #include "common/util.h"
 #include "queen/structs.h"
@@ -68,6 +71,7 @@ private:
 	struct PackedBank {
 		uint32 indexes[MAX_BANK_SIZE];
 		uint8 *data;
+		char name[20];
 	};
 
 	//! unpacked bob frames
@@ -76,9 +80,6 @@ private:
 	 //! banked bob frames
 	PackedBank _banks[MAX_BANKS_NUMBER];
 
-	//! loaded banks names
-	char _loadedBanks[MAX_BANKS_NUMBER][20];
-
 	Resource *_res;
 };
 
