@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2003-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,12 +18,14 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/queen/command.cpp $
- * $Id: command.cpp 22785 2006-05-30 20:46:11Z cyx $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/queen/command.cpp $
+ * $Id: command.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
 #include "common/stdafx.h"
+#include "common/events.h"
+#include "common/system.h"
 #include "queen/command.h"
 
 #include "queen/display.h"
@@ -36,8 +41,9 @@
 
 namespace Queen {
 
-CmdText::CmdText(bool reversed, uint8 y, QueenEngine *vm)
-	: _isReversed(reversed), _y(y), _vm(vm) {
+CmdText::CmdText(uint8 y, QueenEngine *vm)
+	: _y(y), _vm(vm) {
+	_isReversed = (_vm->resource()->getLanguage() == Common::HB_ISR);
 	clear();
 }
 
@@ -45,12 +51,12 @@ void CmdText::clear() {
 	memset(_command, 0, sizeof(_command));
 }
 
-void CmdText::display(uint8 color) {
-	_vm->display()->textCurrentColor(color);
+void CmdText::display(InkColor color) {
+	_vm->display()->textCurrentColor(_vm->display()->getInkColor(color));
 	_vm->display()->setTextCentered(_y, _command, false);
 }
 
-void CmdText::displayTemp(uint8 color, Verb v, const char *name, bool outlined) {
+void CmdText::displayTemp(InkColor color, Verb v, const char *name, bool outlined) {
 	char temp[MAX_COMMAND_LEN] = "";
 	if (_isReversed) {
 		if (name != NULL)
@@ -63,17 +69,17 @@ void CmdText::displayTemp(uint8 color, Verb v, const char *name, bool outlined)
 			strcat(temp, name);
 		}
 	}
-	_vm->display()->textCurrentColor(color);
+	_vm->display()->textCurrentColor(_vm->display()->getInkColor(color));
 	_vm->display()->setTextCentered(_y, temp, outlined);
 }
 
-void CmdText::displayTemp(uint8 color, const char *name, bool outlined) {
+void CmdText::displayTemp(InkColor color, const char *name, bool outlined) {
 	char temp[MAX_COMMAND_LEN];
 	if (_isReversed)
 		sprintf(temp, "%s %s", name, _command);
 	else
 		sprintf(temp, "%s %s", _command, name);
-	_vm->display()->textCurrentColor(color);
+	_vm->display()->textCurrentColor(_vm->display()->getInkColor(color));
 	_vm->display()->setTextCentered(_y, temp, outlined);
 }
 
@@ -124,7 +130,7 @@ void CmdState::init() {
 
 Command::Command(QueenEngine *vm)
 	: _cmdList(NULL), _cmdArea(NULL), _cmdObject(NULL), _cmdInventory(NULL), _cmdGameState(NULL),
-	_cmdText((vm->resource()->getLanguage() == HEBREW), CmdText::COMMAND_Y_POS, vm), _vm(vm) {
+	_cmdText(CmdText::COMMAND_Y_POS, vm), _vm(vm) {
 }
 
 Command::~Command() {
@@ -214,7 +220,7 @@ void Command::executeCurrentAction() {
 		// Joe is stuck behind the waterfall due to a walkbox issue. We could
 		// fix the walkbox issue, but then Joe would walk through the waterfall
 		// which wouldn't look that nice, graphically.
-		//		
+		//
 		// Since this command isn't necessary to complete the game and doesn't
 		// really makes sense here, we just skip it for now. The same cutscene
 		// is already played in command 648, so the user don't miss anything
@@ -225,7 +231,7 @@ void Command::executeCurrentAction() {
 		if (comId == 649) {
 			continue;
 		}
-				
+
 		com = &_cmdList[comId];
 
 		// check the Gamestates and set them if necessary
@@ -268,10 +274,9 @@ void Command::executeCurrentAction() {
 
 void Command::updatePlayer() {
 	if (_vm->logic()->joeWalk() != JWM_MOVE) {
-		int16 cx = _vm->input()->mousePosX();
-		int16 cy = _vm->input()->mousePosY();
-		lookForCurrentObject(cx, cy);
-		lookForCurrentIcon(cx, cy);
+		Common::Point mouse = _vm->input()->getMousePos();
+		lookForCurrentObject(mouse.x, mouse.y);
+		lookForCurrentIcon(mouse.x, mouse.y);
 	}
 
 	if (_vm->input()->keyVerb() != VERB_NONE) {
@@ -529,8 +534,9 @@ int16 Command::makeJoeWalkTo(int16 x, int16 y, int16 objNum, Verb v, bool mustWa
 }
 
 void Command::grabCurrentSelection() {
-	_selPosX = _vm->input()->mousePosX();
-	_selPosY = _vm->input()->mousePosY();
+	Common::Point mouse = _vm->input()->getMousePos();
+	_selPosX = mouse.x;
+	_selPosY = mouse.y;
 
 	uint16 zone = _vm->grid()->findObjectUnderCursor(_selPosX, _selPosY);
 	_state.noun = _vm->grid()->findObjectNumber(zone);
@@ -707,7 +713,7 @@ void Command::grabSelectedVerb() {
 
 bool Command::executeIfCutaway(const char *description) {
 	if (strlen(description) > 4 &&
-		scumm_stricmp(description + strlen(description) - 4, ".cut") == 0) {
+		scumm_stricmp(description + strlen(description) - 4, ".CUT") == 0) {
 
 		_vm->display()->clearTexts(CmdText::COMMAND_Y_POS, CmdText::COMMAND_Y_POS);
 
@@ -724,7 +730,7 @@ bool Command::executeIfCutaway(const char *description) {
 
 bool Command::executeIfDialog(const char *description) {
 	if (strlen(description) > 4 &&
-		scumm_stricmp(description + strlen(description) - 4, ".dog") == 0) {
+		scumm_stricmp(description + strlen(description) - 4, ".DOG") == 0) {
 
 		_vm->display()->clearTexts(CmdText::COMMAND_Y_POS, CmdText::COMMAND_Y_POS);
 
@@ -1211,8 +1217,9 @@ void Command::lookAtSelectedObject() {
 			break;
 		}
 	}
-
-	_vm->logic()->makeJoeSpeak(desc, true);
+	if (desc != 0) {
+		_vm->logic()->makeJoeSpeak(desc, true);
+	}
 	_vm->logic()->joeFace();
 }
 
