@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2003-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,13 +18,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/queen/command.h $
- * $Id: command.h 20516 2006-02-11 10:07:12Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/queen/command.h $
+ * $Id: command.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
-#ifndef QUEENCOMMAND_H
-#define QUEENCOMMAND_H
+#ifndef QUEEN_COMMAND_H
+#define QUEEN_COMMAND_H
 
 #include "common/util.h"
 #include "queen/structs.h"
@@ -32,19 +35,19 @@ class QueenEngine;
 
 struct CmdText {
 
-	CmdText(bool reversed, uint8 y, QueenEngine *vm);
+	CmdText(uint8 y, QueenEngine *vm);
 
 	//! reset the command sentence
 	void clear();
 
 	//! display the command sentence using the specified color
-	void display(uint8 color);
+	void display(InkColor color);
 
 	//! display a temporary command sentence using the specified parameters
-	void displayTemp(uint8 color, Verb v, const char *name = NULL, bool outlined = false);
+	void displayTemp(InkColor color, Verb v, const char *name = NULL, bool outlined = false);
 
 	//! display a temporary command sentence using the specified parameters
-	void displayTemp(uint8 color, const char *name, bool outlined = false);
+	void displayTemp(InkColor color, const char *name, bool outlined = false);
 
 	//! set the verb for the command sentence
 	void setVerb(Verb v);
@@ -185,7 +188,7 @@ private:
 	void lookForCurrentIcon(int16 cx, int16 cy);
 
 	//! returns true if the verb is an action verb
-	bool isVerbAction(Verb v) const { return (v >= VERB_PANEL_COMMAND_FIRST && v <= VERB_PANEL_COMMAND_LAST) || (v == VERB_WALK_TO); };
+	bool isVerbAction(Verb v) const { return (v >= VERB_PANEL_COMMAND_FIRST && v <= VERB_PANEL_COMMAND_LAST) || (v == VERB_WALK_TO); }
 
 	//! return true if the verb is an inventory item
 	bool isVerbInv(Verb v) const { return v >= VERB_INV_FIRST && v <= VERB_INV_LAST; }
