@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2003-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/queen/credits.cpp $
- * $Id: credits.cpp 20516 2006-02-11 10:07:12Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/queen/credits.cpp $
+ * $Id: credits.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -30,14 +33,8 @@
 namespace Queen {
 
 Credits::Credits(QueenEngine *vm, const char* filename) :
-	_vm(vm), _running(true), _count(0), _pause(0), _justify(0), _fontSize(0), _color(0), _zone(0) {
-	uint32 size;
-	char *buf = (char *)_vm->resource()->loadFile(filename, 0, &size);
-	_credits = new LineReader(buf, size);
-}
-
-Credits::~Credits() {
-	delete _credits;
+	_vm(vm), _running(true), _count(0), _pause(0), _justify(0), _fontSize(0), _color(0), _zone(0), _lineNum(0) {
+	_vm->resource()->loadTextFile(filename, _credits);
 }
 
 void Credits::nextRoom() {
@@ -62,8 +59,9 @@ void Credits::update() {
 	if (-1 == _pause)
 		return;
 
-	for (;;) {
-		const char *line = _credits->nextLine();
+	while (_lineNum < _credits.size()) {
+		const char *line = _credits[_lineNum].c_str();
+		++_lineNum;
 
 		if (0 == memcmp(line, "EN", 2)) {
 			_running = false;
@@ -103,6 +101,9 @@ void Credits::update() {
 				return;
 			case 'i' :
 				_color = atoi(&line[3]);
+				if (_vm->resource()->getPlatform() == Common::kPlatformAmiga) {
+					_color &= 31;
+				}
 				break;
 			case '1' :
 			case '2' :
@@ -138,6 +139,7 @@ void Credits::update() {
 			_count++;
 		}
 	}
+	_running = false;
 }
 
 
