@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2003-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,13 +18,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/queen/credits.h $
- * $Id: credits.h 20516 2006-02-11 10:07:12Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/queen/credits.h $
+ * $Id: credits.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
-#ifndef CREDITS_H
-#define CREDITS_H
+#ifndef QUEEN_CREDITS_H
+#define QUEEN_CREDITS_H
 
 #include "common/util.h"
 #include "queen/defs.h"
@@ -35,7 +38,6 @@ class Credits {
 public:
 
 	Credits(QueenEngine *vm, const char* filename);
-	~Credits();
 
 	//! update/display credits for current room
 	void update();
@@ -77,8 +79,10 @@ private:
 	//! current text position
 	int _zone;
 
+	uint _lineNum;
+
 	//! contains the credits description
-	LineReader *_credits;
+	Common::StringList _credits;
 
 	QueenEngine *_vm;
 };
