@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2003-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/queen/cutaway.cpp $
- * $Id: cutaway.cpp 20975 2006-02-28 22:20:13Z cyx $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/queen/cutaway.cpp $
+ * $Id: cutaway.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -66,7 +69,7 @@ void Cutaway::load(const char *filename) {
 
 	ptr = _fileData = _vm->resource()->loadFile(filename, 20);
 
-	if (0 == scumm_stricmp(filename, "comic.cut"))
+	if (0 == scumm_stricmp(filename, "COMIC.CUT"))
 		_songBeforeComic = _vm->sound()->lastOverride();
 
 	strcpy(_basename, filename);
@@ -463,8 +466,7 @@ Cutaway::ObjectType Cutaway::getObjectType(CutawayObject &object) {
 
 const byte *Cutaway::getCutawayAnim(const byte *ptr, int header, CutawayAnim &anim) {
 	// lines 1531-1607 in cutaway.c
-
-	//debug(6, "[Cutaway::getCutawayAnim] header=%i", header);
+	debug(6, "[Cutaway::getCutawayAnim] header=%i", header);
 
 	anim.currentFrame = 0;
 	anim.originalFrame = 0;
@@ -493,6 +495,7 @@ const byte *Cutaway::getCutawayAnim(const byte *ptr, int header, CutawayAnim &an
 		anim.bank = 15;
 	} else {
 		if (anim.bank != 13) {
+			assert(anim.bank - 1 < MAX_BANK_NAME_COUNT);
 			_vm->bankMan()->load(_bankNames[anim.bank-1], CUTAWAY_BANK);
 			anim.bank = 8;
 		} else {
@@ -516,7 +519,8 @@ const byte *Cutaway::getCutawayAnim(const byte *ptr, int header, CutawayAnim &an
 	anim.scale = (int16)READ_BE_INT16(ptr);
 	ptr += 2;
 
-	if (_vm->resource()->isDemo()) {
+	if ((_vm->resource()->isDemo() && _vm->resource()->getPlatform() == Common::kPlatformPC) ||
+		(_vm->resource()->isInterview() && _vm->resource()->getPlatform() == Common::kPlatformAmiga)) {
 		anim.song = 0;
 	} else {
 		anim.song = (int16)READ_BE_INT16(ptr);
@@ -1165,7 +1169,7 @@ void Cutaway::updateGameState() {
 
 void Cutaway::talk(char *nextFilename) {
 	const char *p = strrchr(_talkFile, '.');
-	if (p && 0 == scumm_stricmp(p, ".dog")) {
+	if (p && 0 == scumm_stricmp(p, ".DOG")) {
 		nextFilename[0] = '\0';
 		assert(_talkTo > 0);
 		int personInRoom = _talkTo - _vm->logic()->roomData(_vm->logic()->currentRoom());
@@ -1242,7 +1246,7 @@ void Cutaway::handleText(
 			char voiceFileName[MAX_STRING_SIZE];
 			findCdCut(_basename, index, voiceFileName);
 			strcat(voiceFileName, "1");
-			_vm->sound()->playSfx(voiceFileName, true);
+			_vm->sound()->playSpeech(voiceFileName);
 		}
 
 		if (OBJECT_TYPE_TEXT_SPEAK == type && _vm->sound()->speechOn() && !_vm->subtitles())
