@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2003-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/queen/debug.cpp $
- * $Id: debug.cpp 20516 2006-02-11 10:07:12Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/queen/debug.cpp $
+ * $Id: debug.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -29,24 +32,20 @@
 #include "queen/resource.h"
 #include "queen/sound.h"
 
-#include "common/debugger.cpp"
-
 namespace Queen {
 
 Debugger::Debugger(QueenEngine *vm)
 	: _vm(vm), _flags(0) {
 
-	DCmd_Register("exit", &Debugger::Cmd_Exit);
-	DCmd_Register("help", &Debugger::Cmd_Help);
-	DCmd_Register("areas", &Debugger::Cmd_Areas);
-	DCmd_Register("asm", &Debugger::Cmd_Asm);
-	DCmd_Register("bob", &Debugger::Cmd_Bob);
-	DCmd_Register("bobs", &Debugger::Cmd_PrintBobs);
-	DCmd_Register("gs", &Debugger::Cmd_GameState);
-	DCmd_Register("info", &Debugger::Cmd_Info);
-	DCmd_Register("items", &Debugger::Cmd_Items);
-	DCmd_Register("room", &Debugger::Cmd_Room);
-	DCmd_Register("song", &Debugger::Cmd_Song);
+	DCmd_Register("areas", WRAP_METHOD(Debugger, Cmd_Areas));
+	DCmd_Register("asm",   WRAP_METHOD(Debugger, Cmd_Asm));
+	DCmd_Register("bob",   WRAP_METHOD(Debugger, Cmd_Bob));
+	DCmd_Register("bobs",  WRAP_METHOD(Debugger, Cmd_PrintBobs));
+	DCmd_Register("gs",    WRAP_METHOD(Debugger, Cmd_GameState));
+	DCmd_Register("info",  WRAP_METHOD(Debugger, Cmd_Info));
+	DCmd_Register("items", WRAP_METHOD(Debugger, Cmd_Items));
+	DCmd_Register("room",  WRAP_METHOD(Debugger, Cmd_Room));
+	DCmd_Register("song",  WRAP_METHOD(Debugger, Cmd_Song));
 }
 
 Debugger::~Debugger() {} // we need this here for __SYMBIAN32__
@@ -57,32 +56,6 @@ void Debugger::postEnter() {
 	_vm->graphics()->setupMouseCursor();
 }
 
-bool Debugger::Cmd_Exit(int argc, const char **argv) {
-	_detach_now = true;
-	return false;
-}
-
-bool Debugger::Cmd_Help(int argc, const char **argv) {
-	// console normally has 39 line width
-	// wrap around nicely
-	int width = 0, size, i;
-
-	DebugPrintf("Commands are:\n");
-	for (i = 0 ; i < _dcmd_count ; i++) {
-		size = strlen(_dcmds[i].name) + 1;
-
-		if ((width + size) >= 39) {
-			DebugPrintf("\n");
-			width = size;
-		} else
-			width += size;
-
-		DebugPrintf("%s ", _dcmds[i].name);
-	}
-	DebugPrintf("\n");
-	return true;
-}
-
 bool Debugger::Cmd_Asm(int argc, const char **argv) {
 	if (argc == 2) {
 		uint16 sm = atoi(argv[1]);
@@ -156,8 +129,8 @@ bool Debugger::Cmd_GameState(int argc, const char **argv) {
 }
 
 bool Debugger::Cmd_Info(int argc, const char **argv) {
-	DebugPrintf("Version: %s\n", _vm->resource()->JASVersion());
-	DebugPrintf("Audio compression: %d\n", _vm->resource()->compression());
+	DebugPrintf("Version: %s\n", _vm->resource()->getJASVersion());
+	DebugPrintf("Audio compression: %d\n", _vm->resource()->getCompression());
 	return true;
 }
 
