@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2003-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,13 +18,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/queen/defs.h $
- * $Id: defs.h 21398 2006-03-22 00:24:10Z sev $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/queen/defs.h $
+ * $Id: defs.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
-#ifndef QUEENDEFS_H
-#define QUEENDEFS_H
+#ifndef QUEEN_DEFS_H
+#define QUEEN_DEFS_H
 
 namespace Queen {
 
@@ -41,13 +44,11 @@ enum {
 	PANEL_ZONE_HEIGHT  =  50
 };
 
-
 enum {
 	FRAMES_JOE      = 38,
 	FRAMES_JOURNAL  = 40
 };
 
-
 enum Direction {
 	DIR_LEFT  = 1,
 	DIR_RIGHT = 2,
@@ -55,20 +56,19 @@ enum Direction {
 	DIR_BACK  = 4
 };
 
-
-enum {
-	INK_BG_PANEL      = 226,
-	INK_JOURNAL       = 248,
-	INK_PINNACLE_ROOM = 243,
-	INK_CMD_SELECT    = 255,
-	INK_CMD_NORMAL    = 225,
-	INK_CMD_LOCK      = 234,
-	INK_TALK_NORMAL   = 7,
-	INK_JOE           = 14,
-	INK_OUTLINED_TEXT = 16
+enum InkColor {
+	INK_BG_PANEL = 0,
+	INK_JOURNAL,
+	INK_PINNACLE_ROOM,
+	INK_CMD_SELECT,
+	INK_CMD_NORMAL,
+	INK_TALK_NORMAL,
+	INK_JOE,
+	INK_OUTLINED_TEXT,
+	
+	INK_COUNT
 };
 
-
 enum {
 	ITEM_NONE                     =  0,
 	ITEM_BAT,
@@ -252,7 +252,6 @@ enum {
 	ROOM_JOURNAL              = 200 // dummy value to keep Display methods happy
 };
 
-
 //! GameState vars
 enum {
 	VAR_HOTEL_ITEMS_REMOVED       =   3,
@@ -265,18 +264,6 @@ enum {
 	VAR_AZURA_IN_LOVE             = 167
 };
 
-
-enum Language {
-	ENGLISH  = 'E',
-	FRENCH   = 'F',
-	GERMAN   = 'G',
-	HEBREW   = 'H',
-	ITALIAN  = 'I',
-	SPANISH  = 'S',
-	RUSSIAN  = 'R'
-};
-
-
 enum Verb {
 	VERB_NONE = 0,
 
@@ -317,7 +304,6 @@ enum Verb {
 	VERB_PREP_TO   = 12
 };
 
-
 } // End of namespace Queen
 
 #endif
