@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2003-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,13 +18,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/queen/display.h $
- * $Id: display.h 23992 2006-09-26 06:09:29Z eriktorbjorn $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/queen/display.h $
+ * $Id: display.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
-#ifndef QUEENDISPLAY_H
-#define QUEENDISPLAY_H
+#ifndef QUEEN_DISPLAY_H
+#define QUEEN_DISPLAY_H
 
 #include "common/str.h"
 #include "common/util.h"
@@ -72,6 +75,8 @@ public:
 	//! scroll some palette colors
 	void palScroll(int start, int end);
 
+	void palSetAmigaColor(uint8 color, uint16 rgb);
+
 	//! custom palette effect for the specified room
 	void palCustomColors(uint16 roomNum);
 
@@ -109,8 +114,11 @@ public:
 	void blit(uint8 *dstBuf, uint16 dstPitch, uint16 x, uint16 y, const uint8 *srcBuf, uint16 srcPitch, uint16 w, uint16 h, bool xflip, bool masked);
 	void fill(uint8 *dstBuf, uint16 dstPitch, uint16 x, uint16 y, uint16 w, uint16 h, uint8 color);
 
-	//! decode a PCX stream
-	void readPCX(uint8 *dst, uint16 dstPitch, const uint8 *src, uint16 w, uint16 h);
+	//! decode PCX picture data
+	void decodePCX(const uint8 *src, uint32 srcSize, uint8 *dst, uint16 dstPitch, uint16 *w, uint16 *h, uint8 *pal, uint16 palStart, uint16 palEnd);
+
+	//! decode ILBM picture data
+	void decodeLBM(const uint8 *src, uint32 srcSize, uint8 *dst, uint16 dstPitch, uint16 *w, uint16 *h, uint8 *pal, uint16 palStart, uint16 palEnd, uint8 colorBase = 0);
 
 	void horizontalScrollUpdate(int16 xCamera);
 	void horizontalScroll(int16 scroll);
@@ -146,6 +154,10 @@ public:
 	//! remove entries from the texts list
 	void clearTexts(uint16 y1, uint16 y2);
 
+	void setupInkColors();
+
+	uint8 getInkColor(InkColor color) const { return _inkColors[color]; }
+
 	//! change the current text color
 	void textCurrentColor(uint8 color) { _curTextColor = color; }
 
@@ -223,6 +235,8 @@ private:
 	uint16 _horizontalScroll;
 	uint16 _bdWidth, _bdHeight;
 
+	const char *_imageExt;
+
 	//! texts list
 	TextSlot _texts[GAME_SCREEN_HEIGHT];
 
@@ -232,6 +246,8 @@ private:
 	//! font justification sizes
 	uint8 _charWidth[256];
 
+	uint8 _inkColors[INK_COUNT];
+
 	Common::RandomSource _rnd;
 	Dynalum _dynalum;
 	OSystem *_system;
