@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2003-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/queen/graphics.cpp $
- * $Id: graphics.cpp 20975 2006-02-28 22:20:13Z cyx $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/queen/graphics.cpp $
+ * $Id: graphics.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -246,31 +249,80 @@ Graphics::~Graphics() {
 }
 
 void Graphics::unpackControlBank() {
-	_vm->bankMan()->load("control.BBK",17);
-	_vm->bankMan()->unpack(1, 1, 17); // Mouse pointer
-	// unpack arrows frames and change hotspot to be always on top
-	for (int i = 3; i <= 4; ++i) {
-		_vm->bankMan()->unpack(i, i, 17);
-		BobFrame *bf = _vm->bankMan()->fetchFrame(i);
-		bf->yhotspot += 200;
+	if (_vm->resource()->getPlatform() == Common::kPlatformPC) {
+		_vm->bankMan()->load("CONTROL.BBK",17);
+
+		// unpack mouse pointer frame
+		_vm->bankMan()->unpack(1, 1, 17);
+
+		// unpack arrows frames and change hotspot to be always on top
+		_vm->bankMan()->unpack(3, 3, 17);
+		_vm->bankMan()->fetchFrame(3)->yhotspot += 200;
+		_vm->bankMan()->unpack(4, 4, 17);
+		_vm->bankMan()->fetchFrame(4)->yhotspot += 200;
+
+		_vm->bankMan()->close(17);
 	}
-	_vm->bankMan()->close(17);
 }
 
 void Graphics::setupArrows() {
-	int scrollX = _vm->display()->horizontalScroll();
-	BobSlot *arrow;
-	arrow = bob(ARROW_BOB_UP);
-	arrow->curPos(303 + 8 + scrollX, 150 + 1 + 200);
-	arrow->frameNum = 3;
-	arrow = bob(ARROW_BOB_DOWN);
-	arrow->curPos(303 + scrollX, 175 + 200);
-	arrow->frameNum = 4;
+	if (_vm->resource()->getPlatform() == Common::kPlatformPC) {
+		int scrollX = _vm->display()->horizontalScroll();
+		BobSlot *arrow;
+		arrow = bob(ARROW_BOB_UP);
+		arrow->curPos(303 + 8 + scrollX, 150 + 1 + 200);
+		arrow->frameNum = 3;
+		arrow = bob(ARROW_BOB_DOWN);
+		arrow->curPos(303 + scrollX, 175 + 200);
+		arrow->frameNum = 4;
+	}
 }
 
 void Graphics::setupMouseCursor() {
-	BobFrame *bf = _vm->bankMan()->fetchFrame(1);
-	_vm->display()->setMouseCursor(bf->data, bf->width, bf->height);
+	if (_vm->resource()->getPlatform() == Common::kPlatformAmiga) {
+		static const uint8 defaultAmigaCursor[4 * 15] = {
+			0x00, 0x00, 0xFF, 0xC0,
+			0x7F, 0x80, 0x80, 0x40,
+			0x7F, 0x00, 0x80, 0x80,
+			0x7E, 0x00, 0x81, 0x00,
+			0x7F, 0x00, 0x80, 0x80,
+			0x7F, 0x80, 0x80, 0x40,
+			0x7F, 0xC0, 0x80, 0x20,
+			0x6F, 0xE0, 0x90, 0x10,
+			0x47, 0xF0, 0xA8, 0x08,
+			0x03, 0xF8, 0xC4, 0x04,
+			0x01, 0xFC, 0x02, 0x02,
+			0x00, 0xF8, 0x01, 0x04,
+			0x00, 0x70, 0x00, 0x88,
+			0x00, 0x20, 0x00, 0x50,
+			0x00, 0x00, 0x00, 0x20
+		};
+		uint8 cursorData[16 * 15];
+		memset(cursorData, 0, sizeof(cursorData));
+		const uint8 *src = defaultAmigaCursor;
+		int i = 0;
+		for (int h = 0; h < 15; ++h) {
+			for (int b = 0; b < 16; ++b) {
+				const uint16 mask = (1 << (15 - b));
+				uint8 color = 0;
+				if (READ_BE_UINT16(src + 0) & mask) {
+					color |= 2;
+				}
+				if (READ_BE_UINT16(src + 2) & mask) {
+					color |= 1;
+				}
+				if (color != 0) {
+					cursorData[i] = 0x90 + color - 1;
+				}
+				++i;
+			}
+			src += 4;
+		}
+		_vm->display()->setMouseCursor(cursorData, 16, 15);
+	} else {
+		BobFrame *bf = _vm->bankMan()->fetchFrame(1);
+		_vm->display()->setMouseCursor(bf->data, bf->width, bf->height);
+	}
 }
 
 void Graphics::drawBob(const BobSlot *bs, const BobFrame *bf, const Box *bbox, int16 x, int16 y) {
@@ -380,7 +432,7 @@ void Graphics::sortBobs() {
 			if (pbs->animating) {
 				pbs->animOneStep();
 				if (pbs->frameNum > 500) { // SFX frame
-					_vm->sound()->playSfx(_vm->logic()->currentRoomSfx(), false);
+					_vm->sound()->playSfx(_vm->logic()->currentRoomSfx());
 					pbs->frameNum -= 500;
 				}
 			}
@@ -439,7 +491,7 @@ void Graphics::stopBobs() {
 }
 
 BobSlot *Graphics::bob(int index) {
-	assert(index < MAX_BOBS_NUMBER);
+	assert(index >= 0 && index < MAX_BOBS_NUMBER);
 	return &_bobs[index];
 }
 
@@ -469,7 +521,7 @@ void Graphics::setBobText(const BobSlot *pbs, const char *text, int textX, int t
 
 	// Hebrew strings are written from right to left and should be cut
 	// to lines in reverse
-	if (_vm->resource()->getLanguage() == HEBREW) {
+	if (_vm->resource()->getLanguage() == Common::HB_ISR) {
 		for (i = length - 1; i >= 0; i--) {
 			lineLength++;
 
@@ -545,6 +597,15 @@ void Graphics::setBobText(const BobSlot *pbs, const char *text, int textX, int t
 	else if ((x + maxLineWidth) > 320)
 		x = 320 - maxLineWidth - 4;
 
+	// remap some colors for the Amiga
+	if (_vm->resource()->getPlatform() == Common::kPlatformAmiga) {
+		if (color == 5) {
+			color = (_vm->logic()->currentRoom() == 9) ? 15 : 11;
+		} else if (color == 10 && _vm->logic()->currentRoom() == 100) {
+			color = 11;
+		}
+	}
+
 	_vm->display()->textCurrentColor(color);
 
 	for (i = 0; i < lineCount; i++) {
@@ -1019,7 +1080,6 @@ void Graphics::setupRoomObjects() {
 				curBob = 20 + _numFurnitureStatic + numObjectStatic;
 				++curImage;
 				bob(curBob)->clear();
-
 				_vm->bankMan()->unpack(pgd->firstFrame, curImage, 15);
 				++_numFrames;
 				if (pod->name > 0) {
@@ -1116,16 +1176,19 @@ void Graphics::update(uint16 room) {
 
 BamScene::BamScene(QueenEngine *vm)
 	: _flag(F_STOP), _screenShaked(false), _fightData(_fight1Data), _vm(vm) {
+	if (_vm->resource()->getPlatform() == Common::kPlatformAmiga) {
+		_fightData = _fight4Data;
+	}
 }
 
 void BamScene::playSfx() {
-	// Don't try to play all the sounds. This is only necessary for the 
+	// Don't try to play all the sounds. This is only necessary for the
 	// fight bam, in which the number of 'sfx bam frames' is too much
 	// important / too much closer. The original game does not have
 	// this problem since its playSfx() function returns immediately
 	// if a sound is already being played.
 	if (_lastSoundIndex == 0 || _index - _lastSoundIndex >= SFX_SKIP) {
-		_vm->sound()->playSfx(_vm->logic()->currentRoomSfx(), false);
+		_vm->sound()->playSfx(_vm->logic()->currentRoomSfx());
 		_lastSoundIndex = _index;
 	}
 }
@@ -1229,7 +1292,9 @@ void BamScene::updateFightAnimation() {
 			break;
 		case 99: // end of BAM data
 			_lastSoundIndex = _index = 0;
-			_fightData = fightDataBlocks[_vm->randomizer.getRandomNumber(2)];
+			if (_vm->resource()->getPlatform() == Common::kPlatformPC) {
+				_fightData = fightDataBlocks[_vm->randomizer.getRandomNumber(2)];
+			}
 			if (_flag == F_REQ_STOP) {
 				_flag = F_STOP;
 			}
@@ -1513,6 +1578,58 @@ const BamScene::BamDataBlock BamScene::_fight3Data[] = {
 	{ {  75, 96,  1 }, { 187,  96, -23 }, { 183,  41, 47 },  0 },
 	{ {  75, 96,  1 }, { 187,  96, -23 }, { 183,  41, 47 }, 99 }
 };
+
+const BamScene::BamDataBlock BamScene::_fight4Data[] = {
+	{ {  75, 96,  1 }, { 187,  96, -23 }, { 150,  45, 35 },  0 },
+	{ {  75, 96,  1 }, { 187,  96, -23 }, { 150,  45, 35 },  0 },
+	{ {  75, 96,  1 }, { 187,  96, -23 }, { 150,  45, 35 },  0 },
+	{ {  75, 96,  5 }, { 187,  96, -23 }, { 150,  45, 35 },  0 },
+	{ {  75, 96,  5 }, { 187,  96, -23 }, { 150,  45, 35 },  0 },
+	{ {  75, 96,  6 }, { 187,  96, -24 }, { 150,  45, 35 },  0 },
+	{ {  75, 96,  6 }, { 187,  96, -24 }, { 150,  45, 35 },  0 },
+	{ {  75, 96,  7 }, { 187,  96, -24 }, { 150,  45, 35 },  0 },
+	{ {  75, 96,  8 }, { 187,  96, -25 }, {  79, 101, 59 },  0 },
+	{ {  75, 96,  9 }, { 187,  96, -25 }, {  95, 104, 66 },  0 },
+	{ {  75, 96, 10 }, { 187,  96, -25 }, { 129, 104, 65 },  0 },
+	{ {  75, 96, 10 }, { 187,  96, -25 }, { 160, 104, 64 },  0 },
+	{ {  75, 96,  6 }, { 187,  96, -25 }, { 179, 104, 63 },  0 },
+	{ {  75, 96,  6 }, { 187,  96, -23 }, { 188, 104, 62 },  0 },
+	{ {  75, 96,  6 }, { 187,  96, -29 }, { 191, 104, 36 },  0 },
+	{ {  75, 96,  6 }, { 187,  96, -29 }, { 195, 104, 37 },  0 },
+	{ {  75, 96,  6 }, { 187,  96, -31 }, { 202, 104, 38 },  0 },
+	{ {  75, 96,  5 }, { 187,  96, -32 }, { 210, 104, 39 },  0 },
+	{ {  75, 96,  5 }, { 187,  98, -32 }, { 216, 104, 40 },  0 },
+	{ {  75, 96,  5 }, { 187,  96, -32 }, { 223, 104, 42 },  0 },
+	{ {  75, 96,  5 }, { 187,  98, -32 }, { 223, 104, 42 },  0 },
+	{ {  75, 96,  5 }, { 187,  97, -33 }, { 223, 104, 42 },  0 },
+	{ {  75, 96,  5 }, { 187,  96, -34 }, { 223, 104, 42 },  0 },
+	{ {  75, 96,  5 }, { 187,  96, -23 }, { 223, 104, 42 },  0 },
+	{ {  75, 96,  5 }, { 187,  96, -23 }, { 223, 104, 42 },  0 },
+	{ {  75, 96,  6 }, { 187,  96, -23 }, { 223, 104, 42 },  0 },
+	{ {  75, 96,  6 }, { 187,  96, -24 }, { 223, 104, 42 },  0 },
+	{ {  75, 96,  6 }, { 187,  96, -24 }, { 223, 104, 42 },  0 },
+	{ {  75, 96,  5 }, { 187,  96, -25 }, { 223, 104, 42 },  0 },
+	{ {  75, 96,  5 }, { 187,  96, -25 }, { 223, 104, 42 },  0 },
+	{ {  75, 96,  5 }, { 187,  96, -26 }, { 175,  98, 36 },  0 },
+	{ {  75, 96,  5 }, { 187,  96, -26 }, { 152,  98, 36 },  0 },
+	{ {  75, 96,  6 }, { 187,  96, -27 }, { 124,  98, 37 },  0 },
+	{ {  75, 96,  6 }, { 187,  96, -28 }, { 105,  98, 38 },  0 },
+	{ {  75, 96, 11 }, { 187,  96, -23 }, {  77,  98, 39 },  0 },
+	{ {  75, 96, 13 }, { 187,  96, -23 }, {  63,  98, 40 },  0 },
+	{ {  75, 96, 14 }, { 187,  96, -23 }, {  51,  98, 41 },  0 },
+	{ {  75, 98, 14 }, { 187,  96, -23 }, {  51,  98, 42 },  0 },
+	{ {  75, 94, 14 }, { 187,  96, -23 }, {   0,   0,  0 },  0 },
+	{ {  75, 98, 14 }, { 187,  96, -23 }, {   0,   0,  0 },  0 },
+	{ {  75, 96, 15 }, { 187,  96, -23 }, {   0,   0,  0 },  0 },
+	{ {  75, 96,  1 }, { 187,  96, -23 }, {   0,   0,  0 },  0 },
+	{ {  75, 96,  1 }, { 187,  96, -23 }, {   0,   0,  0 },  0 },
+	{ {  75, 96,  1 }, { 187,  96, -23 }, {   0,   0,  0 },  0 },
+	{ {  75, 96,  1 }, { 187,  96, -23 }, {   0,   0,  0 },  0 },
+	{ {  75, 96,  1 }, { 187,  96, -23 }, {   0,   0,  0 },  0 },
+	{ {  75, 96,  1 }, { 187,  96, -23 }, {   0,   0,  0 },  0 },
+	{ {  75, 96,  1 }, { 187,  96, -23 }, {   0,   0,  0 },  0 },
+	{ {  75, 96,  1 }, { 187,  96, -23 }, {   0,   0,  0 }, 99 }
+};
 #endif
 
 } // End of namespace Queen
