@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2003-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/queen/grid.cpp $
- * $Id: grid.cpp 20516 2006-02-11 10:07:12Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/queen/grid.cpp $
+ * $Id: grid.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -72,6 +75,7 @@ void Grid::readDataFrom(uint16 numObjects, uint16 numRooms, byte *&ptr) {
 
 void Grid::setZone(GridScreen screen, uint16 zoneNum, uint16 x1, uint16 y1, uint16 x2, uint16 y2) {
 	debug(9, "Grid::setZone(%d, %d, (%d,%d), (%d,%d))", screen, zoneNum, x1, y1, x2, y2);
+	assert(zoneNum < MAX_ZONES_NUMBER);
 	ZoneSlot *pzs = &_zones[screen][zoneNum];
 	pzs->valid = true;
 	pzs->box.x1 = x1;
@@ -82,6 +86,7 @@ void Grid::setZone(GridScreen screen, uint16 zoneNum, uint16 x1, uint16 y1, uint
 
 void Grid::setZone(GridScreen screen, uint16 zoneNum, const Box &box) {
 	debug(9, "Grid::setZone(%d, %d, (%d,%d), (%d,%d))", screen, zoneNum, box.x1, box.y1, box.x2, box.y2);
+	assert(zoneNum < MAX_ZONES_NUMBER);
 	ZoneSlot *pzs = &_zones[screen][zoneNum];
 	pzs->valid = true;
 	pzs->box = box;
@@ -132,7 +137,20 @@ void Grid::setupNewRoom(uint16 room, uint16 firstRoomObjNum) {
 	zoneNum = 1;
 	for (i = firstRoomObjNum + 1; i <= firstRoomObjNum + maxObjRoom; ++i) {
 		if (_vm->logic()->objectData(i)->name != 0) {
-			setZone(GS_ROOM, zoneNum, _objectBox[i]);
+			if (room == 41 && i == 303) {
+
+				// WORKAROUND bug #1599009: In the room 41, the bounding box of the
+				// stairs (object 303) doesn't match with the room picture. With the
+				// original box dimensions, Joe could walk "above" the stairs, giving
+				// the impression of floating in the air.
+				// To fix this, the bounding box is set relative to the position of
+				// the cabinet (object 295).
+
+				uint16 y1 = _objectBox[295].y2 + 1;
+				setZone(GS_ROOM, zoneNum, _objectBox[i].x1, y1, _objectBox[i].x2, _objectBox[i].y2);
+			} else {
+				setZone(GS_ROOM, zoneNum, _objectBox[i]);
+			}
 		}
 		++zoneNum;
 	}
