@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2003-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,19 +18,20 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/queen/input.cpp $
- * $Id: input.cpp 21994 2006-04-17 23:55:42Z eriktorbjorn $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/queen/input.cpp $
+ * $Id: input.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
 #include "common/stdafx.h"
+#include "common/events.h"
 #include "common/system.h"
 
 #include "queen/input.h"
 
 namespace Queen {
 
-const char *Input::_commandKeys[LANGUAGE_COUNT] = {
+const char *Input::_commandKeys[] = {
 	"ocmglptu", // English
 	"osbgpnre", // German
 	"ofdnepau", // French
@@ -36,7 +40,7 @@ const char *Input::_commandKeys[LANGUAGE_COUNT] = {
 	"acodmthu"  // Spanish
 };
 
-const Verb Input::_verbKeys[8] = {
+const Verb Input::_verbKeys[] = {
 	VERB_OPEN,
 	VERB_CLOSE,
 	VERB_MOVE,
@@ -47,31 +51,31 @@ const Verb Input::_verbKeys[8] = {
 	VERB_USE
 };
 
-Input::Input(Language language, OSystem *system) :
-	_system(system), _fastMode(false), _keyVerb(VERB_NONE),
-	_cutawayRunning(false), _canQuit(false), _cutawayQuit(false),
-	_dialogueRunning(false), _talkQuit(false), _quickSave(false),
-	_quickLoad(false), _debugger(false), _inKey(0), _mouse_x(0),
-	_mouse_y(0), _mouseButton(0), _idleTime(0) {
+Input::Input(Common::Language language, OSystem *system) :
+	_system(system), _eventMan(system->getEventManager()), _fastMode(false),
+	_keyVerb(VERB_NONE), _cutawayRunning(false), _canQuit(false),
+	_cutawayQuit(false), _dialogueRunning(false), _talkQuit(false),
+	_quickSave(false), _quickLoad(false), _debugger(false), _inKey(0),
+	_mouseButton(0), _idleTime(0) {
 
 	switch (language) {
-	case ENGLISH:
-	case RUSSIAN:
+	case Common::EN_ANY:
+	case Common::RU_RUS:
 		_currentCommandKeys = _commandKeys[0];
 		break;
-	case GERMAN:
+	case Common::DE_DEU:
 		_currentCommandKeys = _commandKeys[1];
 		break;
-	case FRENCH:
+	case Common::FR_FRA:
 		_currentCommandKeys = _commandKeys[2];
 		break;
-	case ITALIAN:
+	case Common::IT_ITA:
 		_currentCommandKeys = _commandKeys[3];
 		break;
-	case HEBREW:
+	case Common::HB_ISR:
 		_currentCommandKeys = _commandKeys[4];
 		break;
-	case SPANISH:
+	case Common::ES_ESP:
 		_currentCommandKeys = _commandKeys[5];
 		break;
 	default:
@@ -80,22 +84,21 @@ Input::Input(Language language, OSystem *system) :
 	}
 }
 
-void Input::delay() {
-	delay(_fastMode ? DELAY_SHORT : DELAY_NORMAL);
-}
-
 void Input::delay(uint amount) {
+	if (_fastMode && amount > DELAY_SHORT) {
+		amount = DELAY_SHORT;
+	}
 	if (_idleTime < DELAY_SCREEN_BLANKER) {
 		_idleTime += amount;
 	}
 	uint32 end = _system->getMillis() + amount;
 	do {
-		OSystem::Event event;
-		while (_system->pollEvent(event)) {
+		Common::Event event;
+		while (_eventMan->pollEvent(event)) {
 			_idleTime = 0;
 			switch (event.type) {
-			case OSystem::EVENT_KEYDOWN:
-				if (event.kbd.flags == OSystem::KBD_CTRL) {
+			case Common::EVENT_KEYDOWN:
+				if (event.kbd.flags == Common::KBD_CTRL) {
 					if (event.kbd.keycode == 'd') {
 						_debugger = true;
 					} else if (event.kbd.keycode == 'f') {
@@ -106,24 +109,15 @@ void Input::delay(uint amount) {
 				}
 				break;
 
-			case OSystem::EVENT_MOUSEMOVE:
-				_mouse_x = event.mouse.x;
-				_mouse_y = event.mouse.y;
-				break;
-
-			case OSystem::EVENT_LBUTTONDOWN:
+			case Common::EVENT_LBUTTONDOWN:
 				_mouseButton |= MOUSE_LBUTTON;
-				_mouse_x = event.mouse.x;
-				_mouse_y = event.mouse.y;
 				break;
 
-			case OSystem::EVENT_RBUTTONDOWN:
+			case Common::EVENT_RBUTTONDOWN:
 				_mouseButton |= MOUSE_RBUTTON;
-				_mouse_x = event.mouse.x;
-				_mouse_y = event.mouse.y;
 				break;
 
-			case OSystem::EVENT_QUIT:
+			case Common::EVENT_QUIT:
 				_system->quit();
 				break;
 
@@ -168,7 +162,7 @@ int Input::checkKeys() {
 	case KEY_DIGIT_4:
 		_keyVerb = VERB_DIGIT_4;
 		break;
-	case KEY_ESCAPE: // slip cutaway / dialogue
+	case KEY_ESCAPE: // skip cutaway / dialogue
 		if (_canQuit) {
 			if (_cutawayRunning) {
 				debug(6, "[Input::checkKeys] Setting _cutawayQuit to true");
@@ -212,5 +206,8 @@ int Input::checkKeys() {
 	return inKey;
 }
 
+Common::Point Input::getMousePos() const {
+	return _eventMan->getMousePos();
+}
 
 } // End of namespace Queen
