@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2003-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,19 +18,24 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/queen/input.h $
- * $Id: input.h 20516 2006-02-11 10:07:12Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/queen/input.h $
+ * $Id: input.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
-#ifndef INPUT_H
-#define INPUT_H
+#ifndef QUEEN_INPUT_H
+#define QUEEN_INPUT_H
 
 #include "common/util.h"
+#include "common/rect.h"
 #include "queen/defs.h"
 
 class OSystem;
 
+namespace Common {
+	class EventManager;
+}
+
 namespace Queen {
 
 class Input {
@@ -36,7 +44,7 @@ public:
 	//! Adjust here to change delays!
 	enum {
 		DELAY_SHORT  =  10,
-		DELAY_NORMAL = 100,
+		DELAY_NORMAL = 100, // 5 * 20ms
 		DELAY_SCREEN_BLANKER = 5 * 60 * 1000
 	};
 	enum {
@@ -44,12 +52,8 @@ public:
 		MOUSE_RBUTTON = 2
 	};
 
-	Input(Language language, OSystem *system);
+	Input(Common::Language language, OSystem *system);
 
-	//! calls the other delay() with a value adjusted depending on _fastMode
-	void delay();
-
-	//! moved QueenEngine::delay() here
 	void delay(uint amount);
 
 	//! convert input to verb
@@ -83,8 +87,7 @@ public:
 
 	Verb keyVerb() const { return _keyVerb; }
 
-	int mousePosX() const { return _mouse_x; }
-	int mousePosY() const { return _mouse_y; }
+	Common::Point getMousePos() const;
 
 	int mouseButton() const { return _mouseButton; }
 	void clearMouseButton() { _mouseButton = 0; }
@@ -114,13 +117,11 @@ private:
 		KEY_F12
 	};
 
-	enum {
-		LANGUAGE_COUNT = 6
-	};
-
 	//! used to get keyboard and mouse events
 	OSystem *_system;
 
+	Common::EventManager *_eventMan;
+
 	//! some cutaways require update() run faster
 	bool _fastMode;
 
@@ -154,9 +155,6 @@ private:
 	//! set by delay();
 	int _inKey;
 
-	//! set by delay();
-	int _mouse_x, _mouse_y;
-
 	//! set by delay();
 	int _mouseButton;
 
@@ -167,10 +165,10 @@ private:
 	const char *_currentCommandKeys;
 
 	//! command keys for all languages
-	static const char *_commandKeys[LANGUAGE_COUNT];
+	static const char *_commandKeys[];
 
 	//! verbs matching the command keys
-	static const Verb _verbKeys[8];
+	static const Verb _verbKeys[];
 };
 
 } // End of namespace Queen
