@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2003-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,12 +18,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/queen/journal.cpp $
- * $Id: journal.cpp 22456 2006-05-13 17:30:04Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/queen/journal.cpp $
+ * $Id: journal.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
 #include "common/stdafx.h"
+#include "common/events.h"
 #include "common/system.h"
 #include "queen/journal.h"
 
@@ -64,22 +68,23 @@ void Journal::use() {
 
 	_quitMode = QM_LOOP;
 	while (_quitMode == QM_LOOP) {
-		OSystem::Event event;
-		while (_system->pollEvent(event)) {
+		Common::Event event;
+		Common::EventManager *eventMan = _system->getEventManager();
+		while (eventMan->pollEvent(event)) {
 			switch (event.type) {
-			case OSystem::EVENT_KEYDOWN:
+			case Common::EVENT_KEYDOWN:
 				handleKeyDown(event.kbd.ascii, event.kbd.keycode);
 				break;
-			case OSystem::EVENT_LBUTTONDOWN:
+			case Common::EVENT_LBUTTONDOWN:
 				handleMouseDown(event.mouse.x, event.mouse.y);
 				break;
-			case OSystem::EVENT_WHEELUP:
+			case Common::EVENT_WHEELUP:
 				handleMouseWheel(-1);
 				break;
-			case OSystem::EVENT_WHEELDOWN:
+			case Common::EVENT_WHEELDOWN:
 				handleMouseWheel(1);
 				break;
-			case OSystem::EVENT_QUIT:
+			case Common::EVENT_QUIT:
 				_system->quit();
 				break;
 			default:
@@ -117,7 +122,7 @@ void Journal::setup() {
 	_vm->graphics()->clearBobs();
 	_vm->display()->clearTexts(0, GAME_SCREEN_HEIGHT - 1);
 	_vm->bankMan()->eraseFrames(false);
-	_vm->display()->textCurrentColor(INK_JOURNAL);
+	_vm->display()->textCurrentColor(_vm->display()->getInkColor(INK_JOURNAL));
 
 	_vm->grid()->clear(GS_ROOM);
 	for (int i = 0; i < MAX_ZONES; ++i) {
@@ -159,7 +164,7 @@ void Journal::update() {
 	if (_textField.enabled) {
 		int16 x = _textField.x + _textField.posCursor;
 		int16 y = _textField.y + _currentSaveSlot * _textField.h + 8;
-		_vm->display()->drawBox(x, y, x + 6, y, INK_JOURNAL);
+		_vm->display()->drawBox(x, y, x + 6, y, _vm->display()->getInkColor(INK_JOURNAL));
 	}
 	_vm->display()->forceFullRefresh();
 	_vm->display()->update();
@@ -268,7 +273,7 @@ void Journal::handleMouseDown(int x, int y) {
 				if (_saveDescriptions[currentSlot][0]) {
 					_vm->graphics()->clearBobs();
 					_vm->display()->palFadeOut(ROOM_JOURNAL);
-					_vm->music()->stopSong();
+					_vm->sound()->stopSong();
 					_vm->loadGameState(currentSlot);
 					_vm->display()->clearTexts(0, GAME_SCREEN_HEIGHT - 1);
 					_quitMode = QM_RESTORE;
@@ -319,8 +324,8 @@ void Journal::handleMouseDown(int x, int y) {
 			drawConfigPanel();
 			break;
 		case ZN_MUSIC_VOLUME:
-			val = (x - 136) * QueenEngine::MAX_MUSIC_VOLUME / (266 - 136);
-			_vm->music()->setVolume(val);
+			val = (x - 136) * Audio::Mixer::kMaxMixerVolume / (266 - 136);
+			_vm->sound()->setVolume(val);
 			drawConfigPanel();
 			break;
 		case ZN_DESC_1:
@@ -357,7 +362,7 @@ void Journal::handleMouseDown(int x, int y) {
 			if (_vm->sound()->musicOn()) {
 				_vm->sound()->playLastSong();
 			} else {
-				_vm->music()->stopSong();
+				_vm->sound()->stopSong();
 			}
 			drawConfigPanel();
 			break;
@@ -387,7 +392,7 @@ void Journal::drawPanelText(int y, const char *text) {
 		_panelTextY[_panelTextCount++] = y;
 	} else {
 		*p++ = '\0';
-		if (_vm->resource()->getLanguage() == HEBREW) {
+		if (_vm->resource()->getLanguage() == Common::HB_ISR) {
 			drawPanelText(y - 5, p);
 			drawPanelText(y + 5, s);
 		} else {
@@ -418,7 +423,11 @@ void Journal::drawPanel(const int *frames, const int *titles, int n) {
 	int y = 8;
 	while (n--) {
 		showBob(bobNum++, 32, y, *frames++);
-		drawPanelText(y + 12, _vm->logic()->joeResponse(*titles++));
+		// trim panel texts for spanish version
+		char buf[128];
+		strncpy(buf, _vm->logic()->joeResponse(*titles++), 128);
+		buf[127] = 0;
+		drawPanelText(y + 12, Common::trim(buf));
 		y += 48;
 	}
 }
@@ -447,7 +456,7 @@ void Journal::drawConfigPanel() {
 	_vm->checkOptionSettings();
 
 	drawSlideBar(_vm->talkSpeed(), QueenEngine::MAX_TEXT_SPEED, BOB_TALK_SPEED, 164, FRAME_BLUE_PIN);
-	drawSlideBar(_vm->music()->volume(), QueenEngine::MAX_MUSIC_VOLUME, BOB_MUSIC_VOLUME, 177, FRAME_GREEN_PIN);
+	drawSlideBar(_vm->sound()->getVolume(), Audio::Mixer::kMaxMixerVolume, BOB_MUSIC_VOLUME, 177, FRAME_GREEN_PIN);
 
 	drawCheckBox(_vm->sound()->sfxOn(), BOB_SFX_TOGGLE, 221, 155, FRAME_CHECK_BOX);
 	drawCheckBox(_vm->sound()->speechOn(), BOB_SPEECH_TOGGLE, 158, 155, FRAME_CHECK_BOX);
@@ -457,7 +466,7 @@ void Journal::drawConfigPanel() {
 
 void Journal::drawInfoPanel() {
 	showBob(BOB_INFO_BOX, 72, 221, FRAME_INFO_BOX);
-	const char *ver = _vm->resource()->JASVersion();
+	const char *ver = _vm->resource()->getJASVersion();
 	switch (ver[0]) {
 	case 'P':
 		_vm->display()->setTextCentered(132, "PC Hard Drive", false);
@@ -468,12 +477,6 @@ void Journal::drawInfoPanel() {
 	case 'a':
 		_vm->display()->setTextCentered(132, "Amiga A500/600", false);
 		break;
-	case 'A':
-		_vm->display()->setTextCentered(132, "Amiga A1200", false);
-		break;
-	case 'c':
-		_vm->display()->setTextCentered(132, "Amiga CD-32", false);
-		break;
 	}
 	switch (ver[1]) {
 	case 'E':
