@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2003-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,15 +18,18 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/queen/logic.cpp $
- * $Id: logic.cpp 20975 2006-02-28 22:20:13Z cyx $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/queen/logic.cpp $
+ * $Id: logic.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
 #include "common/stdafx.h"
+#include "common/config-manager.h"
+#include "common/events.h"
+#include "common/system.h"
+
 #include "queen/logic.h"
 
-#include "common/config-manager.h"
 #include "queen/bankman.h"
 #include "queen/command.h"
 #include "queen/credits.h"
@@ -44,20 +50,6 @@
 
 namespace Queen {
 
-static Common::String trim(const Common::String &s) {
-	const char *p;
-
-	p = s.c_str();
-	while (*p == ' ') ++p;
-	int start = p - s.c_str();
-
-	p = s.c_str() + s.size() - 1;
-	while (p != s.c_str() && *p == ' ') --p;
-	int end = p - s.c_str();
-
-	return Common::String(s.c_str() + start, end - start + 1);
-}
-
 Logic::Logic(QueenEngine *vm)
 	: _credits(NULL), _objectData(NULL), _roomData(NULL), _sfxName(NULL),
 	_itemData(NULL), _graphicData(NULL), _walkOffData(NULL), _objectDescription(NULL),
@@ -70,7 +62,8 @@ Logic::Logic(QueenEngine *vm)
 	_puzzleAttemptCount = 0;
 	_journal = new Journal(vm);
 	_scene = 0;
-	initialise();
+	memset(_specialMoves, 0, sizeof(_specialMoves));
+	readQueenJas();
 }
 
 Logic::~Logic() {
@@ -88,7 +81,7 @@ Logic::~Logic() {
 	delete[] _graphicAnim;
 }
 
-void Logic::initialise() {
+void Logic::readQueenJas() {
 	int16 i;
 
 	uint8 *jas = _vm->resource()->loadFile("QUEEN.JAS", 20);
@@ -112,7 +105,8 @@ void Logic::initialise() {
 	}
 	_roomData[_numRooms + 1] = _numObjects;
 
-	if (_vm->resource()->isDemo()) {
+	if ((_vm->resource()->isDemo() && _vm->resource()->getPlatform() == Common::kPlatformPC) ||
+		(_vm->resource()->isInterview() && _vm->resource()->getPlatform() == Common::kPlatformAmiga)) {
 		_sfxName = NULL;
 	} else {
 		_sfxName = new uint16[_numRooms + 1];
@@ -190,72 +184,29 @@ void Logic::initialise() {
 	_currentRoom = _objectData[_entryObj].room;
 	_entryObj = 0;
 
-	if (memcmp(ptr, _vm->resource()->JASVersion(), 5) != 0) {
+	if (memcmp(ptr, _vm->resource()->getJASVersion(), 5) != 0) {
 		warning("Unexpected queen.jas file format");
 	}
-
 	delete[] jas;
 
-	uint32 size;
-	char *buf = (char *)_vm->resource()->loadFile("QUEEN2.JAS", 0, &size);
-	LineReader queen2jas(buf, size);
-
-	_objDescription.push_back("");
-	for (i = 1; i <= _numDescriptions; i++) {
-		_objDescription.push_back(queen2jas.nextLine());
-	}
+	_vm->resource()->loadTextFile("QUEEN2.JAS", _jasStringList);
+	_jasStringOffset[0] = 0;
+	_jasStringOffset[1] = _jasStringOffset[0] + _numDescriptions;
+	_jasStringOffset[2] = _jasStringOffset[1] + _numNames;
+	_jasStringOffset[3] = _jasStringOffset[2] + _numRooms;
+	_jasStringOffset[4] = _jasStringOffset[3] + 12;
+	_jasStringOffset[5] = _jasStringOffset[4] + JOE_RESPONSE_MAX;
+	_jasStringOffset[6] = _jasStringOffset[5] + _numAAnim;
+	_jasStringOffset[7] = _jasStringOffset[6] + _numAName;
 
 	// Patch for German text bug
-	if (_vm->resource()->getLanguage() == GERMAN) {
-		_objDescription[296] = "Es bringt nicht viel, das festzubinden.";
-	}
-
-	_objName.push_back("");
-	for (i = 1; i <= _numNames; i++) {
-		_objName.push_back(queen2jas.nextLine());
-	}
-
-	_roomName.push_back("");
-	for (i = 1; i <= _numRooms; i++) {
-		_roomName.push_back(queen2jas.nextLine());
-	}
-
-	_verbName.push_back("");
-	for (i = 1; i <= 12; i++) {
-		_verbName.push_back(queen2jas.nextLine());
-	}
-
-	_joeResponse.push_back("");
-	for (i = 1; i <= JOE_RESPONSE_MAX; i++) {
-		_joeResponse.push_back(queen2jas.nextLine());
-	}
-
-	// Spanish version adds some space characters (0x20) at the beginning
-	// and the end of the journal button captions. As the engine computes
-	// the text width to center it, we need to trim those strings.
-	if (_vm->resource()->getLanguage() == SPANISH) {
-		for (i = 30; i <= 35; i++) {
-			_joeResponse[i] = trim(_joeResponse[i]);
-		}
-	}
-
-	_aAnim.push_back("");
-	for (i = 1; i <= _numAAnim; i++) {
-		_aAnim.push_back(queen2jas.nextLine());
-	}
-
-	_aName.push_back("");
-	for (i = 1; i <= _numAName; i++) {
-		_aName.push_back(queen2jas.nextLine());
-	}
-
-	_aFile.push_back("");
-	for (i = 1; i <= _numAFile; i++) {
-		_aFile.push_back(queen2jas.nextLine());
+	if (_vm->resource()->getLanguage() == Common::DE_DEU) {
+		_jasStringList[_jasStringOffset[JSO_OBJECT_DESCRIPTION] + 296 - 1] = "Es bringt nicht viel, das festzubinden.";
 	}
 }
 
 void Logic::start() {
+	setupSpecialMoveTable();
 	_vm->command()->clear(false);
 	_vm->display()->setupPanel();
 	_vm->graphics()->unpackControlBank();
@@ -268,11 +219,6 @@ void Logic::start() {
 	_newRoom = _currentRoom;
 }
 
-ObjectData* Logic::objectData(int index) const {
-	assert(index >= 0 && index <= _numObjects);
-	return &_objectData[index];
-}
-
 uint16 Logic::findBob(uint16 obj) const {
 	assert(obj <= _numObjects);
 
@@ -453,27 +399,45 @@ void Logic::gameState(int index, int16 newValue) {
 
 const char *Logic::roomName(uint16 roomNum) const {
 	assert(roomNum >= 1 && roomNum <= _numRooms);
-	return _roomName[roomNum].c_str();
+	return _jasStringList[_jasStringOffset[JSO_ROOM_NAME] + roomNum - 1].c_str();
 }
 
 const char *Logic::objectName(uint16 objNum) const {
 	assert(objNum >= 1 && objNum <= _numNames);
-	return _objName[objNum].c_str();
+	return _jasStringList[_jasStringOffset[JSO_OBJECT_NAME] + objNum - 1].c_str();
 }
 
 const char *Logic::objectTextualDescription(uint16 objNum) const {
 	assert(objNum >= 1 && objNum <= _numDescriptions);
-	return _objDescription[objNum].c_str();
+	return _jasStringList[_jasStringOffset[JSO_OBJECT_DESCRIPTION] + objNum - 1].c_str();
 }
 
 const char *Logic::joeResponse(int i) const {
 	assert(i >= 1 && i <= JOE_RESPONSE_MAX);
-	return _joeResponse[i].c_str();
+	return _jasStringList[_jasStringOffset[JSO_JOE_RESPONSE] + i - 1].c_str();
 }
 
 const char *Logic::verbName(Verb v) const {
 	assert(v >= 0 && v <= 12);
-	return _verbName[v].c_str();
+	if (v == 0) {
+		return "";
+	}
+	return _jasStringList[_jasStringOffset[JSO_VERB_NAME] + v - 1].c_str();
+}
+
+const char *Logic::actorAnim(int num) const {
+	assert(num >= 1 && num <= _numAAnim);
+	return _jasStringList[_jasStringOffset[JSO_ACTOR_ANIM] + num - 1].c_str();
+}
+
+const char *Logic::actorName(int num) const {
+	assert(num >= 1 && num <= _numAName);
+	return _jasStringList[_jasStringOffset[JSO_ACTOR_NAME] + num - 1].c_str();
+}
+
+const char *Logic::actorFile(int num) const {
+	assert(num >= 1 && num <= _numAFile);
+	return _jasStringList[_jasStringOffset[JSO_ACTOR_FILE] + num - 1].c_str();
 }
 
 void Logic::eraseRoom() {
@@ -571,7 +535,7 @@ ActorData *Logic::findActor(uint16 noun, const char *name) const {
 		for (uint16 i = 1; i <= _numActors; ++i) {
 			ActorData *pad = &_actorData[i];
 			if (pad->room == _currentRoom && gameState(pad->gsSlot) == pad->gsValue) {
-				if (bobNum == pad->bobNum || (name && _aName[pad->name] == name)) {
+				if (bobNum == pad->bobNum || (name && strcmp(actorName(pad->name), name) == 0)) {
 					return pad;
 				}
 			}
@@ -580,18 +544,18 @@ ActorData *Logic::findActor(uint16 noun, const char *name) const {
 	return NULL;
 }
 
-bool Logic::initPerson(uint16 noun, const char *actorName, bool loadBank, Person *pp) {
-	const ActorData *pad = findActor(noun, actorName);
+bool Logic::initPerson(uint16 noun, const char *name, bool loadBank, Person *pp) {
+	const ActorData *pad = findActor(noun, name);
 	if (pad != NULL) {
 		pp->actor = pad;
-		pp->name = _aName[pad->name].c_str();
+		pp->name = actorName(pad->name);
 		if (pad->anim != 0) {
-			pp->anim = _aAnim[pad->anim].c_str();
+			pp->anim = actorAnim(pad->anim);
 		} else {
 			pp->anim = NULL;
 		}
 		if (loadBank && pad->file != 0) {
-			_vm->bankMan()->load(_aFile[pad->file].c_str(), pad->bankNum);
+			_vm->bankMan()->load(actorFile(pad->file), pad->bankNum);
 			// if there is no valid actor file (ie pad->file is 0), the person
 			// data is already loaded as it is included in objects room bank (.bbk)
 		}
@@ -625,7 +589,7 @@ void Logic::loadJoeBanks(const char *animBank, const char *standBank) {
 }
 
 void Logic::setupJoe() {
-	loadJoeBanks("joe_a.BBK", "joe_b.BBK");
+	loadJoeBanks("JOE_A.BBK", "JOE_B.BBK");
 	joePrevFacing(DIR_FRONT);
 	joeFacing(DIR_FRONT);
 }
@@ -697,7 +661,7 @@ void Logic::setupJoeInRoom(bool autoPosition, uint16 scale) {
 
 	if (_currentRoom == 108) {
 		_vm->graphics()->putCameraOnBob(-1);
-		_vm->bankMan()->load("joe_e.act", 7);
+		_vm->bankMan()->load("JOE_E.ACT", 7);
 		_vm->bankMan()->unpack(2, 31, 7);
 
 		_vm->display()->horizontalScroll(320);
@@ -817,14 +781,14 @@ void Logic::joeUseDress(bool showCut) {
 		joeFacing(DIR_FRONT);
 		joeFace();
 		if (gameState(VAR_JOE_DRESSING_MODE) == 0) {
-			playCutaway("cdres.CUT");
+			playCutaway("CDRES.CUT");
 			inventoryInsertItem(ITEM_CLOTHES);
 		} else {
-			playCutaway("cudrs.CUT");
+			playCutaway("CUDRS.CUT");
 		}
 	}
 	_vm->display()->palSetJoeDress();
-	loadJoeBanks("JoeD_A.BBK", "JoeD_B.BBK");
+	loadJoeBanks("JOED_A.BBK", "JOED_B.BBK");
 	inventoryDeleteItem(ITEM_DRESS);
 	gameState(VAR_JOE_DRESSING_MODE, 2);
 }
@@ -833,18 +797,18 @@ void Logic::joeUseClothes(bool showCut) {
 	if (showCut) {
 		joeFacing(DIR_FRONT);
 		joeFace();
-		playCutaway("cdclo.CUT");
+		playCutaway("CDCLO.CUT");
 		inventoryInsertItem(ITEM_DRESS);
 	}
 	_vm->display()->palSetJoeNormal();
-	loadJoeBanks("Joe_A.BBK", "Joe_B.BBK");
+	loadJoeBanks("JOE_A.BBK", "JOE_B.BBK");
 	inventoryDeleteItem(ITEM_CLOTHES);
 	gameState(VAR_JOE_DRESSING_MODE, 0);
 }
 
 void Logic::joeUseUnderwear() {
 	_vm->display()->palSetJoeNormal();
-	loadJoeBanks("JoeU_A.BBK", "JoeU_B.BBK");
+	loadJoeBanks("JOEU_A.BBK", "JOEU_B.BBK");
 	gameState(VAR_JOE_DRESSING_MODE, 1);
 }
 
@@ -883,7 +847,7 @@ void Logic::playCutaway(const char *cutFile, char *next) {
 }
 
 void Logic::makeJoeSpeak(uint16 descNum, bool objectType) {
-	const char *text = objectType ? _objDescription[descNum].c_str() : _joeResponse[descNum].c_str();
+	const char *text = objectType ? objectTextualDescription(descNum) : joeResponse(descNum);
 	if (objectType) {
 		descNum += JOE_RESPONSE_MAX;
 	}
@@ -901,7 +865,7 @@ uint16 Logic::findInventoryItem(int invSlot) const {
 }
 
 void Logic::inventorySetup() {
-	_vm->bankMan()->load("objects.BBK", 14);
+	_vm->bankMan()->load("OBJECTS.BBK", 14);
 	if (_vm->resource()->isInterview()) {
 		_inventoryItem[0] = 1;
 		_inventoryItem[1] = 2;
@@ -920,9 +884,7 @@ void Logic::inventoryRefresh() {
 	for (int i = 0; i < 4; ++i) {
 		uint16 itemNum = _inventoryItem[i];
 		if (itemNum != 0) {
-			// 1st object in inventory uses frame 9,
-			// whereas 2nd, 3rd and 4th uses frame 8
-			uint16 dstFrame = (itemNum != 0) ? 8 : 9;
+			uint16 dstFrame = (i == 0) ? 8 : 9;
 			// unpack frame for object and draw it
 			_vm->bankMan()->unpack(_itemData[itemNum].frame, dstFrame, 14);
 			_vm->graphics()->drawInventoryItem(dstFrame, x, 14);
@@ -1107,53 +1069,53 @@ void Logic::handleSpecialArea(Direction facing, uint16 areaNum, uint16 walkDataN
 		makeJoeSpeak(16);
 		break;
 	case ROOM_JUNGLE_GORILLA_1:
-		playCutaway("c6c.CUT", nextCut);
+		playCutaway("C6C.CUT", nextCut);
 		break;
 	case ROOM_JUNGLE_GORILLA_2:
-		playCutaway("c14b.CUT", nextCut);
+		playCutaway("C14B.CUT", nextCut);
 		break;
 	case ROOM_AMAZON_ENTRANCE:
 		if (areaNum == 3) {
-			playCutaway("c16a.CUT", nextCut);
+			playCutaway("C16A.CUT", nextCut);
 		}
 		break;
 	case ROOM_AMAZON_HIDEOUT:
 		if (walkDataNum == 4) {
-			playCutaway("c17a.CUT", nextCut);
+			playCutaway("C17A.CUT", nextCut);
 		} else if (walkDataNum == 2) {
-			playCutaway("c17b.CUT", nextCut);
+			playCutaway("C17B.CUT", nextCut);
 		}
 		break;
 	case ROOM_FLODA_OUTSIDE:
-		playCutaway("c22a.CUT", nextCut);
+		playCutaway("C22A.CUT", nextCut);
 		break;
 	case ROOM_FLODA_KITCHEN:
-		playCutaway("c26b.CUT", nextCut);
+		playCutaway("C26B.CUT", nextCut);
 		break;
 	case ROOM_FLODA_KLUNK:
-		playCutaway("c30a.CUT", nextCut);
+		playCutaway("C30A.CUT", nextCut);
 		break;
 	case ROOM_FLODA_HENRY:
-		playCutaway("c32c.CUT", nextCut);
+		playCutaway("C32C.CUT", nextCut);
 		break;
 	case ROOM_TEMPLE_ZOMBIES:
 		if (areaNum == 6) {
 			switch (gameState(VAR_BYPASS_ZOMBIES)) {
 			case 0:
-				playCutaway("c50d.CUT", nextCut);
+				playCutaway("C50D.CUT", nextCut);
 				while (nextCut[0] != '\0') {
 					playCutaway(nextCut, nextCut);
 				}
 				gameState(VAR_BYPASS_ZOMBIES, 1);
 				break;
 			case 1:
-				playCutaway("c50h.CUT", nextCut);
+				playCutaway("C50H.CUT", nextCut);
 				break;
 			}
 		}
 		break;
 	case ROOM_TEMPLE_SNAKE:
-		playCutaway("c53b.CUT", nextCut);
+		playCutaway("C53B.CUT", nextCut);
 		break;
 	case ROOM_TEMPLE_LIZARD_LASER:
 		makeJoeSpeak(19);
@@ -1164,17 +1126,17 @@ void Logic::handleSpecialArea(Direction facing, uint16 areaNum, uint16 walkDataN
 	case ROOM_HOTEL_LOBBY:
 		switch (gameState(VAR_HOTEL_ESCAPE_STATE)) {
 		case 0:
-			playCutaway("c73a.CUT");
+			playCutaway("C73A.CUT");
 			joeUseUnderwear();
 			joeFace();
 			gameState(VAR_HOTEL_ESCAPE_STATE, 1);
 			break;
 		case 1:
-			playCutaway("c73b.CUT");
+			playCutaway("C73B.CUT");
 			gameState(VAR_HOTEL_ESCAPE_STATE, 2);
 			break;
 		case 2:
-			playCutaway("c73c.CUT");
+			playCutaway("C73C.CUT");
 			break;
 		}
 		break;
@@ -1185,18 +1147,18 @@ void Logic::handleSpecialArea(Direction facing, uint16 areaNum, uint16 walkDataN
 		break;
 	case ROOM_TEMPLE_MAZE_6:
 		if (areaNum == 5 && gameState(187) == 0) {
-			playCutaway("c101b.CUT", nextCut);
+			playCutaway("C101B.CUT", nextCut);
 		}
 		break;
 	case ROOM_FLODA_FRONTDESK:
 		if (areaNum == 3) {
 			switch (gameState(VAR_BYPASS_FLODA_RECEPTIONIST)) {
 			case 0:
-				playCutaway("c103b.CUT", nextCut);
+				playCutaway("C103B.CUT", nextCut);
 				gameState(VAR_BYPASS_FLODA_RECEPTIONIST, 1);
 				break;
 			case 1:
-				playCutaway("c103e.CUT", nextCut);
+				playCutaway("C103E.CUT", nextCut);
 				break;
 			}
 		}
@@ -1204,7 +1166,7 @@ void Logic::handleSpecialArea(Direction facing, uint16 areaNum, uint16 walkDataN
 	}
 
 	while (strlen(nextCut) > 4 &&
-		scumm_stricmp(nextCut + strlen(nextCut) - 4, ".cut") == 0) {
+		scumm_stricmp(nextCut + strlen(nextCut) - 4, ".CUT") == 0) {
 		playCutaway(nextCut, nextCut);
 	}
 }
@@ -1218,10 +1180,11 @@ void Logic::handlePinnacleRoom() {
 	BobSlot *piton = _vm->graphics()->bob(7);
 
 	// set scrolling value to mouse position to avoid glitch
-	_vm->display()->horizontalScroll(_vm->input()->mousePosX());
+	Common::Point mouse = _vm->input()->getMousePos();
+	_vm->display()->horizontalScroll(mouse.x);
 
-	joe->x = piton->x = 3 * _vm->input()->mousePosX() / 4 + 200;
-	joe->frameNum = _vm->input()->mousePosX() / 36 + 45;
+	joe->x = piton->x = 3 * mouse.x / 4 + 200;
+	joe->frameNum = mouse.x / 36 + 45;
 
 	// bobs have been unpacked from animating objects, we don't need them
 	// to animate anymore ; so turn animation off
@@ -1232,24 +1195,23 @@ void Logic::handlePinnacleRoom() {
 
 	_entryObj = 0;
 	uint16 prevObj = 0;
-	CmdText cmdText((_vm->resource()->getLanguage() == HEBREW), 5, _vm);
+	CmdText cmdText(5, _vm);
 	cmdText.setVerb(VERB_WALK_TO);
 	while (_vm->input()->mouseButton() == 0 || _entryObj == 0) {
 
 		_vm->update();
-		int mx = _vm->input()->mousePosX();
-		int my = _vm->input()->mousePosY();
+		mouse = _vm->input()->getMousePos();
 
 		// update screen scrolling
-		_vm->display()->horizontalScroll(mx);
+		_vm->display()->horizontalScroll(mouse.x);
 
 		// update bobs position / frame
-		joe->x = piton->x = 3 * mx / 4 + 200;
-		joe->frameNum = mx / 36 + 45;
+		joe->x = piton->x = 3 * mouse.x / 4 + 200;
+		joe->frameNum = mouse.x / 36 + 45;
 
 		_vm->display()->clearTexts(5, 5);
 
-		uint16 curObj = _vm->grid()->findObjectUnderCursor(mx, my);
+		uint16 curObj = _vm->grid()->findObjectUnderCursor(mouse.x, mouse.y);
 		if (curObj != 0 && curObj != prevObj) {
 			_entryObj = 0;
 			curObj += currentRoomData(); // global object number
@@ -1391,15 +1353,15 @@ void Logic::setupRestoredGame() {
 	switch (gameState(VAR_JOE_DRESSING_MODE)) {
 	case 0:
 		_vm->display()->palSetJoeNormal();
-		loadJoeBanks("Joe_A.BBK", "Joe_B.BBK");
+		loadJoeBanks("JOE_A.BBK", "JOE_B.BBK");
 		break;
 	case 1:
 		_vm->display()->palSetJoeNormal();
-		loadJoeBanks("JoeU_A.BBK", "JoeU_B.BBK");
+		loadJoeBanks("JOEU_A.BBK", "JOEU_B.BBK");
 		break;
 	case 2:
 		_vm->display()->palSetJoeDress();
-		loadJoeBanks("JoeD_A.BBK", "JoeD_B.BBK");
+		loadJoeBanks("JOED_A.BBK", "JOED_B.BBK");
 		break;
 	}
 
@@ -1459,15 +1421,16 @@ void Logic::sceneStop() {
 }
 
 void Logic::changeRoom() {
-	if (!preChangeRoom())
+	if (!changeToSpecialRoom())
 		displayRoom(currentRoom(), RDM_FADE_JOE, 100, 1, false);
 	_vm->display()->showMouseCursor(true);
 }
 
 void Logic::executeSpecialMove(uint16 sm) {
 	debug(6, "Special move: %d", sm);
-	if (!handleSpecialMove(sm))
-		warning("unhandled / invalid special move : %d", sm);
+	if (sm < ARRAYSIZE(_specialMoves) && _specialMoves[sm] != 0) {
+		(this->*_specialMoves[sm])();
+	}
 }
 
 void Logic::asmMakeJoeUseDress() {
@@ -1518,14 +1481,22 @@ void Logic::asmMakeFrankGrowing() {
 	_vm->bankMan()->unpack(1, 38, 15);
 	BobSlot *bobFrank = _vm->graphics()->bob(5);
 	bobFrank->frameNum = 38;
-	bobFrank->curPos(160, 200);
-
-	int i;
-	for (i = 10; i <= 100; i += 4) {
-		bobFrank->scale = i;
-		_vm->update();
+	if (_vm->resource()->getPlatform() == Common::kPlatformAmiga) {
+		bobFrank->active = true;
+		bobFrank->x = 160;
+		bobFrank->scale = 100;
+		for (int i = 350; i >= 200; i -= 5) {
+			bobFrank->y = i;
+			_vm->update();
+		}
+	} else {
+		bobFrank->curPos(160, 200);
+		for (int i = 10; i <= 100; i += 4) {
+			bobFrank->scale = i;
+			_vm->update();
+		}
 	}
-	for (i = 0; i <= 20; ++i) {
+	for (int i = 0; i <= 20; ++i) {
 		_vm->update();
 	}
 
@@ -1541,14 +1512,22 @@ void Logic::asmMakeRobotGrowing() {
 	_vm->bankMan()->unpack(1, 38, 15);
 	BobSlot *bobRobot = _vm->graphics()->bob(5);
 	bobRobot->frameNum = 38;
-	bobRobot->curPos(160, 200);
-
-	int i;
-	for (i = 10; i <= 100; i += 4) {
-		bobRobot->scale = i;
-		_vm->update();
+	if (_vm->resource()->getPlatform() == Common::kPlatformAmiga) {
+		bobRobot->active = true;
+		bobRobot->x = 160;
+		bobRobot->scale = 100;
+		for (int i = 350; i >= 200; i -= 5) {
+			bobRobot->y = i;
+			_vm->update();
+		}
+	} else {
+		bobRobot->curPos(160, 200);
+		for (int i = 10; i <= 100; i += 4) {
+			bobRobot->scale = i;
+			_vm->update();
+		}
 	}
-	for (i = 0; i <= 20; ++i) {
+	for (int i = 0; i <= 20; ++i) {
 		_vm->update();
 	}
 
@@ -1570,7 +1549,7 @@ void Logic::asmEndGame() {
 	while (n--) {
 		_vm->update();
 	}
-	debug(0, "Game completed.");
+//	printf("Game completed.");
 	_vm->quitGame();
 }
 
@@ -1721,6 +1700,19 @@ void Logic::asmSmooch() {
 	}
 }
 
+void Logic::asmSmoochNoScroll() {
+	_vm->graphics()->putCameraOnBob(-1);
+	BobSlot *bobAzura = _vm->graphics()->bob(5);
+	BobSlot *bobJoe = _vm->graphics()->bob(6);
+	for (int i = 0; i < 320; i += 8) {
+		if (bobJoe->x - bobAzura->x > 128) {
+			bobAzura->x += 2;
+			bobJoe->x -= 2;
+		}
+		_vm->update();
+	}
+}
+
 void Logic::asmMakeLightningHitPlane() {
 	_vm->graphics()->putCameraOnBob(-1);
 	short iy = 0, x, ydir = -1, j, k;
@@ -1730,7 +1722,11 @@ void Logic::asmMakeLightningHitPlane() {
 
 	planeBob->y = 135;
 
-	planeBob->scale = 20;
+	if (_vm->resource()->getPlatform() == Common::kPlatformAmiga) {
+		planeBob->scale = 100;
+	} else {
+		planeBob->scale = 20;
+	}
 
 	for (x = 660; x > 163; x -= 6) {
 		planeBob->x = x;
@@ -1760,7 +1756,7 @@ void Logic::asmMakeLightningHitPlane() {
 	lightningBob->x = 160;
 	lightningBob->y = 0;
 
-	_vm->sound()->playSfx(currentRoomSfx(), false);
+	_vm->sound()->playSfx(currentRoomSfx());
 
 	_vm->bankMan()->unpack(18, lightningBob->frameNum, 15);
 	_vm->bankMan()->unpack(4,  planeBob    ->frameNum, 15);
@@ -1804,11 +1800,13 @@ void Logic::asmScaleBlimp() {
 	BobSlot *bob = _vm->graphics()->bob(7);
 	int16 x = bob->x;
 	int16 y = bob->y;
+	bob->scale = 100;
 	while (bob->x > 150) {
 		bob->x = x * 256 / z + 150;
 		bob->y = y * 256 / z + 112;
-		bob->scale = 100 * 256 / z;
-
+		if (_vm->resource()->getPlatform() != Common::kPlatformAmiga) {
+			bob->scale = 100 * 256 / z;
+		}
 		++z;
 		if (z % 6 == 0) {
 			--x;
@@ -1821,14 +1819,14 @@ void Logic::asmScaleBlimp() {
 void Logic::asmScaleEnding() {
 	_vm->graphics()->bob(7)->active = false; // Turn off blimp
 	BobSlot *b = _vm->graphics()->bob(20);
-	b->x = 160;
-	b->y = 100;
-	int i;
-	for (i = 5; i <= 100; i += 5) {
-		b->scale = i;
-		_vm->update();
+	b->curPos(160, 100);
+	if (_vm->resource()->getPlatform() != Common::kPlatformAmiga) {
+		for (int i = 5; i <= 100; i += 5) {
+			b->scale = i;
+			_vm->update();
+		}
 	}
-	for (i = 0; i < 50; ++i) {
+	for (int i = 0; i < 50; ++i) {
 		_vm->update();
 	}
 	_vm->display()->palFadeOut(_currentRoom);
@@ -1871,6 +1869,18 @@ void Logic::asmScaleTitle() {
 	}
 }
 
+void Logic::asmScrollTitle() {
+	BobSlot *bob = _vm->graphics()->bob(5);
+	bob->animating = false;
+	bob->x = 161;
+	bob->y = 300;
+	bob->scale = 100;
+	while (bob->y >= 120) {
+		_vm->update();
+		bob->y -= 4;
+	}
+}
+
 void Logic::asmPanRightToHugh() {
 	BobSlot *bob_thugA1 = _vm->graphics()->bob(20);
 	BobSlot *bob_thugA2 = _vm->graphics()->bob(21);
@@ -1993,7 +2003,7 @@ void Logic::asmPanLeftToBomb() {
 }
 
 void Logic::asmEndDemo() {
-	debug(0, "Flight of the Amazon Queen, released January 95.");
+//	printf("Flight of the Amazon Queen, released January 95.");
 	_vm->quitGame();
 }
 
@@ -2038,7 +2048,7 @@ void Logic::asmInterviewIntro() {
 }
 
 void Logic::asmEndInterview() {
-	debug(0, "Interactive Interview copyright (c) 1995, IBI.");
+//	printf("Interactive Interview copyright (c) 1995, IBI.");
 	_vm->quitGame();
 }
 
@@ -2059,16 +2069,16 @@ void LogicDemo::useJournal() {
 	makePersonSpeak("This is a demo, so I can't load or save games*14", NULL, "");
 }
 
-bool LogicDemo::preChangeRoom() {
+bool LogicDemo::changeToSpecialRoom() {
 	if (currentRoom() == FOTAQ_LOGO && gameState(VAR_INTRO_PLAYED) == 0) {
 		currentRoom(79);
 		displayRoom(currentRoom(), RDM_FADE_NOJOE, 100, 2, true);
-		playCutaway("clogo.cut");
+		playCutaway("CLOGO.CUT");
 		sceneReset();
 		currentRoom(ROOM_HOTEL_LOBBY);
 		entryObj(584);
 		displayRoom(currentRoom(), RDM_FADE_JOE, 100, 2, true);
-		playCutaway("c70d.cut");
+		playCutaway("C70D.CUT");
 		gameState(VAR_INTRO_PLAYED, 1);
 		inventoryRefresh();
 		return true;
@@ -2076,32 +2086,23 @@ bool LogicDemo::preChangeRoom() {
 	return false;
 }
 
-bool LogicDemo::handleSpecialMove(uint16 sm) {
-	switch (sm) {
-	case 4:
-		asmMakeJoeUseUnderwear();
-		break;
-	case 5:
-		asmSwitchToDressPalette();
-		break;
-	case 14:
-		asmEndDemo();
-		break;
-	default:
-		return false;
+void LogicDemo::setupSpecialMoveTable() {
+	_specialMoves[4] = &LogicDemo::asmMakeJoeUseUnderwear;
+	_specialMoves[14] = &LogicDemo::asmEndDemo;
+	if (_vm->resource()->getPlatform() == Common::kPlatformPC) {
+		_specialMoves[5]  = &LogicDemo::asmSwitchToDressPalette;
 	}
-	return true;
 }
 
 void LogicInterview::useJournal() {
 	// no-op
 }
 
-bool LogicInterview::preChangeRoom() {
+bool LogicInterview::changeToSpecialRoom() {
 	if (currentRoom() == 2 && gameState(2) == 0) {
 		currentRoom(6);
 		displayRoom(currentRoom(), RDM_FADE_NOJOE, 100, 2, true);
-		playCutaway("start.cut");
+		playCutaway("START.CUT");
 		gameState(2, 1);
 		inventoryRefresh();
 		return true;
@@ -2109,18 +2110,9 @@ bool LogicInterview::preChangeRoom() {
 	return false;
 }
 
-bool LogicInterview::handleSpecialMove(uint16 sm) {
-	switch (sm) {
-	case 1:
-		asmInterviewIntro();
-		break;
-	case 2:
-		asmEndInterview();
-		break;
-	default:
-		return false;
-	}
-	return true;
+void LogicInterview::setupSpecialMoveTable() {
+	_specialMoves[1] = &LogicInterview::asmInterviewIntro;
+	_specialMoves[2] = &LogicInterview::asmEndInterview;
 }
 
 void LogicGame::useJournal() {
@@ -2129,28 +2121,28 @@ void LogicGame::useJournal() {
 	_vm->walk()->stopJoe();
 }
 
-bool LogicGame::preChangeRoom() {
+bool LogicGame::changeToSpecialRoom() {
 	if (currentRoom() == ROOM_JUNGLE_PINNACLE) {
 		handlePinnacleRoom();
 		return true;
 	} else if (currentRoom() == FOTAQ_LOGO && gameState(VAR_INTRO_PLAYED) == 0) {
 		displayRoom(currentRoom(), RDM_FADE_NOJOE, 100, 2, true);
-		playCutaway("copy.cut");
-		playCutaway("clogo.cut");
-
-		if (ConfMan.getBool("alt_intro") && _vm->resource()->isCD()) {
-			playCutaway("cintr.cut");
-		} else {
-			playCutaway("cdint.cut");
+		playCutaway("COPY.CUT");
+		playCutaway("CLOGO.CUT");
+		if (_vm->resource()->getPlatform() != Common::kPlatformAmiga) {
+			if (ConfMan.getBool("alt_intro") && _vm->resource()->isCD()) {
+				playCutaway("CINTR.CUT");
+			} else {
+				playCutaway("CDINT.CUT");
+			}
 		}
-
-		playCutaway("cred.cut");
+		playCutaway("CRED.CUT");
 		_vm->display()->palSetPanel();
 		sceneReset();
 		currentRoom(ROOM_HOTEL_LOBBY);
 		entryObj(584);
 		displayRoom(currentRoom(), RDM_FADE_JOE, 100, 2, true);
-		playCutaway("c70d.cut");
+		playCutaway("C70D.CUT");
 		gameState(VAR_INTRO_PLAYED, 1);
 		inventoryRefresh();
 		return true;
@@ -2158,64 +2150,48 @@ bool LogicGame::preChangeRoom() {
 	return false;
 }
 
-bool LogicGame::handleSpecialMove(uint16 sm) {
-	typedef void (LogicGame::*SpecialMoveProc)();
-	static const SpecialMoveProc asmTable[] = {
-		/* 00 */
-		0,
-		0,
-		&LogicGame::asmMakeJoeUseDress,
-		&LogicGame::asmMakeJoeUseNormalClothes,
-		/* 04 */
-		&LogicGame::asmMakeJoeUseUnderwear,
-		&LogicGame::asmSwitchToDressPalette,
-		&LogicGame::asmSwitchToNormalPalette,
-		&LogicGame::asmStartCarAnimation,       // room 74
-		/* 08 */
-		&LogicGame::asmStopCarAnimation,        // room 74
-		&LogicGame::asmStartFightAnimation,     // room 69
-		&LogicGame::asmWaitForFrankPosition,    // c69e.cut
-		&LogicGame::asmMakeFrankGrowing,        // c69z.cut
-		/* 12 */
-		&LogicGame::asmMakeRobotGrowing,        // c69z.cut
-		&LogicGame::asmShrinkRobot,
-		&LogicGame::asmEndGame,
-		&LogicGame::asmPutCameraOnDino,
-		/* 16 */
-		&LogicGame::asmPutCameraOnJoe,
-		&LogicGame::asmAltIntroPanRight,        // cintr.cut
-		&LogicGame::asmAltIntroPanLeft,         // cintr.cut
-		&LogicGame::asmSetAzuraInLove,
-		/* 20 */
-		&LogicGame::asmPanRightFromJoe,
-		&LogicGame::asmSetLightsOff,
-		&LogicGame::asmSetLightsOn,
-		&LogicGame::asmSetManequinAreaOn,
-		/* 24 */
-		&LogicGame::asmPanToJoe,
-		&LogicGame::asmTurnGuardOn,
-		&LogicGame::asmPanLeft320To144,
-		&LogicGame::asmSmooch,
-		/* 28 */
-		&LogicGame::asmMakeLightningHitPlane,
-		&LogicGame::asmScaleBlimp,
-		&LogicGame::asmScaleEnding,
-		&LogicGame::asmWaitForCarPosition,
-		/* 32 */
-		&LogicGame::asmShakeScreen,
-		&LogicGame::asmAttemptPuzzle,
-		&LogicGame::asmScaleTitle,
-		0,
-		/* 36 */
-		&LogicGame::asmPanRightToHugh,
-		&LogicGame::asmMakeWhiteFlash,
-		&LogicGame::asmPanRightToJoeAndRita,
-		&LogicGame::asmPanLeftToBomb            // cdint.cut
-	};
-	if (sm >= ARRAYSIZE(asmTable) || asmTable[sm] == 0)
-		return false;
-	(this->*asmTable[sm])();
-	return true;
+void LogicGame::setupSpecialMoveTable() {
+	_specialMoves[2] = &LogicGame::asmMakeJoeUseDress;
+	_specialMoves[3] = &LogicGame::asmMakeJoeUseNormalClothes;
+	_specialMoves[4] = &LogicGame::asmMakeJoeUseUnderwear;
+	_specialMoves[7] = &LogicGame::asmStartCarAnimation;       // room 74
+	_specialMoves[8] = &LogicGame::asmStopCarAnimation;        // room 74
+	_specialMoves[9] = &LogicGame::asmStartFightAnimation;     // room 69
+	_specialMoves[10] = &LogicGame::asmWaitForFrankPosition;   // c69e.cut
+	_specialMoves[11] = &LogicGame::asmMakeFrankGrowing;       // c69z.cut
+	_specialMoves[12] = &LogicGame::asmMakeRobotGrowing;       // c69z.cut
+	_specialMoves[14] = &LogicGame::asmEndGame;
+	_specialMoves[15] = &LogicGame::asmPutCameraOnDino;
+	_specialMoves[16] = &LogicGame::asmPutCameraOnJoe;
+	_specialMoves[19] = &LogicGame::asmSetAzuraInLove;
+	_specialMoves[20] = &LogicGame::asmPanRightFromJoe;
+	_specialMoves[21] = &LogicGame::asmSetLightsOff;
+	_specialMoves[22] = &LogicGame::asmSetLightsOn;
+	_specialMoves[23] = &LogicGame::asmSetManequinAreaOn;
+	_specialMoves[24] = &LogicGame::asmPanToJoe;
+	_specialMoves[25] = &LogicGame::asmTurnGuardOn;
+	_specialMoves[26] = &LogicGame::asmPanLeft320To144;
+	_specialMoves[27] = &LogicGame::asmSmoochNoScroll;
+	_specialMoves[28] = &LogicGame::asmMakeLightningHitPlane;
+	_specialMoves[29] = &LogicGame::asmScaleBlimp;
+	_specialMoves[30] = &LogicGame::asmScaleEnding;
+	_specialMoves[31] = &LogicGame::asmWaitForCarPosition;
+	_specialMoves[33] = &LogicGame::asmAttemptPuzzle;
+	_specialMoves[34] = &LogicGame::asmScrollTitle;
+	if (_vm->resource()->getPlatform() == Common::kPlatformPC) {
+		_specialMoves[5]  = &LogicGame::asmSwitchToDressPalette;
+		_specialMoves[6]  = &LogicGame::asmSwitchToNormalPalette;
+		_specialMoves[13] = &LogicGame::asmShrinkRobot;
+		_specialMoves[17] = &LogicGame::asmAltIntroPanRight;      // cintr.cut
+		_specialMoves[18] = &LogicGame::asmAltIntroPanLeft;       // cintr.cut
+		_specialMoves[27] = &LogicGame::asmSmooch;
+		_specialMoves[32] = &LogicGame::asmShakeScreen;
+		_specialMoves[34] = &LogicGame::asmScaleTitle;
+		_specialMoves[36] = &LogicGame::asmPanRightToHugh;
+		_specialMoves[37] = &LogicGame::asmMakeWhiteFlash;
+		_specialMoves[38] = &LogicGame::asmPanRightToJoeAndRita;
+		_specialMoves[39] = &LogicGame::asmPanLeftToBomb;         // cdint.cut
+	}
 }
 
 } // End of namespace Queen
