@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2003-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,13 +18,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/queen/logic.h $
- * $Id: logic.h 20516 2006-02-11 10:07:12Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/queen/logic.h $
+ * $Id: logic.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
-#ifndef QUEENLOGIC_H
-#define QUEENLOGIC_H
+#ifndef QUEEN_LOGIC_H
+#define QUEEN_LOGIC_H
 
 #include "common/str.h"
 #include "common/util.h"
@@ -43,14 +46,25 @@ enum JoeWalkMode {
 	JWM_SPEAK   = 3
 };
 
+enum {
+	JSO_OBJECT_DESCRIPTION = 0,
+	JSO_OBJECT_NAME,
+	JSO_ROOM_NAME,
+	JSO_VERB_NAME,
+	JSO_JOE_RESPONSE,
+	JSO_ACTOR_ANIM,
+	JSO_ACTOR_NAME,
+	JSO_ACTOR_FILE,
+	JSO_COUNT
+};
+
 class Credits;
 class Journal;
-class LineReader;
 class QueenEngine;
 
 class Logic {
-
 public:
+
 	Logic(QueenEngine *vm);
 	virtual ~Logic();
 
@@ -72,7 +86,7 @@ public:
 		_newRoom = room;
 	}
 
-	ObjectData *objectData(int index) const;
+	ObjectData *objectData(int index) const { return &_objectData[index]; }
 	uint16 roomData(int room) const { return _roomData[room]; }
 	GraphicData *graphicData(int index) const { return &_graphicData[index]; }
 	ItemData *itemData(int index) const { return &_itemData[index]; }
@@ -117,6 +131,9 @@ public:
 	const char *objectTextualDescription(uint16 objNum) const;
 	const char *joeResponse(int i) const;
 	const char *verbName(Verb v) const;
+	const char *actorAnim(int num) const;
+	const char *actorName(int num) const;
+	const char *actorFile(int num) const;
 
 	void eraseRoom();
 	void setupRoom(const char *room, int comPanel, bool inCutaway);
@@ -222,9 +239,11 @@ public:
 		TALK_SELECTED_COUNT = 86
 	};
 
+	typedef void (Logic::*SpecialMoveProc)();
+
 protected:
 
-	void initialise();
+	void readQueenJas();
 
 	void asmMakeJoeUseDress();
 	void asmMakeJoeUseNormalClothes();
@@ -252,6 +271,7 @@ protected:
 	void asmTurnGuardOn();
 	void asmPanLeft320To144();
 	void asmSmooch();
+	void asmSmoochNoScroll();
 	void asmMakeLightningHitPlane();
 	void asmScaleBlimp();
 	void asmScaleEnding();
@@ -259,6 +279,7 @@ protected:
 	void asmShakeScreen();
 	void asmAttemptPuzzle();
 	void asmScaleTitle();
+	void asmScrollTitle();
 	void asmPanRightToHugh();
 	void asmMakeWhiteFlash();
 	void asmPanRightToJoeAndRita();
@@ -267,8 +288,8 @@ protected:
 	void asmInterviewIntro();
 	void asmEndInterview();
 
-	virtual bool preChangeRoom() = 0;
-	virtual bool handleSpecialMove(uint16 sm) = 0;
+	virtual bool changeToSpecialRoom() = 0;
+	virtual void setupSpecialMoveTable() = 0;
 
 
 	uint16 _currentRoom;
@@ -316,30 +337,13 @@ protected:
 	//! actor initial position in room is _walkOffData[_entryObj]
 	int16 _entryObj;
 
-	//! object description (Look At)
-	Common::StringList _objDescription;
+	Common::StringList _jasStringList;
+	int _jasStringOffset[JSO_COUNT];
+
 	uint16 _numDescriptions;
-
-	Common::StringList _objName;
 	uint16 _numNames;
-
-	//! room name, prefix for data files (PCX, LUM...)
-	Common::StringList _roomName;
-
-	Common::StringList _verbName;
-
-	Common::StringList _joeResponse;
-
-	//! actor animation strings
-	Common::StringList _aAnim;
 	uint16 _numAAnim;
-
-	//! actor names
-	Common::StringList _aName;
 	uint16 _numAName;
-
-	//! actor filenames
-	Common::StringList _aFile;
 	uint16 _numAFile;
 
 	struct {
@@ -362,6 +366,8 @@ protected:
 	//! cutscene counter
 	int _scene;
 
+	SpecialMoveProc _specialMoves[40];
+
 	Credits *_credits;
 	Journal *_journal;
 
@@ -376,8 +382,8 @@ public:
 
 protected:
 
-	bool preChangeRoom();
-	bool handleSpecialMove(uint16 sm);
+	bool changeToSpecialRoom();
+	void setupSpecialMoveTable();
 };
 
 class LogicInterview : public Logic {
@@ -388,8 +394,8 @@ public:
 
 protected:
 
-	bool preChangeRoom();
-	bool handleSpecialMove(uint16 sm);
+	bool changeToSpecialRoom();
+	void setupSpecialMoveTable();
 };
 
 class LogicGame : public Logic {
@@ -400,8 +406,8 @@ public:
 
 protected:
 
-	bool preChangeRoom();
-	bool handleSpecialMove(uint16 sm);
+	bool changeToSpecialRoom();
+	void setupSpecialMoveTable();
 };
 
 
