@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2003-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,13 +18,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/queen/music.h $
- * $Id: music.h 20516 2006-02-11 10:07:12Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/queen/music.h $
+ * $Id: music.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
-#ifndef QUEENMUSIC_H
-#define QUEENMUSIC_H
+#ifndef QUEEN_MUSIC_H
+#define QUEEN_MUSIC_H
 
 #include "common/util.h"
 #include "sound/mididrv.h"
@@ -30,16 +33,20 @@ class MidiParser;
 
 namespace Queen {
 
+struct tuneData;
+
 class QueenEngine;
 
-class MusicPlayer : public MidiDriver {
+class MidiMusic : public MidiDriver {
 public:
-	MusicPlayer(MidiDriver *driver, byte *data, uint32 size);
-	~MusicPlayer();
+	MidiMusic(MidiDriver *driver, QueenEngine *vm);
+	~MidiMusic();
 	void setVolume(int volume);
 	int getVolume()			{ return _masterVolume; }
 
 	void hasNativeMT32(bool b)	{ _nativeMT32 = b; }
+	void playSong(uint16 songNum);
+	void stopSong() { stopMusic(); }
 	void playMusic();
 	void stopMusic();
 	void setLoop(bool loop)		{ _looping = loop; }
@@ -47,6 +54,7 @@ public:
 	bool queueSong(uint16 songNum);
 	void queueClear();
 	void setPassThrough(bool b)		{ _passThrough = b; }
+	void toggleVChange();
 
 	//MidiDriver interface implementation
 	int open();
@@ -93,31 +101,11 @@ protected:
 	int16 _songQueue[MUSIC_QUEUE_SIZE];
 
 	uint16 _numSongs;
-	byte *_musicData;
 	uint16 *_buf;
 	uint32 _musicDataSize;
-};
-
-class Music {
-public:
-	Music(MidiDriver *_driver, QueenEngine *vm);
-	~Music();
-	void hasNativeMT32(bool b)		{ _player->hasNativeMT32(b); }
-	void playSong(uint16 songNum);
-	void queueTuneList(int16 tuneList)	{ _player->queueTuneList(tuneList); }
-	void playMusic()			{ _player->playMusic(); }
-	void stopSong()				{ _player->stopMusic(); }
-	void setPassThrough(bool b)		{ _player->setPassThrough(b); }
-
-	void toggleVChange();
-	void setVolume(int vol)			{ _player->setVolume(vol); }
-	int volume()				{ return _player->getVolume(); }
-
-protected:
 	bool _vToggle;
 	byte *_musicData;
-	uint32 _musicDataSize;
-	MusicPlayer *_player;
+	const tuneData *_tune;
 };
 
 } // End of namespace Queen
