@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2003-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,19 +18,18 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/queen/queen.cpp $
- * $Id: queen.cpp 22456 2006-05-13 17:30:04Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/queen/queen.cpp $
+ * $Id: queen.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
 #include "common/stdafx.h"
 
-#include "backends/fs/fs.h"
-
 #include "base/plugins.h"
 
 #include "common/config-manager.h"
 #include "common/file.h"
+#include "common/fs.h"
 #include "common/savefile.h"
 #include "common/system.h"
 
@@ -41,86 +43,54 @@
 #include "queen/grid.h"
 #include "queen/input.h"
 #include "queen/logic.h"
-#include "queen/music.h"
 #include "queen/resource.h"
 #include "queen/sound.h"
 #include "queen/talk.h"
 #include "queen/walk.h"
 
-#include "sound/mididrv.h"
-
-#ifdef _WIN32_WCE
-bool isSmartphone();
-#endif
-
-/* Flight of the Amazon Queen */
-static const PlainGameDescriptor queen_setting[] = {
-	{ "queen", "Flight of the Amazon Queen" },
-	{ "queen", "Flight of the Amazon Queen (Demo)" },
-	{ "queen", "Flight of the Amazon Queen (Interview)" },
-	{ 0, 0 }
+static const PlainGameDescriptor queenGameDescriptor = {
+	"queen", "Flight of the Amazon Queen"
 };
 
 GameList Engine_QUEEN_gameIDList() {
 	GameList games;
-	games.push_back(queen_setting[0]);
+	games.push_back(queenGameDescriptor);
 	return games;
 }
 
 GameDescriptor Engine_QUEEN_findGameID(const char *gameid) {
-	if (0 == scumm_stricmp(gameid, queen_setting[0].gameid))
-		return queen_setting[0];
+	if (0 == scumm_stricmp(gameid, queenGameDescriptor.gameid)) {
+		return queenGameDescriptor;
+	}
 	return GameDescriptor();
 }
 
-
-GameDescriptor determineTarget(uint32 size) {
-	switch (size) {
-	case 3724538:	//regular demo
-	case 3732177:
-		return queen_setting[1];
-		break;
-	case 1915913:   //interview demo
-		return queen_setting[2];
-		break;
-	default:	//non-demo
-		return queen_setting[0];
-		break;
-	}
-	return queen_setting[0];
-}
-
-DetectedGameList Engine_QUEEN_detectGames(const FSList &fslist) {
-	DetectedGameList detectedGames;
+GameList Engine_QUEEN_detectGames(const FSList &fslist) {
+	GameList detectedGames;
 
 	// Iterate over all files in the given directory
 	for (FSList::const_iterator file = fslist.begin(); file != fslist.end(); ++file) {
-		if (!file->isDirectory()) {
-			const char *gameName = file->displayName().c_str();
-
-			if (0 == scumm_stricmp("queen.1", gameName) || 0 == scumm_stricmp("queen.1c", gameName)) {
-				Common::File dataFile;
-				dataFile.open(file->path());
-				assert(dataFile.isOpen());
-
-				if (0 == scumm_stricmp("queen.1", gameName)) {	//an unmodified file
-					detectedGames.push_back(determineTarget(dataFile.size()));
-				} else if (0 == scumm_stricmp("queen.1c", gameName)) { //oh joy, it's a rebuilt file
-					char header[9];
-					dataFile.read(header, 9);
-					if (0 == scumm_strnicmp("QTBL", header, 4)) { //check validity
-						uint8 version = 0;	//default to full/normal version
-
-						if (0 == scumm_strnicmp("PE100", header + 4, 5)) //One of the 2 regular demos
-							version = 1;
-						if (0 == scumm_strnicmp("PEint", header + 4, 5)) //Interview demo
-							version = 2;
-
-						detectedGames.push_back(queen_setting[version]);
-					}
+		if (file->isDirectory()) {
+			continue;
+		}
+		if (file->name().equalsIgnoreCase("queen.1") || file->name().equalsIgnoreCase("queen.1c")) {
+			Common::File dataFile;
+			if (!dataFile.open(*file)) {
+				continue;
+			}
+			Queen::DetectedGameVersion version;
+			if (Queen::Resource::detectVersion(&version, &dataFile)) {
+				GameDescriptor dg(queenGameDescriptor.gameid, queenGameDescriptor.description, version.language, version.platform);
+				if (version.features & Queen::GF_DEMO) {
+					dg.updateDesc("Demo");
+				} else if (version.features & Queen::GF_INTERVIEW) {
+					dg.updateDesc("Interview");
+				} else if (version.features & Queen::GF_FLOPPY) {
+					dg.updateDesc("Floppy");
+				} else if (version.features & Queen::GF_TALKIE) {
+					dg.updateDesc("Talkie");
 				}
-
-				dataFile.close();
+				detectedGames.push_back(dg);
 				break;
 			}
 		}
@@ -134,7 +104,7 @@ PluginError Engine_QUEEN_create(OSystem *syst, Engine **engine) {
 	return kNoError;
 }
 
-REGISTER_PLUGIN(QUEEN, "Flight of the Amazon Queen");
+REGISTER_PLUGIN(QUEEN, "Flight of the Amazon Queen", "Flight of the Amazon Queen (C) John Passfield and Steve Stamatiadis");
 
 namespace Queen {
 
@@ -153,7 +123,6 @@ QueenEngine::~QueenEngine() {
 	delete _grid;
 	delete _input;
 	delete _logic;
-	delete _music;
 	delete _sound;
 	delete _walk;
 }
@@ -161,6 +130,7 @@ QueenEngine::~QueenEngine() {
 void QueenEngine::registerDefaultSettings() {
 	ConfMan.registerDefault("talkspeed", Logic::DEFAULT_TALK_SPEED);
 	ConfMan.registerDefault("subtitles", true);
+	_subtitles = true;
 }
 
 void QueenEngine::checkOptionSettings() {
@@ -171,29 +141,29 @@ void QueenEngine::checkOptionSettings() {
 		_talkSpeed = MAX_TEXT_SPEED;
 	}
 
-	// ensure text is always on when voice is off
-	if (!_sound->speechOn()) {
-		_subtitles = true;
-	}
-
 	// demo and interview versions don't have speech at all
 	if (_sound->speechOn() && (_resource->isDemo() || _resource->isInterview())) {
 		_sound->speechToggle(false);
 	}
+
+	// ensure text is always on when voice is off
+	if (!_sound->speechOn()) {
+		_subtitles = true;
+	}
 }
 
 void QueenEngine::readOptionSettings() {
-	_music->setVolume(ConfMan.getInt("music_volume"));
+	_sound->setVolume(ConfMan.getInt("music_volume"));
 	_sound->musicToggle(!ConfMan.getBool("music_mute"));
 	_sound->sfxToggle(!ConfMan.getBool("sfx_mute"));
-	_talkSpeed = (ConfMan.getInt("talkspeed") * (MAX_TEXT_SPEED - MIN_TEXT_SPEED) + 255 / 2) / 255 + MIN_TEXT_SPEED;
 	_sound->speechToggle(!ConfMan.getBool("speech_mute"));
+	_talkSpeed = (ConfMan.getInt("talkspeed") * (MAX_TEXT_SPEED - MIN_TEXT_SPEED) + 255 / 2) / 255 + MIN_TEXT_SPEED;
 	_subtitles = ConfMan.getBool("subtitles");
 	checkOptionSettings();
 }
 
 void QueenEngine::writeOptionSettings() {
-	ConfMan.setInt("music_volume", _music->volume());
+	ConfMan.setInt("music_volume", _sound->getVolume());
 	ConfMan.setBool("music_mute", !_sound->musicOn());
 	ConfMan.setBool("sfx_mute", !_sound->sfxOn());
 	ConfMan.setInt("talkspeed", ((_talkSpeed - MIN_TEXT_SPEED) * 255 + (MAX_TEXT_SPEED - MIN_TEXT_SPEED) / 2) / (MAX_TEXT_SPEED - MIN_TEXT_SPEED));
@@ -210,7 +180,12 @@ void QueenEngine::update(bool checkPlayerInput) {
 	_graphics->update(_logic->currentRoom());
 	_logic->update();
 
-	_input->delay();
+	int frameDelay = (_lastUpdateTime + Input::DELAY_NORMAL - _system->getMillis());
+	if (frameDelay <= 0) {
+		frameDelay = 1;
+	}
+	_input->delay(frameDelay);
+	_lastUpdateTime = _system->getMillis();
 
 	if (!_resource->isInterview()) {
 		_display->palCustomScroll(_logic->currentRoom());
@@ -245,6 +220,7 @@ void QueenEngine::update(bool checkPlayerInput) {
 			_display->blankScreen();
 		}
 	}
+	_sound->updateMusic();
 }
 
 bool QueenEngine::canLoadOrSave() const {
@@ -268,18 +244,18 @@ void QueenEngine::saveGameState(uint16 slot, const char *desc) {
 		assert(dataSize < SAVESTATE_MAX_SIZE);
 
 		// write header
-		GameStateHeader header;
-		memset(&header, 0, sizeof(header));
 		file->writeUint32BE('SCVM');
-		header.version = TO_BE_32(SAVESTATE_CUR_VER);
-		header.flags = TO_BE_32(0);
-		header.dataSize = TO_BE_32(dataSize);
-		strncpy(header.description, desc, sizeof(header.description) - 1);
-		file->write(&header, sizeof(header));
+		file->writeUint32BE(SAVESTATE_CUR_VER);
+		file->writeUint32BE(0);
+		file->writeUint32BE(dataSize);
+		char description[32];
+		memset(description, 0, 32);
+		strncpy(description, desc, 31);
+		file->write(description, 32);
 
 		// write save data
 		file->write(saveData, dataSize);
-		file->flush();
+		file->finalize();
 
 		// check for errors
 		if (file->ioFailed()) {
@@ -321,7 +297,7 @@ Common::InSaveFile *QueenEngine::readGameStateHeader(uint16 slot, GameStateHeade
 	char name[20];
 	makeGameStateName(slot, name);
 	Common::InSaveFile *file = _saveFileMan->openForLoading(name);
-	if (file && file->readUint32BE() == 'SCVM') {
+	if (file && file->readUint32BE() == MKID_BE('SCVM')) {
 		gsh->version = file->readUint32BE();
 		gsh->flags = file->readUint32BE();
 		gsh->dataSize = file->readUint32BE();
@@ -356,22 +332,8 @@ void QueenEngine::findGameStateDescriptions(char descriptions[100][32]) {
 	}
 }
 
-void QueenEngine::errorString(const char *buf1, char *buf2) {
-	strcpy(buf2, buf1);
-
-#ifdef _WIN32_WCE
-	if (isSmartphone())
-		return;
-#endif
-
-	// Unless an error -originated- within the debugger, spawn the
-	// debugger. Otherwise exit out normally.
-	if (_debugger && !_debugger->isAttached()) {
-		// (Print it again in case debugger segfaults)
-		printf("%s\n", buf2);
-		_debugger->attach(buf2);
-		_debugger->onFrame();
-	}
+GUI::Debugger *QueenEngine::getDebugger() {
+	return _debugger;
 }
 
 int QueenEngine::go() {
@@ -379,7 +341,7 @@ int QueenEngine::go() {
 	if (ConfMan.hasKey("save_slot") && canLoadOrSave()) {
 		loadGameState(ConfMan.getInt("save_slot"));
 	}
-	_lastSaveTime = _system->getMillis();
+	_lastSaveTime = _lastUpdateTime = _system->getMillis();
 	_quit = false;
 	while (!_quit) {
 		if (_logic->newRoom() > 0) {
@@ -408,8 +370,9 @@ int QueenEngine::init() {
 		_system->initSize(GAME_SCREEN_WIDTH, GAME_SCREEN_HEIGHT);
 	_system->endGFXTransaction();
 
-	_bam = new BamScene(this);
 	_resource = new Resource();
+
+	_bam = new BamScene(this);
 	_bankMan = new BankManager(_resource);
 	_command = new Command(this);
 	_debugger = new Debugger(this);
@@ -432,17 +395,7 @@ int QueenEngine::init() {
 	// Set mixer music volume to maximum, since music volume is regulated by MusicPlayer's MIDI messages
 	_mixer->setVolumeForSoundType(Audio::Mixer::kMusicSoundType, Audio::Mixer::kMaxMixerVolume);
 
-	int midiDriver = MidiDriver::detectMusicDriver(MDT_MIDI | MDT_ADLIB | MDT_PREFER_MIDI);
-	bool native_mt32 = ((midiDriver == MD_MT32) || ConfMan.getBool("native_mt32"));
-
-	MidiDriver *driver = MidiDriver::createMidi(midiDriver);
-	if (native_mt32)
-		driver->property(MidiDriver::PROP_CHANNEL_MASK, 0x03FE);
-
-	_music = new Music(driver, this);
-	_music->hasNativeMT32(native_mt32);
-
-	_sound = Sound::giveSound(_mixer, this, _resource->compression());
+	_sound = Sound::makeSoundInstance(_mixer, this, _resource->getCompression());
 	_walk = new Walk(this);
 	//_talkspeedScale = (MAX_TEXT_SPEED - MIN_TEXT_SPEED) / 255.0;
 
