@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2003-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,15 +18,15 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/queen/queen.h $
- * $Id: queen.h 23018 2006-06-10 22:26:46Z knakos $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/queen/queen.h $
+ * $Id: queen.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
 #ifndef QUEEN_H
 #define QUEEN_H
 
-#include "base/engine.h"
+#include "engines/engine.h"
 
 namespace Common {
 	class InSaveFile;
@@ -49,20 +52,12 @@ FORCEINLINE int16 READ_BE_INT16(const void *ptr) {
 
 namespace Queen {
 
-#if !defined(__GNUC__)
-	#pragma START_PACK_STRUCTS
-#endif
-
 struct GameStateHeader {
 	uint32 version;
 	uint32 flags;
 	uint32 dataSize;
 	char description[32];
-} GCC_PACK;
-
-#if !defined(__GNUC__)
-	#pragma END_PACK_STRUCTS
-#endif
+};
 
 class BamScene;
 class BankManager;
@@ -73,7 +68,6 @@ class Graphics;
 class Grid;
 class Input;
 class Logic;
-class Music;
 class Resource;
 class Sound;
 class Walk;
@@ -93,7 +87,6 @@ public:
 	Grid *grid() const { return _grid; }
 	Input *input() const { return _input; }
 	Logic *logic() const { return _logic; }
-	Music *music() const { return _music; }
 	Resource *resource() const { return _resource; }
 	Sound *sound() const { return _sound; }
 	Walk *walk() const { return _walk; }
@@ -128,13 +121,12 @@ public:
 		AUTOSAVE_SLOT      = 0xFF,
 
 		MIN_TEXT_SPEED     = 4,
-		MAX_TEXT_SPEED     = 100,
-		MAX_MUSIC_VOLUME   = 255
+		MAX_TEXT_SPEED     = 100
 	};
 
 protected:
 
-	void errorString(const char *buf_input, char *buf_output);
+	GUI::Debugger *getDebugger();
 
 	int go();
 	int init();
@@ -144,6 +136,7 @@ protected:
 	bool _subtitles;
 	bool _quit;
 	uint32 _lastSaveTime;
+	uint32 _lastUpdateTime;
 
 	BamScene *_bam;
 	BankManager *_bankMan;
@@ -154,7 +147,6 @@ protected:
 	Grid *_grid;
 	Input *_input;
 	Logic *_logic;
-	Music *_music;
 	Resource *_resource;
 	Sound *_sound;
 	Walk *_walk;
