@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2003-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,22 +18,24 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/queen/sound.h $
- * $Id: sound.h 20516 2006-02-11 10:07:12Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/queen/sound.h $
+ * $Id: sound.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
-#ifndef QUEENSOUND_H
-#define QUEENSOUND_H
+#ifndef QUEEN_SOUND_H
+#define QUEEN_SOUND_H
 
 #include "common/util.h"
 #include "sound/mixer.h"
+#include "sound/mods/rjp1.h"
 #include "queen/defs.h"
 
-namespace Queen {
+namespace Common {
+	class File;
+}
 
-class Input;
-class Resource;
+namespace Queen {
 
 struct songData {
 	int16 tuneList[5];
@@ -48,35 +53,48 @@ struct tuneData {
 	int16 delay;
 };
 
+class MidiMusic;
 class QueenEngine;
 
 class Sound {
 public:
 	Sound(Audio::Mixer *mixer, QueenEngine *vm);
-	virtual ~Sound();
-	virtual bool sfxPlay(const char *name, Audio::SoundHandle *soundHandle) = 0;
-	static Sound *giveSound(Audio::Mixer *mixer, QueenEngine *vm, uint8 compression);
-	void playSfx(uint16 sfx, bool isSpeech);
-	void playSfx(const char *base, bool isSpeech);
-	void playSong(int16 songNum);
+	virtual ~Sound() {}
+
+	/**
+	 * Factory method for subclasses of class Sound.
+	 */
+	static Sound *makeSoundInstance(Audio::Mixer *mixer, QueenEngine *vm, uint8 compression);
+
+	virtual void playSfx(uint16 sfx) {}
+	virtual void playSong(int16 songNum) {}
+	virtual void playSpeech(const char *base) {}
+
+	virtual void stopSfx() {}
+	virtual void stopSong() {}
+	virtual void stopSpeech() {}
+
+	virtual bool isSpeechActive() const	{ return false; }
+	virtual bool isSfxActive() const { return false; }
+
+	virtual void updateMusic() {}
+
+	virtual void setVolume(int vol);
+	virtual int getVolume()				{ return _musicVolume; }
+
 	void playLastSong()		{ playSong(_lastOverride); }
-	void stopSpeech()		{ _mixer->stopHandle(_speechHandle); }
-	void stopSfx()			{ _mixer->stopHandle(_sfxHandle); }
 
 	bool sfxOn() const			{ return _sfxToggle; }
 	void sfxToggle(bool val)	{ _sfxToggle = val; }
-	void toggleSfx()			{ _sfxToggle ^= true; }
+	void toggleSfx()			{ _sfxToggle = !_sfxToggle; }
 
 	bool speechOn()	const		{ return _speechToggle; }
 	void speechToggle(bool val)	{ _speechToggle = val; }
-	void toggleSpeech()			{ _speechToggle ^= true; }
+	void toggleSpeech()			{ _speechToggle = !_speechToggle; }
 
 	bool musicOn() const		{ return _musicToggle; }
 	void musicToggle(bool val)	{ _musicToggle = val; }
-	void toggleMusic()			{ _musicToggle ^= true; }
-
-	bool isSpeechActive() const	{ return _mixer->isSoundHandleActive(_speechHandle); }
-	bool isSfxActive() const 	{ return _mixer->isSoundHandleActive(_sfxHandle); }
+	void toggleMusic()			{ _musicToggle = !_musicToggle; }
 
 	bool speechSfxExists() const	{ return _speechSfxExists; }
 
@@ -102,7 +120,6 @@ public:
 #endif
 
 protected:
-	void waitFinished(bool isSpeech);
 
 	Audio::Mixer *_mixer;
 	QueenEngine *_vm;
@@ -113,45 +130,64 @@ protected:
 	bool _speechSfxExists;
 
 	int16 _lastOverride;
+	int _musicVolume;
+};
+
+class PCSound : public Sound {
+public:
+	PCSound(Audio::Mixer *mixer, QueenEngine *vm);
+	~PCSound();
+
+	void playSfx(uint16 sfx);
+	void playSpeech(const char *base);
+	void playSong(int16 songNum);
+
+	void stopSfx()			{ _mixer->stopHandle(_sfxHandle); }
+	void stopSong();
+	void stopSpeech()		{ _mixer->stopHandle(_speechHandle); }
+
+	bool isSpeechActive() const	{ return _mixer->isSoundHandleActive(_speechHandle); }
+	bool isSfxActive() const 	{ return _mixer->isSoundHandleActive(_sfxHandle); }
+
+	void setVolume(int vol);
+
+protected:
+	void waitFinished(bool isSpeech);
+	void playSound(const char *base, bool isSpeech);
+
+	virtual void playSoundData(Common::File *f, uint32 size, Audio::SoundHandle *soundHandle) = 0;
+
 	Audio::SoundHandle _sfxHandle;
 	Audio::SoundHandle _speechHandle;
+	MidiMusic *_music;
 };
 
-class SilentSound : public Sound {
+class AmigaSound : public Sound {
 public:
-	SilentSound(Audio::Mixer *mixer, QueenEngine *vm) : Sound(mixer, vm) {};
-	bool sfxPlay(const char *name, Audio::SoundHandle *soundHandle);
-};
+	AmigaSound(Audio::Mixer *mixer, QueenEngine *vm);
 
-class SBSound : public Sound {
-public:
-	SBSound(Audio::Mixer *mixer, QueenEngine *vm) : Sound(mixer, vm) {};
-	bool sfxPlay(const char *name, Audio::SoundHandle *soundHandle);
-};
+	void playSfx(uint16 sfx);
+	void playSong(int16 song);
 
-#ifdef USE_MAD
-class MP3Sound : public Sound {
-public:
-	MP3Sound(Audio::Mixer *mixer, QueenEngine *vm) : Sound(mixer, vm) {};
-	bool sfxPlay(const char *name, Audio::SoundHandle *soundHandle);
-};
-#endif
+	void stopSfx();
+	void stopSong();
 
-#ifdef USE_VORBIS
-class OGGSound : public Sound {
-public:
-	OGGSound(Audio::Mixer *mixer, QueenEngine *vm) : Sound(mixer, vm) {};
-	bool sfxPlay(const char *name, Audio::SoundHandle *soundHandle);
-};
-#endif
+	bool isSfxActive() const { return _mixer->isSoundHandleActive(_sfxHandle); }
 
-#ifdef USE_FLAC
-class FLACSound : public Sound {
-public:
-	FLACSound(Audio::Mixer *mixer, QueenEngine *vm) : Sound(mixer, vm) {};
-	bool sfxPlay(const char *name, Audio::SoundHandle *soundHandle);
+	void updateMusic();
+
+protected:
+
+	void playSound(const char *base);
+	void playModule(const char *base, int song);
+	void playRandomPatternJungle();
+	bool playSpecialSfx(int16 sfx);
+
+	int16 _fanfareRestore;
+	int _fanfareCount, _fluteCount;
+	Audio::SoundHandle _modHandle;
+	Audio::SoundHandle _sfxHandle;
 };
-#endif // #ifdef USE_FLAC
 
 } // End of namespace Queen
 
