@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2003-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,13 +18,16 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/queen/talk.cpp $
- * $Id: talk.cpp 23992 2006-09-26 06:09:29Z eriktorbjorn $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/queen/talk.cpp $
+ * $Id: talk.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
 #include "common/stdafx.h"
+#include "common/events.h"
+#include "common/system.h"
 #include "common/rect.h"
+
 #include "queen/talk.h"
 
 #include "queen/bankman.h"
@@ -348,15 +354,15 @@ void Talk::findDialogueString(uint16 offset, int16 id, int16 max, char *str) {
 byte *Talk::loadDialogFile(const char *filename) {
 	static const struct {
 		const char *filename;
-		Language lang;
+		Common::Language language;
 	} dogFiles[] = {
-		{ "chief1.dog", FRENCH  },
-		{ "chief2.dog", FRENCH  },
-		{ "bud1.dog",   ITALIAN }
+		{ "CHIEF1.DOG", Common::FR_FRA },
+		{ "CHIEF2.DOG", Common::FR_FRA },
+		{ "BUD1.DOG",   Common::IT_ITA }
 	};
 	for (int i = 0; i < ARRAYSIZE(dogFiles); ++i) {
 		if (!scumm_stricmp(filename, dogFiles[i].filename) &&
-			_vm->resource()->getLanguage() == dogFiles[i].lang) {
+			_vm->resource()->getLanguage() == dogFiles[i].language) {
 			Common::File fdog;
 			fdog.open(filename);
 			if (fdog.isOpen()) {
@@ -677,7 +683,7 @@ void Talk::stringAnimation(const SpeechParameters *parameters, int startFrame, i
 
 		if (frame > 500) {
 			frame -= 500;
-			_vm->sound()->playSfx(_vm->logic()->currentRoomSfx(), false);
+			_vm->sound()->playSfx(_vm->logic()->currentRoomSfx());
 		}
 
 		if (torso) {
@@ -798,9 +804,8 @@ void Talk::speakSegment(
 
 	// French talkie version has a useless voice file ;	c30e_102 file is the same as c30e_101,
 	// so there is no need to play it. This voice was used in room 30 (N8) when talking to Klunk.
-	if (!(_vm->resource()->getLanguage() == FRENCH && !strcmp(voiceFileName, "c30e_102"))
-		&& _vm->sound()->speechOn())
-		_vm->sound()->playSfx(voiceFileName, true);
+	if (!(_vm->resource()->getLanguage() == Common::FR_FRA && !strcmp(voiceFileName, "c30e_102")))
+		_vm->sound()->playSpeech(voiceFileName);
 
 	int faceDirectionCommand = 0;
 
@@ -853,11 +858,22 @@ void Talk::speakSegment(
 		// talk.c lines 1491-1533
 		switch (_vm->logic()->currentRoom()) {
 		case FAYE_HEAD:
+			textX = 15;
+			if (_vm->resource()->getPlatform() == Common::kPlatformAmiga) {
+				color = isJoe ? 15 : 29;
+			}
+			break;
 		case AZURA_HEAD:
 			textX = 15;
+			if (_vm->resource()->getPlatform() == Common::kPlatformAmiga) {
+				color = isJoe ? 6 : 30;
+			}
 			break;
 		default: // FRANK_HEAD
 			textX = 150;
+			if (_vm->resource()->getPlatform() == Common::kPlatformAmiga) {
+				color = 17;
+			}
 			break;
 		}
 		textY = isJoe ? 30 : 60;
@@ -871,10 +887,11 @@ void Talk::speakSegment(
 	BobFrame *pbf = _vm->bankMan()->fetchFrame(bob->frameNum);
 
 	int height = (pbf->height * bob->scale) / 100;
-	
+
 	Common::Rect focus(textX - 96, textY - height - 64, textX + 96, textY + height + 64);
 	_vm->display()->setFocusRect(focus);
-	
+
+
 	//int SF = _vm->grid()->findScale(textX, textY);
 
 	const SpeechParameters *parameters = NULL;
@@ -1029,7 +1046,6 @@ const Talk::SpeechParameters *Talk::findSpeechParameters(
 	const SpeechParameters *iterator = _speechParameters;
 	if (faceDirection == DIR_RIGHT)
 		faceDirection = DIR_LEFT;
-
 	while (iterator->name[0] != '*') {
 		if (0 == scumm_stricmp(iterator->name, name) &&
 				iterator->state == state &&
@@ -1037,7 +1053,6 @@ const Talk::SpeechParameters *Talk::findSpeechParameters(
 			break;
 		iterator++;
 	}
-
 	return iterator;
 }
 
@@ -1075,10 +1090,10 @@ int Talk::splitOption(const char *str, char optionText[5][MAX_STRING_SIZE]) {
 	}
 	int lines;
 	memset(optionText, 0, 5 * MAX_STRING_SIZE);
-	if (_vm->resource()->getLanguage() == ENGLISH || _vm->display()->textWidth(option) <= MAX_TEXT_WIDTH) {
+	if (_vm->resource()->getLanguage() == Common::EN_ANY || _vm->display()->textWidth(option) <= MAX_TEXT_WIDTH) {
 		strcpy(optionText[0], option);
 		lines = 1;
-	} else if (_vm->resource()->getLanguage() == HEBREW) {
+	} else if (_vm->resource()->getLanguage() == Common::HB_ISR) {
 		lines = splitOptionHebrew(option, optionText);
 	} else {
 		lines = splitOptionDefault(option, optionText);
@@ -1178,7 +1193,7 @@ int16 Talk::selectSentence() {
 	int talkZone[5];
 	int i;
 
-	_vm->display()->textCurrentColor(INK_TALK_NORMAL);
+	_vm->display()->textCurrentColor(_vm->display()->getInkColor(INK_TALK_NORMAL));
 
 	_vm->graphics()->setupArrows();
 	BobSlot *arrowBobUp   = _vm->graphics()->bob(Graphics::ARROW_BOB_UP);
@@ -1195,7 +1210,7 @@ int16 Talk::selectSentence() {
 
 		_vm->grid()->clear(GS_PANEL);
 
-		if (_vm->resource()->getLanguage() != ENGLISH) {
+		if (_vm->resource()->getLanguage() != Common::EN_ANY) {
 			_vm->grid()->setZone(GS_PANEL, ARROW_ZONE_UP,   MAX_TEXT_WIDTH + 1, 0,  319, 24);
 			_vm->grid()->setZone(GS_PANEL, ARROW_ZONE_DOWN, MAX_TEXT_WIDTH + 1, 25, 319, 49);
 		}
@@ -1218,7 +1233,7 @@ int16 Talk::selectSentence() {
 							i,
 							0,
 							yOffset * LINE_HEIGHT - PUSHUP,
-							(_vm->resource()->getLanguage() == ENGLISH) ? 319 : MAX_TEXT_WIDTH,
+							(_vm->resource()->getLanguage() == Common::EN_ANY) ? 319 : MAX_TEXT_WIDTH,
 							(yOffset + optionLines) * LINE_HEIGHT - PUSHUP);
 				}
 
@@ -1241,7 +1256,7 @@ int16 Talk::selectSentence() {
 
 		// Up and down dialogue arrows
 
-		if (_vm->resource()->getLanguage() != ENGLISH) {
+		if (_vm->resource()->getLanguage() != Common::EN_ANY) {
 			arrowBobUp->active    = (startOption > 1);
 			arrowBobDown->active  = (yOffset > 4);
 		}
@@ -1259,7 +1274,8 @@ int16 Talk::selectSentence() {
 
 				_vm->update();
 
-				zone = _vm->grid()->findZoneForPos(GS_PANEL, _vm->input()->mousePosX(), _vm->input()->mousePosY());
+				Common::Point mouse = _vm->input()->getMousePos();
+				zone = _vm->grid()->findZoneForPos(GS_PANEL, mouse.x, mouse.y);
 
 				int mouseButton = _vm->input()->mouseButton();
 				_vm->input()->clearMouseButton();
@@ -1269,7 +1285,7 @@ int16 Talk::selectSentence() {
 						int16 y;
 						const Box *b = _vm->grid()->zone(GS_PANEL, oldZone);
 						for (y = b->y1; y < b->y2; y += 10)
-							_vm->display()->textColor(150 + y, INK_TALK_NORMAL);
+							_vm->display()->textColor(150 + y, _vm->display()->getInkColor(INK_TALK_NORMAL));
 						oldZone = 0;
 					}
 					if (mouseButton != 0) {
@@ -1292,13 +1308,13 @@ int16 Talk::selectSentence() {
 						if (zone > 0) {
 							const Box *b = _vm->grid()->zone(GS_PANEL, zone);
 							for (y = b->y1; y < b->y2; y += 10)
-								_vm->display()->textColor(150 + y, INK_JOE);
+								_vm->display()->textColor(150 + y, _vm->display()->getInkColor(INK_JOE));
 						}
 
 						if (oldZone > 0) {
 							const Box *b = _vm->grid()->zone(GS_PANEL, oldZone);
 							for (y = b->y1; y < b->y2; y += 10)
-								_vm->display()->textColor(150 + y, INK_TALK_NORMAL);
+								_vm->display()->textColor(150 + y, _vm->display()->getInkColor(INK_TALK_NORMAL));
 						}
 
 						oldZone = zone;
@@ -1317,8 +1333,7 @@ int16 Talk::selectSentence() {
 					}
 
 					_vm->input()->clearKeyVerb();
-				}
-				else if (mouseButton) {
+				} else if (mouseButton) {
 					selectedSentence = zone;
 				}
 
