@@ -1,7 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2004-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
  *
- * The ReInherit Engine is (C)2000-2003 by Daniel Balsom.
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -17,15 +18,15 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/saga/actor.h $
- * $Id: actor.h 20532 2006-02-11 12:44:16Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/saga/actor.h $
+ * $Id: actor.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
 // Actor management module header file
 
-#ifndef SAGA_ACTOR_H__
-#define SAGA_ACTOR_H__
+#ifndef SAGA_ACTOR_H
+#define SAGA_ACTOR_H
 
 #include "common/savefile.h"
 
@@ -50,6 +51,8 @@ class HitZone;
 
 #define ACTOR_LMULT 4
 
+#define ACTOR_SPEED 72
+
 #define ACTOR_CLIMB_SPEED 8
 
 #define ACTOR_COLLISION_WIDTH       32
@@ -147,8 +150,7 @@ enum DragonMoveTypes {
 
 struct PathDirectionData {
 	int8 direction;
-	int16	x;
-	int16 y;
+	Point coord;	
 };
 
 struct ActorFrameRange {
@@ -622,15 +624,19 @@ private:
 			(testPoint.y < 0) || (testPoint.y >= _yCellCount));
 	}
 	void setPathCell(const Point &testPoint, int8 value) {
+#ifdef ACTOR_DEBUG
 		if (!validPathCellPoint(testPoint)) {
 			error("Actor::setPathCell wrong point");
 		}
+#endif
 		_pathCell[testPoint.x + testPoint.y * _xCellCount] = value;
 	}
 	int8 getPathCell(const Point &testPoint) {
+#ifdef ACTOR_DEBUG
 		if (!validPathCellPoint(testPoint)) {
 			error("Actor::getPathCell wrong point");
 		}
+#endif
 		return _pathCell[testPoint.x + testPoint.y * _xCellCount];
 	}
 	bool scanPathLine(const Point &point1, const Point &point2);
@@ -767,12 +773,6 @@ public:
 #endif
 };
 
-inline int16 quickDistance(const Point &point1, const Point &point2) {
-	Point delta;
-	delta.x = ABS(point1.x - point2.x) / 2;
-	delta.y = ABS(point1.y - point2.y);
-	return ((delta.x < delta.y) ? (delta.y + delta.x / 2) : (delta.x + delta.y / 2));
-}
 } // End of namespace Saga
 
 #endif
