@@ -1,7 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2004-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
  *
- * The ReInherit Engine is (C)2000-2003 by Daniel Balsom.
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -17,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/saga/console.cpp $
- * $Id: console.cpp 20532 2006-02-11 12:44:16Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/saga/console.cpp $
+ * $Id: console.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -32,26 +33,21 @@
 
 #include "saga/console.h"
 
-#include "common/debugger.cpp"
-
 namespace Saga {
 
-Console::Console(SagaEngine *vm) : Common::Debugger<Console>() {
+Console::Console(SagaEngine *vm) : GUI::Debugger() {
 	_vm = vm;
 
-	DCmd_Register("continue", &Console::Cmd_Exit);
-	DCmd_Register("exit", &Console::Cmd_Exit);
-	DCmd_Register("quit", &Console::Cmd_Exit);
-	DCmd_Register("help", &Console::Cmd_Help);
+	DCmd_Register("continue",         WRAP_METHOD(Console, Cmd_Exit));
 
 	// CVAR_Register_I(&_soundEnabled, "sound", NULL, CVAR_CFG, 0, 1);
 	// CVAR_Register_I(&_musicEnabled, "music", NULL, CVAR_CFG, 0, 1);
 
 	// Actor commands
-	DCmd_Register("actor_walk_to", &Console::cmdActorWalkTo);
+	DCmd_Register("actor_walk_to",    WRAP_METHOD(Console, cmdActorWalkTo));
 
 	// Animation commands
-	DCmd_Register("anim_info", &Console::Cmd_AnimInfo);
+	DCmd_Register("anim_info",        WRAP_METHOD(Console, Cmd_AnimInfo));
 
 	// Game stuff
 
@@ -66,63 +62,14 @@ Console::Console(SagaEngine *vm) : Common::Debugger<Console>() {
 #endif
 
 	// Scene commands
-	DCmd_Register("scene_change", &Console::cmdSceneChange);
-	DCmd_Register("action_map_info", &Console::cmdActionMapInfo);
-	DCmd_Register("object_map_info", &Console::cmdObjectMapInfo);
+	DCmd_Register("scene_change",     WRAP_METHOD(Console, cmdSceneChange));
+	DCmd_Register("action_map_info",  WRAP_METHOD(Console, cmdActionMapInfo));
+	DCmd_Register("object_map_info",  WRAP_METHOD(Console, cmdObjectMapInfo));
 }
 
 Console::~Console() {
 }
 
-void Console::preEnter() {
-}
-
-void Console::postEnter() {
-}
-
-bool Console::Cmd_Exit(int argc, const char **argv) {
-	_detach_now = true;
-	return false;
-}
-
-bool Console::Cmd_Help(int argc, const char **argv) {
-	// console normally has 39 line width
-	// wrap around nicely
-	int width = 0, size, i;
-
-	DebugPrintf("Commands are:\n");
-	for (i = 0 ; i < _dcmd_count ; i++) {
-		size = strlen(_dcmds[i].name) + 1;
-
-		if ((width + size) >= 39) {
-			DebugPrintf("\n");
-			width = size;
-		} else
-			width += size;
-
-		DebugPrintf("%s ", _dcmds[i].name);
-	}
-
-	width = 0;
-
-	DebugPrintf("\n\nVariables are:\n");
-	for (i = 0 ; i < _dvar_count ; i++) {
-		size = strlen(_dvars[i].name) + 1;
-
-		if ((width + size) >= 39) {
-			DebugPrintf("\n");
-			width = size;
-		} else
-			width += size;
-
-		DebugPrintf("%s ", _dvars[i].name);
-	}
-
-	DebugPrintf("\n");
-
-	return true;
-}
-
 bool Console::cmdActorWalkTo(int argc, const char **argv) {
 	if (argc != 4)
 		DebugPrintf("Usage: %s <Actor id> <lx> <ly>\n", argv[0]);
