@@ -1,7 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2004-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
  *
- * The ReInherit Engine is (C)2000-2003 by Daniel Balsom.
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -17,33 +18,26 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/saga/console.h $
- * $Id: console.h 20532 2006-02-11 12:44:16Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/saga/console.h $
+ * $Id: console.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
  // Console module header file
 
-#ifndef SAGA_CONSOLE_H_
-#define SAGA_CONSOLE_H_
+#ifndef SAGA_CONSOLE_H
+#define SAGA_CONSOLE_H
 
-#include "common/debugger.h"
+#include "gui/debugger.h"
 
 namespace Saga {
 
-class Console : public Common::Debugger<Console> {
+class Console : public GUI::Debugger {
 public:
 	Console(SagaEngine *vm);
 	virtual ~Console(void);
 
-protected:
-	virtual void preEnter();
-	virtual void postEnter();
-
 private:
-	bool Cmd_Exit(int argc, const char **argv);
-	bool Cmd_Help(int argc, const char **argv);
-
 	bool cmdActorWalkTo(int argc, const char **argv);
 
 	bool Cmd_AnimInfo(int argc, const char **argv);
