@@ -1,7 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2004-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
  *
- * The ReInherit Engine is (C)2000-2003 by Daniel Balsom.
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -17,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/saga/events.cpp $
- * $Id: events.cpp 21141 2006-03-08 11:37:25Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/saga/events.cpp $
+ * $Id: events.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -125,8 +126,8 @@ int Events::handleContinuous(Event *event) {
 	Surface *backGroundSurface;
 	BGInfo bgInfo;
 	Rect rect;
-	if(event->duration != 0) {
-	event_pc = ((double)event->duration - event->time) / event->duration;
+	if (event->duration != 0) {
+		event_pc = ((double)event->duration - event->time) / event->duration;
 	} else {
 		event_pc = 1.0;
 	}
@@ -207,8 +208,8 @@ int Events::handleImmediate(Event *event) {
 	bool event_done = false;
 
 	// Duration might be 0 so dont do division then
-	if(event->duration != 0) {
-	event_pc = ((double)event->duration - event->time) / event->duration;
+	if (event->duration != 0) {
+		event_pc = ((double)event->duration - event->time) / event->duration;
 	} else {
 		// Just make sure that event_pc is 1.0 so event_done is true
 		event_pc = 1.0;
@@ -364,7 +365,6 @@ int Events::handleOneShot(Event *event) {
 		case kEventEnd:
 			_vm->_scene->nextScene();
 			return kEvStBreak;
-			break;
 		default:
 			break;
 		}
@@ -438,6 +438,15 @@ int Events::handleOneShot(Event *event) {
 		case kEventHide:
 			_vm->_gfx->showCursor(false);
 			break;
+		case kEventSetNormalCursor:
+			// in ITE there is just one cursor
+			if (_vm->getGameType() == GType_IHNM)
+				_vm->_gfx->setCursor(kCursorNormal);
+			break;
+		case kEventSetBusyCursor:
+			if (_vm->getGameType() == GType_IHNM)
+				_vm->_gfx->setCursor(kCursorBusy);
+			break;
 		default:
 			break;
 		}
@@ -514,7 +523,6 @@ int Events::initializeEvent(Event *event) {
 		break;
 	default:
 		return FAILURE;
-		break;
 	}
 
 	return SUCCESS;
