@@ -1,7 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2004-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
  *
- * The ReInherit Engine is (C)2000-2003 by Daniel Balsom.
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -17,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/saga/gfx.cpp $
- * $Id: gfx.cpp 22639 2006-05-25 22:51:42Z eriktorbjorn $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/saga/gfx.cpp $
+ * $Id: gfx.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -168,7 +169,7 @@ void Surface::transitionDissolve(const byte *sourceBuffer, const Common::Rect &s
 }
 
 void Gfx::initPalette() {
-	if(_vm->getGameType() != GType_IHNM)
+	if (_vm->getGameType() != GType_IHNM)
 		return;
 
 	ResourceContext *resourceContext = _vm->_resource->getContext(GAME_RESOURCEFILE);
@@ -184,7 +185,7 @@ void Gfx::initPalette() {
 
 	MemoryReadStream metaS(resourcePointer, resourceLength);
 
-	for(int i = 0; i < 256; i++) {
+	for (int i = 0; i < 256; i++) {
 		_globalPalette[i].red = metaS.readByte();
 		_globalPalette[i].green = metaS.readByte();
 		_globalPalette[i].blue = metaS.readByte();
@@ -215,6 +216,10 @@ void Gfx::setPalette(const PalEntry *pal, bool full) {
 		ppal[3] = 0;
 	}
 
+	// Color 0 should always be black in IHNM
+	if (_vm->getGameType() == GType_IHNM)
+		memset(&_currentPal[0 * 4], 0, 4);
+
 	// Make 256th color black. See bug #1256368
 	if ((_vm->getPlatform() == Common::kPlatformMacintosh) && !_vm->_scene->isInIntro())
 		memset(&_currentPal[255 * 4], 0, 4);
@@ -320,6 +325,10 @@ void Gfx::palToBlack(PalEntry *srcPal, double percent) {
 		ppal[3] = 0;
 	}
 
+	// Color 0 should always be black in IHNM
+	if (_vm->getGameType() == GType_IHNM)
+		memset(&_currentPal[0 * 4], 0, 4);
+
 	// Make 256th color black. See bug #1256368
 	if ((_vm->getPlatform() == Common::kPlatformMacintosh) && !_vm->_scene->isInIntro())
 		memset(&_currentPal[255 * 4], 0, 4);
@@ -385,6 +394,10 @@ void Gfx::blackToPal(PalEntry *srcPal, double percent) {
 		ppal[3] = 0;
 	}
 
+	// Color 0 should always be black in IHNM
+	if (_vm->getGameType() == GType_IHNM)
+		memset(&_currentPal[0 * 4], 0, 4);
+
 	// Make 256th color black. See bug #1256368
 	if ((_vm->getPlatform() == Common::kPlatformMacintosh) && !_vm->_scene->isInIntro())
 		memset(&_currentPal[255 * 4], 0, 4);
