@@ -1,7 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2004-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
  *
- * The ReInherit Engine is (C)2000-2003 by Daniel Balsom.
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -17,15 +18,15 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/saga/gfx.h $
- * $Id: gfx.h 21916 2006-04-15 20:36:41Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/saga/gfx.h $
+ * $Id: gfx.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
 // Graphics maniuplation routines - private header file
 
-#ifndef SAGA_GFX_H_
-#define SAGA_GFX_H_
+#ifndef SAGA_GFX_H
+#define SAGA_GFX_H
 
 #include "graphics/surface.h"
 
@@ -78,14 +79,15 @@ struct ClipData {
 	}
 };
 
-#pragma START_PACK_STRUCTS
+#include "common/pack-start.h"	// START STRUCT PACKING
+
 struct PalEntry {
 	byte red;
 	byte green;
 	byte blue;
-} GCC_PACK;
+};
 
-#pragma END_PACK_STRUCTS
+#include "common/pack-end.h"	// END STRUCT PACKING
 
 struct Color {
 	int red;
@@ -147,9 +149,9 @@ public:
 	void palToBlack(PalEntry *src_pal, double percent);
 	void blackToPal(PalEntry *src_pal, double percent);
 	void showCursor(bool state);
+	void setCursor(CursorType cursorType = kCursorNormal);
 
 private:
-	void setCursor(CursorType cursorType = kCursorNormal);
 	int _init;
 	Surface _backBuffer;
 	byte _currentPal[PAL_ENTRIES * 4];
