@@ -1,7 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2004-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
  *
- * The ReInherit Engine is (C)2000-2003 by Daniel Balsom.
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -17,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/saga/input.cpp $
- * $Id: input.cpp 20532 2006-02-11 12:44:16Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/saga/input.cpp $
+ * $Id: input.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 #include "saga/saga.h"
@@ -32,19 +33,18 @@
 #include "saga/script.h"
 #include "saga/isomap.h"
 
+#include "common/events.h"
 #include "common/system.h"
 
 namespace Saga {
 
 int SagaEngine::processInput() {
-	OSystem::Event event;
+	Common::Event event;
 
-//	Point imousePt;
-
-	while (g_system->pollEvent(event)) {
+	while (_eventMan->pollEvent(event)) {
 		switch (event.type) {
-		case OSystem::EVENT_KEYDOWN:
-			if (event.kbd.flags == OSystem::KBD_CTRL) {
+		case Common::EVENT_KEYDOWN:
+			if (event.kbd.flags == Common::KBD_CTRL) {
 				if (event.kbd.keycode == 'd')
 					_console->attach();
 			}
@@ -119,35 +119,29 @@ int SagaEngine::processInput() {
 				break;
 			}
 			break;
-		case OSystem::EVENT_KEYUP:
-			_interface->processKeyUp(event.kbd.ascii);
-			break;
-		case OSystem::EVENT_LBUTTONUP:
+		case Common::EVENT_LBUTTONUP:
 			_leftMouseButtonPressed = false;
 			break;
-		case OSystem::EVENT_RBUTTONUP:
+		case Common::EVENT_RBUTTONUP:
 			_rightMouseButtonPressed = false;
 			break;
-		case OSystem::EVENT_LBUTTONDOWN:
+		case Common::EVENT_LBUTTONDOWN:
 			_leftMouseButtonPressed = true;
-			_mousePos = event.mouse;
-			_interface->update(_mousePos, UPDATE_LEFTBUTTONCLICK);
+			_interface->update(event.mouse, UPDATE_LEFTBUTTONCLICK);
 			break;
-		case OSystem::EVENT_RBUTTONDOWN:
+		case Common::EVENT_RBUTTONDOWN:
 			_rightMouseButtonPressed = true;
-			_mousePos = event.mouse;
-			_interface->update(_mousePos, UPDATE_RIGHTBUTTONCLICK);
+			_interface->update(event.mouse, UPDATE_RIGHTBUTTONCLICK);
 			break;
-		case OSystem::EVENT_WHEELUP:
-			_interface->update(_mousePos, UPDATE_WHEELUP);
+		case Common::EVENT_WHEELUP:
+			_interface->update(event.mouse, UPDATE_WHEELUP);
 			break;
-		case OSystem::EVENT_WHEELDOWN:
-			_interface->update(_mousePos, UPDATE_WHEELDOWN);
+		case Common::EVENT_WHEELDOWN:
+			_interface->update(event.mouse, UPDATE_WHEELDOWN);
 			break;
-		case OSystem::EVENT_MOUSEMOVE:
-			_mousePos = event.mouse;
+		case Common::EVENT_MOUSEMOVE:
 			break;
-		case OSystem::EVENT_QUIT:
+		case Common::EVENT_QUIT:
 			shutDown();
 			break;
 		default:
@@ -158,6 +152,9 @@ int SagaEngine::processInput() {
 	return SUCCESS;
 }
 
+Point SagaEngine::mousePos() const {
+	return _eventMan->getMousePos();
+}
 
 } // End of namespace Saga
 
