@@ -1,7 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2004-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
  *
- * The ReInherit Engine is (C)2000-2003 by Daniel Balsom.
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -17,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/saga/interface.cpp $
- * $Id: interface.cpp 23031 2006-06-11 20:30:43Z sev $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/saga/interface.cpp $
+ * $Id: interface.cpp 27107 2007-06-05 13:19:47Z thebluegr $
  *
  */
 
@@ -28,6 +29,7 @@
 #include "saga/gfx.h"
 #include "saga/actor.h"
 #include "saga/console.h"
+#include "saga/displayinfo.h"
 #include "saga/events.h"
 #include "saga/font.h"
 #include "saga/objectmap.h"
@@ -68,14 +70,14 @@ static int verbTypeToTextStringsIdLUT[2][kVerbTypeIdsMax] = {
 	-1,
 	-1},
 	{-1, 
-	3, //TODO:check
-	2, 
-	1, 
-	5, 
-	6, //TODO:check
-	8, //TODO:check
-	7, 
-	4}
+	kVerbIHNMWalk,
+	kVerbIHNMLookAt,
+	kVerbIHNMTake, 
+	kVerbIHNMUse, 
+	kVerbIHNMTalkTo,
+	kVerbIHNMSwallow,
+	kVerbIHNMGive, 
+	kVerbIHNMPush}
 };
 
 Interface::Interface(SagaEngine *vm) : _vm(vm) {
@@ -212,7 +214,6 @@ Interface::Interface(SagaEngine *vm) : _vm(vm) {
 		error("Interface::Interface(): not enough memory");
 	}
 
-	_textInputRepeatPhase = 0;
 	_textInput = false;
 	_statusTextInput = false;
 	_statusTextInputState = kStatusTextInputFirstRun;
@@ -316,7 +317,6 @@ void Interface::setMode(int mode) {
 		_textInput = true;
 		_textInputStringLength = strlen(_textInputString);
 		_textInputPos = _textInputStringLength + 1;
-		_textInputRepeatPhase = 0;
 		break;
 	case kPanelMap:
 		mapPanelShow();
@@ -337,14 +337,13 @@ void Interface::setMode(int mode) {
 		_textInputString[0] = 0;
 		_textInputStringLength = 0;
 		_textInputPos = _textInputStringLength + 1;
-		_textInputRepeatPhase = 0;
 		break;
 	}
 
 	draw();
 }
 
-bool Interface::processAscii(uint16 ascii, bool synthetic) {
+bool Interface::processAscii(uint16 ascii) {
 	// TODO: Checking for Esc and Enter below is a bit hackish, and
 	// and probably only works with the English version. Maybe we should
 	// add a flag to the button so it can indicate if it's the default or
@@ -352,8 +351,6 @@ bool Interface::processAscii(uint16 ascii, bool synthetic) {
 
 	int i;
 	PanelButton *panelButton;
-	if (!synthetic)
-		_textInputRepeatPhase = 0;
 	if (_statusTextInput) {
 		processStatusTextInput(ascii);
 		return true;
@@ -536,41 +533,12 @@ bool Interface::processAscii(uint16 ascii, bool synthetic) {
 	return false;
 }
 
-#define KEYBOARD_REPEAT_DELAY1 300000L
-#define KEYBOARD_REPEAT_DELAY2 50000L
-
-void Interface::textInputRepeatCallback(void *refCon) {
-	((Interface *)refCon)->textInputRepeat();
-}
-
-void Interface::textInputStartRepeat(uint16 ascii) {
-	if (!_textInputRepeatPhase) {
-		_textInputRepeatPhase = 1;
-		Common::g_timer->removeTimerProc(&textInputRepeatCallback);
-		Common::g_timer->installTimerProc(&textInputRepeatCallback, KEYBOARD_REPEAT_DELAY1, this);
-	}
-
-	_textInputRepeatChar = ascii;
-}
-
-void Interface::textInputRepeat() {
-	if (_textInputRepeatPhase == 1) {
-		_textInputRepeatPhase = 2;
-		Common::g_timer->removeTimerProc(&textInputRepeatCallback);
-		Common::g_timer->installTimerProc(&textInputRepeatCallback, KEYBOARD_REPEAT_DELAY2, this);
-	} else if (_textInputRepeatPhase == 2) {
-		processAscii(_textInputRepeatChar, true);
-	}
-}
-
-void Interface::processKeyUp(uint16 ascii) {
-	if (_textInputRepeatPhase) {
-		Common::g_timer->removeTimerProc(&textInputRepeatCallback);
-		_textInputRepeatPhase = 0;
-	}
-}
-
 void Interface::setStatusText(const char *text, int statusColor) {
+
+	// Disable the status text in IHNM when the chapter is 8
+	if (_vm->getGameType() == GType_IHNM && _vm->_scene->currentChapterNumber() == 8)
+		return;
+
 	assert(text != NULL);
 	assert(strlen(text) < STATUS_TEXT_LEN);
 
@@ -735,14 +703,20 @@ void Interface::drawPanelText(Surface *ds, InterfacePanel *panel, PanelButton *p
 	text = _vm->getTextString(panelButton->id);
 	panel->calcPanelButtonRect(panelButton, rect);
 	if (panelButton->xOffset < 0) {
-		textWidth = _vm->_font->getStringWidth(kKnownFontMedium, text, 0, kFontNormal);
+		if (_vm->getGameType() == GType_ITE)
+			textWidth = _vm->_font->getStringWidth(kKnownFontMedium, text, 0, kFontNormal);
+		else
+			textWidth = _vm->_font->getStringWidth(kKnownFontVerb, text, 0, kFontNormal);
 		rect.left += 2 + (panel->imageWidth - 1 - textWidth) / 2;
 	}
 
 	textPoint.x = rect.left;
 	textPoint.y = rect.top + 1;
 
-	_vm->_font->textDraw(kKnownFontMedium, ds, text, textPoint, _vm->KnownColor2ColorId(kKnownColorVerbText), _vm->KnownColor2ColorId(kKnownColorVerbTextShadow), kFontShadow);
+	if (_vm->getGameType() == GType_ITE)
+		_vm->_font->textDraw(kKnownFontMedium, ds, text, textPoint, _vm->KnownColor2ColorId(kKnownColorVerbText), _vm->KnownColor2ColorId(kKnownColorVerbTextShadow), kFontShadow);
+	else
+		_vm->_font->textDraw(kKnownFontVerb, ds, text, textPoint, _vm->KnownColor2ColorId(kKnownColorVerbText), _vm->KnownColor2ColorId(kKnownColorVerbTextShadow), kFontShadow);
 }
 
 void Interface::drawOption() {
@@ -757,7 +731,7 @@ void Interface::drawOption() {
 	Rect rect2;
 	PanelButton *panelButton;
 	Point textPoint;
-	if (_optionSaveFileSlider == NULL) return;//TODO:REMOVE
+	Point point;
 	
 	backBuffer = _vm->_gfx->getBackBuffer();
 
@@ -766,6 +740,15 @@ void Interface::drawOption() {
 
 	for (i = 0; i < _optionPanel.buttonsCount; i++) {
 		panelButton = &_optionPanel.buttons[i];
+
+		// TODO: This probably works for the button background, but the resources are still not loeaded
+		// (_optionPanel.sprites)
+		/*
+		point.x = _optionPanel.x + panelButton->xOffset;
+		point.y = _optionPanel.y + panelButton->yOffset;
+		_vm->_sprite->draw(backBuffer, _vm->getDisplayClip(), _optionPanel.sprites, i, point, 256);
+		*/
+
 		if (panelButton->type == kPanelButtonOption) {
 			drawPanelButtonText(backBuffer, &_optionPanel, panelButton);
 		}
@@ -775,9 +758,17 @@ void Interface::drawOption() {
 	}
 
 	if (_optionSaveRectTop.height() > 0) {
-		backBuffer->drawRect(_optionSaveRectTop, kITEColorDarkGrey);
+		if (_vm->getGameType() == GType_ITE) {
+			backBuffer->drawRect(_optionSaveRectTop, kITEColorDarkGrey);
+		} else {
+			// TODO: Draw the button graphic properly for IHNM
+		}
 	}
 
+	// FIXME: The _optionSaveFileSlider checks exist for IHNM, where 
+	// _optionSaveFileSlider is not initialized correctly yet
+	if (_optionSaveFileSlider == NULL) return; //TODO:REMOVE
+
 	drawButtonBox(backBuffer, _optionSaveRectSlider, kSlider, _optionSaveFileSlider->state > 0);
 
 	if (_optionSaveRectBottom.height() > 0) {
@@ -928,7 +919,6 @@ void Interface::setLoad(PanelButton *panelButton) {
 
 void Interface::processStatusTextInput(uint16 ascii) {
 
-	textInputStartRepeat(ascii);
 	switch (ascii) {
 	case 27: // esc
 		_statusTextInputState = kStatusTextInputAborted;
@@ -968,8 +958,6 @@ bool Interface::processTextInput(uint16 ascii) {
 	memset(tempString, 0, SAVE_TITLE_SIZE);
 	ch[1] = 0;
 
-	textInputStartRepeat(ascii);
-
 	switch (ascii) {
 	case 13:
 		return false;
@@ -1538,14 +1526,12 @@ void Interface::drawStatusBar() {
 	int stringWidth;
 	int color;
 
-	if (_panelMode == kPanelChapterSelection)
+	// Disable the status text in IHNM when the chapter is 8
+	if (_vm->getGameType() == GType_IHNM && _vm->_scene->currentChapterNumber() == 8)
 		return;
 
 	backBuffer = _vm->_gfx->getBackBuffer();
 
-	// Disable this for IHNM for now, since that game uses the full screen
-	// in some cases.
-
 	// Erase background of status bar
 	rect.left = _vm->getDisplayInfo().statusXOffset;
 	rect.top = _vm->getDisplayInfo().statusYOffset;
@@ -1905,7 +1891,7 @@ void Interface::drawPanelButtonText(Surface *ds, InterfacePanel *panel, PanelBut
 	Rect rect;
 
 	textId = panelButton->id;
-	switch(panelButton->id) {
+	switch (panelButton->id) {
 	case kTextReadingSpeed:
 		if (_vm->getFeatures() & GF_CD_FX) {
 			if (_vm->_subtitlesEnabled)
@@ -1931,8 +1917,13 @@ void Interface::drawPanelButtonText(Surface *ds, InterfacePanel *panel, PanelBut
 	}
 	text = _vm->getTextString(textId);
 
-	textWidth = _vm->_font->getStringWidth(kKnownFontMedium, text, 0, kFontNormal);
-	textHeight = _vm->_font->getHeight(kKnownFontMedium);
+	if (_vm->getGameType() == GType_ITE) {
+		textWidth = _vm->_font->getStringWidth(kKnownFontMedium, text, 0, kFontNormal);
+		textHeight = _vm->_font->getHeight(kKnownFontMedium);
+	} else {
+		textWidth = _vm->_font->getStringWidth(kKnownFontVerb, text, 0, kFontNormal);
+		textHeight = _vm->_font->getHeight(kKnownFontVerb);
+	}
 
 	point.x = panel->x + panelButton->xOffset + (panelButton->width / 2) - (textWidth / 2);
 	point.y = panel->y + panelButton->yOffset + (panelButton->height / 2) - (textHeight / 2);
@@ -1946,8 +1937,12 @@ void Interface::drawPanelButtonText(Surface *ds, InterfacePanel *panel, PanelBut
 	panel->calcPanelButtonRect(panelButton, rect);
 	drawButtonBox(ds, rect, kButton, panelButton->state > 0);
 
-	_vm->_font->textDraw(kKnownFontMedium, ds, text, point,
-		_vm->KnownColor2ColorId(textColor), _vm->KnownColor2ColorId(kKnownColorVerbTextShadow), kFontShadow);
+	if (_vm->getGameType() == GType_ITE)
+		_vm->_font->textDraw(kKnownFontMedium, ds, text, point,
+			_vm->KnownColor2ColorId(textColor), _vm->KnownColor2ColorId(kKnownColorVerbTextShadow), kFontShadow);
+	else
+		_vm->_font->textDraw(kKnownFontVerb, ds, text, point,
+			_vm->KnownColor2ColorId(textColor), _vm->KnownColor2ColorId(kKnownColorVerbTextShadow), kFontShadow);
 }
 
 void Interface::drawPanelButtonArrow(Surface *ds, InterfacePanel *panel, PanelButton *panelButton) {
@@ -1967,7 +1962,10 @@ void Interface::drawPanelButtonArrow(Surface *ds, InterfacePanel *panel, PanelBu
 	point.x = panel->x + panelButton->xOffset;
 	point.y = panel->y + panelButton->yOffset;
 
-	_vm->_sprite->draw(ds, _vm->getDisplayClip(), _vm->_sprite->_mainSprites, spriteNumber, point, 256);
+	if (_vm->getGameType() == GType_ITE)
+		_vm->_sprite->draw(ds, _vm->getDisplayClip(), _vm->_sprite->_mainSprites, spriteNumber, point, 256);
+	else
+		_vm->_sprite->draw(ds, _vm->getDisplayClip(), _vm->_sprite->_arrowSprites, spriteNumber, point, 256);
 }
 
 void Interface::drawVerbPanelText(Surface *ds, PanelButton *panelButton, KnownColor textKnownColor, KnownColor textShadowKnownColor) {
@@ -1979,7 +1977,7 @@ void Interface::drawVerbPanelText(Surface *ds, PanelButton *panelButton, KnownCo
 	if (_vm->getGameType() == GType_ITE) {
 		textId = verbTypeToTextStringsIdLUT[0][panelButton->id];
 		text = _vm->getTextString(textId);
-	} else {
+	} else {	
 		textId = verbTypeToTextStringsIdLUT[1][panelButton->id];
 		text = _vm->_script->_mainStrings.getString(textId + 1);
 		textShadowKnownColor = kKnownColorTransparent;
