@@ -1,7 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2004-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
  *
- * The ReInherit Engine is (C)2000-2003 by Daniel Balsom.
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -17,18 +18,19 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/saga/interface.h $
- * $Id: interface.h 20532 2006-02-11 12:44:16Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/saga/interface.h $
+ * $Id: interface.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
 // Game interface module private header file
 
-#ifndef SAGA_INTERFACE_H__
-#define SAGA_INTERFACE_H__
+#ifndef SAGA_INTERFACE_H
+#define SAGA_INTERFACE_H
 
 #include "common/savefile.h"
 
+#include "saga/displayinfo.h"
 #include "saga/sprite.h"
 #include "saga/script.h"
 
@@ -222,8 +224,7 @@ public:
 	void drawStatusBar();
 	void setVerbState(int verb, int state);
 
-	bool processAscii(uint16 ascii, bool synthetic = false);
-	void processKeyUp(uint16 ascii);
+	bool processAscii(uint16 ascii);
 
 	void keyBoss();
 	void keyBossExit();
@@ -243,8 +244,6 @@ public:
 	}
 
 private:
-	static void textInputRepeatCallback(void *refCon);
-
 	void drawInventory(Surface *backBuffer);
 	void updateInventory(int pos);
 	void inventoryChangePos(int chg);
@@ -343,8 +342,6 @@ private:
 	void calcOptionSaveSlider();
 	bool processTextInput(uint16 ascii);
 	void processStatusTextInput(uint16 ascii);
-	void textInputStartRepeat(uint16 ascii);
-	void textInputRepeat(void);
 
 public:
 	void converseInit(void);
@@ -452,9 +449,6 @@ private:
 
 	uint _statusTextInputPos;
 
-	int _textInputRepeatPhase;
-	uint16 _textInputRepeatChar;
-
 	PalEntry _mapSavedPal[PAL_ENTRIES];
 	bool _mapPanelCrossHairState;
 
@@ -463,4 +457,4 @@ private:
 
 } // End of namespace Saga
 
-#endif				/* INTERFACE_H__ */
+#endif
