@@ -1,7 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2004-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
  *
- * The ReInherit Engine is (C)2000-2003 by Daniel Balsom.
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -17,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/saga/music.cpp $
- * $Id: music.cpp 22433 2006-05-13 10:30:38Z h00ligan $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/saga/music.cpp $
+ * $Id: music.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 #include "saga/saga.h"
@@ -30,9 +31,6 @@
 #include "sound/audiostream.h"
 #include "sound/mididrv.h"
 #include "sound/midiparser.h"
-#include "sound/mp3.h"
-#include "sound/vorbis.h"
-#include "sound/flac.h"
 #include "common/config-manager.h"
 #include "common/file.h"
 
@@ -40,24 +38,6 @@ namespace Saga {
 
 #define BUFFER_SIZE 4096
 
-struct TrackFormat {
-	Audio::DigitalTrackInfo* (*openTrackFunction)(int);
-};
-
-static const TrackFormat TRACK_FORMATS[] = {
-#ifdef USE_FLAC
-	{ Audio::getFlacTrack },
-#endif
-#ifdef USE_VORBIS
-	{ Audio::getVorbisTrack },
-#endif
-#ifdef USE_MAD
-	{ Audio::getMP3Track },
-#endif
-
-	{ NULL } // Terminator
-};
-
 // I haven't decided yet if it's a good idea to make looping part of the audio
 // stream class, or if I should use a "wrapper" class, like I did for Broken
 // Sword 2, to make it easier to add support for compressed music... but I'll
@@ -307,8 +287,6 @@ Music::Music(SagaEngine *vm, Audio::Mixer *mixer, MidiDriver *driver, int enable
 	_songTableLen = 0;
 	_songTable = 0;
 
-	_track = NULL;
-
 	_midiMusicData = NULL;
 }
 
@@ -346,7 +324,7 @@ void Music::musicVolumeGauge() {
 	_player->setVolume(volume);
 
 	if (_currentVolumePercent == 100) {
-		Common::g_timer->removeTimerProc(&musicVolumeGaugeCallback);
+		_vm->_timer->removeTimerProc(&musicVolumeGaugeCallback);
 		_currentVolume = _targetVolume;
 	}
 }
@@ -361,12 +339,12 @@ void Music::setVolume(int volume, int time) {
 	if (time == 1) {
 		_mixer->setVolumeForSoundType(Audio::Mixer::kMusicSoundType, volume);
 		_player->setVolume(volume);
-		Common::g_timer->removeTimerProc(&musicVolumeGaugeCallback);
+		_vm->_timer->removeTimerProc(&musicVolumeGaugeCallback);
 		_currentVolume = volume;
 		return;
 	}
 
-	Common::g_timer->installTimerProc(&musicVolumeGaugeCallback, time * 100L, this);
+	_vm->_timer->installTimerProc(&musicVolumeGaugeCallback, time * 100L, this);
 }
 
 bool Music::isPlaying() {
@@ -411,13 +389,18 @@ void Music::play(uint32 resourceId, MusicFlags flags) {
 	}
 
 	// Try to open standalone digital track
-	for (int i = 0; i < ARRAYSIZE(TRACK_FORMATS) - 1; ++i)
-		if ((_track = TRACK_FORMATS[i].openTrackFunction(realTrackNumber))) {
-			break;
+	char trackName[2][16];
+	sprintf(trackName[0], "track%d", realTrackNumber);
+	sprintf(trackName[1], "track%02d", realTrackNumber);
+	Audio::AudioStream *stream = 0;
+	for (int i = 0; i < 2; ++i) {
+		// We multiply by 40 / 3 = 1000 / 75 to convert frames to milliseconds
+		// FIXME: Do we really want a duration of 10000 frames = 133 seconds, or is that just a random value?
+		stream = Audio::AudioStream::openStreamFile(trackName[i], 0, 10000 * 40 / 3, (flags == MUSIC_LOOP) ? 0 : 1);
+		if (stream) {
+			_mixer->playInputStream(Audio::Mixer::kMusicSoundType, &_musicHandle, stream);
+			return;
 		}
-	if (_track) {
-		_track->play(_mixer, &_musicHandle, (flags == MUSIC_LOOP) ? -1 : 1, 10000);
-		return;
 	}
 
 	if (_vm->getGameType() == GType_ITE) {
