@@ -1,7 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2004-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
  *
- * The ReInherit Engine is (C)2000-2003 by Daniel Balsom.
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -17,15 +18,15 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/saga/music.h $
- * $Id: music.h 22231 2006-04-29 22:33:31Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/saga/music.h $
+ * $Id: music.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
 // Music class
 
-#ifndef SAGA_MUSIC_H_
-#define SAGA_MUSIC_H_
+#ifndef SAGA_MUSIC_H
+#define SAGA_MUSIC_H
 
 #include "sound/audiocd.h"
 #include "sound/mididrv.h"
@@ -137,8 +138,6 @@ private:
 	MidiParser *xmidiParser;
 	MidiParser *smfParser;
 
-	Audio::DigitalTrackInfo *_track;
-
 	byte *_midiMusicData;
 
 	static void musicVolumeGaugeCallback(void *refCon);
