@@ -1,7 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2004-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
  *
- * The ReInherit Engine is (C)2000-2003 by Daniel Balsom.
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -17,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/saga/objectmap.cpp $
- * $Id: objectmap.cpp 20532 2006-02-11 12:44:16Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/saga/objectmap.cpp $
+ * $Id: objectmap.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -41,7 +42,7 @@
 
 namespace Saga {
 
-HitZone::HitZone(MemoryReadStreamEndian *readStream, int index): _index(index) {
+HitZone::HitZone(MemoryReadStreamEndian *readStream, int index, int sceneNumber): _index(index) {
 	int i, j;
 	HitZone::ClickArea *clickArea;
 	Point *point;
@@ -74,6 +75,13 @@ HitZone::HitZone(MemoryReadStreamEndian *readStream, int index): _index(index) {
 			point = &clickArea->points[j];
 			point->x = readStream->readSint16();
 			point->y = readStream->readSint16();
+
+			// WORKAROUND: bug #1259608: "ITE: Riff ignores command in Ferret merchant center"
+			// Apparently ITE Mac version has bug in game data. Both ObjectMap and ActionMap
+			// for exit area are little taller (y = 123) and thus Riff goes to exit 
+			// when clicked on barrel of nails.
+			if (sceneNumber == 18 && index == 0 && i == 0 && j == 0 && point->y == 123)
+				point->y = 129;
 		}
 	}
 }
@@ -198,7 +206,7 @@ void ObjectMap::load(const byte *resourcePointer, size_t resourceLength) {
 	}
 
 	for (i = 0; i < _hitZoneListCount; i++) {
-		_hitZoneList[i] = new HitZone(&readS, i);
+		_hitZoneList[i] = new HitZone(&readS, i, _vm->_scene->currentSceneNumber());
 	}
 }
 
