@@ -1,7 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2004-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
  *
- * The ReInherit Engine is (C)2000-2003 by Daniel Balsom.
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -17,15 +18,15 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/saga/objectmap.h $
- * $Id: objectmap.h 20532 2006-02-11 12:44:16Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/saga/objectmap.h $
+ * $Id: objectmap.h 27072 2007-06-03 22:00:18Z thebluegr $
  *
  */
 
 // Object map / Object click-area module header file
 
-#ifndef SAGA_OBJECTMAP_H_
-#define SAGA_OBJECTMAP_H_
+#ifndef SAGA_OBJECTMAP_H
+#define SAGA_OBJECTMAP_H
 
 #include "saga/stream.h"
 
@@ -40,7 +41,7 @@ private:
 	};
 
 public:
-	HitZone(MemoryReadStreamEndian *readStream, int index);
+	HitZone(MemoryReadStreamEndian *readStream, int index, int sceneNumber);
 	~HitZone();
 
 	int getNameIndex() const {
@@ -109,7 +110,12 @@ public:
 	int hitTest(const Point& testPoint);
 	HitZone *getHitZone(int16 index) {
 		if ((index < 0) || (index >= _hitZoneListCount)) {
-			error("ObjectMap::getHitZone wrong index 0x%X", index);
+			// HACK: If we get a wrong hitzone, return the last hitzone in the list
+			// Normally, we don't get wrong hitzones in ITE, however IHNM still seems
+			// to have problems with some, therefore just throw a warning for now and
+			// continue with a valid hitzone
+			warning("ObjectMap::getHitZone wrong index 0x%X, adjusting it to 0x%X", index, _hitZoneListCount - 1);
+			index = _hitZoneListCount - 1;
 		}
 		return _hitZoneList[index];
 	}
