@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2005-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/saga/puzzle.cpp $
- * $Id: puzzle.cpp 22433 2006-05-13 10:30:38Z h00ligan $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/saga/puzzle.cpp $
+ * $Id: puzzle.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -119,7 +122,7 @@ void Puzzle::initPieceInfo(int i, int16 curX, int16 curY, byte offX, byte offY,
 
 void Puzzle::execute(void) {
 	_active = true;
-	Common::g_timer->installTimerProc(&hintTimerCallback, kPuzzleHintTime, this);
+	_vm->_timer->installTimerProc(&hintTimerCallback, kPuzzleHintTime, this);
 
 	initPieces();
 
@@ -134,7 +137,7 @@ void Puzzle::execute(void) {
 void Puzzle::exitPuzzle(void) {
 	_active = false;
 
-	Common::g_timer->removeTimerProc(&hintTimerCallback);
+	_vm->_timer->removeTimerProc(&hintTimerCallback);
 
 	_vm->_scene->changeScene(ITE_SCENE_LODGE, 0, kTransitionNoFade);
 	_vm->_interface->setMode(kPanelMain);
@@ -383,17 +386,17 @@ void Puzzle::solicitHint(void) {
 
 	_vm->_actor->setSpeechColor(1, kITEColorBlack);
 
-	Common::g_timer->removeTimerProc(&hintTimerCallback);
+	_vm->_timer->removeTimerProc(&hintTimerCallback);
 
 	switch (_hintRqState) {
 	case kRQSpeaking:
 		if (_vm->_actor->isSpeaking()) {
-			Common::g_timer->installTimerProc(&hintTimerCallback, 50000, this);
+			_vm->_timer->installTimerProc(&hintTimerCallback, 50000, this);
 			break;
 		}
 
 		_hintRqState = _hintNextRqState;
-		Common::g_timer->installTimerProc(&hintTimerCallback, 333333, this);
+		_vm->_timer->installTimerProc(&hintTimerCallback, 333333, this);
 		break;
 
 	case kRQNoHint:
@@ -416,11 +419,11 @@ void Puzzle::solicitHint(void) {
 		//	Roll to see if Sakka scolds
 		if (_vm->_rnd.getRandomNumber(1)) {
 			_hintRqState = kRQSakkaDenies;
-			Common::g_timer->installTimerProc(&hintTimerCallback, 200000, this);
+			_vm->_timer->installTimerProc(&hintTimerCallback, 200000, this);
 		} else {
 			_hintRqState = kRQSpeaking;
 			_hintNextRqState = kRQHintRequested;
-			Common::g_timer->installTimerProc(&hintTimerCallback, 50000, this);
+			_vm->_timer->installTimerProc(&hintTimerCallback, 50000, this);
 		}
 
 		break;
@@ -433,7 +436,7 @@ void Puzzle::solicitHint(void) {
 
 		_hintRqState = kRQSpeaking;
 		_hintNextRqState = kRQHintRequestedStage2;
-		Common::g_timer->installTimerProc(&hintTimerCallback, 50000, this);
+		_vm->_timer->installTimerProc(&hintTimerCallback, 50000, this);
 
 		_vm->_interface->converseClear();
 		_vm->_interface->converseAddText(optionsStr[_lang][kROAccept], 1, 0, 0 );
@@ -460,7 +463,7 @@ void Puzzle::solicitHint(void) {
 		_vm->_interface->converseAddText(optionsStr[_lang][kROLater], 0, 0, 0);
 		_vm->_interface->converseDisplayText();
 
-		Common::g_timer->installTimerProc(&hintTimerCallback, kPuzzleHintTime, this);
+		_vm->_timer->installTimerProc(&hintTimerCallback, kPuzzleHintTime, this);
 
 		_hintRqState = kRQSkipEverything;
 		break;
@@ -471,7 +474,7 @@ void Puzzle::solicitHint(void) {
 }
 
 void Puzzle::handleReply(int reply) {
-	switch(reply) {
+	switch (reply) {
 	case 0:		// Quit the puzzle
 		exitPuzzle();
 		break;
@@ -483,8 +486,8 @@ void Puzzle::handleReply(int reply) {
 	case 2:		// Decline the hint
 		_vm->_actor->abortSpeech();
 		_hintRqState = kRQNoHint;
-		Common::g_timer->removeTimerProc(&hintTimerCallback);
-		Common::g_timer->installTimerProc(&hintTimerCallback, kPuzzleHintTime * 2, this);
+		_vm->_timer->removeTimerProc(&hintTimerCallback);
+		_vm->_timer->installTimerProc(&hintTimerCallback, kPuzzleHintTime * 2, this);
 		clearHint();
 		break;
 	}
@@ -546,8 +549,8 @@ void Puzzle::giveHint(void) {
 	_vm->_interface->converseAddText(optionsStr[_lang][kROLater], 0, 0, 0);
 	_vm->_interface->converseDisplayText();
 
-	Common::g_timer->removeTimerProc(&hintTimerCallback);
-	Common::g_timer->installTimerProc(&hintTimerCallback, kPuzzleHintTime, this);
+	_vm->_timer->removeTimerProc(&hintTimerCallback);
+	_vm->_timer->installTimerProc(&hintTimerCallback, kPuzzleHintTime, this);
 }
 
 void Puzzle::clearHint(void) {
