@@ -1,7 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2004-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
  *
- * The ReInherit Engine is (C)2000-2003 by Daniel Balsom.
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -17,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/saga/render.cpp $
- * $Id: render.cpp 20532 2006-02-11 12:44:16Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/saga/render.cpp $
+ * $Id: render.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -48,7 +49,7 @@ Render::Render(SagaEngine *vm, OSystem *system) {
 	_initialized = false;
 
 	// Initialize FPS timer callback
-	Common::g_timer->installTimerProc(&fpsTimerCallback, 1000000, this);
+	_vm->_timer->installTimerProc(&fpsTimerCallback, 1000000, this);
 
 	_backGroundSurface.create(_vm->getDisplayWidth(), _vm->getDisplayHeight(), 1);
 
@@ -58,7 +59,7 @@ Render::Render(SagaEngine *vm, OSystem *system) {
 }
 
 Render::~Render(void) {
-	Common::g_timer->removeTimerProc(&fpsTimerCallback);
+	_vm->_timer->removeTimerProc(&fpsTimerCallback);
 	_backGroundSurface.free();
 
 	_initialized = false;
