@@ -1,7 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2004-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
  *
- * The ReInherit Engine is (C)2000-2003 by Daniel Balsom.
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -17,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/saga/rscfile.cpp $
- * $Id: rscfile.cpp 21013 2006-03-02 18:25:56Z h00ligan $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/saga/rscfile.cpp $
+ * $Id: rscfile.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -34,6 +35,8 @@
 #include "saga/sndres.h"
 #include "saga/stream.h"
 
+#include "common/advancedDetector.h"
+
 namespace Saga {
 
 struct MacResMap {
@@ -261,7 +264,7 @@ bool Resource::loadMacContext(ResourceContext *context) {
 bool Resource::loadContext(ResourceContext *context) {
 	size_t i;
 	int j;
-	GamePatchDescription *patchDescription;
+	const GamePatchDescription *patchDescription;
 	ResourceData *resourceData;
 	uint16 subjectResourceType;
 	ResourceContext *subjectContext;
@@ -317,8 +320,8 @@ bool Resource::loadContext(ResourceContext *context) {
 	}
 
 	//process external patch files
-	for (j = 0; j < _vm->getGameDescription()->patchesCount; j++) {
-		patchDescription = &_vm->getGameDescription()->patchDescriptions[j];
+	for (j = 0; j < _vm->getPatchesCount(); j++) {
+		patchDescription = &_vm->getPatchDescriptions()[j];
 		if ((patchDescription->fileType & context->fileType) != 0) {
 			if (patchDescription->resourceId < context->count) {
 				resourceData = &context->table[patchDescription->resourceId];
@@ -340,14 +343,18 @@ bool Resource::loadContext(ResourceContext *context) {
 bool Resource::createContexts() {
 	int i;
 	ResourceContext *context;
-	_contextsCount = _vm->getGameDescription()->filesCount;
+
+	_contextsCount = 0;
+	for (i = 0; _vm->getFilesDescriptions()[i].fileName; i++)
+		_contextsCount++;
+
 	_contexts = (ResourceContext*)calloc(_contextsCount, sizeof(*_contexts));
 
 	for (i = 0; i < _contextsCount; i++) {
 		context = &_contexts[i];
 		context->file = new Common::File();
-		context->fileName = _vm->getGameDescription()->filesDescriptions[i].fileName;
-		context->fileType = _vm->getGameDescription()->filesDescriptions[i].fileType;
+		context->fileName = _vm->getFilesDescriptions()[i].fileName;
+		context->fileType = _vm->getFilesDescriptions()[i].fileType;
 		context->serial = 0;
 
 		// IHNM has serveral different voice files, so we need to allow
@@ -377,11 +384,11 @@ void Resource::clearContexts() {
 	if (_contexts == NULL) {
 		return;
 	}
-	for(i = 0; i < _contextsCount; i++) {
+	for (i = 0; i < _contextsCount; i++) {
 		context = &_contexts[i];
 		delete context->file;
 		if (context->table != NULL) {
-			for(j = 0; j < context->count; j++) {
+			for (j = 0; j < context->count; j++) {
 				delete context->table[j].patchData;
 			}
 		}
@@ -398,7 +405,7 @@ uint32 Resource::convertResourceId(uint32 resourceId) {
 			return resourceId - 2;
 		} else {
 			if (resourceId == 1535 || resourceId == 1536) {
-				error ("Wrong resource number %d for Mac ITE", resourceId);
+				error("Wrong resource number %d for Mac ITE", resourceId);
 			}
 		}
 	}
