@@ -1,7 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2004-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
  *
- * The ReInherit Engine is (C)2000-2003 by Daniel Balsom.
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -17,15 +18,15 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/saga/rscfile.h $
- * $Id: rscfile.h 20532 2006-02-11 12:44:16Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/saga/rscfile.h $
+ * $Id: rscfile.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
 // RSC Resource file management header file
 
-#ifndef SAGA_RSCFILE_H__
-#define SAGA_RSCFILE_H__
+#ifndef SAGA_RSCFILE_H
+#define SAGA_RSCFILE_H
 
 #include "common/file.h"
 
@@ -40,9 +41,9 @@ namespace Saga {
 struct PatchData {
 	bool _deletePatchFile;
 	Common::File *_patchFile;
-	GamePatchDescription *_patchDescription;
+	const GamePatchDescription *_patchDescription;
 
-	PatchData(GamePatchDescription *patchDescription): _patchDescription(patchDescription), _deletePatchFile(true) {
+	PatchData(const GamePatchDescription *patchDescription): _patchDescription(patchDescription), _deletePatchFile(true) {
 		_patchFile = new Common::File();
 	}
 	PatchData(Common::File *patchFile): _patchDescription(NULL), _patchFile(patchFile), _deletePatchFile(false) {
