@@ -1,7 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2004-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
  *
- * The ReInherit Engine is (C)2000-2003 by Daniel Balsom.
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -17,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/saga/saga.cpp $
- * $Id: saga.cpp 23031 2006-06-11 20:30:43Z sev $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/saga/saga.cpp $
+ * $Id: saga.cpp 27070 2007-06-03 19:15:59Z thebluegr $
  *
  */
 #include "common/stdafx.h"
@@ -142,10 +143,6 @@ SagaEngine::~SagaEngine() {
 	delete _resource;
 }
 
-void SagaEngine::errorString(const char *buf1, char *buf2) {
-	strcpy(buf2, buf1);
-}
-
 int SagaEngine::init() {
 	_soundVolume = ConfMan.getInt("sfx_volume") / 25;
 	_musicVolume = ConfMan.getInt("music_volume") / 25;
@@ -160,7 +157,7 @@ int SagaEngine::init() {
 
 	// Detect game and open resource files
 	if (!initGame()) {
-		GUIErrorMessage("No valid games were found in the specified directory.");
+		GUIErrorMessage("Error loading game resources.");
 		return FAILURE;
 	}
 
