@@ -1,7 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2004-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
  *
- * The ReInherit Engine is (C)2000-2003 by Daniel Balsom.
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -17,21 +18,25 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/saga/saga.h $
- * $Id: saga.h 23031 2006-06-11 20:30:43Z sev $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/saga/saga.h $
+ * $Id: saga.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
 #ifndef SAGA_H
 #define SAGA_H
 
-#include "base/engine.h"
+#include "engines/engine.h"
 
 #include "common/stream.h"
 
 #include "saga/gfx.h"
 #include "saga/list.h"
 
+namespace Common {
+struct ADGameFileDescription;
+}
+
 namespace Saga {
 
 class SndRes;
@@ -83,8 +88,237 @@ enum ERRORCODE {
 	SUCCESS = 0
 };
 
-#include "sagagame.h"
+enum SAGAGameType {
+	GType_ITE = 0,
+	GType_IHNM = 1
+};
 
+enum GameIds {
+	// Dreamers Guild
+	GID_ITE_DEMO_G = 0,
+	GID_ITE_DISK_G,
+	GID_ITE_DISK_G2,
+	GID_ITE_CD_G,
+	GID_ITE_CD_G2,
+	GID_ITE_MACCD_G,
+
+	// Wyrmkeep
+	GID_ITE_CD,       // data for Win rerelease is same as in old DOS
+	GID_ITE_WINCD,    // but it has a bunch of patch files
+	GID_ITE_MACCD,
+	GID_ITE_LINCD,
+	GID_ITE_MULTICD,  // Wyrmkeep combined Windows/Mac/Linux version
+	GID_ITE_WINDEMO1, // older Wyrmkeep windows demo
+	GID_ITE_MACDEMO1, // older Wyrmkeep mac demo
+	GID_ITE_LINDEMO,
+	GID_ITE_WINDEMO2,
+	GID_ITE_WINDEMO3,
+	GID_ITE_MACDEMO2,
+
+	// German
+	GID_ITE_DISK_DE,
+	GID_ITE_DISK_DE2,
+	GID_ITE_AMIGACD_DE, // TODO
+	GID_ITE_OLDMAC_DE,  // TODO
+	GID_ITE_AMIGA_FL_DE,// TODO
+	GID_ITE_CD_DE,      // reported by mld. Bestsellergamers cover disk
+	GID_ITE_CD_DE2,
+	GID_ITE_AMIGA_AGA_DEMO, // TODO
+	GID_ITE_AMIGA_ECS_DEMO, // TODO
+
+	GID_IHNM_DEMO,
+	GID_IHNM_CD,
+	GID_IHNM_CD_DE,   // reported by mld. German retail
+	GID_IHNM_CD_ES,
+	GID_IHNM_CD_RU,
+	GID_IHNM_CD_FR
+};
+
+enum GameFileTypes {
+	GAME_RESOURCEFILE = 1 << 0,
+	GAME_SCRIPTFILE   = 1 << 1,
+	GAME_SOUNDFILE    = 1 << 2,
+	GAME_VOICEFILE    = 1 << 3,
+	GAME_DEMOFILE     = 1 << 4,
+	GAME_MUSICFILE    = 1 << 5,
+	GAME_MUSICFILE_GM = 1 << 6,
+	GAME_MUSICFILE_FM = 1 << 7,
+	GAME_PATCHFILE    = 1 << 8,
+	GAME_MACBINARY    = 1 << 9,
+	GAME_SWAPENDIAN   = 1 << 10
+};
+
+enum GameFeatures {
+	GF_BIG_ENDIAN_DATA   = 1 << 0,
+	GF_WYRMKEEP          = 1 << 1,
+	GF_CD_FX             = 1 << 2,
+	GF_SCENE_SUBSTITUTES = 1 << 3
+};
+
+enum VerbTypeIds {
+        kVerbITENone = 0,
+        kVerbITEPickUp = 1,
+        kVerbITELookAt = 2,
+        kVerbITEWalkTo = 3,
+        kVerbITETalkTo = 4,
+        kVerbITEOpen = 5,
+        kVerbITEClose = 6,
+        kVerbITEGive = 7,
+        kVerbITEUse = 8,
+        kVerbITEOptions = 9,
+        kVerbITEEnter = 10,
+        kVerbITELeave = 11,
+        kVerbITEBegin = 12,
+        kVerbITEWalkOnly = 13,
+        kVerbITELookOnly = 14,
+
+
+        kVerbIHNMNone = 0,
+        kVerbIHNMWalk = 1,
+        kVerbIHNMLookAt = 2,
+        kVerbIHNMTake = 3,
+        kVerbIHNMUse = 4,
+        kVerbIHNMTalkTo = 5,
+        kVerbIHNMSwallow = 6,
+        kVerbIHNMGive = 7,
+        kVerbIHNMPush = 8,
+        kVerbIHNMOptions = 9,
+        kVerbIHNMEnter = 10,
+        kVerbIHNMLeave = 11,
+        kVerbIHNMBegin = 12,
+        kVerbIHNMWalkOnly = 13,
+        kVerbIHNMLookOnly = 14,
+
+        kVerbTypeIdsMax = kVerbITELookOnly + 1
+};
+
+enum PanelButtonType {
+	kPanelButtonVerb = 1 << 0,
+	kPanelButtonArrow = 1 << 1,
+	kPanelButtonConverseText = 1 << 2,
+	kPanelButtonInventory = 1 << 3,
+
+	kPanelButtonOption = 1 << 4,
+	kPanelButtonOptionSlider = 1 << 5,
+	kPanelButtonOptionSaveFiles = 1 << 6,
+	kPanelButtonOptionText = 1 << 7,
+
+	kPanelButtonQuit = 1 << 8,
+	kPanelButtonQuitText = 1 << 9,
+
+	kPanelButtonLoad = 1 << 10,
+	kPanelButtonLoadText = 1 << 11,
+
+	kPanelButtonSave = 1 << 12,
+	kPanelButtonSaveText = 1 << 13,
+	kPanelButtonSaveEdit = 1 << 14,
+
+	kPanelButtonProtectText = 1 << 15,
+	kPanelButtonProtectEdit = 1 << 16,
+
+	kPanelAllButtons = 0xFFFFF
+};
+
+enum GameSoundTypes {
+	kSoundPCM = 0,
+	kSoundVOX = 1,
+	kSoundVOC = 2,
+	kSoundWAV = 3,
+	kSoundMacPCM = 4
+};
+
+enum TextStringIds {
+	kTextWalkTo,
+	kTextLookAt,
+	kTextPickUp,
+	kTextTalkTo,
+	kTextOpen,
+	kTextClose,
+	kTextUse,
+	kTextGive,
+	kTextOptions,
+	kTextTest,
+	kTextDemo,
+	kTextHelp,
+	kTextQuitGame,
+	kTextFast,
+	kTextSlow,
+	kTextOn,
+	kTextOff,
+	kTextContinuePlaying,
+	kTextLoad,
+	kTextSave,
+	kTextGameOptions,
+	kTextReadingSpeed,
+	kTextMusic,
+	kTextSound,
+	kTextCancel,
+	kTextQuit,
+	kTextOK,
+	kTextMid,
+	kTextClick,
+	kText10Percent,
+	kText20Percent,
+	kText30Percent,
+	kText40Percent,
+	kText50Percent,
+	kText60Percent,
+	kText70Percent,
+	kText80Percent,
+	kText90Percent,
+	kTextMax,
+	kTextQuitTheGameQuestion,
+	kTextLoadSuccessful,
+	kTextEnterSaveGameName,
+	kTextGiveTo,
+	kTextUseWidth,
+	kTextNewSave,
+	kTextICantPickup,
+	kTextNothingSpecial,
+	kTextNoPlaceToOpen,
+	kTextNoOpening,
+	kTextDontKnow,
+	kTextShowDialog,
+	kTextEnterProtectAnswer
+};
+
+
+struct GameResourceDescription {
+	uint32 sceneLUTResourceId;
+	uint32 moduleLUTResourceId;
+	uint32 mainPanelResourceId;
+	uint32 conversePanelResourceId;
+	uint32 optionPanelResourceId;
+	uint32 mainSpritesResourceId;
+	uint32 mainPanelSpritesResourceId;
+	uint32 defaultPortraitsResourceId;
+	uint32 mainStringsResourceId;
+	uint32 actorsStringsResourceId;
+};
+
+struct GameFontDescription {
+	uint32 fontResourceId;
+};
+
+struct GameDisplayInfo;
+
+struct GameSoundInfo {
+	GameSoundTypes resourceType;
+	long frequency;
+	int sampleBits;
+	bool stereo;
+	bool isBigEndian;
+	bool isSigned;
+};
+
+struct GamePatchDescription {
+	const char *fileName;
+	uint16 fileType;
+	uint32 resourceId;
+	const GameSoundInfo *soundInfo;
+};
+
+struct SAGAGameDescription;
 
 enum GameObjectTypes {
 	kGameObjectNone = 0,
@@ -230,10 +464,6 @@ inline int clamp(int minValue, int value, int maxValue) {
 	}
 }
 
-inline int integerCompare(int i1, int i2) {
-	return ((i1) > (i2) ? 1 : ((i1) < (i2) ? -1 : 0));
-}
-
 inline int objectTypeId(uint16 objectId) {
 	return objectId >> OBJECT_TYPE_SHIFT;
 }
@@ -246,12 +476,9 @@ inline uint16 objectIndexToId(int type, int index) {
 	return (type << OBJECT_TYPE_SHIFT) | (OBJECT_TYPE_MASK & index);
 }
 
-
 class SagaEngine : public Engine {
 	friend class Scene;
 
-	void errorString(const char *buf_input, char *buf_output);
-
 protected:
 	int go();
 	int init();
@@ -330,9 +557,7 @@ public:
 	const char *getObjectName(uint16 objectId);
 public:
 	int processInput(void);
-	const Point &mousePos() const {
-		return _mousePos;
-	}
+	Point mousePos() const;
 
 	const bool leftMouseButtonPressed() const {
 		return _leftMouseButtonPressed;
@@ -353,7 +578,6 @@ public:
 	bool _saveMarks[MAX_SAVES];
 	SaveGameHeader _saveHeader;
 
-	Point _mousePos;
 	bool _leftMouseButtonPressed;
 	bool _rightMouseButtonPressed;
 
@@ -361,46 +585,43 @@ public:
 
 //current game description
 	int _gameNumber;
-	GameDescription *_gameDescription;
+	const SAGAGameDescription *_gameDescription;
 	Common::String _gameTitle;
 	Common::Rect _displayClip;
 
-protected:
-	GameDisplayInfo _gameDisplayInfo;
-
 public:
 	int32 _frameCount;
 
 public:
 	bool initGame(void);
-public:
-	const GameDescription *getGameDescription() const { return _gameDescription; }
-	const bool isBigEndian() const { return (_gameDescription->features & GF_BIG_ENDIAN_DATA) != 0; }
-	const bool isMacResources() const { return (getPlatform() == Common::kPlatformMacintosh); }
-	const GameResourceDescription *getResourceDescription() { return _gameDescription->resourceDescription; }
-	const GameSoundInfo *getVoiceInfo() const { return _gameDescription->voiceInfo; }
-	const GameSoundInfo *getSfxInfo() const { return _gameDescription->sfxInfo; }
-	const GameSoundInfo *getMusicInfo() const { return _gameDescription->musicInfo; }
 
-	const GameFontDescription *getFontDescription(int index) {
-		assert(index < _gameDescription->fontsCount);
-		return &_gameDescription->fontDescriptions[index];
-	}
-	int getFontsCount() const { return _gameDescription->fontsCount; }
+	const bool isBigEndian() const;
+	const bool isMacResources() const;
+	const GameResourceDescription *getResourceDescription();
+	const GameSoundInfo *getVoiceInfo() const;
+	const GameSoundInfo *getSfxInfo() const;
+	const GameSoundInfo *getMusicInfo() const;
 
-	int getGameId() const { return _gameDescription->gameId; }
-	int getGameType() const { return _gameDescription->gameType; }
-	uint32 getFeatures() const { return _gameDescription->features; }
-	Common::Language getLanguage() const { return _gameDescription->language; }
-	Common::Platform getPlatform() const { return _gameDescription->platform; }
-	int getGameNumber() const { return _gameNumber; }
-	int getStartSceneNumber() const { return _gameDescription->startSceneNumber; }
+	const GameFontDescription *getFontDescription(int index);
+	int getFontsCount() const;
 
+	int getGameId() const;
+	int getGameType() const;
+	uint32 getFeatures() const;
+	Common::Language getLanguage() const;
+	Common::Platform getPlatform() const;
+	int getGameNumber() const;
+	int getStartSceneNumber() const;
+
+	int getPatchesCount() const;
+	const GamePatchDescription *getPatchDescriptions() const;
+
+	const Common::ADGameFileDescription *getFilesDescriptions() const;
 
 	const Common::Rect &getDisplayClip() const { return _displayClip;}
-	int getDisplayWidth() const { return _gameDisplayInfo.logicalWidth; }
-	int getDisplayHeight() const { return _gameDisplayInfo.logicalHeight;}
-	const GameDisplayInfo & getDisplayInfo() { return _gameDisplayInfo; }
+	int getDisplayWidth() const;
+	int getDisplayHeight() const;
+	const GameDisplayInfo &getDisplayInfo();
 
 	const char *getTextString(int textStringId);
 	void getExcuseInfo(int verb, const char *&textString, int &soundResourceId);
