@@ -1,7 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2004-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
  *
- * The ReInherit Engine is (C)2000-2003 by Daniel Balsom.
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -17,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/saga/saveload.cpp $
- * $Id: saveload.cpp 22977 2006-06-07 09:31:52Z sev $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/saga/saveload.cpp $
+ * $Id: saveload.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -211,7 +212,7 @@ void SagaEngine::save(const char *fileName, const char *saveName) {
 	out->writeSint16LE(_isoMap->getMapPosition().x);
 	out->writeSint16LE(_isoMap->getMapPosition().y);
 
-	out->flush();
+	out->finalize();
 
 	// TODO: Check out->ioFailed()
 
