@@ -1,7 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2004-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
  *
- * The ReInherit Engine is (C)2000-2003 by Daniel Balsom.
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -17,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/saga/scene.cpp $
- * $Id: scene.cpp 22433 2006-05-13 10:30:38Z h00ligan $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/saga/scene.cpp $
+ * $Id: scene.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -45,7 +46,7 @@
 #include "saga/rscfile.h"
 #include "saga/sagaresnames.h"
 
-#include "graphics/ilbm.h"
+#include "graphics/iff.h"
 #include "common/util.h"
 
 namespace Saga {
@@ -437,7 +438,7 @@ void Scene::changeScene(int16 sceneNumber, int actorsEntrance, SceneTransitionTy
 				_vm->_interface->setMode(kPanelSceneSubstitute);
 
 				if (file.open(sceneSubstitutes[i].image)) {
-					Graphics::decodeILBM(file, bbmBuffer, pal);
+					Graphics::decodePBM(file, bbmBuffer, pal);
 					colors = pal;
 					rect.setWidth(bbmBuffer.w);
 					rect.setHeight(bbmBuffer.h);
@@ -502,28 +503,6 @@ void Scene::getBGInfo(BGInfo &bgInfo) {
 	bgInfo.bounds.setHeight(_bg.h);
 }
 
-int Scene::getBGMaskType(const Point &testPoint) {
-	uint offset;
-	if (!_bgMask.loaded) {
-		return 0;
-	}
-	offset = testPoint.x + testPoint.y * _bgMask.w;
-	if (offset >= _bgMask.buf_len) {
-		error("Scene::getBGMaskType offset 0x%X exceed bufferLength 0x%X", offset, (int)_bgMask.buf_len);
-	}
-
-	return (_bgMask.buf[offset] >> 4) & 0x0f;
-}
-
-bool Scene::validBGMaskPoint(const Point &testPoint) {
-	if (!_bgMask.loaded) {
-		error("Scene::validBGMaskPoint _bgMask not loaded");
-	}
-
-	return !((testPoint.x < 0) || (testPoint.x >= _bgMask.w) ||
-		(testPoint.y < 0) || (testPoint.y >= _bgMask.h));
-}
-
 bool Scene::canWalk(const Point &testPoint) {
 	int maskType;
 
@@ -571,20 +550,6 @@ void Scene::getBGMaskInfo(int &width, int &height, byte *&buffer, size_t &buffer
 	bufferLength = _bgMask.buf_len;
 }
 
-void Scene::setDoorState(int doorNumber, int doorState) {
-	if ((doorNumber < 0) || (doorNumber >= SCENE_DOORS_MAX))
-		error("Scene::setDoorState wrong doorNumber");
-
-	_sceneDoors[doorNumber] = doorState;
-}
-
-int Scene::getDoorState(int doorNumber) {
-	if ((doorNumber < 0) || (doorNumber >= SCENE_DOORS_MAX))
-		error("Scene::getDoorState wrong doorNumber");
-
-	return _sceneDoors[doorNumber];
-}
-
 void Scene::initDoorsState() {
 	memcpy(_sceneDoors, initSceneDoors, sizeof (_sceneDoors) );
 }
@@ -594,7 +559,14 @@ void Scene::loadScene(LoadSceneParams *loadSceneParams) {
 	Event event;
 	Event *q_event;
 	static PalEntry current_pal[PAL_ENTRIES];
-	
+
+	// Change the cursor to an hourglass in IHNM
+	event.type = kEvTOneshot;
+	event.code = kCursorEvent;
+	event.op = kEventSetBusyCursor;
+	event.time = 0;
+	_vm->_events->queue(&event);
+
 	if ((_vm->getGameType() == GType_IHNM) && (loadSceneParams->chapter != NO_CHAPTER_CHANGE)) {
 		if (loadSceneParams->loadFlag != kLoadBySceneNumber) {
 			error("loadScene wrong usage");
@@ -680,7 +652,7 @@ void Scene::loadScene(LoadSceneParams *loadSceneParams) {
 			_vm->_resource->loadResource(_sceneContext, _resourceList[i].resourceId,
 				_resourceList[i].buffer, _resourceList[i].size);
 
-			
+
 			if (_resourceList[i].size >= 6) {
 				if (!memcmp(_resourceList[i].buffer, "DUMMY!", 6)) {
 					_resourceList[i].invalid = true;
@@ -874,8 +846,6 @@ void Scene::loadScene(LoadSceneParams *loadSceneParams) {
 		loadSceneParams->sceneProc(SCENE_BEGIN, this);
 	}
 
-
-
 	// We probably don't want "followers" to go into scene -1 , 0. At the very
 	// least we don't want garbage to be drawn that early in the ITE intro.
 	if (_sceneNumber > 0 && _sceneNumber != ITE_SCENE_PUZZLE)
@@ -892,12 +862,19 @@ void Scene::loadScene(LoadSceneParams *loadSceneParams) {
 		event.time = 0;
 		_vm->_events->queue(&event);
 	}
+
+	// Change the cursor back to a crosshair in IHNM
+	event.type = kEvTOneshot;
+	event.code = kCursorEvent;
+	event.op = kEventSetNormalCursor;
+	event.time = 0;
+	_vm->_events->queue(&event);
 }
 
 void Scene::loadSceneDescriptor(uint32 resourceId) {
 	byte *sceneDescriptorData;
 	size_t sceneDescriptorDataLength;
-	
+
 	memset(&_sceneDescription, 0, sizeof(_sceneDescription));
 
 	if (resourceId == 0) {
@@ -970,7 +947,7 @@ void Scene::processSceneResources() {
 	SAGAResourceTypes resType;
 
 	getResourceTypes(types, typesCount);
-	
+
 	// Process the scene resource list
 	for (i = 0; i < _resourceListCount; i++) {
 		if (_resourceList[i].invalid) {
