@@ -1,7 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2004-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
  *
- * The ReInherit Engine is (C)2000-2003 by Daniel Balsom.
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -17,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/saga/scene.h $
- * $Id: scene.h 20532 2006-02-11 12:44:16Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/saga/scene.h $
+ * $Id: scene.h 27067 2007-06-03 16:14:57Z thebluegr $
  *
  */
 
@@ -34,6 +35,8 @@
 
 namespace Saga {
 
+//#define SCENE_DEBUG // for scene debugging
+
 #define SCENE_DOORS_MAX 16
 
 #define NO_CHAPTER_CHANGE -2
@@ -233,13 +236,55 @@ class Scene {
 
 	void getBGMaskInfo(int &width, int &height, byte *&buffer, size_t &bufferLength);
 	int isBGMaskPresent() { return _bgMask.loaded; }
-	int getBGMaskType(const Point &testPoint);
-	bool validBGMaskPoint(const Point &testPoint);
+
+	int getBGMaskType(const Point &testPoint) {
+		uint offset;
+		if (!_bgMask.loaded) {
+			return 0;
+		}
+		offset = testPoint.x + testPoint.y * _bgMask.w;
+
+		#ifdef SCENE_DEBUG
+		if (offset >= _bgMask.buf_len) {
+			error("Scene::getBGMaskType offset 0x%X exceed bufferLength 0x%X", offset, (int)_bgMask.buf_len);
+		}
+		#endif
+
+		return (_bgMask.buf[offset] >> 4) & 0x0f;
+	}
+
+	bool validBGMaskPoint(const Point &testPoint) {
+		#ifdef SCENE_DEBUG
+		if (!_bgMask.loaded) {
+			error("Scene::validBGMaskPoint _bgMask not loaded");
+		}
+		#endif
+
+		return !((testPoint.x < 0) || (testPoint.x >= _bgMask.w) ||
+			(testPoint.y < 0) || (testPoint.y >= _bgMask.h));
+	}
+
 	bool canWalk(const Point &testPoint);
 	bool offscreenPath(Point &testPoint);
 
-	void setDoorState(int doorNumber, int doorState);
-	int getDoorState(int doorNumber);
+	void setDoorState(int doorNumber, int doorState) {
+		#ifdef SCENE_DEBUG
+		if ((doorNumber < 0) || (doorNumber >= SCENE_DOORS_MAX))
+			error("Scene::setDoorState wrong doorNumber");
+		#endif
+
+		_sceneDoors[doorNumber] = doorState;
+	}
+
+	int getDoorState(int doorNumber) {
+		#ifdef SCENE_DEBUG
+		if ((doorNumber < 0) || (doorNumber >= SCENE_DOORS_MAX))
+			error("Scene::getDoorState wrong doorNumber");
+		#endif
+
+		return _sceneDoors[doorNumber];
+	}
+
 	void initDoorsState();
 
 	void getBGInfo(BGInfo &bgInfo);
@@ -257,9 +302,11 @@ class Scene {
 	bool isSceneLoaded() const { return _sceneLoaded; }
 
 	int getSceneResourceId(int sceneNumber) {
+	#ifdef SCENE_DEBUG
 		if ((sceneNumber < 0) || (sceneNumber >= _sceneCount)) {
  			error("getSceneResourceId: wrong sceneNumber %i", sceneNumber);
 		}
+	#endif
 		return _sceneLUT[sceneNumber];
 	}
 	int currentSceneNumber() const { return _sceneNumber; }
@@ -277,10 +324,10 @@ class Scene {
 	void drawTextList(Surface *ds);
 
 	int getHeight() const {
-		if (_vm->_interface->getMode() == kPanelChapterSelection)
-			return _vm->_gameDisplayInfo.logicalHeight;
+		if (_vm->getGameType() == GType_IHNM && _vm->_scene->currentChapterNumber() == 8)
+			return _vm->getDisplayInfo().logicalHeight;
 		else
-			return _vm->_gameDisplayInfo.sceneHeight;
+			return _vm->getDisplayInfo().sceneHeight;
 	}
 
  private:
