@@ -1,7 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2004-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
  *
- * The ReInherit Engine is (C)2000-2003 by Daniel Balsom.
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -17,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/saga/script.cpp $
- * $Id: script.cpp 21141 2006-03-08 11:37:25Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/saga/script.cpp $
+ * $Id: script.cpp 27121 2007-06-06 00:48:27Z thebluegr $
  *
  */
 
@@ -287,7 +288,10 @@ void Script::showVerb(int statusColor) {
 		return;
 	}
 
-	verbName = _mainStrings.getString(_leftButtonVerb - 1);
+	if (_vm->getGameType() == GType_ITE)
+		verbName = _mainStrings.getString(_leftButtonVerb - 1);
+	else
+		verbName = _mainStrings.getString(_leftButtonVerb + 1);
 
 	if (objectTypeId(_currentObject[0]) == kGameObjectNone) {
 		_vm->_interface->setStatusText(verbName, statusColor);
@@ -355,27 +359,29 @@ int Script::getVerbType(VerbTypes verbType) {
 		}
 	}
 	else {
+		// TODO: This is ugly and needs rewriting, but
+		// it works for now
 		switch (verbType) {
 		case kVerbNone:
 			return kVerbIHNMNone;
 		case kVerbWalkTo:
 			return kVerbIHNMWalk;
-		case kVerbGive:
-			return kVerbIHNMGive;
-		case kVerbUse:
-			return kVerbIHNMUse;
-		case kVerbEnter:
-			return kVerbIHNMEnter;
 		case kVerbLookAt:
 			return kVerbIHNMLookAt;
 		case kVerbPickUp:
 			return kVerbIHNMTake;
-		case kVerbOpen:
-			return -2;
-		case kVerbClose:
-			return -2;
+		case kVerbUse:
+			return kVerbIHNMUse;
 		case kVerbTalkTo:
 			return kVerbIHNMTalkTo;
+		case kVerbOpen:
+			return kVerbIHNMSwallow;
+		case kVerbGive:
+			return kVerbIHNMGive;
+		case kVerbClose:
+			return kVerbIHNMPush;
+		case kVerbEnter:
+			return kVerbIHNMEnter;
 		case kVerbWalkOnly:
 			return kVerbIHNMWalkOnly;
 		case kVerbLookOnly:
@@ -458,7 +464,9 @@ void Script::doVerb() {
 			}
 		}
 
-		if (objectType == kGameObjectHitZone) {
+		if (objectType == 0)
+			return;
+		else if (objectType == kGameObjectHitZone) {
 			scriptModuleNumber = _vm->_scene->getScriptModuleNumber();
 			hitZone = _vm->_scene->_objectMap->getHitZone(objectIdToIndex(_pendingObject[0]));
 			if ((hitZone->getFlags() & kHitZoneExit) == 0) {
@@ -709,8 +717,10 @@ void Script::whichObject(const Point& mousePoint) {
 						(_currentVerb == getVerbType(kVerbClose)) ||
 						((_currentVerb == getVerbType(kVerbGive)) && !_firstObjectSet) ||
 						((_currentVerb == getVerbType(kVerbUse)) && !(actor->_flags & kFollower))) {
-							objectId = ID_NOTHING;
-							newObjectId = ID_NOTHING;
+							if (_vm->getGameType() == GType_ITE) {
+								objectId = ID_NOTHING;
+								newObjectId = ID_NOTHING;
+							}
 						}
 				}
 			}
