@@ -1,7 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2004-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
  *
- * The ReInherit Engine is (C)2000-2003 by Daniel Balsom.
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -17,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/saga/script.h $
- * $Id: script.h 22433 2006-05-13 10:30:38Z h00ligan $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/saga/script.h $
+ * $Id: script.h 27055 2007-06-02 15:26:05Z thebluegr $
  *
  */
 
@@ -325,6 +326,11 @@ public:
 		_sleepTime = sleepTime;
 	}
 
+	void waitFrames(int frames) {
+		wait(kWaitTypeWaitFrames);
+		_frameWait = frames;
+	}
+
 	ScriptThread() {
 		memset(this, 0xFE, sizeof(*this));
 		_stackBuf = NULL;
