@@ -1,7 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2004-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
  *
- * The ReInherit Engine is (C)2000-2003 by Daniel Balsom.
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -17,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/saga/sfuncs.cpp $
- * $Id: sfuncs.cpp 22433 2006-05-13 10:30:38Z h00ligan $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/saga/sfuncs.cpp $
+ * $Id: sfuncs.cpp 27119 2007-06-05 22:39:36Z Kirben $
  *
  */
 
@@ -147,7 +148,7 @@ static const ScriptFunctionDescription IHNMscriptFunctionsList[IHNM_SCRIPT_FUNCT
 		OPCODE(sfSetActorFacing),
 		OPCODE(sfStartBgdAnim),
 		OPCODE(sfStopBgdAnim),
-		OPCODE(sfNull),
+		OPCODE(sfLockUser),
 		OPCODE(sfPreDialog),
 		OPCODE(sfKillActorThreads),
 		OPCODE(sfFaceTowards),
@@ -385,11 +386,13 @@ void Script::sfScriptDoAction(SCRIPTFUNC_PARAMS) {
 			break;
 		case kGameObjectHitZone:
 		case kGameObjectStepZone:
-			if (objectTypeId(objectId) == kGameObjectHitZone) {
+			if (objectTypeId(objectId) == 0)
+				return;
+			else if (objectTypeId(objectId) == kGameObjectHitZone)
 				hitZone = _vm->_scene->_objectMap->getHitZone(objectIdToIndex(objectId));
-			} else {
+			else
 				hitZone = _vm->_scene->_actionMap->getHitZone(objectIdToIndex(objectId));
-			}
+
 			scriptEntryPointNumber = hitZone->getScriptNumber();
 			moduleNumber = _vm->_scene->getScriptModuleNumber();
 			break;
@@ -728,12 +731,21 @@ void Script::sfEnableZone(SCRIPTFUNC_PARAMS) {
 	int16 flag = thread->pop();
 	HitZone *hitZone;
 
-	if (objectTypeId(objectId) == kGameObjectHitZone) {
-		hitZone = _vm->_scene->_objectMap->getHitZone(objectIdToIndex(objectId));
-	} else {
-		hitZone = _vm->_scene->_actionMap->getHitZone(objectIdToIndex(objectId));
+	// HACK: Don't disable the tear in scene 14, to keep the staircase functioning
+	// FIXME: Investigate why this hack is needed and remove it
+	if (_vm->getGameType() == GType_IHNM && _vm->_scene->currentChapterNumber() == 1 &&
+		_vm->_scene->currentSceneNumber() == 14) {
+		warning("sfEnableZone: HACK: Prevent unusable staircase");
+		return;		// Do nothing
 	}
 
+	if (objectTypeId(objectId) == 0)
+		return;
+	else if (objectTypeId(objectId) == kGameObjectHitZone)
+		hitZone = _vm->_scene->_objectMap->getHitZone(objectIdToIndex(objectId));
+	else
+		hitZone = _vm->_scene->_actionMap->getHitZone(objectIdToIndex(objectId));
+
 	if (flag) {
 		hitZone->setFlag(kHitZoneEnabled);
 	} else {
@@ -1115,7 +1127,7 @@ void Script::sfPlaceActor(SCRIPTFUNC_PARAMS) {
 		frameRange = _vm->_actor->getActorFrameRange(actorId, frameType);
 
 		if (frameRange->frameCount <= frameOffset) {
-			error("Wrong frameOffset 0x%X", frameOffset);
+			warning("Wrong frameOffset 0x%X", frameOffset);
 		}
 
 		actor->_frameNumber = frameRange->frameIndex + frameOffset;
@@ -1886,8 +1898,14 @@ void Script::sfResetMouseClicks(SCRIPTFUNC_PARAMS) {
 	SF_stub("sfResetMouseClicks", thread, nArgs);
 }
 
+// Used in IHNM only
+// Param1: frames
 void Script::sfWaitFrames(SCRIPTFUNC_PARAMS) {
-	SF_stub("sfWaitFrames", thread, nArgs);
+	int16 frames;
+	frames = thread->pop();
+
+	if (!_skipSpeeches)
+		thread->waitFrames(_vm->_frameCount + frames);
 }
 
 void Script::sfScriptFade(SCRIPTFUNC_PARAMS) {
