@@ -1,6 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2001  Ludvig Strigeus
- * Copyright (C) 2001-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -16,14 +18,14 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/scumm/actor.h $
- * $Id: actor.h 23590 2006-07-24 12:52:53Z kirben $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/scumm/actor.h $
+ * $Id: actor.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
 
-#ifndef ACTOR_H
-#define ACTOR_H
+#ifndef SCUMM_ACTOR_H
+#define SCUMM_ACTOR_H
 
 #include "common/scummsys.h"
 #include "scumm/saveload.h"
@@ -32,6 +34,15 @@
 
 namespace Scumm {
 
+
+enum {
+	V12_X_MULTIPLIER = 8,
+	V12_Y_MULTIPLIER = 2,
+
+	V12_X_SHIFT = 3,
+	V12_Y_SHIFT = 1
+};
+
 enum MoveFlags {
 	MF_NEW_LEG = 1,
 	MF_IN_LEG = 2,
@@ -40,21 +51,6 @@ enum MoveFlags {
 	MF_FROZEN = 0x80
 };
 
-struct ActorWalkData {
-	Common::Point dest;           // Final destination point
-	byte destbox;                 // Final destination box
-	int16 destdir;                // Final destination, direction to face at
-
-	Common::Point cur;            // Last position
-	byte curbox;                  // Last box
-
-	Common::Point next;           // Next position on our way to the destination, i.e. our intermediate destination
-
-	Common::Point point3;
-	int32 deltaXFactor, deltaYFactor;
-	uint16 xfrac, yfrac;
-};
-
 struct CostumeData {
 	byte active[16];
 	uint16 animCounter;
@@ -83,17 +79,22 @@ struct AdjustBoxResult {	/* Result type of AdjustBox functions */
 	byte box;
 };
 
-class Actor : public Serializable {
+enum {
+	kOldInvalidBox = 255,	// For small header games
+	kNewInavlidBox = 0
+};
 
+class Actor : public Serializable {
 public:
 	static byte kInvalidBox;
 
-	static void initActorClass(ScummEngine *scumm);
+protected:
+	ScummEngine *_vm;
 
-public:
 	/** The position of the actor inside the virtual screen. */
 	Common::Point _pos;
 
+public:
 	/** HE specific: This rect is used to clip actor drawing. */
 	Common::Rect _clipOverride;
 
@@ -103,6 +104,8 @@ public:
 	byte _number;
 	uint16 _costume;
 	byte _room;
+
+public:
 	byte _talkColor;
 	int _talkFrequency;
 	byte _talkPan;
@@ -133,9 +136,6 @@ public:
 	uint16 _sound[32];
 	CostumeData _cost;
 
-	/* C64 maniac specific */
-	byte _miscflags;
-
 	/* HE specific */
 	bool _heNoTalkAnimation;
 	bool _heSkipLimbs;
@@ -155,6 +155,22 @@ public:
 	} _heTalkQueue[16];
 
 protected:
+	struct ActorWalkData {
+		Common::Point dest;           // Final destination point
+		byte destbox;                 // Final destination box
+		int16 destdir;                // Final destination, direction to face at
+
+		Common::Point cur;            // Last position
+		byte curbox;                  // Last box
+
+		Common::Point next;           // Next position on our way to the destination, i.e. our intermediate destination
+
+		Common::Point point3;
+		int32 deltaXFactor, deltaYFactor;
+		uint16 xfrac, yfrac;
+	};
+
+
 	byte _palette[256];
 	int _elevation;
 	uint16 _facing;
@@ -165,37 +181,48 @@ protected:
 	ActorWalkData _walkdata;
 	int16 _animVariable[27];
 
-	static ScummEngine *_vm;
-
 public:
 
-	Actor();
+	Actor(ScummEngine *scumm, int id);
+	virtual ~Actor() {}
 
 //protected:
 	void hideActor();
 	void showActor();
 
-	void initActor(int mode);
-	void putActor(int x, int y, byte room);
+	virtual void initActor(int mode);
+
+	void putActor() {
+		putActor(_pos.x, _pos.y, _room);
+	}
+
+	void putActor(int room) {
+		putActor(_pos.x, _pos.y, room);
+	}
+
+	void putActor(int x, int y) {
+		putActor(x, y, _room);
+	}
+
+	void putActor(int x, int y, int room);
 	void setActorWalkSpeed(uint newSpeedX, uint newSpeedY);
 protected:
 	int calcMovementFactor(const Common::Point& next);
 	int actorWalkStep();
 	int remapDirection(int dir, bool is_walking);
-	void setupActorScale();
+	virtual void setupActorScale();
 
 	void setBox(int box);
 	int updateActorDirection(bool is_walking);
 
 public:
 	void adjustActorPos();
-	AdjustBoxResult adjustXYToBeInBox(int dstX, int dstY);
+	virtual AdjustBoxResult adjustXYToBeInBox(int dstX, int dstY);
 
 	void setDirection(int direction);
 	void faceToObject(int obj);
 	void turnToDirection(int newdir);
-	void walkActor();
-	void walkActorOld();
+	virtual void walkActor();
 	void drawActorToBackBuf(int x, int y);
 	void drawActorCostume(bool hitTestMode = false);
 	void animateCostume();
@@ -205,7 +232,7 @@ public:
 
 	bool actorHitTest(int x, int y);
 
-	byte *getActorName();
+	const byte *getActorName();
 	void startWalkActor(int x, int y, int dir);
 	void stopActorMoving();
 protected:
@@ -223,7 +250,18 @@ public:
 		return _room == _vm->_currentRoom;
 	}
 
-	int getActorXYPos(int &x, int &y) const;
+	Common::Point getPos() const {
+		Common::Point p(_pos);
+		if (_vm->_game.version <= 2) {
+			p.x *= V12_X_MULTIPLIER;
+			p.y *= V12_Y_MULTIPLIER;
+		}
+		return p;
+	}
+
+	const Common::Point& getRealPos() const {
+		return _pos;
+	}
 
 	int getRoom() const {
 		return _room;
@@ -293,12 +331,53 @@ public:
 protected:
 	bool isInClass(int cls);
 
-	bool isPlayer();
+	virtual bool isPlayer();
 
 	bool findPathTowards(byte box, byte box2, byte box3, Common::Point &foundPath);
+};
+
+class Actor_v3 : public Actor {
+public:
+	Actor_v3(ScummEngine *scumm, int id) : Actor(scumm, id) {}
+
+	virtual void walkActor();
+
+protected:
+	virtual void setupActorScale();
 	void findPathTowardsOld(byte box, byte box2, byte box3, Common::Point &p2, Common::Point &p3);
 };
 
+class Actor_v2 : public Actor_v3 {
+public:
+	Actor_v2(ScummEngine *scumm, int id) : Actor_v3(scumm, id) {}
+
+	virtual void initActor(int mode);
+	virtual void walkActor();
+	virtual AdjustBoxResult adjustXYToBeInBox(int dstX, int dstY);
+
+protected:
+	virtual bool isPlayer();
+};
+
+class ActorC64 : public Actor_v2 {
+public:
+	// FIXME: This flag is never saved, which might lead to broken save states.
+	byte _miscflags;
+
+public:
+	ActorC64(ScummEngine *scumm, int id) : Actor_v2(scumm, id) {}
+	virtual void initActor(int mode) {
+		Actor_v2::initActor(mode);
+		if (mode == -1) {
+			_miscflags = 0;
+		}
+	}
+
+protected:
+
+};
+
+
 } // End of namespace Scumm
 
 #endif
