@@ -1,6 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2001  Ludvig Strigeus
- * Copyright (C) 2001-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -16,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/scumm/akos.cpp $
- * $Id: akos.cpp 24105 2006-10-04 05:16:44Z eriktorbjorn $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/scumm/akos.cpp $
+ * $Id: akos.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -36,9 +38,7 @@
 
 namespace Scumm {
 
-#if !defined(__GNUC__)
-#pragma START_PACK_STRUCTS
-#endif
+#include "common/pack-start.h"	// START STRUCT PACKING
 
 struct AkosHeader {
 	byte unk_1[2];
@@ -47,16 +47,14 @@ struct AkosHeader {
 	uint16 num_anims;
 	uint16 unk_3;
 	uint16 codec;
-} GCC_PACK;
+};
 
 struct AkosOffset {
-	uint32 akcd;
-	uint16 akci;
-} GCC_PACK;
+	uint32 akcd;	// offset into the akcd data
+	uint16 akci;	// offset into the akci data
+};
 
-#if !defined(__GNUC__)
-#pragma END_PACK_STRUCTS
-#endif
+#include "common/pack-end.h"	// END STRUCT PACKING
 
 
 enum AkosOpcodes {
@@ -304,30 +302,30 @@ void AkosRenderer::setPalette(byte *new_palette) {
 
 	if (_vm->_game.heversion >= 99 && _paletteNum) {
 		for (i = 0; i < size; i++)
-			palette[i] = (byte)_vm->_hePalettes[_paletteNum * 1024 + 768 + akpl[i]];
+			_palette[i] = (byte)_vm->_hePalettes[_paletteNum * 1024 + 768 + akpl[i]];
 	} else {
 		for (i = 0; i < size; i++) {
-			palette[i] = new_palette[i] != 0xFF ? new_palette[i] : akpl[i];
+			_palette[i] = new_palette[i] != 0xFF ? new_palette[i] : akpl[i];
 		}
 	}
 
 	if (_vm->_game.heversion == 70) {
 		for (i = 0; i < size; i++)
-			palette[i] = _vm->_HEV7ActorPalette[palette[i]];
+			_palette[i] = _vm->_HEV7ActorPalette[_palette[i]];
 	} 
 
 	if (size == 256) {
 		byte color = new_palette[0];
 		if (color == 255) {
-			palette[0] = color;
+			_palette[0] = color;
 		} else {
-			_vm->_bompActorPalettePtr = palette;
+			_useBompPalette = true;
 		}
 	}
 }
 
 void AkosRenderer::setCostume(int costume, int shadow) {
-	akos = _vm->getResourceAddress(rtCostume, costume);
+	const byte *akos = _vm->getResourceAddress(rtCostume, costume);
 	assert(akos);
 
 	akhd = (const AkosHeader *) _vm->findResourceData(MKID_BE('AKHD'), akos);
@@ -336,7 +334,7 @@ void AkosRenderer::setCostume(int costume, int shadow) {
 	aksq = _vm->findResourceData(MKID_BE('AKSQ'), akos);
 	akcd = _vm->findResourceData(MKID_BE('AKCD'), akos);
 	akpl = _vm->findResourceData(MKID_BE('AKPL'), akos);
-	codec = READ_LE_UINT16(&akhd->codec);
+	_codec = READ_LE_UINT16(&akhd->codec);
 	akct = _vm->findResourceData(MKID_BE('AKCT'), akos);
 
 	xmap = 0;
@@ -424,7 +422,7 @@ byte AkosRenderer::drawLimb(const Actor *a, int limb) {
 		_xmove += (int16)READ_LE_UINT16(&costumeInfo->move_x);
 		_ymove -= (int16)READ_LE_UINT16(&costumeInfo->move_y);
 
-		switch (codec) {
+		switch (_codec) {
 		case 1:
 			result |= codec1(xmoveCur, ymoveCur);
 			break;
@@ -435,7 +433,7 @@ byte AkosRenderer::drawLimb(const Actor *a, int limb) {
 			result |= codec16(xmoveCur, ymoveCur);
 			break;
 		default:
-			error("akos_drawLimb: invalid codec %d", codec);
+			error("akos_drawLimb: invalid _codec %d", _codec);
 		}
 	} else {
 		if (code == AKC_ComplexChan2)  {
@@ -507,7 +505,7 @@ byte AkosRenderer::drawLimb(const Actor *a, int limb) {
 					_shadow_mode = 3;
 			}
 
-			switch (codec) {
+			switch (_codec) {
 			case 1:
 				result |= codec1(xmoveCur, ymoveCur);
 				break;
@@ -521,7 +519,7 @@ byte AkosRenderer::drawLimb(const Actor *a, int limb) {
 				result |= codec32(xmoveCur, ymoveCur);
 				break;
 			default:
-				error("akos_drawLimb: invalid codec %d", codec);
+				error("akos_drawLimb: invalid _codec %d", _codec);
 			}
 		}
 	}
@@ -561,7 +559,7 @@ void AkosRenderer::codec1_genericDecode(Codec1 &v1) {
 			len = *src++;
 
 		do {
-			if (*scaleytab++ < _scaleY) {
+			if (_scaleY == 255 || *scaleytab++ < _scaleY) {
 				if (_actorHitMode) {
 					if (color && y == _actorHitY && v1.x == _actorHitX) {
 						_actorHitResult = true;
@@ -571,7 +569,7 @@ void AkosRenderer::codec1_genericDecode(Codec1 &v1) {
 					masked = (y < v1.boundsRect.top || y >= v1.boundsRect.bottom) || (v1.x < 0 || v1.x >= v1.boundsRect.right) || (*mask & maskbit);
 
 					if (color && !masked && !skip_column) {
-						pcolor = palette[color];
+						pcolor = _palette[color];
 						if (_shadow_mode == 1) {
 							if (pcolor == 13)	
 								pcolor = _shadow_table[*dst];
@@ -781,7 +779,7 @@ byte AkosRenderer::codec1(int xmoveCur, int ymoveCur) {
 	// So I had to put copy of it back here as it was before 1.227 revision
 	// of this file.
 	v1.scaletable = (_vm->_game.heversion >= 61) ? smallCostumeScaleTableAKOS : bigCostumeScaleTable;
-	if (_vm->VAR_CUSTOMSCALETABLE != 0xFF && _vm->res.isResourceLoaded(rtString, _vm->VAR(_vm->VAR_CUSTOMSCALETABLE))) {
+	if (_vm->VAR_CUSTOMSCALETABLE != 0xFF && _vm->_res->isResourceLoaded(rtString, _vm->VAR(_vm->VAR_CUSTOMSCALETABLE))) {
 		v1.scaletable = _vm->getStringAddressVar(_vm->VAR_CUSTOMSCALETABLE);
 	}
 
@@ -1029,14 +1027,7 @@ byte AkosRenderer::codec5(int xmoveCur, int ymoveCur) {
 
 	BompDrawData bdd;
 
-	bdd.srcwidth = _width;
-	bdd.srcheight = _height;
 	bdd.dst = _out;
-	bdd.dataptr = _srcptr;
-	bdd.scale_x = 255;
-	bdd.scale_y = 255;
-	bdd.shadowMode = _shadow_mode;
-
 	if (!_mirror) {
 		bdd.x = (_actorX - xmoveCur - _width) + 1;
 	} else {
@@ -1044,33 +1035,48 @@ byte AkosRenderer::codec5(int xmoveCur, int ymoveCur) {
 	}
 	bdd.y = _actorY + ymoveCur;
 
-	bdd.maskPtr = _vm->getMaskBuffer(0, 0, _zbuf);
-	_vm->drawBomp(bdd, !_mirror);
+	bdd.src = _srcptr;
+	bdd.srcwidth = _width;
+	bdd.srcheight = _height;
 
-	_vm->_bompActorPalettePtr = NULL;
+	bdd.scale_x = 255;
+	bdd.scale_y = 255;
+
+	bdd.maskPtr = _vm->getMaskBuffer(0, 0, _zbuf);
+	bdd.numStrips = _numStrips;
+
+	bdd.shadowMode = _shadow_mode;
+	bdd.shadowPalette = _vm->_shadowPalette;
+
+	bdd.actorPalette = _useBompPalette ? _palette : 0;
+	bdd.mirror = !_mirror;
+
+	drawBomp(bdd);
+	
+	_useBompPalette = false;
 	
 	return 0;
 }
 
 void AkosRenderer::akos16SetupBitReader(const byte *src) {
-	akos16.unk5 = 0;
-	akos16.numbits = 16;
-	akos16.mask = (1 << *src) - 1;
-	akos16.shift = *(src);
-	akos16.color = *(src + 1);
-	akos16.bits = (*(src + 2) | *(src + 3) << 8);
-	akos16.dataptr = src + 4;
+	_akos16.repeatMode = false;
+	_akos16.numbits = 16;
+	_akos16.mask = (1 << *src) - 1;
+	_akos16.shift = *(src);
+	_akos16.color = *(src + 1);
+	_akos16.bits = (*(src + 2) | *(src + 3) << 8);
+	_akos16.dataptr = src + 4;
 }
 
 #define AKOS16_FILL_BITS()                                        \
-        if (akos16.numbits <= 8) {                                \
-          akos16.bits |= (*akos16.dataptr++) << akos16.numbits;   \
-          akos16.numbits += 8;                                    \
+        if (_akos16.numbits <= 8) {                                \
+          _akos16.bits |= (*_akos16.dataptr++) << _akos16.numbits;   \
+          _akos16.numbits += 8;                                    \
         }
 
 #define AKOS16_EAT_BITS(n)                                        \
-		akos16.numbits -= (n);                                    \
-		akos16.bits >>= (n);
+		_akos16.numbits -= (n);                                    \
+		_akos16.bits >>= (n);
 
 
 void AkosRenderer::akos16SkipData(int32 numbytes) {
@@ -1082,39 +1088,41 @@ void AkosRenderer::akos16DecodeLine(byte *buf, int32 numbytes, int32 dir) {
 
 	while (numbytes != 0) {
 		if (buf) {
-			*buf = akos16.color;
+			*buf = _akos16.color;
 			buf += dir;
 		}
 		
-		if (akos16.unk5 == 0) {
+		if (!_akos16.repeatMode) {
 			AKOS16_FILL_BITS()
-			bits = akos16.bits & 3;
+			bits = _akos16.bits & 3;
 			if (bits & 1) {
 				AKOS16_EAT_BITS(2)
 				if (bits & 2) {
-					tmp_bits = akos16.bits & 7;
+					tmp_bits = _akos16.bits & 7;
 					AKOS16_EAT_BITS(3)
 					if (tmp_bits != 4) {
-						akos16.color += (tmp_bits - 4);
+						// A color change
+						_akos16.color += (tmp_bits - 4);
 					} else {
-						akos16.unk5 = 1;
+						// Color does not change, but rather identical pixels get repeated
+						_akos16.repeatMode = true;
 						AKOS16_FILL_BITS()
-						akos16.unk6 = (akos16.bits & 0xff) - 1;
+						_akos16.repeatCount = (_akos16.bits & 0xff) - 1;
 						AKOS16_EAT_BITS(8)
 						AKOS16_FILL_BITS()
 					}
 				} else {
 					AKOS16_FILL_BITS()
-					akos16.color = ((byte)akos16.bits) & akos16.mask;
-					AKOS16_EAT_BITS(akos16.shift)
+					_akos16.color = ((byte)_akos16.bits) & _akos16.mask;
+					AKOS16_EAT_BITS(_akos16.shift)
 					AKOS16_FILL_BITS()
 				}
 			} else {
 				AKOS16_EAT_BITS(1);
 			}
 		} else {
-			if (--akos16.unk6 == 0) {
-				akos16.unk5 = 0;
+			if (--_akos16.repeatCount == 0) {
+				_akos16.repeatMode = false;
 			}
 		}
 		numbytes--;
@@ -1123,7 +1131,7 @@ void AkosRenderer::akos16DecodeLine(byte *buf, int32 numbytes, int32 dir) {
 
 void AkosRenderer::akos16Decompress(byte *dest, int32 pitch, const byte *src, int32 t_width, int32 t_height, int32 dir,
 		int32 numskip_before, int32 numskip_after, byte transparency, int maskLeft, int maskTop, int zBuf) {
-	byte *tmp_buf = akos16.buffer;
+	byte *tmp_buf = _akos16.buffer;
 	int maskpitch;
 	byte *maskptr;
 	const byte maskbit = revBitMask(maskLeft & 7);
@@ -1147,9 +1155,9 @@ void AkosRenderer::akos16Decompress(byte *dest, int32 pitch, const byte *src, in
 	assert(t_width > 0);
 	while (t_height--) {
 		akos16DecodeLine(tmp_buf, t_width, dir);
-		bompApplyMask(akos16.buffer, maskptr, maskbit, t_width, transparency);
+		bompApplyMask(_akos16.buffer, maskptr, maskbit, t_width, transparency);
 		bool HE7Check = (_vm->_game.heversion == 70);
-		bompApplyShadow(_shadow_mode, _shadow_table, akos16.buffer, dest, t_width, transparency, HE7Check);
+		bompApplyShadow(_shadow_mode, _shadow_table, _akos16.buffer, dest, t_width, transparency, HE7Check);
 
 		if (numskip_after != 0)	{
 			akos16SkipData(numskip_after);
@@ -1163,7 +1171,7 @@ byte AkosRenderer::codec16(int xmoveCur, int ymoveCur) {
 	Common::Rect clip;
 	int32 minx, miny, maxw, maxh;
 	int32 skip_x, skip_y, cur_x, cur_y;
-	byte transparency = (_vm->_game.heversion >= 61) ? palette[0] : 255;
+	byte transparency = (_vm->_game.heversion >= 61) ? _palette[0] : 255;
 
 	if (_actorHitMode) {
 		error("codec16: _actorHitMode not yet implemented");
@@ -1325,9 +1333,13 @@ byte AkosRenderer::codec32(int xmoveCur, int ymoveCur) {
 
 	byte *dstPtr = (byte *)_out.pixels + dst.left + dst.top * _out.pitch;
 	if (_shadow_mode == 3) {
-		Wiz::decompressWizImage(dstPtr, _out.pitch, dst, _srcptr, src, 0, palPtr, xmap);
+		Wiz::decompressWizImage<kWizXMap>(dstPtr, _out.pitch, _srcptr, src, 0, palPtr, xmap);
 	} else {
-		Wiz::decompressWizImage(dstPtr, _out.pitch, dst, _srcptr, src, 0, palPtr);
+		if (palPtr != NULL) {
+			Wiz::decompressWizImage<kWizRMap>(dstPtr, _out.pitch, _srcptr, src, 0, palPtr);
+		} else {
+			Wiz::decompressWizImage<kWizCopy>(dstPtr, _out.pitch, _srcptr, src, 0);
+		}
 	}
 #endif
 	return 0;
@@ -1721,7 +1733,7 @@ bool ScummEngine_v6::akos_increaseAnim(Actor *a, int chan, const byte *aksq, con
 
 void ScummEngine_v6::akos_queCommand(byte cmd, Actor *a, int param_1, int param_2) {
 	_akosQueuePos++;
-	checkRange(31, 0, _akosQueuePos, "akos_queCommand overflow");
+	assertRange(0, _akosQueuePos, 31, "akos_queCommand: _akosQueuePos");
 
 	_akosQueue[_akosQueuePos].cmd = cmd;
 	_akosQueue[_akosQueuePos].actor = a->_number;
