@@ -1,6 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2001  Ludvig Strigeus
- * Copyright (C) 2001-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -16,13 +18,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/scumm/akos.h $
- * $Id: akos.h 20511 2006-02-11 09:55:41Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/scumm/akos.h $
+ * $Id: akos.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
-#ifndef AKOS_H
-#define AKOS_H
+#ifndef SCUMM_AKOS_H
+#define SCUMM_AKOS_H
 
 #include "scumm/base-costume.h"
 
@@ -55,24 +57,27 @@ protected:
 
 class AkosRenderer : public BaseCostumeRenderer {
 protected:
-	uint16 codec;
+	uint16 _codec;
 
-	// actor palette
-	byte palette[256];
+	// actor _palette
+	byte _palette[256];
+	bool _useBompPalette;
 
 	// pointer to various parts of the costume resource
-	const byte *akos;
-	const AkosHeader *akhd;
+	const AkosHeader *akhd;	// header
 
-	const byte *akpl, *akci, *aksq;
-	const AkosOffset *akof;
-	const byte *akcd;
-	const byte *akct;
-	const uint8 *xmap;
+	const byte *akpl;		// palette data
+	const byte *akci;		// CostumeInfo table
+	const byte *aksq;		// command sequence
+	const AkosOffset *akof;	// offsets into ci and cd table
+	const byte *akcd;		// costume data (contains the data for the codecs)
+
+	const byte *akct;		// HE specific: condition table
+	const uint8 *xmap;		// HE specific: shadow color table ?!?
 
 	struct {
-		byte unk5;
-		int unk6;
+		bool repeatMode;
+		int repeatCount;
 		byte mask;
 		byte color;
 		byte shift;
@@ -80,11 +85,11 @@ protected:
 		byte numbits;
 		const byte *dataptr;
 		byte buffer[336];
-	} akos16;
+	} _akos16;
 
 public:
 	AkosRenderer(ScummEngine *scumm) : BaseCostumeRenderer(scumm) {
-		akos = 0;
+		_useBompPalette = false;
 		akhd = 0;
 		akpl = 0;
 		akci = 0;
@@ -100,7 +105,7 @@ public:
 	int16 _actorHitX, _actorHitY;
 	bool _actorHitResult;
 
-	void setPalette(byte *palette);
+	void setPalette(byte *_palette);
 	void setFacing(const Actor *a);
 	void setCostume(int costume, int shadow);
 
