@@ -1,6 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2001  Ludvig Strigeus
- * Copyright (C) 2001-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -16,32 +18,28 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/scumm/base-costume.h $
- * $Id: base-costume.h 23144 2006-06-16 11:30:23Z kirben $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/scumm/base-costume.h $
+ * $Id: base-costume.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
-#ifndef BASE_COSTUME_H
-#define BASE_COSTUME_H
+#ifndef SCUMM_BASE_COSTUME_H
+#define SCUMM_BASE_COSTUME_H
 
 #include "common/scummsys.h"
 #include "scumm/actor.h"		// for CostumeData
 
 namespace Scumm {
 
-#if !defined(__GNUC__)
-#pragma START_PACK_STRUCTS
-#endif
+#include "common/pack-start.h"	// START STRUCT PACKING
 
 struct CostumeInfo {
 	uint16 width, height;
 	int16 rel_x, rel_y;
 	int16 move_x, move_y;
-} GCC_PACK;
+};
 
-#if !defined(__GNUC__)
-#pragma END_PACK_STRUCTS
-#endif
+#include "common/pack-end.h"	// END STRUCT PACKING
 
 
 
