@@ -1,6 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2001  Ludvig Strigeus
- * Copyright (C) 2001-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +17,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/scumm/bomp.cpp $
- * $Id: bomp.cpp 20511 2006-02-11 09:55:41Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/scumm/bomp.cpp $
+ * $Id: bomp.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -33,7 +35,7 @@ static int32 setupBompScale(byte *scaling, int32 size, byte scale);
 
 static void bompScaleFuncX(byte *line_buffer, byte *scaling_x_ptr, byte skip, int32 size);
 
-static void bompApplyShadow0(const byte *shadowPalette, const byte *line_buffer, byte *dst, int32 size, byte transparency, byte HE7Check);
+static void bompApplyShadow0(const byte *shadowPalette, const byte *line_buffer, byte *dst, int32 size, byte transparency, bool HE7Check);
 static void bompApplyShadow1(const byte *shadowPalette, const byte *line_buffer, byte *dst, int32 size, byte transparency);
 static void bompApplyShadow3(const byte *shadowPalette, const byte *line_buffer, byte *dst, int32 size, byte transparency);
 static void bompApplyActorPalette(byte *actorPalette, byte *line_buffer, int32 size);
@@ -115,7 +117,7 @@ void bompApplyMask(byte *line_buffer, byte *mask, byte maskbit, int32 size, byte
 	}
 }
 
-void bompApplyShadow(int shadowMode, const byte *shadowPalette, const byte *line_buffer, byte *dst, int32 size, byte transparency, byte HE7Check) {
+void bompApplyShadow(int shadowMode, const byte *shadowPalette, const byte *line_buffer, byte *dst, int32 size, byte transparency, bool HE7Check) {
 	assert(size > 0);
 	switch (shadowMode) {
 	case 0:
@@ -131,7 +133,7 @@ void bompApplyShadow(int shadowMode, const byte *shadowPalette, const byte *line
 		error("Unknown shadow mode %d", shadowMode);
 	}
 }
-void bompApplyShadow0(const byte *shadowPalette, const byte *line_buffer, byte *dst, int32 size, byte transparency, byte HE7Check = false) {
+void bompApplyShadow0(const byte *shadowPalette, const byte *line_buffer, byte *dst, int32 size, byte transparency, bool HE7Check) {
 	while (size-- > 0) {
 		byte tmp = *line_buffer++;
 		if (tmp != transparency) {
@@ -171,12 +173,10 @@ void bompApplyShadow3(const byte *shadowPalette, const byte *line_buffer, byte *
 }
 
 void bompApplyActorPalette(byte *actorPalette, byte *line_buffer, int32 size) {
-	if (actorPalette != 0) {
-		actorPalette[255] = 255;
-		while (size-- > 0) {
-			*line_buffer = actorPalette[*line_buffer];
-			line_buffer++;
-		}
+	actorPalette[255] = 255;
+	while (size-- > 0) {
+		*line_buffer = actorPalette[*line_buffer];
+		line_buffer++;
 	}
 }
 
@@ -199,7 +199,7 @@ void bompScaleFuncX(byte *line_buffer, byte *scaling_x_ptr, byte skip, int32 siz
 	}
 }
 
-void ScummEngine::drawBomp(const BompDrawData &bd, bool mirror) {
+void drawBomp(const BompDrawData &bd) {
 	const byte *src;
 	byte *dst;
 	byte *mask = 0;
@@ -211,7 +211,6 @@ void ScummEngine::drawBomp(const BompDrawData &bd, bool mirror) {
 	byte bomp_scaling_x[64];
 	byte bomp_scaling_y[64];
 
-
 	if (bd.x < 0) {
 		clip.left = -bd.x;
 	} else {
@@ -234,14 +233,14 @@ void ScummEngine::drawBomp(const BompDrawData &bd, bool mirror) {
 		clip.bottom = bd.dst.h - bd.y;
 	}
 
-	src = bd.dataptr;
+	src = bd.src;
 	dst = (byte *)bd.dst.pixels + bd.y * bd.dst.pitch + (bd.x + clip.left);
 
 	const byte maskbit = revBitMask((bd.x + clip.left) & 7);
 
 	// Mask against any additionally imposed mask
 	if (bd.maskPtr) {
-		mask = bd.maskPtr + (bd.y * gdi._numStrips) + ((bd.x + clip.left) / 8);
+		mask = bd.maskPtr + (bd.y * bd.numStrips) + ((bd.x + clip.left) / 8);
 	}
 
 	// Setup vertical scaling
@@ -279,7 +278,7 @@ void ScummEngine::drawBomp(const BompDrawData &bd, bool mirror) {
 	// Loop over all lines
 	while (pos_y < clip.bottom) {
 		// Decode a single (bomp encoded) line, reversed if we are in mirror mode
-		if (mirror)
+		if (bd.mirror)
 			bompDecodeLineReverse(line_buffer, src + 2, bd.srcwidth);
 		else
 			bompDecodeLine(line_buffer, src + 2, bd.srcwidth);
@@ -316,78 +315,56 @@ void ScummEngine::drawBomp(const BompDrawData &bd, bool mirror) {
 				bompApplyMask(line_ptr, mask, maskbit, width, 255);
 
 			// Apply custom color map, if available
-			if (_bompActorPalettePtr)
-				bompApplyActorPalette(_bompActorPalettePtr, line_ptr, width);
+			if (bd.actorPalette)
+				bompApplyActorPalette(bd.actorPalette, line_ptr, width);
 
 			// Finally, draw the decoded, scaled, masked and recolored line onto
 			// the target surface, using the specified shadow mode
-			bompApplyShadow(bd.shadowMode, _shadowPalette, line_ptr, dst, width, 255);
+			bompApplyShadow(bd.shadowMode, bd.shadowPalette, line_ptr, dst, width, 255);
 		}
 
 		// Advance to the next line
 		pos_y++;
-		mask += gdi._numStrips;
+		mask += bd.numStrips;
 		dst += bd.dst.pitch;
 	}
 }
 
-static const byte bitCount[] = {
-	8, 7, 7, 6, 7, 6, 6, 5, 7, 6, 6, 5, 6, 5, 5, 4,
-	7, 6, 6, 5, 6, 5, 5, 4, 6, 5, 5, 4, 5, 4, 4, 3,
-	7, 6, 6, 5, 6, 5, 5, 4, 6, 5, 5, 4, 5, 4, 4, 3,
-	6, 5, 5, 4, 5, 4, 4, 3, 5, 4, 4, 3, 4, 3, 3, 2,
-	7, 6, 6, 5, 6, 5, 5, 4, 6, 5, 5, 4, 5, 4, 4, 3,
-	6, 5, 5, 4, 5, 4, 4, 3, 5, 4, 4, 3, 4, 3, 3, 2,
-	6, 5, 5, 4, 5, 4, 4, 3, 5, 4, 4, 3, 4, 3, 3, 2,
-	5, 4, 4, 3, 4, 3, 3, 2, 4, 3, 3, 2, 3, 2, 2, 1,
-	7, 6, 6, 5, 6, 5, 5, 4, 6, 5, 5, 4, 5, 4, 4, 3,
-	6, 5, 5, 4, 5, 4, 4, 3, 5, 4, 4, 3, 4, 3, 3, 2,
-	6, 5, 5, 4, 5, 4, 4, 3, 5, 4, 4, 3, 4, 3, 3, 2,
-	5, 4, 4, 3, 4, 3, 3, 2, 4, 3, 3, 2, 3, 2, 2, 1,
-	6, 5, 5, 4, 5, 4, 4, 3, 5, 4, 4, 3, 4, 3, 3, 2,
-	5, 4, 4, 3, 4, 3, 3, 2, 4, 3, 3, 2, 3, 2, 2, 1,
-	5, 4, 4, 3, 4, 3, 3, 2, 4, 3, 3, 2, 3, 2, 2, 1,
-	4, 3, 3, 2, 3, 2, 2, 1, 3, 2, 2, 1, 2, 1, 1, 0,
-};
-
 int32 setupBompScale(byte *scaling, int32 size, byte scale) {
-	byte tmp;
+	static const int offsets[8] = { 3, 2, 1, 0, 7, 6, 5, 4 };
 	int32 count;
-	const byte *tmp_ptr;
-	byte *tmp_scaling = scaling;
-	byte a = 0;
-	byte ret_value = 0;
-	const int offsets[8] = { 3, 2, 1, 0, 7, 6, 5, 4 };
+	byte bitsCount = 0;
 
 	count = (256 - size / 2);
 	assert(0 <= count && count < 768);
-	tmp_ptr = bigCostumeScaleTable + count;
+	const byte *scaleTable = bigCostumeScaleTable + count;
 
 	count = (size + 7) / 8;
 	while (count--) {
-		a = 0;
+		byte scaleMask = 0;
 		for (int i = 0; i < 8; i++) {
-			tmp = *(tmp_ptr + offsets[i]);
-			a <<= 1;
-			if (scale < tmp) {
-				a |= 1;
+			byte scaleTest = *(scaleTable + offsets[i]);
+			scaleMask <<= 1;
+			if (scale < scaleTest) {
+				scaleMask |= 1;
+			} else {
+				bitsCount++;
 			}
 		}
-		tmp_ptr += 8;
+		scaleTable += 8;
 
-		*tmp_scaling++ = a;
+		*scaling++ = scaleMask;
 	}
-	if ((size & 7) != 0) {
-		*(tmp_scaling - 1) |= revBitMask(size & 7);
+	size &= 7;
+	if (size != 0) {
+		--scaling;
+		if ((*scaling & revBitMask(size)) == 0) {
+			*scaling |= revBitMask(size);
+			bitsCount--;
+		}
 	}
 
-	count = (size + 7) / 8;
-	while (count--) {
-		tmp = *scaling++;
-		ret_value += bitCount[tmp];
-	}
-
-	return ret_value;
+	return bitsCount;
 }
 
 } // End of namespace Scumm
