@@ -1,6 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2001  Ludvig Strigeus
- * Copyright (C) 2001-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,25 +17,50 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/scumm/bomp.h $
- * $Id: bomp.h 20511 2006-02-11 09:55:41Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/scumm/bomp.h $
+ * $Id: bomp.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
-#ifndef BOMP_H
-#define BOMP_H
+#ifndef SCUMM_BOMP_H
+#define SCUMM_BOMP_H
 
 #include "common/scummsys.h"
+#include "graphics/surface.h"
 
 namespace Scumm {
 
 void bompApplyMask(byte *line_buffer, byte *mask, byte maskbit, int32 size, byte transparency);
-void bompApplyShadow(int shadowMode, const byte *shadowPalette, const byte *line_buffer, byte *dst, int32 size, byte transparency, byte HE7Check = false);
+void bompApplyShadow(int shadowMode, const byte *shadowPalette, const byte *line_buffer, byte *dst, int32 size, byte transparency, bool HE7Check = false);
 
 void decompressBomp(byte *dst, const byte *src, int w, int h);
 void bompDecodeLine(byte *dst, const byte *src, int size);
 void bompDecodeLineReverse(byte *dst, const byte *src, int size);
 
+
+/** Bomp graphics data */
+struct BompDrawData {
+	Graphics::Surface dst;
+	int x, y;
+
+	const byte *src;
+	int srcwidth, srcheight;
+
+	byte scale_x, scale_y;
+
+	byte *maskPtr;
+	int numStrips;
+
+	uint16 shadowMode;
+	byte *shadowPalette;
+
+	byte *actorPalette;
+
+	bool mirror;
+};
+
+void drawBomp(const BompDrawData &bd);
+
 } // End of namespace Scumm
 
 #endif
