@@ -1,6 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2001  Ludvig Strigeus
- * Copyright (C) 2001-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -16,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/scumm/boxes.cpp $
- * $Id: boxes.cpp 22732 2006-05-29 12:45:11Z kirben $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/scumm/boxes.cpp $
+ * $Id: boxes.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -25,15 +27,14 @@
 #include "scumm/scumm.h"
 #include "scumm/actor.h"
 #include "scumm/boxes.h"
+#include "scumm/intern.h"
 #include "scumm/util.h"
 
 #include "common/util.h"
 
 namespace Scumm {
 
-#if !defined(__GNUC__)
-	#pragma START_PACK_STRUCTS
-#endif
+#include "common/pack-start.h"	// START STRUCT PACKING
 
 struct Box {				/* Internal walkbox file format */
 	union {
@@ -43,7 +44,7 @@ struct Box {				/* Internal walkbox file format */
 			byte y1;
 			byte y2;
 			byte mask;
-		} GCC_PACK c64;
+		} c64;
 
 		struct {
 			byte uy;
@@ -54,7 +55,7 @@ struct Box {				/* Internal walkbox file format */
 			byte lrx;
 			byte mask;
 			byte flags;
-		} GCC_PACK v2;
+		} v2;
 
 		struct {
 			int16 ulx, uly;
@@ -64,7 +65,7 @@ struct Box {				/* Internal walkbox file format */
 			byte mask;
 			byte flags;
 			uint16 scale;
-		} GCC_PACK old;
+		} old;
 
 		struct {
 			int32 ulx, uly;
@@ -77,24 +78,95 @@ struct Box {				/* Internal walkbox file format */
 			uint32 scale;
 			uint32 unk2;
 			uint32 unk3;
-		} GCC_PACK v8;
-	} GCC_PACK;
-} GCC_PACK;
+		} v8;
+	};
+};
 
-#if !defined(__GNUC__)
-	#pragma END_PACK_STRUCTS
-#endif
+#include "common/pack-end.h"	// END STRUCT PACKING
 
 #define BOX_MATRIX_SIZE 2000
 #define BOX_DEBUG 0
 
 
-static bool compareSlope(int X1, int Y1, int X2, int Y2, int X3, int Y3);
-static Common::Point closestPtOnLine(const Common::Point &start, const Common::Point &end, int x, int y);
+static void getGates(const BoxCoords &box1, const BoxCoords &box2, Common::Point gateA[2], Common::Point gateB[2]);
 
+static bool compareSlope(const Common::Point &p1, const Common::Point &p2, const Common::Point &p3) {
+	return (p2.y - p1.y) * (p3.x - p1.x) <= (p3.y - p1.y) * (p2.x - p1.x);
+}
+
+/**
+ * Find the point on a line segment which is closest to a given point.
+ *
+ * @param lineStart	the start of the line segment
+ * @param lineEnd	the end of the line segment
+ * @param p		the point which we want to 'project' on the line segment
+ * @return the point on the line segment closest to the given point
+ */
+static Common::Point closestPtOnLine(const Common::Point &lineStart, const Common::Point &lineEnd, const Common::Point &p) {
+	Common::Point result;
+
+	const int lxdiff = lineEnd.x - lineStart.x;
+	const int lydiff = lineEnd.y - lineStart.y;
+
+	if (lineEnd.x == lineStart.x) {	// Vertical line?
+		result.x = lineStart.x;
+		result.y = p.y;
+	} else if (lineEnd.y == lineStart.y) {	// Horizontal line?
+		result.x = p.x;
+		result.y = lineStart.y;
+	} else {
+		const int dist = lxdiff * lxdiff + lydiff * lydiff;
+		int a, b, c;
+		if (ABS(lxdiff) > ABS(lydiff)) {
+			a = lineStart.x * lydiff / lxdiff;
+			b = p.x * lxdiff / lydiff;
+
+			c = (a + b - lineStart.y + p.y) * lydiff * lxdiff / dist;
+
+			result.x = c;
+			result.y = c * lydiff / lxdiff - a + lineStart.y;
+		} else {
+			a = lineStart.y * lxdiff / lydiff;
+			b = p.y * lydiff / lxdiff;
+
+			c = (a + b - lineStart.x + p.x) * lydiff * lxdiff / dist;
+
+			result.x = c * lxdiff / lydiff - a + lineStart.x;
+			result.y = c;
+		}
+	}
+
+	if (ABS(lydiff) < ABS(lxdiff)) {
+		if (lxdiff > 0) {
+			if (result.x < lineStart.x)
+				result = lineStart;
+			else if (result.x > lineEnd.x)
+				result = lineEnd;
+		} else {
+			if (result.x > lineStart.x)
+				result = lineStart;
+			else if (result.x < lineEnd.x)
+				result = lineEnd;
+		}
+	} else {
+		if (lydiff > 0) {
+			if (result.y < lineStart.y)
+				result = lineStart;
+			else if (result.y > lineEnd.y)
+				result = lineEnd;
+		} else {
+			if (result.y > lineStart.y)
+				result = lineStart;
+			else if (result.y < lineEnd.y)
+				result = lineEnd;
+		}
+	}
+
+	return result;
+}
 
 byte ScummEngine::getMaskFromBox(int box) {
-	// Fix for bug #740244 and #755863. This appears to have been a
+	// WORKAROUND for bug #740244 and #755863. This appears to have been a
 	// long standing bug in the original engine?
 	if (_game.version <= 3 && box == 255)
 		return 1;
@@ -170,7 +242,7 @@ void ScummEngine::setBoxScaleSlot(int box, int slot) {
 }
 
 int ScummEngine::getScale(int box, int x, int y) {
-	if (_game.features & GF_NO_SCALING)
+	if (_game.version <= 3)
 		return 255;
 
 	Box *ptr = getBoxBaseAddr(box);
@@ -236,7 +308,7 @@ int ScummEngine::getScaleFromSlot(int slot, int x, int y) {
 }
 
 int ScummEngine::getBoxScale(int box) {
-	if (_game.features & GF_NO_SCALING)
+	if (_game.version <= 3)
 		return 255;
 	Box *ptr = getBoxBaseAddr(box);
 	if (!ptr)
@@ -373,7 +445,9 @@ byte ScummEngine::getNumBoxes() {
 	if (!ptr)
 		return 0;
 	if (_game.version == 8)
-		return (byte) READ_LE_UINT32(ptr);
+		return (byte)READ_LE_UINT32(ptr);
+	else if (_game.features >= 5)
+		return (byte)READ_LE_UINT16(ptr);
 	else
 		return ptr[0];
 }
@@ -383,29 +457,29 @@ Box *ScummEngine::getBoxBaseAddr(int box) {
 	if (!ptr || box == 255)
 		return NULL;
 
-	// The NES version of Maniac Mansion attempts to set flags for boxes 2-4
+	// WORKAROUND: The NES version of Maniac Mansion attempts to set flags for boxes 2-4
 	// when there are only three boxes (0-2) when walking out to the garage.
 	if ((_game.id == GID_MANIAC) && (_game.platform == Common::kPlatformNES) && (box >= ptr[0]))
 		return NULL;
 
-	// FIXME: In "pass to adventure", the loom demo, when bobbin enters
+	// WORKAROUND: In "pass to adventure", the loom demo, when bobbin enters
 	// the tent to the elders, box = 2, but ptr[0] = 2 -> errors out.
-	// Hence we disable the check for now. Maybe in PASS (and other old games)
-	// we shouldn't subtract 1 from ptr[0] when performing the check?
-	// this also seems to be incorrect for atari st demo of zak
-	// and assumingly other v2 games
-	// The same happens in Indy3EGA (see bug #770351)
-	// Also happens in ZakEGA (see bug #771803).
+	// Also happens in Indy3EGA (see bug #770351) and ZakEGA (see bug #771803).
 	//
 	// This *might* mean that we have a bug in our box implementation
 	// OTOH, the original engine, unlike ScummVM, performed no bound
 	// checking at all. All the problems so far have been cases where
 	// the value was exactly one more than what we consider the maximum.
-	// So it's very well possible that all of these are script errors.
+	// So it seems to be most likely that all of these are script errors.
+	//
+	// As a workaround, we simply use the last box if the last+1 box is requested.
+	// Note that this may cause different behavior than the original game
+	// engine exhibited! To faithfully reproduce the behavior of the original
+	// engine, we would have to know the data coming *after* the walkbox table.
 	if (_game.version <= 4 && ptr[0] == box)
-	    box--;
+		box--;
 
-	checkRange(ptr[0] - 1, 0, box, "Illegal box %d");
+	assertRange(0, box, ptr[0] - 1, "box");
 	if (_game.version == 0)
 		return (Box *)(ptr + box * SIZEOF_BOX_C64 + 1);
 	else if (_game.version <= 2)
@@ -420,7 +494,7 @@ Box *ScummEngine::getBoxBaseAddr(int box) {
 		return (Box *)(ptr + box * SIZEOF_BOX + 2);
 }
 
-int ScummEngine::getSpecialBox(int x, int y) {
+int ScummEngine_v6::getSpecialBox(int x, int y) {
 	int i;
 	int numOfBoxes;
 	byte flag;
@@ -440,14 +514,21 @@ int ScummEngine::getSpecialBox(int x, int y) {
 	return (-1);
 }
 
-bool ScummEngine::checkXYInBoxBounds(int b, int x, int y) {
-	BoxCoords box;
-
-	if (b < 0 || b == Actor::kInvalidBox)
+bool ScummEngine::checkXYInBoxBounds(int boxnum, int x, int y) {
+	// Since this method is called by many other methods that take params
+	// from e.g. script opcodes, but do not validate the boxnum, we
+	// make a check here to filter out invalid boxes.
+	// See also bug #1599113.
+	if (boxnum < 0 || boxnum == Actor::kInvalidBox)
 		return false;
 
-	getBoxCoordinates(b, &box);
+	BoxCoords box = getBoxCoordinates(boxnum);
+	const Common::Point p(x, y);
 
+	// Quick check: If the x (resp. y) coordinate of the point is
+	// strictly smaller (bigger) than the x (y) coordinates of all
+	// corners of the quadrangle, then it certainly is *not* contained
+	// inside the quadrangle.
 	if (x < box.ul.x && x < box.ur.x && x < box.lr.x && x < box.ll.x)
 		return false;
 
@@ -460,31 +541,40 @@ bool ScummEngine::checkXYInBoxBounds(int b, int x, int y) {
 	if (y > box.ul.y && y > box.ur.y && y > box.lr.y && y > box.ll.y)
 		return false;
 
-	if (box.ul.x == box.ur.x && box.ul.y == box.ur.y && box.lr.x == box.ll.x && box.lr.y == box.ll.y ||
-		box.ul.x == box.ll.x && box.ul.y == box.ll.y && box.ur.x == box.lr.x && box.ur.y == box.lr.y) {
+	// Corner case: If the box is a simple line segment, we consider the
+	// point to be contained "in" (or rather, lying on) the line if it
+	// is very close to its projection to the line segment.
+	if (box.ul == box.ur && box.lr == box.ll ||
+		box.ul == box.ll && box.ur == box.lr) {
 
-		Common::Point pt;
-		pt = closestPtOnLine(box.ul, box.lr, x, y);
-		if (distanceFromPt(x, y, pt.x, pt.y) <= 4)
+		Common::Point tmp;
+		tmp = closestPtOnLine(box.ul, box.lr, p);
+		if (p.sqrDist(tmp) <= 4)
 			return true;
 	}
+	
+	// Finally, fall back to the classic algorithm to compute containment
+	// in a convex polygon: For each (oriented) side of the polygon
+	// (quadrangle in this case), compute whether p is "left" or "right"
+	// from it.
 
-	if (!compareSlope(box.ul.x, box.ul.y, box.ur.x, box.ur.y, x, y))
+	if (!compareSlope(box.ul, box.ur, p))
 		return false;
 
-	if (!compareSlope(box.ur.x, box.ur.y, box.lr.x, box.lr.y, x, y))
+	if (!compareSlope(box.ur, box.lr, p))
 		return false;
 
-	if (!compareSlope(box.ll.x, box.ll.y, x, y, box.lr.x, box.lr.y))
+	if (!compareSlope(box.lr, box.ll, p))
 		return false;
 
-	if (!compareSlope(box.ul.x, box.ul.y, x, y, box.ll.x, box.ll.y))
+	if (!compareSlope(box.ll, box.ul, p))
 		return false;
 
 	return true;
 }
 
-void ScummEngine::getBoxCoordinates(int boxnum, BoxCoords *box) {
+BoxCoords ScummEngine::getBoxCoordinates(int boxnum) {
+	BoxCoords tmp, *box = &tmp;
 	Box *bp = getBoxBaseAddr(boxnum);
 	assert(bp);
 
@@ -499,13 +589,10 @@ void ScummEngine::getBoxCoordinates(int boxnum, BoxCoords *box) {
 		box->lr.x = (short)FROM_LE_32(bp->v8.lrx);
 		box->lr.y = (short)FROM_LE_32(bp->v8.lry);
 
-		// FIXME: Some walkboxes in CMI appear to have been flipped,
-		// in the sense that for instance the lower boundary is above
-		// the upper one. Can that really be the case, or is there
-		// some more sinister problem afoot?
-		//
-		// Is this fix sufficient, or will we need something more
-		// elaborate?
+		// WORKAROUND (see patch #684732): Some walkboxes in CMI appear
+		// to have been flipped, in the sense that for instance the
+		// lower boundary is above the upper one. We work around this
+		// by simply flipping them back.
 
 		if (box->ul.y > box->ll.y && box->ur.y > box->lr.y) {
 			SWAP(box->ul, box->ll);
@@ -517,25 +604,25 @@ void ScummEngine::getBoxCoordinates(int boxnum, BoxCoords *box) {
 			SWAP(box->ll, box->lr);
 		}
 	} else if (_game.version == 0) {
-		box->ul.x = bp->c64.x1 * 8;
-		box->ul.y = bp->c64.y1 * 2;
-		box->ur.x = bp->c64.x2 * 8;
-		box->ur.y = bp->c64.y1 * 2;
+		box->ul.x = bp->c64.x1;
+		box->ul.y = bp->c64.y1;
+		box->ur.x = bp->c64.x2;
+		box->ur.y = bp->c64.y1;
 
-		box->ll.x = bp->c64.x1 * 8;
-		box->ll.y = bp->c64.y2 * 2;
-		box->lr.x = bp->c64.x2 * 8;
-		box->lr.y = bp->c64.y2 * 2;
+		box->ll.x = bp->c64.x1;
+		box->ll.y = bp->c64.y2;
+		box->lr.x = bp->c64.x2;
+		box->lr.y = bp->c64.y2;
 	} else if (_game.version <= 2) {
-		box->ul.x = bp->v2.ulx * 8;
-		box->ul.y = bp->v2.uy * 2;
-		box->ur.x = bp->v2.urx * 8;
-		box->ur.y = bp->v2.uy * 2;
+		box->ul.x = bp->v2.ulx;
+		box->ul.y = bp->v2.uy;
+		box->ur.x = bp->v2.urx;
+		box->ur.y = bp->v2.uy;
 
-		box->ll.x = bp->v2.llx * 8;
-		box->ll.y = bp->v2.ly * 2;
-		box->lr.x = bp->v2.lrx * 8;
-		box->lr.y = bp->v2.ly * 2;
+		box->ll.x = bp->v2.llx;
+		box->ll.y = bp->v2.ly;
+		box->lr.x = bp->v2.lrx;
+		box->lr.y = bp->v2.ly;
 	} else {
 		box->ul.x = (int16)READ_LE_UINT16(&bp->old.ulx);
 		box->ul.y = (int16)READ_LE_UINT16(&bp->old.uly);
@@ -547,165 +634,45 @@ void ScummEngine::getBoxCoordinates(int boxnum, BoxCoords *box) {
 		box->lr.x = (int16)READ_LE_UINT16(&bp->old.lrx);
 		box->lr.y = (int16)READ_LE_UINT16(&bp->old.lry);
 	}
+	return *box;
 }
 
-uint ScummEngine::distanceFromPt(int x, int y, int ptx, int pty) {
-	int diffx, diffy;
-
-	diffx = ABS(ptx - x);
-
-	if (diffx >= 0x1000)
-		return 0xFFFFFF;
-
-	diffy = ABS(pty - y);
-
-	if (diffy >= 0x1000)
-		return 0xFFFFFF;
-	diffx *= diffx;
-	diffy *= diffy;
-	return diffx + diffy;
-}
-
-
-bool compareSlope(int X1, int Y1, int X2, int Y2, int X3, int Y3) {
-	return (Y2 - Y1) * (X3 - X1) <= (Y3 - Y1) * (X2 - X1);
-}
-
-/**
- * Find the point on a line segment which is closest to a given point.
- *
- * @param start	the start of the line segment
- * @param end	the end of the line segment
- * @param x		the x coordinate of the point which we want to 'project' on the line segment
- * @param y		the y coordinate of the point which we want to 'project' on the line segment
- * @return the point on the line segmen closes to the given point
- */
-Common::Point closestPtOnLine(const Common::Point &start, const Common::Point &end, int x, int y) {
-	Common::Point pt;
-
-	const int lxdiff = end.x - start.x;
-	const int lydiff = end.y - start.y;
-
-	if (end.x == start.x) {	// Vertical line?
-		pt.x = start.x;
-		pt.y = y;
-	} else if (end.y == start.y) {	// Horizontal line?
-		pt.x = x;
-		pt.y = start.y;
-	} else {
-		const int dist = lxdiff * lxdiff + lydiff * lydiff;
-		int a, b, c;
-		if (ABS(lxdiff) > ABS(lydiff)) {
-			a = start.x * lydiff / lxdiff;
-			b = x * lxdiff / lydiff;
-
-			c = (a + b - start.y + y) * lydiff * lxdiff / dist;
-
-			pt.x = c;
-			pt.y = c * lydiff / lxdiff - a + start.y;
-		} else {
-			a = start.y * lxdiff / lydiff;
-			b = y * lydiff / lxdiff;
-
-			c = (a + b - start.x + x) * lydiff * lxdiff / dist;
-
-			pt.y = c;
-			pt.x = c * lxdiff / lydiff - a + start.x;
-		}
-	}
-
-	if (ABS(lydiff) < ABS(lxdiff)) {
-		if (lxdiff > 0) {
-			if (pt.x < start.x)
-				pt = start;
-			else if (pt.x > end.x)
-				pt = end;
-		} else {
-			if (pt.x > start.x)
-				pt = start;
-			else if (pt.x < end.x)
-				pt = end;
-		}
-	} else {
-		if (lydiff > 0) {
-			if (pt.y < start.y)
-				pt = start;
-			else if (pt.y > end.y)
-				pt = end;
-		} else {
-			if (pt.y > start.y)
-				pt = start;
-			else if (pt.y < end.y)
-				pt = end;
-		}
-	}
-
-	return pt;
-}
-
-bool ScummEngine::inBoxQuickReject(int b, int x, int y, int threshold) {
-	int t;
-	BoxCoords box;
-
-	getBoxCoordinates(b, &box);
-
-	t = x - threshold;
-	if (t > box.ul.x && t > box.ur.x && t > box.lr.x && t > box.ll.x)
-		return true;
-
-	t = x + threshold;
-	if (t < box.ul.x && t < box.ur.x && t < box.lr.x && t < box.ll.x)
-		return true;
-
-	t = y - threshold;
-	if (t > box.ul.y && t > box.ur.y && t > box.lr.y && t > box.ll.y)
-		return true;
-
-	t = y + threshold;
-	if (t < box.ul.y && t < box.ur.y && t < box.lr.y && t < box.ll.y)
-		return true;
-
-	return false;
-}
-
-int ScummEngine::getClosestPtOnBox(int b, int x, int y, int16& outX, int16& outY) {
-	Common::Point pt;
+int getClosestPtOnBox(const BoxCoords &box, int x, int y, int16& outX, int16& outY) {
+	const Common::Point p(x, y);
+	Common::Point tmp;
 	uint dist;
 	uint bestdist = 0xFFFFFF;
-	BoxCoords box;
 
-	getBoxCoordinates(b, &box);
-
-	pt = closestPtOnLine(box.ul, box.ur, x, y);
-	dist = distanceFromPt(x, y, pt.x, pt.y);
+	tmp = closestPtOnLine(box.ul, box.ur, p);
+	dist = p.sqrDist(tmp);
 	if (dist < bestdist) {
 		bestdist = dist;
-		outX = pt.x;
-		outY = pt.y;
+		outX = tmp.x;
+		outY = tmp.y;
 	}
 
-	pt = closestPtOnLine(box.ur, box.lr, x, y);
-	dist = distanceFromPt(x, y, pt.x, pt.y);
+	tmp = closestPtOnLine(box.ur, box.lr, p);
+	dist = p.sqrDist(tmp);
 	if (dist < bestdist) {
 		bestdist = dist;
-		outX = pt.x;
-		outY = pt.y;
+		outX = tmp.x;
+		outY = tmp.y;
 	}
 
-	pt = closestPtOnLine(box.lr, box.ll, x, y);
-	dist = distanceFromPt(x, y, pt.x, pt.y);
+	tmp = closestPtOnLine(box.lr, box.ll, p);
+	dist = p.sqrDist(tmp);
 	if (dist < bestdist) {
 		bestdist = dist;
-		outX = pt.x;
-		outY = pt.y;
+		outX = tmp.x;
+		outY = tmp.y;
 	}
 
-	pt = closestPtOnLine(box.ll, box.ul, x, y);
-	dist = distanceFromPt(x, y, pt.x, pt.y);
+	tmp = closestPtOnLine(box.ll, box.ul, p);
+	dist = p.sqrDist(tmp);
 	if (dist < bestdist) {
 		bestdist = dist;
-		outX = pt.x;
-		outY = pt.y;
+		outX = tmp.x;
+		outY = tmp.y;
 	}
 
 	return bestdist;
@@ -719,13 +686,13 @@ byte *ScummEngine::getBoxMatrixBaseAddr() {
 	return ptr;
 }
 
-/*
+/**
  * Compute if there is a way that connects box 'from' with box 'to'.
- * Returns the number of a box adjactant to 'from' that is the next on the
+ * Returns the number of a box adjacent to 'from' that is the next on the
  * way to 'to' (this can be 'to' itself or a third box).
  * If there is no connection -1 is return.
  */
-int ScummEngine::getPathToDestBox(byte from, byte to) {
+int ScummEngine::getNextBox(byte from, byte to) {
 	const byte *boxm;
 	byte i;
 	const int numOfBoxes = getNumBoxes();
@@ -813,16 +780,14 @@ int ScummEngine::getPathToDestBox(byte from, byte to) {
  * line in order to get from box1 to box3 via box2.
  */
 bool Actor::findPathTowards(byte box1nr, byte box2nr, byte box3nr, Common::Point &foundPath) {
-	BoxCoords box1;
-	BoxCoords box2;
+	assert(_vm->_game.version >= 3);
+	BoxCoords box1 = _vm->getBoxCoordinates(box1nr);
+	BoxCoords box2 = _vm->getBoxCoordinates(box2nr);
 	Common::Point tmp;
 	int i, j;
 	int flag;
 	int q, pos;
 
-	_vm->getBoxCoordinates(box1nr, &box1);
-	_vm->getBoxCoordinates(box2nr, &box2);
-
 	for (i = 0; i < 4; i++) {
 		for (j = 0; j < 4; j++) {
 			if (box1.ul.x == box1.ur.x && box1.ul.x == box2.ul.x && box1.ul.x == box2.ur.x) {
@@ -1041,9 +1006,9 @@ void ScummEngine::createBoxMatrix() {
 	// itineray box. Assuming we are in the 5th "row" and encounter
 	// the triplet 7,11,15: this means to get from box 5 to any of
 	// the boxes 7,8,9,10,11 the shortest way is to go via box 15.
-	// See also getPathToDestBox.
+	// See also getNextBox.
 
-	byte *matrixStart = res.createResource(rtMatrix, 1, BOX_MATRIX_SIZE);
+	byte *matrixStart = _res->createResource(rtMatrix, 1, BOX_MATRIX_SIZE);
 	const byte *matrixEnd = matrixStart + BOX_MATRIX_SIZE;
 
 	#define addToMatrix(b)	do { *matrixStart++ = (b); assert(matrixStart < matrixEnd); } while (0)
@@ -1081,17 +1046,18 @@ bool ScummEngine::areBoxesNeighbours(int box1nr, int box2nr) {
 	BoxCoords box;
 	BoxCoords box2;
 
-	if (getBoxFlags(box1nr) & kBoxInvisible || getBoxFlags(box2nr) & kBoxInvisible)
+	if ((getBoxFlags(box1nr) & kBoxInvisible) || (getBoxFlags(box2nr) & kBoxInvisible))
 		return false;
 
-	getBoxCoordinates(box1nr, &box2);
-	getBoxCoordinates(box2nr, &box);
+	assert(_game.version >= 3);
+	box2 = getBoxCoordinates(box1nr);
+	box = getBoxCoordinates(box2nr);
 
-	// Roughly, the idea of this algorithm is to check if we can find
-	// two sides of the two given boxes which touch.
+	// Roughly, the idea of this algorithm is to search for sies of the given
+	// boxes that touch each other.
 	// In order to keep te code simple, we only match the upper sides;
 	// then, we "rotate" the box coordinates four times each, for a total
-	// of 16 comparisions (four sides compared with four sides each).
+	// of 16 comparisions.
 	for (int j = 0; j < 4; j++) {
 		for (int k = 0; k < 4; k++) {
 			// Are the "upper" sides of the boxes on a single vertical line
@@ -1099,11 +1065,11 @@ bool ScummEngine::areBoxesNeighbours(int box1nr, int box2nr) {
 			if (box2.ur.x == box2.ul.x && box.ul.x == box2.ul.x && box.ur.x == box2.ul.x) {
 				bool swappedBox2 = false, swappedBox1 = false;
 				if (box2.ur.y < box2.ul.y) {
-					swappedBox2 = 1;
+					swappedBox2 = true;
 					SWAP(box2.ur.y, box2.ul.y);
 				}
 				if (box.ur.y < box.ul.y) {
-					swappedBox1 = 1;
+					swappedBox1 = true;
 					SWAP(box.ur.y, box.ul.y);
 				}
 				if (box.ur.y < box2.ul.y ||
@@ -1128,11 +1094,11 @@ bool ScummEngine::areBoxesNeighbours(int box1nr, int box2nr) {
 			if (box2.ur.y == box2.ul.y && box.ul.y == box2.ul.y && box.ur.y == box2.ul.y) {
 				bool swappedBox2 = false, swappedBox1 = false;
 				if (box2.ur.x < box2.ul.x) {
-					swappedBox2 = 1;
+					swappedBox2 = true;
 					SWAP(box2.ur.x, box2.ul.x);
 				}
 				if (box.ur.x < box.ul.x) {
-					swappedBox1 = 1;
+					swappedBox1 = true;
 					SWAP(box.ur.x, box.ul.x);
 				}
 				if (box.ur.x < box2.ul.x ||
@@ -1172,12 +1138,12 @@ bool ScummEngine::areBoxesNeighbours(int box1nr, int box2nr) {
 	return false;
 }
 
-void Actor::findPathTowardsOld(byte box1, byte box2, byte finalBox, Common::Point &p2, Common::Point &p3) {
+void Actor_v3::findPathTowardsOld(byte box1, byte box2, byte finalBox, Common::Point &p2, Common::Point &p3) {
 	Common::Point pt;
 	Common::Point gateA[2];
 	Common::Point gateB[2];
 
-	_vm->getGates(box1, box2, gateA, gateB);
+	getGates(_vm->getBoxCoordinates(box1), _vm->getBoxCoordinates(box2), gateA, gateB);
 
 	p2.x = 32000;
 	p3.x = 32000;
@@ -1185,24 +1151,22 @@ void Actor::findPathTowardsOld(byte box1, byte box2, byte finalBox, Common::Poin
 	// next box (box2) = final box?
 	if (box2 == finalBox) {
 		// In Indy3, the masks (= z-level) have to match, too -- needed for the
-		// 'maze' in the zeppeling (see bug #1032964).
+		// 'maze' in the zeppelin (see bug #1032964).
 		if (_vm->_game.id != GID_INDY3 || _vm->getMaskFromBox(box1) == _vm->getMaskFromBox(box2)) {
 			// Is the actor (x,y) between both gates?
-			if (compareSlope(_pos.x, _pos.y, _walkdata.dest.x, _walkdata.dest.y, gateA[0].x, gateA[0].y) !=
-					compareSlope(_pos.x, _pos.y, _walkdata.dest.x, _walkdata.dest.y, gateB[0].x, gateB[0].y) &&
-					compareSlope(_pos.x, _pos.y, _walkdata.dest.x, _walkdata.dest.y, gateA[1].x, gateA[1].y) !=
-					compareSlope(_pos.x, _pos.y, _walkdata.dest.x, _walkdata.dest.y, gateB[1].x, gateB[1].y)) {
+			if (compareSlope(_pos, _walkdata.dest, gateA[0]) !=
+					compareSlope(_pos, _walkdata.dest, gateB[0]) &&
+					compareSlope(_pos, _walkdata.dest, gateA[1]) !=
+					compareSlope(_pos, _walkdata.dest, gateB[1])) {
 				return;
 			}
 		}
 	}
 
-	p3 = pt = closestPtOnLine(gateA[1], gateB[1], _pos.x, _pos.y);
+	p3 = closestPtOnLine(gateA[1], gateB[1], _pos);
 
-	if (compareSlope(_pos.x, _pos.y, p3.x, p3.y, gateA[0].x, gateA[0].y) ==
-			compareSlope(_pos.x, _pos.y, p3.x, p3.y, gateB[0].x, gateB[0].y)) {
-		closestPtOnLine(gateA[0], gateB[0], _pos.x, _pos.y);
-		p2 = pt;	// if point 2 between gates, ignore!
+	if (compareSlope(_pos, p3, gateA[0]) == compareSlope(_pos, p3, gateB[0])) {
+		p2 = closestPtOnLine(gateA[0], gateB[0], _pos);
 	}
 }
 
@@ -1210,38 +1174,35 @@ void Actor::findPathTowardsOld(byte box1, byte box2, byte finalBox, Common::Poin
  * Compute the "gate" between two boxes. The gate is a pair of two lines which
  * both start on box 'box1' and end on 'box2'. For both lines, one of its
  * end points is the corner point of one of the two boxes. The other end point
- * is a point on the other point closest to first end point.
- * This way the lines bound a 'corridor' between the two boxes, through which
- * the actor has to walk to get from box1 to box2.
+ * is the point on the other box closest to the first end point.
+ * This way the lines form the boundary of a 'corridor' between the two boxes,
+ * through which the actor has to walk to get from box1 to box2.
  */
-void ScummEngine::getGates(int box1, int box2, Common::Point gateA[2], Common::Point gateB[2]) {
+void getGates(const BoxCoords &box1, const BoxCoords &box2, Common::Point gateA[2], Common::Point gateB[2]) {
 	int i, j;
 	int dist[8];
 	int minDist[3];
 	int closest[3];
 	int box[3];
-	BoxCoords coords;
 	Common::Point closestPoint[8];
 	Common::Point boxCorner[8];
 	int line1, line2;
 
 	// For all corner coordinates of the first box, compute the point closest
 	// to them on the second box (and also compute the distance of these points).
-	getBoxCoordinates(box1, &coords);
-	boxCorner[0] = coords.ul;
-	boxCorner[1] = coords.ur;
-	boxCorner[2] = coords.lr;
-	boxCorner[3] = coords.ll;
+	boxCorner[0] = box1.ul;
+	boxCorner[1] = box1.ur;
+	boxCorner[2] = box1.lr;
+	boxCorner[3] = box1.ll;
 	for (i = 0; i < 4; i++) {
 		dist[i] = getClosestPtOnBox(box2, boxCorner[i].x, boxCorner[i].y, closestPoint[i].x, closestPoint[i].y);
 	}
 
 	// Now do the same but with the roles of the first and second box swapped.
-	getBoxCoordinates(box2, &coords);
-	boxCorner[4] = coords.ul;
-	boxCorner[5] = coords.ur;
-	boxCorner[6] = coords.lr;
-	boxCorner[7] = coords.ll;
+	boxCorner[4] = box2.ul;
+	boxCorner[5] = box2.ur;
+	boxCorner[6] = box2.lr;
+	boxCorner[7] = box2.ll;
 	for (i = 4; i < 8; i++) {
 		dist[i] = getClosestPtOnBox(box1, boxCorner[i].x, boxCorner[i].y, closestPoint[i].x, closestPoint[i].y);
 	}
