@@ -1,6 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2001  Ludvig Strigeus
- * Copyright (C) 2001-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -16,13 +18,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/scumm/boxes.h $
- * $Id: boxes.h 21084 2006-03-05 03:46:41Z kirben $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/scumm/boxes.h $
+ * $Id: boxes.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
-#ifndef BOXES_H
-#define BOXES_H
+#ifndef SCUMM_BOXES_H
+#define SCUMM_BOXES_H
 
 #include "common/rect.h"
 
@@ -50,6 +52,8 @@ struct BoxCoords {			/* Box coordinates */
 	Common::Point lr;
 };
 
+int getClosestPtOnBox(const BoxCoords &box, int x, int y, int16& outX, int16& outY);
+
 } // End of namespace Scumm
 
 #endif
