@@ -1,6 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2001  Ludvig Strigeus
- * Copyright (C) 2001-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +17,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/scumm/camera.cpp $
- * $Id: camera.cpp 21355 2006-03-18 10:30:47Z kirben $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/scumm/camera.cpp $
+ * $Id: camera.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -29,7 +31,7 @@
 namespace Scumm {
 
 void ScummEngine::setCameraAtEx(int at) {
-	if (!(_game.features & GF_NEW_CAMERA)) {
+	if (_game.version < 7) {
 		camera._mode = kNormalCameraMode;
 		camera._cur.x = at;
 		setCameraAt(at, 0);
@@ -69,18 +71,18 @@ void ScummEngine::setCameraFollows(Actor *a, bool setCamera) {
 	if (!a->isInCurrentRoom()) {
 		startScene(a->getRoom(), 0, 0);
 		camera._mode = kFollowActorCameraMode;
-		camera._cur.x = a->_pos.x;
+		camera._cur.x = a->getPos().x;
 		setCameraAt(camera._cur.x, 0);
 	}
 
-	t = a->_pos.x / 8 - _screenStartStrip;
+	t = a->getPos().x / 8 - _screenStartStrip;
 
 	if (t < camera._leftTrigger || t  > camera._rightTrigger || setCamera == true)
-		setCameraAt(a->_pos.x, 0);
+		setCameraAt(a->getPos().x, 0);
 
 	for (i = 1; i < _numActors; i++) {
-		if (_actors[i].isInCurrentRoom())
-			_actors[i]._needRedraw = true;
+		if (_actors[i]->isInCurrentRoom())
+			_actors[i]->_needRedraw = true;
 	}
 	runInventoryScript(0);
 }
@@ -101,13 +103,14 @@ void ScummEngine::clampCameraPos(Common::Point *pt) {
 
 void ScummEngine::moveCamera() {
 	int pos = camera._cur.x;
-	int actorx, t;
+	int t;
 	Actor *a = NULL;
+	const bool snapToX = (VAR_CAMERA_FAST_X != 0xFF && VAR(VAR_CAMERA_FAST_X));
 
 	camera._cur.x &= 0xFFF8;
 
 	if (VAR_CAMERA_MIN_X != 0xFF && camera._cur.x < VAR(VAR_CAMERA_MIN_X)) {
-		if (VAR_CAMERA_FAST_X != 0xFF && VAR(VAR_CAMERA_FAST_X))
+		if (snapToX)
 			camera._cur.x = (short) VAR(VAR_CAMERA_MIN_X);
 		else
 			camera._cur.x += 8;
@@ -116,7 +119,7 @@ void ScummEngine::moveCamera() {
 	}
 
 	if (VAR_CAMERA_MAX_X != 0xFF && camera._cur.x > VAR(VAR_CAMERA_MAX_X)) {
-		if (VAR_CAMERA_FAST_X != 0xFF && VAR(VAR_CAMERA_FAST_X))
+		if (snapToX)
 			camera._cur.x = (short) VAR(VAR_CAMERA_MAX_X);
 		else
 			camera._cur.x -= 8;
@@ -127,12 +130,12 @@ void ScummEngine::moveCamera() {
 	if (camera._mode == kFollowActorCameraMode) {
 		a = derefActor(camera._follows, "moveCamera");
 
-		actorx = a->_pos.x;
+		int actorx = a->getPos().x;
 		t = actorx / 8 - _screenStartStrip;
 
 		if (t < camera._leftTrigger || t > camera._rightTrigger) {
-			if (VAR_CAMERA_FAST_X != 0xFF && VAR(VAR_CAMERA_FAST_X)) {
-				if (t > 35)
+			if (snapToX) {
+				if (t > 40-5)
 					camera._dest.x = actorx + 80;
 				if (t < 5)
 					camera._dest.x = actorx - 80;
@@ -143,7 +146,7 @@ void ScummEngine::moveCamera() {
 
 	if (camera._movingToActor) {
 		a = derefActor(camera._follows, "moveCamera(2)");
-		camera._dest.x = a->_pos.x;
+		camera._dest.x = a->getPos().x;
 	}
 
 	if (VAR_CAMERA_MIN_X != 0xFF && camera._dest.x < VAR(VAR_CAMERA_MIN_X))
@@ -152,7 +155,7 @@ void ScummEngine::moveCamera() {
 	if (VAR_CAMERA_MAX_X != 0xFF && camera._dest.x > VAR(VAR_CAMERA_MAX_X))
 		camera._dest.x = (short) VAR(VAR_CAMERA_MAX_X);
 
-	if (VAR_CAMERA_FAST_X != 0xFF && VAR(VAR_CAMERA_FAST_X)) {
+	if (snapToX) {
 		camera._cur.x = camera._dest.x;
 	} else {
 		if (camera._cur.x < camera._dest.x)
@@ -161,8 +164,8 @@ void ScummEngine::moveCamera() {
 			camera._cur.x -= 8;
 	}
 
-	/* a is set a bit above */
-	if (camera._movingToActor && (camera._cur.x / 8) == (a->_pos.x / 8)) {
+	/* Actor 'a' is set a bit above */
+	if (camera._movingToActor && (camera._cur.x / 8) == (a->getPos().x / 8)) {
 		camera._movingToActor = false;
 	}
 
@@ -176,7 +179,7 @@ void ScummEngine::moveCamera() {
 
 void ScummEngine::cameraMoved() {
 	int screenLeft;
-	if (_game.features & GF_NEW_CAMERA) {
+	if (_game.version >= 7) {
 		assert(camera._cur.x >= (_screenWidth / 2) && camera._cur.y >= (_screenHeight / 2));
 	} else {
 		if (camera._cur.x < (_screenWidth / 2)) {
@@ -186,11 +189,11 @@ void ScummEngine::cameraMoved() {
 		}
 	}
 
-	_screenStartStrip = camera._cur.x / 8 - gdi._numStrips / 2;
-	_screenEndStrip = _screenStartStrip + gdi._numStrips - 1;
+	_screenStartStrip = camera._cur.x / 8 - _gdi->_numStrips / 2;
+	_screenEndStrip = _screenStartStrip + _gdi->_numStrips - 1;
 
 	_screenTop = camera._cur.y - (_screenHeight / 2);
-	if (_game.features & GF_NEW_CAMERA) {
+	if (_game.version >= 7) {
 		screenLeft = camera._cur.x - (_screenWidth / 2);
 	} else {
 		screenLeft = _screenStartStrip * 8;
@@ -206,7 +209,7 @@ void ScummEngine::panCameraTo(int x, int y) {
 }
 
 void ScummEngine::actorFollowCamera(int act) {
-	if (!(_game.features & GF_NEW_CAMERA)) {
+	if (_game.version < 7) {
 		int old;
 
 		old = camera._follows;
@@ -262,11 +265,11 @@ void ScummEngine_v7::setCameraFollows(Actor *a, bool setCamera) {
 		startScene(a->getRoom(), 0, 0);
 	}
 
-	ax = ABS(a->_pos.x - camera._cur.x);
-	ay = ABS(a->_pos.y - camera._cur.y);
+	ax = ABS(a->getPos().x - camera._cur.x);
+	ay = ABS(a->getPos().y - camera._cur.y);
 
 	if (ax > VAR(VAR_CAMERA_THRESHOLD_X) || ay > VAR(VAR_CAMERA_THRESHOLD_Y) || ax > (_screenWidth / 2) || ay > (_screenHeight / 2)) {
-		setCameraAt(a->_pos.x, a->_pos.y);
+		setCameraAt(a->getPos().x, a->getPos().y);
 	}
 
 	if (a->_number != oldfollow)
@@ -279,13 +282,13 @@ void ScummEngine_v7::moveCamera() {
 
 	if (camera._follows) {
 		a = derefActor(camera._follows, "moveCamera");
-		if (ABS(camera._cur.x - a->_pos.x) > VAR(VAR_CAMERA_THRESHOLD_X) ||
-				ABS(camera._cur.y - a->_pos.y) > VAR(VAR_CAMERA_THRESHOLD_Y)) {
+		if (ABS(camera._cur.x - a->getPos().x) > VAR(VAR_CAMERA_THRESHOLD_X) ||
+				ABS(camera._cur.y - a->getPos().y) > VAR(VAR_CAMERA_THRESHOLD_Y)) {
 			camera._movingToActor = true;
 			if (VAR(VAR_CAMERA_THRESHOLD_X) == 0)
-				camera._cur.x = a->_pos.x;
+				camera._cur.x = a->getPos().x;
 			if (VAR(VAR_CAMERA_THRESHOLD_Y) == 0)
-				camera._cur.y = a->_pos.y;
+				camera._cur.y = a->getPos().y;
 			clampCameraPos(&camera._cur);
 		}
 	} else {
@@ -293,8 +296,8 @@ void ScummEngine_v7::moveCamera() {
 	}
 
 	if (camera._movingToActor) {
-		VAR(VAR_CAMERA_DEST_X) = camera._dest.x = a->_pos.x;
-		VAR(VAR_CAMERA_DEST_Y) = camera._dest.y = a->_pos.y;
+		VAR(VAR_CAMERA_DEST_X) = camera._dest.x = a->getPos().x;
+		VAR(VAR_CAMERA_DEST_Y) = camera._dest.y = a->getPos().y;
 	}
 
 	assert(camera._cur.x >= (_screenWidth / 2) && camera._cur.y >= (_screenHeight / 2));
