@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2001-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/scumm/charset.cpp $
- * $Id: charset.cpp 22397 2006-05-09 14:16:43Z sev $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/scumm/charset.cpp $
+ * $Id: charset.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  */
 
 #include "common/stdafx.h"
@@ -24,10 +27,24 @@
 #include "scumm/scumm.h"
 #include "scumm/nut_renderer.h"
 #include "scumm/util.h"
+#include "scumm/he/intern_he.h"
 #include "scumm/he/wiz_he.h"
 
 namespace Scumm {
 
+/*
+TODO:
+Right now our charset renderers directly access _textSurface, as well as the
+virtual screens of ScummEngine. Ideally, this would not be the case. Instead,
+ScummVM would simply pass the appropriate Surface to the resp. methods. 
+Of course it is not quite as simple, various flags and offsets have to
+be taken into account for that.
+
+The advantage will be cleaner coder (easier to debug, in particular), and a 
+better separation of the various modules.
+*/
+
+
 void ScummEngine::loadCJKFont() {
 	Common::File fp;
 	_useCJKMode = false;
@@ -185,10 +202,6 @@ byte *ScummEngine::get2byteCharPtr(int idx) {
 
 
 CharsetRenderer::CharsetRenderer(ScummEngine *vm) {
-
-	_nextLeft = 0;
-	_nextTop = 0;
-
 	_top = 0;
 	_left = 0;
 	_startLeft = 0;
@@ -205,18 +218,9 @@ CharsetRenderer::CharsetRenderer(ScummEngine *vm) {
 
 	_vm = vm;
 	_curId = 0;
-
-	const int size = _vm->_screenWidth * _vm->_screenHeight;
-	_textSurface.pixels = malloc(size);
-	_textSurface.w = _vm->_screenWidth;
-	_textSurface.h = _vm->_screenHeight;
-	_textSurface.pitch = _vm->_screenWidth;
-	_textSurface.bytesPerPixel = 1;
-	clearTextSurface();
 }
 
 CharsetRenderer::~CharsetRenderer() {
-	free(_textSurface.pixels);
 }
 
 CharsetRendererCommon::CharsetRendererCommon(ScummEngine *vm)
@@ -226,7 +230,7 @@ CharsetRendererCommon::CharsetRendererCommon(ScummEngine *vm)
 }
 
 void CharsetRendererCommon::setCurID(byte id) {
-	checkRange(_vm->_numCharsets - 1, 0, id, "Printing with bad charset %d");
+	assertRange(0, id, _vm->_numCharsets - 1, "charset");
 
 	_curId = id;
 
@@ -245,7 +249,7 @@ void CharsetRendererCommon::setCurID(byte id) {
 }
 
 void CharsetRendererV3::setCurID(byte id) {
-	checkRange(_vm->_numCharsets - 1, 0, id, "Printing with bad charset %d");
+	assertRange(0, id, _vm->_numCharsets - 1, "charset");
 
 	_curId = id;
 
@@ -429,14 +433,14 @@ void CharsetRenderer::addLinebreaks(int a, byte *str, int pos, int maxwidth) {
 }
 
 #ifdef PALMOS_68K
-static byte *englishCharsetDataV2;
-static byte *germanCharsetDataV2;
-static byte *frenchCharsetDataV2;
-static byte *italianCharsetDataV2;
-static byte *spanishCharsetDataV2;
+static const byte *englishCharsetDataV2;
+static const byte *germanCharsetDataV2;
+static const byte *frenchCharsetDataV2;
+static const byte *italianCharsetDataV2;
+static const byte *spanishCharsetDataV2;
 #else
 // English Zak font
-static byte englishCharsetDataV2[] = {
+static const byte englishCharsetDataV2[] = {
 	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
 	0x01, 0x03, 0x06, 0x0C, 0x18, 0x3E, 0x03, 0x00,
 	0x80, 0xC0, 0x60, 0x30, 0x18, 0x7C, 0xC0, 0x00,
@@ -568,7 +572,7 @@ static byte englishCharsetDataV2[] = {
 };
 
 // German Zak font
-static byte germanCharsetDataV2[] = {
+static const byte germanCharsetDataV2[] = {
 	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
 	0x01, 0x03, 0x06, 0x0c, 0x18, 0x3e, 0x03, 0x00,
 	0x80, 0xc0, 0x60, 0x30, 0x18, 0x7c, 0xc0, 0x00,
@@ -700,7 +704,7 @@ static byte germanCharsetDataV2[] = {
 };
 
 // French Zak font.
-static byte frenchCharsetDataV2[] = {
+static const byte frenchCharsetDataV2[] = {
 	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
 	0x01, 0x03, 0x06, 0x0c, 0x18, 0x3e, 0x03, 0x00,
 	0x80, 0xc0, 0x60, 0x30, 0x18, 0x7c, 0xc0, 0x00,
@@ -832,7 +836,7 @@ static byte frenchCharsetDataV2[] = {
 };
 
 // Italian Zak font.
-static byte italianCharsetDataV2[] = {
+static const byte italianCharsetDataV2[] = {
 	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
 	0x01, 0x03, 0x06, 0x0c, 0x18, 0x3e, 0x03, 0x00,
 	0x80, 0xc0, 0x60, 0x30, 0x18, 0x7c, 0xc0, 0x00,
@@ -965,7 +969,7 @@ static byte italianCharsetDataV2[] = {
 
 // Spanish Zak font.
 // FIXME: This is identical to germanCharsetDataV2 it seems?!
-static byte spanishCharsetDataV2[] = {
+static const byte spanishCharsetDataV2[] = {
 	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
 	0x01, 0x03, 0x06, 0x0c, 0x18, 0x3e, 0x03, 0x00,
 	0x80, 0xc0, 0x60, 0x30, 0x18, 0x7c, 0xc0, 0x00,
@@ -1203,10 +1207,11 @@ void CharsetRendererV3::printChar(int chr, bool ignoreCharsetMask) {
 	// Indy3 / Zak256 / Loom
 	int width, height, origWidth = 0, origHeight;
 	VirtScreen *vs;
-	byte *charPtr, *dst;
+	const byte *charPtr;
+	byte *dst;
 	int is2byte = (chr >= 0x80 && _vm->_useCJKMode) ? 1 : 0;
 
-	checkRange(_vm->_numCharsets - 1, 0, _curId, "Printing with bad charset %d");
+	assertRange(0, _curId, _vm->_numCharsets - 1, "charset");
 
 	if ((vs = _vm->findVirtScreen(_top)) == NULL)
 		return;
@@ -1256,8 +1261,8 @@ void CharsetRendererV3::printChar(int chr, bool ignoreCharsetMask) {
 		dst = vs->getPixels(_left, drawTop);
 		drawBits1(*vs, dst, charPtr, drawTop, origWidth, origHeight);
 	} else {
-		dst = (byte *)_textSurface.pixels + _top * _textSurface.pitch + _left;
-		drawBits1(_textSurface, dst, charPtr, drawTop, origWidth, origHeight);
+		dst = (byte *)_vm->_textSurface.pixels + _top * _vm->_textSurface.pitch + _left;
+		drawBits1(_vm->_textSurface, dst, charPtr, drawTop, origWidth, origHeight);
 	}
 
 	if (_str.left > _left)
@@ -1276,7 +1281,8 @@ void CharsetRendererV3::printChar(int chr, bool ignoreCharsetMask) {
 }
 
 void CharsetRendererV3::drawChar(int chr, const Graphics::Surface &s, int x, int y) {
-	byte *charPtr, *dst;
+	const byte *charPtr;
+	byte *dst;
 	int width, height;
 	int is2byte = (chr >= 0x80 && _vm->_useCJKMode) ? 1 : 0;
 	if (is2byte) {
@@ -1296,27 +1302,41 @@ void CharsetRendererV3::drawChar(int chr, const Graphics::Surface &s, int x, int
 void CharsetRenderer::translateColor() {
 	// Based on disassembly
 	if (_vm->_renderMode == Common::kRenderCGA) {
-		static byte CGAtextColorMap[16] = {0,  3, 3, 3, 5, 5, 5,  15,
+		static const byte CGAtextColorMap[16] = {0,  3, 3, 3, 5, 5, 5,  15,
 										   15, 3, 3, 3, 5, 5, 15, 15};
 		_color = CGAtextColorMap[_color & 0x0f];
 	}
 
 	if (_vm->_renderMode == Common::kRenderHercA || _vm->_renderMode == Common::kRenderHercG) {
-		static byte HercTextColorMap[16] = {0, 15,  2, 15, 15,  5, 15,  15,
+		static const byte HercTextColorMap[16] = {0, 15,  2, 15, 15,  5, 15,  15,
 										   8, 15, 15, 15, 15, 15, 15, 15};
 		_color = HercTextColorMap[_color & 0x0f];
 	}
 }
 
+void CharsetRenderer::saveLoadWithSerializer(Serializer *ser) {
+	static const SaveLoadEntry charsetRendererEntries[] = {
+		MKLINE(CharsetRenderer, _curId, sleByte, VER(73)),
+		MKLINE(CharsetRenderer, _color, sleByte, VER(73)),
+		MKEND()
+	};
+
+	ser->saveLoadEntries(this, charsetRendererEntries);
+
+	if (ser->isLoading()) {
+		setCurID(_curId);
+		setColor(_color);
+	}
+}
 
 void CharsetRendererClassic::printChar(int chr, bool ignoreCharsetMask) {
 	int width, height, origWidth, origHeight;
 	int offsX, offsY;
 	VirtScreen *vs;
 	const byte *charPtr;
-	int is2byte = (chr >= 0x80 && _vm->_useCJKMode) ? 1 : 0;
+	bool is2byte = (chr >= 0x80 && _vm->_useCJKMode);
 
-	checkRange(_vm->_numCharsets - 1, 1, _curId, "Printing with bad charset %d");
+	assertRange(1, _curId, _vm->_numCharsets - 1, "charset");
 
 	if ((vs = _vm->findVirtScreen(_top)) == NULL && (vs = _vm->findVirtScreen(_top + getFontHeight())) == NULL)
 		return;
@@ -1397,20 +1417,38 @@ void CharsetRendererClassic::printChar(int chr, bool ignoreCharsetMask) {
 
 	_vm->markRectAsDirty(vs->number, _left, _left + width, drawTop, drawTop + height);
 
-	byte *dstPtr;
-	byte *back = NULL;
-
 	if (!ignoreCharsetMask) {
 		_hasMask = true;
 		_textScreenID = vs->number;
 	}
 
+	printCharIntern(is2byte, charPtr, origWidth, origHeight, width, height, vs, ignoreCharsetMask);
+
+	_left += origWidth;
+
+	if (_str.right < _left) {
+		_str.right = _left;
+		if (_shadowMode != kNoShadowMode)
+			_str.right++;
+	}
+
+	if (_str.bottom < _top + height)
+		_str.bottom = _top + height;
+
+	_top -= offsY;
+}
+
+void CharsetRendererClassic::printCharIntern(bool is2byte, const byte *charPtr, int origWidth, int origHeight, int width, int height, VirtScreen *vs, bool ignoreCharsetMask) {
+	byte *dstPtr;
+	byte *back = NULL;
+	int drawTop = _top - vs->topline;
+
 	if ((_vm->_game.heversion >= 71 && _bitDepth >= 8) || (_vm->_game.heversion >= 90 && _bitDepth == 0)) {
 #ifndef DISABLE_HE
 		if (ignoreCharsetMask || !vs->hasTwoBuffers) {
 			dstPtr = vs->getPixels(0, 0);
 		} else {
-			dstPtr = (byte *)_textSurface.pixels;
+			dstPtr = (byte *)_vm->_textSurface.pixels;
 		}
 
 		if (_blitAlso && vs->hasTwoBuffers) {
@@ -1429,7 +1467,7 @@ void CharsetRendererClassic::printChar(int chr, bool ignoreCharsetMask) {
 
 		if (_blitAlso && vs->hasTwoBuffers) {
 			Common::Rect dst(_left, _top, _left + origWidth, _top + origHeight);
-			_vm->gdi.copyVirtScreenBuffers(dst);
+			((ScummEngine_v71he *)_vm)->restoreBackgroundHE(dst);
 		}
 #endif
 	} else {
@@ -1439,8 +1477,8 @@ void CharsetRendererClassic::printChar(int chr, bool ignoreCharsetMask) {
 			dstSurface = *vs;
 			dstPtr = vs->getPixels(_left, drawTop);
 		} else {
-			dstSurface = _textSurface;
-			dstPtr = (byte *)_textSurface.pixels + (_top - _vm->_screenTop) * _textSurface.pitch + _left;
+			dstSurface = _vm->_textSurface;
+			dstPtr = (byte *)_vm->_textSurface.pixels + (_top - _vm->_screenTop) * _vm->_textSurface.pitch + _left;
 		}
 
 		if (_blitAlso && vs->hasTwoBuffers) {
@@ -1495,19 +1533,6 @@ void CharsetRendererClassic::printChar(int chr, bool ignoreCharsetMask) {
 			}
 		}
 	}
-
-	_left += origWidth;
-
-	if (_str.right < _left) {
-		_str.right = _left;
-		if (_shadowMode != kNoShadowMode)
-			_str.right++;
-	}
-
-	if (_str.bottom < _top + height)
-		_str.bottom = _top + height;
-
-	_top -= offsY;
 }
 
 void CharsetRendererClassic::drawChar(int chr, const Graphics::Surface &s, int x, int y) {
@@ -1601,29 +1626,25 @@ CharsetRendererNut::CharsetRendererNut(ScummEngine *vm)
 	_current = 0;
 
 	for (int i = 0; i < 5; i++) {
-		char fontname[256];
-		if ((_vm->_game.id == GID_CMI) && (_vm->_game.features & GF_DEMO) && (i == 4))
-			break;
-		sprintf(fontname, "font%d.nut", i);
-		_fr[i] = new NutRenderer(_vm);
-		if (!(_fr[i]->loadFont(fontname))) {
-			delete _fr[i];
-			_fr[i] = NULL;
-		}
+		_fr[i] = NULL;
 	}
 }
 
 CharsetRendererNut::~CharsetRendererNut() {
 	for (int i = 0; i < 5; i++) {
-		if ((_vm->_game.id == GID_CMI) && (_vm->_game.features & GF_DEMO) && (i == 4))
-			break;
 		delete _fr[i];
 	}
 }
 
 void CharsetRendererNut::setCurID(byte id) {
-	assert(id < 5);
+	int numFonts = ((_vm->_game.id == GID_CMI) && (_vm->_game.features & GF_DEMO)) ? 4 : 5;
+	assert(id < numFonts);
 	_curId = id;
+	if (!_fr[id]) {
+		char fontname[11];
+		sprintf(fontname, "font%d.nut", id);
+		_fr[id] = new NutRenderer(_vm, fontname);
+	}
 	_current = _fr[id];
 	assert(_current);
 }
@@ -1651,12 +1672,8 @@ void CharsetRendererNut::printChar(int chr, bool ignoreCharsetMask) {
 	if (chr == '@')
 		return;
 
-	shadow.left = _left - 1;
-	shadow.top = _top - 1;
-
-	// Note that the character is drawn with a shadow, so it is slightly
-	// larger than the advertised dimensions. See drawShadowChar() for
-	// details.
+	shadow.left = _left;
+	shadow.top = _top;
 
 	if (_firstChar) {
 		_str.left = (shadow.left >= 0) ? shadow.left : 0;
@@ -1672,8 +1689,8 @@ void CharsetRendererNut::printChar(int chr, bool ignoreCharsetMask) {
 	if (chr >= 256 && _vm->_useCJKMode)
 		width = _vm->_2byteWidth;
 
-	shadow.right = _left + width + 2;
-	shadow.bottom = _top + height + 2;
+	shadow.right = _left + width;
+	shadow.bottom = _top + height;
 
 	Graphics::Surface s;
 	if (!ignoreCharsetMask) {
@@ -1687,11 +1704,14 @@ void CharsetRendererNut::printChar(int chr, bool ignoreCharsetMask) {
 		s = *vs;
 		s.pixels = vs->getPixels(0, 0);
 	} else {
-		s = _textSurface;
+		s = _vm->_textSurface;
 		drawTop -= _vm->_screenTop;
 	}
 
-	_current->drawShadowChar(s, chr, _left, drawTop, _color, _curId != 3);
+	if (chr >= 256 && _vm->_useCJKMode)
+		_current->draw2byte(s, chr, _left, drawTop, _color);
+	else
+		_current->drawChar(s, (byte)chr, _left, drawTop, _color);
 	_vm->markRectAsDirty(kMainVirtScreen, shadow);
 
 	if (_str.left > _left)
@@ -1754,8 +1774,8 @@ void CharsetRendererNES::printChar(int chr, bool ignoreCharsetMask) {
 		dst = vs->getPixels(_left, drawTop);
 		drawBits1(*vs, dst, charPtr, drawTop, origWidth, origHeight);
 	} else {
-		dst = (byte *)_textSurface.pixels + _top * _textSurface.pitch + _left;
-		drawBits1(_textSurface, dst, charPtr, drawTop, origWidth, origHeight);
+		dst = (byte *)_vm->_textSurface.pixels + _top * _vm->_textSurface.pitch + _left;
+		drawBits1(_vm->_textSurface, dst, charPtr, drawTop, origWidth, origHeight);
 	}
 
 	if (_str.left > _left)
