@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2001-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,16 +18,17 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/scumm/charset.h $
- * $Id: charset.h 22311 2006-05-03 16:23:32Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/scumm/charset.h $
+ * $Id: charset.h 27024 2007-05-30 21:56:52Z fingolfin $
  */
 
-#ifndef CHARSET_H
-#define CHARSET_H
+#ifndef SCUMM_CHARSET_H
+#define SCUMM_CHARSET_H
 
 #include "common/scummsys.h"
 #include "common/rect.h"
 #include "scumm/gfx.h"
+#include "scumm/saveload.h"
 
 namespace Scumm {
 
@@ -43,7 +47,6 @@ class CharsetRenderer {
 public:
 
 	Common::Rect _str;
-	int _nextLeft, _nextTop;
 
 	int _top;
 	int _left, _startLeft;
@@ -62,12 +65,6 @@ public:
 	bool _firstChar;
 	bool _disableOffsX;
 
-	/**
-	 * All text is normally rendered into this overlay surface. Then later
-	 * drawStripToScreen() composits it over the game graphics.
-	 */
-	Graphics::Surface _textSurface;
-
 protected:
 	ScummEngine *_vm;
 	byte _curId;
@@ -76,10 +73,6 @@ public:
 	CharsetRenderer(ScummEngine *vm);
 	virtual ~CharsetRenderer();
 
-	void restoreCharsetBg();
-	void clearCharsetMask();
-	void clearTextSurface();
-
 	virtual void printChar(int chr, bool ignoreCharsetMask) = 0;
 	virtual void drawChar(int chr, const Graphics::Surface &s, int x, int y) {}
 
@@ -95,11 +88,13 @@ public:
 	virtual int getCharWidth(byte chr) = 0;
 
 	virtual void setColor(byte color) { _color = color; translateColor(); }
+
+	void saveLoadWithSerializer(Serializer *ser);
 };
 
 class CharsetRendererCommon : public CharsetRenderer {
 protected:
-	byte *_fontPtr;
+	const byte *_fontPtr;
 	int _bitDepth;
 	int _fontHeight;
 	int _numChars;
@@ -127,6 +122,8 @@ class CharsetRendererClassic : public CharsetRendererCommon {
 protected:
 	void drawBitsN(const Graphics::Surface &s, byte *dst, const byte *src, byte bpp, int drawTop, int width, int height);
 
+	void printCharIntern(bool is2byte, const byte *charPtr, int origWidth, int origHeight, int width, int height, VirtScreen *vs, bool ignoreCharsetMask);
+
 public:
 	CharsetRendererClassic(ScummEngine *vm) : CharsetRendererCommon(vm) {}
 
@@ -155,7 +152,7 @@ public:
 
 class CharsetRendererV3 : public CharsetRendererCommon {
 protected:
-	byte *_widthTable;
+	const byte *_widthTable;
 
 public:
 	CharsetRendererV3(ScummEngine *vm) : CharsetRendererCommon(vm) {}
