@@ -1,6 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2001  Ludvig Strigeus
- * Copyright (C) 2001-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -16,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/scumm/costume.cpp $
- * $Id: costume.cpp 24105 2006-10-04 05:16:44Z eriktorbjorn $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/scumm/costume.cpp $
+ * $Id: costume.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -88,12 +90,14 @@ byte ClassicCostumeRenderer::mainRoutine(int xmoveCur, int ymoveCur) {
 	int step;
 	Codec1 v1;
 
-	
+	if (_vm->_game.id == GID_LOOM && _vm->_game.platform == Common::kPlatformPCEngine) {
+		// FIXME: Unknown costume format
+		return 0;
+	}
+
 	const int scaletableSize = 128;
 	const bool newAmiCost = (_vm->_game.version == 5) && (_vm->_game.platform == Common::kPlatformAmiga);
 
-	CHECK_HEAP
-
 	v1.scaletable = smallCostumeScaleTable;
 
 	if (_loaded._numColors == 32) {
@@ -298,7 +302,6 @@ byte ClassicCostumeRenderer::mainRoutine(int xmoveCur, int ymoveCur) {
 		_draw_bottom = rect.bottom;
 
 	if (_height + rect.top >= 256) {
-		CHECK_HEAP
 		return 2;
 	}
 
@@ -306,8 +309,6 @@ byte ClassicCostumeRenderer::mainRoutine(int xmoveCur, int ymoveCur) {
 
 	v1.mask_ptr = _vm->getMaskBuffer(0, v1.y, _zbuf);
 
-	CHECK_HEAP
-
 	if (_loaded._format == 0x57) {
 		// The v1 costume renderer needs the actor number, which is
 		// the same thing as the costume renderer's _actorID.
@@ -317,7 +318,6 @@ byte ClassicCostumeRenderer::mainRoutine(int xmoveCur, int ymoveCur) {
 	else
 		proc3(v1);
 
-	CHECK_HEAP
 	return drawFlag;
 }
 
@@ -557,6 +557,11 @@ void ClassicCostumeLoader::loadCostume(int id) {
 	_id = id;
 	byte *ptr = _vm->getResourceAddress(rtCostume, id);
 
+	if (_vm->_game.id == GID_LOOM && _vm->_game.platform == Common::kPlatformPCEngine) {
+		// FIXME: Unknown costume format
+		return;
+	}
+
 	if (_vm->_game.version >= 6)
 		ptr += 8;
 	else if (_vm->_game.features & GF_OLD_BUNDLE)
@@ -661,6 +666,11 @@ byte NESCostumeRenderer::drawLimb(const Actor *a, int limb) {
 			x = -x;
 		}
 
+		left = MIN(left, _actorX + x);
+		right = MAX(right, _actorX + x + 8);
+		top = MIN(top, _actorY + y);
+		bottom = MAX(bottom, _actorY + y + 8);
+
 		if ((_actorX + x < 0) || (_actorX + x + 8 >= _out.w))
 			continue;
 		if ((_actorY + y < 0) || (_actorY + y + 8 >= _out.h))
@@ -687,10 +697,6 @@ byte NESCostumeRenderer::drawLimb(const Actor *a, int limb) {
 					*((byte *)_out.pixels + my * _out.pitch + mx) = palette[c];
 			}
 		}
-		left = MIN(left, _actorX + x);
-		right = MAX(right, _actorX + x + 8);
-		top = MIN(top, _actorY + y);
-		bottom = MAX(bottom, _actorY + y + 8);
 	}
 
 	_draw_top = top;
