@@ -1,6 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2001  Ludvig Strigeus
- * Copyright (C) 2001-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +17,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/scumm/cursor.cpp $
- * $Id: cursor.cpp 22732 2006-05-29 12:45:11Z kirben $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/scumm/cursor.cpp $
+ * $Id: cursor.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -24,7 +26,6 @@
 #include "common/system.h"
 #include "common/util.h"
 #include "graphics/cursorman.h"
-#include "graphics/paletteman.h"
 #include "scumm/bomp.h"
 #include "scumm/charset.h"
 #include "scumm/intern.h"
@@ -155,7 +156,7 @@ void ScummEngine_v70he::setCursorFromImg(uint img, uint room, uint imgindex) {
 void ScummEngine_v70he::setDefaultCursor() {
 	const uint16 *src;
 	int i, j;
-	static byte palette[] = { 0xff, 0xff, 0xff, 0,
+	static const byte palette[] = { 0xff, 0xff, 0xff, 0,
 							  0,    0,    0,    0};
 
 	memset(_grabbedCursor, 5, sizeof(_grabbedCursor));
@@ -178,7 +179,7 @@ void ScummEngine_v70he::setDefaultCursor() {
 
 	// Since white color position is not guaranteed
 	// we setup our own palette if supported by backend
-	PaletteMan.replaceCursorPalette(palette, 0xfe, 2);
+	CursorMan.replaceCursorPalette(palette, 0xfe, 2);
 
 	updateCursor();
 }
@@ -257,10 +258,10 @@ void ScummEngine_v6::useIm01Cursor(const byte *im, int w, int h) {
 	drawBox(0, 0, w - 1, h - 1, 0xFF);
 
 	vs->hasTwoBuffers = false;
-	gdi.disableZBuffer();
-	gdi.drawBitmap(im, vs, _screenStartStrip, 0, w, h, 0, w / 8, 0);
+	_gdi->disableZBuffer();
+	_gdi->drawBitmap(im, vs, _screenStartStrip, 0, w, h, 0, w / 8, 0);
 	vs->hasTwoBuffers = true;
-	gdi.enableZBuffer();
+	_gdi->enableZBuffer();
 
 	// Grab the data we just drew and setup the cursor with it
 	setCursorFromBuffer(vs->getPixels(0, 0), w, h, vs->pitch);
@@ -361,7 +362,7 @@ void ScummEngine_v5::redefineBuiltinCursorHotspot(int index, int x, int y) {
 	_cursorHotspots[index * 2 + 1] = y;
 }
 
-void ScummEngine_v5::setBuiltinCursor(int idx) {
+void ScummEngine_v2::setBuiltinCursor(int idx) {
 	int i, j;
 	byte color;
 
@@ -389,7 +390,7 @@ void ScummEngine_v5::setBuiltinCursor(int idx) {
 				*dst++ = palette[((c0 >> (7 - j)) & 1) | (((c1 >> (7 - j)) & 1) << 1) | ((idx == 3) ? 4 : 0)];
 		}
 
-	} else if (_game.version <= 2 && _game.platform == Common::kPlatformAmiga) {
+	} else if (_game.platform == Common::kPlatformAmiga) {
 		_cursor.width = 15;
 		_cursor.height = 15;
 		_cursor.hotspotX = 7;
@@ -418,7 +419,7 @@ void ScummEngine_v5::setBuiltinCursor(int idx) {
 			*(hotspot - _cursor.width * (3 + i) + i) = color;
 			*(hotspot + _cursor.width * (3 + i) + i) = color;
 		}
-	} else if (_game.version <= 2) {
+	} else {
 		_cursor.width = 23;
 		_cursor.height = 21;
 		_cursor.hotspotX = 11;
@@ -463,21 +464,27 @@ void ScummEngine_v5::setBuiltinCursor(int idx) {
 		*(hotspot - (_cursor.width * 5) + 1) = color;
 		*(hotspot + (_cursor.width * 5) - 1) = color;
 		*(hotspot + (_cursor.width * 5) + 1) = color;
-	} else {
-		const uint16 *src;
+	}
 
-		_cursor.hotspotX = _cursorHotspots[2 * _currentCursor];
-		_cursor.hotspotY = _cursorHotspots[2 * _currentCursor + 1];
-		src = _cursorImages[_currentCursor];
+	updateCursor();
+}
 
-		_cursor.width = 16;
-		_cursor.height = 16;
+void ScummEngine_v5::setBuiltinCursor(int idx) {
+	int i, j;
+	byte color = default_cursor_colors[idx];
+	const uint16 *src = _cursorImages[_currentCursor];
 
-		for (i = 0; i < 16; i++) {
-			for (j = 0; j < 16; j++) {
-				if (src[i] & (1 << j))
-					_grabbedCursor[16 * i + 15 - j] = color;
-			}
+	memset(_grabbedCursor, 0xFF, sizeof(_grabbedCursor));
+
+	_cursor.hotspotX = _cursorHotspots[2 * _currentCursor];
+	_cursor.hotspotY = _cursorHotspots[2 * _currentCursor + 1];
+	_cursor.width = 16;
+	_cursor.height = 16;
+
+	for (i = 0; i < 16; i++) {
+		for (j = 0; j < 16; j++) {
+			if (src[i] & (1 << j))
+				_grabbedCursor[16 * i + 15 - j] = color;
 		}
 	}
 
