@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2001-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/scumm/debugger.cpp $
- * $Id: debugger.cpp 22110 2006-04-23 17:33:16Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/scumm/debugger.cpp $
+ * $Id: debugger.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 #include "common/stdafx.h"
@@ -36,31 +39,15 @@
 #include "scumm/scumm.h"
 #include "scumm/sound.h"
 
-#include "common/debugger.cpp"
-
 namespace Scumm {
 
-// Debug channel lookup table for Debugger console
-static const dbgChannelDesc debugChannels[] = {
-	{"SCRIPTS", "Track script execution", DEBUG_SCRIPTS},
-	{"OPCODES", "Track opcode execution", DEBUG_OPCODES},
-	{"IMUSE", "Track iMUSE events", DEBUG_IMUSE},
-	{"RESOURCE", "Track resource loading/management", DEBUG_RESOURCE},
-	{"VARS", "Track variable changes", DEBUG_VARS},
-	{"ACTORS", "Actor-related debug", DEBUG_ACTORS},
-	{"SOUND", "Sound related debug", DEBUG_SOUND},
-	{"INSANE", "Track INSANE", DEBUG_INSANE},
-	{"SMUSH", "Track SMUSH", DEBUG_SMUSH}
-};
-
-
 void CDECL debugC(int channel, const char *s, ...) {
 	char buf[STRINGBUFLEN];
 	va_list va;
 
 	// FIXME: Still spew all debug at -d9, for crashes in startup etc.
 	//	  Add setting from commandline ( / abstract channel interface)
-	if (!(g_scumm->_debugFlags & channel) && (gDebugLevel < 9))
+	if (!(Common::getEnabledSpecialDebugLevels() & channel) && (gDebugLevel < 9))
 		return;
 
 	va_start(va, s);
@@ -71,7 +58,7 @@ void CDECL debugC(int channel, const char *s, ...) {
 }
 
 ScummDebugger::ScummDebugger(ScummEngine *s)
-	: Common::Debugger<ScummDebugger>() {
+	: GUI::Debugger() {
 	_vm = s;
 
 	// Register variables
@@ -82,46 +69,41 @@ ScummDebugger::ScummDebugger(ScummEngine *s)
 	DVar_Register("scumm_roomresource", &_vm->_roomResource, DVAR_INT, 0);
 	DVar_Register("scumm_vars", &_vm->_scummVars, DVAR_INTARRAY, _vm->_numVariables);
 
-//	DVar_Register("scumm_gamename", &_vm->_targetName, DVAR_STRING, 0);
-//	DVar_Register("scumm_exename", &_vm->_baseName, DVAR_STRING, 0);
-	DVar_Register("scumm_gameid", &_vm->_game.id, DVAR_BYTE, 0);
-
 	// Register commands
-	DCmd_Register("continue", &ScummDebugger::Cmd_Exit);
-	DCmd_Register("exit", &ScummDebugger::Cmd_Exit);
-	DCmd_Register("quit", &ScummDebugger::Cmd_Exit);
-	DCmd_Register("restart", &ScummDebugger::Cmd_Restart);
+	DCmd_Register("continue",  WRAP_METHOD(ScummDebugger, Cmd_Exit));
+	DCmd_Register("restart",   WRAP_METHOD(ScummDebugger, Cmd_Restart));
 
-	DCmd_Register("actor", &ScummDebugger::Cmd_Actor);
-	DCmd_Register("actors", &ScummDebugger::Cmd_PrintActor);
-	DCmd_Register("box", &ScummDebugger::Cmd_PrintBox);
-	DCmd_Register("matrix", &ScummDebugger::Cmd_PrintBoxMatrix);
-	DCmd_Register("camera", &ScummDebugger::Cmd_Camera);
-	DCmd_Register("room", &ScummDebugger::Cmd_Room);
-	DCmd_Register("objects", &ScummDebugger::Cmd_PrintObjects);
-	DCmd_Register("object", &ScummDebugger::Cmd_Object);
-	DCmd_Register("script", &ScummDebugger::Cmd_Script);
-	DCmd_Register("scr", &ScummDebugger::Cmd_Script);
-	DCmd_Register("scripts", &ScummDebugger::Cmd_PrintScript);
-	DCmd_Register("importres", &ScummDebugger::Cmd_ImportRes);
+	DCmd_Register("actor",     WRAP_METHOD(ScummDebugger, Cmd_Actor));
+	DCmd_Register("actors",    WRAP_METHOD(ScummDebugger, Cmd_PrintActor));
+	DCmd_Register("box",       WRAP_METHOD(ScummDebugger, Cmd_PrintBox));
+	DCmd_Register("matrix",    WRAP_METHOD(ScummDebugger, Cmd_PrintBoxMatrix));
+	DCmd_Register("camera",    WRAP_METHOD(ScummDebugger, Cmd_Camera));
+	DCmd_Register("room",      WRAP_METHOD(ScummDebugger, Cmd_Room));
+	DCmd_Register("objects",   WRAP_METHOD(ScummDebugger, Cmd_PrintObjects));
+	DCmd_Register("object",    WRAP_METHOD(ScummDebugger, Cmd_Object));
+	DCmd_Register("script",    WRAP_METHOD(ScummDebugger, Cmd_Script));
+	DCmd_Register("scr",       WRAP_METHOD(ScummDebugger, Cmd_Script));
+	DCmd_Register("scripts",   WRAP_METHOD(ScummDebugger, Cmd_PrintScript));
+	DCmd_Register("importres", WRAP_METHOD(ScummDebugger, Cmd_ImportRes));
 
 	if (_vm->_game.id == GID_LOOM)
-		DCmd_Register("drafts", &ScummDebugger::Cmd_PrintDraft);
+		DCmd_Register("drafts",  WRAP_METHOD(ScummDebugger, Cmd_PrintDraft));
 
-	DCmd_Register("loadgame", &ScummDebugger::Cmd_LoadGame);
-	DCmd_Register("savegame", &ScummDebugger::Cmd_SaveGame);
+	DCmd_Register("loadgame",  WRAP_METHOD(ScummDebugger, Cmd_LoadGame));
+	DCmd_Register("savegame",  WRAP_METHOD(ScummDebugger, Cmd_SaveGame));
 
-	DCmd_Register("level", &ScummDebugger::Cmd_DebugLevel);
-	DCmd_Register("debug", &ScummDebugger::Cmd_Debug);
-	DCmd_Register("help", &ScummDebugger::Cmd_Help);
+	DCmd_Register("level",     WRAP_METHOD(ScummDebugger, Cmd_DebugLevel));
+	DCmd_Register("debug",     WRAP_METHOD(ScummDebugger, Cmd_Debug));
 
-	DCmd_Register("show", &ScummDebugger::Cmd_Show);
-	DCmd_Register("hide", &ScummDebugger::Cmd_Hide);
+	DCmd_Register("show",      WRAP_METHOD(ScummDebugger, Cmd_Show));
+	DCmd_Register("hide",      WRAP_METHOD(ScummDebugger, Cmd_Hide));
 
-	DCmd_Register("imuse", &ScummDebugger::Cmd_IMuse);
+	DCmd_Register("imuse",     WRAP_METHOD(ScummDebugger, Cmd_IMuse));
 }
 
-ScummDebugger::~ScummDebugger() {} // we need this here for __SYMBIAN32__
+ScummDebugger::~ScummDebugger() {
+	 // we need this destructor, even if it is empty, for __SYMBIAN32__
+}
 
 void ScummDebugger::preEnter() {
 	// Pause sound output
@@ -138,11 +120,6 @@ void ScummDebugger::postEnter() {
 // Now the fun stuff:
 
 // Commands
-bool ScummDebugger::Cmd_Exit(int argc, const char **argv) {
-	_detach_now = true;
-	return false;
-}
-
 bool ScummDebugger::Cmd_Restart(int argc, const char **argv) {
 	_vm->restart();
 
@@ -203,7 +180,7 @@ bool ScummDebugger::Cmd_IMuse(int argc, const char **argv) {
 bool ScummDebugger::Cmd_Room(int argc, const char **argv) {
 	if (argc > 1) {
 		int room = atoi(argv[1]);
-		_vm->_actors[_vm->VAR(_vm->VAR_EGO)]._room = room;
+		_vm->_actors[_vm->VAR(_vm->VAR_EGO)]->_room = room;
 		_vm->_sound->stopAllSounds();
 		_vm->startScene(room, 0, 0);
 		_vm->_fullRedraw = true;
@@ -339,7 +316,7 @@ bool ScummDebugger::Cmd_ImportRes(int argc, const char** argv) {
 			file.seek(-8, SEEK_CUR);
 		}
 
-		file.read(_vm->res.createResource(rtScript, resnum, size), size);
+		file.read(_vm->_res->createResource(rtScript, resnum, size), size);
 
 	} else
 		DebugPrintf("Unknown importres type '%s'\n", argv[1]);
@@ -381,7 +358,7 @@ bool ScummDebugger::Cmd_Actor(int argc, const char **argv) {
 		return true;
 	}
 
-	a = &_vm->_actors[actnum];
+	a = _vm->_actors[actnum];
 	if (argc > 3)
 		value = atoi(argv[3]);
 	if (argc > 4)
@@ -397,20 +374,20 @@ bool ScummDebugger::Cmd_Actor(int argc, const char **argv) {
 		a->_ignoreBoxes = (value > 0);
 		DebugPrintf("Actor[%d].ignoreBoxes = %d\n", actnum, a->_ignoreBoxes);
 	} else if (!strcmp(argv[2], "x")) {
-		a->putActor(value, a->_pos.y, a->_room);
-		DebugPrintf("Actor[%d].x = %d\n", actnum, a->_pos.x);
+		a->putActor(value, a->getRealPos().y);
+		DebugPrintf("Actor[%d].x = %d\n", actnum, a->getRealPos().x);
 		_vm->_fullRedraw = true;
 	} else if (!strcmp(argv[2], "y")) {
-		a->putActor(a->_pos.x, value, a->_room);
-		DebugPrintf("Actor[%d].y = %d\n", actnum, a->_pos.y);
+		a->putActor(a->getRealPos().x, value);
+		DebugPrintf("Actor[%d].y = %d\n", actnum, a->getRealPos().y);
 		_vm->_fullRedraw = true;
 	} else if (!strcmp(argv[2], "_elevation")) {
 		a->setElevation(value);
 		DebugPrintf("Actor[%d]._elevation = %d\n", actnum, a->getElevation());
 		_vm->_fullRedraw = true;
 	} else if (!strcmp(argv[2], "costume")) {
-		if (value >= _vm->res.num[rtCostume])
-			DebugPrintf("Costume not changed as %d exceeds max of %d\n", value, _vm->res.num[rtCostume]);
+		if (value >= _vm->_res->num[rtCostume])
+			DebugPrintf("Costume not changed as %d exceeds max of %d\n", value, _vm->_res->num[rtCostume]);
 		else {
 			a->setActorCostume(value);
 			_vm->_fullRedraw = true;
@@ -438,10 +415,10 @@ bool ScummDebugger::Cmd_PrintActor(int argc, const char **argv) {
 	DebugPrintf("|# |  x |  y | w |elev|cos|box|mov| zp|frm|scl|dir|   cls   |\n");
 	DebugPrintf("+--+----+----+---+----+---+---+---+---+---+---+---+---------+\n");
 	for (i = 1; i < _vm->_numActors; i++) {
-		a = &_vm->_actors[i];
+		a = _vm->_actors[i];
 		if (a->_visible)
 			DebugPrintf("|%2d|%4d|%4d|%3d|%4d|%3d|%3d|%3d|%3d|%3d|%3d|%3d|$%08x|\n",
-						 a->_number, a->_pos.x, a->_pos.y, a->_width, a->getElevation(),
+						 a->_number, a->getRealPos().x, a->getRealPos().y, a->_width, a->getElevation(),
 						 a->_costume, a->_walkbox, a->_moving, a->_forceClip, a->_frame,
 						 a->_scalex, a->getFacing(), _vm->_classData[a->_number]);
 	}
@@ -505,77 +482,39 @@ bool ScummDebugger::Cmd_Object(int argc, const char **argv) {
 		_vm->markObjectRectAsDirty(obj);
 		_vm->clearDrawObjectQueue();
 		_vm->runInventoryScript(obj);
-	} else if (!strcmp(argv[2], "state")) {
+	} else if (!strcmp(argv[2], "state") && argc == 4) {
 		_vm->putState(obj, atoi(argv[3]));
 		//is BgNeedsRedraw enough?
 		_vm->_bgNeedsRedraw = true;
 	} else if (!strcmp(argv[2], "name")) {
 		DebugPrintf("Name of object %d: %s\n", obj, _vm->getObjOrActorName(obj));
 	} else {
-		DebugPrintf("Unknown object command '%s'\nUse <pickup | state> as command\n", argv[2]);
+		DebugPrintf("Unknown object command '%s'\nUse <pickup | state | name> as command\n", argv[2]);
 	}
 
 	return true;
 }
 
-bool ScummDebugger::Cmd_Help(int argc, const char **argv) {
-	// console normally has 39 line width
-	// wrap around nicely
-	int width = 0, size, i;
-
-	DebugPrintf("Commands are:\n");
-	for (i = 0 ; i < _dcmd_count ; i++) {
-		size = strlen(_dcmds[i].name) + 1;
-
-		if ((width + size) >= 39) {
-			DebugPrintf("\n");
-			width = size;
-		} else
-			width += size;
-
-		DebugPrintf("%s ", _dcmds[i].name);
-	}
-
-	width = 0;
-
-	DebugPrintf("\n\nVariables are:\n");
-	for (i = 0 ; i < _dvar_count ; i++) {
-		size = strlen(_dvars[i].name) + 1;
-
-		if ((width + size) >= 39) {
-			DebugPrintf("\n");
-			width = size;
-		} else
-			width += size;
-
-		DebugPrintf("%s ", _dvars[i].name);
-	}
-
-	DebugPrintf("\n");
-
-	return true;
-}
-
 bool ScummDebugger::Cmd_Debug(int argc, const char **argv) {
-	int numChannels = sizeof(debugChannels) / sizeof(dbgChannelDesc);
+	Common::Array<Common::EngineDebugLevel> lvls = Common::listSpecialDebugLevels();
 
 	bool setFlag = false;	// Remove or add debug channel?
 
-	if ((argc == 1) && (_vm->_debugFlags == 0)) {
+	if ((argc == 1) && (Common::getEnabledSpecialDebugLevels() == 0)) {
 		DebugPrintf("No debug flags are enabled\n");
 		DebugPrintf("Available Channels: ");
-		for (int i = 0; i < numChannels; i++) {
-			DebugPrintf("%s, ", debugChannels[i].channel);
+		for (uint i = 0; i < lvls.size(); i++) {
+			DebugPrintf("%s, ", lvls[i].option.c_str());
 		}
 		DebugPrintf("\n");
 		return true;
 	}
 
-	if ((argc == 1) && (_vm->_debugFlags > 0)) {
-		for (int i = 0; i < numChannels; i++) {
-			if (_vm->_debugFlags & debugChannels[i].flag)
-				DebugPrintf("%s - %s\n", debugChannels[i].channel,
-							 debugChannels[i].desc);
+	if ((argc == 1) && (Common::getEnabledSpecialDebugLevels() > 0)) {
+		for (uint i = 0; i < lvls.size(); i++) {
+			if (lvls[i].enabled)
+				DebugPrintf("%s - %s\n", lvls[i].option.c_str(),
+							 lvls[i].description.c_str());
 		}
 		return true;
 	}
@@ -588,25 +527,25 @@ bool ScummDebugger::Cmd_Debug(int argc, const char **argv) {
 	} else {
 		DebugPrintf("Syntax: Debug +CHANNEL, or Debug -CHANNEL\n");
 		DebugPrintf("Available Channels: ");
-		for (int i = 0; i < numChannels; i++) {
-			DebugPrintf("%s, ", debugChannels[i].channel);
+		for (uint i = 0; i < lvls.size(); i++) {
+			DebugPrintf("%s, ", lvls[i].option.c_str());
 			DebugPrintf("\n");
 		}
 	}
 
 	// Identify flag
 	const char *realFlag = argv[1] + 1;
-	for (int i = 0; i < numChannels; i++) {
-		if ((scumm_stricmp(debugChannels[i].channel, realFlag)) == 0) {
+	for (uint i = 0; i < lvls.size(); i++) {
+		if ((scumm_stricmp(lvls[i].option.c_str(), realFlag)) == 0) {
 			if (setFlag) {
-				_vm->_debugFlags |= debugChannels[i].flag;
+				enableSpecialDebugLevel(lvls[i].option);
 				DebugPrintf("Enable ");
 			} else {
-				_vm->_debugFlags &= ~debugChannels[i].flag;
+				disableSpecialDebugLevel(lvls[i].option);
 				DebugPrintf("Disable ");
 			}
 
-			DebugPrintf("%s\n", debugChannels[i].desc);
+			DebugPrintf("%s\n", lvls[i].description.c_str());
 			return true;
 		}
 	}
@@ -694,7 +633,7 @@ void ScummDebugger::printBox(int box) {
 	int mask = _vm->getMaskFromBox(box);
 	int scale = _vm->getBoxScale(box);
 
-	_vm->getBoxCoordinates(box, &coords);
+	coords = _vm->getBoxCoordinates(box);
 
 	// Print out coords, flags, zbuffer mask
 	DebugPrintf("%d: [%d x %d] [%d x %d] [%d x %d] [%d x %d], flags=0x%02x, mask=%d, scale=%d\n",
@@ -800,12 +739,19 @@ void ScummDebugger::drawBox(int box) {
 	BoxCoords coords;
 	Common::Point r[4];
 
-	_vm->getBoxCoordinates(box, &coords);
+	coords = _vm->getBoxCoordinates(box);
 
 	r[0] = coords.ul;
 	r[1] = coords.ur;
 	r[2] = coords.lr;
 	r[3] = coords.ll;
+	
+	if (_vm->_game.version <= 2) {
+		for (int i = 0; i < 4; ++i) {
+			r[i].x *= V12_X_MULTIPLIER;
+			r[i].y *= V12_Y_MULTIPLIER;
+		}
+	}
 
 	// TODO - maybe use different colors for each box, and/or print the box number inside it?
 	fillQuad(_vm, r, 13);
