@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2002-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/scumm/dialogs.cpp $
- * $Id: dialogs.cpp 23988 2006-09-25 21:08:03Z agent-q $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/scumm/dialogs.cpp $
+ * $Id: dialogs.cpp 27068 2007-06-03 17:32:42Z fingolfin $
  */
 
 #include "common/stdafx.h"
@@ -24,9 +27,14 @@
 #include "common/config-manager.h"
 #include "common/savefile.h"
 #include "common/system.h"
+#include "common/events.h"
 
 #include "graphics/scaler.h"
 
+#ifdef __DS__
+#include "scummhelp.h"
+#endif
+
 #include "gui/about.h"
 #include "gui/chooser.h"
 #include "gui/eval.h"
@@ -43,10 +51,6 @@
 #include "sound/mididrv.h"
 #include "sound/mixer.h"
 
-#ifdef __DS__
-#include "scummhelp.h"
-#endif
-
 #ifndef DISABLE_HELP
 #include "scumm/help.h"
 #endif
@@ -78,11 +82,11 @@ struct ResString {
 };
 
 #ifdef PALMOS_68K
-static ResString *string_map_table_v7;
-static ResString *string_map_table_v6;
-static ResString *string_map_table_v5;
+static const ResString *string_map_table_v7;
+static const ResString *string_map_table_v6;
+static const ResString *string_map_table_v5;
 #else
-static ResString string_map_table_v8[] = {
+static const ResString string_map_table_v8[] = {
 	{0, "/BT_100/Please insert disk %d. Press ENTER"},
 	{0, "/BT__003/Unable to Find %s, (%s %d) Press Button."},
 	{0, "/BT__004/Error reading disk %c, (%c%d) Press Button."},
@@ -105,7 +109,7 @@ static ResString string_map_table_v8[] = {
 	{0, "/BT__020/Select a game to LOAD"}
 };
 
-static ResString string_map_table_v7[] = {
+static const ResString string_map_table_v7[] = {
 	{96, "game name and version"}, //that's how it's supposed to be
 	{83, "Unable to Find %s, (%c%d) Press Button."},
 	{84, "Error reading disk %c, (%c%d) Press Button."},
@@ -163,7 +167,7 @@ static ResString string_map_table_v7[] = {
 
 };
 
-static ResString string_map_table_v6[] = {
+static const ResString string_map_table_v6[] = {
 	{90, "Insert Disk %c and Press Button to Continue."},
 	{91, "Unable to Find %s, (%c%d) Press Button."},
 	{92, "Error reading disk %c, (%c%d) Press Button."},
@@ -187,7 +191,7 @@ static ResString string_map_table_v6[] = {
 	{117, "How may I serve you?"}
 };
 
-static ResString string_map_table_v345[] = {
+static const ResString string_map_table_v345[] = {
 	{1, "Insert Disk %c and Press Button to Continue."},
 	{2, "Unable to Find %s, (%c%d) Press Button."},
 	{3, "Error reading disk %c, (%c%d) Press Button."},
@@ -261,8 +265,6 @@ SaveLoadChooser::SaveLoadChooser(const String &title, const String &buttonLabel,
 	new GUI::ButtonWidget(this, "scummsaveload_cancel", "Cancel", kCloseCmd, 0);
 	_chooseButton = new GUI::ButtonWidget(this, "scummsaveload_choose", buttonLabel, kChooseCmd, 0);
 	_chooseButton->setEnabled(false);
-
-	handleScreenChanged();
 }
 
 SaveLoadChooser::~SaveLoadChooser() {
@@ -322,13 +324,13 @@ void SaveLoadChooser::handleCommand(CommandSender *sender, uint32 cmd, uint32 da
 	}
 }
 
-void SaveLoadChooser::handleScreenChanged() {
+void SaveLoadChooser::reflowLayout() {
 	if (g_gui.evaluator()->getVar("scummsaveload_extinfo.visible") == 1) {
 		int thumbX = g_gui.evaluator()->getVar("scummsaveload_thumbnail.x");
 		int thumbY = g_gui.evaluator()->getVar("scummsaveload_thumbnail.y");
 		int hPad = g_gui.evaluator()->getVar("scummsaveload_thumbnail.hPad");
 		int vPad = g_gui.evaluator()->getVar("scummsaveload_thumbnail.vPad");
-		int thumbH = ((_vm->_system->getHeight() % 200 && _vm->_system->getHeight() != 350) ? kThumbnailHeight2 : kThumbnailHeight1);
+		int thumbH = ((g_system->getHeight() % 200 && g_system->getHeight() != 350) ? kThumbnailHeight2 : kThumbnailHeight1);
 
 		_container->resize(thumbX - hPad, thumbY - vPad, kThumbnailWidth + hPad * 2, thumbH + vPad * 2 + kLineHeight * 4);
 	
@@ -365,7 +367,7 @@ void SaveLoadChooser::handleScreenChanged() {
 		_playtime->setFlags(GUI::WIDGET_INVISIBLE);
 	}
 
-	Dialog::handleScreenChanged();
+	Dialog::reflowLayout();
 }
 
 void SaveLoadChooser::updateInfos() {
@@ -512,20 +514,6 @@ void MainMenuDialog::handleCommand(CommandSender *sender, uint32 cmd, uint32 dat
 	}
 }
 
-void MainMenuDialog::handleScreenChanged() {
-	ScummDialog::handleScreenChanged();
-
-	_optionsDialog->handleScreenChanged();
-	_aboutDialog->handleScreenChanged();
-
-	_saveDialog->handleScreenChanged();
-	_loadDialog->handleScreenChanged();
-
-#ifndef DISABLE_HELP
-	_helpDialog->handleScreenChanged();
-#endif
-}
-
 void MainMenuDialog::save() {
 	int idx;
 	_saveDialog->setList(generateSavegameList(_vm, true));
@@ -555,11 +543,6 @@ void MainMenuDialog::load() {
 	}
 }
 
-void MainMenuDialog::open() {
-	handleScreenChanged();
-	Dialog::open();
-}
-
 #pragma mark -
 
 enum {
@@ -686,8 +669,8 @@ HelpDialog::HelpDialog(const GameSettings &game)
 
 }
 
-void HelpDialog::handleScreenChanged() {
-	ScummDialog::handleScreenChanged();
+void HelpDialog::reflowLayout() {
+	ScummDialog::reflowLayout();
 
 	_drawingHints &= ~GUI::THEME_HINT_SPECIAL_COLOR;
 
@@ -767,24 +750,29 @@ void HelpDialog::handleCommand(CommandSender *sender, uint32 cmd, uint32 data) {
 
 InfoDialog::InfoDialog(ScummEngine *scumm, int res)
 : ScummDialog("scummDummyDialog"), _vm(scumm) { // dummy x and w
-	setInfoText(queryResString(res));
+
+	_message = queryResString(res);
+
+	// Width and height are dummy
+	_text = new StaticTextWidget(this, 4, 4, 10, 10, _message, kTextAlignCenter);
 }
 
 InfoDialog::InfoDialog(ScummEngine *scumm, const String& message)
 : ScummDialog("scummDummyDialog"), _vm(scumm) { // dummy x and w
-	setInfoText(message);
-}
 
-void InfoDialog::setInfoText(const String& message) {
 	_message = message;
 
 	// Width and height are dummy
 	_text = new StaticTextWidget(this, 4, 4, 10, 10, _message, kTextAlignCenter);
-
-	handleScreenChanged();
 }
 
-void InfoDialog::handleScreenChanged() {
+void InfoDialog::setInfoText(const String& message) {
+	_message = message;
+	_text->setLabel(_message);
+	//reflowLayout(); // FIXME: Should we call this here? Depends on the usage patterns, I guess...
+}
+
+void InfoDialog::reflowLayout() {
 	const int screenW = g_system->getOverlayWidth();
 	const int screenH = g_system->getOverlayHeight();
 
@@ -801,13 +789,13 @@ void InfoDialog::handleScreenChanged() {
 
 const Common::String InfoDialog::queryResString(int stringno) {
 	byte buf[256];
-	byte *result;
+	const byte *result;
 
 	if (stringno == 0)
 		return String();
 
 	if (_vm->_game.version == 8)
-		result = (byte *)string_map_table_v8[stringno - 1].string;
+		result = (const byte *)string_map_table_v8[stringno - 1].string;
 	else if (_vm->_game.version == 7)
 		result = _vm->getStringAddressVar(string_map_table_v7[stringno - 1].num);
 	else if (_vm->_game.version == 6)
@@ -869,10 +857,31 @@ void ConfirmDialog::handleKeyDown(uint16 ascii, int keycode, int modifiers) {
 
 #pragma mark -
 
-ValueDisplayDialog::ValueDisplayDialog(const Common::String& label, int minVal, int maxVal, int val, uint16 incKey, uint16 decKey)
-	: GUI::Dialog("scummDummyDialog"), _label(label), _min(minVal), _max(maxVal), _value(val), _incKey(incKey), _decKey(decKey) {
+ValueDisplayDialog::ValueDisplayDialog(const Common::String& label, int minVal, int maxVal,
+		int val, uint16 incKey, uint16 decKey)
+	: GUI::Dialog("scummDummyDialog", false),
+	_label(label), _min(minVal), _max(maxVal),
+	_value(val), _incKey(incKey), _decKey(decKey) {
 	assert(_min <= _value && _value <= _max);
+}
 
+void ValueDisplayDialog::drawDialog() {
+	const int labelWidth = _w - 8 - _percentBarWidth;
+	g_gui.theme()->drawDialogBackground(Common::Rect(_x, _y, _x+_w, _y+_h),
+				GUI::THEME_HINT_SAVE_BACKGROUND | GUI::THEME_HINT_FIRST_DRAW);
+	g_gui.theme()->drawText(Common::Rect(_x+4, _y+4, _x+labelWidth+4,
+				_y+g_gui.theme()->getFontHeight()+4), _label);
+	g_gui.theme()->drawSlider(Common::Rect(_x+4+labelWidth, _y+4, _x+_w-4, _y+_h-4),
+				_percentBarWidth * (_value - _min) / (_max - _min));
+}
+
+void ValueDisplayDialog::handleTickle() {
+	if (getMillis() > _timer) {
+		close();
+	}
+}
+
+void ValueDisplayDialog::reflowLayout() {
 	const int screenW = g_system->getOverlayWidth();
 	const int screenH = g_system->getOverlayHeight();
 
@@ -882,7 +891,7 @@ ValueDisplayDialog::ValueDisplayDialog(const Common::String& label, int minVal,
 		_percentBarWidth = kPercentBarWidth;
 	}
 
-	int width = g_gui.getStringWidth(label) + 16 + _percentBarWidth;
+	int width = g_gui.getStringWidth(_label) + 16 + _percentBarWidth;
 	int height = g_gui.getFontHeight() + 4 * 2;
 
 	_x = (screenW - width) / 2;
@@ -891,19 +900,6 @@ ValueDisplayDialog::ValueDisplayDialog(const Common::String& label, int minVal,
 	_h = height;
 }
 
-void ValueDisplayDialog::drawDialog() {
-	const int labelWidth = _w - 8 - _percentBarWidth;
-	g_gui.theme()->drawDialogBackground(Common::Rect(_x, _y, _x+_w, _y+_h), GUI::THEME_HINT_SAVE_BACKGROUND | GUI::THEME_HINT_FIRST_DRAW);
-	g_gui.theme()->drawText(Common::Rect(_x+4, _y+4, _x+labelWidth+4, _y+g_gui.theme()->getFontHeight()+4), _label);
-	g_gui.theme()->drawSlider(Common::Rect(_x+4+labelWidth, _y+4, _x+_w-4, _y+_h-4), _percentBarWidth * (_value - _min) / (_max - _min));
-}
-
-void ValueDisplayDialog::handleTickle() {
-	if (getMillis() > _timer) {
-		close();
-	}
-}
-
 void ValueDisplayDialog::handleKeyDown(uint16 ascii, int keycode, int modifiers) {
 	if (ascii == _incKey || ascii == _decKey) {
 		if (ascii == _incKey && _value < _max)
@@ -925,7 +921,58 @@ void ValueDisplayDialog::open() {
 	_timer = getMillis() + kDisplayDelay;
 }
 
+SubtitleSettingsDialog::SubtitleSettingsDialog(ScummEngine *scumm, int value) 
+	: InfoDialog(scumm, ""), _value(value) {
 
+}
+
+void SubtitleSettingsDialog::handleTickle() {
+	InfoDialog::handleTickle();
+	if (getMillis() > _timer)
+		close();
+}
+
+void SubtitleSettingsDialog::handleKeyDown(uint16 ascii, int keycode, int modifiers) {
+	if (keycode == 't' && modifiers == Common::KBD_CTRL) {
+		cycleValue();
+
+		reflowLayout();
+		draw();
+	} else {
+		close();
+	}
+}
+
+void SubtitleSettingsDialog::open() {
+	cycleValue();
+	InfoDialog::open();
+}
+
+void SubtitleSettingsDialog::cycleValue() {
+	static const char* subtitleDesc[] = {
+		"Speech Only",
+		"Speech and Subtitles",
+		"Subtitles Only"
+	};
+	
+	_value = (_value + 1) % 3;
+
+	setInfoText(subtitleDesc[_value]);
+
+	setResult(_value);
+	_timer = getMillis() + 1500;
+}
+
+Indy3IQPointsDialog::Indy3IQPointsDialog(ScummEngine *scumm, char* text)
+	: InfoDialog(scumm, text) {
+}
+
+void Indy3IQPointsDialog::handleKeyDown(uint16 ascii, int keycode, int modifiers) {
+	if (ascii == 'i') 
+		close();
+	else
+		ScummDialog::handleKeyDown(ascii, keycode, modifiers);
+}
 
 } // End of namespace Scumm
 
