@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2002-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/scumm/dialogs.h $
- * $Id: dialogs.h 22745 2006-05-29 14:40:18Z lordhoto $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/scumm/dialogs.h $
+ * $Id: dialogs.h 27068 2007-06-03 17:32:42Z fingolfin $
  */
 
 #ifndef SCUMM_DIALOGS_H
@@ -27,7 +30,7 @@
 #include "gui/options.h"
 #include "gui/widget.h"
 
-#include "scumm/plugin.h"
+#include "scumm/detection.h"
 #ifndef DISABLE_HELP
 #include "scumm/help.h"
 #endif
@@ -76,8 +79,7 @@ public:
 	void setList(const StringList& list);
 	int runModal();
 
-	void handleScreenChanged();
-	
+	virtual void reflowLayout();
 };
 
 class MainMenuDialog : public ScummDialog {
@@ -85,7 +87,6 @@ public:
 	MainMenuDialog(ScummEngine *scumm);
 	~MainMenuDialog();
 	virtual void handleCommand(GUI::CommandSender *sender, uint32 cmd, uint32 data);
-	virtual void handleScreenChanged();
 
 protected:
 	ScummEngine		*_vm;
@@ -100,7 +101,6 @@ protected:
 
 	void save();
 	void load();
-	void open();
 };
 
 #ifndef DISABLE_HELP
@@ -110,7 +110,7 @@ public:
 	HelpDialog(const GameSettings &game);
 	virtual void handleCommand(GUI::CommandSender *sender, uint32 cmd, uint32 data);
 
-	virtual void handleScreenChanged();
+	virtual void reflowLayout();
 
 protected:
 	typedef Common::String String;
@@ -165,6 +165,8 @@ public:
 	// from resources
 	InfoDialog(ScummEngine *scumm, int res);
 
+	void setInfoText(const String& message);
+
 	virtual void handleMouseDown(int x, int y, int button, int clickCount) {
 		setResult(0);
 		close();
@@ -174,10 +176,9 @@ public:
 		close();
 	}
 
-	virtual void handleScreenChanged();
+	virtual void reflowLayout();
 
 protected:
-	void setInfoText (const String& message);
 
 	// Query a string from the resources
 	const String queryResString(int stringno);
@@ -219,6 +220,8 @@ public:
 	}
 	virtual void handleKeyDown(uint16 ascii, int keycode, int modifiers);
 
+	virtual void reflowLayout();
+
 protected:
 	enum {
 		kPercentBarWidth = 50,
@@ -233,6 +236,34 @@ protected:
 	uint32 _timer;
 };
 
+/**
+ * A dialog used to display and cycle subtitle settings.
+ * Automatically closes after a brief time has passed.
+ */
+class SubtitleSettingsDialog : public InfoDialog {
+public:
+	SubtitleSettingsDialog(ScummEngine *scumm, int value);
+
+	virtual void open();
+	virtual void handleTickle();
+	virtual void handleMouseDown(int x, int y, int button, int clickCount) {
+		close();
+	}
+	virtual void handleKeyDown(uint16 ascii, int keycode, int modifiers);
+protected:
+	int _value;
+	uint32 _timer;
+	
+	void cycleValue();
+};
+
+//The Indy IQ dialog
+class Indy3IQPointsDialog : public InfoDialog {
+public:
+	Indy3IQPointsDialog(ScummEngine *scumm, char* text);
+	virtual void handleKeyDown(uint16 ascii, int keycode, int modifiers);
+};
+
 } // End of namespace Scumm
 
 #endif
