@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2002-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/scumm/file.h $
- * $Id: file.h 21939 2006-04-16 13:01:41Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/scumm/file.h $
+ * $Id: file.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -25,6 +28,8 @@
 
 #include "common/file.h"
 
+#include "scumm/detection.h"
+
 namespace Scumm {
 
 class BaseScummFile : public Common::File {
@@ -87,7 +92,7 @@ private:
 
 	bool generateIndex();
 	bool generateResource(int res);
-	uint16 extractResource(Common::WriteStream *out, Resource *res);
+	uint16 extractResource(Common::WriteStream *out, const Resource *res);
 
 	byte fileReadByte();
 	uint16 fileReadUint16LE();
@@ -109,14 +114,15 @@ public:
 };
 
 
-class ScummC64File : public BaseScummFile {
+class ScummDiskImage : public BaseScummFile {
 private:
 	Common::MemoryReadStream *_stream;
 	byte _roomDisks[59], _roomTracks[59], _roomSectors[59];
 
 	byte *_buf;
 
-	bool _maniac;
+	GameSettings _game;
+
 	Common::String _disk1, _disk2;
 	int _openedDisk;
 
@@ -139,7 +145,7 @@ private:
 	uint16 fileReadUint16LE();
 
 public:
-	ScummC64File(const char *disk1, const char *disk2, bool maniac);
+	ScummDiskImage(const char *disk1, const char *disk2, GameSettings game);
 	void setEnc(byte value);
 
 	bool open(const Common::String &filename, AccessMode mode = kFileReadMode);
