@@ -1,6 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2001  Ludvig Strigeus
- * Copyright (C) 2001-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -16,13 +18,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/scumm/gfx.h $
- * $Id: gfx.h 20511 2006-02-11 09:55:41Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/scumm/gfx.h $
+ * $Id: gfx.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
-#ifndef GFX_H
-#define GFX_H
+#ifndef SCUMM_GFX_H
+#define SCUMM_GFX_H
 
 #include "graphics/surface.h"
 
@@ -170,37 +172,14 @@ struct ColorCycle {
 	byte end;
 };
 
-/** Bomp graphics data, used as parameter to ScummEngine::drawBomp. */
-struct BompDrawData {
-	Graphics::Surface dst;
-
-	int x, y;
-	byte scale_x, scale_y;
-	const byte *dataptr;
-	int srcwidth, srcheight;
-	uint16 shadowMode;
-
-	byte *maskPtr;
-
-	BompDrawData() { memset(this, 0, sizeof(*this)); }
-};
-
 struct StripTable;
 
 #define CHARSET_MASK_TRANSPARENCY	253
 
 class Gdi {
+protected:
 	ScummEngine *_vm;
 
-public:
-	int _numZBuffer;
-	int _imgBufOffs[8];
-	int32 _numStrips;
-
-	Gdi(ScummEngine *vm);
-	~Gdi();
-
-protected:
 	byte _paletteMod;
 	byte *_roomPalette;
 	byte _transparentColor;
@@ -212,31 +191,16 @@ protected:
 	/** Flag which is true when an object is being rendered, false otherwise. */
 	bool _objectMode;
 
-	/** Render settings which are specific to the C64 graphic decoders. */
-	struct {
-		byte colors[4];
-		byte charMap[2048], objectMap[2048], picMap[4096], colorMap[4096];
-		byte maskMap[4096], maskChar[4096];
-	} _C64;
-
-	struct {
-		byte nametable[16][64], nametableObj[16][64];
-		byte attributes[64], attributesObj[64];
-		byte masktable[16][8], masktableObj[16][8];
-		int  objX;
-		bool hasmask;
-	} _NES;
-
-	/** For V2 games, we cache offsets into the room graphics, to speed up things. */
-	StripTable *_roomStrips;
+public:
+	int _numZBuffer;
+	int _imgBufOffs[8];
+	int32 _numStrips;
 
+protected:
 	/* Bitmap decompressors */
 	bool decompressBitmap(byte *dst, int dstPitch, const byte *src, int numLinesToProcess);
 
 	void drawStripEGA(byte *dst, int dstPitch, const byte *src, int height) const;
-	void drawStripC64Object(byte *dst, int dstPitch, int stripnr, int width, int height);
-	void drawStripC64Background(byte *dst, int dstPitch, int stripnr, int height);
-	void drawStripNES(byte *dst, byte *mask, int dstPitch, int stripnr, int top, int height);
 
 	void drawStripComplex(byte *dst, int dstPitch, const byte *src, int height, const bool transpCheck) const;
 	void drawStripBasicH(byte *dst, int dstPitch, const byte *src, int height, const bool transpCheck) const;
@@ -252,35 +216,40 @@ protected:
 	void drawStripHE(byte *dst, int dstPitch, const byte *src, int width, int height, const bool transpCheck) const;
 
 	/* Mask decompressors */
-	void drawStripC64Mask(byte *dst, int stripnr, int width, int height) const;
-	void drawStripNESMask(byte *dst, int stripnr, int top, int height) const;
 	void decompressTMSK(byte *dst, const byte *tmsk, const byte *src, int height) const;
 	void decompressMaskImgOr(byte *dst, const byte *src, int height) const;
 	void decompressMaskImg(byte *dst, const byte *src, int height) const;
 
 	/* Misc */
-	void decodeC64Gfx(const byte *src, byte *dst, int size) const;
-
 	int getZPlanes(const byte *smap_ptr, const byte *zplane_list[9], bool bmapImage) const;
 
-	StripTable *generateStripTable(const byte *src, int width, int height, StripTable *table) const;
-	void drawBitmapV2Helper(const byte *ptr, VirtScreen *vs, int x, int y, const int width, const int height,
+	virtual bool drawStrip(byte *dstPtr, VirtScreen *vs,
+					int x, int y, const int width, const int height,
+					int stripnr, const byte *smap_ptr);
+
+	virtual void decodeMask(int x, int y, const int width, const int height,
+	                int stripnr, int numzbuf, const byte *zplane_list[9],
+	                bool transpStrip, byte flag, const byte *tmsk_ptr);
+
+	virtual void prepareDrawBitmap(const byte *ptr, VirtScreen *vs,
+					const int x, const int y, const int width, const int height,
 	                int stripnr, int numstrip);
 
 public:
-	void init();
-	void roomChanged(byte *roomptr, uint32 IM00_offs, byte transparentColor);
+	Gdi(ScummEngine *vm);
+	virtual ~Gdi();
+
+	virtual void init();
+	virtual void roomChanged(byte *roomptr);
+	void setTransparentColor(byte transparentColor) { _transparentColor = transparentColor; }
 
 	void drawBitmap(const byte *ptr, VirtScreen *vs, int x, int y, const int width, const int height,
 	                int stripnr, int numstrip, byte flag);
 
-	void decodeNESGfx(const byte *room);
-	void decodeNESObject(const byte *ptr, int xpos, int ypos, int width, int height);
-
+#ifndef DISABLE_HE
 	void drawBMAPBg(const byte *ptr, VirtScreen *vs);
 	void drawBMAPObject(const byte *ptr, VirtScreen *vs, int obj, int x, int y, int w, int h);
-
-	void copyVirtScreenBuffers(Common::Rect rect, int dirtybit = 0);
+#endif
 
 	byte *getMaskBuffer(int x, int y, int z);
 	void disableZBuffer() { _zbufferDisabled = true; }
@@ -295,6 +264,101 @@ public:
 	};
 };
 
+class GdiNES : public Gdi {
+protected:
+	struct {
+		byte nametable[16][64], nametableObj[16][64];
+		byte attributes[64], attributesObj[64];
+		byte masktable[16][8], masktableObj[16][8];
+		int  objX;
+		bool hasmask;
+	} _NES;
+
+protected:
+	void decodeNESGfx(const byte *room);
+	void decodeNESObject(const byte *ptr, int xpos, int ypos, int width, int height);
+
+	void drawStripNES(byte *dst, byte *mask, int dstPitch, int stripnr, int top, int height);
+	void drawStripNESMask(byte *dst, int stripnr, int top, int height) const;
+
+	virtual bool drawStrip(byte *dstPtr, VirtScreen *vs,
+					int x, int y, const int width, const int height,
+					int stripnr, const byte *smap_ptr);
+
+	virtual void decodeMask(int x, int y, const int width, const int height,
+	                int stripnr, int numzbuf, const byte *zplane_list[9],
+	                bool transpStrip, byte flag, const byte *tmsk_ptr);
+
+	virtual void prepareDrawBitmap(const byte *ptr, VirtScreen *vs,
+					const int x, const int y, const int width, const int height,
+	                int stripnr, int numstrip);
+
+public:
+	GdiNES(ScummEngine *vm);
+
+	virtual void roomChanged(byte *roomptr);
+};
+
+class GdiV1 : public Gdi {
+protected:
+	/** Render settings which are specific to the C64 graphic decoders. */
+	struct {
+		byte colors[4];
+		byte charMap[2048], objectMap[2048], picMap[4096], colorMap[4096];
+		byte maskMap[4096], maskChar[4096];
+	} _C64;
+
+protected:
+	void decodeC64Gfx(const byte *src, byte *dst, int size) const;
+
+	void drawStripC64Object(byte *dst, int dstPitch, int stripnr, int width, int height);
+	void drawStripC64Background(byte *dst, int dstPitch, int stripnr, int height);
+	void drawStripC64Mask(byte *dst, int stripnr, int width, int height) const;
+
+	virtual bool drawStrip(byte *dstPtr, VirtScreen *vs,
+					int x, int y, const int width, const int height,
+					int stripnr, const byte *smap_ptr);
+
+	virtual void decodeMask(int x, int y, const int width, const int height,
+	                int stripnr, int numzbuf, const byte *zplane_list[9],
+	                bool transpStrip, byte flag, const byte *tmsk_ptr);
+
+	virtual void prepareDrawBitmap(const byte *ptr, VirtScreen *vs,
+					const int x, const int y, const int width, const int height,
+	                int stripnr, int numstrip);
+
+public:
+	GdiV1(ScummEngine *vm);
+
+	virtual void roomChanged(byte *roomptr);
+};
+
+class GdiV2 : public Gdi {
+protected:
+	/** For V2 games, we cache offsets into the room graphics, to speed up things. */
+	StripTable *_roomStrips;
+
+protected:
+	StripTable *generateStripTable(const byte *src, int width, int height, StripTable *table) const;
+
+	virtual bool drawStrip(byte *dstPtr, VirtScreen *vs,
+					int x, int y, const int width, const int height,
+					int stripnr, const byte *smap_ptr);
+
+	virtual void decodeMask(int x, int y, const int width, const int height,
+	                int stripnr, int numzbuf, const byte *zplane_list[9],
+	                bool transpStrip, byte flag, const byte *tmsk_ptr);
+
+	virtual void prepareDrawBitmap(const byte *ptr, VirtScreen *vs,
+					const int x, const int y, const int width, const int height,
+	                int stripnr, int numstrip);
+
+public:
+	GdiV2(ScummEngine *vm);
+	~GdiV2();
+
+	virtual void roomChanged(byte *roomptr);
+};
 
 } // End of namespace Scumm
 
