@@ -1,6 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2001  Ludvig Strigeus
- * Copyright (C) 2001-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -16,57 +18,48 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/scumm/he/animation_he.h $
- * $Id: animation_he.h 22384 2006-05-08 10:21:17Z kirben $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/scumm/he/animation_he.h $
+ * $Id: animation_he.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
-#ifndef ANIMATION_H
-#define ANIMATION_H
+#ifndef SCUMM_HE_ANIMATION_H
+#define SCUMM_HE_ANIMATION_H
 
 #include "common/file.h"
 
+#include "graphics/dxa_player.h"
+
+#include "sound/mixer.h"
+
 namespace Scumm {
 
 class ScummEngine_v90he;
 
-class MoviePlayer {
+class MoviePlayer : public Graphics::DXAPlayer {
 	ScummEngine_v90he *_vm;
 
-	Common::File _fd;
-	uint8 *_frameBuffer1;
-	uint8 *_frameBuffer2;
-	uint16 _width;
-	uint16 _height;
-	uint16 _framesCount;
-	uint32 _framesPerSec;
-	uint16 _frameNum;
-	uint32 _frameSize;
-	uint32 _frameTicks;
+	Audio::Mixer *_mixer;
 
+	Audio::SoundHandle _bgSound;
+	Audio::AudioStream *_bgSoundStream;
+
+	char baseName[40];
 	uint32 _flags;
 	uint32 _wizResNum;
 	
 public:
-	MoviePlayer(ScummEngine_v90he *vm);
-	~MoviePlayer();
+	MoviePlayer(ScummEngine_v90he *vm, Audio::Mixer *mixer);
 
-	int getWidth();
-	int getHeight();
-	int getCurFrame();
-	int getFrameCount();
 	int getImageNum();
 	int load(const char *filename, int flags, int image = 0);
 
 	void handleNextFrame();
-	void close();
 
-private:
-	void copyFrame(byte *dst, uint x, uint y);
-	void decodeFrame();
-	void decodeZlib(uint8 *data, int size, int totalSize);
+protected:
+	virtual void setPalette(byte *pal);
 };
 
-} // End of namespace Simon
+} // End of namespace Scumm
 
 #endif
