@@ -1,6 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2001  Ludvig Strigeus
- * Copyright (C) 2001-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -16,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/scumm/he/floodfill_he.cpp $
- * $Id: floodfill_he.cpp 20862 2006-02-25 02:12:58Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/scumm/he/floodfill_he.cpp $
+ * $Id: floodfill_he.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -220,7 +222,7 @@ void floodFill(FloodFillParameters *ffp, ScummEngine_v90he *vm) {
 	
 	if (r.left <= r.right && r.top <= r.bottom) {
 		if (ffp->flags & 0x8000) {
-			vm->gdi.copyVirtScreenBuffers(r);
+			vm->restoreBackgroundHE(r);
 		} else {
 			++r.bottom;
 			vm->markRectAsDirty(kMainVirtScreen, r);
@@ -287,7 +289,7 @@ void Wiz::fillWizFlood(const WizParameters *params) {
 			}
 		}
 	}
-	_vm->res.setModified(rtImage, params->img.resNum);
+	_vm->_res->setModified(rtImage, params->img.resNum);
 }
 
 } // End of namespace Scumm
