@@ -1,6 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2001  Ludvig Strigeus
- * Copyright (C) 2001-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -16,13 +18,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/scumm/he/floodfill_he.h $
- * $Id: floodfill_he.h 20696 2006-02-15 00:57:50Z sev $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/scumm/he/floodfill_he.h $
+ * $Id: floodfill_he.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
-#if !defined(FLOODFILL_HE_H) && !defined(DISABLE_HE)
-#define FLOODFILL_HE_H
+#if !defined(SCUMM_HE_FLOODFILL_HE_H) && !defined(DISABLE_HE)
+#define SCUMM_HE_FLOODFILL_HE_H
 
 #include "common/rect.h"
 
