@@ -1,6 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2001  Ludvig Strigeus
- * Copyright (C) 2001-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -16,13 +18,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/scumm/he/intern_he.h $
- * $Id: intern_he.h 23819 2006-09-02 05:51:19Z kirben $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/scumm/he/intern_he.h $
+ * $Id: intern_he.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
-#ifndef SCUMM_INTERN_HE_H
-#define SCUMM_INTERN_HE_H
+#ifndef SCUMM_HE_INTERN_HE_H
+#define SCUMM_HE_INTERN_HE_H
 
 #include "scumm/intern.h"
 #ifndef DISABLE_HE
@@ -42,6 +44,7 @@ class ResExtractor;
 class LogicHE;
 class MoviePlayer;
 class Sprite;
+class CUP_Player;
 #endif
 
 class ScummEngine_v60he : public ScummEngine_v6 {
@@ -55,7 +58,6 @@ protected:
 	const OpcodeEntryv60he *_opcodesv60he;
 
 public:
-	//Common::File _hFileTable[17];
 	Common::SeekableReadStream *_hInFileTable[17];
 	Common::WriteStream *_hOutFileTable[17];
 
@@ -129,6 +131,9 @@ protected:
 
 	bool _skipProcessActors;
 
+	int _numStoredFlObjects;
+	ObjectData *_storedFlObjects;
+
 public:
 	ScummEngine_v70he(OSystem *syst, const DetectorResult &dr);
 	~ScummEngine_v70he();
@@ -139,6 +144,8 @@ public:
 	byte *heFindResource(uint32 tag, byte *ptr);
 	byte *findWrappedBlock(uint32 tag, byte *ptr, int state, bool flagError);
 
+	void restoreBackgroundHE(Common::Rect rect, int dirtybit = 0);
+
 protected:
 	virtual void setupOpcodes();
 	virtual void executeOpcode(byte i);
@@ -153,8 +160,15 @@ protected:
 	virtual void readGlobalObjects();
 	virtual void readIndexBlock(uint32 blocktype, uint32 itemsize);
 
+	virtual void clearRoomObjects();
+	virtual void resetRoomObjects();
+
 	virtual int getActorFromPos(int x, int y);
 
+	virtual void loadFlObject(uint object, uint room);
+	void storeFlObject(int slot);
+	void restoreFlObjects();
+
 	int getStringCharWidth(byte chr);
 	virtual int setupStringArray(int size);
 	void appendSubstring(int dst, int src, int len2, int len);
@@ -185,7 +199,8 @@ protected:
 	void o70_writeINI();
 	void o70_getStringLenForWidth();
 	void o70_getCharIndexInString();
-	void o70_setFilePath();
+	void o70_createDirectory();
+	void o70_findBox();
 	void o70_setSystemMessage();
 	void o70_polygonOps();
 	void o70_polygonHit();
@@ -226,9 +241,7 @@ protected:
 		const char *desc;
 	};
 
-#if !defined(__GNUC__)
-	#pragma START_PACK_STRUCTS
-#endif
+#include "common/pack-start.h"	// START STRUCT PACKING
 
 	struct ArrayHeader {
 		int32 type;      //0
@@ -237,11 +250,9 @@ protected:
 		int32 dim2start; //0C
 		int32 dim2end;   //10
 		byte data[1];    //14
-	} GCC_PACK;
+	};
 
-#if !defined(__GNUC__)
-	#pragma END_PACK_STRUCTS
-#endif
+#include "common/pack-end.h"	// END STRUCT PACKING
 
 	const OpcodeEntryV72he *_opcodesV72he;
 
@@ -269,7 +280,7 @@ protected:
 
 	virtual void redrawBGAreas();
 
-	ArrayHeader *defineArray(int array, int type, int dim2start, int dim2end, int dim1start, int dim1end);
+	byte *defineArray(int array, int type, int dim2start, int dim2end, int dim1start, int dim1end);
 	virtual int readArray(int array, int idx2, int idx1);
 	virtual void writeArray(int array, int idx2, int idx1, int value);
 	void redimArray(int arrayId, int newDim2start, int newDim2end,
@@ -337,7 +348,7 @@ protected:
 	void o72_readINI();
 	void o72_writeINI();
 	void o72_getResourceSize();
-	void o72_setFilePath();
+	void o72_createDirectory();
 	void o72_setSystemMessage();
 
 	byte VAR_NUM_ROOMS;
@@ -398,7 +409,7 @@ protected:
 	void o80_pickVarRandom();
 
 	byte VAR_PLATFORM;
-	byte VAR_WINDOWS_VERSION;
+	byte VAR_PLATFORM_VERSION;
 	byte VAR_CURRENT_CHARSET;
 	byte VAR_COLOR_DEPTH;
 };
@@ -436,27 +447,32 @@ protected:
 	int32 _curSpriteId;
 	int32 _curSpriteGroupId;
 
-public:
-	ScummEngine_v90he(OSystem *syst, const DetectorResult &dr);
-	~ScummEngine_v90he();
-
-	virtual void resetScumm();
-
 	LogicHE *_logicHE;
 	MoviePlayer *_moviePlay;
 	Sprite *_sprite;
 
+public:
+	ScummEngine_v90he(OSystem *syst, const DetectorResult &dr);
+	~ScummEngine_v90he();
+
 protected:
 	virtual void allocateArrays();
 	virtual void setupOpcodes();
 	virtual void executeOpcode(byte i);
 	virtual const char *getOpcodeDesc(byte i);
 
-	virtual void scummLoop_handleDrawing();
+	virtual void resetScumm();
 
 	virtual void setupScummVars();
 	virtual void resetScummVars();
 
+	virtual int scummLoop(int delta);
+	virtual void scummLoop_handleDrawing();
+	virtual void runBootscript();
+
+	virtual void processInput();
+	virtual void clearClickedStatus();
+
 	virtual void saveOrLoad(Serializer *s);
 
 	virtual void readMAXS(int blockSize);
@@ -617,6 +633,27 @@ protected:
 	void o100_getWizData();
 	void o100_getVideoData();
 };
+
+class ScummEngine_vCUPhe : public Engine {
+protected:
+	CUP_Player *_cupPlayer;
+	FilenamePattern _filenamePattern;
+
+public:
+	ScummEngine_vCUPhe(OSystem *syst, const DetectorResult &dr);
+	~ScummEngine_vCUPhe();
+
+	int init();
+	int go();
+
+	void parseEvents();
+
+	bool _quit;
+	OSystem *_syst;
+
+	GameSettings _game;
+};
+
 #endif
 
 
