@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2005-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/scumm/he/logic_he.cpp $
- * $Id: logic_he.cpp 20799 2006-02-20 20:41:13Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/scumm/he/logic_he.cpp $
+ * $Id: logic_he.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -28,12 +31,22 @@
 namespace Scumm {
 
 LogicHE::LogicHE(ScummEngine_v90he *vm) : _vm(vm) {
-	// Originally it used 0x930 and stored both floats and doubles inside
-	_userData = (float *)calloc(550, sizeof(float));
-	_userDataD = (double *)calloc(30, sizeof(double));
 }
 
 LogicHE::~LogicHE() {
+}
+
+LogicHErace::LogicHErace(ScummEngine_v90he *vm) : LogicHE(vm) {
+	// Originally it used 0x930 and stored both floats and doubles inside
+	_userData = (float *)calloc(550, sizeof(float));
+	_userDataD = (double *)calloc(30, sizeof(double));
+
+	// FIXME: of the 550 entries in _userData, only 516 till 532 are used
+	// FIXME: similarly, in _userDataD only 9 till 17 are used for computations
+	//       (some of the other entries are also set, but never read, hence useless).
+}
+
+LogicHErace::~LogicHErace() {
 	free(_userData);
 	free(_userDataD);
 }
@@ -172,29 +185,34 @@ int32 LogicHErace::op_1004(int32 *args) {
 }
 
 int32 LogicHErace::op_1100(int32 *args) {
+	// _userData 516,517,518 describe a 3D translation?
 	_userData[516] = (float)args[0] / args[10];
 	_userData[517] = (float)args[1] / args[10];
 	_userData[518] = (float)args[2] / args[10];
+
+	// _userData 519,520,521 describe rotation angles around the x,y,z axes?
 	_userData[519] = (float)args[3] / args[10];
 	_userData[520] = (float)args[4] / args[10];
-
-	op_sub1(_userData[520]);
-
 	_userData[521] = (float)args[5] / args[10];
 
+	op_sub1(_userData[520]);
 	op_sub2(_userData[521]);
 
+	// _userData[532] seems to be some kind of global scale factor
 	_userData[532] = (float)args[10];
 
-	_userData[524] = (float)args[8];
-	_userData[525] = (float)args[9];
-	_userData[522] = (float)args[6] / args[10];
-	_userData[523] = (float)args[7] / args[10];
+	_userData[524] = (float)args[8];	// not used
+	_userData[525] = (float)args[9];	// not used
+	_userData[522] = (float)args[6] / args[10];	// not used
+	_userData[523] = (float)args[7] / args[10];	// only used to compute 528 and 529
+
+	// The following two are some kind of scale factors
 	_userData[526] = (float)args[6] / args[8] / args[10];
 	_userData[527] = (float)args[7] / args[9] / args[10];
 
+	// Set var 108 and 109 -- the value set here corresponds to the values
+	// set by op_1110!
 	writeScummVar(108, (int32)((float)args[6] / args[8] * args[10]));
-
 	writeScummVar(109, (int32)((float)args[7] / args[9] * args[10]));
 
 	_userData[528] = (float)(_userData[519] - _userData[523] * 0.5);
@@ -203,6 +221,8 @@ int32 LogicHErace::op_1100(int32 *args) {
 	writeScummVar(110, (int32)(_userData[528] * args[10]));
 	writeScummVar(111, (int32)(_userData[529] * args[10]));
 
+	// 530 and 531 are only used to set vars 112 and 113, so no need
+	// to store them permanently
 	_userData[530] = (float)(_userData[517] / tan(_userData[529] * DEG2RAD));
 	_userData[531] = (float)(_userData[517] / tan(_userData[528] * DEG2RAD));
 
@@ -213,21 +233,20 @@ int32 LogicHErace::op_1100(int32 *args) {
 }
 
 int32 LogicHErace::op_1101(int32 *args) {
+	// Update rotation params?
 	int32 retval;
 	float temp;
 
     temp = args[0] / _userData[532];
-
-	if (_userData[519] == temp) {
-		retval = (int32)temp;
-	} else {
+	if (_userData[519] != temp) {
 		_userData[519] = temp;
 		op_sub3(temp);
 		retval = 1;
+	} else {
+		retval = (int32)temp;
 	}
 
 	temp = args[1] / _userData[532];
-
 	if (_userData[520] != temp) {
 		_userData[520] = temp;
 		op_sub1(temp);
@@ -235,7 +254,6 @@ int32 LogicHErace::op_1101(int32 *args) {
 	}
 
 	temp = args[2] / _userData[532];
-
 	if (_userData[521] != temp) {
 		_userData[521] = temp;
 		op_sub2(temp);
@@ -246,6 +264,7 @@ int32 LogicHErace::op_1101(int32 *args) {
 }
 
 int32 LogicHErace::op_1102(int32 *args) {
+	// Update translation params?
 	int32 retval;
 	float temp;
 
@@ -290,19 +309,21 @@ int32 LogicHErace::op_1110() {
 }
 
 int32 LogicHErace::op_1120(int32 *args) {
-	double a0, a1, a2, expr;
+	double a0, a1, a2;
+	double b0, b1, b2;
 	double res1, res2;
 
 	a0 = args[0] / _userData[532] - _userData[516];
 	a1 = args[1] / _userData[532] - _userData[517];
 	a2 = args[2] / _userData[532] - _userData[518];
 
-	expr = a2 * _userDataD[17] + a1 * _userDataD[14] + a0 * _userDataD[11];
+	// Perform matrix multiplication (multiplying by a rotation matrix)
+	b2 = a2 * _userDataD[17] + a1 * _userDataD[14] + a0 * _userDataD[11];
+	b1 = a2 * _userDataD[16] + a1 * _userDataD[13] + a0 * _userDataD[10];
+	b0 = a2 * _userDataD[15] + a1 * _userDataD[12] + a0 * _userDataD[9];
 
-	res1 = (atan2(a2 * _userDataD[15] + a1 * _userDataD[12] + a0 * _userDataD[9], expr) * RAD2DEG)
-			/ _userData[526];
-	res2 = (atan2(a2 * _userDataD[16] + a1 * _userDataD[13] + a0 * _userDataD[10], expr) * RAD2DEG
-			- _userData[528]) / _userData[527];
+	res1 = (atan2(b0, b2) * RAD2DEG) / _userData[526];
+	res2 = (atan2(b1, b2) * RAD2DEG - _userData[528]) / _userData[527];
 
 	writeScummVar(108, (int32)res1);
 	writeScummVar(109, (int32)res2);
@@ -315,36 +336,44 @@ int32 LogicHErace::op_1130(int32 *args) {
 	double sn = sin(args[0] / _userData[532] * DEG2RAD);
 
 	writeScummVar(108, (int32)(cs * args[1] + sn * args[2]));
-
 	writeScummVar(109, (int32)(cs * args[2] - sn * args[1]));
 
 	return 1;
 }
 
 int32 LogicHErace::op_1140(int32 *args) {
-	double arg2 = -args[2] * args[2];
-	double arg3 = -args[3] * args[3];
-	double sq = sqrt(arg2 + arg3);
-	double res;
+	// This functions seems to perform some kind of projection: We project
+	// the vector (arg2,arg3) onto the vector (arg0,arg1), but also apply
+	// some kind of distortion factor ?!?
+	double x = args[2], y = args[3];
 
-	arg2 = arg2 / sq;
-	arg3 = arg3 / sq;
+	// We start by normalizing the vector described by arg2 and arg3.
+	// So compute its length and divide the x and y coordinates
+	const double sq = sqrt(x*x + y*y);
+	x /= sq;
+	y /= sq;
 
-	res = (args[0] - 2 * (arg2 * args[0] + arg3 * args[1]) * arg2) * 0.86956525;
+	// Compute the scalar product of the vectors (arg0,arg1) and (x,y)
+	const double scalarProduct = x * args[0] + y * args[1];
 
-	writeScummVar(108, (int32)res);
+	// Finally compute the projection of (arg2,arg3) onto (arg0,arg1)
+	double projX = args[0] - 2 * scalarProduct * args[2];
+	double projY = args[1] - 2 * scalarProduct * args[3];
 
-	res = args[1] - 2 * (arg2 * args[0] + arg3 * args[1]) * arg3;
+	projX = projX * 20.0 / 23.0;	// FIXME: Why is this here?
 
-	if (-args[3] * args[3] >= 0)
-		res *= 0.83333331f;
+	writeScummVar(108, (int32)projX);
 
-	writeScummVar(109, (int32)res);
+	if (args[3] >= 0)	// FIXME: Why is this here?
+		projY = projY * 5.0 / 6.0;
+
+	writeScummVar(109, (int32)projY);
 
 	return 1;
 }
 
 void LogicHErace::op_sub1(float arg) {
+	// Setup a rotation matrix
 	_userDataD[10] = _userDataD[12] = _userDataD[14] = _userDataD[16] = 0;
 	_userDataD[13] = 1;
 
@@ -355,6 +384,7 @@ void LogicHErace::op_sub1(float arg) {
 }
 
 void LogicHErace::op_sub2(float arg) {
+	// Setup a rotation matrix -- but it is NEVER USED!
 	_userDataD[20] = _userDataD[21] = _userDataD[24] = _userDataD[25] = 0;
 	_userDataD[26] = 1;
 
@@ -365,6 +395,7 @@ void LogicHErace::op_sub2(float arg) {
 }
 
 void LogicHErace::op_sub3(float arg) {
+	// Setup a rotation matrix -- but it is NEVER USED!
 	_userDataD[1] = _userDataD[2] = _userDataD[3] = _userDataD[6] = 0;
 	_userDataD[0] = 1;
 
@@ -397,7 +428,6 @@ int32 LogicHEfunshop::dispatch(int op, int numArgs, int32 *args) {
 
 	default:
 		break;
-
 	}
 
 	return 0;
@@ -460,14 +490,14 @@ void LogicHEfunshop::op_1004(int32 *args) {
 void LogicHEfunshop::op_1005(int32 *args) {
 	double data[8];
 	double args1, args2;
-	int i=0;
+	int i;
 	for (i = 520; i <= 526; i += 2) {
 		data[i - 520] = getFromArray(args[0], 0, i - 1);
 		data[i - 520 + 1] = getFromArray(args[0], 0, i);
 	}
 
-	args1 = args[1] * 0.01 + 1;
-	args2 = args[2] * 0.01 + 1;
+	args1 = args[1] / 100.0 + 1;
+	args2 = args[2] / 100.0 + 1;
 
 	for (i = 0; i < 4; i++) {
 		data[2 * i] *= args1;
@@ -503,7 +533,7 @@ int LogicHEfunshop::checkShape(int32 data0, int32 data1, int32 data4, int32 data
 	diff7_3 = data7 - data3;
 	diff2 = data3 * data6 - data2 * data7;
 	sum1 = diff2_6 * data1 + diff2 + diff7_3 * data0;
-	sum2 = diff2_6 * data5 + diff2 + diff7_3 * data4;;
+	sum2 = diff2_6 * data5 + diff2 + diff7_3 * data4;
 
 	if (sum1 != 0 && sum2 != 0) {
 		sum2 ^= sum1;
@@ -549,6 +579,7 @@ int LogicHEfunshop::checkShape(int32 data0, int32 data1, int32 data4, int32 data
 
 /***********************
  * Backyard Football
+ * Backyard Football 2002
  * Backyard Football Demo
  *
  */
@@ -623,13 +654,14 @@ int32 LogicHEfootball::dispatch(int op, int numArgs, int32 *args) {
 
 	default:
 		LogicHE::dispatch(op, numArgs, args);
-		error("Tell sev how to reproduce it");
+		warning("Tell sev how to reproduce it (%d)", op);
 	}
 
 	return res;
 }
 
 int LogicHEfootball::op_1004(int32 *args) {
+	// Identical to LogicHEsoccer::op_1004
 	double res, a2, a4, a5;
 
 	a5 = ((double)args[4] - (double)args[1]) / ((double)args[5] - (double)args[2]);
@@ -647,13 +679,26 @@ int LogicHEfootball::op_1004(int32 *args) {
 }
 
 int LogicHEfootball::op_1006(int32 *args) {
+	// This seems to be more or less the inverse of op_1010
+	const double a1 = args[1];
 	double res;
 
-	res = (1.0 - args[1] * 2.9411764e-4 * 5.3050399e-2) * args[0] * 1.2360656e-1 +
-		args[1] * 1.1764706e-2 + 46;
+	// 2.9411764e-4 = 1/3400
+	// 5.3050399e-2 = 1/18.85 = 20/377
+	// 1.1764706e-2 = 1/85 = 40/3400
+	// 1.2360656e-1 = 377/3050
+	res = (1.0 - a1 * 2.9411764e-4 * 5.3050399e-2) * 1.2360656e-1 * args[0] +
+		a1 * 1.1764706e-2 + 46;
+
+	// Shortened / optimized version of that formula:
+	// res = (377.0 - a1 / 170.0) / 3050.0 * args[0] + a1 / 85.0 + 46;
+
 	writeScummVar(108, (int32)res);
 
-	res = 640.0 - args[2] * 1.2360656e-1 - args[1] * 1.1588235e-1 - 26;
+	// 1.2360656e-1 = 377/3050
+	// 1.1588235e-1 = 197/1700 = 394/3400
+	res = 640.0 - args[2] * 1.2360656e-1 - a1 * 1.1588235e-1 - 26;
+
 	writeScummVar(109, (int32)res);
 
 	return 1;
@@ -681,13 +726,17 @@ int LogicHEfootball::op_1007(int32 *args) {
 }
 
 int LogicHEfootball::op_1010(int32 *args) {
-	double a1 = (640.0 - (double)args[1] - 26.0) * 8.6294413;
-	double res;
+	// This seems to be more or less the inverse of op_1006
+	double a1 = (640.0 - (double)args[1] - 26.0) / 1.1588235e-1;
 
-	res = ((double)args[0] - 46 - a1 * 1.1764706e-2) /
+	// 2.9411764e-4 = 1/3400
+	// 5.3050399e-2 = 1/18.85 = 20/377
+	// 1.1764706e-2 = 1/85 = 40/3400
+	// 1.2360656e-1 = 377/3050
+	double a0 = ((double)args[0] - 46 - a1 * 1.1764706e-2) /
 		((1.0 - a1 * 2.9411764e-4 * 5.3050399e-2) * 1.2360656e-1);
-	writeScummVar(108, (int32)res);
 
+	writeScummVar(108, (int32)a0);
 	writeScummVar(109, (int32)a1);
 
 	return 1;
@@ -802,7 +851,6 @@ int32 LogicHEsoccer::dispatch(int op, int numArgs, int32 *args) {
 	default:
 		// original range is 1001 - 1021
 		LogicHE::dispatch(op, numArgs, args);
-		warning("Tell sev how to reproduce it");
 	}
 
 	return res;
@@ -817,6 +865,7 @@ int LogicHEsoccer::op_1002(int32 *args) {
 }
 
 int LogicHEsoccer::op_1004(int32 *args) {
+	// Identical to LogicHEfootball::op_1004
 	double res, a2, a4, a5;
 
 	a5 = ((double)args[4] - (double)args[1]) / ((double)args[5] - (double)args[2]);
