@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2005-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,13 +18,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/scumm/he/logic_he.h $
- * $Id: logic_he.h 20696 2006-02-15 00:57:50Z sev $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/scumm/he/logic_he.h $
+ * $Id: logic_he.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
-#if !defined(LOGIC_HE_H) && !defined(DISABLE_HE)
-#define LOGIC_HE_H
+#if !defined(SCUMM_HE_LOGIC_HE_H) && !defined(DISABLE_HE)
+#define SCUMM_HE_LOGIC_HE_H
 
 #include "common/stdafx.h"
 
@@ -31,8 +34,6 @@ class ScummEngine_v90he;
 
 class LogicHE {
 public:
-	float *_userData;
-	double *_userDataD;
 	ScummEngine_v90he *_vm;
 
 	LogicHE(ScummEngine_v90he *vm);
@@ -42,19 +43,23 @@ public:
 	int getFromArray(int arg0, int idx2, int idx1);
 	void putInArray(int arg0, int idx2, int idx1, int val);
 
-	void beforeBootScript(void) {};
-	void initOnce() {};
-	void startOfFrame() {};
-	void endOfFrame() {};
-	void processKeyStroke(int keyPressed) {};
+	void beforeBootScript(void) {}
+	void initOnce() {}
+	void startOfFrame() {}
+	void endOfFrame() {}
+	void processKeyStroke(int keyPressed) {}
 
 	virtual int versionID();
 	virtual int32 dispatch(int op, int numArgs, int32 *args);
 };
 
 class LogicHErace : public LogicHE {
+private:
+	float *_userData;
+	double *_userDataD;
 public:
-	LogicHErace(ScummEngine_v90he *vm) : LogicHE(vm) {}
+	LogicHErace(ScummEngine_v90he *vm);
+	~LogicHErace();
 
 	int versionID();
 	int32 dispatch(int op, int numArgs, int32 *args);
