@@ -1,6 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2001  Ludvig Strigeus
- * Copyright (C) 2001-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +17,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/scumm/he/palette_he.cpp $
- * $Id: palette_he.cpp 20862 2006-02-25 02:12:58Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/scumm/he/palette_he.cpp $
+ * $Id: palette_he.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -78,13 +80,13 @@ uint8 *ScummEngine_v90he::getHEPaletteIndex(int palSlot) {
 }
 
 int ScummEngine_v90he::getHEPaletteSimilarColor(int palSlot, int red, int green, int start, int end) {
-	checkRange(_numPalettes, 1, palSlot, "Invalid palette %d");
-	checkRange(255, 0, start, "Invalid palette slot %d");
-	checkRange(255, 0, end, "Invalid palette slot %d");
+	assertRange(1, palSlot, _numPalettes, "palette");
+	assertRange(0, start, 255, "start palette slot");
+	assertRange(0, end, 255, "pend alette slot");
 
 	uint8 *pal = _hePalettes + palSlot * 1024 + start * 3;
 
-	int bestsum = 0xFFFFFFFF;
+	int bestsum = 0x7FFFFFFF;
 	int bestitem = start;
 
 	for (int i = start; i <= end; i++) {
@@ -104,22 +106,22 @@ int ScummEngine_v90he::getHEPaletteSimilarColor(int palSlot, int red, int green,
 }
 
 int ScummEngine_v90he::getHEPaletteColorComponent(int palSlot, int color, int component) {
-	checkRange(_numPalettes, 1, palSlot, "Invalid palette %d");
-	checkRange(255, 0, color, "Invalid palette slot %d");
+	assertRange(1, palSlot, _numPalettes, "palette");
+	assertRange(0, color, 255, "palette slot");
 
 	return _hePalettes[palSlot * 1024 + color * 3 + component % 3];
 }
 
 int ScummEngine_v90he::getHEPaletteColor(int palSlot, int color) {
-	checkRange(_numPalettes, 1, palSlot, "Invalid palette %d");
-	checkRange(255, 0, color, "Invalid palette slot %d");
+	assertRange(1, palSlot, _numPalettes, "palette");
+	assertRange(0, color, 255, "palette slot");
 
 	return _hePalettes[palSlot * 1024 + 768 + color];
 }
 
 void ScummEngine_v90he::setHEPaletteColor(int palSlot, uint8 color, uint8 r, uint8 g, uint8 b) {
 	debug(7, "setHEPaletteColor(%d, %d, %d, %d, %d)", palSlot, color, r, g, b);
-	checkRange(_numPalettes, 1, palSlot, "Invalid palette %d");
+	assertRange(1, palSlot, _numPalettes, "palette");
 	uint8 *p = _hePalettes + palSlot * 1024 + color * 3;
 	*(p + 0) = r;
 	*(p + 1) = g;
@@ -128,7 +130,7 @@ void ScummEngine_v90he::setHEPaletteColor(int palSlot, uint8 color, uint8 r, uin
 }
 
 void ScummEngine_v90he::setHEPaletteFromPtr(int palSlot, const uint8 *palData) {
-	checkRange(_numPalettes, 1, palSlot, "Invalid palette %d");
+	assertRange(1, palSlot, _numPalettes, "palette");
 	uint8 *pc = _hePalettes + palSlot * 1024;
 	uint8 *pi = pc + 768;
 	for (int i = 0; i < 256; ++i) {
@@ -141,7 +143,7 @@ void ScummEngine_v90he::setHEPaletteFromPtr(int palSlot, const uint8 *palData) {
 
 void ScummEngine_v90he::setHEPaletteFromCostume(int palSlot, int resId) {
 	debug(7, "setHEPaletteFromCostume(%d, %d)", palSlot, resId);
-	checkRange(_numPalettes, 1, palSlot, "Invalid palette %d");
+	assertRange(1, palSlot, _numPalettes, "palette");
 	const uint8 *data = getResourceAddress(rtCostume, resId);
 	assert(data);
 	const uint8 *rgbs = findResourceData(MKID_BE('RGBS'), data);
@@ -151,7 +153,7 @@ void ScummEngine_v90he::setHEPaletteFromCostume(int palSlot, int resId) {
 
 void ScummEngine_v90he::setHEPaletteFromImage(int palSlot, int resId, int state) {
 	debug(7, "setHEPaletteFromImage(%d, %d, %d)", palSlot, resId, state);
-	checkRange(_numPalettes, 1, palSlot, "Invalid palette %d");
+	assertRange(1, palSlot, _numPalettes, "palette");
 	uint8 *data = getResourceAddress(rtImage, resId);
 	assert(data);
 	const uint8 *rgbs = findWrappedBlock(MKID_BE('RGBS'), data, state, 0);
@@ -161,7 +163,7 @@ void ScummEngine_v90he::setHEPaletteFromImage(int palSlot, int resId, int state)
 
 void ScummEngine_v90he::setHEPaletteFromRoom(int palSlot, int resId, int state) {
 	debug(7, "setHEPaletteFromRoom(%d, %d, %d)", palSlot, resId, state);
-	checkRange(_numPalettes, 1, palSlot, "Invalid palette %d");
+	assertRange(1, palSlot, _numPalettes, "palette");
 	const uint8 *data = getResourceAddress(rtRoom, resId);
 	assert(data);
 	const uint8 *pals = findResourceData(MKID_BE('PALS'), data);
@@ -173,7 +175,7 @@ void ScummEngine_v90he::setHEPaletteFromRoom(int palSlot, int resId, int state)
 
 void ScummEngine_v90he::restoreHEPalette(int palSlot) {
 	debug(7, "restoreHEPalette(%d)", palSlot);
-	checkRange(_numPalettes, 1, palSlot, "Invalid palette %d");
+	assertRange(1, palSlot, _numPalettes, "palette");
 	if (palSlot != 1) {
 		memcpy(_hePalettes + palSlot * 1024, _hePalettes + 1024, 1024);
 	}
@@ -190,7 +192,7 @@ void ScummEngine_v90he::copyHEPalette(int dstPalSlot, int srcPalSlot) {
 
 void ScummEngine_v90he::copyHEPaletteColor(int palSlot, uint8 dstColor, uint8 srcColor) {
 	debug(7, "copyHEPaletteColor(%d, %d, %d)", palSlot, dstColor, srcColor);
-	checkRange(_numPalettes, 1, palSlot, "Invalid palette %d");
+	assertRange(1, palSlot, _numPalettes, "palette");
 	uint8 *dstPal = _hePalettes + palSlot * 1024 + dstColor * 3;
 	uint8 *srcPal = _hePalettes + 1024 + srcColor * 3;
 	memcpy(dstPal, srcPal, 3);
@@ -205,7 +207,7 @@ void ScummEngine_v99he::setPaletteFromPtr(const byte *ptr, int numcolor) {
 		numcolor = getResourceDataSize(ptr) / 3;
 	}
 
-	checkRange(256, 0, numcolor, "Too many colors (%d) in Palette");
+	assertRange(0, numcolor, 256, "setPaletteFromPtr: numcolor");
 
 	dest = _hePalettes + 1024;
 
