@@ -1,9 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2004-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
  *
- * Parts of code heavily based on:
- * icoutils - A set of programs dealing with MS Windows icons and cursors.
- * Copyright (C) 1998-2001 Oskar Liljeblad
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -19,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/scumm/he/resource_he.cpp $
- * $Id: resource_he.cpp 22548 2006-05-20 15:09:15Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/scumm/he/resource_he.cpp $
+ * $Id: resource_he.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -33,13 +32,17 @@
 #include "scumm/he/sound_he.h"
 
 #include "sound/wave.h"
-#include "graphics/paletteman.h"
+#include "graphics/cursorman.h"
 
 #include "common/stream.h"
 #include "common/system.h"
 
 namespace Scumm {
 
+#define LE16(x)      ((x) = TO_LE_16(x))
+#define LE32(x)      ((x) = TO_LE_32(x))
+
+
 ResExtractor::ResExtractor(ScummEngine_v70he *scumm)
 	: _vm(scumm) {
 
@@ -108,7 +111,7 @@ void ResExtractor::setCursor(int id) {
 	}
 
 	if (cc->palette)
-		PaletteMan.replaceCursorPalette(cc->palette, 0, cc->palSize);
+		CursorMan.replaceCursorPalette(cc->palette, 0, cc->palSize);
 
 	_vm->setCursorHotspot(cc->hotspot_x, cc->hotspot_y);
 	_vm->setCursorFromBuffer(cc->bitmap, cc->w, cc->h, cc->w);
@@ -145,7 +148,6 @@ int Win32ResExtractor::extractResource_(const char *resType, char *resName, byte
 	char *arg_language = NULL;
 	const char *arg_type = resType;
 	char *arg_name = resName;
-	int arg_action = ACTION_LIST;
 	int ressize = 0;
 
 	_arg_raw = false;
@@ -193,11 +195,8 @@ int Win32ResExtractor::extractResource_(const char *resType, char *resName, byte
 		goto cleanup;
 	}
 
-	//	verbose_printf("file is a %s\n",
-	//		fi.is_PE_binary ? "Windows NT `PE' binary" : "Windows 3.1 `NE' binary");
-
 	/* errors will be printed by the callback */
-	ressize = do_resources(&fi, arg_type, arg_name, arg_language, arg_action, data);
+	ressize = do_resources(&fi, arg_type, arg_name, arg_language, data);
 
 	/* free stuff and close file */
 	cleanup:
@@ -245,6 +244,20 @@ const char *Win32ResExtractor::res_type_string_to_id(const char *type) {
 	return type;
 }
 
+/* return the resource id quoted if it's a string, otherwise just return it */
+char *Win32ResExtractor::WinResource::get_resource_id_quoted() {
+	// FIXME: Using a static var here is EVIL and in fact, broken when
+	// used multiple times in a row, e.g. in a single call to printf()
+	// or debug()... which is in fact how we use this function... :-)
+	static char tmp[WINRES_ID_MAXLEN+2];
+
+	if (numeric_id || id[0] == '\0')
+		return id;
+
+	sprintf(tmp, "'%s'", id);
+	return tmp;
+}
+
 int Win32ResExtractor::extract_resources(WinLibrary *fi, WinResource *wr,
                             WinResource *type_wr, WinResource *name_wr,
 							WinResource *lang_wr, byte **data) {
@@ -271,9 +284,9 @@ int Win32ResExtractor::extract_resources(WinLibrary *fi, WinResource *wr,
 		type = res_type_id_to_string(id);
 
 	debugC(DEBUG_RESOURCE, "extractCursor(). Found cursor name: %s%s%s [size=%d]",
-	  get_resource_id_quoted(name_wr),
+	  name_wr->get_resource_id_quoted(),
 	  (lang_wr->id[0] != '\0' ? " language: " : ""),
-	  get_resource_id_quoted(lang_wr), size);
+	  lang_wr->get_resource_id_quoted(), size);
 
 	return size;
 }
@@ -339,7 +352,7 @@ byte *Win32ResExtractor::extract_group_icon_cursor_resource(WinLibrary *fi, WinR
 	/* calculate total size of output file */
 	RETURN_IF_BAD_POINTER(NULL, icondir->count);
 	skipped = 0;
-	for (c = 0 ; c < icondir->count ; c++) {
+	for (c = 0 ; c < FROM_LE_16(icondir->count) ; c++) {
 		int level;
 		int iconsize;
 		char name[14];
@@ -347,14 +360,14 @@ byte *Win32ResExtractor::extract_group_icon_cursor_resource(WinLibrary *fi, WinR
 
 		RETURN_IF_BAD_POINTER(NULL, icondir->entries[c]);
 		/*printf("%d. bytes_in_res=%d width=%d height=%d planes=%d bit_count=%d\n", c,
-			icondir->entries[c].bytes_in_res,
-			(is_icon ? icondir->entries[c].res_info.icon.width : icondir->entries[c].res_info.cursor.width),
-			(is_icon ? icondir->entries[c].res_info.icon.height : icondir->entries[c].res_info.cursor.height),
-			icondir->entries[c].plane_count,
-			icondir->entries[c].bit_count);*/
+			FROM_LE_32(icondir->entries[c].bytes_in_res),
+			(is_icon ? icondir->entries[c].res_info.icon.width : FROM_LE_16(icondir->entries[c].res_info.cursor.width)),
+			(is_icon ? icondir->entries[c].res_info.icon.height : FROM_LE_16(icondir->entries[c].res_info.cursor.height)),
+			FROM_LE_16(icondir->entries[c].plane_count),
+			FROM_LE_16(icondir->entries[c].bit_count));*/
 
 		/* find the corresponding icon resource */
-		snprintf(name, sizeof(name)/sizeof(char), "-%d", icondir->entries[c].res_id);
+		snprintf(name, sizeof(name)/sizeof(char), "-%d", FROM_LE_16(icondir->entries[c].res_id));
 		fwr = find_resource(fi, (is_icon ? "-3" : "-1"), name, lang, &level);
 		if (fwr == NULL) {
 			error("%s: could not find `%s' in `%s' resource.",
@@ -368,11 +381,11 @@ byte *Win32ResExtractor::extract_group_icon_cursor_resource(WinLibrary *fi, WinR
 				skipped++;
 				continue;
 		    }
-		    if ((uint32)iconsize != icondir->entries[c].bytes_in_res) {
+		    if ((uint32)iconsize != FROM_LE_32(icondir->entries[c].bytes_in_res)) {
 				debugC(DEBUG_RESOURCE, "%s: mismatch of size in icon resource `%s' and group (%d != %d)",
-					fi->file->name(), name, iconsize, icondir->entries[c].bytes_in_res);
+					fi->file->name(), name, iconsize, FROM_LE_32(icondir->entries[c].bytes_in_res));
 		    }
-		    size += iconsize; /* size += icondir->entries[c].bytes_in_res; */
+		    size += iconsize; /* size += FROM_LE_32(icondir->entries[c].bytes_in_res); */
 
 		    /* cursor resources have two additional WORDs that contain
 		     * hotspot info */
@@ -380,7 +393,7 @@ byte *Win32ResExtractor::extract_group_icon_cursor_resource(WinLibrary *fi, WinR
 			size -= sizeof(uint16)*2;
 		}
 	}
-	offset = sizeof(Win32CursorIconFileDir) + (icondir->count-skipped) * sizeof(Win32CursorIconFileDirEntry);
+	offset = sizeof(Win32CursorIconFileDir) + (FROM_LE_16(icondir->count)-skipped) * sizeof(Win32CursorIconFileDirEntry);
 	size += offset;
 	*ressize = size;
 
@@ -391,18 +404,18 @@ byte *Win32ResExtractor::extract_group_icon_cursor_resource(WinLibrary *fi, WinR
 	/* transfer Win32CursorIconDir structure members */
 	fileicondir->reserved = icondir->reserved;
 	fileicondir->type = icondir->type;
-	fileicondir->count = icondir->count - skipped;
+	fileicondir->count = TO_LE_16(FROM_LE_16(icondir->count) - skipped);
 
 	/* transfer each cursor/icon: Win32CursorIconDirEntry and data */
 	skipped = 0;
-	for (c = 0 ; c < icondir->count ; c++) {
+	for (c = 0 ; c < FROM_LE_16(icondir->count) ; c++) {
 		int level;
 		char name[14];
 		WinResource *fwr;
 		byte *data;
 
 		/* find the corresponding icon resource */
-		snprintf(name, sizeof(name)/sizeof(char), "-%d", icondir->entries[c].res_id);
+		snprintf(name, sizeof(name)/sizeof(char), "-%d", FROM_LE_16(icondir->entries[c].res_id));
 		fwr = find_resource(fi, (is_icon ? "-3" : "-1"), name, lang, &level);
 		if (fwr == NULL) {
 			error("%s: could not find `%s' in `%s' resource.",
@@ -416,7 +429,7 @@ byte *Win32ResExtractor::extract_group_icon_cursor_resource(WinLibrary *fi, WinR
 			/* get_resource_entry has printed error */
 			return NULL;
 		}
-    	    	if (size == 0) {
+    	if (size == 0) {
 		    skipped++;
 		    continue;
 		}
@@ -428,27 +441,27 @@ byte *Win32ResExtractor::extract_group_icon_cursor_resource(WinLibrary *fi, WinR
 		/* special treatment for cursors */
 		if (!is_icon) {
 			fileicondir->entries[c-skipped].width = icondir->entries[c].res_info.cursor.width;
-			fileicondir->entries[c-skipped].height = icondir->entries[c].res_info.cursor.height / 2;
+			fileicondir->entries[c-skipped].height = TO_LE_16(FROM_LE_16(icondir->entries[c].res_info.cursor.height) / 2);
 			fileicondir->entries[c-skipped].color_count = 0;
 			fileicondir->entries[c-skipped].reserved = 0;
 		}
 
 		/* set image offset and increase it */
-		fileicondir->entries[c-skipped].dib_offset = offset;
+		fileicondir->entries[c-skipped].dib_offset = TO_LE_32(offset);
 
 		/* transfer resource into file memory */
 		if (is_icon) {
-			memcpy(&memory[offset], data, icondir->entries[c].bytes_in_res);
+			memcpy(&memory[offset], data, FROM_LE_32(icondir->entries[c].bytes_in_res));
 		} else {
 			fileicondir->entries[c-skipped].hotspot_x = ((uint16 *) data)[0];
 			fileicondir->entries[c-skipped].hotspot_y = ((uint16 *) data)[1];
 			memcpy(&memory[offset], data+sizeof(uint16)*2,
-				   icondir->entries[c].bytes_in_res-sizeof(uint16)*2);
+				   FROM_LE_32(icondir->entries[c].bytes_in_res)-sizeof(uint16)*2);
 			offset -= sizeof(uint16)*2;
 		}
 
 		/* increase the offset pointer */
-		offset += icondir->entries[c].bytes_in_res;
+		offset += FROM_LE_32(icondir->entries[c].bytes_in_res);
 	}
 
 	return memory;
@@ -477,7 +490,7 @@ bool Win32ResExtractor::check_offset(byte *memory, int total_size, const char *n
 /* do_resources:
  *   Do something for each resource matching type, name and lang.
  */
-int Win32ResExtractor::do_resources(WinLibrary *fi, const char *type, char *name, char *lang, int action, byte **data) {
+int Win32ResExtractor::do_resources(WinLibrary *fi, const char *type, char *name, char *lang, byte **data) {
 	WinResource *type_wr;
 	WinResource *name_wr;
 	WinResource *lang_wr;
@@ -487,7 +500,7 @@ int Win32ResExtractor::do_resources(WinLibrary *fi, const char *type, char *name
 	name_wr = type_wr + 1;
 	lang_wr = type_wr + 2;
 
-	size = do_resources_recurs(fi, NULL, type_wr, name_wr, lang_wr, type, name, lang, action, data);
+	size = do_resources_recurs(fi, NULL, type_wr, name_wr, lang_wr, type, name, lang, data);
 
 	free(type_wr);
 
@@ -502,7 +515,7 @@ int Win32ResExtractor::do_resources(WinLibrary *fi, const char *type, char *name
 
 int Win32ResExtractor::do_resources_recurs(WinLibrary *fi, WinResource *base,
 		  WinResource *type_wr, WinResource *name_wr, WinResource *lang_wr,
-		  const char *type, char *name, char *lang, int action, byte **data) {
+		  const char *type, char *name, char *lang, byte **data) {
 	int c, rescnt;
 	WinResource *wr;
 	uint32 size = 0;
@@ -523,7 +536,7 @@ int Win32ResExtractor::do_resources_recurs(WinLibrary *fi, WinResource *base,
 		/* go deeper unless there is something that does NOT match */
 		if (LEVEL_MATCHES(type) && LEVEL_MATCHES(name) && LEVEL_MATCHES(lang)) {
 			if (wr->is_directory)
-				size = do_resources_recurs(fi, wr+c, type_wr, name_wr, lang_wr, type, name, lang, action, data);
+				size = do_resources_recurs(fi, wr+c, type_wr, name_wr, lang_wr, type, name, lang, data);
 			else
 				size = extract_resources(fi, wr+c, type_wr, name_wr, lang_wr, data);
 		}
@@ -536,17 +549,6 @@ int Win32ResExtractor::do_resources_recurs(WinLibrary *fi, WinResource *base,
 	return size;
 }
 
-/* return the resource id quoted if it's a string, otherwise just return it */
-char *Win32ResExtractor::get_resource_id_quoted(WinResource *wr) {
-	static char tmp[WINRES_ID_MAXLEN+2];
-
-	if (wr->numeric_id || wr->id[0] == '\0')
-		return wr->id;
-
-	sprintf(tmp, "'%s'", wr->id);
-	return tmp;
-}
-
 bool Win32ResExtractor::compare_resource_id(WinResource *wr, const char *id) {
 	if (wr->numeric_id) {
 		int32 cmp1, cmp2;
@@ -576,65 +578,37 @@ bool Win32ResExtractor::decode_pe_resource_id(WinLibrary *fi, WinResource *wr, u
 
 		/* copy each char of the string, and terminate it */
 		RETURN_IF_BAD_POINTER(false, *mem);
-		len = mem[0];
+		len = FROM_LE_16(mem[0]);
 		RETURN_IF_BAD_OFFSET(false, &mem[1], sizeof(uint16) * len);
 
-		len = MIN(mem[0], (uint16)WINRES_ID_MAXLEN);
+		len = MIN(FROM_LE_16(mem[0]), (uint16)WINRES_ID_MAXLEN);
 		for (c = 0 ; c < len ; c++)
-			wr->id[c] = mem[c+1] & 0x00FF;
+			wr->id[c] = FROM_LE_16(mem[c+1]) & 0x00FF;
 		wr->id[len] = '\0';
+		wr->numeric_id = false;
 	} else {					/* Unicode string id */
 		/* translate id into a string */
 		snprintf(wr->id, WINRES_ID_MAXLEN, "%d", value);
+		wr->numeric_id = true;
 	}
 
-	wr->numeric_id = (value & IMAGE_RESOURCE_NAME_IS_STRING ? false:true);
 	return true;
 }
 
 byte *Win32ResExtractor::get_resource_entry(WinLibrary *fi, WinResource *wr, int *size) {
-	if (fi->is_PE_binary) {
-		Win32ImageResourceDataEntry *dataent;
+	byte *result;
 
-		dataent = (Win32ImageResourceDataEntry *) wr->children;
-		RETURN_IF_BAD_POINTER(NULL, *dataent);
-		*size = dataent->size;
-		RETURN_IF_BAD_OFFSET(NULL, fi->memory + dataent->offset_to_data, *size);
+	Win32ImageResourceDataEntry *dataent;
 
-		return fi->memory + dataent->offset_to_data;
-	} else {
-		Win16NENameInfo *nameinfo;
-		int sizeshift;
+	dataent = (Win32ImageResourceDataEntry *) wr->children;
+	RETURN_IF_BAD_POINTER(NULL, *dataent);
+	*size = FROM_LE_32(dataent->size);
 
-		nameinfo = (Win16NENameInfo *) wr->children;
-		sizeshift = *((uint16 *) fi->first_resource - 1);
-		*size = nameinfo->length << sizeshift;
-		RETURN_IF_BAD_OFFSET(NULL, fi->memory + (nameinfo->offset << sizeshift), *size);
+	result = fi->memory + FROM_LE_32(dataent->offset_to_data);
 
-		return fi->memory + (nameinfo->offset << sizeshift);
-	}
-}
+	RETURN_IF_BAD_OFFSET(NULL, result, *size);
 
-bool Win32ResExtractor::decode_ne_resource_id(WinLibrary *fi, WinResource *wr, uint16 value) {
-	if (value & NE_RESOURCE_NAME_IS_NUMERIC) {		/* numeric id */
-		/* translate id into a string */
-		snprintf(wr->id, WINRES_ID_MAXLEN, "%d", value & ~NE_RESOURCE_NAME_IS_NUMERIC);
-	} else {					/* ASCII string id */
-		int len;
-		char *mem = (char *)NE_HEADER(fi->memory)
-		                     + NE_HEADER(fi->memory)->rsrctab
-		                     + value;
-
-		/* copy each char of the string, and terminate it */
-		RETURN_IF_BAD_POINTER(false, *mem);
-		len = mem[0];
-		RETURN_IF_BAD_OFFSET(false, &mem[1], sizeof(char) * len);
-		memcpy(wr->id, &mem[1], len);
-		wr->id[len] = '\0';
-	}
-
-	wr->numeric_id = (value & NE_RESOURCE_NAME_IS_NUMERIC ? true:false);
-	return true;
+	return result;
 }
 
 Win32ResExtractor::WinResource *Win32ResExtractor::list_pe_resources(WinLibrary *fi, Win32ImageResourceDirectory *pe_res, int level, int *count) {
@@ -645,7 +619,7 @@ Win32ResExtractor::WinResource *Win32ResExtractor::list_pe_resources(WinLibrary
 
 	/* count number of `type' resources */
 	RETURN_IF_BAD_POINTER(NULL, *dirent);
-	rescnt = pe_res->number_of_named_entries + pe_res->number_of_id_entries;
+	rescnt = FROM_LE_16(pe_res->number_of_named_entries) + FROM_LE_16(pe_res->number_of_id_entries);
 	*count = rescnt;
 
 	/* allocate WinResource's */
@@ -656,99 +630,30 @@ Win32ResExtractor::WinResource *Win32ResExtractor::list_pe_resources(WinLibrary
 		RETURN_IF_BAD_POINTER(NULL, dirent[c]);
 		wr[c].this_ = pe_res;
 		wr[c].level = level;
-		wr[c].is_directory = (dirent[c].u2.s.data_is_directory);
-		wr[c].children = fi->first_resource + dirent[c].u2.s.offset_to_directory;
+		wr[c].is_directory = ((FROM_LE_32(dirent[c].offset_to_data) & IMAGE_RESOURCE_DATA_IS_DIRECTORY) != 0);
+		wr[c].children = fi->first_resource + (FROM_LE_32(dirent[c].offset_to_data) & ~IMAGE_RESOURCE_DATA_IS_DIRECTORY);
 
 		/* fill in wr->id, wr->numeric_id */
-		if (!decode_pe_resource_id (fi, wr + c, dirent[c].u1.name))
+		if (!decode_pe_resource_id(fi, wr + c, FROM_LE_32(dirent[c].name)))
 			return NULL;
 	}
 
 	return wr;
 }
 
-Win32ResExtractor::WinResource *Win32ResExtractor::list_ne_name_resources(WinLibrary *fi, WinResource *typeres, int *count) {
-	int c, rescnt;
-	WinResource *wr;
-	Win16NETypeInfo *typeinfo = (Win16NETypeInfo *) typeres->this_;
-	Win16NENameInfo *nameinfo = (Win16NENameInfo *) typeres->children;
-
-	/* count number of `type' resources */
-	RETURN_IF_BAD_POINTER(NULL, typeinfo->count);
-	*count = rescnt = typeinfo->count;
-
-	/* allocate WinResource's */
-	wr = (WinResource *)malloc(sizeof(WinResource) * rescnt);
-
-	/* fill in the WinResource's */
-	for (c = 0 ; c < rescnt ; c++) {
-		RETURN_IF_BAD_POINTER(NULL, nameinfo[c]);
-		wr[c].this_ = nameinfo+c;
-		wr[c].is_directory = false;
-		wr[c].children = nameinfo+c;
-		wr[c].level = 1;
-
-		/* fill in wr->id, wr->numeric_id */
-		if (!decode_ne_resource_id(fi, wr + c, (nameinfo+c)->id))
-			return NULL;
-	}
-
-	return wr;
-}
-
-Win32ResExtractor::WinResource *Win32ResExtractor::list_ne_type_resources(WinLibrary *fi, int *count) {
-	int c, rescnt;
-	WinResource *wr;
-	Win16NETypeInfo *typeinfo;
-
-	/* count number of `type' resources */
-	typeinfo = (Win16NETypeInfo *) fi->first_resource;
-	RETURN_IF_BAD_POINTER(NULL, *typeinfo);
-	for (rescnt = 0 ; typeinfo->type_id != 0 ; rescnt++) {
-		typeinfo = NE_TYPEINFO_NEXT(typeinfo);
-		RETURN_IF_BAD_POINTER(NULL, *typeinfo);
-	}
-	*count = rescnt;
-
-	/* allocate WinResource's */
-	wr = (WinResource *)malloc(sizeof(WinResource) * rescnt);
-
-	/* fill in the WinResource's */
-	typeinfo = (Win16NETypeInfo *) fi->first_resource;
-	for (c = 0 ; c < rescnt ; c++) {
-		wr[c].this_ = typeinfo;
-		wr[c].is_directory = (typeinfo->count != 0);
-		wr[c].children = typeinfo+1;
-		wr[c].level = 0;
-
-		/* fill in wr->id, wr->numeric_id */
-		if (!decode_ne_resource_id(fi, wr + c, typeinfo->type_id))
-			return NULL;
-
-		typeinfo = NE_TYPEINFO_NEXT(typeinfo);
-	}
-
-	return wr;
-}
 
 /* list_resources:
  *   Return an array of WinResource's in the current
- *   resource level specified by res.
+ *   resource level specified by _res->
  */
 Win32ResExtractor::WinResource *Win32ResExtractor::list_resources(WinLibrary *fi, WinResource *res, int *count) {
 	if (res != NULL && !res->is_directory)
 		return NULL;
 
-	if (fi->is_PE_binary) {
-		return list_pe_resources(fi, (Win32ImageResourceDirectory *)
-				 (res == NULL ? fi->first_resource : res->children),
-				 (res == NULL ? 0 : res->level+1),
-				 count);
-	} else {
-		return (res == NULL
-				? list_ne_type_resources(fi, count)
-				: list_ne_name_resources(fi, res, count));
-	}
+	return list_pe_resources(fi, (Win32ImageResourceDirectory *)
+			 (res == NULL ? fi->first_resource : res->children),
+			 (res == NULL ? 0 : res->level+1),
+			 count);
 }
 
 /* read_library:
@@ -759,44 +664,30 @@ Win32ResExtractor::WinResource *Win32ResExtractor::list_resources(WinLibrary *fi
 bool Win32ResExtractor::read_library(WinLibrary *fi) {
 	/* check for DOS header signature `MZ' */
 	RETURN_IF_BAD_POINTER(false, MZ_HEADER(fi->memory)->magic);
-	if (MZ_HEADER(fi->memory)->magic == IMAGE_DOS_SIGNATURE) {
+	if (FROM_LE_16(MZ_HEADER(fi->memory)->magic) == IMAGE_DOS_SIGNATURE) {
 		DOSImageHeader *mz_header = MZ_HEADER(fi->memory);
 
 		RETURN_IF_BAD_POINTER(false, mz_header->lfanew);
+		
+		// Apply endian fix (currently only lfanew is used from the DOSImageHeader,
+		// so we don't bother to 'fix' the rest).
+		LE32(mz_header->lfanew);
+
 		if (mz_header->lfanew < sizeof(DOSImageHeader)) {
 			error("%s: not a Windows library", fi->file->name());
 			return false;
 		}
 	}
 
-	/* check for OS2 (Win16) header signature `NE' */
-	RETURN_IF_BAD_POINTER(false, NE_HEADER(fi->memory)->magic);
-	if (NE_HEADER(fi->memory)->magic == IMAGE_OS2_SIGNATURE) {
-		OS2ImageHeader *header = NE_HEADER(fi->memory);
-
-		RETURN_IF_BAD_POINTER(false, header->rsrctab);
-		RETURN_IF_BAD_POINTER(false, header->restab);
-		if (header->rsrctab >= header->restab) {
-			error("%s: no resource directory found", fi->file->name());
-			return false;
-		}
-
-		fi->is_PE_binary = false;
-		fi->first_resource = (byte *) NE_HEADER(fi->memory)
-		  + header->rsrctab + sizeof(uint16);
-		RETURN_IF_BAD_POINTER(false, *(Win16NETypeInfo *) fi->first_resource);
-
-		return true;
-	}
-
 	/* check for NT header signature `PE' */
 	RETURN_IF_BAD_POINTER(false, PE_HEADER(fi->memory)->signature);
-	if (PE_HEADER(fi->memory)->signature == IMAGE_NT_SIGNATURE) {
-		Win32ImageSectionHeader *pe_sec;
-		Win32ImageDataDirectory *dir;
+	if (FROM_LE_32(PE_HEADER(fi->memory)->signature) == IMAGE_NT_SIGNATURE) {
 		Win32ImageNTHeaders *pe_header;
 		int d;
 
+		// Fix image header endianess
+		fix_win32_image_header_endian(PE_HEADER(fi->memory));
+
 		/* allocate new memory */
 		fi->total_size = calc_vma_size(fi);
 		if (fi->total_size == 0) {
@@ -812,32 +703,33 @@ bool Win32ResExtractor::read_library(WinLibrary *fi) {
 		/* we don't need to do OFFSET checking for the sections.
 		 * calc_vma_size has already done that */
 		for (d = pe_header->file_header.number_of_sections - 1; d >= 0 ; d--) {
-			pe_sec = PE_SECTIONS(fi->memory) + d;
+			Win32ImageSectionHeader *pe_sec = PE_SECTIONS(fi->memory) + d;
 
-	            	if (pe_sec->characteristics & IMAGE_SCN_CNT_UNINITIALIZED_DATA)
-			    continue;
+			if (pe_sec->characteristics & IMAGE_SCN_CNT_UNINITIALIZED_DATA)
+				continue;
 
-    	    	    	//if (pe_sec->virtual_address + pe_sec->size_of_raw_data > fi->total_size)
+			//if (pe_sec->virtual_address + pe_sec->size_of_raw_data > fi->total_size)
 
-    	    	    	RETURN_IF_BAD_OFFSET(0, fi->memory + pe_sec->virtual_address, pe_sec->size_of_raw_data);
-    	    	    	RETURN_IF_BAD_OFFSET(0, fi->memory + pe_sec->pointer_to_raw_data, pe_sec->size_of_raw_data);
-    	    	    	if (pe_sec->virtual_address != pe_sec->pointer_to_raw_data) {
-    	    	    	    memmove(fi->memory + pe_sec->virtual_address,
-				    fi->memory + pe_sec->pointer_to_raw_data,
-				    pe_sec->size_of_raw_data);
+			RETURN_IF_BAD_OFFSET(0, fi->memory + pe_sec->virtual_address, pe_sec->size_of_raw_data);
+			RETURN_IF_BAD_OFFSET(0, fi->memory + pe_sec->pointer_to_raw_data, pe_sec->size_of_raw_data);
+			if (FROM_LE_32(pe_sec->virtual_address) != pe_sec->pointer_to_raw_data) {
+				memmove(fi->memory + pe_sec->virtual_address,
+						fi->memory + pe_sec->pointer_to_raw_data,
+						pe_sec->size_of_raw_data);
 			}
 		}
 
 		/* find resource directory */
 		RETURN_IF_BAD_POINTER(false, pe_header->optional_header.data_directory[IMAGE_DIRECTORY_ENTRY_RESOURCE]);
-		dir = pe_header->optional_header.data_directory + IMAGE_DIRECTORY_ENTRY_RESOURCE;
+		Win32ImageDataDirectory *dir = pe_header->optional_header.data_directory + IMAGE_DIRECTORY_ENTRY_RESOURCE;
 		if (dir->size == 0) {
 			error("%s: file contains no resources", fi->file->name());
 			return false;
 		}
+		
+		fix_win32_image_data_directory(dir);
 
 		fi->first_resource = fi->memory + dir->virtual_address;
-		fi->is_PE_binary = true;
 		return true;
 	}
 
@@ -869,11 +761,12 @@ int Win32ResExtractor::calc_vma_size(WinLibrary *fi) {
 	RETURN_IF_BAD_POINTER(-1, *seg);
     for (c = 0 ; c < segcount ; c++) {
 		RETURN_IF_BAD_POINTER(0, *seg);
+		fix_win32_image_section_header(seg);
 
-        size = MAX((uint32)size, seg->virtual_address + seg->size_of_raw_data);
+		size = MAX((uint32)size, seg->virtual_address + seg->size_of_raw_data);
 		/* I have no idea what misc.virtual_size is for... */
-        size = MAX((uint32)size, seg->virtual_address + seg->misc.virtual_size);
-        seg++;
+		size = MAX((uint32)size, seg->virtual_address + seg->misc.virtual_size);
+		seg++;
     }
 
     return size;
@@ -1110,7 +1003,7 @@ int Win32ResExtractor::convertIcons(byte *data, int datasize, byte **cursor, int
 				goto cleanup;
 			} else {
 				if (entries[c].dib_offset > offset)
-						min_offset = MIN(min_offset, entries[c].dib_offset);
+					min_offset = MIN(min_offset, entries[c].dib_offset);
 			}
 		}
 
@@ -1119,6 +1012,7 @@ int Win32ResExtractor::convertIcons(byte *data, int datasize, byte **cursor, int
 				error("offset of bitmap header incorrect (too low)");
 				goto cleanup;
 			}
+			assert(min_offset != 0x7fffffff);
 			debugC(DEBUG_RESOURCE, "skipping %d bytes of garbage at %d", min_offset-offset, offset);
 			in->seek(min_offset - offset, SEEK_CUR);
 			offset = min_offset;
@@ -1155,9 +1049,6 @@ uint32 Win32ResExtractor::simple_vec(byte *data, uint32 ofs, byte size) {
 	return 0;
 }
 
-#define LE16(x)      ((x) = TO_LE_16(x))
-#define LE32(x)      ((x) = TO_LE_32(x))
-
 void Win32ResExtractor::fix_win32_cursor_icon_file_dir_endian(Win32CursorIconFileDir *obj) {
     LE16(obj->reserved);
 	LE16(obj->type);
@@ -1197,35 +1088,6 @@ void Win32ResExtractor::fix_win32_image_section_header(Win32ImageSectionHeader *
     LE32(obj->characteristics);
 }
 
-void Win32ResExtractor::fix_os2_image_header_endian(OS2ImageHeader *obj) {
-    LE16(obj->magic);
-    LE16(obj->enttab);
-    LE16(obj->cbenttab);
-    LE32(obj->crc);
-    LE16(obj->flags);
-    LE16(obj->autodata);
-    LE16(obj->heap);
-    LE16(obj->stack);
-    LE32(obj->csip);
-    LE32(obj->sssp);
-    LE16(obj->cseg);
-    LE16(obj->cmod);
-    LE16(obj->cbnrestab);
-    LE16(obj->segtab);
-    LE16(obj->rsrctab);
-    LE16(obj->restab);
-    LE16(obj->modtab);
-    LE16(obj->imptab);
-    LE32(obj->nrestab);
-    LE16(obj->cmovent);
-    LE16(obj->align);
-    LE16(obj->cres);
-    LE16(obj->fastload_offset);
-    LE16(obj->fastload_length);
-    LE16(obj->swaparea);
-    LE16(obj->expver);
-}
-
 /* fix_win32_image_header_endian:
  * NOTE: This assumes that the optional header is always available.
  */
@@ -1238,6 +1100,10 @@ void Win32ResExtractor::fix_win32_image_header_endian(Win32ImageNTHeaders *obj)
     LE32(obj->file_header.number_of_symbols);
     LE16(obj->file_header.size_of_optional_header);
     LE16(obj->file_header.characteristics);
+
+	// FIXME: Does this assert ever trigger? If so, we should modify this function
+	// to properly deal with it.
+	assert(obj->file_header.size_of_optional_header >= sizeof(obj->optional_header));
     LE16(obj->optional_header.magic);
     LE32(obj->optional_header.size_of_code);
     LE32(obj->optional_header.size_of_initialized_data);
@@ -1630,7 +1496,7 @@ void ScummEngine_v70he::readRoomsOffsets() {
 	num = READ_LE_UINT16(_heV7RoomOffsets);
 	ptr = _heV7RoomOffsets + 2;
 	for (i = 0; i < num; i++) {
-		res.roomoffs[rtRoom][i] = READ_LE_UINT32(ptr);
+		_res->roomoffs[rtRoom][i] = READ_LE_UINT32(ptr);
 		ptr += 4;
 	}
 }
