@@ -1,9 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2004-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
  *
- * Parts of code heavily based on:
- * icoutils - A set of programs dealing with MS Windows icons and cursors.
- * Copyright (C) 1998-2001 Oskar Liljeblad
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -19,13 +18,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/scumm/he/resource_he.h $
- * $Id: resource_he.h 22548 2006-05-20 15:09:15Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/scumm/he/resource_he.h $
+ * $Id: resource_he.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
-#if !defined(RESOURCE_HE_H) && !defined(DISABLE_HE)
-#define RESOURCE_HE_H
+#if !defined(SCUMM_HE_RESOURCE_HE_H) && !defined(DISABLE_HE)
+#define SCUMM_HE_RESOURCE_HE_H
 
 namespace Scumm {
 
@@ -35,17 +34,7 @@ namespace Scumm {
  * Definitions
  */
 
-#define ACTION_LIST 				1	/* command: list resources */
-#define ACTION_EXTRACT				2	/* command: extract resources */
-#define CALLBACK_STOP				0	/* results of ResourceCallback */
-#define CALLBACK_CONTINUE			1
-#define CALLBACK_CONTINUE_RECURS	2
-
 #define MZ_HEADER(x)	((DOSImageHeader *)(x))
-#define NE_HEADER(x)	((OS2ImageHeader *)PE_HEADER(x))
-#define NE_TYPEINFO_NEXT(x) ((Win16NETypeInfo *)((byte *)(x) + sizeof(Win16NETypeInfo) + \
-						    ((Win16NETypeInfo *)x)->count * sizeof(Win16NENameInfo)))
-#define NE_RESOURCE_NAME_IS_NUMERIC (0x8000)
 
 #define STRIP_RES_ID_FORMAT(x) (x != NULL && (x[0] == '-' || x[0] == '+') ? ++x : x)
 
@@ -64,18 +53,21 @@ namespace Scumm {
                            PE_HEADER(module)->file_header.size_of_optional_header))
 
 #define IMAGE_DOS_SIGNATURE    0x5A4D     /* MZ */
-#define IMAGE_OS2_SIGNATURE    0x454E     /* NE */
-#define IMAGE_OS2_SIGNATURE_LE 0x454C     /* LE */
-#define IMAGE_OS2_SIGNATURE_LX 0x584C     /* LX */
-#define IMAGE_VXD_SIGNATURE    0x454C     /* LE */
 #define IMAGE_NT_SIGNATURE     0x00004550 /* PE00 */
 
-#if !defined (WIN32)
+/* The following symbols below and another group a few lines below are defined in
+ * the windows header, at least in wince and most likely in plain win32 as well.
+ * Defining them out silences a redefinition warning in gcc.
+ * If the same problem arises in win32 builds as well, please replace
+ * _WIN32_WCE with _WIN32 which is also defined in the wince platform.
+ */
+#ifndef _WIN32_WCE
 #define IMAGE_SCN_CNT_CODE			0x00000020
 #define IMAGE_SCN_CNT_INITIALIZED_DATA		0x00000040
 #define IMAGE_SCN_CNT_UNINITIALIZED_DATA	0x00000080
 #endif
 
+// Only IMAGE_DIRECTORY_ENTRY_RESOURCE is used:
 #define	IMAGE_DIRECTORY_ENTRY_EXPORT		0
 #define	IMAGE_DIRECTORY_ENTRY_IMPORT		1
 #define	IMAGE_DIRECTORY_ENTRY_RESOURCE		2
@@ -92,7 +84,8 @@ namespace Scumm {
 #define	IMAGE_DIRECTORY_ENTRY_DELAY_IMPORT	13
 #define	IMAGE_DIRECTORY_ENTRY_COM_DESCRIPTOR	14
 
-#if !defined (WIN32)
+#ifndef _WIN32_WCE
+// Only RT_GROUP_CURSOR and RT_GROUP_ICON are used
 #define RT_CURSOR        1
 #define RT_BITMAP        2
 #define RT_ICON          3
@@ -124,10 +117,10 @@ public:
 
 	void setCursor(int id);
 
-	virtual int extractResource(int id, byte **buf) { return 0; };
+	virtual int extractResource(int id, byte **buf) { return 0; }
 	virtual int convertIcons(byte *data, int datasize, byte **cursor, int *w, int *h,
 							 int *hotspot_x, int *hotspot_y, int *keycolor,
-							 byte **palette, int *palSize) { return 0; };
+							 byte **palette, int *palSize) { return 0; }
 
 	enum {
 		MAX_CACHED_CURSORS = 10
@@ -160,7 +153,7 @@ public:
 class Win32ResExtractor : public ResExtractor {
  public:
 	Win32ResExtractor(ScummEngine_v70he *scumm);
-	~Win32ResExtractor() {};
+	~Win32ResExtractor() {}
 	int extractResource(int id, byte **data);
 	void setCursor(int id);
 	int convertIcons(byte *data, int datasize, byte **cursor, int *w, int *h,
@@ -172,15 +165,12 @@ class Win32ResExtractor : public ResExtractor {
  * Structures
  */
 
-#if !defined(__GNUC__)
-	#pragma START_PACK_STRUCTS
-#endif
+#include "common/pack-start.h"	// START STRUCT PACKING
 
 	struct WinLibrary {
 		Common::File *file;
 		byte *memory;
 		byte *first_resource;
-		bool is_PE_binary;
 		int total_size;
 	};
 
@@ -191,6 +181,8 @@ class Win32ResExtractor : public ResExtractor {
 		int level;
 		bool numeric_id;
 		bool is_directory;
+		
+		char *get_resource_id_quoted();
 	};
 
 
@@ -221,7 +213,7 @@ class Win32ResExtractor : public ResExtractor {
 		uint16 reserved;
 		uint16 type;
 		uint16 count;
-		Win32CursorIconDirEntry entries[1] GCC_PACK;
+		Win32CursorIconDirEntry entries[1];
 	};
 
 	struct Win32CursorIconFileDirEntry {
@@ -264,39 +256,8 @@ class Win32ResExtractor : public ResExtractor {
 	};
 
 	struct Win32ImageResourceDirectoryEntry {
-		union {
-			struct {
-				#ifdef SCUMM_BIGENDIAN
-				unsigned name_is_string:1;
-				unsigned name_offset:31;
-	    	    #else
-				unsigned name_offset:31;
-				unsigned name_is_string:1;
-	    	    #endif
-			} s1;
-			uint32 name;
-			struct {
-	    	    #ifdef SCUMM_BIG_ENDIAN
-				uint16 __pad;
-				uint16 id;
-	    	    #else
-				uint16 id;
-				uint16 __pad;
-	    	    #endif
-			} s2;
-		} u1;
-		union {
-			uint32 offset_to_data;
-			struct {
-	    	    #ifdef SCUMM_BIG_ENDIAN
-				unsigned data_is_directory:1;
-				unsigned offset_to_directory:31;
-	    	    #else
-				unsigned offset_to_directory:31;
-				unsigned data_is_directory:1;
-	    	    #endif
-			} s;
-		} u2;
+		uint32 name;
+		uint32 offset_to_data;
 	};
 
 	struct Win16NETypeInfo {
@@ -305,48 +266,6 @@ class Win32ResExtractor : public ResExtractor {
 		uint32 resloader;     // FARPROC16 - smaller? uint16?
 	};
 
-	struct Win16NENameInfo {
-		uint16 offset;
-		uint16 length;
-		uint16 flags;
-		uint16 id;
-		uint16 handle;
-		uint16 usage;
-	};
-
-	struct OS2ImageHeader {
-		uint16 magic;
-		byte ver;
-		byte rev;
-		uint16 enttab;
-		uint16 cbenttab;
-		int32 crc;
-		uint16 flags;
-		uint16 autodata;
-		uint16 heap;
-		uint16 stack;
-		uint32 csip;
-		uint32 sssp;
-		uint16 cseg;
-		uint16 cmod;
-		uint16 cbnrestab;
-		uint16 segtab;
-		uint16 rsrctab;
-		uint16 restab;
-		uint16 modtab;
-		uint16 imptab;
-		uint32 nrestab;
-		uint16 cmovent;
-		uint16 align;
-		uint16 cres;
-		byte exetyp;
-		byte flagsothers;
-		uint16 fastload_offset;
-		uint16 fastload_length;
-		uint16 swaparea;
-		uint16 expver;
-	};
-
 	struct DOSImageHeader {
 		uint16 magic;
 		uint16 cblp;
@@ -456,9 +375,7 @@ class Win32ResExtractor : public ResExtractor {
 		uint16 number_of_id_entries;
 	};
 
-#if !defined(__GNUC__)
-	#pragma END_PACK_STRUCTS
-#endif
+#include "common/pack-end.h"	// END STRUCT PACKING
 
 /*
  * Function Prototypes
@@ -468,7 +385,7 @@ class Win32ResExtractor : public ResExtractor {
 	bool read_library(WinLibrary *);
 	WinResource *find_resource(WinLibrary *, const char *, const char *, const char *, int *);
 	byte *get_resource_entry(WinLibrary *, WinResource *, int *);
-	int do_resources(WinLibrary *, const char *, char *, char *, int, byte **);
+	int do_resources(WinLibrary *, const char *, char *, char *, byte **);
 	bool compare_resource_id(WinResource *, const char *);
 	const char *res_type_string_to_id(const char *);
 
@@ -480,13 +397,9 @@ class Win32ResExtractor : public ResExtractor {
 	byte *extract_group_icon_cursor_resource(WinLibrary *, WinResource *, char *, int *, bool);
 
 	bool decode_pe_resource_id(WinLibrary *, WinResource *, uint32);
-	bool decode_ne_resource_id(WinLibrary *, WinResource *, uint16);
-	WinResource *list_ne_type_resources(WinLibrary *, int *);
-	WinResource *list_ne_name_resources(WinLibrary *, WinResource *, int *);
 	WinResource *list_pe_resources(WinLibrary *, Win32ImageResourceDirectory *, int, int *);
 	int calc_vma_size(WinLibrary *);
-	int do_resources_recurs(WinLibrary *, WinResource *, WinResource *, WinResource *, WinResource *, const char *, char *, char *, int, byte **);
-	char *get_resource_id_quoted(WinResource *);
+	int do_resources_recurs(WinLibrary *, WinResource *, WinResource *, WinResource *, WinResource *, const char *, char *, char *, byte **);
 	WinResource *find_with_resource_array(WinLibrary *, WinResource *, const char *);
 
 	bool check_offset(byte *, int, const char *, void *, int);
@@ -497,7 +410,6 @@ class Win32ResExtractor : public ResExtractor {
 	void fix_win32_bitmap_info_header_endian(Win32BitmapInfoHeader *obj);
 	void fix_win32_cursor_icon_file_dir_entry_endian(Win32CursorIconFileDirEntry *obj);
 	void fix_win32_image_section_header(Win32ImageSectionHeader *obj);
-	void fix_os2_image_header_endian(OS2ImageHeader *obj);
 	void fix_win32_image_header_endian(Win32ImageNTHeaders *obj);
 	void fix_win32_image_data_directory(Win32ImageDataDirectory *obj);
 };
