@@ -1,6 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2001  Ludvig Strigeus
- * Copyright (C) 2001-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -16,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/scumm/he/script_v100he.cpp $
- * $Id: script_v100he.cpp 23472 2006-07-10 10:16:24Z kirben $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/scumm/he/script_v100he.cpp $
+ * $Id: script_v100he.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -27,6 +29,7 @@
 
 #include "scumm/actor.h"
 #include "scumm/charset.h"
+#include "scumm/he/animation_he.h"
 #include "scumm/he/intern_he.h"
 #include "scumm/object.h"
 #include "scumm/resource.h"
@@ -70,7 +73,7 @@ void ScummEngine_v100he::setupOpcodes() {
 		/* 14 */
 		OPCODE(o6_loadRoomWithEgo),
 		OPCODE(o6_invalid),
-		OPCODE(o72_setFilePath),
+		OPCODE(o72_createDirectory),
 		OPCODE(o100_createSound),
 		/* 18 */
 		OPCODE(o6_cutscene),
@@ -259,7 +262,7 @@ void ScummEngine_v100he::setupOpcodes() {
 		OPCODE(o72_findAllObjects),
 		/* AC */
 		OPCODE(o90_findAllObjectsWithClassOf),
-		OPCODE(o6_invalid),
+		OPCODE(o70_findBox),
 		OPCODE(o6_findInventory),
 		OPCODE(o72_findObject),
 		/* B0 */
@@ -409,7 +412,7 @@ void ScummEngine_v100he::o100_actorOps() {
 	case 6:
 		j = pop();
 		i = pop();
-		a->putActor(i, j, a->_room);
+		a->putActor(i, j);
 		break;
 	case 8:
 		a->_drawToBackBuf = false;
@@ -417,7 +420,7 @@ void ScummEngine_v100he::o100_actorOps() {
 		a->_needBgReset = true;
 		break;
 	case 9:
-		a->drawActorToBackBuf(a->_pos.x, a->_pos.y);
+		a->drawActorToBackBuf(a->getPos().x, a->getPos().y);
 		break;
 	case 14:
 		a->_charset = pop();
@@ -455,7 +458,7 @@ void ScummEngine_v100he::o100_actorOps() {
 	case 57:		// SO_PALETTE
 		j = pop();
 		i = pop();
-		checkRange(255, 0, i, "o100_actorOps: Illegal palette slot %d");
+		assertRange(0, i, 255, "palette slot");
 		a->remapActorPaletteColor(i, j);
 		a->_needRedraw = true;
 		break;
@@ -533,13 +536,13 @@ void ScummEngine_v100he::o100_actorOps() {
 		a->_ignoreBoxes = 0;
 		a->_forceClip = 0;
 		if (a->isInCurrentRoom())
-			a->putActor(a->_pos.x, a->_pos.y, a->_room);
+			a->putActor();
 		break;
 	case 135:		// SO_IGNORE_BOXES
 		a->_ignoreBoxes = 1;
 		a->_forceClip = 0;
 		if (a->isInCurrentRoom())
-			a->putActor(a->_pos.x, a->_pos.y, a->_room);
+			a->putActor();
 		break;
 	case 136:		// SO_ACTOR_IGNORE_TURNS_OFF
 		a->_ignoreTurns = false;
@@ -580,7 +583,7 @@ void ScummEngine_v100he::o100_actorOps() {
 }
 
 void ScummEngine_v100he::o100_arrayOps() {
-	ArrayHeader *ah;
+	byte *data;
 	byte string[1024];
 	int dim1end, dim1start, dim2end, dim2start;
 	int id, len, b, c, list[128];
@@ -595,14 +598,14 @@ void ScummEngine_v100he::o100_arrayOps() {
 	case 35:
 		decodeScriptString(string);
 		len = resStrLen(string);
-		ah = defineArray(array, kStringArray, 0, 0, 0, len);
-		memcpy(ah->data, string, len);
+		data = defineArray(array, kStringArray, 0, 0, 0, len);
+		memcpy(data, string, len);
 		break;
 	case 77:			// SO_ASSIGN_STRING
 		copyScriptString(string, sizeof(string));
 		len = resStrLen(string);
-		ah = defineArray(array, kStringArray, 0, 0, 0, len);
-		memcpy(ah->data, string, len);
+		data = defineArray(array, kStringArray, 0, 0, 0, len);
+		memcpy(data, string, len);
 		break;
 
 	case 128:		// SO_ASSIGN_2DIM_LIST
@@ -1122,16 +1125,16 @@ void ScummEngine_v100he::o100_resourceRoutines() {
 			objidx = getObjectIndex(_heResId);
 			if (objidx == -1)
 				break;
-			res.lock(rtFlObject, _objs[objidx].fl_object_index);
+			_res->lock(rtFlObject, _objs[objidx].fl_object_index);
 		} else {
-			res.lock(_heResType, _heResId);
+			_res->lock(_heResType, _heResId);
 		}
 		break;
 	case 133:
 		if (_heResType == rtCharset)
 			nukeCharset(_heResId);
 		else
-			res.nukeResource(_heResType, _heResId);
+			_res->nukeResource(_heResType, _heResId);
 		break;
 	case 134:
 	case 135:
@@ -1151,9 +1154,9 @@ void ScummEngine_v100he::o100_resourceRoutines() {
 			objidx = getObjectIndex(_heResId);
 			if (objidx == -1)
 				break;
-			res.unlock(rtFlObject, _objs[objidx].fl_object_index);
+			_res->unlock(rtFlObject, _objs[objidx].fl_object_index);
 		} else {
-			res.unlock(_heResType, _heResId);
+			_res->unlock(_heResType, _heResId);
 		}
 		break;
 	default:
@@ -1208,6 +1211,7 @@ void ScummEngine_v100he::o100_wizImageOps() {
 		_wizParams.box.right = pop();
 		_wizParams.box.top = pop();
 		_wizParams.box.left = pop();
+		adjustRect(_wizParams.box);
 		break;
 	case 21:
 		b = pop();
@@ -1615,7 +1619,7 @@ void ScummEngine_v100he::o100_roomOps() {
 
 	case 134:		// SO_ROOM_NEW_PALETTE
 		a = pop();
-		setPalette(a);
+		setCurrentPalette(a);
 		break;
 
 	case 135:
@@ -1967,7 +1971,7 @@ void ScummEngine_v100he::o100_setSpriteInfo() {
 			spriteId++;
 
 		for (; spriteId <= _curMaxSpriteId; spriteId++)
-			switch(args[1]) {
+			switch (args[1]) {
 			case 0:
 				_sprite->setSpriteFlagXFlipped(spriteId, args[0]);
 				break;
@@ -2150,7 +2154,7 @@ void ScummEngine_v100he::o100_systemOps() {
 		debug(0, "Start executable (%s)", string);
 		break;
 	case 75:
-		gdi.copyVirtScreenBuffers(Common::Rect(_screenWidth, _screenHeight));
+		restoreBackgroundHE(Common::Rect(_screenWidth, _screenHeight));
 		updatePalette();
 		break;
 	default:
@@ -2247,16 +2251,15 @@ void ScummEngine_v100he::o100_videoOps() {
 			if (_videoParams.flags == 0)
 				_videoParams.flags = 4;
 
-			//const char *filename = (char *)_videoParams.filename + convertFilePath(_videoParams.filename);
+			const char *filename = (char *)_videoParams.filename + convertFilePath(_videoParams.filename);
 			if (_videoParams.flags == 2) {
-				// VAR(119) = startVideo(_videoParams.filename, _videoParams.flags, _videoParams.wizResNum);
-				VAR(119) = -1;
+				VAR(119) = _moviePlay->load(filename, _videoParams.flags, _videoParams.wizResNum);
 			} else {
-				// VAR(119) = startVideo(_videoParams.filename, _videoParams.flags);
-				VAR(119) = -1;
+				VAR(119) = _moviePlay->load(filename, _videoParams.flags);
 			}
 		} else if (_videoParams.status == 19) {
 			// Stop video
+			_moviePlay->closeFile();
 		}
 		break;
 	default:
@@ -2433,7 +2436,7 @@ void ScummEngine_v100he::o100_getSpriteGroupInfo() {
 		type = pop();
 		spriteGroupId = pop();
 		if (spriteGroupId) {
-			switch(type) {
+			switch (type) {
 			case 0:
 				push(_sprite->getGroupXMul(spriteGroupId));
 				break;
@@ -2774,7 +2777,7 @@ void ScummEngine_v100he::o100_getSpriteInfo() {
 		flags = pop();
 		spriteId = pop();
 		if (spriteId) {
-			switch(flags) {
+			switch (flags) {
 			case 0:
 				push(_sprite->getSpriteFlagXFlipped(spriteId));
 				break;
@@ -2891,29 +2894,31 @@ void ScummEngine_v100he::o100_getVideoData() {
 	switch (subOp) {
 	case 0:
 		pop();
+		push(_moviePlay->getFrameCount());
 		break;
 	case 13:
 		pop();
+		push(_moviePlay->getHeight());
 		break;
 	case 14:
 		pop();
+		push(_moviePlay->getImageNum());
 		break;
 	case 28:
-		pop();
-		pop();
+		debug(0, "o100_getVideoData: subOp 28 stub (%d, %d)", pop(), pop());
+		push(0);
 		break;
 	case 47:
 		pop();
+		push(_moviePlay->getCurFrame());
 		break;
 	case 58:
 		pop();
+		push(_moviePlay->getWidth());
 		break;
 	default:
 		error("o100_getVideoData: unhandled case %d", subOp);
 	}
-
-	push(-1);
-	debug(1,"o100_getVideoData stub (%d)", subOp);
 }
 
 void ScummEngine_v100he::decodeParseString(int m, int n) {
