@@ -1,6 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2001  Ludvig Strigeus
- * Copyright (C) 2001-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -16,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/scumm/he/script_v60he.cpp $
- * $Id: script_v60he.cpp 23215 2006-06-21 06:58:22Z kirben $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/scumm/he/script_v60he.cpp $
+ * $Id: script_v60he.cpp 27126 2007-06-06 03:38:57Z Kirben $
  *
  */
 
@@ -279,7 +281,7 @@ void ScummEngine_v60he::setupOpcodes() {
 		OPCODE(o6_getActorWidth),
 		OPCODE(o6_wait),
 		OPCODE(o6_getActorScaleX),
-		OPCODE(o6_getActorAnimCounter1),
+		OPCODE(o6_getActorAnimCounter),
 		/* AC */
 		OPCODE(o6_invalid),
 		OPCODE(o6_isAnyOf),
@@ -403,16 +405,12 @@ int ScummEngine_v60he::convertFilePath(byte *dst) {
 	debug(1, "convertFilePath: original filePath is %s", dst);
 
 	int len = resStrLen(dst);
-	if (dst[0] == ':') {
+	if (_game.platform == Common::kPlatformMacintosh) {
 		// Switch all : to / for portablity
-		int j = 0;
-		for (int i = 1; i < len; i++) {
+		for (int i = 0; i < len; i++) {
 			if (dst[i] == ':')
-				dst[j++] = '/';
-			else
-				dst[j++] = dst[i];
+				dst[i] = '/';
 		}
-		dst[j] = 0;
 	} else {
 		// Switch all \ to / for portablity
 		for (int i = 0; i < len; i++) {
@@ -423,9 +421,11 @@ int ScummEngine_v60he::convertFilePath(byte *dst) {
 
 	// Strip path
 	int r = 0;
-	if (dst[0] == '.' && dst[1] == '/') {
+	if (dst[0] == '.' && dst[1] == '/') { // Game Data Path
 		r = 2;
-	} else if (dst[0] == 'c' && dst[1] == ':') {
+	} else if (dst[0] == '*' && dst[1] == '/') { // Save Game Path (HE72 - HE100)
+		r = 2;
+	} else if (dst[0] == 'c' && dst[1] == ':') { // Save Game Path (HE60 - HE71)
 		for (r = len; r != 0; r--) {
 			if (dst[r - 1] == '/')
 				break;
@@ -554,13 +554,13 @@ void ScummEngine_v60he::o60_roomOps() {
 	case 187:		// SO_CYCLE_SPEED
 		b = pop();
 		a = pop();
-		checkRange(16, 1, a, "o60_roomOps: 187: color cycle out of range (%d)");
+		assertRange(1, a, 16, "o60_roomOps: 187: color cycle");
 		_colorCycle[a - 1].delay = (b != 0) ? 0x4000 / (b * 0x4C) : 0;
 		break;
 
 	case 213:		// SO_ROOM_NEW_PALETTE
 		a = pop();
-		setPalette(a);
+		setCurrentPalette(a);
 		break;
 	case 220:
 		a = pop();
@@ -682,7 +682,7 @@ void ScummEngine_v60he::o60_actorOps() {
 	case 86:		// SO_PALETTE
 		j = pop();
 		i = pop();
-		checkRange(255, 0, i, "Illegal palette slot %d");
+		assertRange(0, i, 255, "o60_actorOps: palette slot");
 		a->remapActorPaletteColor(i, j);
 		a->_needRedraw = true;
 		break;
@@ -712,13 +712,13 @@ void ScummEngine_v60he::o60_actorOps() {
 		a->_ignoreBoxes = 1;
 		a->_forceClip = 0;
 		if (a->isInCurrentRoom())
-			a->putActor(a->_pos.x, a->_pos.y, a->_room);
+			a->putActor();
 		break;
 	case 96:		// SO_FOLLOW_BOXES
 		a->_ignoreBoxes = 0;
 		a->_forceClip = 0;
 		if (a->isInCurrentRoom())
-			a->putActor(a->_pos.x, a->_pos.y, a->_room);
+			a->putActor();
 		break;
 	case 97:		// SO_ANIMATION_SPEED
 		a->setAnimSpeed(pop());
@@ -748,7 +748,7 @@ void ScummEngine_v60he::o60_actorOps() {
 		a->initActor(2);
 		break;
 	case 218:
-		a->drawActorToBackBuf(a->_pos.x, a->_pos.y);
+		a->drawActorToBackBuf(a->getPos().x, a->getPos().y);
 		break;
 	case 219:
 		a->_drawToBackBuf = false;
@@ -856,7 +856,7 @@ uint8 virtScreenLoadUnpack(vsUnpackCtx *ctx, byte *data) {
 
 void ScummEngine_v60he::o60_kernelGetFunctions() {
 	int args[29];
-	ArrayHeader *ah;
+	byte *data;
 	getStackList(args, ARRAYSIZE(args));
 
 	switch (args[0]) {
@@ -864,8 +864,8 @@ void ScummEngine_v60he::o60_kernelGetFunctions() {
 		// Used to store images when decorating cake in
 		// Fatty Bear's Birthday Surprise
 		writeVar(0, 0);
-		ah = defineArray(0, kByteArray, 0, virtScreenSave(0, args[1], args[2], args[3], args[4]));
-		virtScreenSave(ah->data, args[1], args[2], args[3], args[4]);
+		data = defineArray(0, kByteArray, 0, virtScreenSave(0, args[1], args[2], args[3], args[4]));
+		virtScreenSave(data, args[1], args[2], args[3], args[4]);
 		push(readVar(0));
 		break;
 	default:
@@ -999,7 +999,7 @@ void ScummEngine_v60he::o60_openFile() {
 	}
 
 	if (slot != -1) {
-		switch(mode) {
+		switch (mode) {
 		case 1:
 			// TODO / FIXME: Consider using listSavefiles to avoid unneccessary openForLoading calls
 			_hInFileTable[slot] = _saveFileMan->openForLoading(filename);
@@ -1078,8 +1078,8 @@ int ScummEngine_v60he::readFileToArray(int slot, int32 size) {
 		size = _hInFileTable[slot]->size() - _hInFileTable[slot]->pos();
 
 	writeVar(0, 0);
-	ArrayHeader *ah = defineArray(0, kByteArray, 0, size);
-	_hInFileTable[slot]->read(ah->data, size);
+	byte *data = defineArray(0, kByteArray, 0, size);
+	_hInFileTable[slot]->read(data, size);
 
 	return readVar(0);
 }
@@ -1090,7 +1090,7 @@ void ScummEngine_v60he::o60_readFile() {
 	int val;
 
 	// Fatty Bear uses positive values
-	if ((_game.platform == Common::kPlatformPC) && (_game.id == GID_FBEAR))
+	if (_game.platform == Common::kPlatformPC && _game.id == GID_FBEAR)
 		size = -size;
 
 	assert(_hInFileTable[slot]);
@@ -1120,7 +1120,7 @@ void ScummEngine_v60he::o60_writeFile() {
 	int slot = pop();
 
 	// Fatty Bear uses positive values
-	if ((_game.platform == Common::kPlatformPC) && (_game.id == GID_FBEAR))
+	if (_game.platform == Common::kPlatformPC && _game.id == GID_FBEAR)
 		size = -size;
 
 	assert(_hOutFileTable[slot]);
@@ -1138,13 +1138,15 @@ void ScummEngine_v60he::o60_soundOps() {
 	int arg = pop();
 
 	switch (subOp) {
-	case 0xde:
-		_imuse->setMusicVolume(arg);
+	case 222:
+		if (_imuse) {
+			_imuse->setMusicVolume(arg);
+		}
 		break;
-	case 0xdf:
-		// Used in fbear introduction
+	case 223:
+		// WORKAROUND: For error in room script 228 (room 2) of fbear.
 		break;
-	case 0xe0:
+	case 224:
 		// Fatty Bear's Birthday surprise uses this when playing the
 		// piano, but only when using one of the digitized instruments.
 		// See also o6_startSound().
