@@ -1,6 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2001  Ludvig Strigeus
- * Copyright (C) 2001-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -16,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/scumm/he/script_v70he.cpp $
- * $Id: script_v70he.cpp 23147 2006-06-16 13:16:11Z kirben $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/scumm/he/script_v70he.cpp $
+ * $Id: script_v70he.cpp 27036 2007-06-01 08:47:22Z Kirben $
  *
  */
 
@@ -256,7 +258,7 @@ void ScummEngine_v70he::setupOpcodes() {
 		OPCODE(o6_getActorWidth),
 		OPCODE(o6_wait),
 		OPCODE(o6_getActorScaleX),
-		OPCODE(o6_getActorAnimCounter1),
+		OPCODE(o6_getActorAnimCounter),
 		/* AC */
 		OPCODE(o6_invalid),
 		OPCODE(o6_isAnyOf),
@@ -351,10 +353,10 @@ void ScummEngine_v70he::setupOpcodes() {
 		OPCODE(o70_writeINI),
 		OPCODE(o70_getStringLenForWidth),
 		OPCODE(o70_getCharIndexInString),
-		OPCODE(o6_invalid),
+		OPCODE(o70_findBox),
 		/* F8 */
 		OPCODE(o6_invalid),
-		OPCODE(o70_setFilePath),
+		OPCODE(o70_createDirectory),
 		OPCODE(o70_setSystemMessage),
 		OPCODE(o70_polygonOps),
 		/* FC */
@@ -459,6 +461,9 @@ void ScummEngine_v70he::o70_startSound() {
 	case 164:
 		_heSndFlags |= 2;
 		break;
+	case 222:
+		// WORKAROUND: For errors in room script 240 (room 4) of maze
+		break;
 	case 224:
 		_heSndSoundFreq = pop();
 		break;
@@ -541,62 +546,62 @@ void ScummEngine_v70he::o70_resourceRoutines() {
 		break;
 	case 104:		// SO_NUKE_SCRIPT
 		resid = pop();
-		res.nukeResource(rtScript, resid);
+		_res->nukeResource(rtScript, resid);
 		break;
 	case 105:		// SO_NUKE_SOUND
 		resid = pop();
-		res.nukeResource(rtSound, resid);
+		_res->nukeResource(rtSound, resid);
 		break;
 	case 106:		// SO_NUKE_COSTUME
 		resid = pop();
-		res.nukeResource(rtCostume, resid);
+		_res->nukeResource(rtCostume, resid);
 		break;
 	case 107:		// SO_NUKE_ROOM
 		resid = pop();
-		res.nukeResource(rtRoom, resid);
-		res.nukeResource(rtRoomImage, resid);
+		_res->nukeResource(rtRoom, resid);
+		_res->nukeResource(rtRoomImage, resid);
 		break;
 	case 108:		// SO_LOCK_SCRIPT
 		resid = pop();
 		if (resid >= _numGlobalScripts)
 			break;
-		res.lock(rtScript, resid);
+		_res->lock(rtScript, resid);
 		break;
 	case 109:		// SO_LOCK_SOUND
 		resid = pop();
-		res.lock(rtSound, resid);
+		_res->lock(rtSound, resid);
 		break;
 	case 110:		// SO_LOCK_COSTUME
 		resid = pop();
-		res.lock(rtCostume, resid);
+		_res->lock(rtCostume, resid);
 		break;
 	case 111:		// SO_LOCK_ROOM
 		resid = pop();
 		if (_game.heversion <= 71 && resid > 0x7F)
 			resid = _resourceMapper[resid & 0x7F];
-		res.lock(rtRoom, resid);
-		res.lock(rtRoomImage, resid);
+		_res->lock(rtRoom, resid);
+		_res->lock(rtRoomImage, resid);
 		break;
 	case 112:		// SO_UNLOCK_SCRIPT
 		resid = pop();
 		if (resid >= _numGlobalScripts)
 			break;
-		res.unlock(rtScript, resid);
+		_res->unlock(rtScript, resid);
 		break;
 	case 113:		// SO_UNLOCK_SOUND
 		resid = pop();
-		res.unlock(rtSound, resid);
+		_res->unlock(rtSound, resid);
 		break;
 	case 114:		// SO_UNLOCK_COSTUME
 		resid = pop();
-		res.unlock(rtCostume, resid);
+		_res->unlock(rtCostume, resid);
 		break;
 	case 115:		// SO_UNLOCK_ROOM
 		resid = pop();
 		if (_game.heversion <= 71 && resid > 0x7F)
 			resid = _resourceMapper[resid & 0x7F];
-		res.unlock(rtRoom, resid);
-		res.unlock(rtRoomImage, resid);
+		_res->unlock(rtRoom, resid);
+		_res->unlock(rtRoomImage, resid);
 		break;
 	case 116:
 		break;
@@ -635,11 +640,11 @@ void ScummEngine_v70he::o70_resourceRoutines() {
 		break;
 	case 159:
 		resid = pop();
-		res.unlock(rtImage, resid);
+		_res->unlock(rtImage, resid);
 		break;
 	case 192:
 		resid = pop();
-		res.nukeResource(rtImage, resid);
+		_res->nukeResource(rtImage, resid);
 		break;
 	case 201:
 		resid = pop();
@@ -647,7 +652,7 @@ void ScummEngine_v70he::o70_resourceRoutines() {
 		break;
 	case 202:
 		resid = pop();
-		res.lock(rtImage, resid);
+		_res->lock(rtImage, resid);
 		break;
 	case 203:
 		resid = pop();
@@ -658,14 +663,14 @@ void ScummEngine_v70he::o70_resourceRoutines() {
 		objidx = getObjectIndex(resid);
 		if (objidx == -1)
 			break;
-		res.lock(rtFlObject, _objs[objidx].fl_object_index);
+		_res->lock(rtFlObject, _objs[objidx].fl_object_index);
 		break;
 	case 235:
 		resid = pop();
 		objidx = getObjectIndex(resid);
 		if (objidx == -1)
 			break;
-		res.unlock(rtFlObject, _objs[objidx].fl_object_index);
+		_res->unlock(rtFlObject, _objs[objidx].fl_object_index);
 		break;
 	case 239:
 		// Used in airport
@@ -784,7 +789,7 @@ void ScummEngine_v70he::o70_kernelSetFunctions() {
 		_skipDrawObject = 0;
 		break;
 	case 23:
-		_charset->clearCharsetMask();
+		clearCharsetMask();
 		_fullRedraw = true;
 		break;
 	case 24:
@@ -927,7 +932,7 @@ void ScummEngine_v70he::o70_isResourceLoaded() {
 
 void ScummEngine_v70he::o70_readINI() {
 	byte option[256];
-	ArrayHeader *ah;
+	byte *data;
 	const char *entry;
 	int len, type;
 
@@ -951,8 +956,8 @@ void ScummEngine_v70he::o70_readINI() {
 
 		writeVar(0, 0);
 		len = resStrLen((const byte *)entry);
-		ah = defineArray(0, kStringArray, 0, len);
-		memcpy(ah->data, entry, len);
+		data = defineArray(0, kStringArray, 0, len);
+		memcpy(data, entry, len);
 
 		push(readVar(0));
 		break;
@@ -1056,15 +1061,21 @@ void ScummEngine_v70he::o70_getCharIndexInString() {
 	push(-1);
 }
 
-void ScummEngine_v70he::o70_setFilePath() {
-	int len;
-	byte filename[100];
+void ScummEngine_v70he::o70_findBox() {
+	int y = pop();
+	int x = pop();
+	push(getSpecialBox(x, y));
+}
 
-	convertMessageToString(_scriptPointer, filename, sizeof(filename));
+void ScummEngine_v70he::o70_createDirectory() {
+	int len;
+	byte directoryName[100];
+
+	convertMessageToString(_scriptPointer, directoryName, sizeof(directoryName));
 	len = resStrLen(_scriptPointer);
 	_scriptPointer += len + 1;
 
-	debug(1,"stub o70_setFilePath(%s)", filename);
+	debug(1,"stub o70_createDirectory(%s)", directoryName);
 }
 
 void ScummEngine_v70he::o70_setSystemMessage() {
