@@ -1,6 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2001  Ludvig Strigeus
- * Copyright (C) 2001-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -16,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/scumm/he/script_v72he.cpp $
- * $Id: script_v72he.cpp 23472 2006-07-10 10:16:24Z kirben $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/scumm/he/script_v72he.cpp $
+ * $Id: script_v72he.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -259,7 +261,7 @@ void ScummEngine_v72he::setupOpcodes() {
 		OPCODE(o6_getActorWidth),
 		OPCODE(o6_wait),
 		OPCODE(o6_getActorScaleX),
-		OPCODE(o6_getActorAnimCounter1),
+		OPCODE(o6_getActorAnimCounter),
 		/* AC */
 		OPCODE(o6_invalid),
 		OPCODE(o6_isAnyOf),
@@ -354,10 +356,10 @@ void ScummEngine_v72he::setupOpcodes() {
 		OPCODE(o72_writeINI),
 		OPCODE(o70_getStringLenForWidth),
 		OPCODE(o70_getCharIndexInString),
-		OPCODE(o6_invalid),
+		OPCODE(o70_findBox),
 		/* F8 */
 		OPCODE(o72_getResourceSize),
-		OPCODE(o72_setFilePath),
+		OPCODE(o72_createDirectory),
 		OPCODE(o72_setSystemMessage),
 		OPCODE(o70_polygonOps),
 		/* FC */
@@ -381,7 +383,7 @@ const char *ScummEngine_v72he::getOpcodeDesc(byte i) {
 
 static const int arrayDataSizes[] = { 0, 1, 4, 8, 8, 16, 32 };
 
-ScummEngine_v72he::ArrayHeader *ScummEngine_v72he::defineArray(int array, int type, int dim2start, int dim2end,
+byte *ScummEngine_v72he::defineArray(int array, int type, int dim2start, int dim2end,
 											int dim1start, int dim1end) {
 	int id;
 	int size;
@@ -419,7 +421,7 @@ ScummEngine_v72he::ArrayHeader *ScummEngine_v72he::defineArray(int array, int ty
 	size *= dim1end - dim1start + 1;
 	size >>= 3;
 
-	ah = (ArrayHeader *)res.createResource(rtString, id, size + sizeof(ArrayHeader));
+	ah = (ArrayHeader *)_res->createResource(rtString, id, size + sizeof(ArrayHeader));
 
 	ah->type = TO_LE_32(type);
 	ah->dim1start = TO_LE_32(dim1start);
@@ -427,7 +429,7 @@ ScummEngine_v72he::ArrayHeader *ScummEngine_v72he::defineArray(int array, int ty
 	ah->dim2start = TO_LE_32(dim2start);
 	ah->dim2end = TO_LE_32(dim2end);
 
-	return ah;
+	return ah->data;
 }
 
 int ScummEngine_v72he::readArray(int array, int idx2, int idx1) {
@@ -594,7 +596,7 @@ void ScummEngine_v72he::decodeScriptString(byte *dst, bool scriptString) {
 		chr = string[num++];
 		if (chr == '%') {
 			chr = string[num++];
-			switch(chr) {
+			switch (chr) {
 			case 'b':
 				//dst += sprintf((char *)dst, "%b", args[val++]);
 				break;
@@ -962,7 +964,7 @@ void ScummEngine_v72he::o72_getArrayDimSize() {
 }
 
 void ScummEngine_v72he::o72_getNumFreeArrays() {
-	byte **addr = res.address[rtString];
+	byte **addr = _res->address[rtString];
 	int i, num = 0;
 
 	for (i = 1; i < _numArray; i++) {
@@ -1038,7 +1040,7 @@ void ScummEngine_v72he::o72_roomOps() {
 
 	case 213:		// SO_ROOM_NEW_PALETTE
 		a = pop();
-		setPalette(a);
+		setCurrentPalette(a);
 		break;
 
 	case 220:
@@ -1049,6 +1051,8 @@ void ScummEngine_v72he::o72_roomOps() {
 
 	case 221:
 		copyScriptString(filename, sizeof(filename));
+		debug(1, "o72_roomOps: case 221: filename %s", filename);
+
 		_saveLoadFlag = pop();
 		_saveLoadSlot = 1;
 		_saveTemporaryState = true;
@@ -1123,7 +1127,7 @@ void ScummEngine_v72he::o72_actorOps() {
 	case 65: // HE 98+
 		j = pop();
 		i = pop();
-		a->putActor(i, j, a->_room);
+		a->putActor(i, j);
 		break;
 	case 68: // HE 90+
 		k = pop();
@@ -1174,7 +1178,7 @@ void ScummEngine_v72he::o72_actorOps() {
 	case 86:		// SO_PALETTE
 		j = pop();
 		i = pop();
-		checkRange(255, 0, i, "Illegal palette slot %d");
+		assertRange(0, i, 255, "palette slot");
 		a->remapActorPaletteColor(i, j);
 		a->_needRedraw = true;
 		break;
@@ -1205,13 +1209,13 @@ void ScummEngine_v72he::o72_actorOps() {
 		a->_ignoreBoxes = 1;
 		a->_forceClip = 0;
 		if (a->isInCurrentRoom())
-			a->putActor(a->_pos.x, a->_pos.y, a->_room);
+			a->putActor();
 		break;
 	case 96:		// SO_FOLLOW_BOXES
 		a->_ignoreBoxes = 0;
 		a->_forceClip = 0;
 		if (a->isInCurrentRoom())
-			a->putActor(a->_pos.x, a->_pos.y, a->_room);
+			a->putActor();
 		break;
 	case 97:		// SO_ANIMATION_SPEED
 		a->setAnimSpeed(pop());
@@ -1245,7 +1249,7 @@ void ScummEngine_v72he::o72_actorOps() {
 		a->initActor(2);
 		break;
 	case 218:
-		a->drawActorToBackBuf(a->_pos.x, a->_pos.y);
+		a->drawActorToBackBuf(a->getPos().x, a->getPos().y);
 		break;
 	case 219:
 		a->_drawToBackBuf = false;
@@ -1279,7 +1283,7 @@ void ScummEngine_v72he::o72_verbOps() {
 	if (subOp == 196) {
 		_curVerb = pop();
 		_curVerbSlot = getVerbSlot(_curVerb, 0);
-		checkRange(_numVerbs - 1, 0, _curVerbSlot, "Illegal new verb slot %d");
+		assertRange(0, _curVerbSlot, _numVerbs - 1, "new verb slot");
 		return;
 	}
 	vs = &_verbs[_curVerbSlot];
@@ -1395,7 +1399,7 @@ void ScummEngine_v72he::o72_findObject() {
 }
 
 void ScummEngine_v72he::o72_arrayOps() {
-	ArrayHeader *ah;
+	byte *data;
 	byte string[1024];
 	int dim1end, dim1start, dim2end, dim2start;
 	int id, len, b, c, list[128];
@@ -1410,8 +1414,8 @@ void ScummEngine_v72he::o72_arrayOps() {
 	case 7:			// SO_ASSIGN_STRING
 		copyScriptString(string, sizeof(string));
 		len = resStrLen(string);
-		ah = defineArray(array, kStringArray, 0, 0, 0, len);
-		memcpy(ah->data, string, len);
+		data = defineArray(array, kStringArray, 0, 0, 0, len);
+		memcpy(data, string, len);
 		break;
 
 	case 126:
@@ -1489,8 +1493,8 @@ void ScummEngine_v72he::o72_arrayOps() {
 	case 194:
 		decodeScriptString(string);
 		len = resStrLen(string);
-		ah = defineArray(array, kStringArray, 0, 0, 0, len);
-		memcpy(ah->data, string, len);
+		data = defineArray(array, kStringArray, 0, 0, 0, len);
+		memcpy(data, string, len);
 		break;
 	case 208:		// SO_ASSIGN_INT_LIST
 		b = pop();
@@ -1528,7 +1532,7 @@ void ScummEngine_v72he::o72_systemOps() {
 		clearDrawObjectQueue();
 		break;
 	case 26: // HE80+
-		gdi.copyVirtScreenBuffers(Common::Rect(_screenWidth, _screenHeight));
+		restoreBackgroundHE(Common::Rect(_screenWidth, _screenHeight));
 		updatePalette();
 		break;
 	case 158:
@@ -1661,14 +1665,14 @@ void ScummEngine_v72he::o72_traceStatus() {
 
 void ScummEngine_v72he::o72_kernelGetFunctions() {
 	int args[29];
-	ArrayHeader *ah;
+	byte *data;
 	getStackList(args, ARRAYSIZE(args));
 
 	switch (args[0]) {
 	case 1:
 		writeVar(0, 0);
-		ah = defineArray(0, kByteArray, 0, 0, 0, virtScreenSave(0, args[1], args[2], args[3], args[4]));
-		virtScreenSave(ah->data, args[1], args[2], args[3], args[4]);
+		data = defineArray(0, kByteArray, 0, 0, 0, virtScreenSave(0, args[1], args[2], args[3], args[4]));
+		virtScreenSave(data, args[1], args[2], args[3], args[4]);
 		push(readVar(0));
 		break;
 	default:
@@ -1721,14 +1725,6 @@ void ScummEngine_v72he::o72_openFile() {
 	const char *filename = (char *)buffer + convertFilePath(buffer);
 	debug(1, "Final filename to %s", filename);
 
-	// Work around for lost, to avoid debug code been triggered.
-	// The 'TEST.FYL' file is always deleted after been created
-	// but we currently don't support deleting files.
-	if (!strcmp(filename, "TEST.FYL")) {
-		push(-1);
-		return;
-	}
-
 	slot = -1;
 	for (i = 1; i < 17; i++) {
 		if (_hInFileTable[i] == 0 && _hOutFileTable[i] == 0) {
@@ -1738,7 +1734,7 @@ void ScummEngine_v72he::o72_openFile() {
 	}
 
 	if (slot != -1) {
-		switch(mode) {
+		switch (mode) {
 		case 1:
 			// TODO / FIXME: Consider using listSavefiles to avoid unneccessary openForLoading calls
 			_hInFileTable[slot] = _saveFileMan->openForLoading(filename);
@@ -1772,8 +1768,8 @@ int ScummEngine_v72he::readFileToArray(int slot, int32 size) {
 		size = _hInFileTable[slot]->size() - _hInFileTable[slot]->pos();
 
 	writeVar(0, 0);
-	ArrayHeader *ah = defineArray(0, kByteArray, 0, 0, 0, size);
-	_hInFileTable[slot]->read(ah->data, size + 1);
+	byte *data = defineArray(0, kByteArray, 0, 0, 0, size);
+	_hInFileTable[slot]->read(data, size + 1);
 
 	return readVar(0);
 }
@@ -1870,6 +1866,7 @@ void ScummEngine_v72he::o72_deleteFile() {
 	byte filename[256];
 
 	copyScriptString(filename, sizeof(filename));
+
 	debug(1, "stub o72_deleteFile(%s)", filename);
 }
 
@@ -2107,9 +2104,7 @@ void ScummEngine_v72he::copyArrayHelper(ArrayHeader *ah, int idx2, int idx1, int
 
 void ScummEngine_v72he::o72_readINI() {
 	byte option[128];
-	ArrayHeader *ah;
-	const char *entry;
-	int len;
+	byte *data;
 
 	copyScriptString(option, sizeof(option));
 	byte subOp = fetchScriptByte();
@@ -2117,7 +2112,9 @@ void ScummEngine_v72he::o72_readINI() {
 	switch (subOp) {
 	case 43: // HE 100
 	case 6: // number
-		if (!strcmp((char *)option, "NoPrinting")) {
+		if (!strcmp((char *)option, "NoFontsInstalled")) {
+			push(1);
+		} else if (!strcmp((char *)option, "NoPrinting")) {
 			push(1);
 		} else if (!strcmp((char *)option, "TextOn")) {
 			push(ConfMan.getBool("subtitles"));
@@ -2127,13 +2124,19 @@ void ScummEngine_v72he::o72_readINI() {
 		break;
 	case 77: // HE 100
 	case 7: // string
-		entry = (ConfMan.get((char *)option).c_str());
-
 		writeVar(0, 0);
-		len = resStrLen((const byte *)entry);
-		ah = defineArray(0, kStringArray, 0, 0, 0, len);
-		memcpy(ah->data, entry, len);
-
+		if (!strcmp((char *)option, "SaveGamePath")) {
+			// We set SaveGamePath in order to detect where it used
+			// in convertFilePath and to avoid warning about invalid
+			// path in Macintosh verisons.
+			data = defineArray(0, kStringArray, 0, 0, 0, 1);
+			memcpy(data, (const char *)"*", 1);
+		} else {
+			const char *entry = (ConfMan.get((char *)option).c_str());
+			int len = resStrLen((const byte *)entry);
+			data = defineArray(0, kStringArray, 0, 0, 0, len);
+			memcpy(data, entry, len);
+		}
 		push(readVar(0));
 		break;
 	default:
@@ -2154,13 +2157,15 @@ void ScummEngine_v72he::o72_writeINI() {
 	case 6: // number
 		value = pop();
 		copyScriptString(option, sizeof(option));
-		ConfMan.setInt((char *)option, value);
 		debug(1, "o72_writeINI: Option %s Value %d", option, value);
+
+		ConfMan.setInt((char *)option, value);
 		break;
 	case 77: // HE 100
 	case 7: // string
 		copyScriptString(string, sizeof(string));
 		copyScriptString(option, sizeof(option));
+		debug(1, "o72_writeINI: Option %s String %s", option, string);
 
 		// Filter out useless setting
 		if (!strcmp((char *)option, "HETest"))
@@ -2175,7 +2180,6 @@ void ScummEngine_v72he::o72_writeINI() {
 			return;
 
 		ConfMan.set((char *)option, (char *)string);
-		debug(1, "o72_writeINI: Option %s String %s", option, string);
 		break;
 	default:
 		error("o72_writeINI: default type %d", subOp);
@@ -2222,11 +2226,11 @@ void ScummEngine_v72he::o72_getResourceSize() {
 	push(size);
 }
 
-void ScummEngine_v72he::o72_setFilePath() {
-	byte filename[255];
+void ScummEngine_v72he::o72_createDirectory() {
+	byte directoryName[255];
 
-	copyScriptString(filename, sizeof(filename));
-	debug(1,"o72_setFilePath: %s", filename);
+	copyScriptString(directoryName, sizeof(directoryName));
+	debug(1,"o72_createDirectory: %s", directoryName);
 }
 
 void ScummEngine_v72he::o72_setSystemMessage() {
