@@ -1,6 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2001  Ludvig Strigeus
- * Copyright (C) 2001-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -16,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/scumm/he/script_v80he.cpp $
- * $Id: script_v80he.cpp 22273 2006-05-02 03:23:03Z kirben $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/scumm/he/script_v80he.cpp $
+ * $Id: script_v80he.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -25,6 +27,7 @@
 
 #include "common/config-file.h"
 #include "common/config-manager.h"
+#include "common/savefile.h"
 #include "common/str.h"
 
 #include "scumm/actor.h"
@@ -256,7 +259,7 @@ void ScummEngine_v80he::setupOpcodes() {
 		OPCODE(o6_getActorWidth),
 		OPCODE(o6_wait),
 		OPCODE(o6_getActorScaleX),
-		OPCODE(o6_getActorAnimCounter1),
+		OPCODE(o6_getActorAnimCounter),
 		/* AC */
 		OPCODE(o80_drawWizPolygon),
 		OPCODE(o6_isAnyOf),
@@ -351,10 +354,10 @@ void ScummEngine_v80he::setupOpcodes() {
 		OPCODE(o72_writeINI),
 		OPCODE(o70_getStringLenForWidth),
 		OPCODE(o70_getCharIndexInString),
-		OPCODE(o6_invalid),
+		OPCODE(o70_findBox),
 		/* F8 */
 		OPCODE(o72_getResourceSize),
-		OPCODE(o72_setFilePath),
+		OPCODE(o72_createDirectory),
 		OPCODE(o72_setSystemMessage),
 		OPCODE(o70_polygonOps),
 		/* FC */
@@ -403,13 +406,20 @@ void ScummEngine_v80he::o80_getFileSize() {
 	copyScriptString(filename, sizeof(filename));
 	convertFilePath(filename);
 
-	Common::File f;
-	if (!f.open((char *)filename)) {
+	Common::SeekableReadStream *f = _saveFileMan->openForLoading((const char *)filename);
+	if (!f) {
+		Common::File *file = new Common::File();
+		file->open((const char *)filename, Common::File::kFileReadMode);
+		f = file;
+	}
+
+	if (!f) {
 		push(-1);
 	} else {
-		push(f.size());
-		f.close();
+		push(f->size());
 	}
+
+	delete f;
 }
 
 void ScummEngine_v80he::o80_stringToInt() {
@@ -447,7 +457,7 @@ void ScummEngine_v80he::o80_sourceDebug() {
 
 void ScummEngine_v80he::o80_readConfigFile() {
 	byte option[128], section[128], filename[256];
-	ArrayHeader *ah;
+	byte *data;
 	Common::String entry;
 	int len, r;
 
@@ -477,8 +487,8 @@ void ScummEngine_v80he::o80_readConfigFile() {
 
 		writeVar(0, 0);
 		len = resStrLen((const byte *)entry.c_str());
-		ah = defineArray(0, kStringArray, 0, 0, 0, len);
-		memcpy(ah->data, entry.c_str(), len);
+		data = defineArray(0, kStringArray, 0, 0, 0, len);
+		memcpy(data, entry.c_str(), len);
 		push(readVar(0));
 		break;
 	default:
