@@ -1,6 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2001  Ludvig Strigeus
- * Copyright (C) 2001-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -16,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/scumm/he/script_v90he.cpp $
- * $Id: script_v90he.cpp 23302 2006-06-25 00:59:01Z kirben $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/scumm/he/script_v90he.cpp $
+ * $Id: script_v90he.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -351,10 +353,10 @@ void ScummEngine_v90he::setupOpcodes() {
 		OPCODE(o72_writeINI),
 		OPCODE(o70_getStringLenForWidth),
 		OPCODE(o70_getCharIndexInString),
-		OPCODE(o6_invalid),
+		OPCODE(o70_findBox),
 		/* F8 */
 		OPCODE(o72_getResourceSize),
-		OPCODE(o72_setFilePath),
+		OPCODE(o72_createDirectory),
 		OPCODE(o72_setSystemMessage),
 		OPCODE(o70_polygonOps),
 		/* FC */
@@ -466,7 +468,7 @@ void ScummEngine_v90he::o90_getActorData() {
 		push(a->isUserConditionSet(val));
 		break;
 	case 2:
-		checkRange(15, 0, val, "Limb %d out of range");
+		assertRange(0, val, 15, "o90_getActorData: Limb");
 		push(a->_cost.frame[val]);
 		break;
 	case 3:
@@ -550,7 +552,7 @@ void ScummEngine_v90he::o90_videoOps() {
 			}
 		} else if (_videoParams.status == 165) {
 			// Stop video
-			_moviePlay->close();
+			_moviePlay->closeFile();
 		}
 		break;
 	default:
@@ -703,6 +705,7 @@ void ScummEngine_v90he::o90_wizImageOps() {
 		_wizParams.box.right = pop();
 		_wizParams.box.top = pop();
 		_wizParams.box.left = pop();
+		adjustRect(_wizParams.box);
 		break;
 	case 40: // HE99+
 		_wizParams.processFlags |= kWPFPaletteNum;
@@ -951,7 +954,7 @@ void ScummEngine_v90he::o90_getSpriteInfo() {
 		flags = pop();
 		spriteId = pop();
 		if (spriteId) {
-			switch(flags) {
+			switch (flags) {
 			case 0:
 				push(_sprite->getSpriteFlagXFlipped(spriteId));
 				break;
@@ -1166,7 +1169,7 @@ void ScummEngine_v90he::o90_setSpriteInfo() {
 			spriteId++;
 
 		for (; spriteId <= _curMaxSpriteId; spriteId++)
-			switch(args[1]) {
+			switch (args[1]) {
 			case 0:
 				_sprite->setSpriteFlagXFlipped(spriteId, args[0]);
 				break;
@@ -1478,7 +1481,7 @@ void ScummEngine_v90he::o90_getSpriteGroupInfo() {
 		type = pop();
 		spriteGroupId = pop();
 		if (spriteGroupId) {
-			switch(type) {
+			switch (type) {
 			case 0:
 				push(_sprite->getGroupXMul(spriteGroupId));
 				break;
@@ -2593,7 +2596,7 @@ void ScummEngine_v90he::o90_kernelSetFunctions() {
 		_skipDrawObject = 0;
 		break;
 	case 23:
-		_charset->clearCharsetMask();
+		clearCharsetMask();
 		_fullRedraw = true;
 		break;
 	case 24:
