@@ -1,6 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2001  Ludvig Strigeus
- * Copyright (C) 2001-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -16,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/scumm/he/sound_he.cpp $
- * $Id: sound_he.cpp 23446 2006-07-09 05:47:14Z kirben $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/scumm/he/sound_he.cpp $
+ * $Id: sound_he.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -46,10 +48,11 @@
 
 namespace Scumm {
 
-SoundHE::SoundHE(ScummEngine *parent)
+SoundHE::SoundHE(ScummEngine *parent, Audio::Mixer *mixer)
 	:
-	Sound(parent),
+	Sound(parent, mixer),
 	_vm((ScummEngine_v60he *)parent),
+	_overrideFreq(0),
 	_heMusic(0),
 	_heMusicTracks(0) {
 
@@ -113,7 +116,7 @@ void SoundHE::processSoundQueues() {
 int SoundHE::isSoundRunning(int sound) const {
 	if (_vm->_game.heversion >= 70) {
 		if (sound >= 10000) {
-			return _vm->_mixer->getSoundID(_heSoundChannels[sound - 10000]);
+			return _mixer->getSoundID(_heSoundChannels[sound - 10000]);
 		}
 	} else if (_vm->_game.heversion >= 60) {
 		if (sound == -2) {
@@ -123,7 +126,7 @@ int SoundHE::isSoundRunning(int sound) const {
 		}
 	}
 
-	if (_vm->_mixer->isSoundIDActive(sound))
+	if (_mixer->isSoundIDActive(sound))
 		return sound;
 
 	if (isSoundInQueue(sound))
@@ -188,7 +191,7 @@ void SoundHE::stopSoundChannel(int chan) {
 		_vm->_talkDelay = 0;
 	}
 
-	_vm->_mixer->stopHandle(_heSoundChannels[chan]);
+	_mixer->stopHandle(_heSoundChannels[chan]);
 
 	_heChannel[chan].sound = 0;
 	_heChannel[chan].priority = 0;
@@ -218,7 +221,7 @@ int SoundHE::findFreeSoundChannel() {
 
 	if (min < 8) {
 		for (chan = min; chan < ARRAYSIZE(_heChannel); chan++) {
-			if (_vm->_mixer->isSoundHandleActive(_heSoundChannels[chan]) == 0)
+			if (_mixer->isSoundHandleActive(_heSoundChannels[chan]) == 0)
 				return chan;
 		}
 	} else {
@@ -235,7 +238,7 @@ int SoundHE::isSoundCodeUsed(int sound) {
 			chan = i;
 	}
 
-	if (_vm->_mixer->isSoundHandleActive(_heSoundChannels[chan]) && chan != -1) {
+	if (_mixer->isSoundHandleActive(_heSoundChannels[chan]) && chan != -1) {
 		return _heChannel[chan].sbngBlock;
 	} else {
 		return 0;
@@ -249,7 +252,7 @@ int SoundHE::getSoundPos(int sound) {
 			chan = i;
 	}
 
-	if (_vm->_mixer->isSoundHandleActive(_heSoundChannels[chan]) && chan != -1) {
+	if (_mixer->isSoundHandleActive(_heSoundChannels[chan]) && chan != -1) {
 		int time =  _vm->getHETimer(chan + 4) * 11025 / 1000;
 		return time;
 	} else {
@@ -262,7 +265,7 @@ int SoundHE::getSoundVar(int sound, int var) {
 		return isSoundCodeUsed(sound);
 	}
 
-	checkRange(25, 0, var, "Illegal sound variable %d");
+	assertRange(0, var, 25, "sound variable");
 
 	int chan = -1;
 	for (int i = 0; i < ARRAYSIZE(_heChannel); i ++) {
@@ -270,7 +273,7 @@ int SoundHE::getSoundVar(int sound, int var) {
 			chan = i;
 	}
 
-	if (_vm->_mixer->isSoundHandleActive(_heSoundChannels[chan]) && chan != -1) {
+	if (_mixer->isSoundHandleActive(_heSoundChannels[chan]) && chan != -1) {
 		debug(5, "getSoundVar: sound %d var %d result %d", sound, var, _heChannel[chan].soundVars[var]);
 		return _heChannel[chan].soundVars[var];
 	} else {
@@ -279,7 +282,7 @@ int SoundHE::getSoundVar(int sound, int var) {
 }
 
 void SoundHE::setSoundVar(int sound, int var, int val) {
-	checkRange(25, 0, var, "Illegal sound variable %d");
+	assertRange(0, var, 25, "sound variable");
 
 	int chan = -1;
 	for (int i = 0; i < ARRAYSIZE(_heChannel); i ++) {
@@ -369,7 +372,7 @@ void SoundHE::processSoundCode() {
 		assert(codePtr);
 		codePtr += _heChannel[chan].codeOffs;
 
-		while(1) {
+		while (1) {
 			size = READ_LE_UINT16(codePtr);
 			time = READ_LE_UINT32(codePtr + 2);
 
@@ -414,7 +417,7 @@ void SoundHE::processSoundCode() {
 void SoundHE::processSoundOpcodes(int sound, byte *codePtr, int *soundVars) {
 	int arg, opcode, var, val;
 
-	while(READ_LE_UINT16(codePtr) != 0) {
+	while (READ_LE_UINT16(codePtr) != 0) {
 		codePtr += 2;
 		opcode = READ_LE_UINT16(codePtr); codePtr += 2;
 		opcode = (opcode & 0xFFF) >> 4;
@@ -527,14 +530,14 @@ void SoundHE::playHESound(int soundID, int heOffset, int heChannel, int heFlags)
 
 		musicFile.seek(music_offs, SEEK_SET);
 
-		_vm->_mixer->stopHandle(_heSoundChannels[heChannel]);
-		spoolPtr = _vm->res.createResource(rtSpoolBuffer, heChannel, size);
+		_mixer->stopHandle(_heSoundChannels[heChannel]);
+		spoolPtr = _vm->_res->createResource(rtSpoolBuffer, heChannel, size);
 		assert(spoolPtr);
 		musicFile.read(spoolPtr, size);
 		musicFile.close();
 
 		if (_vm->_game.heversion == 70) {
-			_vm->_mixer->playRaw(&_heSoundChannels[heChannel], spoolPtr, size, 11025, flags, soundID, 255, 0, 0,0, type);
+			_mixer->playRaw(type, &_heSoundChannels[heChannel], spoolPtr, size, 11025, flags, soundID);
 			return;
 		}
 	}
@@ -584,8 +587,8 @@ void SoundHE::playHESound(int soundID, int heOffset, int heChannel, int heFlags)
 			flags |= Audio::Mixer::FLAG_LOOP;
 		}
 
-		_vm->_mixer->stopHandle(_heSoundChannels[heChannel]);
-		_vm->_mixer->playRaw(&_heSoundChannels[heChannel], sound, size, rate, flags, soundID, 255, 0, 0,0, type);
+		_mixer->stopHandle(_heSoundChannels[heChannel]);
+		_mixer->playRaw(type, &_heSoundChannels[heChannel], sound, size, rate, flags, soundID);
 	}
 	// Support for sound in Humongous Entertainment games
 	else if (READ_BE_UINT32(ptr) == MKID_BE('DIGI') || READ_BE_UINT32(ptr) == MKID_BE('TALK')) {
@@ -597,7 +600,7 @@ void SoundHE::playHESound(int soundID, int heOffset, int heChannel, int heFlags)
 		// Skip DIGI/TALK (8) and HSHD (24) blocks
 		ptr += 32;
 
-		if (_vm->_mixer->isSoundHandleActive(_heSoundChannels[heChannel])) {
+		if (_mixer->isSoundHandleActive(_heSoundChannels[heChannel])) {
 			int curSnd = _heChannel[heChannel].sound;
 			if (curSnd == 1 && soundID != 1)
 				return;
@@ -640,8 +643,8 @@ void SoundHE::playHESound(int soundID, int heOffset, int heChannel, int heFlags)
 			_heChannel[heChannel].timer = size * 1000 / rate;
 		}
 
-		_vm->_mixer->stopHandle(_heSoundChannels[heChannel]);
-		_vm->_mixer->playRaw(&_heSoundChannels[heChannel], ptr + heOffset + 8, size, rate, flags, soundID, 255, 0, 0,0, type);
+		_mixer->stopHandle(_heSoundChannels[heChannel]);
+		_mixer->playRaw(type, &_heSoundChannels[heChannel], ptr + heOffset + 8, size, rate, flags, soundID);
 
 	}
 	// Support for PCM music in 3DO versions of Humongous Entertainment games
@@ -657,9 +660,9 @@ void SoundHE::playHESound(int soundID, int heOffset, int heChannel, int heFlags)
 
 		flags = Audio::Mixer::FLAG_AUTOFREE;
 
-		_vm->_mixer->stopID(_currentMusic);
+		_mixer->stopID(_currentMusic);
 		_currentMusic = soundID;
-		_vm->_mixer->playRaw(NULL, ptr + 8, size, rate, flags, soundID, 255, 0, 0,0, Audio::Mixer::kMusicSoundType);
+		_mixer->playRaw(Audio::Mixer::kMusicSoundType, NULL, ptr + 8, size, rate, flags, soundID);
 	}
 	else if (READ_BE_UINT32(ptr) == MKID_BE('MIDI')) {
 		if (_vm->_imuse) {
@@ -686,13 +689,13 @@ void SoundHE::startHETalkSound(uint32 offset) {
 	}
 
 	_sfxMode |= 2;
-	_vm->res.nukeResource(rtSound, 1);
+	_vm->_res->nukeResource(rtSound, 1);
 
 	_sfxFile->seek(offset + 4, SEEK_SET);
 	 size = _sfxFile->readUint32BE();
 	_sfxFile->seek(offset, SEEK_SET);
 
-	_vm->res.createResource(rtSound, 1, size);
+	_vm->_res->createResource(rtSound, 1, size);
 	ptr = _vm->getResourceAddress(rtSound, 1);
 	_sfxFile->read(ptr, size);
 
