@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2002-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/scumm/he/sound_he.h $
- * $Id: sound_he.h 23446 2006-07-09 05:47:14Z kirben $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/scumm/he/sound_he.h $
+ * $Id: sound_he.h 27024 2007-05-30 21:56:52Z fingolfin $
  */
 
 #ifndef SCUMM_HE_SOUND_HE_H
@@ -33,6 +36,8 @@ class SoundHE : public Sound {
 protected:
 	ScummEngine_v60he *_vm;
 
+	int _overrideFreq;
+
 	struct HEMusic {
 		int32 id;
 		int32 offset;
@@ -54,7 +59,7 @@ public: // Used by createSound()
 	} _heChannel[8];
 
 public:
-	SoundHE(ScummEngine *parent);
+	SoundHE(ScummEngine *parent, Audio::Mixer *mixer);
 	~SoundHE();
 	
 	virtual void addSoundToQueue(int sound, int heOffset = 0, int heChannel = 0, int heFlags = 0);
