@@ -1,6 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2001  Ludvig Strigeus
- * Copyright (C) 2001-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -16,15 +18,15 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/scumm/he/wiz_he.cpp $
- * $Id: wiz_he.cpp 23098 2006-06-13 18:40:35Z eriktorbjorn $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/scumm/he/wiz_he.cpp $
+ * $Id: wiz_he.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
 #include "common/stdafx.h"
 
 #include "common/system.h"
-#include "graphics/paletteman.h"
+#include "graphics/cursorman.h"
 #include "scumm/he/intern_he.h"
 #include "scumm/resource.h"
 #include "scumm/scumm.h"
@@ -354,7 +356,21 @@ void Wiz::copyWizImage(uint8 *dst, const uint8 *src, int dstw, int dsth, int src
 	Common::Rect r1, r2;
 	if (calcClipRects(dstw, dsth, srcx, srcy, srcw, srch, rect, r1, r2)) {
 		dst += r2.left + r2.top * dstw;
-		decompressWizImage(dst, dstw, r2, src, r1, flags, palPtr, xmapPtr);
+		if (flags & kWIFFlipY) {
+			const int dy = (srcy < 0) ? srcy : (srch - r1.height());
+			r1.translate(0, dy);
+		}
+		if (flags & kWIFFlipX) {
+			const int dx = (srcx < 0) ? srcx : (srcw - r1.width());
+			r1.translate(dx, 0);
+		}
+		if (xmapPtr) {
+			decompressWizImage<kWizXMap>(dst, dstw, src, r1, flags, palPtr, xmapPtr);
+		} else if (palPtr) {
+			decompressWizImage<kWizRMap>(dst, dstw, src, r1, flags, palPtr);
+		} else {
+			decompressWizImage<kWizCopy>(dst, dstw, src, r1, flags);
+		}
 	}
 }
 
@@ -365,7 +381,7 @@ static void decodeWizMask(uint8 *&dst, uint8 &mask, int w, int maskType) {
 			mask >>= 1;
 			if (mask == 0) {
 				mask = 0x80;
-				++dst;	
+				++dst;
 			}
 		}
 		break;
@@ -481,53 +497,6 @@ void Wiz::copyWizImageWithMask(uint8 *dst, const uint8 *src, int dstw, int dsth,
 	}
 }
 
-void Wiz::copyRaw16BitWizImage(uint8 *dst, const uint8 *src, int dstw, int dsth, int srcx, int srcy, int srcw, int srch, const Common::Rect *rect, int flags, const uint8 *palPtr, int transColor) {
-	// RAW 16 bits in 555 format
-
-	// HACK: Skip every second bit for now
-	Common::Rect r1, r2;
-	if (calcClipRects(dstw, dsth, srcx, srcy, srcw, srch, rect, r1, r2)) {
-		if (flags & kWIFFlipX) {
-			int l = r1.left;
-			int r = r1.right;
-			r1.left = srcw - r;
-			r1.right = srcw - l;
-		}
-		if (flags & kWIFFlipY) {
-			int t = r1.top;
-			int b = r1.bottom;
-			r1.top = srch - b;
-			r1.bottom = srch - t;
-		}
-		byte imagePal[256];
-		if (!palPtr) {
-			for (int i = 0; i < 256; i++) {
-				imagePal[i] = i;
-			}
-			palPtr = imagePal;
-		}
-
-		int h = r1.height();
-		int w = r1.width();
-		src += r1.left + r1.top * srcw * 2;
-		dst += r2.left + r2.top * dstw;
-
-		while (h--) {
-			const uint8 *p = src;
-			for (int i = 0; i < w; ++i) {
-				uint8 col = *p;
-				if (transColor == -1 || transColor != col) {
-					dst[i] = palPtr[col];
-				}
-				p += 2;
-			}
-			src += srcw * 2;
-			dst += dstw;
-		}
-
-	}
-}
-
 void Wiz::copyRawWizImage(uint8 *dst, const uint8 *src, int dstw, int dsth, int srcx, int srcy, int srcw, int srch, const Common::Rect *rect, int flags, const uint8 *palPtr, int transColor) {
 	Common::Rect r1, r2;
 	if (calcClipRects(dstw, dsth, srcx, srcy, srcw, srch, rect, r1, r2)) {
@@ -543,6 +512,27 @@ void Wiz::copyRawWizImage(uint8 *dst, const uint8 *src, int dstw, int dsth, int
 			r1.top = srch - b;
 			r1.bottom = srch - t;
 		}
+		int h = r1.height();
+		int w = r1.width();
+		src += r1.left + r1.top * srcw;
+		dst += r2.left + r2.top * dstw;
+		if (palPtr) {
+			decompressRawWizImage<kWizRMap>(dst, dstw, src, srcw, w, h, transColor, palPtr);
+		} else {
+			decompressRawWizImage<kWizCopy>(dst, dstw, src, srcw, w, h, transColor);
+		}
+	}
+}
+
+void Wiz::copyRaw16BitWizImage(uint8 *dst, const uint8 *src, int dstw, int dsth, int srcx, int srcy, int srcw, int srch, const Common::Rect *rect, int flags, const uint8 *palPtr, int transColor) {
+	// RAW 16 bits in 555 format
+
+	// HACK: Skip every second bit for now
+	Common::Rect r1, r2;
+	if (calcClipRects(dstw, dsth, srcx, srcy, srcw, srch, rect, r1, r2)) {
+		if (flags & (kWIFFlipY | kWIFFlipX)) {
+			warning("Unhandled Wiz flags (kWIFFlipY | kWIFFlipX)");
+		}
 		byte imagePal[256];
 		if (!palPtr) {
 			for (int i = 0; i < 256; i++) {
@@ -552,41 +542,32 @@ void Wiz::copyRawWizImage(uint8 *dst, const uint8 *src, int dstw, int dsth, int
 		}
 		int h = r1.height();
 		int w = r1.width();
-		src += r1.left + r1.top * srcw;
+		src += r1.left + r1.top * srcw * 2;
 		dst += r2.left + r2.top * dstw;
 		while (h--) {
-			const uint8 *p = src;
 			for (int i = 0; i < w; ++i) {
-				uint8 col = *p++;
+				uint8 col = src[2 * i];
 				if (transColor == -1 || transColor != col) {
 					dst[i] = palPtr[col];
 				}
 			}
-			src += srcw;
+			src += srcw * 2;
 			dst += dstw;
 		}
 	}
 }
 
-void Wiz::decompressWizImage(uint8 *dst, int dstPitch, const Common::Rect &dstRect, const uint8 *src, const Common::Rect &srcRect, int flags, const uint8 *palPtr, const uint8 *xmapPtr) {
-	if (flags & kWIFFlipX) {
-		debug(1, "decompressWizImage: Unhandled flag kWIFFlipX");
-	}
-	if (flags & kWIFFlipY) {
-		debug(1, "decompressWizImage: Unhandled flag kWIFFlipY");
-	}
-
+template <int type>
+void Wiz::decompressWizImage(uint8 *dst, int dstPitch, const uint8 *src, const Common::Rect &srcRect, int flags, const uint8 *palPtr, const uint8 *xmapPtr) {
 	const uint8 *dataPtr, *dataPtrNext;
 	uint8 code, *dstPtr, *dstPtrNext;
-	int h, w, xoff;
-	uint16 off;
+	int h, w, xoff, dstInc;
 
-	byte imagePal[256];
-	if (!palPtr) {
-		for (int i = 0; i < 256; i++) {
-			imagePal[i] = i;
-		}
-		palPtr = imagePal;
+	if (type == kWizXMap) {
+		assert(xmapPtr != 0);
+	}
+	if (type == kWizRMap) {
+		assert(palPtr != 0);
 	}
 
 	dstPtr = dst;
@@ -602,13 +583,23 @@ void Wiz::decompressWizImage(uint8 *dst, int dstPitch, const Common::Rect &dstRe
 	if (h <= 0 || w <= 0)
 		return;
 
+	if (flags & kWIFFlipY) {
+		dstPtr += (h - 1) * dstPitch;
+		dstPitch = -dstPitch;
+	}
+	dstInc = 1;
+	if (flags & kWIFFlipX) {
+		dstPtr += w - 1;
+		dstInc = -1;
+	}
+
 	while (h--) {
 		xoff = srcRect.left;
 		w = srcRect.width();
-		off = READ_LE_UINT16(dataPtr); dataPtr += 2;
+		uint16 lineSize = READ_LE_UINT16(dataPtr); dataPtr += 2;
 		dstPtrNext = dstPtr + dstPitch;
-		dataPtrNext = dataPtr + off;
-		if (off != 0) {
+		dataPtrNext = dataPtr + lineSize;
+		if (lineSize != 0) {
 			while (w > 0) {
 				code = *dataPtr++;
 				if (code & 1) {
@@ -620,8 +611,7 @@ void Wiz::decompressWizImage(uint8 *dst, int dstPitch, const Common::Rect &dstRe
 
 						code = -xoff;
 					}
-
-					dstPtr += code;
+					dstPtr += dstInc * code;
 					w -= code;
 				} else if (code & 2) {
 					code = (code >> 2) + 1;
@@ -639,12 +629,16 @@ void Wiz::decompressWizImage(uint8 *dst, int dstPitch, const Common::Rect &dstRe
 						code += w;
 					}
 					while (code--) {
-						if (xmapPtr) {
-							*dstPtr = xmapPtr[palPtr[*dataPtr] * 256 + *dstPtr];
-							dstPtr++;
-						} else {
-							*dstPtr++ = palPtr[*dataPtr];
+						if (type == kWizXMap) {
+							*dstPtr = xmapPtr[*dataPtr * 256 + *dstPtr];
 						}
+						if (type == kWizRMap) {
+							*dstPtr = palPtr[*dataPtr];
+						}
+						if (type == kWizCopy) {
+							*dstPtr = *dataPtr;
+						}
+						dstPtr += dstInc;
 					}
 					dataPtr++;
 				} else {
@@ -663,12 +657,16 @@ void Wiz::decompressWizImage(uint8 *dst, int dstPitch, const Common::Rect &dstRe
 						code += w;
 					}
 					while (code--) {
-						if (xmapPtr) {
-							*dstPtr = xmapPtr[palPtr[*dataPtr++] * 256 + *dstPtr];
-							dstPtr++;
-						} else {
-							*dstPtr++ = palPtr[*dataPtr++];
+						if (type == kWizXMap) {
+							*dstPtr = xmapPtr[*dataPtr++ * 256 + *dstPtr];
 						}
+						if (type == kWizRMap) {
+							*dstPtr = palPtr[*dataPtr++];
+						}
+						if (type == kWizCopy) {
+							*dstPtr = *dataPtr++;
+						}
+						dstPtr += dstInc;
 					}
 				}
 			}
@@ -676,7 +674,32 @@ void Wiz::decompressWizImage(uint8 *dst, int dstPitch, const Common::Rect &dstRe
 		dataPtr = dataPtrNext;
 		dstPtr = dstPtrNext;
 	}
+}
 
+template <int type>
+void Wiz::decompressRawWizImage(uint8 *dst, int dstPitch, const uint8 *src, int srcPitch, int w, int h, int transColor, const uint8 *palPtr) {
+	if (type == kWizRMap) {
+		assert(palPtr != 0);
+	}
+
+	if (w <= 0 || h <= 0) {
+		return;
+	}
+	while (h--) {
+		for (int i = 0; i < w; ++i) {
+			uint8 col = src[i];
+			if (transColor == -1 || transColor != col) {
+				if (type == kWizRMap) {
+					dst[i] = palPtr[col];
+				}
+				if (type == kWizCopy) {
+					dst[i] = col;
+				}
+			}
+		}
+		src += srcPitch;
+		dst += dstPitch;
+	}
 }
 
 int Wiz::isWizPixelNonTransparent(const uint8 *data, int x, int y, int w, int h) {
@@ -923,7 +946,7 @@ static int wizPackType1(uint8 *dst, const uint8 *src, int srcPitch, const Common
 					runCountSame = 0;
 				}
 				assert(runCountDiff < ARRAYSIZE(diffBuffer));
-				diffBuffer[runCountDiff++] = color;				
+				diffBuffer[runCountDiff++] = color;
 				if (runCountDiff == 0x40) {
 					if (dst) {
 						*dst++ = ((runCountDiff - 1) << 2) | 0;
@@ -1025,7 +1048,7 @@ void Wiz::captureWizImage(int resNum, const Common::Rect& r, bool backBuffer, in
 		dataSize = (dataSize + 1) & ~1;
 		int wizSize = headerSize + dataSize;
 		// write header
-		uint8 *wizImg = _vm->res.createResource(rtImage, resNum, dataSize + headerSize);
+		uint8 *wizImg = _vm->_res->createResource(rtImage, resNum, dataSize + headerSize);
 		WRITE_BE_UINT32(wizImg + 0x00, 'AWIZ');
 		WRITE_BE_UINT32(wizImg + 0x04, wizSize);
 		WRITE_BE_UINT32(wizImg + 0x08, 'WIZH');
@@ -1063,7 +1086,7 @@ void Wiz::captureWizImage(int resNum, const Common::Rect& r, bool backBuffer, in
 			break;
 		}
 	}
-	_vm->res.setModified(rtImage, resNum);
+	_vm->_res->setModified(rtImage, resNum);
 }
 
 void Wiz::displayWizImage(WizImage *pwi) {
@@ -1089,7 +1112,7 @@ void Wiz::displayWizImage(WizImage *pwi) {
 }
 
 uint8 *Wiz::drawWizImage(int resNum, int state, int x1, int y1, int zorder, int shadow, int field_390, const Common::Rect *clipBox, int flags, int dstResNum, int palette) {
-	debug(2, "drawWizImage(resNum %d, x1 %d y1 %d flags 0x%X zorder %d shadow %d field_390 %d dstResNum %d palette %d)", resNum, x1, y1, flags, zorder, shadow, field_390, dstResNum, palette);
+	debug(3, "drawWizImage(resNum %d, x1 %d y1 %d flags 0x%X zorder %d shadow %d field_390 %d dstResNum %d palette %d)", resNum, x1, y1, flags, zorder, shadow, field_390, dstResNum, palette);
 	uint8 *dataPtr;
 	uint8 *dst = NULL;
 
@@ -1112,13 +1135,13 @@ uint8 *Wiz::drawWizImage(int resNum, int state, int x1, int y1, int zorder, int
 
 	dataPtr = _vm->getResourceAddress(rtImage, resNum);
 	assert(dataPtr);
-	
+
 	uint8 *wizh = _vm->findWrappedBlock(MKID_BE('WIZH'), dataPtr, state, 0);
 	assert(wizh);
 	uint32 comp   = READ_LE_UINT32(wizh + 0x0);
 	uint32 width  = READ_LE_UINT32(wizh + 0x4);
 	uint32 height = READ_LE_UINT32(wizh + 0x8);
-	debug(2, "wiz_header.comp = %d wiz_header.w = %d wiz_header.h = %d", comp, width, height);
+	debug(3, "wiz_header.comp = %d wiz_header.w = %d wiz_header.h = %d", comp, width, height);
 
 	uint8 *wizd = _vm->findWrappedBlock(MKID_BE('WIZD'), dataPtr, state, 0);
 	assert(wizd);
@@ -1226,7 +1249,7 @@ uint8 *Wiz::drawWizImage(int resNum, int state, int x1, int y1, int zorder, int
 				++rImage.bottom;
 				_vm->markRectAsDirty(kMainVirtScreen, rImage);
 			} else {
-				_vm->gdi.copyVirtScreenBuffers(rImage);
+				_vm->restoreBackgroundHE(rImage);
 			}
 		}
 	}
@@ -1342,7 +1365,7 @@ void Wiz::drawWizPolygon(int resNum, int state, int id, int flags, int shadow, i
 }
 
 void Wiz::drawWizPolygonTransform(int resNum, int state, Common::Point *wp, int flags, int shadow, int dstResNum, int palette) {
-	debug(2, "drawWizPolygonTransform(resNum %d, flags 0x%X, shadow %d dstResNum %d palette %d)", resNum, flags, shadow, dstResNum, palette);
+	debug(3, "drawWizPolygonTransform(resNum %d, flags 0x%X, shadow %d dstResNum %d palette %d)", resNum, flags, shadow, dstResNum, palette);
 	int i;
 
 	if (flags & 0x800000) {
@@ -1401,12 +1424,12 @@ void Wiz::drawWizPolygonTransform(int resNum, int state, Common::Point *wp, int
   		int16 xmin_b, xmax_b, ymin_b, ymax_b;
   		xmin_b = ymin_b = (int16)0x7FFF;
   		xmax_b = ymax_b = (int16)0x8000;
-		
+
 		for (i = 0; i < 4; ++i) {
   			xmin_b = MIN(bbox[i].x, xmin_b);
   			xmax_b = MAX(bbox[i].x, xmax_b);
   			ymin_b = MIN(bbox[i].y, ymin_b);
-  			ymax_b = MAX(bbox[i].y, ymax_b);			
+  			ymax_b = MAX(bbox[i].y, ymax_b);
 		}
 
 		PolygonDrawData pdd(ymax_p - ymin_p + 1);
@@ -1485,7 +1508,7 @@ void Wiz::drawWizPolygonTransform(int resNum, int state, Common::Point *wp, int
 		if (flags & kWIFMarkBufferDirty) {
 			_vm->markRectAsDirty(kMainVirtScreen, bound);
 		} else {
-			_vm->gdi.copyVirtScreenBuffers(bound);
+			_vm->restoreBackgroundHE(bound);
 		}
 
 		free(srcWizBuf);
@@ -1527,7 +1550,7 @@ void Wiz::loadWizCursor(int resId) {
 	_vm->setCursorFromBuffer(cursor, cw, ch, cw);
 
 	// Since we set up cursor palette for default cursor, disable it now
-	PaletteMan.disableCursorPalette(true);
+	CursorMan.disableCursorPalette(true);
 
 	free(cursor);
 }
@@ -1637,7 +1660,7 @@ void Wiz::createWizEmptyImage(int resNum, int img_x, int img_y, int img_w, int i
 	} else {
 		palPtr = _vm->_currentPalette;
 	}
-	uint8 *res_data = _vm->res.createResource(rtImage, resNum, res_size);
+	uint8 *res_data = _vm->_res->createResource(rtImage, resNum, res_size);
 	if (!res_data) {
 		_vm->VAR(119) = -1;
 	} else {
@@ -1671,7 +1694,7 @@ void Wiz::createWizEmptyImage(int resNum, int img_x, int img_y, int img_w, int i
 		WRITE_BE_UINT32(res_data, 'WIZD'); res_data += 4;
 		WRITE_BE_UINT32(res_data, 8 + img_w * img_h); res_data += 4;
 	}
-	_vm->res.setModified(rtImage, resNum);
+	_vm->_res->setModified(rtImage, resNum);
 }
 
 void Wiz::fillWizRect(const WizParameters *params) {
@@ -1716,7 +1739,7 @@ void Wiz::fillWizRect(const WizParameters *params) {
 			}
 		}
 	}
-	_vm->res.setModified(rtImage, params->img.resNum);
+	_vm->_res->setModified(rtImage, params->img.resNum);
 }
 
 void Wiz::fillWizLine(const WizParameters *params) {
@@ -1808,7 +1831,7 @@ void Wiz::fillWizLine(const WizParameters *params) {
 			}
 		}
 	}
-	_vm->res.setModified(rtImage, params->img.resNum);
+	_vm->_res->setModified(rtImage, params->img.resNum);
 }
 
 void Wiz::fillWizPixel(const WizParameters *params) {
@@ -1845,7 +1868,7 @@ void Wiz::fillWizPixel(const WizParameters *params) {
 			}
 		}
 	}
-	_vm->res.setModified(rtImage, params->img.resNum);
+	_vm->_res->setModified(rtImage, params->img.resNum);
 }
 
 void Wiz::remapWizImagePal(const WizParameters *params) {
@@ -1861,13 +1884,13 @@ void Wiz::remapWizImagePal(const WizParameters *params) {
 		uint8 idx = *index++;
 		rmap[4 + idx] = params->remapColor[idx];
 	}
-	_vm->res.setModified(rtImage, params->img.resNum);
+	_vm->_res->setModified(rtImage, params->img.resNum);
 }
 
 void Wiz::processWizImage(const WizParameters *params) {
 	byte filename[260];
 
-	debug(2, "processWizImage: processMode %d", params->processMode);
+	debug(3, "processWizImage: processMode %d", params->processMode);
 	switch (params->processMode) {
 	case 0:
 		// Used in racedemo
@@ -1890,14 +1913,14 @@ void Wiz::processWizImage(const WizParameters *params) {
 				if (id == MKID_BE('AWIZ') || id == MKID_BE('MULT')) {
 					uint32 size = f.readUint32BE();
 					f.seek(0, SEEK_SET);
-					byte *p = _vm->res.createResource(rtImage, params->img.resNum, size);
+					byte *p = _vm->_res->createResource(rtImage, params->img.resNum, size);
 					if (f.read(p, size) != size) {
-						_vm->res.nukeResource(rtImage, params->img.resNum);
+						_vm->_res->nukeResource(rtImage, params->img.resNum);
 						error("i/o error when reading '%s'", filename);
 						_vm->VAR(_vm->VAR_GAME_LOADED) = -2;
 						_vm->VAR(119) = -2;
 					} else {
-						_vm->res.setModified(rtImage, params->img.resNum);
+						_vm->_res->setModified(rtImage, params->img.resNum);
 						_vm->VAR(_vm->VAR_GAME_LOADED) = 0;
 						_vm->VAR(119) = 0;
 					}
@@ -1917,7 +1940,7 @@ void Wiz::processWizImage(const WizParameters *params) {
 		if (params->processFlags & kWPFUseFile) {
 			Common::File f;
 
-			switch(params->fileWriteMode) {
+			switch (params->fileWriteMode) {
 			case 2:
 				_vm->VAR(119) = -1;
 				break;
@@ -1955,7 +1978,7 @@ void Wiz::processWizImage(const WizParameters *params) {
 	case 7:
 		// Used in PuttsFunShop/SamsFunShop/soccer2004
 		// TODO: Capture polygon
-		_vm->res.setModified(rtImage, params->img.resNum);
+		_vm->_res->setModified(rtImage, params->img.resNum);
 		break;
 	case 8: {
 			int img_w = 640;
@@ -2006,7 +2029,7 @@ void Wiz::processWizImage(const WizParameters *params) {
 	case 17:
 		// Used in to draw circles in FreddisFunShop/PuttsFunShop/SamsFunShop
 		// TODO: Ellipse
-		_vm->res.setModified(rtImage, params->img.resNum);
+		_vm->_res->setModified(rtImage, params->img.resNum);
 		break;
 	default:
 		error("Unhandled processWizImage mode %d", params->processMode);
