@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2002-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/scumm/help.cpp $
- * $Id: help.cpp 22872 2006-06-03 13:33:05Z kirben $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/scumm/help.cpp $
+ * $Id: help.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -35,10 +38,8 @@ int ScummHelp::numPages(byte gameId) {
 	case GID_MANIAC:
 	case GID_ZAK:
 		return 4;
-		break;
 	case GID_INDY3:
 		return 6;
-		break;
 	case GID_LOOM:
 	case GID_MONKEY_EGA:
 	case GID_MONKEY_VGA:
@@ -51,7 +52,6 @@ int ScummHelp::numPages(byte gameId) {
 	case GID_FT:
 	case GID_CMI:
 		return 3;
-		break;
 /*	TODO - I don't know the controls for these games
 	case GID_PUTTDEMO:
 	case GID_PUTTPUTT:
@@ -119,10 +119,7 @@ void ScummHelp::updateStrings(byte gameId, byte version, Common::Platform platfo
 		switch (gameId) {
 		case GID_ZAK:
 		case GID_MANIAC:
-			// HACK. I know use of g_scumm here is evil, however, 
-			// introducing new GID and putting it everywhere will
-			// pollute code much more that this single instance
-			if (g_scumm->_game.platform == Common::kPlatformNES) {
+			if (platform == Common::kPlatformNES) {
 				ADD_BIND("q", "Push");
 				ADD_BIND("a", "Pull");
 				ADD_BIND("z", "Give");
@@ -218,10 +215,8 @@ void ScummHelp::updateStrings(byte gameId, byte version, Common::Platform platfo
 			ADD_BIND("s", "puSh");
 			ADD_BIND("y", "pull (Yank)");
 			if (platform == Common::kPlatformSegaCD) {
-			// FIXME look at scripts to figure all options out...
-			// keys 1->4 seem to do something as well
-				ADD_BIND("6", "Highlight prev dialogue");
-				ADD_BIND("7", "Highlight next dialogue");
+				ADD_BIND("KeyUp", "Highlight prev dialogue");
+				ADD_BIND("KeyDown", "Highlight next dialogue");
 			}
 			break;
 		case GID_SAMNMAX:
@@ -267,17 +262,6 @@ void ScummHelp::updateStrings(byte gameId, byte version, Common::Platform platfo
 			ADD_BIND("o", "Upper right item");
 			ADD_BIND("l", "Lower right item");
 			ADD_LINE;
-			ADD_TEXT("Switching characters:");
-			if (gameId == GID_MANIAC) {
-				ADD_BIND("F1", "Dave");
-				ADD_BIND("F2", "Second kid");
-				ADD_BIND("F3", "Third kid");
-			} else {
-				ADD_BIND("F1", "Zak");
-				ADD_BIND("F2", "Annie");
-				ADD_BIND("F3", "Melissa");
-				ADD_BIND("F4", "Leslie");
-			}
 		} else if (gameId == GID_INDY3 || gameId == GID_ZAK) {
 			// Indy3, or FM-TOWNS Zak
 			ADD_TEXT("Inventory:");
@@ -289,6 +273,19 @@ void ScummHelp::updateStrings(byte gameId, byte version, Common::Platform platfo
 			ADD_BIND("m", "Lower right item");
 			ADD_BIND("o", "Scroll list up");
 			ADD_BIND("l", "Scroll list down");
+			ADD_LINE;
+		}
+		if (gameId == GID_MANIAC) {
+			ADD_TEXT("Switching characters:");
+			ADD_BIND("F1", "Dave");
+			ADD_BIND("F2", "Second kid");
+			ADD_BIND("F3", "Third kid");
+		} else if (gameId == GID_ZAK) {
+			ADD_TEXT("Switching characters:");
+			ADD_BIND("F1", "Zak");
+			ADD_BIND("F2", "Annie");
+			ADD_BIND("F3", "Melissa");
+			ADD_BIND("F4", "Leslie");
 		}
 		break;
 	case 5:
