@@ -1,6 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2001  Ludvig Strigeus
- * Copyright (C) 2001-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -16,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/scumm/imuse/imuse.cpp $
- * $Id: imuse.cpp 21607 2006-04-04 22:04:51Z sev $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/scumm/imuse/imuse.cpp $
+ * $Id: imuse.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -149,7 +151,7 @@ bool IMuseInternal::isMT32(int sound) {
 	if (ptr[8] == 'S' && ptr[9] == 'O')
 		return false;
 
-	error("Unknown music type: '%c%c%c%c'", (char)tag>>24, (char)tag>>16, (char)tag>>8, (char)tag);
+	error("Unknown music type: '%c%c%c%c'", (char)tag >> 24, (char)tag >> 16, (char)tag >> 8, (char)tag);
 
 	return false;
 }
@@ -191,7 +193,7 @@ bool IMuseInternal::isMIDI(int sound) {
 	if (ptr[8] == 'S' && ptr[9] == 'O')
 		return true;
 
-	error("Unknown music type: '%c%c%c%c'", (char)tag>>24, (char)tag>>16, (char)tag>>8, (char)tag);
+	error("Unknown music type: '%c%c%c%c'", (char)tag >> 24, (char)tag >> 16, (char)tag >> 8, (char)tag);
 
 	return false;
 }
@@ -387,12 +389,12 @@ int IMuseInternal::save_or_load(Serializer *ser, ScummEngine *scumm) {
 
 bool IMuseInternal::get_sound_active(int sound) const {
 	Common::StackLock lock(_mutex, "IMuseInternal::get_sound_active()");
-	return getSoundStatus_internal (sound, false) != 0;
+	return getSoundStatus_internal(sound, false) != 0;
 }
 
 int32 IMuseInternal::doCommand(int numargs, int a[]) {
 	Common::StackLock lock(_mutex, "IMuseInternal::doCommand()");
-	return doCommand_internal (numargs, a);
+	return doCommand_internal(numargs, a);
 }
 
 void IMuseInternal::setBase(byte **base) {
@@ -452,7 +454,7 @@ uint32 IMuseInternal::property(int prop, uint32 value) {
 	return 0;
 }
 
-void IMuseInternal::addSysexHandler (byte mfgID, sysexfunc handler) {
+void IMuseInternal::addSysexHandler(byte mfgID, sysexfunc handler) {
 	// TODO: Eventually support multiple sysEx handlers and pay
 	// attention to the client-supplied manufacturer ID.
 	Common::StackLock lock(_mutex, "IMuseInternal::property()");
@@ -467,7 +469,7 @@ void IMuseInternal::addSysexHandler (byte mfgID, sysexfunc handler) {
 //
 ////////////////////////////////////////
 
-void IMuseInternal::setMusicVolume (int vol) {
+void IMuseInternal::setMusicVolume(int vol) {
 	Common::StackLock lock(_mutex, "IMuseInternal::setMusicVolume()");
 	if (vol > 255)
 		vol = 255;
@@ -484,12 +486,12 @@ void IMuseInternal::setMusicVolume (int vol) {
 
 void IMuseInternal::startSound(int sound) {
 	Common::StackLock lock(_mutex, "IMuseInternal::startSound()");
-	startSound_internal (sound);
+	startSound_internal(sound);
 }
 
 void IMuseInternal::stopSound(int sound) {
 	Common::StackLock lock(_mutex, "IMuseInternal::stopSound()");
-	stopSound_internal (sound);
+	stopSound_internal(sound);
 }
 
 void IMuseInternal::stopAllSounds() {
@@ -497,7 +499,7 @@ void IMuseInternal::stopAllSounds() {
 	stopAllSounds_internal();
 }
 
-int IMuseInternal::getSoundStatus (int sound) const {
+int IMuseInternal::getSoundStatus(int sound) const {
 	Common::StackLock lock(_mutex, "IMuseInternal::getSoundStatus()");
 	return getSoundStatus_internal (sound, true);
 }
@@ -557,7 +559,7 @@ void IMuseInternal::terminate() {
 //
 ////////////////////////////////////////
 
-bool IMuseInternal::startSound_internal (int sound) {
+bool IMuseInternal::startSound_internal(int sound) {
 	// Do not start a sound if it is already set to start on an ImTrigger
 	// event. This fixes carnival music problems where a sound has been set
 	// to trigger at the right time, but then is started up immediately
@@ -633,7 +635,7 @@ bool IMuseInternal::startSound_internal (int sound) {
 	return player->startSound(sound, driver, _direct_passthrough);
 }
 
-int IMuseInternal::stopSound_internal (int sound) {
+int IMuseInternal::stopSound_internal(int sound) {
 	int r = -1;
 	Player *player = findActivePlayer(sound);
 	if (player) {
@@ -652,7 +654,7 @@ int IMuseInternal::stopAllSounds_internal() {
 	return 0;
 }
 
-int IMuseInternal::getSoundStatus_internal (int sound, bool ignoreFadeouts) const {
+int IMuseInternal::getSoundStatus_internal(int sound, bool ignoreFadeouts) const {
 	const Player *player = _players;
 	for (int i = ARRAYSIZE(_players); i; i--, player++) {
 		if (player->isActive() && (!ignoreFadeouts || !player->isFadingOut())) {
@@ -677,10 +679,10 @@ int32 IMuseInternal::doCommand_internal
 	args[5] = f;
 	args[6] = g;
 	args[7] = h;
-	return doCommand_internal (8, args);
+	return doCommand_internal(8, args);
 }
 
-int32 IMuseInternal::doCommand_internal (int numargs, int a[]) {
+int32 IMuseInternal::doCommand_internal(int numargs, int a[]) {
 	if (numargs < 1)
 		return -1;
 
@@ -713,9 +715,9 @@ int32 IMuseInternal::doCommand_internal (int numargs, int a[]) {
 			debug(0, "IMuse doCommand(7) - getMasterVolume (%d)", a[1]);
 			return _master_volume / 2; // Convert from 0-255 to 0-127
 		case 8:
-			return startSound_internal (a[1]) ? 0 : -1;
+			return startSound_internal(a[1]) ? 0 : -1;
 		case 9:
-			return stopSound_internal (a[1]);
+			return stopSound_internal(a[1]);
 		case 10: // FIXME: Sam and Max - Not sure if this is correct
 			return stopAllSounds_internal();
 		case 11:
@@ -960,6 +962,8 @@ void IMuseInternal::handle_marker(uint id, byte data) {
 	_trigger_count--;
 	_queue_cleared = false;
 	do {
+		bool skip_cmd = false;
+
 		pos = (pos + 1) % ARRAYSIZE(_cmd_queue);
 		if (_queue_pos == pos)
 			break;
@@ -968,7 +972,29 @@ void IMuseInternal::handle_marker(uint id, byte data) {
 			break;
 		_queue_end = pos;
 
-		doCommand_internal (p[0], p[1], p[2], p[3], p[4], p[5], p[6], 0);
+		// WORKAROUND for bug #1324106. When playing the "flourishes"
+		// as Rapp's body appears from his ashes, MI2 sets up a trigger
+		// to pause the music, in case the animation plays too slowly,
+		// and then the music is manually unpaused for the next part of
+		// the animation.
+		//
+		// However, with ScummVM the animation finishes slightly too
+		// quickly instead, and the pause command is run *after* the
+		// unpause, so the whole thing is thrown out of sync. I think
+		// we can assume that any platform running ScummVM is fast
+		// enough to keep up with the animation here, so ignore the
+		// pause command.
+		//
+		// Since setting up a trigger is a multi-step operation (first
+		// the trigger, and then the commands), it's probably easiest
+		// to catch it here.
+
+		if (_game_id == GID_MONKEY2 && p[0] == 262 && p[1] == 183 && p[2] == 0) {
+			skip_cmd = true;
+		}
+
+		if (!skip_cmd)
+			doCommand_internal(p[0], p[1], p[2], p[3], p[4], p[5], p[6], 0);
 
 		if (_queue_cleared)
 			return;
@@ -1231,8 +1257,8 @@ int32 IMuseInternal::ImSetTrigger(int sound, int id, int a, int b, int c, int d,
 	// NOTE: We ONLY do this if the sound that will trigger the command is actually
 	// playing. Otherwise, there's a problem when exiting and re-entering the
 	// Bumpusville mansion. Ref Bug #780918.
-	if (trig->command[0] == 8 && getSoundStatus_internal (trig->command[1],true) && getSoundStatus_internal (sound,true))
-		stopSound_internal (trig->command[1]);
+	if (trig->command[0] == 8 && getSoundStatus_internal(trig->command[1],true) && getSoundStatus_internal(sound,true))
+		stopSound_internal(trig->command[1]);
 	return 0;
 }
 
@@ -1257,7 +1283,7 @@ int32 IMuseInternal::ImFireAllTriggers(int sound) {
 	for (i = 0; i < ARRAYSIZE(_snm_triggers); ++i) {
 		if (_snm_triggers[i].sound == sound) {
 			_snm_triggers[i].sound = _snm_triggers[i].id = 0;
-			doCommand_internal (8, _snm_triggers[i].command);
+			doCommand_internal(8, _snm_triggers[i].command);
 			++count;
 		}
 	}
@@ -1384,12 +1410,12 @@ int IMuseInternal::initialize(OSystem *syst, MidiDriver *native_midi, MidiDriver
 	if (native_midi != NULL) {
 		_timer_info_native.imuse = this;
 		_timer_info_native.driver = native_midi;
-		initMidiDriver (&_timer_info_native);
+		initMidiDriver(&_timer_info_native);
 	}
 	if (adlib_midi != NULL) {
 		_timer_info_adlib.imuse = this;
 		_timer_info_adlib.driver = adlib_midi;
-		initMidiDriver (&_timer_info_adlib);
+		initMidiDriver(&_timer_info_adlib);
 	}
 
 	if (!_tempoFactor)
@@ -1408,14 +1434,14 @@ int IMuseInternal::initialize(OSystem *syst, MidiDriver *native_midi, MidiDriver
 	return 0;
 }
 
-void IMuseInternal::initMidiDriver (TimerCallbackInfo *info) {
+void IMuseInternal::initMidiDriver(TimerCallbackInfo *info) {
 	// Open MIDI driver
 	int result = info->driver->open();
 	if (result)
 		error("IMuse initialization - %s", MidiDriver::getErrorName(result));
 
 	// Connect to the driver's timer
-	info->driver->setTimerCallback (info, &IMuseInternal::midiTimerCallback);
+	info->driver->setTimerCallback(info, &IMuseInternal::midiTimerCallback);
 }
 
 void IMuseInternal::initMT32(MidiDriver *midi) {
@@ -1460,7 +1486,7 @@ void IMuseInternal::initGM(MidiDriver *midi) {
 
 	// General MIDI System On message
 	// Resets all GM devices to default settings
-	memcpy(&buffer[0], "\xF0\x7E\x7F\x09\x01\xF7", 6);
+	memcpy(&buffer[0], "\x7E\x7F\x09\x01", 4);
 	midi->sysEx(buffer, 6);
 	debug(2, "GM SysEx: GM System On");
 	_system->delayMillis(200);
@@ -1475,11 +1501,11 @@ void IMuseInternal::initGM(MidiDriver *midi) {
 		// require individual SysEx commands with unique IDs.
 
 		// Roland GS SysEx ID
-		memcpy(&buffer[0], "\xF0\x41\x10\x42\x12", 5);
+		memcpy(&buffer[0], "\x41\x10\x42\x12", 4);
 
 		// GS Reset
-		memcpy(&buffer[5], "\x40\x00\x7F\x00\x41\xF7", 6);
-		midi->sysEx(buffer, 11);
+		memcpy(&buffer[4], "\x40\x00\x7F\x00\x41", 5);
+		midi->sysEx(buffer, 9);
 		debug(2, "GS SysEx: GS Reset");
 		_system->delayMillis(200);
 
@@ -1508,8 +1534,8 @@ void IMuseInternal::initGM(MidiDriver *midi) {
 		}
 
 		// Set Master Chorus to 0. The MT-32 has no chorus capability.
-		memcpy(&buffer[5], "\x40\x01\x3A\x00\x05\xF7", 6);
-		midi->sysEx(buffer, 11);
+		memcpy(&buffer[4], "\x40\x01\x3A\x00\x05", 5);
+		midi->sysEx(buffer, 9);
 		debug(2, "GS SysEx: Master Chorus Level is 0");
 
 		// Set Channels 1-16 Reverb to 64, which is the
@@ -1521,48 +1547,17 @@ void IMuseInternal::initGM(MidiDriver *midi) {
 		// Set Channels 1-16 Pitch Bend Sensitivity to
 		// 12 semitones; then lock the RPN by setting null.
 		for (i = 0; i < 16; ++i) {
-			midi->send((  0    << 16) | (100 << 8) | (0xB0 | i));
-			midi->send((  0    << 16) | (101 << 8) | (0xB0 | i));
-			midi->send((  12   << 16) | (6   << 8) | (0xB0 | i));
-			midi->send((  0    << 16) | (38  << 8) | (0xB0 | i));
-			midi->send((  127  << 16) | (100 << 8) | (0xB0 | i));
-			midi->send((  127  << 16) | (101 << 8) | (0xB0 | i));
+			midi->setPitchBendRange(i, 12);
 		}
 		debug(2, "GM Controller 6 Change: Channels 1-16 Pitch Bend Sensitivity is 12 semitones");
 
 		// Set channels 1-16 Mod. LFO1 Pitch Depth to 4
-		memcpy(&buffer[5], "\x40\x20\x04\x04\x18\xF7", 6);
-		midi->sysEx(buffer, 11);
-		memcpy(&buffer[5], "\x40\x21\x04\x04\x17\xF7", 6);
-		midi->sysEx(buffer, 11);
-		memcpy(&buffer[5], "\x40\x22\x04\x04\x16\xF7", 6);
-		midi->sysEx(buffer, 11);
-		memcpy(&buffer[5], "\x40\x23\x04\x04\x15\xF7", 6);
-		midi->sysEx(buffer, 11);
-		memcpy(&buffer[5], "\x40\x24\x04\x04\x14\xF7", 6);
-		midi->sysEx(buffer, 11);
-		memcpy(&buffer[5], "\x40\x25\x04\x04\x13\xF7", 6);
-		midi->sysEx(buffer, 11);
-		memcpy(&buffer[5], "\x40\x26\x04\x04\x12\xF7", 6);
-		midi->sysEx(buffer, 11);
-		memcpy(&buffer[5], "\x40\x27\x04\x04\x11\xF7", 6);
-		midi->sysEx(buffer, 11);
-		memcpy(&buffer[5], "\x40\x28\x04\x04\x10\xF7", 6);
-		midi->sysEx(buffer, 11);
-		memcpy(&buffer[5], "\x40\x29\x04\x04\x0F\xF7", 6);
-		midi->sysEx(buffer, 11);
-		memcpy(&buffer[5], "\x40\x2A\x04\x04\x0E\xF7", 6);
-		midi->sysEx(buffer, 11);
-		memcpy(&buffer[5], "\x40\x2B\x04\x04\x0D\xF7", 6);
-		midi->sysEx(buffer, 11);
-		memcpy(&buffer[5], "\x40\x2C\x04\x04\x0C\xF7", 6);
-		midi->sysEx(buffer, 11);
-		memcpy(&buffer[5], "\x40\x2D\x04\x04\x0B\xF7", 6);
-		midi->sysEx(buffer, 11);
-		memcpy(&buffer[5], "\x40\x2E\x04\x04\x0A\xF7", 6);
-		midi->sysEx(buffer, 11);
-		memcpy(&buffer[5], "\x40\x2F\x04\x04\x09\xF7", 6);
-		midi->sysEx(buffer, 11);
+		memcpy(&buffer[4], "\x40\x20\x04\x04\x18", 5);
+		for (i = 0; i < 16; ++i) {
+			buffer[5] = 0x20 + i;
+			buffer[8] = 0x18 - i;
+			midi->sysEx(buffer, 9);
+		}
 		debug(2, "GS SysEx: Channels 1-16 Mod. LFO1 Pitch Depth Level is 4");
 
 		// Set Percussion Channel Expression to 80
@@ -1572,26 +1567,26 @@ void IMuseInternal::initGM(MidiDriver *midi) {
 		// Turn off Percussion Channel Rx. Expression so that
 		// Expression cannot be modified. I don't know why, but
 		// Roland does it this way.
-		memcpy(&buffer[5], "\x40\x10\x0E\x00\x22\xF7", 6);
-		midi->sysEx(buffer, 11);
+		memcpy(&buffer[4], "\x40\x10\x0E\x00\x22", 5);
+		midi->sysEx(buffer, 9);
 		debug(2, "GS SysEx: Percussion Channel Rx. Expression is OFF");
 
 		// Change Reverb Character to 0. I don't think this
 		// sounds most like MT-32, but apparently Roland does.
-		memcpy(&buffer[5], "\x40\x01\x31\x00\x0E\xF7", 6);
-		midi->sysEx(buffer, 11);
+		memcpy(&buffer[4], "\x40\x01\x31\x00\x0E", 5);
+		midi->sysEx(buffer, 9);
 		debug(2, "GS SysEx: Reverb Character is 0");
 
 		// Change Reverb Pre-LF to 4, which is similar to
 		// what MT-32 reverb does.
-		memcpy(&buffer[5], "\x40\x01\x32\x04\x09\xF7", 6);
-		midi->sysEx(buffer, 11);
+		memcpy(&buffer[4], "\x40\x01\x32\x04\x09", 5);
+		midi->sysEx(buffer, 9);
 		debug(2, "GS SysEx: Reverb Pre-LF is 4");
 
 		// Change Reverb Time to 106; the decay on Hall 2
 		// Reverb is too fast compared to the MT-32's
-		memcpy(&buffer[5], "\x40\x01\x34\x6A\x21\xF7", 6);
-		midi->sysEx(buffer, 11);
+		memcpy(&buffer[4], "\x40\x01\x34\x6A\x21", 5);
+		midi->sysEx(buffer, 9);
 		debug(2, "GS SysEx: Reverb Time is 106");
 	}
 }
@@ -1612,7 +1607,7 @@ void IMuseInternal::handleDeferredCommands(MidiDriver *midi) {
 		if (!ptr->time_left)
 			continue;
 		if (ptr->time_left <= advance) {
-			doCommand_internal (ptr->a, ptr->b, ptr->c, ptr->d, ptr->e, ptr->f, 0, 0);
+			doCommand_internal(ptr->a, ptr->b, ptr->c, ptr->d, ptr->e, ptr->f, 0, 0);
 			ptr->time_left = advance;
 		}
 		ptr->time_left -= advance;
@@ -1671,9 +1666,9 @@ void IMuseInternal::fix_players_after_load(ScummEngine *scumm) {
 //
 ////////////////////////////////////////
 
-void IMuseInternal::midiTimerCallback (void *data) {
-	TimerCallbackInfo *info = (TimerCallbackInfo *) data;
-	info->imuse->on_timer (info->driver);
+void IMuseInternal::midiTimerCallback(void *data) {
+	TimerCallbackInfo *info = (TimerCallbackInfo *)data;
+	info->imuse->on_timer(info->driver);
 }
 
 void IMuseInternal::reallocateMidiChannels(MidiDriver *midi) {
