@@ -1,6 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2001  Ludvig Strigeus
- * Copyright (C) 2001-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -16,14 +18,14 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/scumm/imuse/imuse_player.cpp $
- * $Id: imuse_player.cpp 22479 2006-05-15 13:19:30Z eriktorbjorn $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/scumm/imuse/imuse_player.cpp $
+ * $Id: imuse_player.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  */
 
 #include "common/stdafx.h"
 
 #include "common/util.h"
-#include "base/engine.h"
+#include "engines/engine.h"
 
 #include "scumm/imuse/imuse_internal.h"
 #include "scumm/saveload.h"
@@ -211,8 +213,8 @@ int Player::start_seq_sound(int sound, bool reset_vars) {
 }
 
 void Player::uninit_parts() {
-	if (_parts && _parts->_player != this)
-		error("asd");
+	assert(!_parts || _parts->_player == this);
+
 	while (_parts)
 		_parts->uninit();
 
@@ -397,12 +399,13 @@ void Player::sysEx(const byte *p, uint16 len) {
 		debugC(DEBUG_IMUSE, "[%02d] SysEx:%s", _id, buf);
 	}
 
-	if (_se->_sysex) (*_se->_sysex) (this, p, len);
+	if (_se->_sysex)
+		(*_se->_sysex)(this, p, len);
 }
 
 void Player::decode_sysex_bytes(const byte *src, byte *dst, int len) {
 	while (len >= 0) {
-		*dst++ = ((src[0] << 4)&0xFF) | (src[1] & 0xF);
+		*dst++ = ((src[0] << 4) & 0xFF) | (src[1] & 0xF);
 		src += 2;
 		len -= 2;
 	}
@@ -885,7 +888,7 @@ int Player::addParameterFader(int param, int target, int time) {
 		break;
 
 	default:
-		debug(0, "Player::addParameterFader (%d, %d, %d): Unknown parameter", param, target, time);
+		debug(0, "Player::addParameterFader(%d, %d, %d): Unknown parameter", param, target, time);
 		return 0; // Should be -1, but we'll let the script think it worked.
 	}
 
