@@ -1,6 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2001  Ludvig Strigeus
- * Copyright (C) 2001-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -16,12 +18,12 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/scumm/imuse/instrument.h $
- * $Id: instrument.h 20952 2006-02-27 01:59:07Z Jamieson630 $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/scumm/imuse/instrument.h $
+ * $Id: instrument.h 27024 2007-05-30 21:56:52Z fingolfin $
  */
 
-#ifndef INSTRUMENT_H
-#define INSTRUMENT_H
+#ifndef SCUMM_IMUSE_INSTRUMENT_H
+#define SCUMM_IMUSE_INSTRUMENT_H
 
 #include "common/stdafx.h"
 #include "common/scummsys.h"
@@ -36,9 +38,9 @@ class Instrument;
 class InstrumentInternal {
 public:
 	virtual ~InstrumentInternal() {}
-	virtual void saveOrLoad (Serializer *s) = 0;
-	virtual void send (MidiChannel *mc) = 0;
-	virtual void copy_to (Instrument *dest) = 0;
+	virtual void saveOrLoad(Serializer *s) = 0;
+	virtual void send(MidiChannel *mc) = 0;
+	virtual void copy_to(Instrument *dest) = 0;
 	virtual bool is_valid() = 0;
 	virtual operator int() { return 255; }
 };
@@ -58,20 +60,28 @@ public:
 
 	Instrument() : _type (0), _instrument (0) { }
 	~Instrument() { delete _instrument; }
-	static void nativeMT32 (bool native);
+	static void nativeMT32(bool native);
 	static const byte _gmRhythmMap[35];
 
 	void clear();
-	void copy_to (Instrument *dest) { if (_instrument) _instrument->copy_to (dest); else dest->clear(); }
+	void copy_to(Instrument *dest) {
+		if (_instrument)
+			_instrument->copy_to(dest);
+		else
+			dest->clear();
+	}
 
-	void program (byte program, bool mt32);
-	void adlib (const byte *instrument);
-	void roland (const byte *instrument);
+	void program(byte program, bool mt32);
+	void adlib(const byte *instrument);
+	void roland(const byte *instrument);
 
 	byte getType() { return _type; }
 	bool isValid() { return (_instrument ? _instrument->is_valid() : false); }
-	void saveOrLoad (Serializer *s);
-	void send (MidiChannel *mc) { if (_instrument) _instrument->send (mc); }
+	void saveOrLoad(Serializer *s);
+	void send(MidiChannel *mc) {
+		if (_instrument)
+			_instrument->send(mc);
+	}
 };
 
 } // End of namespace Scumm
