@@ -1,6 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2001  Ludvig Strigeus
- * Copyright (C) 2001-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -16,12 +18,12 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/scumm/imuse/sysex.h $
- * $Id: sysex.h 21241 2006-03-12 18:06:22Z Jamieson630 $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/scumm/imuse/sysex.h $
+ * $Id: sysex.h 27024 2007-05-30 21:56:52Z fingolfin $
  */
 
-#ifndef DEFINED_SYSEX_H
-#define DEFINED_SYSEX_H
+#ifndef SCUMM_IMUSE_SYSEX_H
+#define SCUMM_IMUSE_SYSEX_H
 
 #include "common/stdafx.h"
 #include "common/util.h"
@@ -30,8 +32,8 @@ namespace Scumm {
 
 class Player;
 
-extern void sysexHandler_Scumm (Player *, const byte *, uint16);
-extern void sysexHandler_SamNMax (Player *, const byte *, uint16);
+extern void sysexHandler_Scumm(Player *, const byte *, uint16);
+extern void sysexHandler_SamNMax(Player *, const byte *, uint16);
 
 } // End of namespace Scumm
 
