@@ -1,6 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2001  Ludvig Strigeus
- * Copyright (C) 2001-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -16,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/scumm/imuse/sysex_scumm.cpp $
- * $Id: sysex_scumm.cpp 21604 2006-04-04 21:32:19Z anotherguest $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/scumm/imuse/sysex_scumm.cpp $
+ * $Id: sysex_scumm.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  */
 
 #include "common/stdafx.h"
@@ -37,7 +39,7 @@
 
 namespace Scumm {
 
-void sysexHandler_Scumm (Player *player, const byte *msg, uint16 len) {
+void sysexHandler_Scumm(Player *player, const byte *msg, uint16 len) {
 	Part *part;
 	byte a;
 	byte buf[128];
@@ -67,12 +69,12 @@ void sysexHandler_Scumm (Player *player, const byte *msg, uint16 len) {
 		part = player->getPart(p[0] & 0x0F);
 		if (part) {
 			part->set_onoff(p[2] & 0x01);
-			part->effectLevel ((p[2] & 0x02) ? 127 : 0);
+			part->effectLevel((p[2] & 0x02) ? 127 : 0);
 			part->set_pri(p[4]);
 			part->volume((p[5] & 0x0F) << 4 |(p[6] & 0x0F));
 			part->set_pan((p[7] & 0x0F) << 4 | (p[8] & 0x0F));
 			part->_percussion = player->_isMIDI ? ((p[9] & 0x08) > 0) : false;
-			part->pitchBendFactor ((p[13] & 0x0F) << 4 | (p[14] & 0x0F));
+			part->pitchBendFactor((p[13] & 0x0F) << 4 | (p[14] & 0x0F));
 			if (part->_percussion) {
 				if (part->_mc) {
 					part->off();
@@ -93,8 +95,9 @@ void sysexHandler_Scumm (Player *player, const byte *msg, uint16 len) {
 
 	case 1:
 		// Shut down a part. [Bug 1088045, comments]
-		part = player->getPart (p[0]);
-		if (part != NULL) part->uninit();
+		part = player->getPart(p[0]);
+		if (part != NULL)
+			part->uninit();
 		break;
 
 	case 2: // Start of song. Ignore for now.
@@ -105,11 +108,11 @@ void sysexHandler_Scumm (Player *player, const byte *msg, uint16 len) {
 		++p; // Skip hardware type
 		part = player->getPart(a);
 		if (part) {
-			if (len == 63) {
-				player->decode_sysex_bytes(p, buf, len - 3);
+			if (len == 62) {
+				player->decode_sysex_bytes(p, buf, len - 2);
 				part->set_instrument((byte *)buf);
 			} else {
-				// SPK tracks have len == 49 here, and are not supported
+				// SPK tracks have len == 48 here, and are not supported
 				part->programChange(254); // Must be invalid, but not 255 (which is reserved)
 			}
 		}
@@ -118,14 +121,14 @@ void sysexHandler_Scumm (Player *player, const byte *msg, uint16 len) {
 	case 17: // Adlib instrument definition(Global)
 		p += 2; // Skip hardware type and... whatever came right before it
 		a = *p++;
-		player->decode_sysex_bytes(p, buf, len - 4);
+		player->decode_sysex_bytes(p, buf, len - 3);
 		se->setGlobalAdlibInstrument(a, buf);
 		break;
 
 	case 33: // Parameter adjust
 		a = *p++ & 0x0F;
 		++p; // Skip hardware type
-		player->decode_sysex_bytes(p, buf, len - 3);
+		player->decode_sysex_bytes(p, buf, len - 2);
 		part = player->getPart(a);
 		if (part)
 			part->set_param(READ_BE_UINT16(buf), READ_BE_UINT16(buf + 2));
@@ -134,49 +137,49 @@ void sysexHandler_Scumm (Player *player, const byte *msg, uint16 len) {
 	case 48: // Hook - jump
 		if (player->_scanning)
 			break;
-		player->decode_sysex_bytes(p + 1, buf, len - 2);
+		player->decode_sysex_bytes(p + 1, buf, len - 1);
 		player->maybe_jump(buf[0], READ_BE_UINT16(buf + 1), READ_BE_UINT16(buf + 3), READ_BE_UINT16(buf + 5));
 		break;
 
 	case 49: // Hook - global transpose
-		player->decode_sysex_bytes(p + 1, buf, len - 2);
+		player->decode_sysex_bytes(p + 1, buf, len - 1);
 		player->maybe_set_transpose(buf);
 		break;
 
 	case 50: // Hook - part on/off
 		buf[0] = *p++ & 0x0F;
-		player->decode_sysex_bytes(p, buf + 1, len - 2);
+		player->decode_sysex_bytes(p, buf + 1, len - 1);
 		player->maybe_part_onoff(buf);
 		break;
 
 	case 51: // Hook - set volume
 		buf[0] = *p++ & 0x0F;
-		player->decode_sysex_bytes(p, buf + 1, len - 2);
+		player->decode_sysex_bytes(p, buf + 1, len - 1);
 		player->maybe_set_volume(buf);
 		break;
 
 	case 52: // Hook - set program
 		buf[0] = *p++ & 0x0F;
-		player->decode_sysex_bytes(p, buf + 1, len - 2);
+		player->decode_sysex_bytes(p, buf + 1, len - 1);
 		player->maybe_set_program(buf);
 		break;
 
 	case 53: // Hook - set transpose
 		buf[0] = *p++ & 0x0F;
-		player->decode_sysex_bytes(p, buf + 1, len - 2);
+		player->decode_sysex_bytes(p, buf + 1, len - 1);
 		player->maybe_set_transpose_part(buf);
 		break;
 
 	case 64: // Marker
 		p++;
-		len -= 2;
+		len--;
 		while (len--) {
 			se->handle_marker(player->_id, *p++);
 		}
 		break;
 
 	case 80: // Loop
-		player->decode_sysex_bytes(p + 1, buf, len - 2);
+		player->decode_sysex_bytes(p + 1, buf, len - 1);
 		player->setLoop
 			(READ_BE_UINT16(buf), READ_BE_UINT16(buf + 2),
 			 READ_BE_UINT16(buf + 4), READ_BE_UINT16(buf + 6),
@@ -189,7 +192,7 @@ void sysexHandler_Scumm (Player *player, const byte *msg, uint16 len) {
 
 	case 96: // Set instrument
 		part = player->getPart(p[0] & 0x0F);
-		a = (p[1] & 0x0F) << 12 |(p[2] & 0x0F) << 8 |(p[4] & 0x0F) << 4 |(p[4] & 0x0F);
+		a = (p[1] & 0x0F) << 12 | (p[2] & 0x0F) << 8 | (p[3] & 0x0F) << 4 | (p[4] & 0x0F);
 		if (part)
 			part->set_instrument(a);
 		break;
