@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2001-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,12 +18,12 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/scumm/imuse_digi/dimuse.h $
- * $Id: dimuse.h 22526 2006-05-18 21:46:07Z eriktorbjorn $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/scumm/imuse_digi/dimuse.h $
+ * $Id: dimuse.h 27049 2007-06-02 09:07:29Z aquadran $
  */
 
-#if !defined(IMUSE_DIGI_H) && !defined(DISABLE_SCUMM_7_8)
-#define IMUSE_DIGI_H
+#if !defined(SCUMM_IMUSE_DIGI_H) && !defined(DISABLE_SCUMM_7_8)
+#define SCUMM_IMUSE_DIGI_H
 
 #include "common/scummsys.h"
 #include "common/util.h"
@@ -42,45 +45,63 @@ namespace Scumm {
 struct imuseDigTable;
 struct imuseComiTable;
 class Serializer;
+class ScummEngine_v7;
+
+// These flag bits correspond exactly to the sound mixer flags of March 2007.
+// We don't want to use the mixer flags directly, because then our saved games
+// will break in interesting ways if the mixer flags are ever assigned new
+// values. Now they should keep working, as long as these flags don't change.
+
+enum {
+	kFlagUnsigned = 1 << 0,
+	kFlag16Bits = 1 << 1,
+	kFlagLittleEndian = 1 << 2,
+	kFlagStereo = 1 << 3,
+	kFlagReverseStereo = 1 << 4
+
+	// Not used by Digital iMUSE
+	// kFlagAutoFree = 1 << 5,
+	// kFlagLoop = 1 << 6
+};
 
 class IMuseDigital : public MusicEngine {
 private:
 
-	int _callbackFps;
+	int _callbackFps;		// value how many times callback needs to be called per second
 
 	struct Track {
-		int trackId;
+		int trackId;		// used to identify track by value (0-15)
 
-		int8 pan;			// pan
-		int32 vol;			// volume
-		int32 volFadeDest;	//
-		int32 volFadeStep;	//
-		int32 volFadeDelay;	//
-		bool volFadeUsed;	//
+		int8 pan;			// panning value of sound
+		int32 vol;			// volume level (values 0-127 * 1000)
+		int32 volFadeDest;	// volume level which fading target (values 0-127 * 1000)
+		int32 volFadeStep;	// delta of step while changing volume at each imuse callback
+		int32 volFadeDelay;	// time in ms how long fading volume must be
+		bool volFadeUsed;	// flag if fading is in progress
 
-		int32 soundId;
-		char soundName[15];
-		bool used;
-		bool toBeRemoved;
-		bool readyToRemove;
-		bool started;
-		bool souStream;
-		bool compressed;
-		int32 priority;
-		int32 regionOffset;
-		int32 dataOffset;
-		int32 curRegion;
-		int32 curHookId;
-		int32 volGroupId;
-		int32 soundType;
-		int32 iteration;
-		int32 mod;
-		int32 mixerFlags;
+		int32 soundId;		// sound id used by scumm script
+		char soundName[15]; // sound name but also filename of sound in bundle data
+		bool used;			// flag mean that track is used
+		bool toBeRemoved;   // flag mean that track need to be free
+		bool readyToRemove; // flag mean that track is ready to stop
+		bool mixerStreamRunning;	// flag mean sound mixer's stream is running
+		bool souStreamUsed;	// flag mean that track use stream from sou file
+		bool sndDataExtComp;// flag mean that sound data is compressed by scummvm tools
+		int32 soundPriority;// priority level of played sound (0-127)
+		int32 regionOffset; // offset to sound data relative to begining of current region
+		int32 dataOffset;	// offset to sound data relative to begining of 'DATA' chunk
+		int32 curRegion;	// id of current used region
+		int32 curHookId;	// id of current used hook id
+		int32 volGroupId;	// id of volume group (IMUSE_VOLGRP_VOICE, IMUSE_VOLGRP_SFX, IMUSE_VOLGRP_MUSIC)
+		int32 soundType;	// type of sound data (kSpeechSoundType, kSFXSoundType, kMusicSoundType)
+		int32 feedSize;		// size of sound data needed to be filled at each callback iteration
+		int32 dataMod12Bit;	// value used between all callback to align 12 bit source of data
+		int32 mixerFlags;	// flags for sound mixer's channel (kFlagStereo, kFlag16Bits, kFlagReverseStereo, kFlagUnsigned, kFlagLittleEndian)
 
-		ImuseDigiSndMgr::soundStruct *soundHandle;
-		Audio::SoundHandle handle;
-		Audio::AppendableAudioStream *stream;
-		Audio::AudioStream *stream2;
+		ImuseDigiSndMgr::soundStruct *soundHandle;	// sound handle used by iMuse sound manager
+		Audio::SoundHandle mixChanHandle;					// sound mixer's channel handle
+		Audio::AppendableAudioStream *stream;		// sound mixer's audio stream handle for *.la1 and *.bun
+		Audio::AudioStream *streamSou;				// sound mixer's audio stream handle for *.sou
 
 		Track();
 	};
@@ -88,20 +109,22 @@ private:
 	Track *_track[MAX_DIGITAL_TRACKS + MAX_DIGITAL_FADETRACKS];
 
 	Common::Mutex _mutex;
-	ScummEngine *_vm;
+	ScummEngine_v7 *_vm;
+	Audio::Mixer *_mixer;
 	ImuseDigiSndMgr *_sound;
 
-	char *_audioNames;
-	int32 _numAudioNames;
+	char *_audioNames;		// filenames of sound SFX used in FT
+	int32 _numAudioNames;	// number of above filenames
 
-	bool _pause;
+	bool _pause;			// flag mean that iMuse callback should be idle
 
-	int32 _attributes[188];
-	int32 _nextSeqToPlay;
-	int32 _curMusicState;
-	int32 _curMusicSeq;
-	int32 _curMusicCue;
+	int32 _attributes[188];	// internal atributes for each music file to store and check later
+	int32 _nextSeqToPlay;	// id of sequence type of music needed played
+	int32 _curMusicState;	// current or previous id of music
+	int32 _curMusicSeq;		// current or previous id of sequence music
+	int32 _curMusicCue;		// current cue for current music. used in FT
 
+	int32 makeMixerFlags(int32 flags);
 	static void timer_handler(void *refConf);
 	void callback();
 	void switchToNextRegion(Track *track);
@@ -114,7 +137,7 @@ private:
 
 	int getSoundIdByName(const char *soundName);
 	void fadeOutMusic(int fadeDelay);
-	Track *cloneToFadeOutTrack(Track *track, int fadeDelay);
+	Track *cloneToFadeOutTrack(const Track *track, int fadeDelay);
 
 	void setFtMusicState(int stateId);
 	void setFtMusicSequence(int seqId);
@@ -130,7 +153,7 @@ private:
 	void playDigMusic(const char *songName, const imuseDigTable *table, int atribPos, bool sequence);
 
 public:
-	IMuseDigital(ScummEngine *scumm, int fps);
+	IMuseDigital(ScummEngine_v7 *scumm, Audio::Mixer *mixer, int fps);
 	virtual ~IMuseDigital();
 
 	void setAudioNames(int32 num, char *names);
@@ -141,7 +164,7 @@ public:
 	void startMusic(const char *soundName, int soundId, int hookId, int volume);
 	void startSfx(int soundId, int priority);
 	void startSound(int sound)
-		{ error("MusicEngine::startSound() Should be never called"); }
+		{ error("IMuseDigital::startSound(int) should be never called"); }
 
 	void saveOrLoad(Serializer *ser);
 	void resetState();
@@ -178,7 +201,7 @@ struct imuseRoomMap {
 };
 
 struct imuseDigTable {
-	byte opcode;
+	byte transitionType;
 	int16 soundId;
 	char name[20];
 	byte atribPos;
@@ -187,12 +210,12 @@ struct imuseDigTable {
 };
 
 struct imuseComiTable {
-	byte opcode;
+	byte transitionType;
 	int16 soundId;
 	char name[20];
 	byte atribPos;
 	byte hookId;
-	int16 fadeOut60TicksDelay;
+	int16 fadeOutDelay;
 	char filename[13];
 };
 
@@ -203,14 +226,14 @@ struct imuseFtNames {
 
 struct imuseFtStateTable {
 	char audioName[9];
-	byte opcode;
+	byte transitionType;
 	byte volume;
 	char name[21];
 };
 
 struct imuseFtSeqTable {
 	char audioName[9];
-	byte opcode;
+	byte transitionType;
 	byte volume;
 };
 
