@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2001-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/scumm/imuse_digi/dimuse_music.cpp $
- * $Id: dimuse_music.cpp 20779 2006-02-19 23:25:27Z aquadran $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/scumm/imuse_digi/dimuse_music.cpp $
+ * $Id: dimuse_music.cpp 27049 2007-06-02 09:07:29Z aquadran $
  */
 
 #include "common/stdafx.h"
@@ -107,7 +110,7 @@ void IMuseDigital::setDigMusicSequence(int seqId) {
 			_nextSeqToPlay = 0;
 			_attributes[DIG_SEQ_OFFSET + num] = 1;
 		} else {
-			if ((_digSeqMusicTable[_curMusicSeq].opcode == 4) || (_digSeqMusicTable[_curMusicSeq].opcode == 6)) {
+			if ((_digSeqMusicTable[_curMusicSeq].transitionType == 4) || (_digSeqMusicTable[_curMusicSeq].transitionType == 6)) {
 				_nextSeqToPlay = num;
 				return;
 			} else {
@@ -171,7 +174,7 @@ void IMuseDigital::playDigMusic(const char *songName, const imuseDigTable *table
 
 	fadeOutMusic(120);
 
-	switch(table->opcode) {
+	switch (table->transitionType) {
 		case 0:
 		case 5:
 		case 6:
@@ -249,7 +252,7 @@ void IMuseDigital::setComiMusicSequence(int seqId) {
 			playComiMusic(_comiSeqMusicTable[num].name, &_comiSeqMusicTable[num], 0, true);
 			_nextSeqToPlay = 0;
 		} else {
-			if ((_comiSeqMusicTable[_curMusicSeq].opcode == 4) || (_comiSeqMusicTable[_curMusicSeq].opcode == 6)) {
+			if ((_comiSeqMusicTable[_curMusicSeq].transitionType == 4) || (_comiSeqMusicTable[_curMusicSeq].transitionType == 6)) {
 				_nextSeqToPlay = num;
 				return;
 			} else {
@@ -292,7 +295,7 @@ void IMuseDigital::playComiMusic(const char *songName, const imuseComiTable *tab
 		}
 	}
 
-	switch (table->opcode) {
+	switch (table->transitionType) {
 		case 0:
 			fadeOutMusic(120);
 			break;
@@ -314,7 +317,7 @@ void IMuseDigital::playComiMusic(const char *songName, const imuseComiTable *tab
 				fadeOutMusic(60);
 				return;
 			}
-			fadeOutMusic(table->fadeOut60TicksDelay);
+			fadeOutMusic(table->fadeOutDelay);
 			startMusic(table->filename, table->soundId, table->hookId, 127);
 			break;
 		case 3:
@@ -324,13 +327,13 @@ void IMuseDigital::playComiMusic(const char *songName, const imuseComiTable *tab
 				fadeOutMusic(60);
 				return;
 			}
-			fadeOutMusic(table->fadeOut60TicksDelay);
+			fadeOutMusic(table->fadeOutDelay);
 			if ((!sequence) && (table->atribPos != 0) &&
 					(table->atribPos == _comiStateMusicTable[_curMusicState].atribPos)) {
 				startMusic(table->filename, table->soundId, 0, 127);
 				return;
 			}
-			if (table->opcode == 12) {
+			if (table->transitionType == 12) {
 				startMusic(table->filename, table->soundId, table->hookId, 127);
 			} else {
 				startMusic(table->filename, table->soundId, hookId, 127);
@@ -352,7 +355,7 @@ void IMuseDigital::setFtMusicState(int stateId) {
 		if (stateId == 0)
 			playFtMusic(NULL, 0, 0);
 		else
-			playFtMusic(_ftStateMusicTable[stateId].audioName, _ftStateMusicTable[stateId].opcode, _ftStateMusicTable[stateId].volume);
+			playFtMusic(_ftStateMusicTable[stateId].audioName, _ftStateMusicTable[stateId].transitionType, _ftStateMusicTable[stateId].volume);
 	}
 
 	_curMusicState = stateId;
@@ -371,11 +374,11 @@ void IMuseDigital::setFtMusicSequence(int seqId) {
 		if (_curMusicState == 0)
 			playFtMusic(NULL, 0, 0);
 		else {
-			playFtMusic(_ftStateMusicTable[_curMusicState].audioName, _ftStateMusicTable[_curMusicState].opcode, _ftStateMusicTable[_curMusicState].volume);
+			playFtMusic(_ftStateMusicTable[_curMusicState].audioName, _ftStateMusicTable[_curMusicState].transitionType, _ftStateMusicTable[_curMusicState].volume);
 		}
 	} else {
 		int seq = (seqId - 1) * 4;
-		playFtMusic(_ftSeqMusicTable[seq].audioName, _ftSeqMusicTable[seq].opcode, _ftSeqMusicTable[seq].volume);
+		playFtMusic(_ftSeqMusicTable[seq].audioName, _ftSeqMusicTable[seq].transitionType, _ftSeqMusicTable[seq].volume);
 	}
 
 	_curMusicSeq = seqId;
@@ -398,7 +401,7 @@ void IMuseDigital::setFtMusicCuePoint(int cueId) {
 		playFtMusic(NULL, 0, 0);
 	else {
 		int seq = ((_curMusicSeq - 1) * 4) + cueId;
-		playFtMusic(_ftSeqMusicTable[seq].audioName, _ftSeqMusicTable[seq].opcode, _ftSeqMusicTable[seq].volume);
+		playFtMusic(_ftSeqMusicTable[seq].audioName, _ftSeqMusicTable[seq].transitionType, _ftSeqMusicTable[seq].volume);
 	}
 
 	_curMusicCue = cueId;
@@ -425,7 +428,7 @@ int IMuseDigital::getSoundIdByName(const char *soundName) {
 void IMuseDigital::playFtMusic(const char *songName, int opcode, int volume) {
 	fadeOutMusic(200);
 
-	switch(opcode) {
+	switch (opcode) {
 		case 0:
 		case 4:
 			break;
