@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2001-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/scumm/imuse_digi/dimuse_script.cpp $
- * $Id: dimuse_script.cpp 22231 2006-04-29 22:33:31Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/scumm/imuse_digi/dimuse_script.cpp $
+ * $Id: dimuse_script.cpp 27039 2007-06-01 11:22:49Z aquadran $
  */
 
 #include "common/stdafx.h"
@@ -24,7 +27,7 @@
 #include "common/timer.h"
 
 #include "scumm/actor.h"
-#include "scumm/scumm.h"
+#include "scumm/intern.h"
 #include "scumm/sound.h"
 #include "scumm/imuse_digi/dimuse.h"
 #include "scumm/imuse_digi/dimuse_bndmgr.h"
@@ -165,26 +168,25 @@ void IMuseDigital::flushTracks() {
 	debug(5, "flushTracks()");
 	for (int l = 0; l < MAX_DIGITAL_TRACKS + MAX_DIGITAL_FADETRACKS; l++) {
 		Track *track = _track[l];
-		if (track->used && (track->readyToRemove ||
-				(_vm->_insaneRunning && track->toBeRemoved))) { // INSANE hack for sync timer mode
+		if (track->used && track->readyToRemove) {
 			if (track->stream) {
-				if (!track->stream->endOfStream()) {
-	 				track->stream->finish();
-	 			}
-				if (track->stream->endOfStream()
-						|| _vm->_mixer->isPaused() // hack for paused Mixer
-						|| _vm->_insaneRunning) { // INSANE hack for sync timer mode
-					_vm->_mixer->stopHandle(track->handle);
+				// Finalize the appendable stream
+ 				track->stream->finish();
+ 				// There might still be some data left in the buffers of the
+ 				// appendable stream. We play it nice and wait till all of it
+ 				// played.
+				if (track->stream->endOfStream()) {
+					_mixer->stopHandle(track->mixChanHandle);
 					delete track->stream;
 					track->stream = NULL;
 					_sound->closeSound(track->soundHandle);
 					track->soundHandle = NULL;
 					track->used = false;
 				}
-			} else if (track->stream2) {
-				_vm->_mixer->stopHandle(track->handle);
-				delete track->stream2;
-				track->stream2 = NULL;
+			} else if (track->streamSou) {
+				_mixer->stopHandle(track->mixChanHandle);
+				delete track->streamSou;
+				track->streamSou = NULL;
 				track->used = false;
 			}
 		}
@@ -271,7 +273,7 @@ int32 IMuseDigital::getPosInMs(int soundId) {
 	for (int l = 0; l < MAX_DIGITAL_TRACKS; l++) {
 		Track *track = _track[l];
 		if ((track->soundId == soundId) && track->used && !track->toBeRemoved) {
-			int32 pos = (5 * (track->dataOffset + track->regionOffset)) / (track->iteration / 200);
+			int32 pos = (5 * (track->dataOffset + track->regionOffset)) / (track->feedSize / 200);
 			return pos;
 		}
 	}
@@ -285,7 +287,7 @@ int IMuseDigital::getSoundStatus(int sound) const {
 	for (int l = 0; l < MAX_DIGITAL_TRACKS; l++) {
 		Track *track = _track[l];
 		if (track->soundId == sound) {
-			if ((track->stream2 && _vm->_mixer->isSoundHandleActive(track->handle)) ||
+			if ((track->streamSou && _mixer->isSoundHandleActive(track->mixChanHandle)) ||
 				(track->stream && track->used && !track->readyToRemove)) {
 					return 1;
 			}
@@ -381,24 +383,30 @@ int32 IMuseDigital::getCurMusicLipSyncHeight(int syncId) {
 }
 
 void IMuseDigital::stopAllSounds() {
-	debug(5, "IMuseDigital::stopAllSounds");
+	Common::StackLock lock(_mutex, "IMuseDigital::stopAllSounds()");
+	debug(0, "IMuseDigital::stopAllSounds");
 
-	for (;;) {
-		bool foundNotRemoved = false;
-		for (int l = 0; l < MAX_DIGITAL_TRACKS + MAX_DIGITAL_FADETRACKS; l++) {
-			Track *track = _track[l];
-			if (track->used) {
-				track->toBeRemoved = true;
-				foundNotRemoved = true;
+	for (int l = 0; l < MAX_DIGITAL_TRACKS + MAX_DIGITAL_FADETRACKS; l++) {
+		Track *track = _track[l];
+		if (track->used) {
+			// Stop the sound output, *now*. No need to use toBeRemoved etc.
+			// as we are protected by a mutex, and this method is never called
+			// from IMuseDigital::callback either.
+			if (track->stream) {
+				_mixer->stopHandle(track->mixChanHandle);
+				delete track->stream;
+				track->stream = NULL;
+				_sound->closeSound(track->soundHandle);
+				track->soundHandle = NULL;
+			} else if (track->streamSou) {
+				_mixer->stopHandle(track->mixChanHandle);
+				delete track->streamSou;
+				track->streamSou = NULL;
 			}
+			
+			// Mark the track as unused
+			track->used = false;
 		}
-		if (!foundNotRemoved)
-			break;
-		flushTracks();
-		_vm->_system->delayMillis(50);
-#if defined(_WIN32_WCE) || defined (PALMOS_MODE) || defined(__SYMBIAN32__)
-		_vm->parseEvents(); // timers are events, we need to consume them
-#endif
 	}
 }
 
