@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2001-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/scumm/imuse_digi/dimuse_sndmgr.cpp $
- * $Id: dimuse_sndmgr.cpp 22231 2006-04-29 22:33:31Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/scumm/imuse_digi/dimuse_sndmgr.cpp $
+ * $Id: dimuse_sndmgr.cpp 27039 2007-06-01 11:22:49Z aquadran $
  */
 
 #include "common/stdafx.h"
@@ -60,7 +63,7 @@ void ImuseDigiSndMgr::countElements(byte *ptr, int &numRegions, int &numJumps, i
 
 	do {
 		tag = READ_BE_UINT32(ptr); ptr += 4;
-		switch(tag) {
+		switch (tag) {
 		case MKID_BE('TEXT'):
 		case MKID_BE('STOP'):
 		case MKID_BE('FRMT'):
@@ -90,7 +93,7 @@ void ImuseDigiSndMgr::prepareSoundFromRMAP(Common::File *file, soundStruct *soun
 
 	file->seek(offset, SEEK_SET);
 	uint32 tag = file->readUint32BE();
-	assert(tag == 'RMAP');
+	assert(tag == MKID_BE('RMAP'));
 	int32 version = file->readUint32BE();
 	if (version != 2) {
 		error("ImuseDigiSndMgr::prepareSoundFromRMAP: Wrong version number, expected 2, but it's: %d.", version);
@@ -154,7 +157,7 @@ void ImuseDigiSndMgr::prepareSound(byte *ptr, soundStruct *sound) {
 			}
 			offset += 4;
 			len >>= 8;
-			switch(code) {
+			switch (code) {
 			case 0:
 				quit = true;
 				break;
@@ -211,7 +214,7 @@ void ImuseDigiSndMgr::prepareSound(byte *ptr, soundStruct *sound) {
 
 		do {
 			tag = READ_BE_UINT32(ptr); ptr += 4;
-			switch(tag) {
+			switch (tag) {
 			case MKID_BE('FRMT'):
 				ptr += 12;
 				sound->bits = READ_BE_UINT32(ptr); ptr += 4;
@@ -356,7 +359,7 @@ ImuseDigiSndMgr::soundStruct *ImuseDigiSndMgr::openSound(int32 soundId, const ch
 		assert(soundName[0] == 0);	// Paranoia check
 
 		_vm->ensureResourceLoaded(rtSound, soundId);
-		_vm->res.lock(rtSound, soundId);
+		_vm->_res->lock(rtSound, soundId);
 		ptr = _vm->getResourceAddress(rtSound, soundId);
 		if (ptr == NULL) {
 			closeSound(sound);
@@ -430,7 +433,7 @@ void ImuseDigiSndMgr::closeSound(soundStruct *soundHandle) {
 				found = true;
 		}
 		if (!found)
-			_vm->res.unlock(rtSound, soundHandle->soundId);
+			_vm->_res->unlock(rtSound, soundHandle->soundId);
 	}
 
 	if (soundHandle->compressedStream)
@@ -462,7 +465,7 @@ bool ImuseDigiSndMgr::checkForProperHandle(soundStruct *soundHandle) {
 	return false;
 }
 
-bool ImuseDigiSndMgr::isCompressed(soundStruct *soundHandle) {
+bool ImuseDigiSndMgr::isSndDataExtComp(soundStruct *soundHandle) {
 	assert(checkForProperHandle(soundHandle));
 	return soundHandle->compressed;
 }
@@ -583,11 +586,11 @@ int32 ImuseDigiSndMgr::getDataFromRegion(soundStruct *soundHandle, int region, b
 	if ((soundHandle->bundle) && (!soundHandle->compressed)) {
 		size = soundHandle->bundle->decompressSampleByCurIndex(start + offset, size, buf, header_size, header_outside);
 	} else if (soundHandle->resPtr) {
-		*buf = (byte *)malloc(size);
+		*buf = new byte[size];
 		assert(*buf);
 		memcpy(*buf, soundHandle->resPtr + start + offset + header_size, size);
 	} else if ((soundHandle->bundle) && (soundHandle->compressed)) {
-		*buf = (byte *)malloc(size);
+		*buf = new byte[size];
 		assert(*buf);
 		char fileName[24];
 		sprintf(fileName, "%s_reg%03d", soundHandle->name, region);
