@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2001-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,12 +18,12 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/scumm/imuse_digi/dimuse_sndmgr.h $
- * $Id: dimuse_sndmgr.h 22231 2006-04-29 22:33:31Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/scumm/imuse_digi/dimuse_sndmgr.h $
+ * $Id: dimuse_sndmgr.h 27039 2007-06-01 11:22:49Z aquadran $
  */
 
-#ifndef IMUSE_DIGI_SNDMGR_H
-#define IMUSE_DIGI_SNDMGR_H
+#ifndef SCUMM_IMUSE_DIGI_SNDMGR_H
+#define SCUMM_IMUSE_DIGI_SNDMGR_H
 
 #include "common/stdafx.h"
 #include "common/scummsys.h"
@@ -117,7 +120,7 @@ public:
 	void closeSound(soundStruct *soundHandle);
 	soundStruct *cloneSound(soundStruct *soundHandle);
 
-	bool isCompressed(soundStruct *soundHandle);
+	bool isSndDataExtComp(soundStruct *soundHandle);
 	int getFreq(soundStruct *soundHandle);
 	int getBits(soundStruct *soundHandle);
 	int getChannels(soundStruct *soundHandle);
