@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2001-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,15 +18,15 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/scumm/imuse_digi/dimuse_track.cpp $
- * $Id: dimuse_track.cpp 22526 2006-05-18 21:46:07Z eriktorbjorn $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/scumm/imuse_digi/dimuse_track.cpp $
+ * $Id: dimuse_track.cpp 27039 2007-06-01 11:22:49Z aquadran $
  */
 
 #include "common/stdafx.h"
 #include "common/timer.h"
 
 #include "scumm/actor.h"
-#include "scumm/scumm.h"
+#include "scumm/intern.h"
 #include "scumm/sound.h"
 #include "scumm/imuse_digi/dimuse.h"
 #include "scumm/imuse_digi/dimuse_bndmgr.h"
@@ -45,21 +48,21 @@ int IMuseDigital::allocSlot(int priority) {
 	}
 
 	if (trackId == -1) {
-		debug(5, "IMuseDigital::startSound(): All slots are full");
+		debug(5, "IMuseDigital::allocSlot(): All slots are full");
 		for (l = 0; l < MAX_DIGITAL_TRACKS; l++) {
 			Track *track = _track[l];
 			if (track->used && !track->toBeRemoved &&
-					(lowest_priority > track->priority) && !track->stream2) {
-				lowest_priority = track->priority;
+					(lowest_priority > track->soundPriority) && !track->streamSou) {
+				lowest_priority = track->soundPriority;
 				trackId = l;
 			}
 		}
 		if (lowest_priority <= priority) {
 			assert(trackId != -1);
 			_track[trackId]->toBeRemoved = true;
-			debug(5, "IMuseDigital::startSound(): Removed sound %d from track %d", _track[trackId]->soundId, trackId);
+			debug(5, "IMuseDigital::allocSlot(): Removed sound %d from track %d", _track[trackId]->soundId, trackId);
 		} else {
-			debug(5, "IMuseDigital::startSound(): Priority sound too low");
+			debug(5, "IMuseDigital::allocSlot(): Priority sound too low");
 			return -1;
 		}
 	}
@@ -95,14 +98,14 @@ void IMuseDigital::startSound(int soundId, const char *soundName, int soundType,
 	track->volFadeDelay = 0;
 	track->volFadeUsed = false;
 	track->soundId = soundId;
-	track->started = false;
+	track->mixerStreamRunning = false;
 	track->volGroupId = volGroupId;
 	track->curHookId = hookId;
-	track->priority = priority;
+	track->soundPriority = priority;
 	track->curRegion = -1;
 	track->dataOffset = 0;
 	track->regionOffset = 0;
-	track->mod = 0;
+	track->dataMod12Bit = 0;
 	track->mixerFlags = 0;
 	track->toBeRemoved = false;
 	track->readyToRemove = false;
@@ -111,18 +114,19 @@ void IMuseDigital::startSound(int soundId, const char *soundName, int soundType,
 	int bits = 0, freq = 0, channels = 0;
 
 	if (input) {
-		track->iteration = 0;
-		track->souStream = true;
+		track->feedSize = 0;
+		track->souStreamUsed = true;
 		track->soundName[0] = 0;
+		track->soundHandle = NULL;
 	} else {
-		track->souStream = false;
+		track->souStreamUsed = false;
 		strcpy(track->soundName, soundName);
 		track->soundHandle = _sound->openSound(soundId, soundName, soundType, volGroupId, -1);
 
 		if (track->soundHandle == NULL)
 			return;
 
-		track->compressed = _sound->isCompressed(track->soundHandle);
+		track->sndDataExtComp = _sound->isSndDataExtComp(track->soundHandle);
 
 		bits = _sound->getBits(track->soundHandle);
 		channels = _sound->getChannels(track->soundHandle);
@@ -141,28 +145,28 @@ void IMuseDigital::startSound(int soundId, const char *soundName, int soundType,
 		assert(channels == 1 || channels == 2);
 		assert(0 < freq && freq <= 65535);
 
-		track->iteration = freq * channels;
+		track->feedSize = freq * channels;
 		if (channels == 2)
-			track->mixerFlags = Audio::Mixer::FLAG_STEREO | Audio::Mixer::FLAG_REVERSE_STEREO;
+			track->mixerFlags = kFlagStereo | kFlagReverseStereo;
 
 		if ((bits == 12) || (bits == 16)) {
-			track->mixerFlags |= Audio::Mixer::FLAG_16BITS;
-			track->iteration *= 2;
+			track->mixerFlags |= kFlag16Bits;
+			track->feedSize *= 2;
 		} else if (bits == 8) {
-			track->mixerFlags |= Audio::Mixer::FLAG_UNSIGNED;
+			track->mixerFlags |= kFlagUnsigned;
 		} else
 			error("IMuseDigital::startSound(): Can't handle %d bit samples", bits);
 
 #ifdef SCUMM_LITTLE_ENDIAN
-		if (track->compressed)
-			track->mixerFlags |= Audio::Mixer::FLAG_LITTLE_ENDIAN;
+		if (track->sndDataExtComp)
+			track->mixerFlags |= kFlagLittleEndian;
 #endif
 	}
 
 	if (input) {
-		track->stream2 = input;
+		track->streamSou = input;
 		track->stream = NULL;
-		track->started = false;
+		track->mixerStreamRunning = false;
 	} else {
 		const int pan = (track->pan != 64) ? 2 * track->pan - 127 : 0;
 		const int vol = track->vol / 1000;
@@ -175,12 +179,10 @@ void IMuseDigital::startSound(int soundId, const char *soundName, int soundType,
 		if (track->volGroupId == 3)
 			type = Audio::Mixer::kMusicSoundType;
 
-		// setup 1 second stream wrapped buffer
-		int32 streamBufferSize = track->iteration;
-		track->stream2 = NULL;
-		track->stream = Audio::makeAppendableAudioStream(freq, track->mixerFlags, streamBufferSize);
-		_vm->_mixer->playInputStream(type, &track->handle, track->stream, -1, vol, pan, false);
-		track->started = true;
+		track->streamSou = NULL;
+		track->stream = Audio::makeAppendableAudioStream(freq, makeMixerFlags(track->mixerFlags));
+		_mixer->playInputStream(type, &track->mixChanHandle, track->stream, -1, vol, pan, false);
+		track->mixerStreamRunning = true;
 	}
 
 	track->used = true;
@@ -194,7 +196,7 @@ void IMuseDigital::setPriority(int soundId, int priority) {
 	for (int l = 0; l < MAX_DIGITAL_TRACKS; l++) {
 		Track *track = _track[l];
 		if ((track->soundId == soundId) && track->used && !track->toBeRemoved) {
-			track->priority = priority;
+			track->soundPriority = priority;
 		}
 	}
 }
@@ -306,12 +308,12 @@ void IMuseDigital::fadeOutMusic(int fadeDelay) {
 	}
 }
 
-IMuseDigital::Track *IMuseDigital::cloneToFadeOutTrack(Track *track, int fadeDelay) {
+IMuseDigital::Track *IMuseDigital::cloneToFadeOutTrack(const Track *track, int fadeDelay) {
 	Common::StackLock lock(_mutex, "IMuseDigital::cloneToFadeOutTrack()");
 	assert(track);
 	Track *fadeTrack = 0;
 
-	debug(5, "IMuseDigital::cloneToFadeOutTrack(%d, %d)", track->trackId, fadeDelay);
+	debug(0, "IMuseDigital::cloneToFadeOutTrack(%d, %d)", track->trackId, fadeDelay);
 
 	if (_track[track->trackId + MAX_DIGITAL_TRACKS]->used) {
 		warning("IMuseDigital::cloneToFadeOutTrack: Not free fade track");
@@ -319,47 +321,46 @@ IMuseDigital::Track *IMuseDigital::cloneToFadeOutTrack(Track *track, int fadeDel
 	}
 
 	fadeTrack = _track[track->trackId + MAX_DIGITAL_TRACKS];
-	fadeTrack->pan = track->pan;
-	fadeTrack->vol = track->vol;
-	fadeTrack->volGroupId = track->volGroupId;
-	fadeTrack->priority = track->priority;
-	fadeTrack->soundId = track->soundId;
-	fadeTrack->dataOffset = track->dataOffset;
-	fadeTrack->regionOffset = track->regionOffset;
-	fadeTrack->curRegion = track->curRegion;
-	fadeTrack->curHookId = track->curHookId;
-	fadeTrack->iteration = track->iteration;
-	fadeTrack->mixerFlags = track->mixerFlags;
-	fadeTrack->mod = track->mod;
-	fadeTrack->toBeRemoved = track->toBeRemoved;
-	fadeTrack->readyToRemove = track->readyToRemove;
-	fadeTrack->souStream = track->souStream;
-	fadeTrack->started = track->started;
-	fadeTrack->stream2 = track->stream2;
-	strcpy(fadeTrack->soundName, track->soundName);
-	fadeTrack->soundType = track->soundType;
+	
+	// Clone the settings of the given track
+	memcpy(fadeTrack, track, sizeof(Track));
+
+	// Clone the soundhandle
+	// FIXME: Shouldn't we check here whether track->soundHandle is NULL, resp. whether stream2
+	// is being used (as in, we are using compressed data)...
+	//
+	// -- aquadran -- nope :) this is called only for bundle files and sound data in *.la1
+	// from switchToNextRegion and fadeOutMusic func.
+	// stream2 is used only for sou VOICE type sound data (FT) --
 	fadeTrack->soundHandle = _sound->cloneSound(track->soundHandle);
 	assert(fadeTrack->soundHandle);
 
+	// Set the volume fading parameters to indicate a fade out
 	fadeTrack->volFadeDelay = fadeDelay;
 	fadeTrack->volFadeDest = 0;
 	fadeTrack->volFadeStep = (fadeTrack->volFadeDest - fadeTrack->vol) * 60 * (1000 / _callbackFps) / (1000 * fadeDelay);
 	fadeTrack->volFadeUsed = true;
 
-	Audio::Mixer::SoundType type = Audio::Mixer::kPlainSoundType;
-
-	if (fadeTrack->volGroupId == 1)
+	// Create an appendable output buffer
+	Audio::Mixer::SoundType type;
+	switch (fadeTrack->volGroupId) {
+	case 1:
 		type = Audio::Mixer::kSpeechSoundType;
-	if (fadeTrack->volGroupId == 2)
+		break;
+	case 2:
 		type = Audio::Mixer::kSFXSoundType;
-	if (fadeTrack->volGroupId == 3)
+		break;
+	case 3:
 		type = Audio::Mixer::kMusicSoundType;
+		break;
+	default:
+		type = Audio::Mixer::kPlainSoundType;
+		break;
+	}
+	fadeTrack->stream = Audio::makeAppendableAudioStream(_sound->getFreq(fadeTrack->soundHandle), makeMixerFlags(fadeTrack->mixerFlags));
+	_mixer->playInputStream(type, &fadeTrack->mixChanHandle, fadeTrack->stream, -1, fadeTrack->vol / 1000, fadeTrack->pan, false);
 
-	// setup 1 second stream wrapped buffer
-	int32 streamBufferSize = fadeTrack->iteration;
-	fadeTrack->stream = Audio::makeAppendableAudioStream(_sound->getFreq(fadeTrack->soundHandle), fadeTrack->mixerFlags, streamBufferSize);
-	_vm->_mixer->playInputStream(type, &fadeTrack->handle, fadeTrack->stream, -1, fadeTrack->vol / 1000, fadeTrack->pan, false);
-	fadeTrack->started = true;
+	fadeTrack->mixerStreamRunning = true;
 	fadeTrack->used = true;
 
 	return fadeTrack;
