@@ -1,6 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2001  Ludvig Strigeus
- * Copyright (C) 2001-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -16,14 +18,14 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/scumm/insane/insane.cpp $
- * $Id: insane.cpp 22417 2006-05-12 18:00:41Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/scumm/insane/insane.cpp $
+ * $Id: insane.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
 #include "common/stdafx.h"
 
-#include "base/engine.h"
+#include "engines/engine.h"
 
 #include "common/system.h"
 
@@ -37,7 +39,6 @@
 
 #include "scumm/smush/smush_player.h"
 #include "scumm/smush/smush_font.h"
-#include "scumm/smush/chunk_type.h"
 #include "scumm/smush/chunk.h"
 
 #include "scumm/insane/insane.h"
@@ -66,16 +67,12 @@ Insane::Insane(ScummEngine_v7 *scumm) {
 		readFileToMem("toranch.flu", &_smush_toranchFlu);
 		readFileToMem("minedriv.flu", &_smush_minedrivFlu);
 		readFileToMem("minefite.flu", &_smush_minefiteFlu);
-		_smush_bensgoggNut = new NutRenderer(_vm);
-		_smush_bensgoggNut->loadFont("bensgogg.nut");
-		_smush_bencutNut = new NutRenderer(_vm);
-		_smush_bencutNut->loadFont("bencut.nut");
+		_smush_bensgoggNut = new NutRenderer(_vm, "bensgogg.nut");
+		_smush_bencutNut = new NutRenderer(_vm, "bencut.nut");
 	}
 
-	_smush_iconsNut = new NutRenderer(_vm);
-	_smush_iconsNut->loadFont("icons.nut");
-	_smush_icons2Nut = new NutRenderer(_vm);
-	_smush_icons2Nut->loadFont("icons2.nut");
+	_smush_iconsNut = new NutRenderer(_vm, "icons.nut");
+	_smush_icons2Nut = new NutRenderer(_vm, "icons2.nut");
 }
 
 Insane::~Insane(void) {
@@ -620,7 +617,7 @@ void Insane::startVideo(const char *filename, int num, int argC, int frameRate,
 		smush_setupSanFromStart(filename, 0, -1, -1, 0);
 	}
 
-	_player->play(filename, offset, startFrame);
+	_player->play(filename, _speed, offset, startFrame);
 }
 
 void Insane::smush_warpMouse(int x, int y, int buttons) {
@@ -877,7 +874,7 @@ void Insane::smush_rewindCurrentSan(int arg_0, int arg_4, int arg_8) {
 }
 
 int32 Insane::weaponMaxRange(int32 actornum) {
-	static int map[8] = {135, 125, 130, 125, 120, 104, 104, 104};
+	static const int map[8] = {135, 125, 130, 125, 120, 104, 104, 104};
 
 	if (_actor[actornum].weapon == -1)
 		return 104;
@@ -886,7 +883,7 @@ int32 Insane::weaponMaxRange(int32 actornum) {
 }
 
 int32 Insane::weaponMinRange(int32 actornum) {
-	static int map[8] = {80, 40, 80, 40, 80, 80, 40, 50};
+	static const int map[8] = {80, 40, 80, 40, 80, 80, 40, 50};
 
 	if (_actor[actornum].weapon == -1)
 		return 40;
@@ -895,7 +892,7 @@ int32 Insane::weaponMinRange(int32 actornum) {
 }
 
 int32 Insane::weaponDamage(int32 actornum) {
-	static int map[8] = {20, 300, 25, 40, 15, 10, 10, 5};
+	static const int map[8] = {20, 300, 25, 40, 15, 10, 10, 5};
 
 	if (_actor[actornum].weapon == -1)
 		return 10;
@@ -934,7 +931,7 @@ void Insane::reinitActors(void) {
 }
 
 int Insane::calcTilt(int speed) {
-	const int tilt[7] = {-5, -4, -2, 0, 2, 4, 5};
+	static const int tilt[7] = {-5, -4, -2, 0, 2, 4, 5};
 	if (speed + 3 > 6)
 		return 0;
 
@@ -944,7 +941,7 @@ int Insane::calcTilt(int speed) {
 bool Insane::actor1StateFlags(int state) {
 	// This is compressed table. It contains indexes where state
 	// changes. I.e. 0-33: true, 34-38: false 39-72: true, etc.
-	const int spans[] = {0, 34, 39, 73, 89, 90, 92, 93, 99, 100, 117};
+	static const int spans[] = {0, 34, 39, 73, 89, 90, 92, 93, 99, 100, 117};
 	bool retvalue = 0;
 	unsigned int i;
 
@@ -1099,7 +1096,7 @@ void Insane::escapeKeyHandler(void) {
 }
 
 bool Insane::actor0StateFlags1(int state) {
-	const int spans[] = {0, 2, 34, 35, 39, 69, 98, 100, 117};
+	static const int spans[] = {0, 2, 34, 35, 39, 69, 98, 100, 117};
 	bool retvalue = 1;
 	unsigned int i;
 
@@ -1112,7 +1109,7 @@ bool Insane::actor0StateFlags1(int state) {
 }
 
 bool Insane::actor0StateFlags2(int state) {
-	const int spans[] = {0, 10, 14, 34, 39, 73, 75, 79, 81, 90, 93, 94,
+	static const int spans[] = {0, 10, 14, 34, 39, 73, 75, 79, 81, 90, 93, 94,
 		 98, 100, 117, 133, 136, 153, 158, 200, 202, 209, 212, 213, 217,
 		 219, 236, 256, 259, 272, 277, 311, 312, 315, 317, 328, 331, 332,
 		 336, 338, 355, 379, 382, 391, 396, 440, 441, 447, 450, 451, 455,
@@ -1148,7 +1145,7 @@ int Insane::smlayer_loadSound(int id, int flag, int phase) {
 	if (phase == 2)
 		_vm->ensureResourceLoaded(rtSound, resid);
 
-	_vm->res.setResourceCounter(rtSound, resid, 1);
+	_vm->_res->setResourceCounter(rtSound, resid, 1);
 
 	if (phase == 1) {
 		_objArray2Idx2++;
@@ -1170,7 +1167,7 @@ int Insane::smlayer_loadCostume(int id, int phase) {
 		return 0;
 
 	_vm->ensureResourceLoaded(rtCostume, resid);
-	_vm->res.setResourceCounter(rtCostume, resid, 1);
+	_vm->_res->setResourceCounter(rtCostume, resid, 1);
 
 	// smlayer_lock(rtCostume, resid); // FIXME
 
@@ -1256,7 +1253,7 @@ void Insane::smlayer_overrideDrawActorAt(byte *arg_0, byte arg_4, byte arg_8) {
 void Insane::smlayer_showStatusMsg(int32 arg_0, byte *renderBitmap, int32 codecparam,
 					   int32 pos_x, int32 pos_y, int32 arg_14, int32 arg_18,
 					   int32 flags, const char *formatString, const char *strng) {
-	SmushFont *sf = _player->_sf[0];
+	SmushFont *sf = _player->getFont(0);
 	int color = 1;
 	int32 top = 0;
 	char *str = NULL, *string;
@@ -1277,7 +1274,7 @@ void Insane::smlayer_showStatusMsg(int32 arg_0, byte *renderBitmap, int32 codecp
 			{
 				int id = str[3] - '0';
 				str += 4;
-				sf = _player->_sf[id];
+				sf = _player->getFont(id);
 			}
 			break;
 		case 'c':
@@ -1320,17 +1317,17 @@ void Insane::procSKIP(Chunk &b) {
 	_player->_skipNext = false;
 
 	if ((_vm->_game.features & GF_DEMO) && (_vm->_game.platform == Common::kPlatformPC)) {
-		_player->checkBlock(b, TYPE_SKIP, 2);
-		par1 = b.getWord();
+		_player->checkBlock(b, MKID_BE('SKIP'), 2);
+		par1 = b.readUint16LE();
 		if (isBitSet(par1))
 			_player->_skipNext = true;
 		return;
 	}
 
-	_player->checkBlock(b, TYPE_SKIP, 4);
+	_player->checkBlock(b, MKID_BE('SKIP'), 4);
 
-	par1 = b.getWord();
-	par2 = b.getWord();
+	par1 = b.readUint16LE();
+	par2 = b.readUint16LE();
 
 
 	if (!par2) {
@@ -1427,7 +1424,7 @@ int32 Insane::smush_setupSanWithFlu(const char *filename, int32 setupsan2, int32
 	_smush_setupsan1 = setupsan1;
 
 	/* skip FLUP marker */
-	if (READ_BE_UINT32(fluPtr) == 'FLUP')
+	if (READ_BE_UINT32(fluPtr) == MKID_BE('FLUP'))
 		tmp += 8;
 
 	_smush_setupsan2 = setupsan2;
