@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2002-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,15 +18,15 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/scumm/insane/insane.h $
- * $Id: insane.h 22417 2006-05-12 18:00:41Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/scumm/insane/insane.h $
+ * $Id: insane.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
-#if !defined(INSANE_H) && !defined(DISABLE_SCUMM_7_8)
-#define INSANE_H
+#if !defined(SCUMM_INSANE_H) && !defined(DISABLE_SCUMM_7_8)
+#define SCUMM_INSANE_H
 
-#include "base/engine.h"
+#include "engines/engine.h"
 #include "scumm/intern.h"
 #include "scumm/nut_renderer.h"
 
@@ -64,13 +67,14 @@ class Insane {
 	void procPostRendering(byte *renderBitmap, int32 codecparam, int32 setupsan12,
 						   int32 setupsan13, int32 curFrame, int32 maxFrame);
 	void procIACT(byte *renderBitmap, int32 codecparam, int32 setupsan12,
-				  int32 setupsan13, Chunk &b, int32 size, int32 flags);
+				  int32 setupsan13, Chunk &b, int32 size, int32 flags, int16 par1,
+				  int16 par2, int16 par3, int16 par4);
 	void procSKIP(Chunk &b);
 	void escapeKeyHandler(void);
 
  private:
 
-	ScummEngine_v6 *_vm;
+	ScummEngine_v7 *_vm;
 	SmushPlayer *_player;
 
 	int32 _speed;
@@ -79,7 +83,6 @@ class Insane {
 	int32 _numberArray;
    	int32 _emulTimerId;
 	int32 _emulateInterrupt;
-	byte _errbuf[0x7d0];
 	int32 _flag1d;
 	int32 _mainTimerId;
 	int32 _objArray1Idx;
@@ -431,17 +434,23 @@ class Insane {
 	void ouchSoundEnemy(void);
 	bool weaponEnemyIsEffective(void);
 	void iactScene1(byte *renderBitmap, int32 codecparam, int32 setupsan12,
-				  int32 setupsan13, Chunk &b, int32 size, int32 flags);
+				  int32 setupsan13, Chunk &b, int32 size, int32 flags,
+				  int16 par1, int16 par2, int16 par3, int16 par4);
 	void iactScene3(byte *renderBitmap, int32 codecparam, int32 setupsan12,
-				  int32 setupsan13, Chunk &b, int32 size, int32 flags);
+				  int32 setupsan13, Chunk &b, int32 size, int32 flags,
+				  int16 command, int16 par1, int16, int16);
 	void iactScene4(byte *renderBitmap, int32 codecparam, int32 setupsan12,
-				  int32 setupsan13, Chunk &b, int32 size, int32 flags);
+				  int32 setupsan13, Chunk &b, int32 size, int32 flags,
+				  int16 par1, int16 par2, int16 par3, int16 par4);
 	void iactScene6(byte *renderBitmap, int32 codecparam, int32 setupsan12,
-				  int32 setupsan13, Chunk &b, int32 size, int32 flags);
+				  int32 setupsan13, Chunk &b, int32 size, int32 flags,
+				  int16 par1, int16 par2, int16 par3, int16 par4);
 	void iactScene17(byte *renderBitmap, int32 codecparam, int32 setupsan12,
-				  int32 setupsan13, Chunk &b, int32 size, int32 flags);
+				  int32 setupsan13, Chunk &b, int32 size, int32 flags,
+				  int16 par1, int16 par2, int16 par3, int16 par4);
 	void iactScene21(byte *renderBitmap, int32 codecparam, int32 setupsan12,
-				  int32 setupsan13, Chunk &b, int32 size, int32 flags);
+				  int32 setupsan13, Chunk &b, int32 size, int32 flags,
+				  int16 par1, int16 par2, int16 par3, int16 par4);
 	bool isBitSet(int n);
 	void setBit(int n);
 	void clearBit(int n);
