@@ -1,6 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2001  Ludvig Strigeus
- * Copyright (C) 2001-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -16,14 +18,14 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/scumm/insane/insane_ben.cpp $
- * $Id: insane_ben.cpp 21506 2006-03-31 00:39:53Z kirben $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/scumm/insane/insane_ben.cpp $
+ * $Id: insane_ben.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
 #include "common/stdafx.h"
 
-#include "base/engine.h"
+#include "engines/engine.h"
 
 #include "scumm/insane/insane.h"
 
@@ -501,7 +503,7 @@ int32 Insane::calcBenDamage(bool arg_0, bool arg_4) {
 void Insane::actor02Reaction(int32 buttons) {
 	int32 tmp, tmp2;
 
-	switch(_actor[0].act[2].state) {
+	switch (_actor[0].act[2].state) {
 	case 1:
 		smlayer_setActorLayer(0, 2, 5);
 		_actor[0].weaponClass = 2;
