@@ -1,6 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2001  Ludvig Strigeus
- * Copyright (C) 2001-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -16,14 +18,14 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/scumm/insane/insane_enemy.cpp $
- * $Id: insane_enemy.cpp 21506 2006-03-31 00:39:53Z kirben $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/scumm/insane/insane_enemy.cpp $
+ * $Id: insane_enemy.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
 #include "common/stdafx.h"
 
-#include "base/engine.h"
+#include "engines/engine.h"
 
 #include "scumm/insane/insane.h"
 
@@ -33,31 +35,22 @@ int32 Insane::enemyInitializer(int num, int32 actor1, int32 actor2, int32 probab
 	switch (num) {
 	case EN_ROTT1:
 		return enemy0initializer(actor1, actor2, probability);
-		break;
 	case EN_ROTT2:
 		return enemy1initializer(actor1, actor2, probability);
-		break;
 	case EN_ROTT3:
 		return enemy2initializer(actor1, actor2, probability);
-		break;
 	case EN_VULTF1:
 		return enemy3initializer(actor1, actor2, probability);
-		break;
 	case EN_VULTM1:
 		return enemy4initializer(actor1, actor2, probability);
-		break;
 	case EN_VULTF2:
 		return enemy5initializer(actor1, actor2, probability);
-		break;
 	case EN_VULTM2:
 		return enemy6initializer(actor1, actor2, probability);
-		break;
 	case EN_CAVEFISH:
 		return enemy7initializer(actor1, actor2, probability);
-		break;
 	case EN_TORQUE:
 		return enemy8initializer(actor1, actor2, probability);
-		break;
 	case -1:
 		// nothing
 		break;
@@ -71,34 +64,24 @@ int32 Insane::enemyHandler(int num, int32 actor1, int32 actor2, int32 probabilit
 	switch (num) {
 	case EN_ROTT1:
 		return enemy0handler(actor1, actor2, probability);
-		break;
 	case EN_ROTT2:
 		return enemy1handler(actor1, actor2, probability);
-		break;
 	case EN_ROTT3:
 		return enemy2handler(actor1, actor2, probability);
-		break;
 	case EN_VULTF1:
 		return enemy3handler(actor1, actor2, probability);
-		break;
 	case EN_VULTM1:
 		return enemy4handler(actor1, actor2, probability);
-		break;
 	case EN_VULTF2:
 		return enemy5handler(actor1, actor2, probability);
-		break;
 	case EN_VULTM2:
 		return enemy6handler(actor1, actor2, probability);
-		break;
 	case EN_CAVEFISH:
 		return enemy7handler(actor1, actor2, probability);
-		break;
 	case EN_TORQUE:
 		return enemy8handler(actor1, actor2, probability);
-		break;
 	case EN_BEN:
 		return enemyBenHandler(actor1, actor2, probability);
-		break;
 	case -1:
 		// nothing
 		break;
@@ -197,7 +180,7 @@ int32 Insane::enemy0handler(int32 actor1, int32 actor2, int32 probability) {
 					break;
 				}
 			} else {
-				switch(_vm->_rnd.getRandomNumber(14)) {
+				switch (_vm->_rnd.getRandomNumber(14)) {
 				case 2:
 					if (!_enemyState[EN_ROTT1][2]) {
 						_enemyState[EN_ROTT1][2] = 1;
@@ -361,7 +344,7 @@ int32 Insane::enemy1handler(int32 actor1, int32 actor2, int32 probability) {
 					break;
 				}
 			} else {
-				switch(_vm->_rnd.getRandomNumber(14)) {
+				switch (_vm->_rnd.getRandomNumber(14)) {
 				case 2:
 					if (!_enemyState[EN_ROTT2][2]) {
 						_enemyState[EN_ROTT2][2] = 1;
@@ -1372,7 +1355,7 @@ void Insane::actor11Reaction(int32 buttons) {
 
 	chooseEnemyWeaponAnim(buttons);
 
-	switch(_actor[1].tilt) {
+	switch (_actor[1].tilt) {
 	case -3:
 		if (_actor[1].act[1].state != 41 || _actor[1].weaponClass != _actor[1].animWeaponClass) {
 			setEnemyAnimation(1, 6);
@@ -1636,7 +1619,7 @@ void Insane::setEnemyState(void) {
 void Insane::actor12Reaction(int32 buttons) {
 	int32 tmp, tmp2;
 
-	switch(_actor[1].act[2].state) {
+	switch (_actor[1].act[2].state) {
 	case 1:
 		smlayer_setActorLayer(1, 2, 5);
 		_actor[1].weaponClass = 2;
