@@ -1,6 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2001  Ludvig Strigeus
- * Copyright (C) 2001-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -16,19 +18,18 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/scumm/insane/insane_iact.cpp $
- * $Id: insane_iact.cpp 21504 2006-03-30 13:47:18Z kirben $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/scumm/insane/insane_iact.cpp $
+ * $Id: insane_iact.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
 #include "common/stdafx.h"
 
-#include "base/engine.h"
+#include "engines/engine.h"
 
 #include "scumm/scumm.h"
 
 #include "scumm/smush/smush_player.h"
-#include "scumm/smush/chunk_type.h"
 #include "scumm/smush/chunk.h"
 
 #include "scumm/insane/insane.h"
@@ -36,51 +37,46 @@
 namespace Scumm {
 
 void Insane::procIACT(byte *renderBitmap, int32 codecparam, int32 setupsan12,
-					  int32 setupsan13, Chunk &b, int32 size, int32 flags) {
+					  int32 setupsan13, Chunk &b, int32 size, int32 flags,
+					  int16 par1, int16 par2, int16 par3, int16 par4) {
 	if (_keyboardDisable)
 		return;
 
 	switch (_currSceneId) {
 	case 1:
-		iactScene1(renderBitmap, codecparam, setupsan12, setupsan13, b, size, flags);
+		iactScene1(renderBitmap, codecparam, setupsan12, setupsan13, b, size, flags, par1, par2, par3, par4);
 		break;
 	case 3:
 	case 13:
-		iactScene3(renderBitmap, codecparam, setupsan12, setupsan13, b, size, flags);
+		iactScene3(renderBitmap, codecparam, setupsan12, setupsan13, b, size, flags, par1, par2, par3, par4);
 		break;
 	case 4:
 	case 5:
-		iactScene4(renderBitmap, codecparam, setupsan12, setupsan13, b, size, flags);
+		iactScene4(renderBitmap, codecparam, setupsan12, setupsan13, b, size, flags, par1, par2, par3, par4);
 		break;
 	case 6:
-		iactScene6(renderBitmap, codecparam, setupsan12, setupsan13, b, size, flags);
+		iactScene6(renderBitmap, codecparam, setupsan12, setupsan13, b, size, flags, par1, par2, par3, par4);
 		break;
 	case 17:
-		iactScene17(renderBitmap, codecparam, setupsan12, setupsan13, b, size, flags);
+		iactScene17(renderBitmap, codecparam, setupsan12, setupsan13, b, size, flags, par1, par2, par3, par4);
 		break;
 	case 21:
-		iactScene21(renderBitmap, codecparam, setupsan12, setupsan13, b, size, flags);
+		iactScene21(renderBitmap, codecparam, setupsan12, setupsan13, b, size, flags, par1, par2, par3, par4);
 		break;
 	}
 }
 
 void Insane::iactScene1(byte *renderBitmap, int32 codecparam, int32 setupsan12,
-					  int32 setupsan13, Chunk &b, int32 size, int32 flags) {
-	_player->checkBlock(b, TYPE_IACT, 8);
-
-	int16 par1, par2, par3, par4, par5, par6, par7, par9, par11, par13, tmp;
-
-	par1 = b.getWord(); // cx
-	par2 = b.getWord(); // dx
-	par3 = b.getWord(); // si
-	par4 = b.getWord(); // bx
+					  int32 setupsan13, Chunk &b, int32 size, int32 flags,
+					  int16 par1, int16 par2, int16 par3, int16 par4) {
+	int16 par5, par6, par7, par9, par11, par13, tmp;
 
 	switch (par1) {
 	case 2: // PATCH
 		if (par3 != 1)
 			break;
 
-		par5 = b.getWord(); // si
+		par5 = b.readUint16LE(); // si
 		if (_actor[0].field_8 == 112) {
 			setBit(par5);
 			break;
@@ -98,27 +94,27 @@ void Insane::iactScene1(byte *renderBitmap, int32 codecparam, int32 setupsan12,
 		break;
 	case 3:
 		if (par3 == 1) {
-			setBit(b.getWord());
+			setBit(b.readUint16LE());
 			_approachAnim = -1;
 		}
 		break;
 	case 4:
 		if (par3 == 1 && (_approachAnim < 0 || _approachAnim > 4))
-			setBit(b.getWord());
+			setBit(b.readUint16LE());
 		break;
 	case 5:
 		if (par2 != 13)
 			break;
 
-		tmp = b.getWord();   // +8
-		tmp = b.getWord();   // +10
-		par7 = b.getWord();  // +12 dx
-		tmp = b.getWord();   // +14
-		par9 = b.getWord();  // +16 bx
-		tmp = b.getWord();   // +18
-		par11 = b.getWord(); // +20 cx
-		tmp = b.getWord();   // +22
-		par13 = b.getWord(); // +24 ax
+		tmp = b.readUint16LE();   // +8
+		tmp = b.readUint16LE();   // +10
+		par7 = b.readUint16LE();  // +12 dx
+		tmp = b.readUint16LE();   // +14
+		par9 = b.readUint16LE();  // +16 bx
+		tmp = b.readUint16LE();   // +18
+		par11 = b.readUint16LE(); // +20 cx
+		tmp = b.readUint16LE();   // +22
+		par13 = b.readUint16LE(); // +24 ax
 		
 		if (par13 > _actor[0].x || par11 < _actor[0].x) {
 			_tiresRustle = true;
@@ -149,8 +145,8 @@ void Insane::iactScene1(byte *renderBitmap, int32 codecparam, int32 setupsan12,
 								 _continueFrame1, 1300);
 			break;
 		case 9:
-			par5 = b.getWord(); // si
-			par6 = b.getWord(); // bx
+			par5 = b.readUint16LE(); // si
+			par6 = b.readUint16LE(); // bx
 			smlayer_setFluPalette(_smush_roadrsh3Rip, 0);
 			if (par5 == par6 - 1)
 				smlayer_setFluPalette(_smush_roadrashRip, 0);
@@ -159,10 +155,10 @@ void Insane::iactScene1(byte *renderBitmap, int32 codecparam, int32 setupsan12,
 	case 7:
 		switch (par4) {
 		case 1:
-			_actor[0].x -= (b.getWord() - 160) / 10;
+			_actor[0].x -= (b.readUint16LE() - 160) / 10;
 			break;
 		case 2:
-			par5 = b.getWord();
+			par5 = b.readUint16LE();
 
 			if (par5 - 8 > _actor[0].x || par5 + 8 < _actor[0].x) {
 				if (smlayer_isSoundRunning(86))
@@ -298,18 +294,13 @@ void Insane::removeEnemyFromMetList(int32 enemy1) {
 }
 
 void Insane::iactScene3(byte *renderBitmap, int32 codecparam, int32 setupsan12,
-					  int32 setupsan13, Chunk &b, int32 size, int32 flags) {
-	_player->checkBlock(b, TYPE_IACT, 8);
-
-	int command, par1, par2, par3, tmp;
-	command = b.getWord();
-	par1 = b.getWord();
+					  int32 setupsan13, Chunk &b, int32 size, int32 flags,
+					  int16 command, int16 par1, int16, int16) {
+	int par2, par3;
 	if (command == 6) {
 		if (par1 == 9) {
-			tmp = b.getWord();  // ptr + 4
-			tmp = b.getWord();  // ptr + 6
-			par2 = b.getWord(); // ptr + 8
-			par3 = b.getWord(); // ptr + 10
+			par2 = b.readUint16LE(); // ptr + 8
+			par3 = b.readUint16LE(); // ptr + 10
 
 			if (!par2)
 				smlayer_setFluPalette(_smush_roadrsh3Rip, 0);
@@ -326,20 +317,14 @@ void Insane::iactScene3(byte *renderBitmap, int32 codecparam, int32 setupsan12,
 }
 
 void Insane::iactScene4(byte *renderBitmap, int32 codecparam, int32 setupsan12,
-					  int32 setupsan13, Chunk &b, int32 size, int32 flags) {
-	_player->checkBlock(b, TYPE_IACT, 8);
-
-	int16 par1, par2, par3, par4, par5;
-
-	par1 = b.getWord(); // edx
-	par2 = b.getWord(); // bx
-	par3 = b.getWord();
-	par4 = b.getWord(); // cx
+					  int32 setupsan13, Chunk &b, int32 size, int32 flags,
+					  int16 par1, int16 par2, int16 par3, int16 par4) {
+	int16 par5;
 
 	switch (par1) {
 	case 2:
 	case 4:
-		par5 = b.getWord(); // si
+		par5 = b.readUint16LE(); // si
 		switch (par3) {
 		case 1:
 			if (par4 == 1) {
@@ -408,19 +393,13 @@ void Insane::iactScene4(byte *renderBitmap, int32 codecparam, int32 setupsan12,
 }
 
 void Insane::iactScene6(byte *renderBitmap, int32 codecparam, int32 setupsan12,
-					  int32 setupsan13, Chunk &b, int32 size, int32 flags) {
-	_player->checkBlock(b, TYPE_IACT, 8);
-
-	int16 par1, par2, par3, par4, par5;
-
-	par1 = b.getWord();
-	par2 = b.getWord(); // bx
-	par3 = b.getWord();
-	par4 = b.getWord();
+					  int32 setupsan13, Chunk &b, int32 size, int32 flags,
+					  int16 par1, int16 par2, int16 par3, int16 par4) {
+	int16 par5;
 
 	switch (par1) {
 	case 7:
-		par5 = b.getWord();
+		par5 = b.readUint16LE();
 		if (par4 != 3)
 			break;
 
@@ -431,7 +410,7 @@ void Insane::iactScene6(byte *renderBitmap, int32 codecparam, int32 setupsan12,
 		break;
 	case 2:
 	case 4:
-		par5 = b.getWord();
+		par5 = b.readUint16LE();
 		switch (par3) {
 		case 1:
 			if (par4 == 1) {
@@ -499,21 +478,14 @@ void Insane::iactScene6(byte *renderBitmap, int32 codecparam, int32 setupsan12,
 }
 
 void Insane::iactScene17(byte *renderBitmap, int32 codecparam, int32 setupsan12,
-					  int32 setupsan13, Chunk &b, int32 size, int32 flags) {
-	_player->checkBlock(b, TYPE_IACT, 8);
-	int16 par1, par2, par3, par4;
-
-	par1 = b.getWord(); // dx
-	par2 = b.getWord(); // cx
-	par3 = b.getWord(); // di
-	par4 = b.getWord();
-
+					  int32 setupsan13, Chunk &b, int32 size, int32 flags,
+					  int16 par1, int16 par2, int16 par3, int16 par4) {
 	switch (par1) {
 	case 2:
 	case 3:
 	case 4:
 		if (par3 == 1) {
-			setBit(b.getWord());
+			setBit(b.readUint16LE());
 			_approachAnim = -1;
 		}
 		break;
@@ -552,7 +524,8 @@ void Insane::iactScene17(byte *renderBitmap, int32 codecparam, int32 setupsan12,
 }
 
 void Insane::iactScene21(byte *renderBitmap, int32 codecparam, int32 setupsan12,
-					  int32 setupsan13, Chunk &b, int32 size, int32 flags) {
+					  int32 setupsan13, Chunk &b, int32 size, int32 flags,
+  					  int16 par1, int16 par2, int16 par3, int16 par4) {
 	// void implementation
 }
 
