@@ -1,6 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2001  Ludvig Strigeus
- * Copyright (C) 2001-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -16,14 +18,14 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/scumm/insane/insane_scenes.cpp $
- * $Id: insane_scenes.cpp 24293 2006-10-13 09:17:35Z sev $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/scumm/insane/insane_scenes.cpp $
+ * $Id: insane_scenes.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
 #include "common/stdafx.h"
 
-#include "base/engine.h"
+#include "engines/engine.h"
 
 #include "common/config-manager.h"
 
@@ -36,7 +38,7 @@ namespace Scumm {
 
 void Insane::runScene(int arraynum) {
 	_insaneIsRunning = true;
-	_player = new SmushPlayer(_vm, _speed);
+	_player = _vm->_splayer;
 	_player->insanity(true);
 
 	_numberArray = arraynum;
@@ -171,8 +173,6 @@ void Insane::runScene(int arraynum) {
 	}
 	// insane_unlock(); // FIXME
 	_vm->_sound->stopAllSounds(); // IMUSE_StopAllSounds();
-
-	delete _player;
 }
 
 int Insane::initScene(int sceneId) {
@@ -345,7 +345,7 @@ int Insane::loadSceneData(int scene, int flag, int phase) {
 		break;
 	case 3:
 	case 13:
-		switch(_currEnemy) {
+		switch (_currEnemy) {
 		case EN_TORQUE:
 			smlayer_loadSound(59, flag, phase);
 			smlayer_loadSound(93, flag, phase);
@@ -633,12 +633,10 @@ void Insane::setSceneCostumes(int sceneId) {
 		smlayer_setFluPalette(_smush_roadrashRip, 0);
 		setupValues();
 		return;
-		break;
 	case 17:
 		smlayer_setFluPalette(_smush_goglpaltRip, 0);
 		setupValues();
 		return;
-		break;
 	case 2:
 		if ((_vm->_game.features & GF_DEMO) && (_vm->_game.platform == Common::kPlatformPC))
 			smlayer_setActorCostume(0, 2, readArray(9));
@@ -646,19 +644,16 @@ void Insane::setSceneCostumes(int sceneId) {
 			smlayer_setActorCostume(0, 2, readArray(10));
 		setupValues();
 		return;
-		break;
 	case 13:
 		setEnemyCostumes();
 		smlayer_setFluPalette(_smush_roadrashRip, 0);
 		return;
-		break;
 	case 21:
 		_currEnemy = EN_ROTT3; //PATCH
 		setEnemyCostumes();
 		_actor[1].y = 200;
 		smlayer_setFluPalette(_smush_roadrashRip, 0);
 		return;
-		break;
 	case 4:
 	case 5:
 	case 6:
@@ -669,7 +664,6 @@ void Insane::setSceneCostumes(int sceneId) {
 		smlayer_putActor(0, 2, _actor[0].x, _actor[0].y1+190, _smlayer_room2);
 		setupValues();
 		return;
-		break;
 	case 7:
 	case 8:
 		writeArray(4, 0);
