@@ -1,6 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2001  Ludvig Strigeus
- * Copyright (C) 2001-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -16,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/scumm/intern.h $
- * $Id: intern.h 23590 2006-07-24 12:52:53Z kirben $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/scumm/intern.h $
+ * $Id: intern.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -28,6 +30,10 @@
 
 namespace Scumm {
 
+class Insane;
+class SmushMixer;
+class SmushPlayer;
+
 // This is to help devices with small memory (PDA, smartphones, ...)
 // to save abit of memory used by opcode names in the Scumm engine.
 #ifndef REDUCE_MEMORY_USAGE
@@ -55,6 +61,8 @@ protected:
 		uint16 xStrips, yStrips;
 		bool isDrawn;
 	} _flashlight;
+	
+	char _saveLoadVarsFilename[256];
 
 public:
 	ScummEngine_v5(OSystem *syst, const DetectorResult &dr);
@@ -78,6 +86,8 @@ protected:
 	int getWordVararg(int *ptr);
 	void saveVars();
 	void loadVars();
+	void saveIQPoints();
+	void loadIQPoints();
 
 	virtual int getVar();
 	virtual int getVarOrDirectByte(byte mask);
@@ -85,7 +95,7 @@ protected:
 
 	virtual void animateCursor();
 
-	void setBuiltinCursor(int index);
+	virtual void setBuiltinCursor(int index);
 	void redefineBuiltinCursorFromChar(int index, int chr);
 	void redefineBuiltinCursorHotspot(int index, int x, int y);
 
@@ -210,6 +220,7 @@ public:
 	virtual void resetScumm();
 
 protected:
+	virtual void readResTypeList(int id);
 	virtual void readIndexFile();
 	virtual void loadCharset(int no);
 	virtual void resetRoomObjects();
@@ -229,6 +240,8 @@ public:
 protected:
 	virtual void readRoomsOffsets();
 	virtual void loadCharset(int no);
+
+	virtual void processKeyboard(int lastKeyHit);
 };
 
 /**
@@ -239,7 +252,7 @@ public:
 	ScummEngine_v3old(OSystem *syst, const DetectorResult &dr);
 
 protected:
-	virtual void readResTypeList(int id, const char *name);
+	virtual void readResTypeList(int id);
 	virtual void readIndexFile();
 	virtual void setupRoomSubBlocks();
 	virtual void resetRoomSubBlocks();
@@ -267,6 +280,12 @@ protected:
 
 	V2MouseoverBox _mouseOverBoxesV2[7];
 	int8 _mouseOverBoxV2;
+	
+	char _sentenceBuf[256];
+
+	int _activeInventory;
+	int _activeObject;
+	int _activeVerb;
 
 public:
 	ScummEngine_v2(OSystem *syst, const DetectorResult &dr);
@@ -286,6 +305,8 @@ protected:
 	virtual void resetScummVars();
 	virtual void decodeParseString();
 
+	virtual void processKeyboard(int lastKeyHit);
+
 	virtual void readIndexFile();
 	void readClassicIndexFile();	// V1
 	void readEnhancedIndexFile();	// V2
@@ -314,6 +335,8 @@ protected:
 	void initV2MouseOver();
 	void initNESMouseOver();
 
+	virtual void setBuiltinCursor(int index);
+
 	void runObject(int obj, int entry);
 
 	/* Version 2 script opcodes */
@@ -321,7 +344,6 @@ protected:
 	void o2_actorOps();
 	void o2_add();
 	void o2_addIndirect();
-	void o2_animateActor();
 	void o2_assignVarByte();
 	void o2_assignVarWordIndirect();
 	void o2_beginOverride();
@@ -402,11 +424,11 @@ protected:
 };
 
 /**
- * Engine for Commodore 64 version of Maniac Mansion
+ * Engine for Apple II and Commodore 64 versions of Maniac Mansion
  */
-class ScummEngine_c64 : public ScummEngine_v2 {
+class ScummEngine_v0 : public ScummEngine_v2 {
 protected:
-	typedef void (ScummEngine_c64::*OpcodeProcC64)();
+	typedef void (ScummEngine_v0::*OpcodeProcC64)();
 	struct OpcodeEntryC64 {
 		OpcodeProcC64 proc;
 		const char *desc;
@@ -416,7 +438,7 @@ protected:
 
 	int _currentMode;
 public:
-	ScummEngine_c64(OSystem *syst, const DetectorResult &dr);
+	ScummEngine_v0(OSystem *syst, const DetectorResult &dr);
 
 	virtual void resetScumm();
 
@@ -431,13 +453,18 @@ protected:
 	virtual void resetScummVars();
 	virtual void decodeParseString();
 
+	virtual void processInput();
+
 	virtual void saveOrLoad(Serializer *s);
 
 	virtual void checkExecVerbs();
 	virtual void handleMouseOver(bool updateInventory);
-	void initC64Verbs();
+	void resetVerbs();
+	void setNewKidVerbs();
 	void drawSentence();
 
+	void switchActor(int slot);
+
 	virtual int getVarOrDirectWord(byte mask);
 	virtual uint fetchScriptWord();
 
@@ -520,25 +547,19 @@ protected:
 		kDwordArray = 6
 	};
 
-	#if !defined(__GNUC__)
-		#pragma START_PACK_STRUCTS
-	#endif
+#include "common/pack-start.h"	// START STRUCT PACKING
 
 	struct ArrayHeader {
 		int16 dim1;
 		int16 type;
 		int16 dim2;
 		byte data[1];
-	} GCC_PACK;
+	};
 
-	#if !defined(__GNUC__)
-		#pragma END_PACK_STRUCTS
-	#endif
+#include "common/pack-end.h"	// END STRUCT PACKING
 
 	const OpcodeEntryV6 *_opcodesV6;
 
-	int _smushFrameRate;
-
 	struct TextObject {
 		int16 xpos, ypos;
 		byte color;
@@ -575,6 +596,9 @@ protected:
 	} _akosQueue[32];
 	int16 _akosQueuePos;
 
+	byte _curActor;
+	int _curVerb;
+	int _curVerbSlot;
 
 public:
 	ScummEngine_v6(OSystem *syst, const DetectorResult &dr);
@@ -587,6 +611,7 @@ protected:
 	virtual const char *getOpcodeDesc(byte i);
 
 	virtual void scummLoop_handleActors();
+	virtual void processKeyboard(int lastKeyHit);
 
 	virtual void setupScummVars();
 	virtual void decodeParseString(int a, int b);
@@ -602,7 +627,7 @@ protected:
 	int popRoomAndObj(int *room);
 
 	ArrayHeader *getArray(int array);
-	ArrayHeader *defineArray(int array, int type, int dim2, int dim1);
+	byte *defineArray(int array, int type, int dim2, int dim1);
 	int findFreeArrayId();
 	void nukeArray(int array);
 	virtual int readArray(int array, int index, int base);
@@ -640,6 +665,10 @@ protected:
 
 	virtual void processActors();
 
+	int getSpecialBox(int x, int y);
+
+	int getDistanceBetween(bool is_obj_1, int b, int c, bool is_obj_2, int e, int f);
+
 	/* Version 6 script opcodes */
 	void o6_setBlastObjectWindow();
 	void o6_pushByte();
@@ -754,7 +783,7 @@ protected:
 	void o6_getActorWidth();
 	void o6_wait();
 	void o6_getActorScaleX();
-	void o6_getActorAnimCounter1();
+	void o6_getActorAnimCounter();
 	void o6_soundKludge();
 	void o6_isAnyOf();
 	void o6_systemOps();
@@ -780,7 +809,7 @@ protected:
 	void o6_distObjectObject();
 	void o6_distObjectPt();
 	void o6_distPtPt();
-	void o6_kernelSetFunctions();
+	virtual void o6_kernelSetFunctions();
 	void o6_delayFrames();
 	void o6_pickOneOf();
 	void o6_pickOneOfDefault();
@@ -815,16 +844,41 @@ protected:
 
 #ifndef DISABLE_SCUMM_7_8
 class ScummEngine_v7 : public ScummEngine_v6 {
+	friend class SmushPlayer;
+	friend class Insane;
 public:
 	ScummEngine_v7(OSystem *syst, const DetectorResult &dr);
 	~ScummEngine_v7();
 
+
+protected:
+	int _smushFrameRate;
+
+	/**
+	 * Flag which signals that the SMUSH video playback should end now
+	 * (e.g. because it was aborted by the user or it's simply finished).
+	 */
+	bool _smushVideoShouldFinish;
+
+	bool _smushActive;
+
+	Insane *_insane;
+
+public:
+	/** This flag tells IMuseDigital that INSANE is running. */
+	bool _insaneRunning;	// Used by IMuseDigital::flushTracks()
+
+	SmushMixer *_smixer;
+	SmushPlayer *_splayer;
+
+
 	struct LangIndexNode {
 		char tag[12+1];
 		int32 offset;
 	};
 	
-	int _verbCharset, _verbLineSpacing;
+protected:
+	int _verbLineSpacing;
 	bool _existLanguageFile;
 	char *_languageBuffer;
 	LangIndexNode *_languageIndex;
@@ -848,13 +902,19 @@ public:
 	int _subtitleQueuePos;
 	SubtitleText _subtitleQueue[20];
 
+public:
 	void processSubtitleQueue();
 	void addSubtitleToQueue(const byte *text, const Common::Point &pos, byte color, byte charset);
 	void clearSubtitleQueue();
 
 protected:
+	virtual int runDialog(Dialog &dialog);
+
 	virtual void scummLoop_handleSound();
 	virtual void scummLoop_handleDrawing();
+	virtual void processKeyboard(int lastKeyHit);
+
+	virtual void setupScumm();
 
 	virtual void setupScummVars();
 	virtual void resetScummVars();
@@ -880,6 +940,9 @@ protected:
 	void playSpeech(const byte *ptr);
 
 	virtual void drawVerb(int verb, int mode);
+
+
+	virtual void o6_kernelSetFunctions();
 };
 
 class ScummEngine_v8 : public ScummEngine_v7 {
@@ -899,6 +962,8 @@ protected:
 	int _objectIDMapSize;
 	ObjectNameId *_objectIDMap;
 
+	int _keyScriptKey, _keyScriptNo;
+
 public:
 	ScummEngine_v8(OSystem *syst, const DetectorResult &dr);
 	~ScummEngine_v8();
@@ -908,6 +973,11 @@ protected:
 	virtual void executeOpcode(byte i);
 	virtual const char *getOpcodeDesc(byte i);
 
+	virtual void printString(int m, const byte *msg);
+	virtual void CHARSET_1();
+
+	virtual void scummLoop_handleSaveLoad();
+
 	virtual void setupScummVars();
 	virtual void resetScummVars();
 	virtual void decodeParseString(int m, int n);
@@ -924,6 +994,7 @@ protected:
 
 	virtual int getObjectIdFromOBIM(const byte *obim);
 
+	virtual void processKeyboard(int lastKeyHit);
 
 	void desaturatePalette(int hueScale, int satScale, int lightScale, int startColor, int endColor);
 
