@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2002-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,12 +18,12 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/scumm/nut_renderer.h $
- * $Id: nut_renderer.h 20511 2006-02-11 09:55:41Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/scumm/nut_renderer.h $
+ * $Id: nut_renderer.h 27024 2007-05-30 21:56:52Z fingolfin $
  */
 
-#if !defined(NUT_RENDERER_H) && !defined(DISABLE_SCUMM_7_8)
-#define NUT_RENDERER_H
+#if !defined(SCUMM_NUT_RENDERER_H) && !defined(DISABLE_SCUMM_7_8)
+#define SCUMM_NUT_RENDERER_H
 
 #include "common/file.h"
 #include "graphics/surface.h"
@@ -31,29 +34,41 @@ class ScummEngine;
 
 class NutRenderer {
 protected:
+
+	enum {
+		kDefaultTransparentColor = 0,
+		kSmush44TransparentColor = 2
+	};
+
 	ScummEngine *_vm;
-	bool _loaded;
 	int _numChars;
+	int _maxCharSize;
+	byte *_charBuffer;
+	byte *_decodedData;
+	byte *_paletteMap;
+	byte _bpp;
+	byte _palette[16];
 	struct {
-		int xoffs;
-		int yoffs;
-		int width;
-		int height;
+		uint16 width;
+		uint16 height;
 		byte *src;
+		byte transparency;
 	} _chars[256];
 
-	void drawChar(const Graphics::Surface &s, byte c, int x, int y, byte color);
-	void draw2byte(const Graphics::Surface &s, int c, int x, int y, byte color);
+	void codec1(byte *dst, const byte *src, int width, int height, int pitch);
+	void codec21(byte *dst, const byte *src, int width, int height, int pitch);
+
+	void loadFont(const char *filename);
+	byte *unpackChar(byte c);
 
 public:
-	NutRenderer(ScummEngine *vm);
+	NutRenderer(ScummEngine *vm, const char *filename);
 	virtual ~NutRenderer();
 	int getNumChars() const { return _numChars; }
 
-	bool loadFont(const char *filename);
-
 	void drawFrame(byte *dst, int c, int x, int y);
-	void drawShadowChar(const Graphics::Surface &s, int c, int x, int y, byte color, bool showShadow);
+	void drawChar(const Graphics::Surface &s, byte c, int x, int y, byte color);
+	void draw2byte(const Graphics::Surface &s, int c, int x, int y, byte color);
 
 	int getCharWidth(byte c) const;
 	int getCharHeight(byte c) const;
