@@ -1,6 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2001  Ludvig Strigeus
- * Copyright (C) 2001-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -16,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/scumm/object.cpp $
- * $Id: object.cpp 22732 2006-05-29 12:45:11Z kirben $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/scumm/object.cpp $
+ * $Id: object.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -36,32 +38,166 @@
 
 namespace Scumm {
 
-#if !defined(__GNUC__)
-	#pragma START_PACK_STRUCTS
-#endif
+void ScummEngine::addObjectToInventory(uint obj, uint room) {
+	int idx, slot;
+	uint32 size;
+	const byte *ptr;
+	byte *dst;
+	FindObjectInRoom foir;
 
-struct BompHeader {			/* Bomp header */
-	union {
-		struct {
-			uint16 unk;
-			uint16 width, height;
-		} GCC_PACK old;
+	debug(1, "Adding object %d from room %d into inventory", obj, room);
 
-		struct {
-			uint32 width, height;
-		} GCC_PACK v8;
-	} GCC_PACK;
-} GCC_PACK;
+	if (whereIsObject(obj) == WIO_FLOBJECT) {
+		idx = getObjectIndex(obj);
+		assert(idx >= 0);
+		ptr = getResourceAddress(rtFlObject, _objs[idx].fl_object_index) + 8;
+		size = READ_BE_UINT32(ptr + 4);
+	} else {
+		findObjectInRoom(&foir, foCodeHeader, obj, room);
+		if (_game.features & GF_OLD_BUNDLE)
+			size = READ_LE_UINT16(foir.obcd);
+		else if (_game.features & GF_SMALL_HEADER)
+			size = READ_LE_UINT32(foir.obcd);
+		else
+			size = READ_BE_UINT32(foir.obcd + 4);
+		ptr = foir.obcd;
+	}
 
-#if !defined(__GNUC__)
-	#pragma END_PACK_STRUCTS
-#endif
+	slot = getInventorySlot();
+	_inventory[slot] = obj;
+	dst = _res->createResource(rtInventory, slot, size);
+	assert(dst);
+	memcpy(dst, ptr, size);
+}
 
+int ScummEngine::getInventorySlot() {
+	int i;
+	for (i = 0; i < _numInventory; i++) {
+		if (_inventory[i] == 0)
+			return i;
+	}
+	error("Inventory full, %d max items", _numInventory);
+	return -1;
+}
+
+int ScummEngine::findInventory(int owner, int idx) {
+	int count = 1, i, obj;
+	for (i = 0; i < _numInventory; i++) {
+		obj = _inventory[i];
+		if (obj && getOwner(obj) == owner && count++ == idx)
+			return obj;
+	}
+	return 0;
+}
+
+int ScummEngine::getInventoryCount(int owner) {
+	int i, obj;
+	int count = 0;
+	for (i = 0; i < _numInventory; i++) {
+		obj = _inventory[i];
+		if (obj && getOwner(obj) == owner)
+			count++;
+	}
+	return count;
+}
+
+void ScummEngine::setOwnerOf(int obj, int owner) {
+	ScriptSlot *ss;
+
+	// In Sam & Max this is necessary, or you won't get your stuff back
+	// from the Lost and Found tent after riding the Cone of Tragedy. But
+	// it probably applies to all V6+ games. See bugs #493153 and #907113.
+	// FT disassembly is checked, behaviour is correct. [sev]
+
+	int arg = (_game.version >= 6) ? obj : 0;
+
+	if (owner == 0) {
+		clearOwnerOf(obj);
+		
+		// FIXME: See bug #1535358 and many others. Essentially, the following
+		// code, while matching disasm of various versions of the SCUMM engine,
+		// is total bullocks, and leads to odd crashes due to out-of-bounds
+		// array (read) access. Three "famous" crashes were caused by this:
+		//   Monkey Island 1: Using meat with flower
+		//   FOA: Using ribcage with another item
+		//   DOTT: Using stamp with contract
+		//
+		// The bad code:
+		//   if (ss->where == WIO_INVENTORY && _inventory[ss->number] == obj) {
+		// That check makes no sense at all: _inventory only contains 80 items,
+		// which are in the order the player picked up items. We can only 
+		// guess that the SCUMM coders meant to write
+		//   if (ss->where == WIO_INVENTORY && ss->number == obj) {
+		// which would ensure that an object script that nukes itself gets
+		// stopped. Alas, we can't just make that change, since it could
+		// lead to new regressions.
+		// Another fix would be to completely remove this check, which should
+		// not cause much problems, since it'll only succeed by pure chance.
+		// 
+		// For now we follow a more defensive route: We perform the check
+		// if ss->number is small enough.
+		
+		ss = &vm.slot[_currentScript];
+		if (ss->where == WIO_INVENTORY) {
+			if (ss->number < _numInventory && _inventory[ss->number] == obj) {
+				warning("Odd setOwnerOf case #1: Please report to Fingolfin where you encountered this");
+				putOwner(obj, 0);
+				runInventoryScript(arg);
+				stopObjectCode();
+				return;
+			}
+			if (ss->number == obj)
+				warning("Odd setOwnerOf case #2: Please report to Fingolfin where you encountered this");
+		}
+	}
+
+	putOwner(obj, owner);
+	runInventoryScript(arg);
+}
+
+void ScummEngine::clearOwnerOf(int obj) {
+	int i, j;
+	uint16 *a;
+
+	stopObjectScript(obj);
+
+	if (getOwner(obj) == OF_OWNER_ROOM) {
+		for (i = 0; i < _numLocalObjects; i++)  {
+			if (_objs[i].obj_nr == obj && _objs[i].fl_object_index) {
+				// Removing an flObject from a room means we can nuke it
+				_res->nukeResource(rtFlObject, _objs[i].fl_object_index);
+				_objs[i].obj_nr = 0;
+				_objs[i].fl_object_index = 0;
+			}
+		}
+		return;
+	}
+
+	for (i = 0; i < _numInventory; i++) {
+		if (_inventory[i] == obj) {
+			j = whereIsObject(obj);
+			if (j == WIO_INVENTORY) {
+				_res->nukeResource(rtInventory, i);
+				_inventory[i] = 0;
+			}
+			a = _inventory;
+			for (i = 0; i < _numInventory - 1; i++, a++) {
+				if (!a[0] && a[1]) {
+					a[0] = a[1];
+					a[1] = 0;
+					_res->address[rtInventory][i] = _res->address[rtInventory][i + 1];
+					_res->address[rtInventory][i + 1] = NULL;
+				}
+			}
+			return;
+		}
+	}
+}
 
 bool ScummEngine::getClass(int obj, int cls) const {
-	checkRange(_numGlobalObjects - 1, 0, obj, "Object %d out of range in getClass");
+	assertRange(0, obj, _numGlobalObjects - 1, "object");
 	cls &= 0x7F;
-	checkRange(32, 1, cls, "Class %d out of range in getClass");
+	assertRange(1, cls, 32, "class");
 
 	if (_game.features & GF_SMALL_HEADER) {
 		// Translate the new (V5) object classes to the old classes
@@ -86,9 +222,9 @@ bool ScummEngine::getClass(int obj, int cls) const {
 }
 
 void ScummEngine::putClass(int obj, int cls, bool set) {
-	checkRange(_numGlobalObjects - 1, 0, obj, "Object %d out of range in putClass");
+	assertRange(0, obj, _numGlobalObjects - 1, "object");
 	cls &= 0x7F;
-	checkRange(32, 1, cls, "Class %d out of range in putClass");
+	assertRange(1, cls, 32, "class");
 
 	if (_game.features & GF_SMALL_HEADER) {
 		// Translate the new (V5) object classes to the old classes
@@ -115,23 +251,23 @@ void ScummEngine::putClass(int obj, int cls, bool set) {
 		_classData[obj] &= ~(1 << (cls - 1));
 
 	if (_game.version <= 4 && obj >= 1 && obj < _numActors) {
-		_actors[obj].classChanged(cls, set);
+		_actors[obj]->classChanged(cls, set);
 	}
 }
 
 int ScummEngine::getOwner(int obj) const {
-	checkRange(_numGlobalObjects - 1, 0, obj, "Object %d out of range in getOwner");
+	assertRange(0, obj, _numGlobalObjects - 1, "object");
 	return _objectOwnerTable[obj];
 }
 
 void ScummEngine::putOwner(int obj, int owner) {
-	checkRange(_numGlobalObjects - 1, 0, obj, "Object %d out of range in putOwner");
-	checkRange(0xFF, 0, owner, "Owner %d out of range in putOwner");
+	assertRange(0, obj, _numGlobalObjects - 1, "object");
+	assertRange(0, owner, 0xFF, "owner");
 	_objectOwnerTable[obj] = owner;
 }
 
 int ScummEngine::getState(int obj) {
-	checkRange(_numGlobalObjects - 1, 0, obj, "Object %d out of range in getState");
+	assertRange(0, obj, _numGlobalObjects - 1, "object");
 
 	if (!_copyProtection) {
 		// I knew LucasArts sold cracked copies of the original Maniac Mansion,
@@ -140,24 +276,24 @@ int ScummEngine::getState(int obj) {
 		//
 		// This will keep the security door open at all times. I can only
 		// assume that 182 and 193 each correspond to one particular side of
-		// the it. Fortunately it does not prevent frustrated players from
+		// it. Fortunately this does not prevent frustrated players from
 		// blowing up the mansion, should they feel the urge to.
 
 		if (_game.id == GID_MANIAC && (obj == 182 || obj == 193))
-			_objectStateTable[obj] |= 0x08;
+			_objectStateTable[obj] |= kObjectState_08;
 	}
 
 	return _objectStateTable[obj];
 }
 
 void ScummEngine::putState(int obj, int state) {
-	checkRange(_numGlobalObjects - 1, 0, obj, "Object %d out of range in putState");
-	checkRange(0xFF, 0, state, "State %d out of range in putState");
+	assertRange(0, obj, _numGlobalObjects - 1, "object");
+	assertRange(0, state, 0xFF, "state");
 	_objectStateTable[obj] = state;
 }
 
 int ScummEngine::getObjectRoom(int obj) const {
-	checkRange(_numGlobalObjects - 1, 0, obj, "Object %d out of range in getObjectRoom");
+	assertRange(0, obj, _numGlobalObjects - 1, "object");
 	return _objectRoomTable[obj];
 }
 
@@ -201,11 +337,15 @@ int ScummEngine::whereIsObject(int object) const {
 }
 
 int ScummEngine::getObjectOrActorXY(int object, int &x, int &y) {
+	Actor *act;
+
 	if (object < _numActors) {
-		Actor *act = derefActorSafe(object, "getObjectOrActorXY");
-		if (act)
-			return act->getActorXYPos(x, y);
-		else
+		act = derefActorSafe(object, "getObjectOrActorXY");
+		if (act && act->isInCurrentRoom()) {
+			x = act->getRealPos().x;
+			y = act->getRealPos().y;
+			return 0;
+		} else
 			return -1;
 	}
 
@@ -213,10 +353,15 @@ int ScummEngine::getObjectOrActorXY(int object, int &x, int &y) {
 	case WIO_NOT_FOUND:
 		return -1;
 	case WIO_INVENTORY:
-		if (_objectOwnerTable[object] < _numActors)
-			return derefActor(_objectOwnerTable[object], "getObjectOrActorXY(2)")->getActorXYPos(x, y);
-		else
-			return -1;
+		if (_objectOwnerTable[object] < _numActors) {
+			act = derefActor(_objectOwnerTable[object], "getObjectOrActorXY(2)");
+			if (act && act->isInCurrentRoom()) {
+				x = act->getRealPos().x;
+				y = act->getRealPos().y;
+				return 0;
+			}
+		}
+		return -1;
 	}
 	getObjectXYPos(object, x, y);
 	return 0;
@@ -269,6 +414,9 @@ void ScummEngine::getObjectXYPos(int object, int &x, int &y, int &dir) {
 			x = od.x_pos + (int16)READ_LE_UINT16(&imhd->old.hotspot[state].x);
 			y = od.y_pos + (int16)READ_LE_UINT16(&imhd->old.hotspot[state].y);
 		}
+	} else if (_game.version <= 2) {
+		x = od.walk_x >> V12_X_SHIFT;
+		y = od.walk_y >> V12_Y_SHIFT;
 	} else {
 		x = od.walk_x;
 		y = od.walk_y;
@@ -315,34 +463,25 @@ int ScummEngine::getObjActToObjActDist(int a, int b) {
 		y2 = r.y;
 	}
 
-	// Now compute the distance between the two points
-	if (_game.version <= 2) {
-		// For V1/V2 games, distances are measured in the original "character"
-		// based coordinate system, instead of pixels. Otherwise various scripts
-		// will break. See bugs #853874, #774529
-		x /= 8;
-		y /= 2;
-		x2 /= 8;
-		y2 /= 2;
-	}
 
+	// Now compute the distance between the two points
 	return getDist(x, y, x2, y2);
 }
 
 int ScummEngine::findObject(int x, int y) {
 	int i, b;
 	byte a;
-	const int mask = (_game.version <= 2) ? 0x8 : 0xF;
+	const int mask = (_game.version <= 2) ? kObjectState_08 : 0xF;
 
 	for (i = 1; i < _numLocalObjects; i++) {
 		if ((_objs[i].obj_nr < 1) || getClass(_objs[i].obj_nr, kObjectClassUntouchable))
 			continue;
 
 		if (_game.version == 0) {
-			if (_objs[i].flags == 0 && _objs[i].state & 0x2)
+			if (_objs[i].flags == 0 && _objs[i].state & kObjectStateUntouchable)
 				continue;
 		} else {
-			if (_game.version <= 2 && _objs[i].state & 0x2)
+			if (_game.version <= 2 && _objs[i].state & kObjectStateUntouchable)
 				continue;
 		}
 
@@ -371,7 +510,7 @@ int ScummEngine::findObject(int x, int y) {
 void ScummEngine::drawRoomObject(int i, int arg) {
 	ObjectData *od;
 	byte a;
-	const int mask = (_game.version <= 2) ? 0x8 : 0xF;
+	const int mask = (_game.version <= 2) ? kObjectState_08 : 0xF;
 
 	od = &_objs[i];
 	if ((i < 1) || (od->obj_nr < 1) || !od->state)
@@ -390,7 +529,7 @@ void ScummEngine::drawRoomObject(int i, int arg) {
 
 void ScummEngine::drawRoomObjects(int arg) {
 	int i;
-	const int mask = (_game.version <= 2) ? 0x8 : 0xF;
+	const int mask = (_game.version <= 2) ? kObjectState_08 : 0xF;
 
 	if (_game.heversion >= 60) {
 		// In HE games, normal objects are drawn, followed by FlObjects.
@@ -431,7 +570,7 @@ void ScummEngine::drawObject(int obj, int arg) {
 	if (od.obj_nr == 0)
 		return;
 
-	checkRange(_numGlobalObjects - 1, 0, od.obj_nr, "Object %d out of range in drawObject");
+	assertRange(0, od.obj_nr, _numGlobalObjects - 1, "object");
 
 	const int xpos = od.x_pos / 8;
 	const int ypos = od.y_pos;
@@ -478,10 +617,12 @@ void ScummEngine::drawObject(int obj, int arg) {
 		    (_game.id == GID_FT && getClass(od.obj_nr, kObjectClassPlayer)))
 			flags |= Gdi::dbDrawMaskOnAll;
 
+#ifndef DISABLE_HE
 		if (_game.heversion >= 70 && findResource(MKID_BE('SMAP'), ptr) == NULL)
-			gdi.drawBMAPObject(ptr, &virtscr[0], obj, od.x_pos, od.y_pos, od.width, od.height);
+			_gdi->drawBMAPObject(ptr, &virtscr[0], obj, od.x_pos, od.y_pos, od.width, od.height);
 		else
-			gdi.drawBitmap(ptr, &virtscr[0], x, ypos, width * 8, height, x - xpos, numstrip, flags);
+#endif
+			_gdi->drawBitmap(ptr, &virtscr[0], x, ypos, width * 8, height, x - xpos, numstrip, flags);
 	}
 }
 
@@ -493,8 +634,6 @@ void ScummEngine::clearRoomObjects() {
 			_objs[i].obj_nr = 0;
 		}
 	} else {
-		storeFlObject(-1);
-
 		for (i = 0; i < _numLocalObjects; i++) {
 			if (_objs[i].obj_nr < 1)	// Optimise codepath
 				continue;
@@ -504,12 +643,8 @@ void ScummEngine::clearRoomObjects() {
 				_objs[i].obj_nr = 0;
 			} else {
 				// Nuke all unlocked flObjects
-				if (!res.isLocked(rtFlObject, _objs[i].fl_object_index)) {
-					res.nukeResource(rtFlObject, _objs[i].fl_object_index);
-					_objs[i].obj_nr = 0;
-					_objs[i].fl_object_index = 0;
-				} else if (_game.heversion >= 70) {
-					storeFlObject(i);
+				if (!_res->isLocked(rtFlObject, _objs[i].fl_object_index)) {
+					_res->nukeResource(rtFlObject, _objs[i].fl_object_index);
 					_objs[i].obj_nr = 0;
 					_objs[i].fl_object_index = 0;
 				}
@@ -518,21 +653,43 @@ void ScummEngine::clearRoomObjects() {
 	}
 }
 
-void ScummEngine::storeFlObject(int slot) {
-	if (slot == -1) {
-		_numStoredFlObjects = 0;
-	} else {
-		memcpy(&_storedFlObjects[_numStoredFlObjects], &_objs[slot], sizeof(_objs[slot]));
-		_numStoredFlObjects++;
-		if (_numStoredFlObjects > 100)
-			error("Too many flobjects saved on room transition.");
-	}
+#ifndef DISABLE_HE
+void ScummEngine_v70he::resetRoomObjects() {
+	ScummEngine_v60he::resetRoomObjects();
+	restoreFlObjects();
 }
 
-void ScummEngine::restoreFlObjects() {
-	if (!_numStoredFlObjects)
-		return;
+void ScummEngine_v70he::clearRoomObjects() {
+	_numStoredFlObjects = 0;
 
+	for (int i = 0; i < _numLocalObjects; i++) {
+		if (_objs[i].obj_nr < 1)	// Optimise codepath
+			continue;
+
+		if (_objs[i].fl_object_index != 0) {
+			if (!_res->isLocked(rtFlObject, _objs[i].fl_object_index)) {
+				_res->nukeResource(rtFlObject, _objs[i].fl_object_index);
+			} else {
+				storeFlObject(i);
+			}
+		}
+		_objs[i].fl_object_index = 0;
+		_objs[i].obj_nr = 0;
+	}
+
+	if (_currentRoom == 0)
+		restoreFlObjects();
+}
+
+
+void ScummEngine_v70he::storeFlObject(int slot) {
+	memcpy(&_storedFlObjects[_numStoredFlObjects], &_objs[slot], sizeof(_objs[slot]));
+	_numStoredFlObjects++;
+	if (_numStoredFlObjects > 100)
+		error("Too many flobjects saved on room transition.");
+}
+
+void ScummEngine_v70he::restoreFlObjects() {
 	int i, slot;
 
 	for (i = 0; i < _numStoredFlObjects; i++) {
@@ -542,6 +699,7 @@ void ScummEngine::restoreFlObjects() {
 
 	_numStoredFlObjects = 0;
 }
+#endif
 
 void ScummEngine::resetRoomObjects() {
 	int i, j;
@@ -551,7 +709,6 @@ void ScummEngine::resetRoomObjects() {
 	const byte *room, *searchptr, *rootptr;
 	const CodeHeader *cdhd;
 
-	CHECK_HEAP
 	room = getResourceAddress(rtRoom, _roomResource);
 
 	if (_numObjectsInRoom == 0)
@@ -613,8 +770,6 @@ void ScummEngine::resetRoomObjects() {
 		if (_objs[i].obj_nr && !_objs[i].fl_object_index)
 			resetRoomObject(&_objs[i], room);
 	}
-
-	CHECK_HEAP
 }
 
 void ScummEngine_v3old::resetRoomObjects() {
@@ -622,7 +777,6 @@ void ScummEngine_v3old::resetRoomObjects() {
 	ObjectData *od;
 	const byte *room, *ptr;
 
-	CHECK_HEAP
 	room = getResourceAddress(rtRoom, _roomResource);
 
 	if (_numObjectsInRoom == 0)
@@ -658,8 +812,6 @@ void ScummEngine_v3old::resetRoomObjects() {
 			dumpResource(buf, od->obj_nr, room + od->OBCDoffset);
 		}
 	}
-
-	CHECK_HEAP
 }
 
 void ScummEngine_v4::resetRoomObjects() {
@@ -669,7 +821,6 @@ void ScummEngine_v4::resetRoomObjects() {
 	uint16 obim_id;
 	const byte *room;
 
-	CHECK_HEAP
 	room = getResourceAddress(rtRoom, _roomResource);
 
 	if (_numObjectsInRoom == 0)
@@ -697,9 +848,11 @@ void ScummEngine_v4::resetRoomObjects() {
 
 	ResourceIterator	obims(room, true);
 	for (i = 0; i < _numObjectsInRoom; i++) {
+		// In the PC Engine version of Loom, there aren't image blocks
+		// for all objects.
 		ptr = obims.findNext(MKID_BE('OBIM'));
 		if (ptr == NULL)
-			error("Room %d missing image blocks(s)", _roomResource);
+			break;
 
 		obim_id = READ_LE_UINT16(ptr + 6);
 
@@ -714,11 +867,9 @@ void ScummEngine_v4::resetRoomObjects() {
 			resetRoomObject(&_objs[i], room);
 		}
 	}
-
-	CHECK_HEAP
 }
 
-void ScummEngine_c64::resetRoomObject(ObjectData *od, const byte *room, const byte *searchptr) {
+void ScummEngine_v0::resetRoomObject(ObjectData *od, const byte *room, const byte *searchptr) {
 	assert(room);
 	const byte *ptr = room + od->OBCDoffset;
 	ptr -= 2;
@@ -751,27 +902,41 @@ void ScummEngine_v4::resetRoomObject(ObjectData *od, const byte *room, const byt
 
 	od->obj_nr = READ_LE_UINT16(ptr + 6);
 
-	od->x_pos = *(ptr + 9) * 8;
-	od->y_pos = ((*(ptr + 10)) & 0x7F) * 8;
+	if (_game.id == GID_LOOM && _game.platform == Common::kPlatformPCEngine) {
+		od->x_pos = *(ptr + 8) * 8;
+		od->y_pos = ((*(ptr + 9)) & 0x7F) * 8;
 
-	od->parentstate = (*(ptr + 10) & 0x80) ? 1 : 0;
-	if (_game.version <= 2)
-		od->parentstate *= 8;
+		od->parentstate = (*(ptr + 11) & 0x80) ? 1 : 0;
+		od->width = *(ptr + 10) * 8;
 
-	od->width = *(ptr + 11) * 8;
-
-	od->parent = *(ptr + 12);
-
-	if (_game.version <= 2) {
+		od->parent = *(ptr + 11);
 		od->walk_x = *(ptr + 13) * 8;
 		od->walk_y = (*(ptr + 14) & 0x1f) * 8;
 		od->actordir = (*(ptr + 15)) & 7;
 		od->height = *(ptr + 15) & 0xf8;
 	} else {
-		od->walk_x = READ_LE_UINT16(ptr + 13);
-		od->walk_y = READ_LE_UINT16(ptr + 15);
-		od->actordir = (*(ptr + 17)) & 7;
-		od->height = *(ptr + 17) & 0xf8;
+		od->x_pos = *(ptr + 9) * 8;
+		od->y_pos = ((*(ptr + 10)) & 0x7F) * 8;
+
+		od->parentstate = (*(ptr + 10) & 0x80) ? 1 : 0;
+		if (_game.version <= 2)
+			od->parentstate *= 8;
+
+		od->width = *(ptr + 11) * 8;
+
+		od->parent = *(ptr + 12);
+
+		if (_game.version <= 2) {
+			od->walk_x = *(ptr + 13) * 8;
+			od->walk_y = (*(ptr + 14) & 0x1f) * 8;
+			od->actordir = (*(ptr + 15)) & 7;
+			od->height = *(ptr + 15) & 0xf8;
+		} else {
+			od->walk_x = READ_LE_UINT16(ptr + 13);
+			od->walk_y = READ_LE_UINT16(ptr + 15);
+			od->actordir = (*(ptr + 17)) & 7;
+			od->height = *(ptr + 17) & 0xf8;
+		}
 	}
 }
 
@@ -926,47 +1091,6 @@ void ScummEngine_v80he::clearDrawQueues() {
 }
 #endif
 
-void ScummEngine::clearOwnerOf(int obj) {
-	int i, j;
-	uint16 *a;
-
-	stopObjectScript(obj);
-
-	if (getOwner(obj) == OF_OWNER_ROOM) {
-		i = 0;
-		do {
-			if (_objs[i].obj_nr == obj) {
-				if (!_objs[i].fl_object_index)
-					return;
-				res.nukeResource(rtFlObject, _objs[i].fl_object_index);
-				_objs[i].obj_nr = 0;
-				_objs[i].fl_object_index = 0;
-			}
-		} while (++i < _numLocalObjects);
-		return;
-	}
-
-	for (i = 0; i < _numInventory; i++) {
-		if (_inventory[i] == obj) {
-			j = whereIsObject(obj);
-			if (j == WIO_INVENTORY) {
-				res.nukeResource(rtInventory, i);
-				_inventory[i] = 0;
-			}
-			a = _inventory;
-			for (i = 0; i < _numInventory - 1; i++, a++) {
-				if (!a[0] && a[1]) {
-					a[0] = a[1];
-					a[1] = 0;
-					res.address[rtInventory][i] = res.address[rtInventory][i + 1];
-					res.address[rtInventory][i + 1] = NULL;
-				}
-			}
-			return;
-		}
-	}
-}
-
 /**
  * Mark the rectangle covered by the given object as dirty, thus eventually
  * ensuring a redraw of that area. This function is typically invoked when an
@@ -1034,7 +1158,7 @@ void ScummEngine::setObjectName(int obj) {
 
 	for (i = 0; i < _numNewNames; i++) {
 		if (_newNames[i] == obj) {
-			res.nukeResource(rtObjectName, i);
+			_res->nukeResource(rtObjectName, i);
 			_newNames[i] = 0;
 			break;
 		}
@@ -1211,41 +1335,6 @@ int ScummEngine::getObjectIdFromOBIM(const byte *obim) {
 	return READ_LE_UINT16(&imhd->old.obj_id);
 }
 
-void ScummEngine::addObjectToInventory(uint obj, uint room) {
-	int idx, slot;
-	uint32 size;
-	const byte *ptr;
-	byte *dst;
-	FindObjectInRoom foir;
-
-	debug(1, "Adding object %d from room %d into inventory", obj, room);
-
-	CHECK_HEAP
-	if (whereIsObject(obj) == WIO_FLOBJECT) {
-		idx = getObjectIndex(obj);
-		assert(idx >= 0);
-		ptr = getResourceAddress(rtFlObject, _objs[idx].fl_object_index) + 8;
-		size = READ_BE_UINT32(ptr + 4);
-	} else {
-		findObjectInRoom(&foir, foCodeHeader, obj, room);
-		if (_game.features & GF_OLD_BUNDLE)
-			size = READ_LE_UINT16(foir.obcd);
-		else if (_game.features & GF_SMALL_HEADER)
-			size = READ_LE_UINT32(foir.obcd);
-		else
-			size = READ_BE_UINT32(foir.obcd + 4);
-		ptr = foir.obcd;
-	}
-
-	slot = getInventorySlot();
-	_inventory[slot] = obj;
-	dst = res.createResource(rtInventory, slot, size);
-	assert(dst);
-	memcpy(dst, ptr, size);
-
-	CHECK_HEAP
-}
-
 void ScummEngine::findObjectInRoom(FindObjectInRoom *fo, byte findWhat, uint id, uint room) {
 
 	const CodeHeader *cdhd;
@@ -1368,46 +1457,11 @@ void ScummEngine::findObjectInRoom(FindObjectInRoom *fo, byte findWhat, uint id,
 	}
 }
 
-int ScummEngine::getInventorySlot() {
-	int i;
-	for (i = 0; i < _numInventory; i++) {
-		if (_inventory[i] == 0)
-			return i;
-	}
-	error("Inventory full, %d max items", _numInventory);
-	return -1;
-}
-
-void ScummEngine::setOwnerOf(int obj, int owner) {
-	ScriptSlot *ss;
-
-	// In Sam & Max this is necessary, or you won't get your stuff back
-	// from the Lost and Found tent after riding the Cone of Tragedy. But
-	// it probably applies to all V6+ games. See bugs #493153 and #907113.
-	// FT disassembly is checked, behaviour is correct. [sev]
-
-	int arg = (_game.version >= 6) ? obj : 0;
-
-	if (owner == 0) {
-		clearOwnerOf(obj);
-		ss = &vm.slot[_currentScript];
-		if (ss->where == WIO_INVENTORY && _inventory[ss->number] == obj) {
-			putOwner(obj, 0);
-			runInventoryScript(arg);
-			stopObjectCode();
-			return;
-		}
-	}
-
-	putOwner(obj, owner);
-	runInventoryScript(arg);
-}
-
 int ScummEngine::getObjX(int obj) {
 	if (obj < _numActors) {
 		if (obj < 1)
 			return 0;									/* fix for indy4's map */
-		return derefActor(obj, "getObjX")->_pos.x;
+		return derefActor(obj, "getObjX")->getRealPos().x;
 	} else {
 		if (whereIsObject(obj) == WIO_NOT_FOUND)
 			return -1;
@@ -1421,7 +1475,7 @@ int ScummEngine::getObjY(int obj) {
 	if (obj < _numActors) {
 		if (obj < 1)
 			return 0;									/* fix for indy4's map */
-		return derefActor(obj, "getObjY")->_pos.y;
+		return derefActor(obj, "getObjY")->getRealPos().y;
 	} else {
 		if (whereIsObject(obj) == WIO_NOT_FOUND)
 			return -1;
@@ -1446,27 +1500,6 @@ int ScummEngine::getObjNewDir(int obj) {
 	return dir;
 }
 
-int ScummEngine::findInventory(int owner, int idx) {
-	int count = 1, i, obj;
-	for (i = 0; i < _numInventory; i++) {
-		obj = _inventory[i];
-		if (obj && getOwner(obj) == owner && count++ == idx)
-			return obj;
-	}
-	return 0;
-}
-
-int ScummEngine::getInventoryCount(int owner) {
-	int i, obj;
-	int count = 0;
-	for (i = 0; i < _numInventory; i++) {
-		obj = _inventory[i];
-		if (obj && getOwner(obj) == owner)
-			count++;
-	}
-	return count;
-}
-
 void ScummEngine::setObjectState(int obj, int state, int x, int y) {
 	int i;
 
@@ -1500,7 +1533,7 @@ void ScummEngine::setObjectState(int obj, int state, int x, int y) {
 	putState(obj, state);
 }
 
-int ScummEngine::getDistanceBetween(bool is_obj_1, int b, int c, bool is_obj_2, int e, int f) {
+int ScummEngine_v6::getDistanceBetween(bool is_obj_1, int b, int c, bool is_obj_2, int e, int f) {
 	int i, j;
 	int x, y;
 	int x2, y2;
@@ -1538,7 +1571,7 @@ void ScummEngine::nukeFlObjects(int min, int max) {
 
 	for (i = (_numLocalObjects-1), od = _objs; --i >= 0; od++)
 		if (od->fl_object_index && od->obj_nr >= min && od->obj_nr <= max) {
-			res.nukeResource(rtFlObject, od->fl_object_index);
+			_res->nukeResource(rtFlObject, od->fl_object_index);
 			od->obj_nr = 0;
 			od->fl_object_index = 0;
 		}
@@ -1595,7 +1628,7 @@ void ScummEngine_v6::drawBlastObject(BlastObject *eo) {
 
 	vs = &virtscr[0];
 
-	checkRange(_numGlobalObjects - 1, 30, eo->number, "Illegal Blast object %d");
+	assertRange(30, eo->number, _numGlobalObjects - 1, "blast object");
 
 	objnum = getObjectIndex(eo->number);
 	if (objnum == -1)
@@ -1619,34 +1652,42 @@ void ScummEngine_v6::drawBlastObject(BlastObject *eo) {
 	if (!bomp)
 		error("object %d is not a blast object", eo->number);
 
-	if (_game.version == 8) {
-		bdd.srcwidth = READ_LE_UINT32(&((const BompHeader *)bomp)->v8.width);
-		bdd.srcheight = READ_LE_UINT32(&((const BompHeader *)bomp)->v8.height);
-	} else {
-		bdd.srcwidth = READ_LE_UINT16(&((const BompHeader *)bomp)->old.width);
-		bdd.srcheight = READ_LE_UINT16(&((const BompHeader *)bomp)->old.height);
-	}
-
 	bdd.dst = *vs;
 	bdd.dst.pixels = vs->getPixels(0, 0);
-	// Skip the bomp header
-	if (_game.version == 8) {
-		bdd.dataptr = bomp + 8;
-	} else {
-		bdd.dataptr = bomp + 10;
-	}
 	bdd.x = eo->rect.left;
 	bdd.y = eo->rect.top;
+
+	// Skip the bomp header
+	if (_game.version == 8) {
+		bdd.src = bomp + 8;
+	} else {
+		bdd.src = bomp + 10;
+	}
+	if (_game.version == 8) {
+		bdd.srcwidth = READ_LE_UINT32(bomp);
+		bdd.srcheight = READ_LE_UINT32(bomp+4);
+	} else {
+		bdd.srcwidth = READ_LE_UINT16(bomp+2);
+		bdd.srcheight = READ_LE_UINT16(bomp+4);
+	}
+
 	bdd.scale_x = (byte)eo->scaleX;
 	bdd.scale_y = (byte)eo->scaleY;
+
 	bdd.maskPtr = NULL;
+	bdd.numStrips = _gdi->_numStrips;
 
 	if ((bdd.scale_x != 255) || (bdd.scale_y != 255)) {
 		bdd.shadowMode = 0;
 	} else {
 		bdd.shadowMode = eo->mode;
 	}
-	drawBomp(bdd, false);
+	bdd.shadowPalette = _shadowPalette;
+
+	bdd.actorPalette = 0;
+	bdd.mirror = false;
+
+	drawBomp(bdd);
 
 	markRectAsDirty(vs->number, bdd.x, bdd.x + bdd.srcwidth, bdd.y, bdd.y + bdd.srcheight);
 }
@@ -1681,10 +1722,10 @@ void ScummEngine_v6::removeBlastObject(BlastObject *eo) {
 
 	if (left_strip < 0)
 		left_strip = 0;
-	if (right_strip > gdi._numStrips - 1)
-		right_strip = gdi._numStrips - 1;
+	if (right_strip > _gdi->_numStrips - 1)
+		right_strip = _gdi->_numStrips - 1;
 	for (i = left_strip; i <= right_strip; i++)
-		gdi.resetBackground(r.top, r.bottom, i);
+		_gdi->resetBackground(r.top, r.bottom, i);
 
 	markRectAsDirty(kMainVirtScreen, r, USAGE_BIT_RESTORED);
 }
@@ -1705,16 +1746,28 @@ int ScummEngine::findLocalObjectSlot() {
 int ScummEngine::findFlObjectSlot() {
 	int i;
 	for (i = 1; i < _numFlObject; i++) {
-		if (res.address[rtFlObject][i] == NULL)
+		if (_res->address[rtFlObject][i] == NULL)
 			return i;
 	}
 	error("findFlObjectSlot: Out of FLObject slots");
 	return -1;
 }
 
+#ifndef DISABLE_HE
+void ScummEngine_v70he::loadFlObject(uint object, uint room) {
+	// Don't load an already stored object
+	for (int i = 0; i < _numStoredFlObjects; i++) {
+		if (_storedFlObjects[i].obj_nr == object)
+			return;
+	}
+
+	ScummEngine_v60he::loadFlObject(object, room);
+}
+#endif
+
 void ScummEngine::loadFlObject(uint object, uint room) {
 	FindObjectInRoom foir;
-	int i, slot, objslot;
+	int slot, objslot;
 	ObjectData *od;
 	byte *flob;
 	uint32 obcd_size, obim_size, flob_size;
@@ -1724,12 +1777,6 @@ void ScummEngine::loadFlObject(uint object, uint room) {
 	if (getObjectIndex(object) != -1)
 		return;
 
-	// Don't load an already stored object
-	for (i = 0; i < _numStoredFlObjects; i++) {
-		if (_storedFlObjects[i].obj_nr == object)
-			return;
-	}
-
 	// Locate the object in the room resource
 	findObjectInRoom(&foir, foImageHeader | foCodeHeader, object, room);
 
@@ -1758,16 +1805,16 @@ void ScummEngine::loadFlObject(uint object, uint room) {
 
 	// Lock room/roomScripts for the given room. They contains the OBCD/OBIM
 	// data, and a call to createResource might expire them, hence we lock them.
-	isRoomLocked = res.isLocked(rtRoom, room);
-	isRoomScriptsLocked = res.isLocked(rtRoomScripts, room);
+	isRoomLocked = _res->isLocked(rtRoom, room);
+	isRoomScriptsLocked = _res->isLocked(rtRoomScripts, room);
 	if (!isRoomLocked)
-		res.lock(rtRoom, room);
+		_res->lock(rtRoom, room);
 	if (_game.version == 8 && !isRoomScriptsLocked)
-		res.lock(rtRoomScripts, room);
+		_res->lock(rtRoomScripts, room);
 
 	// Allocate slot & memory for floating object
 	slot = findFlObjectSlot();
-	flob = res.createResource(rtFlObject, slot, flob_size);
+	flob = _res->createResource(rtFlObject, slot, flob_size);
 	assert(flob);
 
 	// Copy object code + object image to floating object
@@ -1779,9 +1826,9 @@ void ScummEngine::loadFlObject(uint object, uint room) {
 
 	// Unlock room/roomScripts
 	if (!isRoomLocked)
-		res.unlock(rtRoom, room);
+		_res->unlock(rtRoom, room);
 	if (_game.version == 8 && !isRoomScriptsLocked)
-		res.unlock(rtRoomScripts, room);
+		_res->unlock(rtRoomScripts, room);
 
 	// Setup local object flags
 	resetRoomObject(od, flob, flob);
