@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2002-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,12 +18,12 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/scumm/object.h $
- * $Id: object.h 20511 2006-02-11 09:55:41Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/scumm/object.h $
+ * $Id: object.h 27024 2007-05-30 21:56:52Z fingolfin $
  */
 
-#ifndef OBJECT_H
-#define OBJECT_H
+#ifndef SCUMM_OBJECT_H
+#define SCUMM_OBJECT_H
 
 namespace Scumm {
 
@@ -34,6 +37,20 @@ enum ObjectClass {
 	kObjectClassUntouchable = 32
 };
 
+enum ObjectStateV2 {
+	kObjectStatePickupable = 1,
+	kObjectStateUntouchable = 2,
+	kObjectStateLocked = 4,
+
+	// FIXME: Not quite sure how to name state 8. It seems to mark some kind
+	// of "activation state" for the given object. E.g. is a door open?
+	// Is a drawer extended? In addition it is used to toggle the look
+	// of objects that the user can "pick up" (i.e. it is set in 
+	// o2_pickupObject together with kObjectStateUntouchable). So in a sense,
+	// it can also mean "invisible" in some situations.
+	kObjectState_08 = 8
+};
+
 struct ObjectData {
 	uint32 OBIMoffset;
 	uint32 OBCDoffset;
@@ -51,22 +68,20 @@ struct ObjectData {
 	byte flags;
 };
 
-#if !defined(__GNUC__)
-	#pragma START_PACK_STRUCTS
-#endif
+#include "common/pack-start.h"	// START STRUCT PACKING
 
 struct RoomHeader {
 	union {
 		struct {
 			uint16 width, height;
 			uint16 numObjects;
-		} GCC_PACK old;
+		} old;
 
 		struct {
 			uint32 version;
 			uint16 width, height;
 			uint16 numObjects;
-		} GCC_PACK v7;
+		} v7;
 
 		struct {
 			uint32 version;
@@ -74,9 +89,9 @@ struct RoomHeader {
 			uint32 numObjects;
 			uint32 numZBuffer;
 			uint32 transparency;
-		} GCC_PACK v8;
-	} GCC_PACK;
-} GCC_PACK;
+		} v8;
+	};
+};
 
 struct CodeHeader {
 	union {
@@ -88,7 +103,7 @@ struct CodeHeader {
 			int16 walk_x;
 			int16 walk_y;
 			byte actordir;
-		} GCC_PACK v5;
+		} v5;
 
 		struct {
 			uint16 obj_id;
@@ -98,17 +113,17 @@ struct CodeHeader {
 			uint16 unk1;
 			uint16 unk2;
 			byte actordir;
-		} GCC_PACK v6;
+		} v6;
 
 		struct {
 			uint32 version;
 			uint16 obj_id;
 			byte parent;
 			byte parentstate;
-		} GCC_PACK v7;
+		} v7;
 
-	} GCC_PACK;
-} GCC_PACK;
+	};
+};
 
 struct ImageHeader { /* file format */
 	union {
@@ -124,8 +139,8 @@ struct ImageHeader { /* file format */
 			uint16 hotspot_num;
 			struct {
 				int16 x, y;
-			} GCC_PACK hotspot[15];
-		} GCC_PACK old;
+			} hotspot[15];
+		} old;
 
 		struct {
 			uint32 version;
@@ -138,8 +153,8 @@ struct ImageHeader { /* file format */
 			uint16 hotspot_num;
 			struct {
 				int16 x, y;
-			} GCC_PACK hotspot[15];
-		} GCC_PACK v7;
+			} hotspot[15];
+		} v7;
 
 		struct {
 			char name[32];
@@ -154,14 +169,12 @@ struct ImageHeader { /* file format */
 			uint32 flags;	// This field is missing in the COMI demo (version == 800) !
 			struct {
 				int32 x, y;
-			} GCC_PACK hotspot[15];
-		} GCC_PACK v8;
-	} GCC_PACK;
-} GCC_PACK;
+			} hotspot[15];
+		} v8;
+	};
+};
 
-#if !defined(__GNUC__)
-	#pragma END_PACK_STRUCTS
-#endif
+#include "common/pack-end.h"	// END STRUCT PACKING
 
 struct FindObjectInRoom {
 	const CodeHeader *cdhd;
