@@ -1,6 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2001  Ludvig Strigeus
- * Copyright (C) 2001-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +17,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/scumm/palette.cpp $
- * $Id: palette.cpp 22732 2006-05-29 12:45:11Z kirben $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/scumm/palette.cpp $
+ * $Id: palette.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -33,7 +35,10 @@ namespace Scumm {
 
 void ScummEngine::resetPalette() {
 	if (_game.version <= 1) {
-		if (_game.platform == Common::kPlatformC64) {
+		if (_game.platform == Common::kPlatformApple2GS) {
+			// TODO: unique palette?
+			setC64Palette();
+		} else if (_game.platform == Common::kPlatformC64) {
 			setC64Palette();
 		} else if (_game.platform == Common::kPlatformNES) {
 			setNESPalette();
@@ -88,13 +93,13 @@ void ScummEngine::setC64Palette() {
 	setPalColor(15, 0xA4, 0xA7, 0xA2);
 
 	// Use 17 color table for v1 games to allow correct color for inventory and
-	// sentence line Original games used some kind of dynamic color table
+	// sentence line. Original games used some kind of dynamic color table
 	// remapping between rooms.
 	setPalColor(16, 255,  85, 255);
 }
 
 void ScummEngine::setNESPalette() {
-	setPalColor(0x00,0x24,0x24,0x24); // 0x1D
+	setPalColor(0x00,0x00,0x00,0x00); // 0x1D
 	setPalColor(0x01,0x00,0x24,0x92);
 	setPalColor(0x02,0x00,0x00,0xDB);
 	setPalColor(0x03,0x6D,0x49,0xDB);
@@ -268,7 +273,7 @@ void ScummEngine::setPaletteFromPtr(const byte *ptr, int numcolor) {
 		}
 	}
 
-	checkRange(256, 0, numcolor, "Too many colors (%d) in Palette");
+	assertRange(0, numcolor, 256, "setPaletteFromPtr: numcolor");
 
 	dest = _currentPalette;
 
@@ -357,7 +362,7 @@ void ScummEngine::initCycl(const byte *ptr) {
 void ScummEngine::stopCycle(int i) {
 	ColorCycle *cycl;
 
-	checkRange(16, 0, i, "Stop Cycle %d Out Of Range");
+	assertRange(0, i, 16, "stopCycle: cycle");
 	if (i != 0) {
 		_colorCycle[i - 1].delay = 0;
 		return;
@@ -905,7 +910,10 @@ void ScummEngine::setPalColor(int idx, int r, int g, int b) {
 	setDirtyColors(idx, idx);
 }
 
-void ScummEngine::setPalette(int palindex) {
+void ScummEngine::setCurrentPalette(int palindex) {
+	// TODO: This method could almost be moved to ScummEngin_v6, the only
+	// problem is that it is called by ScummEngine::resetRoomSubBlocks().
+	// But it should be possible to get rid of that, too (with some care).
 	const byte *pals;
 
 	_curPalIndex = palindex;
