@@ -1,6 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2001  Ludvig Strigeus
- * Copyright (C) 2001-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -16,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/scumm/player_mod.cpp $
- * $Id: player_mod.cpp 22231 2006-04-29 22:33:31Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/scumm/player_mod.cpp $
+ * $Id: player_mod.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -28,9 +30,9 @@
 
 namespace Scumm {
 
-Player_MOD::Player_MOD(ScummEngine *scumm) {
+Player_MOD::Player_MOD(Audio::Mixer *mixer) {
 	int i;
-	_mixer = scumm->_mixer;
+	_mixer = mixer;
 	_samplerate = _mixer->getOutputRate();
 	_mixamt = 0;
 	_mixpos = 0;
@@ -46,12 +48,11 @@ Player_MOD::Player_MOD(ScummEngine *scumm) {
 	_playproc = NULL;
 	_playparam = NULL;
 
-	_mixer->setupPremix(this);
+	_mixer->playInputStream(Audio::Mixer::kPlainSoundType, &_soundHandle, this, -1, Audio::Mixer::kMaxChannelVolume, 0, false, true);
 }
 
 Player_MOD::~Player_MOD() {
-	// Detach the premix callback handler
-	_mixer->setupPremix(0);
+	_mixer->stopHandle(_soundHandle);
 	for (int i = 0; i < MOD_MAXCHANS; i++) {
 		if (!_channels[i].id)
 			continue;
@@ -92,7 +93,7 @@ void Player_MOD::startChannel(int id, void *data, int size, int rate, uint8 vol,
 	_channels[i].vol = vol;
 	_channels[i].pan = pan;
 	_channels[i].freq = rate;
-	_channels[i].input = Audio::makeLinearInputStream(rate, Audio::Mixer::FLAG_AUTOFREE | (loopStart != loopEnd ? Audio::Mixer::FLAG_LOOP : 0), (const byte*)data, size, loopStart, loopEnd - loopStart);
+	_channels[i].input = Audio::makeLinearInputStream((const byte*)data, size, rate, Audio::Mixer::FLAG_AUTOFREE | (loopStart != loopEnd ? Audio::Mixer::FLAG_LOOP : 0), loopStart, loopEnd);
 	_channels[i].converter = Audio::makeRateConverter(rate, _mixer->getOutputRate(), false, false);
 }
 
