@@ -1,6 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2001  Ludvig Strigeus
- * Copyright (C) 2001-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -16,16 +18,17 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/scumm/player_mod.h $
- * $Id: player_mod.h 22231 2006-04-29 22:33:31Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/scumm/player_mod.h $
+ * $Id: player_mod.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
-#ifndef PLAYER_MOD_H
-#define PLAYER_MOD_H
+#ifndef SCUMM_PLAYER_MOD_H
+#define SCUMM_PLAYER_MOD_H
 
 #include "scumm/scumm.h"
 #include "sound/audiostream.h"
+#include "sound/mixer.h"
 
 namespace Audio {
 	class RateConverter;
@@ -38,7 +41,7 @@ namespace Scumm {
  */
 class Player_MOD : public Audio::AudioStream {
 public:
-	Player_MOD(ScummEngine *scumm);
+	Player_MOD(Audio::Mixer *mixer);
 	virtual ~Player_MOD();
 	virtual void setMusicVolume(int vol);
 
@@ -77,6 +80,7 @@ private:
 	};
 
 	Audio::Mixer *_mixer;
+	Audio::SoundHandle _soundHandle;
 
 	uint32 _mixamt;
 	uint32 _mixpos;
