@@ -1,6 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2001  Ludvig Strigeus
- * Copyright (C) 2001-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -16,25 +18,25 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/scumm/player_nes.h $
- * $Id: player_nes.h 22231 2006-04-29 22:33:31Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/scumm/player_nes.h $
+ * $Id: player_nes.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
-#ifndef PLAYER_NES_H
-#define PLAYER_NES_H
+#ifndef SCUMM_PLAYER_NES_H
+#define SCUMM_PLAYER_NES_H
 
 #include "common/scummsys.h"
 #include "scumm/music.h"
 #include "sound/audiostream.h"
-
-namespace Audio {
-	class Mixer;
-}
+#include "sound/mixer.h"
 
 namespace Scumm {
 
 class ScummEngine;
+namespace APUe {
+	class APU;
+}
 
 static const int MAXVOLUME = 0x7F;
 static const int NUMSLOTS = 3;
@@ -45,7 +47,7 @@ static const int NUMCHANS = 4;
  */
 class Player_NES : public Audio::AudioStream, public MusicEngine {
 public:
-	Player_NES(ScummEngine *scumm);
+	Player_NES(ScummEngine *scumm, Audio::Mixer *mixer);
 	virtual ~Player_NES();
 
 	virtual void setMusicVolume(int vol);
@@ -77,6 +79,8 @@ private:
 
 	ScummEngine *_vm;
 	Audio::Mixer *_mixer;
+	Audio::SoundHandle _soundHandle;
+	APUe::APU *_apu;
 	int _sample_rate;
 	int _samples_per_frame;
 	int _current_sample;
