@@ -1,6 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2001  Ludvig Strigeus
- * Copyright (C) 2001-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -16,13 +18,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/scumm/player_v2.cpp $
- * $Id: player_v2.cpp 21827 2006-04-12 14:54:12Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/scumm/player_v2.cpp $
+ * $Id: player_v2.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
 #include "common/stdafx.h"
-#include "base/engine.h"
+#include "engines/engine.h"
 #include "scumm/player_v2.h"
 #include "scumm/scumm.h"
 #include "sound/mididrv.h"
@@ -342,12 +344,12 @@ static const uint16 pcjr_freq_table[12] = {
 #endif
 
 
-Player_V2::Player_V2(ScummEngine *scumm, bool pcjr) {
+Player_V2::Player_V2(ScummEngine *scumm, Audio::Mixer *mixer, bool pcjr) {
 	int i;
 
 	_isV3Game = (scumm->_game.version >= 3);
 	_vm = scumm;
-	_mixer = scumm->_mixer;
+	_mixer = mixer;
 	_sample_rate = _mixer->getOutputRate();
 
 	_header_len = (scumm->_game.features & GF_OLD_BUNDLE) ? 4 : 6;
@@ -375,13 +377,12 @@ Player_V2::Player_V2(ScummEngine *scumm, bool pcjr) {
 	set_pcjr(pcjr);
 	setMusicVolume(255);
 
-	_mixer->setupPremix(this);
+	_mixer->playInputStream(Audio::Mixer::kPlainSoundType, &_soundHandle, this, -1, Audio::Mixer::kMaxChannelVolume, 0, false, true);
 }
 
 Player_V2::~Player_V2() {
 	mutex_up();
-	// Detach the premix callback handler
-	_mixer->setupPremix(0);
+	_mixer->stopHandle(_soundHandle);
 	mutex_down();
 }
 
