@@ -1,6 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2001  Ludvig Strigeus
- * Copyright (C) 2001-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -16,30 +18,25 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/scumm/player_v2.h $
- * $Id: player_v2.h 22231 2006-04-29 22:33:31Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/scumm/player_v2.h $
+ * $Id: player_v2.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
-#ifndef PLAYER_V2_H
-#define PLAYER_V2_H
+#ifndef SCUMM_PLAYER_V2_H
+#define SCUMM_PLAYER_V2_H
 
 #include "common/scummsys.h"
 #include "common/mutex.h"
 #include "scumm/music.h"
 #include "sound/audiostream.h"
-
-namespace Audio {
-	class Mixer;
-}
+#include "sound/mixer.h"
 
 namespace Scumm {
 
 class ScummEngine;
 
-#if !defined(__GNUC__)
-	#pragma START_PACK_STRUCTS
-#endif
+#include "common/pack-start.h"	// START STRUCT PACKING
 
 struct channel_data {
 	uint16 time_left;          // 00
@@ -64,11 +61,9 @@ struct channel_data {
 	uint16 unknown[4];         // 38 - 44
 	uint16 music_timer;        // 46
 	uint16 music_script_nr;    // 48
-} GCC_PACK;
+};
 
-#if !defined(__GNUC__)
-	#pragma END_PACK_STRUCTS
-#endif
+#include "common/pack-end.h"	// END STRUCT PACKING
 
 
 /**
@@ -78,7 +73,7 @@ struct channel_data {
  */
 class Player_V2 : public Audio::AudioStream, public MusicEngine {
 public:
-	Player_V2(ScummEngine *scumm, bool pcjr);
+	Player_V2(ScummEngine *scumm, Audio::Mixer *mixer, bool pcjr);
 	virtual ~Player_V2();
 
 	virtual void setMusicVolume(int vol);
@@ -100,6 +95,7 @@ public:
 protected:
 	bool _isV3Game;
 	Audio::Mixer *_mixer;
+	Audio::SoundHandle _soundHandle;
 	ScummEngine *_vm;
 
 	bool _pcjr;
