@@ -1,6 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2001  Ludvig Strigeus
- * Copyright (C) 2001-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -16,13 +18,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/scumm/player_v2a.cpp $
- * $Id: player_v2a.cpp 20511 2006-02-11 09:55:41Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/scumm/player_v2a.cpp $
+ * $Id: player_v2a.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
 #include "common/stdafx.h"
-#include "base/engine.h"
+#include "engines/engine.h"
 #include "scumm/player_v2a.h"
 #include "scumm/scumm.h"
 
@@ -63,7 +65,7 @@ static uint32 GetCRC (byte *data, int len)
 class V2A_Sound {
 public:
 	V2A_Sound() : _id(0), _mod(NULL) { }
-	virtual ~V2A_Sound() {};
+	virtual ~V2A_Sound() {}
 	virtual void start(Player_MOD *mod, int id, const byte *data) = 0;
 	virtual bool update() = 0;
 	virtual void stop() = 0;
@@ -1303,7 +1305,7 @@ static V2A_Sound *findSound (unsigned long crc) {
 	return NULL;
 }
 
-Player_V2A::Player_V2A(ScummEngine *scumm) {
+Player_V2A::Player_V2A(ScummEngine *scumm, Audio::Mixer *mixer) {
 	int i;
 	_vm = scumm;
 
@@ -1317,7 +1319,7 @@ Player_V2A::Player_V2A(ScummEngine *scumm) {
 		_slot[i].sound = NULL;
 	}
 
-	_mod = new Player_MOD(scumm);
+	_mod = new Player_MOD(mixer);
 	_mod->setUpdateProc(update_proc, this, 60);
 }
 
