@@ -1,6 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2001  Ludvig Strigeus
- * Copyright (C) 2001-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -16,13 +18,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/scumm/player_v2a.h $
- * $Id: player_v2a.h 20511 2006-02-11 09:55:41Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/scumm/player_v2a.h $
+ * $Id: player_v2a.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
-#ifndef PLAYER_V2A_H
-#define PLAYER_V2A_H
+#ifndef SCUMM_PLAYER_V2A_H
+#define SCUMM_PLAYER_V2A_H
 
 #include "common/scummsys.h"
 #include "scumm/music.h"
@@ -40,7 +42,7 @@ class V2A_Sound;
  */
 class Player_V2A : public MusicEngine {
 public:
-	Player_V2A(ScummEngine *scumm);
+	Player_V2A(ScummEngine *scumm, Audio::Mixer *mixer);
 	virtual ~Player_V2A();
 
 	virtual void setMusicVolume(int vol);
