@@ -1,6 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2001  Ludvig Strigeus
- * Copyright (C) 2001-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -16,13 +18,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/scumm/player_v3a.cpp $
- * $Id: player_v3a.cpp 20795 2006-02-20 16:51:30Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/scumm/player_v3a.cpp $
+ * $Id: player_v3a.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
 #include "common/stdafx.h"
-#include "base/engine.h"
+#include "engines/engine.h"
 #include "scumm/player_v3a.h"
 #include "scumm/scumm.h"
 
@@ -35,7 +37,7 @@ static const uint16 note_freqs[4][12] = {
 	{0x00D6, 0x00CA, 0x00BE, 0x00B4, 0x00A9, 0x00A0, 0x0097, 0x008E, 0x0086, 0x007F, 0x00F0, 0x00E2}
 };
 
-Player_V3A::Player_V3A(ScummEngine *scumm) {
+Player_V3A::Player_V3A(ScummEngine *scumm, Audio::Mixer *mixer) {
 	int i;
 	_vm = scumm;
 	for (i = 0; i < V3A_MAXMUS; i++) {
@@ -56,7 +58,7 @@ Player_V3A::Player_V3A(ScummEngine *scumm) {
 
 	_isinit = false;
 
-	_mod = new Player_MOD(scumm);
+	_mod = new Player_MOD(mixer);
 	_mod->setUpdateProc(update_proc, this, 60);
 }
 
@@ -223,24 +225,27 @@ void Player_V3A::startSound(int nr) {
 		int rate = 3579545 / READ_BE_UINT16(data + 20);
 		char *sound = (char *)malloc(size);
 		int vol = (data[24] << 1) | (data[24] >> 5);	// if I boost this to 0-255, it gets too loud and starts to clip
-		memcpy(sound,data + READ_BE_UINT16(data + 8),size);
+		memcpy(sound, data + READ_BE_UINT16(data + 8), size);
 		int loopStart = 0, loopEnd = 0;
-		bool looped = false;
-		if ((READ_BE_UINT16(data + 16) || READ_BE_UINT16(data + 6))) {
+		int loopcount = data[27];
+		if (loopcount > 1) {
 			loopStart = READ_BE_UINT16(data + 10) - READ_BE_UINT16(data + 8);
 			loopEnd = READ_BE_UINT16(data + 14);
-			looped = true;
 		}
 		int i = getSfxChan();
-		if (i == -1)
-		{
+		if (i == -1) {
 			free(sound);
 			return;
 		}
 		_sfx[i].id = nr;
-		_sfx[i].dur = looped ? -1 : (1 + 60 * size / rate);
-		if ((_vm->_game.id == GID_INDY3) && (nr == 60))
-			_sfx[i].dur = 240;
+		_sfx[i].dur = 1 + loopcount * 60 * size / rate;
+		if (READ_BE_UINT16(data + 16)) {
+			_sfx[i].rate = READ_BE_UINT16(data + 20) << 16;
+			_sfx[i].delta = (int32)READ_BE_UINT32(data + 32);
+			_sfx[i].dur = READ_BE_UINT32(data + 40);
+		} else {
+			_sfx[i].delta = 0;
+		}
 		_mod->startChannel(nr | 0x100, sound, size, rate, vol, loopStart, loopEnd);
 	}
 }
@@ -262,6 +267,15 @@ void Player_V3A::playMusic() {
 	}
 	for (i = 0; i < V3A_MAXSFX; i++) {
 		if (_sfx[i].id) {
+			if (_sfx[i].delta) {
+				uint16 oldrate = _sfx[i].rate >> 16;
+				_sfx[i].rate += _sfx[i].delta;
+				if (_sfx[i].rate < (55 << 16))
+					_sfx[i].rate = 55 << 16;	// at rates below 55, frequency
+				uint16 newrate = _sfx[i].rate >> 16;	// exceeds 65536, which is bad
+				if (oldrate != newrate)
+					_mod->setChannelFreq(_sfx[i].id | 0x100, 3579545 / newrate);
+			}
 			_sfx[i].dur--;
 			if (_sfx[i].dur)
 				continue;
@@ -320,8 +334,7 @@ void Player_V3A::playMusic() {
 			memcpy(data + _wavetable[inst]->_ilen[oct], _wavetable[inst]->_ldat[oct], _wavetable[inst]->_llen[oct]);
 
 		i = getMusChan();
-		if (i == -1)
-		{
+		if (i == -1) {
 			free(data);
 			return;
 		}
