@@ -1,6 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2001  Ludvig Strigeus
- * Copyright (C) 2001-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -16,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/scumm/resource.cpp $
- * $Id: resource.cpp 22120 2006-04-23 22:39:55Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/scumm/resource.cpp $
+ * $Id: resource.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -85,18 +87,18 @@ void ScummEngine::openRoom(const int room) {
 	// Load the disk numer / room offs (special case for room 0 exists because
 	// room 0 contains the data which is used to create the roomno / roomoffs
 	// tables -- hence obviously we mustn't use those when loading room 0.
-	const uint32 diskNumber = room ? res.roomno[rtRoom][room] : 0;
-	const uint32 room_offs = room ? res.roomoffs[rtRoom][room] : 0;
+	const uint32 diskNumber = room ? _res->roomno[rtRoom][room] : 0;
+	const uint32 room_offs = room ? _res->roomoffs[rtRoom][room] : 0;
 
 	// FIXME: Since room_offs is const, clearly the following loop either
 	// is never entered, or loops forever (if it wasn't for the return/error
 	// statements in it, that is). -> This should be cleaned up!
 	// Maybe we should re-enabled the looping properly, to deal with disc
 	// changes in COMI ?
-	while (room_offs != 0xFFFFFFFF) {
+	while (room_offs != RES_INVALID_OFFSET) {
 
 		if (room_offs != 0 && room != 0 && _game.heversion < 98) {
-			_fileOffset = res.roomoffs[rtRoom][room];
+			_fileOffset = _res->roomoffs[rtRoom][room];
 			return;
 		}
 		
@@ -124,7 +126,7 @@ void ScummEngine::openRoom(const int room) {
 				return;
 			deleteRoomOffsets();
 			readRoomsOffsets();
-			_fileOffset = res.roomoffs[rtRoom][room];
+			_fileOffset = _res->roomoffs[rtRoom][room];
 
 			if (_fileOffset != 8)
 				return;
@@ -159,8 +161,8 @@ void ScummEngine::closeRoom() {
 /** Delete the currently loaded room offsets. */
 void ScummEngine::deleteRoomOffsets() {
 	for (int i = 0; i < _numRooms; i++) {
-		if (res.roomoffs[rtRoom][i] != 0xFFFFFFFF)
-			res.roomoffs[rtRoom][i] = 0;
+		if (_res->roomoffs[rtRoom][i] != RES_INVALID_OFFSET)
+			_res->roomoffs[rtRoom][i] = 0;
 	}
 }
 
@@ -179,8 +181,8 @@ void ScummEngine::readRoomsOffsets() {
 	num = _fileHandle->readByte();
 	while (num--) {
 		room = _fileHandle->readByte();
-		if (res.roomoffs[rtRoom][room] != 0xFFFFFFFF) {
-			res.roomoffs[rtRoom][room] = _fileHandle->readUint32LE();
+		if (_res->roomoffs[rtRoom][room] != RES_INVALID_OFFSET) {
+			_res->roomoffs[rtRoom][room] = _fileHandle->readUint32LE();
 		} else {
 			_fileHandle->readUint32LE();
 		}
@@ -367,7 +369,7 @@ void ScummEngine_v7::readIndexBlock(uint32 blocktype, uint32 itemsize) {
 		break;
 
 	case MKID_BE('DRSC'):		// Used by: COMI
-		readResTypeList(rtRoomScripts, "room script");
+		readResTypeList(rtRoomScripts);
 		break;
 
 	default:
@@ -381,15 +383,15 @@ void ScummEngine_v70he::readIndexBlock(uint32 blocktype, uint32 itemsize) {
 	int i;
 	switch (blocktype) {
 	case MKID_BE('DIRI'):
-		readResTypeList(rtRoomImage, "room image");
+		readResTypeList(rtRoomImage);
 		break;
 
 	case MKID_BE('DIRM'):
-		readResTypeList(rtImage, "images");
+		readResTypeList(rtImage);
 		break;
 
 	case MKID_BE('DIRT'):
-		readResTypeList(rtTalkie, "talkie");
+		readResTypeList(rtTalkie);
 		break;
 
 	case MKID_BE('DLFL'):
@@ -426,7 +428,7 @@ void ScummEngine::readIndexBlock(uint32 blocktype, uint32 itemsize) {
 	switch (blocktype) {
 	case MKID_BE('DCHR'):
 	case MKID_BE('DIRF'):
-		readResTypeList(rtCharset, "charset");
+		readResTypeList(rtCharset);
 		break;
 
 	case MKID_BE('DOBJ'):
@@ -460,17 +462,17 @@ void ScummEngine::readIndexBlock(uint32 blocktype, uint32 itemsize) {
 
 	case MKID_BE('DROO'):
 	case MKID_BE('DIRR'):
-		readResTypeList(rtRoom, "room");
+		readResTypeList(rtRoom);
 		break;
 
 	case MKID_BE('DSCR'):
 	case MKID_BE('DIRS'):
-		readResTypeList(rtScript, "script");
+		readResTypeList(rtScript);
 		break;
 
 	case MKID_BE('DCOS'):
 	case MKID_BE('DIRC'):
-		readResTypeList(rtCostume, "costume");
+		readResTypeList(rtCostume);
 		break;
 
 	case MKID_BE('MAXS'):
@@ -480,7 +482,7 @@ void ScummEngine::readIndexBlock(uint32 blocktype, uint32 itemsize) {
 
 	case MKID_BE('DIRN'):
 	case MKID_BE('DSOU'):
-		readResTypeList(rtSound, "sound");
+		readResTypeList(rtSound);
 		break;
 
 	case MKID_BE('AARY'):
@@ -497,71 +499,62 @@ void ScummEngine::readArrayFromIndexFile() {
 	error("readArrayFromIndexFile() not supported in pre-V6 games");
 }
 
-void ScummEngine::readResTypeList(int id, const char *name) {
+void ScummEngine::readResTypeList(int id) {
 	int num;
 	int i;
 
-	debug(9, "readResTypeList(%s,%s)", resTypeFromId(id), name);
+	debug(9, "readResTypeList(%s)", resTypeFromId(id));
 
 	if (_game.version == 8)
 		num = _fileHandle->readUint32LE();
 	else
 		num = _fileHandle->readUint16LE();
 
-	if (num != res.num[id]) {
-		error("Invalid number of %ss (%d) in directory", name, num);
+	if (num != _res->num[id]) {
+		error("Invalid number of %ss (%d) in directory", resTypeFromId(id), num);
 	}
 
-	if (_game.features & GF_SMALL_HEADER) {
-		for (i = 0; i < num; i++) {
-			res.roomno[id][i] = _fileHandle->readByte();
-			res.roomoffs[id][i] = _fileHandle->readUint32LE();
-		}
-	} else {
-		for (i = 0; i < num; i++) {
-			res.roomno[id][i] = _fileHandle->readByte();
-		}
-		for (i = 0; i < num; i++) {
-			res.roomoffs[id][i] = _fileHandle->readUint32LE();
+	for (i = 0; i < num; i++) {
+		_res->roomno[id][i] = _fileHandle->readByte();
+	}
+	for (i = 0; i < num; i++) {
+		_res->roomoffs[id][i] = _fileHandle->readUint32LE();
 
-			if (id == rtRoom && _game.heversion >= 70)
-				_heV7RoomIntOffsets[i] = res.roomoffs[id][i];
-		}
+		if (id == rtRoom && _game.heversion >= 70)
+			_heV7RoomIntOffsets[i] = _res->roomoffs[id][i];
+	}
 
-		if (_game.heversion >= 70) {
-			for (i = 0; i < num; i++) {
-				res.globsize[id][i] = _fileHandle->readUint32LE();
-			}
+	if (_game.heversion >= 70) {
+		for (i = 0; i < num; i++) {
+			_res->globsize[id][i] = _fileHandle->readUint32LE();
 		}
 	}
 }
 
-void ScummEngine::allocResTypeData(int id, uint32 tag, int num, const char *name, int mode) {
-	debug(9, "allocResTypeData(%s/%s,%s,%d,%d)", resTypeFromId(id), name, tag2str(TO_BE_32(tag)), num, mode);
-	assert(id >= 0 && id < (int)(ARRAYSIZE(res.mode)));
+void ResourceManager::allocResTypeData(int id, uint32 tag, int num_, const char *name_, int mode_) {
+	debug(9, "allocResTypeData(%s/%s,%s,%d,%d)", resTypeFromId(id), name_, tag2str(TO_BE_32(tag)), num_, mode_);
+	assert(id >= 0 && id < (int)(ARRAYSIZE(this->mode)));
 
-	if (num >= 8000)
-		error("Too many %ss (%d) in directory", name, num);
+	if (num_ >= 8000)
+		error("Too many %ss (%d) in directory", name_, num_);
 
-	res.mode[id] = mode;
-	res.num[id] = num;
-	res.tags[id] = tag;
-	res.name[id] = name;
-	res.address[id] = (byte **)calloc(num, sizeof(void *));
-	res.flags[id] = (byte *)calloc(num, sizeof(byte));
-	res.status[id] = (byte *)calloc(num, sizeof(byte));
+	mode[id] = mode_;
+	num[id] = num_;
+	tags[id] = tag;
+	name[id] = name_;
+	address[id] = (byte **)calloc(num_, sizeof(void *));
+	flags[id] = (byte *)calloc(num_, sizeof(byte));
+	status[id] = (byte *)calloc(num_, sizeof(byte));
 
-	if (mode) {
-		res.roomno[id] = (byte *)calloc(num, sizeof(byte));
-		res.roomoffs[id] = (uint32 *)calloc(num, sizeof(uint32));
+	if (mode_) {
+		roomno[id] = (byte *)calloc(num_, sizeof(byte));
+		roomoffs[id] = (uint32 *)calloc(num_, sizeof(uint32));
 	}
 
-	if (_game.heversion >= 70) {
-		res.globsize[id] = (uint32 *)calloc(num, sizeof(uint32));
-
-		if (id == rtRoom)
-			_heV7RoomIntOffsets = (uint32 *)calloc(num, sizeof(uint32));
+	if (_vm->_game.heversion >= 70) {
+		globsize[id] = (uint32 *)calloc(num_, sizeof(uint32));
 	}
+
 }
 
 void ScummEngine::loadCharset(int no) {
@@ -581,7 +574,7 @@ void ScummEngine::loadCharset(int no) {
 	}
 
 	assert(no < (int)sizeof(_charsetData) / 16);
-	checkRange(_numCharsets - 1, 1, no, "Loading illegal charset %d");
+	assertRange(1, no, _numCharsets - 1, "charset");
 
 	ptr = getResourceAddress(rtCharset, no);
 
@@ -591,8 +584,8 @@ void ScummEngine::loadCharset(int no) {
 }
 
 void ScummEngine::nukeCharset(int i) {
-	checkRange(_numCharsets - 1, 1, i, "Nuking illegal charset %d");
-	res.nukeResource(rtCharset, i);
+	assertRange(1, i, _numCharsets - 1, "charset");
+	_res->nukeResource(rtCharset, i);
 }
 
 void ScummEngine::ensureResourceLoaded(int type, int i) {
@@ -618,8 +611,8 @@ void ScummEngine::ensureResourceLoaded(int type, int i) {
 	if (type != rtCharset && i == 0)
 		return;
 
-	if (i <= res.num[type])
-		addr = res.address[type][i];
+	if (i <= _res->num[type])
+		addr = _res->address[type][i];
 
 	if (addr)
 		return;
@@ -635,7 +628,7 @@ int ScummEngine::loadResource(int type, int idx) {
 	uint32 fileOffs;
 	uint32 size, tag;
 
-	debugC(DEBUG_RESOURCE, "loadResource(%s,%d)", resTypeFromId(type),idx);
+	debugC(DEBUG_RESOURCE, "loadResource(%s,%d)", resTypeFromId(type), idx);
 
 	if (type == rtCharset && (_game.features & GF_SMALL_HEADER)) {
 		loadCharset(idx);
@@ -644,8 +637,8 @@ int ScummEngine::loadResource(int type, int idx) {
 
 	roomNr = getResourceRoomNr(type, idx);
 
-	if (idx >= res.num[type])
-		error("%s %d undefined %d %d", res.name[type], idx, res.num[type], roomNr);
+	if (idx >= _res->num[type])
+		error("%s %d undefined %d %d", _res->name[type], idx, _res->num[type], roomNr);
 
 	if (roomNr == 0)
 		roomNr = _roomResource;
@@ -658,8 +651,8 @@ int ScummEngine::loadResource(int type, int idx) {
 		else
 			fileOffs = 0;
 	} else {
-		fileOffs = res.roomoffs[type][idx];
-		if (fileOffs == 0xFFFFFFFF)
+		fileOffs = _res->roomoffs[type][idx];
+		if (fileOffs == RES_INVALID_OFFSET)
 			return 0;
 	}
 
@@ -690,16 +683,16 @@ int ScummEngine::loadResource(int type, int idx) {
 
 		tag = _fileHandle->readUint32BE();
 
-		if (tag != res.tags[type] && _game.heversion < 70) {
+		if (tag != _res->tags[type] && _game.heversion < 70) {
 			error("%s %d not in room %d at %d+%d in file %s",
-					res.name[type], idx, roomNr,
+					_res->name[type], idx, roomNr,
 					_fileOffset, fileOffs, _fileHandle->name());
 		}
 
 		size = _fileHandle->readUint32BE();
 		_fileHandle->seek(-8, SEEK_CUR);
 	}
-	_fileHandle->read(res.createResource(type, idx, size), size);
+	_fileHandle->read(_res->createResource(type, idx, size), size);
 
 	// dump the resource if requested
 	if (_dumpScripts && type == rtScript) {
@@ -710,7 +703,7 @@ int ScummEngine::loadResource(int type, int idx) {
 		return 1;
 	}
 
-	res.nukeResource(type, idx);
+	_res->nukeResource(type, idx);
 
 	error("Cannot read resource");
 }
@@ -718,11 +711,12 @@ int ScummEngine::loadResource(int type, int idx) {
 int ScummEngine::getResourceRoomNr(int type, int idx) {
 	if (type == rtRoom && _game.heversion < 70)
 		return idx;
-	return res.roomno[type][idx];
+	return _res->roomno[type][idx];
 }
 
 int ScummEngine::getResourceSize(int type, int idx) {
 	byte *ptr = getResourceAddress(type, idx);
+	assert(ptr);
 	MemBlkHeader *hdr = (MemBlkHeader *)(ptr - sizeof(MemBlkHeader));
 
 	return hdr->size;
@@ -731,29 +725,27 @@ int ScummEngine::getResourceSize(int type, int idx) {
 byte *ScummEngine::getResourceAddress(int type, int idx) {
 	byte *ptr;
 
-	CHECK_HEAP
-
 	if (_game.heversion >= 80 && type == rtString)
 		idx &= ~0x33539000;
 
-	if (!res.validateResource("getResourceAddress", type, idx))
+	if (!_res->validateResource("getResourceAddress", type, idx))
 		return NULL;
 
-	if (!res.address[type]) {
-		debugC(DEBUG_RESOURCE, "getResourceAddress(%s,%d), res.address[type] == NULL", resTypeFromId(type), idx);
+	if (!_res->address[type]) {
+		debugC(DEBUG_RESOURCE, "getResourceAddress(%s,%d), _res->address[type] == NULL", resTypeFromId(type), idx);
 		return NULL;
 	}
 
-	if (res.mode[type] && !res.address[type][idx]) {
+	if (_res->mode[type] && !_res->address[type][idx]) {
 		ensureResourceLoaded(type, idx);
 	}
 
-	if (!(ptr = (byte *)res.address[type][idx])) {
+	if (!(ptr = (byte *)_res->address[type][idx])) {
 		debugC(DEBUG_RESOURCE, "getResourceAddress(%s,%d) == NULL", resTypeFromId(type), idx);
 		return NULL;
 	}
 
-	res.setResourceCounter(type, idx, 1);
+	_res->setResourceCounter(type, idx, 1);
 
 	debugC(DEBUG_RESOURCE, "getResourceAddress(%s,%d) == %p", resTypeFromId(type), idx, ptr + sizeof(MemBlkHeader));
 	return ptr + sizeof(MemBlkHeader);
@@ -768,13 +760,20 @@ byte *ScummEngine_v6::getStringAddress(int i) {
 	byte *addr = getResourceAddress(rtString, i);
 	if (addr == NULL)
 		return NULL;
-	return ((ScummEngine_v6::ArrayHeader *)addr)->data;
+	// Skip over the ArrayHeader
+	return addr + 6;
 }
 
 byte *ScummEngine::getStringAddressVar(int i) {
 	return getStringAddress(_scummVars[i]);
 }
 
+void ResourceManager::increaseExpireCounter() {
+	if (!(++_expireCounter)) {
+		increaseResourceCounter();
+	}
+}
+
 void ResourceManager::increaseResourceCounter() {
 	int i, j;
 	byte counter;
@@ -800,8 +799,7 @@ void ResourceManager::setResourceCounter(int type, int idx, byte flag) {
 byte *ResourceManager::createResource(int type, int idx, uint32 size) {
 	byte *ptr;
 
-	CHECK_HEAP
-	debugC(DEBUG_RESOURCE, "res.createResource(%s,%d,%d)", resTypeFromId(type), idx, size);
+	debugC(DEBUG_RESOURCE, "_res->createResource(%s,%d,%d)", resTypeFromId(type), idx, size);
 
 	if (!validateResource("allocating", type, idx))
 		return NULL;
@@ -819,7 +817,6 @@ byte *ResourceManager::createResource(int type, int idx, uint32 size) {
 
 	expireResources(size);
 
-	CHECK_HEAP
 	ptr = (byte *)calloc(size + sizeof(MemBlkHeader) + SAFETY_AREA, 1);
 	if (ptr == NULL) {
 		error("Out of memory while allocating %d", size);
@@ -839,6 +836,17 @@ ResourceManager::ResourceManager(ScummEngine *vm) {
 //	_allocatedSize = 0;
 }
 
+ResourceManager::~ResourceManager() {
+	freeResources();
+}
+
+void ResourceManager::setHeapThreshold(int min, int max) {
+	assert(0 < max);
+	assert(min <= max);
+	_maxHeapThreshold = max;
+	_minHeapThreshold = min;
+}
+
 bool ResourceManager::validateResource(const char *str, int type, int idx) const {
 	if (type < rtFirst || type > rtLast || (uint) idx >= (uint)num[type]) {
 		error("%s Illegal Glob type %s (%d) num %d", str, resTypeFromId(type), type, idx);
@@ -850,7 +858,6 @@ bool ResourceManager::validateResource(const char *str, int type, int idx) const
 void ResourceManager::nukeResource(int type, int idx) {
 	byte *ptr;
 
-	CHECK_HEAP
 	if (!address[type])
 		return;
 
@@ -911,7 +918,7 @@ bool ResourceManager::isLocked(int type, int i) const {
 }
 
 bool ScummEngine::isResourceInUse(int type, int i) const {
-	if (!res.validateResource("isResourceInUse", type, i))
+	if (!_res->validateResource("isResourceInUse", type, i))
 		return false;
 	switch (type) {
 	case rtRoom:
@@ -929,7 +936,7 @@ bool ScummEngine::isResourceInUse(int type, int i) const {
 	case rtCharset:
 		return _charset->getCurID() == i;
 	case rtImage:
-		return res.isModified(type, i) != 0;
+		return _res->isModified(type, i) != 0;
 	case rtSpoolBuffer:
 		return _sound->isSoundRunning(10000 + i) != 0;
 	default:
@@ -1013,14 +1020,14 @@ void ScummEngine::loadPtrToResource(int type, int resindex, const byte *source)
 	byte *alloced;
 	int i, len;
 
-	res.nukeResource(type, resindex);
+	_res->nukeResource(type, resindex);
 
 	len = resStrLen(source) + 1;
 
 	if (len <= 0)
 		return;
 
-	alloced = res.createResource(type, resindex, len);
+	alloced = _res->createResource(type, resindex, len);
 
 	if (!source) {
 		alloced[0] = fetchScriptByte();
@@ -1273,32 +1280,30 @@ void ScummEngine::allocateArrays() {
 	if (_game.heversion >= 60) {
 		_arraySlot = (byte *)calloc(_numArray, 1);
 	}
-	if (_game.heversion >= 70) {
-		_storedFlObjects = (ObjectData *)calloc(100, sizeof(ObjectData));
-	}
 
-	allocResTypeData(rtCostume, (_game.features & GF_NEW_COSTUMES) ? MKID_BE('AKOS') : MKID_BE('COST'),
+	_res->allocResTypeData(rtCostume, (_game.features & GF_NEW_COSTUMES) ? MKID_BE('AKOS') : MKID_BE('COST'),
 								_numCostumes, "costume", 1);
-	allocResTypeData(rtRoom, MKID_BE('ROOM'), _numRooms, "room", 1);
-	allocResTypeData(rtRoomImage, MKID_BE('RMIM'), _numRooms, "room image", 1);
-	allocResTypeData(rtRoomScripts, MKID_BE('RMSC'), _numRooms, "room script", 1);
-	allocResTypeData(rtSound, MKID_BE('SOUN'), _numSounds, "sound", 2);
-	allocResTypeData(rtScript, MKID_BE('SCRP'), _numScripts, "script", 1);
-	allocResTypeData(rtCharset, MKID_BE('CHAR'), _numCharsets, "charset", 1);
-	allocResTypeData(rtObjectName, 0, _numNewNames, "new name", 0);
-	allocResTypeData(rtInventory, 0, _numInventory, "inventory", 0);
-	allocResTypeData(rtTemp, 0, 10, "temp", 0);
-	allocResTypeData(rtScaleTable, 0, 5, "scale table", 0);
-	allocResTypeData(rtActorName, 0, _numActors, "actor name", 0);
-	allocResTypeData(rtVerb, 0, _numVerbs, "verb", 0);
-	allocResTypeData(rtString, 0, _numArray, "array", 0);
-	allocResTypeData(rtFlObject, 0, _numFlObject, "flobject", 0);
-	allocResTypeData(rtMatrix, 0, 10, "boxes", 0);
-	allocResTypeData(rtImage, MKID_BE('AWIZ'), _numImages, "images", 1);
-	allocResTypeData(rtTalkie, MKID_BE('TLKE'), _numTalkies, "talkie", 1);
+	_res->allocResTypeData(rtRoom, MKID_BE('ROOM'), _numRooms, "room", 1);
+	_res->allocResTypeData(rtRoomImage, MKID_BE('RMIM'), _numRooms, "room image", 1);
+	_res->allocResTypeData(rtRoomScripts, MKID_BE('RMSC'), _numRooms, "room script", 1);
+	_res->allocResTypeData(rtSound, MKID_BE('SOUN'), _numSounds, "sound", 2);
+	_res->allocResTypeData(rtScript, MKID_BE('SCRP'), _numScripts, "script", 1);
+	_res->allocResTypeData(rtCharset, MKID_BE('CHAR'), _numCharsets, "charset", 1);
+	_res->allocResTypeData(rtObjectName, 0, _numNewNames, "new name", 0);
+	_res->allocResTypeData(rtInventory, 0, _numInventory, "inventory", 0);
+	_res->allocResTypeData(rtTemp, 0, 10, "temp", 0);
+	_res->allocResTypeData(rtScaleTable, 0, 5, "scale table", 0);
+	_res->allocResTypeData(rtActorName, 0, _numActors, "actor name", 0);
+	_res->allocResTypeData(rtVerb, 0, _numVerbs, "verb", 0);
+	_res->allocResTypeData(rtString, 0, _numArray, "array", 0);
+	_res->allocResTypeData(rtFlObject, 0, _numFlObject, "flobject", 0);
+	_res->allocResTypeData(rtMatrix, 0, 10, "boxes", 0);
+	_res->allocResTypeData(rtImage, MKID_BE('AWIZ'), _numImages, "images", 1);
+	_res->allocResTypeData(rtTalkie, MKID_BE('TLKE'), _numTalkies, "talkie", 1);
 
 	if (_game.heversion >= 70) {
-		allocResTypeData(rtSpoolBuffer, 0, 9, "spool buffer", 1);
+		_res->allocResTypeData(rtSpoolBuffer, 0, 9, "spool buffer", 1);
+		_heV7RoomIntOffsets = (uint32 *)calloc(_numRooms, sizeof(uint32));
 	}
 }
 
@@ -1316,11 +1321,7 @@ void ScummEngine::dumpResource(const char *tag, int idx, const byte *ptr, int le
 	else
 		size = READ_BE_UINT32(ptr + 4);
 
-#if defined(MACOS_CARBON)
-	sprintf(buf, ":dumps:%s%d.dmp", tag, idx);
-#else
 	sprintf(buf, "dumps/%s%d.dmp", tag, idx);
-#endif
 
 	out.open(buf, Common::File::kFileWriteMode);
 	if (out.isOpen() == false)
