@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2002-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/scumm/resource_v2.cpp $
- * $Id: resource_v2.cpp 22732 2006-05-29 12:45:11Z kirben $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/scumm/resource_v2.cpp $
+ * $Id: resource_v2.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -33,7 +36,7 @@ void ScummEngine_v2::readClassicIndexFile() {
 	int i;
 
 	if (_game.id == GID_MANIAC) {
-		if (_game.platform == Common::kPlatformC64) {
+		if (_game.version == 0) {
 			_numGlobalObjects = 256;
 			_numRooms = 55;
 			_numCostumes = 25;
@@ -87,40 +90,40 @@ void ScummEngine_v2::readClassicIndexFile() {
 	}
 
 	for (i = 0; i < _numRooms; i++) {
-		res.roomno[rtRoom][i] = i;
+		_res->roomno[rtRoom][i] = i;
 	}
 	_fileHandle->seek(_numRooms, SEEK_CUR);
 	for (i = 0; i < _numRooms; i++) {
-		res.roomoffs[rtRoom][i] = _fileHandle->readUint16LE();
-		if (res.roomoffs[rtRoom][i] == 0xFFFF)
-			res.roomoffs[rtRoom][i] = 0xFFFFFFFF;
+		_res->roomoffs[rtRoom][i] = _fileHandle->readUint16LE();
+		if (_res->roomoffs[rtRoom][i] == 0xFFFF)
+			_res->roomoffs[rtRoom][i] = (uint32)RES_INVALID_OFFSET;
 	}
 
 	for (i = 0; i < _numCostumes; i++) {
-		res.roomno[rtCostume][i] = _fileHandle->readByte();
+		_res->roomno[rtCostume][i] = _fileHandle->readByte();
 	}
 	for (i = 0; i < _numCostumes; i++) {
-		res.roomoffs[rtCostume][i] = _fileHandle->readUint16LE();
-		if (res.roomoffs[rtCostume][i] == 0xFFFF)
-			res.roomoffs[rtCostume][i] = 0xFFFFFFFF;
+		_res->roomoffs[rtCostume][i] = _fileHandle->readUint16LE();
+		if (_res->roomoffs[rtCostume][i] == 0xFFFF)
+			_res->roomoffs[rtCostume][i] = (uint32)RES_INVALID_OFFSET;
 	}
 
 	for (i = 0; i < _numScripts; i++) {
-		res.roomno[rtScript][i] = _fileHandle->readByte();
+		_res->roomno[rtScript][i] = _fileHandle->readByte();
 	}
 	for (i = 0; i < _numScripts; i++) {
-		res.roomoffs[rtScript][i] = _fileHandle->readUint16LE();
-		if (res.roomoffs[rtScript][i] == 0xFFFF)
-			res.roomoffs[rtScript][i] = 0xFFFFFFFF;
+		_res->roomoffs[rtScript][i] = _fileHandle->readUint16LE();
+		if (_res->roomoffs[rtScript][i] == 0xFFFF)
+			_res->roomoffs[rtScript][i] = (uint32)RES_INVALID_OFFSET;
 	}
 
 	for (i = 0; i < _numSounds; i++) {
-		res.roomno[rtSound][i] = _fileHandle->readByte();
+		_res->roomno[rtSound][i] = _fileHandle->readByte();
 	}
 	for (i = 0; i < _numSounds; i++) {
-		res.roomoffs[rtSound][i] = _fileHandle->readUint16LE();
-		if (res.roomoffs[rtSound][i] == 0xFFFF)
-			res.roomoffs[rtSound][i] = 0xFFFFFFFF;
+		_res->roomoffs[rtSound][i] = _fileHandle->readUint16LE();
+		if (_res->roomoffs[rtSound][i] == 0xFFFF)
+			_res->roomoffs[rtSound][i] = (uint32)RES_INVALID_OFFSET;
 	}
 }
 
@@ -144,10 +147,10 @@ void ScummEngine_v2::readEnhancedIndexFile() {
 
 	_fileHandle->readUint16LE(); /* version magic number */
 	readGlobalObjects();
-	readResTypeList(rtRoom, "room");
-	readResTypeList(rtCostume, "costume");
-	readResTypeList(rtScript, "script");
-	readResTypeList(rtSound, "sound");
+	readResTypeList(rtRoom);
+	readResTypeList(rtCostume);
+	readResTypeList(rtScript);
+	readResTypeList(rtSound);
 }
 
 void ScummEngine_v2::readGlobalObjects() {
@@ -197,6 +200,11 @@ void ScummEngine_v2::readIndexFile() {
 				assert(_game.version == 1);
 			readClassicIndexFile();
 			break;
+		case 0x032:
+			printf("Apple II V1 game detected\n");
+			assert(_game.version == 0);
+			readClassicIndexFile();
+			break;
 		default:
 			error("Unknown magic id (0x%X) - this version is unsupported", magic);
 			break;
