@@ -1,6 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2001  Ludvig Strigeus
- * Copyright (C) 2001-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -16,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/scumm/resource_v3.cpp $
- * $Id: resource_v3.cpp 21038 2006-03-03 15:16:02Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/scumm/resource_v3.cpp $
+ * $Id: resource_v3.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -31,30 +33,30 @@ namespace Scumm {
 
 extern const char *resTypeFromId(int id);
 
-void ScummEngine_v3old::readResTypeList(int id, const char *name) {
+void ScummEngine_v3old::readResTypeList(int id) {
 	int num;
 	int i;
 
-	debug(9, "readResTypeList(%s,%s)", resTypeFromId(id), name);
+	debug(9, "readResTypeList(%s)", resTypeFromId(id));
 
 	num = _fileHandle->readByte();
 
 	if (num >= 0xFF) {
-		error("Too many %ss (%d) in directory", name, num);
+		error("Too many %ss (%d) in directory", resTypeFromId(id), num);
 	}
 
 	if (id == rtRoom) {
 		for (i = 0; i < num; i++)
-			res.roomno[id][i] = i;
+			_res->roomno[id][i] = i;
 		_fileHandle->seek(num, SEEK_CUR);
 	} else {
 		for (i = 0; i < num; i++)
-			res.roomno[id][i] = _fileHandle->readByte();
+			_res->roomno[id][i] = _fileHandle->readByte();
 	}
 	for (i = 0; i < num; i++) {
-		res.roomoffs[id][i] = _fileHandle->readUint16LE();
-		if (res.roomoffs[id][i] == 0xFFFF)
-			res.roomoffs[id][i] = 0xFFFFFFFF;
+		_res->roomoffs[id][i] = _fileHandle->readUint16LE();
+		if (_res->roomoffs[id][i] == 0xFFFF)
+			_res->roomoffs[id][i] = (uint32)RES_INVALID_OFFSET;
 	}
 }
 
@@ -87,10 +89,10 @@ void ScummEngine_v3old::readIndexFile() {
 
 	_fileHandle->readUint16LE(); /* version magic number */
 	readGlobalObjects();
-	readResTypeList(rtRoom, "room");
-	readResTypeList(rtCostume, "costume");
-	readResTypeList(rtScript, "script");
-	readResTypeList(rtSound, "sound");
+	readResTypeList(rtRoom);
+	readResTypeList(rtCostume);
+	readResTypeList(rtScript);
+	readResTypeList(rtSound);
 
 	closeRoom();
 }
@@ -102,7 +104,7 @@ void ScummEngine_v3::loadCharset(int no) {
 	uint32 size;
 	memset(_charsetData, 0, sizeof(_charsetData));
 
-	checkRange(2, 0, no, "Loading illegal charset %d");
+	assertRange(0, no, 2, "charset");
 	closeRoom();
 
 	Common::File file;
@@ -116,7 +118,7 @@ void ScummEngine_v3::loadCharset(int no) {
 	}
 
 	size = file.readUint16LE();
-	file.read(res.createResource(rtCharset, no, size), size);
+	file.read(_res->createResource(rtCharset, no, size), size);
 }
 
 } // End of namespace Scumm
