@@ -1,6 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2001  Ludvig Strigeus
- * Copyright (C) 2001-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -16,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/scumm/resource_v4.cpp $
- * $Id: resource_v4.cpp 23373 2006-07-05 22:28:43Z kirben $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/scumm/resource_v4.cpp $
+ * $Id: resource_v4.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -30,6 +32,26 @@
 
 namespace Scumm {
 
+extern const char *resTypeFromId(int id);
+
+void ScummEngine_v4::readResTypeList(int id) {
+	int num;
+	int i;
+
+	debug(9, "readResTypeList(%s)", resTypeFromId(id));
+
+	num = _fileHandle->readUint16LE();
+
+	if (num != _res->num[id]) {
+		error("Invalid number of %ss (%d) in directory", resTypeFromId(id), num);
+	}
+
+	for (i = 0; i < num; i++) {
+		_res->roomno[id][i] = _fileHandle->readByte();
+		_res->roomoffs[id][i] = _fileHandle->readUint32LE();
+	}
+}
+
 void ScummEngine_v4::readIndexFile() {
 	uint16 blocktype;
 	uint32 itemsize;
@@ -105,19 +127,19 @@ void ScummEngine_v4::readIndexFile() {
 			break;
 
 		case 0x5230:	// 'R0'
-			readResTypeList(rtRoom, "room");
+			readResTypeList(rtRoom);
 			break;
 
 		case 0x5330:	// 'S0'
-			readResTypeList(rtScript, "script");
+			readResTypeList(rtScript);
 			break;
 
 		case 0x4E30:	// 'N0'
-			readResTypeList(rtSound, "sound");
+			readResTypeList(rtSound);
 			break;
 
 		case 0x4330:	// 'C0'
-			readResTypeList(rtCostume, "costume");
+			readResTypeList(rtCostume);
 			break;
 
 		case 0x4F30:	// 'O0'
@@ -135,7 +157,7 @@ void ScummEngine_v4::loadCharset(int no) {
 	uint32 size;
 	memset(_charsetData, 0, sizeof(_charsetData));
 
-	checkRange(4, 0, no, "Loading illegal charset %d");
+	assertRange(0, no, 4, "charset");
 	closeRoom();
 
 	Common::File file;
@@ -149,7 +171,7 @@ void ScummEngine_v4::loadCharset(int no) {
 	}
 
 	size = file.readUint32LE() + 11;
-	file.read(res.createResource(rtCharset, no, size), size);
+	file.read(_res->createResource(rtCharset, no, size), size);
 }
 
 void ScummEngine_v4::readMAXS(int blockSize) {
