@@ -1,6 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2001  Ludvig Strigeus
- * Copyright (C) 2001-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -16,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/scumm/room.cpp $
- * $Id: room.cpp 22732 2006-05-29 12:45:11Z kirben $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/scumm/room.cpp $
+ * $Id: room.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -44,7 +46,6 @@ namespace Scumm {
 void ScummEngine::startScene(int room, Actor *a, int objectNr) {
 	int i, where;
 
-	CHECK_HEAP;
 	debugC(DEBUG_GENERAL, "Loading room %d", room);
 
 	stopTalk();
@@ -97,7 +98,7 @@ void ScummEngine::startScene(int room, Actor *a, int objectNr) {
 	nukeArrays(0xFF);
 
 	for (i = 1; i < _numActors; i++) {
-		_actors[i].hideActor();
+		_actors[i]->hideActor();
 	}
 
 	if (_game.version >= 7) {
@@ -118,7 +119,7 @@ void ScummEngine::startScene(int room, Actor *a, int objectNr) {
 	VAR(VAR_ROOM) = room;
 	_fullRedraw = true;
 
-	res.increaseResourceCounter();
+	_res->increaseResourceCounter();
 
 	_currentRoom = room;
 	VAR(VAR_ROOM) = room;
@@ -139,7 +140,6 @@ void ScummEngine::startScene(int room, Actor *a, int objectNr) {
 	if (_currentRoom == 0) {
 		_ENCD_offs = _EXCD_offs = 0;
 		_numObjectsInRoom = 0;
-		restoreFlObjects();
 		return;
 	}
 
@@ -149,7 +149,6 @@ void ScummEngine::startScene(int room, Actor *a, int objectNr) {
 	initBGBuffers(_roomHeight);
 
 	resetRoomObjects();
-	restoreFlObjects();
 
 	if (VAR_ROOM_WIDTH != 0xFF && VAR_ROOM_HEIGHT != 0xFF) {
 		VAR(VAR_ROOM_WIDTH) = _roomWidth;
@@ -161,7 +160,7 @@ void ScummEngine::startScene(int room, Actor *a, int objectNr) {
 	if (VAR_CAMERA_MAX_X != 0xFF)
 		VAR(VAR_CAMERA_MAX_X) = _roomWidth - (_screenWidth / 2);
 
-	if (_game.features & GF_NEW_CAMERA) {
+	if (_game.version >= 7) {
 		VAR(VAR_CAMERA_MIN_Y) = _screenHeight / 2;
 		VAR(VAR_CAMERA_MAX_Y) = _roomHeight - (_screenHeight / 2);
 		setCameraAt(_screenWidth / 2, _screenHeight / 2);
@@ -188,8 +187,8 @@ void ScummEngine::startScene(int room, Actor *a, int objectNr) {
 		a->setDirection(dir + 180);
 		a->stopActorMoving();
 		if (_game.id == GID_SAMNMAX) {
-			camera._cur.x = camera._dest.x = a->_pos.x;
-			setCameraAt(a->_pos.x, a->_pos.y);
+			camera._cur.x = camera._dest.x = a->getPos().x;
+			setCameraAt(a->getPos().x, a->getPos().y);
 		}
 	}
 
@@ -209,13 +208,11 @@ void ScummEngine::startScene(int room, Actor *a, int objectNr) {
 	} else if (_game.version >= 7) {
 		if (camera._follows) {
 			a = derefActor(camera._follows, "startScene: follows");
-			setCameraAt(a->_pos.x, a->_pos.y);
+			setCameraAt(a->getPos().x, a->getPos().y);
 		}
 	}
 
 	_doEffect = true;
-
-	CHECK_HEAP;
 }
 
 /**
@@ -331,7 +328,7 @@ void ScummEngine::setupRoomSubBlocks() {
 
 			id = READ_LE_UINT32(ptr);
 
-			checkRange(_numLocalScripts + _numGlobalScripts, _numGlobalScripts, id, "Invalid local script %d");
+			assertRange(_numGlobalScripts, id, _numLocalScripts + _numGlobalScripts, "local script");
 			_localScriptOffsets[id - _numGlobalScripts] = ptr + 4 - roomResPtr;
 
 			if (_dumpScripts) {
@@ -366,11 +363,11 @@ void ScummEngine::setupRoomSubBlocks() {
 
 			if (_game.version == 8) {
 				id = READ_LE_UINT32(ptr);
-				checkRange(_numLocalScripts + _numGlobalScripts, _numGlobalScripts, id, "Invalid local script %d");
+				assertRange(_numGlobalScripts, id, _numLocalScripts + _numGlobalScripts, "local script");
 				_localScriptOffsets[id - _numGlobalScripts] = ptr + 4 - roomResPtr;
 			} else if (_game.version == 7) {
 				id = READ_LE_UINT16(ptr);
-				checkRange(_numLocalScripts + _numGlobalScripts, _numGlobalScripts, id, "Invalid local script %d");
+				assertRange(_numGlobalScripts, id, _numLocalScripts + _numGlobalScripts, "local script");
 				_localScriptOffsets[id - _numGlobalScripts] = ptr + 2 - roomResPtr;
 			} else {
 				id = ptr[0];
@@ -440,7 +437,8 @@ void ScummEngine::setupRoomSubBlocks() {
 		putClass(182, kObjectClassUntouchable, 0);
 	}
 
-	gdi.roomChanged(roomptr, _IM00_offs, trans);
+	_gdi->roomChanged(roomptr);
+	_gdi->setTransparentColor(trans);
 }
 
 /**
@@ -464,8 +462,8 @@ void ScummEngine::resetRoomSubBlocks() {
 	//
 	memset(_extraBoxFlags, 0, sizeof(_extraBoxFlags));
 
-	res.nukeResource(rtMatrix, 1);
-	res.nukeResource(rtMatrix, 2);
+	_res->nukeResource(rtMatrix, 1);
+	_res->nukeResource(rtMatrix, 2);
 	if (_game.features & GF_SMALL_HEADER) {
 		ptr = findResourceData(MKID_BE('BOXD'), roomptr);
 		if (ptr) {
@@ -476,13 +474,13 @@ void ScummEngine::resetRoomSubBlocks() {
 			else
 				size = numOfBoxes * SIZEOF_BOX + 1;
 
-			res.createResource(rtMatrix, 2, size);
+			_res->createResource(rtMatrix, 2, size);
 			memcpy(getResourceAddress(rtMatrix, 2), ptr, size);
 			ptr += size;
 
 			size = getResourceDataSize(ptr - size - _resourceHeaderSize) - size;
 			if (size > 0) {					// do this :)
-				res.createResource(rtMatrix, 1, size);
+				_res->createResource(rtMatrix, 1, size);
 				memcpy(getResourceAddress(rtMatrix, 1), ptr, size);
 			}
 
@@ -491,7 +489,7 @@ void ScummEngine::resetRoomSubBlocks() {
 		ptr = findResourceData(MKID_BE('BOXD'), roomptr);
 		if (ptr) {
 			int size = getResourceDataSize(ptr);
-			res.createResource(rtMatrix, 2, size);
+			_res->createResource(rtMatrix, 2, size);
 			roomptr = getResourceAddress(rtRoom, _roomResource);
 			ptr = findResourceData(MKID_BE('BOXD'), roomptr);
 			memcpy(getResourceAddress(rtMatrix, 2), ptr, size);
@@ -500,7 +498,7 @@ void ScummEngine::resetRoomSubBlocks() {
 		ptr = findResourceData(MKID_BE('BOXM'), roomptr);
 		if (ptr) {
 			int size = getResourceDataSize(ptr);
-			res.createResource(rtMatrix, 1, size);
+			_res->createResource(rtMatrix, 1, size);
 			roomptr = getResourceAddress(rtRoom, _roomResource);
 			ptr = findResourceData(MKID_BE('BOXM'), roomptr);
 			memcpy(getResourceAddress(rtMatrix, 1), ptr, size);
@@ -510,14 +508,14 @@ void ScummEngine::resetRoomSubBlocks() {
 	//
 	// Load scale data
 	//
-	for (i = 1; i < res.num[rtScaleTable]; i++)
-		res.nukeResource(rtScaleTable, i);
+	for (i = 1; i < _res->num[rtScaleTable]; i++)
+		_res->nukeResource(rtScaleTable, i);
 
 	ptr = findResourceData(MKID_BE('SCAL'), roomptr);
 	if (ptr) {
 		int s1, s2, y1, y2;
 		if (_game.version == 8) {
-			for (i = 1; i < res.num[rtScaleTable]; i++, ptr += 16) {
+			for (i = 1; i < _res->num[rtScaleTable]; i++, ptr += 16) {
 				s1 = READ_LE_UINT32(ptr);
 				y1 = READ_LE_UINT32(ptr + 4);
 				s2 = READ_LE_UINT32(ptr + 8);
@@ -525,7 +523,7 @@ void ScummEngine::resetRoomSubBlocks() {
 				setScaleSlot(i, 0, y1, s1, 0, y2, s2);
 			}
 		} else {
-			for (i = 1; i < res.num[rtScaleTable]; i++, ptr += 8) {
+			for (i = 1; i < _res->num[rtScaleTable]; i++, ptr += 8) {
 				s1 = READ_LE_UINT16(ptr);
 				y1 = READ_LE_UINT16(ptr + 2);
 				s2 = READ_LE_UINT16(ptr + 4);
@@ -557,7 +555,7 @@ void ScummEngine::resetRoomSubBlocks() {
 #endif
 
 	if (_PALS_offs || _CLUT_offs)
-		setPalette(0);
+		setCurrentPalette(0);
 }
 
 
@@ -694,7 +692,7 @@ void ScummEngine_v3old::setupRoomSubBlocks() {
 		}
 	}
 
-	gdi.roomChanged(roomptr, _IM00_offs, 255);
+	_gdi->roomChanged(roomptr);
 }
 
 void ScummEngine_v3old::resetRoomSubBlocks() {
@@ -714,8 +712,8 @@ void ScummEngine_v3old::resetRoomSubBlocks() {
 	//
 	// Load box data
 	//
-	res.nukeResource(rtMatrix, 1);
-	res.nukeResource(rtMatrix, 2);
+	_res->nukeResource(rtMatrix, 1);
+	_res->nukeResource(rtMatrix, 2);
 
 	if (_game.version <= 2)
 		ptr = roomptr + *(roomptr + 0x15);
@@ -735,7 +733,7 @@ void ScummEngine_v3old::resetRoomSubBlocks() {
 			ptr = roomptr + *(roomptr + 0x15);
 			size = numOfBoxes * SIZEOF_BOX_C64 + 1;
 
-			res.createResource(rtMatrix, 2, size + 1);
+			_res->createResource(rtMatrix, 2, size + 1);
 			getResourceAddress(rtMatrix, 2)[0] = numOfBoxes;
 			memcpy(getResourceAddress(rtMatrix, 2) + 1, ptr, size);
 		} else {
@@ -745,7 +743,7 @@ void ScummEngine_v3old::resetRoomSubBlocks() {
 			else
 				size = numOfBoxes * SIZEOF_BOX_V3 + 1;
 
-			res.createResource(rtMatrix, 2, size);
+			_res->createResource(rtMatrix, 2, size);
 			memcpy(getResourceAddress(rtMatrix, 2), ptr, size);
 		}
 
@@ -771,7 +769,7 @@ void ScummEngine_v3old::resetRoomSubBlocks() {
 		}
 
 		if (size > 0) {					// do this :)
-			res.createResource(rtMatrix, 1, size);
+			_res->createResource(rtMatrix, 1, size);
 			memcpy(getResourceAddress(rtMatrix, 1), ptr, size);
 		}
 
@@ -780,8 +778,8 @@ void ScummEngine_v3old::resetRoomSubBlocks() {
 	//
 	// No scale data in old bundle games
 	//
-	for (i = 1; i < res.num[rtScaleTable]; i++)
-		res.nukeResource(rtScaleTable, i);
+	for (i = 1; i < _res->num[rtScaleTable]; i++)
+		_res->nukeResource(rtScaleTable, i);
 
 }
 
