@@ -1,6 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2001  Ludvig Strigeus
- * Copyright (C) 2001-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -16,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/scumm/saveload.cpp $
- * $Id: saveload.cpp 22793 2006-05-31 10:48:40Z kirben $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/scumm/saveload.cpp $
+ * $Id: saveload.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -53,10 +55,6 @@ struct SaveGameHeader {
 	char name[32];
 };
 
-#if !defined(__GNUC__)
-	#pragma START_PACK_STRUCTS
-#endif
-
 struct SaveInfoSection {
 	uint32 type;
 	uint32 version;
@@ -67,11 +65,9 @@ struct SaveInfoSection {
 	
 	uint32 date;
 	uint16 time;
-} GCC_PACK;
+};
 
-#if !defined(__GNUC__)
-	#pragma END_PACK_STRUCTS
-#endif
+#define SaveInfoSectionSize (4+4+4 + 4+4 + 4+2)
 
 #define INFOSECTION_VERSION 2
 
@@ -115,8 +111,8 @@ bool ScummEngine::saveState(int slot, bool compat) {
 
 	Serializer ser(0, out, CURRENT_VER);
 	saveOrLoad(&ser);
-	out->flush();
-	if(out->ioFailed()) {
+	out->finalize();
+	if (out->ioFailed()) {
 		delete out;
 		debug(1, "State save as '%s' FAILED", filename);
 		return false;
@@ -239,8 +235,6 @@ bool ScummEngine::loadState(int slot, bool compat) {
 
 	_sound->pauseSounds(true);
 
-	CHECK_HEAP
-
 	closeRoom();
 
 	memset(_inventory, 0, sizeof(_inventory[0]) * _numInventory);
@@ -253,8 +247,8 @@ bool ScummEngine::loadState(int slot, bool compat) {
 	// Nuke all resources
 	for (i = rtFirst; i <= rtLast; i++)
 		if (i != rtTemp && i != rtBuffer && (i != rtSound || _saveSound || !compat))
-			for (j = 0; j < res.num[i]; j++) {
-				res.nukeResource(i, j);
+			for (j = 0; j < _res->num[i]; j++) {
+				_res->nukeResource(i, j);
 			}
 
 	resetScummVars();
@@ -284,9 +278,11 @@ bool ScummEngine::loadState(int slot, bool compat) {
 	if (_screenTop < 0)
 		_screenTop = 0;
 
-	// WORKAROUND bug #795214: Object 819 could be set to a state of 1 in old save games
-	// Object 819 is part of the exit of the church and should not be drawn.
-	if (hdr.ver < VER(59) && _game.id == GID_MONKEY_VGA) {
+	// WORKAROUND bug #795214: For unknown reasons, object 819 sometimes is in
+	// state 1 in old save games, implying it should be drawn. This in turn
+	// results in a crash when entering the church, as object 819 is part of the
+	//  exitof the church and there are no graphics assigned to it.
+	if (_game.id == GID_MONKEY_VGA) {
 		putState(819, 0);
 	}
 
@@ -326,7 +322,7 @@ bool ScummEngine::loadState(int slot, bool compat) {
 	// loading such an old save game, try to upgrade the old to new format.
 	if (hdr.ver < VER(22)) {
 		// Convert all rtScaleTable resources to matching scale items
-		for (i = 1; i < res.num[rtScaleTable]; i++) {
+		for (i = 1; i < _res->num[rtScaleTable]; i++) {
 			convertScaleTableToScaleSlot(i);
 		}
 	}
@@ -340,7 +336,7 @@ bool ScummEngine::loadState(int slot, bool compat) {
 	// Load the static room data
 	setupRoomSubBlocks();
 
-	if (!(_game.features & GF_NEW_CAMERA)) {
+	if (_game.version < 7) {
 		camera._last.x = camera._cur.x;
 	}
 
@@ -361,7 +357,7 @@ bool ScummEngine::loadState(int slot, bool compat) {
 
 	// Reset charset mask
 	_charset->_hasMask = false;
-	_charset->clearTextSurface();
+	clearTextSurface();
 
 	_lastCodePtr = NULL;
 	_drawObjectQueNr = 0;
@@ -378,7 +374,6 @@ bool ScummEngine::loadState(int slot, bool compat) {
 	if (VAR_VOICE_MODE != 0xFF)
 		VAR(VAR_VOICE_MODE) = ConfMan.getBool("subtitles");
 
-	CHECK_HEAP
 	debug(1, "State loaded from '%s'", filename);
 
 	_sound->pauseSounds(false);
@@ -518,8 +513,9 @@ bool ScummEngine::loadInfos(Common::InSaveFile *file, InfoStuff *stuff) {
 	section.version = file->readUint32BE();
 	section.size = file->readUint32BE();
 
-	// if we extend this we should add a table for the special sizes at the versions to check it
-	if (section.version == INFOSECTION_VERSION && section.size != sizeof(SaveInfoSection)) {
+	// If we ever extend this we should add a table containing the sizes corresponding to each
+	// version, so that we are able to properly verify their correctness.
+	if (section.version == INFOSECTION_VERSION && section.size != SaveInfoSectionSize) {
 		warning("Info section is corrupt");
 		file->skip(section.size);
 		return false;
@@ -528,11 +524,10 @@ bool ScummEngine::loadInfos(Common::InSaveFile *file, InfoStuff *stuff) {
 	section.timeTValue = file->readUint32BE();
 	section.playtime = file->readUint32BE();
 
-	// for compatibility for older version we
-	// to load in with our old method
+	// For header version 1, we load the data in with our old method
 	if (section.version == 1) {
-		time_t curTime_ = section.timeTValue;
-		tm *curTime = localtime(&curTime_);	
+		time_t tmp = section.timeTValue;
+		tm *curTime = localtime(&tmp);	
 		stuff->date = (curTime->tm_mday & 0xFF) << 24 | ((curTime->tm_mon + 1) & 0xFF) << 16 | (curTime->tm_year + 1900) & 0xFFFF;
 		stuff->time = (curTime->tm_hour & 0xFF) << 8 | (curTime->tm_min) & 0xFF;
 	}
@@ -547,10 +542,9 @@ bool ScummEngine::loadInfos(Common::InSaveFile *file, InfoStuff *stuff) {
 	
 	stuff->playtime = section.playtime;
 
-	// skip all newer features, this could make problems if some older version uses more space for
-	// saving informations, but this should NOT happen
-	if (section.size > sizeof(SaveInfoSection)) {
-		file->skip(section.size - sizeof(SaveInfoSection));
+	// Skip over the remaining (unsupported) data
+	if (section.size > SaveInfoSectionSize) {
+		file->skip(section.size - SaveInfoSectionSize);
 	}
 
 	return true;
@@ -560,7 +554,7 @@ void ScummEngine::saveInfos(Common::OutSaveFile* file) {
 	SaveInfoSection section;
 	section.type = MKID_BE('INFO');
 	section.version = INFOSECTION_VERSION;
-	section.size = sizeof(SaveInfoSection);
+	section.size = SaveInfoSectionSize;
 
 	// still save old format for older versions
 	section.timeTValue = time(0);
@@ -718,7 +712,7 @@ void ScummEngine::saveOrLoad(Serializer *s) {
 		MKLINE(ScummEngine, _userPut, sleByte, VER(8)),
 		MKLINE(ScummEngine, _userState, sleUint16, VER(17)),
 		MKLINE(ScummEngine, _cursor.state, sleByte, VER(8)),
-		MK_OBSOLETE(ScummEngine, gdi._cursorActive, sleByte, VER(8), VER(20)),
+		MK_OBSOLETE(ScummEngine, _gdi->_cursorActive, sleByte, VER(8), VER(20)),
 		MKLINE(ScummEngine, _currentCursor, sleByte, VER(8)),
 		MKARRAY(ScummEngine, _grabbedCursor[0], sleByte, 8192, VER(20)),
 		MKLINE(ScummEngine, _cursor.width, sleInt16, VER(20)),
@@ -747,7 +741,7 @@ void ScummEngine::saveOrLoad(Serializer *s) {
 		MKARRAY_OLD(ScummEngine, gfxUsageBits[0], sleUint32, 410, VER(10), VER(13)),
 		MKARRAY(ScummEngine, gfxUsageBits[0], sleUint32, 3 * 410, VER(14)),
 
-		MK_OBSOLETE(ScummEngine, gdi._transparentColor, sleByte, VER(8), VER(50)),
+		MK_OBSOLETE(ScummEngine, _gdi->_transparentColor, sleByte, VER(8), VER(50)),
 		MKARRAY(ScummEngine, _currentPalette[0], sleByte, 768, VER(8)),
 		MKARRAY(ScummEngine, _darkenPalette[0], sleByte, 768, VER(53)),
 
@@ -758,13 +752,13 @@ void ScummEngine::saveOrLoad(Serializer *s) {
 
 		MKLINE(ScummEngine, _egoPositioned, sleByte, VER(8)),
 
-		// gdi._imgBufOffs grew from 4 to 5 entries. Then one day we realized
+		// _gdi->_imgBufOffs grew from 4 to 5 entries. Then one day we realized
 		// that we don't have to store it since initBGBuffers() recomputes it.
-		MK_OBSOLETE_ARRAY(ScummEngine, gdi._imgBufOffs[0], sleUint16, 4, VER(8), VER(9)),
-		MK_OBSOLETE_ARRAY(ScummEngine, gdi._imgBufOffs[0], sleUint16, 5, VER(10), VER(26)),
+		MK_OBSOLETE_ARRAY(ScummEngine, _gdi->_imgBufOffs[0], sleUint16, 4, VER(8), VER(9)),
+		MK_OBSOLETE_ARRAY(ScummEngine, _gdi->_imgBufOffs[0], sleUint16, 5, VER(10), VER(26)),
 
 		// See _imgBufOffs: _numZBuffer is recomputed by initBGBuffers().
-		MK_OBSOLETE(ScummEngine, gdi._numZBuffer, sleByte, VER(8), VER(26)),
+		MK_OBSOLETE(ScummEngine, _gdi->_numZBuffer, sleByte, VER(8), VER(26)),
 
 		MKLINE(ScummEngine, _screenEffectFlag, sleByte, VER(8)),
 
@@ -842,6 +836,8 @@ void ScummEngine::saveOrLoad(Serializer *s) {
 		MKLINE(StringTab, _default.overhead, sleByte, VER(8)),
 		MKLINE(StringTab, no_talk_anim, sleByte, VER(8)),
 		MKLINE(StringTab, _default.no_talk_anim, sleByte, VER(8)),
+		MKLINE(StringTab, wrapping, sleByte, VER(71)),
+		MKLINE(StringTab, _default.wrapping, sleByte, VER(71)),
 		MKEND()
 	};
 
@@ -908,7 +904,7 @@ void ScummEngine::saveOrLoad(Serializer *s) {
 		debug(2, "Saved game MD5: %s", (s->getVersion() >= 39) ? md5str1 : "unknown");
 
 		if (memcmp(md5Backup, _gameMD5, 16) != 0) {
-			warning("Game was saved with different gamedata - you may encounter problems.");
+			warning("Game was saved with different gamedata - you may encounter problems");
 			debug(1, "You have %s and save is %s.", md5str2, md5str1);
  			memcpy(_gameMD5, md5Backup, 16);
 		}
@@ -943,7 +939,7 @@ void ScummEngine::saveOrLoad(Serializer *s) {
 	// Save/load actors
 	//
 	for (i = 0; i < _numActors; i++)
-		_actors[i].saveLoadWithSerializer(s);
+		_actors[i]->saveLoadWithSerializer(s);
 
 
 	//
@@ -1008,11 +1004,11 @@ void ScummEngine::saveOrLoad(Serializer *s) {
 		// number of script resources, savegames won't break.
 		if (s->isSaving()) {
 			for (type = rtFirst; type <= rtLast; type++) {
-				if (res.mode[type] != 1 && type != rtTemp && type != rtBuffer) {
+				if (_res->mode[type] != 1 && type != rtTemp && type != rtBuffer) {
 					s->saveUint16(type);	// Save the res type...
-					for (idx = 0; idx < res.num[type]; idx++) {
+					for (idx = 0; idx < _res->num[type]; idx++) {
 						// Only save resources which actually exist...
-						if (res.address[type][idx]) {
+						if (_res->address[type][idx]) {
 							s->saveUint16(idx);	// Save the index of the resource
 							saveResource(s, type, idx);
 						}
@@ -1024,7 +1020,7 @@ void ScummEngine::saveOrLoad(Serializer *s) {
 		} else {
 			while ((type = s->loadUint16()) != 0xFFFF) {
 				while ((idx = s->loadUint16()) != 0xFFFF) {
-					assert(0 <= idx && idx < res.num[type]);
+					assert(0 <= idx && idx < _res->num[type]);
 					loadResource(s, type, idx);
 				}
 			}
@@ -1034,13 +1030,13 @@ void ScummEngine::saveOrLoad(Serializer *s) {
 		// with index 0, and breaks whenever we change the limit on a given
 		// resource type.
  		for (type = rtFirst; type <= rtLast; type++)
- 			if (res.mode[type] != 1 && type != rtTemp && type != rtBuffer) {
+ 			if (_res->mode[type] != 1 && type != rtTemp && type != rtBuffer) {
  				// For V1-V5 games, there used to be no object name resources.
  				// At some point this changed. But since old savegames rely on
  				// unchanged resource counts, we have to hard code the following check
  				if (_game.version < 6 && type == rtObjectName)
  					continue;
- 				for (idx = 1; idx < res.num[type]; idx++)
+ 				for (idx = 1; idx < _res->num[type]; idx++)
  					saveLoadResource(s, type, idx);
  			}
 	}
@@ -1121,8 +1117,8 @@ void ScummEngine::saveOrLoad(Serializer *s) {
 	//
 	if (s->isSaving()) {
 		for (i = rtFirst; i <= rtLast; i++)
-			for (j = 1; j < res.num[i]; j++) {
-				if (res.isLocked(i, j)) {
+			for (j = 1; j < _res->num[i]; j++) {
+				if (_res->isLocked(i, j)) {
 					s->saveByte(i);
 					s->saveUint16(j);
 				}
@@ -1131,7 +1127,7 @@ void ScummEngine::saveOrLoad(Serializer *s) {
 	} else {
 		while ((i = s->loadByte()) != 0xFF) {
 			j = s->loadUint16();
-			res.lock(i, j);
+			_res->lock(i, j);
 		}
 	}
 
@@ -1158,9 +1154,25 @@ void ScummEngine::saveOrLoad(Serializer *s) {
 	if (_imuse && (_saveSound || !_saveTemporaryState)) {
 		_imuse->save_or_load(s, this);
 	}
+
+	//
+	// Save/load the charset renderer state
+	//
+	if (s->getVersion() >= VER(73)) {
+		_charset->saveLoadWithSerializer(s);
+	} else if (s->isLoading()) {
+		if (s->getVersion() == VER(72)) {
+			_charset->setCurID(s->loadByte());
+		} else {
+			// Before V72, the charset id wasn't saved. This used to cause issues such
+			// as the one described in the bug report #1722153. For these savegames,
+			// we reinitialize the id using a, hopefully, sane value.
+			_charset->setCurID(_string[0]._default.charset);
+		}
+	}
 }
 
-void ScummEngine_c64::saveOrLoad(Serializer *s) {
+void ScummEngine_v0::saveOrLoad(Serializer *s) {
 	ScummEngine::saveOrLoad(s);
 
 	// TODO: Save additional variables
@@ -1197,7 +1209,7 @@ void ScummEngine_v7::saveOrLoad(Serializer *s) {
 
 	const SaveLoadEntry V7Entries[] = {
 		MKLINE(ScummEngine_v7, _subtitleQueuePos, sleInt32, VER(61)),
-		MKLINE(ScummEngine_v7, _verbCharset, sleInt32, VER(68)),
+		MK_OBSOLETE(ScummEngine_v7, _verbCharset, sleInt32, VER(68), VER(68)),
 		MKLINE(ScummEngine_v7, _verbLineSpacing, sleInt32, VER(68)),
 		MKEND()
 	};
@@ -1313,9 +1325,9 @@ void ScummEngine::saveLoadResource(Serializer *ser, int type, int idx) {
 	byte *ptr;
 	uint32 size;
 
-	if (!res.mode[type]) {
+	if (!_res->mode[type]) {
 		if (ser->isSaving()) {
-			ptr = res.address[type][idx];
+			ptr = _res->address[type][idx];
 			if (ptr == NULL) {
 				ser->saveUint32(0);
 				return;
@@ -1335,7 +1347,7 @@ void ScummEngine::saveLoadResource(Serializer *ser, int type, int idx) {
 		} else {
 			size = ser->loadUint32();
 			if (size) {
-				res.createResource(type, idx, size);
+				_res->createResource(type, idx, size);
 				ser->loadBytes(getResourceAddress(type, idx), size);
 				if (type == rtInventory) {
 					_inventory[idx] = ser->loadUint16();
@@ -1352,10 +1364,10 @@ void ScummEngine::saveLoadResource(Serializer *ser, int type, int idx) {
 				}
 			}
 		}
-	} else if (res.mode[type] == 2 && ser->getVersion() >= VER(23)) {
-		// Save/load only a list of resource numbers that need reloaded.
+	} else if (_res->mode[type] == 2 && ser->getVersion() >= VER(23)) {
+		// Save/load only a list of resource numbers that need to be reloaded.
 		if (ser->isSaving()) {
-			ser->saveUint16(res.address[type][idx] ? 1 : 0);
+			ser->saveUint16(_res->address[type][idx] ? 1 : 0);
 		} else {
 			if (ser->loadUint16())
 				ensureResourceLoaded(type, idx);
@@ -1364,10 +1376,10 @@ void ScummEngine::saveLoadResource(Serializer *ser, int type, int idx) {
 }
 
 void ScummEngine::saveResource(Serializer *ser, int type, int idx) {
-	assert(res.address[type][idx]);
+	assert(_res->address[type][idx]);
 
-	if (res.mode[type] == 0) {
-		byte *ptr = res.address[type][idx];
+	if (_res->mode[type] == 0) {
+		byte *ptr = _res->address[type][idx];
 		uint32 size = ((MemBlkHeader *)ptr)->size;
 
 		ser->saveUint32(size);
@@ -1387,12 +1399,12 @@ void ScummEngine::loadResource(Serializer *ser, int type, int idx) {
 		((type == rtSound && idx == 1) || (type == rtSpoolBuffer))) {
 		uint32 size = ser->loadUint32();
 		assert(size);
-		res.createResource(type, idx, size);
+		_res->createResource(type, idx, size);
 		ser->loadBytes(getResourceAddress(type, idx), size);
-	} else if (res.mode[type] == 0) {
+	} else if (_res->mode[type] == 0) {
 		uint32 size = ser->loadUint32();
 		assert(size);
-		res.createResource(type, idx, size);
+		_res->createResource(type, idx, size);
 		ser->loadBytes(getResourceAddress(type, idx), size);
 
 		if (type == rtInventory) {
@@ -1401,7 +1413,7 @@ void ScummEngine::loadResource(Serializer *ser, int type, int idx) {
 		if (type == rtObjectName) {
 			_newNames[idx] = ser->loadUint16();
 		}
-	} else if (res.mode[type] == 2) {
+	} else if (_res->mode[type] == 2) {
 		// HE Games use sound resource 1 for speech
 		if (_game.heversion >= 60 && idx == 1)
 			return;
@@ -1468,7 +1480,7 @@ void Serializer::saveArrayOf(void *b, int len, int datasize, byte filetype) {
 			data = *(uint32 *)at;
 			at += 4;
 		} else {
-			error("saveLoadArrayOf: invalid size %d", datasize);
+			error("saveArrayOf: invalid size %d", datasize);
 		}
 		switch (filetype) {
 		case sleByte:
@@ -1483,7 +1495,7 @@ void Serializer::saveArrayOf(void *b, int len, int datasize, byte filetype) {
 			saveUint32(data);
 			break;
 		default:
-			error("saveLoadArrayOf: invalid filetype %d", filetype);
+			error("saveArrayOf: invalid filetype %d", filetype);
 		}
 	}
 }
@@ -1516,7 +1528,7 @@ void Serializer::loadArrayOf(void *b, int len, int datasize, byte filetype) {
 			data = (int32)loadUint32();
 			break;
 		default:
-			error("saveLoadArrayOf: invalid filetype %d", filetype);
+			error("loadArrayOf: invalid filetype %d", filetype);
 		}
 		if (datasize == 0) {
 			// Do nothing for obsolete data
@@ -1530,7 +1542,7 @@ void Serializer::loadArrayOf(void *b, int len, int datasize, byte filetype) {
 			*(uint32 *)at = data;
 			at += 4;
 		} else {
-			error("saveLoadArrayOf: invalid size %d", datasize);
+			error("loadArrayOf: invalid size %d", datasize);
 		}
 	}
 }
