@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2002-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,16 +18,16 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/scumm/saveload.h $
- * $Id: saveload.h 22793 2006-05-31 10:48:40Z kirben $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/scumm/saveload.h $
+ * $Id: saveload.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
-#ifndef SAVELOAD_H
-#define SAVELOAD_H
+#ifndef SCUMM_SAVELOAD_H
+#define SCUMM_SAVELOAD_H
 
 #include "common/scummsys.h"
-#include <stddef.h>
+#include <stddef.h>	// for ptrdiff_t
 
 namespace Common {
 	class InSaveFile;
@@ -47,7 +50,7 @@ namespace Scumm {
  * only saves/loads those which are valid for the version of the savegame
  * which is being loaded/saved currently.
  */
-#define CURRENT_VER 68
+#define CURRENT_VER 73
 
 /**
  * An auxillary macro, used to specify savegame versions. We use this instead
@@ -71,7 +74,8 @@ namespace Scumm {
  * what our OFFS macro does. Now, for non-POD types this is not really legal, because
  * member need not be at a fixed offset relative to the variable, even if they are in
  * current reality (many of our complex structs are non-POD; for an explanation of
- * what POD means refer to http://www-cpd.fnal.gov/personal/wb/boost/ISOcxx/doc/POD.html)
+ * what POD means refer to <http://en.wikipedia.org/wiki/Plain_Old_Data_Structures> or
+ * to <http://www.informit.com/guides/content.asp?g=cplusplus&seqNum=32&rl=1>)
  */
 #define OFFS(type,item) (((ptrdiff_t)(&((type*)42)->type::item))-42)
 
