@@ -1,6 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2001  Ludvig Strigeus
- * Copyright (C) 2001-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -16,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/scumm/script.cpp $
- * $Id: script.cpp 23821 2006-09-02 12:48:14Z kirben $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/scumm/script.cpp $
+ * $Id: script.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -150,6 +152,12 @@ int ScummEngine::getVerbEntrypoint(int obj, int entry) {
 	const byte *objptr, *verbptr;
 	int verboffs;
 
+	// WORKAROUND for bug #1555938: Disallow pulling the rope if it's
+	// already in the player's inventory.
+	if (_game.id == GID_MONKEY2 && obj == 1047 && entry == 6 && whereIsObject(obj) == WIO_INVENTORY) {
+		return 0;
+	}
+
 	if (whereIsObject(obj) == WIO_NOT_FOUND)
 		return 0;
 
@@ -160,7 +168,8 @@ int ScummEngine::getVerbEntrypoint(int obj, int entry) {
 		verbptr = objptr + 14;
 	else if (_game.version <= 2)
 		verbptr = objptr + 15;
-	else if (_game.features & GF_OLD_BUNDLE)
+	else if ((_game.id == GID_LOOM && _game.platform == Common::kPlatformPCEngine) ||
+		_game.features & GF_OLD_BUNDLE)
 		verbptr = objptr + 17;
 	else if (_game.features & GF_SMALL_HEADER)
 		verbptr = objptr + 19;
@@ -205,7 +214,9 @@ int ScummEngine::getVerbEntrypoint(int obj, int entry) {
 			verbptr += 3;
 		} while (1);
 
-		if (_game.features & GF_SMALL_HEADER)
+		if (_game.id == GID_LOOM && _game.platform == Common::kPlatformPCEngine)
+			return verboffs + READ_LE_UINT16(verbptr + 1) + 3;
+		else if (_game.features & GF_SMALL_HEADER)
 			return READ_LE_UINT16(verbptr + 1);
 		else
 			return verboffs + READ_LE_UINT16(verbptr + 1);
@@ -215,8 +226,7 @@ int ScummEngine::getVerbEntrypoint(int obj, int entry) {
 /* Stop script 'script' */
 void ScummEngine::stopScript(int script) {
 	ScriptSlot *ss;
-	NestedScript *nest;
-	int i, num;
+	int i;
 
 	if (script == 0)
 		return;
@@ -236,19 +246,14 @@ void ScummEngine::stopScript(int script) {
 		}
 	}
 
-	nest = vm.nest;
-	num = vm.numNestedScripts;
-
-	while (num > 0) {
-		if (nest->number == script &&
-				(nest->where == WIO_GLOBAL || nest->where == WIO_LOCAL)) {
-			nukeArrays(nest->slot);
-			nest->number = 0xFF;
-			nest->slot = 0xFF;
-			nest->where = 0xFF;
+	for (i = 0; i < vm.numNestedScripts; ++i) {
+		if (vm.nest[i].number == script &&
+				(vm.nest[i].where == WIO_GLOBAL || vm.nest[i].where == WIO_LOCAL)) {
+			nukeArrays(vm.nest[i].slot);
+			vm.nest[i].number = 0xFF;
+			vm.nest[i].slot = 0xFF;
+			vm.nest[i].where = 0xFF;
 		}
-		nest++;
-		num--;
 	}
 }
 
@@ -371,7 +376,7 @@ void ScummEngine::nukeArrays(byte scriptSlot) {
 
 	for (i = 1; i < _numArray; i++) {
 		if (_arraySlot[i] == scriptSlot) {
-			res.nukeResource(rtString, i);
+			_res->nukeResource(rtString, i);
 			_arraySlot[i] = 0;
 		}
 	}
@@ -393,26 +398,26 @@ void ScummEngine::getScriptBaseAddress() {
 				break;
 		_scriptOrgPointer = getResourceAddress(rtInventory, idx);
 		assert(idx < _numInventory);
-		_lastCodePtr = &res.address[rtInventory][idx];
+		_lastCodePtr = &_res->address[rtInventory][idx];
 		break;
 
 	case WIO_LOCAL:
 	case WIO_ROOM:								/* room script */
 		if (_game.version == 8) {
 			_scriptOrgPointer = getResourceAddress(rtRoomScripts, _roomResource);
-			assert(_roomResource < res.num[rtRoomScripts]);
-			_lastCodePtr = &res.address[rtRoomScripts][_roomResource];
+			assert(_roomResource < _res->num[rtRoomScripts]);
+			_lastCodePtr = &_res->address[rtRoomScripts][_roomResource];
 		} else {
 			_scriptOrgPointer = getResourceAddress(rtRoom, _roomResource);
 			assert(_roomResource < _numRooms);
-			_lastCodePtr = &res.address[rtRoom][_roomResource];
+			_lastCodePtr = &_res->address[rtRoom][_roomResource];
 		}
 		break;
 
 	case WIO_GLOBAL:							/* global script */
 		_scriptOrgPointer = getResourceAddress(rtScript, ss->number);
 		assert(ss->number < _numScripts);
-		_lastCodePtr = &res.address[rtScript][ss->number];
+		_lastCodePtr = &_res->address[rtScript][ss->number];
 		break;
 
 	case WIO_FLOBJECT:						/* flobject script */
@@ -421,7 +426,7 @@ void ScummEngine::getScriptBaseAddress() {
 		idx = _objs[idx].fl_object_index;
 		_scriptOrgPointer = getResourceAddress(rtFlObject, idx);
 		assert(idx < _numFlObject);
-		_lastCodePtr = &res.address[rtFlObject][idx];
+		_lastCodePtr = &_res->address[rtFlObject][idx];
 		break;
 	default:
 		error("Bad type while getting base address");
@@ -454,7 +459,8 @@ void ScummEngine::executeScript() {
 			printf("\n");
 		}
 		_opcode = fetchScriptByte();
-		vm.slot[_currentScript].didexec = 1;
+		if (_game.version > 2) // V0-V2 games didn't use the didexec flag
+			vm.slot[_currentScript].didexec = true;
 		debugC(DEBUG_OPCODES, "Script %d, offset 0x%x: [%X] %s()",
 				vm.slot[_currentScript].number,
 				_scriptPointer - _scriptOrgPointer,
@@ -470,7 +476,6 @@ void ScummEngine::executeScript() {
 		executeOpcode(_opcode);
 
 	}
-	CHECK_HEAP;
 }
 
 byte ScummEngine::fetchScriptByte() {
@@ -542,17 +547,18 @@ int ScummEngine::readVar(uint var) {
 			return !ConfMan.getBool("subtitles");
 		}
 
-		checkRange(_numVariables - 1, 0, var, "Variable %d out of range(r)");
+		assertRange(0, var, _numVariables - 1, "variable (reading)");
 		return _scummVars[var];
 	}
 
 	if (var & 0x8000) {
 		if (_game.heversion >= 80) {
 			var &= 0xFFF;
-			checkRange(_numRoomVariables - 1, 0, var, "Room variable %d out of range(w)");
+			assertRange(0, var, _numRoomVariables - 1, "room variable (reading)");
 			return _roomVars[var];
 
-		} else if (_game.version <= 3 && !(_game.id == GID_INDY3 && (_game.platform == Common::kPlatformFMTowns))) {
+		} else if (_game.version <= 3 && !(_game.id == GID_INDY3 && _game.platform == Common::kPlatformFMTowns) &&
+			!(_game.id == GID_LOOM && _game.platform == Common::kPlatformPCEngine)) {
 			int bit = var & 0xF;
 			var = (var >> 4) & 0xFF;
 
@@ -564,7 +570,7 @@ int ScummEngine::readVar(uint var) {
 				}
 			}
 
-			checkRange(_numVariables - 1, 0, var, "Variable %d out of range(rzb)");
+			assertRange(0, var, _numVariables - 1, "variable (reading)");
 			return (_scummVars[ var ] & ( 1 << bit ) ) ? 1 : 0;
 		} else {
 			var &= 0x7FFF;
@@ -573,7 +579,7 @@ int ScummEngine::readVar(uint var) {
 					return 0;
 			}
 
-			checkRange(_numBitVariables - 1, 0, var, "Bit variable %d out of range(r)");
+			assertRange(0, var, _numBitVariables - 1, "variable (reading)");
 			return (_bitVars[var >> 3] & (1 << (var & 7))) ? 1 : 0;
 		}
 	}
@@ -586,9 +592,9 @@ int ScummEngine::readVar(uint var) {
 		}
 
 		if (_game.heversion >= 80)
-			checkRange(25, 0, var, "Local variable %d out of range(r)");
+			assertRange(0, var, 25, "local variable (reading)");
 		else
-			checkRange(20, 0, var, "Local variable %d out of range(r)");
+			assertRange(0, var, 20, "local variable (reading)");
 		return vm.localvar[_currentScript][var];
 	}
 
@@ -600,7 +606,7 @@ void ScummEngine::writeVar(uint var, int value) {
 	debugC(DEBUG_VARS, "writeVar(%d, %d)", var, value);
 
 	if (!(var & 0xF000)) {
-		checkRange(_numVariables - 1, 0, var, "Variable %d out of range(w)");
+		assertRange(0, var, _numVariables - 1, "variable (writing)");
 
 		if (VAR_SUBTITLES != 0xFF && var == VAR_SUBTITLES) {
 			// Ignore default setting in HE72-73 games
@@ -641,22 +647,23 @@ void ScummEngine::writeVar(uint var, int value) {
 	if (var & 0x8000) {
 		if (_game.heversion >= 80) {
 			var &= 0xFFF;
-			checkRange(_numRoomVariables - 1, 0, var, "Room variable %d out of range(w)");
+			assertRange(0, var, _numRoomVariables - 1, "room variable (writing)");
 			_roomVars[var] = value;
 
-		} else if (_game.version <= 3 && !(_game.id == GID_INDY3 && (_game.platform == Common::kPlatformFMTowns))) {
+		} else if (_game.version <= 3 && !(_game.id == GID_INDY3 && _game.platform == Common::kPlatformFMTowns) &&
+			!(_game.id == GID_LOOM && _game.platform == Common::kPlatformPCEngine)) {
 			// In the old games, the bit variables were using the same memory
 			// as the normal variables!
 			int bit = var & 0xF;
 			var = (var >> 4) & 0xFF;
-			checkRange(_numVariables - 1, 0, var, "Variable %d out of range(wzb)");
+			assertRange(0, var, _numVariables - 1, "variable (writing)");
 			if (value)
 				_scummVars[var] |= ( 1 << bit );
 			else
 				_scummVars[var] &= ~( 1 << bit );
 		} else {
 			var &= 0x7FFF;
-			checkRange(_numBitVariables - 1, 0, var, "Bit variable %d out of range(w)");
+			assertRange(0, var, _numBitVariables - 1, "bit variable (writing)");
 
 			if (value)
 				_bitVars[var >> 3] |= (1 << (var & 7));
@@ -674,9 +681,9 @@ void ScummEngine::writeVar(uint var, int value) {
 		}
 
 		if (_game.heversion >= 80)
-			checkRange(25, 0, var, "Local variable %d out of range(w)");
+			assertRange(0, var, 25, "local variable (writing)");
 		else
-			checkRange(20, 0, var, "Local variable %d out of range(w)");
+			assertRange(0, var, 20, "local variable (writing)");
 
 		vm.localvar[_currentScript][var] = value;
 		return;
@@ -723,27 +730,36 @@ void ScummEngine::stopObjectCode() {
 	ScriptSlot *ss;
 
 	ss = &vm.slot[_currentScript];
-	if (ss->cutsceneOverride == 255) {	/* FIXME: What does this? */
-		warning("Cutscene for script %d has overflown. Resetting.", ss->number);
-		ss->cutsceneOverride = 0;
-	}
+	if (_game.version <= 2) {
+		if (ss->where == WIO_GLOBAL || ss->where == WIO_LOCAL) {
+			stopScript(ss->number);
+		} else {
+			ss->number = 0;
+			ss->status = ssDead;
+		}
+	} else if (_game.version <= 5) {
+		if (ss->where != WIO_GLOBAL && ss->where != WIO_LOCAL) {
+			stopObjectScript(ss->number);
+		} else {
+			if (_game.version >= 4 && ss->cutsceneOverride)
+				error("Script %d ending with active cutscene/override (%d)", ss->number, ss->cutsceneOverride);
 
-	if (ss->where != WIO_GLOBAL && ss->where != WIO_LOCAL) {
-		if (ss->cutsceneOverride) {
-			if (_game.version >= 5)
-				warning("Object %d ending with active cutscene/override (%d)", ss->number, ss->cutsceneOverride);
-			ss->cutsceneOverride = 0;
+			ss->number = 0;
+			ss->status = ssDead;
 		}
 	} else {
-		if (ss->cutsceneOverride) {
-			if (_game.version >= 5)
-				warning("Script %d ending with active cutscene/override (%d)", ss->number, ss->cutsceneOverride);
-			ss->cutsceneOverride = 0;
+		if (ss->where != WIO_GLOBAL && ss->where != WIO_LOCAL) {
+			if (ss->cutsceneOverride)
+				error("Object %d ending with active cutscene/override (%d)", ss->number, ss->cutsceneOverride);
+		} else {
+			if (ss->cutsceneOverride)
+				error("Script %d ending with active cutscene/override (%d)", ss->number, ss->cutsceneOverride);
 		}
+		ss->number = 0;
+		ss->status = ssDead;
 	}
+
 	nukeArrays(_currentScript);
-	ss->number = 0;
-	ss->status = ssDead;
 	_currentScript = 0xFF;
 }
 
@@ -857,7 +873,6 @@ void ScummEngine::freezeScripts(int flag) {
 		for (i = 0; i < NUM_SCRIPT_SLOT; i++) {
 			if (_currentScript != i && vm.slot[i].status != ssDead && !vm.slot[i].freezeResistant) {
 				vm.slot[i].status |= 0x80;
-				vm.slot[i].freezeCount = 1;
 			}
 		}
 		return;
@@ -885,7 +900,6 @@ void ScummEngine::unfreezeScripts() {
 	if (_game.version <= 2) {
 		for (i = 0; i < NUM_SCRIPT_SLOT; i++) {
 			vm.slot[i].status &= 0x7F;
-			vm.slot[i].freezeCount = 0;
 		}
 		return;
 	}
@@ -909,14 +923,14 @@ void ScummEngine::runAllScripts() {
 	int i;
 
 	for (i = 0; i < NUM_SCRIPT_SLOT; i++)
-		vm.slot[i].didexec = 0;
+		vm.slot[i].didexec = false;
 
 	_currentScript = 0xFF;
 	int numCycles = (_game.heversion >= 90) ? VAR(VAR_NUM_SCRIPT_CYCLES) : 1;
 
 	for (int cycle = 1; cycle <= numCycles; cycle++) {
 		for (i = 0; i < NUM_SCRIPT_SLOT; i++) {
-			if (vm.slot[i].cycle == cycle && vm.slot[i].status == ssRunning && vm.slot[i].didexec == 0) {
+			if (vm.slot[i].cycle == cycle && vm.slot[i].status == ssRunning && !vm.slot[i].didexec) {
 				_currentScript = (byte)i;
 				getScriptBaseAddress();
 				getScriptEntryPoint();
@@ -1024,7 +1038,7 @@ void ScummEngine::killScriptsAndResources() {
 						continue;
 
 					_newNames[i] = 0;
-					res.nukeResource(rtObjectName, i);
+					_res->nukeResource(rtObjectName, i);
 				}
 			}
 		}
@@ -1049,7 +1063,7 @@ void ScummEngine::doSentence(int verb, int objectA, int objectB) {
 		if (objectA == objectB)
 			return;
 
-		if (_sentenceNum) {
+		if (_sentenceNum > 0) {
 			st = &_sentence[_sentenceNum - 1];
 
 			// Check if this doSentence request is identical to the previous one;
@@ -1060,6 +1074,7 @@ void ScummEngine::doSentence(int verb, int objectA, int objectB) {
 
 	}
 
+	assert(_sentenceNum < NUM_SENTENCE);
 	st = &_sentence[_sentenceNum++];
 
 	st->verb = verb;
@@ -1074,6 +1089,7 @@ void ScummEngine::checkAndRunSentenceScript() {
 	int localParamList[24];
 	const ScriptSlot *ss;
 	int sentenceScript;
+
 	if (_game.version <= 2)
 		sentenceScript = 2;
 	else
@@ -1091,20 +1107,21 @@ void ScummEngine::checkAndRunSentenceScript() {
 		return;
 
 	_sentenceNum--;
+	SentenceTab &st = _sentence[_sentenceNum];
 
 	if (_game.version < 7)
-		if (_sentence[_sentenceNum].preposition && _sentence[_sentenceNum].objectB == _sentence[_sentenceNum].objectA)
+		if (st.preposition && st.objectB == st.objectA)
 			return;
 
 	if (_game.version <= 2) {
-		VAR(VAR_ACTIVE_VERB) = _sentence[_sentenceNum].verb;
-		VAR(VAR_ACTIVE_OBJECT1) = _sentence[_sentenceNum].objectA;
-		VAR(VAR_ACTIVE_OBJECT2) = _sentence[_sentenceNum].objectB;
-		VAR(VAR_VERB_ALLOWED) = (0 != getVerbEntrypoint(_sentence[_sentenceNum].objectA, _sentence[_sentenceNum].verb));
+		VAR(VAR_ACTIVE_VERB) = st.verb;
+		VAR(VAR_ACTIVE_OBJECT1) = st.objectA;
+		VAR(VAR_ACTIVE_OBJECT2) = st.objectB;
+		VAR(VAR_VERB_ALLOWED) = (0 != getVerbEntrypoint(st.objectA, st.verb));
 	} else {
-		localParamList[0] = _sentence[_sentenceNum].verb;
-		localParamList[1] = _sentence[_sentenceNum].objectA;
-		localParamList[2] = _sentence[_sentenceNum].objectB;
+		localParamList[0] = st.verb;
+		localParamList[1] = st.objectA;
+		localParamList[2] = st.objectB;
 
 
 		if (_game.id == GID_FT && !isValidActor(localParamList[1]) && !isValidActor(localParamList[2])) {
@@ -1125,7 +1142,7 @@ void ScummEngine::checkAndRunSentenceScript() {
 			// For now we assume that if there are more than 460 scripts, then
 			// the pair 29/104 is used, else the pair 28/103.
 			
-			if (res.num[rtScript] > 460) {
+			if (_res->num[rtScript] > 460) {
 				if (sentenceScript == 104)
 					sentenceScript = 29;
 			} else {
