@@ -1,6 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2001  Ludvig Strigeus
- * Copyright (C) 2001-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -16,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/scumm/script_v2.cpp $
- * $Id: script_v2.cpp 23590 2006-07-24 12:52:53Z kirben $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/scumm/script_v2.cpp $
+ * $Id: script_v2.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -59,7 +61,7 @@ void ScummEngine_v2::setupOpcodes() {
 		OPCODE(o2_ifNotState08),
 		/* 10 */
 		OPCODE(o5_getObjectOwner),
-		OPCODE(o2_animateActor),
+		OPCODE(o5_animateActor),
 		OPCODE(o2_panCameraTo),
 		OPCODE(o2_actorOps),
 		/* 14 */
@@ -139,7 +141,7 @@ void ScummEngine_v2::setupOpcodes() {
 		OPCODE(o2_ifState08),
 		/* 50 */
 		OPCODE(o2_pickupObject),
-		OPCODE(o2_animateActor),
+		OPCODE(o5_animateActor),
 		OPCODE(o5_actorFollowCamera),
 		OPCODE(o2_actorOps),
 		/* 54 */
@@ -219,7 +221,7 @@ void ScummEngine_v2::setupOpcodes() {
 		OPCODE(o2_ifNotState08),
 		/* 90 */
 		OPCODE(o5_getObjectOwner),
-		OPCODE(o2_animateActor),
+		OPCODE(o5_animateActor),
 		OPCODE(o2_panCameraTo),
 		OPCODE(o2_actorOps),
 		/* 94 */
@@ -299,7 +301,7 @@ void ScummEngine_v2::setupOpcodes() {
 		OPCODE(o2_ifState08),
 		/* D0 */
 		OPCODE(o2_pickupObject),
-		OPCODE(o2_animateActor),
+		OPCODE(o5_animateActor),
 		OPCODE(o5_actorFollowCamera),
 		OPCODE(o2_actorOps),
 		/* D4 */
@@ -434,13 +436,13 @@ int ScummEngine_v2::readVar(uint var) {
 	if (var >= 14 && var <= 16)
 		var = _scummVars[var];
 
-	checkRange(_numVariables - 1, 0, var, "Variable %d out of range(r)");
+	assertRange(0, var, _numVariables - 1, "variable (reading)");
 	debugC(DEBUG_VARS, "readvar(%d) = %d", var, _scummVars[var]);
 	return _scummVars[var];
 }
 
 void ScummEngine_v2::writeVar(uint var, int value) {
-	checkRange(_numVariables - 1, 0, var, "Variable %d out of range(r)");
+	assertRange(0, var, _numVariables - 1, "variable (writing)");
 	debugC(DEBUG_VARS, "writeVar(%d) = %d", var, value);
 
 	if (VAR_CUTSCENEEXIT_KEY != 0xFF && var == VAR_CUTSCENEEXIT_KEY) {
@@ -472,40 +474,40 @@ void ScummEngine_v2::clearStateCommon(byte type) {
 
 void ScummEngine_v2::o2_setState08() {
 	int obj = getVarOrDirectWord(PARAM_1);
-	putState(obj, getState(obj) | 0x08);
+	putState(obj, getState(obj) | kObjectState_08);
 	markObjectRectAsDirty(obj);
 	clearDrawObjectQueue();
 }
 
 void ScummEngine_v2::o2_clearState08() {
 	int obj = getVarOrDirectWord(PARAM_1);
-	putState(obj, getState(obj) & ~0x08);
+	putState(obj, getState(obj) & ~kObjectState_08);
 	markObjectRectAsDirty(obj);
 	clearDrawObjectQueue();
 }
 
 void ScummEngine_v2::o2_setState04() {
-	setStateCommon(0x04);
+	setStateCommon(kObjectStateLocked);
 }
 
 void ScummEngine_v2::o2_clearState04() {
-	clearStateCommon(0x04);
+	clearStateCommon(kObjectStateLocked);
 }
 
 void ScummEngine_v2::o2_setState02() {
-	setStateCommon(0x02);
+	setStateCommon(kObjectStateUntouchable);
 }
 
 void ScummEngine_v2::o2_clearState02() {
-	clearStateCommon(0x02);
+	clearStateCommon(kObjectStateUntouchable);
 }
 
 void ScummEngine_v2::o2_setState01() {
-	setStateCommon(0x01);
+	setStateCommon(kObjectStatePickupable);
 }
 
 void ScummEngine_v2::o2_clearState01() {
-	clearStateCommon(0x01);
+	clearStateCommon(kObjectStatePickupable);
 }
 
 void ScummEngine_v2::o2_assignVarWordIndirect() {
@@ -575,51 +577,51 @@ void ScummEngine_v2::o2_getBitVar() {
 void ScummEngine_v2::ifStateCommon(byte type) {
 	int obj = getVarOrDirectWord(PARAM_1);
 
-	if ((getState(obj) & type) == 0)
-		o5_jumpRelative();
-	else
+	if ((getState(obj) & type) != 0)
 		ignoreScriptWord();
+	else
+		o5_jumpRelative();
 }
 
 void ScummEngine_v2::ifNotStateCommon(byte type) {
 	int obj = getVarOrDirectWord(PARAM_1);
 
-	if ((getState(obj) & type) != 0)
-		o5_jumpRelative();
-	else
+	if ((getState(obj) & type) == 0)
 		ignoreScriptWord();
+	else
+		o5_jumpRelative();
 }
 
 void ScummEngine_v2::o2_ifState08() {
-	ifStateCommon(0x08);
+	ifStateCommon(kObjectState_08);
 }
 
 void ScummEngine_v2::o2_ifNotState08() {
-	ifNotStateCommon(0x08);
+	ifNotStateCommon(kObjectState_08);
 }
 
 void ScummEngine_v2::o2_ifState04() {
-	ifStateCommon(0x04);
+	ifStateCommon(kObjectStateLocked);
 }
 
 void ScummEngine_v2::o2_ifNotState04() {
-	ifNotStateCommon(0x04);
+	ifNotStateCommon(kObjectStateLocked);
 }
 
 void ScummEngine_v2::o2_ifState02() {
-	ifStateCommon(0x02);
+	ifStateCommon(kObjectStateUntouchable);
 }
 
 void ScummEngine_v2::o2_ifNotState02() {
-	ifNotStateCommon(0x02);
+	ifNotStateCommon(kObjectStateUntouchable);
 }
 
 void ScummEngine_v2::o2_ifState01() {
-	ifStateCommon(0x01);
+	ifStateCommon(kObjectStatePickupable);
 }
 
 void ScummEngine_v2::o2_ifNotState01() {
-	ifNotStateCommon(0x01);
+	ifNotStateCommon(kObjectStatePickupable);
 }
 
 void ScummEngine_v2::o2_addIndirect() {
@@ -753,10 +755,10 @@ void ScummEngine_v2::o2_drawObject() {
 	i = _numLocalObjects;
 	while (i--) {
 		if (_objs[i].obj_nr && _objs[i].x_pos == x && _objs[i].y_pos == y && _objs[i].width == w && _objs[i].height == h)
-			putState(_objs[i].obj_nr, getState(_objs[i].obj_nr) & ~0x08);
+			putState(_objs[i].obj_nr, getState(_objs[i].obj_nr) & ~kObjectState_08);
 	}
 
-	putState(obj, getState(od->obj_nr) | 0x08);
+	putState(obj, getState(od->obj_nr) | kObjectState_08);
 }
 
 void ScummEngine_v2::o2_resourceRoutines() {
@@ -787,9 +789,9 @@ void ScummEngine_v2::o2_resourceRoutines() {
 		ensureResourceLoaded(type, resid);
 	} else {
 		if (opcode & 1)
-			res.lock(type, resid);
+			_res->lock(type, resid);
 		else
-			res.unlock(type, resid);
+			_res->unlock(type, resid);
 	}
 }
 
@@ -801,8 +803,6 @@ void ScummEngine_v2::o2_verbOps() {
 	case 0:		// SO_DELETE_VERBS
 		slot = getVarOrDirectByte(PARAM_1) + 1;
 		assert(0 < slot && slot < _numVerbs);
-
-		//printf("o2_verbOps delete slot = %d\n", slot);
 		killVerb(slot);
 		break;
 
@@ -810,11 +810,7 @@ void ScummEngine_v2::o2_verbOps() {
 		verb = fetchScriptByte();
 		state = fetchScriptByte();
 		slot = getVerbSlot(verb, 0);
-
-		//printf("o2_verbOps Verb On/Off: verb = %d, slot = %d, state = %d\n", verb, slot, state);
-
 		_verbs[slot].curmode = state;
-
 		break;
 
 	default: {	// New Verb
@@ -828,9 +824,6 @@ void ScummEngine_v2::o2_verbOps() {
 		else if ((_game.id == GID_MANIAC) && (_game.version == 1))
 			y += 8;
 
-		//printf("o2_verbOps: verb = %d, slot = %d, x = %d, y = %d, unk = %d, name = %s\n",
-		//		verb, slot, x, y, prep, _scriptPointer);
-
 		VerbSlot *vs;
 		assert(0 < slot && slot < _numVerbs);
 
@@ -861,16 +854,26 @@ void ScummEngine_v2::o2_verbOps() {
 		vs->curRect.left = x;
 		vs->curRect.top = y;
 
-		// FIXME: again, this map depends on the language of the game.
+		// FIXME: these keyboard map depends on the language of the game.
 		// E.g. a german keyboard has 'z' and 'y' swapped, while a french
-		// keyboard starts with "awert", etc.
-		const char keyboard[] = {
-				'q','w','e','r','t',
-				'a','s','d','f','g',
-				'z','x','c','v','b'
-			};
-		if (1 <= slot && slot <= ARRAYSIZE(keyboard))
-			vs->key = keyboard[slot - 1];
+		// keyboard starts with "azerty", etc.
+		if (_game.platform == Common::kPlatformNES) {
+			static const char keyboard[] = {
+					'q','w','e','r',
+					'a','s','d','f',
+					'z','x','c','v'
+				};
+			if (1 <= slot && slot <= ARRAYSIZE(keyboard))
+				vs->key = keyboard[slot - 1];
+		} else {
+			static const char keyboard[] = {
+					'q','w','e','r','t',
+					'a','s','d','f','g',
+					'z','x','c','v','b'
+				};
+			if (1 <= slot && slot <= ARRAYSIZE(keyboard))
+				vs->key = keyboard[slot - 1];
+		}
 
 		// It follows the verb name
 		loadPtrToResource(rtVerb, slot, NULL);
@@ -898,6 +901,7 @@ void ScummEngine_v2::o2_doSentence() {
 		return;
 	}
 
+	assert(_sentenceNum < NUM_SENTENCE);
 	st = &_sentence[_sentenceNum++];
 
 	st->verb = a;
@@ -975,7 +979,6 @@ void ScummEngine_v2::o2_doSentence() {
 
 void ScummEngine_v2::o2_drawSentence() {
 	Common::Rect sentenceline;
-	static char sentence[256];
 	const byte *temp;
 	int slot = getVerbSlot(VAR(VAR_SENTENCE_VERB), 0);
 
@@ -983,15 +986,15 @@ void ScummEngine_v2::o2_drawSentence() {
 		return;
 
 	if (getResourceAddress(rtVerb, slot))
-		strcpy(sentence, (char*)getResourceAddress(rtVerb, slot));
+		strcpy(_sentenceBuf, (char*)getResourceAddress(rtVerb, slot));
 	else
 		return;
 
 	if (VAR(VAR_SENTENCE_OBJECT1) > 0) {
 		temp = getObjOrActorName(VAR(VAR_SENTENCE_OBJECT1));
 		if (temp) {
-			strcat(sentence, " ");
-			strcat(sentence, (const char*)temp);
+			strcat(_sentenceBuf, " ");
+			strcat(_sentenceBuf, (const char*)temp);
 		}
 
 		// For V1 games, the engine must compute the preposition.
@@ -1036,16 +1039,16 @@ void ScummEngine_v2::o2_drawSentence() {
 		}
 
 		if (_game.platform == Common::kPlatformNES) {
-			strcat(sentence, (const char *)(getResourceAddress(rtCostume, 78) + VAR(VAR_SENTENCE_PREPOSITION) * 8 + 2));
+			strcat(_sentenceBuf, (const char *)(getResourceAddress(rtCostume, 78) + VAR(VAR_SENTENCE_PREPOSITION) * 8 + 2));
 		} else
-			strcat(sentence, prepositions[lang][VAR(VAR_SENTENCE_PREPOSITION)]);
+			strcat(_sentenceBuf, prepositions[lang][VAR(VAR_SENTENCE_PREPOSITION)]);
 	}
 
 	if (VAR(VAR_SENTENCE_OBJECT2) > 0) {
 		temp = getObjOrActorName(VAR(VAR_SENTENCE_OBJECT2));
 		if (temp) {
-			strcat(sentence, " ");
-			strcat(sentence, (const char*)temp);
+			strcat(_sentenceBuf, " ");
+			strcat(_sentenceBuf, (const char*)temp);
 		}
 	}
 
@@ -1062,7 +1065,7 @@ void ScummEngine_v2::o2_drawSentence() {
 		_string[2].color = 13;
 
 	byte string[80];
-	char *ptr = sentence;
+	char *ptr = _sentenceBuf;
 	int i = 0, len = 0;
 
 	// Maximum length of printable characters
@@ -1094,7 +1097,7 @@ void ScummEngine_v2::o2_drawSentence() {
 		sentenceline.left = 0;
 		sentenceline.right = virtscr[kVerbVirtScreen].w - 1;
 	}
-	restoreBG(sentenceline);
+	restoreBackground(sentenceline);
 
 	drawString(2, (byte*)string);
 }
@@ -1130,8 +1133,8 @@ void ScummEngine_v2::o2_walkActorTo() {
 
 	a = derefActor(act, "o2_walkActorTo");
 
-	x = getVarOrDirectByte(PARAM_2) * 8;
-	y = getVarOrDirectByte(PARAM_3) * 2;
+	x = getVarOrDirectByte(PARAM_2);
+	y = getVarOrDirectByte(PARAM_3);
 
 	a->startWalkActor(x, y, -1);
 }
@@ -1143,13 +1146,13 @@ void ScummEngine_v2::o2_putActor() {
 
 	a = derefActor(act, "o2_putActor");
 
-	x = getVarOrDirectByte(PARAM_2) * 8;
-	y = getVarOrDirectByte(PARAM_3) * 2;
+	x = getVarOrDirectByte(PARAM_2);
+	y = getVarOrDirectByte(PARAM_3);
 
 	if (_game.id == GID_MANIAC && _game.version <= 1 && _game.platform != Common::kPlatformNES)
 		a->setFacing(180);
 
-	a->putActor(x, y, a->_room);
+	a->putActor(x, y);
 }
 
 void ScummEngine_v2::o2_startScript() {
@@ -1171,14 +1174,6 @@ void ScummEngine_v2::o2_stopScript() {
 
 	script = getVarOrDirectByte(PARAM_1);
 
-	if (_game.id == GID_ZAK && _roomResource == 7 && vm.slot[_currentScript].number == 10001) {
-	// FIXME: Nasty hack for bug #771499
-	// Don't let the exit script for room 7 stop the buy script (24), when
-	// switching to the number selection keypad (script 15)
-		if ((script == 24) && isScriptRunning(15))
-			return;
-	}
-
 	if (_game.id == GID_MANIAC && _roomResource == 26 && vm.slot[_currentScript].number == 10001) {
 	// FIXME: Nasty hack for bug #915575
 	// Don't let the exit script for room 26 stop the script (116), when
@@ -1197,7 +1192,7 @@ void ScummEngine_v2::o2_stopScript() {
 }
 
 void ScummEngine_v2::o2_panCameraTo() {
-	panCameraTo(getVarOrDirectByte(PARAM_1) * 8, 0);
+	panCameraTo(getVarOrDirectByte(PARAM_1) * V12_X_MULTIPLIER, 0);
 }
 
 void ScummEngine_v2::o2_walkActorToObject() {
@@ -1209,6 +1204,9 @@ void ScummEngine_v2::o2_walkActorToObject() {
 	if (whereIsObject(obj) != WIO_NOT_FOUND) {
 		int x, y, dir;
 		getObjectXYPos(obj, x, y, dir);
+		AdjustBoxResult r = a->adjustXYToBeInBox(x, y);
+		x = r.x;
+		y = r.y;
 		a->startWalkActor(x, y, dir);
 	}
 }
@@ -1218,16 +1216,18 @@ void ScummEngine_v2::o2_putActorAtObject() {
 	Actor *a;
 
 	a = derefActor(getVarOrDirectByte(PARAM_1), "o2_putActorAtObject");
-
 	obj = getVarOrDirectWord(PARAM_2);
-	if (whereIsObject(obj) != WIO_NOT_FOUND)
+	if (whereIsObject(obj) != WIO_NOT_FOUND) {
 		getObjectXYPos(obj, x, y);
-	else {
-		x = 240;
-		y = 120;
+		AdjustBoxResult r = a->adjustXYToBeInBox(x, y);
+		x = r.x;
+		y = r.y;
+	} else {
+		x = 30;
+		y = 60;
 	}
 
-	a->putActor(x, y, a->_room);
+	a->putActor(x, y);
 }
 
 void ScummEngine_v2::o2_putActorInRoom() {
@@ -1250,7 +1250,7 @@ void ScummEngine_v2::o2_getActorElevation() {
 	getResultPos();
 	int act = getVarOrDirectByte(PARAM_1);
 	Actor *a = derefActor(act, "o2_getActorElevation");
-	setResult(a->getElevation() / 2);
+	setResult(a->getElevation());
 }
 
 void ScummEngine_v2::o2_setActorElevation() {
@@ -1258,30 +1258,22 @@ void ScummEngine_v2::o2_setActorElevation() {
 	int elevation = (int8)getVarOrDirectByte(PARAM_2);
 
 	Actor *a = derefActor(act, "o2_setActorElevation");
-	a->setElevation(elevation * 2);
-}
-
-void ScummEngine_v2::o2_animateActor() {
-	int act = getVarOrDirectByte(PARAM_1);
-	int anim = getVarOrDirectByte(PARAM_2);
-
-	Actor *a = derefActor(act, "o2_animateActor");
-	a->animateActor(anim);
+	a->setElevation(elevation);
 }
 
 void ScummEngine_v2::o2_actorFromPos() {
 	int x, y;
 	getResultPos();
-	x = getVarOrDirectByte(PARAM_1) * 8;
-	y = getVarOrDirectByte(PARAM_2) * 2;
+	x = getVarOrDirectByte(PARAM_1) * V12_X_MULTIPLIER;
+	y = getVarOrDirectByte(PARAM_2) * V12_Y_MULTIPLIER;
 	setResult(getActorFromPos(x, y));
 }
 
 void ScummEngine_v2::o2_findObject() {
 	int obj;
 	getResultPos();
-	int x = getVarOrDirectByte(PARAM_1) * 8;
-	int y = getVarOrDirectByte(PARAM_2) * 2;
+	int x = getVarOrDirectByte(PARAM_1) * V12_X_MULTIPLIER;
+	int y = getVarOrDirectByte(PARAM_2) * V12_Y_MULTIPLIER;
 	obj = findObject(x, y);
 	if (obj == 0 && (_game.platform == Common::kPlatformNES) && (_userState & 0x40)) {
 		if (_mouseOverBoxV2 >= 0 && _mouseOverBoxV2 < 4)
@@ -1295,7 +1287,7 @@ void ScummEngine_v2::o2_getActorX() {
 	getResultPos();
 
 	a = getVarOrDirectByte(PARAM_1);
-	setResult(getObjX(a) / 8);
+	setResult(getObjX(a));
 }
 
 void ScummEngine_v2::o2_getActorY() {
@@ -1303,7 +1295,7 @@ void ScummEngine_v2::o2_getActorY() {
 	getResultPos();
 
 	a = getVarOrDirectByte(PARAM_1);
-	setResult(getObjY(a) / 2);
+	setResult(getObjY(a));
 }
 
 void ScummEngine_v2::o2_isGreater() {
@@ -1353,7 +1345,7 @@ void ScummEngine_v2::o2_lights() {
 	if (c == 0) {
 		if (_game.id == GID_MANIAC && _game.version == 1 && !(_game.platform == Common::kPlatformNES)) {
 			// Convert older light mode values into
-			// equivalent values.of later games
+			// equivalent values of later games.
 			// 0 Darkness
 			// 1 Flashlight
 			// 2 Lighted area
@@ -1384,17 +1376,20 @@ void ScummEngine_v2::o2_loadRoomWithEgo() {
 	a->putActor(0, 0, room);
 	_egoPositioned = false;
 
-	x = (int8)fetchScriptByte() * 8;
-	y = (int8)fetchScriptByte() * 2;
+	x = (int8)fetchScriptByte();
+	y = (int8)fetchScriptByte();
 
 	startScene(a->_room, a, obj);
 
 	getObjectXYPos(obj, x2, y2, dir);
+	AdjustBoxResult r = a->adjustXYToBeInBox(x2, y2);
+	x2 = r.x;
+	y2 = r.y;
 	a->putActor(x2, y2, _currentRoom);
 	a->setDirection(dir + 180);
 
-	camera._dest.x = camera._cur.x = a->_pos.x;
-	setCameraAt(a->_pos.x, a->_pos.y);
+	camera._dest.x = camera._cur.x = a->getPos().x;
+	setCameraAt(a->getPos().x, a->getPos().y);
 	setCameraFollows(a);
 
 	_fullRedraw = true;
@@ -1436,7 +1431,7 @@ void ScummEngine_v2::o2_setBoxFlags() {
 }
 
 void ScummEngine_v2::o2_setCameraAt() {
-	setCameraAtEx(getVarOrDirectByte(PARAM_1) * 8);
+	setCameraAtEx(getVarOrDirectByte(PARAM_1) * V12_X_MULTIPLIER);
 }
 
 void ScummEngine_v2::o2_roomOps() {
@@ -1481,10 +1476,6 @@ void ScummEngine_v2::o2_cutscene() {
 
 	VAR(VAR_CURSORSTATE) = 200;
 
-	// FIXME allows quotes script (173) to start during introudction of
-	// demo mode of V1 Maniac Mansion. setUserState was halting script
-	// 173 before it started.
-	if (!(_game.id == GID_MANIAC && (_game.features & GF_DEMO)))
 	// Hide inventory, freeze scripts, hide cursor
 	setUserState(15);
 
@@ -1551,7 +1542,7 @@ void ScummEngine_v2::o2_pickupObject() {
 	addObjectToInventory(obj, _roomResource);
 	markObjectRectAsDirty(obj);
 	putOwner(obj, VAR(VAR_EGO));
-	putState(obj, getState(obj) | 0xA);
+	putState(obj, getState(obj) | kObjectState_08 | kObjectStateUntouchable);
 	clearDrawObjectQueue();
 
 	runInventoryScript(1);
@@ -1607,7 +1598,7 @@ void ScummEngine_v2::setUserState(byte state) {
 	} else {
 		rect.left = 0;
 	}
-	restoreBG(rect);
+	restoreBackground(rect);
 
 	// Draw all verbs and inventory
 	redrawVerbs();
