@@ -1,6 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2001  Ludvig Strigeus
- * Copyright (C) 2001-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -16,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/scumm/script_v5.cpp $
- * $Id: script_v5.cpp 24392 2006-10-19 12:46:55Z kirben $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/scumm/script_v5.cpp $
+ * $Id: script_v5.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -462,7 +464,7 @@ void ScummEngine_v5::o5_actorOps() {
 		case 11:		// SO_PALETTE
 			i = getVarOrDirectByte(PARAM_1);
 			j = getVarOrDirectByte(PARAM_2);
-			checkRange(31, 0, i, "Illegal palette slot %d");
+			assertRange(0, i, 31, "o5_actorOps: palette slot");
 			a->setPalette(i, j);
 			break;
 		case 12:		// SO_TALK_COLOR
@@ -499,7 +501,7 @@ void ScummEngine_v5::o5_actorOps() {
 			a->_ignoreBoxes = !(_opcode & 1);
 			a->_forceClip = 0;
 			if (a->isInCurrentRoom())
-				a->putActor(a->_pos.x, a->_pos.y, a->_room);
+				a->putActor();
 			break;
 
 		case 22:		// SO_ANIMATION_SPEED
@@ -587,6 +589,18 @@ void ScummEngine_v5::o5_animateActor() {
 		return;
 	}
 
+	// WORKAROUND bug #859513: While on mars, going outside without your helmet
+	// (or missing some other part of your "space suite" will cause your
+	// character to complain ("I can't breathe."). Unfortunately, this is 
+	// coupled with an animate command, making it very difficult to return to
+	// safety (from where you came). The following hack works around this by
+	// ignoring that particular turn command.
+	if (_game.id == GID_ZAK && _currentRoom == 182 && anim == 246 &&
+			((_game.version < 3 && vm.slot[_currentScript].number == 82)
+			|| (_game.version == 3 && vm.slot[_currentScript].number == 131))) {
+		return;
+	}
+
 	Actor *a = derefActor(act, "o5_animateActor");
 	a->animateActor(anim);
 }
@@ -613,6 +627,9 @@ void ScummEngine_v5::o5_chainScript() {
 	// called via chainScript by script 32, and in there Local[5] is defined
 	// to the  actor ID of the opposing soldier. So, we copy that value over
 	// to the Local[5] variable of script 33.
+	// FIXME: This workaround is meant for Indy3 VGA, but we make no checks
+	// to exclude the EGA/Mac/FM-TOWNS versions. Maybe we should check those
+	// (by looking for GF_OLD256 && !platform=FM-TOWNS
 	if (_game.id == GID_INDY3 && vm.slot[cur].number == 32 && script == 33) {
 		vars[5] = vm.localvar[cur][5];
 	}
@@ -746,7 +763,6 @@ void ScummEngine_v5::o5_divide() {
 
 void ScummEngine_v5::o5_doSentence() {
 	int verb;
-	SentenceTab *st;
 
 	verb = getVarOrDirectByte(PARAM_1);
 	if (verb == 0xFE) {
@@ -756,13 +772,9 @@ void ScummEngine_v5::o5_doSentence() {
 		return;
 	}
 
-	st = &_sentence[_sentenceNum++];
-
-	st->verb = verb;
-	st->objectA = getVarOrDirectWord(PARAM_2);
-	st->objectB = getVarOrDirectWord(PARAM_3);
-	st->preposition = (st->objectB != 0);
-	st->freezeCount = 0;
+	int objectA = getVarOrDirectWord(PARAM_2);
+	int objectB = getVarOrDirectWord(PARAM_3);
+	doSentence(verb, objectA, objectB);
 }
 
 void ScummEngine_v5::o5_drawBox() {
@@ -848,10 +860,23 @@ void ScummEngine_v5::o5_getStringWidth() {
 	width = _charset->getStringWidth(0, ptr);
 
 	setResult(width);
-	debug(0, "o5_getStringWidth, result %d", width);
 }
 
+enum StringIds {
+	// The string IDs used by Indy3 to store the episode resp. series IQ points.
+	// Note that we save the episode IQ points but load the series IQ points,
+	// which matches the original Indy3 save/load code. See also the notes
+	// on Feature Request #1666521.
+	STRINGID_IQ_EPISODE = 7,
+	STRINGID_IQ_SERIES = 9
+};
+
 void ScummEngine_v5::o5_saveLoadVars() {
+	// The KIXX XL release of Monkey Island 2 (Amiga disk) used this opcode
+	// as dummy, in order to remove copy protection and keep level selection.
+	if (_game.version == 5)
+		return;
+
 	if (fetchScriptByte() == 1)
 		saveVars();
 	else
@@ -873,20 +898,26 @@ void ScummEngine_v5::saveVars() {
 		case 0x02: // write a range of string variables
 			a = getVarOrDirectByte(PARAM_1);
 			b = getVarOrDirectByte(PARAM_2);
-			debug(0, "stub saveVars: strings %d -> %d", a, b);
+
+			if (a == STRINGID_IQ_EPISODE && b == STRINGID_IQ_EPISODE) {
+				if (_game.id == GID_INDY3) {
+					saveIQPoints();
+				}
+				break;
+			}
+			// FIXME: changing savegame-names not supported
 			break;
 		case 0x03: // open file
 			a = resStrLen(_scriptPointer);
-			debug(0, "stub saveVars to %s", _scriptPointer);
+			strncpy(_saveLoadVarsFilename, (const char *)_scriptPointer, a);
+			_saveLoadVarsFilename[a] = '\0';
 			_scriptPointer += a + 1;
 			break;
 		case 0x04:
 			return;
-			break;
 		case 0x1F: // close file
-			debug(0, "stub saveVars close file");
+			_saveLoadVarsFilename[0] = '\0';
 			return;
-			break;
 		}
 	}
 }
@@ -894,7 +925,6 @@ void ScummEngine_v5::saveVars() {
 void ScummEngine_v5::loadVars() {
 	int a, b;
 
-//	Common::hexdump(_scriptPointer, 64);
 	while ((_opcode = fetchScriptByte()) != 0) {
 		switch (_opcode & 0x1F) {
 		case 0x01: // read a range of variables
@@ -905,26 +935,98 @@ void ScummEngine_v5::loadVars() {
 			debug(0, "stub loadVars: vars %d -> %d", a, b);
 			break;
 		case 0x02: // read a range of string variables
-			a = getVarOrDirectByte(0x80);
-			b = getVarOrDirectByte(0x40);
-			debug(0, "stub loadVars: strings %d -> %d", a, b);
+			a = getVarOrDirectByte(PARAM_1);
+			b = getVarOrDirectByte(PARAM_2);
+
+			int slot;
+			int slotSize;
+			byte* slotContent;
+			int savegameId;
+			char name[32];
+			bool avail_saves[100];
+
+			if (a == STRINGID_IQ_SERIES && b == STRINGID_IQ_SERIES) {
+				// Zak256 loads the IQ script-slot but does not use it -> ignore it
+				if(_game.id == GID_INDY3) {
+					loadIQPoints();
+				}
+				break;
+			} 
+			
+			listSavegames(avail_saves, ARRAYSIZE(avail_saves));							
+			for (slot = a; slot <= b; ++slot) {
+				slotSize = getResourceSize(rtString, slot);
+				slotContent = getResourceAddress(rtString, slot);
+
+				// load savegame names
+				savegameId = slot - a + 1;
+				if (avail_saves[savegameId] && getSavegameName(savegameId, name)) {
+					int pos;
+					char *ptr = name;
+					// slotContent ends with {'\0','@'} -> max. length = slotSize-2
+					for (pos = 0; pos < slotSize - 2; ++pos) {
+						if (!ptr[pos]) 
+							break;
+						// replace special characters
+						if(ptr[pos] >= 32 && ptr[pos] <= 122 && ptr[pos] != 64)
+							slotContent[pos] = ptr[pos];
+						else
+							slotContent[pos] = '_';
+					}
+					slotContent[pos] = '\0';						
+				} else {
+					slotContent[0] = '\0';
+				}
+			}
 			break;
 		case 0x03: // open file
 			a = resStrLen(_scriptPointer);
-			debug(0, "stub loadVars from %s", _scriptPointer);
+			strncpy(_saveLoadVarsFilename, (const char *)_scriptPointer, a);
+			_saveLoadVarsFilename[a] = '\0';
 			_scriptPointer += a + 1;
 			break;
 		case 0x04:
 			return;
-			break;
 		case 0x1F: // close file
-			debug(0, "stub loadVars close file");
+			_saveLoadVarsFilename[0] = '\0';
 			return;
-			break;
 		}
 	}
 }
 
+void ScummEngine_v5::saveIQPoints() {
+	// save Indy3 IQ-points
+	Common::OutSaveFile *file;
+	Common::String filename = _targetName + ".iq";
+
+	file = _saveFileMan->openForSaving(filename.c_str());
+	if (file != NULL) {
+		int size = getResourceSize(rtString, STRINGID_IQ_EPISODE);
+		byte *ptr = getResourceAddress(rtString, STRINGID_IQ_EPISODE);
+		file->write(ptr, size);
+		delete file;
+	}
+}
+
+void ScummEngine_v5::loadIQPoints() {
+	// load Indy3 IQ-points
+	Common::InSaveFile *file;
+	Common::String filename = _targetName + ".iq";
+
+	file = _saveFileMan->openForLoading(filename.c_str());
+	if (file != NULL) {
+		int size = getResourceSize(rtString, STRINGID_IQ_SERIES);
+		byte *ptr = getResourceAddress(rtString, STRINGID_IQ_SERIES);
+		byte *tmp = (byte*)malloc(size);
+		int nread = file->read(tmp, size);
+		if (nread == size) {
+			memcpy(ptr, tmp, size); 
+		}
+		free(tmp);
+		delete file;
+	}
+}
+
 void ScummEngine_v5::o5_expression() {
 	int dst, i;
 
@@ -1107,9 +1209,13 @@ void ScummEngine_v5::o5_getActorY() {
 void ScummEngine_v5::o5_saveLoadGame() {
 	getResultPos();
 	byte a = getVarOrDirectByte(PARAM_1);
-	byte slot = (a & 0x1F) + 1;
+	byte slot = a & 0x1F;
 	byte result = 0;
 
+	// Slot numbers in older games start with 0, in newer games with 1
+	if (_game.version <= 2)
+		slot++;
+
 	if ((_game.id == GID_MANIAC) && (_game.version <= 1)) {
 		// Convert older load/save screen
 		// 1 Load
@@ -1144,20 +1250,22 @@ void ScummEngine_v5::o5_saveLoadGame() {
 			result = 5; // failed to load
 		break;
 	case 0x80: // save
-		if (saveState(slot, _saveTemporaryState))
-			result = 0;
-		else
-			result = 2;
+		//if (saveState(slot, _saveTemporaryState))
+		//	result = 0; // sucess
+		//else
+			result = 2; // failed to save
 		break;
 	case 0xC0: // test if save exists
+		Common::InSaveFile *file;
 		bool avail_saves[100];
 		char filename[256];
 
 		listSavegames(avail_saves, ARRAYSIZE(avail_saves));
 		makeSavegameName(filename, slot, false);
-		if (avail_saves[slot] && (_saveFileMan->openForLoading(filename)))
+		if (avail_saves[slot] && (file = _saveFileMan->openForLoading(filename))) {
 			result = 6; // save file exists
-		else
+			delete file;
+		} else
 			result = 7; // save file does not exist
 		break;
 	default:
@@ -1313,7 +1421,7 @@ void ScummEngine_v5::o5_isActorInBox() {
 	int box = getVarOrDirectByte(PARAM_2);
 	Actor *a = derefActor(act, "o5_isActorInBox");
 
-	if (!checkXYInBoxBounds(box, a->_pos.x, a->_pos.y))
+	if (!checkXYInBoxBounds(box, a->getRealPos().x, a->getRealPos().y))
 		o5_jumpRelative();
 	else
 		ignoreScriptWord();
@@ -1472,7 +1580,7 @@ void ScummEngine_v5::o5_loadRoomWithEgo() {
 
 	a = derefActor(VAR(VAR_EGO), "o5_loadRoomWithEgo");
 
-	a->putActor(a->_pos.x, a->_pos.y, room);
+	a->putActor(room);
 	oldDir = a->getFacing();
 	_egoPositioned = false;
 
@@ -1496,9 +1604,9 @@ void ScummEngine_v5::o5_loadRoomWithEgo() {
 	}
 
 	// This is based on disassembly
-	camera._cur.x = camera._dest.x = a->_pos.x;
+	camera._cur.x = camera._dest.x = a->getPos().x;
 	if ((_game.id == GID_ZAK || _game.id == GID_LOOM) && (_game.platform == Common::kPlatformFMTowns)) {
-		setCameraAt(a->_pos.x, a->_pos.y);
+		setCameraAt(a->getPos().x, a->getPos().y);
 	}
 	setCameraFollows(a);
 
@@ -1618,7 +1726,7 @@ void ScummEngine_v5::o5_putActor() {
 	a = derefActor(getVarOrDirectByte(PARAM_1), "o5_putActor");
 	x = getVarOrDirectWord(PARAM_2);
 	y = getVarOrDirectWord(PARAM_3);
-	a->putActor(x, y, a->_room);
+	a->putActor(x, y);
 }
 
 void ScummEngine_v5::o5_putActorAtObject() {
@@ -1633,7 +1741,7 @@ void ScummEngine_v5::o5_putActorAtObject() {
 		x = 240;
 		y = 120;
 	}
-	a->putActor(x, y, a->_room);
+	a->putActor(x, y);
 }
 
 void ScummEngine_v5::o5_putActorInRoom() {
@@ -1686,6 +1794,12 @@ void ScummEngine_v5::o5_resourceRoutines() {
 
 	int op = _opcode & 0x3F;
 
+	// FIXME: Sound resources are currently missing
+	if (_game.id == GID_LOOM && _game.platform == Common::kPlatformPCEngine &&
+		(op == 2 || op == 6)) {
+			return;
+	}
+
 	switch (op) {
 	case 1:			// SO_LOAD_SCRIPT
 	case 2:			// SO_LOAD_SOUND
@@ -1699,7 +1813,7 @@ void ScummEngine_v5::o5_resourceRoutines() {
 				resid = _resourceMapper[resid & 0x7F];
 
 			if (_currentRoom != resid) {
-				res.setResourceCounter(rtRoom, resid, 1);
+				_res->setResourceCounter(rtRoom, resid, 1);
 			}
 		} else
 			ensureResourceLoaded(rtRoom, resid);
@@ -1712,40 +1826,46 @@ void ScummEngine_v5::o5_resourceRoutines() {
 		if (_game.id == GID_ZAK && (_game.platform == Common::kPlatformFMTowns))
 			error("o5_resourceRoutines %d should not occur in Zak256", op);
 		else
-			res.setResourceCounter(resType[op-5], resid, 0x7F);
+			_res->setResourceCounter(resType[op-5], resid, 0x7F);
 		break;
 	case 9:			// SO_LOCK_SCRIPT
 		if (resid >= _numGlobalScripts)
 			break;
-		res.lock(rtScript, resid);
+		_res->lock(rtScript, resid);
 		break;
 	case 10:		// SO_LOCK_SOUND
-		res.lock(rtSound, resid);
+		// FIXME: Sound resources are currently missing
+		if (_game.id == GID_LOOM && _game.platform == Common::kPlatformPCEngine)
+			break;
+		_res->lock(rtSound, resid);
 		break;
 	case 11:		// SO_LOCK_COSTUME
-		res.lock(rtCostume, resid);
+		_res->lock(rtCostume, resid);
 		break;
 	case 12:		// SO_LOCK_ROOM
 		if (resid > 0x7F)
 			resid = _resourceMapper[resid & 0x7F];
-		res.lock(rtRoom, resid);
+		_res->lock(rtRoom, resid);
 		break;
 
 	case 13:		// SO_UNLOCK_SCRIPT
 		if (resid >= _numGlobalScripts)
 			break;
-		res.unlock(rtScript, resid);
+		_res->unlock(rtScript, resid);
 		break;
 	case 14:		// SO_UNLOCK_SOUND
-		res.unlock(rtSound, resid);
+		// FIXME: Sound resources are currently missing
+		if (_game.id == GID_LOOM && _game.platform == Common::kPlatformPCEngine)
+			break;
+		_res->unlock(rtSound, resid);
 		break;
 	case 15:		// SO_UNLOCK_COSTUME
-		res.unlock(rtCostume, resid);
+		_res->unlock(rtCostume, resid);
 		break;
 	case 16:		// SO_UNLOCK_ROOM
 		if (resid > 0x7F)
 			resid = _resourceMapper[resid & 0x7F];
-		res.unlock(rtRoom, resid);
+		_res->unlock(rtRoom, resid);
 		break;
 
 	case 17:		// SO_CLEAR_HEAP
@@ -1797,7 +1917,7 @@ void ScummEngine_v5::o5_resourceRoutines() {
 void ScummEngine_v5::o5_roomOps() {
 	int a = 0, b = 0, c, d, e;
 
-	if (_game.version == 3) {
+	if (_game.version == 3 && _game.platform != Common::kPlatformPCEngine) {
 		a = getVarOrDirectWord(PARAM_1);
 		b = getVarOrDirectWord(PARAM_2);
 	}
@@ -1805,7 +1925,7 @@ void ScummEngine_v5::o5_roomOps() {
 	_opcode = fetchScriptByte();
 	switch (_opcode & 0x1F) {
 	case 1:		// SO_ROOM_SCROLL
-		if (_game.version != 3) {
+		if (_game.version != 3 || _game.platform == Common::kPlatformPCEngine) {
 			a = getVarOrDirectWord(PARAM_1);
 			b = getVarOrDirectWord(PARAM_2);
 		}
@@ -1822,11 +1942,11 @@ void ScummEngine_v5::o5_roomOps() {
 		break;
 	case 2:		// SO_ROOM_COLOR
 		if (_game.features & GF_SMALL_HEADER) {
-			if (_game.version != 3) {
+			if (_game.version != 3 || _game.platform == Common::kPlatformPCEngine) {
 				a = getVarOrDirectWord(PARAM_1);
 				b = getVarOrDirectWord(PARAM_2);
 			}
-			checkRange(256, 0, a, "o5_roomOps: 2: Illegal room color slot (%d)");
+			assertRange(0, a, 256, "o5_roomOps: 2: room color slot");
 			_roomPalette[b] = a;
 			_fullRedraw = true;
 		} else {
@@ -1835,7 +1955,7 @@ void ScummEngine_v5::o5_roomOps() {
 		break;
 
 	case 3:		// SO_ROOM_SCREEN
-		if (_game.version != 3) {
+		if (_game.version != 3 || _game.platform == Common::kPlatformPCEngine) {
 			a = getVarOrDirectWord(PARAM_1);
 			b = getVarOrDirectWord(PARAM_2);
 		}
@@ -1843,11 +1963,11 @@ void ScummEngine_v5::o5_roomOps() {
 		break;
 	case 4:		// SO_ROOM_PALETTE
 		if (_game.features & GF_SMALL_HEADER) {
-			if (_game.version != 3) {
+			if (_game.version != 3 || _game.platform == Common::kPlatformPCEngine) {
 				a = getVarOrDirectWord(PARAM_1);
 				b = getVarOrDirectWord(PARAM_2);
 			}
-			checkRange(256, 0, a, "o5_roomOps: 2: Illegal room color slot (%d)");
+			assertRange(0, a, 256, "o5_roomOps: 4: room color slot");
 			_shadowPalette[b] = a;
 			setDirtyColors(b, b);
 		} else {
@@ -1876,17 +1996,9 @@ void ScummEngine_v5::o5_roomOps() {
 		setScaleSlot(e - 1, 0, b, a, 0, d, c);
 		break;
 	case 8:		// SO_ROOM_INTENSITY
-		if (_game.features & GF_SMALL_HEADER) {
-			if (_game.version != 3) {
-				a = getVarOrDirectWord(PARAM_1);
-				b = getVarOrDirectWord(PARAM_2);
-			}
-			c = getVarOrDirectWord(PARAM_3);
-		} else {
-			a = getVarOrDirectByte(PARAM_1);
-			b = getVarOrDirectByte(PARAM_2);
-			c = getVarOrDirectByte(PARAM_3);
-		}
+		a = getVarOrDirectByte(PARAM_1);
+		b = getVarOrDirectByte(PARAM_2);
+		c = getVarOrDirectByte(PARAM_3);
 		darkenPalette(a, a, a, b, c);
 		break;
 	case 9:		// SO_ROOM_SAVEGAME
@@ -1916,7 +2028,6 @@ void ScummEngine_v5::o5_roomOps() {
 				case 30:
 					debug(0, "o5_roomOps: unhandled FM-TOWNS fadeEffect %d", a);
 					return;
-					break;
 				}
 			}
 			_switchRoomEffect = (byte)(a & 0xFF);
@@ -1978,9 +2089,12 @@ void ScummEngine_v5::o5_roomOps() {
 				int len = 256, cnt = 0;
 				ptr = (byte *)malloc(len);
 				while (ptr) {
-				  int r = file->read(ptr + cnt, len - cnt);
-				  if ((cnt += r) < len) break;
-				  ptr = (byte *)realloc(ptr, len *= 2);
+					int r = file->read(ptr + cnt, len - cnt);
+					cnt += r;
+					if (cnt < len)
+						break;
+					len *= 2;
+					ptr = (byte *)realloc(ptr, len);
 				}
 				ptr[cnt] = '\0';
 				loadPtrToResource(rtString, a, ptr);
@@ -2002,7 +2116,7 @@ void ScummEngine_v5::o5_roomOps() {
 	case 16:	// SO_CYCLE_SPEED
 		a = getVarOrDirectByte(PARAM_1);
 		b = getVarOrDirectByte(PARAM_2);
-		checkRange(16, 1, a, "o5_roomOps: 16: color cycle out of range (%d)");
+		assertRange(1, a, 16, "o5_roomOps: 16: color cycle");
 		_colorCycle[a - 1].delay = (b != 0) ? 0x4000 / (b * 0x4C) : 0;
 		break;
 	default:
@@ -2071,8 +2185,8 @@ void ScummEngine_v5::o5_setObjectName() {
 void ScummEngine_v5::o5_setOwnerOf() {
 	int obj, owner;
 
-	obj = getVarOrDirectWord(0x80);
-	owner = getVarOrDirectByte(0x40);
+	obj = getVarOrDirectWord(PARAM_1);
+	owner = getVarOrDirectByte(PARAM_2);
 
 	setOwnerOf(obj, owner);
 }
@@ -2164,7 +2278,6 @@ void ScummEngine_v5::o5_startSound() {
 	// WORKAROUND: In the scene where Largo is talking to Mad Marty, the
 	// Woodtick music often resumes before Largo's theme has finished. As
 	// far as I can tell, this is a script bug.
-
 	if (_game.id == GID_MONKEY2 && sound == 110 && _sound->isSoundRunning(151)) {
 		debug(1, "Delaying Woodtick music until Largo's theme has finished\n");
 		_scriptPointer = oldaddr;
@@ -2233,8 +2346,8 @@ void ScummEngine_v5::o5_startScript() {
 
 	getWordVararg(data);
 
-	// FIXME: Script 171 loads a complete room resource, instead of the actual script.
-	// Causing invalid opcode cases, see bug #1290485
+	// WORKAROUND bug #1290485: Script 171 loads a complete room resource,
+	// instead of the actual script, causing invalid opcode cases
 	if (_game.id == GID_ZAK && _game.platform == Common::kPlatformFMTowns && script == 171)
 		return;
 
@@ -2243,15 +2356,15 @@ void ScummEngine_v5::o5_startScript() {
 		// Copy protection was disabled in LucasArts Classic Adventures (PC Disk)
 		if (_game.id == GID_LOOM && _game.platform == Common::kPlatformPC && _game.version == 3 && _currentRoom == 69 && script == 201)
 			script = 205;
-		// Copy protection was disabled in LucasArts Classic Adventures (PC Disk)
-		if (_game.id == GID_MONKEY_VGA && _game.platform == Common::kPlatformPC && script == 152)
+		// Copy protection was disabled in KIXX XL release (Amiga Disk) and
+		// in LucasArts Classic Adventures (PC Disk)
+		if (_game.id == GID_MONKEY_VGA && script == 152)
 			return;
 		// Copy protection was disabled in LucasArts Mac CD Game Pack II (Macintosh CD)
 		if (_game.id == GID_MONKEY && _game.platform == Common::kPlatformMacintosh && script == 155)
 			return;
 	}
 
-
 	runScript(script, (op & 0x20) != 0, (op & 0x40) != 0, data);
 }
 
@@ -2297,7 +2410,7 @@ void ScummEngine_v5::o5_stringOps() {
 	case 2:											/* copystring */
 		a = getVarOrDirectByte(PARAM_1);
 		b = getVarOrDirectByte(PARAM_2);
-		res.nukeResource(rtString, a);
+		_res->nukeResource(rtString, a);
 		ptr = getResourceAddress(rtString, b);
 		if (ptr)
 			loadPtrToResource(rtString, a, ptr);
@@ -2325,9 +2438,9 @@ void ScummEngine_v5::o5_stringOps() {
 	case 5:											/* create empty string */
 		a = getVarOrDirectByte(PARAM_1);
 		b = getVarOrDirectByte(PARAM_2);
-		res.nukeResource(rtString, a);
+		_res->nukeResource(rtString, a);
 		if (b) {
-			ptr = res.createResource(rtString, a, b);
+			ptr = _res->createResource(rtString, a, b);
 			if (ptr) {
 				for (i = 0; i < b; i++)
 					ptr[i] = 0;
@@ -2353,7 +2466,7 @@ void ScummEngine_v5::o5_verbOps() {
 	verb = getVarOrDirectByte(PARAM_1);
 
 	slot = getVerbSlot(verb, 0);
-	checkRange(_numVerbs - 1, 0, slot, "Illegal new verb slot %d");
+	assertRange(0, slot, _numVerbs - 1, "new verb slot");
 
 	vs = &_verbs[slot];
 	vs->verbid = verb;
@@ -2370,7 +2483,7 @@ void ScummEngine_v5::o5_verbOps() {
 		case 2:		// SO_VERB_NAME
 			loadPtrToResource(rtVerb, slot, NULL);
 			if (slot == 0)
-				res.nukeResource(rtVerb, slot);
+				_res->nukeResource(rtVerb, slot);
 			vs->type = kTextVerbType;
 			vs->imgindex = 0;
 			break;
@@ -2383,7 +2496,7 @@ void ScummEngine_v5::o5_verbOps() {
 		case 5:		// SO_VERB_AT
 			vs->curRect.left = getVarOrDirectWord(PARAM_1);
 			vs->curRect.top = getVarOrDirectWord(PARAM_2);
-			// Macintosh verison of indy3ega used different interface, so adjust values.
+			// Macintosh version of indy3ega used different interface, so adjust values.
 			if ((_game.platform == Common::kPlatformMacintosh) && (_game.id == GID_INDY3)) {
 				switch (verb) {
 				case 1:
@@ -2490,12 +2603,12 @@ void ScummEngine_v5::o5_verbOps() {
 		case 20:	// SO_VERB_NAME_STR
 			ptr = getResourceAddress(rtString, getVarOrDirectWord(PARAM_1));
 			if (!ptr)
-				res.nukeResource(rtVerb, slot);
+				_res->nukeResource(rtVerb, slot);
 			else {
 				loadPtrToResource(rtVerb, slot, ptr);
 			}
 			if (slot == 0)
-				res.nukeResource(rtVerb, slot);
+				_res->nukeResource(rtVerb, slot);
 			vs->type = kTextVerbType;
 			vs->imgindex = 0;
 			break;
@@ -2580,11 +2693,6 @@ void ScummEngine_v5::o5_walkActorToActor() {
 	int nr2 = getVarOrDirectByte(PARAM_2);
 	int dist = fetchScriptByte();
 
-	if (nr == 106 && _game.id == GID_INDY4) {
-		printf("Bypassing Indy4 bug\n");
-		return;
-	}
-
 	if (_game.id == GID_LOOM && _game.version == 4 && nr == 1 && nr2 == 0 &&
 		dist == 255 && vm.slot[_currentScript].number == 98) {
 		// WORKAROUND bug #743615: LoomCD script 98 contains this:
@@ -2613,19 +2721,23 @@ void ScummEngine_v5::o5_walkActorToActor() {
 	if (!a2->isInCurrentRoom())
 		return;
 
-	if (_game.version <= 2)
-		dist *= 8;
-	else if (dist == 0xFF) {
+	if (_game.version <= 2) {
+		dist *= V12_X_MULTIPLIER;
+	} else if (dist == 0xFF) {
 		dist = a->_scalex * a->_width / 0xFF;
 		dist += (a2->_scalex * a2->_width / 0xFF) / 2;
 	}
-	x = a2->_pos.x;
-	y = a2->_pos.y;
-	if (x < a->_pos.x)
+	x = a2->getPos().x;
+	y = a2->getPos().y;
+	if (x < a->getPos().x)
 		x += dist;
 	else
 		x -= dist;
 
+	if (_game.version <= 2) {
+		x /= V12_X_MULTIPLIER;
+		y /= V12_Y_MULTIPLIER;
+	}
 	if (_game.version <= 3) {
 		AdjustBoxResult abr = a->adjustXYToBeInBox(x, y);
 		x = abr.x;
@@ -2741,15 +2853,29 @@ void ScummEngine_v5::decodeParseString() {
 				}
 			}
 			break;
-		case 15:	// SO_TEXTSTRING
-			// WORKAROUND: This happens when Chaos introduces
-			// herself to bishop Mandible. Of all the places to put
-			// a typo...
-			if (_game.id == GID_LOOM && strcmp((const char *) _scriptPointer, "I am Choas.") == 0)
-				printString(textSlot, (const byte *) "I am Chaos.");
-			else
-				printString(textSlot, _scriptPointer);
-			_scriptPointer += resStrLen(_scriptPointer) + 1;
+		case 15:{	// SO_TEXTSTRING
+				const int len = resStrLen(_scriptPointer);
+	
+				if (_game.id == GID_LOOM && strcmp((const char *) _scriptPointer, "I am Choas.") == 0) {
+					// WORKAROUND: This happens when Chaos introduces
+					// herself to bishop Mandible. Of all the places to put
+					// a typo...
+					printString(textSlot, (const byte *) "I am Chaos.");
+				} else if (_game.id == GID_INDY4 && _roomResource == 23 && vm.slot[_currentScript].number == 167 &&
+						len == 24 && 0==memcmp(_scriptPointer+16, "pregod", 6)) {
+					// WORKAROUND for bug #1621210.
+					byte tmpBuf[25];
+					memcpy(tmpBuf, _scriptPointer, 25);
+					if (tmpBuf[22] == '8')
+						strcpy((char *)tmpBuf+16, "^18^");
+					else
+						strcpy((char *)tmpBuf+16, "^19^");
+					printString(textSlot, tmpBuf);
+				} else {
+					printString(textSlot, _scriptPointer);
+				}
+				_scriptPointer += len + 1;
+			}
 
 
  			// In SCUMM V1-V3, there were no 'default' values for the text slot
@@ -2804,7 +2930,7 @@ void ScummEngine_v5::o5_oldRoomEffect() {
 				// For now, we force a redraw of the screen background. This
 				// way the Zak end credits seem to work mostly correct.
 				VirtScreen *vs = &virtscr[0];
-				restoreBG(Common::Rect(0,vs->topline, vs->w, vs->topline + vs->h));
+				restoreBackground(Common::Rect(0,vs->topline, vs->w, vs->topline + vs->h));
 				vs->setDirtyRange(0, vs->h);
 				updateDirtyScreen(kMainVirtScreen);
 
