@@ -1,6 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2001  Ludvig Strigeus
- * Copyright (C) 2001-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -16,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/scumm/script_v6.cpp $
- * $Id: script_v6.cpp 23215 2006-06-21 06:58:22Z kirben $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/scumm/script_v6.cpp $
+ * $Id: script_v6.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -263,7 +265,7 @@ void ScummEngine_v6::setupOpcodes() {
 		OPCODE(o6_getActorWidth),
 		OPCODE(o6_wait),
 		OPCODE(o6_getActorScaleX),
-		OPCODE(o6_getActorAnimCounter1),
+		OPCODE(o6_getActorAnimCounter),
 		/* AC */
 		OPCODE(o6_soundKludge),
 		OPCODE(o6_isAnyOf),
@@ -397,7 +399,7 @@ int ScummEngine_v6::popRoomAndObj(int *room) {
 	return obj;
 }
 
-ScummEngine_v6::ArrayHeader *ScummEngine_v6::defineArray(int array, int type, int dim2, int dim1) {
+byte *ScummEngine_v6::defineArray(int array, int type, int dim2, int dim1) {
 	int id;
 	int size;
 	ArrayHeader *ah;
@@ -413,7 +415,7 @@ ScummEngine_v6::ArrayHeader *ScummEngine_v6::defineArray(int array, int type, in
 		// integer arrays. There seems to be no reason for this, and it wastes
 		// space. However, we can't just remove this either, as that would
 		// break savegame compatibility. So do not touch this unless you are
-		// also adding code which updated old savegames, too. And of course
+		// also adding code which updates old savegames, too. And of course
 		// readArray() and writeArray() would have to be updated, too...
 		if (type != kStringArray)
 			type = kIntArray;
@@ -448,13 +450,13 @@ ScummEngine_v6::ArrayHeader *ScummEngine_v6::defineArray(int array, int type, in
 	size *= dim2 + 1;
 	size *= dim1 + 1;
 
-	ah = (ArrayHeader *)res.createResource(rtString, id, size + sizeof(ArrayHeader));
+	ah = (ArrayHeader *)_res->createResource(rtString, id, size + sizeof(ArrayHeader));
 
 	ah->type = TO_LE_16(type);
 	ah->dim1 = TO_LE_16(dim1 + 1);
 	ah->dim2 = TO_LE_16(dim2 + 1);
 
-	return ah;
+	return ah->data;
 }
 
 void ScummEngine_v6::nukeArray(int a) {
@@ -466,7 +468,7 @@ void ScummEngine_v6::nukeArray(int a) {
 		data &= ~0x33539000;
 
 	if (data)
-		res.nukeResource(rtString, data);
+		_res->nukeResource(rtString, data);
 	if (_game.heversion >= 60)
 		_arraySlot[data] = 0;
 
@@ -474,7 +476,7 @@ void ScummEngine_v6::nukeArray(int a) {
 }
 
 int ScummEngine_v6::findFreeArrayId() {
-	byte **addr = res.address[rtString];
+	byte **addr = _res->address[rtString];
 	int i;
 
 	for (i = 1; i < _numArray; i++) {
@@ -492,10 +494,11 @@ ScummEngine_v6::ArrayHeader *ScummEngine_v6::getArray(int array) {
 	if (!ah)
 		return 0;
 
-	// Workaround for a long standing bug where we save array headers in native
-	// endianness, instead of a fixed endianness. We try to detect savegames
-	// which were created on a big endian system and convert them to little
-	// endian.
+	// Workaround for a long standing bug where we saved array headers in native
+	// endianness, instead of a fixed endianness. We now always store the
+	// dimensions in little endian byte order. But to stay compatible with older
+	// savegames, we try to detect savegames which were created on a big endian
+	// system and convert them to the proper little endian format on the fly.
 	if ((FROM_LE_16(ah->dim1) & 0xF000) || (FROM_LE_16(ah->dim2) & 0xF000) || (FROM_LE_16(ah->type) & 0xFF00)) {
 		SWAP16(ah->dim1);
 		SWAP16(ah->dim2);
@@ -518,6 +521,8 @@ int ScummEngine_v6::readArray(int array, int idx, int base) {
 	// [03BD] (5D)           if ((localvar13 != -1) && (localvar14 != -1)) {
 	// [03CF] (B6)             printDebug.begin()
 	// ...
+	// So it checks for invalid array indices only *after* using them to access
+	// the array. Ouch.
 	if (_game.id == GID_FT && array == 447 && _currentRoom == 95 && vm.slot[_currentScript].number == 2010 && idx == -1 && base == -1) {
 		return 0;
 	}
@@ -996,6 +1001,7 @@ void ScummEngine_v6::o6_cursorCommand() {
 	case 0x9A:		// SO_CURSOR_HOTSPOT Set cursor hotspot
 		a = pop();
 		setCursorHotspot(pop(), a);
+		updateCursor();
 		break;
 	case 0x9C:		// SO_CHARSET_SET
 		initCharset(pop());
@@ -1096,7 +1102,7 @@ void ScummEngine_v6::o6_startSound() {
 		offset = pop();
 
 #ifndef DISABLE_SCUMM_7_8
-	if (_game.features & GF_DIGI_IMUSE)
+	if (_game.version >= 7)
 		_imuseDigital->startSfx(pop(), 64);
 	else
 #endif
@@ -1108,7 +1114,7 @@ void ScummEngine_v6::o6_stopSound() {
 }
 
 void ScummEngine_v6::o6_startMusic() {
-	if (_game.features & GF_DIGI_IMUSE)
+	if (_game.version >= 7)
 		error("o6_startMusic() It shouldn't be called here for imuse digital");
 
 	_sound->addSoundToQueue(pop());
@@ -1201,9 +1207,9 @@ void ScummEngine_v6::o6_walkActorToObj() {
 			dist = a2->_scalex * a2->_width / 0xFF;
 			dist += dist / 2;
 		}
-		x = a2->_pos.x;
-		y = a2->_pos.y;
-		if (x < a->_pos.x)
+		x = a2->getPos().x;
+		y = a2->getPos().y;
+		if (x < a->getPos().x)
 			x += dist;
 		else
 			x -= dist;
@@ -1346,7 +1352,7 @@ void ScummEngine_v6::o6_loadRoomWithEgo() {
 	VAR(VAR_WALKTO_OBJ) = 0;
 
 	if (_game.version == 6) {
-		camera._cur.x = camera._dest.x = a->_pos.x;
+		camera._cur.x = camera._dest.x = a->getPos().x;
 		setCameraFollows(a, (_game.heversion >= 60));
 	}
 
@@ -1434,11 +1440,11 @@ void ScummEngine_v6::o6_getActorWidth() {
 }
 
 void ScummEngine_v6::o6_getActorScaleX() {
-	Actor *a = derefActor(pop(), "o6_getActorScale");
+	Actor *a = derefActor(pop(), "o6_getActorScaleX");
 	push(a->_scalex);
 }
 
-void ScummEngine_v6::o6_getActorAnimCounter1() {
+void ScummEngine_v6::o6_getActorAnimCounter() {
 	Actor *a = derefActor(pop(), "o6_getActorAnimCounter");
 	push(a->_cost.animCounter);
 }
@@ -1452,7 +1458,7 @@ void ScummEngine_v6::o6_getAnimateVariable() {
 void ScummEngine_v6::o6_isActorInBox() {
 	int box = pop();
 	Actor *a = derefActor(pop(), "o6_isActorInBox");
-	push(checkXYInBoxBounds(box, a->_pos.x, a->_pos.y));
+	push(checkXYInBoxBounds(box, a->getPos().x, a->getPos().y));
 }
 
 void ScummEngine_v6::o6_getActorLayer() {
@@ -1566,59 +1572,59 @@ void ScummEngine_v6::o6_resourceRoutines() {
 		if (_game.version >= 7)
 			if (resid >= _numGlobalScripts)
 				break;
-		res.setResourceCounter(rtScript, resid, 0x7F);
+		_res->setResourceCounter(rtScript, resid, 0x7F);
 		break;
 	case 105:		// SO_NUKE_SOUND
 		resid = pop();
-		res.setResourceCounter(rtSound, resid, 0x7F);
+		_res->setResourceCounter(rtSound, resid, 0x7F);
 		break;
 	case 106:		// SO_NUKE_COSTUME
 		resid = pop();
-		res.setResourceCounter(rtCostume, resid, 0x7F);
+		_res->setResourceCounter(rtCostume, resid, 0x7F);
 		break;
 	case 107:		// SO_NUKE_ROOM
 		resid = pop();
-		res.setResourceCounter(rtRoom, resid, 0x7F);
+		_res->setResourceCounter(rtRoom, resid, 0x7F);
 		break;
 	case 108:		// SO_LOCK_SCRIPT
 		resid = pop();
 		if (resid >= _numGlobalScripts)
 			break;
-		res.lock(rtScript, resid);
+		_res->lock(rtScript, resid);
 		break;
 	case 109:		// SO_LOCK_SOUND
 		resid = pop();
-		res.lock(rtSound, resid);
+		_res->lock(rtSound, resid);
 		break;
 	case 110:		// SO_LOCK_COSTUME
 		resid = pop();
-		res.lock(rtCostume, resid);
+		_res->lock(rtCostume, resid);
 		break;
 	case 111:		// SO_LOCK_ROOM
 		resid = pop();
 		if (resid > 0x7F)
 			resid = _resourceMapper[resid & 0x7F];
-		res.lock(rtRoom, resid);
+		_res->lock(rtRoom, resid);
 		break;
 	case 112:		// SO_UNLOCK_SCRIPT
 		resid = pop();
 		if (resid >= _numGlobalScripts)
 			break;
-		res.unlock(rtScript, resid);
+		_res->unlock(rtScript, resid);
 		break;
 	case 113:		// SO_UNLOCK_SOUND
 		resid = pop();
-		res.unlock(rtSound, resid);
+		_res->unlock(rtSound, resid);
 		break;
 	case 114:		// SO_UNLOCK_COSTUME
 		resid = pop();
-		res.unlock(rtCostume, resid);
+		_res->unlock(rtCostume, resid);
 		break;
 	case 115:		// SO_UNLOCK_ROOM
 		resid = pop();
 		if (resid > 0x7F)
 			resid = _resourceMapper[resid & 0x7F];
-		res.unlock(rtRoom, resid);
+		_res->unlock(rtRoom, resid);
 		break;
 	case 116:		// SO_CLEAR_HEAP
 		/* this is actually a scumm message */
@@ -1749,7 +1755,7 @@ void ScummEngine_v6::o6_roomOps() {
 	case 187:		// SO_CYCLE_SPEED
 		b = pop();
 		a = pop();
-		checkRange(16, 1, a, "o6_roomOps: 187: color cycle out of range (%d)");
+		assertRange(1, a, 16, "o6_roomOps: 187: color cycle");
 		_colorCycle[a - 1].delay = (b != 0) ? 0x4000 / (b * 0x4C) : 0;
 		break;
 
@@ -1765,7 +1771,7 @@ void ScummEngine_v6::o6_roomOps() {
 		if (_game.id == GID_SAMNMAX && vm.slot[_currentScript].number == 64)
 			setDirtyColors(0, 255);
 		else
-			setPalette(a);
+			setCurrentPalette(a);
 		break;
 	default:
 		error("o6_roomOps: default case %d", subOp);
@@ -1833,7 +1839,7 @@ void ScummEngine_v6::o6_actorOps() {
 	case 86:		// SO_PALETTE
 		j = pop();
 		i = pop();
-		checkRange(255, 0, i, "Illegal palette slot %d");
+		assertRange(0, i, 255, "o6_actorOps: palette slot");
 		a->setPalette(i, j);
 		break;
 	case 87:		// SO_TALK_COLOR
@@ -1863,13 +1869,13 @@ void ScummEngine_v6::o6_actorOps() {
 		a->_ignoreBoxes = 1;
 		a->_forceClip = (_game.version >= 7) ? 100 : 0;
 		if (a->isInCurrentRoom())
-			a->putActor(a->_pos.x, a->_pos.y, a->_room);
+			a->putActor();
 		break;
 	case 96:		// SO_FOLLOW_BOXES
 		a->_ignoreBoxes = 0;
 		a->_forceClip = (_game.version >= 7) ? 100 : 0;
 		if (a->isInCurrentRoom())
-			a->putActor(a->_pos.x, a->_pos.y, a->_room);
+			a->putActor();
 		break;
 	case 97:		// SO_ANIMATION_SPEED
 		a->setAnimSpeed(pop());
@@ -1933,7 +1939,7 @@ void ScummEngine_v6::o6_verbOps() {
 	if (subOp == 196) {
 		_curVerb = pop();
 		_curVerbSlot = getVerbSlot(_curVerb, 0);
-		checkRange(_numVerbs - 1, 0, _curVerbSlot, "Illegal new verb slot %d");
+		assertRange(0, _curVerbSlot, _numVerbs - 1, "new verb slot");
 		return;
 	}
 	vs = &_verbs[_curVerbSlot];
@@ -2081,15 +2087,15 @@ void ScummEngine_v6::o6_arrayOps() {
 	byte subOp = fetchScriptByte();
 	int array = fetchScriptWord();
 	int b, c, d, len;
-	ArrayHeader *ah;
+	byte *data;
 	int list[128];
 
 	switch (subOp) {
 	case 205:		// SO_ASSIGN_STRING
 		b = pop();
 		len = resStrLen(_scriptPointer);
-		ah = defineArray(array, kStringArray, 0, len + 1);
-		copyScriptString(ah->data + b);
+		data = defineArray(array, kStringArray, 0, len + 1);
+		copyScriptString(data + b);
 		break;
 	case 208:		// SO_ASSIGN_INT_LIST
 		b = pop();
@@ -2503,6 +2509,104 @@ void ScummEngine_v6::o6_setBlastObjectWindow() {
 	// So, we just handle this as no-op opcode.
 }
 
+#ifndef DISABLE_SCUMM_7_8
+void ScummEngine_v7::o6_kernelSetFunctions() {
+	int args[30];
+	int num;
+	Actor *a;
+
+	num = getStackList(args, ARRAYSIZE(args));
+
+	switch (args[0]) {
+	case 4:
+		grabCursor(args[1], args[2], args[3], args[4]);
+		break;
+	case 6: {
+			if (_smushFrameRate == 0)
+				_smushFrameRate = 14;
+
+			// SMUSH movie playback
+			if (args[1] == 0) {
+				const char *videoname = (const char *)getStringAddressVar(VAR_VIDEONAME);
+				assert(videoname);
+				if (strcmp(videoname, "sq3.san") == 0)
+					_smushFrameRate = 14;
+
+				// Correct incorrect smush filename in Macintosh FT demo
+				if ((_game.id == GID_FT) && (_game.features & GF_DEMO) && (_game.platform == Common::kPlatformMacintosh) &&
+					(strcmp(videoname, "jumpgorge.san") == 0))
+					_splayer->play("jumpgorg.san", _smushFrameRate);
+				else
+					_splayer->play(videoname, _smushFrameRate);
+
+				if (_game.id == GID_DIG) {
+					_disableFadeInEffect = true;
+				}
+			} else if (_game.id == GID_FT) {
+				const int insaneVarNum = ((_game.features & GF_DEMO) && (_game.platform == Common::kPlatformPC))
+					? 232 : 233;
+
+				_insaneRunning = true;
+				_insane->setSmushParams(_smushFrameRate);
+				_insane->runScene(insaneVarNum);
+				_insaneRunning = false;
+			}
+		}
+		break;
+	case 12:
+		setCursorFromImg(args[1], (uint) - 1, args[2]);
+		break;
+	case 13:
+		derefActor(args[1], "o6_kernelSetFunctions:13")->remapActorPalette(args[2], args[3], args[4], -1);
+		break;
+	case 14:
+		derefActor(args[1], "o6_kernelSetFunctions:14")->remapActorPalette(args[2], args[3], args[4], args[5]);
+		break;
+	case 15:
+		_smushFrameRate = args[1];
+		break;
+	case 16:
+	case 17:
+		enqueueText(getStringAddressVar(VAR_STRING2DRAW), args[3], args[4], args[2], args[1], (args[0] == 16));
+		break;
+	case 20:
+		// it's used for turn on/off 'RadioChatter' effect for voice in the dig, but i's not needed
+		break;
+	case 107:
+		a = derefActor(args[1], "o6_kernelSetFunctions: 107");
+		a->setScale((unsigned char)args[2], -1);
+		break;
+	case 108:
+		setShadowPalette(args[1], args[2], args[3], args[4], args[5], args[6]);
+		break;
+	case 109:
+		setShadowPalette(0, args[1], args[2], args[3], args[4], args[5]);
+		break;
+	case 114:
+		error("o6_kernelSetFunctions: stub114()");
+		break;
+	case 117:
+		freezeScripts(2);
+		break;
+	case 118:
+		enqueueObject(args[1], args[2], args[3], args[4], args[5], args[6], args[7], args[8], 3);
+		break;
+	case 119:
+		enqueueObject(args[1], args[2], args[3], args[4], args[5], args[6], args[7], args[8], 0);
+		break;
+	case 124:
+		_saveSound = args[1];
+		break;
+	case 215:
+		ConfMan.setBool("subtitles", args[1] != 0);
+		break;
+	default:
+		error("o6_kernelSetFunctions: default case %d (param count %d)", args[0], num);
+		break;
+	}
+}
+#endif
+
 void ScummEngine_v6::o6_kernelSetFunctions() {
 	int args[30];
 	int num;
@@ -2510,189 +2614,99 @@ void ScummEngine_v6::o6_kernelSetFunctions() {
 
 	num = getStackList(args, ARRAYSIZE(args));
 
-	if (_game.version >= 7) {
-		switch (args[0]) {
-		case 4:
-			grabCursor(args[1], args[2], args[3], args[4]);
-			break;
-#ifndef DISABLE_SCUMM_7_8
-		case 6: {
-				if (_smushFrameRate == 0)
-					_smushFrameRate = 14;
-
-				// SMUSH movie playback
-				if (args[1] == 0) {
-					assert(getStringAddressVar(VAR_VIDEONAME));
-					if (strcmp((char *)getStringAddressVar(VAR_VIDEONAME), "sq3.san") == 0)
-						_smushFrameRate = 14;
-
-					SmushPlayer *sp = new SmushPlayer(this, _smushFrameRate);
-
-					// Correct incorrect smush filename in Macintosh FT demo
-					if ((_game.id == GID_FT) && (_game.features & GF_DEMO) && (_game.platform == Common::kPlatformMacintosh) &&
-					    (strcmp((char *)getStringAddressVar(VAR_VIDEONAME), "jumpgorge.san") == 0))
-						sp->play("jumpgorg.san");
-					else
-						sp->play((char *)getStringAddressVar(VAR_VIDEONAME));
-					delete sp;
-				} else if (_game.id == GID_FT) {
-					const int insaneVarNum = ((_game.features & GF_DEMO) && (_game.platform == Common::kPlatformPC))
-						? 232 : 233;
-
-					_insaneRunning = true;
-					_insane->setSmushParams(_smushFrameRate);
-					_insane->runScene(insaneVarNum);
-					_insaneRunning = false;
-				}
-			}
-			break;
-#endif
-		case 12:
-			setCursorFromImg(args[1], (uint) - 1, args[2]);
-			break;
-		case 13:
-			derefActor(args[1], "o6_kernelSetFunctions:13")->remapActorPalette(args[2], args[3], args[4], -1);
-			break;
-		case 14:
-			derefActor(args[1], "o6_kernelSetFunctions:14")->remapActorPalette(args[2], args[3], args[4], args[5]);
-			break;
-		case 15:
-			_smushFrameRate = args[1];
-			break;
-		case 16:
-		case 17:
-			enqueueText(getStringAddressVar(VAR_STRING2DRAW), args[3], args[4], args[2], args[1], (args[0] == 16));
-			break;
-		case 20:
-			// it's used for turn on/off 'RadioChatter' effect for voice in the dig, but i's not needed
-			break;
-		case 107:
-			a = derefActor(args[1], "o6_kernelSetFunctions: 107");
-			a->setScale((unsigned char)args[2], -1);
-			break;
-		case 108:
-			setShadowPalette(args[1], args[2], args[3], args[4], args[5], args[6]);
-			break;
-		case 109:
-			setShadowPalette(0, args[1], args[2], args[3], args[4], args[5]);
-			break;
-		case 114:
-			error("o6_kernelSetFunctions: stub114()");
-			break;
-		case 117:
-			freezeScripts(2);
-			break;
-		case 118:
-			enqueueObject(args[1], args[2], args[3], args[4], args[5], args[6], args[7], args[8], 3);
-			break;
-		case 119:
-			enqueueObject(args[1], args[2], args[3], args[4], args[5], args[6], args[7], args[8], 0);
-			break;
-		case 124:
-			_saveSound = args[1];
-			break;
-		case 215:
-			ConfMan.setBool("subtitles", args[1] != 0);
-			break;
-		default:
-			error("o6_kernelSetFunctions: default case %d (param count %d)", args[0], num);
-			break;
-		}
-	} else {
-		switch (args[0]) {
-		case 3:
-			// Dummy case
-			break;
-		case 4:
-			grabCursor(args[1], args[2], args[3], args[4]);
-			break;
-		case 5:
-			fadeOut(args[1]);
-			break;
-		case 6:
-			_fullRedraw = true;
-			redrawBGAreas();
-			setActorRedrawFlags();
-			processActors();
-			fadeIn(args[1]);
-			break;
-		case 8:
-			startManiac();
-			break;
-		case 9:
-			killAllScriptsExceptCurrent();
-			break;
-		case 104:									/* samnmax */
-			nukeFlObjects(args[2], args[3]);
-			break;
-		case 107:									/* set actor scale */
-			a = derefActor(args[1], "o6_kernelSetFunctions: 107");
-			a->setScale((unsigned char)args[2], -1);
-			break;
-		case 108:									/* create proc_special_palette */
-		case 109:
-			// Case 108 and 109 share the same function
-			if (num != 6)
-				error("o6_kernelSetFunctions sub op %d: expected 6 params but got %d", args[0], num);
-			setShadowPalette(args[3], args[4], args[5], args[1], args[2], 0, 256);
-			break;
-		case 110:
-			_charset->clearCharsetMask();
-			break;
-		case 111:
-			a = derefActor(args[1], "o6_kernelSetFunctions: 111");
-			a->_shadowMode = args[2] + args[3];
-			break;
-		case 112:									/* palette shift? */
-			setShadowPalette(args[3], args[4], args[5], args[1], args[2], args[6], args[7]);
-			break;
-		case 114:
-			// Sam & Max film noir mode
-			if (_game.id == GID_SAMNMAX) {
-				// At this point ScummVM will already have set
-				// variable 0x8000 to indicate that the game is
-				// in film noir mode. All we have to do here is
-				// to mark the palette as "dirty", because
-				// updatePalette() will desaturate the colors
-				// as they are uploaded to the backend.
-				//
-				// This actually works better than the original
-				// interpreter, where actors would sometimes
-				// still be drawn in color.
-				setDirtyColors(0, 255);
-			} else
-				error("stub o6_kernelSetFunctions_114()");
-			break;
-		case 117:
-			// Sam & Max uses this opcode in script-43, right
-			// before a screensaver is selected.
+	switch (args[0]) {
+	case 3:
+		// Dummy case
+		break;
+	case 4:
+		grabCursor(args[1], args[2], args[3], args[4]);
+		break;
+	case 5:
+		fadeOut(args[1]);
+		break;
+	case 6:
+		_fullRedraw = true;
+		redrawBGAreas();
+		setActorRedrawFlags();
+		processActors();
+		fadeIn(args[1]);
+		break;
+	case 8:
+		startManiac();
+		break;
+	case 9:
+		killAllScriptsExceptCurrent();
+		break;
+	case 104:									/* samnmax */
+		nukeFlObjects(args[2], args[3]);
+		break;
+	case 107:									/* set actor scale */
+		a = derefActor(args[1], "o6_kernelSetFunctions: 107");
+		a->setScale((unsigned char)args[2], -1);
+		break;
+	case 108:									/* create proc_special_palette */
+	case 109:
+		// Case 108 and 109 share the same function
+		if (num != 6)
+			error("o6_kernelSetFunctions sub op %d: expected 6 params but got %d", args[0], num);
+		setShadowPalette(args[3], args[4], args[5], args[1], args[2], 0, 256);
+		break;
+	case 110:
+		clearCharsetMask();
+		break;
+	case 111:
+		a = derefActor(args[1], "o6_kernelSetFunctions: 111");
+		a->_shadowMode = args[2] + args[3];
+		break;
+	case 112:									/* palette shift? */
+		setShadowPalette(args[3], args[4], args[5], args[1], args[2], args[6], args[7]);
+		break;
+	case 114:
+		// Sam & Max film noir mode
+		if (_game.id == GID_SAMNMAX) {
+			// At this point ScummVM will already have set
+			// variable 0x8000 to indicate that the game is
+			// in film noir mode. All we have to do here is
+			// to mark the palette as "dirty", because
+			// updatePalette() will desaturate the colors
+			// as they are uploaded to the backend.
 			//
-			// Sam & Max uses variable 132 to specify the number of
-			// minutes of inactivity (no mouse movements) before
-			// starting the screensaver, so setting it to 0 will
-			// help in debugging.
-			freezeScripts(0x80);
-			break;
-		case 119:
-			enqueueObject(args[1], args[2], args[3], args[4], args[5], args[6], args[7], args[8], 0);
-			break;
-		case 120:
-			swapPalColors(args[1], args[2]);
-			break;
-		case 122:
-			VAR(VAR_SOUNDRESULT) =
-				(short)_imuse->doCommand (num - 1, &args[1]);
-			break;
-		case 123:
-			copyPalColor(args[2], args[1]);
-			break;
-		case 124:
-			_saveSound = args[1];
-			break;
-		default:
-			error("o6_kernelSetFunctions: default case %d (param count %d)", args[0], num);
-			break;
-		}
+			// This actually works better than the original
+			// interpreter, where actors would sometimes
+			// still be drawn in color.
+			setDirtyColors(0, 255);
+		} else
+			error("stub o6_kernelSetFunctions_114()");
+		break;
+	case 117:
+		// Sam & Max uses this opcode in script-43, right
+		// before a screensaver is selected.
+		//
+		// Sam & Max uses variable 132 to specify the number of
+		// minutes of inactivity (no mouse movements) before
+		// starting the screensaver, so setting it to 0 will
+		// help in debugging.
+		freezeScripts(0x80);
+		break;
+	case 119:
+		enqueueObject(args[1], args[2], args[3], args[4], args[5], args[6], args[7], args[8], 0);
+		break;
+	case 120:
+		swapPalColors(args[1], args[2]);
+		break;
+	case 122:
+		VAR(VAR_SOUNDRESULT) =
+			(short)_imuse->doCommand (num - 1, &args[1]);
+		break;
+	case 123:
+		copyPalColor(args[2], args[1]);
+		break;
+	case 124:
+		_saveSound = args[1];
+		break;
+	default:
+		error("o6_kernelSetFunctions: default case %d (param count %d)", args[0], num);
+		break;
 	}
 }
 
@@ -2790,35 +2804,26 @@ int ScummEngine::getKeyState(int key) {
 	switch (key) {
 	case 0x147: // Home
 		return (_keyDownMap[0x107] || _keyDownMap[0x115]) ? 1 : 0;
-		break;
 	case 0x148: // Up
 		return (_keyDownMap[0x108] || _keyDownMap[0x111] ||
 				_keyDownMap[0x38]) ? 1 : 0;
-		break;
 	case 0x149: // PgUp
 		return (_keyDownMap[0x109] || _keyDownMap[0x118]) ? 1 : 0;
-		break;
 	case 0x14B: // Left
 		return (_keyDownMap[0x104] || _keyDownMap[0x114] ||
 				_keyDownMap[0x34]) ? 1 : 0;
-		break;
 	case 0x14D: // Right
 		return (_keyDownMap[0x106] || _keyDownMap[0x113] ||
 				_keyDownMap[0x36]) ? 1 : 0;
-		break;
 	case 0x14F: // End
 		return (_keyDownMap[0x101] || _keyDownMap[0x117]) ? 1 : 0;
-		break;
 	case 0x150: // Down
 		return (_keyDownMap[0x102] || _keyDownMap[0x112] ||
 				_keyDownMap[0x32]) ? 1 : 0;
-		break;
 	case 0x151: // PgDn
 		return (_keyDownMap[0x103] || _keyDownMap[0x119]) ? 1 : 0;
-		break;
 	default:
 		return (_keyDownMap[key]) ? 1 : 0;
-		break;
 	}
 }
 
@@ -2969,7 +2974,7 @@ void ScummEngine_v6::o6_pickVarRandom() {
 
 	num = readArray(value, 0, 0);
 
-	ArrayHeader *ah = (ArrayHeader *)getResourceAddress(rtString, readVar(value));
+	ArrayHeader *ah = getArray(value);
 	dim1 = FROM_LE_16(ah->dim1) - 1;
 
 	if (dim1 < num) {
@@ -3040,7 +3045,7 @@ void ScummEngine_v6::o6_setBoxSet() {
 		error("ScummEngine_v6::o6_setBoxSet: Can't find dboxes for set %d", arg);
 
 	dboxSize = READ_BE_UINT32(boxd + 4) - 8;
-	byte *matrix = res.createResource(rtMatrix, 2, dboxSize);
+	byte *matrix = _res->createResource(rtMatrix, 2, dboxSize);
 
 	assert(matrix);
 	memcpy(matrix, boxd + 8, dboxSize);
@@ -3053,7 +3058,7 @@ void ScummEngine_v6::o6_setBoxSet() {
 		error("ScummEngine_v6::o6_setBoxSet: Can't find mboxes for set %d", arg);
 
 	mboxSize = READ_BE_UINT32(boxm + 4) - 8;
-	matrix = res.createResource(rtMatrix, 1, mboxSize);
+	matrix = _res->createResource(rtMatrix, 1, mboxSize);
 
 	assert(matrix);
 	memcpy(matrix, boxm + 8, mboxSize);
