@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2002-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/scumm/script_v8.cpp $
- * $Id: script_v8.cpp 22854 2006-06-03 04:10:14Z kirben $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/scumm/script_v8.cpp $
+ * $Id: script_v8.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -393,19 +396,19 @@ int ScummEngine_v8::readVar(uint var) {
 	debugC(DEBUG_VARS, "readvar(%d)", var);
 
 	if (!(var & 0xF0000000)) {
-		checkRange(_numVariables - 1, 0, var, "Variable %d out of range(r)");
+		assertRange(0, var, _numVariables - 1, "variable");
 		return _scummVars[var];
 	}
 
 	if (var & 0x80000000) {
 		var &= 0x7FFFFFFF;
-		checkRange(_numBitVariables - 1, 0, var, "Bit variable %d out of range(r)");
+		assertRange(0, var, _numBitVariables - 1, "bit variable (reading)");
 		return (_bitVars[var >> 3] & (1 << (var & 7))) ? 1 : 0;
 	}
 
 	if (var & 0x40000000) {
 		var &= 0xFFFFFFF;
-		checkRange(25, 0, var, "Local variable %d out of range(r)");
+		assertRange(0, var, 25, "local variable (reading)");
 		return vm.localvar[_currentScript][var];
 	}
 
@@ -417,7 +420,7 @@ void ScummEngine_v8::writeVar(uint var, int value) {
 	debugC(DEBUG_VARS, "writeVar(%d, %d)", var, value);
 
 	if (!(var & 0xF0000000)) {
-		checkRange(_numVariables - 1, 0, var, "Variable %d out of range(w)");
+		assertRange(0, var, _numVariables - 1, "variable (writing)");
 
 		if (var == VAR_CHARINC) {
 			if (ConfMan.hasKey("talkspeed")) {
@@ -441,7 +444,7 @@ void ScummEngine_v8::writeVar(uint var, int value) {
 
 	if (var & 0x80000000) {
 		var &= 0x7FFFFFFF;
-		checkRange(_numBitVariables - 1, 0, var, "Bit variable %d out of range(w)");
+		assertRange(0, var, _numBitVariables - 1, "bit variable (writing)");
 
 		if (value)
 			_bitVars[var >> 3] |= (1 << (var & 7));
@@ -452,7 +455,7 @@ void ScummEngine_v8::writeVar(uint var, int value) {
 
 	if (var & 0x40000000) {
 		var &= 0xFFFFFFF;
-		checkRange(25, 0, var, "Local variable %d out of range(w)");
+		assertRange(0, var, 25, "local variable (writing)");
 		vm.localvar[_currentScript][var] = value;
 		return;
 	}
@@ -488,7 +491,7 @@ void ScummEngine_v8::decodeParseString(int m, int n) {
 		_string[m].charset = pop();
 		break;
 	case 0xCE:		// SO_PRINT_LEFT
-		_string[m].center = false;
+		_string[m].wrapping = false;
 		_string[m].overhead = false;
 		break;
 	case 0xCF:		// SO_PRINT_OVERHEAD
@@ -499,15 +502,12 @@ void ScummEngine_v8::decodeParseString(int m, int n) {
 		_string[m].no_talk_anim = true;
 		break;
 	case 0xD1:		// SO_PRINT_STRING
-		if (m == 5)
-			enqueueText(_scriptPointer, _string[m].xpos, _string[m].ypos, _string[m].color, _string[m].charset, _string[m].center);
-		else
-			printString(m, _scriptPointer);
+		printString(m, _scriptPointer);
 		_scriptPointer += resStrLen(_scriptPointer) + 1;
-
 		break;
 	case 0xD2:		// SO_PRINT_WRAP Set print wordwrap
-		//debug(0, "decodeParseString: SO_PRINT_WRAP");
+		_string[m].wrapping = true;
+		_string[m].overhead = false;
 		break;
 	default:
 		error("decodeParseString: default case 0x%x", b);
@@ -633,15 +633,15 @@ void ScummEngine_v8::o8_arrayOps() {
 	byte subOp = fetchScriptByte();
 	int array = fetchScriptWord();
 	int b, c, d, len;
-	ArrayHeader *ah;
+	byte *data;
 	int list[128];
 
 	switch (subOp) {
 	case 0x14:		// SO_ASSIGN_STRING
 		b = pop();
 		len = resStrLen(_scriptPointer);
-		ah = defineArray(array, kStringArray, 0, len + 1);
-		copyScriptString(ah->data + b);
+		data = defineArray(array, kStringArray, 0, len + 1);
+		copyScriptString(data + b);
 		break;
 	case 0x15:		// SO_ASSIGN_SCUMMVAR_LIST
 		b = pop();
@@ -671,14 +671,16 @@ void ScummEngine_v8::o8_arrayOps() {
 }
 
 void ScummEngine_v8::o8_blastText() {
-	// FIXME
-	decodeParseString(5, 0);
+	// Original V8 interpreter uses StringSlot 2 for o_blastText and 4 for o_printDebug.
+	// Since slot 2 is already mapped to printDebug for V6 (see ScummEngine::printString()),
+	// we just "swap" the slots, and use slot 4 here.
+	decodeParseString(4, 0);
 }
 
 void ScummEngine_v8::o8_cursorCommand() {
 	byte subOp = fetchScriptByte();
-	int a, i;
-	int args[16];
+	int a;
+	int args[4];
 
 	switch (subOp) {
 	case 0xDC:		// SO_CURSOR_ON Turn cursor on
@@ -725,12 +727,11 @@ void ScummEngine_v8::o8_cursorCommand() {
 		setCursorTransparency(pop());
 		break;
 	case 0xE7: 		// SO_CHARSET_SET
-		_verbCharset = pop();
+		_string[0]._default.charset = pop();
 		break;
 	case 0xE8:		// SO_CHARSET_COLOR
 		getStackList(args, ARRAYSIZE(args));
-		for (i = 0; i < 16; i++)
-			_charsetColorMap[i] = _charsetData[_string[1]._default.charset][i] = (unsigned char)args[i];
+		// This opcode does nothing (confirmed with disasm)
 		break;
 	case 0xE9: 		// SO_CURSOR_PUT
 		{
@@ -775,40 +776,40 @@ void ScummEngine_v8::o8_resourceRoutines() {
 		break;
 
 	case 0x42:		// SO_HEAP_LOCK_COSTUME Lock costume in heap
-		res.lock(rtCostume, resid);
+		_res->lock(rtCostume, resid);
 		break;
 	case 0x43:		// SO_HEAP_LOCK_ROOM Lock room in heap
-		res.lock(rtRoom, resid);
+		_res->lock(rtRoom, resid);
 		break;
 	case 0x44:		// SO_HEAP_LOCK_SCRIPT Lock script in heap
-		res.lock(rtScript, resid);
+		_res->lock(rtScript, resid);
 		break;
 	case 0x45:		// SO_HEAP_LOCK_SOUND Lock sound in heap
-		res.lock(rtSound, resid);
+		_res->lock(rtSound, resid);
 		break;
 	case 0x46:		// SO_HEAP_UNLOCK_COSTUME Unlock costume
-		res.unlock(rtCostume, resid);
+		_res->unlock(rtCostume, resid);
 		break;
 	case 0x47:		// SO_HEAP_UNLOCK_ROOM Unlock room
-		res.unlock(rtRoom, resid);
+		_res->unlock(rtRoom, resid);
 		break;
 	case 0x48:		// SO_HEAP_UNLOCK_SCRIPT Unlock script
-		res.unlock(rtScript, resid);
+		_res->unlock(rtScript, resid);
 		break;
 	case 0x49:		// SO_HEAP_UNLOCK_SOUND Unlock sound
-		res.unlock(rtSound, resid);
+		_res->unlock(rtSound, resid);
 		break;
 	case 0x4A:		// SO_HEAP_NUKE_COSTUME Remove costume from heap
-		res.setResourceCounter(rtCostume, resid, 0x7F);
+		_res->setResourceCounter(rtCostume, resid, 0x7F);
 		break;
 	case 0x4B:		// SO_HEAP_NUKE_ROOM Remove room from heap
-		res.setResourceCounter(rtRoom, resid, 0x7F);
+		_res->setResourceCounter(rtRoom, resid, 0x7F);
 		break;
 	case 0x4C:		// SO_HEAP_NUKE_SCRIPT Remove script from heap
-		res.setResourceCounter(rtScript, resid, 0x7F);
+		_res->setResourceCounter(rtScript, resid, 0x7F);
 		break;
 	case 0x4D:		// SO_HEAP_NUKE_SOUND Remove sound from heap
-		res.setResourceCounter(rtSound, resid, 0x7F);
+		_res->setResourceCounter(rtSound, resid, 0x7F);
 		break;
 	default:
 		error("o8_resourceRoutines: default case 0x%x", subOp);
@@ -853,17 +854,21 @@ void ScummEngine_v8::o8_roomOps() {
 		break;
 	case 0x5C:		// SO_ROOM_NEW_PALETTE New palette
 		a = pop();
-		setPalette(a);
+		setCurrentPalette(a);
 		break;
 	case 0x5D:		// SO_ROOM_SAVE_GAME Save game
+		_saveSound = 0;
 		_saveTemporaryState = true;
 		_saveLoadSlot = 1;
 		_saveLoadFlag = 1;
 		break;
 	case 0x5E:		// SO_ROOM_LOAD_GAME Load game
-		_saveTemporaryState = true;
-		_saveLoadSlot = 1;
-		_saveLoadFlag = 2;
+		_saveSound = pop();
+		if (!_saveLoadFlag) {
+			_saveTemporaryState = true;
+			_saveLoadSlot = 1;
+			_saveLoadFlag = 2;
+		}
 		break;
 	case 0x5F:		// SO_ROOM_SATURATION Set saturation of room colors
 		e = pop();
@@ -934,7 +939,7 @@ void ScummEngine_v8::o8_actorOps() {
 	case 0x6F:		// SO_ACTOR_PALETTE Set actor palette
 		j = pop();
 		i = pop();
-		checkRange(31, 0, i, "Illegal palette slot %d");
+		assertRange(0, i, 31, "o8_actorOps: palette slot");
 		a->setPalette(i, j);
 		break;
 	case 0x70:		// SO_ACTOR_TALK_COLOR Set actor talk color
@@ -963,13 +968,13 @@ void ScummEngine_v8::o8_actorOps() {
 		a->_ignoreBoxes = true;
 		a->_forceClip = 100;
 		if (a->isInCurrentRoom())
-			a->putActor(a->_pos.x, a->_pos.y, a->_room);
+			a->putActor();
 		break;
 	case 0x77:		// SO_ACTOR_FOLLOW_BOXES Make actor follow boxes
 		a->_ignoreBoxes = false;
 		a->_forceClip = 100;
 		if (a->isInCurrentRoom())
-			a->putActor(a->_pos.x, a->_pos.y, a->_room);
+			a->putActor();
 		break;
 	case 0x78:		// SO_ACTOR_SPECIAL_DRAW
 		a->_shadowMode = pop();
@@ -1055,7 +1060,7 @@ void ScummEngine_v8::o8_verbOps() {
 	if (subOp == 0x96) {
 		_curVerb = pop();
 		_curVerbSlot = getVerbSlot(_curVerb, 0);
-		checkRange(_numVerbs - 1, 0, _curVerbSlot, "Illegal new verb slot %d");
+		assertRange(0, _curVerbSlot, _numVerbs - 1, "new verb slot");
 		//printf("Setting current actor to %d\n", _curActor);
 		return;
 	}
@@ -1085,7 +1090,7 @@ void ScummEngine_v8::o8_verbOps() {
 		vs->hicolor = 0;
 		vs->dimcolor = 8;
 		vs->type = kTextVerbType;
-		vs->charset_nr = _verbCharset;
+		vs->charset_nr = _string[0]._default.charset;
 		vs->curmode = 0;
 		vs->saveid = 0;
 		vs->key = 0;
@@ -1176,9 +1181,7 @@ void ScummEngine_v8::o8_systemOps() {
 void ScummEngine_v8::o8_startVideo() {
 	int len = resStrLen(_scriptPointer);
 
-	SmushPlayer *sp = new SmushPlayer(this, 12);
-	sp->play((const char*)_scriptPointer);
-	delete sp;
+	_splayer->play((const char*)_scriptPointer, 12);
 
 	_scriptPointer += len + 1;
 }
@@ -1193,13 +1196,13 @@ void ScummEngine_v8::o8_kernelSetFunctions() {
 	case 11: {	// lockObject
 		int objidx = getObjectIndex(args[1]);
 		assert(objidx != -1);
-		res.lock(rtFlObject, _objs[objidx].fl_object_index);
+		_res->lock(rtFlObject, _objs[objidx].fl_object_index);
 		break;
 	}
 	case 12: {	// unlockObject
 		int objidx = getObjectIndex(args[1]);
 		assert(objidx != -1);
-		res.unlock(rtFlObject, _objs[objidx].fl_object_index);
+		_res->unlock(rtFlObject, _objs[objidx].fl_object_index);
 		break;
 	}
 	case 13:	// remapCostume
@@ -1232,32 +1235,31 @@ void ScummEngine_v8::o8_kernelSetFunctions() {
 		removeBlastTexts();
 		break;
 	case 25: {	// saveGameReadName
-		char *address = (char*)getStringAddress(args[2]);
 		char name[30];
+		if (getSavegameName(args[1], name)) {
+			int size = resStrLen((const byte *)name) + 1;
+			_res->nukeResource(rtString, args[2]);
 
-		if (!address) {
-			error("o8_kernelSetFunctions: saveGameReadName failed finding slot string %d", args[2]);
-			break;
+			ArrayHeader *ah = (ArrayHeader *)_res->createResource(rtString, args[2], size + sizeof(ArrayHeader));
+			ah->type = TO_LE_16(kStringArray);
+			ah->dim1 = TO_LE_16(size + 1);
+			ah->dim2 = TO_LE_16(1);
+
+			memcpy(getStringAddress(args[2]), name, size);
 		}
-		getSavegameName(args[1] - 1, name);
-		if (strlen(name) > 0 && strlen(name) < 30)
-			strcpy(address, name);
 		break;
 	}
-	case 26: {	// saveGame?
-		//char *address = (char*)getStringAddress(args[2]);
-		char address[30];
-		debug(0, "o8_kernelSetFunctions: saveGame?(%d, %s)", args[1], address);
+	case 26: { // saveGameWrite
+		// FIXME: This doesn't work
+		char *address = (char*)getStringAddress(args[2]);
+		debug(0, "o8_kernelSetFunctions: saveGame(%d, %s)", args[1], address);
 		break;
 	}
-	case 27: {	// FIXME: This doesn't work
-			// saveGameRead
+	case 27: // saveGameRead
 		_saveLoadSlot = args[1];
 		_saveLoadFlag = 2;
 		_saveTemporaryState = false;
-		debug(0, "Sgl: %d", args[1]);
 		break;
-	}
 	case 28:	// saveGameStampScreenshot
 		debug(0, "o8_kernelSetFunctions: saveGameStampScreenshot(%d, %d, %d, %d, %d, %d)", args[1], args[2], args[3], args[4], args[5], args[6]);
 		break;
@@ -1275,16 +1277,16 @@ void ScummEngine_v8::o8_kernelSetFunctions() {
 		{
 		int idx = args[1];
 		int value = args[2];
-		ArrayHeader *ah = (ArrayHeader *)getResourceAddress(rtString, idx);
+		const char *str = (const char *)getStringAddress(idx);
 
-		debugC(DEBUG_GENERAL,"o8_kernelSetFunctions: writeRegistryValue(%s, %d)", (char *)ah->data, value);
+		debugC(DEBUG_GENERAL,"o8_kernelSetFunctions: writeRegistryValue(%s, %d)", str, value);
 		}
 		break;
 	case 33:	// paletteSetIntensity
 		debug(0, "o8_kernelSetFunctions: paletteSetIntensity(%d, %d)", args[1], args[2]);
 		break;
 	case 34:	// queryQuit
-		if (_confirmExit)
+		if (ConfMan.getBool("confirm_exit"))
 			confirmExitDialog();
 		else
 			_quit = true;
@@ -1373,22 +1375,22 @@ void ScummEngine_v8::o8_kernelGetFunctions() {
 	case 0xE0:		// readRegistryValue
 		{
 		int idx = args[1];
-		ArrayHeader *ah = (ArrayHeader *)getResourceAddress(rtString, idx);
-		if (!strcmp((char *)ah->data, "SFX Volume"))
+		const char *str = (const char *)getStringAddress(idx);
+		if (!strcmp(str, "SFX Volume"))
 			push(ConfMan.getInt("sfx_volume") / 2);
-		else if (!strcmp((char *)ah->data, "Voice Volume"))
+		else if (!strcmp(str, "Voice Volume"))
 			push(ConfMan.getInt("speech_volume") / 2);
-		else if (!strcmp((char *)ah->data, "Music Volume"))
+		else if (!strcmp(str, "Music Volume"))
 			push(ConfMan.getInt("music_volume") / 2);
-		else if (!strcmp((char *)ah->data, "Text Status"))
+		else if (!strcmp(str, "Text Status"))
 			push(ConfMan.getBool("subtitles"));
-		else if (!strcmp((char *)ah->data, "Object Names"))
+		else if (!strcmp(str, "Object Names"))
 			push(ConfMan.getBool("object_labels"));
-		else if (!strcmp((char *)ah->data, "Saveload Page"))
+		else if (!strcmp(str, "Saveload Page"))
 			push(14);
 		else 		// Use defaults
 			push(-1);
-		debugC(DEBUG_GENERAL,"o8_kernelGetFunctions: readRegistryValue(%s)", (char *)ah->data);
+		debugC(DEBUG_GENERAL,"o8_kernelGetFunctions: readRegistryValue(%s)", str);
 		}
 		break;
 	case 0xE1:		// imGetMusicPosition
@@ -1419,8 +1421,8 @@ void ScummEngine_v8::o8_getActorZPlane() {
 	int z = a->_forceClip;
 	if (z == 100) {
 		z = getMaskFromBox(a->_walkbox);
-		if (z > gdi._numZBuffer - 1)
-			z = gdi._numZBuffer - 1;
+		if (z > _gdi->_numZBuffer - 1)
+			z = _gdi->_numZBuffer - 1;
 	}
 
 	push(z);
