@@ -1,6 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2001  Ludvig Strigeus
- * Copyright (C) 2001-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -16,15 +18,15 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/scumm/scumm.h $
- * $Id: scumm.h 23819 2006-09-02 05:51:19Z kirben $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/scumm/scumm.h $
+ * $Id: scumm.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
 #ifndef SCUMM_H
 #define SCUMM_H
 
-#include "base/engine.h"
+#include "engines/engine.h"
 #include "common/endian.h"
 #include "common/file.h"
 #include "common/rect.h"
@@ -32,7 +34,7 @@
 #include "graphics/surface.h"
 
 #include "scumm/gfx.h"
-#include "scumm/plugin.h"
+#include "scumm/detection.h"
 #include "scumm/script.h"
 
 #include "sound/mididrv.h"
@@ -55,8 +57,6 @@ class BaseScummFile;
 class CharsetRenderer;
 class IMuse;
 class IMuseDigital;
-class Insane;
-class SmushMixer;
 class MusicEngine;
 class ScummEngine;
 class ScummDebugger;
@@ -82,15 +82,9 @@ enum {
  * Note that some of them could be replaced by checks for the SCUMM version.
  */
 enum GameFeatures {
-	/** Games with the new camera system (ScummEngine_v7 and subclasses). */
-	GF_NEW_CAMERA          = 1 << 1,
-
 	/** Games with the AKOS costume system (ScummEngine_v7 and subclasses, HE games). */
 	GF_NEW_COSTUMES        = 1 << 2,
 
-	/** Games with digital iMUSE (ScummEngine_v7 and subclasses). */
-	GF_DIGI_IMUSE          = 1 << 3,
-
 	/** Games using XOR encrypted data files. */
 	GF_USE_KEY             = 1 << 4,
 
@@ -109,9 +103,6 @@ enum GameFeatures {
 	/** Games which have Audio CD tracks. */
 	GF_AUDIOTRACKS         = 1 << 9,
 
-	/** Games without actor scaling (ScummEngine_v3 and subclasses). */
-	GF_NO_SCALING          = 1 << 10,
-
 	/**
 	 * Games using only very few local variables in scripts.
 	 * Apparently that is only the case for 256 color version of Indy3.
@@ -125,7 +116,7 @@ enum GameFeatures {
 	 *  HE Games with more global scripts and different sprite handling
 	 *  i.e. read it as HE version 9.85. Used for HE98 only.
 	 */
-	GF_HE_985		= 1 << 14,
+	GF_HE_985             = 1 << 14,
 
 	/** HE games with 16 bit color */
 	GF_16BIT_COLOR         = 1 << 15,
@@ -140,11 +131,6 @@ enum GameFeatures {
 /* SCUMM Debug Channels */
 void CDECL debugC(int level, const char *s, ...);
 
-struct dbgChannelDesc {
-	const char *channel, *desc;
-	uint32 flag;
-};
-
 enum {
 	DEBUG_GENERAL	=	1 << 0,		// General debug
 	DEBUG_SCRIPTS	=	1 << 2,		// Track script execution (start/stop/pause)
@@ -237,7 +223,8 @@ enum ScummGameId {
 	GID_PUTTRACE,
 	GID_FUNSHOP,	// Used for all three funshops
 	GID_FOOTBALL,
-	GID_SOCCER
+	GID_SOCCER,
+	GID_HECUP		// CUP demos
 };
 
 struct SentenceTab {
@@ -258,6 +245,7 @@ struct StringSlot {
 	bool center;
 	bool overhead;
 	bool no_talk_anim;
+	bool wrapping;
 };
 
 struct StringTab : StringSlot {
@@ -343,15 +331,17 @@ enum ResTypes {
 	rtNumTypes = 22
 };
 
-
+enum {
+	RES_INVALID_OFFSET = 0xFFFFFFFF
+};
 
 /**
  * The 'resource manager' class. Currently doesn't really deserve to be called
  * a 'class', at least until somebody gets around to OOfying this more.
  */
 class ResourceManager {
-	friend class ScummDebugger;
-	friend class ScummEngine;
+	//friend class ScummDebugger;
+	//friend class ScummEngine;
 protected:
 	ScummEngine *_vm;
 
@@ -369,18 +359,23 @@ public:
 	uint32 *roomoffs[rtNumTypes];
 	uint32 *globsize[rtNumTypes];
 
+protected:
 	uint32 _allocatedSize;
 	uint32 _maxHeapThreshold, _minHeapThreshold;
 	byte _expireCounter;
 
 public:
 	ResourceManager(ScummEngine *vm);
+	~ResourceManager();
+
+	void setHeapThreshold(int min, int max);
+
+	void allocResTypeData(int id, uint32 tag, int num, const char *name, int mode);
+	void freeResources();
 
 	byte *createResource(int type, int index, uint32 size);
 	void nukeResource(int type, int i);
 
-	void freeResources();
-
 	bool isResourceLoaded(int type, int index) const;
 
 	void lock(int type, int i);
@@ -390,14 +385,16 @@ public:
 	void setModified(int type, int i);
 	bool isModified(int type, int i) const;
 
+	void increaseExpireCounter();
 	void setResourceCounter(int type, int index, byte flag);
 	void increaseResourceCounter();
 
 	void resourceStats();
-	void expireResources(uint32 size);
-
-protected:
+	
+//protected:
 	bool validateResource(const char *str, int type, int index) const;
+protected:
+	void expireResources(uint32 size);
 };
 
 /**
@@ -405,12 +402,12 @@ protected:
  */
 class ScummEngine : public Engine {
 	friend class ScummDebugger;
-	friend class SmushPlayer;
-	friend class Insane;
 	friend class CharsetRenderer;
 	friend class ResourceManager;
 
+	GUI::Debugger *getDebugger();
 	void errorString(const char *buf_input, char *buf_output);
+
 public:
 	/* Put often used variables at the top.
 	 * That results in a shorter form of the opcode
@@ -419,11 +416,9 @@ public:
 	IMuseDigital *_imuseDigital;
 	MusicEngine *_musicEngine;
 	Sound *_sound;
-	SmushMixer *_smixer;
 
 	VerbSlot *_verbs;
 	ObjectData *_objs;
-	ObjectData *_storedFlObjects;
 	ScummDebugger *_debugger;
 
 	// Core variables
@@ -434,13 +429,10 @@ public:
 	Common::RandomSource _rnd;
 
 	/** Graphics manager */
-	Gdi gdi;
+	Gdi *_gdi;
 
 	/** Central resource data. */
-	ResourceManager res;
-
-	// Misc utility functions
-	uint32 _debugFlags;	// Used by the debugger
+	ResourceManager *_res;
 
 protected:
 	VirtualMachineState vm;
@@ -473,43 +465,31 @@ protected:
 	void setTalkspeed(int talkspeed);
 	int getTalkspeed();
 
-protected:
 	// Scumm main loop & helper functions.
-	int scummLoop(int delta);
+	virtual int scummLoop(int delta);
 	virtual void scummLoop_updateScummVars();
 	virtual void scummLoop_handleSaveLoad();
 	virtual void scummLoop_handleDrawing();
 	virtual void scummLoop_handleActors() = 0;
 	virtual void scummLoop_handleEffects();
 	virtual void scummLoop_handleSound();
+	
+	virtual void runBootscript();
 
 	// Event handling
 public:
 	void parseEvents();	// Used by IMuseDigital::startSound
 protected:
 	void waitForTimer(int msec_delay);
-	void processKbd(bool smushMode);
-	void clearClickedStatus();
+	virtual void processInput();
+	virtual void processKeyboard(int lastKeyHit);
+	virtual void clearClickedStatus();
 
 	// Cursor/palette
 	void updateCursor();
 	virtual void animateCursor() {}
 	virtual void updatePalette();
 
-	/**
-	 * Flag which signals that the SMUSH video playback should end now
-	 * (e.g. because it was aborted by the user or it's simply finished).
-	 */
-	bool _smushVideoShouldFinish;
-	/** This flag is a hack to allow the pause dialog to pause SMUSH playback, too. */
-	bool _smushPaused;
-public:
-	/** This flag tells IMuseDigital that INSANE is running. */
-	bool _insaneRunning;	// Used by IMuseDigital::flushTracks()
-
-protected:
-	Insane *_insane;
-
 public:
 	void pauseGame();
 	void restart();
@@ -523,7 +503,7 @@ protected:
 	Dialog *_versionDialog;
 	Dialog *_mainMenuDialog;
 
-	int runDialog(Dialog &dialog);
+	virtual int runDialog(Dialog &dialog);
 	void confirmExitDialog();
 	void confirmRestartDialog();
 	void pauseDialog();
@@ -535,7 +515,7 @@ protected:
 	byte _fastMode;
 
 	byte _numActors;
-	Actor *_actors;	// Has _numActors elements
+	Actor **_actors;	// Has _numActors elements
 	Actor **_sortedActors;
 
 	byte *_arraySlot;
@@ -573,7 +553,6 @@ protected:
 	int _numNewNames, _numGlobalScripts;
 	int _numRoomVariables;
 	int _numPalettes, _numSprites, _numTalkies, _numUnk;
-	int _numStoredFlObjects;
 	int _HEHeapSize;
 public:
 	int _numLocalScripts, _numImages, _numRooms, _numScripts, _numSounds;	// Used by HE games
@@ -593,10 +572,6 @@ public:
 	int _NESStartStrip;
 
 protected:
-	/* Current objects - can go in their respective classes */
-	byte _curActor;
-	int _curVerb;
-	int _curVerbSlot;
 	int _curPalIndex;
 
 public:
@@ -610,7 +585,6 @@ public:
 
 protected:
 	int _keyPressed;
-	uint16 _lastKeyHit;
 	bool _keyDownMap[512]; // FIXME - 512 is a guess. it's max(kbd.ascii)
 
 	Common::Point _mouse;
@@ -673,7 +647,6 @@ protected:
 	const byte * const *_lastCodePtr;
 	int _resultVarNumber, _scummStackPos;
 	int _vmStack[150];
-	int _keyScriptKey, _keyScriptNo;
 
 	virtual void setupOpcodes() = 0;
 	virtual void executeOpcode(byte i) = 0;
@@ -769,8 +742,8 @@ protected:
 	bool openResourceFile(const Common::String &filename, byte encByte);	// TODO: Use Common::String
 
 	void loadPtrToResource(int type, int i, const byte *ptr);
-	virtual void readResTypeList(int id, const char *name);
-	void allocResTypeData(int id, uint32 tag, int num, const char *name, int mode);
+	virtual void readResTypeList(int id);
+//	void allocResTypeData(int id, uint32 tag, int num, const char *name, int mode);
 //	byte *createResource(int type, int index, uint32 size);
 	int loadResource(int type, int i);
 //	void nukeResource(int type, int i);
@@ -785,19 +758,13 @@ public:
 
 protected:
 	int readSoundResource(int type, int index);
-	int convert_extraflags(byte *ptr, byte * src_ptr);
-	void convertMac0Resource(int type, int index, byte *ptr, int size);
-	void convertADResource(int type, int index, byte *ptr, int size);
 	int readSoundResourceSmallHeader(int type, int index);
 	bool isResourceInUse(int type, int i) const;
 
 	virtual void setupRoomSubBlocks();
 	virtual void resetRoomSubBlocks();
 
-	void storeFlObject(int slot);
-	void restoreFlObjects();
-
-	void clearRoomObjects();
+	virtual void clearRoomObjects();
 	virtual void resetRoomObjects();
 	virtual void resetRoomObject(ObjectData *od, const byte *room, const byte *searchptr = NULL);
 
@@ -832,7 +799,7 @@ public:
 
 protected:
 	void markObjectRectAsDirty(int obj);
-	void loadFlObject(uint object, uint room);
+	virtual void loadFlObject(uint object, uint room);
 	void nukeFlObjects(int min, int max);
 	int findFlObjectSlot();
 	int findLocalObjectSlot();
@@ -881,8 +848,6 @@ protected:
 	const byte *getObjectImage(const byte *ptr, int state);
 	virtual int getObjectIdFromOBIM(const byte *obim);
 
-	int getDistanceBetween(bool is_obj_1, int b, int c, bool is_obj_2, int e, int f);
-
 protected:
 	/* Should be in Verb class */
 	uint16 _verbMouseOver;
@@ -890,10 +855,6 @@ protected:
 	int8 _userPut;
 	uint16 _userState;
 
-	int _activeInventory;
-	int _activeObject;
-	int _activeVerb;
-
 	virtual void handleMouseOver(bool updateInventory);
 	virtual void redrawVerbs();
 	virtual void checkExecVerbs();
@@ -915,8 +876,6 @@ public:
 	Actor *derefActor(int id, const char *errmsg = 0) const;
 	Actor *derefActorSafe(int id, const char *errmsg) const;
 
-	int getAngleFromPos(int x, int y) const;
-
 protected:
 	void walkActors();
 	void playActorSounds();
@@ -976,6 +935,7 @@ protected:
 	//ender: fullscreen
 	bool _fullRedraw, _bgNeedsRedraw;
 	bool _screenEffectFlag, _completeScreenRedraw;
+	bool _disableFadeInEffect;
 
 	struct {
 		int hotspotX, hotspotY, width, height;
@@ -1009,7 +969,7 @@ protected:
 
 	void moveScreen(int dx, int dy, int height);
 
-	void restoreBG(Common::Rect rect, byte backcolor = 0);
+	void restoreBackground(Common::Rect rect, byte backcolor = 0);
 	void redrawBGStrip(int start, int num);
 	virtual void redrawBGAreas();
 
@@ -1034,7 +994,7 @@ protected:
 
 	void resetPalette();
 
-	void setPalette(int pal);
+	void setCurrentPalette(int pal);
 	void setRoomPalette(int pal, int room);
 	virtual void setPaletteFromPtr(const byte *ptr, int numcolor = -1);
 	virtual void setPalColor(int index, int r, int g, int b);
@@ -1085,11 +1045,6 @@ protected:
 	void dissolveEffect(int width, int height);
 	void scrollEffect(int dir);
 
-	// bomp
-public:
-	byte *_bompActorPalettePtr;
-	void drawBomp(const BompDrawData &bd, bool mirror);
-
 protected:
 	bool _shakeEnabled;
 	uint _shakeFrame;
@@ -1107,7 +1062,7 @@ protected:
 	 * needs to be redrawn.
 	 *
 	 * The second leftmost bit is set by removeBlastObject() and
-	 * restoreBG(), but I'm not yet sure why.
+	 * restoreBackground(), but I'm not yet sure why.
 	 */
 	uint32 gfxUsageBits[410 * 3];
 
@@ -1149,25 +1104,21 @@ protected:
 	bool _enable_gs;
 	MidiDriverFlags _musicType;
 	bool _copyProtection;
-	bool _confirmExit;
 
 public:
 	uint16 _extraBoxFlags[65];
 
 	byte getNumBoxes();
 	byte *getBoxMatrixBaseAddr();
-	int getPathToDestBox(byte from, byte to);
-	void getGates(int trap1, int trap2, Common::Point gateA[2], Common::Point gateB[2]);
-	bool inBoxQuickReject(int box, int x, int y, int threshold);
-	int getClosestPtOnBox(int box, int x, int y, int16& outX, int16& outY);
-	int getSpecialBox(int param1, int param2);
+	int getNextBox(byte from, byte to);
 
 	void setBoxFlags(int box, int val);
 	void setBoxScale(int box, int b);
 
 	bool checkXYInBoxBounds(int box, int x, int y);
-	uint distanceFromPt(int x, int y, int ptx, int pty);
-	void getBoxCoordinates(int boxnum, BoxCoords *bc);
+
+	BoxCoords getBoxCoordinates(int boxnum);
+
 	byte getMaskFromBox(int box);
 	Box *getBoxBaseAddr(int box);
 	byte getBoxFlags(int box);
@@ -1194,6 +1145,13 @@ protected:
 public:
 	CharsetRenderer *_charset;
 	byte _charsetColorMap[16];
+
+	/**
+	 * All text is normally rendered into this overlay surface. Then later
+	 * drawStripToScreen() composits it over the game graphics.
+	 */
+	Graphics::Surface _textSurface;
+
 protected:
 	byte _charsetColor;
 	byte _charsetData[23][16];
@@ -1203,12 +1161,18 @@ protected:
 
 	bool _keepText;
 
+	int _nextLeft, _nextTop;
+
+	void restoreCharsetBg();
+	void clearCharsetMask();
+	void clearTextSurface();
+
 	virtual void initCharset(int charset);
 
-	void printString(int m, const byte *msg);
+	virtual void printString(int m, const byte *msg);
 
 	virtual bool handleNextCharsetCode(Actor *a, int *c);
-	void CHARSET_1();
+	virtual void CHARSET_1();
 	void drawString(int a, const byte *msg);
 	void debugMessage(const byte *msg);
 	void showMessageDialog(const byte *msg);
@@ -1318,7 +1282,7 @@ public:
 	byte VAR_SUBTITLES;
 	byte VAR_V6_EMSSPACE;
 
-	// V7/V8 (=GF_NEW_CAMERA) specific variables
+	// V7/V8 specific variables
 	byte VAR_CAMERA_POS_Y;
 	byte VAR_CAMERA_MIN_Y;
 	byte VAR_CAMERA_MAX_Y;
