@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2002-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,40 +18,29 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/scumm/smush/chunk.h $
- * $Id: chunk.h 22506 2006-05-18 00:20:53Z sev $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/scumm/smush/chunk.h $
+ * $Id: chunk.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
-#ifndef CHUNK_H
-#define CHUNK_H
+#ifndef SCUMM_SMUSH_CHUNK_H
+#define SCUMM_SMUSH_CHUNK_H
 
 #include "common/scummsys.h"
 #include "common/str.h"
+#include "common/stream.h"
 
 namespace Scumm {
 
 class BaseScummFile;
 
-class Chunk {
+class Chunk : public Common::SeekableReadStream {
 public:
-	virtual ~Chunk() {};
-	enum seek_type { seek_start, seek_end, seek_cur };
 	typedef uint32 type;
-	static const char *ChunkString(type t);
+
 	virtual type getType() const = 0;
-	virtual uint32 getSize() const = 0;
 	virtual Chunk *subBlock() = 0;
 	virtual void reseek() = 0;
-	virtual bool eof() const = 0;
-	virtual uint32 tell() const = 0;
-	virtual bool seek(int32 delta, seek_type dir = seek_cur) = 0;
-	virtual bool read(void *buffer, uint32 size) = 0;
-	virtual int8 getChar() = 0;
-	virtual byte getByte() = 0;
-	virtual int16 getShort() = 0;
-	virtual uint16 getWord() = 0;
-	virtual uint32 getDword() = 0;
 };
 
 // Common functionality for concrete chunks (FileChunk, MemoryChunk)
@@ -63,10 +55,10 @@ protected:
 
 public:
 	Chunk::type getType() const;
-	uint32 getSize() const;
-	bool eof() const;
-	uint32 tell() const;
-	bool seek(int32 delta, seek_type dir = seek_cur);
+	uint32 size() const;
+	bool eos() const;
+	uint32 pos() const;
+	void seek(int32 delta, int dir);
 };
 
 class FileChunk : public BaseChunk {
@@ -81,12 +73,7 @@ public:
 	virtual ~FileChunk();
 	Chunk *subBlock();
 	void reseek();
-	bool read(void *buffer, uint32 size);
-	int8 getChar();
-	byte getByte();
-	short getShort();
-	uint16 getWord();
-	uint32 getDword();
+	uint32 read(void *buffer, uint32 size);
 };
 
 class MemoryChunk : public BaseChunk {
@@ -97,12 +84,7 @@ public:
 	MemoryChunk(byte *data);
 	Chunk *subBlock();
 	void reseek();
-	bool read(void *buffer, uint32 size);
-	int8 getChar();
-	byte getByte();
-	int16 getShort();
-	uint16 getWord();
-	uint32 getDword();
+	uint32 read(void *buffer, uint32 size);
 };
 
 } // End of namespace Scumm
