@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2002-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/scumm/smush/codec37.cpp $
- * $Id: codec37.cpp 21500 2006-03-29 15:59:37Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/scumm/smush/codec37.cpp $
+ * $Id: codec37.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -28,8 +31,7 @@
 
 namespace Scumm {
 
-void Codec37Decoder::init(int width, int height) {
-	deinit();
+Codec37Decoder::Codec37Decoder(int width, int height) {
 	_width = width;
 	_height = height;
 	_frameSize = _width * _height;
@@ -40,26 +42,15 @@ void Codec37Decoder::init(int width, int height) {
 	_deltaBufs[0] = _deltaBuf + 0x4D80;
 	_deltaBufs[1] = _deltaBuf + 0xE880 + _frameSize;
 	_offsetTable = new int16[255];
-	_curtable = 0;
 	if (_offsetTable == 0)
 		error("unable to allocate decoder offset table");
-	_tableLastPitch = -1;
-	_tableLastIndex = -1;
-}
-
-Codec37Decoder::Codec37Decoder() {
-	_deltaSize = 0;
-	_deltaBuf = 0;
-	_deltaBufs[0] = 0;
-	_deltaBufs[1] = 0;
 	_curtable = 0;
-	_offsetTable = 0;
+	_prevSeqNb = 0;
 	_tableLastPitch = -1;
 	_tableLastIndex = -1;
-	_prevSeqNb = 0;
 }
 
-void Codec37Decoder::deinit() {
+Codec37Decoder::~Codec37Decoder() {
 	if (_offsetTable) {
 		delete []_offsetTable;
 		_offsetTable = 0;
@@ -75,10 +66,6 @@ void Codec37Decoder::deinit() {
 	}
 }
 
-Codec37Decoder::~Codec37Decoder() {
-	deinit();
-}
-
 void Codec37Decoder::maketable(int pitch, int index) {
 	static const int8 maketable_bytes[] = {
     0,   0,   1,   0,   2,   0,   3,   0,   5,   0,
@@ -519,7 +506,7 @@ void Codec37Decoder::decode(byte *dst, const byte *src) {
 	maketable(pitch, src[1]);
 	int32 tmp;
 
-	switch(src[0]) {
+	switch (src[0]) {
 	case 0:
 		if ((_deltaBufs[_curtable] - _deltaBuf) > 0) {
 			memset(_deltaBuf, 0, _deltaBufs[_curtable] - _deltaBuf);
