@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2002-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,13 +18,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/scumm/smush/codec37.h $
- * $Id: codec37.h 20511 2006-02-11 09:55:41Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/scumm/smush/codec37.h $
+ * $Id: codec37.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
-#ifndef SMUSH_CODEC37_H
-#define SMUSH_CODEC37_H
+#ifndef SCUMM_SMUSH_CODEC37_H
+#define SCUMM_SMUSH_CODEC37_H
 
 #include "common/scummsys.h"
 
@@ -42,10 +45,8 @@ private:
 	int _width, _height;
 
 public:
-	Codec37Decoder();
+	Codec37Decoder(int width, int height);
 	~Codec37Decoder();
-	void init(int width, int height);
-	void deinit();
 protected:
 	void maketable(int, int);
 	void proc1(byte *dst, const byte *src, int32, int, int, int, int16 *);
