@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2002-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/scumm/smush/codec47.cpp $
- * $Id: codec47.cpp 21500 2006-03-29 15:59:37Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/scumm/smush/codec47.cpp $
+ * $Id: codec47.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -69,19 +72,19 @@ namespace Scumm {
 		(dst)[1] = val;	\
 	} while (0)
 
-static int8 codec47_table_small1[] = {
+static const  int8 codec47_table_small1[] = {
   0, 1, 2, 3, 3, 3, 3, 2, 1, 0, 0, 0, 1, 2, 2, 1,
 };
 
-static int8 codec47_table_small2[] = {
+static const int8 codec47_table_small2[] = {
   0, 0, 0, 0, 1, 2, 3, 3, 3, 3, 2, 1, 1, 1, 2, 2,
 };
 
-static int8 codec47_table_big1[] = {
+static const int8 codec47_table_big1[] = {
   0, 2, 5, 7, 7, 7, 7, 7, 7, 5, 2, 0, 0, 0, 0, 0,
 };
 
-static int8 codec47_table_big2[] = {
+static const int8 codec47_table_big2[] = {
   0, 0, 0, 0, 1, 3, 4, 6, 7, 7, 7, 7, 6, 4, 3, 1,
 };
 
@@ -148,7 +151,7 @@ void Codec47Decoder::makeTablesInterpolation(int param) {
 	int32 b1, b2;
 	int32 value_table47_1_2, value_table47_1_1, value_table47_2_2, value_table47_2_1;
 	int32 tableSmallBig[64], tmp, s;
-	int8 *table47_1 = 0, *table47_2 = 0;
+	const int8 *table47_1 = 0, *table47_2 = 0;
 	int32 *ptr_small_big;
 	byte *ptr;
 	int i, x, y;
@@ -156,27 +159,19 @@ void Codec47Decoder::makeTablesInterpolation(int param) {
 	if (param == 8) {
 		table47_1 = codec47_table_big1;
 		table47_2 = codec47_table_big2;
-		ptr = _tableBig + 384;
+		ptr = _tableBig;
 		for (i = 0; i < 256; i++) {
-			*ptr = 0;
-			ptr += 388;
-		}
-		ptr = _tableBig + 385;
-		for (i = 0; i < 256; i++) {
-			*ptr = 0;
+			ptr[384] = 0;
+			ptr[385] = 0;
 			ptr += 388;
 		}
 	} else if (param == 4) {
 		table47_1 = codec47_table_small1;
 		table47_2 = codec47_table_small2;
-		ptr = _tableSmall + 96;
+		ptr = _tableSmall;
 		for (i = 0; i < 256; i++) {
-			*ptr = 0;
-			ptr += 128;
-		}
-		ptr = _tableSmall + 97;
-		for (i = 0; i < 256; i++) {
-			*ptr = 0;
+			ptr[96] = 0;
+			ptr[97] = 0;
 			ptr += 128;
 		}
 	} else {
@@ -509,10 +504,11 @@ void Codec47Decoder::decode2(byte *dst, const byte *src, int width, int height,
 	} while (--bh);
 }
 
-void Codec47Decoder::init(int width, int height) {
-	deinit();
+Codec47Decoder::Codec47Decoder(int width, int height) {
 	_width = width;
 	_height = height;
+	_tableBig = (byte *)malloc(256 * 388);
+	_tableSmall = (byte *)malloc(256 * 128);
 	makeTablesInterpolation(4);
 	makeTablesInterpolation(8);
 
@@ -524,13 +520,15 @@ void Codec47Decoder::init(int width, int height) {
 	_curBuf = _deltaBuf + _frameSize * 2;
 }
 
-Codec47Decoder::Codec47Decoder() {
-	_tableBig = (byte *)malloc(99328);
-	_tableSmall = (byte *)malloc(32768);
-	_deltaBuf = NULL;
-}
-
-void Codec47Decoder::deinit() {
+Codec47Decoder::~Codec47Decoder() {
+	if (_tableBig) {
+		free(_tableBig);
+		_tableBig = NULL;
+	}
+	if (_tableSmall) {
+		free(_tableSmall);
+		_tableSmall = NULL;
+	}
 	_lastTableWidth = -1;
 	if (_deltaBuf) {
 		free(_deltaBuf);
@@ -541,18 +539,6 @@ void Codec47Decoder::deinit() {
 	}
 }
 
-Codec47Decoder::~Codec47Decoder() {
-	if (_tableBig) {
-		free(_tableBig);
-		_tableBig = NULL;
-	}
-	if (_tableSmall) {
-		free(_tableSmall);
-		_tableSmall = NULL;
-	}
-	deinit();
-}
-
 bool Codec47Decoder::decode(byte *dst, const byte *src) {
 	_offset1 = _deltaBufs[1] - _curBuf;
 	_offset2 = _deltaBufs[0] - _curBuf;
@@ -560,7 +546,6 @@ bool Codec47Decoder::decode(byte *dst, const byte *src) {
 	int32 seq_nb = READ_LE_UINT16(src + 0);
 
 	const byte *gfx_data = src + 26;
-	byte *tmp_ptr;
 
 	if (seq_nb == 0) {
 		makeTables47(_width);
@@ -573,11 +558,12 @@ bool Codec47Decoder::decode(byte *dst, const byte *src) {
 		gfx_data += 32896;
 	}
 
-	switch(src[2]) {
+	switch (src[2]) {
 	case 0:
 		memcpy(_curBuf, gfx_data, _frameSize);
 		break;
 	case 1:
+		// Used by Outlaws, but not by any SCUMM game.
 		error("codec47: not implemented decode1 proc");
 		break;
 	case 2:
@@ -600,14 +586,10 @@ bool Codec47Decoder::decode(byte *dst, const byte *src) {
 
 	if (seq_nb == _prevSeqNb + 1) {
 		if (src[3] == 1) {
-			tmp_ptr = _curBuf;
-			_curBuf = _deltaBufs[1];
-			_deltaBufs[1] = tmp_ptr;
+			SWAP(_curBuf, _deltaBufs[1]);
 		} else if (src[3] == 2) {
-			tmp_ptr = _deltaBufs[0];
-			_deltaBufs[0] = _deltaBufs[1];
-			_deltaBufs[1] = _curBuf;
-			_curBuf = tmp_ptr;
+			SWAP(_deltaBufs[0], _deltaBufs[1]);
+			SWAP(_deltaBufs[1], _curBuf);
 		}
 	}
 	_prevSeqNb = seq_nb;
