@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2002-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,13 +18,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/scumm/smush/codec47.h $
- * $Id: codec47.h 20511 2006-02-11 09:55:41Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/scumm/smush/codec47.h $
+ * $Id: codec47.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
-#ifndef SMUSH_CODEC_47_H
-#define SMUSH_CODEC_47_H
+#ifndef SCUMM_SMUSH_CODEC_47_H
+#define SCUMM_SMUSH_CODEC_47_H
 
 #include "common/scummsys.h"
 
@@ -53,10 +56,8 @@ private:
 	void decode2(byte *dst, const byte *src, int width, int height, const byte *param_ptr);
 
 public:
-	Codec47Decoder();
+	Codec47Decoder(int width, int height);
 	~Codec47Decoder();
-	void init(int width, int height);
-	void deinit();
 	bool decode(byte *dst, const byte *src);
 };
 
