@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2002-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,38 +18,22 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/scumm/smush/imuse_channel.cpp $
- * $Id: imuse_channel.cpp 20511 2006-02-11 09:55:41Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/scumm/smush/imuse_channel.cpp $
+ * $Id: imuse_channel.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
 #include "common/stdafx.h"
-#include "scumm/scumm.h"
+#include "common/endian.h"
+
+#include "scumm/scumm.h"	// For DEBUG_SMUSH
+#include "scumm/util.h"
 #include "scumm/smush/channel.h"
 #include "scumm/smush/chunk.h"
-#include "scumm/smush/chunk_type.h"
 
 namespace Scumm {
 
-ImuseChannel::ImuseChannel(int32 track, int32 freq) :
-	_track(track),
-	_tbuffer(0),
-	_tbufferSize(0),
-	_sbuffer(0),
-	_sbufferSize(0),
-	_frequency(freq),
-	_dataSize(-1),
-	_inData(false) {
-}
-
-ImuseChannel::~ImuseChannel() {
-	if (_tbuffer) {
-		delete []_tbuffer;
-	}
-	if (_sbuffer) {
-		warning("_sbuffer should be 0 !!!");
-		delete []_sbuffer;
-	}
+ImuseChannel::ImuseChannel(int32 track) : SmushChannel(track) {
 }
 
 bool ImuseChannel::isTerminated() const {
@@ -76,9 +63,9 @@ bool ImuseChannel::checkParameters(int32 index, int32 nbframes, int32 size, int3
 bool ImuseChannel::appendData(Chunk &b, int32 size) {
 	if (_dataSize == -1) {
 		assert(size > 8);
-		Chunk::type imus_type = b.getDword(); imus_type = SWAP_BYTES_32(imus_type);
-		uint32 imus_size = b.getDword(); imus_size = SWAP_BYTES_32(imus_size);
-		if (imus_type != TYPE_iMUS)
+		Chunk::type imus_type = b.readUint32BE();
+		/*uint32 imus_size =*/ b.readUint32BE();
+		if (imus_type != MKID_BE('iMUS'))
 			error("Invalid Chunk for imuse_channel");
 		size -= 8;
 		_tbufferSize = size;
@@ -107,53 +94,43 @@ bool ImuseChannel::appendData(Chunk &b, int32 size) {
 			b.read(_tbuffer, size);
 		}
 	}
-	return processBuffer();
-}
 
-bool ImuseChannel::handleFormat(Chunk &src) {
-	if (src.getSize() != 20) error("invalid size for FRMT Chunk");
-	uint32 imuse_start = src.getDword();
-	imuse_start = SWAP_BYTES_32(imuse_start);
-	src.seek(4);
-	_bitsize = src.getDword();
-	_bitsize = SWAP_BYTES_32(_bitsize);
-	_rate = src.getDword();
-	_rate = SWAP_BYTES_32(_rate);
-	_channels = src.getDword();
-	_channels = SWAP_BYTES_32(_channels);
-	assert(_channels == 1 || _channels == 2);
-	return true;
-}
+	processBuffer();
 
-bool ImuseChannel::handleRegion(Chunk &src) {
-	if (src.getSize() != 8)
-		error("invalid size for REGN Chunk");
-	return true;
-}
+	_srbufferSize = _sbufferSize;
+	if (_sbuffer && _bitsize == 12)
+		decode();
 
-bool ImuseChannel::handleStop(Chunk &src) {
-	if (src.getSize() != 4)
-		error("invalid size for STOP Chunk");
 	return true;
 }
 
 bool ImuseChannel::handleMap(Chunk &map) {
-	while (!map.eof()) {
+	while (!map.eos()) {
 		Chunk *sub = map.subBlock();
-		switch(sub->getType()) {
-		case TYPE_FRMT:
-			handleFormat(*sub);
+		switch (sub->getType()) {
+		case MKID_BE('FRMT'):
+			if (sub->size() != 20)
+				error("invalid size for FRMT Chunk");
+			/*uint32 imuse_start =*/ sub->readUint32BE();
+			sub->skip(4);
+			_bitsize = sub->readUint32BE();
+			_rate = sub->readUint32BE();
+			_channels = sub->readUint32BE();
+			assert(_channels == 1 || _channels == 2);
 			break;
-		case TYPE_TEXT:
+		case MKID_BE('TEXT'):
+			// Ignore this
 			break;
-		case TYPE_REGN:
-			handleRegion(*sub);
+		case MKID_BE('REGN'):
+			if (sub->size() != 8)
+				error("invalid size for REGN Chunk");
 			break;
-		case TYPE_STOP:
-			handleStop(*sub);
+		case MKID_BE('STOP'):
+			if (sub->size() != 4)
+				error("invalid size for STOP Chunk");
 			break;
 		default:
-			error("Unknown iMUS subChunk found : %s, %d", Chunk::ChunkString(sub->getType()), sub->getSize());
+			error("Unknown iMUS subChunk found : %s, %d", tag2str(sub->getType()), sub->size());
 		}
 		delete sub;
 	}
@@ -184,7 +161,7 @@ void ImuseChannel::decode() {
 		}
 	}
 
-	// FIXME: Code duplication! See decode12BitsSample() in scumm/imuse_digi.cpp
+	// FIXME: Code duplication! See decode12BitsSample() in imuse_digi/dimuse_codecs.cpp
 
 	int loop_size = _sbufferSize / 3;
 	int new_size = loop_size * 4;
@@ -213,15 +190,15 @@ bool ImuseChannel::handleSubTags(int32 &offset) {
 		Chunk::type type = READ_BE_UINT32(_tbuffer + offset);
 		uint32 size = READ_BE_UINT32(_tbuffer + offset + 4);
 		uint32 available_size = _tbufferSize - offset;
-		switch(type) {
-		case TYPE_MAP_:
+		switch (type) {
+		case MKID_BE('MAP '):
 			_inData = false;
 			if (available_size >= (size + 8)) {
 				MemoryChunk c((byte *)_tbuffer + offset);
 				handleMap(c);
 			}
 			break;
-		case TYPE_DATA:
+		case MKID_BE('DATA'):
 			_inData = true;
 			_dataSize = size;
 			offset += 8;
@@ -243,7 +220,7 @@ bool ImuseChannel::handleSubTags(int32 &offset) {
 			}
 			return false;
 		default:
-			error("unknown Chunk in iMUS track : %s ", Chunk::ChunkString(type));
+			error("unknown Chunk in iMUS track : %s ", tag2str(type));
 		}
 		offset += size + 8;
 		return true;
@@ -251,97 +228,16 @@ bool ImuseChannel::handleSubTags(int32 &offset) {
 	return false;
 }
 
-bool ImuseChannel::processBuffer() {
-	assert(_tbuffer != 0);
-	assert(_tbufferSize != 0);
-	assert(_sbuffer == 0);
-	assert(_sbufferSize == 0);
+byte *ImuseChannel::getSoundData() {
+	byte *tmp = _sbuffer;
 
-	if (_inData) {
-		if (_dataSize < _tbufferSize) {
-			int32 offset= _dataSize;
-			while (handleSubTags(offset));
-			_sbufferSize = _dataSize;
-			_sbuffer = _tbuffer;
-			if (offset < _tbufferSize) {
-				int32 new_size = _tbufferSize - offset;
-				_tbuffer = new byte[new_size];
-				if (!_tbuffer) error("imuse_channel failed to allocate memory");
-				memcpy(_tbuffer, _sbuffer + offset, new_size);
-				_tbufferSize = new_size;
-			} else {
-				_tbuffer = 0;
-				_tbufferSize = 0;
-			}
-			if (_sbufferSize == 0) {
-				delete []_sbuffer;
-				_sbuffer = 0;
-			}
-		} else {
-			_sbufferSize = _tbufferSize;
-			_sbuffer = _tbuffer;
-			_tbufferSize = 0;
-			_tbuffer = 0;
-		}
-	} else {
-		int32 offset = 0;
-		while (handleSubTags(offset));
-		if (_inData) {
-			_sbufferSize = _tbufferSize - offset;
-			assert(_sbufferSize);
-			_sbuffer = new byte[_sbufferSize];
-			if (!_sbuffer) error("imuse_channel failed to allocate memory");
-			memcpy(_sbuffer, _tbuffer + offset, _sbufferSize);
-			delete []_tbuffer;
-			_tbuffer = 0;
-			_tbufferSize = 0;
-		} else {
-			if (offset) {
-				byte * old = _tbuffer;
-				int32 new_size = _tbufferSize - offset;
-				_tbuffer = new byte[new_size];
-				if (!_tbuffer) error("imuse_channel failed to allocate memory");
-				memcpy(_tbuffer, old + offset, new_size);
-				_tbufferSize = new_size;
-				delete []old;
-			}
-		}
-	}
-	_srbufferSize = _sbufferSize;
-	if (_sbuffer && _bitsize == 12) decode();
-	return true;
-}
+	assert(_dataSize > 0);
+	_dataSize -= _srbufferSize;
 
-int32 ImuseChannel::availableSoundData(void) const {
-	int32 ret = _sbufferSize;
-	if (_channels == 2) ret /= 2;
-	if (_bitsize > 8) ret /= 2;
-	return ret;
-}
-
-void ImuseChannel::getSoundData(int16 *snd, int32 size) {
-	if (_dataSize <= 0 || _bitsize <= 8) error("invalid call to imuse_channel::read_sound_data()");
-	if (_channels == 2) size *= 2;
-
-	memcpy(snd, _sbuffer, size * 2);
-
-	delete []_sbuffer;
-	assert(_sbufferSize == 2 * size);
 	_sbuffer = 0;
 	_sbufferSize = 0;
-	_dataSize -= _srbufferSize;
-}
-
-void ImuseChannel::getSoundData(int8 *snd, int32 size) {
-	if (_dataSize <= 0 || _bitsize > 8) error("invalid call to imuse_channel::read_sound_data()");
-	if (_channels == 2) size *= 2;
-
-	memcpy(snd, _sbuffer, size);
-
-	delete []_sbuffer;
-	_sbuffer = 0;
-	_sbufferSize = 0;
-	_dataSize -= _srbufferSize;
+	
+	return tmp;
 }
 
 } // End of namespace Scumm
