@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2002-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/scumm/smush/smush_font.cpp $
- * $Id: smush_font.cpp 20795 2006-02-20 16:51:30Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/scumm/smush/smush_font.cpp $
+ * $Id: smush_font.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -29,8 +32,8 @@
 
 namespace Scumm {
 
-SmushFont::SmushFont(ScummEngine *vm, bool use_original_colors, bool new_colors) :
-	NutRenderer(vm),
+SmushFont::SmushFont(ScummEngine *vm, const char *filename, bool use_original_colors, bool new_colors) :
+	NutRenderer(vm, filename),
 	_color(-1),
 	_new_colors(new_colors),
 	_original(use_original_colors) {
@@ -38,10 +41,6 @@ SmushFont::SmushFont(ScummEngine *vm, bool use_original_colors, bool new_colors)
 
 int SmushFont::getStringWidth(const char *str) {
 	assert(str);
-	if (!_loaded) {
-		error("SmushFont::getStringWidth() Font is not loaded");
-		return 0;
-	}
 
 	int width = 0;
 	while (*str) {
@@ -56,10 +55,6 @@ int SmushFont::getStringWidth(const char *str) {
 
 int SmushFont::getStringHeight(const char *str) {
 	assert(str);
-	if (!_loaded) {
-		error("SmushFont::getStringHeight() Font is not loaded");
-		return 0;
-	}
 
 	int height = 0;
 	while (*str) {
@@ -73,7 +68,7 @@ int SmushFont::getStringHeight(const char *str) {
 int SmushFont::drawChar(byte *buffer, int dst_width, int x, int y, byte chr) {
 	int w = _chars[chr].width;
 	int h = _chars[chr].height;
-	const byte *src = _chars[chr].src;
+	const byte *src = unpackChar(chr);
 	byte *dst = buffer + dst_width * y + x;
 
 	assert(dst_width == _vm->_screenWidth);
@@ -82,7 +77,7 @@ int SmushFont::drawChar(byte *buffer, int dst_width, int x, int y, byte chr) {
 		for (int j = 0; j < h; j++) {
 			for (int i = 0; i < w; i++) {
 				int8 value = *src++;
-				if (value)
+				if (value != _chars[chr].transparency)
 					dst[i] = value;
 			}
 			dst += dst_width;
@@ -97,7 +92,7 @@ int SmushFont::drawChar(byte *buffer, int dst_width, int x, int y, byte chr) {
 						dst[i] = 0xFF;
 					} else if (value == -31) {
 						dst[i] = 0;
-					} else if (value) {
+					} else if (value != _chars[chr].transparency) {
 						dst[i] = value;
 					}
 				}
@@ -109,7 +104,7 @@ int SmushFont::drawChar(byte *buffer, int dst_width, int x, int y, byte chr) {
 					int8 value = *src++;
 					if (value == 1) {
 						dst[i] = color;
-					} else if (value) {
+					} else if (value != _chars[chr].transparency) {
 						dst[i] = 0;
 					}
 				}
