@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2002-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,13 +18,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/scumm/smush/smush_font.h $
- * $Id: smush_font.h 20511 2006-02-11 09:55:41Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/scumm/smush/smush_font.h $
+ * $Id: smush_font.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
-#ifndef SMUSH_FONT_H
-#define SMUSH_FONT_H
+#ifndef SCUMM_SMUSH_FONT_H
+#define SCUMM_SMUSH_FONT_H
 
 #include "common/scummsys.h"
 #include "scumm/nut_renderer.h"
@@ -42,7 +45,7 @@ protected:
 	void drawSubstring(const char *str, byte *buffer, int dst_width, int x, int y);
 
 public:
-	SmushFont(ScummEngine *vm, bool use_original_colors, bool new_colors);
+	SmushFont(ScummEngine *vm, const char *filename, bool use_original_colors, bool new_colors);
 
 	void setColor(byte c) { _color = c; }
 	void drawString    (const char *str, byte *buffer, int dst_width, int dst_height, int x, int y, bool center);
