@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2002-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/scumm/smush/smush_mixer.cpp $
- * $Id: smush_mixer.cpp 22526 2006-05-18 21:46:07Z eriktorbjorn $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/scumm/smush/smush_mixer.cpp $
+ * $Id: smush_mixer.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -45,14 +48,12 @@ SmushMixer::SmushMixer(Audio::Mixer *m) :
 }
 
 SmushMixer::~SmushMixer() {
-	Common::StackLock lock(_mutex);
 	for (int32 i = 0; i < NUM_CHANNELS; i++) {
 		_mixer->stopHandle(_channels[i].handle);
 	}
 }
 
 SmushChannel *SmushMixer::findChannel(int32 track) {
-	Common::StackLock lock(_mutex);
 	debugC(DEBUG_SMUSH, "SmushMixer::findChannel(%d)", track);
 	for (int32 i = 0; i < NUM_CHANNELS; i++) {
 		if (_channels[i].id == track)
@@ -62,7 +63,6 @@ SmushChannel *SmushMixer::findChannel(int32 track) {
 }
 
 void SmushMixer::addChannel(SmushChannel *c) {
-	Common::StackLock lock(_mutex);
 	int32 track = c->getTrackIdentifier();
 	int i;
 
@@ -91,7 +91,6 @@ void SmushMixer::addChannel(SmushChannel *c) {
 }
 
 bool SmushMixer::handleFrame() {
-	Common::StackLock lock(_mutex);
 	debugC(DEBUG_SMUSH, "SmushMixer::handleFrame()");
 	for (int i = 0; i < NUM_CHANNELS; i++) {
 		if (_channels[i].id != -1) {
@@ -104,39 +103,33 @@ bool SmushMixer::handleFrame() {
 					_channels[i].stream = 0;
 				}
 			} else {
-				int32 rate, vol, pan;
+				int32 vol, pan;
 				bool stereo, is_16bit;
-				void *data;
 
-				_channels[i].chan->getParameters(rate, stereo, is_16bit, vol, pan);
-				int32 size = _channels[i].chan->availableSoundData();
+				_channels[i].chan->getParameters(stereo, is_16bit, vol, pan);
+
+				// Grab the audio data from the channel
+				int32 size = _channels[i].chan->getAvailableSoundDataSize();
+				byte *data = _channels[i].chan->getSoundData();
+
 				byte flags = stereo ? Audio::Mixer::FLAG_STEREO : 0;
-
 				if (is_16bit) {
-					data = malloc(size * (stereo ? 2 : 1) * 4);
-					_channels[i].chan->getSoundData((int16 *)data, size);
-					size *= stereo ? 4 : 2;
-
 					flags |= Audio::Mixer::FLAG_16BITS;
-
 				} else {
-					data = malloc(size * (stereo ? 2 : 1) * 2);
-					_channels[i].chan->getSoundData((int8 *)data, size);
-					size *= stereo ? 2 : 1;
-
 					flags |= Audio::Mixer::FLAG_UNSIGNED;
 				}
 
 				if (_mixer->isReady()) {
+					// Stream the data
 					if (!_channels[i].stream) {
-						_channels[i].stream = Audio::makeAppendableAudioStream(rate, flags, 500000);
+						_channels[i].stream = Audio::makeAppendableAudioStream(_channels[i].chan->getRate(), flags);
 						_mixer->playInputStream(Audio::Mixer::kSFXSoundType, &_channels[i].handle, _channels[i].stream);
 					}
 					_mixer->setChannelVolume(_channels[i].handle, vol);
 					_mixer->setChannelBalance(_channels[i].handle, pan);
-					_channels[i].stream->append((byte *)data, size);
-				}
-				free(data);
+					_channels[i].stream->queueBuffer(data, size);	// The stream will free the buffer for us
+				} else
+					delete[] data;
 			}
 		}
 	}
@@ -144,7 +137,6 @@ bool SmushMixer::handleFrame() {
 }
 
 bool SmushMixer::stop() {
-	Common::StackLock lock(_mutex);
 	debugC(DEBUG_SMUSH, "SmushMixer::stop()");
 	for (int i = 0; i < NUM_CHANNELS; i++) {
 		if (_channels[i].id != -1) {
@@ -162,7 +154,6 @@ bool SmushMixer::stop() {
 }
 
 bool SmushMixer::flush() {
-	Common::StackLock lock(_mutex);
 	debugC(DEBUG_SMUSH, "SmushMixer::flush()");
 	for (int i = 0; i < NUM_CHANNELS; i++) {
 		if (_channels[i].id != -1) {
