@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2002-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,19 +18,18 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/scumm/smush/smush_player.cpp $
- * $Id: smush_player.cpp 23966 2006-09-21 17:49:42Z sev $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/scumm/smush/smush_player.cpp $
+ * $Id: smush_player.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
 #include "common/stdafx.h"
 
-#include "base/engine.h"
+#include "engines/engine.h"
 
 #include "common/config-manager.h"
 #include "common/file.h"
 #include "common/system.h"
-#include "common/timer.h"
 #include "common/util.h"
 
 #include "graphics/cursorman.h"
@@ -38,9 +40,11 @@
 #include "scumm/imuse/imuse.h"
 #include "scumm/scumm.h"
 #include "scumm/sound.h"
+#include "scumm/util.h"
 #include "scumm/smush/channel.h"
-#include "scumm/smush/chunk_type.h"
 #include "scumm/smush/chunk.h"
+#include "scumm/smush/codec37.h"
+#include "scumm/smush/codec47.h"
 #include "scumm/smush/smush_font.h"
 #include "scumm/smush/smush_mixer.h"
 #include "scumm/smush/smush_player.h"
@@ -61,7 +65,8 @@
 
 namespace Scumm {
 
-const int MAX_STRINGS = 200;
+static const int MAX_STRINGS = 200;
+static const int ETRS_HEADER_LENGTH = 16;
 
 class StringResource {
 private:
@@ -80,7 +85,7 @@ public:
 	StringResource() :
 		_nbStrings(0),
 		_lastId(-1) {
-	};
+	}
 	~StringResource() {
 		for (int32 i = 0; i < _nbStrings; i++) {
 			delete []_strings[i].string;
@@ -192,26 +197,13 @@ static StringResource *getStrings(ScummEngine *vm, const char *file, bool is_enc
 	theFile.read(filebuffer, length);
 	filebuffer[length] = 0;
 
-	if (is_encoded) {
-		enum {
-			ETRS_HEADER_LENGTH = 16
-		};
+	if (is_encoded && READ_BE_UINT32(filebuffer) == MKID_BE('ETRS')) {
 		assert(length > ETRS_HEADER_LENGTH);
-		Chunk::type type = READ_BE_UINT32(filebuffer);
-
-		if (type != TYPE_ETRS) {
-			delete [] filebuffer;
-			return getStrings(vm, file, false);
-		}
-
-		char *old = filebuffer;
-		filebuffer = new char[length - ETRS_HEADER_LENGTH + 1];
-		for (int32 i = ETRS_HEADER_LENGTH; i < length; i++) {
-			filebuffer[i - ETRS_HEADER_LENGTH] = old[i] ^ 0xCC;
-		}
-		filebuffer[length - ETRS_HEADER_LENGTH] = '\0';
-		delete []old;
 		length -= ETRS_HEADER_LENGTH;
+		for (int i = 0; i < length; ++i) {
+			filebuffer[i] = filebuffer[i + ETRS_HEADER_LENGTH] ^ 0xCC;
+		}
+		filebuffer[length] = '\0';
 	}
 	StringResource *sr = new StringResource;
 	assert(sr);
@@ -220,26 +212,20 @@ static StringResource *getStrings(ScummEngine *vm, const char *file, bool is_enc
 	return sr;
 }
 
-void SmushPlayer::timerCallback(void *refCon) {
-	((SmushPlayer *)refCon)->parseNextFrame();
+void SmushPlayer::timerCallback() {
+	parseNextFrame();
 #ifdef _WIN32_WCE
-	((SmushPlayer *)refCon)->_inTimer = true;
-	((SmushPlayer *)refCon)->_inTimerCount++;
-#endif
-#ifdef __SYMBIAN32__
-	if(((SmushPlayer *)refCon)->_closeOnTextTick) {
-		delete ((SmushPlayer *)refCon)->_base;
-		((SmushPlayer *)refCon)->_base = NULL;
-		((SmushPlayer *)refCon)->_closeOnTextTick = false;
-	}
+	_inTimer = true;
+	_inTimerCount++;
 #endif
 }
 
-SmushPlayer::SmushPlayer(ScummEngine_v6 *scumm, int speed) {
+SmushPlayer::SmushPlayer(ScummEngine_v7 *scumm) {
 	_vm = scumm;
-	_version = -1;
 	_nbframes = 0;
-	_smixer = 0;
+	_codec37 = 0;
+	_codec47 = 0;
+	_smixer = NULL;
 	_strings = NULL;
 	_sf[0] = NULL;
 	_sf[1] = NULL;
@@ -259,32 +245,33 @@ SmushPlayer::SmushPlayer(ScummEngine_v6 *scumm, int speed) {
 	_width = 0;
 	_height = 0;
 	_IACTpos = 0;
-	_soundFrequency = 22050;
-	_initDone = false;
-	_speed = speed;
+	_speed = -1;
 	_insanity = false;
 	_middleAudio = false;
 	_skipPalette = false;
 	_IACTstream = NULL;
 	_smixer = _vm->_smixer;
+	_paused = false;
+	_pauseStartTime = 0;
+	_pauseTime = 0;
 #ifdef _WIN32_WCE
 	_inTimer = false;
 	_inTimerCount = 0;
 	_inTimerCountRedraw = ConfMan.getInt("Smush_force_redraw");
 #endif
-#ifdef __SYMBIAN32__
-	_closeOnTextTick = false;
-#endif
 }
 
 SmushPlayer::~SmushPlayer() {
-	release();
 }
 
-void SmushPlayer::init() {
+void SmushPlayer::init(int32 speed) {
 	_frame = 0;
-	_alreadyInit = false;
+	_speed = speed;
+	_endOfFile = false;
+
 	_vm->_smushVideoShouldFinish = false;
+	_vm->_smushActive = true;
+
 	_vm->setDirtyColors(0, 255);
 	_dst = _vm->virtscr[0].getPixels(0, 0);
 
@@ -295,29 +282,17 @@ void SmushPlayer::init() {
 	// width and pitch to be equal, this will require lots of changes. So
 	// we resort to this hackish solution for now.
 	_origPitch = _vm->virtscr[0].pitch;
-	_origNumStrips = _vm->gdi._numStrips;
+	_origNumStrips = _vm->_gdi->_numStrips;
 	_vm->virtscr[0].pitch = _vm->virtscr[0].w;
-	_vm->gdi._numStrips = _vm->virtscr[0].w / 8;
+	_vm->_gdi->_numStrips = _vm->virtscr[0].w / 8;
 
+	_vm->_mixer->stopHandle(_compressedFileSoundHandle);
+	_vm->_mixer->stopHandle(_IACTchannel);
+	_IACTpos = 0;
 	_vm->_smixer->stop();
-
-	Common::g_timer->installTimerProc(&timerCallback, 1000000 / _speed, this);
-
-	_initDone = true;
 }
 
 void SmushPlayer::release() {
-	if (!_initDone)
-		return;
-#ifdef __SYMBIAN32__
-	_closeOnTextTick = true;
-	// Wait for _closeOnTextTick to be set to false to indicate file closure
-	while(_closeOnTextTick) {
-		User::After(15624); 
-	}
-#endif
-	_vm->_timer->removeTimerProc(&timerCallback);
-
 	_vm->_smushVideoShouldFinish = true;
 
 	for (int i = 0; i < 5; i++) {
@@ -337,33 +312,28 @@ void SmushPlayer::release() {
 	free(_frameBuffer);
 	_frameBuffer = NULL;
 
-	_vm->_mixer->stopHandle(_compressedFileSoundHandle);
-
-	_vm->_mixer->stopHandle(_IACTchannel);
 	_IACTstream = NULL;
 
+	_vm->_smushActive = false;
 	_vm->_fullRedraw = true;
 
-	// WORKAROUND bug #1035739: This is hack to workaround some ugly palette
-	// issues, see the mentioned bug report for details.
-	_vm->_doEffect = false;
-
-
 	// HACK HACK HACK: This is an *evil* trick, beware! See above for
 	// some explanation.
 	_vm->virtscr[0].pitch = _origPitch;
-	_vm->gdi._numStrips = _origNumStrips;
+	_vm->_gdi->_numStrips = _origNumStrips;
 
-
-	_initDone = false;
+	delete _codec37;
+	_codec37 = 0;
+	delete _codec47;
+	_codec47 = 0;
 }
 
 void SmushPlayer::checkBlock(const Chunk &b, Chunk::type type_expected, uint32 min_size) {
 	if (type_expected != b.getType()) {
 		error("Chunk type is different from expected : %x != %x", b.getType(), type_expected);
 	}
-	if (min_size > b.getSize()) {
-		error("Chunk size is inferior than minimum required size : %d < %d", b.getSize(), min_size);
+	if (min_size > b.size()) {
+		error("Chunk size is inferior than minimum required size : %d < %d", b.size(), min_size);
 	}
 }
 
@@ -377,11 +347,11 @@ void SmushPlayer::handleSoundBuffer(int32 track_id, int32 index, int32 max_frame
 //	}
 	SmushChannel *c = _smixer->findChannel(track_id);
 	if (c == NULL) {
-		c = new SaudChannel(track_id, _soundFrequency);
+		c = new SaudChannel(track_id);
 		_smixer->addChannel(c);
 	}
 
-	if (_middleAudio || (index == 0)) {
+	if (_middleAudio || index == 0) {
 		c->setParameters(max_frames, flags, vol, pan, index);
 	} else {
 		c->checkParameters(index, max_frames, flags, vol, pan);
@@ -391,41 +361,31 @@ void SmushPlayer::handleSoundBuffer(int32 track_id, int32 index, int32 max_frame
 }
 
 void SmushPlayer::handleSoundFrame(Chunk &b) {
-	checkBlock(b, TYPE_PSAD);
+	checkBlock(b, MKID_BE('PSAD'));
 	debugC(DEBUG_SMUSH, "SmushPlayer::handleSoundFrame()");
 
-	int32 track_id = b.getWord();
-	int32 index = b.getWord();
-	int32 max_frames = b.getWord();
-	int32 flags = b.getWord();
-	int32 vol = b.getByte();
-	int32 pan = b.getChar();
+	int32 track_id = b.readUint16LE();
+	int32 index = b.readUint16LE();
+	int32 max_frames = b.readUint16LE();
+	int32 flags = b.readUint16LE();
+	int32 vol = b.readByte();
+	int32 pan = b.readSByte();
 	if (index == 0) {
 		debugC(DEBUG_SMUSH, "track_id:%d, max_frames:%d, flags:%d, vol:%d, pan:%d", track_id, max_frames, flags, vol, pan);
 	}
-	int32 size = b.getSize() - 10;
+	int32 size = b.size() - 10;
 	handleSoundBuffer(track_id, index, max_frames, flags, vol, pan, b, size);
 }
 
-void SmushPlayer::handleSkip(Chunk &b) {
-	checkBlock(b, TYPE_SKIP, 4);
-	int32 code = b.getDword();
-	debugC(DEBUG_SMUSH, "SmushPlayer::handleSkip(%d)", code);
-	if (code >= 0 && code < 37)
-		_skipNext = _skips[code];
-	else
-		_skipNext = true;
-}
-
 void SmushPlayer::handleStore(Chunk &b) {
 	debugC(DEBUG_SMUSH, "SmushPlayer::handleStore()");
-	checkBlock(b, TYPE_STOR, 4);
+	checkBlock(b, MKID_BE('STOR'), 4);
 	_storeFrame = true;
 }
 
 void SmushPlayer::handleFetch(Chunk &b) {
 	debugC(DEBUG_SMUSH, "SmushPlayer::handleFetch()");
-	checkBlock(b, TYPE_FTCH, 6);
+	checkBlock(b, MKID_BE('FTCH'), 6);
 
 	if (_frameBuffer != NULL) {
 		memcpy(_dst, _frameBuffer, _width * _height);
@@ -433,20 +393,29 @@ void SmushPlayer::handleFetch(Chunk &b) {
 }
 
 void SmushPlayer::handleIACT(Chunk &b) {
-	checkBlock(b, TYPE_IACT, 8);
-	debugC(DEBUG_SMUSH, "SmushPlayer::handleImuseAction()");
+	checkBlock(b, MKID_BE('IACT'), 8);
+	debugC(DEBUG_SMUSH, "SmushPlayer::IACT()");
 
-	/* int code = */ b.getWord();
-	int flags = b.getWord();
-	int unknown = b.getShort();
-	int track_flags = b.getWord();
+	int code = b.readUint16LE();
+	int flags = b.readUint16LE();
+	int unknown = b.readSint16LE();
+	int track_flags = b.readUint16LE();
+
+	if ((code != 8) && (flags != 46)) {
+		_vm->_insane->procIACT(_dst, 0, 0, 0, b, 0, 0, code, flags, unknown, track_flags);
+		return;
+	}
+
+	if (_compressedFileMode) {
+		return;
+	}
 
 	assert(flags == 46 && unknown == 0);
-	int track_id = b.getWord();
-	int index = b.getWord();
-	int nbframes = b.getWord();
-	int32 size = b.getDword();
-	int32 bsize = b.getSize() - 18;
+	int track_id = b.readUint16LE();
+	int index = b.readUint16LE();
+	int nbframes = b.readUint16LE();
+	int32 size = b.readUint32LE();
+	int32 bsize = b.size() - 18;
 
 	if (_vm->_game.id != GID_CMI) {
 		int32 track = track_id;
@@ -463,13 +432,13 @@ void SmushPlayer::handleIACT(Chunk &b) {
 		} else if ((track_flags >= 300) && (track_flags <= 363)) {
 			track = track_id + 600;
 		} else {
-			error("ImuseChannel::handleIACT(): bad track_flags: %d", track_flags);
+			error("SmushPlayer::handleIACT(): bad track_flags: %d", track_flags);
 		}
 		debugC(DEBUG_SMUSH, "SmushPlayer::handleIACT(): %d, %d, %d", track, index, track_flags);
 
 		SmushChannel *c = _smixer->findChannel(track);
 		if (c == 0) {
-			c = new ImuseChannel(track, _soundFrequency);
+			c = new ImuseChannel(track);
 			_smixer->addChannel(c);
 		}
 		if (index == 0)
@@ -478,7 +447,7 @@ void SmushPlayer::handleIACT(Chunk &b) {
 			c->checkParameters(index, nbframes, size, track_flags, unknown);
 		c->appendData(b, bsize);
 	} else {
-		byte output_data[4096];
+		// TODO: Move this code into another SmushChannel subclass?
 		byte *src = (byte *)malloc(bsize);
 		b.read(src, bsize);
 		byte *d_src = src;
@@ -493,6 +462,8 @@ void SmushPlayer::handleIACT(Chunk &b) {
 					_IACTpos += bsize;
 					bsize = 0;
 				} else {
+					byte *output_data = new byte[4096];
+
 					memcpy(_IACToutput + _IACTpos, d_src, len);
 					byte *dst = output_data;
 					byte *d_src2 = _IACToutput;
@@ -523,10 +494,10 @@ void SmushPlayer::handleIACT(Chunk &b) {
 					} while (--count);
 
 					if (!_IACTstream) {
-						_IACTstream = Audio::makeAppendableAudioStream(22050, Audio::Mixer::FLAG_STEREO | Audio::Mixer::FLAG_16BITS, 900000);
+						_IACTstream = Audio::makeAppendableAudioStream(22050, Audio::Mixer::FLAG_STEREO | Audio::Mixer::FLAG_16BITS);
 						_vm->_mixer->playInputStream(Audio::Mixer::kSFXSoundType, &_IACTchannel, _IACTstream);
 					}
-					_IACTstream->append(output_data, 0x1000);
+					_IACTstream->queueBuffer(output_data, 0x1000);
 
 					bsize -= len;
 					d_src += len;
@@ -549,23 +520,23 @@ void SmushPlayer::handleIACT(Chunk &b) {
 }
 
 void SmushPlayer::handleTextResource(Chunk &b) {
-	int pos_x = b.getShort();
-	int pos_y = b.getShort();
-	int flags = b.getShort();
-	int left = b.getShort();
-	int top = b.getShort();
-	int right = b.getShort();
-	/*int32 height =*/ b.getShort();
-	/*int32 unk2 =*/ b.getWord();
+	int pos_x = b.readSint16LE();
+	int pos_y = b.readSint16LE();
+	int flags = b.readSint16LE();
+	int left = b.readSint16LE();
+	int top = b.readSint16LE();
+	int right = b.readSint16LE();
+	/*int32 height =*/ b.readSint16LE();
+	/*int32 unk2 =*/ b.readUint16LE();
 
 	const char *str;
 	char *string = NULL, *string2 = NULL;
-	if (b.getType() == TYPE_TEXT) {
-		string = (char *)malloc(b.getSize() - 16);
+	if (b.getType() == MKID_BE('TEXT')) {
+		string = (char *)malloc(b.size() - 16);
 		str = string;
-		b.read(string, b.getSize() - 16);
+		b.read(string, b.size() - 16);
 	} else {
-		int string_id = b.getWord();
+		int string_id = b.readUint16LE();
 		if (!_strings)
 			return;
 		str = _strings->get(string_id);
@@ -579,7 +550,7 @@ void SmushPlayer::handleTextResource(Chunk &b) {
 	if ((!ConfMan.getBool("subtitles")) && ((flags & 8) == 8))
 		return;
 
-	SmushFont *sf = _sf[0];
+	SmushFont *sf = getFont(0);
 	int color = 15;
 	while (*str == '/') {
 		str++; // For Full Throttle text resources
@@ -605,7 +576,7 @@ void SmushPlayer::handleTextResource(Chunk &b) {
 			{
 				int id = str[3] - '0';
 				str += 4;
-				sf = _sf[id];
+				sf = getFont(id);
 			}
 			break;
 		case 'c':
@@ -725,7 +696,7 @@ bool SmushPlayer::readString(const char *file) {
 		return true;
 	}
 
-	if ((_strings = getStrings(_vm, "digtxt.trs", true)) != 0) {
+	if (_vm->_game.id == GID_DIG && (_strings = getStrings(_vm, "digtxt.trs", true)) != 0) {
 		return true;
 	}
 	return false;
@@ -737,32 +708,28 @@ void SmushPlayer::readPalette(byte *out, Chunk &in) {
 
 static byte delta_color(byte org_color, int16 delta_color) {
 	int t = (org_color * 129 + delta_color) / 128;
-	if (t > 255)
-		t = 255;
-	if (t < 0)
-		t = 0;
-	return (byte)t;
+	return CLIP(t, 0, 255);
 }
 
 void SmushPlayer::handleDeltaPalette(Chunk &b) {
-	checkBlock(b, TYPE_XPAL);
+	checkBlock(b, MKID_BE('XPAL'));
 	debugC(DEBUG_SMUSH, "SmushPlayer::handleDeltaPalette()");
 
-	if (b.getSize() == 0x300 * 3 + 4) {
+	if (b.size() == 0x300 * 3 + 4) {
 
-		b.getWord();
-		b.getWord();
+		b.readUint16LE();
+		b.readUint16LE();
 
 		for (int i = 0; i < 0x300; i++) {
-			_deltaPal[i] = b.getWord();
+			_deltaPal[i] = b.readUint16LE();
 		}
 		readPalette(_pal, b);
 		setDirtyColors(0, 255);
-	} else if (b.getSize() == 6) {
+	} else if (b.size() == 6) {
 
-		b.getWord();
-		b.getWord();
-		b.getWord();
+		b.readUint16LE();
+		b.readUint16LE();
+		b.readUint16LE();
 
 		for (int i = 0; i < 0x300; i++) {
 			_pal[i] = delta_color(_pal[i], _deltaPal[i]);
@@ -774,7 +741,7 @@ void SmushPlayer::handleDeltaPalette(Chunk &b) {
 }
 
 void SmushPlayer::handleNewPalette(Chunk &b) {
-	checkBlock(b, TYPE_NPAL, 0x300);
+	checkBlock(b, MKID_BE('NPAL'), 0x300);
 	debugC(DEBUG_SMUSH, "SmushPlayer::handleNewPalette()");
 
 	if (_skipPalette)
@@ -786,6 +753,57 @@ void SmushPlayer::handleNewPalette(Chunk &b) {
 
 void smush_decode_codec1(byte *dst, const byte *src, int left, int top, int width, int height, int pitch);
 
+void SmushPlayer::decodeFrameObject(int codec, const uint8 *src, int left, int top, int width, int height) {
+	if ((height == 242) && (width == 384)) {
+		if (_specialBuffer == 0)
+			_specialBuffer = (byte *)malloc(242 * 384);
+		_dst = _specialBuffer;
+	} else if ((height > _vm->_screenHeight) || (width > _vm->_screenWidth))
+		return;
+	// FT Insane uses smaller frames to draw overlays with moving objects
+	// Other .san files do have them as well but their purpose in unknown
+	// and often it causes memory overdraw. So just skip those frames
+	else if (!_insanity && ((height != _vm->_screenHeight) || (width != _vm->_screenWidth)))
+		return;
+
+	if ((height == 242) && (width == 384)) {
+		_width = width;
+		_height = height;
+	} else {
+		_width = _vm->_screenWidth;
+		_height = _vm->_screenHeight;
+	}
+
+	switch (codec) {
+	case 1:
+	case 3:
+		smush_decode_codec1(_dst, src, left, top, width, height, _vm->_screenWidth);
+		break;
+	case 37:
+		if (!_codec37)
+			_codec37 = new Codec37Decoder(width, height);
+		if (_codec37)
+			_codec37->decode(_dst, src);
+		break;
+	case 47:
+		if (!_codec47)
+			_codec47 = new Codec47Decoder(width, height);
+		if (_codec47)
+			_codec47->decode(_dst, src);
+		break;
+	default:
+		error("Invalid codec for frame object : %d", codec);
+	}
+
+	if (_storeFrame) {
+		if (_frameBuffer == NULL) {
+			_frameBuffer = (byte *)malloc(_width * _height);
+		}
+		memcpy(_frameBuffer, _dst, _width * _height);
+		_storeFrame = false;
+	}
+}
+
 #ifdef USE_ZLIB
 void SmushPlayer::handleZlibFrameObject(Chunk &b) {
 	if (_skipNext) {
@@ -793,7 +811,7 @@ void SmushPlayer::handleZlibFrameObject(Chunk &b) {
 		return;
 	}
 
-	int32 chunkSize = b.getSize();
+	int32 chunkSize = b.size();
 	byte *chunkBuffer = (byte *)malloc(chunkSize);
 	assert(chunkBuffer);
 	b.read(chunkBuffer, chunkSize);
@@ -812,199 +830,93 @@ void SmushPlayer::handleZlibFrameObject(Chunk &b) {
 	int width = READ_LE_UINT16(ptr); ptr += 2;
 	int height = READ_LE_UINT16(ptr); ptr += 2;
 
-	if ((height == 242) && (width == 384)) {
-		if (_specialBuffer == 0)
-			_specialBuffer = (byte *)malloc(242 * 384);
-		_dst = _specialBuffer;
-	} else if ((height > _vm->_screenHeight) || (width > _vm->_screenWidth))
-		return;
-	// FT Insane uses smaller frames to draw overlays with moving objects
-	// Other .san files do have them as well but their purpose in unknown
-	// and often it causes memory overdraw. So just skip those frames
-	else if (!_insanity && ((height != _vm->_screenHeight) || (width != _vm->_screenWidth)))
-		return;
-
-	if (!_alreadyInit) {
-		_codec37.init(width, height);
-		_codec47.init(width, height);
-		_alreadyInit = true;
-	}
-
-	if ((height == 242) && (width == 384)) {
-		_width = width;
-		_height = height;
-	} else {
-		_width = _vm->_screenWidth;
-		_height = _vm->_screenHeight;
-	}
-
-	switch (codec) {
-	case 1:
-	case 3:
-		smush_decode_codec1(_dst, fobjBuffer + 14, left, top, width, height, _vm->_screenWidth);
-		break;
-	case 37:
-		_codec37.decode(_dst, fobjBuffer + 14);
-		break;
-	case 47:
-		_codec47.decode(_dst, fobjBuffer + 14);
-		break;
-	default:
-		error("Invalid codec for frame object : %d", (int)codec);
-	}
-
-	if (_storeFrame) {
-		if (_frameBuffer == NULL) {
-			_frameBuffer = (byte *)malloc(_width * _height);
-		}
-		memcpy(_frameBuffer, _dst, _width * _height);
-		_storeFrame = false;
-	}
+	decodeFrameObject(codec, fobjBuffer + 14, left, top, width, height);
 
 	free(fobjBuffer);
 }
 #endif
 
 void SmushPlayer::handleFrameObject(Chunk &b) {
-	checkBlock(b, TYPE_FOBJ, 14);
+	checkBlock(b, MKID_BE('FOBJ'), 14);
 	if (_skipNext) {
 		_skipNext = false;
 		return;
 	}
 
-	int codec = b.getWord();
-	int left = b.getWord();
-	int top = b.getWord();
-	int width = b.getWord();
-	int height = b.getWord();
+	int codec = b.readUint16LE();
+	int left = b.readUint16LE();
+	int top = b.readUint16LE();
+	int width = b.readUint16LE();
+	int height = b.readUint16LE();
 
-	if ((height == 242) && (width == 384)) {
-		if (_specialBuffer == 0)
-			_specialBuffer = (byte *)malloc(242 * 384);
-		_dst = _specialBuffer;
-	} else if ((height > _vm->_screenHeight) || (width > _vm->_screenWidth))
-		return;
-	// FT Insane uses smaller frames to draw overlays with moving objects
-	// Other .san files do have them as well but their purpose in unknown
-	// and often it causes memory overdraw. So just skip those frames
-	else if (!_insanity && ((height != _vm->_screenHeight) || (width != _vm->_screenWidth)))
-		return;
+	b.readUint16LE();
+	b.readUint16LE();
 
-	if (!_alreadyInit) {
-		_codec37.init(width, height);
-		_codec47.init(width, height);
-		_alreadyInit = true;
-	}
-
-	if ((height == 242) && (width == 384)) {
-		_width = width;
-		_height = height;
-	} else {
-		_width = _vm->_screenWidth;
-		_height = _vm->_screenHeight;
-	}
-
-	b.getWord();
-	b.getWord();
-
-	int32 chunk_size = b.getSize() - 14;
+	int32 chunk_size = b.size() - 14;
 	byte *chunk_buffer = (byte *)malloc(chunk_size);
 	assert(chunk_buffer);
 	b.read(chunk_buffer, chunk_size);
 
-	switch (codec) {
-	case 1:
-	case 3:
-		smush_decode_codec1(_dst, chunk_buffer, left, top, width, height, _vm->_screenWidth);
-		break;
-	case 37:
-		_codec37.decode(_dst, chunk_buffer);
-		break;
-	case 47:
-		_codec47.decode(_dst, chunk_buffer);
-		break;
-	default:
-		error("Invalid codec for frame object : %d", (int)codec);
-	}
-
-	if (_storeFrame) {
-		if (_frameBuffer == NULL) {
-			_frameBuffer = (byte *)malloc(_width * _height);
-		}
-		memcpy(_frameBuffer, _dst, _width * _height);
-		_storeFrame = false;
-	}
+	decodeFrameObject(codec, chunk_buffer, left, top, width, height);
 
 	free(chunk_buffer);
 }
 
 void SmushPlayer::handleFrame(Chunk &b) {
-	checkBlock(b, TYPE_FRME);
+	checkBlock(b, MKID_BE('FRME'));
 	debugC(DEBUG_SMUSH, "SmushPlayer::handleFrame(%d)", _frame);
 	_skipNext = false;
 
-	uint32 start_time, end_time;
-	start_time = _vm->_system->getMillis();
-
 	if (_insanity) {
 		_vm->_insane->procPreRendering();
 	}
 
-	while (!b.eof()) {
+	while (!b.eos()) {
 		Chunk *sub = b.subBlock();
 		switch (sub->getType()) {
-		case TYPE_NPAL:
+		case MKID_BE('NPAL'):
 			handleNewPalette(*sub);
 			break;
-		case TYPE_FOBJ:
+		case MKID_BE('FOBJ'):
 			handleFrameObject(*sub);
 			break;
 #ifdef USE_ZLIB
-		case TYPE_ZFOB:
+		case MKID_BE('ZFOB'):
 			handleZlibFrameObject(*sub);
 			break;
 #endif
-		case TYPE_PSAD:
+		case MKID_BE('PSAD'):
 			if (!_compressedFileMode)
 				handleSoundFrame(*sub);
 			break;
-		case TYPE_TRES:
+		case MKID_BE('TRES'):
 			handleTextResource(*sub);
 			break;
-		case TYPE_XPAL:
+		case MKID_BE('XPAL'):
 			handleDeltaPalette(*sub);
 			break;
-		case TYPE_IACT:
-			// FIXME: check parameters
-			if (_insanity)
-				_vm->_insane->procIACT(_dst, 0, 0, 0, *sub, 0, 0);
-			else {
-				if (!_compressedFileMode)
-					handleIACT(*sub);
-			}
+		case MKID_BE('IACT'):
+			handleIACT(*sub);
 			break;
-		case TYPE_STOR:
+		case MKID_BE('STOR'):
 			handleStore(*sub);
 			break;
-		case TYPE_FTCH:
+		case MKID_BE('FTCH'):
 			handleFetch(*sub);
 			break;
-		case TYPE_SKIP:
-			if (_insanity)
-				_vm->_insane->procSKIP(*sub);
-			else
-				handleSkip(*sub);
+		case MKID_BE('SKIP'):
+			_vm->_insane->procSKIP(*sub);
 			break;
-		case TYPE_TEXT:
+		case MKID_BE('TEXT'):
 			handleTextResource(*sub);
 			break;
 		default:
-			error("Unknown frame subChunk found : %s, %d", Chunk::ChunkString(sub->getType()), sub->getSize());
+			error("Unknown frame subChunk found : %s, %d", tag2str(sub->getType()), sub->size());
 		}
 
 		b.reseek();
-		if (sub->getSize() & 1)
-			b.seek(1);
+		if (sub->size() & 1)
+			b.skip(1);
 
 		delete sub;
 	}
@@ -1013,8 +925,6 @@ void SmushPlayer::handleFrame(Chunk &b) {
 		_vm->_insane->procPostRendering(_dst, 0, 0, 0, _frame, _nbframes-1);
 	}
 
-	end_time = _vm->_system->getMillis();
-
 	if (_width != 0 && _height != 0) {
 #ifdef _WIN32_WCE
 		if (!_inTimer || _inTimerCount == _inTimerCountRedraw) {
@@ -1027,18 +937,16 @@ void SmushPlayer::handleFrame(Chunk &b) {
 	}
 	_smixer->handleFrame();
 
-	debugC(DEBUG_SMUSH, "Smush stats: FRME( %03d ), Limit(%d)", end_time - start_time, _speed);
-
 	_frame++;
 }
 
 void SmushPlayer::handleAnimHeader(Chunk &b) {
-	checkBlock(b, TYPE_AHDR, 0x300 + 6);
+	checkBlock(b, MKID_BE('AHDR'), 0x300 + 6);
 	debugC(DEBUG_SMUSH, "SmushPlayer::handleAnimHeader()");
 
-	_version = b.getWord();
-	_nbframes = b.getWord();
-	b.getWord();
+	/* _version = */ b.readUint16LE();
+	_nbframes = b.readUint16LE();
+	b.readUint16LE();
 
 	if (_skipPalette)
 		return;
@@ -1048,55 +956,56 @@ void SmushPlayer::handleAnimHeader(Chunk &b) {
 }
 
 void SmushPlayer::setupAnim(const char *file) {
-	int i;
-	char file_font[11];
-
 	if (_insanity) {
 		if (!((_vm->_game.features & GF_DEMO) && (_vm->_game.platform == Common::kPlatformPC)))
 			readString("mineroad.trs");
 	} else
 		readString(file);
+}
+
+SmushFont *SmushPlayer::getFont(int font) {
+	char file_font[11];
+
+	if (_sf[font])
+		return _sf[font];
 
 	if (_vm->_game.id == GID_FT) {
 		if (!((_vm->_game.features & GF_DEMO) && (_vm->_game.platform == Common::kPlatformPC))) {
-			_sf[0] = new SmushFont(_vm, true, false);
-			_sf[1] = new SmushFont(_vm, true, false);
-			_sf[2] = new SmushFont(_vm, true, false);
-			_sf[3] = new SmushFont(_vm, true, false);
-			_sf[0]->loadFont("scummfnt.nut");
-			_sf[1]->loadFont("techfnt.nut");
-			_sf[2]->loadFont("titlfnt.nut");
-			_sf[3]->loadFont("specfnt.nut");
+			const char *ft_fonts[] = {
+				"scummfnt.nut",
+				"techfnt.nut",
+				"titlfnt.nut",
+				"specfnt.nut"
+			};
+
+			assert(font >= 0 && font < ARRAYSIZE(ft_fonts));
+
+			_sf[font] = new SmushFont(_vm, ft_fonts[font], true, false);
 		}
 	} else if (_vm->_game.id == GID_DIG) {
 		if (!(_vm->_game.features & GF_DEMO)) {
-			for (i = 0; i < 4; i++) {
-				sprintf(file_font, "font%d.nut", i);
-				_sf[i] = new SmushFont(_vm, i != 0, false);
-				_sf[i]->loadFont(file_font);
-			}
+			assert(font >= 0 && font < 4);
+
+			sprintf(file_font, "font%d.nut", font);
+			_sf[font] = new SmushFont(_vm, file_font, font != 0, false);
 		}
 	} else if (_vm->_game.id == GID_CMI) {
-		for (i = 0; i < 5; i++) {
-			if ((_vm->_game.features & GF_DEMO) && (i == 4))
-				break;
-			sprintf(file_font, "font%d.nut", i);
-			_sf[i] = new SmushFont(_vm, false, true);
-			_sf[i]->loadFont(file_font);
-		}
+		int numFonts = (_vm->_game.features & GF_DEMO) ? 4 : 5;
+		assert(font >= 0 && font < numFonts);
+
+		sprintf(file_font, "font%d.nut", font);
+		_sf[font] = new SmushFont(_vm, file_font, false, true);
 	} else {
-		error("SmushPlayer::setupAnim() Unknown font setup for game");
+		error("SmushPlayer::getFont() Unknown font setup for game");
 	}
+
+	assert(_sf[font]);
+	return _sf[font];
 }
 
 void SmushPlayer::parseNextFrame() {
-	Common::StackLock lock(_mutex);
-
 	Chunk *sub;
 
-	if (_vm->_smushPaused)
-		return;
-
 	if (_seekPos >= 0) {
 		if (_smixer)
 			_smixer->stop();
@@ -1109,7 +1018,7 @@ void SmushPlayer::parseNextFrame() {
 				assert(_seekPos > 8);
 				// In this case we need to get palette and number of frames
 				sub = _base->subBlock();
-				checkBlock(*sub, TYPE_AHDR);
+				checkBlock(*sub, MKID_BE('AHDR'));
 				handleAnimHeader(*sub);
 				delete sub;
 
@@ -1125,29 +1034,32 @@ void SmushPlayer::parseNextFrame() {
 			_skipPalette = true;
 		}
 
-		_base->seek(_seekPos, FileChunk::seek_start);
+		_base->seek(_seekPos, SEEK_SET);
 		_frame = _seekFrame;
+		_startFrame = _frame;
+		_startTime = _vm->_system->getMillis();
 
 		_seekPos = -1;
 	}
 
 	assert(_base);
-	if (_base->eof()) {
+	if (_base->eos()) {
 		_vm->_smushVideoShouldFinish = true;
+		_endOfFile = true;
 		return;
 	}
 
 	sub = _base->subBlock();
 
 	switch (sub->getType()) {
-	case TYPE_AHDR: // FT INSANE may seek file to the beginning
+	case MKID_BE('AHDR'): // FT INSANE may seek file to the beginning
 		handleAnimHeader(*sub);
 		break;
-	case TYPE_FRME:
+	case MKID_BE('FRME'):
 		handleFrame(*sub);
 		break;
 	default:
-		error("Unknown Chunk found at %x: %x, %d", _base->tell(), sub->getType(), sub->getSize());
+		error("Unknown Chunk found at %x: %x, %d", _base->pos(), sub->getType(), sub->size());
 	}
 	delete sub;
 
@@ -1248,18 +1160,15 @@ void SmushPlayer::insanity(bool flag) {
 }
 
 void SmushPlayer::seekSan(const char *file, int32 pos, int32 contFrame) {
-	Common::StackLock lock(_mutex);
-
 	_seekFile = file ? file : "";
 	_seekPos = pos;
 	_seekFrame = contFrame;
+	_pauseTime = 0;
 }
 
 void SmushPlayer::tryCmpFile(const char *filename) {
-	if (_compressedFile.isOpen()) {
-		_vm->_mixer->stopHandle(_compressedFileSoundHandle);
-		_compressedFile.close();
-	}
+	_vm->_mixer->stopHandle(_compressedFileSoundHandle);
+
 	_compressedFileMode = false;
 	const char *i = strrchr(filename, '.');
 	if (i == NULL) {
@@ -1268,32 +1177,44 @@ void SmushPlayer::tryCmpFile(const char *filename) {
 #if defined(USE_MAD) || defined(USE_VORBIS)
 	char fname[260];
 #endif
-#ifdef USE_MAD
-	memcpy(fname, filename, i - filename);
-	strcpy(fname + (i - filename), ".mp3");
-	_compressedFile.open(fname);
-	if (_compressedFile.isOpen()) {
-		int size = _compressedFile.size();
-		_compressedFileMode = true;
-		_vm->_mixer->playInputStream(Audio::Mixer::kSFXSoundType, &_compressedFileSoundHandle, Audio::makeMP3Stream(&_compressedFile, size));
-		return;
-	}
-#endif
+	Common::File *file = new Common::File();
 #ifdef USE_VORBIS
 	memcpy(fname, filename, i - filename);
 	strcpy(fname + (i - filename), ".ogg");
-	_compressedFile.open(fname);
-	if (_compressedFile.isOpen()) {
-		int size = _compressedFile.size();
+	if (file->open(fname)) {
 		_compressedFileMode = true;
-		_vm->_mixer->playInputStream(Audio::Mixer::kSFXSoundType, &_compressedFileSoundHandle, Audio::makeVorbisStream(&_compressedFile, size));
+		_vm->_mixer->playInputStream(Audio::Mixer::kSFXSoundType, &_compressedFileSoundHandle, Audio::makeVorbisStream(file, true, 0, 0));
 		return;
 	}
 #endif
+#ifdef USE_MAD
+	memcpy(fname, filename, i - filename);
+	strcpy(fname + (i - filename), ".mp3");
+	if (file->open(fname)) {
+		_compressedFileMode = true;
+		_vm->_mixer->playInputStream(Audio::Mixer::kSFXSoundType, &_compressedFileSoundHandle, Audio::makeMP3Stream(file, true, 0, 0));
+		return;
+	}
+#endif
+	delete file;
 }
 
-void SmushPlayer::play(const char *filename, int32 offset, int32 startFrame) {
+void SmushPlayer::pause() {
+	if (!_paused) {
+		_paused = true;
+		_pauseStartTime = _vm->_system->getMillis();
+	}
+}
 
+void SmushPlayer::unpause() {
+	if (_paused) {
+		_paused = false;
+		_pauseTime += (_vm->_system->getMillis() - _pauseStartTime);
+		_pauseStartTime = 0;
+	}
+}
+
+void SmushPlayer::play(const char *filename, int32 speed, int32 offset, int32 startFrame) {
 	// Verify the specified file exists
 	ScummFile f;
 	_vm->openFile(f, filename);
@@ -1318,15 +1239,53 @@ void SmushPlayer::play(const char *filename, int32 offset, int32 startFrame) {
 	_base = 0;
 
 	setupAnim(filename);
-	init();
+	init(speed);
+
+	_startTime = _vm->_system->getMillis();
+	_startFrame = startFrame;
+	_frame = startFrame;
+
+	_pauseTime = 0;
+
+	int skipped = 0;
 
 	for (;;) {
+		uint32 now, elapsed;
+		bool skipFrame = false;
+
+		if (_insanity) {
+			// Seeking makes a mess of trying to sync the audio to
+			// the sound. Synt to time instead.
+			now = _vm->_system->getMillis() - _pauseTime;
+			elapsed = now - _startTime;
+		} else if (_vm->_mixer->isSoundHandleActive(_compressedFileSoundHandle)) {
+			// Compressed SMUSH files.
+			elapsed = _vm->_mixer->getSoundElapsedTime(_compressedFileSoundHandle);
+		} else if (_vm->_mixer->isSoundHandleActive(_IACTchannel)) {
+			// Curse of Monkey Island SMUSH files.
+			elapsed = _vm->_mixer->getSoundElapsedTime(_IACTchannel);
+		} else {
+			// For other SMUSH files, we don't necessarily have any
+			// one channel to sync against, so we have to use
+			// elapsed real time.
+			now = _vm->_system->getMillis() - _pauseTime;
+			elapsed = now - _startTime;
+		}
+
+		if (elapsed >= ((_frame - _startFrame) * 1000) / _speed) {
+			if (elapsed >= ((_frame + 1) * 1000) / _speed)
+				skipFrame = true;
+			else
+				skipFrame = false;
+			timerCallback();
+		}
+
 		if (_warpNeeded) {
 			_vm->_system->warpMouse(_warpX, _warpY);
 			_warpNeeded = false;
 		}
 		_vm->parseEvents();
-		_vm->processKbd(true);
+		_vm->processInput();
 		if (_palDirtyMax >= _palDirtyMin) {
 			byte palette_colors[1024];
 			byte *p = palette_colors;
@@ -1344,40 +1303,40 @@ void SmushPlayer::play(const char *filename, int32 offset, int32 startFrame) {
 
 			_palDirtyMax = -1;
 			_palDirtyMin = 256;
+			skipFrame = false;
 		}
+		if (skipFrame) {
+			if (++skipped > 10) {
+				skipFrame = false;
+				skipped = 0;
+			}
+		} else
+			skipped = 0;
 		if (_updateNeeded) {
-			uint32 end_time, start_time;
-			int w = _width, h = _height;
+			if (!skipFrame) {
+				// Workaround for bug #1386333: "FT DEMO: assertion triggered
+				// when playing movie". Some frames there are 384 x 224
+				int w = MIN(_width, _vm->_screenWidth);
+				int h = MIN(_height, _vm->_screenHeight);
 
-			start_time = _vm->_system->getMillis();
-
-			// Workaround for bug #1386333: "FT DEMO: assertion triggered 
-			// when playing movie". Some frames there are 384 x 224
-			if (w > _vm->_screenWidth)
-				w = _vm->_screenWidth;
-
-			if (h > _vm->_screenHeight)
-				h = _vm->_screenHeight;
-
-			_vm->_system->copyRectToScreen(_dst, _width, 0, 0, w, h);
-			_vm->_system->updateScreen();
-			_updateNeeded = false;
+				_vm->_system->copyRectToScreen(_dst, _width, 0, 0, w, h);
+				_vm->_system->updateScreen();
+				_updateNeeded = false;
+			}
 #ifdef _WIN32_WCE
 			_inTimer = false;
 			_inTimerCount = 0;
 #endif
-
-			end_time = _vm->_system->getMillis();
-
-			debugC(DEBUG_SMUSH, "Smush stats: BackendUpdateScreen( %03d )", end_time - start_time);
-
 		}
-		if (_vm->_quit || _vm->_saveLoadFlag) {
+		if (_endOfFile)
+			break;
+		if (_vm->_quit || _vm->_saveLoadFlag || _vm->_smushVideoShouldFinish) {
 			_smixer->stop();
+			_vm->_mixer->stopHandle(_compressedFileSoundHandle);
+			_vm->_mixer->stopHandle(_IACTchannel);
+			_IACTpos = 0;
 			break;
 		}
-		if (_vm->_smushVideoShouldFinish || _vm->_quit || _vm->_saveLoadFlag)
-			break;
 		_vm->_system->delayMillis(10);
 	}
 
@@ -1388,4 +1347,3 @@ void SmushPlayer::play(const char *filename, int32 offset, int32 startFrame) {
 }
 
 } // End of namespace Scumm
-
