@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2002-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,65 +18,62 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/scumm/smush/smush_player.h $
- * $Id: smush_player.h 23966 2006-09-21 17:49:42Z sev $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/scumm/smush/smush_player.h $
+ * $Id: smush_player.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
-#if !defined(SMUSH_PLAYER_H) && !defined(DISABLE_SCUMM_7_8)
-#define SMUSH_PLAYER_H
+#if !defined(SCUMM_SMUSH_PLAYER_H) && !defined(DISABLE_SCUMM_7_8)
+#define SCUMM_SMUSH_PLAYER_H
 
 #include "common/util.h"
 #include "scumm/smush/chunk.h"
-#include "scumm/smush/codec37.h"
-#include "scumm/smush/codec47.h"
 #include "scumm/sound.h"
 
 namespace Scumm {
 
-class ScummEngine_v6;
+class ScummEngine_v7;
 class SmushFont;
 class SmushMixer;
 class StringResource;
+class Codec37Decoder;
+class Codec47Decoder;
 
 class SmushPlayer {
 	friend class Insane;
 private:
-	ScummEngine_v6 *_vm;
-	int _version;
+	ScummEngine_v7 *_vm;
 	int32 _nbframes;
 	SmushMixer *_smixer;
 	int16 _deltaPal[0x300];
 	byte _pal[0x300];
+	SmushFont *_sf[5];
 	StringResource *_strings;
-	Codec37Decoder _codec37;
-	Codec47Decoder _codec47;
+	Codec37Decoder *_codec37;
+	Codec47Decoder *_codec47;
 	FileChunk *_base;
 	byte *_frameBuffer;
 	byte *_specialBuffer;
 
 	Common::String _seekFile;
+	uint32 _startFrame;
+	uint32 _startTime;
 	int32 _seekPos;
-	int32 _seekFrame;
+	uint32 _seekFrame;
 
 	bool _skipNext;
-	bool _skips[37];
-	int32 _frame;
+	uint32 _frame;
 
 	Audio::SoundHandle _IACTchannel;
 	Audio::AppendableAudioStream *_IACTstream;
 
 	Audio::SoundHandle _compressedFileSoundHandle;
 	bool _compressedFileMode;
-	Common::File _compressedFile;
 	byte _IACToutput[4096];
 	int32 _IACTpos;
 	bool _storeFrame;
-	int _soundFrequency;
-	bool _alreadyInit;
-	bool _initDone;
 	int _speed;
-	bool _outputSound;
+	bool _endOfFile;
 
 	byte *_dst;
 	bool _updateNeeded;
@@ -89,23 +89,25 @@ private:
 	int16 _inTimerCount;
 	int16 _inTimerCountRedraw;
 #endif
-#ifdef __SYMBIAN32__
-	bool _closeOnTextTick;
-#endif
-	Common::Mutex _mutex;
 
 public:
-	SmushPlayer(ScummEngine_v6 *scumm, int speed);
+	SmushPlayer(ScummEngine_v7 *scumm);
 	~SmushPlayer();
 
-	void play(const char *filename, int32 offset = 0, int32 startFrame = 0);
+	void pause();
+	void unpause();
+
+	void play(const char *filename, int32 speed, int32 offset = 0, int32 startFrame = 0);
+	void release();
 	void warpMouse(int x, int y, int buttons);
 
 protected:
-	SmushFont *_sf[5];
 	int _width, _height;
 
 	int _origPitch, _origNumStrips;
+	bool _paused;
+	uint32 _pauseStartTime;
+	uint32 _pauseTime;
 
 	void insanity(bool);
 	void setPalette(const byte *palette);
@@ -115,14 +117,15 @@ protected:
 	const char *getString(int id);
 
 private:
+	SmushFont *getFont(int font);
 	void parseNextFrame();
-	void init();
-	void release();
+	void init(int32 spped);
 	void setupAnim(const char *file);
 	void updateScreen();
 	void tryCmpFile(const char *filename);
 
 	bool readString(const char *file);
+	void decodeFrameObject(int codec, const uint8 *src, int left, int top, int width, int height);
 	void checkBlock(const Chunk &, Chunk::type, uint32 = 0);
 	void handleAnimHeader(Chunk &);
 	void handleFrame(Chunk &);
@@ -133,7 +136,6 @@ private:
 	void handleFrameObject(Chunk &);
 	void handleSoundBuffer(int32, int32, int32, int32, int32, int32, Chunk &, int32);
 	void handleSoundFrame(Chunk &);
-	void handleSkip(Chunk &);
 	void handleStore(Chunk &);
 	void handleFetch(Chunk &);
 	void handleIACT(Chunk &);
@@ -141,7 +143,7 @@ private:
 	void handleDeltaPalette(Chunk &);
 	void readPalette(byte *, Chunk &);
 
-	static void timerCallback(void *ptr);
+	void timerCallback();
 };
 
 } // End of namespace Scumm
