@@ -1,6 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2001  Ludvig Strigeus
- * Copyright (C) 2001-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -16,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/scumm/sound.cpp $
- * $Id: sound.cpp 23205 2006-06-20 21:36:20Z eriktorbjorn $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/scumm/sound.cpp $
+ * $Id: sound.cpp 27254 2007-06-09 18:09:26Z fingolfin $
  *
  */
 
@@ -56,9 +58,10 @@ struct MP3OffsetTable {					/* Compressed Sound (.SO3) */
 };
 
 
-Sound::Sound(ScummEngine *parent)
+Sound::Sound(ScummEngine *parent, Audio::Mixer *mixer)
 	:
 	_vm(parent),
+	_mixer(mixer),
 	_soundQuePos(0),
 	_soundQue2Pos(0),
 	_sfxFile(0),
@@ -74,7 +77,6 @@ Sound::Sound(ScummEngine *parent)
 	_mouthSyncMode(false),
 	_endOfMouthSync(false),
 	_curSoundPos(0),
-	_overrideFreq(0),
 	_currentCDSound(0),
 	_currentMusic(0),
 	_soundsPaused(false),
@@ -111,7 +113,7 @@ void Sound::addSoundToQueue2(int sound, int heOffset, int heChannel, int heFlags
 }
 
 void Sound::processSound() {
-	if (_vm->_game.features & GF_DIGI_IMUSE) {
+	if (_vm->_game.version >= 7) {
 		processSfxQueues();
 	} else if (_vm->_game.heversion >= 60) {
 		processSoundQueues();
@@ -150,7 +152,7 @@ void Sound::processSoundQueues() {
 						data[1], data[2], data[3], data[4], data[5], data[6], data[7]);
 
 			if (_vm->_imuse) {
-				_vm->VAR(_vm->VAR_SOUNDRESULT) = (short)_vm->_imuse->doCommand (num, data);
+				_vm->VAR(_vm->VAR_SOUNDRESULT) = (short)_vm->_imuse->doCommand(num, data);
 			}
 		}
 	}
@@ -165,6 +167,10 @@ void Sound::playSound(int soundID) {
 	int rate;
 	byte flags = Audio::Mixer::FLAG_UNSIGNED | Audio::Mixer::FLAG_AUTOFREE;
 
+	// FIXME: Sound resources are currently missing
+	if (_vm->_game.id == GID_LOOM && _vm->_game.platform == Common::kPlatformPCEngine)
+		return;
+
 	debugC(DEBUG_SOUND, "playSound #%d (room %d)", soundID,
 		_vm->getResourceRoomNr(rtSound, soundID));
 
@@ -188,7 +194,7 @@ void Sound::playSound(int soundID) {
 		// Allocate a sound buffer, copy the data into it, and play
 		sound = (char *)malloc(size);
 		memcpy(sound, ptr, size);
-		_vm->_mixer->playRaw(NULL, sound, size, rate, flags, soundID);
+		_mixer->playRaw(Audio::Mixer::kSFXSoundType, NULL, sound, size, rate, flags, soundID);
 	}
 	// WORKAROUND bug # 1311447
 	else if (READ_BE_UINT32(ptr) == 0x460e200d) {
@@ -210,7 +216,7 @@ void Sound::playSound(int soundID) {
 		// Allocate a sound buffer, copy the data into it, and play
 		sound = (char *)malloc(size);
 		memcpy(sound, ptr, size);
-		_vm->_mixer->playRaw(NULL, sound, size, rate, flags, soundID);
+		_mixer->playRaw(Audio::Mixer::kSFXSoundType, NULL, sound, size, rate, flags, soundID);
 	}
 	// Support for sampled sound effects in Monkey Island 1 and 2
 	else if (READ_BE_UINT32(ptr) == MKID_BE('SBL ')) {
@@ -281,7 +287,7 @@ void Sound::playSound(int soundID) {
 		// Allocate a sound buffer, copy the data into it, and play
 		sound = (char *)malloc(size);
 		memcpy(sound, ptr + 6, size);
-		_vm->_mixer->playRaw(NULL, sound, size, rate, flags, soundID);
+		_mixer->playRaw(Audio::Mixer::kSFXSoundType, NULL, sound, size, rate, flags, soundID);
 	}
 	else if ((_vm->_game.platform == Common::kPlatformFMTowns && _vm->_game.version == 3) || READ_BE_UINT32(ptr) == MKID_BE('SOUN') || READ_BE_UINT32(ptr) == MKID_BE('TOWS')) {
 
@@ -334,7 +340,7 @@ void Sound::playSound(int soundID) {
 				if (loopEnd > 0)
 					flags |= Audio::Mixer::FLAG_LOOP;
 
-				_vm->_mixer->playRaw(NULL, sound, waveSize, rate, flags, soundID, 255, 0, loopStart, loopEnd);
+				_mixer->playRaw(Audio::Mixer::kSFXSoundType, NULL, sound, waveSize, rate, flags, soundID, 255, 0, loopStart, loopEnd);
 			}
 			break;
 		case 1:
@@ -395,7 +401,7 @@ void Sound::playSound(int soundID) {
 		sound = (char *)malloc(size);
 		int vol = ptr[24] * 4;
 		memcpy(sound, ptr + READ_BE_UINT16(ptr + 8), size);
-		_vm->_mixer->playRaw(NULL, sound, size, rate, Audio::Mixer::FLAG_AUTOFREE, soundID, vol, 0);
+		_mixer->playRaw(Audio::Mixer::kSFXSoundType, NULL, sound, size, rate, Audio::Mixer::FLAG_AUTOFREE, soundID, vol, 0);
 	}
 	else {
 
@@ -459,7 +465,7 @@ void Sound::processSfxQueues() {
 		} else if (_vm->_game.heversion >= 60) {
 			finished = !isSoundRunning(1);
 		} else {
-			finished = !_vm->_mixer->isSoundHandleActive(_talkChannelHandle);
+			finished = !_mixer->isSoundHandleActive(_talkChannelHandle);
 		}
 
 		if ((uint) act < 0x80 && ((_vm->_game.version == 8) || (_vm->_game.version <= 7 && !_vm->_string[0].no_talk_anim))) {
@@ -557,7 +563,7 @@ void Sound::startTalkSound(uint32 offset, uint32 b, int mode, Audio::SoundHandle
 
 		if (mode == 1 && (_vm->_game.id == GID_TENTACLE || _vm->_game.id == GID_SAMNMAX)) {
 			id = 777777 + _talk_sound_channel;
-			_vm->_mixer->stopID(id);
+			_mixer->stopID(id);
 		}
 
 		if (b > 8) {
@@ -599,7 +605,7 @@ void Sound::startTalkSound(uint32 offset, uint32 b, int mode, Audio::SoundHandle
 		_mouthSyncMode = true;
 	}
 
-	if (!_soundsPaused && _vm->_mixer->isReady()) {
+	if (!_soundsPaused && _mixer->isReady()) {
 		Audio::AudioStream *input = NULL;
 
 		switch (_soundMode) {
@@ -637,7 +643,7 @@ void Sound::startTalkSound(uint32 offset, uint32 b, int mode, Audio::SoundHandle
 			_vm->_imuseDigital->startVoice(kTalkSoundID, input);
 #endif
 		} else {
-			_vm->_mixer->playInputStream(Audio::Mixer::kSFXSoundType, handle, input, id);
+			_mixer->playInputStream(Audio::Mixer::kSpeechSoundType, handle, input, id);
 		}
 	}
 }
@@ -651,7 +657,7 @@ void Sound::stopTalkSound() {
 		} else if (_vm->_game.heversion >= 60) {
 			stopSound(1);
 		} else {
-			_vm->_mixer->stopHandle(_talkChannelHandle);
+			_mixer->stopHandle(_talkChannelHandle);
 		}
 		_sfxMode &= ~2;
 	}
@@ -683,13 +689,13 @@ int Sound::isSoundRunning(int sound) const {
 	if (sound == _currentCDSound)
 		return pollCD();
 
-	if (_vm->_mixer->isSoundIDActive(sound))
+	if (_mixer->isSoundIDActive(sound))
 		return 1;
 
 	if (isSoundInQueue(sound))
 		return 1;
 
-	if (sound > _vm->_numSounds || !_vm->res.isResourceLoaded(rtSound, sound))
+	if (sound > _vm->_numSounds || !_vm->_res->isResourceLoaded(rtSound, sound))
 		return 0;
 
 	if (_vm->_musicEngine)
@@ -722,13 +728,13 @@ bool Sound::isSoundInUse(int sound) const {
 	if (isSoundInQueue(sound))
 		return true;
 
-	if (!_vm->res.isResourceLoaded(rtSound, sound))
+	if (!_vm->_res->isResourceLoaded(rtSound, sound))
 		return false;
 
 	if (_vm->_imuse)
 		return _vm->_imuse->get_sound_active(sound);
 
-	if (_vm->_mixer->isSoundIDActive(sound))
+	if (_mixer->isSoundIDActive(sound))
 		return 1;
 
 	return false;
@@ -765,8 +771,8 @@ void Sound::stopSound(int sound) {
 		stopCDTimer();
 	}
 
-	if (!(_vm->_game.features & GF_DIGI_IMUSE))
-		_vm->_mixer->stopID(sound);
+	if (_vm->_game.version < 7)
+		_mixer->stopID(sound);
 
 	if (_vm->_musicEngine)
 		_vm->_musicEngine->stopSound(sound);
@@ -798,7 +804,7 @@ void Sound::stopAllSounds() {
 
 	// Stop all SFX
 	if (!_vm->_imuseDigital) {
-		_vm->_mixer->stopAll();
+		_mixer->stopAll();
 	}
 }
 
@@ -875,7 +881,7 @@ void Sound::pauseSounds(bool pause) {
 	}
 #endif
 
-	_vm->_mixer->pauseAll(pause);
+	_mixer->pauseAll(pause);
 
 	if ((_vm->_game.features & GF_AUDIOTRACKS) && _vm->VAR(_vm->VAR_MUSIC_TIMER) > 0) {
 		if (pause)
@@ -905,7 +911,6 @@ BaseScummFile *Sound::openSfxFile() {
 		{ 0, kVOCMode }
 	};
 
-	char buf[256];
 	ScummFile *file = new ScummFile();
 	_offsetTable = NULL;
 
@@ -914,6 +919,7 @@ BaseScummFile *Sound::openSfxFile() {
 	 * same directory */
 
 	Common::String basename[2];
+	Common::String tmp;
 	
 	const char *ptr = strchr(_vm->_filenamePattern.pattern, '.');
 	if (ptr) {
@@ -924,27 +930,26 @@ BaseScummFile *Sound::openSfxFile() {
 	}
 	basename[1] = "monster.";
 
-	for (uint j = 0; j < 2 && !file->isOpen(); ++j) {
-		for (int i = 0; extensions[i].ext; ++i) {
-			Common::String tmp(basename[j]);
-			tmp += extensions[i].ext;
-			if (_vm->openFile(*file, tmp)) {
-				_soundMode = extensions[i].mode;
-				break;
-			}
-		}
-	}
-
-	if (!file->isOpen()) {
+	if (_vm->_game.heversion >= 60) {
 		if ((_vm->_game.heversion <= 61 && _vm->_game.platform == Common::kPlatformMacintosh) || (_vm->_game.heversion >= 70)) {
-			strncpy(buf, _vm->generateFilename(-2).c_str(), sizeof(buf));
+			tmp = _vm->generateFilename(-2);
 		} else {
-			sprintf(buf, "%s.tlk", _vm->_filenamePattern.pattern);
+			tmp = basename[0] + "tlk";
 		}
 
-		if (file->open(buf) && _vm->_game.heversion <= 73)
+		if (file->open(tmp) && _vm->_game.heversion <= 73)
 			file->setEnc(0x69);
 		_soundMode = kVOCMode;
+	} else {
+		for (uint j = 0; j < 2 && !file->isOpen(); ++j) {
+			for (int i = 0; extensions[i].ext; ++i) {
+				tmp = basename[j] + extensions[i].ext;
+				if (_vm->openFile(*file, tmp)) {
+					_soundMode = extensions[i].mode;
+					break;
+				}
+			}
+		}
 	}
 
 	if (_soundMode != kVOCMode) {
@@ -984,7 +989,7 @@ BaseScummFile *Sound::openSfxFile() {
 }
 
 bool Sound::isSfxFinished() const {
-	return !_vm->_mixer->hasActiveChannelOfType(Audio::Mixer::kSFXSoundType);
+	return !_mixer->hasActiveChannelOfType(Audio::Mixer::kSFXSoundType);
 }
 
 // We use a real timer in an attempt to get better sync with CD tracks. This is
@@ -1001,20 +1006,18 @@ static void cd_timer_handler(void *refCon) {
 }
 
 void Sound::startCDTimer() {
-	int timer_interval;
-
-	// The timer interval has been tuned for Loom CD and the Monkey 1
-	// intro. I have to use 100 for Loom, or there will be a nasty stutter
-	// when Chaos first appears, and I have to use 101 for Monkey 1 or the
-	// intro music will be cut short.
-
-	if (_vm->_game.id == GID_LOOM && _vm->_game.version == 4)
-		timer_interval = 100;
-	else
-		timer_interval = 101;
+	// This timer interval is based on two scenes: The Monkey Island 1
+	// intro, and the scene in Loom CD where Chaos appears. In both cases
+	// the game plays the scene as two separate sounds, even though both
+	// halves are right next to each other in the CD track. Probably so
+	// that you can hit Escape to skip the first half.
+	//
+	// Make it too low, and the Monkey Island theme will be cut short. Make
+	// it too high, and there will be a nasty "hiccup" just as Chaos
+	// appears.
 
 	_vm->_timer->removeTimerProc(&cd_timer_handler);
-	_vm->_timer->installTimerProc(&cd_timer_handler, 1000 * timer_interval, _vm);
+	_vm->_timer->installTimerProc(&cd_timer_handler, 100700, _vm);
 }
 
 void Sound::stopCDTimer() {
@@ -1062,6 +1065,9 @@ void Sound::saveLoadWithSerializer(Serializer *ser) {
 #pragma mark --- Sound resource handling ---
 #pragma mark -
 
+static void convertMac0Resource(ResourceManager *res, int type, int idx, byte *src_ptr, int size);
+
+
 /*
  * TODO: The way we handle sound/music resources really is one huge hack.
  * We probably should reconsider how we do this, and maybe come up with a
@@ -1069,7 +1075,6 @@ void Sound::saveLoadWithSerializer(Serializer *ser) {
  * could stand a thorough cleanup!
  */
 
-
 int ScummEngine::readSoundResource(int type, int idx) {
 	uint32 pos, total_size, size, tag, basetag, max_total_size;
 	int pri, best_pri;
@@ -1092,7 +1097,7 @@ int ScummEngine::readSoundResource(int type, int idx) {
 	case MKID_BE('iMUS'):
 		if (_musicType != MDT_PCSPK) {
 			_fileHandle->seek(-8, SEEK_CUR);
-			_fileHandle->read(res.createResource(type, idx, total_size + 8), total_size + 8);
+			_fileHandle->read(_res->createResource(type, idx, total_size + 8), total_size + 8);
 			return 1;
 		}
 		break;
@@ -1155,7 +1160,7 @@ int ScummEngine::readSoundResource(int type, int idx) {
 
 		if (best_pri != -1) {
 			_fileHandle->seek(best_offs - 8, SEEK_SET);
-			ptr = res.createResource(type, idx, best_size);
+			ptr = _res->createResource(type, idx, best_size);
 			_fileHandle->read(ptr, best_size);
 			//dumpResource("sound-", idx, ptr);
 			return 1;
@@ -1167,7 +1172,7 @@ int ScummEngine::readSoundResource(int type, int idx) {
 		ptr = (byte *)calloc(total_size, 1);
 		_fileHandle->read(ptr, total_size);
 		//dumpResource("sound-", idx, ptr);
-		convertMac0Resource(type, idx, ptr, total_size);
+		convertMac0Resource(_res, type, idx, ptr, total_size);
 		free(ptr);
 		return 1;
 
@@ -1179,7 +1184,7 @@ int ScummEngine::readSoundResource(int type, int idx) {
 	case 0x460e200d:	// WORKAROUND bug # 1311447
 		_fileHandle->seek(-12, SEEK_CUR);
 		total_size = _fileHandle->readUint32BE();
-		ptr = res.createResource(type, idx, total_size);
+		ptr = _res->createResource(type, idx, total_size);
 		_fileHandle->read(ptr, total_size - 8);
 		//dumpResource("sound-", idx, ptr);
 		return 1;
@@ -1188,7 +1193,7 @@ int ScummEngine::readSoundResource(int type, int idx) {
 		// HE sound type without SOUN header
 		_fileHandle->seek(-16, SEEK_CUR);
 		total_size = max_total_size + 8;
-		ptr = res.createResource(type, idx, total_size);
+		ptr = _res->createResource(type, idx, total_size);
 		_fileHandle->read(ptr, total_size);
 		//dumpResource("sound-", idx, ptr);
 		return 1;
@@ -1218,14 +1223,14 @@ int ScummEngine::readSoundResource(int type, int idx) {
 		debugC(DEBUG_SOUND, "FMUS file %s", buffer);
 		if (dmuFile.open(buffer) == false) {
 			error("Can't open music file %s*", buffer);
-			res.roomoffs[type][idx] = 0xFFFFFFFF;
+			_res->roomoffs[type][idx] = (uint32)RES_INVALID_OFFSET;
 			return 0;
 		}
 		dmuFile.seek(4, SEEK_SET);
 		total_size = dmuFile.readUint32BE();
 		debugC(DEBUG_SOUND, "dmu file size %d", total_size);
 		dmuFile.seek(-8, SEEK_CUR);
-		dmuFile.read(res.createResource(type, idx, total_size), total_size);
+		dmuFile.read(_res->createResource(type, idx, total_size), total_size);
 		dmuFile.close();
 		}
 		return 1;
@@ -1235,14 +1240,14 @@ int ScummEngine::readSoundResource(int type, int idx) {
 			_fileHandle->seek(-12, SEEK_CUR);
 			total_size = _fileHandle->readUint32BE();
 			_fileHandle->seek(-8, SEEK_CUR);
-			ptr = res.createResource(type, idx, total_size);
+			ptr = _res->createResource(type, idx, total_size);
 			_fileHandle->read(ptr, total_size);
 			//dumpResource("sound-", idx, ptr);
 			return 1;
 		}
 		error("Unrecognized base tag 0x%08x in sound %d", basetag, idx);
 	}
-	res.roomoffs[type][idx] = 0xFFFFFFFF;
+	_res->roomoffs[type][idx] = (uint32)RES_INVALID_OFFSET;
 	return 0;
 }
 
@@ -1296,7 +1301,7 @@ static const uint16 num_steps_table[] = {
 	600, 860, 1200, 1600
 };
 
-int ScummEngine::convert_extraflags(byte * ptr, byte * src_ptr) {
+static int convert_extraflags(byte * ptr, byte * src_ptr) {
 	int flags = src_ptr[0];
 
 	int t1, t2, t3, t4, time;
@@ -1362,7 +1367,7 @@ int ScummEngine::convert_extraflags(byte * ptr, byte * src_ptr) {
 }
 
 #define kMIDIHeaderSize		46
-static inline byte *writeMIDIHeader(byte *ptr, const char *type, int ppqn, int total_size) {
+static byte *writeMIDIHeader(byte *ptr, const char *type, int ppqn, int total_size) {
 	uint32 dw = TO_BE_32(total_size);
 
 	memcpy(ptr, type, 4); ptr += 4;
@@ -1386,7 +1391,7 @@ static inline byte *writeMIDIHeader(byte *ptr, const char *type, int ppqn, int t
 	return ptr;
 }
 
-static inline byte *writeVLQ(byte *ptr, int value) {
+static byte *writeVLQ(byte *ptr, int value) {
 	if (value > 0x7f) {
 		if (value > 0x3fff) {
 			*ptr++ = (value >> 14) | 0x80;
@@ -1399,7 +1404,7 @@ static inline byte *writeVLQ(byte *ptr, int value) {
 	return ptr;
 }
 
-static inline byte Mac0ToGMInstrument(uint32 type, int &transpose) {
+static byte Mac0ToGMInstrument(uint32 type, int &transpose) {
 	transpose = 0;
 	switch (type) {
 	case MKID_BE('MARI'): return 12;
@@ -1421,7 +1426,7 @@ static inline byte Mac0ToGMInstrument(uint32 type, int &transpose) {
 	}
 }
 
-void ScummEngine::convertMac0Resource(int type, int idx, byte *src_ptr, int size) {
+static void convertMac0Resource(ResourceManager *res, int type, int idx, byte *src_ptr, int size) {
 	/*
 	From Markus Magnuson (superqult) we got this information:
 	Mac0
@@ -1474,7 +1479,7 @@ void ScummEngine::convertMac0Resource(int type, int idx, byte *src_ptr, int size
 	*/
 
 #if 0
-	byte *ptr = res.createResource(type, idx, size);
+	byte *ptr = _res->createResource(type, idx, size);
 	memcpy(ptr, src_ptr, size);
 #else
 	const int ppqn = 480;
@@ -1516,7 +1521,7 @@ void ScummEngine::convertMac0Resource(int type, int idx, byte *src_ptr, int size
 	assert(*src_ptr == 0x09);
 
 	// Create sound resource
-	start_ptr = res.createResource(type, idx, total_size);
+	start_ptr = res->createResource(type, idx, total_size);
 
 	// Insert MIDI header
 	ptr = writeMIDIHeader(start_ptr, "GMD ", ppqn, total_size);
@@ -1612,7 +1617,7 @@ void ScummEngine::convertMac0Resource(int type, int idx, byte *src_ptr, int size
 #endif
 }
 
-void ScummEngine::convertADResource(int type, int idx, byte *src_ptr, int size) {
+static void convertADResource(ResourceManager *res, const GameSettings& game, int type, int idx, byte *src_ptr, int size) {
 
 	// We will ignore the PPQN in the original resource, because
 	// it's invalid anyway. We use a constant PPQN of 480.
@@ -1623,7 +1628,7 @@ void ScummEngine::convertADResource(int type, int idx, byte *src_ptr, int size)
 	int total_size = kMIDIHeaderSize + 7 + 8 * sizeof(ADLIB_INSTR_MIDI_HACK) + size;
 	total_size += 24;	// Up to 24 additional bytes are needed for the jump sysex
 
-	ptr = res.createResource(type, idx, total_size);
+	ptr = res->createResource(type, idx, total_size);
 
 	src_ptr += 2;
 	size -= 2;
@@ -1653,17 +1658,15 @@ void ScummEngine::convertADResource(int type, int idx, byte *src_ptr, int size)
 		src_ptr  += 0x11 + 8 * 16;
 		size -= 0x11 + 8 * 16;
 
-		CHECK_HEAP
-
 		track = src_ptr;
 
 		// Convert the ticks into a MIDI tempo.
 		// Unfortunate LOOM and INDY3 have different interpretation
 		// of the ticks value.
-		if (_game.id == GID_INDY3) {
+		if (game.id == GID_INDY3) {
 			// Note: since we fix ppqn at 480, ppqn/473 is almost 1
 			dw = 500000 * 256 / 473 * ppqn / ticks;
-		} else if (_game.id == GID_LOOM && _game.version == 3) {
+		} else if (game.id == GID_LOOM && game.version == 3) {
 			dw = 500000 * ppqn / 4 / ticks;
 		} else {
 			dw = 500000 * 256 / ticks;
@@ -2017,7 +2020,7 @@ int ScummEngine::readSoundResourceSmallHeader(int type, int idx) {
 		_fileHandle->seek(ro_offs + 4, SEEK_SET);
 		_fileHandle->read(src_ptr, ro_size -4);
 
-		ptr = res.createResource(type, idx, ro_size + 2);
+		ptr = _res->createResource(type, idx, ro_size + 2);
 		memcpy(ptr, "RO", 2); ptr += 2;
 		memcpy(ptr, src_ptr, ro_size - 4); ptr += ro_size - 4;
 		return 1;
@@ -2089,24 +2092,24 @@ int ScummEngine::readSoundResourceSmallHeader(int type, int idx) {
 		}
 		ptr = (byte *) calloc(ad_size, 1);
 		_fileHandle->read(ptr, ad_size);
-		convertADResource(type, idx, ptr, ad_size);
+		convertADResource(_res, _game, type, idx, ptr, ad_size);
 		free(ptr);
 		return 1;
 	} else if ((_musicType == MDT_PCSPK) && wa_offs != 0) {
 		if (_game.features & GF_OLD_BUNDLE) {
 			_fileHandle->seek(wa_offs, SEEK_SET);
-			_fileHandle->read(res.createResource(type, idx, wa_size), wa_size);
+			_fileHandle->read(_res->createResource(type, idx, wa_size), wa_size);
 		} else {
 			_fileHandle->seek(wa_offs - 6, SEEK_SET);
-			_fileHandle->read(res.createResource(type, idx, wa_size + 6), wa_size + 6);
+			_fileHandle->read(_res->createResource(type, idx, wa_size + 6), wa_size + 6);
 		}
 		return 1;
 	} else if (ro_offs != 0) {
 		_fileHandle->seek(ro_offs - 2, SEEK_SET);
-		_fileHandle->read(res.createResource(type, idx, ro_size - 4), ro_size - 4);
+		_fileHandle->read(_res->createResource(type, idx, ro_size - 4), ro_size - 4);
 		return 1;
 	}
-	res.roomoffs[type][idx] = 0xFFFFFFFF;
+	_res->roomoffs[type][idx] = (uint32)RES_INVALID_OFFSET;
 	return 0;
 }
 
