@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2002-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/scumm/sound.h $
- * $Id: sound.h 22526 2006-05-18 21:46:07Z eriktorbjorn $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/scumm/sound.h $
+ * $Id: sound.h 27024 2007-05-30 21:56:52Z fingolfin $
  */
 
 #ifndef SCUMM_SOUND_H
@@ -27,8 +30,8 @@
 #include "sound/mixer.h"
 #include "scumm/saveload.h"
 
-namespace Common {
-	class File;
+namespace Audio {
+class Mixer;
 }
 
 namespace Scumm {
@@ -56,6 +59,7 @@ public:
 
 protected:
 	ScummEngine *_vm;
+	Audio::Mixer *_mixer;
 
 	int16 _soundQuePos, _soundQue[0x100];
 	int16 _soundQue2Pos;
@@ -79,8 +83,6 @@ protected:
 	uint16 _mouthSyncTimes[64];
 	uint _curSoundPos;
 
-	int _overrideFreq;
-
 	int16 _currentCDSound;
 	int16 _currentMusic;
 
@@ -91,7 +93,7 @@ public:
 	byte _sfxMode;
 
 public:
-	Sound(ScummEngine *parent);
+	Sound(ScummEngine *parent, Audio::Mixer *mixer);
 	virtual ~Sound();
 	virtual void addSoundToQueue(int sound, int heOffset = 0, int heChannel = 0, int heFlags = 0);
 	virtual void addSoundToQueue2(int sound, int heOffset = 0, int heChannel = 0, int heFlags = 0);
