@@ -1,6 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2001  Ludvig Strigeus
- * Copyright (C) 2001-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -16,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/scumm/string.cpp $
- * $Id: string.cpp 23819 2006-09-02 05:51:19Z kirben $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/scumm/string.cpp $
+ * $Id: string.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -64,12 +66,21 @@ void ScummEngine::printString(int m, const byte *msg) {
 	}
 }
 
+#ifndef DISABLE_SCUMM_7_8
+void ScummEngine_v8::printString(int m, const byte *msg) {
+	if (m == 4) {
+		const StringTab &st = _string[m];
+		enqueueText(msg, st.xpos, st.ypos, st.color, st.charset, st.center);
+	} else {
+		ScummEngine::printString(m, msg);
+	}
+}
+#endif
 
 void ScummEngine::debugMessage(const byte *msg) {
 	byte buffer[500];
 	convertMessageToString(msg, buffer, sizeof(buffer));
 
-//	if ((_game.id == GID_CMI) && _debugMode) {	// In CMI, debugMessage is used for printDebug output
 	if ((buffer[0] != 0xFF) && _debugMode) {
 		debug(0, "DEBUG: %s", buffer);
 		return;
@@ -184,7 +195,7 @@ void ScummEngine_v6::removeBlastTexts() {
 	int i;
 
 	for (i = 0; i < _blastTextQueuePos; i++) {
-		restoreBG(_blastTextQueue[i].rect);
+		restoreBackground(_blastTextQueue[i].rect);
 	}
 	_blastTextQueuePos = 0;
 }
@@ -311,7 +322,7 @@ bool ScummEngine::handleNextCharsetCode(Actor *a, int *code) {
 			_charset->setCurID(*buffer++);
 			buffer += 2;
 			memcpy(_charsetColorMap, _charsetData[_charset->getCurID()], 4);
-			_charset->_nextTop -= _charset->getFontHeight() - oldy;
+			_nextTop -= _charset->getFontHeight() - oldy;
 			break;
 		default:
 			error("handleNextCharsetCode: invalid code %d", c);
@@ -400,7 +411,6 @@ bool ScummEngine_v72he::handleNextCharsetCode(Actor *a, int *code) {
 
 void ScummEngine::CHARSET_1() {
 	Actor *a;
-	int t, c = 0;
 #ifndef DISABLE_SCUMM_7_8
 	byte subtitleBuffer[200];
 	byte *subtitleLine = subtitleBuffer;
@@ -429,11 +439,11 @@ void ScummEngine::CHARSET_1() {
 	if (getTalkingActor() != 0xFF)
 		a = derefActorSafe(getTalkingActor(), "CHARSET_1");
 
-	if (a && _string[0].overhead != 0) {
+	if (a && _string[0].overhead) {
 		int s;
 
-		_string[0].xpos = a->_pos.x - virtscr[0].xstart;
-		_string[0].ypos = a->_pos.y - a->getElevation() - _screenTop;
+		_string[0].xpos = a->getPos().x - virtscr[0].xstart;
+		_string[0].ypos = a->getPos().y - a->getElevation() - _screenTop;
 
 		if (_game.version <= 5) {
 
@@ -482,8 +492,7 @@ void ScummEngine::CHARSET_1() {
 		return;
 
 	if ((_game.version <= 6 && _haveMsg == 1) ||
-	    (_game.version == 7 && _haveMsg != 1) ||
-	    (_game.version == 8 && VAR(VAR_HAVE_MSG))) {
+	    (_game.version == 7 && _haveMsg != 1)) {
 
 		if (_game.heversion >= 60) {
 			if (_sound->isSoundRunning(1) == 0)
@@ -506,32 +515,34 @@ void ScummEngine::CHARSET_1() {
 		if (_game.version >= 7) {
 #ifndef DISABLE_SCUMM_7_8
 			((ScummEngine_v7 *)this)->clearSubtitleQueue();
-			_charset->_nextLeft = _string[0].xpos;
-			_charset->_nextTop = _string[0].ypos;
+			_nextLeft = _string[0].xpos;
+			_nextTop = _string[0].ypos + _screenTop;
 #endif
 		} else {
-			_charset->restoreCharsetBg();
+			restoreCharsetBg();
 		}
 	}
 
-	t = _charset->_right - _string[0].xpos - 1;
-	if (_charset->_center) {
-		if (t > _charset->_nextLeft)
-			t = _charset->_nextLeft;
-		t *= 2;
+	if (_game.version > 3) {
+		int maxwidth = _charset->_right - _string[0].xpos - 1;
+		if (_charset->_center) {
+			if (maxwidth > _nextLeft)
+				maxwidth = _nextLeft;
+			maxwidth *= 2;
+		}
+
+		_charset->addLinebreaks(0, _charsetBuffer + _charsetBufPos, 0, maxwidth);
 	}
 
-	if (_game.version > 3)
-		_charset->addLinebreaks(0, _charsetBuffer + _charsetBufPos, 0, t);
-
 	if (_charset->_center) {
-		_charset->_nextLeft -= _charset->getStringWidth(0, _charsetBuffer + _charsetBufPos) / 2;
-		if (_charset->_nextLeft < 0)
-			_charset->_nextLeft = 0;
+		_nextLeft -= _charset->getStringWidth(0, _charsetBuffer + _charsetBufPos) / 2;
+		if (_nextLeft < 0)
+			_nextLeft = 0;
 	}
 
 	_charset->_disableOffsX = _charset->_firstChar = !_keepText;
 
+	int c = 0;
 	while (handleNextCharsetCode(a, &c)) {
 		if (c == 0) {
 			// End of text reached, set _haveMsg accordingly
@@ -542,7 +553,7 @@ void ScummEngine::CHARSET_1() {
 
 		if (c == 13) {
 		newLine:;
-			_charset->_nextLeft = _string[0].xpos;
+			_nextLeft = _string[0].xpos;
 #ifndef DISABLE_SCUMM_7_8			
 			if (_game.version >= 7 && subtitleLine != subtitleBuffer) {
 				((ScummEngine_v7 *)this)->addSubtitleToQueue(subtitleBuffer, subtitlePos, _charsetColor, _charset->getCurID());
@@ -550,15 +561,15 @@ void ScummEngine::CHARSET_1() {
 			}
 #endif
 			if (_charset->_center) {
-				_charset->_nextLeft -= _charset->getStringWidth(0, _charsetBuffer + _charsetBufPos) / 2;
+				_nextLeft -= _charset->getStringWidth(0, _charsetBuffer + _charsetBufPos) / 2;
 			}
 
 			if (_game.version == 0) {
 				break;
 			} else if (!(_game.platform == Common::kPlatformFMTowns) && _string[0].height) {
-				_charset->_nextTop += _string[0].height;
+				_nextTop += _string[0].height;
 			} else {
-				_charset->_nextTop += _charset->getFontHeight();
+				_nextTop += _charset->getFontHeight();
 			}
 			if (_game.version > 3) {
 				// FIXME - is this really needed?
@@ -568,22 +579,23 @@ void ScummEngine::CHARSET_1() {
 		}
 
 		// Handle line overflow for V3. See also bug #1306269.
-		if (_game.version == 3 && _charset->_nextLeft >= _screenWidth) {
-			_charset->_nextLeft = _screenWidth;
+		if (_game.version == 3 && _nextLeft >= _screenWidth) {
+			_nextLeft = _screenWidth;
 		}
 		// Handle line breaks for V1-V2
-		if (_game.version <= 2 && _charset->_nextLeft >= _screenWidth) {
+		if (_game.version <= 2 && _nextLeft >= _screenWidth) {
 			goto newLine;
 		}
 
-		_charset->_left = _charset->_nextLeft;
-		_charset->_top = _charset->_nextTop;
+		_charset->_left = _nextLeft;
+		_charset->_top = _nextTop;
 
 		if (_game.version >= 7) {
 #ifndef DISABLE_SCUMM_7_8
 			if (subtitleLine == subtitleBuffer) {
 				subtitlePos.x = _charset->_left;
-				subtitlePos.y = _charset->_top;
+				// BlastText position is relative to the top of the screen, adjust y-coordinate
+				subtitlePos.y = _charset->_top - _screenTop;
 			}
 			*subtitleLine++ = c;
 			*subtitleLine = '\0';
@@ -614,8 +626,8 @@ void ScummEngine::CHARSET_1() {
 					_charset->printChar(c, false);
 				}
 			}
-			_charset->_nextLeft = _charset->_left;
-			_charset->_nextTop = _charset->_top;
+			_nextLeft = _charset->_left;
+			_nextTop = _charset->_top;
 		}
 
 		if (_game.version <= 2) {
@@ -633,6 +645,164 @@ void ScummEngine::CHARSET_1() {
 #endif
 }
 
+#ifndef DISABLE_SCUMM_7_8
+void ScummEngine_v8::CHARSET_1() {
+	byte subtitleBuffer[2048];
+	byte *subtitleLine = subtitleBuffer;
+	Common::Point subtitlePos;
+
+	processSubtitleQueue();
+
+	if (!_haveMsg)
+		return;
+
+	Actor *a = NULL;
+	if (getTalkingActor() != 0xFF)
+		a = derefActorSafe(getTalkingActor(), "CHARSET_1");
+
+	StringTab saveStr = _string[0];
+	if (a && _string[0].overhead) {
+		int s;
+
+		_string[0].xpos = a->getPos().x - virtscr[0].xstart;
+		s = a->_scalex * a->_talkPosX / 255;
+		_string[0].xpos += (a->_talkPosX - s) / 2 + s;
+
+		_string[0].ypos = a->getPos().y - a->getElevation() - _screenTop;
+		s = a->_scaley * a->_talkPosY / 255;
+		_string[0].ypos += (a->_talkPosY - s) / 2 + s;
+	}
+
+	_charset->setColor(_charsetColor);
+
+	if (a && a->_charset)
+		_charset->setCurID(a->_charset);
+	else
+		_charset->setCurID(_string[0].charset);
+
+	if (_talkDelay)
+		return;
+
+	if (VAR(VAR_HAVE_MSG)) {
+		if ((_sound->_sfxMode & 2) == 0) {
+			stopTalk();
+		}
+		return;
+	}
+
+	if (a && !_string[0].no_talk_anim) {
+		a->runActorTalkScript(a->_talkStartFrame);
+	}
+
+	if (!_keepText) {
+		clearSubtitleQueue();
+		_nextLeft = _string[0].xpos;
+		_nextTop = _string[0].ypos + _screenTop;
+	}
+
+	_charset->_disableOffsX = _charset->_firstChar = !_keepText;
+
+	_talkDelay = VAR(VAR_DEFAULT_TALK_DELAY);
+	for (int i = _charsetBufPos; _charsetBuffer[i]; ++i) {
+		_talkDelay += VAR(VAR_CHARINC);
+	}
+
+	if (_string[0].wrapping) {
+		_charset->addLinebreaks(0, _charsetBuffer, _charsetBufPos, _screenWidth - 20);
+
+		struct { int pos, w; } substring[10];
+		int count = 0;
+		int maxLineWidth = 0;
+		int lastPos = 0;
+		int code = 0;
+		while (handleNextCharsetCode(a, &code)) {
+			if (code == 13 || code == 0) {
+				*subtitleLine++ = '\0';
+				assert(count < 10);
+				substring[count].w = _charset->getStringWidth(0, subtitleBuffer + lastPos);
+				if (maxLineWidth < substring[count].w) {
+					maxLineWidth = substring[count].w;
+				}
+				substring[count].pos = lastPos;
+				++count;
+				lastPos = subtitleLine - subtitleBuffer;
+			} else {
+				*subtitleLine++ = code;
+				*subtitleLine = '\0';
+			}
+			if (code == 0) {
+				break;
+			}
+		}
+
+		int h = count * _charset->getFontHeight();
+		h += _charset->getFontHeight() / 2;
+		subtitlePos.y = _string[0].ypos;
+		if (subtitlePos.y + h > _screenHeight - 10) {
+			subtitlePos.y = _screenHeight - 10 - h;
+		}
+		if (subtitlePos.y < 10) {
+			subtitlePos.y = 10;
+		}
+
+		for (int i = 0; i < count; ++i) {
+			subtitlePos.x = _string[0].xpos;
+			if (_string[0].center) {
+				if (subtitlePos.x + maxLineWidth / 2 > _screenWidth - 10) {
+					subtitlePos.x = _screenWidth - 10 - maxLineWidth / 2;
+				}
+				if (subtitlePos.x - maxLineWidth / 2 < 10) {
+					subtitlePos.x = 10 + maxLineWidth / 2;
+				}
+				subtitlePos.x -= substring[i].w / 2;
+			} else {
+				if (subtitlePos.x + maxLineWidth > _screenWidth - 10) {
+					subtitlePos.x = _screenWidth - 10 - maxLineWidth;
+				}
+				if (subtitlePos.x - maxLineWidth < 10) {
+					subtitlePos.x = 10;
+				}
+			}
+			if (subtitlePos.y < _screenHeight - 10) {
+				addSubtitleToQueue(subtitleBuffer + substring[i].pos, subtitlePos, _charsetColor, _charset->getCurID());
+			}
+			subtitlePos.y += _charset->getFontHeight();
+		}
+	} else {
+		int code = 0;
+		subtitlePos.y = _string[0].ypos;
+		if (subtitlePos.y < 10) {
+			subtitlePos.y = 10;
+		}
+		while (handleNextCharsetCode(a, &code)) {
+			if (code == 13 || code == 0) {
+				subtitlePos.x = _string[0].xpos;
+				if (_string[0].center) {
+					subtitlePos.x -= _charset->getStringWidth(0, subtitleBuffer) / 2;
+				}
+				if (subtitlePos.x < 10) {
+					subtitlePos.x = 10;
+				}
+				if (subtitlePos.y < _screenHeight - 10) {
+					addSubtitleToQueue(subtitleBuffer, subtitlePos, _charsetColor, _charset->getCurID());
+					subtitlePos.y += _charset->getFontHeight();
+				}
+				subtitleLine = subtitleBuffer;
+			} else {
+				*subtitleLine++ = code;
+			}
+			*subtitleLine = '\0';
+			if (code == 0) {
+				break;
+			}
+		}
+	}
+	_haveMsg = 2;
+	_keepText = false;
+	_string[0] = saveStr;
+}
+#endif
+
 void ScummEngine::drawString(int a, const byte *msg) {
 	byte buf[270];
 	byte *space;
@@ -677,29 +847,37 @@ void ScummEngine::drawString(int a, const byte *msg) {
 
 	fontHeight = _charset->getFontHeight();
 
-	// trim from the right
-	byte *tmp = buf;
-	space = NULL;
-	while (*tmp) {
-		if (*tmp == ' ') {
-			if (!space)
-				space = tmp;
-		} else {
-			space = NULL;
+	if (_game.version >= 4) {
+		// trim from the right
+		byte *tmp = buf;
+		space = NULL;
+		while (*tmp) {
+			if (*tmp == ' ') {
+				if (!space)
+					space = tmp;
+			} else {
+				space = NULL;
+			}
+			tmp++;
 		}
-		tmp++;
+		if (space)
+			*space = '\0';
 	}
-	if (space)
-		*space = '\0';
+
 	if (_charset->_center) {
 		_charset->_left -= _charset->getStringWidth(a, buf) / 2;
 	}
 
-	const bool ignoreCharsetMask = (_game.version < 7);
-
 	if (!buf[0]) {
-		buf[0] = ' ';
-		buf[1] = 0;
+		if (_game.version >= 5) {
+			buf[0] = ' ';
+			buf[1] = 0;
+		} else {
+			_charset->_str.left = _charset->_left;
+			_charset->_str.top = _charset->_top;
+			_charset->_str.right = _charset->_left;
+			_charset->_str.bottom = _charset->_top;
+		}
 	}
 
 	for (i = 0; (c = buf[i++]) != 0;) {
@@ -715,7 +893,7 @@ void ScummEngine::drawString(int a, const byte *msg) {
 				_charset->_top += fontHeight;
 				break;
 			}
-		} else if (c == 0xFF || (_game.version <= 6 && c == 0xFE) && (_game.heversion <= 71)) {
+		} else if ((c == 0xFF || (_game.version <= 6 && c == 0xFE)) && (_game.heversion <= 71)) {
 			c = buf[i++];
 			switch (c) {
 			case 9:
@@ -732,7 +910,7 @@ void ScummEngine::drawString(int a, const byte *msg) {
 					_charset->_left = _charset->_startLeft;
 				}
 				if (!(_game.platform == Common::kPlatformFMTowns) && _string[0].height) {
-					_charset->_nextTop += _string[0].height;
+					_nextTop += _string[0].height;
 				} else {
 					_charset->_top += fontHeight;
 				}
@@ -774,7 +952,7 @@ void ScummEngine::drawString(int a, const byte *msg) {
 					}
 				}
 			}
-			_charset->printChar(c, ignoreCharsetMask);
+			_charset->printChar(c, (_game.version < 7));
 			_charset->_blitAlso = false;
 
 			if (cmi_pos_hack) {
@@ -785,11 +963,11 @@ void ScummEngine::drawString(int a, const byte *msg) {
 	}
 
 	if (a == 0) {
-		_charset->_nextLeft = _charset->_left;
-		_charset->_nextTop = _charset->_top;
+		_nextLeft = _charset->_left;
+		_nextTop = _charset->_top;
 	}
 
-	_string[a].xpos = _charset->_str.right + 8;	// Indy3: Fixes Grail Diary text positioning
+	_string[a].xpos = _charset->_str.right;
 }
 
 int ScummEngine::convertMessageToString(const byte *msg, byte *dst, int dstSize) {
@@ -831,6 +1009,16 @@ int ScummEngine::convertMessageToString(const byte *msg, byte *dst, int dstSize)
 				*dst++ = 0xE1;
 				continue;
 			}
+			
+			// WORKAROUND for bug #1514457: Yet another script bug in Indy3.
+			// Once more a german 'sz' was encoded incorrectly, but this time
+			// they simply encoded it as 0xFF instead of 0xE1. Happens twice
+			// in script 71.
+			if (_game.id == GID_INDY3 && chr == 0x20 && vm.slot[_currentScript].number == 71) {
+				num--;
+				*dst++ = 0xE1;
+				continue;
+			}
 
 			if (chr == 1 || chr == 2 || chr == 3 || chr == 8) {
 				// Simply copy these special codes
@@ -1012,7 +1200,7 @@ void ScummEngine_v80he::initCharset(int charsetno) {
 
 void ScummEngine::initCharset(int charsetno) {
 	if (_game.id == GID_FT) {
-		if (!res.isResourceLoaded(rtCharset, charsetno))
+		if (!_res->isResourceLoaded(rtCharset, charsetno))
 			loadCharset(charsetno);
 	} else {
 		if (!getResourceAddress(rtCharset, charsetno))
@@ -1044,6 +1232,14 @@ void ScummEngine_v7::loadLanguageBundle() {
 	ScummFile file;
 	int32 size;
 
+	// if game is manually set to English, don't try to load localized text
+	if ((_language == Common::EN_ANY) || (_language == Common::EN_USA) || (_language == Common::EN_GRB)) {
+		warning("Language file is forced to be ignored");
+
+		_existLanguageFile = false;
+		return;
+	}
+
 	if (_game.id == GID_DIG) {
 		openFile(file, "language.bnd");
 	} else if (_game.id == GID_CMI) {
@@ -1228,7 +1424,7 @@ void ScummEngine_v7::translateText(const byte *text, byte *trans_buff) {
 			text = (const byte *)"/CATHPLAT.068/left";
 		else if (!strcmp((const char *)text, "right"))
 			text = (const byte *)"/CATHPLAT.070/right";
-		else if (!strcmp((const char *)text, "right"))
+		else if (!strcmp((const char *)text, "top"))
 			text = (const byte *)"/CATHPLAT.067/top";
 		else if (!strcmp((const char *)text, "exit"))
 			text = (const byte *)"/SKY.008/exit";
