@@ -1,6 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2001  Ludvig Strigeus
- * Copyright (C) 2001-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -16,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/scumm/thumbnail.cpp $
- * $Id: thumbnail.cpp 23988 2006-09-25 21:08:03Z agent-q $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/scumm/thumbnail.cpp $
+ * $Id: thumbnail.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -31,22 +33,15 @@ namespace Scumm {
 
 #define THMB_VERSION 1
 
-#if !defined(__GNUC__)
-	#pragma START_PACK_STRUCTS
-#endif
-
 struct ThumbnailHeader {
 	uint32 type;
 	uint32 size;
 	byte version;
 	uint16 width, height;
 	byte bpp;
-} GCC_PACK;
-
-#if !defined(__GNUC__)
-	#pragma END_PACK_STRUCTS
-#endif
+};
 
+#define ThumbnailHeaderSize (4+4+1+2+2+1)
 
 inline void colorToRGB(uint16 color, uint8 &r, uint8 &g, uint8 &b) {
 	r = (((color >> 11) & 0x1F) << 3);
@@ -104,20 +99,14 @@ Graphics::Surface *ScummEngine::loadThumbnail(Common::InSaveFile *file) {
 void ScummEngine::saveThumbnail(Common::OutSaveFile *file) {
 	Graphics::Surface thumb;
 
-#if !defined(PALMOS_68K) && !defined(__DS__)
+#if !defined(PALMOS_68K) || !defined(__DS__)
 	if (!createThumbnailFromScreen(&thumb))
 #endif
 		thumb.create(kThumbnailWidth, kThumbnailHeight2, sizeof(uint16));
 
 	ThumbnailHeader header;
 	header.type = MKID_BE('THMB');
-#if defined(PALMOS_ARM) || defined(__GP32__)
-	// sizeof(header) is hardcoded here, because the compiler add padding to
-	// have a 4byte aligned struct and there is no easy way to pack it.
-	header.size = 14 + thumb.w*thumb.h*thumb.bytesPerPixel;
-#else
-	header.size = sizeof(header) + thumb.w*thumb.h*thumb.bytesPerPixel;
-#endif
+	header.size = ThumbnailHeaderSize + thumb.w*thumb.h*thumb.bytesPerPixel;
 	header.version = THMB_VERSION;
 	header.width = thumb.w;
 	header.height = thumb.h;
