@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2002-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/scumm/util.cpp $
- * $Id: util.cpp 21038 2006-03-03 15:16:02Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/scumm/util.cpp $
+ * $Id: util.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -29,11 +32,9 @@ namespace Scumm {
 #pragma mark --- Utilities ---
 #pragma mark -
 
-void checkRange(int max, int min, int no, const char *str) {
-	if (no < min || no > max) {
-		char buf[256];
-		snprintf(buf, sizeof(buf), str, no);
-		error("Value %d is out of bounds (%d,%d) (%s)", no, min, max, buf);
+void assertRange(int min, int value, int max, const char *desc) {
+	if (value < min || value > max) {
+		error("%s %d is out of bounds (%d,%d)", desc, value, min, max);
 	}
 }
 
