@@ -1,6 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2001  Ludvig Strigeus
- * Copyright (C) 2001-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -16,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/scumm/vars.cpp $
- * $Id: vars.cpp 23728 2006-08-20 00:44:38Z kirben $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/scumm/vars.cpp $
+ * $Id: vars.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -106,7 +108,7 @@ void ScummEngine::setupScummVars() {
 	}
 }
 
-void ScummEngine_c64::setupScummVars() {
+void ScummEngine_v0::setupScummVars() {
 	VAR_EGO = 0;
 	VAR_CAMERA_POS_X = 2;
 	VAR_HAVE_MSG = 3;
@@ -298,8 +300,8 @@ void ScummEngine_v72he::setupScummVars() {
 void ScummEngine_v80he::setupScummVars() {
 	ScummEngine_v72he::setupScummVars();
 
-	VAR_PLATFORM = 78;  		// 1 is PC, 2 is Macintosh
-	VAR_WINDOWS_VERSION = 79; 	// 31 is Windows 3.1, 40 is Windows 95+
+	VAR_PLATFORM = 78;
+	VAR_PLATFORM_VERSION = 79;
 	VAR_CURRENT_CHARSET = 80;
 	VAR_SOUNDCODE_TMR = 84;
 	VAR_KEY_STATE = 86;
@@ -536,7 +538,7 @@ void ScummEngine_v8::setupScummVars() {
 }
 #endif
 
-void ScummEngine_c64::resetScummVars() {
+void ScummEngine_v0::resetScummVars() {
 	_activeInventory = 0;
 	_activeObject = 0;
 	_activeVerb = 13;
@@ -653,10 +655,10 @@ void ScummEngine_v80he::resetScummVars() {
 
 	if (_game.platform == Common::kPlatformMacintosh) {
 		VAR(VAR_PLATFORM) = 2;
-		VAR(VAR_WINDOWS_VERSION) = 70;
+		VAR(VAR_PLATFORM_VERSION) = 70;
 	} else {
 		VAR(VAR_PLATFORM) = 1;
-		VAR(VAR_WINDOWS_VERSION) = 40;
+		VAR(VAR_PLATFORM_VERSION) = 40; 	// 31 is Windows 3.1, 40 is Windows 95+
 	}
 	VAR(VAR_COLOR_DEPTH) = 256;
 }
@@ -684,6 +686,11 @@ void ScummEngine_v99he::resetScummVars() {
 
 	VAR(VAR_NUM_PALETTES) = _numPalettes;
 	VAR(VAR_NUM_UNK) = _numUnk;
+
+	if (_game.heversion >= 100) {
+		// Enable Bink video
+		VAR(140) = 1;
+	}
 }
 #endif
 
@@ -702,22 +709,11 @@ void ScummEngine::resetScummVars() {
 		default:
 			if ((_game.id == GID_MONKEY_EGA || _game.id == GID_MONKEY_VGA || (_game.id == GID_LOOM && _game.version == 3))
 			   &&  (_game.platform == Common::kPlatformPC)) {
-				if (_game.id == GID_LOOM) {
-					char buf[50];
-					Common::File f;
-					for (int i = 82; i < 85; i++) {
-						sprintf(buf, "%d.LFL", i);
-						if (!Common::File::exists(buf))
-							error("Native MIDI support requires Roland patch from LucasArts, but %s is missing", buf);
-					}
-				} else if (_game.id == GID_MONKEY_EGA) {
-					if (!Common::File::exists("DISK09.LEC"))
-						error("Native MIDI support requires Roland patch from LucasArts, but DISK09.LEC is missing");
-				}
 				VAR(VAR_SOUNDCARD) = 4;
 			} else {
 				VAR(VAR_SOUNDCARD) = 3;
 			}
+			break;
 		}
 
 		if (_game.platform == Common::kPlatformFMTowns)
