@@ -1,6 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2001  Ludvig Strigeus
- * Copyright (C) 2001-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -16,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/scumm/verbs.cpp $
- * $Id: verbs.cpp 23963 2006-09-21 12:16:44Z sev $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/scumm/verbs.cpp $
+ * $Id: verbs.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -46,7 +48,7 @@ struct VerbSettings {
 	const char *name;
 };
 
-static const VerbSettings C64VerbTable_English[] = {
+static const VerbSettings v0VerbTable_English[] = {
 	{ 1,  8, 0,   0, "Open"},
 	{ 2,  8, 1,   0, "Close"},
 	{ 3,  0, 2,   4, "Give"},
@@ -65,7 +67,7 @@ static const VerbSettings C64VerbTable_English[] = {
 };
 
 // FIXME: Replace * with the correct character
-static const VerbSettings C64VerbTable_German[] = {
+static const VerbSettings v0VerbTable_German[] = {
 	{ 1,  0, 1,   0, "Ziehe"},
 	{ 2,  0, 0,   0, "Dr[cke"},
 	{ 3,  7, 2,   4, "Lese"},
@@ -83,14 +85,17 @@ static const VerbSettings C64VerbTable_German[] = {
 	{15, 23, 2,   0, "Benutz"}
 };
 
-void ScummEngine_c64::initC64Verbs() {
+void ScummEngine_v0::resetVerbs() {
 	VirtScreen *virt = &virtscr[kVerbVirtScreen];
 	VerbSlot *vs;
 	int i;
 
+	for (i = 1; i < 16; i++)
+		killVerb(i);
+
 	for (i = 1; i < 16; i++) {
 		vs = &_verbs[i];
-		vs->verbid = C64VerbTable_English[i - 1].id;
+		vs->verbid = v0VerbTable_English[i - 1].id;
 		vs->color = 5;
 		vs->hicolor = 7;
 		vs->dimcolor = 11;
@@ -101,20 +106,58 @@ void ScummEngine_c64::initC64Verbs() {
 		vs->key = 0;
 		vs->center = 0;
 		vs->imgindex = 0;
-		vs->prep = C64VerbTable_English[i - 1].prep;
+		vs->prep = v0VerbTable_English[i - 1].prep;
 
 		if (_language == Common::DE_DEU) {
-			vs->curRect.left = C64VerbTable_German[i - 1].x_pos * 8;
-			vs->curRect.top = C64VerbTable_German[i - 1].y_pos * 8 + virt->topline + 8;
-			loadPtrToResource(rtVerb, i, (const byte*)C64VerbTable_German[i - 1].name);
+			vs->curRect.left = v0VerbTable_German[i - 1].x_pos * 8;
+			vs->curRect.top = v0VerbTable_German[i - 1].y_pos * 8 + virt->topline + 8;
+			loadPtrToResource(rtVerb, i, (const byte*)v0VerbTable_German[i - 1].name);
 		} else {
-			vs->curRect.left = C64VerbTable_English[i - 1].x_pos * 8;
-			vs->curRect.top = C64VerbTable_English[i - 1].y_pos * 8 + virt->topline + 8;
-			loadPtrToResource(rtVerb, i, (const byte*)C64VerbTable_English[i - 1].name);
+			vs->curRect.left = v0VerbTable_English[i - 1].x_pos * 8;
+			vs->curRect.top = v0VerbTable_English[i - 1].y_pos * 8 + virt->topline + 8;
+			loadPtrToResource(rtVerb, i, (const byte*)v0VerbTable_English[i - 1].name);
 		}
 	}
 }
 
+void ScummEngine_v0::setNewKidVerbs() {
+	VirtScreen *virt = &virtscr[kVerbVirtScreen];
+	VerbSlot *vs;
+	int i;
+
+	for (i = 1; i < 16; i++)
+		killVerb(i);
+
+	for (i = 1; i < 4; i++) {
+		vs = &_verbs[i];
+		vs->verbid = i;
+		vs->color = 5;
+		vs->hicolor = 7;
+		vs->dimcolor = 11;
+		vs->type = kTextVerbType;
+		vs->charset_nr = _string[0]._default.charset;
+		vs->curmode = 1;
+		vs->saveid = 0;
+		vs->key = 0;
+		vs->center = 0;
+		vs->imgindex = 0;
+		vs->prep = 0;
+		vs->curRect.left = (i * 8) * 8;
+		vs->curRect.top = virt->topline + 8;
+
+		Actor *a = derefActor(VAR(96 + i), "setNewKidVerbs");
+		loadPtrToResource(rtVerb, i, (const byte*)(const byte*)a->getActorName());
+	}
+	setUserState(191);
+}
+
+void ScummEngine_v0::switchActor(int slot) {
+	VAR(VAR_EGO) = VAR(97 + slot);
+	actorFollowCamera(VAR(VAR_EGO));
+	resetVerbs();
+	setUserState(247);
+}
+
 void ScummEngine_v2::initV2MouseOver() {
 	int i;
 	int arrow_color, color, hi_color;
@@ -365,7 +408,7 @@ void ScummEngine_v2::redrawV2Inventory() {
 	inventoryBox.bottom = vs->topline + virtscr[kVerbVirtScreen].h;
 	inventoryBox.left = 0;
 	inventoryBox.right = vs->w;
-	restoreBG(inventoryBox);
+	restoreBackground(inventoryBox);
 
 	_string[1].charset = 1;
 
@@ -464,7 +507,7 @@ void ScummEngine_v2::handleMouseOver(bool updateInventory) {
 	checkV2MouseOver(_mouse);
 }
 
-void ScummEngine_c64::handleMouseOver(bool updateInventory) {
+void ScummEngine_v0::handleMouseOver(bool updateInventory) {
 	ScummEngine_v2::handleMouseOver(updateInventory);
 
 	drawSentence();
@@ -485,13 +528,18 @@ void ScummEngine::checkExecVerbs() {
 
 	if (_mouseAndKeyboardStat < MBS_MAX_KEY) {
 		/* Check keypresses */
-		vs = &_verbs[1];
-		for (i = 1; i < _numVerbs; i++, vs++) {
-			if (vs->verbid && vs->saveid == 0 && vs->curmode == 1) {
-				if (_mouseAndKeyboardStat == vs->key) {
-					// Trigger verb as if the user clicked it
-					runInputScript(1, vs->verbid, 1);
-					return;
+		if (!(_game.id == GID_MONKEY && _game.platform == Common::kPlatformSegaCD)) {
+			// This is disabled in the SegaCD version as the "vs->key" values setup
+			// by script-17 conflict with the values expected by the generic keyboard
+			// input script. See tracker item #1193185.
+			vs = &_verbs[1];
+			for (i = 1; i < _numVerbs; i++, vs++) {
+				if (vs->verbid && vs->saveid == 0 && vs->curmode == 1) {
+					if (_mouseAndKeyboardStat == vs->key) {
+						// Trigger verb as if the user clicked it
+						runInputScript(1, vs->verbid, 1);
+						return;
+					}
 				}
 			}
 		}
@@ -503,14 +551,41 @@ void ScummEngine::checkExecVerbs() {
 			// part of the "Passport to Adventure" demo expects the same keyboard
 			// mapping, even though the full game doesn't.
 			static const int numpad[10] = {
-					'0',
-					335, 336, 337,
-					331, 332, 333,
-					327, 328, 329
-				};
+				'0',
+				335, 336, 337,
+				331, 332, 333,
+				327, 328, 329
+			};
 			_mouseAndKeyboardStat = numpad[_mouseAndKeyboardStat - '0'];
 		}
 
+		if (_game.platform == Common::kPlatformFMTowns && (_game.id == GID_ZAK || _game.id == GID_INDY3)) {
+			// HACK: In the FM-Towns games Indy3 and Zak the most significant bit is set for special keys
+			// like F5 (=0x8005) or joystick buttons (mask 0xFE00, e.g. SELECT=0xFE40 for the save/load menu).
+			// Hence the distinction with (_mouseAndKeyboardStat < MBS_MAX_KEY) between mouse- and key-events is not applicable
+			// to this games, so we have to remap the special keys here.
+			if(_mouseAndKeyboardStat == 319) {
+				_mouseAndKeyboardStat = 0x8005;
+			}
+		}
+
+		if ((_game.platform == Common::kPlatformFMTowns && _game.id == GID_ZAK) &&
+			(_mouseAndKeyboardStat >= 315 && _mouseAndKeyboardStat <= 318)) {
+			// Hack: Handle switching to a person via F1-F4 keys.
+			// This feature isn't available in the scripts of the FM-TOWNS verison.
+			int fKey = _mouseAndKeyboardStat - 314;
+			int switchSlot = getVerbSlot(36, 0);
+			// check if switch-verb is enabled
+			if (_verbs[switchSlot].curmode == 1) {
+				// Check if person is available (see script 23 from ZAK_FM-TOWNS and script 4 from ZAK_PC).
+				// Zak: Var[144 Bit 15], Annie: Var[145 Bit 0], Melissa: Var[145 Bit 1], Leslie: Var[145 Bit 2]
+				if (!readVar(0x890E + fKey)) {
+					runInputScript(1, 36 + fKey, 0);
+				}
+			}
+			return;
+		}
+
 		// Generic keyboard input
 		runInputScript(4, _mouseAndKeyboardStat, 1);
 	} else if (_mouseAndKeyboardStat & MBS_MOUSE_MASK) {
@@ -555,7 +630,7 @@ void ScummEngine_v2::runObject(int obj, int entry) {
 	_activeVerb = 13;
 }
 
-void ScummEngine_c64::checkExecVerbs() {
+void ScummEngine_v0::checkExecVerbs() {
 	Actor *a;
 	VirtScreen *zone = findVirtScreen(_mouse.y);
 
@@ -573,14 +648,29 @@ void ScummEngine_c64::checkExecVerbs() {
 			checkV2Inventory(_mouse.x, _mouse.y);
 		} else {
 			int over = findVerbAtPos(_mouse.x, _mouse.y);
+
+			// Handle New Kid verb options
+			if (_activeVerb == 7) {
+				if (over) {
+					_activeVerb = 13;
+					switchActor(_verbs[over].verbid - 1);
+				}
+				return;
+			}
+
 			if (over) {
 				_activeVerb = _verbs[over].verbid;
+				// Selected New Kid verb
+				if (_activeVerb == 7)
+					setNewKidVerbs();
+
 				return;
 			}
 
 			int act = getActorFromPos(_virtualMouse.x, _virtualMouse.y);
 			int obj = findObject(_virtualMouse.x, _virtualMouse.y);
 			if (act != 0 && _activeVerb == 3 && _activeInventory != 0) {
+				// Give inventory item to actor
 				VAR(5) = act;
 				runObject(_activeInventory, _activeVerb);
 			} else if (obj) {
@@ -601,7 +691,7 @@ void ScummEngine_c64::checkExecVerbs() {
 				runObject(_activeObject, entry);
 			} else if (zone->number == kMainVirtScreen) {
 				a = derefActor(VAR(VAR_EGO), "checkExecVerbs");
-				a->startWalkActor(_virtualMouse.x, _virtualMouse.y, -1);
+				a->startWalkActor(_virtualMouse.x / V12_X_MULTIPLIER, _virtualMouse.y / V12_Y_MULTIPLIER, -1);
 			}
 
 			_activeInventory = 0;
@@ -619,17 +709,16 @@ void ScummEngine::verbMouseOver(int verb) {
 	if (_game.id == GID_FT)
 		return;
 
-	if (_verbMouseOver == verb)
-		return;
+	if (_verbMouseOver != verb) {
+		if (_verbs[_verbMouseOver].type != kImageVerbType) {
+			drawVerb(_verbMouseOver, 0);
+			_verbMouseOver = verb;
+		}
 
-	if (_verbs[_verbMouseOver].type != kImageVerbType) {
-		drawVerb(_verbMouseOver, 0);
-		_verbMouseOver = verb;
-	}
-
-	if (_verbs[verb].type != kImageVerbType && _verbs[verb].hicolor) {
-		drawVerb(verb, 1);
-		_verbMouseOver = verb;
+		if (_verbs[verb].type != kImageVerbType && _verbs[verb].hicolor) {
+			drawVerb(verb, 1);
+			_verbMouseOver = verb;
+		}
 	}
 }
 
@@ -800,7 +889,7 @@ void ScummEngine::restoreVerbBG(int verb) {
 	vs = &_verbs[verb];
 
 	if (vs->oldRect.left != -1) {
-		restoreBG(vs->oldRect, vs->bkcolor);
+		restoreBackground(vs->oldRect, vs->bkcolor);
 		vs->oldRect.left = -1;
 	}
 }
@@ -814,13 +903,12 @@ void ScummEngine::drawVerbBitmap(int verb, int x, int y) {
 	int imgw, imgh;
 	int i, tmp;
 	byte *obim;
-	const ImageHeader *imhd;
 	uint32 size;
 
 	if ((vs = findVirtScreen(y)) == NULL)
 		return;
 
-	gdi.disableZBuffer();
+	_gdi->disableZBuffer();
 
 	twobufs = vs->hasTwoBuffers;
 	vs->hasTwoBuffers = false;
@@ -841,7 +929,7 @@ void ScummEngine::drawVerbBitmap(int verb, int x, int y) {
 		imgh = (*(obim + size + 17)) / 8;
 		imptr = getObjectImage(obim, 1);
 	} else {
-		imhd = (const ImageHeader *)findResourceData(MKID_BE('IMHD'), obim);
+		const ImageHeader *imhd = (const ImageHeader *)findResourceData(MKID_BE('IMHD'), obim);
 		if (_game.version >= 7) {
 			imgw = READ_LE_UINT16(&imhd->v7.width) / 8;
 			imgh = READ_LE_UINT16(&imhd->v7.height) / 8;
@@ -854,7 +942,7 @@ void ScummEngine::drawVerbBitmap(int verb, int x, int y) {
 	assert(imptr);
 	for (i = 0; i < imgw; i++) {
 		tmp = xstrip + i;
-		gdi.drawBitmap(imptr, vs, tmp, ydiff, imgw * 8, imgh * 8, i, 1, Gdi::dbAllowMaskOr | Gdi::dbObjectMode);
+		_gdi->drawBitmap(imptr, vs, tmp, ydiff, imgw * 8, imgh * 8, i, 1, Gdi::dbAllowMaskOr | Gdi::dbObjectMode);
 	}
 
 	vst = &_verbs[verb];
@@ -862,7 +950,7 @@ void ScummEngine::drawVerbBitmap(int verb, int x, int y) {
 	vst->curRect.bottom = vst->curRect.top + imgh * 8;
 	vst->oldRect = vst->curRect;
 
-	gdi.enableZBuffer();
+	_gdi->enableZBuffer();
 
 	vs->hasTwoBuffers = twobufs;
 }
@@ -887,7 +975,7 @@ void ScummEngine::killVerb(int slot) {
 	vs->verbid = 0;
 	vs->curmode = 0;
 
-	res.nukeResource(rtVerb, slot);
+	_res->nukeResource(rtVerb, slot);
 
 	if (_game.version <= 6 && vs->saveid == 0) {
 		drawVerb(slot, 0);
@@ -915,7 +1003,7 @@ void ScummEngine::setVerbObject(uint room, uint object, uint verb) {
 			if (_objs[i].obj_nr == object) {
 				findObjectInRoom(&foir, foImageHeader, object, room);
 				size = READ_LE_UINT16(foir.obim);
-				byte *ptr = res.createResource(rtVerb, verb, size + 2);
+				byte *ptr = _res->createResource(rtVerb, verb, size + 2);
 				obcdptr = getResourceAddress(rtRoom, room) + getOBCDOffs(object);
 				ptr[0] = *(obcdptr + 9);	// Width
 				ptr[1] = *(obcdptr + 15);	// Height
@@ -934,7 +1022,7 @@ void ScummEngine::setVerbObject(uint room, uint object, uint verb) {
 				size = READ_LE_UINT32(foir.obim);
 				obcdptr = getResourceAddress(rtRoom, room) + getOBCDOffs(object);
 				size2 = READ_LE_UINT32(obcdptr);
-				res.createResource(rtVerb, verb, size + size2);
+				_res->createResource(rtVerb, verb, size + size2);
 				obimptr = getResourceAddress(rtRoom, room) - foir.roomptr + foir.obim;
 				obcdptr = getResourceAddress(rtRoom, room) + getOBCDOffs(object);
 				memcpy(getResourceAddress(rtVerb, verb), obimptr, size);
@@ -945,7 +1033,7 @@ void ScummEngine::setVerbObject(uint room, uint object, uint verb) {
 	} else {
 		findObjectInRoom(&foir, foImageHeader, object, room);
 		size = READ_BE_UINT32(foir.obim + 4);
-		res.createResource(rtVerb, verb, size);
+		_res->createResource(rtVerb, verb, size);
 		obimptr = getResourceAddress(rtRoom, room) - foir.roomptr + foir.obim;
 		memcpy(getResourceAddress(rtVerb, verb), obimptr, size);
 	}
