@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2003-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/sky/compact.cpp $
- * $Id: compact.cpp 21607 2006-04-04 22:04:51Z sev $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/sky/compact.cpp $
+ * $Id: compact.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -27,6 +30,8 @@
 #include "sky/compact.h"
 #include "gui/message.h"
 
+extern int gDebugLevel;
+
 namespace Sky {
 
 #define	SKY_CPT_SIZE	419427	
@@ -263,6 +268,11 @@ Compact *SkyCompact::fetchCpt(uint16 cptId) {
 	if (cptId == 0xFFFF) // is this really still necessary?
 		return NULL;
 	assert(((cptId >> 12) < _numDataLists) && ((cptId & 0xFFF) < _dataListLen[cptId >> 12]));
+	
+	if (gDebugLevel >= 8) {
+		debug(8, "Loading Compact %s [%s] (%04X=%d,%d)", _cptNames[cptId >> 12][cptId & 0xFFF], nameForType(_cptTypes[cptId >> 12][cptId & 0xFFF]), cptId, cptId >> 12, cptId & 0xFFF);
+	}		
+
 	return _compacts[cptId >> 12][cptId & 0xFFF];
 }
 
@@ -272,11 +282,22 @@ Compact *SkyCompact::fetchCptInfo(uint16 cptId, uint16 *elems, uint16 *type, cha
 		*elems = _cptSizes[cptId >> 12][cptId & 0xFFF];
 	if (type)
 		*type  = _cptTypes[cptId >> 12][cptId & 0xFFF];
-	if (name)
-		strcpy(name, _cptNames[cptId >> 12][cptId & 0xFFF]);
+	if (name) {
+		if (_cptNames[cptId >> 12][cptId & 0xFFF] != NULL)
+			strcpy(name, _cptNames[cptId >> 12][cptId & 0xFFF]);
+		else
+			strcpy(name, "(null)");
+	}
 	return fetchCpt(cptId);
 }
 
+const char *SkyCompact::nameForType(uint16 type) {
+	if (type >= NUM_CPT_TYPES)
+		return "unknown";
+	else
+		return _typeNames[type];
+}
+
 uint16 *SkyCompact::getSub(Compact *cpt, uint16 mode) {
 	switch (mode) {
 	case 0:
@@ -454,4 +475,15 @@ uint16 SkyCompact::giveDataListLen(uint16 listNum) {
 		return _dataListLen[listNum];
 }
 
+const char *SkyCompact::_typeNames[NUM_CPT_TYPES] = {
+	"null",
+	"COMPACT",
+	"TURNTABLE",
+	"ANIM SEQ",
+	"UNKNOWN",
+	"GETTOTABLE",
+	"AR BUFFER",
+	"MAIN LIST"
+};
+
 } // End of namespace Sky
