@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2003-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,13 +18,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/sky/compact.h $
- * $Id: compact.h 20514 2006-02-11 10:03:24Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/sky/compact.h $
+ * $Id: compact.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
-#ifndef SKYCOMPACT_H
-#define SKYCOMPACT_H
+#ifndef SKY_COMPACT_H
+#define SKY_COMPACT_H
 
 #include "sky/sky.h"
 #include "sky/struc.h"
@@ -50,7 +53,8 @@ enum CptTypeIds {
 	MISCBIN,
 	GETTOTAB,
 	ROUTEBUF,
-	MAINLIST
+	MAINLIST,
+	NUM_CPT_TYPES
 };
 
 namespace Sky {
@@ -75,6 +79,7 @@ public:
 	uint16 findCptId(const char *cptName);
 	uint16 giveNumDataLists(void);
 	uint16 giveDataListLen(uint16 listNum);
+	const char *nameForType(uint16 type);
 private:
 	uint16  _numDataLists;
 	uint16  *_dataListLen;
@@ -86,6 +91,7 @@ private:
 	uint16  **_cptTypes;
 	Common::File	*_cptFile;
 	uint32	_resetDataPos;
+	static const char *_typeNames[NUM_CPT_TYPES];
 };
 
 } // End of namespace Sky
