@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2003-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/sky/control.cpp $
- * $Id: control.cpp 23213 2006-06-21 06:18:40Z eriktorbjorn $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/sky/control.cpp $
+ * $Id: control.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -24,6 +27,7 @@
 #include "common/endian.h"
 #include "common/config-manager.h"
 #include "common/file.h"
+#include "common/events.h"
 #include "common/system.h"
 #include "common/savefile.h"
 #include "common/util.h"
@@ -253,9 +257,6 @@ void Control::initPanel(void) {
 	_screenBuf = (uint8 *)malloc(GAME_SCREEN_WIDTH * FULL_SCREEN_HEIGHT);
 	memset(_screenBuf, 0, GAME_SCREEN_WIDTH * FULL_SCREEN_HEIGHT);
 
-	_mouseX = _skyMouse->giveMouseX();
-	_mouseY = _skyMouse->giveMouseY();
-
 	uint16 volY = (127 - _skyMusic->giveVolume()) / 4 + 59 - MPNL_Y; // volume slider's Y coordinate
 	uint16 spdY = (SkyEngine::_systemVars.gameSpeed - 2) / SPEED_MULTIPLY;
 	spdY += MPNL_Y + 83; // speed slider's initial position
@@ -284,15 +285,12 @@ void Control::initPanel(void) {
 	_savePanButton    = createResource(      _sprites.button, 3, 0, 58,  39, 48, SAVE_GAME_PANEL, MAINPANEL);
 	_dosPanButton     = createResource(      _sprites.button, 3, 0, 58,  59, 93,     QUIT_TO_DOS, MAINPANEL);
 	_restartPanButton = createResource(      _sprites.button, 3, 0, 58,  79, 94,         RESTART, MAINPANEL);
-	if (SkyEngine::_systemVars.systemFlags & SF_FX_OFF)
-		_fxPanButton  = createResource(      _sprites.button, 3, 0, 58,  99, 87,       TOGGLE_FX, MAINPANEL);
-	else
-		_fxPanButton  = createResource(      _sprites.button, 3, 2, 58,  99, 86,       TOGGLE_FX, MAINPANEL);
+	_fxPanButton      = createResource(      _sprites.button, 3, 0, 58,  99, 90,       TOGGLE_FX, MAINPANEL);
 
 	if (SkyEngine::isCDVersion()) { // CD Version: Toggle text/speech
-		_musicPanButton = createResource(      _sprites.button, 3, 0, 58, 119, 52,     TOGGLE_TEXT, MAINPANEL);
+		_musicPanButton = createResource(    _sprites.button, 3, 0, 58, 119, 52,     TOGGLE_TEXT, MAINPANEL);
 	} else {                       // disk version: toggle music on/off
-		_musicPanButton = createResource(      _sprites.button, 3, 0, 58, 119, 91,       TOGGLE_MS, MAINPANEL);
+		_musicPanButton = createResource(    _sprites.button, 3, 0, 58, 119, 91,       TOGGLE_MS, MAINPANEL);
 	}
 	_bodge            = createResource(  _sprites.musicBodge, 2, 1, 98, 115,  0,      DO_NOTHING, MAINPANEL);
 	_yesNo            = createResource(       _sprites.yesNo, 1, 0, -2,  40,  0,      DO_NOTHING, MAINPANEL);
@@ -360,8 +358,9 @@ void Control::buttonControl(ConResource *pButton) {
 		} else
 			_text->setSprite(NULL);
 	}
-	int destY = (_mouseY - 16 >= 0) ? _mouseY - 16 : 0;
-	_text->setXY(_mouseX + 12, destY);
+	Common::Point mouse = _system->getEventManager()->getMousePos();
+	int destY = (mouse.y - 16 >= 0) ? mouse.y - 16 : 0;
+	_text->setXY(mouse.x + 12, destY);
 }
 
 void Control::drawTextCross(uint32 flags) {
@@ -441,7 +440,7 @@ void Control::doLoadSavePanel(void) {
 
 	_savedMouse = _skyMouse->giveCurrentMouseType();
 	_savedCharSet = _skyText->giveCurrentCharSet();
-	_skyText->fnSetFont(0);
+	_skyText->fnSetFont(2);
 	_skyMouse->spriteMouse(MOUSE_NORMAL, 0, 0);
 	_lastButton = -1;
 	_curButtonText = 0;
@@ -466,7 +465,7 @@ void Control::doControlPanel(void) {
 	initPanel();
 
 	_savedCharSet = _skyText->giveCurrentCharSet();
-	_skyText->fnSetFont(0);
+	_skyText->fnSetFont(2);
 
 	_skyScreen->clearScreen();
 	if (SkyEngine::_systemVars.gameVersion < 331)
@@ -474,6 +473,15 @@ void Control::doControlPanel(void) {
 	else
 		_skyScreen->setPalette(60510);
 
+	// Set initial button lights
+	_fxPanButton->_curSprite =
+		(SkyEngine::_systemVars.systemFlags & SF_FX_OFF ? 0 : 2);
+
+	// music button only available in floppy version
+	if (!SkyEngine::isCDVersion())
+		_musicPanButton->_curSprite =
+			(SkyEngine::_systemVars.systemFlags & SF_MUS_OFF ? 0 : 2);
+
 	drawMainPanel();
 
 	_savedMouse = _skyMouse->giveCurrentMouseType();
@@ -494,8 +502,9 @@ void Control::doControlPanel(void) {
 			quitPanel = true;
 		}
 		bool haveButton = false;
+		Common::Point mouse = _system->getEventManager()->getMousePos();
 		for (uint8 lookCnt = 0; lookCnt < 9; lookCnt++) {
-			if (_controlPanLookList[lookCnt]->isMouseOver(_mouseX, _mouseY)) {
+			if (_controlPanLookList[lookCnt]->isMouseOver(mouse.x, mouse.y)) {
 				haveButton = true;
 				buttonControl(_controlPanLookList[lookCnt]);
 				if (_mouseClicked && _controlPanLookList[lookCnt]->_onClick) {
@@ -529,7 +538,7 @@ uint16 Control::handleClick(ConResource *pButton) {
 	char quitDos[] = "Quit to DOS?";
 	char restart[] = "Restart?";
 
-	switch(pButton->_onClick) {
+	switch (pButton->_onClick) {
 	case DO_NOTHING:
 		return 0;
 	case REST_GAME_PANEL:
@@ -573,10 +582,10 @@ uint16 Control::handleClick(ConResource *pButton) {
 		_mouseClicked = true;
 		return doMusicSlide();
 	case TOGGLE_FX:
-		return toggleFx(pButton);
+		toggleFx(pButton);
+		return TOGGLED;
 	case TOGGLE_MS:
-		animClick(pButton);
-		toggleMusic();
+		toggleMusic(pButton);
 		return TOGGLED;
 	case TOGGLE_TEXT:
 		animClick(pButton);
@@ -628,14 +637,15 @@ bool Control::getYesNo(char *text) {
 		}
 		_system->updateScreen();
 		delay(50);
-		if ((_mouseY >= 83) && (_mouseY <= 110)) {
-			if ((_mouseX >= 77) && (_mouseX <= 114)) { // over 'yes'
+		Common::Point mouse = _system->getEventManager()->getMousePos();
+		if ((mouse.y >= 83) && (mouse.y <= 110)) {
+			if ((mouse.x >= 77) && (mouse.x <= 114)) { // over 'yes'
 				wantMouse = MOUSE_CROSS;
 				if (_mouseClicked) {
 					quitPanel = true;
 					retVal = true;
 				}
-			} else if ((_mouseX >= 156) && (_mouseX <= 193)) { // over 'no'
+			} else if ((mouse.x >= 156) && (mouse.x <= 193)) { // over 'no'
 				wantMouse = MOUSE_CROSS;
 				if (_mouseClicked) {
 					quitPanel = true;
@@ -647,6 +657,7 @@ bool Control::getYesNo(char *text) {
 			wantMouse = MOUSE_NORMAL;
 	}
 	_mouseClicked = false;
+	_skyMouse->spriteMouse(MOUSE_NORMAL, 0, 0);
 	if (dlgTextDat)
 		free(dlgTextDat);
 	delete dlgText;
@@ -655,11 +666,13 @@ bool Control::getYesNo(char *text) {
 
 uint16 Control::doMusicSlide(void) {
 
-	int ofsY = _slide2->_y - _mouseY;
+	Common::Point mouse = _system->getEventManager()->getMousePos();
+	int ofsY = _slide2->_y - mouse.y;
 	uint8 volume;
 	while (_mouseClicked) {
 		delay(50);
-		int newY = ofsY + _mouseY;
+		mouse = _system->getEventManager()->getMousePos();
+		int newY = ofsY + mouse.y;
 		if (newY < 59) newY = 59;
 		if (newY > 91) newY = 91;
 		if (newY != _slide2->_y) {
@@ -681,13 +694,15 @@ uint16 Control::doMusicSlide(void) {
 
 uint16 Control::doSpeedSlide(void) {
 
-	int ofsY = _slide->_y - _mouseY;
+	Common::Point mouse = _system->getEventManager()->getMousePos();
+	int ofsY = _slide->_y - mouse.y;
 	uint16 speedDelay = _slide->_y - (MPNL_Y + 93);
 	speedDelay *= SPEED_MULTIPLY;
 	speedDelay += 2;
 	while (_mouseClicked) {
 		delay(50);
-		int newY = ofsY + _mouseY;
+		mouse = _system->getEventManager()->getMousePos();
+		int newY = ofsY + mouse.y;
 		if (newY < MPNL_Y + 93) newY = MPNL_Y + 93;
 		if (newY > MPNL_Y + 104) newY = MPNL_Y + 104;
 		if ((newY == 110) || (newY == 108)) newY = 109;
@@ -708,25 +723,21 @@ uint16 Control::doSpeedSlide(void) {
 	return SPEED_CHANGED;
 }
 
-uint16 Control::toggleFx(ConResource *pButton) {
+void Control::toggleFx(ConResource *pButton) {
 
 	SkyEngine::_systemVars.systemFlags ^= SF_FX_OFF;
 	if (SkyEngine::_systemVars.systemFlags & SF_FX_OFF) {
 		pButton->_curSprite = 0;
-		pButton->_text = 0x7000 + 87;
 		_statusBar->setToText(0x7000 + 87);
 	} else {
 		pButton->_curSprite = 2;
-		pButton->_text = 0x7000 + 86;
 		_statusBar->setToText(0x7000 + 86);
 	}
 
 	ConfMan.setBool("sfx_mute", (SkyEngine::_systemVars.systemFlags & SF_FX_OFF) != 0);
 
 	pButton->drawToScreen(WITH_MASK);
-	buttonControl(pButton);
 	_system->updateScreen();
-	return TOGGLED;
 }
 
 uint16 Control::toggleText(void) {
@@ -756,17 +767,23 @@ uint16 Control::toggleText(void) {
 	return TOGGLED;
 }
 
-void Control::toggleMusic(void) {
+void Control::toggleMusic(ConResource *pButton) {
 
+	SkyEngine::_systemVars.systemFlags ^= SF_MUS_OFF;
 	if (SkyEngine::_systemVars.systemFlags & SF_MUS_OFF) {
-		SkyEngine::_systemVars.systemFlags &= ~SF_MUS_OFF;
-		_skyMusic->startMusic(SkyEngine::_systemVars.currentMusic);
-		_statusBar->setToText(0x7000 + 88);
-	} else {
-		SkyEngine::_systemVars.systemFlags |= SF_MUS_OFF;
 		_skyMusic->startMusic(0);
+		pButton->_curSprite = 0;
 		_statusBar->setToText(0x7000 + 89);
+	} else {
+		_skyMusic->startMusic(SkyEngine::_systemVars.currentMusic);
+		pButton->_curSprite = 2;
+		_statusBar->setToText(0x7000 + 88);
 	}
+
+	ConfMan.setBool("music_mute", (SkyEngine::_systemVars.systemFlags & SF_MUS_OFF) != 0);
+
+	pButton->drawToScreen(WITH_MASK);
+	_system->updateScreen();
 }
 
 uint16 Control::shiftDown(uint8 speed) {
@@ -911,8 +928,9 @@ uint16 Control::saveRestorePanel(bool allowSave) {
 		}
 
 		bool haveButton = false;
+		Common::Point mouse = _system->getEventManager()->getMousePos();
 		for (cnt = 0; cnt < lookListLen; cnt++)
-			if (lookList[cnt]->isMouseOver(_mouseX, _mouseY)) {
+			if (lookList[cnt]->isMouseOver(mouse.x, mouse.y)) {
 				buttonControl(lookList[cnt]);
 				haveButton = true;
 
@@ -942,10 +960,10 @@ uint16 Control::saveRestorePanel(bool allowSave) {
 			}
 
 		if (_mouseClicked) {
-			if ((_mouseX >= GAME_NAME_X) && (_mouseX <= GAME_NAME_X + PAN_LINE_WIDTH) &&
-				(_mouseY >= GAME_NAME_Y) && (_mouseY <= GAME_NAME_Y + PAN_CHAR_HEIGHT * MAX_ON_SCREEN)) {
+			if ((mouse.x >= GAME_NAME_X) && (mouse.x <= GAME_NAME_X + PAN_LINE_WIDTH) &&
+				(mouse.y >= GAME_NAME_Y) && (mouse.y <= GAME_NAME_Y + PAN_CHAR_HEIGHT * MAX_ON_SCREEN)) {
 
-					_selectedGame = (_mouseY - GAME_NAME_Y) / PAN_CHAR_HEIGHT + _firstText;
+					_selectedGame = (mouse.y - GAME_NAME_Y) / PAN_CHAR_HEIGHT + _firstText;
 					refreshNames = true;
 			}
 		}
@@ -1129,7 +1147,7 @@ void Control::saveDescriptions(uint8 *srcBuf) {
 	bool ioFailed = true;
 	if (outf) {
 		outf->write(tmpBuf, tmpPos - tmpBuf);
-		outf->flush();
+		outf->finalize();
 		if (!outf->ioFailed())
 			ioFailed = false;
 		delete outf;
@@ -1156,7 +1174,7 @@ void Control::doAutoSave(void) {
 	uint32 fSize = prepareSaveData(saveData);
 
 	outf->write(saveData, fSize);
-	outf->flush();
+	outf->finalize();
 
 	if (outf->ioFailed())
 		displayMessage(0, "Unable to write autosave file '%s' in directory '%s'. Disk full?", fName, _saveFileMan->getSavePath());
@@ -1178,7 +1196,7 @@ uint16 Control::saveGameToFile(void) {
 	uint32 fSize = prepareSaveData(saveData);
 
 	uint32 writeRes = outf->write(saveData, fSize);
-	outf->flush();
+	outf->finalize();
 	if (outf->ioFailed())
 		writeRes = 0;
 	free(saveData);
@@ -1478,7 +1496,7 @@ uint16 Control::quickXRestore(uint16 slot) {
 	_mouseClicked = false;
 
 	_savedCharSet = _skyText->giveCurrentCharSet();
-	_skyText->fnSetFont(0);
+	_skyText->fnSetFont(2);
 
 	_system->copyRectToScreen(_screenBuf, GAME_SCREEN_WIDTH, 0, 0, FULL_SCREEN_WIDTH, FULL_SCREEN_HEIGHT);
 	_system->updateScreen();
@@ -1534,45 +1552,42 @@ void Control::restartGame(void) {
 
 void Control::delay(unsigned int amount) {
 
-	OSystem::Event event;
+	Common::Event event;
 
 	uint32 start = _system->getMillis();
 	uint32 cur = start;
 	_keyPressed = 0;	//reset
 
 	do {
-		while (_system->pollEvent(event)) {
+		Common::EventManager *eventMan = _system->getEventManager();
+		while (eventMan->pollEvent(event)) {
 			switch (event.type) {
-			case OSystem::EVENT_KEYDOWN:
+			case Common::EVENT_KEYDOWN:
 				// Make sure backspace works right (this fixes a small issue on OS X)
 				if (event.kbd.keycode == 8)
 					_keyPressed = 8;
 				else
 					_keyPressed = (byte)event.kbd.ascii;
 				break;
-			case OSystem::EVENT_MOUSEMOVE:
-				_mouseX = event.mouse.x;
-				_mouseY = event.mouse.y;
+			case Common::EVENT_MOUSEMOVE:
+				if (!(SkyEngine::_systemVars.systemFlags & SF_MOUSE_LOCKED))
+					_skyMouse->mouseMoved(event.mouse.x, event.mouse.y);
 				break;
-			case OSystem::EVENT_LBUTTONDOWN:
-#ifdef PALMOS_MODE
-				_mouseX = event.mouse.x;
-				_mouseY = event.mouse.y;
-#endif
+			case Common::EVENT_LBUTTONDOWN:
 				_mouseClicked = true;
 				break;
-			case OSystem::EVENT_LBUTTONUP:
+			case Common::EVENT_LBUTTONUP:
 				_mouseClicked = false;
 				break;
-			case OSystem::EVENT_RBUTTONDOWN:
+			case Common::EVENT_RBUTTONDOWN:
 				break;
-			case OSystem::EVENT_WHEELUP:
+			case Common::EVENT_WHEELUP:
 				_mouseWheel = -1;
 				break;
-			case OSystem::EVENT_WHEELDOWN:
+			case Common::EVENT_WHEELDOWN:
 				_mouseWheel = 1;
 				break;
-			case OSystem::EVENT_QUIT:
+			case Common::EVENT_QUIT:
 				SkyEngine::_systemVars.quitGame = true;
 				break;
 			default:
