@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2003-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,13 +18,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/sky/control.h $
- * $Id: control.h 21459 2006-03-26 06:22:41Z eriktorbjorn $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/sky/control.h $
+ * $Id: control.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
-#ifndef CONTROL_H
-#define CONTROL_H
+#ifndef SKY_CONTROL_H
+#define SKY_CONTROL_H
 
 #include "common/stdafx.h"
 #include "common/scummsys.h"
@@ -133,10 +136,10 @@ struct AllocedMem {
 class ConResource {
 public:
 	ConResource(void *pSpData, uint32 pNSprites, uint32 pCurSprite, uint16 pX, uint16 pY, uint32 pText, uint8 pOnClick, OSystem *system, uint8 *screen);
-	virtual ~ConResource(void) {};
-	void setSprite(void *pSpData) { _spriteData = (dataFileHeader*)pSpData; };
-	void setText(uint32 pText) { if (pText) _text = pText + 0x7000; else _text = 0; };
-	void setXY(uint16 x, uint16 y) { _x = x; _y = y; };
+	virtual ~ConResource(void) {}
+	void setSprite(void *pSpData) { _spriteData = (dataFileHeader*)pSpData; }
+	void setText(uint32 pText) { if (pText) _text = pText + 0x7000; else _text = 0; }
+	void setXY(uint16 x, uint16 y) { _x = x; _y = y; }
 	bool isMouseOver(uint32 mouseX, uint32 mouseY);
 	virtual void drawToScreen(bool doMask);
 
@@ -203,9 +206,9 @@ private:
 	uint16 handleClick(ConResource *pButton);
 	uint16 doMusicSlide(void);
 	uint16 doSpeedSlide(void);
-	uint16 toggleFx(ConResource *pButton);
+	void toggleFx(ConResource *pButton);
 	uint16 toggleText(void);
-	void toggleMusic(void);
+	void toggleMusic(ConResource *pButton);
 	uint16 shiftDown(uint8 speed);
 	uint16 shiftUp(uint8 speed);
 	void drawTextCross(uint32 flags);
@@ -239,7 +242,6 @@ private:
 	Logic *_skyLogic;
 	Sound *_skySound;
 	OSystem *_system;
-	int _mouseX, _mouseY;
 	bool _mouseClicked;
 	byte _keyPressed;
 	int _mouseWheel;
