@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2003-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/sky/debug.h $
- * $Id: debug.h 20514 2006-02-11 10:03:24Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/sky/debug.h $
+ * $Id: debug.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -25,7 +28,7 @@
 
 #include "common/stdafx.h"
 #include "common/scummsys.h"
-#include "common/debugger.h"
+#include "gui/debugger.h"
 
 namespace Sky {
 
@@ -34,7 +37,7 @@ class Mouse;
 class Screen;
 class SkyCompact;
 
-class Debugger : public Common::Debugger<Debugger> {
+class Debugger : public GUI::Debugger {
 public:
 	Debugger(Logic *logic, Mouse *mouse, Screen *screen, SkyCompact *skyCompact);
 	virtual ~Debugger(); // we need this here for __SYMBIAN32__ archaic gcc/UIQ
@@ -44,8 +47,6 @@ protected:
 	virtual void preEnter();
 	virtual void postEnter();
 
-	bool Cmd_Exit(int argc, const char **argv);
-	bool Cmd_Help(int argc, const char **argv);
 	bool Cmd_ShowGrid(int argc, const char **argv);
 	bool Cmd_ReloadGrid(int argc, const char **argv);
 	bool Cmd_ShowCompact(int argc, const char **argv);
@@ -53,6 +54,9 @@ protected:
 	bool Cmd_Info(int argc, const char **argv);
 	bool Cmd_ScriptVar(int argc, const char **argv);
 	bool Cmd_Section(int argc, const char **argv);
+	bool Cmd_LogicList(int argc, const char **argv);
+
+	void dumpCompact(uint16 cptId);
 
 	Logic *_logic;
 	Mouse *_mouse;
@@ -65,7 +69,6 @@ protected:
 
 class Debug {
 public:
-	static void fetchCompact(uint32 a);
 	static void logic(uint32 logic);
 	static void script(uint32 command, uint16 *scriptData);
 	static void mcode(uint32 mcode, uint32 a, uint32 b, uint32 c);
