@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2003-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/sky/disk.cpp $
- * $Id: disk.cpp 22272 2006-05-01 22:27:56Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/sky/disk.cpp $
+ * $Id: disk.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -37,17 +40,17 @@ static const char *dinnerFilename = "sky.dnr";
 
 Disk::Disk() {
 	_dataDiskHandle = new Common::File();
-	_dnrHandle = new Common::File();
+	Common::File *dnrHandle = new Common::File();
 
-	_dnrHandle->open(dinnerFilename);
-	if (!_dnrHandle->isOpen())
+	dnrHandle->open(dinnerFilename);
+	if (!dnrHandle->isOpen())
 		error("Could not open %s", dinnerFilename);
 
-	if (!(_dinnerTableEntries = _dnrHandle->readUint32LE()))
+	if (!(_dinnerTableEntries = dnrHandle->readUint32LE()))
 		error("Error reading from sky.dnr"); //even though it was opened correctly?!
 
 	_dinnerTableArea = (uint8 *)malloc(_dinnerTableEntries * 8);
-	uint32 entriesRead = _dnrHandle->read(_dinnerTableArea, 8 * _dinnerTableEntries) / 8;
+	uint32 entriesRead = dnrHandle->read(_dinnerTableArea, 8 * _dinnerTableEntries) / 8;
 
 	if (entriesRead != _dinnerTableEntries)
 		error("entriesRead != dinnerTableEntries. [%d/%d]", entriesRead, _dinnerTableEntries);
@@ -60,16 +63,16 @@ Disk::Disk() {
 
 	memset(_buildList, 0, 60 * 2);
 	memset(_loadedFilesList, 0, 60 * 4);
+
+	dnrHandle->close();
+	delete dnrHandle;
 }
 
 Disk::~Disk(void) {
-	if (_dnrHandle->isOpen())
-		_dnrHandle->close();
 	if (_dataDiskHandle->isOpen())
 		_dataDiskHandle->close();
 	fnFlushBuffers();
 	free(_dinnerTableArea);
-	delete _dnrHandle;
 	delete _dataDiskHandle;
 }
 
