@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2003-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,13 +18,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/sky/disk.h $
- * $Id: disk.h 22272 2006-05-01 22:27:56Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/sky/disk.h $
+ * $Id: disk.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
-#ifndef SKYDISK_H
-#define SKYDISK_H
+#ifndef SKY_DISK_H
+#define SKY_DISK_H
 
 #include "common/stdafx.h"
 #include "common/scummsys.h"
@@ -54,7 +57,7 @@ public:
 	void fnCacheChip(uint16 *fList);
 	void fnCacheFiles(void);
 	void fnFlushBuffers(void);
-	uint32 *giveLoadedFilesList(void) { return _loadedFilesList; };
+	uint32 *giveLoadedFilesList(void) { return _loadedFilesList; }
 	void refreshFilesList(uint32 *list);
 
 protected:
@@ -64,7 +67,6 @@ protected:
 	uint32 _dinnerTableEntries;
 	uint8 *_dinnerTableArea;
 	Common::File *_dataDiskHandle;
-	Common::File *_dnrHandle;
 	RncDecoder _rncDecoder;
 
 	uint16 _buildList[MAX_FILES_IN_LIST];
