@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2003-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,13 +18,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/sky/grid.h $
- * $Id: grid.h 20514 2006-02-11 10:03:24Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/sky/grid.h $
+ * $Id: grid.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
-#ifndef SKYGRID_H
-#define SKYGRID_H
+#ifndef SKY_GRID_H
+#define SKY_GRID_H
 
 #include "common/stdafx.h"
 #include "common/scummsys.h"
