@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2003-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/sky/logic.cpp $
- * $Id: logic.cpp 23992 2006-09-26 06:09:29Z eriktorbjorn $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/sky/logic.cpp $
+ * $Id: logic.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -224,7 +227,7 @@ void Logic::arAnim() {
 	// On character boundary. Have we been told to wait?
 	// if not - are WE colliding?
 
-	if (_compact->waitingFor == 0xffff) { // 1st cycle of re-route does
+	if (_compact->waitingFor == 0xffff) { // 1st cycle of re-route does not require collision checks
 		mainAnim();
 		return;
 	}
@@ -2512,16 +2515,17 @@ void Logic::stdSpeak(Compact *target, uint32 textNum, uint32 animNum, uint32 bas
 	bool speechFileFound = false;
 	if (SkyEngine::isCDVersion())
 		speechFileFound = _skySound->startSpeech((uint16)textNum);
-	
-	// Calculate the point where the character is
-	int x = (((uint32) (target->xcood)) * GAME_SCREEN_WIDTH) >> 9;
-	int y = ((((uint32) (target->ycood)) * GAME_SCREEN_HEIGHT) >> 9);
+
 
 	// Set the focus region to that area
+	// Calculate the point where the character is
+	int x = target->xcood - TOP_LEFT_X;
+	int y = target->ycood - TOP_LEFT_Y;
 	// TODO: Make the box size change based on the object that has the focus
 	Common::Rect rect(x - 96, y - 64, x + 96, y + 64);
 	_skyScreen->setFocusRectangle(rect);
 
+
 	if ((SkyEngine::_systemVars.systemFlags & SF_ALLOW_TEXT) || !speechFileFound) {
 		// form the text sprite, if player wants subtitles or
 		// if we couldn't find the speech file
