@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2003-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,12 +18,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/sky/mouse.cpp $
- * $Id: mouse.cpp 23234 2006-06-21 15:06:16Z eriktorbjorn $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/sky/mouse.cpp $
+ * $Id: mouse.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
 #include "common/stdafx.h"
+#include "common/events.h"
 #include "common/system.h"
 #include "graphics/cursorman.h"
 #include "sky/disk.h"
@@ -171,24 +175,27 @@ void Mouse::waitMouseNotPressed(int minDelay) {
 
 	bool mousePressed = true;
 	uint32 now = _system->getMillis();
-	OSystem::Event event;
+	Common::Event event;
+	Common::EventManager *eventMan = _system->getEventManager();
 	while (mousePressed || _system->getMillis() < now + minDelay) {
-		while (_system->pollEvent(event)) {
+
+		if (eventMan->shouldQuit()) {
+			SkyEngine::_systemVars.quitGame = true;
+			minDelay = 0;
+			mousePressed = false;
+		}
+
+		if (!eventMan->getButtonState())
+			mousePressed = false;
+
+		while (eventMan->pollEvent(event)) {
 			switch (event.type) {
-			case OSystem::EVENT_LBUTTONUP:
-				mousePressed = false;
-				break;
-			case OSystem::EVENT_KEYDOWN:
+			case Common::EVENT_KEYDOWN:
 				if (event.kbd.ascii == 27) {
 					minDelay = 0;
 					mousePressed = false;
 				}
 				break;
-			case OSystem::EVENT_QUIT:
-				SkyEngine::_systemVars.quitGame = true;
-				minDelay = 0;
-				mousePressed = false;
-				break;
 			default:
 				break;
 			}
@@ -216,15 +223,12 @@ void Mouse::spriteMouse(uint16 frameNum, uint8 mouseX, uint8 mouseY) {
 		CursorMan.showMouse(true);
 }
 
-void Mouse::mouseEngine(uint16 mouseX, uint16 mouseY) {
-	_mouseX = mouseX;
-	_mouseY = mouseY;
-
+void Mouse::mouseEngine() {
 	_logicClick = (_mouseB > 0); // click signal is available for Logic for one gamecycle
 
 	if (!Logic::_scriptVariables[MOUSE_STOP]) {
 		if (Logic::_scriptVariables[MOUSE_STATUS] & (1 << 1)) {
-			pointerEngine(mouseX + TOP_LEFT_X, mouseY + TOP_LEFT_Y);
+			pointerEngine(_mouseX + TOP_LEFT_X, _mouseY + TOP_LEFT_Y);
 			if (Logic::_scriptVariables[MOUSE_STATUS] & (1 << 2)) //buttons enabled?
 				buttonEngine1();
 		}
@@ -271,10 +275,14 @@ void Mouse::pointerEngine(uint16 xPos, uint16 yPos) {
 }
 
 void Mouse::buttonPressed(uint8 button) {
-
 	_mouseB = button;
 }
 
+void Mouse::mouseMoved(uint16 mouseX, uint16 mouseY) {
+	_mouseX = mouseX;
+	_mouseY = mouseY;
+}
+
 void Mouse::buttonEngine1(void) {
 	//checks for clicking on special item
 	//"compare the size of this routine to S1 mouse_button"
