@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2003-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,13 +18,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/sky/mouse.h $
- * $Id: mouse.h 23234 2006-06-21 15:06:16Z eriktorbjorn $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/sky/mouse.h $
+ * $Id: mouse.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
-#ifndef SKYMOUSE_H
-#define SKYMOUSE_H
+#ifndef SKY_MOUSE_H
+#define SKY_MOUSE_H
 
 #include "common/stdafx.h"
 #include "common/scummsys.h"
@@ -41,7 +44,7 @@ public:
 	Mouse(OSystem *system, Disk *skyDisk, SkyCompact *skyCompact);
 	~Mouse(void);
 
-	void mouseEngine(uint16 mouseX, uint16 mouseY);
+	void mouseEngine();
 	void replaceMouseCursors(uint16 fileNo);
 	bool fnAddHuman(void);
 	void fnSaveCoods(void);
@@ -54,6 +57,7 @@ public:
 	void spriteMouse(uint16 frameNum, uint8 mouseX, uint8 mouseY);
 	void useLogicInstance(Logic *skyLogic) { _skyLogic = skyLogic; };
 	void buttonPressed(uint8 button);
+	void mouseMoved(uint16 mouseX, uint16 mouseY);
 	void waitMouseNotPressed(int minDelay = 0);
 	uint16 giveMouseX(void) { return _mouseX; };
 	uint16 giveMouseY(void) { return _mouseY; };
