@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2003-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/sky/music/adlibmusic.cpp $
- * $Id: adlibmusic.cpp 21500 2006-03-29 15:59:37Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/sky/music/adlibmusic.cpp $
+ * $Id: adlibmusic.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -39,46 +42,41 @@ AdlibMusic::AdlibMusic(Audio::Mixer *pMixer, Disk *pDisk)
 
 	_opl = makeAdlibOPL(_sampleRate);
 
-	_mixer->setupPremix(this);
+	_mixer->playInputStream(Audio::Mixer::kMusicSoundType, &_soundHandle, this, -1, Audio::Mixer::kMaxChannelVolume, 0, false, true);
 }
 
 AdlibMusic::~AdlibMusic(void) {
 
-	_mixer->setupPremix(0);
+	_mixer->stopHandle(_soundHandle);
 }
 
-void AdlibMusic::premixerCall(int16 *data, uint len) {
+int AdlibMusic::readBuffer(int16 *data, const int numSamples) {
 
 	if (_musicData == NULL) {
 		// no music loaded
-		memset(data, 0, 2 * len * sizeof(int16));
+		memset(data, 0, numSamples * sizeof(int16));
 	} else if ((_currentMusic == 0) || (_numberOfChannels == 0)) {
 		// music loaded but not played as of yet
-		memset(data, 0, 2 * len * sizeof(int16));
+		memset(data, 0, numSamples * sizeof(int16));
 		// poll anyways as pollMusic() can activate the music
 		pollMusic();
-		_nextMusicPoll = _sampleRate/50;
+		_nextMusicPoll = _sampleRate / 50;
 	} else {
 		uint32 render;
-		int16 *origData = data;
-		uint origLen = len;
-		while (len) {
-			render = (len > _nextMusicPoll) ? (_nextMusicPoll) : (len);
-			len -= render;
+		uint remaining = numSamples;
+		while (remaining) {
+			render = (remaining > _nextMusicPoll) ? _nextMusicPoll : remaining;
+			remaining -= render;
 			_nextMusicPoll -= render;
 			YM3812UpdateOne(_opl, data, render);
 			data += render;
 			if (_nextMusicPoll == 0) {
 				pollMusic();
-				_nextMusicPoll = _sampleRate/50;
+				_nextMusicPoll = _sampleRate / 50;
 			}
 		}
-
-		// Convert mono data to stereo
-		for (int i = (origLen - 1); i >= 0; i--) {
-			origData[2 * i] = origData[2 * i + 1] = origData[i];
-		}
 	}
+	return numSamples;
 }
 
 void AdlibMusic::setupPointers(void) {
@@ -105,20 +103,35 @@ void AdlibMusic::setupChannels(uint8 *channelData) {
 	_numberOfChannels = channelData[0];
 	channelData++;
 	for (uint8 cnt = 0; cnt < _numberOfChannels; cnt++) {
-		uint16 chDataStart = ((channelData[(cnt << 1) | 1] << 8) | channelData[cnt << 1]) + _musicDataLoc;
+		uint16 chDataStart = READ_LE_UINT16((uint16 *)channelData + cnt) + _musicDataLoc;
 		_channels[cnt] = new AdlibChannel(_opl, _musicData, chDataStart);
-		_channels[cnt]->updateVolume(_musicVolume);
 	}
 }
 
 void AdlibMusic::startDriver(void) {
 
 	uint16 cnt = 0;
-	while (_initSequence[cnt] || _initSequence[cnt+1]) {
-		OPLWriteReg (_opl, _initSequence[cnt], _initSequence[cnt+1]);
+	while (_initSequence[cnt] || _initSequence[cnt + 1]) {
+		OPLWriteReg (_opl, _initSequence[cnt], _initSequence[cnt + 1]);
 		cnt += 2;
 	}
-	_allowedCommands = 0xD;
+}
+
+void AdlibMusic::setVolume(uint16 param) {
+	_musicVolume = param;
+	_mixer->setVolumeForSoundType(Audio::Mixer::kMusicSoundType, 2 * param);
+}
+
+bool AdlibMusic::isStereo(void) const { 
+	return false; 
+}
+
+bool AdlibMusic::endOfData(void) const {
+	return false; 
+}
+
+int AdlibMusic::getRate(void) const { 
+	return _sampleRate; 
 }
 
 } // End of namespace Sky
