@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2003-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,21 +18,18 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/sky/music/adlibmusic.h $
- * $Id: adlibmusic.h 22231 2006-04-29 22:33:31Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/sky/music/adlibmusic.h $
+ * $Id: adlibmusic.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
-#ifndef ADLIBMUSIC_H
-#define ADLIBMUSIC_H
+#ifndef SKY_MUSIC_ADLIBMUSIC_H
+#define SKY_MUSIC_ADLIBMUSIC_H
 
 #include "sky/music/musicbase.h"
 #include "sound/audiostream.h"
 #include "sound/fmopl.h"
-
-namespace Audio {
-	class Mixer;
-}
+#include "sound/mixer.h"
 
 namespace Sky {
 
@@ -39,17 +39,16 @@ public:
 	~AdlibMusic(void);
 
 	// AudioStream API
-	int readBuffer(int16 *buffer, const int numSamples) {
-		premixerCall(buffer, numSamples / 2);
-		return numSamples;
-	}
-	bool isStereo() const { return true; }
-	bool endOfData() const { return false; }
-	int getRate() const { return _sampleRate; }
+	int readBuffer(int16 *buffer, const int numSamples);
+	bool isStereo(void) const;
+	bool endOfData(void) const;
+	int getRate(void) const;
+	virtual void setVolume(uint16 param);
 
 private:
 	FM_OPL *_opl;
 	Audio::Mixer *_mixer;
+	Audio::SoundHandle _soundHandle;
 	uint8 *_initSequence;
 	uint32 _sampleRate, _nextMusicPoll;
 	virtual void setupPointers(void);
