@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2003-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,13 +18,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/sky/music/gmchannel.h $
- * $Id: gmchannel.h 20514 2006-02-11 10:03:24Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/sky/music/gmchannel.h $
+ * $Id: gmchannel.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
-#ifndef SKYGMCHANNEL_H
-#define SKYGMCHANNEL_H
+#ifndef SKY_MUSIC_GMCHANNEL_H
+#define SKY_MUSIC_GMCHANNEL_H
 
 #include "sky/music/musicbase.h"
 
@@ -32,15 +35,16 @@ namespace Sky {
 typedef struct {
 	uint16 eventDataPtr;
 	int32 nextEventTime;
-	uint16 startOfData;
+	uint16 loopPoint;
 	uint8 midiChannelNumber;
 	uint8 note;
-	uint8 channelActive;
+	bool channelActive;
 } MidiChannelType;
 
 class GmChannel : public ChannelBase {
 public:
 	GmChannel(uint8 *pMusicData, uint16 startOfData, MidiDriver *pMidiDrv, const byte *pInstMap, const byte *veloTab);
+	~GmChannel(void);
 	virtual void stopNote(void);
 	virtual uint8 process(uint16 aktTime);
 	virtual void updateVolume(uint16 pVolume);
@@ -52,7 +56,7 @@ private:
 	uint8 *_musicData;
 	uint16 _musicVolume;
 	MidiChannelType _channelData;
-	uint8 _lastVolume;
+	uint8 _currentChannelVolume;
 	//-                          normal subs
 	void setRegister(uint8 regNum, uint8 value);
 	int32 getNextEventTime(void);
@@ -69,11 +73,11 @@ private:
 	void com90_getPitch(void);           // 5
 	void com90_getChannelVolume(void);   // 6
 	//void com90_skipTremoVibro(void);   // 7
-	void com90_rewindMusic(void);        // 8
+	void com90_loopMusic(void);        // 8
 	void com90_keyOff(void);             // 9
 	//void com90_error(void);            // 10
 	void com90_getChannelPanValue(void); // 11
-	void com90_setStartOfData(void);     // 12
+	void com90_setLoopPoint(void);     // 12
 	void com90_getChannelControl(void);     // 13
 };
 
