@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2003-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,13 +18,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/sky/music/musicbase.h $
- * $Id: musicbase.h 23160 2006-06-17 15:59:42Z eriktorbjorn $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/sky/music/musicbase.h $
+ * $Id: musicbase.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
-#ifndef MUSICBASE_H
-#define MUSICBASE_H
+#ifndef SKY_MUSIC_MUSICBASE_H
+#define SKY_MUSIC_MUSICBASE_H
 
 #include "common/stdafx.h"
 #include "common/scummsys.h"
@@ -34,14 +37,12 @@ class Disk;
 #define FILES_PER_SECTION 4
 
 typedef struct {
-	bool doReInit, doStopMusic;
 	uint8 musicToProcess;
 } Actions;
 
 class ChannelBase {
 public:
-	virtual ~ChannelBase() {};
-	virtual void stopNote(void) = 0;
+	virtual ~ChannelBase() {}
 	virtual uint8 process(uint16 aktTime) = 0;
 	virtual void updateVolume(uint16 pVolume) = 0;
 	virtual bool isActive(void) = 0;
@@ -53,18 +54,18 @@ public:
 	MusicBase(Disk *pDisk);
 	virtual ~MusicBase(void);
 	void loadSection(uint8 pSection);
-	void startMusic(uint16 param) { _onNextPoll.musicToProcess = param & 0xF; }; // 4
-	void stopMusic();                                                            // 7
+	void startMusic(uint16 param);
+	void stopMusic();
 	bool musicIsPlaying(void);
-	uint8 giveVolume(void) { return (uint8)_musicVolume; };
-	uint8 giveCurrentMusic(void) { return _currentMusic; };
-	void setVolume(uint16 param);
+	uint8 giveVolume(void);
+	uint8 giveCurrentMusic(void);
+	virtual void setVolume(uint16 param) = 0;
 
 protected:
 
 	Disk *_skyDisk;
 	uint8 *_musicData;
-	uint8 _allowedCommands;
+
 	uint16 _musicDataLoc;
 	uint16 _driverFileBase;
 
