@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2003-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/sky/rnc_deco.cpp $
- * $Id: rnc_deco.cpp 21500 2006-03-29 15:59:37Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/sky/rnc_deco.cpp $
+ * $Id: rnc_deco.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -200,7 +203,7 @@ int32 RncDecoder::unpackM1(const void *input, void *output, uint16 key) {
 	inputptr = (((const uint8 *)input) + HEADER_LEN);
 	_srcPtr = inputptr;
 
-	inputHigh = ((const uint8 *)input) + packLen + HEADER_LEN;;
+	inputHigh = ((const uint8 *)input) + packLen + HEADER_LEN;
 	outputLow = (uint8 *)output;
 	outputHigh = *(((const uint8 *)input) + 16) + unpackLen + outputLow;
 
