@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2003-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,13 +18,14 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/sky/screen.cpp $
- * $Id: screen.cpp 23988 2006-09-25 21:08:03Z agent-q $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/sky/screen.cpp $
+ * $Id: screen.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
 #include "common/stdafx.h"
 #include "common/endian.h"
+#include "common/events.h"
 #include "common/system.h"
 
 #include "sky/disk.h"
@@ -154,10 +158,12 @@ void Screen::setPalette(uint16 fileNum) {
 }
 
 void Screen::showScreen(uint16 fileNum) {
-
+	// This is only used for static images in the floppy and cd intro
 	if (_currentScreen)
 		free(_currentScreen);
 	_currentScreen = _skyDisk->loadFile(fileNum);
+	// make sure the last 8 lines are forced to black.
+	memset(_currentScreen + GAME_SCREEN_HEIGHT * GAME_SCREEN_WIDTH, 0, (FULL_SCREEN_HEIGHT - GAME_SCREEN_HEIGHT) * GAME_SCREEN_WIDTH);
 
 	if (_currentScreen)
 		showScreen(_currentScreen);
@@ -390,21 +396,23 @@ void Screen::fnFadeUp(uint32 palNum, uint32 scroll) {
 
 void Screen::waitForTimer(void) {
 
+	Common::EventManager *eventMan = _system->getEventManager();
 	_gotTick = false;
 	while (!_gotTick) {
-		OSystem::Event event;
+		Common::Event event;
 
 		_system->delayMillis(10);
-		while (_system->pollEvent(event));
+		while (eventMan->pollEvent(event));
 	}
 }
 
 void Screen::waitForSequence(void) {
+	Common::EventManager *eventMan = _system->getEventManager();
 	while (_seqInfo.running) {
-		OSystem::Event event;
+		Common::Event event;
 
 		_system->delayMillis(20);
-		while (_system->pollEvent(event));
+		while (eventMan->pollEvent(event));
 	}
 }
 
