@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2003-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,13 +18,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/sky/screen.h $
- * $Id: screen.h 23992 2006-09-26 06:09:29Z eriktorbjorn $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/sky/screen.h $
+ * $Id: screen.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
-#ifndef SKYSCREEN_H
-#define SKYSCREEN_H
+#ifndef SKY_SCREEN_H
+#define SKY_SCREEN_H
 
 #include "common/stdafx.h"
 #include "common/scummsys.h"
@@ -71,18 +74,19 @@ public:
 	void startSequence(uint16 fileNum);
 	void startSequenceItem(uint16 itemNum);
 	void stopSequence(void);
-	bool sequenceRunning(void) { return _seqInfo.running; };
+	bool sequenceRunning(void) { return _seqInfo.running; }
 	void waitForSequence(void);
-	uint32 seqFramesLeft(void) { return _seqInfo.framesLeft; };
-	uint8 *giveCurrent(void) { return _currentScreen; };
+	uint32 seqFramesLeft(void) { return _seqInfo.framesLeft; }
+	uint8 *giveCurrent(void) { return _currentScreen; }
 	void halvePalette(void);
 
 	//- regular screen.asm routines
-	void forceRefresh(void) { memset(_gameGrid, 0x80, GRID_X * GRID_Y); };
+	void forceRefresh(void) { memset(_gameGrid, 0x80, GRID_X * GRID_Y); }
 	void fnFadeUp(uint32 palNum, uint32 scroll);
 	void fnFadeDown(uint32 scroll);
 	void fnDrawScreen(uint32 palette, uint32 scroll);
 	void clearScreen(void);
+	void setFocusRectangle(const Common::Rect& rect);
 
 	void recreate(void);
 	void flip(bool doUpdate = true);
@@ -91,7 +95,6 @@ public:
 
 	void paintBox(uint16 x, uint16 y);
 	void showGrid(uint8 *gridBuf);
-	void setFocusRectangle(const Common::Rect& rect);
 
 private:
 	OSystem *_system;
