@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2003-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,19 +18,19 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/sky/sky.cpp $
- * $Id: sky.cpp 22272 2006-05-01 22:27:56Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/sky/sky.cpp $
+ * $Id: sky.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
 #include "common/stdafx.h"
 
-#include "backends/fs/fs.h"
-
 #include "base/plugins.h"
 
 #include "common/config-manager.h"
 #include "common/file.h"
+#include "common/fs.h"
+#include "common/events.h"
 #include "common/system.h"
 #include "common/timer.h"
 
@@ -56,7 +59,6 @@
 
 extern bool toolbar_drawn;
 extern bool draw_keyboard;
-extern bool isSmartphone(void);
 #endif
 
 /*
@@ -91,22 +93,76 @@ GameDescriptor Engine_SKY_findGameID(const char *gameid) {
 	return GameDescriptor();
 }
 
-DetectedGameList Engine_SKY_detectGames(const FSList &fslist) {
-	DetectedGameList detectedGames;
+struct SkyVersion {
+	int dinnerTableEntries;
+	int dataDiskSize;
+	const char *extraDesc;
+	int version;
+};
+
+// TODO: Would be nice if Disk::determineGameVersion() used this table, too.
+static const SkyVersion skyVersions[] = {
+	{  243, -1, "pc gamer demo", 109 },
+	{  247, -1, "floppy demo", 267 },
+	{ 1404, -1, "floppy", 288 },
+	{ 1413, -1, "floppy", 303 },
+	{ 1445, 8830435, "floppy", 348 },
+	{ 1445, -1, "floppy", 331 },
+	{ 1711, -1, "cd demo", 365 },
+	{ 5099, -1, "cd", 368 },
+	{ 5097, -1, "cd", 372 },
+	{ 0, 0, 0, 0 }
+};
+
+GameList Engine_SKY_detectGames(const FSList &fslist) {
+	GameList detectedGames;
+	bool hasSkyDsk = false;
+	bool hasSkyDnr = false;
+	int dinnerTableEntries = -1;
+	int dataDiskSize = -1;
+
 	// Iterate over all files in the given directory
 	for (FSList::const_iterator file = fslist.begin(); file != fslist.end(); ++file) {
 		if (!file->isDirectory()) {
-			const char *fileName = file->displayName().c_str();
+			const char *fileName = file->name().c_str();
 
 			if (0 == scumm_stricmp("sky.dsk", fileName)) {
-				// Match found, add to list of candidates, then abort inner loop.
-				// The game detector uses US English by default. We want British
-				// English to match the recorded voices better.
-				detectedGames.push_back(DetectedGame(skySetting, Common::EN_GRB, Common::kPlatformUnknown));
-				break;
+				Common::File dataDisk;
+				if (dataDisk.open(file->path())) {
+					hasSkyDsk = true;
+					dataDiskSize = dataDisk.size();
+				}
+			}
+
+			if (0 == scumm_stricmp("sky.dnr", fileName)) {
+				Common::File dinner;
+				if (dinner.open(file->path())) {
+					hasSkyDnr = true;
+					dinnerTableEntries = dinner.readUint32LE();
+				}
 			}
 		}
 	}
+
+	if (hasSkyDsk && hasSkyDnr) {
+		// Match found, add to list of candidates, then abort inner loop.
+		// The game detector uses US English by default. We want British
+		// English to match the recorded voices better.
+		GameDescriptor dg(skySetting.gameid, skySetting.description, Common::UNK_LANG, Common::kPlatformUnknown);
+		const SkyVersion *sv = skyVersions;
+		while (sv->dinnerTableEntries) {
+			if (dinnerTableEntries == sv->dinnerTableEntries &&
+				(sv->dataDiskSize == dataDiskSize || sv->dataDiskSize == -1)) {
+				char buf[32];
+				snprintf(buf, sizeof(buf), "v0.0%d %s", sv->version, sv->extraDesc);
+				dg.updateDesc(buf);
+				break;
+			}
+			++sv;
+		}
+		detectedGames.push_back(dg);
+	}
+
 	return detectedGames;
 }
 
@@ -116,7 +172,7 @@ PluginError Engine_SKY_create(OSystem *syst, Engine **engine) {
 	return kNoError;
 }
 
-REGISTER_PLUGIN(SKY, "Beneath a Steel Sky");
+REGISTER_PLUGIN(SKY, "Beneath a Steel Sky", "Beneath a Steel Sky (C) Revolution");
 
 
 namespace Sky {
@@ -149,22 +205,8 @@ SkyEngine::~SkyEngine() {
 			free(_itemList[i]);
 }
 
-void SkyEngine::errorString(const char *buf1, char *buf2) {
-	strcpy(buf2, buf1);
-
-#ifdef _WIN32_WCE
-	if (isSmartphone())
-		return;
-#endif
-
-	// Unless an error -originated- within the debugger, spawn the
-	// debugger. Otherwise exit out normally.
-	if (_debugger && !_debugger->isAttached()) {
-		// (Print it again in case debugger segfaults)
-		printf("%s\n", buf2);
-		_debugger->attach(buf2);
-		_debugger->onFrame();
-	}
+GUI::Debugger *SkyEngine::getDebugger() {
+	return _debugger;
 }
 
 void SkyEngine::initVirgin() {
@@ -183,7 +225,7 @@ void SkyEngine::handleKey(void) {
 		return;
 	}
 
-	if (_keyFlags == OSystem::KBD_CTRL) {
+	if (_keyFlags == Common::KBD_CTRL) {
 		if (_keyPressed == 'f')
 			_fastMode ^= 1;
 		else if (_keyPressed == 'g')
@@ -225,8 +267,6 @@ int SkyEngine::go() {
 
 	_systemVars.quitGame = false;
 
-	_mouseX = GAME_SCREEN_WIDTH / 2;
-	_mouseY = GAME_SCREEN_HEIGHT / 2;
 	_keyFlags = _keyPressed = 0;
 
 	uint16 result = 0;
@@ -265,7 +305,7 @@ int SkyEngine::go() {
 				_lastSaveTime += 30 * 1000; // try again in 30 secs
 		}
 		_skySound->checkFxQueue();
-		_skyMouse->mouseEngine((uint16)_mouseX, (uint16)_mouseY);
+		_skyMouse->mouseEngine();
 		handleKey();
 		if (_systemVars.paused) {
 			do {
@@ -318,6 +358,9 @@ int SkyEngine::init() {
 
 	if (ConfMan.getBool("sfx_mute")) {
 		SkyEngine::_systemVars.systemFlags |= SF_FX_OFF;
+	}
+	if (ConfMan.getBool("music_mute")) {
+		SkyEngine::_systemVars.systemFlags |= SF_MUS_OFF;
 	}
 	 _mixer->setVolumeForSoundType(Audio::Mixer::kSFXSoundType, ConfMan.getInt("sfx_volume"));
 	 _mixer->setVolumeForSoundType(Audio::Mixer::kMusicSoundType, ConfMan.getInt("music_volume"));
@@ -471,7 +514,7 @@ void SkyEngine::gotTimerTick(void) {
 
 void SkyEngine::delay(int32 amount) {
 
-	OSystem::Event event;
+	Common::Event event;
 
 	uint32 start = _system->getMillis();
 	_keyFlags = _keyPressed = 0;	//reset
@@ -480,36 +523,30 @@ void SkyEngine::delay(int32 amount) {
 		amount = 0;
 
 	do {
-		while (_system->pollEvent(event)) {
+		while (_eventMan->pollEvent(event)) {
 			switch (event.type) {
-			case OSystem::EVENT_KEYDOWN:
+			case Common::EVENT_KEYDOWN:
 				_keyFlags = event.kbd.flags;
-				if (_keyFlags == OSystem::KBD_CTRL)
+				if (_keyFlags == Common::KBD_CTRL)
 					_keyPressed = event.kbd.keycode;
 				else
 					_keyPressed = (byte)event.kbd.ascii;
 				break;
-			case OSystem::EVENT_MOUSEMOVE:
-				if (!(_systemVars.systemFlags & SF_MOUSE_LOCKED)) {
-					_mouseX = event.mouse.x;
-					_mouseY = event.mouse.y;
-				}
+			case Common::EVENT_MOUSEMOVE:
+				if (!(_systemVars.systemFlags & SF_MOUSE_LOCKED))
+					_skyMouse->mouseMoved(event.mouse.x, event.mouse.y);
 				break;
-			case OSystem::EVENT_LBUTTONDOWN:
-#ifdef PALMOS_MODE
-				_mouseX = event.mouse.x;
-				_mouseY = event.mouse.y;
-#endif
+			case Common::EVENT_LBUTTONDOWN:
+				if (!(_systemVars.systemFlags & SF_MOUSE_LOCKED))
+					_skyMouse->mouseMoved(event.mouse.x, event.mouse.y);
 				_skyMouse->buttonPressed(2);
 				break;
-			case OSystem::EVENT_RBUTTONDOWN:
-#ifdef PALMOS_MODE
-				_mouseX = event.mouse.x;
-				_mouseY = event.mouse.y;
-#endif
+			case Common::EVENT_RBUTTONDOWN:
+				if (!(_systemVars.systemFlags & SF_MOUSE_LOCKED))
+					_skyMouse->mouseMoved(event.mouse.x, event.mouse.y);
 				_skyMouse->buttonPressed(1);
 				break;
-			case OSystem::EVENT_QUIT:
+			case Common::EVENT_QUIT:
 				_systemVars.quitGame = true;
 				break;
 			default:
