@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2003-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,16 +18,16 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/sky/sky.h $
- * $Id: sky.h 21916 2006-04-15 20:36:41Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/sky/sky.h $
+ * $Id: sky.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
-#ifndef SKYMAIN_H
-#define SKYMAIN_H
+#ifndef SKY_H
+#define SKY_H
 
 #include "common/stdafx.h"
-#include "base/engine.h"
+#include "engines/engine.h"
 
 namespace Sky {
 
@@ -55,13 +58,11 @@ class Debugger;
 class SkyCompact;
 
 class SkyEngine : public Engine {
-	void errorString(const char *buf_input, char *buf_output);
+	GUI::Debugger *getDebugger();
 protected:
 	byte _keyPressed, _keyFlags;
 	bool _floppyIntro;
 
-	int _mouseX, _mouseY;
-
 	Sound *_skySound;
 	Disk *_skyDisk;
 	Text *_skyText;
