@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2003-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/sky/sound.cpp $
- * $Id: sound.cpp 23217 2006-06-21 11:31:31Z kirben $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/sky/sound.cpp $
+ * $Id: sound.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -37,25 +40,21 @@ namespace Sky {
 #define SFXF_START_DELAY 0x80
 #define SFXF_SAVE 0x20
 
-#if !defined(__GNUC__)
-#pragma START_PACK_STRUCTS
-#endif
+#include "common/pack-start.h"	// START STRUCT PACKING
 
 struct RoomList {
 	uint8 room;
 	uint8 adlibVolume;
 	uint8 rolandVolume;
-} GCC_PACK;
+};
 
 struct Sfx {
 	uint8 soundNo;
 	uint8 flags;
 	RoomList roomList[10];
-} GCC_PACK;
+};
 
-#if !defined(__GNUC__)
-#pragma END_PACK_STRUCTS
-#endif
+#include "common/pack-end.h"	// END STRUCT PACKING
 
 uint16 Sound::_speechConvertTable[8] = {
 	0,									//;Text numbers to file numbers
@@ -69,7 +68,7 @@ uint16 Sound::_speechConvertTable[8] = {
 };
 
 
-static Sfx fx_null = {
+static const Sfx fx_null = {
 	0,
 	0,
 	{
@@ -78,7 +77,7 @@ static Sfx fx_null = {
 	}
 };
 
-static Sfx fx_level_3_ping = {
+static const Sfx fx_level_3_ping = {
 	1,
 	0,
 	{
@@ -89,7 +88,7 @@ static Sfx fx_level_3_ping = {
 	}
 };
 
-static Sfx fx_factory_sound = {
+static const Sfx fx_factory_sound = {
 	1,
 	SFXF_SAVE,
 	{
@@ -97,7 +96,7 @@ static Sfx fx_factory_sound = {
 	}
 };
 
-static Sfx fx_crowbar_plaster = {
+static const Sfx fx_crowbar_plaster = {
 	1,
 	0,
 	{
@@ -105,7 +104,7 @@ static Sfx fx_crowbar_plaster = {
 	}
 };
 
-static Sfx fx_masonry_fall = {
+static const Sfx fx_masonry_fall = {
 	1,
 	0,
 	{
@@ -113,7 +112,7 @@ static Sfx fx_masonry_fall = {
 	}
 };
 
-static Sfx fx_prise_brick = {
+static const Sfx fx_prise_brick = {
 	2,
 	0,
 	{
@@ -121,7 +120,7 @@ static Sfx fx_prise_brick = {
 	}
 };
 
-static Sfx fx_rope_creak = {
+static const Sfx fx_rope_creak = {
 	2,
 	0,
 	{
@@ -129,7 +128,7 @@ static Sfx fx_rope_creak = {
 	}
 };
 
-static Sfx fx_ping = {
+static const Sfx fx_ping = {
 	3,
 	0,
 	{
@@ -137,7 +136,7 @@ static Sfx fx_ping = {
 	}
 };
 
-static Sfx fx_force_fire_door = {
+static const Sfx fx_force_fire_door = {
 	3,
 	0,
 	{
@@ -145,7 +144,7 @@ static Sfx fx_force_fire_door = {
 	}
 };
 
-static Sfx fx_brick_hit_foster = {
+static const Sfx fx_brick_hit_foster = {
 	3,
 	10+SFXF_START_DELAY,
 	{
@@ -153,7 +152,7 @@ static Sfx fx_brick_hit_foster = {
 	}
 };
 
-static Sfx fx_brick_hit_plank = {
+static const Sfx fx_brick_hit_plank = {
 	3,
 	8+SFXF_START_DELAY,
 	{
@@ -161,7 +160,7 @@ static Sfx fx_brick_hit_plank = {
 	}
 };
 
-static Sfx fx_rm3_lift_moving = {
+static const Sfx fx_rm3_lift_moving = {
 	4,
 	SFXF_SAVE,
 	{
@@ -171,7 +170,7 @@ static Sfx fx_rm3_lift_moving = {
 	}
 };
 
-static Sfx fx_weld = {
+static const Sfx fx_weld = {
 	4,
 	0,
 	{
@@ -184,7 +183,7 @@ static Sfx fx_weld = {
 	}
 };
 
-static Sfx fx_weld12 = {
+static const Sfx fx_weld12 = {
 	4,
 	0,
 	{
@@ -193,7 +192,7 @@ static Sfx fx_weld12 = {
 	}
 };
 
-static Sfx fx_spray_on_skin = {
+static const Sfx fx_spray_on_skin = {
 	4,
 	0,
 	{
@@ -201,7 +200,7 @@ static Sfx fx_spray_on_skin = {
 	}
 };
 
-static Sfx fx_plank_vibrating = {
+static const Sfx fx_plank_vibrating = {
 	4,
 	6+SFXF_START_DELAY,
 	{
@@ -209,7 +208,7 @@ static Sfx fx_plank_vibrating = {
 	}
 };
 
-static Sfx fx_press_bang = {
+static const Sfx fx_press_bang = {
 	5,
 	0,
 	{
@@ -218,7 +217,7 @@ static Sfx fx_press_bang = {
 	}
 };
 
-static Sfx fx_spanner_clunk = {
+static const Sfx fx_spanner_clunk = {
 	5,
 	0,
 	{
@@ -226,7 +225,7 @@ static Sfx fx_spanner_clunk = {
 	}
 };
 
-static Sfx fx_break_crystals = {
+static const Sfx fx_break_crystals = {
 	5,
 	0,
 	{
@@ -235,7 +234,7 @@ static Sfx fx_break_crystals = {
 	}
 };
 
-static Sfx fx_press_hiss = {
+static const Sfx fx_press_hiss = {
 	6,
 	0,
 	{
@@ -244,7 +243,7 @@ static Sfx fx_press_hiss = {
 	}
 };
 
-static Sfx fx_open_door = {
+static const Sfx fx_open_door = {
 	6,
 	0,
 	{
@@ -252,7 +251,7 @@ static Sfx fx_open_door = {
 	}
 };
 
-static Sfx fx_open_lamb_door = {
+static const Sfx fx_open_lamb_door = {
 	6,
 	0,
 	{
@@ -262,7 +261,7 @@ static Sfx fx_open_lamb_door = {
 	}
 };
 
-static Sfx fx_splash = {
+static const Sfx fx_splash = {
 	6,
 	22+SFXF_START_DELAY,
 	{
@@ -270,7 +269,7 @@ static Sfx fx_splash = {
 	}
 };
 
-static Sfx fx_disintegrate = {
+static const Sfx fx_disintegrate = {
 	7,
 	0,
 	{
@@ -278,7 +277,7 @@ static Sfx fx_disintegrate = {
 	}
 };
 
-static Sfx fx_buzzer = {
+static const Sfx fx_buzzer = {
 	7,
 	4+SFXF_START_DELAY,
 	{
@@ -286,7 +285,7 @@ static Sfx fx_buzzer = {
 	}
 };
 
-static Sfx fx_lathe = {
+static const Sfx fx_lathe = {
 	7,
 	SFXF_SAVE,
 	{
@@ -296,7 +295,7 @@ static Sfx fx_lathe = {
 	}
 };
 
-static Sfx fx_hit_crowbar_brick = {
+static const Sfx fx_hit_crowbar_brick = {
 	7,
 	9+SFXF_START_DELAY,
 	{
@@ -304,7 +303,7 @@ static Sfx fx_hit_crowbar_brick = {
 	}
 };
 
-static Sfx fx_hello_helga = {
+static const Sfx fx_hello_helga = {
 	8,
 	0,
 	{
@@ -312,7 +311,7 @@ static Sfx fx_hello_helga = {
 	}
 };
 
-static Sfx fx_statue_on_armour = {
+static const Sfx fx_statue_on_armour = {
 	8,
 	0,
 	{
@@ -320,7 +319,7 @@ static Sfx fx_statue_on_armour = {
 	}
 };
 
-static Sfx fx_lift_alarm = {
+static const Sfx fx_lift_alarm = {
 	8,
 	SFXF_SAVE,
 	{
@@ -329,7 +328,7 @@ static Sfx fx_lift_alarm = {
 	}
 };
 
-static Sfx fx_drop_crowbar = {
+static const Sfx fx_drop_crowbar = {
 	8,
 	5+SFXF_START_DELAY,
 	{
@@ -337,7 +336,7 @@ static Sfx fx_drop_crowbar = {
 	}
 };
 
-static Sfx fx_byee_helga = {
+static const Sfx fx_byee_helga = {
 	9,
 	3+SFXF_START_DELAY,
 	{
@@ -345,7 +344,7 @@ static Sfx fx_byee_helga = {
 	}
 };
 
-static Sfx fx_shed_door_creak = {
+static const Sfx fx_shed_door_creak = {
 	10,
 	0,
 	{
@@ -353,7 +352,7 @@ static Sfx fx_shed_door_creak = {
 	}
 };
 
-static Sfx fx_explosion = {
+static const Sfx fx_explosion = {
 	10,
 	0,
 	{
@@ -361,7 +360,7 @@ static Sfx fx_explosion = {
 	}
 };
 
-static Sfx fx_fire_crackle_in_pit = {
+static const Sfx fx_fire_crackle_in_pit = {
 	 9,
 	SFXF_SAVE,
 	{
@@ -369,7 +368,7 @@ static Sfx fx_fire_crackle_in_pit = {
 	}
 };
 
-static Sfx fx_remove_bar_grill = {
+static const Sfx fx_remove_bar_grill = {
 	10,
 	7+SFXF_START_DELAY,
 	{
@@ -377,7 +376,7 @@ static Sfx fx_remove_bar_grill = {
 	}
 };
 
-static Sfx fx_grill_creak = {
+static const Sfx fx_grill_creak = {
 	10,
 	43+SFXF_START_DELAY,
 	{
@@ -385,7 +384,7 @@ static Sfx fx_grill_creak = {
 	}
 };
 
-static Sfx fx_steam1 = {
+static const Sfx fx_steam1 = {
 	11,
 	SFXF_SAVE,
 	{
@@ -394,7 +393,7 @@ static Sfx fx_steam1 = {
 	}
 };
 
-static Sfx fx_steam2 = {
+static const Sfx fx_steam2 = {
 	11,
 	SFXF_SAVE,
 	{
@@ -403,7 +402,7 @@ static Sfx fx_steam2 = {
 	}
 };
 
-static Sfx fx_steam3 = {
+static const Sfx fx_steam3 = {
 	11,
 	SFXF_SAVE,
 	{
@@ -412,7 +411,7 @@ static Sfx fx_steam3 = {
 	}
 };
 
-static Sfx fx_crowbar_wooden = {
+static const Sfx fx_crowbar_wooden = {
 	11,
 	0,
 	{
@@ -420,7 +419,7 @@ static Sfx fx_crowbar_wooden = {
 	}
 };
 
-static Sfx fx_helmet_down_3 = {
+static const Sfx fx_helmet_down_3 = {
 	11,
 	0,
 	{
@@ -428,7 +427,7 @@ static Sfx fx_helmet_down_3 = {
 	}
 };
 
-static Sfx fx_guard_fall = {
+static const Sfx fx_guard_fall = {
 	11,
 	4,
 	{
@@ -437,7 +436,7 @@ static Sfx fx_guard_fall = {
 };
 
 #if 0
-static Sfx fx_furnace = {
+static const Sfx fx_furnace = {
 	11,
 	0,
 	{
@@ -447,7 +446,7 @@ static Sfx fx_furnace = {
 };
 #endif
 
-static Sfx fx_fall_thru_box = {
+static const Sfx fx_fall_thru_box = {
 	12,
 	0,
 	{
@@ -455,7 +454,7 @@ static Sfx fx_fall_thru_box = {
 	}
 };
 
-static Sfx fx_lazer = {
+static const Sfx fx_lazer = {
 	12,
 	0,
 	{
@@ -463,7 +462,7 @@ static Sfx fx_lazer = {
 	}
 };
 
-static Sfx fx_scanner = {
+static const Sfx fx_scanner = {
 	12,
 	0,
 	{
@@ -471,7 +470,7 @@ static Sfx fx_scanner = {
 	}
 };
 
-static Sfx fx_helmet_up_3 = {
+static const Sfx fx_helmet_up_3 = {
 	12,
 	0,
 	{
@@ -479,7 +478,7 @@ static Sfx fx_helmet_up_3 = {
 	}
 };
 
-static Sfx fx_liquid_bubble = {
+static const Sfx fx_liquid_bubble = {
 	12,
 	SFXF_SAVE,
 	{
@@ -489,7 +488,7 @@ static Sfx fx_liquid_bubble = {
 	}
 };
 
-static Sfx fx_liquid_drip = {
+static const Sfx fx_liquid_drip = {
 	13,
 	6+SFXF_START_DELAY,
 	{
@@ -497,7 +496,7 @@ static Sfx fx_liquid_drip = {
 	}
 };
 
-static Sfx fx_goo_drip = {
+static const Sfx fx_goo_drip = {
 	13,
 	5+SFXF_START_DELAY,
 	{
@@ -505,7 +504,7 @@ static Sfx fx_goo_drip = {
 	}
 };
 
-static Sfx fx_comp_bleeps = {
+static const Sfx fx_comp_bleeps = {
 	13,
 	0,
 	{
@@ -513,7 +512,7 @@ static Sfx fx_comp_bleeps = {
 	}
 };
 
-static Sfx fx_use_crowbar_grill = {
+static const Sfx fx_use_crowbar_grill = {
 	13,
 	34+SFXF_START_DELAY,
 	{
@@ -521,7 +520,7 @@ static Sfx fx_use_crowbar_grill = {
 	}
 };
 
-static Sfx fx_helmet_grind = {
+static const Sfx fx_helmet_grind = {
 	14,
 	0,
 	{
@@ -529,7 +528,7 @@ static Sfx fx_helmet_grind = {
 	}
 };
 
-static Sfx fx_lift_moving = {
+static const Sfx fx_lift_moving = {
 	14,
 	SFXF_SAVE,
 	{
@@ -539,7 +538,7 @@ static Sfx fx_lift_moving = {
 	}
 };
 
-static Sfx fx_use_secateurs = {
+static const Sfx fx_use_secateurs = {
 	14,
 	18+SFXF_START_DELAY,
 	{
@@ -547,7 +546,7 @@ static Sfx fx_use_secateurs = {
 	}
 };
 
-static Sfx fx_hit_joey1 = {
+static const Sfx fx_hit_joey1 = {
 	14,
 	7+SFXF_START_DELAY,
 	{
@@ -555,7 +554,7 @@ static Sfx fx_hit_joey1 = {
 	}
 };
 
-static Sfx fx_hit_joey2 = {
+static const Sfx fx_hit_joey2 = {
 	14,
 	13+SFXF_START_DELAY,
 	{
@@ -563,7 +562,7 @@ static Sfx fx_hit_joey2 = {
 	}
 };
 
-static Sfx fx_dani_phone_ring = {
+static const Sfx fx_dani_phone_ring = {
 	15,
 	0,
 	{
@@ -571,7 +570,7 @@ static Sfx fx_dani_phone_ring = {
 	}
 };
 
-static Sfx fx_sc74_pod_down = {
+static const Sfx fx_sc74_pod_down = {
 	15,
 	0,
 	{
@@ -579,7 +578,7 @@ static Sfx fx_sc74_pod_down = {
 	}
 };
 
-static Sfx fx_phone = {
+static const Sfx fx_phone = {
 	15,
 	0,
 	{
@@ -587,7 +586,7 @@ static Sfx fx_phone = {
 	}
 };
 
-static Sfx fx_25_weld = {
+static const Sfx fx_25_weld = {
 	15,
 	0,
 	{
@@ -595,7 +594,7 @@ static Sfx fx_25_weld = {
 	}
 };
 
-static Sfx fx_lift_open_7 = {
+static const Sfx fx_lift_open_7 = {
 	15,
 	0,
 	{
@@ -604,7 +603,7 @@ static Sfx fx_lift_open_7 = {
 	}
 };
 
-static Sfx fx_lift_close_7 = {
+static const Sfx fx_lift_close_7 = {
 	16,
 	0,
 	{
@@ -613,7 +612,7 @@ static Sfx fx_lift_close_7 = {
 	}
 };
 
-static Sfx fx_s2_helmet = {
+static const Sfx fx_s2_helmet = {
 	16,
 	0,
 	{
@@ -621,7 +620,7 @@ static Sfx fx_s2_helmet = {
 	}
 };
 
-static Sfx fx_hiss_in_nitrogen = {
+static const Sfx fx_hiss_in_nitrogen = {
 	16,
 	0,
 	{
@@ -629,7 +628,7 @@ static Sfx fx_hiss_in_nitrogen = {
 	}
 };
 
-static Sfx fx_dog_yap_indoors = {
+static const Sfx fx_dog_yap_indoors = {
 	16,
 	0,
 	{
@@ -638,7 +637,7 @@ static Sfx fx_dog_yap_indoors = {
 	}
 };
 
-static Sfx fx_dog_yap_outdoors = {
+static const Sfx fx_dog_yap_outdoors = {
 	16,
 	0,
 	{
@@ -650,7 +649,7 @@ static Sfx fx_dog_yap_outdoors = {
 	}
 };
 
-static Sfx fx_locker_creak_open = {
+static const Sfx fx_locker_creak_open = {
 	17,
 	0,
 	{
@@ -658,7 +657,7 @@ static Sfx fx_locker_creak_open = {
 	}
 };
 
-static Sfx fx_big_tent_gurgle = {
+static const Sfx fx_big_tent_gurgle = {
 	17,
 	0,
 	{
@@ -666,7 +665,7 @@ static Sfx fx_big_tent_gurgle = {
 	}
 };
 
-static Sfx fx_wind_howl = {
+static const Sfx fx_wind_howl = {
 	17,
 	SFXF_SAVE,
 	{
@@ -675,7 +674,7 @@ static Sfx fx_wind_howl = {
 	}
 };
 
-static Sfx fx_lift_open_29 = {
+static const Sfx fx_lift_open_29 = {
 	17,
 	0,
 	{
@@ -684,7 +683,7 @@ static Sfx fx_lift_open_29 = {
 	}
 };
 
-static Sfx fx_lift_arrive_7 = {
+static const Sfx fx_lift_arrive_7 = {
 	17,
 	0,
 	{
@@ -693,7 +692,7 @@ static Sfx fx_lift_arrive_7 = {
 	}
 };
 
-static Sfx fx_lift_close_29 = {
+static const Sfx fx_lift_close_29 = {
 	18,
 	0,
 	{
@@ -703,7 +702,7 @@ static Sfx fx_lift_close_29 = {
 	}
 };
 
-static Sfx fx_shaft_industrial_noise = {
+static const Sfx fx_shaft_industrial_noise = {
 	18,
 	SFXF_SAVE,
 	{
@@ -711,7 +710,7 @@ static Sfx fx_shaft_industrial_noise = {
 	}
 };
 
-static Sfx fx_gall_drop = {
+static const Sfx fx_gall_drop = {
 	18,
 	29+SFXF_START_DELAY,
 	{
@@ -719,7 +718,7 @@ static Sfx fx_gall_drop = {
 	}
 };
 
-static Sfx fx_door_slam_under = {
+static const Sfx fx_door_slam_under = {
 	19,
 	0,
 	{
@@ -727,7 +726,7 @@ static Sfx fx_door_slam_under = {
 	}
 };
 
-static Sfx fx_reichs_fish = {
+static const Sfx fx_reichs_fish = {
 	19,
 	SFXF_SAVE,
 	{
@@ -735,7 +734,7 @@ static Sfx fx_reichs_fish = {
 	}
 };
 
-static Sfx fx_judges_gavel1 = {
+static const Sfx fx_judges_gavel1 = {
 	19,
 	13+SFXF_START_DELAY,
 	{
@@ -743,7 +742,7 @@ static Sfx fx_judges_gavel1 = {
 	}
 };
 
-static Sfx fx_judges_gavel2 = {
+static const Sfx fx_judges_gavel2 = {
 	19,
 	16+SFXF_START_DELAY,
 	{
@@ -751,7 +750,7 @@ static Sfx fx_judges_gavel2 = {
 	}
 };
 
-static Sfx fx_judges_gavel3 = {
+static const Sfx fx_judges_gavel3 = {
 	19,
 	19+SFXF_START_DELAY,
 	{
@@ -759,7 +758,7 @@ static Sfx fx_judges_gavel3 = {
 	}
 };
 
-static Sfx fx_wind_3 = {
+static const Sfx fx_wind_3 = {
 	20,
 	SFXF_SAVE,
 	{
@@ -767,7 +766,7 @@ static Sfx fx_wind_3 = {
 	}
 };
 
-static Sfx fx_fact_sensor = {
+static const Sfx fx_fact_sensor = {
 	20,
 	SFXF_SAVE,
 	{
@@ -775,7 +774,7 @@ static Sfx fx_fact_sensor = {
 	}
 };
 
-static Sfx fx_medi_stab_gall = {
+static const Sfx fx_medi_stab_gall = {
 	20,
 	17+SFXF_START_DELAY,
 	{
@@ -783,7 +782,7 @@ static Sfx fx_medi_stab_gall = {
 	}
 };
 
-static Sfx fx_computer_3 = {
+static const Sfx fx_computer_3 = {
 	21,
 	SFXF_SAVE,
 	{
@@ -791,7 +790,7 @@ static Sfx fx_computer_3 = {
 	}
 };
 
-static Sfx fx_timber_cracking = {
+static const Sfx fx_timber_cracking = {
 	21,
 	0,
 	{
@@ -799,7 +798,7 @@ static Sfx fx_timber_cracking = {
 	}
 };
 
-static Sfx fx_anchor_fall = {
+static const Sfx fx_anchor_fall = {
 	22,
 	0,
 	{
@@ -807,7 +806,7 @@ static Sfx fx_anchor_fall = {
 	}
 };
 
-static Sfx fx_elevator_4 = {
+static const Sfx fx_elevator_4 = {
 	22,
 	0,
 	{
@@ -815,7 +814,7 @@ static Sfx fx_elevator_4 = {
 	}
 };
 
-static Sfx fx_star_trek_2 = {
+static const Sfx fx_star_trek_2 = {
 	22,
 	SFXF_SAVE,
 	{
@@ -823,7 +822,7 @@ static Sfx fx_star_trek_2 = {
 	}
 };
 
-static Sfx fx_lift_closing = {
+static const Sfx fx_lift_closing = {
 	23,
 	0,
 	{
@@ -831,7 +830,7 @@ static Sfx fx_lift_closing = {
 	}
 };
 
-static Sfx fx_heartbeat = {
+static const Sfx fx_heartbeat = {
 	23,
 	11+SFXF_START_DELAY,
 	{
@@ -847,7 +846,7 @@ static Sfx fx_heartbeat = {
 	}
 };
 
-static Sfx fx_pos_key = {
+static const Sfx fx_pos_key = {
 	25,
 	2+SFXF_START_DELAY,
 	{
@@ -855,7 +854,7 @@ static Sfx fx_pos_key = {
 	}
 };
 
-static Sfx fx_neg_key = {
+static const Sfx fx_neg_key = {
 	26,
 	2+SFXF_START_DELAY,
 	{
@@ -863,7 +862,7 @@ static Sfx fx_neg_key = {
 	}
 };
 
-static Sfx fx_orifice_swallow_drip = {
+static const Sfx fx_orifice_swallow_drip = {
 	28,
 	0,
 	{
@@ -871,7 +870,7 @@ static Sfx fx_orifice_swallow_drip = {
 	}
 };
 
-static Sfx *musicList[] = {
+static const Sfx *musicList[] = {
 	&fx_press_bang, // 256 banging of the press
 	&fx_press_hiss, // 257 hissing press
 	&fx_wind_howl, // 258 howling wind
@@ -1044,7 +1043,7 @@ void Sound::playSound(uint32 id, byte *sound, uint32 size, Audio::SoundHandle *h
 	memcpy(buffer, sound+sizeof(struct dataFileHeader), size);
 
 	_mixer->stopID(id);
-	_mixer->playRaw(handle, buffer, size, 11025, flags, id);
+	_mixer->playRaw(Audio::Mixer::kSFXSoundType, handle, buffer, size, 11025, flags, id);
 }
 
 void Sound::loadSection(uint8 pSection) {
@@ -1122,9 +1121,9 @@ void Sound::playSound(uint16 sound, uint16 volume, uint8 channel) {
 	}
 
 	if (channel == 0)
-		_mixer->playRaw(&_ingameSound0, _soundData + dataOfs, dataSize, sampleRate, flags, SOUND_CH0, volume, 0, loopSta, loopEnd);
+		_mixer->playRaw(Audio::Mixer::kSFXSoundType, &_ingameSound0, _soundData + dataOfs, dataSize, sampleRate, flags, SOUND_CH0, volume, 0, loopSta, loopEnd);
 	else
-		_mixer->playRaw(&_ingameSound1, _soundData + dataOfs, dataSize, sampleRate, flags, SOUND_CH1, volume, 0, loopSta, loopEnd);
+		_mixer->playRaw(Audio::Mixer::kSFXSoundType, &_ingameSound1, _soundData + dataOfs, dataSize, sampleRate, flags, SOUND_CH1, volume, 0, loopSta, loopEnd);
 }
 
 void Sound::fnStartFx(uint32 sound, uint8 channel) {
@@ -1139,8 +1138,8 @@ void Sound::fnStartFx(uint32 sound, uint8 channel) {
 
 	sound &= ~(1 << 8);
 
-	Sfx *sfx = musicList[sound];
-	RoomList *roomList = sfx->roomList;
+	const Sfx *sfx = musicList[sound];
+	const RoomList *roomList = sfx->roomList;
 
 	int i = 0;
 	if (roomList[i].room != 0xff) // if room list empty then do all rooms
@@ -1249,7 +1248,7 @@ bool Sound::startSpeech(uint16 textNum) {
 		rate = 11025;
 
 	_mixer->stopID(SOUND_SPEECH);
-	_mixer->playRaw(&_ingameSpeech, playBuffer, speechSize, rate, Audio::Mixer::FLAG_UNSIGNED | Audio::Mixer::FLAG_AUTOFREE, SOUND_SPEECH);
+	_mixer->playRaw(Audio::Mixer::kSpeechSoundType, &_ingameSpeech, playBuffer, speechSize, rate, Audio::Mixer::FLAG_UNSIGNED | Audio::Mixer::FLAG_AUTOFREE, SOUND_SPEECH);
 	return true;
 }
 
