@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2003-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,13 +18,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/sky/sound.h $
- * $Id: sound.h 23217 2006-06-21 11:31:31Z kirben $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/sky/sound.h $
+ * $Id: sound.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
-#ifndef SKYSOUND_H
-#define SKYSOUND_H
+#ifndef SKY_SOUND_H
+#define SKY_SOUND_H
 
 #include "common/stdafx.h"
 #include "common/scummsys.h"
@@ -70,7 +73,7 @@ public:
 	void playSound(uint16 sound, uint16 volume, uint8 channel);
 	void fnStartFx(uint32 sound, uint8 channel);
 	bool startSpeech(uint16 textNum);
-	bool speechFinished(void) { return !_mixer->isSoundHandleActive(_ingameSpeech); };
+	bool speechFinished(void) { return !_mixer->isSoundHandleActive(_ingameSpeech); }
 	void fnPauseFx(void);
 	void fnUnPauseFx(void);
 	void fnStopFx(void);
