@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2003-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,13 +18,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/sky/struc.h $
- * $Id: struc.h 20514 2006-02-11 10:03:24Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/sky/struc.h $
+ * $Id: struc.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
-#ifndef SKYSTRUC_H
-#define SKYSTRUC_H
+#ifndef SKY_STRUC_H
+#define SKY_STRUC_H
 
 namespace Sky {
 
@@ -36,9 +39,7 @@ struct displayText_t {
 	uint32 textWidth;
 };
 
-#if !defined(__GNUC__)
-#pragma START_PACK_STRUCTS
-#endif
+#include "common/pack-start.h"	// START STRUCT PACKING
 
 struct dataFileHeader {
 	uint16 flag; // bit 0: set for colour data, clear for not
@@ -54,7 +55,7 @@ struct dataFileHeader {
 	int16 s_offset_x;
 	int16 s_offset_y;
 	uint16 s_compressed_size;
-} GCC_PACK;
+};
 
 struct TurnTable {
 	uint16 turnTableUp[5];
@@ -62,7 +63,7 @@ struct TurnTable {
 	uint16 turnTableLeft[5];
 	uint16 turnTableRight[5];
 	uint16 turnTableTalk[5];
-} GCC_PACK;
+};
 
 struct MegaSet {
 	uint16 gridWidth;	 //  0
@@ -81,7 +82,7 @@ struct MegaSet {
 	uint16 standRightId; // 11
 	uint16 standTalkId;	 // 12
 	uint16 turnTableId;	 // 13
-} GCC_PACK;
+};
 
 struct Compact {
 	uint16 logic;		 //  0: Entry in logic table to run (byte as <256entries in logic table
@@ -164,11 +165,9 @@ struct Compact {
 	MegaSet megaSet1;	 //
 	MegaSet megaSet2;	 //
 	MegaSet megaSet3;	 //
-} GCC_PACK;
+};
 
-#if !defined(__GNUC__)
-#pragma END_PACK_STRUCTS
-#endif
+#include "common/pack-end.h"	// END STRUCT PACKING
 
 } // End of namespace Sky
 
