@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2003-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,13 +18,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/sky/text.h $
- * $Id: text.h 20514 2006-02-11 10:03:24Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/sky/text.h $
+ * $Id: text.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
-#ifndef SKYTEXT_H
-#define SKYTEXT_H
+#ifndef SKY_TEXT_H
+#define SKY_TEXT_H
 
 #include "common/stdafx.h"
 #include "common/scummsys.h"
@@ -38,7 +41,7 @@ struct HuffTree {
 	unsigned char value;
 };
 
-#define NUM_PATCH_MSG 8
+#define NUM_PATCH_MSG 9
 
 struct PatchMessage {
 	uint32 textNr;
@@ -68,8 +71,6 @@ private:
 	bool getTextBit(uint8 **data, uint32 *bitPos);
 	void makeGameCharacter(uint8 textChar, uint8 *charSetPtr, uint8 *&data, uint8 color, uint16 bufPitch);
 
-	void patchChar(byte *charSetPtr, int width, int height, int c, const uint16 *data);
-	void patchLINCCharset();
 	bool patchMessage(uint32 textNum);
 
 	Disk *_skyDisk;
