@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2003-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,16 +18,16 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/sword1/control.cpp $
- * $Id: control.cpp 23847 2006-09-09 14:36:20Z chrilith $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/sword1/control.cpp $
+ * $Id: control.cpp 27485 2007-06-16 19:29:14Z eriktorbjorn $
  *
  */
 
 #include "common/stdafx.h"
-#include "common/endian.h"
 #include "common/file.h"
 #include "common/util.h"
 #include "common/savefile.h"
+#include "common/events.h"
 #include "common/system.h"
 
 #include "gui/message.h"
@@ -42,11 +45,6 @@
 
 namespace Sword1 {
 
-enum {
-	kKeyRepeatInitialDelay = 400,
-	kKeyRepeatSustainDelay = 100
-};
-
 enum LangStrings {
 	STR_PAUSED = 0,
 	STR_INSERT_CD_A,
@@ -118,8 +116,8 @@ ControlButton::ControlButton(uint16 x, uint16 y, uint32 resId, uint8 id, uint8 f
 	_frameIdx = 0;
 	_resMan->resOpen(_resId);
 	FrameHeader *tmp = _resMan->fetchFrame(_resMan->fetchRes(_resId), 0);
-	_width = FROM_LE_16(tmp->width);
-	_height = FROM_LE_16(tmp->height);
+	_width = _resMan->getUint16(tmp->width);
+	_height = _resMan->getUint16(tmp->height);
 	if ((x == 0) && (y == 0)) { // center the frame (used for panels);
 		_x = (640 - _width) / 2;
 		_y = (480 - _height) / 2;
@@ -140,12 +138,12 @@ void ControlButton::draw(void) {
 	FrameHeader *fHead = _resMan->fetchFrame(_resMan->fetchRes(_resId), _frameIdx);
 	uint8 *src = (uint8*)fHead + sizeof(FrameHeader);
 	uint8 *dst = _dstBuf;
-	for (uint16 cnt = 0; cnt < READ_LE_UINT16(&fHead->height); cnt++) {
-		for (uint16 cntx = 0; cntx < READ_LE_UINT16(&fHead->width); cntx++)
+	for (uint16 cnt = 0; cnt < _resMan->readUint16(&fHead->height); cnt++) {
+		for (uint16 cntx = 0; cntx < _resMan->readUint16(&fHead->width); cntx++)
 			if (src[cntx])
 				dst[cntx] = src[cntx];
 		dst += SCREEN_WIDTH;
-		src += READ_LE_UINT16(&fHead->width);
+		src += _resMan->readUint16(&fHead->width);
 	}
 	_system->copyRectToScreen(_dstBuf, SCREEN_WIDTH, _x, _y, _width, _height);
 }
@@ -171,8 +169,6 @@ Control::Control(Common::SaveFileManager *saveFileMan, ResMan *pResMan, ObjectMa
 	_music = pMusic;
 	_sound = pSound;
 	_lStrings = _languageStrings + SwordEngine::_systemVars.language * 20;
-	_keyRepeat = 0;
-	_keyRepeatTime = 0;
 	_selectedButton = 255;
 }
 
@@ -194,7 +190,6 @@ void Control::askForCd(void) {
 	_system->setPalette(palOut, 0, 256);
 	free(palOut);
 
-	Common::File test;
 	char fName[10];
 	uint8 textA[50];
 	sprintf(fName, "cd%d.id", SwordEngine::_systemVars.currentCD);
@@ -210,15 +205,13 @@ void Control::askForCd(void) {
 		}
 		delay(300);
 		if (_keyPressed) {
-			test.open(fName);
-			if (!test.isOpen()) {
+			if (!Common::File::exists(fName)) {
 				memset(_screenBuf, 0, 640 * 480);
 				renderText(_lStrings[STR_INCORRECT_CD], 320, 230, TEXT_CENTER);
 				_system->copyRectToScreen(_screenBuf, 640, 0, 0, 640, 480);
 				delay(2000);
 				refreshText = true;
 			} else {
-				test.close();
 				notAccepted = false;
 			}
 		}
@@ -337,7 +330,8 @@ uint8 Control::getClicks(uint8 mode, uint8 *retVal) {
 	uint8 flag = 0;
 	if (_keyPressed == 27)
 		flag = kButtonCancel;
-	else if (_keyPressed == '\r' || _keyPressed == '\n')
+	// 3 is num keypad Enter on Macs. See FR #1273746
+	else if (_keyPressed == '\r' || _keyPressed == '\n' || _keyPressed == 3)
 		flag = kButtonOk;
 
 	if (flag) {
@@ -488,8 +482,8 @@ void Control::setupMainPanel(void) {
 
 void Control::setupSaveRestorePanel(bool saving) {
 	FrameHeader *savePanel = _resMan->fetchFrame(_resMan->openFetchRes(SR_WINDOW), 0);
-	uint16 panelX = (640 - FROM_LE_16(savePanel->width)) / 2;
-	uint16 panelY = (480 - FROM_LE_16(savePanel->height)) / 2;
+	uint16 panelX = (640 - _resMan->getUint16(savePanel->width)) / 2;
+	uint16 panelY = (480 - _resMan->getUint16(savePanel->height)) / 2;
 	ControlButton *panel = new ControlButton(panelX, panelY, SR_WINDOW, 0, 0, _resMan, _screenBuf, _system);
 	panel->draw();
 	delete panel;
@@ -706,6 +700,9 @@ void Control::readSavegameDescriptions(void) {
 	inf = _saveFileMan->openForLoading("SAVEGAME.INF");
 	_saveScrollPos = _saveFiles = 0;
 	_selectedSavegame = 255;
+	for (uint8 cnt = 0; cnt < 64; cnt++) {
+		memset(_saveNames[cnt], 0, sizeof(_saveNames[cnt]));
+	}
 	if (inf) {
 		uint8 curFileNum = 0;
 		uint8 ch;
@@ -713,20 +710,18 @@ void Control::readSavegameDescriptions(void) {
 			uint8 pos = 0;
 			do {
 				ch = inf->readByte();
-				if ((ch == 10) || (ch == 255))
-					_saveNames[curFileNum][pos] = '\0';
-				else
-					_saveNames[curFileNum][pos] = ch;
-				pos++;
-			} while ((ch != 10) && (ch != 255));
-			curFileNum++;
-		} while (ch != 255);
+				if (pos < sizeof(_saveNames[curFileNum]) - 1) {
+					if ((ch == 10) || (ch == 255) || (inf->eos()))
+						_saveNames[curFileNum][pos++] = '\0';
+					else if (ch >= 32)
+						_saveNames[curFileNum][pos++] = ch;
+				}
+			} while ((ch != 10) && (ch != 255) && (!inf->eos()));
+			if (_saveNames[curFileNum][0] != 0)
+				curFileNum++;
+		} while ((ch != 255) && (!inf->eos()));
 		_saveFiles = curFileNum;
-		for (uint8 cnt = _saveFiles; cnt < 64; cnt++)
-			_saveNames[cnt][0] = '\0';
-	} else
-		for (uint8 cnt = 0; cnt < 64; cnt++)
-			_saveNames[cnt][0] = '\0';
+	}
 	delete inf;
 }
 
@@ -767,7 +762,7 @@ void Control::writeSavegameDescriptions(void) {
 		else
 			outf->writeByte(255);
 	}
-	outf->flush();
+	outf->finalize();
 	if (outf->ioFailed())
 		displayMessage(0, "Can't write to SAVEGAME.INF in directory '%s'. Device full?", _saveFileMan->getSavePath());
 	delete outf;
@@ -873,7 +868,7 @@ void Control::destroyButtons(void) {
 uint16 Control::getTextWidth(const uint8 *str) {
 	uint16 width = 0;
 	while (*str) {
-		width += FROM_LE_16(_resMan->fetchFrame(_font, *str - 32)->width) - 3;
+		width += _resMan->getUint16(_resMan->fetchFrame(_font, *str - 32)->width) - 3;
 		str++;
 	}
 	return width;
@@ -897,15 +892,15 @@ void Control::renderText(const uint8 *str, uint16 x, uint16 y, uint8 mode) {
 
 		FrameHeader *chSpr = _resMan->fetchFrame(font, *str - 32);
 		uint8 *sprData = (uint8*)chSpr + sizeof(FrameHeader);
-		for (uint16 cnty = 0; cnty < FROM_LE_16(chSpr->height); cnty++) {
-			for (uint16 cntx = 0; cntx < FROM_LE_16(chSpr->width); cntx++) {
+		for (uint16 cnty = 0; cnty < _resMan->getUint16(chSpr->height); cnty++) {
+			for (uint16 cntx = 0; cntx < _resMan->getUint16(chSpr->width); cntx++) {
 				if (sprData[cntx])
 					dst[cntx] = sprData[cntx];
 			}
-			sprData += FROM_LE_16(chSpr->width);
+			sprData += _resMan->getUint16(chSpr->width);
 			dst += SCREEN_WIDTH;
 		}
-		destX += FROM_LE_16(chSpr->width) - 3;
+		destX += _resMan->getUint16(chSpr->width) - 3;
 		str++;
 	}
 	_system->copyRectToScreen(_screenBuf + y * SCREEN_WIDTH + x, SCREEN_WIDTH, x, y, (destX - x) + 3, 28);
@@ -920,12 +915,12 @@ void Control::renderVolumeBar(uint8 id, uint8 volL, uint8 volR) {
 		FrameHeader *frHead = _resMan->fetchFrame(_resMan->openFetchRes(SR_VLIGHT), (vol + 15) >> 4);
 		uint8 *destMem = _screenBuf + destY * SCREEN_WIDTH + destX;
 		uint8 *srcMem = (uint8*)frHead + sizeof(FrameHeader);
-		for (uint16 cnty = 0; cnty < FROM_LE_16(frHead->height); cnty++) {
-			memcpy(destMem, srcMem, FROM_LE_16(frHead->width));
-			srcMem += FROM_LE_16(frHead->width);
+		for (uint16 cnty = 0; cnty < _resMan->getUint16(frHead->height); cnty++) {
+			memcpy(destMem, srcMem, _resMan->getUint16(frHead->width));
+			srcMem += _resMan->getUint16(frHead->width);
 			destMem += SCREEN_WIDTH;
 		}
-		_system->copyRectToScreen(_screenBuf + destY * SCREEN_WIDTH + destX, SCREEN_WIDTH, destX, destY, FROM_LE_16(frHead->width), FROM_LE_16(frHead->height));
+		_system->copyRectToScreen(_screenBuf + destY * SCREEN_WIDTH + destX, SCREEN_WIDTH, destX, destY, _resMan->getUint16(frHead->width), _resMan->getUint16(frHead->height));
 		_resMan->resClose(SR_VLIGHT);
 		destX += 32;
 	}
@@ -962,7 +957,7 @@ void Control::saveGameToFile(uint8 slot) {
 	uint32 *playerRaw = (uint32*)cpt;
 	for (uint32 cnt2 = 0; cnt2 < playerSize; cnt2++)
 		outf->writeUint32LE(playerRaw[cnt2]);
-	outf->flush();
+	outf->finalize();
 	if (outf->ioFailed())
 		displayMessage(0, "Couldn't write to file '%s' in directory '%s'. Device full?", fName, _saveFileMan->getSavePath());
 	delete outf;
@@ -1038,7 +1033,7 @@ void Control::doRestore(void) {
 }
 
 void Control::delay(uint32 msecs) {
-	OSystem::Event event;
+	Common::Event event;
 
 	uint32 now = _system->getMillis();
 	uint32 endTime = now + msecs;
@@ -1046,29 +1041,24 @@ void Control::delay(uint32 msecs) {
 	_mouseState = 0;
 
 	do {
-		while (_system->pollEvent(event)) {
+		Common::EventManager *eventMan = _system->getEventManager();
+		while (eventMan->pollEvent(event)) {
 			switch (event.type) {
-			case OSystem::EVENT_KEYDOWN:
+			case Common::EVENT_KEYDOWN:
 
 				// Make sure backspace works right (this fixes a small issue on OS X)
 				if (event.kbd.keycode == 8)
 					_keyPressed = 8;
 				else
 					_keyPressed = (byte)event.kbd.ascii;
-				_keyRepeatTime = now + kKeyRepeatInitialDelay;
-				_keyRepeat = _keyPressed;
 				// we skip the rest of the delay and return immediately
 				// to handle keyboard input
 				return;
-			case OSystem::EVENT_KEYUP:
-				_keyRepeatTime = 0;
-				_keyRepeat = 0;
-				break;
-			case OSystem::EVENT_MOUSEMOVE:
+			case Common::EVENT_MOUSEMOVE:
 				_mouseX = event.mouse.x;
 				_mouseY = event.mouse.y;
 				break;
-			case OSystem::EVENT_LBUTTONDOWN:
+			case Common::EVENT_LBUTTONDOWN:
 				_mouseDown = true;
 				_mouseState |= BS1L_BUTTON_DOWN;
 #if defined(_WIN32_WCE) || defined(PALMOS_MODE)
@@ -1076,29 +1066,25 @@ void Control::delay(uint32 msecs) {
 				_mouseY = event.mouse.y;
 #endif
 				break;
-			case OSystem::EVENT_LBUTTONUP:
+			case Common::EVENT_LBUTTONUP:
 				_mouseDown = false;
 				_mouseState |= BS1L_BUTTON_UP;
 				break;
-			case OSystem::EVENT_WHEELUP:
+			case Common::EVENT_WHEELUP:
 				_mouseDown = false;
 				_mouseState |= BS1_WHEEL_UP;
 				break;
-			case OSystem::EVENT_WHEELDOWN:
+			case Common::EVENT_WHEELDOWN:
 				_mouseDown = false;
 				_mouseState |= BS1_WHEEL_DOWN;
 				break;
-			case OSystem::EVENT_QUIT:
+			case Common::EVENT_QUIT:
 				SwordEngine::_systemVars.engineQuit = true;
 				break;
 			default:
 				break;
 			}
 		}
-		if (_keyRepeatTime && now > _keyRepeatTime) {
-			_keyRepeatTime += kKeyRepeatSustainDelay;
-			_keyPressed = _keyRepeat;
-		}
 
 		_system->updateScreen();
 		_system->delayMillis(10);
