@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2003-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,13 +18,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/sword1/control.h $
- * $Id: control.h 22009 2006-04-18 18:23:57Z eriktorbjorn $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/sword1/control.h $
+ * $Id: control.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
-#ifndef BSCONTROL_H
-#define BSCONTROL_H
+#ifndef SWORD1_CONTROL_H
+#define SWORD1_CONTROL_H
 
 #include "common/scummsys.h"
 #include "sword1/sworddefs.h"
@@ -139,8 +142,6 @@ private:
 	uint8 *_font, *_redFont;
 	uint8 *_screenBuf;
 	uint8 _keyPressed;
-	uint8 _keyRepeat;
-	uint32 _keyRepeatTime;
 	void delay(uint32 msecs);
 	uint16 _mouseX, _mouseY, _mouseState;
 	bool _mouseDown;
