@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2003-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/sword1/credits.cpp $
- * $Id: credits.cpp 22231 2006-04-29 22:33:31Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/sword1/credits.cpp $
+ * $Id: credits.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -32,6 +35,7 @@
 
 #include "common/file.h"
 #include "common/util.h"
+#include "common/events.h"
 #include "common/system.h"
 
 
@@ -271,12 +275,13 @@ uint8 CreditsPlayer::getPalIdx(uint8 r, uint8 g, uint8 b) {
 
 void CreditsPlayer::delay(int msecs) {
 
-	OSystem::Event event;
+	Common::Event event;
 	uint32 start = _system->getMillis();
 	do {
-		while (_system->pollEvent(event)) {
+		Common::EventManager *eventMan = _system->getEventManager();
+		while (eventMan->pollEvent(event)) {
 			switch (event.type) {
-			case OSystem::EVENT_QUIT:
+			case Common::EVENT_QUIT:
 				SwordEngine::_systemVars.engineQuit = true;
 				break;
 			default:
