@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2003-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,13 +18,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/sword1/credits.h $
- * $Id: credits.h 20512 2006-02-11 10:01:01Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/sword1/credits.h $
+ * $Id: credits.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
-#ifndef BS1CREDITS_H
-#define BS1CREDITS_H
+#ifndef SWORD1_CREDITS_H
+#define SWORD1_CREDITS_H
 
 #include "common/util.h"
 
@@ -40,7 +43,7 @@ public:
 	uint8 *fetchFile(uint32 fileId, uint32 *size = NULL);
 	uint8 *decompressFile(uint32 fileId);
 	void enterPath(uint32 id);
-	void backToRoot(void) { _bufPos = _buf; };
+	void backToRoot(void) { _bufPos = _buf; }
 private:
 	uint8 *_bufPos;
 	uint8 *_buf;
