@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2003-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/sword1/logic.cpp $
- * $Id: logic.cpp 23847 2006-09-09 14:36:20Z chrilith $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/sword1/logic.cpp $
+ * $Id: logic.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -355,7 +358,7 @@ int Logic::speechDriver(Object *compact) {
 	}
 	if (compact->o_anim_resource) {
 		uint8 *animData = ((uint8*)_resMan->openFetchRes(compact->o_anim_resource)) + sizeof(Header);
-		int32 numFrames = READ_LE_UINT32(animData);
+		int32 numFrames = _resMan->readUint32(animData);
 		animData += 4;
 		compact->o_anim_pc++; // go to next frame of anim
 
@@ -365,10 +368,10 @@ int Logic::speechDriver(Object *compact) {
 
 		AnimUnit *animPtr = (AnimUnit*)(animData + sizeof(AnimUnit) * compact->o_anim_pc);
 		if (!(compact->o_status & STAT_SHRINK)) {
-			compact->o_anim_x = FROM_LE_32(animPtr->animX);
-			compact->o_anim_y = FROM_LE_32(animPtr->animY);
+			compact->o_anim_x = _resMan->getUint32(animPtr->animX);
+			compact->o_anim_y = _resMan->getUint32(animPtr->animY);
 		}
-		compact->o_frame = FROM_LE_32(animPtr->animFrame);
+		compact->o_frame = _resMan->getUint32(animPtr->animFrame);
 		_resMan->resClose(compact->o_anim_resource);
 	}
 	return 0;
@@ -380,13 +383,13 @@ int Logic::fullAnimDriver(Object *compact) {
 		return 1;
 	}
 	uint8 *data = ((uint8*)_resMan->openFetchRes(compact->o_anim_resource)) + sizeof(Header);
-	uint32 numFrames = READ_LE_UINT32(data);
+	uint32 numFrames = _resMan->readUint32(data);
 	data += 4;
 	AnimUnit *animPtr = (AnimUnit*)(data + compact->o_anim_pc * sizeof(AnimUnit));
 
-	compact->o_anim_x = compact->o_xcoord = FROM_LE_32(animPtr->animX);
-	compact->o_anim_y = compact->o_ycoord = FROM_LE_32(animPtr->animY);
-	compact->o_frame = FROM_LE_32(animPtr->animFrame);
+	compact->o_anim_x = compact->o_xcoord = _resMan->getUint32(animPtr->animX);
+	compact->o_anim_y = compact->o_ycoord = _resMan->getUint32(animPtr->animY);
+	compact->o_frame = _resMan->getUint32(animPtr->animFrame);
 
 	compact->o_anim_pc++;
 	if (compact->o_anim_pc == (int)numFrames)
@@ -402,15 +405,15 @@ int Logic::animDriver(Object *compact) {
 		return 1;
 	}
 	uint8 *data = ((uint8*)_resMan->openFetchRes(compact->o_anim_resource)) + sizeof(Header);
-	uint32 numFrames = READ_LE_UINT32(data);
+	uint32 numFrames = _resMan->readUint32(data);
 	AnimUnit *animPtr = (AnimUnit*)(data + 4 + compact->o_anim_pc * sizeof(AnimUnit));
 
 	if (!(compact->o_status & STAT_SHRINK)) {
-		compact->o_anim_x = FROM_LE_32(animPtr->animX);
-		compact->o_anim_y = FROM_LE_32(animPtr->animY);
+		compact->o_anim_x = _resMan->getUint32(animPtr->animX);
+		compact->o_anim_y = _resMan->getUint32(animPtr->animY);
 	}
 
-	compact->o_frame = FROM_LE_32(animPtr->animFrame);
+	compact->o_frame = _resMan->getUint32(animPtr->animFrame);
 	compact->o_anim_pc++;
 	if (compact->o_anim_pc == (int)numFrames)
 		compact->o_logic = LOGIC_script;
@@ -787,7 +790,7 @@ void Logic::setupMcodeTable() {
 		&Logic::fnPurple,
 		&Logic::fnBlack
 	};
-
+	
 	_mcodeTable = mcodeTable;
 }
 
@@ -828,8 +831,8 @@ int Logic::fnAnim(Object *cpt, int32 id, int32 cdt, int32 spr, int32 e, int32 f,
 		animTab = (AnimSet*)((uint8*)_resMan->openFetchRes(cdt) + sizeof(Header));
 		animTab += cpt->o_dir;
 
-		cpt->o_anim_resource = FROM_LE_32(animTab->cdt);
-		cpt->o_resource = FROM_LE_32(animTab->spr);
+		cpt->o_anim_resource = _resMan->getUint32(animTab->cdt);
+		cpt->o_resource = _resMan->getUint32(animTab->spr);
 		_resMan->resClose(cdt);
 	} else {
 		cpt->o_anim_resource = cdt;
@@ -862,14 +865,14 @@ int Logic::fnSetFrame(Object *cpt, int32 id, int32 cdt, int32 spr, int32 frameNo
 	uint8 *data = (uint8*)_resMan->openFetchRes(cdt);
 	data += sizeof(Header);
 	if (frameNo == LAST_FRAME)
-		frameNo = READ_LE_UINT32(data) - 1;
+		frameNo = _resMan->readUint32(data) - 1;
 
 	data += 4;
 	animPtr = (AnimUnit*)(data + frameNo * sizeof(AnimUnit));
 
-	cpt->o_anim_x = FROM_LE_32(animPtr->animX);
-	cpt->o_anim_y = FROM_LE_32(animPtr->animY);
-	cpt->o_frame = FROM_LE_32(animPtr->animFrame);
+	cpt->o_anim_x = _resMan->getUint32(animPtr->animX);
+	cpt->o_anim_y = _resMan->getUint32(animPtr->animY);
+	cpt->o_frame = _resMan->getUint32(animPtr->animFrame);
 
 	cpt->o_resource = spr;
 	cpt->o_status &= ~STAT_SHRINK;
@@ -892,13 +895,13 @@ int Logic::fnFullSetFrame(Object *cpt, int32 id, int32 cdt, int32 spr, int32 fra
 	uint8 *data = (uint8*)_resMan->openFetchRes(cdt) + sizeof(Header);
 
 	if (frameNo == LAST_FRAME)
-		frameNo = READ_LE_UINT32(data) - 1;
+		frameNo = _resMan->readUint32(data) - 1;
 	data += 4;
 
 	AnimUnit *animPtr = (AnimUnit*)(data + sizeof(AnimUnit) * frameNo);
-	cpt->o_anim_x = cpt->o_xcoord = FROM_LE_32(animPtr->animX);
-	cpt->o_anim_y = cpt->o_ycoord = FROM_LE_32(animPtr->animY);
-	cpt->o_frame = FROM_LE_32(animPtr->animFrame);
+	cpt->o_anim_x = cpt->o_xcoord = _resMan->getUint32(animPtr->animX);
+	cpt->o_anim_y = cpt->o_ycoord = _resMan->getUint32(animPtr->animY);
+	cpt->o_frame = _resMan->getUint32(animPtr->animFrame);
 
 	cpt->o_resource = spr;
 	cpt->o_status &= ~STAT_SHRINK;
@@ -957,8 +960,12 @@ int Logic::fnPlaySequence(Object *cpt, int32 id, int32 sequenceId, int32 d, int3
 		CreditsPlayer player(_system, _mixer);
 		player.play();
 	} else {
-		MoviePlayer player(_screen, _mixer, _system);
-		player.play(sequenceId);
+		MoviePlayer *player = makeMoviePlayer(sequenceId, _screen, _textMan, _mixer, _system);
+		if (player) {
+			if (player->load(sequenceId))
+				player->play();
+			delete player;
+		}
 	}
 	return SCRIPT_CONT;
 }
@@ -1112,9 +1119,9 @@ int Logic::fnISpeak(Object *cpt, int32 id, int32 cdt, int32 textNo, int32 spr, i
 		AnimSet *animTab = (AnimSet*)((uint8*)_resMan->openFetchRes(cdt) + sizeof(Header));
 		animTab += cpt->o_dir;
 
-		cpt->o_anim_resource = FROM_LE_32(animTab->cdt);
+		cpt->o_anim_resource = _resMan->getUint32(animTab->cdt);
 		if (animTab->cdt)
-			cpt->o_resource = FROM_LE_32(animTab->spr);
+			cpt->o_resource = _resMan->getUint32(animTab->spr);
 		_resMan->resClose(cdt);
 	} else {
 		cpt->o_anim_resource = cdt;
@@ -1154,8 +1161,8 @@ int Logic::fnISpeak(Object *cpt, int32 id, int32 cdt, int32 textNo, int32 spr, i
 		textCpt->o_target = textCptId;
 
 		// the graphic is a property of Text, so we don't lock/unlock it.
-		uint16 textSpriteWidth  = FROM_LE_16(_textMan->giveSpriteData(textCpt->o_target)->width);
-		uint16 textSpriteHeight = FROM_LE_16(_textMan->giveSpriteData(textCpt->o_target)->height);
+		uint16 textSpriteWidth  = _resMan->getUint16(_textMan->giveSpriteData(textCpt->o_target)->width);
+		uint16 textSpriteHeight = _resMan->getUint16(_textMan->giveSpriteData(textCpt->o_target)->height);
 
 		cpt->o_text_id = textCptId;
 
@@ -1629,7 +1636,7 @@ int Logic::fnQuitGame(Object *cpt, int32 id, int32 a, int32 b, int32 c, int32 d,
 		SwordEngine::_systemVars.engineQuit = true;
 	} else
 		error("fnQuitGame() called");
-	return SCRIPT_STOP;
+	return fnQuit(cpt, id, 0, 0, 0, 0, 0, 0);;
 }
 
 int Logic::fnDeathScreen(Object *cpt, int32 id, int32 a, int32 b, int32 c, int32 d, int32 z, int32 x) {
@@ -1713,6 +1720,8 @@ void Logic::startPosCallFn(uint8 fnId, uint32 param1, uint32 param2, uint32 para
 }
 
 void Logic::runStartScript(const uint8 *data) {
+	// Here data is a static resource defined in staticres.cpp
+	// It is always in little endian
 	uint16 varId = 0;
 	uint8 fnId = 0;
 	uint32 param1 = 0;
