@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2003-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/sword1/music.cpp $
- * $Id: music.cpp 22231 2006-04-29 22:33:31Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/sword1/music.cpp $
+ * $Id: music.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -27,6 +30,7 @@
 #include "common/system.h"
 
 #include "sword1/music.h"
+#include "sound/aiff.h"
 #include "sound/mixer.h"
 #include "sound/mp3.h"
 #include "sound/vorbis.h"
@@ -36,69 +40,156 @@
 
 namespace Sword1 {
 
-WaveAudioStream *makeWaveStream(Common::File *source, uint32 size) {
-	return new WaveAudioStream(source, size);
-}
+class BaseAudioStream : public Audio::AudioStream {
+public:
+	BaseAudioStream(Common::SeekableReadStream *source, bool loop);
+	virtual ~BaseAudioStream();
+	virtual int readBuffer(int16 *buffer, const int numSamples);
+	virtual bool isStereo() const { return _isStereo; }
+	virtual bool endOfData() const { return (_samplesLeft == 0); }
+	virtual int getRate() const { return _rate; }
+protected:
+	Common::SeekableReadStream	*_sourceStream;
+	uint8	*_sampleBuf;
+	uint32	_rate;
+	bool	_isStereo;
+	uint32	_samplesLeft;
+	uint16	_bitsPerSample;
+	bool	_loop;
 
-WaveAudioStream::WaveAudioStream(Common::File *source, uint32 pSize) {
-	int rate, size;
-	byte flags;
+	virtual void rewind() = 0;
+	virtual void reinit(int size, int rate, byte flags);
+};
 
-	_sourceFile = source;
+BaseAudioStream::BaseAudioStream(Common::SeekableReadStream *source, bool loop) {
+	_sourceStream = source;
 	_sampleBuf = (uint8*)malloc(SMP_BUFSIZE);
-	_sourceFile->incRef();
-	if (_sourceFile->isOpen() && Audio::loadWAVFromStream(*_sourceFile, size, rate, flags)) {
-		_isStereo = (flags & Audio::Mixer::FLAG_STEREO) != 0;
-		_rate = rate;
-		if (pSize && (int)pSize < size)
-			size = pSize;
-		assert((uint32)size <= (source->size() - source->pos()));
-		_bitsPerSample = ((flags & Audio::Mixer::FLAG_16BITS) != 0) ? 16 : 8;
-		_samplesLeft = (size * 8) / _bitsPerSample;
-		if ((_bitsPerSample != 16) && (_bitsPerSample != 8))
-			error("WaveAudioStream: unknown wave type");
-	} else {
-		_samplesLeft = 0;
-		_isStereo = false;
-		_bitsPerSample = 16;
-		_rate = 22050;
-	}
+
+	_samplesLeft = 0;
+	_isStereo = false;
+	_bitsPerSample = 16;
+	_rate = 22050;
+	_loop = loop;
 }
 
-WaveAudioStream::~WaveAudioStream(void) {
+BaseAudioStream::~BaseAudioStream() {
 	free(_sampleBuf);
-	_sourceFile->decRef();
 }
 
-int WaveAudioStream::readBuffer(int16 *buffer, const int numSamples) {
-	int samples = MIN((int)_samplesLeft, numSamples);
-	int retVal = samples;
+void BaseAudioStream::reinit(int size, int rate, byte flags) {
+	_isStereo = (flags & Audio::Mixer::FLAG_STEREO) != 0;
+	_rate = rate;
+	assert((uint)size <= (_sourceStream->size() - _sourceStream->pos()));
+	_bitsPerSample = ((flags & Audio::Mixer::FLAG_16BITS) != 0) ? 16 : 8;
+	_samplesLeft = (size * 8) / _bitsPerSample;
+	if ((_bitsPerSample != 16) && (_bitsPerSample != 8))
+		error("BaseAudioStream: unknown sound type");
+}
 
-	while (samples > 0) {
-		int readBytes = MIN(samples * (_bitsPerSample >> 3), SMP_BUFSIZE);
-		_sourceFile->read(_sampleBuf, readBytes);
-		if (_bitsPerSample == 16) {
-			readBytes >>= 1;
-			samples -= readBytes;
-			int16 *src = (int16*)_sampleBuf;
-			while (readBytes--)
-				*buffer++ = (int16)READ_LE_UINT16(src++);
-		} else {
-			samples -= readBytes;
-			int8 *src = (int8*)_sampleBuf;
-			while (readBytes--)
-				*buffer++ = (int16)*src++ << 8;
+int BaseAudioStream::readBuffer(int16 *buffer, const int numSamples) {
+	int retVal = 0;
+
+	while (retVal < numSamples && _samplesLeft > 0) {
+		int samples = MIN((int)_samplesLeft, numSamples - retVal);
+		retVal += samples;
+		_samplesLeft -= samples;
+		while (samples > 0) {
+			int readBytes = MIN(samples * (_bitsPerSample >> 3), SMP_BUFSIZE);
+			_sourceStream->read(_sampleBuf, readBytes);
+			if (_bitsPerSample == 16) {
+				samples -= (readBytes / 2);
+				memcpy(buffer, _sampleBuf, readBytes);
+				buffer += (readBytes / 2);
+			} else {
+				samples -= readBytes;
+				int8 *src = (int8*)_sampleBuf;
+				while (readBytes--)
+					*buffer++ = (int16)*src++ << 8;
+			}
+		}
+		
+		if (!_samplesLeft && _loop) {
+			rewind();
 		}
 	}
-	_samplesLeft -= retVal;
+	
 	return retVal;
 }
 
-bool WaveAudioStream::endOfData(void) const {
+class WaveAudioStream : public BaseAudioStream {
+public:
+	WaveAudioStream(Common::SeekableReadStream *source, bool loop);
+	virtual int readBuffer(int16 *buffer, const int numSamples);
+private:
+	virtual void rewind();
+};
+
+WaveAudioStream::WaveAudioStream(Common::SeekableReadStream *source, bool loop) : BaseAudioStream(source, loop) {
+	rewind();
+
 	if (_samplesLeft == 0)
-		return true;
-	else
-		return false;
+		_loop = false;
+}
+
+void WaveAudioStream::rewind() {
+	int rate, size;
+	byte flags;
+
+	_sourceStream->seek(0);
+
+	if (Audio::loadWAVFromStream(*_sourceStream, size, rate, flags)) {
+		reinit(size, rate, flags);
+	}
+}
+
+int WaveAudioStream::readBuffer(int16 *buffer, const int numSamples) {
+	int retVal = BaseAudioStream::readBuffer(buffer, numSamples);
+
+	if (_bitsPerSample == 16) {
+		for (int i = 0; i < retVal; i++) {
+			buffer[i] = (int16)READ_LE_UINT16(buffer + i);
+		}
+	}
+
+	return retVal;
+}
+
+class AiffAudioStream : public BaseAudioStream {
+public:
+	AiffAudioStream(Common::SeekableReadStream *source, bool loop);
+	virtual int readBuffer(int16 *buffer, const int numSamples);
+private:
+	void rewind();
+};
+
+AiffAudioStream::AiffAudioStream(Common::SeekableReadStream *source, bool loop) : BaseAudioStream(source, loop) {
+	rewind();
+
+	if (_samplesLeft == 0)
+		_loop = false;
+}
+
+void AiffAudioStream::rewind() {
+	int rate, size;
+	byte flags;
+	
+	_sourceStream->seek(0);
+
+	if (Audio::loadAIFFFromStream(*_sourceStream, size, rate, flags)) {
+		reinit(size, rate, flags);
+	}
+}
+
+int AiffAudioStream::readBuffer(int16 *buffer, const int numSamples) {
+	int retVal = BaseAudioStream::readBuffer(buffer, numSamples);
+
+	if (_bitsPerSample == 16) {
+		for (int i = 0; i < retVal; i++) {
+			buffer[i] = (int16)READ_BE_UINT16(buffer + i);
+		}
+	}
+	
+	return retVal;
 }
 
 // This means fading takes 3 seconds.
@@ -107,53 +198,39 @@ bool WaveAudioStream::endOfData(void) const {
 // These functions are only called from Music, so I'm just going to
 // assume that if locking is needed it has already been taken care of.
 
-Audio::AudioStream *MusicHandle::createAudioSource(void) {
-	_file.seek(0);
-	switch (_musicMode) {
-#ifdef USE_MAD
-	case MusicMp3:
-		return Audio::makeMP3Stream(&_file, _file.size());
-#endif
-#ifdef USE_VORBIS
-	case MusicVorbis:
-		return Audio::makeVorbisStream(&_file, _file.size());
-#endif
-	case MusicWave:
-		return makeWaveStream(&_file, 0);
-	case MusicNone: // shouldn't happen
-		warning("createAudioSource ran into null create\n");
-		return NULL;
-	default:
-		error("MusicHandle::createAudioSource: called with illegal MusicMode");
-	}
-	return NULL; // never reached
-}
-
 bool MusicHandle::play(const char *fileBase, bool loop) {
 	char fileName[30];
 	stop();
-	_musicMode = MusicNone;
-#ifdef USE_MAD
-	sprintf(fileName, "%s.mp3", fileBase);
-	if (_file.open(fileName))
-		_musicMode = MusicMp3;
-#endif
+	
 #ifdef USE_VORBIS
-	if (!_file.isOpen()) { // mp3 doesn't exist (or not compiled with MAD support)
+	if (!_audioSource) {
 		sprintf(fileName, "%s.ogg", fileBase);
 		if (_file.open(fileName))
-			_musicMode = MusicVorbis;
+			_audioSource = Audio::makeVorbisStream(&_file, false, 0, 0, loop ? 0 : 1);
 	}
 #endif
-	if (!_file.isOpen()) {
+#ifdef USE_MAD
+	if (!_audioSource) {
+		sprintf(fileName, "%s.mp3", fileBase);
+		if (_file.open(fileName))
+			_audioSource = Audio::makeMP3Stream(&_file, false, 0, 0, loop ? 0 : 1);
+	}
+#endif
+	if (!_audioSource) {
 		sprintf(fileName, "%s.wav", fileBase);
 		if (_file.open(fileName))
-			_musicMode = MusicWave;
-		else
-			return false;
+			_audioSource = new WaveAudioStream(&_file, loop);
 	}
-	_audioSource = createAudioSource();
-	_looping = loop;
+
+	if (!_audioSource) {
+		sprintf(fileName, "%s.aif", fileBase);
+		if (_file.open(fileName))
+			_audioSource = new AiffAudioStream(&_file, loop);
+	}
+	
+	if (!_audioSource)
+		return false;
+
 	fadeUp();
 	return true;
 }
@@ -209,19 +286,14 @@ int MusicHandle::readBuffer(int16 *buffer, const int numSamples) {
 		expectedSamples -= samplesReturned;
 		if ((expectedSamples > 0) && _audioSource->endOfData()) {
 			debug(2, "Music reached EOF");
-			_audioSource->endOfData();
-			if (_looping) {
-				delete _audioSource; // recreate same source.
-				_audioSource = createAudioSource();
-			}
-			if ((!_looping) || (!_audioSource))
-				stop();
+			stop();
 		}
 	}
 	// buffer was filled, now do the fading (if necessary)
 	int samplePos = 0;
 	while ((_fading > 0) && (samplePos < totalSamples)) { // fade down
-		bufStart[samplePos] = (bufStart[samplePos] * --_fading) / _fadeSamples;
+		--_fading;
+		bufStart[samplePos] = (bufStart[samplePos] * _fading) / _fadeSamples;
 		samplePos++;
 		if (_fading == 0) {
 			stop();
@@ -246,7 +318,6 @@ void MusicHandle::stop() {
 	if (_file.isOpen())
 		_file.close();
 	_fading = 0;
-	_looping = false;
 }
 
 Music::Music(Audio::Mixer *pMixer) {
@@ -255,11 +326,11 @@ Music::Music(Audio::Mixer *pMixer) {
 	_converter[0] = NULL;
 	_converter[1] = NULL;
 	_volumeL = _volumeR = 192;
-	_mixer->setupPremix(this);
+	_mixer->playInputStream(Audio::Mixer::kPlainSoundType, &_soundHandle, this, -1, Audio::Mixer::kMaxChannelVolume, 0, false, true);
 }
 
 Music::~Music() {
-	_mixer->setupPremix(0);
+	_mixer->stopHandle(_soundHandle);
 	delete _converter[0];
 	delete _converter[1];
 }
