@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2003-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,63 +18,33 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/sword1/music.h $
- * $Id: music.h 22231 2006-04-29 22:33:31Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/sword1/music.h $
+ * $Id: music.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
-#ifndef BSMUSIC_H
-#define BSMUSIC_H
+#ifndef SWORD1_MUSIC_H
+#define SWORD1_MUSIC_H
 
 #include "common/scummsys.h"
 #include "common/mutex.h"
 #include "common/file.h"
 #include "sound/audiostream.h"
+#include "sound/mixer.h"
 #include "sound/rate.h"
 
-namespace Audio {
-	class Mixer;
-}
-
 namespace Sword1 {
 
 #define TOTAL_TUNES 270
 
-enum MusicMode {
-	MusicNone = 0,
-	MusicWave,
-	MusicMp3,
-	MusicVorbis
-};
-
-class WaveAudioStream : public Audio::AudioStream {
-public:
-	WaveAudioStream(Common::File *source, uint32 pSize);
-	virtual ~WaveAudioStream();
-	virtual int readBuffer(int16 *buffer, const int numSamples);
-	virtual bool isStereo(void) const { return _isStereo; };
-	virtual bool endOfData(void) const;
-	virtual int getRate(void) const { return _rate; };
-private:
-	Common::File	*_sourceFile;
-	uint8	*_sampleBuf;
-	uint32	 _rate;
-	bool	 _isStereo;
-	uint32   _samplesLeft;
-	uint16	 _bitsPerSample;
-};
-
 class MusicHandle : public Audio::AudioStream {
 private:
 	Common::File _file;
-	bool _looping;
 	int32 _fading;
 	int32 _fadeSamples;
-	MusicMode _musicMode;
 	Audio::AudioStream *_audioSource;
-	Audio::AudioStream *createAudioSource(void);
 public:
-	MusicHandle() : _looping(false), _fading(0), _audioSource(NULL) {}
+	MusicHandle() : _fading(0), _audioSource(NULL) {}
 	virtual int readBuffer(int16 *buffer, const int numSamples);
 	bool play(const char *filename, bool loop);
 	void stop();
@@ -108,6 +81,7 @@ private:
 	MusicHandle _handles[2];
 	Audio::RateConverter *_converter[2];
 	Audio::Mixer *_mixer;
+	Audio::SoundHandle _soundHandle;
 	uint32 _sampleRate;
 	Common::Mutex _mutex;
 
