@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2003-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/sword1/object.h $
- * $Id: object.h 21526 2006-04-01 12:47:09Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/sword1/object.h $
+ * $Id: object.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -32,14 +35,12 @@ namespace Sword1 {
 #define	O_GRID_SIZE		200
 #define	EXTRA_GRID_SIZE	20
 
-#if !defined(__GNUC__)
-	#pragma START_PACK_STRUCTS
-#endif
+#include "common/pack-start.h"	// START STRUCT PACKING
 
 struct OEventSlot {			//receiving event list in the compact -
 	int32	o_event;		//array of these with O_TOTAL_EVENTS elements
 	int32	o_event_script;
-} GCC_PACK;	// size = 2*int32 = 8 bytes
+};	// size = 2*int32 = 8 bytes
 
 #define	TOTAL_script_levels	5
 
@@ -47,12 +48,12 @@ struct ScriptTree {			//this is a logic tree, used by OBJECTs
 	int32	o_script_level;						//logic level
 	int32	o_script_id[TOTAL_script_levels];	//script id's (are unique to each level)
 	int32	o_script_pc[TOTAL_script_levels];	//pc of script for each (if script_manager)
-} GCC_PACK;	// size = 11*int32 = 44 bytes
+};	// size = 11*int32 = 44 bytes
 
 struct TalkOffset {
 	int32	x;
 	int32	y;
-} GCC_PACK;	// size = 2*int32 = 8 bytes
+};	// size = 2*int32 = 8 bytes
 
 struct WalkData {
 	int32	frame;
@@ -60,7 +61,7 @@ struct WalkData {
 	int32	y;
 	int32	step;
 	int32	dir;
-} GCC_PACK;	// size = 5*int32 = 20 bytes
+};	// size = 5*int32 = 20 bytes
 
 struct Object {
 	int32	o_type;						// 0  broad description of type - object, floor, etc.
@@ -118,11 +119,9 @@ struct Object {
 
 	WalkData	o_route[O_WALKANIM_SIZE];	// 340	size = 600*20 bytes = 12000
 				// mega size = 12340 bytes (+ 8 byte offset table + 20 byte header = 12368)
-} GCC_PACK;
+};
 
-#if !defined(__GNUC__)
-	#pragma END_PACK_STRUCTS
-#endif
+#include "common/pack-end.h"	// END STRUCT PACKING
 
 } // End of namespace Sword1
 
