@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2003-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,13 +18,12 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/sword1/objectman.cpp $
- * $Id: objectman.cpp 21500 2006-03-29 15:59:37Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/sword1/objectman.cpp $
+ * $Id: objectman.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
 #include "common/stdafx.h"
-#include "common/endian.h"
 #include "common/util.h"
 
 #include "sword1/objectman.h"
@@ -88,7 +90,7 @@ uint8 ObjectMan::fnCheckForTextLine(uint32 textId) {
 
 	uint8 lang = SwordEngine::_systemVars.language;
 	uint32 *textData = (uint32*)((uint8*)_resMan->openFetchRes(_textList[textId / ITM_PER_SEC][lang]) + sizeof(Header));
-	if ((textId & ITM_ID) < READ_LE_UINT32(textData)) {
+	if ((textId & ITM_ID) < _resMan->readUint32(textData)) {
 		textData++;
 		if (textData[textId & ITM_ID])
 			retVal = 1;
@@ -100,11 +102,11 @@ uint8 ObjectMan::fnCheckForTextLine(uint32 textId) {
 char *ObjectMan::lockText(uint32 textId) {
 	uint8 lang = SwordEngine::_systemVars.language;
 	char *addr = (char*)_resMan->openFetchRes(_textList[textId / ITM_PER_SEC][lang]) + sizeof(Header);
-	if ((textId & ITM_ID) >= READ_LE_UINT32(addr)) {
-		warning("ObjectMan::lockText(%d): only %d texts in file", textId & ITM_ID, READ_LE_UINT32(addr));
+	if ((textId & ITM_ID) >= _resMan->readUint32(addr)) {
+		warning("ObjectMan::lockText(%d): only %d texts in file", textId & ITM_ID, _resMan->readUint32(addr));
 		textId = 0; // get first line instead
 	}
-	uint32 offset = READ_LE_UINT32(addr + ((textId & ITM_ID) + 1)* 4);
+	uint32 offset = _resMan->readUint32(addr + ((textId & ITM_ID) + 1)* 4);
 	if (offset == 0) {
 		warning("ObjectMan::lockText(%d): text number has no text lines", textId);
 		return _errorStr;
@@ -118,7 +120,7 @@ void ObjectMan::unlockText(uint32 textId) {
 
 uint32 ObjectMan::lastTextNumber(int section) {
 	uint8 *data = (uint8*)_resMan->openFetchRes(_textList[section][SwordEngine::_systemVars.language]) + sizeof(Header);
-	uint32 result = READ_LE_UINT32(data) - 1;
+	uint32 result = _resMan->readUint32(data) - 1;
 	_resMan->resClose(_textList[section][SwordEngine::_systemVars.language]);
 	return result;
 }
