@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2003-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,15 +18,15 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/sword1/objectman.h $
- * $Id: objectman.h 20512 2006-02-11 10:01:01Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/sword1/objectman.h $
+ * $Id: objectman.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
 // this is the object manager. our equivalent to protocol.c and coredata.c
 
-#ifndef OBJECTMAN_H
-#define OBJECTMAN_H
+#ifndef SWORD1_OBJECTMAN_H
+#define SWORD1_OBJECTMAN_H
 
 #include "sword1/resman.h"
 #include "sword1/sworddefs.h"
