@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2003-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,17 +18,18 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/sword1/resman.h $
- * $Id: resman.h 20512 2006-02-11 10:01:01Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/sword1/resman.h $
+ * $Id: resman.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
-#ifndef RESMAN_H
-#define RESMAN_H
+#ifndef SWORD1_RESMAN_H
+#define SWORD1_RESMAN_H
 
 #include "sword1/memman.h"
 #include "common/file.h"
 #include "sword1/sworddefs.h"
+#include "common/endian.h"
 
 namespace Sword1 {
 
@@ -62,7 +66,7 @@ struct Prj {
 
 class ResMan {
 public:
-	ResMan(const char *fileName);
+	ResMan(const char *fileName, bool isMacFile);
 	~ResMan(void);
 	void flush(void);
 	void resClose(uint32 id);
@@ -74,6 +78,36 @@ public:
 	Header *lockScript(uint32 scrID);
 	void unlockScript(uint32 scrID);
 	FrameHeader *fetchFrame(void *resourceData, uint32 frameNo);
+	
+	uint16 getUint16(uint16 value) {
+		return (_isBigEndian) ? FROM_BE_16(value): FROM_LE_16(value);
+	}
+	uint32 getUint32(uint32 value) {
+		return (_isBigEndian) ? FROM_BE_32(value): FROM_LE_32(value);
+	}
+	uint16 getLEUint16(uint16 value) {
+		return FROM_LE_16(value);
+	}
+	uint32 getLEUint32(uint32 value) {
+		return FROM_LE_32(value);
+	}
+	uint16 readUint16(const void *ptr) {
+		return (_isBigEndian) ? READ_BE_UINT16(ptr): READ_LE_UINT16(ptr);
+	}
+	uint32 readUint32(const void *ptr) {
+		return (_isBigEndian) ? READ_BE_UINT32(ptr):READ_LE_UINT32(ptr);
+	}
+	uint32 readLEUint32(const void *ptr) {
+		return READ_LE_UINT32(ptr);
+	}
+	uint16 toUint16(uint16 value) {
+		return (_isBigEndian) ? TO_BE_16(value): TO_LE_16(value);
+	}
+	uint32 toUint32(uint32 value) {
+		return (_isBigEndian) ? TO_BE_32(value): TO_LE_32(value);
+	}
+
+
 private:
 	uint32	   resLength(uint32 id);
 	MemHandle *resHandle(uint32 id);
@@ -82,6 +116,8 @@ private:
 
 	void openCptResourceBigEndian(uint32 id);
 	void openScriptResourceBigEndian(uint32 id);
+	void openCptResourceLittleEndian(uint32 id);
+	void openScriptResourceLittleEndian(uint32 id);
 
 	void loadCluDescript(const char *fileName);
 	void freeCluDescript(void);
@@ -91,6 +127,7 @@ private:
 	static uint32 _srIdList[29];
 	Clu *_openCluStart, *_openCluEnd;
 	int  _openClus;
+	bool _isBigEndian;
 };
 
 } // End of namespace Sword1
