@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) Revolution Software Ltd.
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/sword1/router.h $
- * $Id: router.h 21526 2006-04-01 12:47:09Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/sword1/router.h $
+ * $Id: router.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -27,9 +30,7 @@
 
 namespace Sword1 {
 
-#if !defined(__GNUC__)
-	#pragma START_PACK_STRUCTS
-#endif
+#include "common/pack-start.h"	// START STRUCT PACKING
 
 struct BarData {
 	int16 x1;
@@ -43,7 +44,7 @@ struct BarData {
 	int16 dx;	// x2 - x1
 	int16 dy;	// y2 - y1
 	int32 co;	// co = (y1*dx) - (x1*dy) from an equation for a line y*dx = x*dy + co
-} GCC_PACK;
+};
 
 struct NodeData {
 	int16 x;
@@ -51,11 +52,9 @@ struct NodeData {
 	int16 level;
 	int16 prev;
 	int16 dist;
-} GCC_PACK;
+};
 
-#if !defined(__GNUC__)
-	#pragma END_PACK_STRUCTS
-#endif
+#include "common/pack-end.h"	// END STRUCT PACKING
 
 struct FloorData {
 	int32		nbars;
