@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2003-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/sword1/sound.cpp $
- * $Id: sound.cpp 23918 2006-09-17 15:37:10Z eriktorbjorn $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/sword1/sound.cpp $
+ * $Id: sound.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -69,10 +72,6 @@ int Sound::addToQueue(int32 fxNo) {
 			warning("Sound queue overflow");
 			return 0;
 		}
-		if ((fxNo == 168) && (SwordEngine::_systemVars.isDemo)) {
-			// this sound doesn't exist in demo
-			return 0;
-		}
 		_resMan->resOpen(_fxList[fxNo].sampleId);
 		_fxQueue[_endOfQueue].id = fxNo;
 		if (_fxList[fxNo].type == FX_SPOT)
@@ -172,7 +171,7 @@ void Sound::playSample(QueueElement *elem) {
 						flags |= Audio::Mixer::FLAG_STEREO;
 					if (_fxList[elem->id].type == FX_LOOP)
 						flags |= Audio::Mixer::FLAG_LOOP;
-					_mixer->playRaw(&elem->handle, sampleData + 0x2C, size, 11025, flags, elem->id, volume, pan);
+					_mixer->playRaw(Audio::Mixer::kSFXSoundType, &elem->handle, sampleData + 0x2C, size, 11025, flags, elem->id, volume, pan);
 			}
 		} else
 			break;
@@ -196,12 +195,12 @@ bool Sound::startSpeech(uint16 roomNo, uint16 localNo) {
 			uint32 size;
 			int16 *data = uncompressSpeech(index + _cowHeaderSize, sampleSize, &size);
 			if (data)
-				_mixer->playRaw(&_speechHandle, data, size, 11025, SPEECH_FLAGS, SOUND_SPEECH_ID, speechVol, speechPan);
+				_mixer->playRaw(Audio::Mixer::kSpeechSoundType, &_speechHandle, data, size, 11025, SPEECH_FLAGS, SOUND_SPEECH_ID, speechVol, speechPan);
 		}
 #ifdef USE_MAD
 		else if (_cowMode == CowMp3) {
 			_cowFile.seek(index);
-			_mixer->playInputStream(Audio::Mixer::kSFXSoundType, &_speechHandle, Audio::makeMP3Stream(&_cowFile, sampleSize), SOUND_SPEECH_ID, speechVol, speechPan);
+			_mixer->playInputStream(Audio::Mixer::kSpeechSoundType, &_speechHandle, Audio::makeMP3Stream(&_cowFile, sampleSize), SOUND_SPEECH_ID, speechVol, speechPan);
 			// with compressed audio, we can't calculate the wave volume.
 			// so default to talking.
 			for (int cnt = 0; cnt < 480; cnt++)
@@ -212,7 +211,7 @@ bool Sound::startSpeech(uint16 roomNo, uint16 localNo) {
 #ifdef USE_VORBIS
 		else if (_cowMode == CowVorbis) {
 			_cowFile.seek(index);
-			_mixer->playInputStream(Audio::Mixer::kSFXSoundType, &_speechHandle, Audio::makeVorbisStream(&_cowFile, sampleSize), SOUND_SPEECH_ID, speechVol, speechPan);
+			_mixer->playInputStream(Audio::Mixer::kSpeechSoundType, &_speechHandle, Audio::makeVorbisStream(&_cowFile, sampleSize), SOUND_SPEECH_ID, speechVol, speechPan);
 			for (int cnt = 0; cnt < 480; cnt++)
 				_waveVolume[cnt] = true;
 			_waveVolPos = 0;
@@ -333,14 +332,6 @@ void Sound::initCowSystem(void) {
 	/* look for speech1/2.clu in the data dir
 	   and speech/speech.clu (running from cd or using cd layout)
 	*/
-#ifdef USE_MAD
-	sprintf(cowName, "SPEECH%d.CL3", SwordEngine::_systemVars.currentCD);
-	_cowFile.open(cowName);
-	if (_cowFile.isOpen()) {
-		debug(1, "Using MP3 compressed Speech Cluster");
-		_cowMode = CowMp3;
-	}
-#endif
 #ifdef USE_VORBIS
 	if (!_cowFile.isOpen()) {
 		sprintf(cowName, "SPEECH%d.CLV", SwordEngine::_systemVars.currentCD);
@@ -350,6 +341,16 @@ void Sound::initCowSystem(void) {
 			_cowMode = CowVorbis;
 		}
 	}
+#endif
+#ifdef USE_MAD
+	if (!_cowFile.isOpen()) {
+		sprintf(cowName, "SPEECH%d.CL3", SwordEngine::_systemVars.currentCD);
+		_cowFile.open(cowName);
+		if (_cowFile.isOpen()) {
+			debug(1, "Using MP3 compressed Speech Cluster");
+			_cowMode = CowMp3;
+		}
+	}
 #endif
 	if (!_cowFile.isOpen()) {
 		sprintf(cowName, "SPEECH%d.CLU", SwordEngine::_systemVars.currentCD);
@@ -380,10 +381,8 @@ void Sound::initCowSystem(void) {
 }
 
 void Sound::closeCowSystem(void) {
-	if (_cowFile.isOpen())
-		_cowFile.close();
-	if (_cowHeader)
-		free(_cowHeader);
+	_cowFile.close();
+	free(_cowHeader);
 	_cowHeader = NULL;
 	_currentCowFile = 0;
 }
