@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2003-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,20 +18,20 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/sword1/sword1.cpp $
- * $Id: sword1.cpp 22315 2006-05-03 20:43:26Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/sword1/sword1.cpp $
+ * $Id: sword1.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
 #include "common/stdafx.h"
 #include "sword1/sword1.h"
 
-#include "backends/fs/fs.h"
-
 #include "base/plugins.h"
 #include "common/config-manager.h"
 #include "common/file.h"
+#include "common/fs.h"
 #include "common/timer.h"
+#include "common/events.h"
 #include "common/system.h"
 
 #include "sword1/resman.h"
@@ -52,17 +55,31 @@ static const PlainGameDescriptor sword1FullSettings =
 	{"sword1", "Broken Sword 1: The Shadow of the Templars"};
 static const PlainGameDescriptor sword1DemoSettings =
 	{"sword1demo", "Broken Sword 1: The Shadow of the Templars (Demo)"};
+static const PlainGameDescriptor sword1MacFullSettings =
+	{"sword1mac", "Broken Sword 1: The Shadow of the Templars (Mac)"};
+static const PlainGameDescriptor sword1MacDemoSettings =
+	{"sword1macdemo", "Broken Sword 1: The Shadow of the Templars (Mac demo)"};
 
 // check these subdirectories (if present)
 static const char *g_dirNames[] = {	"clusters",	"speech" };
 
-#define NUM_FILES_TO_CHECK 5
+#define NUM_COMMON_FILES_TO_CHECK 1
+#define NUM_PC_FILES_TO_CHECK 3
+#define NUM_MAC_FILES_TO_CHECK 4
+#define NUM_DEMO_FILES_TO_CHECK 1
+#define NUM_MAC_DEMO_FILES_TO_CHECK 1
+#define NUM_FILES_TO_CHECK NUM_COMMON_FILES_TO_CHECK + NUM_PC_FILES_TO_CHECK + NUM_MAC_FILES_TO_CHECK + NUM_DEMO_FILES_TO_CHECK + NUM_MAC_DEMO_FILES_TO_CHECK
 static const char *g_filesToCheck[NUM_FILES_TO_CHECK] = { // these files have to be found
-	"swordres.rif",
-	"general.clu",
-	"compacts.clu",
-	"scripts.clu",
+	"swordres.rif", // Mac and PC version
+	"general.clu", // PC version only
+	"compacts.clu", // PC version only
+	"scripts.clu", // PC version only
+	"general.clm", // Mac version only
+	"compacts.clm", // Mac version only
+	"scripts.clm", // Mac version only
+	"paris2.clm", // Mac version (full game only)
 	"cows.mad",	// this one should only exist in the demo version
+	"scripts.clm", // Mac version both demo and full game
 	// the engine needs several more files to work, but checking these should be sufficient
 };
 
@@ -70,6 +87,8 @@ GameList Engine_SWORD1_gameIDList() {
 	GameList games;
 	games.push_back(sword1FullSettings);
 	games.push_back(sword1DemoSettings);
+	games.push_back(sword1MacFullSettings);
+	games.push_back(sword1MacDemoSettings);
 	return games;
 }
 
@@ -78,19 +97,23 @@ GameDescriptor Engine_SWORD1_findGameID(const char *gameid) {
 		return sword1FullSettings;
 	if (0 == scumm_stricmp(gameid, sword1DemoSettings.gameid))
 		return sword1DemoSettings;
+	if (0 == scumm_stricmp(gameid, sword1MacFullSettings.gameid))
+		return sword1MacFullSettings;
+	if (0 == scumm_stricmp(gameid, sword1MacDemoSettings.gameid))
+		return sword1MacDemoSettings;
 	return GameDescriptor();
 }
 
 void Sword1CheckDirectory(const FSList &fslist, bool *filesFound) {
 	for (FSList::const_iterator file = fslist.begin(); file != fslist.end(); ++file) {
 		if (!file->isDirectory()) {
-			const char *fileName = file->displayName().c_str();
+			const char *fileName = file->name().c_str();
 			for (int cnt = 0; cnt < NUM_FILES_TO_CHECK; cnt++)
 				if (scumm_stricmp(fileName, g_filesToCheck[cnt]) == 0)
 					filesFound[cnt] = true;
 		} else {
 			for (int cnt = 0; cnt < ARRAYSIZE(g_dirNames); cnt++)
-				if (scumm_stricmp(file->displayName().c_str(), g_dirNames[cnt]) == 0) {
+				if (scumm_stricmp(file->name().c_str(), g_dirNames[cnt]) == 0) {
 					FSList fslist2;
 					if (file->listDir(fslist2, FilesystemNode::kListFilesOnly))
 						Sword1CheckDirectory(fslist2, filesFound);
@@ -99,23 +122,43 @@ void Sword1CheckDirectory(const FSList &fslist, bool *filesFound) {
 	}
 }
 
-DetectedGameList Engine_SWORD1_detectGames(const FSList &fslist) {
-	int i;
-	DetectedGameList detectedGames;
+GameList Engine_SWORD1_detectGames(const FSList &fslist) {
+	int i, j;
+	GameList detectedGames;
 	bool filesFound[NUM_FILES_TO_CHECK];
 	for (i = 0; i < NUM_FILES_TO_CHECK; i++)
 		filesFound[i] = false;
-
+		
 	Sword1CheckDirectory(fslist, filesFound);
 	bool mainFilesFound = true;
-	for (i = 0; i < NUM_FILES_TO_CHECK -1; i++)
+	bool pcFilesFound = true;
+	bool macFilesFound = true;
+	bool demoFilesFound = true;
+	bool macDemoFilesFound = true;
+	for (i = 0; i < NUM_COMMON_FILES_TO_CHECK; i++)
 		if (!filesFound[i])
 			mainFilesFound = false;
+	for (j = 0; j < NUM_PC_FILES_TO_CHECK; i++, j++)
+		if (!filesFound[i])
+			pcFilesFound = false;
+	for (j = 0; j < NUM_MAC_FILES_TO_CHECK; i++, j++)
+		if (!filesFound[i])
+			macFilesFound = false;
+	for (j = 0; j < NUM_DEMO_FILES_TO_CHECK; i++, j++)
+		if (!filesFound[i])
+			demoFilesFound = false;
+	for (j = 0; j < NUM_DEMO_FILES_TO_CHECK; i++, j++)
+		if (!filesFound[i])
+			macDemoFilesFound = false;
 
-	if (mainFilesFound && filesFound[NUM_FILES_TO_CHECK - 1])
+	if (mainFilesFound && pcFilesFound && demoFilesFound)
 		detectedGames.push_back(sword1DemoSettings);
-	else if (mainFilesFound)
+	else if (mainFilesFound && pcFilesFound)
 		detectedGames.push_back(sword1FullSettings);
+	else if (mainFilesFound && macFilesFound)
+		detectedGames.push_back(sword1MacFullSettings);
+	else if (mainFilesFound && macDemoFilesFound)
+		detectedGames.push_back(sword1MacDemoSettings);
 
 	return detectedGames;
 }
@@ -126,24 +169,19 @@ PluginError Engine_SWORD1_create(OSystem *syst, Engine **engine) {
 	return kNoError;
 }
 
-REGISTER_PLUGIN(SWORD1, "Broken Sword");
+REGISTER_PLUGIN(SWORD1, "Broken Sword", "Broken Sword Games (C) Revolution");
 
 namespace Sword1 {
 
 SystemVars SwordEngine::_systemVars;
 
-void SwordEngine::errorString(const char *buf1, char *buf2) {
-	strcpy(buf2, buf1);
-}
-
 SwordEngine::SwordEngine(OSystem *syst)
 	: Engine(syst) {
 
-	if (0 == scumm_stricmp(ConfMan.get("gameid").c_str(), "sword1demo"))
+	if (!scumm_stricmp(ConfMan.get("gameid").c_str(), "sword1demo"))
 		_features = GF_DEMO;
 	else
-		_features = 0;
-	
+		_features = 0;	
 
 	if (!_mixer->isReady())
 		warning("Sound initialization failed");
@@ -177,11 +215,17 @@ int SwordEngine::init() {
 		initCommonGFX(true);
 		_system->initSize(640, 480);
 	_system->endGFXTransaction();
+	
+	if ( 0 == scumm_stricmp(ConfMan.get("gameid").c_str(), "sword1mac") ||
+	     0 == scumm_stricmp(ConfMan.get("gameid").c_str(), "sword1macdemo") )
+		_systemVars.isMac = true;
+	else
+		_systemVars.isMac = false;	
 
 	checkCdFiles();
 
 	debug(5, "Starting resource manager");
-	_resMan = new ResMan("swordres.rif");
+	_resMan = new ResMan("swordres.rif", _systemVars.isMac);
 	debug(5, "Starting object manager");
 	_objectMan = new ObjectMan(_resMan);
 	_mixer->setVolumeForSoundType(Audio::Mixer::kSFXSoundType, Audio::Mixer::kMaxMixerVolume);
@@ -292,7 +336,7 @@ static const char *errorMsgs[] = {
 	"Please copy these files from their corresponding CDs:\n"
 };
 
-const CdFile SwordEngine::_cdFileList[] = {
+const CdFile SwordEngine::_pcCdFileList[] = {
 	{ "paris2.clu", FLAG_CD1 },
 	{ "ireland.clu", FLAG_CD2 },
 	{ "paris3.clu", FLAG_CD1 },
@@ -321,29 +365,83 @@ const CdFile SwordEngine::_cdFileList[] = {
 #endif
 };
 
+const CdFile SwordEngine::_macCdFileList[] = {
+	{ "paris2.clm", FLAG_CD1 },
+	{ "ireland.clm", FLAG_CD2 },
+	{ "paris3.clm", FLAG_CD1 },
+	{ "paris4.clm", FLAG_CD1 },
+	{ "scotland.clm", FLAG_CD2 },
+	{ "spain.clm", FLAG_CD2 },
+	{ "syria.clm", FLAG_CD2 },
+	{ "train.clm", FLAG_CD2 },
+	{ "compacts.clm", FLAG_CD1 | FLAG_DEMO | FLAG_IMMED },
+	{ "general.clm", FLAG_CD1 | FLAG_DEMO | FLAG_IMMED },
+	{ "maps.clm", FLAG_CD1 | FLAG_DEMO },
+	{ "paris1.clm", FLAG_CD1 | FLAG_DEMO },
+	{ "scripts.clm", FLAG_CD1 | FLAG_DEMO | FLAG_IMMED },
+	{ "swordres.rif", FLAG_CD1 | FLAG_DEMO | FLAG_IMMED },
+	{ "text.clm", FLAG_CD1 | FLAG_DEMO },
+	{ "speech1.clu", FLAG_SPEECH1 },
+	 { "speech2.clu", FLAG_SPEECH2 }
+#ifdef USE_MAD
+	,{ "speech1.cl3", FLAG_SPEECH1 },
+	 { "speech2.cl3", FLAG_SPEECH2 }
+#endif
+#ifdef USE_VORBIS
+	,{ "speech1.clv", FLAG_SPEECH1 },
+	 { "speech2.clv", FLAG_SPEECH2 }
+#endif
+};
+
+
 void SwordEngine::showFileErrorMsg(uint8 type, bool *fileExists) {
 	char msg[1024];
 	int missCnt = 0, missNum = 0;
-	for (int i = 0; i < ARRAYSIZE(_cdFileList); i++)
-		if (!fileExists[i]) {
-			missCnt++;
-			missNum = i;
-		}
-	assert(missCnt > 0); // this function shouldn't get called if there's nothing missing.
-	warning("%d files missing", missCnt);
-	int msgId = (type == TYPE_IMMED) ? 0 : 2;
-	if (missCnt == 1) {
-		sprintf(msg, errorMsgs[msgId],
-				_cdFileList[missNum].name, (_cdFileList[missNum].flags & FLAG_CD2) ? 2 : 1);
-		warning(msg);
-	} else {
-		char *pos = msg + sprintf(msg, errorMsgs[msgId + 1], missCnt);
-		warning(msg);
-		for (int i = 0; i < ARRAYSIZE(_cdFileList); i++)
+
+	if (_systemVars.isMac) {
+		for (int i = 0; i < ARRAYSIZE(_macCdFileList); i++)
 			if (!fileExists[i]) {
-				warning("\"%s\" (CD %d)", _cdFileList[i].name, (_cdFileList[i].flags & FLAG_CD2) ? 2 : 1);
-				pos += sprintf(pos, "\"%s\" (CD %d)\n", _cdFileList[i].name, (_cdFileList[i].flags & FLAG_CD2) ? 2 : 1);
+				missCnt++;
+				missNum = i;
 			}
+		assert(missCnt > 0); // this function shouldn't get called if there's nothing missing.
+		warning("%d files missing", missCnt);
+		int msgId = (type == TYPE_IMMED) ? 0 : 2;
+		if (missCnt == 1) {
+			sprintf(msg, errorMsgs[msgId],
+					_macCdFileList[missNum].name, (_macCdFileList[missNum].flags & FLAG_CD2) ? 2 : 1);
+			warning(msg);
+		} else {
+			char *pos = msg + sprintf(msg, errorMsgs[msgId + 1], missCnt);
+			warning(msg);
+			for (int i = 0; i < ARRAYSIZE(_macCdFileList); i++)
+				if (!fileExists[i]) {
+					warning("\"%s\" (CD %d)", _macCdFileList[i].name, (_macCdFileList[i].flags & FLAG_CD2) ? 2 : 1);
+					pos += sprintf(pos, "\"%s\" (CD %d)\n", _macCdFileList[i].name, (_macCdFileList[i].flags & FLAG_CD2) ? 2 : 1);
+				}
+		}
+	} else {
+		for (int i = 0; i < ARRAYSIZE(_pcCdFileList); i++)
+			if (!fileExists[i]) {
+				missCnt++;
+				missNum = i;
+			}
+		assert(missCnt > 0); // this function shouldn't get called if there's nothing missing.
+		warning("%d files missing", missCnt);
+		int msgId = (type == TYPE_IMMED) ? 0 : 2;
+		if (missCnt == 1) {
+			sprintf(msg, errorMsgs[msgId],
+					_pcCdFileList[missNum].name, (_pcCdFileList[missNum].flags & FLAG_CD2) ? 2 : 1);
+			warning(msg);
+		} else {
+			char *pos = msg + sprintf(msg, errorMsgs[msgId + 1], missCnt);
+			warning(msg);
+			for (int i = 0; i < ARRAYSIZE(_pcCdFileList); i++)
+				if (!fileExists[i]) {
+					warning("\"%s\" (CD %d)", _pcCdFileList[i].name, (_pcCdFileList[i].flags & FLAG_CD2) ? 2 : 1);
+					pos += sprintf(pos, "\"%s\" (CD %d)\n", _pcCdFileList[i].name, (_pcCdFileList[i].flags & FLAG_CD2) ? 2 : 1);
+				}
+		}
 	}
 	GUI::MessageDialog dialog(msg);
 	dialog.runModal();
@@ -352,7 +450,6 @@ void SwordEngine::showFileErrorMsg(uint8 type, bool *fileExists) {
 }
 
 void SwordEngine::checkCdFiles(void) { // check if we're running from cd, hdd or what...
-	Common::File test;
 	bool fileExists[30];
 	bool isFullVersion = false; // default to demo version
 	bool missingTypes[8] = { false, false, false, false, false, false, false, false };
@@ -362,18 +459,33 @@ void SwordEngine::checkCdFiles(void) { // check if we're running from cd, hdd or
 	_systemVars.playSpeech = true;
 
 	// check all files and look out if we can find a file that wouldn't exist if this was the demo version
-	for (int fcnt = 0; fcnt < ARRAYSIZE(_cdFileList); fcnt++) {
-		if (test.open(_cdFileList[fcnt].name)) {
-			test.close();
-			fileExists[fcnt] = true;
-			flagsToBool(foundTypes, _cdFileList[fcnt].flags);
-			if (!(_cdFileList[fcnt].flags & FLAG_DEMO))
-				isFullVersion = true;
-			if (_cdFileList[fcnt].flags & FLAG_CD2)
-				cd2FilesFound = true;
-		} else {
-			flagsToBool(missingTypes, _cdFileList[fcnt].flags);
-			fileExists[fcnt] = false;
+	if (_systemVars.isMac) {
+		for (int fcnt = 0; fcnt < ARRAYSIZE(_macCdFileList); fcnt++) {
+			if (Common::File::exists(_macCdFileList[fcnt].name)) {
+				fileExists[fcnt] = true;
+				flagsToBool(foundTypes, _macCdFileList[fcnt].flags);
+				if (!(_macCdFileList[fcnt].flags & FLAG_DEMO))
+					isFullVersion = true;
+				if (_macCdFileList[fcnt].flags & FLAG_CD2)
+					cd2FilesFound = true;
+			} else {
+				flagsToBool(missingTypes, _macCdFileList[fcnt].flags);
+				fileExists[fcnt] = false;
+			}
+		}
+	} else {
+		for (int fcnt = 0; fcnt < ARRAYSIZE(_pcCdFileList); fcnt++) {
+			if (Common::File::exists(_pcCdFileList[fcnt].name)) {
+				fileExists[fcnt] = true;
+				flagsToBool(foundTypes, _pcCdFileList[fcnt].flags);
+				if (!(_pcCdFileList[fcnt].flags & FLAG_DEMO))
+					isFullVersion = true;
+				if (_pcCdFileList[fcnt].flags & FLAG_CD2)
+					cd2FilesFound = true;
+			} else {
+				flagsToBool(missingTypes, _pcCdFileList[fcnt].flags);
+				fileExists[fcnt] = false;
+			}
 		}
 	}
 
@@ -395,13 +507,23 @@ void SwordEngine::checkCdFiles(void) { // check if we're running from cd, hdd or
 		somethingMissing |= missingTypes[i];
 	if (somethingMissing) { // okay, there *are* files missing
 		// first, update the fileExists[] array depending on our changed missingTypes
-		for (int fileCnt = 0; fileCnt < ARRAYSIZE(_cdFileList); fileCnt++)
-			if (!fileExists[fileCnt]) {
-				fileExists[fileCnt] = true;
-				for (int flagCnt = 0; flagCnt < 8; flagCnt++)
-					if (missingTypes[flagCnt] && ((_cdFileList[fileCnt].flags & (1 << flagCnt)) != 0))
-						fileExists[fileCnt] = false; // this is one of the files we were looking for
-			}
+		if (_systemVars.isMac) {
+			for (int fileCnt = 0; fileCnt < ARRAYSIZE(_macCdFileList); fileCnt++)
+				if (!fileExists[fileCnt]) {
+					fileExists[fileCnt] = true;
+					for (int flagCnt = 0; flagCnt < 8; flagCnt++)
+						if (missingTypes[flagCnt] && ((_macCdFileList[fileCnt].flags & (1 << flagCnt)) != 0))
+							fileExists[fileCnt] = false; // this is one of the files we were looking for
+				}
+		} else {
+			for (int fileCnt = 0; fileCnt < ARRAYSIZE(_pcCdFileList); fileCnt++)
+				if (!fileExists[fileCnt]) {
+					fileExists[fileCnt] = true;
+					for (int flagCnt = 0; flagCnt < 8; flagCnt++)
+						if (missingTypes[flagCnt] && ((_pcCdFileList[fileCnt].flags & (1 << flagCnt)) != 0))
+							fileExists[fileCnt] = false; // this is one of the files we were looking for
+				}
+		}
 		if (missingTypes[TYPE_IMMED]) {
 			// important files missing, can't start the game without them
 			showFileErrorMsg(TYPE_IMMED, fileExists);
@@ -442,9 +564,8 @@ void SwordEngine::checkCdFiles(void) { // check if we're running from cd, hdd or
 	_systemVars.isDemo = (_features & GF_DEMO) != 0;
 	_systemVars.cutscenePackVersion = 0;
 #ifdef USE_MPEG2
-	if (test.open("intro.snd")) {
+	if (Common::File::exists("intro.snd")) {
 		_systemVars.cutscenePackVersion = 1;
-		test.close();
 	}
 #endif
 }
@@ -584,44 +705,44 @@ uint8 SwordEngine::mainLoop(void) {
 
 void SwordEngine::delay(int32 amount) { //copied and mutilated from sky.cpp
 
-	OSystem::Event event;
+	Common::Event event;
 	uint32 start = _system->getMillis();
 
 	do {
-		while (_system->pollEvent(event)) {
+		while (_eventMan->pollEvent(event)) {
 			switch (event.type) {
-			case OSystem::EVENT_KEYDOWN:
+			case Common::EVENT_KEYDOWN:
 				// Make sure backspace works right (this fixes a small issue on OS X)
 				if (event.kbd.keycode == 8)
 					_keyPressed = 8;
 				else
 					_keyPressed = (uint8)event.kbd.ascii;
 				break;
-			case OSystem::EVENT_MOUSEMOVE:
+			case Common::EVENT_MOUSEMOVE:
 				_mouseX = event.mouse.x;
 				_mouseY = event.mouse.y;
 				break;
-			case OSystem::EVENT_LBUTTONDOWN:
+			case Common::EVENT_LBUTTONDOWN:
 				_mouseState |= BS1L_BUTTON_DOWN;
 #if defined(_WIN32_WCE) || defined(PALMOS_MODE)
 				_mouseX = event.mouse.x;
 				_mouseY = event.mouse.y;
 #endif
 				break;
-			case OSystem::EVENT_RBUTTONDOWN:
+			case Common::EVENT_RBUTTONDOWN:
 				_mouseState |= BS1R_BUTTON_DOWN;
 #if defined(_WIN32_WCE) || defined(PALMOS_MODE)
 				_mouseX = event.mouse.x;
 				_mouseY = event.mouse.y;
 #endif
 				break;
-			case OSystem::EVENT_LBUTTONUP:
+			case Common::EVENT_LBUTTONUP:
 				_mouseState |= BS1L_BUTTON_UP;
 				break;
-			case OSystem::EVENT_RBUTTONUP:
+			case Common::EVENT_RBUTTONUP:
 				_mouseState |= BS1R_BUTTON_UP;
 				break;
-			case OSystem::EVENT_QUIT:
+			case Common::EVENT_QUIT:
 				_systemVars.engineQuit = true;
 				break;
 			default:
