@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2003-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,15 +18,15 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/sword1/sword1.h $
- * $Id: sword1.h 21916 2006-04-15 20:36:41Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/sword1/sword1.h $
+ * $Id: sword1.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
-#ifndef BSSWORD1_H
-#define BSSWORD1_H
+#ifndef SWORD1_H
+#define SWORD1_H
 
-#include "base/engine.h"
+#include "engines/engine.h"
 #include "common/util.h"
 #include "sword1/sworddefs.h"
 
@@ -63,12 +66,12 @@ struct SystemVars {
 	uint8	showText;
 	uint8	language;
 	bool    isDemo;
+	bool    isMac;
 
 	uint8	cutscenePackVersion;
 };
 
 class SwordEngine : public Engine {
-	void errorString(const char *buf_input, char *buf_output);
 public:
 	SwordEngine(OSystem *syst);
 	virtual ~SwordEngine();
@@ -101,7 +104,8 @@ private:
 	Music		*_music;
 	Control		*_control;
 	static const uint8  _cdList[TOTAL_SECTIONS];
-	static const CdFile	_cdFileList[];
+	static const CdFile	_pcCdFileList[];
+	static const CdFile	_macCdFileList[];
 };
 
 } // End of namespace Sword1
