@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2003-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,13 +18,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/sword1/sworddefs.h $
- * $Id: sworddefs.h 20512 2006-02-11 10:01:01Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/sword1/sworddefs.h $
+ * $Id: sworddefs.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
-#ifndef SWORDDEFS_H
-#define SWORDDEFS_H
+#ifndef SWORD1_SWORDDEFS_H
+#define SWORD1_SWORDDEFS_H
 
 #include "common/scummsys.h"
 
@@ -89,9 +92,7 @@ namespace Sword1 {
 #define	MAX_text_obs	2			//text compacts
 #define	TEXT_sect		149			//text compacts exist in section 149, probably after all the megas
 
-#if !defined(__GNUC__)
-	#pragma START_PACK_STRUCTS
-#endif
+#include "common/pack-start.h"	// START STRUCT PACKING
 
 struct Header {
 	char type[6];
@@ -99,7 +100,7 @@ struct Header {
 	uint32	comp_length;
 	char	compression[4];
 	uint32	decomp_length;
-} GCC_PACK;
+};
 
 struct FrameHeader {
 	uint8 runTimeComp[4];
@@ -108,35 +109,33 @@ struct FrameHeader {
 	uint16 height;
 	int16 offsetX;
 	int16 offsetY;
-} GCC_PACK;
+};
 
 struct ParallaxHeader {
 	char type[16];
 	uint16 sizeX;
 	uint16 sizeY;
-} GCC_PACK;
+};
 
 struct AnimUnit {
 	uint32 animX;
 	uint32 animY;
 	uint32 animFrame;
-} GCC_PACK;
+};
 
 struct AnimSet {
 	uint32 cdt;
 	uint32 spr;
-} GCC_PACK;
+};
 
 struct WalkGridHeader {
 	int32 scaleA;
 	int32 scaleB;
 	int32 numBars;
 	int32 numNodes;
-} GCC_PACK;
+};
 
-#if !defined(__GNUC__)
-	#pragma END_PACK_STRUCTS
-#endif
+#include "common/pack-end.h"	// END STRUCT PACKING
 
 enum fileTypes {
 	TYPE_CD1 = 0,
