@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2003-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/sword1/text.cpp $
- * $Id: text.cpp 21500 2006-03-29 15:59:37Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/sword1/text.cpp $
+ * $Id: text.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -34,9 +37,6 @@ namespace Sword1 {
 
 #define OVERLAP 3
 #define SPACE ' '
-#define BORDER_COL		200
-#define LETTER_COL		193
-#define NO_COL			0		// sprite background - 0 for transparency
 #define MAX_LINES		30
 
 
@@ -48,15 +48,14 @@ Text::Text(ObjectMan *pObjMan, ResMan *pResMan, bool czechVersion) {
 	_font = (uint8*)_resMan->openFetchRes(_fontId);
 
 	_joinWidth = charWidth( SPACE ) - 2 * OVERLAP;
-	_charHeight = FROM_LE_16(_resMan->fetchFrame(_font, 0)->height); // all chars have the same height
-	_textBlocks[0] = _textBlocks[1] = NULL;
+	_charHeight = _resMan->getUint16(_resMan->fetchFrame(_font, 0)->height); // all chars have the same height
+	for (int i = 0; i < MAX_TEXT_OBS; i++)
+		_textBlocks[i] = NULL;
 }
 
 Text::~Text(void) {
-	if (_textBlocks[0])
-		free(_textBlocks[0]);
-	if (_textBlocks[1])
-		free(_textBlocks[1]);
+	for (int i = 0; i < MAX_TEXT_OBS; i++)
+		free(_textBlocks[i]);
 	//_resMan->resClose(_fontId); => wiped automatically by _resMan->flush();
 }
 
@@ -92,8 +91,8 @@ void Text::makeTextSprite(uint8 slot, uint8 *text, uint16 maxWidth, uint8 pen) {
 
 	memcpy( _textBlocks[slot]->runTimeComp, "Nu  ", 4);
 	_textBlocks[slot]->compSize	= 0;
-	_textBlocks[slot]->width	= TO_LE_16(sprWidth);
-	_textBlocks[slot]->height	= TO_LE_16(sprHeight);
+	_textBlocks[slot]->width	= _resMan->toUint16(sprWidth);
+	_textBlocks[slot]->height	= _resMan->toUint16(sprHeight);
 	_textBlocks[slot]->offsetX	= 0;
 	_textBlocks[slot]->offsetY	= 0;
 
@@ -111,7 +110,7 @@ void Text::makeTextSprite(uint8 slot, uint8 *text, uint16 maxWidth, uint8 pen) {
 uint16 Text::charWidth(uint8 ch) {
 	if (ch < SPACE)
 		ch = 64;
-	return FROM_LE_16(_resMan->fetchFrame(_font, ch - SPACE)->width);
+	return _resMan->getUint16(_resMan->fetchFrame(_font, ch - SPACE)->width);
 }
 
 uint16 Text::analyzeSentence(uint8 *text, uint16 maxWidth, LineInfo *line) {
@@ -158,8 +157,8 @@ uint16 Text::copyChar(uint8 ch, uint8 *sprPtr, uint16 sprWidth, uint8 pen) {
 	FrameHeader *chFrame = _resMan->fetchFrame(_font, ch - SPACE);
 	uint8 *chData = ((uint8*)chFrame) + sizeof(FrameHeader);
 	uint8 *dest = sprPtr;
-	for (uint16 cnty = 0; cnty < FROM_LE_16(chFrame->height); cnty++) {
-		for (uint16 cntx = 0; cntx < FROM_LE_16(chFrame->width); cntx++) {
+	for (uint16 cnty = 0; cnty < _resMan->getUint16(chFrame->height); cnty++) {
+		for (uint16 cntx = 0; cntx < _resMan->getUint16(chFrame->width); cntx++) {
 			if (*chData == LETTER_COL)
 				dest[cntx] = pen;
 			else if ((*chData == BORDER_COL) && (!dest[cntx])) // don't do a border if there's already a color underneath (chars can overlap)
@@ -168,21 +167,21 @@ uint16 Text::copyChar(uint8 ch, uint8 *sprPtr, uint16 sprWidth, uint8 pen) {
 		}
 		dest += sprWidth;
 	}
-	return FROM_LE_16(chFrame->width);
+	return _resMan->getUint16(chFrame->width);
 }
 
 FrameHeader *Text::giveSpriteData(uint32 textTarget) {
 	// textTarget is the resource ID of the Compact linking the textdata.
 	// that's 0x950000 for slot 0 and 0x950001 for slot 1. easy, huh? :)
 	textTarget &= ITM_ID;
-	assert(textTarget <= 1);
+	assert(textTarget < MAX_TEXT_OBS);
 
 	return _textBlocks[textTarget];
 }
 
 void Text::releaseText(uint32 id) {
 	id &= ITM_ID;
-	assert(id <= 1);
+	assert(id < MAX_TEXT_OBS);
 	if (_textBlocks[id]) {
 		free(_textBlocks[id]);
 		_textBlocks[id] = NULL;
