@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2003-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,20 +18,24 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/sword1/text.h $
- * $Id: text.h 20512 2006-02-11 10:01:01Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/sword1/text.h $
+ * $Id: text.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
-#ifndef BSTEXT_H
-#define BSTEXT_H
+#ifndef SWORD1_TEXT_H
+#define SWORD1_TEXT_H
 
 #include "sword1/object.h"
 #include "sword1/sworddefs.h"
 
 namespace Sword1 {
 
-#define MAX_TEXT_OBS 2
+#define MAX_TEXT_OBS 3
+
+#define BORDER_COL	200
+#define LETTER_COL	193
+#define NO_COL		0	// sprite background - 0 for transparency
 
 class ObjectMan;
 class ResMan;
@@ -44,10 +51,10 @@ public:
 	~Text(void);
 	FrameHeader *giveSpriteData(uint32 textTarget);
 	uint32 lowTextManager(uint8 *text, int32 width, uint8 pen);
+	void makeTextSprite(uint8 slot, uint8 *text, uint16 maxWidth, uint8 pen);
 	void releaseText(uint32 id);
 
 private:
-	void makeTextSprite(uint8 slot, uint8 *text, uint16 maxWidth, uint8 pen);
 	uint16 analyzeSentence(uint8 *text, uint16 maxWidth, LineInfo *info);
 	uint16 charWidth(uint8 ch);
 	uint16 copyChar(uint8 ch, uint8 *sprPtr, uint16 sprWidth, uint8 pen);
