@@ -1,5 +1,11 @@
-/* Copyright (C) 1994-1998 Revolution Software Ltd.
- * Copyright (C) 2003-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
+ *
+ * Additional copyright for this file:
+ * Copyright (C) 1994-1998 Revolution Software Ltd.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +21,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/sword2/debug.cpp $
- * $Id: debug.cpp 20741 2006-02-17 15:07:36Z eriktorbjorn $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/sword2/debug.cpp $
+ * $Id: debug.cpp 27030 2007-05-31 20:28:29Z fingolfin $
  */
 
 #include "common/stdafx.h"
@@ -172,8 +178,6 @@ void Debugger::buildDebugText() {
 	// general debug info
 
 	if (_displayDebugText) {
-		byte name[NAME_LEN];
-
 /*
 		// CD in use
 		sprintf(buf, "CD-%d", currentCD);
@@ -187,7 +191,7 @@ void Debugger::buildDebugText() {
 				_vm->_logic->readVar(MOUSE_X),
 				_vm->_logic->readVar(MOUSE_Y),
 				_vm->_logic->readVar(CLICKED_ID),
-				_vm->_resman->fetchName(_vm->_logic->readVar(CLICKED_ID), name));
+				_vm->_resman->fetchName(_vm->_logic->readVar(CLICKED_ID)));
 		else
 			sprintf(buf, "last click at %d,%d (---)",
 				_vm->_logic->readVar(MOUSE_X),
@@ -206,7 +210,7 @@ void Debugger::buildDebugText() {
 				mouseX + screenInfo->scroll_offset_x,
 				mouseY + screenInfo->scroll_offset_y,
 				mouseTouching,
-				_vm->_resman->fetchName(mouseTouching, name));
+				_vm->_resman->fetchName(mouseTouching));
 		else
 			sprintf(buf, "mouse %d,%d (not touching)",
 				mouseX + screenInfo->scroll_offset_x,
@@ -221,7 +225,7 @@ void Debugger::buildDebugText() {
 			sprintf(buf, "player %d,%d %s (%d) #%d/%d",
 				screenInfo->player_feet_x,
 				screenInfo->player_feet_y,
-				_vm->_resman->fetchName(_graphAnimRes, name),
+				_vm->_resman->fetchName(_graphAnimRes),
 				_graphAnimRes,
 				_graphAnimPc,
 				_graphNoFrames);
@@ -287,7 +291,7 @@ void Debugger::buildDebugText() {
 
 		if (_speechScriptWaiting) {
 			sprintf(buf, "script waiting for %s (%d)",
-				_vm->_resman->fetchName(_speechScriptWaiting, name),
+				_vm->_resman->fetchName(_speechScriptWaiting),
 				_speechScriptWaiting);
 			makeDebugTextBlock(buf, 0, 90);
 		}
