@@ -1,5 +1,11 @@
-/* Copyright (C) 1994-1998 Revolution Software Ltd.
- * Copyright (C) 2003-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
+ *
+ * Additional copyright for this file:
+ * Copyright (C) 1994-1998 Revolution Software Ltd.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +21,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/sword2/function.cpp $
- * $Id: function.cpp 20741 2006-02-17 15:07:36Z eriktorbjorn $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/sword2/function.cpp $
+ * $Id: function.cpp 27030 2007-05-31 20:28:29Z fingolfin $
  */
 
 #include "common/stdafx.h"
@@ -876,9 +882,7 @@ int32 Logic::fnISpeak(int32 *params) {
 		if (readVar(PLAYER_ID) != CUR_PLAYER_ID)
 			debug(5, "(%d) Nico: %s", _officialTextNumber, text + 2);
 		else {
-			byte buf[NAME_LEN];
-
-			debug(5, "(%d) %s: %s", _officialTextNumber, _vm->_resman->fetchName(readVar(ID), buf), text + 2);
+			debug(5, "(%d) %s: %s", _officialTextNumber, _vm->_resman->fetchName(readVar(ID)), text + 2);
 		}
 
 		// Set up the speech animation
@@ -1883,11 +1887,11 @@ int32 Logic::fnColour(int32 *params) {
 #define GREEN	3
 #define BLUE	4
 
-static uint8 black[4]	= {  0,    0,   0,   0 };
-static uint8 white[4]	= { 255, 255, 255,   0 };
-static uint8 red[4]	= { 255,   0,   0,   0 };
-static uint8 green[4]	= {   0, 255,   0,   0 };
-static uint8 blue[4]	= {   0,   0, 255,   0 };
+static const uint8 black[4]	= {  0,    0,   0,   0 };
+static const uint8 white[4]	= { 255, 255, 255,   0 };
+static const uint8 red[4]	= { 255,   0,   0,   0 };
+static const uint8 green[4]	= {   0, 255,   0,   0 };
+static const uint8 blue[4]	= {   0,   0, 255,   0 };
 #endif
 
 int32 Logic::fnFlash(int32 *params) {
@@ -2104,7 +2108,6 @@ int32 Logic::fnPlaySequence(int32 *params) {
 	// 		1 number of frames in the sequence, used for PSX.
 
 	char filename[30];
-	MovieTextObject *sequenceSpeechArray[MAX_SEQUENCE_TEXT_LINES + 1];
 
 	// The original code had some #ifdef blocks for skipping or muting the
 	// cutscenes - fondly described as "the biggest fudge in the history
@@ -2120,28 +2123,21 @@ int32 Logic::fnPlaySequence(int32 *params) {
 	// Write to walkthrough file (zebug0.txt)
  	debug(5, "PLAYING SEQUENCE \"%s\"", filename);
 
-	// now create the text sprites, if any
-
-	if (_sequenceTextLines)
-		createSequenceSpeech(sequenceSpeechArray);
-
-	// don't want to carry on streaming game music when smacker starts!
+	// don't want to carry on streaming game music when cutscene starts!
 	fnStopMusic(NULL);
 
 	// pause sfx during sequence
 	_vm->_sound->pauseFx();
 
-	MoviePlayer player(_vm);
- 	uint32 rv;
+	MoviePlayer *player = makeMoviePlayer(_vm, filename);
 
-	if (_sequenceTextLines && !readVar(DEMO))
-		rv = player.play(filename, sequenceSpeechArray, _smackerLeadIn, _smackerLeadOut);
-	else
-		rv = player.play(filename, NULL, _smackerLeadIn, _smackerLeadOut);
+	if (player->load()) {
+		player->play(_sequenceTextList, _sequenceTextLines, _smackerLeadIn, _smackerLeadOut);
+	}
 
-	// check the error return-value
-	if (rv)
-		debug(5, "MoviePlayer.play(\"%s\") returned 0x%.8x", filename, rv);
+	_sequenceTextLines = 0;
+
+	delete player;
 
 	// unpause sound fx again, in case we're staying in same location
 	_vm->_sound->unpauseFx();
@@ -2149,11 +2145,6 @@ int32 Logic::fnPlaySequence(int32 *params) {
 	_smackerLeadIn = 0;
 	_smackerLeadOut = 0;
 
-	// now clear the text sprites, if any
-
-	if (_sequenceTextLines)
-		clearSequenceSpeech(sequenceSpeechArray);
-
 	// now clear the screen in case the Sequence was quitted (using ESC)
 	// rather than fading down to black
 
@@ -2225,12 +2216,15 @@ int32 Logic::fnAddSequenceText(int32 *params) {
 	//		1 frame number to start the text displaying
 	//		2 frame number to stop the text dispalying
 
-	assert(_sequenceTextLines < MAX_SEQUENCE_TEXT_LINES);
+	if (!readVar(DEMO)) {
+		assert(_sequenceTextLines < MAX_SEQUENCE_TEXT_LINES);
+
+		_sequenceTextList[_sequenceTextLines].textNumber = params[0];
+		_sequenceTextList[_sequenceTextLines].startFrame = params[1];
+		_sequenceTextList[_sequenceTextLines].endFrame = params[2];
+		_sequenceTextLines++;
+	}
 
-	_sequenceTextList[_sequenceTextLines].textNumber = params[0];
-	_sequenceTextList[_sequenceTextLines].startFrame = params[1];
-	_sequenceTextList[_sequenceTextLines].endFrame = params[2];
-	_sequenceTextLines++;
 	return IR_CONT;
 }
 
@@ -2339,7 +2333,7 @@ int32 Logic::fnSmackerLeadOut(int32 *params) {
 int32 Logic::fnStopAllFx(int32 *params) {
 	// params:	none
 
-	_vm->_sound->clearFxQueue();
+	_vm->_sound->clearFxQueue(false);
 	return IR_CONT;
 }
 
