@@ -1,5 +1,11 @@
-/* Copyright (C) 1994-1998 Revolution Software Ltd.
- * Copyright (C) 2003-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
+ *
+ * Additional copyright for this file:
+ * Copyright (C) 1994-1998 Revolution Software Ltd.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +21,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/sword2/layers.cpp $
- * $Id: layers.cpp 20741 2006-02-17 15:07:36Z eriktorbjorn $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/sword2/layers.cpp $
+ * $Id: layers.cpp 27030 2007-05-31 20:28:29Z fingolfin $
  */
 
 // high level layer initialising
@@ -48,15 +54,14 @@ namespace Sword2 {
  */
 
 void Screen::initBackground(int32 res, int32 new_palette) {
-	byte buf[NAME_LEN];
 	int i;
 
 	assert(res);
 
-	_vm->_sound->clearFxQueue();
+	_vm->_sound->clearFxQueue(false);
 	waitForFade();
 
-	debug(1, "CHANGED TO LOCATION \"%s\"", _vm->_resman->fetchName(res, buf));
+	debug(1, "CHANGED TO LOCATION \"%s\"", _vm->_resman->fetchName(res));
 
 	// if last screen was using a shading mask (see below)
 	if (_thisScreen.mask_flag) {
