@@ -1,5 +1,11 @@
-/* Copyright (C) 1994-1998 Revolution Software Ltd.
- * Copyright (C) 2003-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
+ *
+ * Additional copyright for this file:
+ * Copyright (C) 1994-1998 Revolution Software Ltd.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +21,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/sword2/logic.cpp $
- * $Id: logic.cpp 23847 2006-09-09 14:36:20Z chrilith $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/sword2/logic.cpp $
+ * $Id: logic.cpp 27030 2007-05-31 20:28:29Z fingolfin $
  */
 
 #include "common/stdafx.h"
@@ -41,7 +47,7 @@ Logic::Logic(Sword2Engine *vm) :
 	memset(_eventList, 0, sizeof(_eventList));
 	memset(_syncList, 0, sizeof(_syncList));
 	_router = new Router(_vm);
-
+	
 	setupOpcodes();
 }
 
@@ -216,7 +222,7 @@ void Logic::expressChangeSession(uint32 sesh_id) {
 
 	// Various clean-ups
 	_router->clearWalkGridList();
-	_vm->_sound->clearFxQueue();
+	_vm->_sound->clearFxQueue(false);
 	_router->freeAllRouteMem();
 }
 
