@@ -1,5 +1,11 @@
-/* Copyright (C) 1994-1998 Revolution Software Ltd.
- * Copyright (C) 2003-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
+ *
+ * Additional copyright for this file:
+ * Copyright (C) 1994-1998 Revolution Software Ltd.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +21,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/sword2/logic.h $
- * $Id: logic.h 23847 2006-09-09 14:36:20Z chrilith $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/sword2/logic.h $
+ * $Id: logic.h 27030 2007-05-31 20:28:29Z fingolfin $
  */
 
 // logic management
@@ -24,13 +30,12 @@
 #ifndef SWORD2_LOGIC_H
 #define SWORD2_LOGIC_H
 
-#include "sword2/memory.h"
 #include "common/endian.h"
+#include "sword2/animation.h"
+#include "sword2/memory.h"
 
 namespace Sword2 {
 
-struct MovieTextObject;
-
 #define MAX_events 10
 
 #define TREE_SIZE 3
@@ -79,22 +84,8 @@ private:
 	// keeps count of number of text lines to disaply during the sequence
 	uint32 _sequenceTextLines;
 
-	// FOR TEXT LINES IN SEQUENCE PLAYER
-
-	struct SequenceTextInfo {
-		uint32 textNumber;
-		uint16 startFrame;
-		uint16 endFrame;
-		byte *text_mem;
-		uint32 speechBufferSize;
-		uint16 *speech_mem;
-	};
-
 	SequenceTextInfo _sequenceTextList[MAX_SEQUENCE_TEXT_LINES];
 
-	void createSequenceSpeech(MovieTextObject *sequenceText[]);
-	void clearSequenceSpeech(MovieTextObject *sequenceText[]);
-
 	// when not playing a wav we calculate the speech time based upon
 	// length of ascii
 
