@@ -1,5 +1,11 @@
-/* Copyright (C) 1994-1998 Revolution Software Ltd.
- * Copyright (C) 2003-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
+ *
+ * Additional copyright for this file:
+ * Copyright (C) 1994-1998 Revolution Software Ltd.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,12 +21,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/sword2/mouse.cpp $
- * $Id: mouse.cpp 22639 2006-05-25 22:51:42Z eriktorbjorn $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/sword2/mouse.cpp $
+ * $Id: mouse.cpp 27030 2007-05-31 20:28:29Z fingolfin $
  */
 
 #include "common/stdafx.h"
 #include "common/system.h"
+#include "common/events.h"
 
 #include "graphics/cursorman.h"
 
@@ -64,7 +71,6 @@ enum {
 Mouse::Mouse(Sword2Engine *vm) {
 	_vm = vm;
 
-	setPos(0, 0);
 	resetMouseList();
 
 	_mouseTouching = 0;
@@ -118,13 +124,25 @@ Mouse::~Mouse() {
 }
 
 void Mouse::getPos(int &x, int &y) {
-	x = _pos.x;
-	y = _pos.y;
+	Common::EventManager *eventMan = _vm->_system->getEventManager();
+	Common::Point pos = eventMan->getMousePos();
+
+	x = pos.x;
+	y = pos.y - MENUDEEP;
 }
 
-void Mouse::setPos(int x, int y) {
-	_pos.x = x;
-	_pos.y = y;
+int Mouse::getX() {
+	int x, y;
+
+	getPos(x, y);
+	return x;
+}
+
+int Mouse::getY() {
+	int x, y;
+
+	getPos(x, y);
+	return y;
 }
 
 /**
@@ -238,7 +256,7 @@ void Mouse::mouseEngine() {
 		systemMenuMouse();
 		break;
 	case MOUSE_holding:
-		if (_pos.y < 400) {
+		if (getY() < 400) {
 			_mouseMode = MOUSE_normal;
 			debug(5, "   releasing");
 		}
@@ -261,13 +279,15 @@ bool Mouse::heldIsInInventory() {
 #endif
 
 int Mouse::menuClick(int menu_items) {
-	if (_pos.x < RDMENU_ICONSTART)
+	int x = getX();
+
+	if (x < RDMENU_ICONSTART)
 		return -1;
 
-	if (_pos.x > RDMENU_ICONSTART + menu_items * (RDMENU_ICONWIDE + RDMENU_ICONSPACING) - RDMENU_ICONSPACING)
+	if (x > RDMENU_ICONSTART + menu_items * (RDMENU_ICONWIDE + RDMENU_ICONSPACING) - RDMENU_ICONSPACING)
 		return -1;
 
-	return (_pos.x - RDMENU_ICONSTART) / (RDMENU_ICONWIDE + RDMENU_ICONSPACING);
+	return (x - RDMENU_ICONSTART) / (RDMENU_ICONWIDE + RDMENU_ICONSPACING);
 }
 
 void Mouse::systemMenuMouse() {
@@ -287,7 +307,9 @@ void Mouse::systemMenuMouse() {
 	// If the mouse is moved off the menu, close it. Unless the player is
 	// dead, in which case the menu should always be visible.
 
-	if (_pos.y > 0 && !_vm->_logic->readVar(DEAD)) {
+	int y = getY();
+
+	if (y > 0 && !_vm->_logic->readVar(DEAD)) {
 		_mouseMode = MOUSE_normal;
 		hideMenu(RDMENU_TOP);
 		return;
@@ -300,7 +322,7 @@ void Mouse::systemMenuMouse() {
 	if (!me || !(me->buttons & RD_LEFTBUTTONDOWN))
 		return;
 
-	if (_pos.y > 0)
+	if (y > 0)
 		return;
 
 	hit = menuClick(ARRAYSIZE(icon_list));
@@ -426,7 +448,11 @@ void Mouse::dragMouse() {
 	// objects in the scene, so if the mouse moves off the inventory menu,
 	// then close it.
 
-	if (_pos.y < 400) {
+	int x, y;
+
+	getPos(x, y);
+
+	if (y < 400) {
 		_mouseMode = MOUSE_normal;
 		hideMenu(RDMENU_BOTTOM);
 		return;
@@ -477,8 +503,8 @@ void Mouse::dragMouse() {
 		// These might be required by the action script about to be run
 		ScreenInfo *screenInfo = _vm->_screen->getScreenInfo();
 
-		_vm->_logic->writeVar(MOUSE_X, _pos.x + screenInfo->scroll_offset_x);
-		_vm->_logic->writeVar(MOUSE_Y, _pos.y + screenInfo->scroll_offset_y);
+		_vm->_logic->writeVar(MOUSE_X, x + screenInfo->scroll_offset_x);
+		_vm->_logic->writeVar(MOUSE_Y, y + screenInfo->scroll_offset_y);
 
 		// For scripts to know what's been clicked. First used for
 		// 'room_13_turning_script' in object 'biscuits_13'
@@ -539,13 +565,12 @@ void Mouse::dragMouse() {
 }
 
 void Mouse::menuMouse() {
-	byte buf[NAME_LEN];
 	MouseEvent *me;
 	int hit;
 
 	// If the mouse is moved off the menu, close it.
 
-	if (_pos.y < 400) {
+	if (getY() < 400) {
 		_mouseMode = MOUSE_normal;
 		hideMenu(RDMENU_BOTTOM);
 		return;
@@ -587,7 +612,7 @@ void Mouse::menuMouse() {
 		hideMouse();
 
 		debug(2, "Right-click on \"%s\" icon",
-			_vm->_resman->fetchName(_vm->_logic->readVar(OBJECT_HELD), buf));
+			_vm->_resman->fetchName(_vm->_logic->readVar(OBJECT_HELD)));
 
 		return;
 	}
@@ -615,7 +640,7 @@ void Mouse::menuMouse() {
 		setLuggage(_masterMenuList[hit].luggage_resource);
 
 		debug(2, "Left-clicked on \"%s\" icon - switch to drag mode",
-			_vm->_resman->fetchName(_vm->_logic->readVar(OBJECT_HELD), buf));
+			_vm->_resman->fetchName(_vm->_logic->readVar(OBJECT_HELD)));
 	}
 }
 
@@ -629,7 +654,11 @@ void Mouse::normalMouse() {
 	// big-object menu lock situation, of if the player is dragging an
 	// object.
 
-	if (_pos.y < 0 && !_mouseModeLocked && !_vm->_logic->readVar(OBJECT_HELD)) {
+	int x, y;
+
+	getPos(x, y);
+
+	if (y < 0 && !_mouseModeLocked && !_vm->_logic->readVar(OBJECT_HELD)) {
 		_mouseMode = MOUSE_system_menu;
 
 		if (_mouseTouching) {
@@ -648,7 +677,7 @@ void Mouse::normalMouse() {
 	// Check if the cursor has moved onto the inventory menu area. No
 	// inventory in big-object menu lock situation,
 
-	if (_pos.y > 399 && !_mouseModeLocked) {
+	if (y > 399 && !_mouseModeLocked) {
 		// If an object is being held, i.e. if the mouse cursor has a
 		// luggage, go to drag mode instead of menu mode, but the menu
 		// is still opened.
@@ -699,8 +728,8 @@ void Mouse::normalMouse() {
 
 			if (button_down) {
 				// set both (x1,y1) and (x2,y2) to this point
-				_vm->_debugger->_rectX1 = _vm->_debugger->_rectX2 = (uint32)_pos.x + screenInfo->scroll_offset_x;
-				_vm->_debugger->_rectY1 = _vm->_debugger->_rectY2 = (uint32)_pos.y + screenInfo->scroll_offset_y;
+				_vm->_debugger->_rectX1 = _vm->_debugger->_rectX2 = (uint32)x + screenInfo->scroll_offset_x;
+				_vm->_debugger->_rectY1 = _vm->_debugger->_rectY2 = (uint32)y + screenInfo->scroll_offset_y;
 				_vm->_debugger->_draggingRectangle = 1;
 			}
 		} else if (_vm->_debugger->_draggingRectangle == 1) {
@@ -712,8 +741,8 @@ void Mouse::normalMouse() {
 				_vm->_debugger->_draggingRectangle = 2;
 			} else {
 				// drag rectangle
-				_vm->_debugger->_rectX2 = (uint32)_pos.x + screenInfo->scroll_offset_x;
-				_vm->_debugger->_rectY2 = (uint32)_pos.y + screenInfo->scroll_offset_y;
+				_vm->_debugger->_rectX2 = (uint32)x + screenInfo->scroll_offset_x;
+				_vm->_debugger->_rectY2 = (uint32)y + screenInfo->scroll_offset_y;
 			}
 		} else {
 			// currently locked to avoid knocking out of place
@@ -775,8 +804,8 @@ void Mouse::normalMouse() {
 	// These might be required by the action script about to be run
 	ScreenInfo *screenInfo = _vm->_screen->getScreenInfo();
 
-	_vm->_logic->writeVar(MOUSE_X, _pos.x + screenInfo->scroll_offset_x);
-	_vm->_logic->writeVar(MOUSE_Y, _pos.y + screenInfo->scroll_offset_y);
+	_vm->_logic->writeVar(MOUSE_X, x + screenInfo->scroll_offset_x);
+	_vm->_logic->writeVar(MOUSE_Y, y + screenInfo->scroll_offset_y);
 
 	if (_mouseTouching == _vm->_logic->readVar(EXIT_CLICK_ID) && (me->buttons & RD_LEFTBUTTONDOWN)) {
 		// It's the exit double click situation. Let the existing
@@ -817,10 +846,10 @@ void Mouse::normalMouse() {
 				_vm->_resman->fetchName(_vm->_logic->readVar(CLICKED_ID), buf2));
 		else if (_vm->_logic->readVar(LEFT_BUTTON))
 			debug(2, "Left-clicked on \"%s\"",
-				_vm->_resman->fetchName(_vm->_logic->readVar(CLICKED_ID), buf1));
+				_vm->_resman->fetchName(_vm->_logic->readVar(CLICKED_ID)));
 		else	// RIGHT BUTTON
 			debug(2, "Right-clicked on \"%s\"",
-				_vm->_resman->fetchName(_vm->_logic->readVar(CLICKED_ID), buf1));
+				_vm->_resman->fetchName(_vm->_logic->readVar(CLICKED_ID)));
 	}
 }
 
@@ -950,7 +979,9 @@ void Mouse::mouseOnOff() {
 	// don't detect objects that are hidden behind the menu bars (ie. in
 	// the scrolled-off areas of the screen)
 
-	if (_pos.y < 0 || _pos.y > 399) {
+	int y = getY();
+
+	if (y < 0 || y > 399) {
 		pointer_type = 0;
 		_mouseTouching = 0;
 	} else {
@@ -993,9 +1024,7 @@ void Mouse::mouseOnOff() {
 				setLuggage(_currentLuggageResource);
 			}
 		} else {
-			byte buf[NAME_LEN];
-
-			error("ERROR: mouse.pointer==0 for object %d (%s) - update logic script!", _mouseTouching, _vm->_resman->fetchName(_mouseTouching, buf));
+			error("ERROR: mouse.pointer==0 for object %d (%s) - update logic script!", _mouseTouching, _vm->_resman->fetchName(_mouseTouching));
 		}
 	} else if (_oldMouseTouching && !_mouseTouching) {
 		// the cursor has moved off something - reset cursor to
@@ -1083,8 +1112,11 @@ void Mouse::setObjectHeld(uint32 res) {
 
 uint32 Mouse::checkMouseList() {
 	ScreenInfo *screenInfo = _vm->_screen->getScreenInfo();
+	int x, y;
 
-	Common::Point mousePos(_pos.x + screenInfo->scroll_offset_x, _pos.y + screenInfo->scroll_offset_y);
+	getPos(x, y);
+
+	Common::Point mousePos(x + screenInfo->scroll_offset_x, y + screenInfo->scroll_offset_y);
 
 	// Number of priorities subject to implementation needs
 	for (int priority = 0; priority < 10; priority++) {
@@ -1255,9 +1287,13 @@ void Mouse::createPointerText(uint32 text_id, uint32 pointer_res) {
 	// 'text+2' to skip the first 2 bytes which form the
 	// line reference number
 
+	int x, y;
+
+	getPos(x, y);
+
 	_pointerTextBlocNo = _vm->_fontRenderer->buildNewBloc(
-		text + 2, _pos.x + xOffset,
-		_pos.y + yOffset,
+		text + 2, x + xOffset,
+		y + yOffset,
 		POINTER_TEXT_WIDTH, POINTER_TEXT_PEN,
 		RDSPR_TRANS | RDSPR_DISPLAYALIGN,
 		_vm->_speechFontId, justification);
@@ -1339,7 +1375,7 @@ void Mouse::addHuman() {
 	}
 
 	// If mouse is over menu area
-	if (_pos.y > 399) {
+	if (getY() > 399) {
 		if (_mouseMode != MOUSE_holding) {
 			// VITAL - reset things & rebuild the menu
 			_mouseMode = MOUSE_normal;
@@ -1359,7 +1395,7 @@ void Mouse::addHuman() {
 		_vm->_screen->setPalette(0, 1, white, RDPAL_INSTANT);
 
 		// Stops all fx & clears the queue - eg. when leaving a room
-		_vm->_sound->clearFxQueue();
+		_vm->_sound->clearFxQueue(false);
 
 		// Trash all object resources so they load in fresh & restart
 		// their logic scripts
@@ -1392,7 +1428,7 @@ void Mouse::startConversation() {
 void Mouse::endConversation() {
 	hideMenu(RDMENU_BOTTOM);
 
-	if (_pos.y > 399) {
+	if (getY() > 399) {
 		// Will wait for cursor to move off the bottom menu
 		_mouseMode = MOUSE_holding;
 	}
