@@ -1,5 +1,11 @@
-/* Copyright (C) 1994-1998 Revolution Software Ltd.
- * Copyright (C) 2003-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
+ *
+ * Additional copyright for this file:
+ * Copyright (C) 1994-1998 Revolution Software Ltd.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,15 +21,15 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/sword2/mouse.h $
- * $Id: mouse.h 20829 2006-02-23 17:51:27Z aquadran $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/sword2/mouse.h $
+ * $Id: mouse.h 27030 2007-05-31 20:28:29Z fingolfin $
  */
 
-#include "common/rect.h"
-
 #ifndef SWORD2_MOUSE_H
 #define SWORD2_MOUSE_H
 
+#include "common/rect.h"
+
 #define MENU_MASTER_OBJECT	44
 
 #define	MAX_SUBJECT_LIST 30	// is that enough?
@@ -123,8 +129,6 @@ class Mouse {
 private:
 	Sword2Engine *_vm;
 
-	Common::Point _pos;
-
 	MouseUnit _mouseList[TOTAL_mouse_list];
 	uint32 _curMouse;
 
@@ -197,7 +201,8 @@ public:
 	~Mouse();
 
 	void getPos(int &x, int &y);
-	void setPos(int x, int y);
+	int getX();
+	int getY();
 
 	bool getObjectLabels() { return _objectLabels; }
 	void setObjectLabels(bool b) { _objectLabels = b; }
