@@ -1,5 +1,11 @@
-/* Copyright (C) 1994-1998 Revolution Software Ltd.
- * Copyright (C) 2003-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
+ *
+ * Additional copyright for this file:
+ * Copyright (C) 1994-1998 Revolution Software Ltd.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +21,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/sword2/music.cpp $
- * $Id: music.cpp 22231 2006-04-29 22:33:31Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/sword2/music.cpp $
+ * $Id: music.cpp 27030 2007-05-31 20:28:29Z fingolfin $
  */
 
 // One feature still missing is the original's DipMusic() function which, as
@@ -53,14 +59,14 @@ static Audio::AudioStream *getAudioStream(SoundFileHandle *fh, const char *base,
 			const char *ext;
 			int mode;
 		} file_types[] = {
-	#ifdef USE_MAD
-			{ "cl3", kMP3Mode },
+	#ifdef USE_FLAC
+			{ "clf", kFlacMode },
 	#endif
 	#ifdef USE_VORBIS
 			{ "clg", kVorbisMode },
 	#endif
-	#ifdef USE_FLAC
-			{ "clf", kFlacMode },
+	#ifdef USE_MAD
+			{ "cl3", kMP3Mode },
 	#endif
 			{ "clu", kCLUMode }
 		};
@@ -90,7 +96,7 @@ static Audio::AudioStream *getAudioStream(SoundFileHandle *fh, const char *base,
 		fh->file.open(filename);
 		fh->fileType = soundMode;
 		if (!fh->file.isOpen()) {
-			warning("Very strange fopen error");
+			warning("BS2 getAudioStream: Failed opening file '%s'", filename);
 			return NULL;
 		}
 		if (fh->fileSize != fh->file.size()) {
@@ -165,8 +171,6 @@ static Audio::AudioStream *getAudioStream(SoundFileHandle *fh, const char *base,
 CLUInputStream::CLUInputStream(Common::File *file, int size)
 	: _file(file), _firstTime(true), _bufferEnd(_outbuf + BUFFER_SIZE) {
 
-	_file->incRef();
-
 	// Determine the end position.
 	_file_pos = _file->pos();
 	_end_pos = _file_pos + size;
@@ -176,7 +180,6 @@ CLUInputStream::CLUInputStream(Common::File *file, int size)
 }
 
 CLUInputStream::~CLUInputStream() {
-	_file->decRef();
 }
 
 int CLUInputStream::readBuffer(int16 *buffer, const int numSamples) {
@@ -267,6 +270,7 @@ MusicInputStream::MusicInputStream(int cd, SoundFileHandle *fh, uint32 musicId,
 
 MusicInputStream::~MusicInputStream() {
 	delete _decoder;
+	_decoder = NULL;
 }
 
 int MusicInputStream::readBuffer(int16 *buffer, const int numSamples) {
@@ -474,12 +478,8 @@ int Sound::readBuffer(int16 *buffer, const int numSamples) {
 }
 
 bool Sound::endOfData() const {
-	for (int i = 0; i < MAXMUS; i++) {
-		if (_musicFile[i].file.isOpen())
-			return false;
-	}
-
-	return true;
+	// The music never stops. It just goes quiet.
+	return false;
 }
 
 // ----------------------------------------------------------------------------
@@ -704,46 +704,6 @@ int32 Sound::amISpeaking() {
 	return RDSE_QUIET;
 }
 
-/**
- * This function loads and decompresses a list of speech from a cluster, but
- * does not play it. This is used for cutscene voice-overs, presumably to
- * avoid having to read from more than one file on the CD during playback.
- * @param speechId the text line id used to reference the speech
- * @param buf a pointer to the buffer that will be allocated for the sound
- */
-
-uint32 Sound::preFetchCompSpeech(uint32 speechId, uint16 **buf) {
-	int cd = _vm->_resman->getCD();
-	uint32 numSamples;
-
-	SoundFileHandle *fh = (cd == 1) ? &_speechFile[0] : &_speechFile[1];
-
-	Audio::AudioStream *input = getAudioStream(fh, "speech", cd, speechId, &numSamples);
-
-	if (!input)
-		return 0;
-
-	*buf = NULL;
-
-	// Decompress data into speech buffer.
-
-	uint32 bufferSize = 2 * numSamples;
-
-	*buf = (uint16 *)malloc(bufferSize);
-	if (!*buf) {
-		delete input;
-		fh->file.close();
-		return 0;
-	}
-
-	uint32 readSamples = input->readBuffer((int16 *)*buf, numSamples);
-
-	fh->file.close();
-	delete input;
-
-	return 2 * readSamples;
-}
-
 /**
  * This function loads, decompresses and plays a line of speech. An error
  * occurs if speech is already playing.
