@@ -1,5 +1,11 @@
-/* Copyright (C) 1994-1998 Revolution Software Ltd.
- * Copyright (C) 2003-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
+ *
+ * Additional copyright for this file:
+ * Copyright (C) 1994-1998 Revolution Software Ltd.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +21,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/sword2/render.cpp $
- * $Id: render.cpp 21500 2006-03-29 15:59:37Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/sword2/render.cpp $
+ * $Id: render.cpp 27030 2007-05-31 20:28:29Z fingolfin $
  */
 
 #include "common/stdafx.h"
@@ -29,13 +35,8 @@
 #include "sword2/defs.h"
 #include "sword2/screen.h"
 
-#ifdef BACKEND_8BIT
-#include "sword2/animation.h"
-#endif
-
 namespace Sword2 {
 
-#define MILLISECSPERCYCLE  83
 #define RENDERAVERAGETOTAL 4
 
 void Screen::updateRect(Common::Rect *r) {
@@ -301,7 +302,7 @@ void Screen::renderParallax(byte *ptr, int16 l) {
 
 void Screen::initialiseRenderCycle() {
 	_initialTime = _vm->_system->getMillis();
-	_totalTime = _initialTime + MILLISECSPERCYCLE;
+	_totalTime = _initialTime + (1000 / _vm->getFramesPerSecond());
 }
 
 /**
@@ -358,7 +359,7 @@ bool Screen::endRenderCycle() {
 	}
 
 	if (_startTime + _renderAverageTime >= _totalTime) {
-		_totalTime += MILLISECSPERCYCLE;
+		_totalTime += (1000 / _vm->getFramesPerSecond());
 		_initialTime = time;
 		return true;
 	}
@@ -369,7 +370,7 @@ bool Screen::endRenderCycle() {
 		// rest of the render cycle.
 		_vm->sleepUntil(_totalTime);
 		_initialTime = _vm->_system->getMillis();
-		_totalTime += MILLISECSPERCYCLE;
+		_totalTime += (1000 / _vm->getFramesPerSecond());
 		return true;
 	}
 #endif
@@ -555,31 +556,4 @@ void Screen::closeBackgroundLayer() {
 	_layer = 0;
 }
 
-#ifdef BACKEND_8BIT
-void Screen::plotYUV(byte *lut, int width, int height, byte *const *dat) {
-	byte *buf = _buffer + ((480 - height) / 2) * RENDERWIDE + (640 - width) / 2;
-
-	int x, y;
-
-	int ypos = 0;
-	int cpos = 0;
-	int linepos = 0;
-
-	for (y = 0; y < height; y += 2) {
-		for (x = 0; x < width; x += 2) {
-			int i = ((((dat[2][cpos] + ROUNDADD) >> SHIFT) * (BITDEPTH + 1)) + ((dat[1][cpos] + ROUNDADD) >> SHIFT)) * (BITDEPTH + 1);
-			cpos++;
-
-			buf[linepos               ] = lut[i + ((dat[0][        ypos  ] + ROUNDADD) >> SHIFT)];
-			buf[RENDERWIDE + linepos++] = lut[i + ((dat[0][width + ypos++] + ROUNDADD) >> SHIFT)];
-			buf[linepos               ] = lut[i + ((dat[0][        ypos  ] + ROUNDADD) >> SHIFT)];
-			buf[RENDERWIDE + linepos++] = lut[i + ((dat[0][width + ypos++] + ROUNDADD) >> SHIFT)];
-		}
-		linepos += (2 * RENDERWIDE - width);
-		ypos += width;
-	}
-}
-#endif
-
-
 } // End of namespace Sword2
