@@ -1,5 +1,11 @@
-/* Copyright (C) 1994-1998 Revolution Software Ltd.
- * Copyright (C) 2003-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
+ *
+ * Additional copyright for this file:
+ * Copyright (C) 1994-1998 Revolution Software Ltd.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +21,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/sword2/resman.cpp $
- * $Id: resman.cpp 22594 2006-05-24 04:48:39Z eriktorbjorn $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/sword2/resman.cpp $
+ * $Id: resman.cpp 27030 2007-05-31 20:28:29Z fingolfin $
  */
 
 #include "common/stdafx.h"
@@ -276,7 +282,7 @@ byte *ResourceManager::openResource(uint32 res, bool dump) {
 
 		file->read(_resList[res].ptr, len);
 
-		debug(3, "Loaded resource '%s' from '%s' on CD %d (%d)", fetchName(_resList[res].ptr), _resFiles[cluFileNum].fileName, getCD(), _resFiles[cluFileNum].cd);
+		debug(3, "Loaded resource '%s' (%d) from '%s' on CD %d (%d)", fetchName(_resList[res].ptr), res, _resFiles[cluFileNum].fileName, getCD(), _resFiles[cluFileNum].cd);
 
 		if (dump) {
 			char buf[256];
@@ -328,11 +334,7 @@ byte *ResourceManager::openResource(uint32 res, bool dump) {
 				break;
 			}
 
-#if defined(MACOS_CARBON)
-			sprintf(buf, ":dumps:%s-%d.dmp", tag, res);
-#else
 			sprintf(buf, "dumps/%s-%d.dmp", tag, res);
-#endif
 
 			if (!Common::File::exists(buf)) {
 				if (out.open(buf, Common::File::kFileWriteMode))
@@ -427,31 +429,27 @@ Common::File *ResourceManager::openCluFile(uint16 fileNum) {
 }
 
 void ResourceManager::readCluIndex(uint16 fileNum, Common::File *file) {
-	if (_resFiles[fileNum].entryTab == NULL) {
-		// we didn't read from this file before, get its index table
-		if (file == NULL)
-			file = openCluFile(fileNum);
-		else
-			file->incRef();
+	// we didn't read from this file before, get its index table
+	assert(_resFiles[fileNum].entryTab == NULL);
+	assert(file);
 
-		// 1st DWORD of a cluster is an offset to the look-up table
-		uint32 table_offset = file->readUint32LE();
-		debug(6, "table offset = %d", table_offset);
-		uint32 tableSize = file->size() - table_offset; // the table is stored at the end of the file
-		file->seek(table_offset);
+	// 1st DWORD of a cluster is an offset to the look-up table
+	uint32 table_offset = file->readUint32LE();
+	debug(6, "table offset = %d", table_offset);
+	uint32 tableSize = file->size() - table_offset; // the table is stored at the end of the file
+	file->seek(table_offset);
+
+	assert((tableSize % 8) == 0);
+	_resFiles[fileNum].entryTab = (uint32*)malloc(tableSize);
+	_resFiles[fileNum].numEntries = tableSize / 8;
+	file->read(_resFiles[fileNum].entryTab, tableSize);
+	if (file->ioFailed())
+		error("unable to read index table from file %s\n", _resFiles[fileNum].fileName);
 
-		assert((tableSize % 8) == 0);
-		_resFiles[fileNum].entryTab = (uint32*)malloc(tableSize);
-		_resFiles[fileNum].numEntries = tableSize / 8;
-		file->read(_resFiles[fileNum].entryTab, tableSize);
-		if (file->ioFailed())
-			error("unable to read index table from file %s\n", _resFiles[fileNum].fileName);
 #ifdef SCUMM_BIG_ENDIAN
-		for (int tabCnt = 0; tabCnt < _resFiles[fileNum].numEntries * 2; tabCnt++)
-			_resFiles[fileNum].entryTab[tabCnt] = FROM_LE_32(_resFiles[fileNum].entryTab[tabCnt]);
+	for (int tabCnt = 0; tabCnt < _resFiles[fileNum].numEntries * 2; tabCnt++)
+	  _resFiles[fileNum].entryTab[tabCnt] = FROM_LE_32(_resFiles[fileNum].entryTab[tabCnt]);
 #endif
-		file->decRef();
-	}
 }
 
 /**
@@ -495,7 +493,9 @@ uint32 ResourceManager::fetchLen(uint32 res) {
 	// open the cluster file
 
 	if (_resFiles[parent_res_file].entryTab == NULL) {
-		readCluIndex(parent_res_file);
+		Common::File *file = openCluFile(parent_res_file);
+		readCluIndex(parent_res_file, file);
+		delete file;
 	}
 	return _resFiles[parent_res_file].entryTab[actual_res * 2 + 1];
 }
@@ -538,10 +538,10 @@ void ResourceManager::remove(int res) {
 
 void ResourceManager::removeAll() {
 	// We need to clear the FX queue, because otherwise the sound system
-	// will still believe that the sound resources are in memory, and that
-	// it's ok to close them.
+	// will still believe that the sound resources are in memory. We also
+ 	// need to kill the movie lead-in/out.
 
-	_vm->_sound->clearFxQueue();
+	_vm->_sound->clearFxQueue(true);
 
 	for (uint i = 0; i < _totalResFiles; i++)
 		remove(i);
@@ -555,10 +555,10 @@ void ResourceManager::killAll(bool wantInfo) {
 	int nuked = 0;
 
 	// We need to clear the FX queue, because otherwise the sound system
-	// will still believe that the sound resources are in memory, and that
-	// it's ok to close them.
+	// will still believe that the sound resources are in memory. We also
+	// need to kill the movie lead-in/out.
 
-	_vm->_sound->clearFxQueue();
+	_vm->_sound->clearFxQueue(true);
 
 	for (uint i = 0; i < _totalResFiles; i++) {
 		// Don't nuke the global variables or the player object!
