@@ -1,5 +1,11 @@
-/* Copyright (C) 1994-1998 Revolution Software Ltd.
- * Copyright (C) 2003-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
+ *
+ * Additional copyright for this file:
+ * Copyright (C) 1994-1998 Revolution Software Ltd.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +21,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/sword2/resman.h $
- * $Id: resman.h 23847 2006-09-09 14:36:20Z chrilith $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/sword2/resman.h $
+ * $Id: resman.h 27030 2007-05-31 20:28:29Z fingolfin $
  */
 
 #ifndef	SWORD2_RESMAN_H
@@ -54,7 +60,7 @@ struct ResourceFile {
 class ResourceManager {
 private:
 	Common::File *openCluFile(uint16 fileNum);
-	void readCluIndex(uint16 fileNum, Common::File *file = NULL);
+	void readCluIndex(uint16 fileNum, Common::File *file);
 	void removeFromCacheList(Resource *res);
 	void addToCacheList(Resource *res);
 	void checkMemUsage();
@@ -102,7 +108,12 @@ public:
 		return ptr[0];
 	}
 
-	byte *fetchName(uint32 res, byte *buf) {
+	byte *fetchName(uint32 res, byte *buf = NULL) {
+		static byte tempbuf[NAME_LEN];
+
+		if (!buf)
+			buf = tempbuf;
+
 		byte *ptr = openResource(res);
 		memcpy(buf, ptr + 10, NAME_LEN);
 		closeResource(res);
