@@ -1,5 +1,11 @@
-/* Copyright (C) 1994-1998 Revolution Software Ltd.
- * Copyright (C) 2003-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
+ *
+ * Additional copyright for this file:
+ * Copyright (C) 1994-1998 Revolution Software Ltd.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +21,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/sword2/saveload.cpp $
- * $Id: saveload.cpp 20741 2006-02-17 15:07:36Z eriktorbjorn $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/sword2/saveload.cpp $
+ * $Id: saveload.cpp 27030 2007-05-31 20:28:29Z fingolfin $
  */
 
 // ---------------------------------------------------------------------------
@@ -44,17 +50,19 @@
 
 namespace Sword2 {
 
-// A savegame consists of a header and the global variables
+char *Sword2Engine::getSaveFileName(uint16 slotNo) {
+	static char buf[128];
 
-// Max length of a savegame filename
-#define	MAX_FILENAME_LEN 128
+	snprintf(buf, sizeof(buf), "%s.%.3d", _targetName.c_str(), slotNo);
+	return buf;
+}
 
 /**
- * Calculate size of required savegame buffer
+ * Calculate size of required savegame buffer. A savegame consists of a header
+ * and the global variables.
  */
 
 uint32 Sword2Engine::findBufferSize() {
-	// Size of savegame header + size of global variables
 	return 212 + _resman->fetchLen(1);
 }
 
@@ -121,9 +129,7 @@ uint32 Sword2Engine::saveGame(uint16 slotNo, byte *desc) {
 }
 
 uint32 Sword2Engine::saveData(uint16 slotNo, byte *buffer, uint32 bufferSize) {
-	char saveFileName[MAX_FILENAME_LEN];
-
-	sprintf(saveFileName, "%s.%.3d", _targetName.c_str(), slotNo);
+	char *saveFileName = getSaveFileName(slotNo);
 
 	Common::OutSaveFile *out;
 
@@ -132,7 +138,7 @@ uint32 Sword2Engine::saveData(uint16 slotNo, byte *buffer, uint32 bufferSize) {
 	}
 
 	out->write(buffer, bufferSize);
-	out->flush();
+	out->finalize();
 
 	if (!out->ioFailed()) {
 		delete out;
@@ -201,9 +207,7 @@ uint32 Sword2Engine::restoreGame(uint16 slotNo) {
 }
 
 uint32 Sword2Engine::restoreData(uint16 slotNo, byte *buffer, uint32 bufferSize) {
-	char saveFileName[MAX_FILENAME_LEN];
-
-	sprintf(saveFileName, "%s.%.3d", _targetName.c_str(), slotNo);
+	char *saveFileName = getSaveFileName(slotNo);
 
 	Common::InSaveFile *in;
 
@@ -368,9 +372,7 @@ uint32 Sword2Engine::restoreFromBuffer(byte *buffer, uint32 size) {
  */
 
 uint32 Sword2Engine::getSaveDescription(uint16 slotNo, byte *description) {
-	char saveFileName[MAX_FILENAME_LEN];
-
-	sprintf(saveFileName, "%s.%.3d", _targetName.c_str(), slotNo);
+	char *saveFileName = getSaveFileName(slotNo);
 
 	Common::InSaveFile *in;
 
@@ -393,9 +395,7 @@ bool Sword2Engine::saveExists() {
 }
 
 bool Sword2Engine::saveExists(uint16 slotNo) {
-	char saveFileName[MAX_FILENAME_LEN];
-
-	sprintf(saveFileName, "%s.%.3d", _targetName.c_str(), slotNo);
+	char *saveFileName = getSaveFileName(slotNo);
 
 	Common::InSaveFile *in;
 
