@@ -1,5 +1,11 @@
-/* Copyright (C) 1994-1998 Revolution Software Ltd.
- * Copyright (C) 2003-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
+ *
+ * Additional copyright for this file:
+ * Copyright (C) 1994-1998 Revolution Software Ltd.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +21,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/sword2/screen.cpp $
- * $Id: screen.cpp 21573 2006-04-03 09:54:05Z eriktorbjorn $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/sword2/screen.cpp $
+ * $Id: screen.cpp 27030 2007-05-31 20:28:29Z fingolfin $
  */
 
 // ---------------------------------------------------------------------------
@@ -491,12 +497,10 @@ void Screen::processLayer(byte *file, uint32 layer_number) {
 	uint32 current_layer_area = layer_head.width * layer_head.height;
 
 	if (current_layer_area > _largestLayerArea) {
-		byte buf[NAME_LEN];
-
 		_largestLayerArea = current_layer_area;
 		sprintf(_largestLayerInfo,
 			"largest layer:  %s layer(%d) is %dx%d",
-			_vm->_resman->fetchName(_thisScreen.background_layer_id, buf),
+			_vm->_resman->fetchName(_thisScreen.background_layer_id),
 			layer_number, layer_head.width, layer_head.height);
 	}
 
@@ -583,12 +587,10 @@ void Screen::processImage(BuildUnit *build_unit) {
 	uint32 current_sprite_area = frame_head.width * frame_head.height;
 
 	if (current_sprite_area > _largestSpriteArea) {
-		byte buf[NAME_LEN];
-
 		_largestSpriteArea = current_sprite_area;
 		sprintf(_largestSpriteInfo,
 			"largest sprite: %s frame(%d) is %dx%d",
-			_vm->_resman->fetchName(build_unit->anim_resource, buf),
+			_vm->_resman->fetchName(build_unit->anim_resource),
 			build_unit->anim_pc,
 			frame_head.width,
 			frame_head.height);
@@ -618,11 +620,9 @@ void Screen::processImage(BuildUnit *build_unit) {
 
 	uint32 rv = drawSprite(&spriteInfo);
 	if (rv) {
-		byte buf[NAME_LEN];
-
 		error("Driver Error %.8x with sprite %s (%d) in processImage",
 			rv,
-			_vm->_resman->fetchName(build_unit->anim_resource, buf),
+			_vm->_resman->fetchName(build_unit->anim_resource),
 			build_unit->anim_resource);
 	}
 
@@ -804,7 +804,7 @@ struct CreditsLine {
 	CreditsLine() {
 		str = NULL;
 		sprite = NULL;
-	};
+	}
 
 	~CreditsLine() {
 		free(str);
@@ -934,13 +934,19 @@ void Screen::rollCredits() {
 			continue;
 		}
 
-		// The German credits contains character code 9. We don't want
-		// the credits to show the 'dud' symbol, so we replace them
-		// with spaces.
+		// Replace invalid character codes to avoid the credits to show
+		// the 'dud' symbol.
 
-		for (char *ptr = line; *ptr; ptr++) {
-			if (*ptr < 32)
+		for (byte *ptr = (byte *)line; *ptr; ptr++) {
+			// The German credits contains character code 9. We
+			// replace them with spaces.
+			if (*ptr == 9)
 				*ptr = 32;
+
+			// The Spanish credits contains character code 170. We
+			// replace them with dots.
+			if (*ptr == 170)
+				*ptr = '.';
 		}
 
 		char *center_mark = strchr(line, '^');
