@@ -1,5 +1,11 @@
-/* Copyright (C) 1994-1998 Revolution Software Ltd.
- * Copyright (C) 2003-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
+ *
+ * Additional copyright for this file:
+ * Copyright (C) 1994-1998 Revolution Software Ltd.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +21,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/sword2/sound.cpp $
- * $Id: sound.cpp 22231 2006-04-29 22:33:31Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/sword2/sound.cpp $
+ * $Id: sound.cpp 27030 2007-05-31 20:28:29Z fingolfin $
  */
 
 // ---------------------------------------------------------------------------
@@ -83,13 +89,13 @@ Sound::Sound(Sword2Engine *vm) {
 	_mixBuffer = NULL;
 	_mixBufferLen = 0;
 
-	_vm->_mixer->setupPremix(this, Audio::Mixer::kMusicSoundType);
+	_vm->_mixer->playInputStream(Audio::Mixer::kMusicSoundType, &_mixerSoundHandle, this, -1, Audio::Mixer::kMaxChannelVolume, 0, false, true);
 }
 
 Sound::~Sound() {
-	_vm->_mixer->setupPremix(0);
+	_vm->_mixer->stopHandle(_mixerSoundHandle);
 
-	clearFxQueue();
+	clearFxQueue(true);
 	stopMusic(true);
 	stopSpeech();
 
@@ -121,15 +127,25 @@ void Sound::setReverseStereo(bool reverse) {
 }
 
 /**
- * Stop all sounds, close their resources and clear the FX queue.
+ * Stop all sounds, close their resources and clear the FX queue. This is used
+ * when going from one room to another, among other things.
  */
 
-void Sound::clearFxQueue() {
+void Sound::clearFxQueue(bool killMovieSounds) {
 	for (int i = 0; i < FXQ_LENGTH; i++) {
 		if (_fxQueue[i].resource) {
 			stopFx(i);
 		}
 	}
+
+	// We aren't just going to change rooms or anything like that, we are
+	// killing off resources (e.g. when restoring or restarting). We need
+	// to also kill any movie lead-in/out sounds.
+
+	if (killMovieSounds) {
+		_vm->_mixer->stopHandle(_leadInHandle);
+		_vm->_mixer->stopHandle(_leadOutHandle);
+	}
 }
 
 /**
@@ -174,6 +190,33 @@ void Sound::processFxQueue() {
 	}
 }
 
+/**
+ * This function is used by the cutscene player to play the movie lead-in/out.
+ * @param res The sound resource to play
+ * @param type Either kLeadInSound or kLeadOutSound
+ */
+
+void Sound::playMovieSound(int32 res, int type) {
+	Audio::SoundHandle *handle;
+
+	if (type == kLeadInSound)
+		handle = &_leadInHandle;
+	else
+		handle = &_leadOutHandle;
+
+	if (_vm->_mixer->isSoundHandleActive(*handle)) {
+		_vm->_mixer->stopHandle(*handle);
+	}
+
+	byte *data = _vm->_resman->openResource(res);
+	uint32 len = _vm->_resman->fetchLen(res) - ResHeader::size();
+
+	assert(_vm->_resman->fetchType(data) == WAV_FILE);
+	data += ResHeader::size();
+
+	_vm->_sound->playFx(handle, data, len, Audio::Mixer::kMaxChannelVolume, 0, false, Audio::Mixer::kMusicSoundType);
+}
+
 /**
  * Queue a sound effect for playing later.
  * @param res the sound resource number
@@ -202,9 +245,7 @@ void Sound::queueFx(int32 res, int32 type, int32 delay, int32 volume, int32 pan)
 			break;
 		}
 
-		byte buf[NAME_LEN];
-
-		debug(0, "SFX (sample=\"%s\", vol=%d, pan=%d, delay=%d, type=%s)", _vm->_resman->fetchName(res, buf), volume, pan, delay, typeStr);
+		debug(0, "SFX (sample=\"%s\", vol=%d, pan=%d, delay=%d, type=%s)", _vm->_resman->fetchName(res), volume, pan, delay, typeStr);
 	}
 
 	for (int i = 0; i < FXQ_LENGTH; i++) {
@@ -221,7 +262,7 @@ void Sound::queueFx(int32 res, int32 type, int32 delay, int32 volume, int32 pan)
 				// effects, however, it's the average number of
 				// seconds between playing the sound, so we
 				// have to multiply by the frame rate.
-				delay *= 12;
+				delay *= FRAMES_PER_SECOND;
 			}
 
 			volume = (volume * Audio::Mixer::kMaxChannelVolume) / 16;
@@ -276,7 +317,7 @@ int32 Sound::playFx(Audio::SoundHandle *handle, byte *data, uint32 len, uint8 vo
 	if (loop)
 		flags |= Audio::Mixer::FLAG_LOOP;
 
-	_vm->_mixer->playRaw(handle, data + stream.pos(), size, rate, flags, -1, vol, pan, 0, 0, soundType);
+	_vm->_mixer->playRaw(soundType, handle, data + stream.pos(), size, rate, flags, -1, vol, pan, 0, 0);
 	return RD_OK;
 }
 
