@@ -1,5 +1,11 @@
-/* Copyright (C) 1994-1998 Revolution Software Ltd.
- * Copyright (C) 2003-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
+ *
+ * Additional copyright for this file:
+ * Copyright (C) 1994-1998 Revolution Software Ltd.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +21,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/sword2/sound.h $
- * $Id: sound.h 22231 2006-04-29 22:33:31Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/sword2/sound.h $
+ * $Id: sound.h 27030 2007-05-31 20:28:29Z fingolfin $
  */
 
 /*****************************************************************************
@@ -53,6 +59,11 @@ enum {
 	kFlacMode
 };
 
+enum {
+	kLeadInSound,
+	kLeadOutSound
+};
+
 enum {
 	// These three types correspond to types set by the scripts
 	FX_SPOT		= 0,
@@ -171,6 +182,10 @@ private:
 
 	Common::Mutex _mutex;
 
+	Audio::SoundHandle _mixerSoundHandle;
+	Audio::SoundHandle _leadInHandle;
+	Audio::SoundHandle _leadOutHandle;
+
 	struct FxQueueEntry {
 		Audio::SoundHandle handle;	// sound handle
 		uint32 resource;		// resource id of sample
@@ -220,7 +235,7 @@ public:
 
 	// End of AudioStream API
 
-	void clearFxQueue();
+	void clearFxQueue(bool killMovieSounds);
 	void processFxQueue();
 
 	void setReverseStereo(bool reverse);
@@ -250,6 +265,8 @@ public:
 	void pauseAllSound();
 	void unpauseAllSound();
 
+	void playMovieSound(int32 res, int type);
+
 	void queueFx(int32 res, int32 type, int32 delay, int32 volume, int32 pan);
 	int32 playFx(FxQueueEntry *fx);
 	int32 playFx(Audio::SoundHandle *handle, byte *data, uint32 len, uint8 vol, int8 pan, bool loop, Audio::Mixer::SoundType soundType);
@@ -259,7 +276,6 @@ public:
 	int32 getSpeechStatus();
 	int32 amISpeaking();
 	int32 playCompSpeech(uint32 speechId, uint8 vol, int8 pan);
-	uint32 preFetchCompSpeech(uint32 speechId, uint16 **buf);
 	int32 stopSpeech();
 
 	int32 streamCompMusic(uint32 musicId, bool loop);
