@@ -1,5 +1,11 @@
-/* Copyright (C) 1994-1998 Revolution Software Ltd.
- * Copyright (C) 2003-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
+ *
+ * Additional copyright for this file:
+ * Copyright (C) 1994-1998 Revolution Software Ltd.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +21,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/sword2/startup.cpp $
- * $Id: startup.cpp 23437 2006-07-08 20:26:13Z eriktorbjorn $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/sword2/startup.cpp $
+ * $Id: startup.cpp 27030 2007-05-31 20:28:29Z fingolfin $
  */
 
 #include "common/stdafx.h"
@@ -80,9 +86,7 @@ bool Sword2Engine::initStartMenu() {
 		}
 
 		if (_resman->fetchType(id) != SCREEN_MANAGER) {
-			byte name[NAME_LEN];
-
-			warning("startup.inf:%d: '%s' (%d) is not a screen manager", lineno, _resman->fetchName(id, name), id);
+			warning("startup.inf:%d: '%s' (%d) is not a screen manager", lineno, _resman->fetchName(id), id);
 			continue;
 		}
 
@@ -136,7 +140,7 @@ void Sword2Engine::registerStartPoint(int32 key, char *name) {
 void Sword2Engine::runStart(int start) {
 	// Restarting - stop sfx, music & speech!
 
-	_sound->clearFxQueue();
+	_sound->clearFxQueue(true);
 	_logic->fnStopMusic(NULL);
 	_sound->unpauseSpeech();
 	_sound->stopSpeech();
