@@ -1,5 +1,11 @@
-/* Copyright (C) 1994-1998 Revolution Software Ltd.
- * Copyright (C) 2003-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
+ *
+ * Additional copyright for this file:
+ * Copyright (C) 1994-1998 Revolution Software Ltd.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,18 +21,18 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/sword2/sword2.cpp $
- * $Id: sword2.cpp 23755 2006-08-26 12:02:29Z eriktorbjorn $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/sword2/sword2.cpp $
+ * $Id: sword2.cpp 27030 2007-05-31 20:28:29Z fingolfin $
  */
 
 #include "common/stdafx.h"
 
-#include "backends/fs/fs.h"
-
 #include "base/plugins.h"
 
 #include "common/config-manager.h"
 #include "common/file.h"
+#include "common/fs.h"
+#include "common/events.h"
 #include "common/system.h"
 
 #include "sword2/sword2.h"
@@ -43,10 +49,6 @@
 #include "sword2/screen.h"
 #include "sword2/sound.h"
 
-#ifdef _WIN32_WCE
-extern bool isSmartphone();
-#endif
-
 namespace Sword2 {
 
 struct GameSettings {
@@ -70,7 +72,7 @@ GameList Engine_SWORD2_gameIDList() {
 	const Sword2::GameSettings *g = Sword2::sword2_settings;
 	GameList games;
 	while (g->gameid) {
-		games.push_back(*g);
+		games.push_back(GameDescriptor(g->gameid, g->description));
 		g++;
 	}
 	return games;
@@ -83,12 +85,13 @@ GameDescriptor Engine_SWORD2_findGameID(const char *gameid) {
 			break;
 		g++;
 	}
-	return *g;
+	return GameDescriptor(g->gameid, g->description);
 }
 
-DetectedGameList Engine_SWORD2_detectGames(const FSList &fslist) {
-	DetectedGameList detectedGames;
+GameList Engine_SWORD2_detectGames(const FSList &fslist) {
+	GameList detectedGames;
 	const Sword2::GameSettings *g;
+	FSList::const_iterator file;
 
 	// TODO: It would be nice if we had code here which distinguishes
 	// between the 'sword2' and 'sword2demo' targets. The current code
@@ -96,18 +99,42 @@ DetectedGameList Engine_SWORD2_detectGames(const FSList &fslist) {
 
 	for (g = Sword2::sword2_settings; g->gameid; ++g) {
 		// Iterate over all files in the given directory
-		for (FSList::const_iterator file = fslist.begin(); file != fslist.end(); ++file) {
+		for (file = fslist.begin(); file != fslist.end(); ++file) {
 			if (!file->isDirectory()) {
-				const char *gameName = file->displayName().c_str();
+				const char *fileName = file->name().c_str();
 
-				if (0 == scumm_stricmp(g->detectname, gameName)) {
+				if (0 == scumm_stricmp(g->detectname, fileName)) {
 					// Match found, add to list of candidates, then abort inner loop.
-					detectedGames.push_back(*g);
+					detectedGames.push_back(GameDescriptor(g->gameid, g->description));
 					break;
 				}
 			}
 		}
 	}
+
+	
+	if (detectedGames.empty()) {
+		// Nothing found -- try to recurse into the 'clusters' subdirectory,
+		// present e.g. if the user copied the data straight from CD.
+		for (file = fslist.begin(); file != fslist.end(); ++file) {
+			if (file->isDirectory()) {
+				const char *fileName = file->name().c_str();
+
+				if (0 == scumm_stricmp("clusters", fileName)) {
+					FSList recList;
+					if (file->listDir(recList, FilesystemNode::kListAll)) {
+						GameList recGames(Engine_SWORD2_detectGames(recList));
+						if (!recGames.empty()) {
+							detectedGames.push_back(recGames);
+							break;
+						}
+					}
+				}
+			}
+		}
+	}
+
+
 	return detectedGames;
 }
 
@@ -117,27 +144,25 @@ PluginError Engine_SWORD2_create(OSystem *syst, Engine **engine) {
 
 	FSList fslist;
 	FilesystemNode dir(ConfMan.get("path"));
-	if (!dir.listDir(fslist, FilesystemNode::kListFilesOnly)) {
-		warning("Sword2Engine: invalid game path '%s'", dir.path().c_str());
+	if (!dir.listDir(fslist, FilesystemNode::kListAll)) {
 		return kInvalidPathError;
 	}
 
 	// Invoke the detector
 	Common::String gameid = ConfMan.get("gameid");
-	DetectedGameList detectedGames = Engine_SWORD2_detectGames(fslist);
+	GameList detectedGames = Engine_SWORD2_detectGames(fslist);
 
 	for (uint i = 0; i < detectedGames.size(); i++) {
-		if (detectedGames[i].gameid == gameid) {
+		if (detectedGames[i].gameid() == gameid) {
 			*engine = new Sword2::Sword2Engine(syst);
 			return kNoError;
 		}
 	}
 
-	warning("Sword2Engine: Unable to locate game data at path '%s'", dir.path().c_str());
 	return kNoGameDataFoundError;
 }
 
-REGISTER_PLUGIN(SWORD2, "Broken Sword 2");
+REGISTER_PLUGIN(SWORD2, "Broken Sword 2", "Broken Sword Games (C) Revolution");
 
 namespace Sword2 {
 
@@ -168,7 +193,6 @@ Sword2Engine::Sword2Engine(OSystem *syst) : Engine(syst) {
 	_debugger = NULL;
 
 	_keyboardEvent.pending = false;
-	_keyboardEvent.repeat = 0;
 	_mouseEvent.pending = false;
 
 	_wantSfxDebug = false;
@@ -182,6 +206,7 @@ Sword2Engine::Sword2Engine(OSystem *syst) : Engine(syst) {
 	_graphicsLevelFudged = false;
 
 	_gameCycle = 0;
+	_gameSpeed = 1;
 
 	_quit = false;
 }
@@ -197,22 +222,8 @@ Sword2Engine::~Sword2Engine() {
 	delete _memory;
 }
 
-void Sword2Engine::errorString(const char *buf1, char *buf2) {
-	strcpy(buf2, buf1);
-
-#ifdef _WIN32_WCE
-	if (isSmartphone())
-		return;
-#endif
-
-	// Unless an error -originated- within the debugger, spawn the
-	// debugger. Otherwise exit out normally.
-	if (_debugger && !_debugger->isAttached()) {
-		// (Print it again in case debugger segfaults)
-		printf("%s\n", buf2);
-		_debugger->attach(buf2);
-		_debugger->onFrame();
-	}
+GUI::Debugger *Sword2Engine::getDebugger() {
+	return _debugger;
 }
 
 void Sword2Engine::registerDefaultSettings() {
@@ -248,6 +259,10 @@ void Sword2Engine::writeSettings() {
 	ConfMan.flushToDisk();
 }
 
+int Sword2Engine::getFramesPerSecond() {
+	return _gameSpeed * FRAMES_PER_SECOND;
+}
+
 /**
  * The global script variables and player object should be kept open throughout
  * the game, so that they are never expelled by the resource manager.
@@ -366,9 +381,9 @@ int Sword2Engine::go() {
 		KeyboardEvent *ke = keyboardEvent();
 
 		if (ke) {
-			if ((ke->modifiers == OSystem::KBD_CTRL && ke->keycode == 'd') || ke->ascii == '#' || ke->ascii == '~') {
+			if ((ke->modifiers == Common::KBD_CTRL && ke->keycode == 'd') || ke->ascii == '#' || ke->ascii == '~') {
 				_debugger->attach();
-			} else if (ke->modifiers == 0 || ke->modifiers == OSystem::KBD_SHIFT) {
+			} else if (ke->modifiers == 0 || ke->modifiers == Common::KBD_SHIFT) {
 				switch (ke->keycode) {
 				case 'p':
 					if (_gamePaused)
@@ -518,94 +533,74 @@ uint32 Sword2Engine::setInputEventFilter(uint32 filter) {
 	return oldFilter;
 }
 
-/**
- * Clear the input events. This is so that we won't get any keyboard repeat
- * right after using the debugging console.
- */
-
-void Sword2Engine::clearInputEvents() {
-	_keyboardEvent.pending = false;
-	_keyboardEvent.repeat = 0;
-	_mouseEvent.pending = false;
-}
-
 /**
  * OSystem Event Handler. Full of cross platform goodness and 99% fat free!
  */
 
 void Sword2Engine::parseInputEvents() {
-	OSystem::Event event;
+	Common::Event event;
 
-	uint32 now = _system->getMillis();
-
-	while (_system->pollEvent(event)) {
+	while (_eventMan->pollEvent(event)) {
 		switch (event.type) {
-		case OSystem::EVENT_KEYDOWN:
+		case Common::EVENT_KEYDOWN:
+			if (event.kbd.flags == Common::KBD_CTRL) {
+				if (event.kbd.keycode == 'f') {
+					if (_gameSpeed == 1)
+						_gameSpeed = 2;
+					else
+						_gameSpeed = 1;
+				}
+			}
 			if (!(_inputEventFilter & RD_KEYDOWN)) {
 				_keyboardEvent.pending = true;
-				_keyboardEvent.repeat = now + 400;
 				_keyboardEvent.ascii = event.kbd.ascii;
 				_keyboardEvent.keycode = event.kbd.keycode;
 				_keyboardEvent.modifiers = event.kbd.flags;
 			}
 			break;
-		case OSystem::EVENT_KEYUP:
-			_keyboardEvent.repeat = 0;
-			break;
-		case OSystem::EVENT_MOUSEMOVE:
-			if (!(_inputEventFilter & RD_KEYDOWN)) {
-				_mouse->setPos(event.mouse.x, event.mouse.y - MENUDEEP);
-			}
-			break;
-		case OSystem::EVENT_LBUTTONDOWN:
+		case Common::EVENT_LBUTTONDOWN:
 			if (!(_inputEventFilter & RD_LEFTBUTTONDOWN)) {
 				_mouseEvent.pending = true;
 				_mouseEvent.buttons = RD_LEFTBUTTONDOWN;
 			}
 			break;
-		case OSystem::EVENT_RBUTTONDOWN:
+		case Common::EVENT_RBUTTONDOWN:
 			if (!(_inputEventFilter & RD_RIGHTBUTTONDOWN)) {
 				_mouseEvent.pending = true;
 				_mouseEvent.buttons = RD_RIGHTBUTTONDOWN;
 			}
 			break;
-		case OSystem::EVENT_LBUTTONUP:
+		case Common::EVENT_LBUTTONUP:
 			if (!(_inputEventFilter & RD_LEFTBUTTONUP)) {
 				_mouseEvent.pending = true;
 				_mouseEvent.buttons = RD_LEFTBUTTONUP;
 			}
 			break;
-		case OSystem::EVENT_RBUTTONUP:
+		case Common::EVENT_RBUTTONUP:
 			if (!(_inputEventFilter & RD_RIGHTBUTTONUP)) {
 				_mouseEvent.pending = true;
 				_mouseEvent.buttons = RD_RIGHTBUTTONUP;
 			}
 			break;
-		case OSystem::EVENT_WHEELUP:
+		case Common::EVENT_WHEELUP:
 			if (!(_inputEventFilter & RD_WHEELUP)) {
 				_mouseEvent.pending = true;
 				_mouseEvent.buttons = RD_WHEELUP;
 			}
 			break;
-		case OSystem::EVENT_WHEELDOWN:
+		case Common::EVENT_WHEELDOWN:
 			if (!(_inputEventFilter & RD_WHEELDOWN)) {
 				_mouseEvent.pending = true;
 				_mouseEvent.buttons = RD_WHEELDOWN;
 			}
 			break;
-		case OSystem::EVENT_QUIT:
+		case Common::EVENT_QUIT:
 			closeGame();
 			break;
 		default:
 			break;
 		}
 	}
-
-	// Handle keyboard auto-repeat
-	if (!_keyboardEvent.pending && _keyboardEvent.repeat && now >= _keyboardEvent.repeat) {
-		_keyboardEvent.pending = true;
-		_keyboardEvent.repeat = now + 100;
-	}
 }
 
 void Sword2Engine::gameCycle() {
