@@ -1,5 +1,11 @@
-/* Copyright (C) 1994-1998 Revolution Software Ltd.
- * Copyright (C) 2003-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
+ *
+ * Additional copyright for this file:
+ * Copyright (C) 1994-1998 Revolution Software Ltd.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,13 +21,15 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/engines/sword2/sword2.h $
- * $Id: sword2.h 21916 2006-04-15 20:36:41Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/engines/sword2/sword2.h $
+ * $Id: sword2.h 27030 2007-05-31 20:28:29Z fingolfin $
  */
 
 #ifndef	SWORD2_H
 #define	SWORD2_H
 
+#define FRAMES_PER_SECOND 12
+
 // Enable this to make it possible to clear the mouse cursor luggage by
 // right-clicking. The original didn't do this, but it feels natural to me.
 // However, I'm afraid that it'll interfer badly with parts of the game, so
@@ -29,7 +37,7 @@
 
 #define RIGHT_CLICK_CLEARS_LUGGAGE 0
 
-#include "base/engine.h"
+#include "engines/engine.h"
 
 #include "common/util.h"
 
@@ -61,8 +69,7 @@ enum {
 	RD_RIGHTBUTTONUP		= 0x08,
 	RD_WHEELUP			= 0x10,
 	RD_WHEELDOWN			= 0x20,
-	RD_KEYDOWN			= 0x40,
-	RD_MOUSEMOVE			= 0x80
+	RD_KEYDOWN			= 0x40
 };
 
 struct MouseEvent {
@@ -72,7 +79,6 @@ struct MouseEvent {
 
 struct KeyboardEvent {
 	bool pending;
-	uint32 repeat;
 	uint16 ascii;
 	int keycode;
 	int modifiers;
@@ -116,6 +122,7 @@ private:
 	uint32 _startRes;
 
 	bool _useSubtitles;
+	int _gameSpeed;
 
 	StartUp _startList[MAX_starts];
 
@@ -125,6 +132,8 @@ public:
 	int go();
 	int init();
 
+	int getFramesPerSecond();
+
 	void registerDefaultSettings();
 	void readSettings();
 	void writeSettings();
@@ -156,7 +165,6 @@ public:
 
 	uint32 setInputEventFilter(uint32 filter);
 
-	void clearInputEvents();
 	void parseInputEvents();
 
 	bool checkForMouseEvents();
@@ -197,6 +205,7 @@ public:
 	bool saveExists();
 	bool saveExists(uint16 slotNo);
 	uint32 restoreFromBuffer(byte *buffer, uint32 size);
+	char *getSaveFileName(uint16 slotNo);
 	uint32 findBufferSize();
 
 	bool _gamePaused;
@@ -209,7 +218,7 @@ public:
 
 	void sleepUntil(uint32 time);
 
-	void errorString(const char *buf_input, char *buf_output);
+	GUI::Debugger *getDebugger();
 	void initialiseFontResourceFlags();
 	void initialiseFontResourceFlags(uint8 language);
 
