@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/graphics/cursorman.cpp $
- * $Id: cursorman.cpp 22673 2006-05-27 11:47:44Z wjpalenstijn $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/graphics/cursorman.cpp $
+ * $Id: cursorman.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  */
 
 #include "graphics/cursorman.h"
@@ -34,6 +37,7 @@ CursorManager::CursorManager() {
 	if (!g_initialized) {
 		g_initialized = true;
 		_cursorStack.clear();
+		_cursorPaletteStack.clear();
 	}
 }
 
@@ -107,4 +111,83 @@ void CursorManager::replaceCursor(const byte *buf, uint w, uint h, int hotspotX,
 	g_system->setMouseCursor(cur->_data, w, h, hotspotX, hotspotY, keycolor, targetScale);
 }
 
+void CursorManager::disableCursorPalette(bool disable) {
+	if (!g_system->hasFeature(OSystem::kFeatureCursorHasPalette))
+		return;
+
+	if (_cursorPaletteStack.empty())
+		return;
+
+	Palette *pal = _cursorPaletteStack.top();
+	pal->_disabled = disable;
+
+	g_system->disableCursorPalette(true);
+}
+
+void CursorManager::pushCursorPalette(const byte *colors, uint start, uint num) {
+	if (!g_system->hasFeature(OSystem::kFeatureCursorHasPalette))
+		return;
+
+	Palette *pal = new Palette(colors, start, num);
+	_cursorPaletteStack.push(pal);
+
+	if (num)
+		g_system->setCursorPalette(colors, start, num);
+	else
+		g_system->disableCursorPalette(true);
+}
+
+void CursorManager::popCursorPalette() {
+	if (!g_system->hasFeature(OSystem::kFeatureCursorHasPalette))
+		return;
+
+	if (_cursorPaletteStack.empty())
+		return;
+
+	Palette *pal = _cursorPaletteStack.pop();
+	delete pal;
+
+	if (_cursorPaletteStack.empty()) {
+		g_system->disableCursorPalette(true);
+		return;
+	}
+
+	pal = _cursorPaletteStack.top();
+
+	if (pal->_num && !pal->_disabled)
+		g_system->setCursorPalette(pal->_data, pal->_start, pal->_num);
+	else
+		g_system->disableCursorPalette(true);
+}
+
+void CursorManager::replaceCursorPalette(const byte *colors, uint start, uint num) {
+	if (!g_system->hasFeature(OSystem::kFeatureCursorHasPalette))
+		return;
+
+	if (_cursorPaletteStack.empty()) {
+		pushCursorPalette(colors, start, num);
+		return;
+	}
+
+	Palette *pal = _cursorPaletteStack.top();
+	uint size = 4 * num;
+
+	if (pal->_size < size) {
+		// Could not re-use the old buffer. Create a new one.
+		delete[] pal->_data;
+		pal->_data = new byte[size];
+		pal->_size = size;
+	}
+
+	pal->_start = start;
+	pal->_num = num;
+
+	if (num) {
+		memcpy(pal->_data, colors, 4 * num);
+		g_system->setCursorPalette(pal->_data, pal->_start, pal->_num);
+	} else {
+		g_system->disableCursorPalette(true);
+	}
+}
+
 } // End of namespace Graphics
