@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2002-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/graphics/font.h $
- * $Id: font.h 23209 2006-06-20 22:52:01Z eriktorbjorn $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/graphics/font.h $
+ * $Id: font.h 27024 2007-05-30 21:56:52Z fingolfin $
  */
 
 #ifndef GRAPHICS_FONT_H
@@ -81,7 +84,7 @@ public:
 class ScummFont : public Font {
 public:
 	virtual int getFontHeight() const { return 8; }
-	virtual int getMaxCharWidth() const { return 8; };
+	virtual int getMaxCharWidth() const { return 8; }
 
 	virtual int getCharWidth(byte chr) const;
 	virtual void drawChar(Surface *dst, byte chr, int x, int y, uint32 color) const;
@@ -126,7 +129,7 @@ public:
 	~NewFont();
 
 	virtual int getFontHeight() const { return desc.height; }
-	virtual int getMaxCharWidth() const { return desc.maxwidth; };
+	virtual int getMaxCharWidth() const { return desc.maxwidth; }
 
 	virtual int getCharWidth(byte chr) const;
 	virtual void drawChar(Surface *dst, byte chr, int x, int y, uint32 color) const;
