@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2005-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/graphics/fontman.h $
- * $Id: fontman.h 21921 2006-04-15 21:29:41Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/graphics/fontman.h $
+ * $Id: fontman.h 27024 2007-05-30 21:56:52Z fingolfin $
  */
 
 #ifndef GRAPHICS_FONTMAN_H
@@ -27,6 +30,7 @@
 #include "common/singleton.h"
 #include "common/str.h"
 #include "common/hashmap.h"
+#include "common/hash-str.h"
 #include "graphics/font.h"
 
 
@@ -81,7 +85,7 @@ private:
 	friend class Common::Singleton<SingletonBaseType>;
 	FontManager();
 
-	Common::HashMap<Common::String, const Font *> _fontMap;
+	Common::HashMap<Common::String, const Font *, Common::CaseSensitiveString_Hash, Common::CaseSensitiveString_EqualTo> _fontMap;
 };
 
 
