@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/graphics/imageman.cpp $
- * $Id: imageman.cpp 20519 2006-02-11 10:10:44Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/graphics/imageman.cpp $
+ * $Id: imageman.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  */
 
 #include "graphics/imagedec.h"
@@ -43,7 +46,7 @@ ImageManager::~ImageManager() {
 	_surfaces.clear();
 #ifdef USE_ZLIB
 	for (ZipIterator pos2 = _archives.begin(); pos2 != _archives.end(); ++pos2) {
-		unzClose(*pos2);
+		unzClose(pos2->file);
 	}
 	_archives.clear();
 #endif
@@ -54,11 +57,26 @@ bool ImageManager::addArchive(const Common::String &name) {
 	unzFile newFile = unzOpen(name.c_str());
 	if (!newFile)
 		return false;
-	_archives.push_back(newFile);
+	Archive arch;
+	arch.file = newFile;
+	arch.filename = name;
+	_archives.push_back(arch);
 #endif
 	return true;
 }
 
+void ImageManager::remArchive(const Common::String &name) {
+#ifdef USE_ZLIB
+	for (ZipIterator pos = _archives.begin(); pos != _archives.end(); ++pos) {
+		if (pos->filename.compareToIgnoreCase(name) == 0) {
+			unzClose(pos->file);
+			_archives.erase(pos);
+			break;
+		}
+	}
+#endif
+}
+
 bool ImageManager::registerSurface(const Common::String &name, Surface *surf) {
 	if (getSurface(name)) {
 		return false;
@@ -74,7 +92,7 @@ bool ImageManager::registerSurface(const Common::String &name, Surface *surf) {
 #ifdef USE_ZLIB
 			ZipIterator file = _archives.end();
 			for (ZipIterator pos = _archives.begin(); pos != _archives.end(); ++pos) {
-				if (unzLocateFile(*pos, name.c_str(), 2) == UNZ_OK) {
+				if (unzLocateFile(pos->file, name.c_str(), 2) == UNZ_OK) {
 					file = pos;
 					break;
 				}
@@ -84,12 +102,12 @@ bool ImageManager::registerSurface(const Common::String &name, Surface *surf) {
 				return false;
 
 			unz_file_info fileInfo;
-			unzOpenCurrentFile(*file);
-			unzGetCurrentFileInfo(*file, &fileInfo, NULL, 0, NULL, 0, NULL, 0);
+			unzOpenCurrentFile(file->file);
+			unzGetCurrentFileInfo(file->file, &fileInfo, NULL, 0, NULL, 0, NULL, 0);
 			uint8 *buffer = new uint8[fileInfo.uncompressed_size];
 			assert(buffer);
-			unzReadCurrentFile(*file, buffer, fileInfo.uncompressed_size);
-			unzCloseCurrentFile(*file);
+			unzReadCurrentFile(file->file, buffer, fileInfo.uncompressed_size);
+			unzCloseCurrentFile(file->file);
 			Common::MemoryReadStream stream(buffer, fileInfo.uncompressed_size);
 			surf = ImageDecoder::loadFile(stream);
 			delete [] buffer;
