@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/graphics/imageman.h $
- * $Id: imageman.h 20519 2006-02-11 10:10:44Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/graphics/imageman.h $
+ * $Id: imageman.h 27024 2007-05-30 21:56:52Z fingolfin $
  */
 
 #ifndef GRAPHICS_IMAGEMAN_H
@@ -37,7 +40,7 @@ public:
 	~ImageManager();
 
 	/**
-	 * adds an .zip archive to the pool there the ImagaManager searches
+	 * adds an .zip archive to the pool where the ImageManager searches
 	 * for image files
 	 *
 	 * @param name the name of the archive
@@ -45,6 +48,14 @@ public:
 	 */
 	bool addArchive(const Common::String &name);
 
+	/**
+	 * deletes an .zip archive from the pool where the Image Manager searches
+	 * for image files
+	 *
+	 * @param name the name of the archive
+	 */
+	void remArchive(const Common::String &name);
+
 	/** 
 	 * registers a surface to the ImageManager.
 	 * surf->free(), also delete surf, will be called when the ImageManager will
@@ -84,14 +95,18 @@ private:
 	};
 	typedef Common::List<Entry*>::iterator Iterator;
 #ifdef USE_ZLIB
-	typedef Common::List<unzFile>::iterator ZipIterator;
+	struct Archive {
+		unzFile file;
+		Common::String filename;
+	};
+	typedef Common::List<Archive>::iterator ZipIterator;
 #endif
 	
 	Iterator searchHandle(const Common::String &name);
 
 	Common::List<Entry*> _surfaces;
 #ifdef USE_ZLIB
-	Common::List<unzFile> _archives;
+	Common::List<Archive> _archives;
 #endif
 };
 
