@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2002-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/graphics/surface.cpp $
- * $Id: surface.cpp 20519 2006-02-11 10:10:44Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/graphics/surface.cpp $
+ * $Id: surface.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  */
 
 #include "common/stdafx.h"
@@ -164,4 +167,59 @@ void Surface::frameRect(const Common::Rect &r, uint32 color) {
 	vLine(r.right-1, r.top, r.bottom-1, color);
 }
 
+void Surface::move(int dx, int dy, int height) {
+	// Short circuit check - do we have to do anything anyway?
+	if ((dx == 0 && dy == 0) || height <= 0)
+		return;
+
+	byte *src, *dst;
+	int x, y;
+
+	// vertical movement
+	if (dy > 0) {
+		// move down - copy from bottom to top
+		dst = (byte *)pixels + (height - 1) * w;
+		src = dst - dy * w;
+		for (y = dy; y < height; y++) {
+			memcpy(dst, src, w);
+			src -= w;
+			dst -= w;
+		}
+	} else if (dy < 0) {
+		// move up - copy from top to bottom
+		dst = (byte *)pixels;
+		src = dst - dy * w;
+		for (y = -dy; y < height; y++) {
+			memcpy(dst, src, w);
+			src += w;
+			dst += w;
+		}
+	}
+
+	// horizontal movement
+	if (dx > 0) {
+		// move right - copy from right to left
+		dst = (byte *)pixels + (w - 1);
+		src = dst - dx;
+		for (y = 0; y < height; y++) {
+			for (x = dx; x < w; x++) {
+				*dst-- = *src--;
+			}
+			src += w + (w - dx);
+			dst += w + (w - dx);
+		}
+	} else if (dx < 0)  {
+		// move left - copy from left to right
+		dst = (byte *)pixels;
+		src = dst - dx;
+		for (y = 0; y < height; y++) {
+			for (x = -dx; x < w; x++) {
+				*dst++ = *src++;
+			}
+			src += w - (w + dx);
+			dst += w - (w + dx);
+		}
+	}
+}
+
 } // End of namespace Graphics
