@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2002-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/graphics/surface.h $
- * $Id: surface.h 20519 2006-02-11 10:10:44Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/graphics/surface.h $
+ * $Id: surface.h 27024 2007-05-30 21:56:52Z fingolfin $
  */
 
 #ifndef GRAPHICS_SURFACE_H
@@ -65,6 +68,7 @@ struct Surface {
 	void vLine(int x, int y, int y2, uint32 color);
 	void fillRect(const Common::Rect &r, uint32 color);
 	void frameRect(const Common::Rect &r, uint32 color);
+	void move(int dx, int dy, int height);
 };
 
 
