@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2002-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/gui/EditTextWidget.cpp $
- * $Id: EditTextWidget.cpp 22928 2006-06-05 12:22:51Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/gui/EditTextWidget.cpp $
+ * $Id: EditTextWidget.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  */
 
 #include "common/stdafx.h"
@@ -32,7 +35,6 @@ EditTextWidget::EditTextWidget(GuiObject *boss, int x, int y, int w, int h, cons
 	_flags = WIDGET_ENABLED | WIDGET_CLEARBG | WIDGET_RETAIN_FOCUS | WIDGET_WANT_TICKLE;
 	_type = kEditTextWidget;
 
-	handleScreenChanged();
 	setEditString(text);
 }
 
@@ -42,7 +44,6 @@ EditTextWidget::EditTextWidget(GuiObject *boss, const String &name, const String
 	_type = kEditTextWidget;
 	_hints |= THEME_HINT_USE_SHADOW;
 
-	handleScreenChanged();
 	setEditString(text);
 }
 
@@ -51,12 +52,13 @@ void EditTextWidget::setEditString(const String &str) {
 	_backupString = str;
 }
 
-void EditTextWidget::handleScreenChanged() {
-	EditableWidget::handleScreenChanged();
+void EditTextWidget::reflowLayout() {
 	_leftPadding = g_gui.evaluator()->getVar("EditTextWidget.leftPadding", 0);
 	_rightPadding = g_gui.evaluator()->getVar("EditTextWidget.rightPadding", 0);
 
 	_font = (Theme::FontStyle)g_gui.evaluator()->getVar("EditTextWidget.font", Theme::kFontStyleNormal);
+
+	EditableWidget::reflowLayout();
 }
 
 
@@ -71,7 +73,7 @@ void EditTextWidget::handleMouseDown(int x, int y, int button, int clickCount) {
 	uint i;
 
 	for (i = 0; i < _editString.size(); ++i) {
-		width += g_gui.getCharWidth(_editString[i]);
+		width += g_gui.theme()->getCharWidth(_editString[i], _font);
 		if (width >= x)
 			break;
 	}
