@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2001-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/gui/KeysDialog.cpp $
- * $Id: KeysDialog.cpp 22997 2006-06-10 07:21:26Z anotherguest $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/gui/KeysDialog.cpp $
+ * $Id: KeysDialog.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -31,27 +34,14 @@
 
 namespace GUI {
 
-/*
-using GUI::ListWidget;
-using GUI::kListNumberingZero;
-using GUI::WIDGET_CLEARBG;
-using GUI::kListSelectionChangedCmd;
-using GUI::kCloseCmd;
-using GUI::StaticTextWidget;
-using GUI::CommandSender;
-*/
-
 enum {
-	kMapCmd					= 'map ',
-	kOKCmd					= 'ok  '
+	kMapCmd	= 'map ',
+	kOKCmd	= 'ok  '
 };
 
 KeysDialog::KeysDialog(const Common::String &title)
 	: GUI::Dialog("keysdialog") {
 
-//tmp
-//	addButton(this, _w - (buttonWidth + 10), _h - buttonHeight - 8, "Choose", kChooseCmd, 0, ws);
-//tmp
 	new ButtonWidget(this, "keysdialog_map", "Map", kMapCmd, 0);
 	new ButtonWidget(this, "keysdialog_ok", "OK", kOKCmd, 0);
 	new ButtonWidget(this, "keysdialog_cancel", "Cancel", kCloseCmd, 0);
@@ -60,7 +50,7 @@ KeysDialog::KeysDialog(const Common::String &title)
 	_actionsList->setNumberingMode(kListNumberingZero);
 
 	_actionTitle = new StaticTextWidget(this, "keysdialog_action", title);
-	_keyMapping = new StaticTextWidget(this, "keysdialog_mapping", "");
+	_keyMapping = new StaticTextWidget(this, "keysdialog_mapping", "Select an action and click 'Map'");
 
 	_actionTitle->setFlags(WIDGET_CLEARBG);
 	_keyMapping->setFlags(WIDGET_CLEARBG);
@@ -78,30 +68,25 @@ KeysDialog::KeysDialog(const Common::String &title)
 }
 
 void KeysDialog::handleCommand(CommandSender *sender, uint32 cmd, uint32 data) {
-	switch(cmd) {
+	switch (cmd) {
 
 	case kListSelectionChangedCmd:
 		if (_actionsList->getSelected() >= 0) {
-				char selection[100];
-#ifdef __SYMBIAN32__
-				uint16 key = Actions::Instance()->getMapping(_actionsList->getSelected());
-				
-				if(key != 0) {
-					// ScummVM mappings for F1-F9 are different from SDL so remap back to sdl
-					if(key >= 315 && key <= 323) {
-						key = key - 315 + SDLK_F1;
-					}
-				}
+			char selection[100];
 
-				if(key != 0)
-					sprintf(selection, "Associated key : %s", SDL_GetKeyName((SDLKey)key));
-				else
-					sprintf(selection, "Associated key : none");
-#else
-				sprintf(selection, "Associated key : %s", CEDevice::getKeyName(Actions::Instance()->getMapping((ActionType)(_actionsList->getSelected()))).c_str());
+			uint16 key = Actions::Instance()->getMapping(_actionsList->getSelected());
+#ifdef __SYMBIAN32__
+			// ScummVM mappings for F1-F9 are different from SDL so remap back to sdl
+			if (key >= 315 && key <= 323)
+				key = key - 315 + SDLK_F1;
 #endif
-				_keyMapping->setLabel(selection);
-				_keyMapping->draw();
+			if (key != 0)
+				sprintf(selection, "Associated key : %s", SDL_GetKeyName((SDLKey)key));
+			else
+				sprintf(selection, "Associated key : none");
+
+			_keyMapping->setLabel(selection);
+			_keyMapping->draw();
 		}
 		break;
 	case kMapCmd:
@@ -109,29 +94,25 @@ void KeysDialog::handleCommand(CommandSender *sender, uint32 cmd, uint32 data) {
 				_actionTitle->setLabel("Please select an action");
 		}
 		else {
-				char selection[100];
+			char selection[100];
 
-				_actionSelected = _actionsList->getSelected();
+			_actionSelected = _actionsList->getSelected();
+			uint16 key = Actions::Instance()->getMapping(_actionSelected);
 #ifdef __SYMBIAN32__
-				uint16 key = Actions::Instance()->getMapping(_actionSelected);
-				if(key != 0) {
-					// ScummVM mappings for F1-F9 are different from SDL so remap back to sdl
-					if(key >= 315 && key <= 323) {
-						key = key - 315 + SDLK_F1;
-					}
-
-					sprintf(selection, "Associated key : %s", SDL_GetKeyName((SDLKey)key));
-				}
-				else
-					sprintf(selection, "Associated key : none");
-#else
-				sprintf(selection, "Associated key : %s", CEDevice::getKeyName(Actions::Instance()->getMapping((ActionType)_actionSelected)).c_str());
+			// ScummVM mappings for F1-F9 are different from SDL so remap back to sdl
+			if (key >= 315 && key <= 323)
+				key = key - 315 + SDLK_F1;
 #endif
-				_actionTitle->setLabel("Press the key to associate");
-				_keyMapping->setLabel(selection);
-				_keyMapping->draw();
-				Actions::Instance()->beginMapping(true);
-				_actionsList->setEnabled(false);
+			if (key != 0) 
+				sprintf(selection, "Associated key : %s", SDL_GetKeyName((SDLKey)key));
+			else
+				sprintf(selection, "Associated key : none");
+
+			_actionTitle->setLabel("Press the key to associate");
+			_keyMapping->setLabel(selection);
+			_keyMapping->draw();
+			Actions::Instance()->beginMapping(true);
+			_actionsList->setEnabled(false);
 		}
 		_actionTitle->draw();
 		break;
@@ -147,29 +128,25 @@ void KeysDialog::handleCommand(CommandSender *sender, uint32 cmd, uint32 data) {
 }
 
 void KeysDialog::handleKeyDown(uint16 ascii, int keycode, int modifiers){
-		if (!Actions::Instance()->mappingActive()) {
-			Dialog::handleKeyDown(ascii,keycode,modifiers);
-		}
+	if (!Actions::Instance()->mappingActive())
+		Dialog::handleKeyDown(ascii,keycode,modifiers);
 }
 
 void KeysDialog::handleKeyUp(uint16 ascii, int keycode, int modifiers) {
 #ifdef __SYMBIAN32__
 	if (Actions::Instance()->mappingActive()) {
 #else
-		// GAPI key was selected
-		if (modifiers == 0xff  && Actions::Instance()->mappingActive()) {
+	if (modifiers == 0xff  && Actions::Instance()->mappingActive()) {	// GAPI key was selected
 #endif
 		char selection[100];
 
 		Actions::Instance()->setMapping((ActionType)_actionSelected, ascii);
-#ifdef __SYMBIAN32__
-		if(ascii != 0)
+
+		if (ascii != 0)
 			sprintf(selection, "Associated key : %s", SDL_GetKeyName((SDLKey) keycode));
 		else
 			sprintf(selection, "Associated key : none");
-#else
-		sprintf(selection, "Associated key : %s", CEDevice::getKeyName(Actions::Instance()->getMapping((ActionType)_actionSelected)).c_str());
-#endif
+
 		_actionTitle->setLabel("Choose an action to map");
 		_keyMapping->setLabel(selection);
 		_keyMapping->draw();
@@ -177,10 +154,8 @@ void KeysDialog::handleKeyUp(uint16 ascii, int keycode, int modifiers) {
 		_actionSelected = -1;
 		_actionsList->setEnabled(true);
 		Actions::Instance()->beginMapping(false);
-	}
-	else {
+	} else 
 		Dialog::handleKeyUp(ascii,keycode,modifiers);
-	}
 }
 
 } // namespace GUI
