@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2002-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/gui/ListWidget.cpp $
- * $Id: ListWidget.cpp 22873 2006-06-03 13:33:39Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/gui/ListWidget.cpp $
+ * $Id: ListWidget.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  */
 
 #include "common/stdafx.h"
@@ -32,19 +35,12 @@ namespace GUI {
 ListWidget::ListWidget(GuiObject *boss, const String &name)
 	: EditableWidget(boss, name), CommandSender(boss) {
 
-	_leftPadding = g_gui.evaluator()->getVar("ListWidget.leftPadding", 0);
-	_rightPadding = g_gui.evaluator()->getVar("ListWidget.rightPadding", 0);
-	_topPadding = g_gui.evaluator()->getVar("ListWidget.topPadding", 0);
-	_bottomPadding = g_gui.evaluator()->getVar("ListWidget.bottomPadding", 0);
-	_hlLeftPadding = g_gui.evaluator()->getVar("ListWidget.hlLeftPadding", 0);
-	_hlRightPadding = g_gui.evaluator()->getVar("ListWidget.hlRightPadding", 0);
+	_scrollBar = NULL;
+	_textWidth = NULL;
+
+	// This ensures that _entriesPerPage is properly initialised.
+	reflowLayout();
 
-	if (g_gui.getWidgetSize() == kBigWidgetSize) {
-		_scrollBarWidth =  kBigScrollBarWidth;
-	} else {
-		_scrollBarWidth = kNormalScrollBarWidth;
-	}
-	
 	_scrollBar = new ScrollBarWidget(this, _w - _scrollBarWidth, 0, _scrollBarWidth, _h);
 	_scrollBar->setTarget(this);
 
@@ -53,7 +49,6 @@ ListWidget::ListWidget(GuiObject *boss, const String &name)
 	_type = kListWidget;
 	_editMode = false;
 	_numberingMode = kListNumberingOne;
-	_entriesPerPage = (_h - _topPadding - _bottomPadding) / kLineHeight;
 	_currentPos = 0;
 	_selectedItem = -1;
 	_currentKeyDown = 0;
@@ -65,11 +60,6 @@ ListWidget::ListWidget(GuiObject *boss, const String &name)
 
 	// FIXME: This flag should come from widget definition
 	_editable = true;
-
-	_textWidth = new int[_entriesPerPage];
-
-	for (int i = 0; i < _entriesPerPage; i++)
-		_textWidth[i] = 0;
 }
 
 ListWidget::~ListWidget() {
@@ -144,12 +134,9 @@ void ListWidget::handleMouseDown(int x, int y, int button, int clickCount) {
 		return;
 
 	// First check whether the selection changed
-	int newSelectedItem;
-	newSelectedItem = findItem(x, y);
-	if (newSelectedItem > (int)_list.size() - 1)
-		newSelectedItem = -1;
+	int newSelectedItem = findItem(x, y);
 
-	if (_selectedItem != newSelectedItem) {
+	if (_selectedItem != newSelectedItem && newSelectedItem != -1) {
 		if (_editMode)
 			abortEditMode();
 		_selectedItem = newSelectedItem;
@@ -157,15 +144,17 @@ void ListWidget::handleMouseDown(int x, int y, int button, int clickCount) {
 	}
 
 	// TODO: Determine where inside the string the user clicked and place the
-	// caret accordingly. See _editScrollOffset and EditTextWidget::handleMouseDown.
+	// caret accordingly.
+	// See _editScrollOffset and EditTextWidget::handleMouseDown.
 	draw();
 
 }
 
 void ListWidget::handleMouseUp(int x, int y, int button, int clickCount) {
-	// If this was a double click and the mouse is still over the selected item,
-	// send the double click command
-	if (clickCount == 2 && (_selectedItem == findItem(x, y))) {
+	// If this was a double click and the mouse is still over 
+	// the selected item, send the double click command
+	if (clickCount == 2 && (_selectedItem == findItem(x, y)) &&
+		_selectedItem >= 0) {
 		sendCommand(kListItemDoubleClickedCmd, _selectedItem);
 	}
 }
@@ -176,7 +165,13 @@ void ListWidget::handleMouseWheel(int x, int y, int direction) {
 
 
 int ListWidget::findItem(int x, int y) const {
-	return (y - _topPadding) / kLineHeight + _currentPos;
+	if (y < _topPadding) return -1;
+	int item = (y - _topPadding) / kLineHeight + _currentPos;
+	if (item >= _currentPos && item < _currentPos + _entriesPerPage && 
+		item < (int)_list.size())
+		return item;
+	else
+		return -1;
 }
 
 static int matchingCharsIgnoringCase(const char *x, const char *y, bool &stop) {
@@ -441,8 +436,8 @@ void ListWidget::abortEditMode() {
 	g_system->setFeatureState(OSystem::kFeatureVirtualKeyboard, false);
 }
 
-void ListWidget::handleScreenChanged() {
-	Widget::handleScreenChanged();
+void ListWidget::reflowLayout() {
+	Widget::reflowLayout();
 
 	_leftPadding = g_gui.evaluator()->getVar("ListWidget.leftPadding", 0);
 	_rightPadding = g_gui.evaluator()->getVar("ListWidget.rightPadding", 0);
@@ -465,9 +460,11 @@ void ListWidget::handleScreenChanged() {
 	for (int i = 0; i < _entriesPerPage; i++)
 		_textWidth[i] = 0;
 
-	_scrollBar->resize(_w - _scrollBarWidth, 0, _scrollBarWidth, _h);
-	scrollBarRecalc();
-	scrollToCurrent();
+	if (_scrollBar) {
+		_scrollBar->resize(_w - _scrollBarWidth, 0, _scrollBarWidth, _h);
+		scrollBarRecalc();
+		scrollToCurrent();
+	}
 }
 
 } // End of namespace GUI
