@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2002-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/gui/ListWidget.h $
- * $Id: ListWidget.h 22873 2006-06-03 13:33:39Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/gui/ListWidget.h $
+ * $Id: ListWidget.h 27024 2007-05-30 21:56:52Z fingolfin $
  */
 
 #ifndef GUI_LISTWIDGET_H
@@ -93,7 +96,7 @@ public:
 	virtual bool handleKeyUp(uint16 ascii, int keycode, int modifiers);
 	virtual void handleCommand(CommandSender *sender, uint32 cmd, uint32 data);
 
-	virtual void handleScreenChanged();
+	virtual void reflowLayout();
 
 	virtual bool wantsFocus() { return true; }
 
@@ -104,6 +107,7 @@ public:
 protected:
 	void drawWidget(bool hilite);
 
+	//! Finds the item at position (x,y). Returns -1 if there is no item there.
 	int findItem(int x, int y) const;
 	void scrollBarRecalc();
 
