@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2002-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/gui/PopUpWidget.cpp $
- * $Id: PopUpWidget.cpp 22873 2006-06-03 13:33:39Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/gui/PopUpWidget.cpp $
+ * $Id: PopUpWidget.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  */
 
 #include "common/stdafx.h"
@@ -25,7 +28,7 @@
 #include "gui/eval.h"
 #include "gui/newgui.h"
 #include "gui/PopUpWidget.h"
-#include "base/engine.h"
+#include "engines/engine.h"
 
 namespace GUI {
 
@@ -143,8 +146,7 @@ PopUpDialog::PopUpDialog(PopUpWidget *boss, int clickX, int clickY)
 	_clickX = clickX - _x;
 	_clickY = clickY - _y;
 
-	// Time the popup was opened
-	_openTime = getMillis();
+	_openTime = 0;
 }
 
 void PopUpDialog::drawDialog() {
@@ -164,6 +166,11 @@ void PopUpDialog::drawDialog() {
 	/*if (_twoColumns && (count & 1)) {
 		g_gui.fillRect(_x + 1 + _w / 2, _y + 1 + kLineHeight * (_entriesPerColumn - 1), _w / 2 - 1, kLineHeight, g_gui._bgcolor);
 	}*/
+
+	if (_openTime == 0) {
+		// Time the popup was opened
+		_openTime = getMillis();
+	}
 }
 
 void PopUpDialog::handleMouseUp(int x, int y, int button, int clickCount) {
@@ -193,8 +200,10 @@ void PopUpDialog::handleMouseMoved(int x, int y, int button) {
 	if (item >= 0 && _popUpBoss->_entries[item].name.size() == 0)
 		item = -1;
 
-	if (item == -1 && !isMouseDown())
+	if (item == -1 && !isMouseDown()) {
+		setSelection(_popUpBoss->_selectedItem);
 		return;
+	}
 
 	// ...and update the selection accordingly
 	setSelection(item);
@@ -202,6 +211,8 @@ void PopUpDialog::handleMouseMoved(int x, int y, int button) {
 
 void PopUpDialog::handleKeyDown(uint16 ascii, int keycode, int modifiers) {
 	if (keycode == 27) {	// escape
+		// Don't change the previous selection
+		setResult(-1);
 		close();
 		return;
 	}
@@ -344,8 +355,6 @@ void PopUpDialog::drawMenuEntry(int entry, bool hilite) {
 
 PopUpWidget::PopUpWidget(GuiObject *boss, const String &name, const String &label, uint labelWidth)
 	: Widget(boss, name), CommandSender(boss), _label(label), _labelWidth(labelWidth) {
-	handleScreenChanged();
-
 	_flags = WIDGET_ENABLED | WIDGET_CLEARBG | WIDGET_RETAIN_FOCUS;
 	setHints(THEME_HINT_SAVE_BACKGROUND);
 	_type = kPopUpWidget;
@@ -369,12 +378,12 @@ void PopUpWidget::handleMouseDown(int x, int y, int button, int clickCount) {
 	}
 }
 
-void PopUpWidget::handleScreenChanged() {
+void PopUpWidget::reflowLayout() {
 	_leftPadding = g_gui.evaluator()->getVar("PopUpWidget.leftPadding", 0);
 	_rightPadding = g_gui.evaluator()->getVar("PopUpWidget.rightPadding", 0);
 	_labelSpacing = g_gui.evaluator()->getVar("PopUpWidget.labelSpacing", 0);
 
-	Widget::handleScreenChanged();
+	Widget::reflowLayout();
 }
 
 void PopUpWidget::appendEntry(const String &entry, uint32 tag) {
