@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,24 +18,41 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/gui/ThemeClassic.cpp $
- * $Id: ThemeClassic.cpp 23254 2006-06-21 22:31:11Z lordhoto $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/gui/ThemeClassic.cpp $
+ * $Id: ThemeClassic.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  */
 
-#include "gui/theme.h"
+#include "gui/ThemeClassic.h"
+#include "gui/eval.h"
 
 namespace GUI {
-ThemeClassic::ThemeClassic(OSystem *system) : Theme() {
+ThemeClassic::ThemeClassic(OSystem *system, const Common::String &config, const Common::ConfigFile *cfg) : Theme() {
+	_enableBlending = true;
+	_stylefile = config;
 	_system = system;
 	_initOk = false;
+	_enabled = false;
+	_font = 0;
+	_fontName.clear();
 	memset(&_screen, 0, sizeof(_screen));
 #ifndef CT_NO_TRANSPARENCY
 	memset(&_dialog, 0, sizeof(_dialog));
 #endif
 	_font = 0;
-	
-	// Maybe change this filename
-	_configFile.loadFromFile("classic.ini");
+
+	// 'classic' is always the built in one, we force it and
+	// ignore all 'classic' named config files
+	if (config.compareToIgnoreCase("classic (builtin)") != 0) {
+		if (cfg)
+			_configFile = *cfg;
+		else
+			loadConfigFile(_stylefile);
+	}
+
+	if (_configFile.hasKey("name", "theme"))
+		_configFile.getKey("name", "theme", _stylename);
+	else
+		_stylename = _stylefile;
 }
 
 ThemeClassic::~ThemeClassic() {
@@ -45,22 +65,29 @@ bool ThemeClassic::init() {
 	if (_screen.pixels) {
 		_initOk = true;
 		clearAll();
-		_bgcolor = _system->RGBToColor(0, 0, 0);
-		_color = _system->RGBToColor(104, 104, 104);
-		_shadowcolor = _system->RGBToColor(64, 64, 64);
-		_textcolor = _system->RGBToColor(32, 160, 32);
-		_textcolorhi = _system->RGBToColor(0, 255, 0);
+		resetDrawArea();
+	}
+
+	if (isThemeLoadingRequired()) {
+		loadTheme(_defaultConfig);
+		loadTheme(_configFile, false);
+
+		setupConfig();
+	}
+
+	_bgcolor = _system->RGBToColor(_colors[kBGColor][0], _colors[kBGColor][1], _colors[kBGColor][2]);
+	_color = _system->RGBToColor(_colors[kColor][0], _colors[kColor][1], _colors[kColor][2]);
+	_shadowcolor = _system->RGBToColor(_colors[kShadowColor][0], _colors[kShadowColor][1], _colors[kShadowColor][2]);
+	_textcolor = _system->RGBToColor(_colors[kTextColor][0], _colors[kTextColor][1], _colors[kTextColor][2]);
+	_textcolorhi = _system->RGBToColor(_colors[kTextColorHi][0], _colors[kTextColorHi][1], _colors[kTextColorHi][2]);
+	if (_fontName.empty()) {
 		if (_screen.w >= 400 && _screen.h >= 300) {
 			_font = FontMan.getFontByUsage(Graphics::FontManager::kBigGUIFont);
 		} else {
 			_font = FontMan.getFontByUsage(Graphics::FontManager::kGUIFont);
 		}
-		resetDrawArea();
 	}
 
-	if (isThemeLoadingRequired())
-		loadTheme(_defaultConfig);
-
 	return true;
 }
 
@@ -74,11 +101,6 @@ void ThemeClassic::deinit() {
 
 void ThemeClassic::refresh() {
 	init();
-	_bgcolor = _system->RGBToColor(0, 0, 0);
-	_color = _system->RGBToColor(104, 104, 104);
-	_shadowcolor = _system->RGBToColor(64, 64, 64);
-	_textcolor = _system->RGBToColor(32, 160, 32);
-	_textcolorhi = _system->RGBToColor(0, 255, 0);
 	if (_enabled)
 		_system->showOverlay();
 }
@@ -105,7 +127,10 @@ void ThemeClassic::openDialog(bool topDialog) {
 		_dialog->screen.create(_screen.w, _screen.h, sizeof(OverlayColor));
 	}
 	memcpy(_dialog->screen.pixels, _screen.pixels, _screen.pitch*_screen.h);
-	blendScreenToDialog();
+	if (!_enableBlending)
+		_dialog->screen.fillRect(Common::Rect(0, 0, _screen.w, _screen.h), _bgcolor);
+	else
+		blendScreenToDialog();
 #endif
 }
 
@@ -296,7 +321,7 @@ void ThemeClassic::drawPopUpWidget(const Common::Rect &r, const Common::String &
 		_screen.drawLine(p0.x, p1.y, p1.x, p1.y, color);
 	}
 
-	if (sel != "") {
+	if (!sel.empty()) {
 		Common::Rect text(r.left + 2, r.top + 3, r.right - 4, r.top + 3 + _font->getFontHeight());
 		_font->drawString(&_screen, sel, text.left, text.top, text.width(), color, convertAligment(align), deltax, false);
 	}
@@ -615,5 +640,71 @@ void ThemeClassic::blendScreenToDialog() {
 	}
 }
 #endif
+
+void ThemeClassic::setupConfig() {
+	if (_configFile.hasSection("theme")) {
+		if (loadConfig())
+			return;
+	}
+
+	static const uint8 colors[][3] = {
+		{ 104, 104, 104 },
+		{ 64, 64, 64 },
+		{ 0, 0, 0, },
+		{ 32, 160, 32 },
+		{ 0, 255, 0 }
+	};
+
+	memcpy(_colors, colors, sizeof(colors));
+}
+
+bool ThemeClassic::loadConfig() {
+	Common::String temp;
+	_configFile.getKey("version", "theme", temp);
+	if (atoi(temp.c_str()) != THEME_VERSION) {
+		// TODO: improve this detection and handle it nicer
+		warning("Theme config uses a different version (you have: '%s', needed is: '%d')", temp.c_str(), THEME_VERSION);
+		_configFile.clear();
+
+		// force a theme reload here
+		loadTheme(_defaultConfig);
+		return false;
+	}
+
+	temp.clear();
+	_configFile.getKey("type", "theme", temp);
+	if (0 != temp.compareToIgnoreCase("classic")) {
+		warning("Theme config is not for the classic style theme");
+		_configFile.clear();
+
+		// force a theme reload here
+		loadTheme(_defaultConfig);
+		return false;
+	}
+
+	getColorFromConfig("color", _colors[kColor][0], _colors[kColor][1], _colors[kColor][2]);
+	getColorFromConfig("shadowcolor", _colors[kShadowColor][0], _colors[kShadowColor][1], _colors[kShadowColor][2]);
+	getColorFromConfig("bgcolor", _colors[kBGColor][0], _colors[kBGColor][1], _colors[kBGColor][2]);
+	getColorFromConfig("textcolor", _colors[kTextColor][0], _colors[kTextColor][1], _colors[kTextColor][2]);
+	getColorFromConfig("textcolorhi", _colors[kTextColorHi][0], _colors[kTextColorHi][1], _colors[kTextColorHi][2]);
+
+	temp.clear();
+	temp = _evaluator->getStringVar("font");
+	if (temp.empty() || 0 == temp.compareToIgnoreCase("builtin")) {
+		if (!_fontName.empty())	
+			delete _font;
+		_fontName.clear();
+	} else if (temp != _fontName) {
+		if (!_fontName.empty())
+			delete _font;
+		_font = loadFont(temp.c_str());
+		_fontName = temp;
+	}
+
+	_enableBlending = (_evaluator->getVar("blending") != 0);
+
+	return true;
+}
+
 } // end of namespace GUI
 
