@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2002-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,13 +18,15 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/gui/about.cpp $
- * $Id: about.cpp 22928 2006-06-05 12:22:51Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/gui/about.cpp $
+ * $Id: about.cpp 27392 2007-06-13 20:37:44Z fingolfin $
  */
 
 #include "common/stdafx.h"
-#include "base/engine.h"
+#include "engines/engine.h"
+#include "base/plugins.h"
 #include "base/version.h"
+#include "common/events.h"
 #include "common/system.h"
 #include "common/util.h"
 #include "gui/about.h"
@@ -50,42 +55,39 @@ enum {
 //
 // TODO: Add different font sizes (for bigger headlines)
 // TODO: Allow color change in the middle of a line...
-static const char *credits_intro[] = {
-"\\C""Copyright (C) 2002-2006 The ScummVM project",
+
+static const char *copyright_text[] = {
+"\\C""",
+"\\C""Copyright (C) 2002-2007 The ScummVM project",
 "\\C""http://www.scummvm.org",
 "\\C""",
-"\\C""LucasArts SCUMM Games (C) LucasArts",
-"\\C""Humongous SCUMM Games (C) Humongous",
-"\\C""Simon the Sorcerer (C) Adventure Soft",
-"\\C""Beneath a Steel Sky (C) Revolution",
-"\\C""Broken Sword Games (C) Revolution",
-"\\C""Flight of the Amazon Queen (C) John Passfield",
-"\\C""and Steve Stamatiadis",
-"\\C""Inherit the Earth (C) Wyrmkeep Entertainment",
-"\\C""Goblins Games (C) Coktel Vision",
-"\\C""The Legend of Kyrandia (C) Westwood Studios",
+"\\C""ScummVM is the legal property of its developers, whose names are too numerous to list here. Please refer to the COPYRIGHT file distributed with this binary.",
+"\\C""",
+};
+
+static const char *gpl_text[] = {
 "\\C""",
 "\\C""This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.",
 "\\C""",
 "\\C""This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.",
 "\\C""",
 "\\C""You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.",
-"\\C"""
+"\\C""",
 };
 
 #include "gui/credits.h"
 
 AboutDialog::AboutDialog()
 	: Dialog(10, 20, 300, 174),
-	_scrollPos(0), _scrollTime(0), _modifiers(0), _willClose(false) {
+	_scrollPos(0), _scrollTime(0), _willClose(false) {
 
 	int i;
 
 	const int screenW = g_system->getOverlayWidth();
 	const int screenH = g_system->getOverlayHeight();
 
-	xOff = g_gui.evaluator()->getVar("aboutXOff");
-	yOff = g_gui.evaluator()->getVar("aboutYOff");
+	_xOff = g_gui.evaluator()->getVar("aboutXOff");
+	_yOff = g_gui.evaluator()->getVar("aboutYOff");
 	int outerBorder = g_gui.evaluator()->getVar("aboutOuterBorder");
 
 	_w = screenW - 2 * outerBorder;
@@ -94,7 +96,7 @@ AboutDialog::AboutDialog()
 	_lineHeight = g_gui.getFontHeight() + 3;
 
 	// Heuristic to compute 'optimal' dialog width
-	int maxW = _w - 2*xOff;
+	int maxW = _w - 2*_xOff;
 	_w = 0;
 	for (i = 0; i < ARRAYSIZE(credits); i++) {
 		int tmp = g_gui.getStringWidth(credits[i] + 5);
@@ -102,7 +104,7 @@ AboutDialog::AboutDialog()
 			_w = tmp;
 		}
 	}
-	_w += 2*xOff;
+	_w += 2*_xOff;
 
 	for (i = 0; i < 1; i++)
 		_lines.push_back("");
@@ -116,14 +118,36 @@ AboutDialog::AboutDialog()
 	date += ')';
 	_lines.push_back(date);
 
-	Common::String features("\\C\\c2""Supports: ");
+	for (i = 0; i < ARRAYSIZE(copyright_text); i++)
+		addLine(copyright_text[i]);
+
+	addLine("\\C\\c1""Features compiled in:");
+	Common::String features("\\C");
 	features += gScummVMFeatures;
 	addLine(features.c_str());
 
 	_lines.push_back("");
 
-	for (i = 0; i < ARRAYSIZE(credits_intro); i++)
-		addLine(credits_intro[i]);
+	addLine("\\C\\c1""Available engines:");
+	const PluginList &plugins = PluginManager::instance().getPlugins();
+	PluginList::const_iterator iter = plugins.begin();
+	for (; iter != plugins.end(); ++iter) {
+	  Common::String str;
+	  str = "\\C";
+	  str += (**iter).getName();
+	  addLine(str.c_str());
+
+	  str = "\\C\\c2";
+	  str += (**iter).getCopyright();
+	  addLine(str.c_str());
+
+	  //addLine("");
+	}
+
+	for (i = 0; i < ARRAYSIZE(gpl_text); i++)
+		addLine(gpl_text[i]);
+
+	_lines.push_back("");
 
 	for (i = 0; i < ARRAYSIZE(credits); i++)
 		addLine(credits[i]);
@@ -158,7 +182,7 @@ void AboutDialog::addLine(const char *str) {
 		_lines.push_back(format);
 	} else {
 		Common::StringList wrappedLines;
-		g_gui.getFont().wordWrapText(str, _w - 2 * xOff, wrappedLines);
+		g_gui.getFont().wordWrapText(str, _w - 2 * _xOff, wrappedLines);
 
 		for (Common::StringList::const_iterator i = wrappedLines.begin(); i != wrappedLines.end(); ++i) {
 			_lines.push_back(format + *i);
@@ -170,7 +194,6 @@ void AboutDialog::addLine(const char *str) {
 void AboutDialog::open() {
 	_scrollTime = getMillis() + kScrollStartDelay;
 	_scrollPos = 0;
-	_modifiers = 0;
 	_willClose = false;
 
 	Dialog::open();
@@ -191,12 +214,7 @@ void AboutDialog::drawDialog() {
 	//       in the right way. Should be even faster...
 	const int firstLine = _scrollPos / _lineHeight;
 	const int lastLine = MIN((_scrollPos + _h) / _lineHeight + 1, (uint32)_lines.size());
-	// FIXME: There's some confusion here about the meaning of 'yOff'.
-	//        The setDrawArea call above makes it possible to render text
-	//        vertically from _y to _y+_h, while the line below assumes text
-	//        is drawn from _y+yOff down. This causes leftover pixels
-	//        the top while scrolling.
-	int y = _y + yOff - (_scrollPos % _lineHeight);
+	int y = _y + _yOff - (_scrollPos % _lineHeight);
 
 	for (int line = firstLine; line < lastLine; line++) {
 		const char *str = _lines[line].c_str();
@@ -248,7 +266,7 @@ void AboutDialog::drawDialog() {
 			while (*str && *str == ' ')
 				str++;
 
-		g_gui.theme()->drawText(Common::Rect(_x + xOff, y, _x + _w - xOff, y + g_gui.theme()->getFontHeight()), str, state, align, false, 0, false);
+		g_gui.theme()->drawText(Common::Rect(_x + _xOff, y, _x + _w - _xOff, y + g_gui.theme()->getFontHeight()), str, state, align, false, 0, false);
 		y += _lineHeight;
 	}
 	g_gui.theme()->resetDrawArea();
@@ -258,11 +276,13 @@ void AboutDialog::handleTickle() {
 	const uint32 t = getMillis();
 	int scrollOffset = ((int)t - (int)_scrollTime) / kScrollMillisPerPixel;
 	if (scrollOffset > 0) {
+		int modifiers = g_system->getEventManager()->getModifierState();
+
 		// Scroll faster when shift is pressed
-		if (_modifiers & OSystem::KBD_SHIFT)
+		if (modifiers & Common::KBD_SHIFT)
 			scrollOffset *= 4;
 		// Reverse scrolling when alt is pressed
-		if (_modifiers & OSystem::KBD_ALT)
+		if (modifiers & Common::KBD_ALT)
 			scrollOffset *= -1;
 		_scrollPos += scrollOffset;
 		_scrollTime = t;
@@ -283,24 +303,22 @@ void AboutDialog::handleMouseUp(int x, int y, int button, int clickCount) {
 }
 
 void AboutDialog::handleKeyDown(uint16 ascii, int keycode, int modifiers) {
-	_modifiers = modifiers;
 	if (ascii)
 		_willClose = true;
 }
 
 void AboutDialog::handleKeyUp(uint16 ascii, int keycode, int modifiers) {
-	_modifiers = modifiers;
 	if (ascii && _willClose)
 		close();
 }
 
-void AboutDialog::handleScreenChanged() {
-	Dialog::handleScreenChanged();
+void AboutDialog::reflowLayout() {
+	Dialog::reflowLayout();
 	const int screenW = g_system->getOverlayWidth();
 	const int screenH = g_system->getOverlayHeight();
 
-	xOff = g_gui.evaluator()->getVar("aboutXOff");
-	yOff = g_gui.evaluator()->getVar("aboutYOff");
+	_xOff = g_gui.evaluator()->getVar("aboutXOff");
+	_yOff = g_gui.evaluator()->getVar("aboutYOff");
 	int outerBorder = g_gui.evaluator()->getVar("aboutOuterBorder");
 
 	_w = screenW - 2 * outerBorder;
@@ -309,7 +327,7 @@ void AboutDialog::handleScreenChanged() {
 	_lineHeight = g_gui.getFontHeight() + 3;
 
 	// Heuristic to compute 'optimal' dialog width
-	int maxW = _w - 2*xOff;
+	int maxW = _w - 2*_xOff;
 	_w = 0;
 	for (int i = 0; i < ARRAYSIZE(credits); i++) {
 		int tmp = g_gui.getStringWidth(credits[i] + 5);
@@ -317,7 +335,7 @@ void AboutDialog::handleScreenChanged() {
 			_w = tmp;
 		}
 	}
-	_w += 2*xOff;
+	_w += 2*_xOff;
 
 	_lineHeight = g_gui.getFontHeight() + 3;
 
