@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2002-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/gui/about.h $
- * $Id: about.h 22017 2006-04-19 01:05:28Z lordhoto $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/gui/about.h $
+ * $Id: about.h 27024 2007-05-30 21:56:52Z fingolfin $
  */
 
 #ifndef ABOUT_DIALOG_H
@@ -35,10 +38,9 @@ protected:
 	uint32		_scrollTime;
 	StringList	_lines;
 	uint32		_lineHeight;
-	byte		_modifiers;
 	bool		_willClose;
 
-	int xOff, yOff;
+	int _xOff, _yOff;
 
 	void addLine(const char *str);
 
@@ -53,7 +55,7 @@ public:
 	void handleKeyDown(uint16 ascii, int keycode, int modifiers);
 	void handleKeyUp(uint16 ascii, int keycode, int modifiers);
 
-	void handleScreenChanged();
+	void reflowLayout();
 };
 
 } // End of namespace GUI
