@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2002-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/gui/browser.cpp $
- * $Id: browser.cpp 22670 2006-05-27 05:46:04Z sev $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/gui/browser.cpp $
+ * $Id: browser.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  */
 
 #include "common/stdafx.h"
@@ -24,11 +27,10 @@
 #include "gui/newgui.h"
 #include "gui/ListWidget.h"
 
-#include "backends/fs/fs.h"
-
 #include "common/config-manager.h"
+#include "common/fs.h"
 #include "common/system.h"
-#include "common/func.h"
+#include "common/algorithm.h"
 
 namespace GUI {
 
@@ -67,7 +69,7 @@ int BrowserDialog::runModal() {
 		g_system->setFeatureState(OSystem::kFeatureFullscreenMode, false);
 
 	// Temporarily show the real mouse
-	ShowCursor();
+	CGDisplayShowCursor(kCGDirectMainDisplay);
 
 	err = NavGetDefaultDialogCreationOptions(&options);
 	assert(err == noErr);
@@ -86,7 +88,7 @@ int BrowserDialog::runModal() {
 	err = NavDialogRun(dialogRef);
 	assert(err == noErr);
 
-	HideCursor();
+	CGDisplayHideCursor(kCGDirectMainDisplay);
 
 	result = NavDialogGetUserAction(dialogRef);
 
@@ -222,14 +224,18 @@ void BrowserDialog::handleCommand(CommandSender *sender, uint32 cmd, uint32 data
 void BrowserDialog::updateListing() {
 	// Update the path display
 	_currentPath->setLabel(_node.path());
-	
+
 	// We memorize the last visited path.
 	ConfMan.set("browser_lastpath", _node.path());
 
 	// Read in the data from the file system
-	_node.listDir(_nodeContent, _isDirBrowser ? FilesystemNode::kListDirectoriesOnly
-	                                          : FilesystemNode::kListAll);
-	Common::sort(_nodeContent.begin(), _nodeContent.end());
+	FilesystemNode::ListMode listMode = _isDirBrowser ? FilesystemNode::kListDirectoriesOnly
+	                                                  : FilesystemNode::kListAll;
+	if (!_node.listDir(_nodeContent, listMode)) {
+		_nodeContent.clear();
+	} else {
+		Common::sort(_nodeContent.begin(), _nodeContent.end());
+	}
 
 	// Populate the ListWidget
 	Common::StringList list;
