@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2002-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/gui/browser.h $
- * $Id: browser.h 20518 2006-02-11 10:08:56Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/gui/browser.h $
+ * $Id: browser.h 27024 2007-05-30 21:56:52Z fingolfin $
  */
 
 #ifndef BROWSER_DIALOG_H
@@ -24,7 +27,7 @@
 
 #include "gui/dialog.h"
 #include "common/str.h"
-#include "backends/fs/fs.h"
+#include "common/fs.h"
 
 #ifdef MACOSX
 #include <Carbon/Carbon.h>
