@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2002-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/gui/console.cpp $
- * $Id: console.cpp 23481 2006-07-10 15:45:34Z eriktorbjorn $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/gui/console.cpp $
+ * $Id: console.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  */
 
 #include "common/stdafx.h"
@@ -24,9 +27,10 @@
 #include "gui/ScrollBarWidget.h"
 #include "gui/eval.h"
 
-#include "base/engine.h"
+#include "engines/engine.h"
 #include "base/version.h"
 
+#include "common/events.h"
 #include "common/system.h"
 
 #include "graphics/font.h"
@@ -123,7 +127,7 @@ void ConsoleDialog::init() {
 
 	_pageWidth = (_w - scrollBarWidth - 2 - _leftPadding - _topPadding - scrollBarWidth) / kConsoleCharWidth;
 	_linesPerPage = (_h - 2 - _topPadding - _bottomPadding) / kConsoleLineHeight;
-	_linesInBuffer = kBufferSize / kLineWidth;
+	_linesInBuffer = kBufferSize / kCharsPerLine;
 }
 
 void ConsoleDialog::slideUpAndClose() {
@@ -157,13 +161,6 @@ void ConsoleDialog::open() {
 	_slideTime = g_system->getMillis();
 	_slideMode = kDownSlideMode;
 
-	// The screen may have changed since the console was created. We have
-	// to make sure things are properly adjusted, or we may get garbage in
-	// the console, or even outright crashes. This means _scrollLine is not
-	// preserved, but that's a tiny sacrifice.
-
-	handleScreenChanged();
-
 	Dialog::open();
 	if (_promptStartPos == -1) {
 		print(PROMPT);
@@ -192,7 +189,7 @@ void ConsoleDialog::drawLine(int line, bool restoreBg) {
 	int x = _x + 1 + _leftPadding;
 	int start = _scrollLine - _linesPerPage + 1;
 	int y = _y + 2 + _topPadding;
-	int limit = MIN(_pageWidth, (int)kLineWidth);
+	int limit = MIN(_pageWidth, (int)kCharsPerLine);
 
 	y += line * kConsoleLineHeight;
 
@@ -205,24 +202,24 @@ void ConsoleDialog::drawLine(int line, bool restoreBg) {
 	for (int column = 0; column < limit; column++) {
 #if 0
 		int l = (start + line) % _linesInBuffer;
-		byte c = buffer(l * kLineWidth + column);
+		byte c = buffer(l * kCharsPerLine + column);
 #else
-		byte c = buffer((start + line) * kLineWidth + column);
+		byte c = buffer((start + line) * kCharsPerLine + column);
 #endif
 		g_gui.theme()->drawChar(Common::Rect(x, y, x+kConsoleCharWidth, y+kConsoleLineHeight), c, _font);
 		x += kConsoleCharWidth;
 	}
 }
 
-void ConsoleDialog::handleScreenChanged() {
+void ConsoleDialog::reflowLayout() {
 	init();
 
-	_scrollLine = _promptEndPos / kLineWidth;
+	_scrollLine = _promptEndPos / kCharsPerLine;
 	if (_scrollLine < _linesPerPage - 1)
 		_scrollLine = _linesPerPage - 1;
 	updateScrollBuffer();
 
-	Dialog::handleScreenChanged();
+	Dialog::reflowLayout();
 	draw();
 }
 
@@ -354,7 +351,7 @@ void ConsoleDialog::handleKeyDown(uint16 ascii, int keycode, int modifiers) {
 		drawLine(pos2line(_currentPos));
 		break;
 	case 256 + 24:	// pageup
-		if (modifiers == OSystem::KBD_SHIFT) {
+		if (modifiers == Common::KBD_SHIFT) {
 			_scrollLine -= _linesPerPage - 1;
 			if (_scrollLine < _firstLineInBuffer + _linesPerPage - 1)
 				_scrollLine = _firstLineInBuffer + _linesPerPage - 1;
@@ -363,10 +360,10 @@ void ConsoleDialog::handleKeyDown(uint16 ascii, int keycode, int modifiers) {
 		}
 		break;
 	case 256 + 25:	// pagedown
-		if (modifiers == OSystem::KBD_SHIFT) {
+		if (modifiers == Common::KBD_SHIFT) {
 			_scrollLine += _linesPerPage - 1;
-			if (_scrollLine > _promptEndPos / kLineWidth) {
-				_scrollLine = _promptEndPos / kLineWidth;
+			if (_scrollLine > _promptEndPos / kCharsPerLine) {
+				_scrollLine = _promptEndPos / kCharsPerLine;
 				if (_scrollLine < _firstLineInBuffer + _linesPerPage - 1)
 					_scrollLine = _firstLineInBuffer + _linesPerPage - 1;
 			}
@@ -375,7 +372,7 @@ void ConsoleDialog::handleKeyDown(uint16 ascii, int keycode, int modifiers) {
 		}
 		break;
 	case 256 + 22:	// home
-		if (modifiers == OSystem::KBD_SHIFT) {
+		if (modifiers == Common::KBD_SHIFT) {
 			_scrollLine = _firstLineInBuffer + _linesPerPage - 1;
 			updateScrollBuffer();
 		} else {
@@ -384,8 +381,8 @@ void ConsoleDialog::handleKeyDown(uint16 ascii, int keycode, int modifiers) {
 		draw();
 		break;
 	case 256 + 23:	// end
-		if (modifiers == OSystem::KBD_SHIFT) {
-			_scrollLine = _promptEndPos / kLineWidth;
+		if (modifiers == Common::KBD_SHIFT) {
+			_scrollLine = _promptEndPos / kCharsPerLine;
 			if (_scrollLine < _linesPerPage - 1)
 				_scrollLine = _linesPerPage - 1;
 			updateScrollBuffer();
@@ -413,7 +410,7 @@ void ConsoleDialog::handleKeyDown(uint16 ascii, int keycode, int modifiers) {
 	default:
 		if (ascii == '~' || ascii == '#') {
 			slideUpAndClose();
-		} else if (modifiers == OSystem::KBD_CTRL) {
+		} else if (modifiers == Common::KBD_CTRL) {
 			specialKeys(keycode);
 		} else if ((ascii >= 32 && ascii <= 127) || (ascii >= 160 && ascii <= 255)) {
 			for (i = _promptEndPos - 1; i >= _currentPos; i--)
@@ -559,10 +556,10 @@ void ConsoleDialog::historyScroll(int direction) {
 }
 
 void ConsoleDialog::nextLine() {
-	int line = _currentPos / kLineWidth;
+	int line = _currentPos / kCharsPerLine;
 	if (line == _scrollLine)
 		_scrollLine++;
-	_currentPos = (line + 1) * kLineWidth;
+	_currentPos = (line + 1) * kCharsPerLine;
 
 	updateScrollBuffer();
 }
@@ -572,12 +569,12 @@ void ConsoleDialog::nextLine() {
 // a new line is added
 void ConsoleDialog::updateScrollBuffer() {
 	int lastchar = MAX(_promptEndPos, _currentPos);
-	int line = lastchar / kLineWidth;
+	int line = lastchar / kCharsPerLine;
 	int numlines = (line < _linesInBuffer) ? line + 1 : _linesInBuffer;
 	int firstline = line - numlines + 1;
 	if (firstline > _firstLineInBuffer) {
 		// clear old line from buffer
-		for (int i = lastchar; i < (line+1) * kLineWidth; ++i)
+		for (int i = lastchar; i < (line+1) * kCharsPerLine; ++i)
 			buffer(i) = ' ';
 		_firstLineInBuffer = firstline;
 	}
@@ -629,7 +626,7 @@ void ConsoleDialog::putcharIntern(int c) {
 	else {
 		buffer(_currentPos) = (char)c;
 		_currentPos++;
-		if ((_scrollLine + 1) * kLineWidth == _currentPos) {
+		if ((_scrollLine + 1) * kCharsPerLine == _currentPos) {
 			_scrollLine++;
 			updateScrollBuffer();
 		}
@@ -648,7 +645,7 @@ void ConsoleDialog::print(const char *str) {
 
 void ConsoleDialog::drawCaret(bool erase) {
 	// TODO: use code from EditableWidget::drawCaret here
-	int line = _currentPos / kLineWidth;
+	int line = _currentPos / kCharsPerLine;
 	int displayLine = line - _scrollLine + _linesPerPage - 1;
 
 	// Only draw caret if visible
@@ -657,7 +654,7 @@ void ConsoleDialog::drawCaret(bool erase) {
 		return;
 	}
 
-	int x = _x + 1 + _leftPadding + (_currentPos % kLineWidth) * kConsoleCharWidth;
+	int x = _x + 1 + _leftPadding + (_currentPos % kCharsPerLine) * kConsoleCharWidth;
 	int y = _y + _topPadding + displayLine * kConsoleLineHeight;
 
 	_caretVisible = !erase;
@@ -665,7 +662,7 @@ void ConsoleDialog::drawCaret(bool erase) {
 }
 
 void ConsoleDialog::scrollToCurrent() {
-	int line = _promptEndPos / kLineWidth;
+	int line = _promptEndPos / kCharsPerLine;
 
 	if (line + _linesPerPage <= _scrollLine) {
 		// TODO - this should only occur for loong edit lines, though
