@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2002-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/gui/console.h $
- * $Id: console.h 22670 2006-05-27 05:46:04Z sev $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/gui/console.h $
+ * $Id: console.h 27024 2007-05-30 21:56:52Z fingolfin $
  */
 
 #ifndef CONSOLE_DIALOG_H
@@ -25,26 +28,56 @@
 #include "gui/dialog.h"
 #include "gui/newgui.h"
 
-#include <stdarg.h>
-
 namespace GUI {
 
-enum {
-	kBufferSize	= 32768,
-	kLineWidth = 128,
-	kLineBufferSize = 256,
-
-	kHistorySize = 20
-};
-
 class ScrollBarWidget;
 
+/*
+ FIXME #1: The console dialog code has some fundamental problems. 
+ First of, note the conflict between the (constant) value kCharsPerLine, and the
+ (variable) value _pageWidth. Look a bit at the code get familiar with them,
+ then return...
+ Now, why don't we just drop kCharsPerLine? Because of the problem of resizing!
+ When the user changes the scaler, the console will get resized. If the dialog
+ becomes smaller because of this, we may have to rewrap text. If the resolution
+ is then increased again, we'd end up with garbled content.
+ 
+ One can now either ignore this problem (and modify our code accordingly to 
+ implement this simple rewrapping -- we currently don't do that at all!).
+ 
+ Or, one can go and implement a more complete console, by replacing the
+ _buffer by a real line buffer -- an arrach of char* pointers.
+ This will allow one to implement resizing perfectly, but has the drawback
+ of making things like scrolling, drawing etc. more complicated.
+ 
+ Either way, the current situation is bad, and we should resolve it one way
+ or the other (and if you can think of a thirds, feel free to suggest it).
+ 
+ 
+ 
+ FIXME #2: Another problem is that apparently _pageWidth isn't computed quite
+ correctly. The current line ends well before reaching the right side of the
+ console dialog. That's irritating and should be fixed.
+ 
+ 
+ FIXME #3: The scroll bar is not shown initially, but the area it would 
+ occupy is not used for anything else. As a result, the gap described above
+ becomes even wider and thus even more irritating.
+*/
 class ConsoleDialog : public Dialog {
 public:
 	typedef bool (*InputCallbackProc)(ConsoleDialog *console, const char *input, void *refCon);
 	typedef bool (*CompletionCallbackProc)(ConsoleDialog* console, const char *input, char*& completion, void *refCon);
 
 protected:
+	enum {
+		kBufferSize	= 32768,
+		kCharsPerLine = 128,
+		kLineBufferSize = 256,
+	
+		kHistorySize = 20
+	};
+
 	const Graphics::Font *_font;
 
 	char	_buffer[kBufferSize];
@@ -105,7 +138,7 @@ public:
 	void drawDialog();
 
 	void handleTickle();
-	void handleScreenChanged();
+	void reflowLayout();
 	void handleMouseWheel(int x, int y, int direction);
 	void handleKeyDown(uint16 ascii, int keycode, int modifiers);
 	void handleCommand(CommandSender *sender, uint32 cmd, uint32 data);
@@ -123,6 +156,10 @@ public:
 		_completionCallbackProc = proc;
 		_completionCallbackRefCon = refCon;
 	}
+	
+	int getCharsPerLine() {
+		return _pageWidth;
+	}
 
 protected:
 	inline char &buffer(int idx) {
@@ -131,7 +168,7 @@ protected:
 
 	void init();
 
-	int pos2line(int pos) { return (pos - (_scrollLine - _linesPerPage + 1) * kLineWidth) / kLineWidth; }
+	int pos2line(int pos) { return (pos - (_scrollLine - _linesPerPage + 1) * kCharsPerLine) / kCharsPerLine; }
 
 	void drawLine(int line, bool restoreBg = true);
 	void drawCaret(bool erase);
