@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2005-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/gui/editable.cpp $
- * $Id: editable.cpp 23211 2006-06-21 00:19:05Z eriktorbjorn $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/gui/editable.cpp $
+ * $Id: editable.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  */
 
 #include "common/stdafx.h"
@@ -50,15 +53,19 @@ void EditableWidget::init() {
 EditableWidget::~EditableWidget() {
 }
 
+void EditableWidget::reflowLayout() {
+	Widget::reflowLayout();
+
+	_editScrollOffset = g_gui.getStringWidth(_editString, _font) - getEditRect().width();
+	if (_editScrollOffset < 0)
+		_editScrollOffset = 0;
+}
+
 void EditableWidget::setEditString(const String &str) {
 	// TODO: We probably should filter the input string here,
 	// e.g. using tryInsertChar.
 	_editString = str;
 	_caretPos = _editString.size();
-
-	_editScrollOffset = g_gui.getStringWidth(_editString, _font) - getEditRect().width();
-	if (_editScrollOffset < 0)
-		_editScrollOffset = 0;
 }
 
 bool EditableWidget::tryInsertChar(byte c, int pos) {
@@ -106,9 +113,11 @@ bool EditableWidget::handleKeyDown(uint16 ascii, int keycode, int modifiers) {
 		forcecaret = true;
 		break;
 	case 127:	// delete
-		_editString.deleteChar(_caretPos);
+		if (_caretPos < (int)_editString.size()) {
+			_editString.deleteChar(_caretPos);
+			dirty = true;
+		}
 		forcecaret = true;
-		dirty = true;
 		break;
 	case 256 + 20:	// left arrow
 		if (_caretPos > 0) {
