@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2005-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/gui/editable.h $
- * $Id: editable.h 23209 2006-06-20 22:52:01Z eriktorbjorn $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/gui/editable.h $
+ * $Id: editable.h 27024 2007-05-30 21:56:52Z fingolfin $
  */
 
 #ifndef GUI_EDITABLE_H
@@ -62,6 +65,8 @@ public:
 	virtual void handleTickle();
 	virtual bool handleKeyDown(uint16 ascii, int keycode, int modifiers);
 
+	virtual void reflowLayout();
+
 protected:
 	virtual void startEditMode() = 0;
 	virtual void endEditMode() = 0;
