@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/gui/eval.cpp $
- * $Id: eval.cpp 22964 2006-06-06 23:57:14Z sev $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/gui/eval.cpp $
+ * $Id: eval.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -192,10 +195,10 @@ void Eval::getToken() {
 
 	if (_input[_pos] == '"') {
 		_pos++;
-		while(_input[_pos] != '"' && _input[_pos] != '\n')
+		while (_input[_pos] != '"' && _input[_pos] != '\n')
 			*temp++ = _input[_pos++];
 
-		if(_input[_pos] == '\n')
+		if (_input[_pos] == '\n')
 			exprError(eMissingQuote);
 
 		_pos++;
@@ -302,22 +305,22 @@ int Eval::getBuiltinVar(const char *s) {
 	return EVAL_UNDEF_VAR;
 }
 
-int Eval::getVar_(const char *s, bool includeAliases) {
+int Eval::getVar_(const Common::String &s, bool includeAliases) {
 	int val;
 
-	val = getBuiltinVar(s);
+	val = getBuiltinVar(s.c_str());
 
 	if (val != EVAL_UNDEF_VAR)
 		return val;
 
-	const char *var = s;
+	const Common::String *var = &s;
 	if (includeAliases) {
 		AliasesMap::const_iterator itera = _aliases.find(s);
 		if (itera != _aliases.end())
-			var = itera->_value.c_str();
+			var = &(itera->_value);
 	}
 
-	VariablesMap::const_iterator iterv = _vars.find(var);
+	VariablesMap::const_iterator iterv = _vars.find(*var);
 	if (iterv != _vars.end())
 		return iterv->_value;
 
