@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/gui/eval.h $
- * $Id: eval.h 22964 2006-06-06 23:57:14Z sev $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/gui/eval.h $
+ * $Id: eval.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -26,6 +29,7 @@
 #include "common/stdafx.h"
 #include "common/str.h"
 #include "common/hashmap.h"
+#include "common/hash-str.h"
 
 namespace GUI {
 
@@ -47,20 +51,17 @@ public:
 
 	void setParent(const String &name);
 
-	void setVar(const String &name, int val) { _vars[name.c_str()] = val; }
-	void setStringVar(const String &name, const String &val) { _strings[name.c_str()] = val; }
-	void setAlias(const char *name, const String &val) { _aliases[name] = val; }
+	void setVar(const String &name, int val) { _vars[name] = val; }
+	void setStringVar(const String &name, const String &val) { _strings[name] = val; }
+	void setAlias(const Common::String &name, const String &val) { _aliases[name] = val; }
 
-	int getVar(const char *s) { return getVar_(s); }
-	int getVar(const char *s, int def) {
+	int getVar(const Common::String &s) { return getVar_(s); }
+	int getVar(const Common::String &s, int def) {
 		int val = getVar_(s);
 		return (val == EVAL_UNDEF_VAR) ? def : val;
 	}
 
-	int getVar(const String &s) { return getVar(s.c_str()); }
-	int getVar(const String &s, int def) { return getVar(s.c_str(), def); }
-
-	const String &getStringVar(const char *name) { return _strings[name]; }
+	const String &getStringVar(const Common::String &name) { return _strings[name]; }
 
 	uint getNumVars() { return _vars.size(); }
 
@@ -68,14 +69,9 @@ public:
 
 	char *lastToken() { return _token; }
 
-	struct CharStar_EqualTo {
-		bool operator()(const char *x, const char *y) const { return strcmp(x, y) == 0; }
-	};
-
-	//typedef HashMap<String, int> VariablesMap;
-	typedef HashMap<const char *, int, Common::Hash<const char *>, CharStar_EqualTo> VariablesMap;
-	typedef HashMap<const char *, String, Common::Hash<const char *>, CharStar_EqualTo> AliasesMap;
-	typedef HashMap<const char *, String, Common::Hash<const char *>, CharStar_EqualTo> StringsMap;
+	typedef HashMap<String, int, Common::CaseSensitiveString_Hash, Common::CaseSensitiveString_EqualTo> VariablesMap;
+	typedef HashMap<String, String, Common::CaseSensitiveString_Hash, Common::CaseSensitiveString_EqualTo> AliasesMap;
+	typedef HashMap<String, String, Common::CaseSensitiveString_Hash, Common::CaseSensitiveString_EqualTo> StringsMap;
 
 private:
 	enum TokenTypes {
@@ -105,7 +101,7 @@ private:
 	void arith(char op, int *r, int *h);
 	void unary(char op, int *r);
 	void exprError(EvalErrors error);
-	int getVar_(const char *s, bool includeAliases = true);
+	int getVar_(const Common::String &s, bool includeAliases = true);
 	int getBuiltinVar(const char *s);
 	void loadConstants();
 
