@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2002-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,20 +18,20 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/gui/launcher.cpp $
- * $Id: launcher.cpp 23961 2006-09-21 11:51:06Z sev $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/gui/launcher.cpp $
+ * $Id: launcher.cpp 27420 2007-06-15 16:34:20Z fingolfin $
  */
 
 #include "common/stdafx.h"
 
-#include "backends/fs/fs.h"
-
-#include "base/engine.h"
+#include "engines/engine.h"
 #include "base/game.h"
 #include "base/plugins.h"
 #include "base/version.h"
 
 #include "common/config-manager.h"
+#include "common/events.h"
+#include "common/fs.h"
 #include "common/util.h"
 #include "common/system.h"
 
@@ -37,6 +40,7 @@
 #include "gui/chooser.h"
 #include "gui/eval.h"
 #include "gui/launcher.h"
+#include "gui/massadd.h"
 #include "gui/message.h"
 #include "gui/newgui.h"
 #include "gui/options.h"
@@ -48,6 +52,7 @@
 #include "sound/mididrv.h"
 
 
+
 using Common::ConfigManager;
 
 namespace GUI {
@@ -117,7 +122,7 @@ class EditGameDialog : public OptionsDialog {
 public:
 	EditGameDialog(const String &domain, const String &desc);
 
-	virtual void handleScreenChanged();	
+	virtual void reflowLayout();	
 
 	void open();
 	void close();
@@ -213,15 +218,9 @@ EditGameDialog::EditGameDialog(const String &domain, const String &desc)
 	//
 	// 5) The volume tab
 	//
-	int volControlPos = g_gui.evaluator()->getVar("volumeControlsInAudio", true);
+	tab->addTab("Volume");
 
-	if (!volControlPos) {
-		tab->addTab("Volume");
-
-		_globalVolumeOverride = new CheckboxWidget(tab, "gameoptions_volumeCheckbox", "Override global volume settings", kCmdGlobalVolumeOverride, 0);
-	} else {
-		_globalVolumeOverride = NULL;
-	}
+	_globalVolumeOverride = new CheckboxWidget(tab, "gameoptions_volumeCheckbox", "Override global volume settings", kCmdGlobalVolumeOverride, 0);
 
 	addVolumeControls(tab, "gameoptions_");
 
@@ -268,8 +267,8 @@ EditGameDialog::EditGameDialog(const String &domain, const String &desc)
 	new ButtonWidget(this, "gameoptions_ok", "OK", kOKCmd, 0);
 }
 
-void EditGameDialog::handleScreenChanged() {
-	OptionsDialog::handleScreenChanged();
+void EditGameDialog::reflowLayout() {
+	OptionsDialog::reflowLayout();
 
 	int labelWidth = g_gui.evaluator()->getVar("gameOptionsLabelWidth");
 
@@ -287,26 +286,29 @@ void EditGameDialog::open() {
 
 	// En-/disable dialog items depending on whether overrides are active or not.
 
-	e = ConfMan.hasKey("fullscreen", _domain) ||
+	e = ConfMan.hasKey("gfx_mode", _domain) ||
+		ConfMan.hasKey("render_mode", _domain) ||
+		ConfMan.hasKey("fullscreen", _domain) ||
 		ConfMan.hasKey("aspect_ratio", _domain);
 	_globalGraphicsOverride->setState(e);
 
 	e = ConfMan.hasKey("music_driver", _domain) ||
+		ConfMan.hasKey("output_rate", _domain) ||
 		ConfMan.hasKey("subtitles", _domain) ||
 		ConfMan.hasKey("talkspeed", _domain);
 	_globalAudioOverride->setState(e);
 
-	e = ConfMan.hasKey("multi_midi", _domain) ||
-		ConfMan.hasKey("native_mt32", _domain)||
-		ConfMan.hasKey("enable_gs", _domain);
-	_globalMIDIOverride->setState(e);
-
 	e = ConfMan.hasKey("music_volume", _domain) ||
 		ConfMan.hasKey("sfx_volume", _domain) ||
 		ConfMan.hasKey("speech_volume", _domain);
+	_globalVolumeOverride->setState(e);
 
-	if (_globalVolumeOverride)
-		_globalVolumeOverride->setState(e);
+	e = ConfMan.hasKey("soundfont", _domain) ||
+		ConfMan.hasKey("multi_midi", _domain) ||
+		ConfMan.hasKey("native_mt32", _domain) ||
+		ConfMan.hasKey("enable_gs", _domain) ||
+		ConfMan.hasKey("midi_gain", _domain);
+	_globalMIDIOverride->setState(e);
 
 	// TODO: game path
 
@@ -374,7 +376,7 @@ void EditGameDialog::handleCommand(CommandSender *sender, uint32 cmd, uint32 dat
 	case kCmdGlobalAudioOverride:
 		setAudioSettingsState(data != 0);
 		setSubtitleSettingsState(data != 0);
-		if(_globalVolumeOverride == NULL)
+		if (_globalVolumeOverride == NULL)
 			setVolumeSettingsState(data != 0);
 		draw();
 		break;
@@ -393,6 +395,12 @@ void EditGameDialog::handleCommand(CommandSender *sender, uint32 cmd, uint32 dat
 			// User made this choice...
 			FilesystemNode file(browser.getResult());
 			_soundFont->setLabel(file.path());
+
+			if (!file.path().empty() && (file.path() != "None"))
+				_soundFontClearButton->setEnabled(true);
+			else
+				_soundFontClearButton->setEnabled(false);
+
 			draw();
 		}
 		break;
@@ -475,12 +483,10 @@ LauncherDialog::LauncherDialog()
 
 #ifndef DISABLE_FANCY_THEMES
 	_logo = 0;
-	if (g_gui.evaluator()->getVar("launcher_logo.visible") == 1) {
+	if (g_gui.evaluator()->getVar("launcher_logo.visible") == 1 && g_gui.theme()->supportsImages()) {
 		_logo = new GraphicsWidget(this, "launcher_logo");
 		_logo->useThemeTransparency(true);
-		ThemeNew *th = (ThemeNew *)g_gui.theme();
-
-		_logo->setGfx(th->getImageSurface(th->kThemeLogo));
+		_logo->setGfx(g_gui.theme()->getImageSurface(Theme::kImageLogo));
 
 		new StaticTextWidget(this, "launcher_version", gScummVMVersionDate);
 	} else
@@ -497,7 +503,8 @@ LauncherDialog::LauncherDialog()
 			new ButtonWidget(this, "launcher_start_button", "Start", kStartCmd, 'S');
 
 	// Above the lowest button rows: two more buttons (directly below the list box)
-	new ButtonWidget(this, "launcher_addGame_button", "Add Game...", kAddGameCmd, 'A');
+	_addButton = 
+		new ButtonWidget(this, "launcher_addGame_button", "Add Game...", kAddGameCmd, 'A');
 	_editButton =
 		new ButtonWidget(this, "launcher_editGame_button", "Edit Game...", kEditGameCmd, 'E');
 	_removeButton =
@@ -541,6 +548,16 @@ LauncherDialog::~LauncherDialog() {
 	delete _browser;
 }
 
+void LauncherDialog::open() {
+	// Clear the active domain, in case we return to the dialog from a
+	// failure to launch a game. Otherwise, pressing ESC will attempt to
+	// re-launch the same game again.
+	ConfMan.setActiveDomain("");
+	Dialog::open();
+	
+	updateButtons();
+}
+
 void LauncherDialog::close() {
 	// Save last selection
 	const int sel = _list->getSelected();
@@ -568,9 +585,11 @@ void LauncherDialog::updateListing() {
 			gameid = iter->_key;
 		if (description.empty()) {
 			GameDescriptor g = Base::findGame(gameid);
-			if (!g.description.empty())
-				description = g.description;
+			if (g.contains("description"))
+				description = g.description();
 		}
+		if (description.empty())
+			description = "Unknown (target " + iter->_key + ", gameid " + gameid + ")";
 
 		if (!gameid.empty() && !description.empty()) {
 			// Insert the game into the launcher list
@@ -587,10 +606,30 @@ void LauncherDialog::updateListing() {
 	_list->setList(l);
 	if (oldSel < (int)l.size())
 		_list->setSelected(oldSel);	// Restore the old selection
+	else if (oldSel != -1)
+		// Select the last entry if the list has been reduced
+		_list->setSelected(_list->getList().size() - 1);
 	updateButtons();
 }
 
 void LauncherDialog::addGame() {
+	int modifiers = g_system->getEventManager()->getModifierState();
+	bool massAdd = (modifiers & Common::KBD_SHIFT) != 0;
+	
+	if (massAdd) {
+		MessageDialog alert("Do you really want to run the mass game detector? "
+							"This could potentially add a huge number of games.", "Yes", "No");
+		if (alert.runModal() == GUI::kMessageOK && _browser->runModal() > 0) {
+			MassAddDialog massAddDlg(_browser->getResult());
+			massAddDlg.runModal();
+
+			// Update the ListWidget and force a redraw
+			updateListing();
+			draw();
+		}
+		return;
+	}
+
 	// Allow user to add a new game to the list.
 	// 1) show a dir selection dialog which lets the user pick the directory
 	//    the game data resides in.
@@ -613,7 +652,7 @@ void LauncherDialog::addGame() {
 
 		// ...so let's determine a list of candidates, games that
 		// could be contained in the specified directory.
-		DetectedGameList candidates(PluginManager::instance().detectGames(files));
+		GameList candidates(PluginManager::instance().detectGames(files));
 
 		int idx;
 		if (candidates.empty()) {
@@ -628,63 +667,28 @@ void LauncherDialog::addGame() {
 			// Display the candidates to the user and let her/him pick one
 			StringList list;
 			for (idx = 0; idx < (int)candidates.size(); idx++)
-				list.push_back(candidates[idx].description);
+				list.push_back(candidates[idx].description());
 
 			ChooserDialog dialog("Pick the game:");
 			dialog.setList(list);
 			idx = dialog.runModal();
 		}
 		if (0 <= idx && idx < (int)candidates.size()) {
-			DetectedGame result = candidates[idx];
+			GameDescriptor result = candidates[idx];
 
-			// The auto detector or the user made a choice.
-			// Pick a domain name which does not yet exist (after all, we
-			// are *adding* a game to the config, not replacing).
-			String domain(result.gameid);
-			if (ConfMan.hasGameDomain(domain)) {
-				int suffixN = 1;
-				char suffix[16];
+			// TODO: Change the detectors to set "path" !
+			result["path"] = dir.path();
 
-				while (ConfMan.hasGameDomain(domain)) {
-					snprintf(suffix, 16, "-%d", suffixN);
-					domain = result.gameid + suffix;
-					suffixN++;
-				}
-			}
-
-			// Add the name domain
-			ConfMan.addGameDomain(domain);
-			
-			// TODO: Setting the description field here has the drawback
-			// that the user does never notice when we upgrade our descriptions.
-			// It might be nice ot leave this field empty, and only set it to
-			// a value when the user edits the description string. 
-			// However, at this point, that's impractical. Once we have a method
-			// to query all backends for the proper & full description of a given
-			// game target, we can change this (currently, you can only query
-			// for the generic gameid description; it's not possible to obtain
-			// a description which contains extended information like language, etc.).
-			ConfMan.set("description", result.description, domain);
-
-			ConfMan.set("gameid", result.gameid, domain);
-			ConfMan.set("path", dir.path(), domain);
-
-			// Set language if specified
-			if (result.language != Common::UNK_LANG)
-				ConfMan.set("language", Common::getLanguageCode(result.language), domain);
-
-			// Set platform if specified
-			if (result.platform != Common::kPlatformUnknown)
-				ConfMan.set("platform", Common::getPlatformCode(result.platform), domain);
+			Common::String domain = addGameToConf(result);
 
 			// Display edit dialog for the new entry
-			EditGameDialog editDialog(domain, result.description);
+			EditGameDialog editDialog(domain, result.description());
 			if (editDialog.runModal() > 0) {
 				// User pressed OK, so make changes permanent
-
+		
 				// Write config to disk
 				ConfMan.flushToDisk();
-
+		
 				// Update the ListWidget, select the new item, and force a redraw
 				updateListing();
 				selectGame(domain);
@@ -693,10 +697,57 @@ void LauncherDialog::addGame() {
 				// User aborted, remove the the new domain again
 				ConfMan.removeGameDomain(domain);
 			}
+
 		}
 	}
 }
 
+Common::String addGameToConf(const GameDescriptor &result) {
+	// The auto detector or the user made a choice.
+	// Pick a domain name which does not yet exist (after all, we
+	// are *adding* a game to the config, not replacing).
+	String domain;
+
+	if (result.contains("preferredtarget"))
+		domain = result["preferredtarget"];
+	else
+		domain = result.gameid();
+
+	assert(!domain.empty());
+	if (ConfMan.hasGameDomain(domain)) {
+		int suffixN = 1;
+		char suffix[16];
+		String gameid(domain);
+
+		while (ConfMan.hasGameDomain(domain)) {
+			snprintf(suffix, 16, "-%d", suffixN);
+			domain = gameid + suffix;
+			suffixN++;
+		}
+	}
+
+	// Add the name domain
+	ConfMan.addGameDomain(domain);
+
+	// Copy all non-empty key/value pairs into the new domain
+	for (GameDescriptor::const_iterator iter = result.begin(); iter != result.end(); ++iter) {
+		if (!iter->_value.empty() && iter->_key != "preferredtarget")
+			ConfMan.set(iter->_key, iter->_value, domain);
+	}
+
+	// TODO: Setting the description field here has the drawback
+	// that the user does never notice when we upgrade our descriptions.
+	// It might be nice ot leave this field empty, and only set it to
+	// a value when the user edits the description string. 
+	// However, at this point, that's impractical. Once we have a method
+	// to query all backends for the proper & full description of a given
+	// game target, we can change this (currently, you can only query
+	// for the generic gameid description; it's not possible to obtain
+	// a description which contains extended information like language, etc.).
+	
+	return domain;
+}
+
 void LauncherDialog::removeGame(int item) {
 	MessageDialog alert("Do you really want to remove this game configuration?", "Yes", "No");
 
@@ -725,7 +776,7 @@ void LauncherDialog::editGame(int item) {
 	String gameId(ConfMan.get("gameid", _domains[item]));
 	if (gameId.empty())
 		gameId = _domains[item];
-	EditGameDialog editDialog(_domains[item], Base::findGame(gameId).description);
+	EditGameDialog editDialog(_domains[item], Base::findGame(gameId).description());
 	if (editDialog.runModal() > 0) {
 		// User pressed OK, so make changes permanent
 
@@ -738,6 +789,16 @@ void LauncherDialog::editGame(int item) {
 	}
 }
 
+void LauncherDialog::handleKeyDown(uint16 ascii, int keycode, int modifiers) {
+	Dialog::handleKeyDown(ascii, keycode, modifiers);
+	updateButtons();
+}
+
+void LauncherDialog::handleKeyUp(uint16 ascii, int keycode, int modifiers) {
+	Dialog::handleKeyUp(ascii, keycode, modifiers);
+	updateButtons();
+}
+
 void LauncherDialog::handleCommand(CommandSender *sender, uint32 cmd, uint32 data) {
 	int item = _list->getSelected();
 
@@ -796,11 +857,22 @@ void LauncherDialog::updateButtons() {
 		_removeButton->setEnabled(enable);
 		_removeButton->draw();
 	}
+
+	// Update the label of the "Add" button depending on whether shift is pressed or not
+	int modifiers = g_system->getEventManager()->getModifierState();
+	const char *newAddButtonLabel = ((modifiers & Common::KBD_SHIFT) != 0)
+		? "Mass Add..."
+		: "Add Game...";
+
+	if (_addButton->getLabel() != newAddButtonLabel) {
+		_addButton->setLabel(newAddButtonLabel);
+		_addButton->draw();
+	}
 }
 
-void LauncherDialog::handleScreenChanged() {
+void LauncherDialog::reflowLayout() {
 #ifndef DISABLE_FANCY_THEMES
-	if (g_gui.evaluator()->getVar("launcher_logo.visible") == 1) {
+	if (g_gui.evaluator()->getVar("launcher_logo.visible") == 1 && g_gui.theme()->supportsImages()) {
 		StaticTextWidget *ver = (StaticTextWidget*)findWidget("launcher_version");
 		if (ver) {
 			ver->setAlign((Graphics::TextAlignment)g_gui.evaluator()->getVar("launcher_version.align"));
@@ -810,9 +882,7 @@ void LauncherDialog::handleScreenChanged() {
 		if (!_logo)
 			_logo = new GraphicsWidget(this, "launcher_logo");
 		_logo->useThemeTransparency(true);
-		ThemeNew *th = (ThemeNew *)g_gui.theme();
-
-		_logo->setGfx(th->getImageSurface(th->kThemeLogo));
+		_logo->setGfx(g_gui.theme()->getImageSurface(Theme::kImageLogo));
 	} else {
 		StaticTextWidget *ver = (StaticTextWidget*)findWidget("launcher_version");
 		if (ver) {
@@ -821,7 +891,7 @@ void LauncherDialog::handleScreenChanged() {
 		}
 
 		if (_logo) {
-			deleteWidget(_logo);
+			removeWidget(_logo);
 			_logo->setNext(0);
 			delete _logo;
 			_logo = 0;
@@ -832,7 +902,7 @@ void LauncherDialog::handleScreenChanged() {
 	_w = g_system->getOverlayWidth();
 	_h = g_system->getOverlayHeight();
 
-	Dialog::handleScreenChanged();
+	Dialog::reflowLayout();
 }
 
 } // End of namespace GUI
