@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2002-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/gui/launcher.h $
- * $Id: launcher.h 24441 2006-10-22 15:15:59Z knakos $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/gui/launcher.h $
+ * $Id: launcher.h 27024 2007-05-30 21:56:52Z fingolfin $
  */
 
 #ifndef LAUNCHER_DIALOG_H
@@ -31,10 +34,11 @@ class BrowserDialog;
 class ListWidget;
 class GraphicsWidget;
 
+
+Common::String addGameToConf(const GameDescriptor &result);
+
+
 class LauncherDialog : public Dialog {
-#ifdef _WIN32_WCE
-public:			// pacify MS compiler
-#endif
 	typedef Common::String String;
 	typedef Common::StringList StringList;
 public:
@@ -43,8 +47,12 @@ public:
 
 	virtual void handleCommand(CommandSender *sender, uint32 cmd, uint32 data);
 
+	virtual void handleKeyDown(uint16 ascii, int keycode, int modifiers);
+	virtual void handleKeyUp(uint16 ascii, int keycode, int modifiers);
+
 protected:
 	ListWidget		*_list;
+	ButtonWidget	*_addButton;
 	Widget			*_startButton;
 	Widget			*_editButton;
 	Widget			*_removeButton;
@@ -54,11 +62,12 @@ protected:
 	StringList		_domains;
 	BrowserDialog	*_browser;
 
-	virtual void handleScreenChanged();
+	virtual void reflowLayout();
 
 	void updateListing();
 	void updateButtons();
 
+	void open();
 	void close();
 	virtual void addGame();
 	void removeGame(int item);
