@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2002-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,18 +18,20 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/gui/newgui.cpp $
- * $Id: newgui.cpp 23142 2006-06-16 06:26:40Z eriktorbjorn $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/gui/newgui.cpp $
+ * $Id: newgui.cpp 27061 2007-06-03 08:11:19Z sev $
  */
 
 #include "common/stdafx.h"
+#include "common/events.h"
 #include "common/system.h"
 #include "common/util.h"
 #include "graphics/cursorman.h"
-#include "graphics/paletteman.h"
 #include "gui/newgui.h"
 #include "gui/dialog.h"
 #include "gui/eval.h"
+#include "gui/ThemeModern.h"
+#include "gui/ThemeClassic.h"
 
 #include "common/config-manager.h"
 
@@ -48,19 +53,11 @@ namespace GUI {
 
 enum {
 	kDoubleClickDelay = 500, // milliseconds
-	kCursorAnimateDelay = 250,
-	kKeyRepeatInitialDelay = 400,
-	kKeyRepeatSustainDelay = 100
+	kCursorAnimateDelay = 250
 };
 
-// HACK. FIXME. This doesn't belong here. But otherwise it creates compilation problems
-GuiObject::GuiObject(const Common::String &name) : _firstWidget(0) {
-	_name = name;
-	handleScreenChanged();
-}
-
-void GuiObject::handleScreenChanged() {
-	if (_name != "") {
+void GuiObject::reflowLayout() {
+	if (!_name.empty()) {
 		if ((_x = g_gui.evaluator()->getVar(_name + ".x")) == EVAL_UNDEF_VAR)
 			error("Undefined variable %s.x", _name.c_str());
 		if ((_y = g_gui.evaluator()->getVar(_name + ".y")) == EVAL_UNDEF_VAR)
@@ -68,17 +65,17 @@ void GuiObject::handleScreenChanged() {
 		_w = g_gui.evaluator()->getVar(_name + ".w");
 		_h = g_gui.evaluator()->getVar(_name + ".h");
 	
-		if(_x < 0)
+		if (_x < 0)
 			error("Widget <%s> has x < 0", _name.c_str());
-		if(_x >= g_system->getOverlayWidth())
+		if (_x >= g_system->getOverlayWidth())
 			error("Widget <%s> has x > %d", _name.c_str(), g_system->getOverlayWidth());
-		if(_x + _w > g_system->getOverlayWidth())
+		if (_x + _w > g_system->getOverlayWidth())
 			error("Widget <%s> has x + w > %d (%d)", _name.c_str(), g_system->getOverlayWidth(), _x + _w);
-		if(_y < 0)
+		if (_y < 0)
 			error("Widget <%s> has y < 0", _name.c_str());
-		if(_y >= g_system->getOverlayWidth())
+		if (_y >= g_system->getOverlayWidth())
 			error("Widget <%s> has y > %d", _name.c_str(), g_system->getOverlayHeight());
-		if(_y + _h > g_system->getOverlayWidth())
+		if (_y + _h > g_system->getOverlayWidth())
 			error("Widget <%s> has y + h > %d (%d)", _name.c_str(), g_system->getOverlayHeight(), _y + _h);
 	}
 }
@@ -86,43 +83,99 @@ void GuiObject::handleScreenChanged() {
 // Constructor
 NewGui::NewGui() : _needRedraw(false),
 	_stateIsSaved(false), _cursorAnimateCounter(0), _cursorAnimateTimer(0) {
+	_theme = 0;
+	_useStdCursor = false;
 
 	_system = g_system;
+	_lastScreenChangeID = _system->getScreenChangeID();
 
 	// Clear the cursor
 	memset(_cursor, 0xFF, sizeof(_cursor));
 
-	// Reset key repeat
-	_currentKeyDown.keycode = 0;
-
+	bool loadClassicTheme = true;
 #ifndef DISABLE_FANCY_THEMES
 	ConfMan.registerDefault("gui_theme", "default");
 	Common::String style(ConfMan.get("gui_theme"));
 	// The default theme for now is the 'modern' theme.
-	if (scumm_stricmp(style.c_str(), "default") == 0)
+	if (style.compareToIgnoreCase("default") == 0)
 		style = "modern";
 
-	if (scumm_stricmp(style.c_str(), "classic") == 0) {
-#endif
-		_theme = new ThemeClassic(_system);
-#ifndef DISABLE_FANCY_THEMES
+	Common::String styleType;
+	Common::ConfigFile cfg;
+	if (loadNewTheme(style)) {
+	   loadClassicTheme = false;
 	} else {
-		_theme = new ThemeNew(_system, style.c_str());
+	   loadClassicTheme = true;
+	   warning("falling back to classic style");
 	}
 #endif
-	assert(_theme);
-
-	// Init the theme
-	if (!_theme->init()) {
-		warning("Could not initialize your preferred theme, falling back to classic style");
-		delete _theme;
+	
+	if (loadClassicTheme) {
 		_theme = new ThemeClassic(_system);
 		assert(_theme);
 		if (!_theme->init()) {
 			error("Couldn't initialize classic theme");
 		}
 	}
+
 	_theme->resetDrawArea();
+	_themeChange = false;
+}
+
+bool NewGui::loadNewTheme(const Common::String &style) {
+	Common::String styleType;
+	Common::ConfigFile cfg;
+
+	Common::String oldTheme = (_theme != 0) ? _theme->getStylefileName() : "";
+
+	if (_theme)
+		_theme->disable();
+
+	if (_useStdCursor) {
+		CursorMan.popCursorPalette();
+		CursorMan.popCursor();
+	}
+
+	delete _theme;
+	_theme = 0;
+
+	if (style.compareToIgnoreCase("classic (builtin)") == 0 ||
+		style.compareToIgnoreCase("classic") == 0) {
+		_theme = new ThemeClassic(_system, style);
+	} else {	
+		if (Theme::themeConfigUseable(style, "", &styleType, &cfg)) {
+			if (0 == styleType.compareToIgnoreCase("classic"))
+				_theme = new ThemeClassic(_system, style, &cfg);
+#ifndef DISABLE_FANCY_THEMES
+			else if (0 == styleType.compareToIgnoreCase("modern"))
+				_theme = new ThemeModern(_system, style, &cfg);
+#endif
+			else
+				warning("Unsupported theme type '%s'", styleType.c_str());
+		} else {
+			warning("Config '%s' is NOT usable for themes or not found", style.c_str());
+		}
+	}
+	cfg.clear();
+
+	if (!_theme)
+		return (!oldTheme.empty() ? loadNewTheme(oldTheme) : false);
+
+	if (!_theme->init()) {
+		warning("Could not initialize your preferred theme");
+		delete _theme;
+		_theme = 0;
+		loadNewTheme(oldTheme);
+		return false;
+	}
+	_theme->resetDrawArea();
+
+	if (!oldTheme.empty())
+		screenChange();
+
+	_themeChange = true;
+
+	return true;
 }
 
 void NewGui::redraw() {
@@ -152,8 +205,14 @@ void NewGui::redraw() {
 	_theme->drawAll();
 }
 
+Dialog *NewGui::getTopDialog() const {
+	if (_dialogStack.empty())
+		return 0;
+	return _dialogStack.top();
+}
+
 void NewGui::runLoop() {
-	Dialog *activeDialog = _dialogStack.top();
+	Dialog *activeDialog = getTopDialog();
 	bool didSaveState = false;
 	int button;
 
@@ -164,24 +223,15 @@ void NewGui::runLoop() {
 		saveState();
 		_theme->enable();
 		didSaveState = true;
+
+		_useStdCursor = !_theme->ownCursor();
+		if (_useStdCursor)
+			setupCursor();
 	}
+	
+	Common::EventManager *eventMan = _system->getEventManager();
 
-	bool useStandardCurs = !_theme->ownCursor();
-
-	if (useStandardCurs) {
-		const byte palette[] = {
-			255, 255, 255, 0,
-			255, 255, 255, 0,
-			171, 171, 171, 0,
-			 87,  87,  87, 0
-		};
-
-		PaletteMan.pushCursorPalette(palette, 0, 4);
-		CursorMan.pushCursor(NULL, 0, 0, 0, 0);
-		CursorMan.showMouse(true);
-	}
-
-	while (!_dialogStack.empty() && activeDialog == _dialogStack.top()) {
+	while (!_dialogStack.empty() && activeDialog == getTopDialog()) {
 		if (_needRedraw) {
 			redraw();
 			_needRedraw = false;
@@ -192,45 +242,48 @@ void NewGui::runLoop() {
 
 		activeDialog->handleTickle();
 
-		if (useStandardCurs)
+		if (_useStdCursor)
 			animateCursor();
 		_theme->drawAll();
 		_system->updateScreen();
 
-		OSystem::Event event;
+		Common::Event event;
 		uint32 time = _system->getMillis();
 
-		while (_system->pollEvent(event)) {
-			if (activeDialog != _dialogStack.top() && event.type != OSystem::EVENT_QUIT && event.type != OSystem::EVENT_SCREEN_CHANGED)
+		while (eventMan->pollEvent(event)) {
+			if (activeDialog != getTopDialog() && event.type != Common::EVENT_QUIT && event.type != Common::EVENT_SCREEN_CHANGED)
 				continue;
 
 			Common::Point mouse(event.mouse.x - activeDialog->_x, event.mouse.y - activeDialog->_y);
+
+			// HACK to change the cursor to the new themes one
+			if (_themeChange) {
+				_theme->enable();
+
+				_useStdCursor = !_theme->ownCursor();
+				if (_useStdCursor)
+					setupCursor();
+
+				_theme->refresh();
+
+				_themeChange = false;
+				redraw();
+			}
 			
 			switch (event.type) {
-			case OSystem::EVENT_KEYDOWN:
-#if !defined(PALMOS_MODE)
-				// init continuous event stream
-				// not done on PalmOS because keyboard is emulated and keyup is not generated
-				_currentKeyDown.ascii = event.kbd.ascii;
-				_currentKeyDown.keycode = event.kbd.keycode;
-				_currentKeyDown.flags = event.kbd.flags;
-				_keyRepeatTime = time + kKeyRepeatInitialDelay;
-#endif
+			case Common::EVENT_KEYDOWN:
 				activeDialog->handleKeyDown(event.kbd.ascii, event.kbd.keycode, event.kbd.flags);
 				break;
-			case OSystem::EVENT_KEYUP:
+			case Common::EVENT_KEYUP:
 				activeDialog->handleKeyUp(event.kbd.ascii, event.kbd.keycode, event.kbd.flags);
-				if (event.kbd.keycode == _currentKeyDown.keycode)
-					// only stop firing events if it's the current key
-					_currentKeyDown.keycode = 0;
 				break;
-			case OSystem::EVENT_MOUSEMOVE:
+			case Common::EVENT_MOUSEMOVE:
 				activeDialog->handleMouseMoved(mouse.x, mouse.y, 0);
 				break;
 			// We don't distinguish between mousebuttons (for now at least)
-			case OSystem::EVENT_LBUTTONDOWN:
-			case OSystem::EVENT_RBUTTONDOWN:
-				button = (event.type == OSystem::EVENT_LBUTTONDOWN ? 1 : 2);
+			case Common::EVENT_LBUTTONDOWN:
+			case Common::EVENT_RBUTTONDOWN:
+				button = (event.type == Common::EVENT_LBUTTONDOWN ? 1 : 2);
 				if (_lastClick.count && (time < _lastClick.time + kDoubleClickDelay)
 							&& ABS(_lastClick.x - event.mouse.x) < 3
 							&& ABS(_lastClick.y - event.mouse.y) < 3) {
@@ -243,41 +296,25 @@ void NewGui::runLoop() {
 				_lastClick.time = time;
 				activeDialog->handleMouseDown(mouse.x, mouse.y, button, _lastClick.count);
 				break;
-			case OSystem::EVENT_LBUTTONUP:
-			case OSystem::EVENT_RBUTTONUP:
-				button = (event.type == OSystem::EVENT_LBUTTONUP ? 1 : 2);
+			case Common::EVENT_LBUTTONUP:
+			case Common::EVENT_RBUTTONUP:
+				button = (event.type == Common::EVENT_LBUTTONUP ? 1 : 2);
 				activeDialog->handleMouseUp(mouse.x, mouse.y, button, _lastClick.count);
 				break;
-			case OSystem::EVENT_WHEELUP:
+			case Common::EVENT_WHEELUP:
 				activeDialog->handleMouseWheel(mouse.x, mouse.y, -1);
 				break;
-			case OSystem::EVENT_WHEELDOWN:
+			case Common::EVENT_WHEELDOWN:
 				activeDialog->handleMouseWheel(mouse.x, mouse.y, 1);
 				break;
-			case OSystem::EVENT_QUIT:
+			case Common::EVENT_QUIT:
 				_system->quit();
 				return;
-			case OSystem::EVENT_SCREEN_CHANGED:
-				// reinit the whole theme
-				_theme->refresh();
-				// refresh all dialogs
-				for (int i = 0; i < _dialogStack.size(); ++i) {
-					_dialogStack[i]->handleScreenChanged();
-				}
-				// We need to redraw immediately. Otherwise
-				// some other event may cause a widget to be
-				// redrawn before redraw() has been called.
-				redraw();
+			case Common::EVENT_SCREEN_CHANGED:
+				screenChange();
+				break;
+			default:
 				break;
-			}
-		}
-
-		// check if event should be sent again (keydown)
-		if (_currentKeyDown.keycode != 0 && activeDialog == _dialogStack.top()) {
-			if (_keyRepeatTime < time) {
-				// fire event
-				activeDialog->handleKeyDown(_currentKeyDown.ascii, _currentKeyDown.keycode, _currentKeyDown.flags);
-				_keyRepeatTime = time + kKeyRepeatSustainDelay;
 			}
 		}
 
@@ -286,12 +323,11 @@ void NewGui::runLoop() {
 	}
 
 	_theme->closeDialog();
-	if (useStandardCurs)
-		PaletteMan.popCursorPalette();
 
 	if (didSaveState) {
 		_theme->disable();
 		restoreState();
+		_useStdCursor = false;
 	}
 }
 
@@ -299,7 +335,6 @@ void NewGui::runLoop() {
 
 void NewGui::saveState() {
 	// Backup old cursor
-	_currentKeyDown.keycode = 0;
 	_lastClick.x = _lastClick.y = 0;
 	_lastClick.time = 0;
 	_lastClick.count = 0;
@@ -308,7 +343,10 @@ void NewGui::saveState() {
 }
 
 void NewGui::restoreState() {
-	CursorMan.popCursor();
+	if (_useStdCursor) {
+		CursorMan.popCursor();
+		CursorMan.popCursorPalette();
+	}
 
 	_system->updateScreen();
 
@@ -318,6 +356,23 @@ void NewGui::restoreState() {
 void NewGui::openDialog(Dialog *dialog) {
 	_dialogStack.push(dialog);
 	_needRedraw = true;
+
+	// We reflow the dialog just before opening it. If the screen changed
+	// since the last time we looked, also refresh the loaded theme,
+	// and reflow all other open dialogs, too.
+	int tmpScreenChangeID = _system->getScreenChangeID();
+	if (_lastScreenChangeID != tmpScreenChangeID) {
+		_lastScreenChangeID = tmpScreenChangeID;
+
+		// reinit the whole theme
+		_theme->refresh();
+		// refresh all dialogs
+		for (int i = 0; i < _dialogStack.size(); ++i) {
+			_dialogStack[i]->reflowLayout();
+		}
+	} else {
+		dialog->reflowLayout();
+	}
 }
 
 void NewGui::closeTopDialog() {
@@ -330,6 +385,19 @@ void NewGui::closeTopDialog() {
 	_needRedraw = true;
 }
 
+void NewGui::setupCursor() {
+	const byte palette[] = {
+		255, 255, 255, 0,
+		255, 255, 255, 0,
+		171, 171, 171, 0,
+		 87,  87,  87, 0
+	};
+
+	CursorMan.pushCursorPalette(palette, 0, 4);
+	CursorMan.pushCursor(NULL, 0, 0, 0, 0);
+	CursorMan.showMouse(true);
+}
+
 // Draw the mouse cursor (animated). This is pretty much the same as in old
 // SCUMM games, but the code no longer resembles what we have in cursor.cpp
 // very much. We could plug in a different cursor here if we like to.
@@ -355,12 +423,24 @@ WidgetSize NewGui::getWidgetSize() {
 	return (WidgetSize)(_theme->_evaluator->getVar("widgetSize"));
 }
 
-void NewGui::handleScreenChange() {
-	_theme->refresh();
+void NewGui::clearDragWidget() {
+	if (!_dialogStack.empty())
+		_dialogStack.top()->_dragWidget = 0;
 }
 
-void NewGui::clearDragWidget() {
-	_dialogStack.top()->_dragWidget = 0;
+void NewGui::screenChange() {
+	_lastScreenChangeID = _system->getScreenChangeID();
+
+	// reinit the whole theme
+	_theme->refresh();
+	// refresh all dialogs
+	for (int i = 0; i < _dialogStack.size(); ++i) {
+		_dialogStack[i]->reflowLayout();
+	}
+	// We need to redraw immediately. Otherwise
+	// some other event may cause a widget to be
+	// redrawn before redraw() has been called.
+	redraw();
 }
 
 } // End of namespace GUI
