@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2002-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/gui/newgui.h $
- * $Id: newgui.h 22928 2006-06-05 12:22:51Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/gui/newgui.h $
+ * $Id: newgui.h 27024 2007-05-30 21:56:52Z fingolfin $
  */
 
 #ifndef NEWGUI_H
@@ -28,6 +31,7 @@
 #include "common/str.h"
 #include "graphics/fontman.h"
 #include "gui/theme.h"
+#include "gui/widget.h"
 
 class OSystem;
 
@@ -70,6 +74,7 @@ public:
 
 	bool isActive() const	{ return ! _dialogStack.empty(); }
 
+	bool loadNewTheme(const Common::String &file);
 	Theme *theme() { return _theme; }
 	Eval *evaluator() { return _theme->_evaluator; }
 
@@ -80,27 +85,22 @@ public:
 
 	WidgetSize getWidgetSize();
 
-	void handleScreenChange();
-
 	void clearDragWidget();
 
+	void screenChange();
+
 protected:
 	OSystem			*_system;
 
 	Theme		*_theme;
 
 	bool		_needRedraw;
+	int			_lastScreenChangeID;
 	DialogStack	_dialogStack;
 
 	bool		_stateIsSaved;
 
-	// for continuous events (keyDown)
-	struct {
-		uint16 ascii;
-		byte flags;
-		int keycode;
-	} _currentKeyDown;
-	uint32		_keyRepeatTime;
+	bool		_useStdCursor;
 
 	// position and time of last mouse click (used to detect double clicks)
 	struct {
@@ -114,6 +114,8 @@ protected:
 	int		_cursorAnimateTimer;
 	byte		_cursor[2048];
 
+	bool _themeChange;
+
 	void saveState();
 	void restoreState();
 
@@ -124,7 +126,10 @@ protected:
 
 	void loop();
 
+	void setupCursor();
 	void animateCursor();
+	
+	Dialog *getTopDialog() const;
 };
 
 } // End of namespace GUI
