@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2002-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,12 +18,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/gui/options.cpp $
- * $Id: options.cpp 23989 2006-09-25 21:09:53Z agent-q $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/gui/options.cpp $
+ * $Id: options.cpp 27347 2007-06-11 08:04:11Z sev $
  */
 
 #include "common/stdafx.h"
 #include "gui/browser.h"
+#include "gui/themebrowser.h"
 #include "gui/chooser.h"
 #include "gui/eval.h"
 #include "gui/newgui.h"
@@ -28,35 +32,17 @@
 #include "gui/PopUpWidget.h"
 #include "gui/TabWidget.h"
 
-#include "backends/fs/fs.h"
+#include "common/fs.h"
 #include "common/config-manager.h"
-#include "graphics/scaler.h"
 #include "common/system.h"
+
+#include "graphics/scaler.h"
+
 #include "sound/mididrv.h"
 #include "sound/mixer.h"
 
-#if (!( defined(PALMOS_MODE) || defined(__DC__) || defined(__GP32__) || defined(__amigaos4__) || defined(__DS__) ) && !defined(_MSC_VER))
-#include <sys/param.h>
-#include <unistd.h>
-#endif
-
-#if !(defined(MAXPATHLEN))
-#ifndef PALMOS_MODE
-#define MAXPATHLEN 1024
-#else
-#define MAXPATHLEN 256
-#endif
-#endif
-
 namespace GUI {
 
-// TODO - allow changing options for:
-// - the save path (use _dirBrowser!)
-// - music & graphics driver (but see also the comments on EditGameDialog
-//   for some techincal difficulties with this)
-// - default volumes (sfx/speech/music)
-// - aspect ratio, language, platform, debug mode/level, cd drive, joystick, multi midi, native mt32
-
 enum {
 	kMidiGainChanged		= 'mgch',
 	kMusicVolumeChanged		= 'muvc',
@@ -65,9 +51,11 @@ enum {
 	kSubtitleSpeedChanged	= 'stsc',
 	kSpeechVolumeChanged	= 'vcvc',
 	kChooseSoundFontCmd		= 'chsf',
+	kClearSoundFontCmd      = 'clsf',
 	kChooseSaveDirCmd		= 'chos',
 	kChooseThemeDirCmd		= 'chth',
-	kChooseExtraDirCmd		= 'chex'
+	kChooseExtraDirCmd		= 'chex',
+	kChooseThemeCmd			= 'chtf'
 };
 
 #ifdef SMALL_SCREEN_DEVICE
@@ -76,6 +64,12 @@ enum {
 };
 #endif
 
+static const char *savePeriodLabels[] = { "Never", "every 5 mins", "every 10 mins", "every 15 mins", "every 30 mins", 0 };
+static const int savePeriodValues[] = { 0, 5 * 60, 10 * 60, 15 * 60, 30 * 60, -1 };
+static const char *outputRateLabels[] = { "Default", "22 kHz", "8 kHz", "11kHz", "44 kHz", "48 kHz", 0 };
+static const int outputRateValues[] = { 0, 22050, 8000, 11025, 44100, 48000, -1 };
+
+
 
 OptionsDialog::OptionsDialog(const String &domain, int x, int y, int w, int h)
 	: Dialog(x, y, w, h), _domain(domain) {
@@ -101,6 +95,7 @@ void OptionsDialog::init() {
 	_aspectCheckbox = 0;
 	_enableAudioSettings = false;
 	_midiPopUp = 0;
+	_outputRatePopUp = 0;
 	_enableMIDISettings = false;
 	_multiMidiCheckbox = 0;
 	_mt32Checkbox = 0;
@@ -184,6 +179,15 @@ void OptionsDialog::open() {
 		_midiPopUp->setSelected(md->name ? i : 0);
 	}
 
+	if (_outputRatePopUp) {
+		_outputRatePopUp->setSelected(1);
+		int value = ConfMan.getInt("output_rate", _domain);
+		for	(int i = 0; outputRateLabels[i]; i++) {
+			if (value == outputRateValues[i])
+				_outputRatePopUp->setSelected(i);
+		}
+	}
+
 	if (_multiMidiCheckbox) {
 
 		// Multi midi setting
@@ -196,10 +200,13 @@ void OptionsDialog::open() {
 		_enableGSCheckbox->setState(ConfMan.getBool("enable_gs", _domain));
 
 		String soundFont(ConfMan.get("soundfont", _domain));
-		if (soundFont.empty() || !ConfMan.hasKey("soundfont", _domain))
+		if (soundFont.empty() || !ConfMan.hasKey("soundfont", _domain)) {
 			_soundFont->setLabel("None");
-		else
+			_soundFontClearButton->setEnabled(false);
+		} else {
 			_soundFont->setLabel(soundFont);
+			_soundFontClearButton->setEnabled(true);
+		}
 
 		// MIDI gain setting
 		char buf[10];
@@ -306,6 +313,17 @@ void OptionsDialog::close() {
 			}
 		}
 
+		if (_outputRatePopUp) {
+			if (_enableAudioSettings) {
+				if (_outputRatePopUp->getSelectedTag() != 0)
+					ConfMan.setInt("output_rate", _outputRatePopUp->getSelectedTag(), _domain);
+				else
+					ConfMan.removeKey("output_rate", _domain);
+			} else {
+				ConfMan.removeKey("output_rate", _domain);
+			}
+		}
+
 		// MIDI options
 		if (_multiMidiCheckbox) {
 			if (_enableMIDISettings) {
@@ -317,6 +335,8 @@ void OptionsDialog::close() {
 				String soundFont(_soundFont->getLabel());
 				if (!soundFont.empty() && (soundFont != "None"))
 					ConfMan.set("soundfont", soundFont, _domain);
+				else
+					ConfMan.removeKey("soundfont", _domain);
 			} else {
 				ConfMan.removeKey("multi_midi", _domain);
 				ConfMan.removeKey("native_mt32", _domain);
@@ -430,6 +450,7 @@ void OptionsDialog::setAudioSettingsState(bool enabled) {
 	_enableAudioSettings = enabled;
 
 	_midiPopUp->setEnabled(enabled);
+	_outputRatePopUp->setEnabled(enabled);
 }
 
 void OptionsDialog::setMIDISettingsState(bool enabled) {
@@ -437,6 +458,12 @@ void OptionsDialog::setMIDISettingsState(bool enabled) {
 
 	_soundFontButton->setEnabled(enabled);
 	_soundFont->setEnabled(enabled);
+
+	if (enabled && !_soundFont->getLabel().empty() && (_soundFont->getLabel() != "None"))
+		_soundFontClearButton->setEnabled(enabled);
+	else
+		_soundFontClearButton->setEnabled(false);
+
 	_multiMidiCheckbox->setEnabled(enabled);
 	_mt32Checkbox->setEnabled(enabled);
 	_enableGSCheckbox->setEnabled(enabled);
@@ -521,6 +548,13 @@ void OptionsDialog::addAudioControls(GuiObject *boss, const String &prefix) {
 		md++;
 	}
 
+	// Sample rate settings
+	_outputRatePopUp = new PopUpWidget(boss, prefix + "auSampleRatePopup", "Output rate: ", labelWidth);
+
+	for (int i = 0; outputRateLabels[i]; i++) {
+		_outputRatePopUp->appendEntry(outputRateLabels[i], outputRateValues[i]);
+	}
+
 	_enableAudioSettings = true;
 }
 
@@ -528,6 +562,7 @@ void OptionsDialog::addMIDIControls(GuiObject *boss, const String &prefix) {
 	// SoundFont
 	_soundFontButton = new ButtonWidget(boss, prefix + "mcFontButton", "SoundFont:", kChooseSoundFontCmd, 0);
 	_soundFont = new StaticTextWidget(boss, prefix + "mcFontPath", "None");
+	_soundFontClearButton = new ButtonWidget(boss, prefix + "mcFontClearButton", "C", kClearSoundFontCmd, 0);
 
 	// Multi midi setting
 	_multiMidiCheckbox = new CheckboxWidget(boss, prefix + "mcMixedCheckbox", "Mixed Adlib/MIDI mode", 0, 0);
@@ -604,13 +639,15 @@ int OptionsDialog::getSubtitleMode(bool subtitles, bool speech_mute) {
 	return 2;
 }
 
-void OptionsDialog::handleScreenChanged() {
-	Dialog::handleScreenChanged();
+void OptionsDialog::reflowLayout() {
+	Dialog::reflowLayout();
 
 	int labelWidth = g_gui.evaluator()->getVar("tabPopupsLabelW");
 
 	if (_midiPopUp)
 		_midiPopUp->changeLabelWidth(labelWidth);
+	if (_outputRatePopUp)
+		_outputRatePopUp->changeLabelWidth(labelWidth);
 	if (_gfxPopUp)
 		_gfxPopUp->changeLabelWidth(labelWidth);
 	if (_renderModePopUp)
@@ -638,18 +675,10 @@ GlobalOptionsDialog::GlobalOptionsDialog()
 	//
 	tab->addTab("Audio");
 	addAudioControls(tab, "globaloptions_");
-
-	int volControlPos = g_gui.evaluator()->getVar("volumeControlsInAudio", true);
-
-	if (volControlPos)
-		addVolumeControls(tab, "globaloptions_");
-
 	addSubtitleControls(tab, "globaloptions_");
 
-	if (!volControlPos) {
-		tab->addTab("Volume");
-		addVolumeControls(tab, "globaloptions_");
-	}
+	tab->addTab("Volume");
+	addVolumeControls(tab, "globaloptions_");
 
 	// TODO: cd drive setting
 
@@ -683,6 +712,19 @@ GlobalOptionsDialog::GlobalOptionsDialog()
 	new ButtonWidget(tab, "globaloptions_keysbutton", "Keys", kChooseKeyMappingCmd, 0);
 #endif
 
+	tab->addTab("Misc");
+
+	new ButtonWidget(tab, "globaloptions_themebutton2", "Theme:", kChooseThemeCmd, 0);
+	_curTheme = new StaticTextWidget(tab, "globaloptions_curtheme", g_gui.theme()->getThemeName());
+
+	int labelWidth = g_gui.evaluator()->getVar("tabPopupsLabelW");
+
+	_autosavePeriodPopUp = new PopUpWidget(tab, "globaloptions_autosaveperiod", "Autosave: ", labelWidth);
+
+	for (int i = 0; savePeriodLabels[i]; i++) {
+		_autosavePeriodPopUp->appendEntry(savePeriodLabels[i], savePeriodValues[i]);
+	}
+
 	// TODO: joystick setting
 
 
@@ -713,13 +755,10 @@ void GlobalOptionsDialog::open() {
 	Common::String themePath(ConfMan.get("themepath", _domain));
 	Common::String extraPath(ConfMan.get("extrapath", _domain));
 
-	if (!savePath.empty()) {
-		_savePath->setLabel(savePath);
+	if (savePath.empty() || !ConfMan.hasKey("savepath", _domain)) {
+		_savePath->setLabel("None");
 	} else {
-		// Default to the current directory...
-		char buf[MAXPATHLEN];
-		getcwd(buf, sizeof(buf));
-		_savePath->setLabel(buf);
+		_savePath->setLabel(savePath);
 	}
 
 	if (themePath.empty() || !ConfMan.hasKey("themepath", _domain)) {
@@ -734,20 +773,35 @@ void GlobalOptionsDialog::open() {
 		_extraPath->setLabel(extraPath);
 	}
 #endif
+
+	// Misc Tab
+	_autosavePeriodPopUp->setSelected(1);
+	int value = ConfMan.getInt("autosave_period");
+	for (int i = 0; savePeriodLabels[i]; i++) {
+		if (value == savePeriodValues[i])
+			_autosavePeriodPopUp->setSelected(i);
+	}
 }
 
 void GlobalOptionsDialog::close() {
 	if (getResult()) {
-		// Savepath
-		ConfMan.set("savepath", _savePath->getLabel(), _domain);
+		String savePath(_savePath->getLabel());
+		if (!savePath.empty() && (savePath != "None"))
+			ConfMan.set("savepath", savePath, _domain);
 
 		String themePath(_themePath->getLabel());
 		if (!themePath.empty() && (themePath != "None"))
 			ConfMan.set("themepath", themePath, _domain);
+		else
+			ConfMan.removeKey("themepath", _domain);
 
 		String extraPath(_extraPath->getLabel());
 		if (!extraPath.empty() && (extraPath != "None"))
 			ConfMan.set("extrapath", extraPath, _domain);
+		else
+			ConfMan.removeKey("extrapath", _domain);
+
+		ConfMan.setInt("autosave_period", _autosavePeriodPopUp->getSelectedTag(), _domain);
 	}
 	OptionsDialog::close();
 }
@@ -791,6 +845,32 @@ void GlobalOptionsDialog::handleCommand(CommandSender *sender, uint32 cmd, uint3
 			// User made his choice...
 			FilesystemNode file(browser.getResult());
 			_soundFont->setLabel(file.path());
+
+			if (!file.path().empty() && (file.path() != "None"))
+				_soundFontClearButton->setEnabled(true);
+			else
+				_soundFontClearButton->setEnabled(false);
+
+			draw();
+		}
+		break;
+	}
+	case kClearSoundFontCmd: {
+		_soundFont->setLabel("None");
+		_soundFontClearButton->setEnabled(false);
+		draw();
+		break;
+	}
+	case kChooseThemeCmd: {
+		ThemeBrowser browser;
+		if (browser.runModal() > 0) {
+			// User made his choice...
+			const Common::String &theme = browser.selected();
+			if (0 != theme.compareToIgnoreCase(g_gui.theme()->getStylefileName()))
+				if (g_gui.loadNewTheme(theme)) {
+					_curTheme->setLabel(g_gui.theme()->getThemeName());
+					ConfMan.set("gui_theme", theme);
+				}
 			draw();
 		}
 		break;
