@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2002-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/gui/options.h $
- * $Id: options.h 23033 2006-06-11 20:51:14Z sev $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/gui/options.h $
+ * $Id: options.h 27024 2007-05-30 21:56:52Z fingolfin $
  */
 
 #ifndef OPTIONS_DIALOG_H
@@ -36,6 +39,7 @@ class CheckboxWidget;
 class PopUpWidget;
 class SliderWidget;
 class StaticTextWidget;
+class ListWidget;
 
 class OptionsDialog : public Dialog {
 	typedef Common::String String;
@@ -49,7 +53,7 @@ public:
 	void close();
 	void handleCommand(CommandSender *sender, uint32 cmd, uint32 data);
 
-	virtual void handleScreenChanged();
+	virtual void reflowLayout();
 
 	enum {
 		kOKCmd					= 'ok  '
@@ -61,6 +65,7 @@ protected:
 
 	ButtonWidget *_soundFontButton;
 	StaticTextWidget *_soundFont;
+	ButtonWidget *_soundFontClearButton;
 
 	void addGraphicControls(GuiObject *boss, const String &prefix);
 	void addAudioControls(GuiObject *boss, const String &prefix);
@@ -91,6 +96,7 @@ private:
 	//
 	bool _enableAudioSettings;
 	PopUpWidget *_midiPopUp;
+	PopUpWidget *_outputRatePopUp;
 
 	//
 	// MIDI controls
@@ -152,6 +158,13 @@ protected:
 	StaticTextWidget *_savePath;
 	StaticTextWidget *_themePath;
 	StaticTextWidget *_extraPath;
+
+	//
+	// Misc controls
+	//
+	StaticTextWidget *_curTheme;
+
+	PopUpWidget *_autosavePeriodPopUp;
 };
 
 } // End of namespace GUI
