@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/gui/theme-config.cpp $
- * $Id: theme-config.cpp 23395 2006-07-07 22:02:20Z sev $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/gui/theme-config.cpp $
+ * $Id: theme-config.cpp 27347 2007-06-11 08:04:11Z sev $
  */
 
 #include "gui/theme.h"
@@ -55,6 +58,32 @@ const char *Theme::_defaultConfigINI =
 "def_vcAudioTabSpacing=2\n"
 "use=XxY\n"
 "\n"
+"# Override audio tab\n"
+"set_parent=gameoptions\n"
+"vBorder=5\n"
+"\n"
+"# audio tab\n"
+"opYoffset=vBorder\n"
+"useWithPrefix=audioControls globaloptions_\n"
+"useWithPrefix=subtitleControls globaloptions_\n"
+"\n"
+"# volume tab\n"
+"opYoffset=vBorder\n"
+"useWithPrefix=volumeControls globaloptions_\n"
+"\n"
+"# audio tab\n"
+"opYoffset=vBorder\n"
+"gameoptions_audioCheckbox=gox opYoffset (kFontHeight + 10 + 180) buttonHeight\n"
+"opYoffset=(opYoffset + buttonHeight + 6)\n"
+"useWithPrefix=audioControls gameoptions_\n"
+"useWithPrefix=subtitleControls gameoptions_\n"
+"\n"
+"# volume tab\n"
+"opYoffset=vBorder\n"
+"gameoptions_volumeCheckbox=gox opYoffset (kFontHeight + 10 + 190) buttonHeight\n"
+"opYoffset=(opYoffset + buttonHeight + 6)\n"
+"useWithPrefix=volumeControls gameoptions_\n"
+"\n"
 "TabWidget.tabWidth=0\n"
 "TabWidget.tabHeight=16\n"
 "TabWidget.titleVPad=2\n"
@@ -164,9 +193,12 @@ const char *Theme::_defaultConfigINI =
 "# audio tab\n"
 "opYoffset=vBorder\n"
 "useWithPrefix=audioControls globaloptions_\n"
-"useWithPrefix=volumeControls globaloptions_\n"
 "useWithPrefix=subtitleControls globaloptions_\n"
 "\n"
+"# volume tab\n"
+"opYoffset=vBorder\n"
+"useWithPrefix=volumeControls globaloptions_\n"
+"\n"
 "# MIDI tab\n"
 "opYoffset=vBorder\n"
 "useWithPrefix=midiControls globaloptions_\n"
@@ -185,6 +217,14 @@ const char *Theme::_defaultConfigINI =
 "yoffset=(yoffset + buttonHeight + 4)\n"
 "globaloptions_keysbutton=10 yoffset (buttonWidth + 5) buttonHeight\n"
 "\n"
+"# Misc options\n"
+"yoffset=vBorder\n"
+"glOff=((buttonHeight - kLineHeight) / 2 + 2)\n"
+"globaloptions_themebutton2=10 yoffset buttonWidth buttonHeight\n"
+"globaloptions_curtheme=(prev.x2 + 20) (yoffset + glOff) (parent.w - (prev.w + 20) - 10) kLineHeight\n"
+"yoffset=(yoffset + buttonHeight + 4)\n"
+"globaloptions_autosaveperiod=10 yoffset (parent.w - 10 - 25) (kLineHeight + 2)\n"
+"\n"
 "globaloptions_cancel=(parent.w - 2 * (buttonWidth + 10)) (parent.h - buttonHeight - 8) buttonWidth buttonHeight\n"
 "globaloptions_ok=(prev.x2 + 10) prev.y prev.w prev.h\n"
 "\n"
@@ -237,9 +277,14 @@ const char *Theme::_defaultConfigINI =
 "gameoptions_audioCheckbox=gox opYoffset (kFontHeight + 10 + 180) buttonHeight\n"
 "opYoffset=(opYoffset + buttonHeight)\n"
 "useWithPrefix=audioControls gameoptions_\n"
-"useWithPrefix=volumeControls gameoptions_\n"
 "useWithPrefix=subtitleControls gameoptions_\n"
 "\n"
+"# volume tab\n"
+"opYoffset=vBorder\n"
+"gameoptions_volumeCheckbox=gox opYoffset (kFontHeight + 10 + 190) buttonHeight\n"
+"opYoffset=(opYoffset + buttonHeight)\n"
+"useWithPrefix=volumeControls gameoptions_\n"
+"\n"
 "# midi tab\n"
 "opYoffset=vBorder\n"
 "gameoptions_midiCheckbox=gox opYoffset (kFontHeight + 10 + 174) buttonHeight\n"
@@ -250,15 +295,27 @@ const char *Theme::_defaultConfigINI =
 "gameoptions_ok=(prev.x2 + 10) prev.y prev.w prev.h\n"
 "\n"
 "### keys dialog\n"
-"keysdialog=(w / 20) (h / 10) (w - w / 10) (h - h / 5)\n"
+"keysdialog=(w / 20) (h / 10) (w - w / 10) (h - h / 4)\n"
 "set_parent=keysdialog\n"
-"keysdialog_map=(parent.w - buttonWidth - 10) 20 buttonWidth buttonHeight\n"
+"keysdialog_map=(parent.w - buttonWidth - 10) (10 + 2 * kLineHeight) buttonWidth buttonHeight\n"
 "keysdialog_ok=prev.x (prev.y2 + 4) prev.w prev.h\n"
 "keysdialog_cancel=prev.x (prev.y2 + 4) prev.w prev.h\n"
-"keysdialog_list=10 10 (prev.x - 20) (parent.h - kLineHeight * 4 - self.y)\n"
-"keysdialog_action=prev.x (parent.h - kLineHeight * 3) (parent.w - self.x * 2) kLineHeight\n"
-"keysdialog_mapping=prev.x (prev.y + kLineHeight) prev.w prev.h\n"
+"keysdialog_action=10 10 (parent.w - 20) kLineHeight\n"
+"keysdialog_action.align=kTextAlignCenter\n"
+"keysdialog_list=prev.x (prev.y + 2 * kLineHeight) (parent.w - buttonWidth - 30) (parent.h - kLineHeight * 6)\n"
+"keysdialog_mapping=prev.x (prev.y + prev.h + kLineHeight) (parent.w - buttonWidth - 20) kLineHeight\n"
 "\n"
+"### mass add dialog\n"
+"massadddialog=10 20 300 174\n"
+"set_parent=massadddialog\n"
+"massadddialog_caption=10 (10 + 1 * kLineHeight) (parent.w - 2*10) kLineHeight\n"
+"massadddialog_caption.align=kTextAlignCenter\n"
+"massadddialog_dirprogress=10 (10 + 3 * kLineHeight) (parent.w - 2*10) kLineHeight\n"
+"massadddialog_dirprogress.align=kTextAlignCenter\n"
+"massadddialog_gameprogress=10 (10 + 4 * kLineHeight) (parent.w - 2*10) kLineHeight\n"
+"massadddialog_gameprogress.align=kTextAlignCenter\n"
+"massadddialog_ok=((parent.w - (buttonWidth * 2)) / 2) (parent.h - buttonHeight - 8) buttonWidth buttonHeight\n"
+"massadddialog_cancel=(prev.x2 + 10) prev.y prev.w prev.h\n"
 "\n"
 "\n"
 "##### SCUMM dialogs\n"
@@ -340,6 +397,8 @@ const char *Theme::_defaultConfigINI =
 "auw=(parent.w - 2 * 10)\n"
 "auMidiPopup=(aux - 5) (opYoffset - 1) (auw + 5) (kLineHeight + 2)\n"
 "opYoffset=(opYoffset + buttonHeight + 4)\n"
+"auSampleRatePopup=(aux - 5) (opYoffset - 1) (auw + 5) (kLineHeight + 2)\n"
+"opYoffset=(opYoffset + buttonHeight + 4)\n"
 "\n"
 "[volumeControls]\n"
 "vctextw=(95 + vcAudioTabIndent)\n"
@@ -364,7 +423,8 @@ const char *Theme::_defaultConfigINI =
 "[midiControls]\n"
 "mcx=10\n"
 "mcFontButton=mcx opYoffset buttonWidth buttonHeight\n"
-"mcFontPath=(prev.x2 + 20) (opYoffset + 3) (parent.w - (buttonWidth + 20) - 15) kLineHeight\n"
+"mcFontPath=(prev.x2 + 20) (opYoffset + 3) (parent.w - (buttonWidth + 20) - 15 - kLineHeight - 10) kLineHeight\n"
+"mcFontClearButton=(prev.x2 + 10) (opYoffset + 2) kLineHeight kLineHeight\n"
 "opYoffset=(opYoffset + buttonHeight + 2 * midiControlsSpacing)\n"
 "mcMixedCheckbox=mcx opYoffset (kFontHeight + 10 + 135) buttonHeight\n"
 "opYoffset=(opYoffset + buttonHeight + midiControlsSpacing)\n"
@@ -428,7 +488,6 @@ const char *Theme::_defaultConfigINI =
 "def_insetH=(h - 13 - insetY)\n"
 "def_launcherVersionX=50\n"
 "def_launcherVersionY=5\n"
-"def_volumeControlsInAudio=false\n"
 "def_midiControlsSpacing=2\n"
 "def_gameOptionsOverrideVPad=10\n"
 "def_aboutXOff=3\n"
@@ -436,32 +495,6 @@ const char *Theme::_defaultConfigINI =
 "def_aboutOuterBorder=10\n"
 "\n"
 "use=XxY\n"
-"\n"
-"# Override audio tab\n"
-"set_parent=gameoptions\n"
-"vBorder=5\n"
-"\n"
-"# audio tab\n"
-"opYoffset=vBorder\n"
-"useWithPrefix=audioControls globaloptions_\n"
-"useWithPrefix=subtitleControls globaloptions_\n"
-"\n"
-"# volume tab\n"
-"opYoffset=vBorder\n"
-"useWithPrefix=volumeControls globaloptions_\n"
-"\n"
-"# audio tab\n"
-"opYoffset=vBorder\n"
-"gameoptions_audioCheckbox=gox opYoffset (kFontHeight + 10 + 180) buttonHeight\n"
-"opYoffset=(opYoffset + buttonHeight + 6)\n"
-"useWithPrefix=audioControls gameoptions_\n"
-"useWithPrefix=subtitleControls gameoptions_\n"
-"\n"
-"# volume tab\n"
-"opYoffset=vBorder\n"
-"gameoptions_volumeCheckbox=gox opYoffset (kFontHeight + 10 + 190) buttonHeight\n"
-"opYoffset=(opYoffset + buttonHeight + 6)\n"
-"useWithPrefix=volumeControls gameoptions_\n"
 "";
 
 using Common::String;
