@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2003-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/sound/audiocd.h $
- * $Id: audiocd.h 22244 2006-04-30 13:40:35Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/sound/audiocd.h $
+ * $Id: audiocd.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -31,12 +34,6 @@
 
 namespace Audio {
 
-class DigitalTrackInfo {
-public:
-	virtual void play(Audio::Mixer *mixer, Audio::SoundHandle *handle, int startFrame, int duration) = 0;
-	virtual ~DigitalTrackInfo() { }
-};
-
 
 class AudioCDManager : public Common::Singleton<AudioCDManager> {
 public:
@@ -48,6 +45,16 @@ public:
 		int numLoops;
 	};
 
+	/**
+	 * Start playback of the specified "CD" track. This method mimics
+	 * the interface of OSystem::playCD (which it in fact may call, if an Audio CD is
+	 * present), but also can play digital audio tracks in various formats.
+	 *
+	 * @param track			the track to play.
+	 * @param num_loops		how often playback should be repeated (-1 = infinitely often).
+	 * @param start_frame	the frame at which playback should start (75 frames = 1 second).
+	 * @param duration		the number of frames to play (0: play until end)
+	 */
 	void play(int track, int numLoops, int startFrame, int duration);
 	void stop();
 	bool isPlaying() const;
@@ -60,27 +67,13 @@ private:
 	friend class Common::Singleton<SingletonBaseType>;
 	AudioCDManager();
 
-	int getCachedTrack(int track);
-
-private:
 	/* used for emulated CD music */
 	struct ExtStatus : Status {
-		Audio::SoundHandle handle;
+		SoundHandle handle;
 	};
 	ExtStatus _cd;
 
-	enum {
-#if defined(__PSP__)
-		CACHE_TRACKS = 4	//the PSP can't have more than 8 files open simultaneously
-					//so don't use more than 4 filehandles for CD tracks
-#else
-		CACHE_TRACKS = 10
-#endif
-	};
-	int _cachedTracks[CACHE_TRACKS];
-	DigitalTrackInfo *_trackInfo[CACHE_TRACKS];
-	int _currentCache;
-
+	Mixer	*_mixer;
 };
 
 /** Shortcut for accessing the audio CD manager. */
