@@ -1,6 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 1999-2000 Tatsuyuki Satoh
- * Copyright (C) 2001-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -16,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/sound/fmopl.cpp $
- * $Id: fmopl.cpp 24241 2006-10-08 22:47:29Z djwillis $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/sound/fmopl.cpp $
+ * $Id: fmopl.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  * LGPL licensed version of MAMEs fmopl (V0.37a modified) by
  * Tatsuyuki Satoh. Included from LGPL'ed AdPlug.
@@ -35,7 +37,7 @@
 
 #include "common/util.h"
 
-#if defined (_WIN32_WCE) || defined (__SYMBIAN32__) || defined(PALMOS_MODE) || defined(__GP32__) || defined (GP2X) || defined (__MAEMO__) || defined(__DS__)
+#if defined (_WIN32_WCE) || defined (__SYMBIAN32__) || defined(PALMOS_MODE) || defined(__GP32__) || defined(GP2X) || defined (__MAEMO__) || defined(__DS__)
 #include "common/config-manager.h"
 #endif
 
@@ -61,7 +63,12 @@ static Common::RandomSource oplRnd;			/* OPL random number generator */
 
 /* sinwave entries */
 /* used static memory = SIN_ENT * 4 (byte) */
+#ifdef __DS__
+#include "dsmain.h"
+#define SIN_ENT 256
+#else
 #define SIN_ENT 2048
+#endif
 
 /* output level entries (envelope,sinwave) */
 /* envelope counter lower bits */
@@ -99,8 +106,7 @@ int EG_AED;
 #define ENV_MOD_AR  0x02
 
 /* -------------------- tables --------------------- */
-static const int slot_array[32]=
-{
+static const int slot_array[32] = {
 	 0, 2, 4, 1, 3, 5,-1,-1,
 	 6, 8,10, 7, 9,11,-1,-1,
 	12,14,16,13,15,17,-1,-1,
@@ -152,7 +158,7 @@ static const double KSL_TABLE_SEED[8 * 16] = {
 	19.875, 20.250, 20.625, 21.000
 };
 
-/* sustain lebel table (3db per step) */
+/* sustain level table (3db per step) */
 /* 0 - 15: 0, 3, 6, 9,12,15,18,21,24,27,30,33,36,39,42,93 (dB)*/
 
 static int SL_TABLE[16];
@@ -300,6 +306,7 @@ inline void OPL_KEYON(OPL_SLOT *SLOT) {
 	SLOT->evc = EG_AST;
 	SLOT->eve = EG_AED;
 }
+
 /* ----- key off ----- */
 inline void OPL_KEYOFF(OPL_SLOT *SLOT) {
 	if( SLOT->evm > ENV_MOD_RR) {
@@ -343,11 +350,12 @@ inline void OPL_KEYOFF(OPL_SLOT *SLOT) {
 }
 
 /* ---------- calcrate Envelope Generator & Phase Generator ---------- */
+
 /* return : envelope output */
 inline uint OPL_CALC_SLOT(OPL_SLOT *SLOT) {
 	/* calcrate envelope generator */
 	if((SLOT->evc += SLOT->evs) >= SLOT->eve) {
-		switch( SLOT->evm ){
+		switch( SLOT->evm ) {
 		case ENV_MOD_AR: /* ATTACK -> DECAY1 */
 			/* next DR */
 			SLOT->evm = ENV_MOD_DR;
@@ -391,8 +399,7 @@ inline void CALC_FCSLOT(OPL_CH *CH, OPL_SLOT *SLOT) {
 	SLOT->Incr = CH->fc * SLOT->mul;
 	ksr = CH->kcode >> SLOT->KSR;
 
-	if( SLOT->ksr != ksr )
-	{
+	if( SLOT->ksr != ksr ) {
 		SLOT->ksr = ksr;
 		/* attack , decay rate recalcration */
 		SLOT->evsa = SLOT->AR[ksr];
@@ -464,6 +471,7 @@ inline void set_sl_rr(FM_OPL *OPL, int slot, int v) {
 }
 
 /* operator output calcrator */
+
 #define OP_OUT(slot,env,con)   slot->wavetable[((slot->Cnt + con) / (0x1000000 / SIN_ENT)) & (SIN_ENT-1)][env]
 /* ---------- calcrate one of channel ---------- */
 inline void OPL_CALC_CH(OPL_CH *CH) {
@@ -485,11 +493,10 @@ inline void OPL_CALC_CH(OPL_CH *CH) {
 			int feedback1 = (CH->op1_out[0] + CH->op1_out[1]) >> CH->FB;
 			CH->op1_out[1] = CH->op1_out[0];
 			*CH->connect1 += CH->op1_out[0] = OP_OUT(SLOT, env_out, feedback1);
-		}
-		else 		{
+		} else {
 			*CH->connect1 += OP_OUT(SLOT, env_out, 0);
 		}
-	}else {
+	} else {
 		CH->op1_out[1] = CH->op1_out[0];
 		CH->op1_out[0] = 0;
 	}
@@ -538,7 +545,7 @@ inline void OPL_CALC_RH(OPL_CH *CH) {
 		else {
 			feedback2 = OP_OUT(SLOT, env_out, 0);
 		}
-	}else {
+	} else {
 		feedback2 = 0;
 		CH[6].op1_out[1] = CH[6].op1_out[0];
 		CH[6].op1_out[0] = 0;
@@ -607,7 +614,7 @@ static void init_timetables(FM_OPL *OPL, int ARRATE, int DRRATE) {
 	/* make attack rate & decay rate tables */
 	for (i = 0; i < 4; i++)
 		OPL->AR_TABLE[i] = OPL->DR_TABLE[i] = 0;
-	for (i = 4; i <= 60; i++){
+	for (i = 4; i <= 60; i++) {
 		rate = OPL->freqbase;						/* frequency rate */
 		if(i < 60)
 			rate *= 1.0 + (i & 3) * 0.25;		/* b0-1 : x1 , x1.25 , x1.5 , x1.75 */
@@ -634,24 +641,29 @@ static int OPLOpenTable(void) {
 		ENV_CURVE = (int *)calloc(2 * 4096 + 1, sizeof(int));
 #endif
 
+#ifdef __DS__
+	DS::fastRamReset();
+
+	TL_TABLE = (int *) DS::fastRamAlloc(TL_MAX * 2 * sizeof(int *));
+	SIN_TABLE = (int **) DS::fastRamAlloc(SIN_ENT * 4 * sizeof(int *));
+#else
+
 	/* allocate dynamic tables */
 	if((TL_TABLE = (int *)malloc(TL_MAX * 2 * sizeof(int))) == NULL)
 		return 0;
-#ifdef __DS__
-	// On the DS, use fast RAM for the sine table, since it thrashes the cache otherwise
-	// Unfotunately, this doesn't stop OPL emulation from taking up far too much CPU time, but it helps a fair bit.
-	SIN_TABLE = ((int **) (0x37F8000));
-#else
+
 	if((SIN_TABLE = (int **)malloc(SIN_ENT * 4 * sizeof(int *))) == NULL) {
 		free(TL_TABLE);
 		return 0;
 	}
 #endif
+
 	if((AMS_TABLE = (int *)malloc(AMS_ENT * 2 * sizeof(int))) == NULL) {
 		free(TL_TABLE);
 		free(SIN_TABLE);
 		return 0;
 	}
+
 	if((VIB_TABLE = (int *)malloc(VIB_ENT * 2 * sizeof(int))) == NULL) {
 		free(TL_TABLE);
 		free(SIN_TABLE);
@@ -659,20 +671,20 @@ static int OPLOpenTable(void) {
 		return 0;
 	}
 	/* make total level table */
-	for (t = 0; t < EG_ENT - 1 ; t++){
+	for (t = 0; t < EG_ENT - 1 ; t++) {
 		rate = ((1 << TL_BITS) - 1) / pow(10.0, EG_STEP * t / 20);	/* dB -> voltage */
 		TL_TABLE[         t] =  (int)rate;
 		TL_TABLE[TL_MAX + t] = -TL_TABLE[t];
 	}
 	/* fill volume off area */
-	for (t = EG_ENT - 1; t < TL_MAX; t++){
+	for (t = EG_ENT - 1; t < TL_MAX; t++) {
 		TL_TABLE[t] = TL_TABLE[TL_MAX + t] = 0;
 	}
 
 	/* make sinwave table (total level offet) */
 	/* degree 0 = degree 180                   = off */
-	SIN_TABLE[0] = SIN_TABLE[SIN_ENT /2 ]         = &TL_TABLE[EG_ENT - 1];
-	for (s = 1;s <= SIN_ENT / 4; s++){
+	SIN_TABLE[0] = SIN_TABLE[SIN_ENT /2 ] = &TL_TABLE[EG_ENT - 1];
+	for (s = 1;s <= SIN_ENT / 4; s++) {
 		pom = sin(2 * PI * s / SIN_ENT); /* sin     */
 		pom = 20 * log10(1 / pom);	   /* decibel */
 		j = int(pom / EG_STEP);         /* TL_TABLE steps */
@@ -793,8 +805,7 @@ void OPLWriteReg(FM_OPL *OPL, int r, int v) {
 		case 0x04:	/* IRQ clear / mask and Timer enable */
 			if(v & 0x80) {	/* IRQ flag clear */
 				OPL_STATUS_RESET(OPL, 0x7f);
-			}
-			else {	/* set IRQ mask ,timer enable*/
+			} else {	/* set IRQ mask ,timer enable*/
 				uint8 st1 = v & 1;
 				uint8 st2 = (v >> 1) & 1;
 				/* IRQRST,T1MSK,t2MSK,EOSMSK,BRMSK,x,ST2,ST1 */
@@ -856,8 +867,7 @@ void OPLWriteReg(FM_OPL *OPL, int r, int v) {
 						OPL->P_CH[6].op1_out[0] = OPL->P_CH[6].op1_out[1] = 0;
 						OPL_KEYON(&OPL->P_CH[6].SLOT[SLOT1]);
 						OPL_KEYON(&OPL->P_CH[6].SLOT[SLOT2]);
-					}
-					else {
+					} else {
 						OPL_KEYOFF(&OPL->P_CH[6].SLOT[SLOT1]);
 						OPL_KEYOFF(&OPL->P_CH[6].SLOT[SLOT2]);
 					}
@@ -892,6 +902,9 @@ void OPLWriteReg(FM_OPL *OPL, int r, int v) {
 			}
 			}
 			return;
+
+		default:
+			break;
 		}
 		/* keyon,block,fnum */
 		if((r & 0x0f) > 8)
@@ -899,8 +912,7 @@ void OPLWriteReg(FM_OPL *OPL, int r, int v) {
 		CH = &OPL->P_CH[r & 0x0f];
 		if(!(r&0x10)) {	/* a0-a8 */
 			block_fnum  = (CH->block_fnum & 0x1f00) | v;
-		}
-		else {	/* b0-b8 */
+		} else {	/* b0-b8 */
 			int keyon = (v >> 5) & 1;
 			block_fnum = ((v & 0x1f) << 8) | (CH->block_fnum & 0xff);
 			if(CH->keyon != keyon) {
@@ -908,8 +920,7 @@ void OPLWriteReg(FM_OPL *OPL, int r, int v) {
 					CH->op1_out[0] = CH->op1_out[1] = 0;
 					OPL_KEYON(&CH->SLOT[SLOT1]);
 					OPL_KEYON(&CH->SLOT[SLOT2]);
-				}
-				else {
+				} else {
 					OPL_KEYOFF(&CH->SLOT[SLOT1]);
 					OPL_KEYOFF(&CH->SLOT[SLOT2]);
 				}
@@ -954,8 +965,7 @@ void OPLWriteReg(FM_OPL *OPL, int r, int v) {
 }
 
 /* lock/unlock for common table */
-static int OPL_LockTable(void)
-{
+static int OPL_LockTable(void) {
 	num_lock++;
 	if(num_lock>1)
 		return 0;
@@ -993,7 +1003,8 @@ void YM3812UpdateOne(FM_OPL *OPL, int16 *buffer, int length) {
 	uint8 rythm = OPL->rythm & 0x20;
 	OPL_CH *CH, *R_CH;
 
-	if((void *)OPL != cur_chip){
+
+	if((void *)OPL != cur_chip) {
 		cur_chip = (void *)OPL;
 		/* channel pointers */
 		S_CH = OPL->P_CH;
@@ -1107,7 +1118,6 @@ void OPLDestroy(FM_OPL *OPL) {
 }
 
 /* ----------  Option handlers ----------       */
-
 void OPLSetTimerHandler(FM_OPL *OPL, OPL_TIMERHANDLER TimerHandler,int channelOffset) {
 	OPL->TimerHandler   = TimerHandler;
 	OPL->TimerParam = channelOffset;
@@ -1117,6 +1127,7 @@ void OPLSetIRQHandler(FM_OPL *OPL, OPL_IRQHANDLER IRQHandler, int param) {
 	OPL->IRQHandler     = IRQHandler;
 	OPL->IRQParam = param;
 }
+
 void OPLSetUpdateHandler(FM_OPL *OPL, OPL_UPDATEHANDLER UpdateHandler,int param) {
 	OPL->UpdateHandler = UpdateHandler;
 	OPL->UpdateParam = param;
@@ -1126,8 +1137,7 @@ void OPLSetUpdateHandler(FM_OPL *OPL, OPL_UPDATEHANDLER UpdateHandler,int param)
 int OPLWrite(FM_OPL *OPL,int a,int v) {
 	if(!(a & 1)) {	/* address port */
 		OPL->address = v & 0xff;
-	}
-	else {	/* data port */
+	} else {	/* data port */
 		if(OPL->UpdateHandler)
 			OPL->UpdateHandler(OPL->UpdateParam,0);
 		OPLWriteReg(OPL, OPL->address,v);
@@ -1149,6 +1159,8 @@ unsigned char OPLRead(FM_OPL *OPL,int a) {
 		return 0;
 	case 0x1a: /* PCM-DATA    */
 		return 0;
+	default:
+		break;
 	}
 	return 0;
 }
@@ -1156,8 +1168,7 @@ unsigned char OPLRead(FM_OPL *OPL,int a) {
 int OPLTimerOver(FM_OPL *OPL, int c) {
 	if(c) {	/* Timer B */
 		OPL_STATUS_SET(OPL, 0x20);
-	}
-	else {	/* Timer A */
+	} else {	/* Timer A */
 		OPL_STATUS_SET(OPL, 0x40);
 		/* CSM mode key,TL controll */
 		if(OPL->mode & 0x80) {	/* CSM mode total level latch and auto key on */
@@ -1182,13 +1193,10 @@ FM_OPL *makeAdlibOPL(int rate) {
 	if (ConfMan.hasKey("FM_high_quality") && ConfMan.getBool("FM_high_quality")) {
 		env_bits = FMOPL_ENV_BITS_HQ;
 		eg_ent = FMOPL_EG_ENT_HQ;
-	}
-	else
-	if (ConfMan.hasKey("FM_medium_quality") && ConfMan.getBool("FM_medium_quality")) {
+	} else if (ConfMan.hasKey("FM_medium_quality") && ConfMan.getBool("FM_medium_quality")) {
 		env_bits = FMOPL_ENV_BITS_MQ;
 		eg_ent = FMOPL_EG_ENT_MQ;
-	}
-	else {
+	} else {
 		env_bits = FMOPL_ENV_BITS_LQ;
 		eg_ent = FMOPL_EG_ENT_LQ;
 	}
