@@ -1,6 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2001  Ludvig Strigeus
- * Copyright (C) 2001-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -16,16 +18,17 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/sound/mididrv.cpp $
- * $Id: mididrv.cpp 21472 2006-03-28 09:42:54Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/sound/mididrv.cpp $
+ * $Id: mididrv.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
 #include "common/stdafx.h"
 
-#include "base/engine.h"
+#include "engines/engine.h"
 #include "common/config-manager.h"
 #include "common/str.h"
+#include "common/system.h"
 #include "common/util.h"
 #include "sound/mididrv.h"
 
@@ -208,12 +211,12 @@ int MidiDriver::detectMusicDriver(int flags) {
 }
 
 MidiDriver *MidiDriver::createMidi(int midiDriver) {
-	switch(midiDriver) {
+	switch (midiDriver) {
 	case MD_NULL:      return MidiDriver_NULL_create();
 
-	case MD_ADLIB:     return MidiDriver_ADLIB_create(g_engine->_mixer);
+	case MD_ADLIB:     return MidiDriver_ADLIB_create(g_system->getMixer());
 
-	case MD_TOWNS:     return MidiDriver_YM2612_create(g_engine->_mixer);
+	case MD_TOWNS:     return MidiDriver_YM2612_create(g_system->getMixer());
 
 	// Right now PC Speaker and PCjr are handled
 	// outside the MidiDriver architecture, so
@@ -222,11 +225,11 @@ MidiDriver *MidiDriver::createMidi(int midiDriver) {
 	case MD_PCJR:      return NULL;
 
 #ifdef USE_FLUIDSYNTH
-	case MD_FLUIDSYNTH:	return MidiDriver_FluidSynth_create(g_engine->_mixer);
+	case MD_FLUIDSYNTH:	return MidiDriver_FluidSynth_create(g_system->getMixer());
 #endif
 
 #ifdef USE_MT32EMU
-	case MD_MT32:      return MidiDriver_MT32_create(g_engine->_mixer);
+	case MD_MT32:      return MidiDriver_MT32_create(g_system->getMixer());
 #endif
 
 #if defined(PALMOS_MODE)
