@@ -1,6 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2001  Ludvig Strigeus
- * Copyright (C) 2001-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -16,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/sound/mididrv.h $
- * $Id: mididrv.h 22413 2006-05-12 12:03:36Z eriktorbjorn $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/sound/mididrv.h $
+ * $Id: mididrv.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -165,8 +167,24 @@ public:
 	/** Close the midi driver. */
 	virtual void close() = 0;
 
-	/** Output a packed midi command to the midi stream. */
+	/**
+	 * Output a packed midi command to the midi stream.
+	 * The 'lowest' byte (i.e. b & 0xFF) is the status
+	 * code, then come (if used) the first and second
+	 * opcode.
+	 */
 	virtual void send(uint32 b) = 0;
+	
+	/**
+	 * Output a midi command to the midi stream. Convenience wrapper
+	 * around the usual 'packed' send method.
+	 *
+	 * Do NOT use this for sysEx transmission; instead, use the sysEx()
+	 * method below.
+	 */
+	void send(byte status, byte firstOp, byte secondOp) {
+		send(status | ((uint32)firstOp << 8) | ((uint32)secondOp << 16));
+	}
 
 	/** Get or set a property. */
 	virtual uint32 property(int prop, uint32 param) { return 0; }
@@ -175,22 +193,33 @@ public:
 	static const char *getErrorName(int error_code);
 
 	// HIGH-LEVEL SEMANTIC METHODS
-	virtual void setPitchBendRange(byte channel, uint range)
-	{
-		send((  0   << 16) | (101 << 8) | (0xB0 | channel));
-		send((  0   << 16) | (100 << 8) | (0xB0 | channel));
-		send((range << 16) | (  6 << 8) | (0xB0 | channel));
-		send((  0   << 16) | ( 38 << 8) | (0xB0 | channel));
-		send(( 127  << 16) | (101 << 8) | (0xB0 | channel));
-		send(( 127  << 16) | (100 << 8) | (0xB0 | channel));
+	virtual void setPitchBendRange(byte channel, uint range) {
+		send(0xB0 | channel, 101, 0);
+		send(0xB0 | channel, 100, 0);
+		send(0xB0 | channel,   6, range);
+		send(0xB0 | channel,  38, 0);
+		send(0xB0 | channel, 101, 127);
+		send(0xB0 | channel, 100, 127);
 	}
 
+	/**
+	 * Transmit a sysEx to the midi device.
+	 *
+	 * The given msg MUST NOT contain the usual SysEx frame, i.e. 
+	 * do NOT include the leading 0xF0 and the trailing 0xF7.
+	 *
+	 * Furthermore, the maximal supported length of a SysEx
+	 * is 254 bytes. Passing longer buffers can lead to 
+	 * undefined behavior (most likely, a crash).
+	 */
 	virtual void sysEx(const byte *msg, uint16 length) { }
+
 	virtual void sysEx_customInstrument(byte channel, uint32 type, const byte *instr) { }
+
 	virtual void metaEvent(byte type, byte *data, uint16 length) { }
 
 	// Timing functions - MidiDriver now operates timers
-	virtual void setTimerCallback(void *timer_param, Common::Timer::TimerProc timer_proc) = 0;
+	virtual void setTimerCallback(void *timer_param, Common::TimerManager::TimerProc timer_proc) = 0;
 
 	/** The time in microseconds between invocations of the timer callback. */
 	virtual uint32 getBaseTempo(void) = 0;
