@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2001-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/sound/midiparser_smf.cpp $
- * $Id: midiparser_smf.cpp 21853 2006-04-13 23:13:56Z lordhoto $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/sound/midiparser_smf.cpp $
+ * $Id: midiparser_smf.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -124,6 +127,9 @@ void MidiParser_SMF::parseNextEvent(EventInfo &info) {
 			info.ext.data = _position._play_pos;
 			_position._play_pos += info.length;
 			break;
+		
+		default:
+			warning("MidiParser_SMF::parseNextEvent: Unsupported event code %x", info.event);
 		}
 	}
 }
