@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2001-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/sound/midiparser_xmidi.cpp $
- * $Id: midiparser_xmidi.cpp 21608 2006-04-04 22:21:10Z sev $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/sound/midiparser_xmidi.cpp $
+ * $Id: midiparser_xmidi.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -120,6 +123,9 @@ void MidiParser_XMIDI::parseNextEvent(EventInfo &info) {
 				info.ext.data[2] = 0x20;
 			}
 			break;
+		
+		default:
+			warning("MidiParser_XMIDI::parseNextEvent: Unsupported event code %x", info.event);
 		}
 	}
 }
