@@ -1,6 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2001  Ludvig Strigeus
- * Copyright (C) 2001-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -16,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/sound/mixer.cpp $
- * $Id: mixer.cpp 21557 2006-04-02 14:16:31Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/sound/mixer.cpp $
+ * $Id: mixer.cpp 27066 2007-06-03 15:56:14Z fingolfin $
  *
  */
 
@@ -30,9 +32,6 @@
 #include "sound/mixer.h"
 #include "sound/rate.h"
 #include "sound/audiostream.h"
-#include "sound/flac.h"
-#include "sound/mp3.h"
-#include "sound/vorbis.h"
 
 
 namespace Audio {
@@ -66,8 +65,6 @@ protected:
 	AudioStream *_input;
 
 public:
-
-	Channel(Mixer *mixer, Mixer::SoundType type, int id = -1);
 	Channel(Mixer *mixer, Mixer::SoundType type, AudioStream *input, bool autofreeStream, bool reverseStereo = false, int id = -1, bool permanent = false);
 	virtual ~Channel();
 
@@ -108,49 +105,24 @@ Mixer::Mixer() {
 
 	_handleSeed = 0;
 
-	_premixChannel = 0;
 	int i = 0;
 
 	for (i = 0; i < ARRAYSIZE(_volumeForSoundType); i++)
 		_volumeForSoundType[i] = kMaxMixerVolume;
 
-	_paused = false;
-
 	for (i = 0; i != NUM_CHANNELS; i++)
 		_channels[i] = 0;
 
-	_mixerReady = _syst->setSoundCallback(mixCallback, this);
-	_outputRate = (uint)_syst->getOutputSampleRate();
-
-	if (_outputRate == 0)
-		error("OSystem returned invalid sample rate");
-
-	debug(1, "Output sample rate: %d Hz", _outputRate);
+	_mixerReady = false;
 }
 
 Mixer::~Mixer() {
-	_syst->clearSoundCallback();
-	stopAll(true);
-
-	delete _premixChannel;
-	_premixChannel = 0;
+	for (int i = 0; i != NUM_CHANNELS; i++)
+		delete _channels[i];
 }
 
-bool Mixer::isPaused() {
-	return _paused;
-}
-
-void Mixer::setupPremix(AudioStream *stream, SoundType type) {
-	Common::StackLock lock(_mutex);
-
-	delete _premixChannel;
-	_premixChannel = 0;
-
-	if (stream == 0)
-		return;
-
-	// Create the channel
-	_premixChannel = new Channel(this, type, stream, false);
+uint Mixer::getOutputRate() const {
+	return (uint)_syst->getOutputSampleRate();
 }
 
 void Mixer::insertChannel(SoundHandle *handle, Channel *chan) {
@@ -176,42 +148,29 @@ void Mixer::insertChannel(SoundHandle *handle, Channel *chan) {
 	}
 }
 
-void Mixer::playRaw(SoundHandle *handle, void *sound, uint32 size, uint rate, byte flags,
-			int id, byte volume, int8 balance, uint32 loopStart, uint32 loopEnd, SoundType type) {
-	Common::StackLock lock(_mutex);
-
-	// Prevent duplicate sounds
-	if (id != -1) {
-		for (int i = 0; i != NUM_CHANNELS; i++)
-			if (_channels[i] != 0 && _channels[i]->getId() == id) {
-				if ((flags & Mixer::FLAG_AUTOFREE) != 0)
-					free(sound);
-				return;
-			}
-	}
+void Mixer::playRaw(
+			SoundType type,
+			SoundHandle *handle,
+			void *sound,
+			uint32 size, uint rate, byte flags,
+			int id, byte volume, int8 balance,
+			uint32 loopStart, uint32 loopEnd) {
 
 	// Create the input stream
-	AudioStream *input;
-	if (flags & Mixer::FLAG_LOOP) {
-		if (loopEnd == 0) {
-			input = makeLinearInputStream(rate, flags, (byte *)sound, size, 0, size);
-		} else {
-			assert(loopStart < loopEnd && loopEnd <= size);
-			input = makeLinearInputStream(rate, flags, (byte *)sound, size, loopStart, loopEnd - loopStart);
-		}
-	} else {
-		input = makeLinearInputStream(rate, flags, (byte *)sound, size, 0, 0);
-	}
-
-	// Create the channel
-	Channel *chan = new Channel(this, type, input, true, (flags & Mixer::FLAG_REVERSE_STEREO) != 0, id);
-	chan->setVolume(volume);
-	chan->setBalance(balance);
-	insertChannel(handle, chan);
+	AudioStream *input = makeLinearInputStream((byte *)sound, size, rate, flags, loopStart, loopEnd);
+	
+	// Play it
+	playInputStream(type, handle, input, id, volume, balance, true, false, ((flags & Mixer::FLAG_REVERSE_STEREO) != 0));
 }
 
-void Mixer::playInputStream(SoundType type, SoundHandle *handle, AudioStream *input,
-			int id, byte volume, int8 balance, bool autofreeStream, bool permanent) {
+void Mixer::playInputStream(
+			SoundType type,
+			SoundHandle *handle,
+			AudioStream *input,
+			int id, byte volume, int8 balance,
+			bool autofreeStream,
+			bool permanent,
+			bool reverseStereo) {
 	Common::StackLock lock(_mutex);
 
 	if (input == 0) {
@@ -230,7 +189,7 @@ void Mixer::playInputStream(SoundType type, SoundHandle *handle, AudioStream *in
 	}
 
 	// Create the channel
-	Channel *chan = new Channel(this, type, input, autofreeStream, false, id, permanent);
+	Channel *chan = new Channel(this, type, input, autofreeStream, reverseStereo, id, permanent);
 	chan->setVolume(volume);
 	chan->setBalance(balance);
 	insertChannel(handle, chan);
@@ -238,24 +197,22 @@ void Mixer::playInputStream(SoundType type, SoundHandle *handle, AudioStream *in
 
 void Mixer::mix(int16 *buf, uint len) {
 	Common::StackLock lock(_mutex);
+	
+	// Since the mixer callback has been called, the mixer must be ready...
+	_mixerReady = true;
 
 	//  zero the buf
 	memset(buf, 0, 2 * len * sizeof(int16));
 
-	if (!_paused) {
-		if (_premixChannel)
-			_premixChannel->mix(buf, len);
-
-		// now mix all channels
-		for (int i = 0; i != NUM_CHANNELS; i++)
-			if (_channels[i]) {
-				if (_channels[i]->isFinished()) {
-					delete _channels[i];
-					_channels[i] = 0;
-				} else if (!_channels[i]->isPaused())
-					_channels[i]->mix(buf, len);
-			}
-	}
+	// mix all channels
+	for (int i = 0; i != NUM_CHANNELS; i++)
+		if (_channels[i]) {
+			if (_channels[i]->isFinished()) {
+				delete _channels[i];
+				_channels[i] = 0;
+			} else if (!_channels[i]->isPaused())
+				_channels[i]->mix(buf, len);
+		}
 }
 
 void Mixer::mixCallback(void *s, byte *samples, int len) {
@@ -266,15 +223,14 @@ void Mixer::mixCallback(void *s, byte *samples, int len) {
 	((Mixer *)s)->mix((int16 *)samples, len >> 2);
 }
 
-void Mixer::stopAll(bool force) {
+void Mixer::stopAll() {
 	Common::StackLock lock(_mutex);
-	for (int i = 0; i != NUM_CHANNELS; i++)
-		if (_channels[i] != 0) {
-			if (force || !_channels[i]->isPermanent()) {
-				delete _channels[i];
-				_channels[i] = 0;
-			}
+	for (int i = 0; i != NUM_CHANNELS; i++) {
+		if (_channels[i] != 0 && !_channels[i]->isPermanent()) {
+			delete _channels[i];
+			_channels[i] = 0;
 		}
+	}
 }
 
 void Mixer::stopID(int id) {
@@ -319,14 +275,6 @@ void Mixer::setChannelBalance(SoundHandle handle, int8 balance) {
 	_channels[index]->setBalance(balance);
 }
 
-uint32 Mixer::getSoundElapsedTimeOfSoundID(int id) {
-	Common::StackLock lock(_mutex);
-	for (int i = 0; i != NUM_CHANNELS; i++)
-		if (_channels[i] && _channels[i]->getId() == id)
-			return _channels[i]->getElapsedTime();
-	return 0;
-}
-
 uint32 Mixer::getSoundElapsedTime(SoundHandle handle) {
 	Common::StackLock lock(_mutex);
 
@@ -338,7 +286,12 @@ uint32 Mixer::getSoundElapsedTime(SoundHandle handle) {
 }
 
 void Mixer::pauseAll(bool paused) {
-	_paused = paused;
+	Common::StackLock lock(_mutex);
+	for (int i = 0; i != NUM_CHANNELS; i++) {
+		if (_channels[i] != 0) {
+			_channels[i]->pause(paused);
+		}
+	}
 }
 
 void Mixer::pauseID(int id, bool paused) {
@@ -354,7 +307,7 @@ void Mixer::pauseID(int id, bool paused) {
 void Mixer::pauseHandle(SoundHandle handle, bool paused) {
 	Common::StackLock lock(_mutex);
 
-	// Simply ignore pause/unpause requests for handles of sound that alreayd terminated
+	// Simply ignore (un)pause requests for sounds that already terminated
 	const int index = handle._val % NUM_CHANNELS;
 	if (!_channels[index] || _channels[index]->_handle._val != handle._val)
 		return;
@@ -419,13 +372,6 @@ int Mixer::getVolumeForSoundType(SoundType type) const {
 #pragma mark -
 
 
-Channel::Channel(Mixer *mixer, Mixer::SoundType type, int id)
-	: _type(type), _mixer(mixer), _autofreeStream(true),
-	  _volume(Mixer::kMaxChannelVolume), _balance(0), _paused(false), _id(id), _samplesConsumed(0),
-	  _samplesDecoded(0), _mixerTimeStamp(0), _converter(0), _input(0) {
-	assert(mixer);
-}
-
 Channel::Channel(Mixer *mixer, Mixer::SoundType type, AudioStream *input,
 				bool autofreeStream, bool reverseStereo, int id, bool permanent)
 	: _type(type), _mixer(mixer), _autofreeStream(autofreeStream),
