@@ -1,6 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2001  Ludvig Strigeus
- * Copyright (C) 2001-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -16,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/sound/mixer.h $
- * $Id: mixer.h 22231 2006-04-29 22:33:31Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/sound/mixer.h $
+ * $Id: mixer.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -38,24 +40,43 @@ class AudioStream;
 class Channel;
 class Mixer;
 
+/**
+ * A SoundHandle instances corresponds to a specific sound
+ * being played via the mixer. It can be used to control that
+ * sound (pause it, stop it, etc.).
+ * @see The Mixer class
+ */
 class SoundHandle {
-	friend class Audio::Channel;
-	friend class Audio::Mixer;
+	friend class Channel;
+	friend class Mixer;
 	uint32 _val;
 public:
 	inline SoundHandle() : _val(0xFFFFFFFF) {}
 };
 
+/**
+ * The main audio mixer handles mixing of an arbitrary number of
+ * input audio streams (in the form of AudioStream instances).
+ */
 class Mixer {
 public:
-	enum {
+	/**
+	 * Various flags which can be bit-ORed and then passed to
+	 * Mixer::playRaw resp. makeLinearInputStream to control their
+	 * behavior.
+	 *
+	 * Engine authors are advised not to rely on a certain value or
+	 * order of these flags (in particular, do not store them verbatim
+	 * in savestates).
+	 */
+	enum RawFlags {
 		/** unsigned samples (default: signed) */
 		FLAG_UNSIGNED = 1 << 0,
 
 		/** sound is 16 bits wide (default: 8bit) */
 		FLAG_16BITS = 1 << 1,
 
-		/** sample is little endian (default: big endian) */
+		/** samples are little endian (default: big endian) */
 		FLAG_LITTLE_ENDIAN = 1 << 2,
 
 		/** sound is in stereo (default: mono) */
@@ -92,14 +113,8 @@ private:
 	OSystem *_syst;
 	Common::Mutex _mutex;
 
-	Channel *_premixChannel;
-
-	uint _outputRate;
-
 	int _volumeForSoundType[4];
 
-	bool _paused;
-
 	uint32 _handleSeed;
 	Channel *_channels[NUM_CHANNELS];
 
@@ -114,21 +129,12 @@ public:
 	/**
 	 * Is the mixer ready and setup? This may not be the case on systems which
 	 * don't support digital sound output. In that case, the mixer proc may
-	 * never be called. That in turn can cause breakage in games which use the
-	 * premix callback for syncing. In particular, the Adlib MIDI emulation...
+	 * never be called. That in turn can cause breakage in games which try to
+	 * sync with an audio stream. In particular, the Adlib MIDI emulation...
 	 *
 	 * @return whether the mixer is ready and setup
 	 */
-	bool isReady() const { return _mixerReady; };
-
-
-
-	/**
-	 * Set the premix stream. This is mainly used for the adlib music, but
-	 * is not limited to it. The premix stream is invoked by the mixer whenever
-	 * it needs to generate any data, before any other mixing takes place.
-	 */
-	void setupPremix(AudioStream *stream, SoundType type = kPlainSoundType);
+	bool isReady() const { return _mixerReady; }
 
 
 
@@ -138,25 +144,48 @@ public:
 	 * (using the makeLinearInputStream factory function), which is then
 	 * passed on to playInputStream.
 	 */
-	void playRaw(SoundHandle *handle,
-				void *sound, uint32 size, uint rate, byte flags,
-				int id = -1, byte volume = 255, int8 balance = 0,
-				uint32 loopStart = 0, uint32 loopEnd = 0,
-				SoundType type = kSFXSoundType);
+	void playRaw(
+		SoundType type,
+		SoundHandle *handle,
+		void *sound, uint32 size, uint rate, byte flags,
+		int id = -1, byte volume = 255, int8 balance = 0,
+		uint32 loopStart = 0, uint32 loopEnd = 0);
 
 	/**
 	 * Start playing the given audio input stream.
+	 *
+	 * Note that the sound id assigned below is unique. At most one stream
+	 * with a given idea can play at any given time. Trying to play a sound
+	 * with an id that is already in use causes the new sound to be not played.
+	 *
+	 * @param type	the type (voice/sfx/music) of the stream
+	 * @param handle	a SoundHandle which can be used to reference and control
+	 *                  the stream via suitable mixer methods
+	 * @param input	the actual AudioStream to be played
+	 * @param id	a unique id assigned to this stream
+	 * @param volume	the volume with which to play the sound, ranging from 0 to 255
+	 * @param balance	the balance with which to play the sound, ranging from -128 to 127
+	 * @param autofreeStream	a flag indicating whether the stream should be
+	 *                          freed after playback finished
+	 * @param permanent	a flag indicating whether a plain stopAll call should
+	 *                  not stop this particular stream
+	 * @param reverseStereo	a flag indicating whether left and right channels shall be swapped
 	 */
-	void playInputStream(SoundType type, SoundHandle *handle, AudioStream *input,
-				int id = -1, byte volume = 255, int8 balance = 0,
-				bool autofreeStream = true, bool permanent = false);
+	void playInputStream(
+		SoundType type,
+		SoundHandle *handle,
+		AudioStream *input,
+		int id = -1, byte volume = 255, int8 balance = 0,
+		bool autofreeStream = true,
+		bool permanent = false,
+		bool reverseStereo = false);
 
 
 
 	/**
 	 * Stop all currently playing sounds.
 	 */
-	void stopAll(bool force = false);
+	void stopAll();
 
 	/**
 	 * Stop playing the sound with given ID.
@@ -175,10 +204,10 @@ public:
 
 
 	/**
-	 * Pause/unpause the mixer (this temporarily stops all audio processing,
-	 * including all regular channels and the premix channel).
+	 * Pause/unpause all sounds, including all regular and permanent 
+	 * channels
 	 *
-	 * @param paused true to pause the mixer, false to unpause it
+	 * @param paused true to pause everything, false to unpause
 	 */
 	void pauseAll(bool paused);
 
@@ -217,20 +246,13 @@ public:
 	int getSoundID(SoundHandle handle);
 
 	/**
-	 * Check if a sound with the given hANDLE is active.
+	 * Check if a sound with the given handle is active.
 	 *
 	 * @param handle sound to query
 	 * @return true if the sound is active
 	 */
 	bool isSoundHandleActive(SoundHandle handle);
 
-	/**
-	 * Check if the mixer is paused (using pauseAll).
-	 *
-	 * @return true if the mixer is paused
-	 */
-	bool isPaused();
-
 
 
 	/**
@@ -250,11 +272,6 @@ public:
 	 */
 	void setChannelBalance(SoundHandle handle, int8 balance);
 
-	/**
-	 * Get approximation of for how long the Sound ID has been playing.
-	 */
-	uint32 getSoundElapsedTimeOfSoundID(int id);
-
 	/**
 	 * Get approximation of for how long the channel has been playing.
 	 */
@@ -292,9 +309,9 @@ public:
 	 *
 	 * @return the output sample rate in Hz
 	 */
-	uint getOutputRate() const { return _outputRate; }
+	uint getOutputRate() const;
 
-private:
+protected:
 	void insertChannel(SoundHandle *handle, Channel *chan);
 
 	/**
@@ -302,11 +319,16 @@ private:
 	 */
 	void mix(int16 * buf, uint len);
 
+	// FIXME: temporary "public" to allow access to mixCallback
+	// from within OSystem::makeMixer()
+public:
 	/**
 	 * The mixer callback function, passed on to OSystem::setSoundCallback().
 	 * This simply calls the mix() method.
 	 */
 	static void mixCallback(void *s, byte *samples, int len);
+	
+	void setReady(bool ready) { _mixerReady = ready; }
 };
 
 
