@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2001-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,12 +18,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/sound/mpu401.cpp $
- * $Id: mpu401.cpp 22413 2006-05-12 12:03:36Z eriktorbjorn $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/sound/mpu401.cpp $
+ * $Id: mpu401.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  */
 
 #include "common/stdafx.h"
 #include "sound/mpu401.h"
+#include "common/system.h"
 #include "common/timer.h"
 #include "common/util.h"	// for ARRAYSIZE
 
@@ -41,7 +45,7 @@ MidiDriver *MidiChannel_MPU401::device() {
 }
 
 void MidiChannel_MPU401::send(uint32 b) {
-	_owner->send (b & 0xFFFFFFF0 | (_channel & 0xF));
+	_owner->send(b & 0xFFFFFFF0 | (_channel & 0xF));
 }
 
 void MidiChannel_MPU401::noteOff(byte note) {
@@ -100,17 +104,17 @@ MidiDriver_MPU401::MidiDriver_MPU401() :
 
 void MidiDriver_MPU401::close() {
 	if (_timer_proc)
-		Common::g_timer->removeTimerProc(_timer_proc);
+		g_system->getTimerManager()->removeTimerProc(_timer_proc);
 	_timer_proc = 0;
 	for (int i = 0; i < 16; ++i)
-		send (0x7B << 8 | 0xB0 | i);
+		send(0x7B << 8 | 0xB0 | i);
 }
 
 uint32 MidiDriver_MPU401::property(int prop, uint32 param) {
 	switch (prop) {
-		case PROP_CHANNEL_MASK:
-			_channel_mask = param & 0xFFFF;
-			return 1;
+	case PROP_CHANNEL_MASK:
+		_channel_mask = param & 0xFFFF;
+		return 1;
 	}
 
 	return 0;
@@ -131,12 +135,12 @@ MidiChannel *MidiDriver_MPU401::allocateChannel() {
 	return NULL;
 }
 
-void MidiDriver_MPU401::setTimerCallback(void *timer_param, Common::Timer::TimerProc timer_proc) {
+void MidiDriver_MPU401::setTimerCallback(void *timer_param, Common::TimerManager::TimerProc timer_proc) {
 	if (!_timer_proc || !timer_proc) {
 		if (_timer_proc)
-			Common::g_timer->removeTimerProc(_timer_proc);
+			g_system->getTimerManager()->removeTimerProc(_timer_proc);
 		_timer_proc = timer_proc;
 		if (timer_proc)
-			Common::g_timer->installTimerProc(timer_proc, 10000, timer_param);
+			g_system->getTimerManager()->installTimerProc(timer_proc, 10000, timer_param);
 	}
 }
