@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2001-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/sound/mpu401.h $
- * $Id: mpu401.h 22413 2006-05-12 12:03:36Z eriktorbjorn $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/sound/mpu401.h $
+ * $Id: mpu401.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -71,14 +74,14 @@ public:
 class MidiDriver_MPU401 : public MidiDriver {
 private:
 	MidiChannel_MPU401 _midi_channels[16];
-	Common::Timer::TimerProc _timer_proc;
+	Common::TimerManager::TimerProc _timer_proc;
 	uint16 _channel_mask;
 
 public:
 	MidiDriver_MPU401();
 
 	virtual void close();
-	void setTimerCallback(void *timer_param, Common::Timer::TimerProc timer_proc);
+	void setTimerCallback(void *timer_param, Common::TimerManager::TimerProc timer_proc);
 	uint32 getBaseTempo(void) { return 10000; }
 	uint32 property(int prop, uint32 param);
 
