@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2001-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/sound/softsynth/adlib.cpp $
- * $Id: adlib.cpp 22951 2006-06-06 11:07:03Z kirben $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/sound/softsynth/adlib.cpp $
+ * $Id: adlib.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  */
 
 #include "sound/softsynth/emumidi.h"
@@ -234,14 +237,14 @@ static const AdlibSetParams adlib_setparam_table[] = {
 	{0xC0, 1, 14, 0}    // feedback
 };
 
-const byte param_table_1[16] = {
+static const byte param_table_1[16] = {
 	29, 28, 27, 0,
 	3, 4, 7, 8,
 	13, 16, 17, 20,
 	21, 30, 31, 0
 };
 
-const uint16 maxval_table[16] = {
+static const uint16 maxval_table[16] = {
 	0x2FF, 0x1F, 0x7, 0x3F,
 	0x0F, 0x0F, 0x0F, 0x3,
 	0x3F, 0x0F, 0x0F, 0x0F,
@@ -473,6 +476,7 @@ static const byte gm_percussion_lookup[128] = {
 };
 
 static byte lookup_table[64][32];
+
 static const byte volume_table[] = {
 	0, 4, 7, 11,
 	13, 16, 18, 20,
@@ -564,7 +568,7 @@ public:
 	int getRate() const { return _mixer->getOutputRate(); }
 
 private:
-	bool _game_SmallHeader;
+	bool _scummSmallHeader;	// FIXME: This flag controls a special mode for SCUMM V3 games
 
 	FM_OPL *_opl;
 	byte *_adlib_reg_cache;
@@ -620,6 +624,7 @@ void AdlibPart::init(MidiDriver_ADLIB *owner, byte channel) {
 	_owner = owner;
 	_channel = channel;
 	_pri_eff = 127;
+	programChange(0);
 }
 
 MidiDriver *AdlibPart::device() {
@@ -803,7 +808,7 @@ MidiDriver_ADLIB::MidiDriver_ADLIB(Audio::Mixer *mixer)
 	: MidiDriver_Emulated(mixer) {
 	uint i;
 
-	_game_SmallHeader = false;
+	_scummSmallHeader = false;
 
 	_adlib_reg_cache = 0;
 
@@ -845,7 +850,7 @@ int MidiDriver_ADLIB::open() {
 	adlib_write(0xBD, 0x00);
 	create_lookup_table();
 
-	_mixer->setupPremix(this);
+	_mixer->playInputStream(Audio::Mixer::kPlainSoundType, &_mixerSoundHandle, this, -1, Audio::Mixer::kMaxChannelVolume, 0, false, true);
 
 	return 0;
 }
@@ -855,8 +860,7 @@ void MidiDriver_ADLIB::close() {
 		return;
 	_isOpen = false;
 
-	// Detach the premix callback handler
-	_mixer->setupPremix(0);
+	_mixer->stopHandle(_mixerSoundHandle);
 
 	uint i;
 	for (i = 0; i < ARRAYSIZE(_voices); ++i) {
@@ -920,8 +924,8 @@ void MidiDriver_ADLIB::send(byte chan, uint32 b) {
 uint32 MidiDriver_ADLIB::property(int prop, uint32 param) {
 	switch (prop) {
 		case PROP_OLD_ADLIB: // Older games used a different operator volume algorithm
-			_game_SmallHeader = (param > 0);
-			if (_game_SmallHeader) {
+			_scummSmallHeader = (param > 0);
+			if (_scummSmallHeader) {
 				_timer_p = 473;
 				_timer_q = 1000;
 			} else {
@@ -1039,7 +1043,7 @@ void MidiDriver_ADLIB::mc_inc_stuff(AdlibVoice *voice, Struct10 *s10, Struct11 *
 		switch (s11->param) {
 		case 0:
 			voice->_vol_2 = s10->start_value + s11->modify_val;
-			if (!_game_SmallHeader) {
+			if (!_scummSmallHeader) {
 				adlib_set_param(voice->_channel, 0,
 												volume_table[lookup_table[voice->_vol_2]
 																		 [part->_vol_eff >> 2]]);
@@ -1049,7 +1053,7 @@ void MidiDriver_ADLIB::mc_inc_stuff(AdlibVoice *voice, Struct10 *s10, Struct11 *
 			break;
 		case 13:
 			voice->_vol_1 = s10->start_value + s11->modify_val;
-			if (voice->_twochan && !_game_SmallHeader) {
+			if (voice->_twochan && !_scummSmallHeader) {
 				adlib_set_param(voice->_channel, 13,
 												volume_table[lookup_table[voice->_vol_1]
 												[part->_vol_eff >> 2]]);
@@ -1251,10 +1255,6 @@ void MidiDriver_ADLIB::adlib_playnote(int channel, int note) {
 	adlib_write(channel + 0xB0, oct | 0x20);
 }
 
-// TODO: Replace this with RandomSource? But if so, please note that this
-// function will be called with negative parameters - getRandomNumber(-1) will
-// crash ScummVM, random_nr(-1) won't.
-
 int MidiDriver_ADLIB::random_nr(int a) {
 	static byte _rand_seed = 1;
 	if (_rand_seed & 1) {
@@ -1308,8 +1308,8 @@ AdlibVoice *MidiDriver_ADLIB::allocate_voice(byte pri) {
 		}
 	}
 
-	/* V3 games don't have note priorities, first comes wins. */
-	if (_game_SmallHeader)
+	/* SCUMM V3 games don't have note priorities, first comes wins. */
+	if (_scummSmallHeader)
 		return NULL;
 
 	if (best)
@@ -1339,7 +1339,7 @@ void MidiDriver_ADLIB::mc_key_on(AdlibVoice *voice, AdlibInstrument *instr, byte
 	if (voice->_duration != 0)
 		voice->_duration *= 63;
 
-	if (!_game_SmallHeader)
+	if (!_scummSmallHeader)
 		vol_1 = (instr->mod_scalingOutputLevel & 0x3F) + lookup_table[velocity >> 1][instr->mod_waveformSelect >> 2];
 	else
 		vol_1 = 0x3f - (instr->mod_scalingOutputLevel & 0x3F);
@@ -1347,7 +1347,7 @@ void MidiDriver_ADLIB::mc_key_on(AdlibVoice *voice, AdlibInstrument *instr, byte
 		vol_1 = 0x3F;
 	voice->_vol_1 = vol_1;
 
-	if (!_game_SmallHeader)
+	if (!_scummSmallHeader)
 		vol_2 = (instr->car_scalingOutputLevel & 0x3F) + lookup_table[velocity >> 1][instr->car_waveformSelect >> 2];
 	else
 		vol_2 = 0x3f - (instr->car_scalingOutputLevel & 0x3F);
@@ -1357,7 +1357,7 @@ void MidiDriver_ADLIB::mc_key_on(AdlibVoice *voice, AdlibInstrument *instr, byte
 
 	c = part->_vol_eff >> 2;
 
-	if (!_game_SmallHeader) {
+	if (!_scummSmallHeader) {
 		vol_2 = volume_table[lookup_table[vol_2][c]];
 		if (voice->_twochan)
 			vol_1 = volume_table[lookup_table[vol_1][c]];
@@ -1451,7 +1451,7 @@ void MidiDriver_ADLIB::mc_init_stuff(AdlibVoice *voice, Struct10 * s10,
 
 void MidiDriver_ADLIB::struct10_init(Struct10 *s10, InstrumentExtra *ie) {
 	s10->active = 1;
-	if (!_game_SmallHeader) {
+	if (!_scummSmallHeader) {
 		s10->cur_val = 0;
 	} else {
 		s10->cur_val = s10->start_value;
