@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2001-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/sound/softsynth/emumidi.h $
- * $Id: emumidi.h 22231 2006-04-29 22:33:31Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/sound/softsynth/emumidi.h $
+ * $Id: emumidi.h 27024 2007-05-30 21:56:52Z fingolfin $
  */
 
 #include "common/stdafx.h"
@@ -30,9 +33,10 @@ class MidiDriver_Emulated : public Audio::AudioStream, public MidiDriver {
 protected:
 	bool _isOpen;
 	Audio::Mixer *_mixer;
+	Audio::SoundHandle _mixerSoundHandle;
 
 private:
-	Common::Timer::TimerProc _timerProc;
+	Common::TimerManager::TimerProc _timerProc;
 	void *_timerParam;
 
 	int _nextTick;
@@ -70,7 +74,7 @@ public:
 		return 0;
 	}
 
-	void setTimerCallback(void *timer_param, Common::Timer::TimerProc timer_proc) {
+	void setTimerCallback(void *timer_param, Common::TimerManager::TimerProc timer_proc) {
 		_timerProc = timer_proc;
 		_timerParam = timer_param;
 	}
