@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2001-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/sound/softsynth/mt32.cpp $
- * $Id: mt32.cpp 21607 2006-04-04 22:04:51Z sev $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/sound/softsynth/mt32.cpp $
+ * $Id: mt32.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  */
 
 #include "common/stdafx.h"
@@ -29,10 +32,11 @@
 #include "sound/softsynth/emumidi.h"
 #include "sound/mpu401.h"
 
-#include "common/util.h"
-#include "common/file.h"
 #include "common/config-manager.h"
+#include "common/events.h"
+#include "common/file.h"
 #include "common/system.h"
+#include "common/util.h"
 
 #include "graphics/fontman.h"
 #include "graphics/surface.h"
@@ -75,7 +79,7 @@ public:
 	int getRate() const { return _outputRate; }
 };
 
-class MT32File: public MT32Emu::File {
+class MT32File : public MT32Emu::File {
 	Common::File file;
 public:
 	bool open(const char *filename, OpenMode mode) {
@@ -111,10 +115,11 @@ public:
 };
 
 static int eatSystemEvents() {
-	OSystem::Event event;
-	while (g_system->pollEvent(event)) {
+	Common::Event event;
+	Common::EventManager *eventMan = g_system->getEventManager();
+	while (eventMan->pollEvent(event)) {
 		switch (event.type) {
-		case OSystem::EVENT_QUIT:
+		case Common::EVENT_QUIT:
 			return 1;
 		default:
 			break;
@@ -198,16 +203,16 @@ static int MT32_Report(void *userData, MT32Emu::ReportType type, const void *rep
 		}
 		break;
 	case MT32Emu::ReportType_availableSSE:
-		debug(1, "MT32emu: SSE is avaliable");
+		debug(1, "MT32emu: SSE is available");
 		break;
 	case MT32Emu::ReportType_usingSSE:
 		debug(1, "MT32emu: using SSE");
 		break;
 	case MT32Emu::ReportType_available3DNow:
-		debug(1, "MT32emu: 3DNow! is avaliable");
+		debug(1, "MT32emu: 3DNow! is available");
 		break;
 	case MT32Emu::ReportType_using3DNow:
-		debug(1, "MT32emu: using 3DNow! ");
+		debug(1, "MT32emu: using 3DNow!");
 		break;
 	default:
 		break;
@@ -391,7 +396,7 @@ class MidiDriver_ThreadedMT32 : public MidiDriver_MT32 {
 private:
 	OSystem::Mutex _eventMutex;
 	MidiEvent_MT32 *_events;
-	Timer::TimerProc _timer_proc;
+	TimerManager::TimerProc _timer_proc;
 
 	void pushMidiEvent(MidiEvent_MT32 *event);
 	MidiEvent_MT32 *popMidiEvent();
@@ -405,7 +410,7 @@ public:
 
 	void onTimer();
 	void close();
-	void setTimerCallback(void *timer_param, Timer::TimerProc timer_proc);
+	void setTimerCallback(void *timer_param, TimerManager::TimerProc timer_proc);
 };
 
 
@@ -421,13 +426,13 @@ void MidiDriver_ThreadedMT32::close() {
 	}
 }
 
-void MidiDriver_ThreadedMT32::setTimerCallback(void *timer_param, Timer::TimerProc timer_proc) {
+void MidiDriver_ThreadedMT32::setTimerCallback(void *timer_param, TimerManager::TimerProc timer_proc) {
 	if (!_timer_proc || !timer_proc) {
 		if (_timer_proc)
-			g_timer->removeTimerProc(_timer_proc);
+			_vm->_timer->removeTimerProc(_timer_proc);
 		_timer_proc = timer_proc;
 		if (timer_proc)
-			g_timer->installTimerProc(timer_proc, getBaseTempo(), timer_param);
+			_vm->_timer->installTimerProc(timer_proc, getBaseTempo(), timer_param);
 	}
 }
 
