@@ -1,9 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2001-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
  *
- * YM2612 tone generation code written by Tomoaki Hayasaka.
- * Used under the terms of the GNU General Public License.
- * Adpated to ScummVM by Jamieson Christian.
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -19,14 +18,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/sound/softsynth/ym2612.cpp $
- * $Id: ym2612.cpp 20952 2006-02-27 01:59:07Z Jamieson630 $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/sound/softsynth/ym2612.cpp $
+ * $Id: ym2612.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  */
 
-#include "sound/softsynth/emumidi.h"
-
 #include <math.h>
 
+#include "sound/softsynth/ym2612.h"
 #include "common/util.h"
 
 ////////////////////////////////////////
@@ -42,153 +40,6 @@ static int *keycodeTable = 0;
 static int *keyscaleTable = 0;
 static int *attackOut = 0;
 
-////////////////////////////////////////
-//
-// Class declarations
-//
-////////////////////////////////////////
-
-class Operator2612;
-class Voice2612;
-class MidiChannel_YM2612;
-class MidiDriver_YM2612;
-
-class Operator2612 {
-protected:
-	Voice2612 *_owner;
-	enum State { _s_ready, _s_attacking, _s_decaying, _s_sustaining, _s_releasing };
-	State _state;
-	int32 _currentLevel;
-	int _frequency;
-	uint32 _phase;
-	int _lastOutput;
-	int _feedbackLevel;
-	int _detune;
-	int _multiple;
-	int32 _totalLevel;
-	int _keyScale;
-	int _velocity;
-	int _specifiedTotalLevel;
-	int _specifiedAttackRate;
-	int _specifiedDecayRate;
-	int _specifiedSustainLevel;
-	int _specifiedSustainRate;
-	int _specifiedReleaseRate;
-	int _tickCount;
-	int _attackTime;
-	int32 _decayRate;
-	int32 _sustainLevel;
-	int32 _sustainRate;
-	int32 _releaseRate;
-
-public:
-	Operator2612 (Voice2612 *owner);
-	~Operator2612();
-	void feedbackLevel(int level);
-	void setInstrument(byte const *instrument);
-	void velocity(int velo);
-	void keyOn();
-	void keyOff();
-	void frequency(int freq);
-	void nextTick(const int *phaseShift, int *outbuf, int buflen);
-	bool inUse() { return (_state != _s_ready); }
-};
-
-class Voice2612 {
-public:
-	Voice2612 *next;
-	uint16 _rate;
-
-protected:
-	Operator2612 *_opr[4];
-	int _velocity;
-	int _control7;
-	int _note;
-	int _frequencyOffs;
-	int _frequency;
-	int _algorithm;
-
-	int *_buffer;
-	int _buflen;
-
-public:
-	Voice2612();
-	~Voice2612();
-	void setControlParameter(int control, int value);
-	void setInstrument(byte const *instrument);
-	void velocity(int velo);
-	void nextTick(int *outbuf, int buflen);
-	void noteOn(int n, int onVelo);
-	bool noteOff(int note);
-	void pitchBend(int value);
-	void recalculateFrequency();
-};
-
-class MidiChannel_YM2612 : public MidiChannel {
-protected:
-	uint16 _rate;
-	Voice2612 *_voices;
-	Voice2612 *_next_voice;
-
-public:
-	void removeAllVoices();
-	void nextTick(int *outbuf, int buflen);
-	void rate(uint16 r);
-
-public:
-	MidiChannel_YM2612();
-	virtual ~MidiChannel_YM2612();
-
-	// MidiChannel interface
-	MidiDriver *device() { return 0; }
-	byte getNumber() { return 0; }
-	void release() { }
-	void send(uint32 b) { }
-	void noteOff(byte note);
-	void noteOn(byte note, byte onVelo);
-	void programChange(byte program) { }
-	void pitchBend(int16 value);
-	void controlChange(byte control, byte value);
-	void pitchBendFactor(byte value) { }
-	void sysEx_customInstrument(uint32 type, const byte *instr);
-};
-
-class MidiDriver_YM2612 : public MidiDriver_Emulated {
-protected:
-	MidiChannel_YM2612 *_channel[16];
-
-	int _next_voice;
-	int _volume;
-
-protected:
-	static void createLookupTables();
-	void nextTick(int16 *buf1, int buflen);
-	int volume(int val = -1) { if (val >= 0) _volume = val; return _volume; }
-	void rate(uint16 r);
-
-	void generateSamples(int16 *buf, int len);
-
-public:
-	MidiDriver_YM2612(Audio::Mixer *mixer);
-	virtual ~MidiDriver_YM2612();
-
-	int open();
-	void close();
-	void send(uint32 b);
-	void send(byte channel, uint32 b); // Supports higher than channel 15
-	uint32 property(int prop, uint32 param) { return 0; }
-
-	void setPitchBendRange(byte channel, uint range) { }
-	void sysEx(const byte *msg, uint16 length);
-
-	MidiChannel *allocateChannel() { return 0; }
-	MidiChannel *getPercussionChannel() { return 0; }
-
-
-	// AudioStream API
-	bool isStereo() const { return true; }
-	int getRate() const { return _mixer->getOutputRate(); }
-};
 
 ////////////////////////////////////////
 //
@@ -238,15 +89,15 @@ void Operator2612::setInstrument(byte const *instrument) {
 	_specifiedSustainRate = instrument[24] & 31;
 	_specifiedSustainLevel = (instrument[28] >> 4) & 15;
 	_specifiedReleaseRate = instrument[28] & 15;
-	_state = _s_ready; // ʪǤϤɤʤΤ?
+	_state = _s_ready;
 	velocity(_velocity);
 }
 
 void Operator2612::keyOn() {
 	_state = _s_attacking;
 	_tickCount = 0;
-	_phase = 0;			// ɤ⡢ºݤ餷
-	_currentLevel = ((int32)0x7f << 15); // ⡢ºݤ餷
+	_phase = 0;			
+	_currentLevel = ((int32)0x7f << 15); 
 }
 
 void Operator2612::keyOff() {
@@ -264,7 +115,7 @@ void Operator2612::frequency(int freq) {
 	if (r != 0) {
 		r = r * 2 + (keyscaleTable[freq/262205] >> (3-_keyScale));
 		if (r >= 64)
-		r = 63; // ٤ʤȤϻפ (p.207)
+		r = 63;
 	}
 
 	r = 63 - r;
@@ -273,7 +124,7 @@ void Operator2612::frequency(int freq) {
 	else {
 		value = powtbl[(r&3) << 7];
 		value *= 1 << (r >> 2);
-		value *= 41; // r == 20 ΤȤ0-96[db]  10.01[ms] == 41.00096
+		value *= 41; 
 		value /= 1 << (15 + 5);
 		value *= 127 - _specifiedTotalLevel;
 		value /= 127;
@@ -304,7 +155,7 @@ void Operator2612::frequency(int freq) {
 	if (r != 0) {
 		r = r * 2 + 1;		// (Translated) I cannot know whether the timing is a good choice or not
 		r = r * 2 + (keyscaleTable[freq/262205] >> (3-_keyScale));
-		// KS ˤϤ餷p.206 ǤϵҤƤʤɡ
+		// KS
 		if (r >= 64)
 			r = 63;
 	}
@@ -333,7 +184,6 @@ void Operator2612::nextTick(const int *phasebuf, int *outbuf, int buflen) {
 		switch (_state) {
 		case _s_ready:
 			return;
-			break;
 		case _s_attacking:
 			next_state = _s_attacking;
 			break;
@@ -378,11 +228,11 @@ void Operator2612::nextTick(const int *phasebuf, int *outbuf, int buflen) {
 			}
 
 			if (level < zero_level) {
-				int phaseShift = *phasebuf >> 2; // Ĵ̤?  3 㾮 2 礭褦ʡ
+				int phaseShift = *phasebuf >> 2; 
 				if (_feedbackLevel)
 					phaseShift += (output << (_feedbackLevel - 1)) / 1024;
 				output = sintbl[((_phase >> 7) + phaseShift) & 0x7ff];
-				output >>= (level >> 18);	// ̤?
+				output >>= (level >> 18);	
 				// Here is the original code, which requires 64-bit ints
 //				output *= powtbl[511 - ((level>>25)&511)];
 //				output >>= 16;
@@ -587,13 +437,13 @@ void Voice2612::pitchBend(int value) {
 }
 
 void Voice2612::recalculateFrequency() {
-	// MIDI Ȥ㤦....
-	// ɤͤʤ?
-	// Ȼפä顢ʤȡ () 餷
+	// 
+	// 
+	// 
 	int32 basefreq = frequencyTable[_note];
 	int cfreq = frequencyTable[_note - (_note % 12)];
 	int oct = _note / 12;
-	int fnum = (int) (((double)basefreq * (1 << 13)) / cfreq); // OPL  fnum Ʊ褦ʤΡ
+	int fnum = (int) (((double)basefreq * (1 << 13)) / cfreq);
 	fnum += _frequencyOffs - 0x2000;
 	if (fnum < 0x2000) {
 		fnum += 0x2000;
@@ -604,7 +454,7 @@ void Voice2612::recalculateFrequency() {
 		oct++;
 	}
 
-	// _frequency ϺǽŪ˥Х 256*1024 
+	//
 	_frequency = (int) ((frequencyTable[oct*12] * (double)fnum) / 8);
 
 	int i;
@@ -664,7 +514,7 @@ void MidiChannel_YM2612::noteOff(byte note) {
 }
 
 void MidiChannel_YM2612::controlChange(byte control, byte value) {
-	// Τ?
+	//
 	if (control == 121) {
 		// Reset controller
 		removeAllVoices();
@@ -686,7 +536,7 @@ void MidiChannel_YM2612::sysEx_customInstrument(uint32 type, const byte *fmInst)
 }
 
 void MidiChannel_YM2612::pitchBend(int16 value) {
-  // Τ?
+  //
 	Voice2612 *voice = _voices;
 	for (; voice; voice = voice->next)
 		voice->pitchBend(value);
@@ -727,13 +577,7 @@ MidiDriver_YM2612::~MidiDriver_YM2612() {
 	int i;
 	for (i = 0; i < ARRAYSIZE(_channel); i++)
 		delete _channel[i];
-	delete sintbl;
-	delete powtbl;
-	delete frequencyTable;
-	delete keycodeTable;
-	delete keyscaleTable;
-	delete attackOut;
-	sintbl = powtbl = frequencyTable = keycodeTable = keyscaleTable = attackOut = 0;
+	removeLookupTables();
 }
 
 int MidiDriver_YM2612::open() {
@@ -742,7 +586,7 @@ int MidiDriver_YM2612::open() {
 
 	MidiDriver_Emulated::open();
 
-	_mixer->setupPremix(this);
+	_mixer->playInputStream(Audio::Mixer::kPlainSoundType, &_mixerSoundHandle, this, -1, Audio::Mixer::kMaxChannelVolume, 0, false, true);
 	return 0;
 }
 
@@ -751,8 +595,7 @@ void MidiDriver_YM2612::close() {
 		return;
 	_isOpen = false;
 
-	// Detach the premix callback handler
-	_mixer->setupPremix(0);
+	_mixer->stopHandle(_mixerSoundHandle);
 }
 
 void MidiDriver_YM2612::send(uint32 b) {
@@ -852,8 +695,8 @@ void MidiDriver_YM2612::createLookupTables() {
 			0x03d5, 0x0410, 0x044e, 0x048f,
 		};
 
-		// (int)(880.0 * 256.0 * pow(2.0, (note-0x51)/12.0)); // Х 256 
-		// 0x45  440Hz (a4)0x51  880Hz (a5) 餷
+		// (int)(880.0 * 256.0 * pow(2.0, (note-0x51)/12.0))
+		// 
 		frequencyTable = new int [120];
 		for (block = -1; block < 9; block++) {
 			for (i = 0; i < 12; i++) {
@@ -863,7 +706,7 @@ void MidiDriver_YM2612::createLookupTables() {
 		}
 
 		keycodeTable = new int [120];
-		// detune ̤η׻ KS ˤ rate Ѵ˻Ȥ󤸤ʤ
+		// detune 
 		for (block = -1; block < 9; block++) {
 			for (i = 0; i < 12; i++) {
 				// see p.204
@@ -886,7 +729,7 @@ void MidiDriver_YM2612::createLookupTables() {
 		keyscaleTable[0] = 0;
 		for (freq = 1; freq < 8192; freq++) {
 			keyscaleTable[freq] = (int)(log((double)freq) / 9.03 * 32.0) - 1;
-			// 8368[Hz] (o9c)  32餤9.03 =:= ln 8368
+			// 8368[Hz] (o9c)
 		}
 	}
 
@@ -898,6 +741,16 @@ void MidiDriver_YM2612::createLookupTables() {
 	}
 }
 
+void MidiDriver_YM2612::removeLookupTables() {
+	delete [] sintbl;
+	delete [] powtbl;
+	delete [] frequencyTable;
+	delete [] keycodeTable;
+	delete [] keyscaleTable;
+	delete [] attackOut;
+	sintbl = powtbl = frequencyTable = keycodeTable = keyscaleTable = attackOut = 0;
+}
+
 ////////////////////////////////////////
 //
 // MidiDriver_YM2612 factory
@@ -907,3 +760,4 @@ void MidiDriver_YM2612::createLookupTables() {
 MidiDriver *MidiDriver_YM2612_create(Audio::Mixer *mixer) {
 	return new MidiDriver_YM2612(mixer);
 }
+
