@@ -1,6 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2001  Ludvig Strigeus
- * Copyright (C) 2001-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -16,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/sound/voc.h $
- * $Id: voc.h 22231 2006-04-29 22:33:31Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/sound/voc.h $
+ * $Id: voc.h 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -34,27 +36,23 @@ namespace Audio {
 class AudioStream;
 
 
-#if !defined(__GNUC__)
-#pragma START_PACK_STRUCTS
-#endif
+#include "common/pack-start.h"	// START STRUCT PACKING
 
 struct VocFileHeader {
 	uint8 desc[20];
 	uint16 datablock_offset;
 	uint16 version;
 	uint16 id;
-} GCC_PACK;
+};
 
 struct VocBlockHeader {
 	uint8 blocktype;
 	uint8 size[3];
 	uint8 sr;
 	uint8 pack;
-} GCC_PACK;
+};
 
-#if !defined(__GNUC__)
-#pragma END_PACK_STRUCTS
-#endif
+#include "common/pack-end.h"	// END STRUCT PACKING
 
 /**
  * Take a sample rate parameter as it occurs in a VOC sound header, and
