@@ -1,5 +1,8 @@
-/* ScummVM - Scumm Interpreter
- * Copyright (C) 2005-2006 The ScummVM project
+/* ScummVM - Graphic Adventure Engine
+ *
+ * ScummVM is the legal property of its developers, whose names
+ * are too numerous to list here. Please refer to the COPYRIGHT
+ * file distributed with this source distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/sound/wave.cpp $
- * $Id: wave.cpp 22596 2006-05-24 06:11:39Z kirben $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/sound/wave.cpp $
+ * $Id: wave.cpp 27024 2007-05-30 21:56:52Z fingolfin $
  *
  */
 
@@ -192,7 +195,7 @@ AudioStream *makeWAVStream(Common::SeekableReadStream &stream) {
 	// Since we allocated our own buffer for the data, we must set the autofree flag.
 	flags |= Audio::Mixer::FLAG_AUTOFREE;
 
-	return makeLinearInputStream(rate, flags, data, size, 0, 0);
+	return makeLinearInputStream(data, size, rate, flags, 0, 0);
 }
 
 } // End of namespace Audio
