@@ -179,8 +179,11 @@ sub begin_credits {
 		print "// This file was generated by credits.pl. Do not edit by hand!\n";
 		print "static const char *credits[] = {\n";
 	} elsif ($mode eq "XML") {
+		print "<?xml version='1.0'?>\n";
 		print "<!-- This file was generated by credits.pl. Do not edit by hand! -->\n";
-		print "<appendix>\n";
+		print "<!DOCTYPE appendix PUBLIC '-//OASIS//DTD DocBook XML V4.2//EN'\n";
+		print "       'http://www.oasis-open.org/docbook/xml/4.2/docbookx.dtd'>\n";
+		print "<appendix id='credits'>\n";
 		print "  <title>" . $title . "</title>\n";
 		print "  <informaltable frame='none'>\n";
 		print "  <tgroup cols='3' align='left' colsep='0' rowsep='0'>\n";
@@ -454,8 +457,8 @@ begin_credits("Credits");
   begin_section("ScummVM Team");
     begin_section("Project Leaders");
 	  begin_persons();
-		add_person('James Brown', 'ender', "");
-		add_person('Max Horn', 'Fingolfin', "");
+		add_person("James Brown", "ender", "");
+		add_person("Max Horn", "Fingolfin", "");
 		add_person("Eugene Sandulenko", "sev", "");
 	  end_persons();
     end_section();
@@ -464,9 +467,9 @@ begin_credits("Credits");
     begin_section("Engine Teams");
 	  begin_section("SCUMM");
 		  add_person("Torbj&ouml;rn Andersson", "eriktorbjorn", "");
-		  add_person('James Brown', 'ender', "");
+		  add_person("James Brown", "ender", "");
 		  add_person("Jonathan Gray", "khalek", "");
-		  add_person('Max Horn', 'Fingolfin', "");
+		  add_person("Max Horn", "Fingolfin", "");
 		  add_person("Travis Howell", "Kirben", "");
 		  add_person("Pawe&#322; Ko&#322;odziejski", "aquadran", "Codecs, iMUSE, Smush, etc.");
 		  add_person("Eugene Sandulenko", "sev", "FT INSANE, MM NES, MM C64, game detection, Herc/CGA");
@@ -480,8 +483,17 @@ begin_credits("Credits");
 	  end_section();
 
 	  begin_section("AGI");
+		  add_person("Stuart George", "darkfiber", "");
+		  add_person("Filippos Karapetis", "[md5]", "");
 		  add_person("Pawe&#322; Ko&#322;odziejski", "aquadran", "");
 		  add_person("Eugene Sandulenko", "sev", "");
+		  add_person("David Symonds", "dsymonds", "");
+	  end_section();
+
+	  begin_section("AGOS");
+		  add_person("Torbj&ouml;rn Andersson", "eriktorbjorn", "");
+		  add_person("Travis Howell", "Kirben", "");
+		  add_person("Oliver Kiehl", "olki", "");
 	  end_section();
 
 	  begin_section("BASS");	# Beneath a Steel Sky
@@ -500,11 +512,16 @@ begin_credits("Credits");
 	  end_section();
 
 	  begin_section("Cinematique evo 1");
+		  add_person("Vincent Hamm", "yazoo", "original CinE engine author");
 		  add_person("Pawe&#322; Ko&#322;odziejski", "aquadran", "");
 		  add_person("Gregory Montoir", "cyx", "");
 		  add_person("Eugene Sandulenko", "sev", "");
 	  end_section();
 
+	  begin_section("Cinematique evo 2");
+		  add_person("Vincent Hamm", "yazoo", "");
+	  end_section();
+
 	  begin_section("FOTAQ");	# Flight of the Amazon Queen
 		  add_person("David Eriksson", "twogood", "");
 		  add_person("Gregory Montoir", "cyx", "");
@@ -529,16 +546,21 @@ begin_credits("Credits");
 		  add_person("Paul Gilbert", "dreammaster", "");
 	  end_section();
 
+	  begin_section("Parallaction");
+		  add_person("", "peres", "");
+	  end_section();
+
 	  begin_section("SAGA");
 		  add_person("Torbj&ouml;rn Andersson", "eriktorbjorn", "");
+		  add_person("Filippos Karapetis", "[md5]", "");
 		  add_person("Andrew Kurushin", "ajax16384", "");
 		  add_person("Eugene Sandulenko", "sev", "");
 	  end_section();
 
-	  begin_section("Simon");
-		  add_person("Travis Howell", "Kirben", "");
-		  add_person("Oliver Kiehl", "olki", "");
+	  begin_section("Touch&eacute;");
+		  add_person("Gregory Montoir", "cyx", "");
 	  end_section();
+
     end_section();
 
 
@@ -565,7 +587,7 @@ begin_credits("Credits");
 
 	  begin_section("PocketPC / WinCE");
 		  add_person("Nicolas Bacca", "arisme", "");
-		  add_person("Kostas Nakos", "knakos", "");
+		  add_person("Kostas Nakos", "Jubanka", "");
 	  end_section();
 
 	  begin_section("PlayStation 2");
@@ -577,8 +599,8 @@ begin_credits("Credits");
 	  end_section();
 
 	  begin_section("SDL (Win/Linux/OS X/etc.)");
-		  add_person('Max Horn', 'Fingolfin', "");
-		  add_person('Eugene Sandulenko', 'sev', "Asm routines, GFX layers");
+		  add_person("Max Horn", "Fingolfin", "");
+		  add_person("Eugene Sandulenko", "sev", "Asm routines, GFX layers");
 	  end_section();
 
 	  begin_section("SymbianOS");
@@ -590,7 +612,7 @@ begin_credits("Credits");
 
     begin_section("Other subsystems");
 	  begin_section("Infrastructure");
-		  add_person('Max Horn', 'Fingolfin', "Backend &amp; Engine APIs, file API, sound mixer, audiostreams, data structures, etc.");
+		  add_person("Max Horn", "Fingolfin", "Backend &amp; Engine APIs, file API, sound mixer, audiostreams, data structures, etc.");
 		  add_person("Eugene Sandulenko", "sev", "");
 	  end_section();
 
@@ -612,6 +634,7 @@ begin_credits("Credits");
     begin_section("Documentation");
 	begin_persons();
 		add_person("Joachim Eberhard", "joachimeberhard", "Documentation manager");
+		add_person("Matthew Hoops", "clone2727", "Wiki editor");
     	end_persons();
     end_section();
 
@@ -620,13 +643,12 @@ begin_credits("Credits");
 		add_person("Tore Anderson", "tore", "Former Debian GNU/Linux maintainer");
 		add_person("Ralph Brorsen", "painelf", "Help with GUI implementation");
 		add_person("Jamieson Christian", "jamieson630", "iMUSE, MIDI, all things musical");
-		add_person('Vincent Hamm', 'yazoo', "Co-Founder, original CinE engine author");
 		add_person("R&uuml;diger Hanke", "", "Port: MorphOS");
 		add_person("Felix Jakschitsch", "yot", "Zak256 reverse engineering");
 		add_person("Mutwin Kraus", "mutle", "Original MacOS porter");
 		add_person("Peter Moraliyski", "ph0x", "Port: GP32");
-		add_person('Jeremy Newman', 'laxdragon', "Former webmaster");
-		add_person('Ludvig Strigeus', 'ludde', "Original ScummVM and SimonVM author");
+		add_person("Jeremy Newman", "laxdragon", "Former webmaster");
+		add_person("Ludvig Strigeus", "ludde", "Original ScummVM and SimonVM author");
 		add_person("Lionel Ulmer", "bbrox", "Port: X11");
 	  end_persons();
     end_section();
@@ -658,15 +680,16 @@ begin_credits("Credits");
 	  end_section();
 
 	  begin_section("Mac OS X");
-		  add_person('Max Horn', 'Fingolfin', "");
+		  add_person("Max Horn", "Fingolfin", "");
+		  add_person("Oystein Eftevaag", "vinterstum", "");
 	  end_section();
 
 	  begin_section("Mandriva");
-		  add_person('Dominik Scherer', '', "");
+		  add_person("Dominik Scherer", "", "");
 	  end_section();
 
 	  begin_section("MorphOS");
-		  add_person('Fabien Coeurjoly', '', "");
+		  add_person("Fabien Coeurjoly", "fab1", "");
 	  end_section();
 
 	  begin_section("OS/2");
@@ -681,7 +704,7 @@ begin_credits("Credits");
 		  add_person("Laurent Blume", "laurent", "");
 	  end_section();
 
-	  begin_section("Solaris Spark");
+	  begin_section("Solaris SPARC");
 		  add_person("Markus Strangl", "WooShell", "");
 	  end_section();
 
@@ -699,32 +722,37 @@ begin_credits("Credits");
 		  add_person("Dob&oacute; Bal&aacute;zs", "draven", "Website design");
 		  add_person("Yaroslav Fedevych", "jafd", "HTML/CSS for the website");
 		  add_person("David Jensen", "Tyst", "SVG logo conversion");
-		  add_person("Jean Marc", "", 'ScummVM logo');
-		  add_person("", "Raina", 'ScummVM forum buttons');
-		  add_person("Clemens Steinhuber", "", 'ScummVM forum theme');
+		  add_person("Jean Marc", "", "ScummVM logo");
+		  add_person("", "Raina", "ScummVM forum buttons");
+		  add_person("Clemens Steinhuber", "", "ScummVM forum theme");
 	  end_persons();
 	end_section();
 
 	begin_section("Code contributions");
 	  begin_persons();
-		  add_person("Ori Avtalion", "salty-horse", "Subtitle control options in the GUI");
+		  add_person("Ori Avtalion", "salty-horse", "Subtitle control options in the GUI; BASS GUI fixes");
 		  add_person("Stuart Caie", "", "Decoders for Simon 1 Amiga data files");
 		  add_person("Paolo Costabel", "", "PSP port contributions");
-		  add_person("Benjamin Haisch", "", "Encoder for DXA videos");
+		  add_person("Thierry Crozat", "criezy", "Support for Broken Sword 1 Macintosh version");
+		  add_person("Thomas Fach-Pedersen", "madmoose", "ProTracker module player");
+		  add_person("Benjamin Haisch", "john_doe", "Heavily improved de-/encoder for DXA videos");
 		  add_person("Janne Huttunen", "", "V3 actor mask support, Dig/FT SMUSH audio");
 		  add_person("Kov&aacute;cs Endre J&aacute;nos", "", "Several fixes for Simon1");
-		  add_person("Jeroen Janssen", "", "Numerous readability and bugfix patches");
+		  add_person("Jeroen Janssen", "japj", "Numerous readability and bugfix patches");
+		  add_person("Florian Kagerer", "athrxx", "Various Kyrandia 1 FM-Towns code contributions");
 		  add_person("Andreas Karlsson", "Sprawl", "Initial port for EPOC/SymbianOS");
-		  add_person("Claudio Matsuoka", "", 'Daily Linux builds');
+		  add_person("Claudio Matsuoka", "", "Daily Linux builds");
 		  add_person("Thomas Mayer", "", "PSP port contributions");
-		  add_person("Mikesch Nepomuk", "", "MI1 VGA floppy patches");
+		  add_person("", "n0p", "Windows CE port aspect ratio correction scaler and right click input method");
+		  add_person("Mikesch Nepomuk", "mnepomuk", "MI1 VGA floppy patches");
 		  add_person("Nicolas Noble", "pixels", "Config file and ALSA support");
+		  add_person("Tim Phillips", "realmz", "Initial MI1 CD music support");
 		  add_person("", "Quietust", "Sound support for Amiga SCUMM V2/V3 games, MM NES support");
 		  add_person("Andreas R&ouml;ver", "", "Broken Sword 1/2 MPEG2 cutscene support");
-		  add_person("Edward Rudd", "", "Fixes for playing MP3 versions of MI1/Loom audio");
-		  add_person("Daniel Schepler", "", "Final MI1 CD music support, initial Ogg Vorbis support");
-		  add_person("Andr&eacute; Souza", "", "SDL-based OpenGL renderer");
-		  add_person("Tim Phillips", "realmz", "Initial MI1 CD music support");
+		  add_person("Edward Rudd", "urkle", "Fixes for playing MP3 versions of MI1/Loom audio");
+		  add_person("Daniel Schepler", "dschepler", "Final MI1 CD music support, initial Ogg Vorbis support");
+		  add_person("Andr&eacute; Souza", "luke_br", "SDL-based OpenGL renderer");
+		  add_person("Robin Watts", "robinwatts", "ARM assembly routines for the Windows CE port");
 	  end_persons();
 	end_section();
 
@@ -775,7 +803,8 @@ begin_credits("Credits");
 
   add_paragraph(
   "Alan Bridgman, Simon Woodroffe and everyone at Adventure Soft for ".
-  "sharing the source code of The Feeble Files with us.");
+  "sharing the source code of The Feeble Files and Simon the Sorcerer's ".
+  "Puzzle Pack with us.");
 
   end_section();
 
