@@ -15,8 +15,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-9-1/tools/md5table.c $
- * $Id: md5table.c 21702 2006-04-08 23:12:56Z sev $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-0-10-0/tools/md5table.c $
+ * $Id: md5table.c 25951 2007-03-03 12:59:48Z kirben $
  *
  */
 
@@ -62,6 +62,7 @@ typedef struct {
 	const char *platform;
 	const char *variant;
 	const char *extra;
+	const char *size;
 	const char *desc;
 	const char *comments;
 	const char *infoSource;
@@ -78,11 +79,12 @@ static const StringMap platformMap[] = {
 	{ "Atari",		"kPlatformAtariST" },
 	{ "C64",		"kPlatformC64" },
 	{ "DOS",		"kPlatformPC" },
-	{ "FM-TOWNS",	"kPlatformFMTowns" },
+	{ "FM-TOWNS",		"kPlatformFMTowns" },
 	{ "Mac",		"kPlatformMacintosh" },
 	{ "NES",		"kPlatformNES" },
+	{ "PC-Engine",		"kPlatformPCEngine" },
 	{ "SEGA",		"kPlatformSegaCD" },
-	{ "Windows",	"kPlatformWindows" },
+	{ "Windows",		"kPlatformWindows" },
 
 	{ "All?",		"kPlatformUnknown" },
 	{ "All",		"kPlatformUnknown" },
@@ -97,7 +99,7 @@ static const StringMap langMap[] = {
 	{ "de",		"DE_DEU" },
 	{ "fr",		"FR_FRA" },
 	{ "it",		"IT_ITA" },
-	{ "pt",		"PT_BRA" },
+	{ "br",		"PT_BRA" },
 	{ "es",		"ES_ESP" },
 	{ "jp",		"JA_JPN" },
 	{ "zh",		"ZH_TWN" },
@@ -136,6 +138,7 @@ static const char *c_header =
 	"	const char *gameid;\n"
 	"	const char *variant;\n"
 	"	const char *extra;\n"
+	"	int32 filesize;\n"
 	"	Common::Language language;\n"
 	"	Common::Platform platform;\n"
 	"};\n"
@@ -143,7 +146,7 @@ static const char *c_header =
 	"static const MD5Table md5table[] = {\n";
 
 static const char *c_footer =
-	"	{ 0, 0, 0, 0, Common::UNK_LANG, Common::kPlatformUnknown }\n"
+	"	{ 0, 0, 0, 0, 0, Common::UNK_LANG, Common::kPlatformUnknown }\n"
 	"};\n";
 
 static void parseEntry(Entry *entry, char *line) {
@@ -152,6 +155,7 @@ static void parseEntry(Entry *entry, char *line) {
 	
 	/* Split at the tabs */
 	entry->md5 = strtok(line, "\t\n\r");
+	entry->size = strtok(NULL, "\t\n\r");
 	entry->language = strtok(NULL, "\t\n\r");
 	entry->platform = strtok(NULL, "\t\n\r");
 	entry->variant = strtok(NULL, "\t\n\r");
@@ -273,8 +277,9 @@ int main(int argc, char *argv[])
 					fprintf(outFile, ", \"%s\"", entry.infoSource);
 				fprintf(outFile, ");\n");
 			} else if (outputMode == kTXTOutput) {
-				fprintf(outFile, "\t%s\t%s\t%s\t%s\t%s\t%s\t%s\n",
+				fprintf(outFile, "\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\n",
 					entry.md5,
+					entry.size,
 					entry.language,
 					entry.platform,
 					entry.variant,
@@ -292,8 +297,14 @@ int main(int argc, char *argv[])
 				if (0 == strcmp(entry.extra, "-"))
 					entry.extra = "";
 				snprintf(entriesBuffer + numEntries * entrySize, entrySize,
-					"\t{ \"%s\", \"%s\", \"%s\", \"%s\", Common::%s, Common::%s },\n",
-					entry.md5, gameid, entry.variant, entry.extra, mapStr(entry.language, langMap), mapStr(entry.platform, platformMap));
+					"\t{ \"%s\", \"%s\", \"%s\", \"%s\", %s, Common::%s, Common::%s },\n",
+					entry.md5,
+					gameid,
+					entry.variant,
+					entry.extra,
+					entry.size,
+					mapStr(entry.language, langMap),
+					mapStr(entry.platform, platformMap));
 				numEntries++;
 			}
 		} else {
